(function() {
  var CSRFToken, Click, ComponentUrl, EVENTS, Link, ProgressBar, browserIsntBuggy, browserSupportsCustomEvents, browserSupportsPushState, browserSupportsTurbolinks, bypassOnLoadPopstate, cacheCurrentPage, cacheSize, changePage, clone, constrainPageCacheTo, createDocument, crossOriginRedirect, currentState, enableProgressBar, enableTransitionCache, executeScriptTags, extractTitleAndBody, fetch, fetchHistory, fetchReplacement, historyStateIsDefined, initializeTurbolinks, installDocumentReadyPageEventTriggers, installHistoryChangeHandler, installJqueryAjaxSuccessPageUpdateTrigger, loadedAssets, manuallyTriggerHashChangeForFirefox, pageCache, pageChangePrevented, pagesCached, popCookie, processResponse, progressBar, recallScrollPosition, ref, referer, reflectNewUrl, reflectRedirectedUrl, rememberCurrentState, rememberCurrentUrl, rememberReferer, removeNoscriptTags, requestMethodIsSafe, resetScrollPosition, setAutofocusElement, transitionCacheEnabled, transitionCacheFor, triggerEvent, visit, xhr,
    indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; },
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty,
    slice = [].slice,
    bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  pageCache = {};

  cacheSize = 10;

  transitionCacheEnabled = false;

  progressBar = null;

  currentState = null;

  loadedAssets = null;

  referer = null;

  xhr = null;

  EVENTS = {
    BEFORE_CHANGE: 'page:before-change',
    FETCH: 'page:fetch',
    RECEIVE: 'page:receive',
    CHANGE: 'page:change',
    UPDATE: 'page:update',
    LOAD: 'page:load',
    RESTORE: 'page:restore',
    BEFORE_UNLOAD: 'page:before-unload',
    EXPIRE: 'page:expire'
  };

  fetch = function(url) {
    var cachedPage;
    url = new ComponentUrl(url);
    rememberReferer();
    cacheCurrentPage();
    if (progressBar != null) {
      progressBar.start();
    }
    if (transitionCacheEnabled && (cachedPage = transitionCacheFor(url.absolute))) {
      fetchHistory(cachedPage);
      return fetchReplacement(url, null, false);
    } else {
      return fetchReplacement(url, resetScrollPosition);
    }
  };

  transitionCacheFor = function(url) {
    var cachedPage;
    cachedPage = pageCache[url];
    if (cachedPage && !cachedPage.transitionCacheDisabled) {
      return cachedPage;
    }
  };

  enableTransitionCache = function(enable) {
    if (enable == null) {
      enable = true;
    }
    return transitionCacheEnabled = enable;
  };

  enableProgressBar = function(enable) {
    if (enable == null) {
      enable = true;
    }
    if (!browserSupportsTurbolinks) {
      return;
    }
    if (enable) {
      return progressBar != null ? progressBar : progressBar = new ProgressBar('html');
    } else {
      if (progressBar != null) {
        progressBar.uninstall();
      }
      return progressBar = null;
    }
  };

  fetchReplacement = function(url, onLoadFunction, showProgressBar) {
    if (showProgressBar == null) {
      showProgressBar = true;
    }
    triggerEvent(EVENTS.FETCH, {
      url: url.absolute
    });
    if (xhr != null) {
      xhr.abort();
    }
    xhr = new XMLHttpRequest;
    xhr.open('GET', url.withoutHashForIE10compatibility(), true);
    xhr.setRequestHeader('Accept', 'text/html, application/xhtml+xml, application/xml');
    xhr.setRequestHeader('X-XHR-Referer', referer);
    xhr.onload = function() {
      var doc;
      triggerEvent(EVENTS.RECEIVE, {
        url: url.absolute
      });
      if (doc = processResponse()) {
        reflectNewUrl(url);
        reflectRedirectedUrl();
        changePage.apply(null, extractTitleAndBody(doc));
        manuallyTriggerHashChangeForFirefox();
        if (typeof onLoadFunction === "function") {
          onLoadFunction();
        }
        return triggerEvent(EVENTS.LOAD);
      } else {
        return document.location.href = crossOriginRedirect() || url.absolute;
      }
    };
    if (progressBar && showProgressBar) {
      xhr.onprogress = (function(_this) {
        return function(event) {
          var percent;
          percent = event.lengthComputable ? event.loaded / event.total * 100 : progressBar.value + (100 - progressBar.value) / 10;
          return progressBar.advanceTo(percent);
        };
      })(this);
    }
    xhr.onloadend = function() {
      return xhr = null;
    };
    xhr.onerror = function() {
      return document.location.href = url.absolute;
    };
    return xhr.send();
  };

  fetchHistory = function(cachedPage) {
    if (xhr != null) {
      xhr.abort();
    }
    changePage(cachedPage.title, cachedPage.body);
    recallScrollPosition(cachedPage);
    return triggerEvent(EVENTS.RESTORE);
  };

  cacheCurrentPage = function() {
    var currentStateUrl;
    currentStateUrl = new ComponentUrl(currentState.url);
    pageCache[currentStateUrl.absolute] = {
      url: currentStateUrl.relative,
      body: document.body,
      title: document.title,
      positionY: window.pageYOffset,
      positionX: window.pageXOffset,
      cachedAt: new Date().getTime(),
      transitionCacheDisabled: document.querySelector('[data-no-transition-cache]') != null
    };
    return constrainPageCacheTo(cacheSize);
  };

  pagesCached = function(size) {
    if (size == null) {
      size = cacheSize;
    }
    if (/^[\d]+$/.test(size)) {
      return cacheSize = parseInt(size);
    }
  };

  constrainPageCacheTo = function(limit) {
    var cacheTimesRecentFirst, i, key, len, pageCacheKeys, results;
    pageCacheKeys = Object.keys(pageCache);
    cacheTimesRecentFirst = pageCacheKeys.map(function(url) {
      return pageCache[url].cachedAt;
    }).sort(function(a, b) {
      return b - a;
    });
    results = [];
    for (i = 0, len = pageCacheKeys.length; i < len; i++) {
      key = pageCacheKeys[i];
      if (!(pageCache[key].cachedAt <= cacheTimesRecentFirst[limit])) {
        continue;
      }
      triggerEvent(EVENTS.EXPIRE, pageCache[key]);
      results.push(delete pageCache[key]);
    }
    return results;
  };

  changePage = function(title, body, csrfToken, runScripts) {
    triggerEvent(EVENTS.BEFORE_UNLOAD);
    document.title = title;
    document.documentElement.replaceChild(body, document.body);
    if (csrfToken != null) {
      CSRFToken.update(csrfToken);
    }
    setAutofocusElement();
    if (runScripts) {
      executeScriptTags();
    }
    currentState = window.history.state;
    if (progressBar != null) {
      progressBar.done();
    }
    triggerEvent(EVENTS.CHANGE);
    return triggerEvent(EVENTS.UPDATE);
  };

  executeScriptTags = function() {
    var attr, copy, i, j, len, len1, nextSibling, parentNode, ref, ref1, script, scripts;
    scripts = Array.prototype.slice.call(document.body.querySelectorAll('script:not([data-turbolinks-eval="false"])'));
    for (i = 0, len = scripts.length; i < len; i++) {
      script = scripts[i];
      if (!((ref = script.type) === '' || ref === 'text/javascript')) {
        continue;
      }
      copy = document.createElement('script');
      ref1 = script.attributes;
      for (j = 0, len1 = ref1.length; j < len1; j++) {
        attr = ref1[j];
        copy.setAttribute(attr.name, attr.value);
      }
      if (!script.hasAttribute('async')) {
        copy.async = false;
      }
      copy.appendChild(document.createTextNode(script.innerHTML));
      parentNode = script.parentNode, nextSibling = script.nextSibling;
      parentNode.removeChild(script);
      parentNode.insertBefore(copy, nextSibling);
    }
  };

  removeNoscriptTags = function(node) {
    node.innerHTML = node.innerHTML.replace(/<noscript[\S\s]*?<\/noscript>/ig, '');
    return node;
  };

  setAutofocusElement = function() {
    var autofocusElement, list;
    autofocusElement = (list = document.querySelectorAll('input[autofocus], textarea[autofocus]'))[list.length - 1];
    if (autofocusElement && document.activeElement !== autofocusElement) {
      return autofocusElement.focus();
    }
  };

  reflectNewUrl = function(url) {
    if ((url = new ComponentUrl(url)).absolute !== referer) {
      return window.history.pushState({
        turbolinks: true,
        url: url.absolute
      }, '', url.absolute);
    }
  };

  reflectRedirectedUrl = function() {
    var location, preservedHash;
    if (location = xhr.getResponseHeader('X-XHR-Redirected-To')) {
      location = new ComponentUrl(location);
      preservedHash = location.hasNoHash() ? document.location.hash : '';
      return window.history.replaceState(window.history.state, '', location.href + preservedHash);
    }
  };

  crossOriginRedirect = function() {
    var redirect;
    if (((redirect = xhr.getResponseHeader('Location')) != null) && (new ComponentUrl(redirect)).crossOrigin()) {
      return redirect;
    }
  };

  rememberReferer = function() {
    return referer = document.location.href;
  };

  rememberCurrentUrl = function() {
    return window.history.replaceState({
      turbolinks: true,
      url: document.location.href
    }, '', document.location.href);
  };

  rememberCurrentState = function() {
    return currentState = window.history.state;
  };

  manuallyTriggerHashChangeForFirefox = function() {
    var url;
    if (navigator.userAgent.match(/Firefox/) && !(url = new ComponentUrl).hasNoHash()) {
      window.history.replaceState(currentState, '', url.withoutHash());
      return document.location.hash = url.hash;
    }
  };

  recallScrollPosition = function(page) {
    return window.scrollTo(page.positionX, page.positionY);
  };

  resetScrollPosition = function() {
    if (document.location.hash) {
      return document.location.href = document.location.href;
    } else {
      return window.scrollTo(0, 0);
    }
  };

  clone = function(original) {
    var copy, key, value;
    if ((original == null) || typeof original !== 'object') {
      return original;
    }
    copy = new original.constructor();
    for (key in original) {
      value = original[key];
      copy[key] = clone(value);
    }
    return copy;
  };

  popCookie = function(name) {
    var ref, value;
    value = ((ref = document.cookie.match(new RegExp(name + "=(\\w+)"))) != null ? ref[1].toUpperCase() : void 0) || '';
    document.cookie = name + '=; expires=Thu, 01-Jan-70 00:00:01 GMT; path=/';
    return value;
  };

  triggerEvent = function(name, data) {
    var event;
    if (typeof Prototype !== 'undefined') {
      Event.fire(document, name, data, true);
    }
    event = document.createEvent('Events');
    if (data) {
      event.data = data;
    }
    event.initEvent(name, true, true);
    return document.dispatchEvent(event);
  };

  pageChangePrevented = function(url) {
    return !triggerEvent(EVENTS.BEFORE_CHANGE, {
      url: url
    });
  };

  processResponse = function() {
    var assetsChanged, clientOrServerError, doc, extractTrackAssets, intersection, validContent;
    clientOrServerError = function() {
      var ref;
      return (400 <= (ref = xhr.status) && ref < 600);
    };
    validContent = function() {
      var contentType;
      return ((contentType = xhr.getResponseHeader('Content-Type')) != null) && contentType.match(/^(?:text\/html|application\/xhtml\+xml|application\/xml)(?:;|$)/);
    };
    extractTrackAssets = function(doc) {
      var i, len, node, ref, results;
      ref = doc.querySelector('head').childNodes;
      results = [];
      for (i = 0, len = ref.length; i < len; i++) {
        node = ref[i];
        if ((typeof node.getAttribute === "function" ? node.getAttribute('data-turbolinks-track') : void 0) != null) {
          results.push(node.getAttribute('src') || node.getAttribute('href'));
        }
      }
      return results;
    };
    assetsChanged = function(doc) {
      var fetchedAssets;
      loadedAssets || (loadedAssets = extractTrackAssets(document));
      fetchedAssets = extractTrackAssets(doc);
      return fetchedAssets.length !== loadedAssets.length || intersection(fetchedAssets, loadedAssets).length !== loadedAssets.length;
    };
    intersection = function(a, b) {
      var i, len, ref, results, value;
      if (a.length > b.length) {
        ref = [b, a], a = ref[0], b = ref[1];
      }
      results = [];
      for (i = 0, len = a.length; i < len; i++) {
        value = a[i];
        if (indexOf.call(b, value) >= 0) {
          results.push(value);
        }
      }
      return results;
    };
    if (!clientOrServerError() && validContent()) {
      doc = createDocument(xhr.responseText);
      if (doc && !assetsChanged(doc)) {
        return doc;
      }
    }
  };

  extractTitleAndBody = function(doc) {
    var title;
    title = doc.querySelector('title');
    return [title != null ? title.textContent : void 0, removeNoscriptTags(doc.querySelector('body')), CSRFToken.get(doc).token, 'runScripts'];
  };

  CSRFToken = {
    get: function(doc) {
      var tag;
      if (doc == null) {
        doc = document;
      }
      return {
        node: tag = doc.querySelector('meta[name="csrf-token"]'),
        token: tag != null ? typeof tag.getAttribute === "function" ? tag.getAttribute('content') : void 0 : void 0
      };
    },
    update: function(latest) {
      var current;
      current = this.get();
      if ((current.token != null) && (latest != null) && current.token !== latest) {
        return current.node.setAttribute('content', latest);
      }
    }
  };

  createDocument = function(html) {
    var doc;
    doc = document.documentElement.cloneNode();
    doc.innerHTML = html;
    doc.head = doc.querySelector('head');
    doc.body = doc.querySelector('body');
    return doc;
  };

  ComponentUrl = (function() {
    function ComponentUrl(original1) {
      this.original = original1 != null ? original1 : document.location.href;
      if (this.original.constructor === ComponentUrl) {
        return this.original;
      }
      this._parse();
    }

    ComponentUrl.prototype.withoutHash = function() {
      return this.href.replace(this.hash, '').replace('#', '');
    };

    ComponentUrl.prototype.withoutHashForIE10compatibility = function() {
      return this.withoutHash();
    };

    ComponentUrl.prototype.hasNoHash = function() {
      return this.hash.length === 0;
    };

    ComponentUrl.prototype.crossOrigin = function() {
      return this.origin !== (new ComponentUrl).origin;
    };

    ComponentUrl.prototype._parse = function() {
      var ref;
      (this.link != null ? this.link : this.link = document.createElement('a')).href = this.original;
      ref = this.link, this.href = ref.href, this.protocol = ref.protocol, this.host = ref.host, this.hostname = ref.hostname, this.port = ref.port, this.pathname = ref.pathname, this.search = ref.search, this.hash = ref.hash;
      this.origin = [this.protocol, '//', this.hostname].join('');
      if (this.port.length !== 0) {
        this.origin += ":" + this.port;
      }
      this.relative = [this.pathname, this.search, this.hash].join('');
      return this.absolute = this.href;
    };

    return ComponentUrl;

  })();

  Link = (function(superClass) {
    extend(Link, superClass);

    Link.HTML_EXTENSIONS = ['html'];

    Link.allowExtensions = function() {
      var extension, extensions, i, len;
      extensions = 1 <= arguments.length ? slice.call(arguments, 0) : [];
      for (i = 0, len = extensions.length; i < len; i++) {
        extension = extensions[i];
        Link.HTML_EXTENSIONS.push(extension);
      }
      return Link.HTML_EXTENSIONS;
    };

    function Link(link1) {
      this.link = link1;
      if (this.link.constructor === Link) {
        return this.link;
      }
      this.original = this.link.href;
      this.originalElement = this.link;
      this.link = this.link.cloneNode(false);
      Link.__super__.constructor.apply(this, arguments);
    }

    Link.prototype.shouldIgnore = function() {
      return this.crossOrigin() || this._anchored() || this._nonHtml() || this._optOut() || this._target();
    };

    Link.prototype._anchored = function() {
      return (this.hash.length > 0 || this.href.charAt(this.href.length - 1) === '#') && (this.withoutHash() === (new ComponentUrl).withoutHash());
    };

    Link.prototype._nonHtml = function() {
      return this.pathname.match(/\.[a-z]+$/g) && !this.pathname.match(new RegExp("\\.(?:" + (Link.HTML_EXTENSIONS.join('|')) + ")?$", 'g'));
    };

    Link.prototype._optOut = function() {
      var ignore, link;
      link = this.originalElement;
      while (!(ignore || link === document)) {
        ignore = link.getAttribute('data-no-turbolink') != null;
        link = link.parentNode;
      }
      return ignore;
    };

    Link.prototype._target = function() {
      return this.link.target.length !== 0;
    };

    return Link;

  })(ComponentUrl);

  Click = (function() {
    Click.installHandlerLast = function(event) {
      if (!event.defaultPrevented) {
        document.removeEventListener('click', Click.handle, false);
        return document.addEventListener('click', Click.handle, false);
      }
    };

    Click.handle = function(event) {
      return new Click(event);
    };

    function Click(event1) {
      this.event = event1;
      if (this.event.defaultPrevented) {
        return;
      }
      this._extractLink();
      if (this._validForTurbolinks()) {
        if (!pageChangePrevented(this.link.absolute)) {
          visit(this.link.href);
        }
        this.event.preventDefault();
      }
    }

    Click.prototype._extractLink = function() {
      var link;
      link = this.event.target;
      while (!(!link.parentNode || link.nodeName === 'A')) {
        link = link.parentNode;
      }
      if (link.nodeName === 'A' && link.href.length !== 0) {
        return this.link = new Link(link);
      }
    };

    Click.prototype._validForTurbolinks = function() {
      return (this.link != null) && !(this.link.shouldIgnore() || this._nonStandardClick());
    };

    Click.prototype._nonStandardClick = function() {
      return this.event.which > 1 || this.event.metaKey || this.event.ctrlKey || this.event.shiftKey || this.event.altKey;
    };

    return Click;

  })();

  ProgressBar = (function() {
    var className;

    className = 'turbolinks-progress-bar';

    function ProgressBar(elementSelector) {
      this.elementSelector = elementSelector;
      this._trickle = bind(this._trickle, this);
      this.value = 0;
      this.content = '';
      this.speed = 300;
      this.opacity = 0.99;
      this.install();
    }

    ProgressBar.prototype.install = function() {
      this.element = document.querySelector(this.elementSelector);
      this.element.classList.add(className);
      this.styleElement = document.createElement('style');
      document.head.appendChild(this.styleElement);
      return this._updateStyle();
    };

    ProgressBar.prototype.uninstall = function() {
      this.element.classList.remove(className);
      return document.head.removeChild(this.styleElement);
    };

    ProgressBar.prototype.start = function() {
      return this.advanceTo(5);
    };

    ProgressBar.prototype.advanceTo = function(value) {
      var ref;
      if ((value > (ref = this.value) && ref <= 100)) {
        this.value = value;
        this._updateStyle();
        if (this.value === 100) {
          return this._stopTrickle();
        } else if (this.value > 0) {
          return this._startTrickle();
        }
      }
    };

    ProgressBar.prototype.done = function() {
      if (this.value > 0) {
        this.advanceTo(100);
        return this._reset();
      }
    };

    ProgressBar.prototype._reset = function() {
      var originalOpacity;
      originalOpacity = this.opacity;
      setTimeout((function(_this) {
        return function() {
          _this.opacity = 0;
          return _this._updateStyle();
        };
      })(this), this.speed / 2);
      return setTimeout((function(_this) {
        return function() {
          _this.value = 0;
          _this.opacity = originalOpacity;
          return _this._withSpeed(0, function() {
            return _this._updateStyle(true);
          });
        };
      })(this), this.speed);
    };

    ProgressBar.prototype._startTrickle = function() {
      if (this.trickling) {
        return;
      }
      this.trickling = true;
      return setTimeout(this._trickle, this.speed);
    };

    ProgressBar.prototype._stopTrickle = function() {
      return delete this.trickling;
    };

    ProgressBar.prototype._trickle = function() {
      if (!this.trickling) {
        return;
      }
      this.advanceTo(this.value + Math.random() / 2);
      return setTimeout(this._trickle, this.speed);
    };

    ProgressBar.prototype._withSpeed = function(speed, fn) {
      var originalSpeed, result;
      originalSpeed = this.speed;
      this.speed = speed;
      result = fn();
      this.speed = originalSpeed;
      return result;
    };

    ProgressBar.prototype._updateStyle = function(forceRepaint) {
      if (forceRepaint == null) {
        forceRepaint = false;
      }
      if (forceRepaint) {
        this._changeContentToForceRepaint();
      }
      return this.styleElement.textContent = this._createCSSRule();
    };

    ProgressBar.prototype._changeContentToForceRepaint = function() {
      return this.content = this.content === '' ? ' ' : '';
    };

    ProgressBar.prototype._createCSSRule = function() {
      return this.elementSelector + "." + className + "::before {\n  content: '" + this.content + "';\n  position: fixed;\n  top: 0;\n  left: 0;\n  z-index: 2000;\n  background-color: #0076ff;\n  height: 3px;\n  opacity: " + this.opacity + ";\n  width: " + this.value + "%;\n  transition: width " + this.speed + "ms ease-out, opacity " + (this.speed / 2) + "ms ease-in;\n  transform: translate3d(0,0,0);\n}";
    };

    return ProgressBar;

  })();

  bypassOnLoadPopstate = function(fn) {
    return setTimeout(fn, 500);
  };

  installDocumentReadyPageEventTriggers = function() {
    return document.addEventListener('DOMContentLoaded', (function() {
      triggerEvent(EVENTS.CHANGE);
      return triggerEvent(EVENTS.UPDATE);
    }), true);
  };

  installJqueryAjaxSuccessPageUpdateTrigger = function() {
    if (typeof jQuery !== 'undefined') {
      return jQuery(document).on('ajaxSuccess', function(event, xhr, settings) {
        if (!jQuery.trim(xhr.responseText)) {
          return;
        }
        return triggerEvent(EVENTS.UPDATE);
      });
    }
  };

  installHistoryChangeHandler = function(event) {
    var cachedPage, ref;
    if ((ref = event.state) != null ? ref.turbolinks : void 0) {
      if (cachedPage = pageCache[(new ComponentUrl(event.state.url)).absolute]) {
        cacheCurrentPage();
        return fetchHistory(cachedPage);
      } else {
        return visit(event.target.location.href);
      }
    }
  };

  initializeTurbolinks = function() {
    rememberCurrentUrl();
    rememberCurrentState();
    document.addEventListener('click', Click.installHandlerLast, true);
    window.addEventListener('hashchange', function(event) {
      rememberCurrentUrl();
      return rememberCurrentState();
    }, false);
    return bypassOnLoadPopstate(function() {
      return window.addEventListener('popstate', installHistoryChangeHandler, false);
    });
  };

  historyStateIsDefined = window.history.state !== void 0 || navigator.userAgent.match(/Firefox\/2[6|7]/);

  browserSupportsPushState = window.history && window.history.pushState && window.history.replaceState && historyStateIsDefined;

  browserIsntBuggy = !navigator.userAgent.match(/CriOS\//);

  requestMethodIsSafe = (ref = popCookie('request_method')) === 'GET' || ref === '';

  browserSupportsTurbolinks = browserSupportsPushState && browserIsntBuggy && requestMethodIsSafe;

  browserSupportsCustomEvents = document.addEventListener && document.createEvent;

  if (browserSupportsCustomEvents) {
    installDocumentReadyPageEventTriggers();
    installJqueryAjaxSuccessPageUpdateTrigger();
  }

  if (browserSupportsTurbolinks) {
    visit = fetch;
    initializeTurbolinks();
  } else {
    visit = function(url) {
      return document.location.href = url;
    };
  }

  this.Turbolinks = {
    visit: visit,
    pagesCached: pagesCached,
    enableTransitionCache: enableTransitionCache,
    enableProgressBar: enableProgressBar,
    allowLinkExtensions: Link.allowExtensions,
    supported: browserSupportsTurbolinks,
    EVENTS: clone(EVENTS)
  };

}).call(this);
(function(undefined) {
  if (typeof(this.Opal) !== 'undefined') {
    console.warn('Opal already loaded. Loading twice can cause troubles, please fix your setup.');
    return this.Opal;
  }

  // The Opal object that is exposed globally
  var Opal = this.Opal = {};

  // All bridged classes - keep track to donate methods from Object
  var bridged_classes = Opal.bridged_classes = [];

  // TopScope is used for inheriting constants from the top scope
  var TopScope = function(){};

  // Opal just acts as the top scope
  TopScope.prototype = Opal;

  // To inherit scopes
  Opal.constructor = TopScope;

  // List top scope constants
  Opal.constants = [];

  // This is a useful reference to global object inside ruby files
  Opal.global = this;

  // Minify common function calls
  var $hasOwn = Opal.hasOwnProperty;
  var $slice  = Opal.slice = Array.prototype.slice;

  // Nil object id is always 4
  var nil_id = 4;

  // Generates even sequential numbers greater than 4
  // (nil_id) to serve as unique ids for ruby objects
  var unique_id = nil_id;

  // Return next unique id
  Opal.uid = function() {
    unique_id += 2;
    return unique_id;
  };

  // Table holds all class variables
  Opal.cvars = {};

  // Globals table
  Opal.gvars = {};

  // Exit function, this should be replaced by platform specific implementation
  // (See nodejs and phantom for examples)
  Opal.exit = function(status) { if (Opal.gvars.DEBUG) console.log('Exited with status '+status); };

  /**
    Get a constant on the given scope. Every class and module in Opal has a
    scope used to store, and inherit, constants. For example, the top level
    `Object` in ruby has a scope accessible as `Opal.Object.$$scope`.

    To get the `Array` class using this scope, you could use:

        Opal.Object.$$scope.get("Array")

    If a constant with the given name cannot be found, then a dispatch to the
    class/module's `#const_method` is called, which by default will raise an
    error.

    @param [String] name the name of the constant to lookup
    @returns [RubyObject]
  */
  Opal.get = function(name) {
    var constant = this[name];

    if (constant == null) {
      return this.base.$const_missing(name);
    }

    return constant;
  };

  /*
   * Create a new constants scope for the given class with the given
   * base. Constants are looked up through their parents, so the base
   * scope will be the outer scope of the new klass.
   */
  function create_scope(base, klass, id) {
    var const_alloc = function() {};
    var const_scope = const_alloc.prototype = new base.constructor();

    klass.$$scope       = const_scope;
    klass.$$base_module = base.base;

    const_scope.base        = klass;
    const_scope.constructor = const_alloc;
    const_scope.constants   = [];

    if (id) {
      klass.$$orig_scope = base;
      base[id] = base.constructor[id] = klass;
      base.constants.push(id);
    }
  }

  Opal.create_scope = create_scope;

  /*
   * A `class Foo; end` expression in ruby is compiled to call this runtime
   * method which either returns an existing class of the given name, or creates
   * a new class in the given `base` scope.
   *
   * If a constant with the given name exists, then we check to make sure that
   * it is a class and also that the superclasses match. If either of these
   * fail, then we raise a `TypeError`. Note, superklass may be null if one was
   * not specified in the ruby code.
   *
   * We pass a constructor to this method of the form `function ClassName() {}`
   * simply so that classes show up with nicely formatted names inside debuggers
   * in the web browser (or node/sprockets).
   *
   * The `base` is the current `self` value where the class is being created
   * from. We use this to get the scope for where the class should be created.
   * If `base` is an object (not a class/module), we simple get its class and
   * use that as the base instead.
   *
   * @param [Object] base where the class is being created
   * @param [Class] superklass superclass of the new class (may be null)
   * @param [String] id the name of the class to be created
   * @param [Function] constructor function to use as constructor
   * @return [Class] new or existing ruby class
   */
  Opal.klass = function(base, superklass, id, constructor) {
    // If base is an object, use its class
    if (!base.$$is_class) {
      base = base.$$class;
    }

    // Not specifying a superclass means we can assume it to be Object
    if (superklass === null) {
      superklass = ObjectClass;
    }

    var klass = base.$$scope[id];

    // If a constant exists in the scope, then we must use that
    if ($hasOwn.call(base.$$scope, id) && klass.$$orig_scope === base.$$scope) {
      // Make sure the existing constant is a class, or raise error
      if (!klass.$$is_class) {
        throw Opal.TypeError.$new(id + " is not a class");
      }

      // Make sure existing class has same superclass
      if (superklass !== klass.$$super && superklass !== ObjectClass) {
        throw Opal.TypeError.$new("superclass mismatch for class " + id);
      }
    }
    else if (typeof(superklass) === 'function') {
      // passed native constructor as superklass, so bridge it as ruby class
      return bridge_class(id, superklass, base);
    }
    else {
      // if class doesnt exist, create a new one with given superclass
      klass = boot_class(superklass, constructor);

      // name class using base (e.g. Foo or Foo::Baz)
      klass.$$name = id;

      // every class gets its own constant scope, inherited from current scope
      create_scope(base.$$scope, klass, id);

      // Name new class directly onto current scope (Opal.Foo.Baz = klass)
      base[id] = base.$$scope[id] = klass;

      // Copy all parent constants to child, unless parent is Object
      if (superklass !== ObjectClass && superklass !== BasicObjectClass) {
        donate_constants(superklass, klass);
      }

      // call .inherited() hook with new class on the superclass
      if (superklass.$inherited) {
        superklass.$inherited(klass);
      }
    }

    return klass;
  };

  // Create generic class with given superclass.
  function boot_class(superklass, constructor) {
    var alloc = boot_class_alloc(null, constructor, superklass)

    return boot_class_object(superklass, alloc);
  }

  // Make `boot_class` available to the JS-API
  Opal.boot = boot_class;

  /*
   * The class object itself (as in `Class.new`)
   *
   * @param [(Opal) Class] superklass Another class object (as in `Class.new`)
   * @param [constructor]  alloc      The constructor that holds the prototype
   *                                  that will be used for instances of the
   *                                  newly constructed class.
   */
  function boot_class_object(superklass, alloc) {
    var singleton_class = function() {};
    singleton_class.prototype = superklass.constructor.prototype;

    function OpalClass() {}
    OpalClass.prototype = new singleton_class();

    var klass = new OpalClass();

    setup_module_or_class_object(klass, OpalClass, superklass, alloc.prototype);

    // @property $$alloc This is the constructor of instances of the current
    //                   class. Its prototype will be used for method lookup
    klass.$$alloc = alloc;

    // @property $$proto.$$class Make available to instances a reference to the
    //                           class they belong to.
    klass.$$proto.$$class = klass;

    return klass;
  }

  /*
   * Adds common/required properties to a module or class object
   * (as in `Module.new` / `Class.new`)
   *
   * @param module      The module or class that needs to be prepared
   *
   * @param constructor The constructor of the module or class itself,
   *                    usually it's already assigned by using `new`. Some
   *                    ipothesis on why it's needed can be found below.
   *
   * @param superklass  The superclass of the class/module object, for modules
   *                    is `Module` (of `ModuleClass` in JS context)
   *
   * @param prototype   The prototype on which the class/module methods will
   *                    be stored.
   */
  function setup_module_or_class_object(module, constructor, superklass, prototype) {
    // @property $$id Each class is assigned a unique `id` that helps
    //                comparation and implementation of `#object_id`
    module.$$id = Opal.uid();

    // @property $$proto This is the prototype on which methods will be defined
    module.$$proto = prototype;

    // @property constructor keeps a ref to the constructor, but apparently the
    //                       constructor is already set on:
    //
    //                          `var module = new constructor` is called.
    //
    //                       Maybe there are some browsers not abiding (IE6?)
    module.constructor = constructor;

    // @property $$is_class Clearly mark this as a class-like
    module.$$is_class = true;

    // @property $$super the superclass, doesn't get changed by module inclusions
    module.$$super = superklass;

    // @property $$parent direct parent class or module
    //                    starts with the superclass, after module inclusion is
    //                    the last included module
    module.$$parent = superklass;

    // @property $$methods keeps track of methods defined on the class
    //                     but seems to be used just by `define_basic_object_method`
    //                     and for donating (Ruby) Object methods to bridged classes
    //                     TODO: check if it can be removed
    module.$$methods = [];

    // @property $$inc included modules
    module.$$inc = [];
  }

  /**
    Define new module (or return existing module). The given `base` is basically
    the current `self` value the `module` statement was defined in. If this is
    a ruby module or class, then it is used, otherwise if the base is a ruby
    object then that objects real ruby class is used (e.g. if the base is the
    main object, then the top level `Object` class is used as the base).

    If a module of the given name is already defined in the base, then that
    instance is just returned.

    If there is a class of the given name in the base, then an error is
    generated instead (cannot have a class and module of same name in same base).

    Otherwise, a new module is created in the base with the given name, and that
    new instance is returned back (to be referenced at runtime).

    @param [RubyModule or Class] base class or module this definition is inside
    @param [String] id the name of the new (or existing) module
    @returns [RubyModule]
  */
  Opal.module = function(base, id) {
    var module;

    if (!base.$$is_class) {
      base = base.$$class;
    }

    if ($hasOwn.call(base.$$scope, id)) {
      module = base.$$scope[id];

      if (!module.$$is_mod && module !== ObjectClass) {
        throw Opal.TypeError.$new(id + " is not a module");
      }
    }
    else {
      module = boot_module_object();
      module.$$name = id;

      create_scope(base.$$scope, module, id);

      // Name new module directly onto current scope (Opal.Foo.Baz = module)
      base[id] = base.$$scope[id] = module;
    }

    return module;
  };

  /*
   * Internal function to create a new module instance. This simply sets up
   * the prototype hierarchy and method tables.
   */
  function boot_module_object() {
    var mtor = function() {};
    mtor.prototype = ModuleClass.constructor.prototype;

    function module_constructor() {}
    module_constructor.prototype = new mtor();

    var module = new module_constructor();
    var module_prototype = {};

    setup_module_or_class_object(module, module_constructor, ModuleClass, module_prototype);

    module.$$is_mod = true;
    module.$$dep    = [];

    return module;
  }

  /**
    Return the singleton class for the passed object.

    If the given object alredy has a singleton class, then it will be stored on
    the object as the `$$meta` property. If this exists, then it is simply
    returned back.

    Otherwise, a new singleton object for the class or object is created, set on
    the object at `$$meta` for future use, and then returned.

    @param [RubyObject] object the ruby object
    @returns [RubyClass] the singleton class for object
  */
  Opal.get_singleton_class = function(object) {
    if (object.$$meta) {
      return object.$$meta;
    }

    if (object.$$is_class) {
      return build_class_singleton_class(object);
    }

    return build_object_singleton_class(object);
  };

  /**
    Build the singleton class for an existing class.

    NOTE: Actually in MRI a class' singleton class inherits from its
    superclass' singleton class which in turn inherits from Class.

    @param [RubyClass] klass
    @returns [RubyClass]
   */
  function build_class_singleton_class(klass) {
    var meta = new Opal.Class.$$alloc;

    meta.$$class = Opal.Class;
    meta.$$proto = klass.constructor.prototype;

    meta.$$is_singleton = true;
    meta.$$inc          = [];
    meta.$$methods      = [];
    meta.$$scope        = klass.$$scope;

    return klass.$$meta = meta;
  }

  /**
    Build the singleton class for a Ruby (non class) Object.

    @param [RubyObject] object
    @returns [RubyClass]
   */
  function build_object_singleton_class(object) {
    var orig_class = object.$$class,
        class_id   = "#<Class:#<" + orig_class.$$name + ":" + orig_class.$$id + ">>";

    var Singleton = function () {};
    var meta = Opal.boot(orig_class, Singleton);
    meta.$$name   = class_id;

    meta.$$proto  = object;
    meta.$$class  = orig_class.$$class;
    meta.$$scope  = orig_class.$$scope;
    meta.$$parent = orig_class;
    return object.$$meta = meta;
  }

  /**
    The actual inclusion of a module into a class.

    ## Class `$$parent` and `iclass`

    To handle `super` calls, every class has a `$$parent`. This parent is
    used to resolve the next class for a super call. A normal class would
    have this point to its superclass. However, if a class includes a module
    then this would need to take into account the module. The module would
    also have to then point its `$$parent` to the actual superclass. We
    cannot modify modules like this, because it might be included in more
    then one class. To fix this, we actually insert an `iclass` as the class'
    `$$parent` which can then point to the superclass. The `iclass` acts as
    a proxy to the actual module, so the `super` chain can then search it for
    the required method.

    @param [RubyModule] module the module to include
    @param [RubyClass] klass the target class to include module into
    @returns [null]
  */
  Opal.append_features = function(module, klass) {
    var included = klass.$$inc;

    // check if this module is already included in the klass
    for (var j = 0, jj = included.length; j < jj; j++) {
      if (included[j] === module) {
        return;
      }
    }

    included.push(module);
    module.$$dep.push(klass);

    // iclass
    var iclass = {
      $$name:   module.$$name,
      $$proto:  module.$$proto,
      $$parent: klass.$$parent,
      $$module: module,
      $$iclass: true
    };

    klass.$$parent = iclass;

    var donator   = module.$$proto,
        prototype = klass.$$proto,
        methods   = module.$$methods;

    for (var i = 0, length = methods.length; i < length; i++) {
      var method = methods[i], current;


      if ( prototype.hasOwnProperty(method) &&
          !(current = prototype[method]).$$donated && !current.$$stub ) {
        // if the target class already has a method of the same name defined
        // and that method was NOT donated, then it must be a method defined
        // by the class so we do not want to override it
      }
      else {
        prototype[method] = donator[method];
        prototype[method].$$donated = true;
      }
    }

    if (klass.$$dep) {
      donate_methods(klass, methods.slice(), true);
    }

    donate_constants(module, klass);
  };

  // Boot a base class (makes instances).
  function boot_class_alloc(id, constructor, superklass) {
    if (superklass) {
      var ctor = function() {};
          ctor.prototype   = superklass.$$proto || superklass.prototype;

      if (id) {
        ctor.displayName = id;
      }

      constructor.prototype = new ctor();
    }

    constructor.prototype.constructor = constructor;

    return constructor;
  }

  /*
   * Builds the class object for core classes:
   * - make the class object have a singleton class
   * - make the singleton class inherit from its parent singleton class
   *
   * @param id         [String]      the name of the class
   * @param alloc      [Function]    the constructor for the core class instances
   * @param superclass [Class alloc] the constructor of the superclass
   */
  function boot_core_class_object(id, alloc, superclass) {
    var superclass_constructor = function() {};
        superclass_constructor.prototype = superclass.prototype;

    var singleton_class = function() {};
        singleton_class.prototype = new superclass_constructor();

    singleton_class.displayName = "#<Class:"+id+">";

    // the singleton_class acts as the class object constructor
    var klass = new singleton_class();

    setup_module_or_class_object(klass, singleton_class, superclass, alloc.prototype);

    klass.$$alloc = alloc;
    klass.$$name  = id;

    // Give all instances a ref to their class
    alloc.prototype.$$class = klass;

    Opal[id] = klass;
    Opal.constants.push(id);

    return klass;
  }

  /*
   * For performance, some core ruby classes are toll-free bridged to their
   * native javascript counterparts (e.g. a ruby Array is a javascript Array).
   *
   * This method is used to setup a native constructor (e.g. Array), to have
   * its prototype act like a normal ruby class. Firstly, a new ruby class is
   * created using the native constructor so that its prototype is set as the
   * target for th new class. Note: all bridged classes are set to inherit
   * from Object.
   *
   * Bridged classes are tracked in `bridged_classes` array so that methods
   * defined on Object can be "donated" to all bridged classes. This allows
   * us to fake the inheritance of a native prototype from our Object
   * prototype.
   *
   * Example:
   *
   *    bridge_class("Proc", Function);
   *
   * @param [String] name the name of the ruby class to create
   * @param [Function] constructor native javascript constructor to use
   * @param [Object] base where the bridge class is being created. If none is supplied, the top level scope (Opal) will be used
   * @return [Class] returns new ruby class
   */
  function bridge_class(name, constructor, base) {
    var klass = boot_class_object(ObjectClass, constructor);

    klass.$$name = name;

    if (base === undefined) {
      base = Opal;
    }
    else {
      base = base.$$scope;
    }

    create_scope(base, klass, name);
    bridged_classes.push(klass);

    var object_methods = BasicObjectClass.$$methods.concat(ObjectClass.$$methods);

    for (var i = 0, len = object_methods.length; i < len; i++) {
      var meth = object_methods[i];
      constructor.prototype[meth] = ObjectClass.$$proto[meth];
    }

    add_stubs_subscriber(constructor.prototype);

    return klass;
  }

  /*
   * constant assign
   */
  Opal.casgn = function(base_module, name, value) {
    var scope = base_module.$$scope;

    if (value.$$is_class && value.$$name === nil) {
      value.$$name = name;
    }

    if (value.$$is_class) {
      value.$$base_module = base_module;
    }

    scope.constants.push(name);
    return scope[name] = value;
  };

  /*
   * constant decl
   */
  Opal.cdecl = function(base_scope, name, value) {
    base_scope.constants.push(name);
    return base_scope[name] = value;
  };

  /*
   * When a source module is included into the target module, we must also copy
   * its constants to the target.
   */
  function donate_constants(source_mod, target_mod) {
    var source_constants = source_mod.$$scope.constants,
        target_scope     = target_mod.$$scope,
        target_constants = target_scope.constants;

    for (var i = 0, length = source_constants.length; i < length; i++) {
      target_constants.push(source_constants[i]);
      target_scope[source_constants[i]] = source_mod.$$scope[source_constants[i]];
    }
  };

  /*
   * Methods stubs are used to facilitate method_missing in opal. A stub is a
   * placeholder function which just calls `method_missing` on the receiver.
   * If no method with the given name is actually defined on an object, then it
   * is obvious to say that the stub will be called instead, and then in turn
   * method_missing will be called.
   *
   * When a file in ruby gets compiled to javascript, it includes a call to
   * this function which adds stubs for every method name in the compiled file.
   * It should then be safe to assume that method_missing will work for any
   * method call detected.
   *
   * Method stubs are added to the BasicObject prototype, which every other
   * ruby object inherits, so all objects should handle method missing. A stub
   * is only added if the given property name (method name) is not already
   * defined.
   *
   * Note: all ruby methods have a `$` prefix in javascript, so all stubs will
   * have this prefix as well (to make this method more performant).
   *
   *    Opal.add_stubs(["$foo", "$bar", "$baz="]);
   *
   * All stub functions will have a private `$$stub` property set to true so
   * that other internal methods can detect if a method is just a stub or not.
   * `Kernel#respond_to?` uses this property to detect a methods presence.
   *
   * @param [Array] stubs an array of method stubs to add
   */
  Opal.add_stubs = function(stubs) {
    var subscribers = Opal.stub_subscribers;
    var subscriber;

    for (var i = 0, length = stubs.length; i < length; i++) {
      var method_name = stubs[i], stub = stub_for(method_name);

      for (var j = 0; j < subscribers.length; j++) {
        subscriber = subscribers[j];
        if (!(method_name in subscriber)) {
          subscriber[method_name] = stub;
        }
      }
    }
  };

  /*
   * Add a prototype to the subscribers list, and (TODO) add previously stubbed
   * methods.
   *
   * @param [Prototype]
   */
  function add_stubs_subscriber(prototype) {
    // TODO: Add previously stubbed methods too.
    Opal.stub_subscribers.push(prototype);
  }

  /*
   * Keep a list of prototypes that want method_missing stubs to be added.
   *
   * @default [Prototype List] BasicObject.prototype
   */
  Opal.stub_subscribers = [BasicObject.prototype];

  /*
   * Add a method_missing stub function to the given prototype for the
   * given name.
   *
   * @param [Prototype] prototype the target prototype
   * @param [String] stub stub name to add (e.g. "$foo")
   */
  function add_stub_for(prototype, stub) {
    var method_missing_stub = stub_for(stub);
    prototype[stub] = method_missing_stub;
  }

  /*
   * Generate the method_missing stub for a given method name.
   *
   * @param [String] method_name The js-name of the method to stub (e.g. "$foo")
   */
  function stub_for(method_name) {
    function method_missing_stub() {
      // Copy any given block onto the method_missing dispatcher
      this.$method_missing.$$p = method_missing_stub.$$p;

      // Set block property to null ready for the next call (stop false-positives)
      method_missing_stub.$$p = null;

      // call method missing with correct args (remove '$' prefix on method name)
      return this.$method_missing.apply(this, [method_name.slice(1)].concat($slice.call(arguments)));
    }

    method_missing_stub.$$stub = true;

    return method_missing_stub;
  }

  // Expose for other parts of Opal to use
  Opal.add_stub_for = add_stub_for;

  // Arity count error dispatcher
  Opal.ac = function(actual, expected, object, meth) {
    var inspect = (object.$$is_class ? object.$$name + '.' : object.$$class.$$name + '#') + meth;
    var msg = '[' + inspect + '] wrong number of arguments(' + actual + ' for ' + expected + ')';
    throw Opal.ArgumentError.$new(msg);
  };

  // Super dispatcher
  Opal.find_super_dispatcher = function(obj, jsid, current_func, iter, defs) {
    var dispatcher;

    if (defs) {
      dispatcher = obj.$$is_class ? defs.$$super : obj.$$class.$$proto;
    }
    else {
      if (obj.$$is_class) {
        dispatcher = obj.$$super;
      }
      else {
        dispatcher = find_obj_super_dispatcher(obj, jsid, current_func);
      }
    }

    dispatcher = dispatcher['$' + jsid];
    dispatcher.$$p = iter;

    return dispatcher;
  };

  // Iter dispatcher for super in a block
  Opal.find_iter_super_dispatcher = function(obj, jsid, current_func, iter, defs) {
    if (current_func.$$def) {
      return Opal.find_super_dispatcher(obj, current_func.$$jsid, current_func, iter, defs);
    }
    else {
      return Opal.find_super_dispatcher(obj, jsid, current_func, iter, defs);
    }
  };

  function find_obj_super_dispatcher(obj, jsid, current_func) {
    var klass = obj.$$meta || obj.$$class;
    jsid = '$' + jsid;

    while (klass) {
      if (klass.$$proto[jsid] === current_func) {
        // ok
        break;
      }

      klass = klass.$$parent;
    }

    // if we arent in a class, we couldnt find current?
    if (!klass) {
      throw new Error("could not find current class for super()");
    }

    klass = klass.$$parent;

    // else, let's find the next one
    while (klass) {
      var working = klass.$$proto[jsid];

      if (working && working !== current_func) {
        // ok
        break;
      }

      klass = klass.$$parent;
    }

    return klass.$$proto;
  };

  /*
   * Used to return as an expression. Sometimes, we can't simply return from
   * a javascript function as if we were a method, as the return is used as
   * an expression, or even inside a block which must "return" to the outer
   * method. This helper simply throws an error which is then caught by the
   * method. This approach is expensive, so it is only used when absolutely
   * needed.
   */
  Opal.ret = function(val) {
    Opal.returner.$v = val;
    throw Opal.returner;
  };

  // handles yield calls for 1 yielded arg
  Opal.yield1 = function(block, arg) {
    if (typeof(block) !== "function") {
      throw Opal.LocalJumpError.$new("no block given");
    }

    if (block.length > 1 && arg.$$is_array) {
      return block.apply(null, arg);
    }
    else {
      return block(arg);
    }
  };

  // handles yield for > 1 yielded arg
  Opal.yieldX = function(block, args) {
    if (typeof(block) !== "function") {
      throw Opal.LocalJumpError.$new("no block given");
    }

    if (block.length > 1 && args.length == 1) {
      if (args[0].$$is_array) {
        return block.apply(null, args[0]);
      }
    }

    if (!args.$$is_array) {
      args = $slice.call(args);
    }

    return block.apply(null, args);
  };

  // Finds the corresponding exception match in candidates.  Each candidate can
  // be a value, or an array of values.  Returns null if not found.
  Opal.rescue = function(exception, candidates) {
    for (var i = 0; i < candidates.length; i++) {
      var candidate = candidates[i];

      if (candidate.$$is_array) {
        var result = Opal.rescue(exception, candidate);

        if (result) {
          return result;
        }
      }
      else if (candidate['$==='](exception)) {
        return candidate;
      }
    }

    return null;
  };

  Opal.is_a = function(object, klass) {
    if (object.$$meta === klass) {
      return true;
    }

    var search = object.$$class;

    while (search) {
      if (search === klass) {
        return true;
      }

      for (var i = 0, length = search.$$inc.length; i < length; i++) {
        if (search.$$inc[i] == klass) {
          return true;
        }
      }

      search = search.$$super;
    }

    return false;
  };

  // Helper to convert the given object to an array
  Opal.to_ary = function(value) {
    if (value.$$is_array) {
      return value;
    }
    else if (value.$to_ary && !value.$to_ary.$$stub) {
      return value.$to_ary();
    }

    return [value];
  };

  /**
    Used to get a list of rest keyword arguments. Method takes the given
    keyword args, i.e. the hash literal passed to the method containing all
    keyword arguemnts passed to method, as well as the used args which are
    the names of required and optional arguments defined. This method then
    just returns all key/value pairs which have not been used, in a new
    hash literal.

    @param given_args [Hash] all kwargs given to method
    @param used_args [Object<String: true>] all keys used as named kwargs
    @return [Hash]
   */
  Opal.kwrestargs = function(given_args, used_args) {
    var keys      = [],
        map       = {},
        key       = null,
        given_map = given_args.smap;

    for (key in given_map) {
      if (!used_args[key]) {
        keys.push(key);
        map[key] = given_map[key];
      }
    }

    return Opal.hash2(keys, map);
  };

  /*
   * Call a ruby method on a ruby object with some arguments:
   *
   *   var my_array = [1, 2, 3, 4]
   *   Opal.send(my_array, 'length')     # => 4
   *   Opal.send(my_array, 'reverse!')   # => [4, 3, 2, 1]
   *
   * A missing method will be forwarded to the object via
   * method_missing.
   *
   * The result of either call with be returned.
   *
   * @param [Object] recv the ruby object
   * @param [String] mid ruby method to call
   */
  Opal.send = function(recv, mid) {
    var args = $slice.call(arguments, 2),
        func = recv['$' + mid];

    if (func) {
      return func.apply(recv, args);
    }

    return recv.$method_missing.apply(recv, [mid].concat(args));
  };

  Opal.block_send = function(recv, mid, block) {
    var args = $slice.call(arguments, 3),
        func = recv['$' + mid];

    if (func) {
      func.$$p = block;
      return func.apply(recv, args);
    }

    return recv.$method_missing.apply(recv, [mid].concat(args));
  };

  /*
   * Donate methods for a class/module
   */
  function donate_methods(klass, defined, indirect) {
    var methods = klass.$$methods, included_in = klass.$$dep;

    // if (!indirect) {
      klass.$$methods = methods.concat(defined);
    // }

    if (included_in) {
      for (var i = 0, length = included_in.length; i < length; i++) {
        var includee = included_in[i];
        var dest     = includee.$$proto;

        for (var j = 0, jj = defined.length; j < jj; j++) {
          var method = defined[j];

          dest[method] = klass.$$proto[method];
          dest[method].$$donated = true;
        }

        if (includee.$$dep) {
          donate_methods(includee, defined, true);
        }
      }
    }
  };

  /**
    Define the given method on the module.

    This also handles donating methods to all classes that include this
    module. Method conflicts are also handled here, where a class might already
    have defined a method of the same name, or another included module defined
    the same method.

    @param [RubyModule] module the module method defined on
    @param [String] jsid javascript friendly method name (e.g. "$foo")
    @param [Function] body method body of actual function
  */
  function define_module_method(module, jsid, body) {
    module.$$proto[jsid] = body;
    body.$$owner = module;

    module.$$methods.push(jsid);

    if (module.$$module_function) {
      module[jsid] = body;
    }

    var included_in = module.$$dep;

    if (included_in) {
      for (var i = 0, length = included_in.length; i < length; i++) {
        var includee = included_in[i];
        var dest = includee.$$proto;
        var current = dest[jsid];


        if (dest.hasOwnProperty(jsid) && !current.$$donated && !current.$$stub) {
          // target class has already defined the same method name - do nothing
        }
        else if (dest.hasOwnProperty(jsid) && !current.$$stub) {
          // target class includes another module that has defined this method
          var klass_includees = includee.$$inc;

          for (var j = 0, jj = klass_includees.length; j < jj; j++) {
            if (klass_includees[j] === current.$$owner) {
              var current_owner_index = j;
            }
            if (klass_includees[j] === module) {
              var module_index = j;
            }
          }

          // only redefine method on class if the module was included AFTER
          // the module which defined the current method body. Also make sure
          // a module can overwrite a method it defined before
          if (current_owner_index <= module_index) {
            dest[jsid] = body;
            dest[jsid].$$donated = true;
          }
        }
        else {
          // neither a class, or module included by class, has defined method
          dest[jsid] = body;
          dest[jsid].$$donated = true;
        }

        if (includee.$$dep) {
          donate_methods(includee, [jsid], true);
        }
      }
    }
  }

  /**
    Used to define methods on an object. This is a helper method, used by the
    compiled source to define methods on special case objects when the compiler
    can not determine the destination object, or the object is a Module
    instance. This can get called by `Module#define_method` as well.

    ## Modules

    Any method defined on a module will come through this runtime helper.
    The method is added to the module body, and the owner of the method is
    set to be the module itself. This is used later when choosing which
    method should show on a class if more than 1 included modules define
    the same method. Finally, if the module is in `module_function` mode,
    then the method is also defined onto the module itself.

    ## Classes

    This helper will only be called for classes when a method is being
    defined indirectly; either through `Module#define_method`, or by a
    literal `def` method inside an `instance_eval` or `class_eval` body. In
    either case, the method is simply added to the class' prototype. A special
    exception exists for `BasicObject` and `Object`. These two classes are
    special because they are used in toll-free bridged classes. In each of
    these two cases, extra work is required to define the methods on toll-free
    bridged class' prototypes as well.

    ## Objects

    If a simple ruby object is the object, then the method is simply just
    defined on the object as a singleton method. This would be the case when
    a method is defined inside an `instance_eval` block.

    @param [RubyObject or Class] obj the actual obj to define method for
    @param [String] jsid the javascript friendly method name (e.g. '$foo')
    @param [Function] body the literal javascript function used as method
    @returns [null]
  */
  Opal.defn = function(obj, jsid, body) {
    if (obj.$$is_mod) {
      define_module_method(obj, jsid, body);
    }
    else if (obj.$$is_class) {
      obj.$$proto[jsid] = body;

      if (obj === BasicObjectClass) {
        define_basic_object_method(jsid, body);
      }
      else if (obj === ObjectClass) {
        donate_methods(obj, [jsid]);
      }
    }
    else {
      obj[jsid] = body;
    }

    return nil;
  };

  /*
   * Define a singleton method on the given object.
   */
  Opal.defs = function(obj, jsid, body) {
    if (obj.$$is_class || obj.$$is_mod) {
      obj.constructor.prototype[jsid] = body;
    }
    else {
      obj[jsid] = body;
    }
  };

  function define_basic_object_method(jsid, body) {
    BasicObjectClass.$$methods.push(jsid);
    for (var i = 0, len = bridged_classes.length; i < len; i++) {
      bridged_classes[i].$$proto[jsid] = body;
    }
  }

  /*
   * Called to remove a method.
   */
  Opal.undef = function(obj, jsid) {
    delete obj.$$proto[jsid];
  };

  Opal.hash = function() {
    if (arguments.length == 1 && arguments[0].$$class == Opal.Hash) {
      return arguments[0];
    }

    var hash = new Opal.Hash.$$alloc(),
        keys = [],
        _map = {},
        smap = {},
        key, obj, length, khash, map;

    hash.map   = _map;
    hash.smap  = smap;
    hash.keys  = keys;

    if (arguments.length == 1) {
      if (arguments[0].$$is_array) {
        var args = arguments[0];

        for (var i = 0, ii = args.length; i < ii; i++) {
          var pair = args[i];

          if (pair.length !== 2) {
            throw Opal.ArgumentError.$new("value not of length 2: " + pair.$inspect());
          }

          key = pair[0];
          obj = pair[1];

          if (key.$$is_string) {
            khash = key;
            map = smap;
          } else {
            khash = key.$hash();
            map = _map;
          }

          if (map[khash] == null) {
            keys.push(key);
          }

          map[khash] = obj;
        }
      }
      else {
        obj = arguments[0];
        for (key in obj) {
          khash = key.$hash();
          smap[khash] = obj[khash];
          keys.push(key);
        }
      }
    }
    else {
      length = arguments.length;
      if (length % 2 !== 0) {
        throw Opal.ArgumentError.$new("odd number of arguments for Hash");
      }

      for (var j = 0; j < length; j++) {
        key = arguments[j];
        obj = arguments[++j];

        if (key.$$is_string) {
          khash = key;
          map = smap;
        } else {
          khash = key.$hash();
          map = _map;
        }

        if (map[khash] == null) {
          keys.push(key);
        }

        map[khash] = obj;
      }
    }

    return hash;
  };

  /*
   * hash2 is a faster creator for hashes that just use symbols and
   * strings as keys. The map and keys array can be constructed at
   * compile time, so they are just added here by the constructor
   * function
   */
  Opal.hash2 = function(keys, map) {
    var hash = new Opal.Hash.$$alloc();

    hash.keys = keys;
    hash.map  = {};
    hash.smap = map;

    return hash;
  };

  /*
   * Create a new range instance with first and last values, and whether the
   * range excludes the last value.
   */
  Opal.range = function(first, last, exc) {
    var range         = new Opal.Range.$$alloc();
        range.begin   = first;
        range.end     = last;
        range.exclude = exc;

    return range;
  };

  // Require system
  // --------------
  (function(Opal) {
    var loaded_features = ['corelib/runtime'],
        require_table   = {'corelib/runtime': true},
        modules         = {};

    var current_dir  = '.';

    function mark_as_loaded(filename) {
      if (require_table[filename]) {
        return false;
      }

      loaded_features.push(filename);
      require_table[filename] = true;

      return true;
    }

    function normalize_loadable_path(path) {
      var parts, part, new_parts = [], SEPARATOR = '/';

      if (current_dir !== '.') {
        path = current_dir.replace(/\/*$/, '/') + path;
      }

      path = path.replace(/\.(rb|opal|js)$/, '');
      parts = path.split(SEPARATOR);

      for (var i = 0, ii = parts.length; i < ii; i++) {
        part = parts[i];
        if (part == '') continue;
        (part === '..') ? new_parts.pop() : new_parts.push(part)
      }

      return new_parts.join(SEPARATOR);
    }

    function load(path) {
      mark_as_loaded(path);

      var module = modules[path];

      if (module) {
        module(Opal);
      }
      else {
        var severity = Opal.dynamic_require_severity || 'warning';
        var message  = 'cannot load such file -- ' + path;

        if (severity === "error") {
          Opal.LoadError ? Opal.LoadError.$new(message) : function(){throw message}();
        }
        else if (severity === "warning") {
          console.warn('WARNING: LoadError: ' + message);
        }
      }

      return true;
    }

    function require(path) {
      if (require_table[path]) {
        return false;
      }

      return load(path);
    }

    Opal.modules         = modules;
    Opal.loaded_features = loaded_features;

    Opal.normalize_loadable_path = normalize_loadable_path;
    Opal.mark_as_loaded          = mark_as_loaded;

    Opal.load    = load;
    Opal.require = require;
  })(Opal);

  // Initialization
  // --------------

  // The actual class for BasicObject
  var BasicObjectClass;

  // The actual Object class
  var ObjectClass;

  // The actual Module class
  var ModuleClass;

  // The actual Class class
  var ClassClass;

  // Constructor for instances of BasicObject
  function BasicObject(){}

  // Constructor for instances of Object
  function Object(){}

  // Constructor for instances of Class
  function Class(){}

  // Constructor for instances of Module
  function Module(){}

  // Constructor for instances of NilClass (nil)
  function NilClass(){}

  // Constructors for *instances* of core objects
  boot_class_alloc('BasicObject', BasicObject);
  boot_class_alloc('Object',      Object,       BasicObject);
  boot_class_alloc('Module',      Module,       Object);
  boot_class_alloc('Class',       Class,        Module);

  // Constructors for *classes* of core objects
  BasicObjectClass = boot_core_class_object('BasicObject', BasicObject, Class);
  ObjectClass      = boot_core_class_object('Object',      Object,      BasicObjectClass.constructor);
  ModuleClass      = boot_core_class_object('Module',      Module,      ObjectClass.constructor);
  ClassClass       = boot_core_class_object('Class',       Class,       ModuleClass.constructor);

  // Fix booted classes to use their metaclass
  BasicObjectClass.$$class = ClassClass;
  ObjectClass.$$class      = ClassClass;
  ModuleClass.$$class      = ClassClass;
  ClassClass.$$class       = ClassClass;

  // Fix superclasses of booted classes
  BasicObjectClass.$$super = null;
  ObjectClass.$$super      = BasicObjectClass;
  ModuleClass.$$super      = ObjectClass;
  ClassClass.$$super       = ModuleClass;

  BasicObjectClass.$$parent = null;
  ObjectClass.$$parent      = BasicObjectClass;
  ModuleClass.$$parent      = ObjectClass;
  ClassClass.$$parent       = ModuleClass;

  // Internally, Object acts like a module as it is "included" into bridged
  // classes. In other words, we donate methods from Object into our bridged
  // classes as their prototypes don't inherit from our root Object, so they
  // act like module includes.
  ObjectClass.$$dep = bridged_classes;

  Opal.base                     = ObjectClass;
  BasicObjectClass.$$scope      = ObjectClass.$$scope = Opal;
  BasicObjectClass.$$orig_scope = ObjectClass.$$orig_scope = Opal;
  Opal.Kernel                   = ObjectClass;

  ModuleClass.$$scope      = ObjectClass.$$scope;
  ModuleClass.$$orig_scope = ObjectClass.$$orig_scope;
  ClassClass.$$scope       = ObjectClass.$$scope;
  ClassClass.$$orig_scope  = ObjectClass.$$orig_scope;

  ObjectClass.$$proto.toString = function() {
    return this.$to_s();
  };

  ObjectClass.$$proto.$require = Opal.require;

  Opal.top = new ObjectClass.$$alloc();

  // Nil
  Opal.klass(ObjectClass, ObjectClass, 'NilClass', NilClass);
  var nil = Opal.nil = new NilClass();
  nil.$$id = nil_id;
  nil.call = nil.apply = function() { throw Opal.LocalJumpError.$new('no block given'); };

  Opal.breaker  = new Error('unexpected break');
  Opal.returner = new Error('unexpected return');

  bridge_class('Array',     Array);
  bridge_class('Boolean',   Boolean);
  bridge_class('Numeric',   Number);
  bridge_class('String',    String);
  bridge_class('Proc',      Function);
  bridge_class('Exception', Error);
  bridge_class('Regexp',    RegExp);
  bridge_class('Time',      Date);

  TypeError.$$super = Error;
}).call(this);

if (typeof(global) !== 'undefined') {
  global.Opal = this.Opal;
  Opal.global = global;
}
if (typeof(window) !== 'undefined') {
  window.Opal = this.Opal;
  Opal.global = window;
}
;
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/helpers"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module;

  Opal.add_stubs(['$new', '$class', '$===', '$respond_to?', '$raise', '$type_error', '$__send__', '$coerce_to', '$nil?', '$<=>', '$inspect']);
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    Opal.defs(self, '$type_error', function(object, type, method, coerced) {
      var $a, $b, self = this;

      if (method == null) {
        method = nil
      }
      if (coerced == null) {
        coerced = nil
      }
      if ((($a = (($b = method !== false && method !== nil) ? coerced : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return $scope.get('TypeError').$new("can't convert " + (object.$class()) + " into " + (type) + " (" + (object.$class()) + "#" + (method) + " gives " + (coerced.$class()))
        } else {
        return $scope.get('TypeError').$new("no implicit conversion of " + (object.$class()) + " into " + (type))
      };
    });

    Opal.defs(self, '$coerce_to', function(object, type, method) {
      var $a, self = this;

      if ((($a = type['$==='](object)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return object};
      if ((($a = object['$respond_to?'](method)) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise(self.$type_error(object, type))
      };
      return object.$__send__(method);
    });

    Opal.defs(self, '$coerce_to!', function(object, type, method) {
      var $a, self = this, coerced = nil;

      coerced = self.$coerce_to(object, type, method);
      if ((($a = type['$==='](coerced)) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise(self.$type_error(object, type, method, coerced))
      };
      return coerced;
    });

    Opal.defs(self, '$coerce_to?', function(object, type, method) {
      var $a, self = this, coerced = nil;

      if ((($a = object['$respond_to?'](method)) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        return nil
      };
      coerced = self.$coerce_to(object, type, method);
      if ((($a = coerced['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil};
      if ((($a = type['$==='](coerced)) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise(self.$type_error(object, type, method, coerced))
      };
      return coerced;
    });

    Opal.defs(self, '$try_convert', function(object, type, method) {
      var $a, self = this;

      if ((($a = type['$==='](object)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return object};
      if ((($a = object['$respond_to?'](method)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return object.$__send__(method)
        } else {
        return nil
      };
    });

    Opal.defs(self, '$compare', function(a, b) {
      var $a, self = this, compare = nil;

      compare = a['$<=>'](b);
      if ((($a = compare === nil) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$raise($scope.get('ArgumentError'), "comparison of " + (a.$class()) + " with " + (b.$class()) + " failed")};
      return compare;
    });

    Opal.defs(self, '$destructure', function(args) {
      var self = this;

      
      if (args.length == 1) {
        return args[0];
      }
      else if (args.$$is_array) {
        return args;
      }
      else {
        return $slice.call(args);
      }
    
    });

    Opal.defs(self, '$respond_to?', function(obj, method) {
      var self = this;

      
      if (obj == null || !obj.$$class) {
        return false;
      }
    
      return obj['$respond_to?'](method);
    });

    Opal.defs(self, '$inspect', function(obj) {
      var self = this;

      
      if (obj === undefined) {
        return "undefined";
      }
      else if (obj === null) {
        return "null";
      }
      else if (!obj.$$class) {
        return obj.toString();
      }
      else {
        return obj.$inspect();
      }
    
    });
  })(self)
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/module"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$attr_reader', '$attr_writer', '$coerce_to!', '$raise', '$=~', '$[]', '$!', '$==', '$inject', '$const_get', '$split', '$const_missing', '$to_str', '$===', '$to_proc', '$lambda', '$bind', '$call', '$class', '$append_features', '$included', '$name', '$new', '$to_s', '$__id__']);
  return (function($base, $super) {
    function $Module(){};
    var self = $Module = $klass($base, $super, 'Module', $Module);

    var def = self.$$proto, $scope = self.$$scope, TMP_1, TMP_3, TMP_5, TMP_6;

    Opal.defs(self, '$new', TMP_1 = function() {
      var self = this, $iter = TMP_1.$$p, block = $iter || nil;

      TMP_1.$$p = null;
      
      function AnonModule(){}
      var klass      = Opal.boot(Opal.Module, AnonModule);
      klass.$$name   = nil;
      klass.$$class  = Opal.Module;
      klass.$$dep    = []
      klass.$$is_mod = true;
      klass.$$proto  = {};

      // inherit scope from parent
      Opal.create_scope(Opal.Module.$$scope, klass);

      if (block !== nil) {
        var block_self = block.$$s;
        block.$$s = null;
        block.call(klass);
        block.$$s = block_self;
      }

      return klass;
    
    });

    def['$==='] = function(object) {
      var $a, self = this;

      if ((($a = object == null) !== nil && (!$a.$$is_boolean || $a == true))) {
        return false};
      return Opal.is_a(object, self);
    };

    def['$<'] = function(other) {
      var self = this;

      
      var working = self;

      while (working) {
        if (working === other) {
          return true;
        }

        working = working.$$parent;
      }

      return false;
    
    };

    def.$alias_method = function(newname, oldname) {
      var self = this;

      
      var newjsid = '$' + newname,
          body    = self.$$proto['$' + oldname];

      if (self.$$is_singleton) {
        self.$$proto[newjsid] = body;
      }
      else {
        Opal.defn(self, newjsid, body);
      }

      return self;
    
      return self;
    };

    def.$alias_native = function(mid, jsid) {
      var self = this;

      if (jsid == null) {
        jsid = mid
      }
      return self.$$proto['$' + mid] = self.$$proto[jsid];
    };

    def.$ancestors = function() {
      var self = this;

      
      var parent = self,
          result = [];

      while (parent) {
        result.push(parent);
        result = result.concat(parent.$$inc);

        parent = parent.$$super;
      }

      return result;
    
    };

    def.$append_features = function(klass) {
      var self = this;

      Opal.append_features(self, klass);
      return self;
    };

    def.$attr_accessor = function(names) {
      var $a, $b, self = this;

      names = $slice.call(arguments, 0);
      ($a = self).$attr_reader.apply($a, [].concat(names));
      return ($b = self).$attr_writer.apply($b, [].concat(names));
    };

    Opal.defn(self, '$attr', def.$attr_accessor);

    def.$attr_reader = function(names) {
      var self = this;

      names = $slice.call(arguments, 0);
      
      var proto = self.$$proto;

      for (var i = names.length - 1; i >= 0; i--) {
        var name = names[i],
            id   = '$' + name;

        // the closure here is needed because name will change at the next
        // cycle, I wish we could use let.
        var body = (function(name) {
          return function() {
            return this[name];
          };
        })(name);

        // initialize the instance variable as nil
        proto[name] = nil;

        if (self.$$is_singleton) {
          proto.constructor.prototype[id] = body;
        }
        else {
          Opal.defn(self, id, body);
        }
      }
    
      return nil;
    };

    def.$attr_writer = function(names) {
      var self = this;

      names = $slice.call(arguments, 0);
      
      var proto = self.$$proto;

      for (var i = names.length - 1; i >= 0; i--) {
        var name = names[i],
            id   = '$' + name + '=';

        // the closure here is needed because name will change at the next
        // cycle, I wish we could use let.
        var body = (function(name){
          return function(value) {
            return this[name] = value;
          }
        })(name);

        // initialize the instance variable as nil
        proto[name] = nil;

        if (self.$$is_singleton) {
          proto.constructor.prototype[id] = body;
        }
        else {
          Opal.defn(self, id, body);
        }
      }
    
      return nil;
    };

    def.$autoload = function(const$, path) {
      var self = this;

      
      var autoloaders;

      if (!(autoloaders = self.$$autoload)) {
        autoloaders = self.$$autoload = {};
      }

      autoloaders[const$] = path;
      return nil;
    ;
    };

    def.$class_variable_get = function(name) {
      var $a, self = this;

      name = $scope.get('Opal')['$coerce_to!'](name, $scope.get('String'), "to_str");
      if ((($a = name.length < 3 || name.slice(0,2) !== '@@') !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$raise($scope.get('NameError'), "class vars should start with @@")};
      
      var value = Opal.cvars[name.slice(2)];
      (function() {if ((($a = value == null) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.$raise($scope.get('NameError'), "uninitialized class variable @@a in")
        } else {
        return nil
      }; return nil; })()
      return value;
    
    };

    def.$class_variable_set = function(name, value) {
      var $a, self = this;

      name = $scope.get('Opal')['$coerce_to!'](name, $scope.get('String'), "to_str");
      if ((($a = name.length < 3 || name.slice(0,2) !== '@@') !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$raise($scope.get('NameError'))};
      
      Opal.cvars[name.slice(2)] = value;
      return value;
    
    };

    def.$constants = function() {
      var self = this;

      return self.$$scope.constants;
    };

    def['$const_defined?'] = function(name, inherit) {
      var $a, self = this;

      if (inherit == null) {
        inherit = true
      }
      if ((($a = name['$=~'](/^[A-Z]\w*$/)) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise($scope.get('NameError'), "wrong constant name " + (name))
      };
      
      var scopes = [self.$$scope];

      if (inherit || self === Opal.Object) {
        var parent = self.$$super;

        while (parent !== Opal.BasicObject) {
          scopes.push(parent.$$scope);

          parent = parent.$$super;
        }
      }

      for (var i = 0, length = scopes.length; i < length; i++) {
        if (scopes[i].hasOwnProperty(name)) {
          return true;
        }
      }

      return false;
    
    };

    def.$const_get = function(name, inherit) {
      var $a, $b, TMP_2, self = this;

      if (inherit == null) {
        inherit = true
      }
      if ((($a = ($b = name['$[]']("::"), $b !== false && $b !== nil ?name['$==']("::")['$!']() : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return ($a = ($b = name.$split("::")).$inject, $a.$$p = (TMP_2 = function(o, c){var self = TMP_2.$$s || this;
if (o == null) o = nil;if (c == null) c = nil;
        return o.$const_get(c)}, TMP_2.$$s = self, TMP_2), $a).call($b, self)};
      if ((($a = name['$=~'](/^[A-Z]\w*$/)) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise($scope.get('NameError'), "wrong constant name " + (name))
      };
      
      var scopes = [self.$$scope];

      if (inherit || self == Opal.Object) {
        var parent = self.$$super;

        while (parent !== Opal.BasicObject) {
          scopes.push(parent.$$scope);

          parent = parent.$$super;
        }
      }

      for (var i = 0, length = scopes.length; i < length; i++) {
        if (scopes[i].hasOwnProperty(name)) {
          return scopes[i][name];
        }
      }

      return self.$const_missing(name);
    
    };

    def.$const_missing = function(name) {
      var self = this;

      
      if (self.$$autoload) {
        var file = self.$$autoload[name];

        if (file) {
          self.$require(file);

          return self.$const_get(name);
        }
      }
    
      return self.$raise($scope.get('NameError'), "uninitialized constant " + (self) + "::" + (name));
    };

    def.$const_set = function(name, value) {
      var $a, self = this;

      if ((($a = name['$=~'](/^[A-Z]\w*$/)) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise($scope.get('NameError'), "wrong constant name " + (name))
      };
      try {
      name = name.$to_str()
      } catch ($err) {if (true) {
        self.$raise($scope.get('TypeError'), "conversion with #to_str failed")
        }else { throw $err; }
      };
      Opal.casgn(self, name, value);
      return value;
    };

    def.$define_method = TMP_3 = function(name, method) {
      var $a, $b, $c, TMP_4, self = this, $iter = TMP_3.$$p, block = $iter || nil, $case = nil;

      TMP_3.$$p = null;
      if ((($a = method === undefined && !(block !== nil)) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$raise($scope.get('ArgumentError'), "tried to create a Proc object without a block")};
      ((($a = block) !== false && $a !== nil) ? $a : block = (function() {$case = method;if ($scope.get('Proc')['$===']($case)) {return method}else if ($scope.get('Method')['$===']($case)) {return method.$to_proc()}else if ($scope.get('UnboundMethod')['$===']($case)) {return ($b = ($c = self).$lambda, $b.$$p = (TMP_4 = function(args){var self = TMP_4.$$s || this, $a, bound = nil;
args = $slice.call(arguments, 0);
      bound = method.$bind(self);
        return ($a = bound).$call.apply($a, [].concat(args));}, TMP_4.$$s = self, TMP_4), $b).call($c)}else {return self.$raise($scope.get('TypeError'), "wrong argument type " + (block.$class()) + " (expected Proc/Method)")}})());
      
      var id = '$' + name;

      block.$$jsid = name;
      block.$$s    = null;
      block.$$def  = block;

      if (self.$$is_singleton) {
        self.$$proto[id] = block;
      }
      else {
        Opal.defn(self, id, block);
      }

      return name;
    
    };

    def.$remove_method = function(name) {
      var self = this;

      Opal.undef(self, '$' + name);
      return self;
    };

    def.$include = function(mods) {
      var self = this;

      mods = $slice.call(arguments, 0);
      
      for (var i = mods.length - 1; i >= 0; i--) {
        var mod = mods[i];

        if (mod === self) {
          continue;
        }

        (mod).$append_features(self);
        (mod).$included(self);
      }
    
      return self;
    };

    def['$include?'] = function(mod) {
      var self = this;

      
      for (var cls = self; cls; cls = cls.$$super) {
        for (var i = 0; i != cls.$$inc.length; i++) {
          var mod2 = cls.$$inc[i];
          if (mod === mod2) {
            return true;
          }
        }
      }
      return false;
    
    };

    def.$instance_method = function(name) {
      var self = this;

      
      var meth = self.$$proto['$' + name];

      if (!meth || meth.$$stub) {
        self.$raise($scope.get('NameError'), "undefined method `" + (name) + "' for class `" + (self.$name()) + "'");
      }

      return $scope.get('UnboundMethod').$new(self, meth, name);
    
    };

    def.$instance_methods = function(include_super) {
      var self = this;

      if (include_super == null) {
        include_super = true
      }
      
      var methods = [],
          proto   = self.$$proto;

      for (var prop in proto) {
        if (!(prop.charAt(0) === '$')) {
          continue;
        }

        if (!(typeof(proto[prop]) === "function")) {
          continue;
        }

        if (proto[prop].$$stub) {
          continue;
        }

        if (!self.$$is_mod) {
          if (self !== Opal.BasicObject && proto[prop] === Opal.BasicObject.$$proto[prop]) {
            continue;
          }

          if (!include_super && !proto.hasOwnProperty(prop)) {
            continue;
          }

          if (!include_super && proto[prop].$$donated) {
            continue;
          }
        }

        methods.push(prop.substr(1));
      }

      return methods;
    
    };

    def.$included = function(mod) {
      var self = this;

      return nil;
    };

    def.$extended = function(mod) {
      var self = this;

      return nil;
    };

    def.$module_eval = TMP_5 = function() {
      var self = this, $iter = TMP_5.$$p, block = $iter || nil;

      TMP_5.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        self.$raise($scope.get('ArgumentError'), "no block given")
      };
      
      var old = block.$$s,
          result;

      block.$$s = null;
      result = block.call(self);
      block.$$s = old;

      return result;
    
    };

    Opal.defn(self, '$class_eval', def.$module_eval);

    def.$module_exec = TMP_6 = function() {
      var self = this, $iter = TMP_6.$$p, block = $iter || nil;

      TMP_6.$$p = null;
      
      if (block === nil) {
        throw new Error("no block given");
      }

      var block_self = block.$$s, result;

      block.$$s = null;
      result = block.apply(self, $slice.call(arguments));
      block.$$s = block_self;

      return result;
    
    };

    Opal.defn(self, '$class_exec', def.$module_exec);

    def['$method_defined?'] = function(method) {
      var self = this;

      
      var body = self.$$proto['$' + method];
      return (!!body) && !body.$$stub;
    
    };

    def.$module_function = function(methods) {
      var self = this;

      methods = $slice.call(arguments, 0);
      
      if (methods.length === 0) {
        self.$$module_function = true;
      }
      else {
        for (var i = 0, length = methods.length; i < length; i++) {
          var meth = methods[i], func = self.$$proto['$' + meth];

          self.constructor.prototype['$' + meth] = func;
        }
      }

      return self;
    
    };

    def.$name = function() {
      var self = this;

      
      if (self.$$full_name) {
        return self.$$full_name;
      }

      var result = [], base = self;

      while (base) {
        if (base.$$name === nil) {
          return result.length === 0 ? nil : result.join('::');
        }

        result.unshift(base.$$name);

        base = base.$$base_module;

        if (base === Opal.Object) {
          break;
        }
      }

      if (result.length === 0) {
        return nil;
      }

      return self.$$full_name = result.join('::');
    
    };

    def.$public = function(methods) {
      var self = this;

      methods = $slice.call(arguments, 0);
      
      if (methods.length === 0) {
        self.$$module_function = false;
      }

      return nil;
    
    };

    Opal.defn(self, '$private', def.$public);

    Opal.defn(self, '$protected', def.$public);

    Opal.defn(self, '$nesting', def.$public);

    def.$private_class_method = function(name) {
      var self = this;

      return self['$' + name] || nil;
    };

    Opal.defn(self, '$public_class_method', def.$private_class_method);

    def['$private_method_defined?'] = function(obj) {
      var self = this;

      return false;
    };

    def.$private_constant = function() {
      var self = this;

      return nil;
    };

    Opal.defn(self, '$protected_method_defined?', def['$private_method_defined?']);

    Opal.defn(self, '$public_instance_methods', def.$instance_methods);

    Opal.defn(self, '$public_method_defined?', def['$method_defined?']);

    def.$remove_class_variable = function() {
      var self = this;

      return nil;
    };

    def.$remove_const = function(name) {
      var self = this;

      
      var old = self.$$scope[name];
      delete self.$$scope[name];
      return old;
    
    };

    def.$to_s = function() {
      var $a, self = this;

      return ((($a = self.$$name) !== false && $a !== nil) ? $a : "#<" + (self.$$is_mod ? 'Module' : 'Class') + ":0x" + (self.$__id__().$to_s(16)) + ">");
    };

    return (def.$undef_method = function(symbol) {
      var self = this;

      Opal.add_stub_for(self.$$proto, "$" + symbol);
      return self;
    }, nil) && 'undef_method';
  })(self, null)
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/class"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$raise', '$allocate']);
  self.$require("corelib/module");
  return (function($base, $super) {
    function $Class(){};
    var self = $Class = $klass($base, $super, 'Class', $Class);

    var def = self.$$proto, $scope = self.$$scope, TMP_1, TMP_2;

    Opal.defs(self, '$new', TMP_1 = function(sup) {
      var self = this, $iter = TMP_1.$$p, block = $iter || nil;

      if (sup == null) {
        sup = $scope.get('Object')
      }
      TMP_1.$$p = null;
      
      if (!sup.$$is_class || sup.$$is_mod) {
        self.$raise($scope.get('TypeError'), "superclass must be a Class");
      }

      function AnonClass(){};
      var klass      = Opal.boot(sup, AnonClass)
      klass.$$name   = nil;
      klass.$$parent = sup;

      // inherit scope from parent
      Opal.create_scope(sup.$$scope, klass);

      sup.$inherited(klass);

      if (block !== nil) {
        var block_self = block.$$s;
        block.$$s = null;
        block.call(klass);
        block.$$s = block_self;
      }

      return klass;
    ;
    });

    def.$allocate = function() {
      var self = this;

      
      var obj = new self.$$alloc;
      obj.$$id = Opal.uid();
      return obj;
    
    };

    def.$inherited = function(cls) {
      var self = this;

      return nil;
    };

    def.$new = TMP_2 = function(args) {
      var self = this, $iter = TMP_2.$$p, block = $iter || nil;

      args = $slice.call(arguments, 0);
      TMP_2.$$p = null;
      
      var obj = self.$allocate();

      obj.$initialize.$$p = block;
      obj.$initialize.apply(obj, args);
      return obj;
    ;
    };

    return (def.$superclass = function() {
      var self = this;

      return self.$$super || nil;
    }, nil) && 'superclass';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/basic_object"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$raise', '$inspect']);
  return (function($base, $super) {
    function $BasicObject(){};
    var self = $BasicObject = $klass($base, $super, 'BasicObject', $BasicObject);

    var def = self.$$proto, $scope = self.$$scope, TMP_1, TMP_2, TMP_3, TMP_4;

    Opal.defn(self, '$initialize', function() {
      var self = this;

      return nil;
    });

    Opal.defn(self, '$==', function(other) {
      var self = this;

      return self === other;
    });

    Opal.defn(self, '$__id__', function() {
      var self = this;

      return self.$$id || (self.$$id = Opal.uid());
    });

    Opal.defn(self, '$__send__', TMP_1 = function(symbol, args) {
      var self = this, $iter = TMP_1.$$p, block = $iter || nil;

      args = $slice.call(arguments, 1);
      TMP_1.$$p = null;
      
      var func = self['$' + symbol]

      if (func) {
        if (block !== nil) {
          func.$$p = block;
        }

        return func.apply(self, args);
      }

      if (block !== nil) {
        self.$method_missing.$$p = block;
      }

      return self.$method_missing.apply(self, [symbol].concat(args));
    
    });

    Opal.defn(self, '$!', function() {
      var self = this;

      return false;
    });

    Opal.defn(self, '$eql?', def['$==']);

    Opal.defn(self, '$equal?', def['$==']);

    Opal.defn(self, '$instance_eval', TMP_2 = function() {
      var self = this, $iter = TMP_2.$$p, block = $iter || nil;

      TMP_2.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        $scope.get('Kernel').$raise($scope.get('ArgumentError'), "no block given")
      };
      
      var old = block.$$s,
          result;

      block.$$s = null;
      result = block.call(self, self);
      block.$$s = old;

      return result;
    
    });

    Opal.defn(self, '$instance_exec', TMP_3 = function(args) {
      var self = this, $iter = TMP_3.$$p, block = $iter || nil;

      args = $slice.call(arguments, 0);
      TMP_3.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        $scope.get('Kernel').$raise($scope.get('ArgumentError'), "no block given")
      };
      
      var block_self = block.$$s,
          result;

      block.$$s = null;
      result = block.apply(self, args);
      block.$$s = block_self;

      return result;
    
    });

    return (Opal.defn(self, '$method_missing', TMP_4 = function(symbol, args) {
      var $a, self = this, $iter = TMP_4.$$p, block = $iter || nil;

      args = $slice.call(arguments, 1);
      TMP_4.$$p = null;
      return $scope.get('Kernel').$raise($scope.get('NoMethodError'), (function() {if ((($a = self.$inspect && !self.$inspect.$$stub) !== nil && (!$a.$$is_boolean || $a == true))) {
        return "undefined method `" + (symbol) + "' for " + (self.$inspect()) + ":" + (self.$$class)
        } else {
        return "undefined method `" + (symbol) + "' for " + (self.$$class)
      }; return nil; })());
    }), nil) && 'method_missing';
  })(self, null)
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/kernel"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_gt(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs > rhs : lhs['$>'](rhs);
  }
  function $rb_le(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs <= rhs : lhs['$<='](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $gvars = Opal.gvars, $hash2 = Opal.hash2;

  Opal.add_stubs(['$raise', '$inspect', '$==', '$object_id', '$class', '$new', '$coerce_to?', '$<<', '$allocate', '$copy_instance_variables', '$initialize_clone', '$initialize_copy', '$singleton_class', '$initialize_dup', '$for', '$to_proc', '$each', '$reverse', '$append_features', '$extended', '$length', '$respond_to?', '$[]', '$nil?', '$to_a', '$to_int', '$fetch', '$Integer', '$Float', '$to_ary', '$to_str', '$coerce_to', '$to_s', '$__id__', '$coerce_to!', '$===', '$print', '$format', '$puts', '$empty?', '$rand', '$respond_to_missing?', '$try_convert!', '$expand_path', '$join', '$start_with?']);
  return (function($base) {
    var self = $module($base, 'Kernel');

    var def = self.$$proto, $scope = self.$$scope, TMP_1, TMP_2, TMP_3, TMP_4, TMP_5, TMP_6, TMP_7, TMP_9;

    Opal.defn(self, '$method_missing', TMP_1 = function(symbol, args) {
      var self = this, $iter = TMP_1.$$p, block = $iter || nil;

      args = $slice.call(arguments, 1);
      TMP_1.$$p = null;
      return self.$raise($scope.get('NoMethodError'), "undefined method `" + (symbol) + "' for " + (self.$inspect()));
    });

    Opal.defn(self, '$=~', function(obj) {
      var self = this;

      return false;
    });

    Opal.defn(self, '$===', function(other) {
      var $a, self = this;

      return ((($a = self.$object_id()['$=='](other.$object_id())) !== false && $a !== nil) ? $a : self['$=='](other));
    });

    Opal.defn(self, '$<=>', function(other) {
      var self = this;

      
      var x = self['$=='](other);

      if (x && x !== nil) {
        return 0;
      }

      return nil;
    ;
    });

    Opal.defn(self, '$method', function(name) {
      var self = this;

      
      var meth = self['$' + name];

      if (!meth || meth.$$stub) {
        self.$raise($scope.get('NameError'), "undefined method `" + (name) + "' for class `" + (self.$class()) + "'");
      }

      return $scope.get('Method').$new(self, meth, name);
    
    });

    Opal.defn(self, '$methods', function(all) {
      var self = this;

      if (all == null) {
        all = true
      }
      
      var methods = [];

      for (var key in self) {
        if (key[0] == "$" && typeof(self[key]) === "function") {
          if (all == false || all === nil) {
            if (!Opal.hasOwnProperty.call(self, key)) {
              continue;
            }
          }
          if (self[key].$$stub === undefined) {
            methods.push(key.substr(1));
          }
        }
      }

      return methods;
    
    });

    Opal.defn(self, '$Array', function(object) {
      var self = this;

      
      var coerced;

      if (object === nil) {
        return [];
      }

      if (object.$$is_array) {
        return object;
      }

      coerced = $scope.get('Opal')['$coerce_to?'](object, $scope.get('Array'), "to_ary");
      if (coerced !== nil) { return coerced; }

      coerced = $scope.get('Opal')['$coerce_to?'](object, $scope.get('Array'), "to_a");
      if (coerced !== nil) { return coerced; }

      return [object];
    
    });

    Opal.defn(self, '$at_exit', TMP_2 = function() {
      var $a, self = this, $iter = TMP_2.$$p, block = $iter || nil;
      if ($gvars.__at_exit__ == null) $gvars.__at_exit__ = nil;

      TMP_2.$$p = null;
      ((($a = $gvars.__at_exit__) !== false && $a !== nil) ? $a : $gvars.__at_exit__ = []);
      return $gvars.__at_exit__['$<<'](block);
    });

    Opal.defn(self, '$caller', function() {
      var self = this;

      return [];
    });

    Opal.defn(self, '$class', function() {
      var self = this;

      return self.$$class;
    });

    Opal.defn(self, '$copy_instance_variables', function(other) {
      var self = this;

      
      for (var name in other) {
        if (name.charAt(0) !== '$') {
          self[name] = other[name];
        }
      }
    
    });

    Opal.defn(self, '$clone', function() {
      var self = this, copy = nil;

      copy = self.$class().$allocate();
      copy.$copy_instance_variables(self);
      copy.$initialize_clone(self);
      return copy;
    });

    Opal.defn(self, '$initialize_clone', function(other) {
      var self = this;

      return self.$initialize_copy(other);
    });

    Opal.defn(self, '$define_singleton_method', TMP_3 = function(name, body) {
      var $a, self = this, $iter = TMP_3.$$p, block = $iter || nil;

      if (body == null) {
        body = nil
      }
      TMP_3.$$p = null;
      ((($a = body) !== false && $a !== nil) ? $a : body = block);
      if (body !== false && body !== nil) {
        } else {
        self.$raise($scope.get('ArgumentError'), "tried to create Proc object without a block")
      };
      
      var jsid   = '$' + name;
      body.$$jsid = name;
      body.$$s    = null;
      body.$$def  = body;

      self.$singleton_class().$$proto[jsid] = body;

      return self;
    
    });

    Opal.defn(self, '$dup', function() {
      var self = this, copy = nil;

      copy = self.$class().$allocate();
      copy.$copy_instance_variables(self);
      copy.$initialize_dup(self);
      return copy;
    });

    Opal.defn(self, '$initialize_dup', function(other) {
      var self = this;

      return self.$initialize_copy(other);
    });

    Opal.defn(self, '$enum_for', TMP_4 = function(method, args) {
      var $a, $b, self = this, $iter = TMP_4.$$p, block = $iter || nil;

      args = $slice.call(arguments, 1);
      if (method == null) {
        method = "each"
      }
      TMP_4.$$p = null;
      return ($a = ($b = $scope.get('Enumerator')).$for, $a.$$p = block.$to_proc(), $a).apply($b, [self, method].concat(args));
    });

    Opal.defn(self, '$to_enum', def.$enum_for);

    Opal.defn(self, '$equal?', function(other) {
      var self = this;

      return self === other;
    });

    Opal.defn(self, '$exit', function(status) {
      var $a, $b, self = this;
      if ($gvars.__at_exit__ == null) $gvars.__at_exit__ = nil;

      if (status == null) {
        status = true
      }
      if ((($a = $gvars.__at_exit__) !== nil && (!$a.$$is_boolean || $a == true))) {
        ($a = ($b = $gvars.__at_exit__.$reverse()).$each, $a.$$p = "call".$to_proc(), $a).call($b)};
      if ((($a = status === true) !== nil && (!$a.$$is_boolean || $a == true))) {
        status = 0};
      Opal.exit(status);
      return nil;
    });

    Opal.defn(self, '$extend', function(mods) {
      var self = this;

      mods = $slice.call(arguments, 0);
      
      var singleton = self.$singleton_class();

      for (var i = mods.length - 1; i >= 0; i--) {
        var mod = mods[i];

        (mod).$append_features(singleton);
        (mod).$extended(self);
      }
    ;
      return self;
    });

    Opal.defn(self, '$format', function(format_string, args) {
      var $a, $b, self = this, ary = nil;
      if ($gvars.DEBUG == null) $gvars.DEBUG = nil;

      args = $slice.call(arguments, 1);
      if ((($a = (($b = args.$length()['$=='](1)) ? args['$[]'](0)['$respond_to?']("to_ary") : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
        ary = $scope.get('Opal')['$coerce_to?'](args['$[]'](0), $scope.get('Array'), "to_ary");
        if ((($a = ary['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
          } else {
          args = ary.$to_a()
        };};
      
      var result = '',
          //used for slicing:
          begin_slice = 0,
          end_slice,
          //used for iterating over the format string:
          i,
          len = format_string.length,
          //used for processing field values:
          arg,
          str,
          //used for processing %g and %G fields:
          exponent,
          //used for keeping track of width and precision:
          width,
          precision,
          //used for holding temporary values:
          tmp_num,
          //used for processing %{} and %<> fileds:
          hash_parameter_key,
          closing_brace_char,
          //used for processing %b, %B, %o, %x, and %X fields:
          base_number,
          base_prefix,
          base_neg_zero_regex,
          base_neg_zero_digit,
          //used for processing arguments:
          next_arg,
          seq_arg_num = 1,
          pos_arg_num = 0,
          //used for keeping track of flags:
          flags,
          FNONE  = 0,
          FSHARP = 1,
          FMINUS = 2,
          FPLUS  = 4,
          FZERO  = 8,
          FSPACE = 16,
          FWIDTH = 32,
          FPREC  = 64,
          FPREC0 = 128;

      function CHECK_FOR_FLAGS() {
        if (flags&FWIDTH) { self.$raise($scope.get('ArgumentError'), "flag after width") }
        if (flags&FPREC0) { self.$raise($scope.get('ArgumentError'), "flag after precision") }
      }

      function CHECK_FOR_WIDTH() {
        if (flags&FWIDTH) { self.$raise($scope.get('ArgumentError'), "width given twice") }
        if (flags&FPREC0) { self.$raise($scope.get('ArgumentError'), "width after precision") }
      }

      function GET_NTH_ARG(num) {
        if (num >= args.length) { self.$raise($scope.get('ArgumentError'), "too few arguments") }
        return args[num];
      }

      function GET_NEXT_ARG() {
        switch (pos_arg_num) {
        case -1: self.$raise($scope.get('ArgumentError'), "unnumbered(" + (seq_arg_num) + ") mixed with numbered")
        case -2: self.$raise($scope.get('ArgumentError'), "unnumbered(" + (seq_arg_num) + ") mixed with named")
        }
        pos_arg_num = seq_arg_num++;
        return GET_NTH_ARG(pos_arg_num - 1);
      }

      function GET_POS_ARG(num) {
        if (pos_arg_num > 0) {
          self.$raise($scope.get('ArgumentError'), "numbered(" + (num) + ") after unnumbered(" + (pos_arg_num) + ")")
        }
        if (pos_arg_num === -2) {
          self.$raise($scope.get('ArgumentError'), "numbered(" + (num) + ") after named")
        }
        if (num < 1) {
          self.$raise($scope.get('ArgumentError'), "invalid index - " + (num) + "$")
        }
        pos_arg_num = -1;
        return GET_NTH_ARG(num - 1);
      }

      function GET_ARG() {
        return (next_arg === undefined ? GET_NEXT_ARG() : next_arg);
      }

      function READ_NUM(label) {
        var num, str = '';
        for (;; i++) {
          if (i === len) {
            self.$raise($scope.get('ArgumentError'), "malformed format string - %*[0-9]")
          }
          if (format_string.charCodeAt(i) < 48 || format_string.charCodeAt(i) > 57) {
            i--;
            num = parseInt(str) || 0;
            if (num > 2147483647) {
              self.$raise($scope.get('ArgumentError'), "" + (label) + " too big")
            }
            return num;
          }
          str += format_string.charAt(i);
        }
      }

      function READ_NUM_AFTER_ASTER(label) {
        var arg, num = READ_NUM(label);
        if (format_string.charAt(i + 1) === '$') {
          i++;
          arg = GET_POS_ARG(num);
        } else {
          arg = GET_NEXT_ARG();
        }
        return (arg).$to_int();
      }

      for (i = format_string.indexOf('%'); i !== -1; i = format_string.indexOf('%', i)) {
        str = undefined;

        flags = FNONE;
        width = -1;
        precision = -1;
        next_arg = undefined;

        end_slice = i;

        i++;

        switch (format_string.charAt(i)) {
        case '%':
          begin_slice = i;
        case '':
        case '\n':
        case '\0':
          i++;
          continue;
        }

        format_sequence: for (; i < len; i++) {
          switch (format_string.charAt(i)) {

          case ' ':
            CHECK_FOR_FLAGS();
            flags |= FSPACE;
            continue format_sequence;

          case '#':
            CHECK_FOR_FLAGS();
            flags |= FSHARP;
            continue format_sequence;

          case '+':
            CHECK_FOR_FLAGS();
            flags |= FPLUS;
            continue format_sequence;

          case '-':
            CHECK_FOR_FLAGS();
            flags |= FMINUS;
            continue format_sequence;

          case '0':
            CHECK_FOR_FLAGS();
            flags |= FZERO;
            continue format_sequence;

          case '1':
          case '2':
          case '3':
          case '4':
          case '5':
          case '6':
          case '7':
          case '8':
          case '9':
            tmp_num = READ_NUM('width');
            if (format_string.charAt(i + 1) === '$') {
              if (i + 2 === len) {
                str = '%';
                i++;
                break format_sequence;
              }
              if (next_arg !== undefined) {
                self.$raise($scope.get('ArgumentError'), "value given twice - %" + (tmp_num) + "$")
              }
              next_arg = GET_POS_ARG(tmp_num);
              i++;
            } else {
              CHECK_FOR_WIDTH();
              flags |= FWIDTH;
              width = tmp_num;
            }
            continue format_sequence;

          case '<':
          case '\{':
            closing_brace_char = (format_string.charAt(i) === '<' ? '>' : '\}');
            hash_parameter_key = '';

            i++;

            for (;; i++) {
              if (i === len) {
                self.$raise($scope.get('ArgumentError'), "malformed name - unmatched parenthesis")
              }
              if (format_string.charAt(i) === closing_brace_char) {

                if (pos_arg_num > 0) {
                  self.$raise($scope.get('ArgumentError'), "named " + (hash_parameter_key) + " after unnumbered(" + (pos_arg_num) + ")")
                }
                if (pos_arg_num === -1) {
                  self.$raise($scope.get('ArgumentError'), "named " + (hash_parameter_key) + " after numbered")
                }
                pos_arg_num = -2;

                if (args[0] === undefined || !args[0].$$is_hash) {
                  self.$raise($scope.get('ArgumentError'), "one hash required")
                }

                next_arg = (args[0]).$fetch(hash_parameter_key);

                if (closing_brace_char === '>') {
                  continue format_sequence;
                } else {
                  str = next_arg.toString();
                  if (precision !== -1) { str = str.slice(0, precision); }
                  if (flags&FMINUS) {
                    while (str.length < width) { str = str + ' '; }
                  } else {
                    while (str.length < width) { str = ' ' + str; }
                  }
                  break format_sequence;
                }
              }
              hash_parameter_key += format_string.charAt(i);
            }

          case '*':
            i++;
            CHECK_FOR_WIDTH();
            flags |= FWIDTH;
            width = READ_NUM_AFTER_ASTER('width');
            if (width < 0) {
              flags |= FMINUS;
              width = -width;
            }
            continue format_sequence;

          case '.':
            if (flags&FPREC0) {
              self.$raise($scope.get('ArgumentError'), "precision given twice")
            }
            flags |= FPREC|FPREC0;
            precision = 0;
            i++;
            if (format_string.charAt(i) === '*') {
              i++;
              precision = READ_NUM_AFTER_ASTER('precision');
              if (precision < 0) {
                flags &= ~FPREC;
              }
              continue format_sequence;
            }
            precision = READ_NUM('precision');
            continue format_sequence;

          case 'd':
          case 'i':
          case 'u':
            arg = self.$Integer(GET_ARG());
            if (arg >= 0) {
              str = arg.toString();
              while (str.length < precision) { str = '0' + str; }
              if (flags&FMINUS) {
                if (flags&FPLUS || flags&FSPACE) { str = (flags&FPLUS ? '+' : ' ') + str; }
                while (str.length < width) { str = str + ' '; }
              } else {
                if (flags&FZERO && precision === -1) {
                  while (str.length < width - ((flags&FPLUS || flags&FSPACE) ? 1 : 0)) { str = '0' + str; }
                  if (flags&FPLUS || flags&FSPACE) { str = (flags&FPLUS ? '+' : ' ') + str; }
                } else {
                  if (flags&FPLUS || flags&FSPACE) { str = (flags&FPLUS ? '+' : ' ') + str; }
                  while (str.length < width) { str = ' ' + str; }
                }
              }
            } else {
              str = (-arg).toString();
              while (str.length < precision) { str = '0' + str; }
              if (flags&FMINUS) {
                str = '-' + str;
                while (str.length < width) { str = str + ' '; }
              } else {
                if (flags&FZERO && precision === -1) {
                  while (str.length < width - 1) { str = '0' + str; }
                  str = '-' + str;
                } else {
                  str = '-' + str;
                  while (str.length < width) { str = ' ' + str; }
                }
              }
            }
            break format_sequence;

          case 'b':
          case 'B':
          case 'o':
          case 'x':
          case 'X':
            switch (format_string.charAt(i)) {
            case 'b':
            case 'B':
              base_number = 2;
              base_prefix = '0b';
              base_neg_zero_regex = /^1+/;
              base_neg_zero_digit = '1';
              break;
            case 'o':
              base_number = 8;
              base_prefix = '0';
              base_neg_zero_regex = /^3?7+/;
              base_neg_zero_digit = '7';
              break;
            case 'x':
            case 'X':
              base_number = 16;
              base_prefix = '0x';
              base_neg_zero_regex = /^f+/;
              base_neg_zero_digit = 'f';
              break;
            }
            arg = self.$Integer(GET_ARG());
            if (arg >= 0) {
              str = arg.toString(base_number);
              while (str.length < precision) { str = '0' + str; }
              if (flags&FMINUS) {
                if (flags&FPLUS || flags&FSPACE) { str = (flags&FPLUS ? '+' : ' ') + str; }
                if (flags&FSHARP && arg !== 0) { str = base_prefix + str; }
                while (str.length < width) { str = str + ' '; }
              } else {
                if (flags&FZERO && precision === -1) {
                  while (str.length < width - ((flags&FPLUS || flags&FSPACE) ? 1 : 0) - ((flags&FSHARP && arg !== 0) ? base_prefix.length : 0)) { str = '0' + str; }
                  if (flags&FSHARP && arg !== 0) { str = base_prefix + str; }
                  if (flags&FPLUS || flags&FSPACE) { str = (flags&FPLUS ? '+' : ' ') + str; }
                } else {
                  if (flags&FSHARP && arg !== 0) { str = base_prefix + str; }
                  if (flags&FPLUS || flags&FSPACE) { str = (flags&FPLUS ? '+' : ' ') + str; }
                  while (str.length < width) { str = ' ' + str; }
                }
              }
            } else {
              if (flags&FPLUS || flags&FSPACE) {
                str = (-arg).toString(base_number);
                while (str.length < precision) { str = '0' + str; }
                if (flags&FMINUS) {
                  if (flags&FSHARP) { str = base_prefix + str; }
                  str = '-' + str;
                  while (str.length < width) { str = str + ' '; }
                } else {
                  if (flags&FZERO && precision === -1) {
                    while (str.length < width - 1 - (flags&FSHARP ? 2 : 0)) { str = '0' + str; }
                    if (flags&FSHARP) { str = base_prefix + str; }
                    str = '-' + str;
                  } else {
                    if (flags&FSHARP) { str = base_prefix + str; }
                    str = '-' + str;
                    while (str.length < width) { str = ' ' + str; }
                  }
                }
              } else {
                str = (arg >>> 0).toString(base_number).replace(base_neg_zero_regex, base_neg_zero_digit);
                while (str.length < precision - 2) { str = base_neg_zero_digit + str; }
                if (flags&FMINUS) {
                  str = '..' + str;
                  if (flags&FSHARP) { str = base_prefix + str; }
                  while (str.length < width) { str = str + ' '; }
                } else {
                  if (flags&FZERO && precision === -1) {
                    while (str.length < width - 2 - (flags&FSHARP ? base_prefix.length : 0)) { str = base_neg_zero_digit + str; }
                    str = '..' + str;
                    if (flags&FSHARP) { str = base_prefix + str; }
                  } else {
                    str = '..' + str;
                    if (flags&FSHARP) { str = base_prefix + str; }
                    while (str.length < width) { str = ' ' + str; }
                  }
                }
              }
            }
            if (format_string.charAt(i) === format_string.charAt(i).toUpperCase()) {
              str = str.toUpperCase();
            }
            break format_sequence;

          case 'f':
          case 'e':
          case 'E':
          case 'g':
          case 'G':
            arg = self.$Float(GET_ARG());
            if (arg >= 0 || isNaN(arg)) {
              if (arg === Infinity) {
                str = 'Inf';
              } else {
                switch (format_string.charAt(i)) {
                case 'f':
                  str = arg.toFixed(precision === -1 ? 6 : precision);
                  break;
                case 'e':
                case 'E':
                  str = arg.toExponential(precision === -1 ? 6 : precision);
                  break;
                case 'g':
                case 'G':
                  str = arg.toExponential();
                  exponent = parseInt(str.split('e')[1]);
                  if (!(exponent < -4 || exponent >= (precision === -1 ? 6 : precision))) {
                    str = arg.toPrecision(precision === -1 ? (flags&FSHARP ? 6 : undefined) : precision);
                  }
                  break;
                }
              }
              if (flags&FMINUS) {
                if (flags&FPLUS || flags&FSPACE) { str = (flags&FPLUS ? '+' : ' ') + str; }
                while (str.length < width) { str = str + ' '; }
              } else {
                if (flags&FZERO && arg !== Infinity && !isNaN(arg)) {
                  while (str.length < width - ((flags&FPLUS || flags&FSPACE) ? 1 : 0)) { str = '0' + str; }
                  if (flags&FPLUS || flags&FSPACE) { str = (flags&FPLUS ? '+' : ' ') + str; }
                } else {
                  if (flags&FPLUS || flags&FSPACE) { str = (flags&FPLUS ? '+' : ' ') + str; }
                  while (str.length < width) { str = ' ' + str; }
                }
              }
            } else {
              if (arg === -Infinity) {
                str = 'Inf';
              } else {
                switch (format_string.charAt(i)) {
                case 'f':
                  str = (-arg).toFixed(precision === -1 ? 6 : precision);
                  break;
                case 'e':
                case 'E':
                  str = (-arg).toExponential(precision === -1 ? 6 : precision);
                  break;
                case 'g':
                case 'G':
                  str = (-arg).toExponential();
                  exponent = parseInt(str.split('e')[1]);
                  if (!(exponent < -4 || exponent >= (precision === -1 ? 6 : precision))) {
                    str = (-arg).toPrecision(precision === -1 ? (flags&FSHARP ? 6 : undefined) : precision);
                  }
                  break;
                }
              }
              if (flags&FMINUS) {
                str = '-' + str;
                while (str.length < width) { str = str + ' '; }
              } else {
                if (flags&FZERO && arg !== -Infinity) {
                  while (str.length < width - 1) { str = '0' + str; }
                  str = '-' + str;
                } else {
                  str = '-' + str;
                  while (str.length < width) { str = ' ' + str; }
                }
              }
            }
            if (format_string.charAt(i) === format_string.charAt(i).toUpperCase() && arg !== Infinity && arg !== -Infinity && !isNaN(arg)) {
              str = str.toUpperCase();
            }
            str = str.replace(/([eE][-+]?)([0-9])$/, '$10$2');
            break format_sequence;

          case 'a':
          case 'A':
            // Not implemented because there are no specs for this field type.
            self.$raise($scope.get('NotImplementedError'), "`A` and `a` format field types are not implemented in Opal yet")

          case 'c':
            arg = GET_ARG();
            if ((arg)['$respond_to?']("to_ary")) { arg = (arg).$to_ary()[0]; }
            if ((arg)['$respond_to?']("to_str")) {
              str = (arg).$to_str();
            } else {
              str = String.fromCharCode($scope.get('Opal').$coerce_to(arg, $scope.get('Integer'), "to_int"));
            }
            if (str.length !== 1) {
              self.$raise($scope.get('ArgumentError'), "%c requires a character")
            }
            if (flags&FMINUS) {
              while (str.length < width) { str = str + ' '; }
            } else {
              while (str.length < width) { str = ' ' + str; }
            }
            break format_sequence;

          case 'p':
            str = (GET_ARG()).$inspect();
            if (precision !== -1) { str = str.slice(0, precision); }
            if (flags&FMINUS) {
              while (str.length < width) { str = str + ' '; }
            } else {
              while (str.length < width) { str = ' ' + str; }
            }
            break format_sequence;

          case 's':
            str = (GET_ARG()).$to_s();
            if (precision !== -1) { str = str.slice(0, precision); }
            if (flags&FMINUS) {
              while (str.length < width) { str = str + ' '; }
            } else {
              while (str.length < width) { str = ' ' + str; }
            }
            break format_sequence;

          default:
            self.$raise($scope.get('ArgumentError'), "malformed format string - %" + (format_string.charAt(i)))
          }
        }

        if (str === undefined) {
          self.$raise($scope.get('ArgumentError'), "malformed format string - %")
        }

        result += format_string.slice(begin_slice, end_slice) + str;
        begin_slice = i + 1;
      }

      if ($gvars.DEBUG && pos_arg_num >= 0 && seq_arg_num < args.length) {
        self.$raise($scope.get('ArgumentError'), "too many arguments for format string")
      }

      return result + format_string.slice(begin_slice);
    ;
    });

    Opal.defn(self, '$freeze', function() {
      var self = this;

      self.___frozen___ = true;
      return self;
    });

    Opal.defn(self, '$frozen?', function() {
      var $a, self = this;
      if (self.___frozen___ == null) self.___frozen___ = nil;

      return ((($a = self.___frozen___) !== false && $a !== nil) ? $a : false);
    });

    Opal.defn(self, '$hash', function() {
      var self = this;

      return "" + (self.$class()) + ":" + (self.$class().$__id__()) + ":" + (self.$__id__());
    });

    Opal.defn(self, '$initialize_copy', function(other) {
      var self = this;

      return nil;
    });

    Opal.defn(self, '$inspect', function() {
      var self = this;

      return self.$to_s();
    });

    Opal.defn(self, '$instance_of?', function(klass) {
      var self = this;

      return self.$$class === klass;
    });

    Opal.defn(self, '$instance_variable_defined?', function(name) {
      var self = this;

      return Opal.hasOwnProperty.call(self, name.substr(1));
    });

    Opal.defn(self, '$instance_variable_get', function(name) {
      var self = this;

      
      var ivar = self[name.substr(1)];

      return ivar == null ? nil : ivar;
    
    });

    Opal.defn(self, '$instance_variable_set', function(name, value) {
      var self = this;

      return self[name.substr(1)] = value;
    });

    Opal.defn(self, '$instance_variables', function() {
      var self = this;

      
      var result = [];

      for (var name in self) {
        if (name.charAt(0) !== '$') {
          if (name !== '$$class' && name !== '$$id') {
            result.push('@' + name);
          }
        }
      }

      return result;
    
    });

    Opal.defn(self, '$Integer', function(value, base) {
      var self = this;

      
      var i, str, base_digits;

      if (!value.$$is_string) {
        if (base !== undefined) {
          self.$raise($scope.get('ArgumentError'), "base specified for non string value")
        }
        if (value === nil) {
          self.$raise($scope.get('TypeError'), "can't convert nil into Integer")
        }
        if (value.$$is_number) {
          if (value === Infinity || value === -Infinity || isNaN(value)) {
            self.$raise($scope.get('FloatDomainError'), value)
          }
          return Math.floor(value);
        }
        if (value['$respond_to?']("to_int")) {
          i = value.$to_int();
          if (i !== nil) {
            return i;
          }
        }
        return $scope.get('Opal')['$coerce_to!'](value, $scope.get('Integer'), "to_i");
      }

      if (base === undefined) {
        base = 0;
      } else {
        base = $scope.get('Opal').$coerce_to(base, $scope.get('Integer'), "to_int");
        if (base === 1 || base < 0 || base > 36) {
          self.$raise($scope.get('ArgumentError'), "invalid radix " + (base))
        }
      }

      str = value.toLowerCase();

      str = str.replace(/(\d)_(?=\d)/g, '$1');

      str = str.replace(/^(\s*[+-]?)(0[bodx]?)/, function (_, head, flag) {
        switch (flag) {
        case '0b':
          if (base === 0 || base === 2) {
            base = 2;
            return head;
          }
        case '0':
        case '0o':
          if (base === 0 || base === 8) {
            base = 8;
            return head;
          }
        case '0d':
          if (base === 0 || base === 10) {
            base = 10;
            return head;
          }
        case '0x':
          if (base === 0 || base === 16) {
            base = 16;
            return head;
          }
        }
        self.$raise($scope.get('ArgumentError'), "invalid value for Integer(): \"" + (value) + "\"")
      });

      base = (base === 0 ? 10 : base);

      base_digits = '0-' + (base <= 10 ? base - 1 : '9a-' + String.fromCharCode(97 + (base - 11)));

      if (!(new RegExp('^\\s*[+-]?[' + base_digits + ']+\\s*$')).test(str)) {
        self.$raise($scope.get('ArgumentError'), "invalid value for Integer(): \"" + (value) + "\"")
      }

      i = parseInt(str, base);

      if (isNaN(i)) {
        self.$raise($scope.get('ArgumentError'), "invalid value for Integer(): \"" + (value) + "\"")
      }

      return i;
    ;
    });

    Opal.defn(self, '$Float', function(value) {
      var self = this;

      
      var str;

      if (value === nil) {
        self.$raise($scope.get('TypeError'), "can't convert nil into Float")
      }

      if (value.$$is_string) {
        str = value.toString();

        str = str.replace(/(\d)_(?=\d)/g, '$1');

        //Special case for hex strings only:
        if (/^\s*[-+]?0[xX][0-9a-fA-F]+\s*$/.test(str)) {
          return self.$Integer(str);
        }

        if (!/^\s*[-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?\s*$/.test(str)) {
          self.$raise($scope.get('ArgumentError'), "invalid value for Float(): \"" + (value) + "\"")
        }

        return parseFloat(str);
      }

      return $scope.get('Opal')['$coerce_to!'](value, $scope.get('Float'), "to_f");
    
    });

    Opal.defn(self, '$Hash', function(arg) {
      var $a, $b, self = this;

      if ((($a = ((($b = arg['$nil?']()) !== false && $b !== nil) ? $b : arg['$==']([]))) !== nil && (!$a.$$is_boolean || $a == true))) {
        return $hash2([], {})};
      if ((($a = $scope.get('Hash')['$==='](arg)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return arg};
      return $scope.get('Opal')['$coerce_to!'](arg, $scope.get('Hash'), "to_hash");
    });

    Opal.defn(self, '$is_a?', function(klass) {
      var self = this;

      return Opal.is_a(self, klass);
    });

    Opal.defn(self, '$kind_of?', def['$is_a?']);

    Opal.defn(self, '$lambda', TMP_5 = function() {
      var self = this, $iter = TMP_5.$$p, block = $iter || nil;

      TMP_5.$$p = null;
      block.$$is_lambda = true;
      return block;
    });

    Opal.defn(self, '$load', function(file) {
      var self = this;

      file = $scope.get('Opal')['$coerce_to!'](file, $scope.get('String'), "to_str");
      return Opal.load(Opal.normalize_loadable_path(file));
    });

    Opal.defn(self, '$loop', TMP_6 = function() {
      var self = this, $iter = TMP_6.$$p, block = $iter || nil;

      TMP_6.$$p = null;
      
      while (true) {
        if (block() === $breaker) {
          return $breaker.$v;
        }
      }
    
      return self;
    });

    Opal.defn(self, '$nil?', function() {
      var self = this;

      return false;
    });

    Opal.defn(self, '$object_id', def.$__id__);

    Opal.defn(self, '$printf', function(args) {
      var $a, self = this;

      args = $slice.call(arguments, 0);
      if ($rb_gt(args.$length(), 0)) {
        self.$print(($a = self).$format.apply($a, [].concat(args)))};
      return nil;
    });

    Opal.defn(self, '$private_methods', function() {
      var self = this;

      return [];
    });

    Opal.defn(self, '$private_instance_methods', def.$private_methods);

    Opal.defn(self, '$proc', TMP_7 = function() {
      var self = this, $iter = TMP_7.$$p, block = $iter || nil;

      TMP_7.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        self.$raise($scope.get('ArgumentError'), "tried to create Proc object without a block")
      };
      block.$$is_lambda = false;
      return block;
    });

    Opal.defn(self, '$puts', function(strs) {
      var $a, self = this;
      if ($gvars.stdout == null) $gvars.stdout = nil;

      strs = $slice.call(arguments, 0);
      return ($a = $gvars.stdout).$puts.apply($a, [].concat(strs));
    });

    Opal.defn(self, '$p', function(args) {
      var $a, $b, TMP_8, self = this;

      args = $slice.call(arguments, 0);
      ($a = ($b = args).$each, $a.$$p = (TMP_8 = function(obj){var self = TMP_8.$$s || this;
        if ($gvars.stdout == null) $gvars.stdout = nil;
if (obj == null) obj = nil;
      return $gvars.stdout.$puts(obj.$inspect())}, TMP_8.$$s = self, TMP_8), $a).call($b);
      if ($rb_le(args.$length(), 1)) {
        return args['$[]'](0)
        } else {
        return args
      };
    });

    Opal.defn(self, '$print', function(strs) {
      var $a, self = this;
      if ($gvars.stdout == null) $gvars.stdout = nil;

      strs = $slice.call(arguments, 0);
      return ($a = $gvars.stdout).$print.apply($a, [].concat(strs));
    });

    Opal.defn(self, '$warn', function(strs) {
      var $a, $b, self = this;
      if ($gvars.VERBOSE == null) $gvars.VERBOSE = nil;
      if ($gvars.stderr == null) $gvars.stderr = nil;

      strs = $slice.call(arguments, 0);
      if ((($a = ((($b = $gvars.VERBOSE['$nil?']()) !== false && $b !== nil) ? $b : strs['$empty?']())) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        return ($a = $gvars.stderr).$puts.apply($a, [].concat(strs))
      };
    });

    Opal.defn(self, '$raise', function(exception, string) {
      var self = this;
      if ($gvars["!"] == null) $gvars["!"] = nil;

      
      if (exception == null && $gvars["!"]) {
        throw $gvars["!"];
      }

      if (exception == null) {
        exception = $scope.get('RuntimeError').$new();
      }
      else if (exception.$$is_string) {
        exception = $scope.get('RuntimeError').$new(exception);
      }
      else if (exception.$$is_class) {
        exception = exception.$new(string);
      }

      $gvars["!"] = exception;

      throw exception;
    ;
    });

    Opal.defn(self, '$fail', def.$raise);

    Opal.defn(self, '$rand', function(max) {
      var self = this;

      
      if (max === undefined) {
        return Math.random();
      }
      else if (max.$$is_range) {
        var arr = max.$to_a();

        return arr[self.$rand(arr.length)];
      }
      else {
        return Math.floor(Math.random() *
          Math.abs($scope.get('Opal').$coerce_to(max, $scope.get('Integer'), "to_int")));
      }
    
    });

    Opal.defn(self, '$respond_to?', function(name, include_all) {
      var $a, self = this;

      if (include_all == null) {
        include_all = false
      }
      if ((($a = self['$respond_to_missing?'](name)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return true};
      
      var body = self['$' + name];

      if (typeof(body) === "function" && !body.$$stub) {
        return true;
      }
    
      return false;
    });

    Opal.defn(self, '$respond_to_missing?', function(method_name) {
      var self = this;

      return false;
    });

    Opal.defn(self, '$require', function(file) {
      var self = this;

      file = $scope.get('Opal')['$coerce_to!'](file, $scope.get('String'), "to_str");
      return Opal.require(Opal.normalize_loadable_path(file));
    });

    Opal.defn(self, '$require_relative', function(file) {
      var self = this;

      $scope.get('Opal')['$try_convert!'](file, $scope.get('String'), "to_str");
      file = $scope.get('File').$expand_path($scope.get('File').$join(Opal.current_file, "..", file));
      return Opal.require(Opal.normalize_loadable_path(file));
    });

    Opal.defn(self, '$require_tree', function(path) {
      var self = this;

      path = $scope.get('File').$expand_path(path);
      if (path['$=='](".")) {
        path = ""};
      
      for (var name in Opal.modules) {
        if ((name)['$start_with?'](path)) {
          Opal.require(name);
        }
      }
    ;
      return nil;
    });

    Opal.defn(self, '$send', def.$__send__);

    Opal.defn(self, '$public_send', def.$__send__);

    Opal.defn(self, '$singleton_class', function() {
      var self = this;

      return Opal.get_singleton_class(self);
    });

    Opal.defn(self, '$sprintf', def.$format);

    Opal.defn(self, '$srand', def.$rand);

    Opal.defn(self, '$String', function(str) {
      var $a, self = this;

      return ((($a = $scope.get('Opal')['$coerce_to?'](str, $scope.get('String'), "to_str")) !== false && $a !== nil) ? $a : $scope.get('Opal')['$coerce_to!'](str, $scope.get('String'), "to_s"));
    });

    Opal.defn(self, '$taint', function() {
      var self = this;

      return self;
    });

    Opal.defn(self, '$tainted?', function() {
      var self = this;

      return false;
    });

    Opal.defn(self, '$tap', TMP_9 = function() {
      var self = this, $iter = TMP_9.$$p, block = $iter || nil;

      TMP_9.$$p = null;
      if (Opal.yield1(block, self) === $breaker) return $breaker.$v;
      return self;
    });

    Opal.defn(self, '$to_proc', function() {
      var self = this;

      return self;
    });

    Opal.defn(self, '$to_s', function() {
      var self = this;

      return "#<" + (self.$class()) + ":0x" + (self.$__id__().$to_s(16)) + ">";
    });

    Opal.defn(self, '$untaint', def.$taint);
  })(self)
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/nil_class"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$raise']);
  (function($base, $super) {
    function $NilClass(){};
    var self = $NilClass = $klass($base, $super, 'NilClass', $NilClass);

    var def = self.$$proto, $scope = self.$$scope;

    def['$!'] = function() {
      var self = this;

      return true;
    };

    def['$&'] = function(other) {
      var self = this;

      return false;
    };

    def['$|'] = function(other) {
      var self = this;

      return other !== false && other !== nil;
    };

    def['$^'] = function(other) {
      var self = this;

      return other !== false && other !== nil;
    };

    def['$=='] = function(other) {
      var self = this;

      return other === nil;
    };

    def.$dup = function() {
      var self = this;

      return self.$raise($scope.get('TypeError'));
    };

    def.$inspect = function() {
      var self = this;

      return "nil";
    };

    def['$nil?'] = function() {
      var self = this;

      return true;
    };

    def.$singleton_class = function() {
      var self = this;

      return $scope.get('NilClass');
    };

    def.$to_a = function() {
      var self = this;

      return [];
    };

    def.$to_h = function() {
      var self = this;

      return Opal.hash();
    };

    def.$to_i = function() {
      var self = this;

      return 0;
    };

    Opal.defn(self, '$to_f', def.$to_i);

    return (def.$to_s = function() {
      var self = this;

      return "";
    }, nil) && 'to_s';
  })(self, null);
  return Opal.cdecl($scope, 'NIL', nil);
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/boolean"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$undef_method']);
  (function($base, $super) {
    function $Boolean(){};
    var self = $Boolean = $klass($base, $super, 'Boolean', $Boolean);

    var def = self.$$proto, $scope = self.$$scope;

    def.$$is_boolean = true;

    def.$__id__ = function() {
      var self = this;

      return self.valueOf() ? 2 : 0;
    };

    Opal.defn(self, '$object_id', def.$__id__);

    (function(self) {
      var $scope = self.$$scope, def = self.$$proto;

      return self.$undef_method("new")
    })(self.$singleton_class());

    def['$!'] = function() {
      var self = this;

      return self != true;
    };

    def['$&'] = function(other) {
      var self = this;

      return (self == true) ? (other !== false && other !== nil) : false;
    };

    def['$|'] = function(other) {
      var self = this;

      return (self == true) ? true : (other !== false && other !== nil);
    };

    def['$^'] = function(other) {
      var self = this;

      return (self == true) ? (other === false || other === nil) : (other !== false && other !== nil);
    };

    def['$=='] = function(other) {
      var self = this;

      return (self == true) === other.valueOf();
    };

    Opal.defn(self, '$equal?', def['$==']);

    Opal.defn(self, '$singleton_class', def.$class);

    return (def.$to_s = function() {
      var self = this;

      return (self == true) ? 'true' : 'false';
    }, nil) && 'to_s';
  })(self, null);
  Opal.cdecl($scope, 'TrueClass', $scope.get('Boolean'));
  Opal.cdecl($scope, 'FalseClass', $scope.get('Boolean'));
  Opal.cdecl($scope, 'TRUE', true);
  return Opal.cdecl($scope, 'FALSE', false);
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/error"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $module = Opal.module;

  Opal.add_stubs(['$attr_reader', '$class']);
  (function($base, $super) {
    function $Exception(){};
    var self = $Exception = $klass($base, $super, 'Exception', $Exception);

    var def = self.$$proto, $scope = self.$$scope;

    def.message = nil;
    self.$attr_reader("message");

    Opal.defs(self, '$new', function(message) {
      var self = this;

      if (message == null) {
        message = "Exception"
      }
      
      var err = new self.$$alloc(message);

      if (Error.captureStackTrace) {
        Error.captureStackTrace(err);
      }

      err.name = self.$$name;
      err.$initialize(message);
      return err;
    
    });

    def.$initialize = function(message) {
      var self = this;

      return self.message = message;
    };

    def.$backtrace = function() {
      var self = this;

      
      var backtrace = self.stack;

      if (typeof(backtrace) === 'string') {
        return backtrace.split("\n").slice(0, 15);
      }
      else if (backtrace) {
        return backtrace.slice(0, 15);
      }

      return [];
    
    };

    def.$inspect = function() {
      var self = this;

      return "#<" + (self.$class()) + ": '" + (self.message) + "'>";
    };

    return Opal.defn(self, '$to_s', def.$message);
  })(self, null);
  (function($base, $super) {
    function $ScriptError(){};
    var self = $ScriptError = $klass($base, $super, 'ScriptError', $ScriptError);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('Exception'));
  (function($base, $super) {
    function $SyntaxError(){};
    var self = $SyntaxError = $klass($base, $super, 'SyntaxError', $SyntaxError);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('ScriptError'));
  (function($base, $super) {
    function $LoadError(){};
    var self = $LoadError = $klass($base, $super, 'LoadError', $LoadError);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('ScriptError'));
  (function($base, $super) {
    function $NotImplementedError(){};
    var self = $NotImplementedError = $klass($base, $super, 'NotImplementedError', $NotImplementedError);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('ScriptError'));
  (function($base, $super) {
    function $SystemExit(){};
    var self = $SystemExit = $klass($base, $super, 'SystemExit', $SystemExit);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('Exception'));
  (function($base, $super) {
    function $NoMemoryError(){};
    var self = $NoMemoryError = $klass($base, $super, 'NoMemoryError', $NoMemoryError);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('Exception'));
  (function($base, $super) {
    function $SignalException(){};
    var self = $SignalException = $klass($base, $super, 'SignalException', $SignalException);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('Exception'));
  (function($base, $super) {
    function $Interrupt(){};
    var self = $Interrupt = $klass($base, $super, 'Interrupt', $Interrupt);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('Exception'));
  (function($base, $super) {
    function $StandardError(){};
    var self = $StandardError = $klass($base, $super, 'StandardError', $StandardError);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('Exception'));
  (function($base, $super) {
    function $NameError(){};
    var self = $NameError = $klass($base, $super, 'NameError', $NameError);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('StandardError'));
  (function($base, $super) {
    function $NoMethodError(){};
    var self = $NoMethodError = $klass($base, $super, 'NoMethodError', $NoMethodError);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('NameError'));
  (function($base, $super) {
    function $RuntimeError(){};
    var self = $RuntimeError = $klass($base, $super, 'RuntimeError', $RuntimeError);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('StandardError'));
  (function($base, $super) {
    function $LocalJumpError(){};
    var self = $LocalJumpError = $klass($base, $super, 'LocalJumpError', $LocalJumpError);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('StandardError'));
  (function($base, $super) {
    function $TypeError(){};
    var self = $TypeError = $klass($base, $super, 'TypeError', $TypeError);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('StandardError'));
  (function($base, $super) {
    function $ArgumentError(){};
    var self = $ArgumentError = $klass($base, $super, 'ArgumentError', $ArgumentError);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('StandardError'));
  (function($base, $super) {
    function $IndexError(){};
    var self = $IndexError = $klass($base, $super, 'IndexError', $IndexError);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('StandardError'));
  (function($base, $super) {
    function $StopIteration(){};
    var self = $StopIteration = $klass($base, $super, 'StopIteration', $StopIteration);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('IndexError'));
  (function($base, $super) {
    function $KeyError(){};
    var self = $KeyError = $klass($base, $super, 'KeyError', $KeyError);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('IndexError'));
  (function($base, $super) {
    function $RangeError(){};
    var self = $RangeError = $klass($base, $super, 'RangeError', $RangeError);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('StandardError'));
  (function($base, $super) {
    function $FloatDomainError(){};
    var self = $FloatDomainError = $klass($base, $super, 'FloatDomainError', $FloatDomainError);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('RangeError'));
  (function($base, $super) {
    function $IOError(){};
    var self = $IOError = $klass($base, $super, 'IOError', $IOError);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('StandardError'));
  (function($base, $super) {
    function $SystemCallError(){};
    var self = $SystemCallError = $klass($base, $super, 'SystemCallError', $SystemCallError);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('StandardError'));
  return (function($base) {
    var self = $module($base, 'Errno');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base, $super) {
      function $EINVAL(){};
      var self = $EINVAL = $klass($base, $super, 'EINVAL', $EINVAL);

      var def = self.$$proto, $scope = self.$$scope, TMP_1;

      return (Opal.defs(self, '$new', TMP_1 = function() {
        var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

        TMP_1.$$p = null;
        return Opal.find_super_dispatcher(self, 'new', TMP_1, null, $EINVAL).apply(self, ["Invalid argument"]);
      }), nil) && 'new'
    })(self, $scope.get('SystemCallError'))
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/regexp"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $gvars = Opal.gvars;

  Opal.add_stubs(['$nil?', '$[]', '$raise', '$escape', '$options', '$to_str', '$new', '$join', '$!', '$match', '$begin', '$coerce_to', '$call', '$=~']);
  (function($base, $super) {
    function $RegexpError(){};
    var self = $RegexpError = $klass($base, $super, 'RegexpError', $RegexpError);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('StandardError'));
  return (function($base, $super) {
    function $Regexp(){};
    var self = $Regexp = $klass($base, $super, 'Regexp', $Regexp);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    Opal.cdecl($scope, 'IGNORECASE', 1);

    Opal.cdecl($scope, 'MULTILINE', 4);

    def.$$is_regexp = true;

    (function(self) {
      var $scope = self.$$scope, def = self.$$proto;

      self.$$proto.$escape = function(string) {
        var self = this;

        
        return string.replace(/([-[\]\/{}()*+?.^$\\| ])/g, '\\$1')
                     .replace(/[\n]/g, '\\n')
                     .replace(/[\r]/g, '\\r')
                     .replace(/[\f]/g, '\\f')
                     .replace(/[\t]/g, '\\t');
      
      };
      self.$$proto.$last_match = function(n) {
        var $a, self = this;
        if ($gvars["~"] == null) $gvars["~"] = nil;

        if (n == null) {
          n = nil
        }
        if ((($a = n['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
          return $gvars["~"]
          } else {
          return $gvars["~"]['$[]'](n)
        };
      };
      self.$$proto.$quote = self.$$proto.$escape;
      self.$$proto.$union = function(parts) {
        var self = this;

        parts = $slice.call(arguments, 0);
        
        var is_first_part_array, quoted_validated, part, options, each_part_options;
        if (parts.length == 0) {
          return /(?!)/;
        }
        // cover the 2 arrays passed as arguments case
        is_first_part_array = parts[0].$$is_array;
        if (parts.length > 1 && is_first_part_array) {
          self.$raise($scope.get('TypeError'), "no implicit conversion of Array into String")
        }        
        // deal with splat issues (related to https://github.com/opal/opal/issues/858)
        if (is_first_part_array) {
          parts = parts[0];
        }
        options = undefined;
        quoted_validated = [];
        for (var i=0; i < parts.length; i++) {
          part = parts[i];
          if (part.$$is_string) {
            quoted_validated.push(self.$escape(part));
          }
          else if (part.$$is_regexp) { 
            each_part_options = (part).$options();   
            if (options != undefined && options != each_part_options) {
              self.$raise($scope.get('TypeError'), "All expressions must use the same options")
            }
            options = each_part_options;
            quoted_validated.push('('+part.source+')');
          }
          else {
            quoted_validated.push(self.$escape((part).$to_str()));
          }
        }
      
        return self.$new((quoted_validated).$join("|"), options);
      };
      return (self.$$proto.$new = function(regexp, options) {
        var self = this;

        
        // Play nice with IE8
        if (regexp.$$is_string && regexp.substr(regexp.length-1, 1) == "\\") {
          self.$raise($scope.get('RegexpError'), "too short escape sequence: /" + (regexp) + "/")
        }
        
        if (options == undefined || options['$!']()) {
          options = undefined;
        }
        
        if (options != undefined) {
          if (regexp.$$is_regexp) {
            // options are already in regex
            options = undefined;
          }
          else if (options.$$is_number) {
            var result = '';
            if ($scope.get('IGNORECASE') & options) {
              result += 'i';
            }
            if ($scope.get('MULTILINE') & options) {
              result += 'm';
            }
            options = result;
          }
          else {
            options = 'i';
          }
        }       
        
        return new RegExp(regexp, options);
      ;
      }, nil) && 'new';
    })(self.$singleton_class());

    def['$=='] = function(other) {
      var self = this;

      return other.constructor == RegExp && self.toString() === other.toString();
    };

    def['$==='] = function(string) {
      var self = this;

      return self.$match(string) !== nil;
    };

    def['$=~'] = function(string) {
      var $a, self = this;
      if ($gvars["~"] == null) $gvars["~"] = nil;

      return ($a = self.$match(string), $a !== false && $a !== nil ?$gvars["~"].$begin(0) : $a);
    };

    Opal.defn(self, '$eql?', def['$==']);

    def.$inspect = function() {
      var self = this;

      return self.toString();
    };

    def.$match = TMP_1 = function(string, pos) {
      var self = this, $iter = TMP_1.$$p, block = $iter || nil;
      if ($gvars["~"] == null) $gvars["~"] = nil;

      TMP_1.$$p = null;
      
      if (pos === undefined) {
        pos = 0;
      } else {
        pos = $scope.get('Opal').$coerce_to(pos, $scope.get('Integer'), "to_int");
      }

      if (string === nil) {
        return $gvars["~"] = nil;
      }

      string = $scope.get('Opal').$coerce_to(string, $scope.get('String'), "to_str");

      if (pos < 0) {
        pos += string.length;
        if (pos < 0) {
          return $gvars["~"] = nil;
        }
      }

      // global RegExp maintains state, so not using self/this
      var md, re = new RegExp(self.source, 'gm' + (self.ignoreCase ? 'i' : ''));

      while (true) {
        md = re.exec(string);
        if (md === null) {
          return $gvars["~"] = nil;
        }
        if (md.index >= pos) {
          $gvars["~"] = $scope.get('MatchData').$new(re, md)
          return block === nil ? $gvars["~"] : block.$call($gvars["~"]);
        }
        re.lastIndex = md.index + 1;
      }
    ;
    };

    def['$~'] = function() {
      var self = this;
      if ($gvars._ == null) $gvars._ = nil;

      return self['$=~']($gvars._);
    };

    def.$source = function() {
      var self = this;

      return self.source;
    };

    def.$options = function() {
      var self = this;

      
      var as_string, text_flags, result, text_flag;
      as_string = self.toString();
      if (as_string == "/(?:)/") {
        self.$raise($scope.get('TypeError'), "uninitialized Regexp")
      }
      text_flags = as_string.replace(self.source, '').match(/\w+/);
      result = 0;
      // may have no flags
      if (text_flags == null) {
        return result;
      }
      // first match contains all of our flags
      text_flags = text_flags[0];
      for (var i=0; i < text_flags.length; i++) {
        text_flag = text_flags[i];
        switch(text_flag) {
          case 'i':
            result |= $scope.get('IGNORECASE');
            break;
          case 'm':
            result |= $scope.get('MULTILINE');
            break;
          default:
            self.$raise("RegExp flag " + (text_flag) + " does not have a match in Ruby")
        }
      }
      
      return result;
    
    };

    return Opal.defn(self, '$to_s', def.$source);
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/comparable"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_gt(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs > rhs : lhs['$>'](rhs);
  }
  function $rb_lt(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs < rhs : lhs['$<'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module;

  Opal.add_stubs(['$===', '$equal?', '$<=>', '$normalize', '$raise', '$class']);
  return (function($base) {
    var self = $module($base, 'Comparable');

    var def = self.$$proto, $scope = self.$$scope;

    Opal.defs(self, '$normalize', function(what) {
      var $a, self = this;

      if ((($a = $scope.get('Integer')['$==='](what)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return what};
      if ($rb_gt(what, 0)) {
        return 1};
      if ($rb_lt(what, 0)) {
        return -1};
      return 0;
    });

    Opal.defn(self, '$==', function(other) {
      var $a, self = this, cmp = nil;

      try {
      if ((($a = self['$equal?'](other)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return true};
        if ((($a = cmp = (self['$<=>'](other))) !== nil && (!$a.$$is_boolean || $a == true))) {
          } else {
          return false
        };
        return $scope.get('Comparable').$normalize(cmp) == 0;
      } catch ($err) {if (Opal.rescue($err, [$scope.get('StandardError')])) {
        return false
        }else { throw $err; }
      };
    });

    Opal.defn(self, '$>', function(other) {
      var $a, self = this, cmp = nil;

      if ((($a = cmp = (self['$<=>'](other))) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise($scope.get('ArgumentError'), "comparison of " + (self.$class()) + " with " + (other.$class()) + " failed")
      };
      return $scope.get('Comparable').$normalize(cmp) > 0;
    });

    Opal.defn(self, '$>=', function(other) {
      var $a, self = this, cmp = nil;

      if ((($a = cmp = (self['$<=>'](other))) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise($scope.get('ArgumentError'), "comparison of " + (self.$class()) + " with " + (other.$class()) + " failed")
      };
      return $scope.get('Comparable').$normalize(cmp) >= 0;
    });

    Opal.defn(self, '$<', function(other) {
      var $a, self = this, cmp = nil;

      if ((($a = cmp = (self['$<=>'](other))) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise($scope.get('ArgumentError'), "comparison of " + (self.$class()) + " with " + (other.$class()) + " failed")
      };
      return $scope.get('Comparable').$normalize(cmp) < 0;
    });

    Opal.defn(self, '$<=', function(other) {
      var $a, self = this, cmp = nil;

      if ((($a = cmp = (self['$<=>'](other))) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise($scope.get('ArgumentError'), "comparison of " + (self.$class()) + " with " + (other.$class()) + " failed")
      };
      return $scope.get('Comparable').$normalize(cmp) <= 0;
    });

    Opal.defn(self, '$between?', function(min, max) {
      var self = this;

      if ($rb_lt(self, min)) {
        return false};
      if ($rb_gt(self, max)) {
        return false};
      return true;
    });
  })(self)
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/enumerable"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module;

  Opal.add_stubs(['$raise', '$enum_for', '$flatten', '$map', '$==', '$destructure', '$nil?', '$coerce_to!', '$coerce_to', '$===', '$new', '$<<', '$[]', '$[]=', '$inspect', '$__send__', '$yield', '$enumerator_size', '$respond_to?', '$size', '$private', '$compare', '$<=>', '$dup', '$to_a', '$lambda', '$sort', '$call', '$first', '$zip']);
  return (function($base) {
    var self = $module($base, 'Enumerable');

    var def = self.$$proto, $scope = self.$$scope, TMP_1, TMP_2, TMP_3, TMP_4, TMP_5, TMP_7, TMP_8, TMP_9, TMP_10, TMP_11, TMP_12, TMP_13, TMP_14, TMP_15, TMP_16, TMP_17, TMP_18, TMP_19, TMP_20, TMP_22, TMP_23, TMP_24, TMP_25, TMP_26, TMP_27, TMP_28, TMP_29, TMP_30, TMP_31, TMP_32, TMP_33, TMP_35, TMP_37, TMP_41, TMP_42;

    Opal.defn(self, '$all?', TMP_1 = function() {
      var $a, self = this, $iter = TMP_1.$$p, block = $iter || nil;

      TMP_1.$$p = null;
      
      var result = true;

      if (block !== nil) {
        self.$each.$$p = function() {
          var value = Opal.yieldX(block, arguments);

          if (value === $breaker) {
            result = $breaker.$v;
            return $breaker;
          }

          if ((($a = value) === nil || ($a.$$is_boolean && $a == false))) {
            result = false;
            return $breaker;
          }
        };
      }
      else {
        self.$each.$$p = function(obj) {
          if (arguments.length == 1 && (($a = obj) === nil || ($a.$$is_boolean && $a == false))) {
            result = false;
            return $breaker;
          }
        };
      }

      self.$each();

      return result;
    
    });

    Opal.defn(self, '$any?', TMP_2 = function() {
      var $a, self = this, $iter = TMP_2.$$p, block = $iter || nil;

      TMP_2.$$p = null;
      
      var result = false;

      if (block !== nil) {
        self.$each.$$p = function() {
          var value = Opal.yieldX(block, arguments);

          if (value === $breaker) {
            result = $breaker.$v;
            return $breaker;
          }

          if ((($a = value) !== nil && (!$a.$$is_boolean || $a == true))) {
            result = true;
            return $breaker;
          }
        };
      }
      else {
        self.$each.$$p = function(obj) {
          if (arguments.length != 1 || (($a = obj) !== nil && (!$a.$$is_boolean || $a == true))) {
            result = true;
            return $breaker;
          }
        }
      }

      self.$each();

      return result;
    
    });

    Opal.defn(self, '$chunk', TMP_3 = function(state) {
      var self = this, $iter = TMP_3.$$p, block = $iter || nil;

      TMP_3.$$p = null;
      return self.$raise($scope.get('NotImplementedError'));
    });

    Opal.defn(self, '$collect', TMP_4 = function() {
      var self = this, $iter = TMP_4.$$p, block = $iter || nil;

      TMP_4.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("collect")
      };
      
      var result = [];

      self.$each.$$p = function() {
        var value = Opal.yieldX(block, arguments);

        if (value === $breaker) {
          result = $breaker.$v;
          return $breaker;
        }

        result.push(value);
      };

      self.$each();

      return result;
    
    });

    Opal.defn(self, '$collect_concat', TMP_5 = function() {
      var $a, $b, TMP_6, self = this, $iter = TMP_5.$$p, block = $iter || nil;

      TMP_5.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("collect_concat")
      };
      return ($a = ($b = self).$map, $a.$$p = (TMP_6 = function(item){var self = TMP_6.$$s || this, $a;
if (item == null) item = nil;
      return $a = Opal.yield1(block, item), $a === $breaker ? $a : $a}, TMP_6.$$s = self, TMP_6), $a).call($b).$flatten(1);
    });

    Opal.defn(self, '$count', TMP_7 = function(object) {
      var $a, self = this, $iter = TMP_7.$$p, block = $iter || nil;

      TMP_7.$$p = null;
      
      var result = 0;

      if (object != null) {
        block = function() {
          return $scope.get('Opal').$destructure(arguments)['$=='](object);
        };
      }
      else if (block === nil) {
        block = function() { return true; };
      }

      self.$each.$$p = function() {
        var value = Opal.yieldX(block, arguments);

        if (value === $breaker) {
          result = $breaker.$v;
          return $breaker;
        }

        if ((($a = value) !== nil && (!$a.$$is_boolean || $a == true))) {
          result++;
        }
      }

      self.$each();

      return result;
    
    });

    Opal.defn(self, '$cycle', TMP_8 = function(n) {
      var $a, self = this, $iter = TMP_8.$$p, block = $iter || nil;

      if (n == null) {
        n = nil
      }
      TMP_8.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        return self.$enum_for("cycle", n)
      };
      if ((($a = n['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        n = $scope.get('Opal')['$coerce_to!'](n, $scope.get('Integer'), "to_int");
        if ((($a = n <= 0) !== nil && (!$a.$$is_boolean || $a == true))) {
          return nil};
      };
      
      var result,
          all  = [];

      self.$each.$$p = function() {
        var param = $scope.get('Opal').$destructure(arguments),
            value = Opal.yield1(block, param);

        if (value === $breaker) {
          result = $breaker.$v;
          return $breaker;
        }

        all.push(param);
      }

      self.$each();

      if (result !== undefined) {
        return result;
      }

      if (all.length === 0) {
        return nil;
      }
    
      if ((($a = n['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        
        while (true) {
          for (var i = 0, length = all.length; i < length; i++) {
            var value = Opal.yield1(block, all[i]);

            if (value === $breaker) {
              return $breaker.$v;
            }
          }
        }
      
        } else {
        
        while (n > 1) {
          for (var i = 0, length = all.length; i < length; i++) {
            var value = Opal.yield1(block, all[i]);

            if (value === $breaker) {
              return $breaker.$v;
            }
          }

          n--;
        }
      
      };
    });

    Opal.defn(self, '$detect', TMP_9 = function(ifnone) {
      var $a, self = this, $iter = TMP_9.$$p, block = $iter || nil;

      TMP_9.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("detect", ifnone)
      };
      
      var result = undefined;

      self.$each.$$p = function() {
        var params = $scope.get('Opal').$destructure(arguments),
            value  = Opal.yield1(block, params);

        if (value === $breaker) {
          result = $breaker.$v;
          return $breaker;
        }

        if ((($a = value) !== nil && (!$a.$$is_boolean || $a == true))) {
          result = params;
          return $breaker;
        }
      };

      self.$each();

      if (result === undefined && ifnone !== undefined) {
        if (typeof(ifnone) === 'function') {
          result = ifnone();
        }
        else {
          result = ifnone;
        }
      }

      return result === undefined ? nil : result;
    
    });

    Opal.defn(self, '$drop', function(number) {
      var $a, self = this;

      number = $scope.get('Opal').$coerce_to(number, $scope.get('Integer'), "to_int");
      if ((($a = number < 0) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$raise($scope.get('ArgumentError'), "attempt to drop negative size")};
      
      var result  = [],
          current = 0;

      self.$each.$$p = function() {
        if (number <= current) {
          result.push($scope.get('Opal').$destructure(arguments));
        }

        current++;
      };

      self.$each()

      return result;
    
    });

    Opal.defn(self, '$drop_while', TMP_10 = function() {
      var $a, self = this, $iter = TMP_10.$$p, block = $iter || nil;

      TMP_10.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("drop_while")
      };
      
      var result   = [],
          dropping = true;

      self.$each.$$p = function() {
        var param = $scope.get('Opal').$destructure(arguments);

        if (dropping) {
          var value = Opal.yield1(block, param);

          if (value === $breaker) {
            result = $breaker.$v;
            return $breaker;
          }

          if ((($a = value) === nil || ($a.$$is_boolean && $a == false))) {
            dropping = false;
            result.push(param);
          }
        }
        else {
          result.push(param);
        }
      };

      self.$each();

      return result;
    
    });

    Opal.defn(self, '$each_cons', TMP_11 = function(n) {
      var self = this, $iter = TMP_11.$$p, block = $iter || nil;

      TMP_11.$$p = null;
      return self.$raise($scope.get('NotImplementedError'));
    });

    Opal.defn(self, '$each_entry', TMP_12 = function() {
      var self = this, $iter = TMP_12.$$p, block = $iter || nil;

      TMP_12.$$p = null;
      return self.$raise($scope.get('NotImplementedError'));
    });

    Opal.defn(self, '$each_slice', TMP_13 = function(n) {
      var $a, self = this, $iter = TMP_13.$$p, block = $iter || nil;

      TMP_13.$$p = null;
      n = $scope.get('Opal').$coerce_to(n, $scope.get('Integer'), "to_int");
      if ((($a = n <= 0) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$raise($scope.get('ArgumentError'), "invalid slice size")};
      if ((block !== nil)) {
        } else {
        return self.$enum_for("each_slice", n)
      };
      
      var result,
          slice = []

      self.$each.$$p = function() {
        var param = $scope.get('Opal').$destructure(arguments);

        slice.push(param);

        if (slice.length === n) {
          if (Opal.yield1(block, slice) === $breaker) {
            result = $breaker.$v;
            return $breaker;
          }

          slice = [];
        }
      };

      self.$each();

      if (result !== undefined) {
        return result;
      }

      // our "last" group, if smaller than n then won't have been yielded
      if (slice.length > 0) {
        if (Opal.yield1(block, slice) === $breaker) {
          return $breaker.$v;
        }
      }
    ;
      return nil;
    });

    Opal.defn(self, '$each_with_index', TMP_14 = function(args) {
      var $a, self = this, $iter = TMP_14.$$p, block = $iter || nil;

      args = $slice.call(arguments, 0);
      TMP_14.$$p = null;
      if ((block !== nil)) {
        } else {
        return ($a = self).$enum_for.apply($a, ["each_with_index"].concat(args))
      };
      
      var result,
          index = 0;

      self.$each.$$p = function() {
        var param = $scope.get('Opal').$destructure(arguments),
            value = block(param, index);

        if (value === $breaker) {
          result = $breaker.$v;
          return $breaker;
        }

        index++;
      };

      self.$each.apply(self, args);

      if (result !== undefined) {
        return result;
      }
    
      return self;
    });

    Opal.defn(self, '$each_with_object', TMP_15 = function(object) {
      var self = this, $iter = TMP_15.$$p, block = $iter || nil;

      TMP_15.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("each_with_object", object)
      };
      
      var result;

      self.$each.$$p = function() {
        var param = $scope.get('Opal').$destructure(arguments),
            value = block(param, object);

        if (value === $breaker) {
          result = $breaker.$v;
          return $breaker;
        }
      };

      self.$each();

      if (result !== undefined) {
        return result;
      }
    
      return object;
    });

    Opal.defn(self, '$entries', function(args) {
      var self = this;

      args = $slice.call(arguments, 0);
      
      var result = [];

      self.$each.$$p = function() {
        result.push($scope.get('Opal').$destructure(arguments));
      };

      self.$each.apply(self, args);

      return result;
    
    });

    Opal.defn(self, '$find', def.$detect);

    Opal.defn(self, '$find_all', TMP_16 = function() {
      var $a, self = this, $iter = TMP_16.$$p, block = $iter || nil;

      TMP_16.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("find_all")
      };
      
      var result = [];

      self.$each.$$p = function() {
        var param = $scope.get('Opal').$destructure(arguments),
            value = Opal.yield1(block, param);

        if (value === $breaker) {
          result = $breaker.$v;
          return $breaker;
        }

        if ((($a = value) !== nil && (!$a.$$is_boolean || $a == true))) {
          result.push(param);
        }
      };

      self.$each();

      return result;
    
    });

    Opal.defn(self, '$find_index', TMP_17 = function(object) {
      var $a, self = this, $iter = TMP_17.$$p, block = $iter || nil;

      TMP_17.$$p = null;
      if ((($a = object === undefined && block === nil) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.$enum_for("find_index")};
      
      var result = nil,
          index  = 0;

      if (object != null) {
        self.$each.$$p = function() {
          var param = $scope.get('Opal').$destructure(arguments);

          if ((param)['$=='](object)) {
            result = index;
            return $breaker;
          }

          index += 1;
        };
      }
      else if (block !== nil) {
        self.$each.$$p = function() {
          var value = Opal.yieldX(block, arguments);

          if (value === $breaker) {
            result = $breaker.$v;
            return $breaker;
          }

          if ((($a = value) !== nil && (!$a.$$is_boolean || $a == true))) {
            result = index;
            return $breaker;
          }

          index += 1;
        };
      }

      self.$each();

      return result;
    
    });

    Opal.defn(self, '$first', function(number) {
      var $a, self = this, result = nil;

      if ((($a = number === undefined) !== nil && (!$a.$$is_boolean || $a == true))) {
        result = nil;
        
        self.$each.$$p = function() {
          result = $scope.get('Opal').$destructure(arguments);

          return $breaker;
        };

        self.$each();
      ;
        } else {
        result = [];
        number = $scope.get('Opal').$coerce_to(number, $scope.get('Integer'), "to_int");
        if ((($a = number < 0) !== nil && (!$a.$$is_boolean || $a == true))) {
          self.$raise($scope.get('ArgumentError'), "attempt to take negative size")};
        if ((($a = number == 0) !== nil && (!$a.$$is_boolean || $a == true))) {
          return []};
        
        var current = 0,
            number  = $scope.get('Opal').$coerce_to(number, $scope.get('Integer'), "to_int");

        self.$each.$$p = function() {
          result.push($scope.get('Opal').$destructure(arguments));

          if (number <= ++current) {
            return $breaker;
          }
        };

        self.$each();
      ;
      };
      return result;
    });

    Opal.defn(self, '$flat_map', def.$collect_concat);

    Opal.defn(self, '$grep', TMP_18 = function(pattern) {
      var $a, self = this, $iter = TMP_18.$$p, block = $iter || nil;

      TMP_18.$$p = null;
      
      var result = [];

      if (block !== nil) {
        self.$each.$$p = function() {
          var param = $scope.get('Opal').$destructure(arguments),
              value = pattern['$==='](param);

          if ((($a = value) !== nil && (!$a.$$is_boolean || $a == true))) {
            value = Opal.yield1(block, param);

            if (value === $breaker) {
              result = $breaker.$v;
              return $breaker;
            }

            result.push(value);
          }
        };
      }
      else {
        self.$each.$$p = function() {
          var param = $scope.get('Opal').$destructure(arguments),
              value = pattern['$==='](param);

          if ((($a = value) !== nil && (!$a.$$is_boolean || $a == true))) {
            result.push(param);
          }
        };
      }

      self.$each();

      return result;
    ;
    });

    Opal.defn(self, '$group_by', TMP_19 = function() {
      var $a, $b, $c, self = this, $iter = TMP_19.$$p, block = $iter || nil, hash = nil;

      TMP_19.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("group_by")
      };
      hash = $scope.get('Hash').$new();
      
      var result;

      self.$each.$$p = function() {
        var param = $scope.get('Opal').$destructure(arguments),
            value = Opal.yield1(block, param);

        if (value === $breaker) {
          result = $breaker.$v;
          return $breaker;
        }

        (($a = value, $b = hash, ((($c = $b['$[]']($a)) !== false && $c !== nil) ? $c : $b['$[]=']($a, []))))['$<<'](param);
      }

      self.$each();

      if (result !== undefined) {
        return result;
      }
    
      return hash;
    });

    Opal.defn(self, '$include?', function(obj) {
      var self = this;

      
      var result = false;

      self.$each.$$p = function() {
        var param = $scope.get('Opal').$destructure(arguments);

        if ((param)['$=='](obj)) {
          result = true;
          return $breaker;
        }
      }

      self.$each();

      return result;
    
    });

    Opal.defn(self, '$inject', TMP_20 = function(object, sym) {
      var self = this, $iter = TMP_20.$$p, block = $iter || nil;

      TMP_20.$$p = null;
      
      var result = object;

      if (block !== nil && sym === undefined) {
        self.$each.$$p = function() {
          var value = $scope.get('Opal').$destructure(arguments);

          if (result === undefined) {
            result = value;
            return;
          }

          value = Opal.yieldX(block, [result, value]);

          if (value === $breaker) {
            result = $breaker.$v;
            return $breaker;
          }

          result = value;
        };
      }
      else {
        if (sym === undefined) {
          if (!$scope.get('Symbol')['$==='](object)) {
            self.$raise($scope.get('TypeError'), "" + (object.$inspect()) + " is not a Symbol");
          }

          sym    = object;
          result = undefined;
        }

        self.$each.$$p = function() {
          var value = $scope.get('Opal').$destructure(arguments);

          if (result === undefined) {
            result = value;
            return;
          }

          result = (result).$__send__(sym, value);
        };
      }

      self.$each();

      return result == undefined ? nil : result;
    ;
    });

    Opal.defn(self, '$lazy', function() {
      var $a, $b, TMP_21, self = this;

      return ($a = ($b = (($scope.get('Enumerator')).$$scope.get('Lazy'))).$new, $a.$$p = (TMP_21 = function(enum$, args){var self = TMP_21.$$s || this, $a;
if (enum$ == null) enum$ = nil;args = $slice.call(arguments, 1);
      return ($a = enum$).$yield.apply($a, [].concat(args))}, TMP_21.$$s = self, TMP_21), $a).call($b, self, self.$enumerator_size());
    });

    Opal.defn(self, '$enumerator_size', function() {
      var $a, self = this;

      if ((($a = self['$respond_to?']("size")) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.$size()
        } else {
        return nil
      };
    });

    self.$private("enumerator_size");

    Opal.defn(self, '$map', def.$collect);

    Opal.defn(self, '$max', TMP_22 = function() {
      var self = this, $iter = TMP_22.$$p, block = $iter || nil;

      TMP_22.$$p = null;
      
      var result;

      if (block !== nil) {
        self.$each.$$p = function() {
          var param = $scope.get('Opal').$destructure(arguments);

          if (result === undefined) {
            result = param;
            return;
          }

          var value = block(param, result);

          if (value === $breaker) {
            result = $breaker.$v;
            return $breaker;
          }

          if (value === nil) {
            self.$raise($scope.get('ArgumentError'), "comparison failed");
          }

          if (value > 0) {
            result = param;
          }
        };
      }
      else {
        self.$each.$$p = function() {
          var param = $scope.get('Opal').$destructure(arguments);

          if (result === undefined) {
            result = param;
            return;
          }

          if ($scope.get('Opal').$compare(param, result) > 0) {
            result = param;
          }
        };
      }

      self.$each();

      return result === undefined ? nil : result;
    
    });

    Opal.defn(self, '$max_by', TMP_23 = function() {
      var self = this, $iter = TMP_23.$$p, block = $iter || nil;

      TMP_23.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        return self.$enum_for("max_by")
      };
      
      var result,
          by;

      self.$each.$$p = function() {
        var param = $scope.get('Opal').$destructure(arguments),
            value = Opal.yield1(block, param);

        if (result === undefined) {
          result = param;
          by     = value;
          return;
        }

        if (value === $breaker) {
          result = $breaker.$v;
          return $breaker;
        }

        if ((value)['$<=>'](by) > 0) {
          result = param
          by     = value;
        }
      };

      self.$each();

      return result === undefined ? nil : result;
    
    });

    Opal.defn(self, '$member?', def['$include?']);

    Opal.defn(self, '$min', TMP_24 = function() {
      var self = this, $iter = TMP_24.$$p, block = $iter || nil;

      TMP_24.$$p = null;
      
      var result;

      if (block !== nil) {
        self.$each.$$p = function() {
          var param = $scope.get('Opal').$destructure(arguments);

          if (result === undefined) {
            result = param;
            return;
          }

          var value = block(param, result);

          if (value === $breaker) {
            result = $breaker.$v;
            return $breaker;
          }

          if (value === nil) {
            self.$raise($scope.get('ArgumentError'), "comparison failed");
          }

          if (value < 0) {
            result = param;
          }
        };
      }
      else {
        self.$each.$$p = function() {
          var param = $scope.get('Opal').$destructure(arguments);

          if (result === undefined) {
            result = param;
            return;
          }

          if ($scope.get('Opal').$compare(param, result) < 0) {
            result = param;
          }
        };
      }

      self.$each();

      return result === undefined ? nil : result;
    
    });

    Opal.defn(self, '$min_by', TMP_25 = function() {
      var self = this, $iter = TMP_25.$$p, block = $iter || nil;

      TMP_25.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        return self.$enum_for("min_by")
      };
      
      var result,
          by;

      self.$each.$$p = function() {
        var param = $scope.get('Opal').$destructure(arguments),
            value = Opal.yield1(block, param);

        if (result === undefined) {
          result = param;
          by     = value;
          return;
        }

        if (value === $breaker) {
          result = $breaker.$v;
          return $breaker;
        }

        if ((value)['$<=>'](by) < 0) {
          result = param
          by     = value;
        }
      };

      self.$each();

      return result === undefined ? nil : result;
    
    });

    Opal.defn(self, '$minmax', TMP_26 = function() {
      var self = this, $iter = TMP_26.$$p, block = $iter || nil;

      TMP_26.$$p = null;
      return self.$raise($scope.get('NotImplementedError'));
    });

    Opal.defn(self, '$minmax_by', TMP_27 = function() {
      var self = this, $iter = TMP_27.$$p, block = $iter || nil;

      TMP_27.$$p = null;
      return self.$raise($scope.get('NotImplementedError'));
    });

    Opal.defn(self, '$none?', TMP_28 = function() {
      var $a, self = this, $iter = TMP_28.$$p, block = $iter || nil;

      TMP_28.$$p = null;
      
      var result = true;

      if (block !== nil) {
        self.$each.$$p = function() {
          var value = Opal.yieldX(block, arguments);

          if (value === $breaker) {
            result = $breaker.$v;
            return $breaker;
          }

          if ((($a = value) !== nil && (!$a.$$is_boolean || $a == true))) {
            result = false;
            return $breaker;
          }
        }
      }
      else {
        self.$each.$$p = function() {
          var value = $scope.get('Opal').$destructure(arguments);

          if ((($a = value) !== nil && (!$a.$$is_boolean || $a == true))) {
            result = false;
            return $breaker;
          }
        };
      }

      self.$each();

      return result;
    
    });

    Opal.defn(self, '$one?', TMP_29 = function() {
      var $a, self = this, $iter = TMP_29.$$p, block = $iter || nil;

      TMP_29.$$p = null;
      
      var result = false;

      if (block !== nil) {
        self.$each.$$p = function() {
          var value = Opal.yieldX(block, arguments);

          if (value === $breaker) {
            result = $breaker.$v;
            return $breaker;
          }

          if ((($a = value) !== nil && (!$a.$$is_boolean || $a == true))) {
            if (result === true) {
              result = false;
              return $breaker;
            }

            result = true;
          }
        }
      }
      else {
        self.$each.$$p = function() {
          var value = $scope.get('Opal').$destructure(arguments);

          if ((($a = value) !== nil && (!$a.$$is_boolean || $a == true))) {
            if (result === true) {
              result = false;
              return $breaker;
            }

            result = true;
          }
        }
      }

      self.$each();

      return result;
    
    });

    Opal.defn(self, '$partition', TMP_30 = function() {
      var $a, self = this, $iter = TMP_30.$$p, block = $iter || nil;

      TMP_30.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("partition")
      };
      
      var truthy = [], falsy = [], result;

      self.$each.$$p = function() {
        var param = $scope.get('Opal').$destructure(arguments),
            value = Opal.yield1(block, param);

        if (value === $breaker) {
          result = $breaker.$v;
          return $breaker;
        }

        if ((($a = value) !== nil && (!$a.$$is_boolean || $a == true))) {
          truthy.push(param);
        }
        else {
          falsy.push(param);
        }
      };

      self.$each();

      return [truthy, falsy];
    
    });

    Opal.defn(self, '$reduce', def.$inject);

    Opal.defn(self, '$reject', TMP_31 = function() {
      var $a, self = this, $iter = TMP_31.$$p, block = $iter || nil;

      TMP_31.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("reject")
      };
      
      var result = [];

      self.$each.$$p = function() {
        var param = $scope.get('Opal').$destructure(arguments),
            value = Opal.yield1(block, param);

        if (value === $breaker) {
          result = $breaker.$v;
          return $breaker;
        }

        if ((($a = value) === nil || ($a.$$is_boolean && $a == false))) {
          result.push(param);
        }
      };

      self.$each();

      return result;
    
    });

    Opal.defn(self, '$reverse_each', TMP_32 = function() {
      var self = this, $iter = TMP_32.$$p, block = $iter || nil;

      TMP_32.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("reverse_each")
      };
      
      var result = [];

      self.$each.$$p = function() {
        result.push(arguments);
      };

      self.$each();

      for (var i = result.length - 1; i >= 0; i--) {
        Opal.yieldX(block, result[i]);
      }

      return result;
    
    });

    Opal.defn(self, '$select', def.$find_all);

    Opal.defn(self, '$slice_before', TMP_33 = function(pattern) {
      var $a, $b, TMP_34, self = this, $iter = TMP_33.$$p, block = $iter || nil;

      TMP_33.$$p = null;
      if ((($a = pattern === undefined && block === nil || arguments.length > 1) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$raise($scope.get('ArgumentError'), "wrong number of arguments (" + (arguments.length) + " for 1)")};
      return ($a = ($b = $scope.get('Enumerator')).$new, $a.$$p = (TMP_34 = function(e){var self = TMP_34.$$s || this, $a;
if (e == null) e = nil;
      
        var slice = [];

        if (block !== nil) {
          if (pattern === undefined) {
            self.$each.$$p = function() {
              var param = $scope.get('Opal').$destructure(arguments),
                  value = Opal.yield1(block, param);

              if ((($a = value) !== nil && (!$a.$$is_boolean || $a == true)) && slice.length > 0) {
                e['$<<'](slice);
                slice = [];
              }

              slice.push(param);
            };
          }
          else {
            self.$each.$$p = function() {
              var param = $scope.get('Opal').$destructure(arguments),
                  value = block(param, pattern.$dup());

              if ((($a = value) !== nil && (!$a.$$is_boolean || $a == true)) && slice.length > 0) {
                e['$<<'](slice);
                slice = [];
              }

              slice.push(param);
            };
          }
        }
        else {
          self.$each.$$p = function() {
            var param = $scope.get('Opal').$destructure(arguments),
                value = pattern['$==='](param);

            if ((($a = value) !== nil && (!$a.$$is_boolean || $a == true)) && slice.length > 0) {
              e['$<<'](slice);
              slice = [];
            }

            slice.push(param);
          };
        }

        self.$each();

        if (slice.length > 0) {
          e['$<<'](slice);
        }
      ;}, TMP_34.$$s = self, TMP_34), $a).call($b);
    });

    Opal.defn(self, '$sort', TMP_35 = function() {
      var $a, $b, TMP_36, self = this, $iter = TMP_35.$$p, block = $iter || nil, ary = nil;

      TMP_35.$$p = null;
      ary = self.$to_a();
      if ((block !== nil)) {
        } else {
        block = ($a = ($b = self).$lambda, $a.$$p = (TMP_36 = function(a, b){var self = TMP_36.$$s || this;
if (a == null) a = nil;if (b == null) b = nil;
        return a['$<=>'](b)}, TMP_36.$$s = self, TMP_36), $a).call($b)
      };
      return ary.sort(block);
    });

    Opal.defn(self, '$sort_by', TMP_37 = function() {
      var $a, $b, TMP_38, $c, $d, TMP_39, $e, $f, TMP_40, self = this, $iter = TMP_37.$$p, block = $iter || nil;

      TMP_37.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("sort_by")
      };
      return ($a = ($b = ($c = ($d = ($e = ($f = self).$map, $e.$$p = (TMP_40 = function(){var self = TMP_40.$$s || this, arg = nil;

      arg = $scope.get('Opal').$destructure(arguments);
        return [block.$call(arg), arg];}, TMP_40.$$s = self, TMP_40), $e).call($f)).$sort, $c.$$p = (TMP_39 = function(a, b){var self = TMP_39.$$s || this;
if (a == null) a = nil;if (b == null) b = nil;
      return a['$[]'](0)['$<=>'](b['$[]'](0))}, TMP_39.$$s = self, TMP_39), $c).call($d)).$map, $a.$$p = (TMP_38 = function(arg){var self = TMP_38.$$s || this;
if (arg == null) arg = nil;
      return arg[1];}, TMP_38.$$s = self, TMP_38), $a).call($b);
    });

    Opal.defn(self, '$take', function(num) {
      var self = this;

      return self.$first(num);
    });

    Opal.defn(self, '$take_while', TMP_41 = function() {
      var $a, self = this, $iter = TMP_41.$$p, block = $iter || nil;

      TMP_41.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        return self.$enum_for("take_while")
      };
      
      var result = [];

      self.$each.$$p = function() {
        var param = $scope.get('Opal').$destructure(arguments),
            value = Opal.yield1(block, param);

        if (value === $breaker) {
          result = $breaker.$v;
          return $breaker;
        }

        if ((($a = value) === nil || ($a.$$is_boolean && $a == false))) {
          return $breaker;
        }

        result.push(param);
      };

      self.$each();

      return result;
    
    });

    Opal.defn(self, '$to_a', def.$entries);

    Opal.defn(self, '$zip', TMP_42 = function(others) {
      var $a, self = this, $iter = TMP_42.$$p, block = $iter || nil;

      others = $slice.call(arguments, 0);
      TMP_42.$$p = null;
      return ($a = self.$to_a()).$zip.apply($a, [].concat(others));
    });
  })(self)
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/enumerator"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  function $rb_lt(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs < rhs : lhs['$<'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$include', '$allocate', '$new', '$to_proc', '$coerce_to', '$nil?', '$empty?', '$class', '$__send__', '$===', '$call', '$enum_for', '$destructure', '$inspect', '$[]', '$raise', '$yield', '$each', '$enumerator_size', '$respond_to?', '$try_convert', '$for']);
  self.$require("corelib/enumerable");
  return (function($base, $super) {
    function $Enumerator(){};
    var self = $Enumerator = $klass($base, $super, 'Enumerator', $Enumerator);

    var def = self.$$proto, $scope = self.$$scope, TMP_1, TMP_2, TMP_3, TMP_4;

    def.size = def.args = def.object = def.method = nil;
    self.$include($scope.get('Enumerable'));

    Opal.defs(self, '$for', TMP_1 = function(object, method, args) {
      var self = this, $iter = TMP_1.$$p, block = $iter || nil;

      args = $slice.call(arguments, 2);
      if (method == null) {
        method = "each"
      }
      TMP_1.$$p = null;
      
      var obj = self.$allocate();

      obj.object = object;
      obj.size   = block;
      obj.method = method;
      obj.args   = args;

      return obj;
    ;
    });

    def.$initialize = TMP_2 = function() {
      var $a, $b, self = this, $iter = TMP_2.$$p, block = $iter || nil;

      TMP_2.$$p = null;
      if (block !== false && block !== nil) {
        self.object = ($a = ($b = $scope.get('Generator')).$new, $a.$$p = block.$to_proc(), $a).call($b);
        self.method = "each";
        self.args = [];
        self.size = arguments[0] || nil;
        if ((($a = self.size) !== nil && (!$a.$$is_boolean || $a == true))) {
          return self.size = $scope.get('Opal').$coerce_to(self.size, $scope.get('Integer'), "to_int")
          } else {
          return nil
        };
        } else {
        self.object = arguments[0];
        self.method = arguments[1] || "each";
        self.args = $slice.call(arguments, 2);
        return self.size = nil;
      };
    };

    def.$each = TMP_3 = function(args) {
      var $a, $b, $c, self = this, $iter = TMP_3.$$p, block = $iter || nil;

      args = $slice.call(arguments, 0);
      TMP_3.$$p = null;
      if ((($a = ($b = block['$nil?'](), $b !== false && $b !== nil ?args['$empty?']() : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self};
      args = $rb_plus(self.args, args);
      if ((($a = block['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return ($a = self.$class()).$new.apply($a, [self.object, self.method].concat(args))};
      return ($b = ($c = self.object).$__send__, $b.$$p = block.$to_proc(), $b).apply($c, [self.method].concat(args));
    };

    def.$size = function() {
      var $a, self = this;

      if ((($a = $scope.get('Proc')['$==='](self.size)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return ($a = self.size).$call.apply($a, [].concat(self.args))
        } else {
        return self.size
      };
    };

    def.$with_index = TMP_4 = function(offset) {
      var self = this, $iter = TMP_4.$$p, block = $iter || nil;

      if (offset == null) {
        offset = 0
      }
      TMP_4.$$p = null;
      if (offset !== false && offset !== nil) {
        offset = $scope.get('Opal').$coerce_to(offset, $scope.get('Integer'), "to_int")
        } else {
        offset = 0
      };
      if (block !== false && block !== nil) {
        } else {
        return self.$enum_for("with_index", offset)
      };
      
      var result, index = offset;

      self.$each.$$p = function() {
        var param = $scope.get('Opal').$destructure(arguments),
            value = block(param, index);

        if (value === $breaker) {
          result = $breaker.$v;
          return $breaker;
        }

        index++;
      }

      self.$each();

      if (result !== undefined) {
        return result;
      }

      return self.object;
    
    };

    Opal.defn(self, '$with_object', def.$each_with_object);

    def.$inspect = function() {
      var $a, self = this, result = nil;

      result = "#<" + (self.$class()) + ": " + (self.object.$inspect()) + ":" + (self.method);
      if ((($a = self.args['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        result = $rb_plus(result, "(" + (self.args.$inspect()['$[]']($scope.get('Range').$new(1, -2))) + ")")
      };
      return $rb_plus(result, ">");
    };

    (function($base, $super) {
      function $Generator(){};
      var self = $Generator = $klass($base, $super, 'Generator', $Generator);

      var def = self.$$proto, $scope = self.$$scope, TMP_5, TMP_6;

      def.block = nil;
      self.$include($scope.get('Enumerable'));

      def.$initialize = TMP_5 = function() {
        var self = this, $iter = TMP_5.$$p, block = $iter || nil;

        TMP_5.$$p = null;
        if (block !== false && block !== nil) {
          } else {
          self.$raise($scope.get('LocalJumpError'), "no block given")
        };
        return self.block = block;
      };

      return (def.$each = TMP_6 = function(args) {
        var $a, $b, self = this, $iter = TMP_6.$$p, block = $iter || nil, yielder = nil;

        args = $slice.call(arguments, 0);
        TMP_6.$$p = null;
        yielder = ($a = ($b = $scope.get('Yielder')).$new, $a.$$p = block.$to_proc(), $a).call($b);
        
        try {
          args.unshift(yielder);

          if (Opal.yieldX(self.block, args) === $breaker) {
            return $breaker.$v;
          }
        }
        catch (e) {
          if (e === $breaker) {
            return $breaker.$v;
          }
          else {
            throw e;
          }
        }
      ;
        return self;
      }, nil) && 'each';
    })(self, null);

    (function($base, $super) {
      function $Yielder(){};
      var self = $Yielder = $klass($base, $super, 'Yielder', $Yielder);

      var def = self.$$proto, $scope = self.$$scope, TMP_7;

      def.block = nil;
      def.$initialize = TMP_7 = function() {
        var self = this, $iter = TMP_7.$$p, block = $iter || nil;

        TMP_7.$$p = null;
        return self.block = block;
      };

      def.$yield = function(values) {
        var self = this;

        values = $slice.call(arguments, 0);
        
        var value = Opal.yieldX(self.block, values);

        if (value === $breaker) {
          throw $breaker;
        }

        return value;
      ;
      };

      return (def['$<<'] = function(values) {
        var $a, self = this;

        values = $slice.call(arguments, 0);
        ($a = self).$yield.apply($a, [].concat(values));
        return self;
      }, nil) && '<<';
    })(self, null);

    return (function($base, $super) {
      function $Lazy(){};
      var self = $Lazy = $klass($base, $super, 'Lazy', $Lazy);

      var def = self.$$proto, $scope = self.$$scope, TMP_8, TMP_11, TMP_13, TMP_18, TMP_20, TMP_21, TMP_23, TMP_26, TMP_29;

      def.enumerator = nil;
      (function($base, $super) {
        function $StopLazyError(){};
        var self = $StopLazyError = $klass($base, $super, 'StopLazyError', $StopLazyError);

        var def = self.$$proto, $scope = self.$$scope;

        return nil;
      })(self, $scope.get('Exception'));

      def.$initialize = TMP_8 = function(object, size) {
        var TMP_9, self = this, $iter = TMP_8.$$p, block = $iter || nil;

        if (size == null) {
          size = nil
        }
        TMP_8.$$p = null;
        if ((block !== nil)) {
          } else {
          self.$raise($scope.get('ArgumentError'), "tried to call lazy new without a block")
        };
        self.enumerator = object;
        return Opal.find_super_dispatcher(self, 'initialize', TMP_8, (TMP_9 = function(yielder, each_args){var self = TMP_9.$$s || this, $a, $b, TMP_10;
if (yielder == null) yielder = nil;each_args = $slice.call(arguments, 1);
        try {
          return ($a = ($b = object).$each, $a.$$p = (TMP_10 = function(args){var self = TMP_10.$$s || this;
args = $slice.call(arguments, 0);
            
              args.unshift(yielder);

              if (Opal.yieldX(block, args) === $breaker) {
                return $breaker;
              }
            ;}, TMP_10.$$s = self, TMP_10), $a).apply($b, [].concat(each_args))
          } catch ($err) {if (Opal.rescue($err, [$scope.get('Exception')])) {
            return nil
            }else { throw $err; }
          }}, TMP_9.$$s = self, TMP_9)).apply(self, [size]);
      };

      Opal.defn(self, '$force', def.$to_a);

      def.$lazy = function() {
        var self = this;

        return self;
      };

      def.$collect = TMP_11 = function() {
        var $a, $b, TMP_12, self = this, $iter = TMP_11.$$p, block = $iter || nil;

        TMP_11.$$p = null;
        if (block !== false && block !== nil) {
          } else {
          self.$raise($scope.get('ArgumentError'), "tried to call lazy map without a block")
        };
        return ($a = ($b = $scope.get('Lazy')).$new, $a.$$p = (TMP_12 = function(enum$, args){var self = TMP_12.$$s || this;
if (enum$ == null) enum$ = nil;args = $slice.call(arguments, 1);
        
          var value = Opal.yieldX(block, args);

          if (value === $breaker) {
            return $breaker;
          }

          enum$.$yield(value);
        }, TMP_12.$$s = self, TMP_12), $a).call($b, self, self.$enumerator_size());
      };

      def.$collect_concat = TMP_13 = function() {
        var $a, $b, TMP_14, self = this, $iter = TMP_13.$$p, block = $iter || nil;

        TMP_13.$$p = null;
        if (block !== false && block !== nil) {
          } else {
          self.$raise($scope.get('ArgumentError'), "tried to call lazy map without a block")
        };
        return ($a = ($b = $scope.get('Lazy')).$new, $a.$$p = (TMP_14 = function(enum$, args){var self = TMP_14.$$s || this, $a, $b, TMP_15, $c, TMP_16;
if (enum$ == null) enum$ = nil;args = $slice.call(arguments, 1);
        
          var value = Opal.yieldX(block, args);

          if (value === $breaker) {
            return $breaker;
          }

          if ((value)['$respond_to?']("force") && (value)['$respond_to?']("each")) {
            ($a = ($b = (value)).$each, $a.$$p = (TMP_15 = function(v){var self = TMP_15.$$s || this;
if (v == null) v = nil;
          return enum$.$yield(v)}, TMP_15.$$s = self, TMP_15), $a).call($b)
          }
          else {
            var array = $scope.get('Opal').$try_convert(value, $scope.get('Array'), "to_ary");

            if (array === nil) {
              enum$.$yield(value);
            }
            else {
              ($a = ($c = (value)).$each, $a.$$p = (TMP_16 = function(v){var self = TMP_16.$$s || this;
if (v == null) v = nil;
          return enum$.$yield(v)}, TMP_16.$$s = self, TMP_16), $a).call($c);
            }
          }
        ;}, TMP_14.$$s = self, TMP_14), $a).call($b, self, nil);
      };

      def.$drop = function(n) {
        var $a, $b, TMP_17, self = this, current_size = nil, set_size = nil, dropped = nil;

        n = $scope.get('Opal').$coerce_to(n, $scope.get('Integer'), "to_int");
        if ($rb_lt(n, 0)) {
          self.$raise($scope.get('ArgumentError'), "attempt to drop negative size")};
        current_size = self.$enumerator_size();
        set_size = (function() {if ((($a = $scope.get('Integer')['$==='](current_size)) !== nil && (!$a.$$is_boolean || $a == true))) {
          if ($rb_lt(n, current_size)) {
            return n
            } else {
            return current_size
          }
          } else {
          return current_size
        }; return nil; })();
        dropped = 0;
        return ($a = ($b = $scope.get('Lazy')).$new, $a.$$p = (TMP_17 = function(enum$, args){var self = TMP_17.$$s || this, $a;
if (enum$ == null) enum$ = nil;args = $slice.call(arguments, 1);
        if ($rb_lt(dropped, n)) {
            return dropped = $rb_plus(dropped, 1)
            } else {
            return ($a = enum$).$yield.apply($a, [].concat(args))
          }}, TMP_17.$$s = self, TMP_17), $a).call($b, self, set_size);
      };

      def.$drop_while = TMP_18 = function() {
        var $a, $b, TMP_19, self = this, $iter = TMP_18.$$p, block = $iter || nil, succeeding = nil;

        TMP_18.$$p = null;
        if (block !== false && block !== nil) {
          } else {
          self.$raise($scope.get('ArgumentError'), "tried to call lazy drop_while without a block")
        };
        succeeding = true;
        return ($a = ($b = $scope.get('Lazy')).$new, $a.$$p = (TMP_19 = function(enum$, args){var self = TMP_19.$$s || this, $a, $b;
if (enum$ == null) enum$ = nil;args = $slice.call(arguments, 1);
        if (succeeding !== false && succeeding !== nil) {
            
            var value = Opal.yieldX(block, args);

            if (value === $breaker) {
              return $breaker;
            }

            if ((($a = value) === nil || ($a.$$is_boolean && $a == false))) {
              succeeding = false;

              ($a = enum$).$yield.apply($a, [].concat(args));
            }
          
            } else {
            return ($b = enum$).$yield.apply($b, [].concat(args))
          }}, TMP_19.$$s = self, TMP_19), $a).call($b, self, nil);
      };

      def.$enum_for = TMP_20 = function(method, args) {
        var $a, $b, self = this, $iter = TMP_20.$$p, block = $iter || nil;

        args = $slice.call(arguments, 1);
        if (method == null) {
          method = "each"
        }
        TMP_20.$$p = null;
        return ($a = ($b = self.$class()).$for, $a.$$p = block.$to_proc(), $a).apply($b, [self, method].concat(args));
      };

      def.$find_all = TMP_21 = function() {
        var $a, $b, TMP_22, self = this, $iter = TMP_21.$$p, block = $iter || nil;

        TMP_21.$$p = null;
        if (block !== false && block !== nil) {
          } else {
          self.$raise($scope.get('ArgumentError'), "tried to call lazy select without a block")
        };
        return ($a = ($b = $scope.get('Lazy')).$new, $a.$$p = (TMP_22 = function(enum$, args){var self = TMP_22.$$s || this, $a;
if (enum$ == null) enum$ = nil;args = $slice.call(arguments, 1);
        
          var value = Opal.yieldX(block, args);

          if (value === $breaker) {
            return $breaker;
          }

          if ((($a = value) !== nil && (!$a.$$is_boolean || $a == true))) {
            ($a = enum$).$yield.apply($a, [].concat(args));
          }
        ;}, TMP_22.$$s = self, TMP_22), $a).call($b, self, nil);
      };

      Opal.defn(self, '$flat_map', def.$collect_concat);

      def.$grep = TMP_23 = function(pattern) {
        var $a, $b, TMP_24, $c, TMP_25, self = this, $iter = TMP_23.$$p, block = $iter || nil;

        TMP_23.$$p = null;
        if (block !== false && block !== nil) {
          return ($a = ($b = $scope.get('Lazy')).$new, $a.$$p = (TMP_24 = function(enum$, args){var self = TMP_24.$$s || this, $a;
if (enum$ == null) enum$ = nil;args = $slice.call(arguments, 1);
          
            var param = $scope.get('Opal').$destructure(args),
                value = pattern['$==='](param);

            if ((($a = value) !== nil && (!$a.$$is_boolean || $a == true))) {
              value = Opal.yield1(block, param);

              if (value === $breaker) {
                return $breaker;
              }

              enum$.$yield(Opal.yield1(block, param));
            }
          ;}, TMP_24.$$s = self, TMP_24), $a).call($b, self, nil)
          } else {
          return ($a = ($c = $scope.get('Lazy')).$new, $a.$$p = (TMP_25 = function(enum$, args){var self = TMP_25.$$s || this, $a;
if (enum$ == null) enum$ = nil;args = $slice.call(arguments, 1);
          
            var param = $scope.get('Opal').$destructure(args),
                value = pattern['$==='](param);

            if ((($a = value) !== nil && (!$a.$$is_boolean || $a == true))) {
              enum$.$yield(param);
            }
          ;}, TMP_25.$$s = self, TMP_25), $a).call($c, self, nil)
        };
      };

      Opal.defn(self, '$map', def.$collect);

      Opal.defn(self, '$select', def.$find_all);

      def.$reject = TMP_26 = function() {
        var $a, $b, TMP_27, self = this, $iter = TMP_26.$$p, block = $iter || nil;

        TMP_26.$$p = null;
        if (block !== false && block !== nil) {
          } else {
          self.$raise($scope.get('ArgumentError'), "tried to call lazy reject without a block")
        };
        return ($a = ($b = $scope.get('Lazy')).$new, $a.$$p = (TMP_27 = function(enum$, args){var self = TMP_27.$$s || this, $a;
if (enum$ == null) enum$ = nil;args = $slice.call(arguments, 1);
        
          var value = Opal.yieldX(block, args);

          if (value === $breaker) {
            return $breaker;
          }

          if ((($a = value) === nil || ($a.$$is_boolean && $a == false))) {
            ($a = enum$).$yield.apply($a, [].concat(args));
          }
        ;}, TMP_27.$$s = self, TMP_27), $a).call($b, self, nil);
      };

      def.$take = function(n) {
        var $a, $b, TMP_28, self = this, current_size = nil, set_size = nil, taken = nil;

        n = $scope.get('Opal').$coerce_to(n, $scope.get('Integer'), "to_int");
        if ($rb_lt(n, 0)) {
          self.$raise($scope.get('ArgumentError'), "attempt to take negative size")};
        current_size = self.$enumerator_size();
        set_size = (function() {if ((($a = $scope.get('Integer')['$==='](current_size)) !== nil && (!$a.$$is_boolean || $a == true))) {
          if ($rb_lt(n, current_size)) {
            return n
            } else {
            return current_size
          }
          } else {
          return current_size
        }; return nil; })();
        taken = 0;
        return ($a = ($b = $scope.get('Lazy')).$new, $a.$$p = (TMP_28 = function(enum$, args){var self = TMP_28.$$s || this, $a;
if (enum$ == null) enum$ = nil;args = $slice.call(arguments, 1);
        if ($rb_lt(taken, n)) {
            ($a = enum$).$yield.apply($a, [].concat(args));
            return taken = $rb_plus(taken, 1);
            } else {
            return self.$raise($scope.get('StopLazyError'))
          }}, TMP_28.$$s = self, TMP_28), $a).call($b, self, set_size);
      };

      def.$take_while = TMP_29 = function() {
        var $a, $b, TMP_30, self = this, $iter = TMP_29.$$p, block = $iter || nil;

        TMP_29.$$p = null;
        if (block !== false && block !== nil) {
          } else {
          self.$raise($scope.get('ArgumentError'), "tried to call lazy take_while without a block")
        };
        return ($a = ($b = $scope.get('Lazy')).$new, $a.$$p = (TMP_30 = function(enum$, args){var self = TMP_30.$$s || this, $a;
if (enum$ == null) enum$ = nil;args = $slice.call(arguments, 1);
        
          var value = Opal.yieldX(block, args);

          if (value === $breaker) {
            return $breaker;
          }

          if ((($a = value) !== nil && (!$a.$$is_boolean || $a == true))) {
            ($a = enum$).$yield.apply($a, [].concat(args));
          }
          else {
            self.$raise($scope.get('StopLazyError'));
          }
        ;}, TMP_30.$$s = self, TMP_30), $a).call($b, self, nil);
      };

      Opal.defn(self, '$to_enum', def.$enum_for);

      return (def.$inspect = function() {
        var self = this;

        return "#<" + (self.$class()) + ": " + (self.enumerator.$inspect()) + ">";
      }, nil) && 'inspect';
    })(self, self);
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/array"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_gt(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs > rhs : lhs['$>'](rhs);
  }
  function $rb_lt(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs < rhs : lhs['$<'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $gvars = Opal.gvars, $range = Opal.range, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$include', '$new', '$class', '$raise', '$===', '$to_a', '$respond_to?', '$to_ary', '$coerce_to', '$coerce_to?', '$==', '$to_str', '$clone', '$hash', '$<=>', '$object_id', '$inspect', '$enum_for', '$empty?', '$nil?', '$coerce_to!', '$initialize_clone', '$initialize_dup', '$replace', '$eql?', '$length', '$begin', '$end', '$exclude_end?', '$flatten', '$__id__', '$[]', '$to_s', '$join', '$delete_if', '$to_proc', '$each', '$reverse', '$frozen?', '$rotate', '$!', '$map', '$rand', '$keep_if', '$shuffle!', '$sort', '$times', '$[]=', '$<<', '$at', '$kind_of?', '$last', '$first', '$upto']);
  self.$require("corelib/enumerable");
  return (function($base, $super) {
    function $Array(){};
    var self = $Array = $klass($base, $super, 'Array', $Array);

    var def = self.$$proto, $scope = self.$$scope, TMP_1, TMP_2, TMP_3, TMP_4, TMP_5, TMP_6, TMP_7, TMP_8, TMP_9, TMP_10, TMP_11, TMP_12, TMP_13, TMP_14, TMP_15, TMP_16, TMP_17, TMP_18, TMP_19, TMP_21, TMP_22, TMP_23, TMP_24, TMP_25, TMP_30;

    def.length = nil;
    self.$include($scope.get('Enumerable'));

    def.$$is_array = true;

    Opal.defs(self, '$[]', function(objects) {
      var self = this;

      objects = $slice.call(arguments, 0);
      return objects;
    });

    def.$initialize = function(args) {
      var $a, self = this;

      args = $slice.call(arguments, 0);
      return ($a = self.$class()).$new.apply($a, [].concat(args));
    };

    Opal.defs(self, '$new', TMP_1 = function(size, obj) {
      var $a, self = this, $iter = TMP_1.$$p, block = $iter || nil;

      if (size == null) {
        size = nil
      }
      if (obj == null) {
        obj = nil
      }
      TMP_1.$$p = null;
      if ((($a = arguments.length > 2) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$raise($scope.get('ArgumentError'), "wrong number of arguments (" + (arguments.length) + " for 0..2)")};
      if ((($a = arguments.length === 0) !== nil && (!$a.$$is_boolean || $a == true))) {
        return []};
      if ((($a = arguments.length === 1) !== nil && (!$a.$$is_boolean || $a == true))) {
        if ((($a = $scope.get('Array')['$==='](size)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return size.$to_a()
        } else if ((($a = size['$respond_to?']("to_ary")) !== nil && (!$a.$$is_boolean || $a == true))) {
          return size.$to_ary()}};
      size = $scope.get('Opal').$coerce_to(size, $scope.get('Integer'), "to_int");
      if ((($a = size < 0) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$raise($scope.get('ArgumentError'), "negative array size")};
      
      var result = [];

      if (block === nil) {
        for (var i = 0; i < size; i++) {
          result.push(obj);
        }
      }
      else {
        for (var i = 0, value; i < size; i++) {
          value = block(i);

          if (value === $breaker) {
            return $breaker.$v;
          }

          result[i] = value;
        }
      }

      return result;
    
    });

    Opal.defs(self, '$try_convert', function(obj) {
      var self = this;

      return $scope.get('Opal')['$coerce_to?'](obj, $scope.get('Array'), "to_ary");
    });

    def['$&'] = function(other) {
      var $a, self = this;

      if ((($a = $scope.get('Array')['$==='](other)) !== nil && (!$a.$$is_boolean || $a == true))) {
        other = other.$to_a()
        } else {
        other = $scope.get('Opal').$coerce_to(other, $scope.get('Array'), "to_ary").$to_a()
      };
      
      var result = [],
          seen   = {};

      for (var i = 0, length = self.length; i < length; i++) {
        var item = self[i];

        if (!seen[item]) {
          for (var j = 0, length2 = other.length; j < length2; j++) {
            var item2 = other[j];

            if (!seen[item2] && (item)['$=='](item2)) {
              seen[item] = true;
              result.push(item);
            }
          }
        }
      }

      return result;
    
    };

    def['$|'] = function(other) {
      var $a, self = this;

      if ((($a = $scope.get('Array')['$==='](other)) !== nil && (!$a.$$is_boolean || $a == true))) {
        other = other.$to_a()
        } else {
        other = $scope.get('Opal').$coerce_to(other, $scope.get('Array'), "to_ary").$to_a()
      };
      
      var result = [],
          seen   = {};

      for (var i = 0, length = self.length; i < length; i++) {
        var item = self[i];

        if (!seen[item]) {
          seen[item] = true;
          result.push(item);
        }
      }

      for (var i = 0, length = other.length; i < length; i++) {
        var item = other[i];

        if (!seen[item]) {
          seen[item] = true;
          result.push(item);
        }
      }
      return result;
    
    };

    def['$*'] = function(other) {
      var $a, self = this;

      if ((($a = other['$respond_to?']("to_str")) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.join(other.$to_str())};
      if ((($a = other['$respond_to?']("to_int")) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise($scope.get('TypeError'), "no implicit conversion of " + (other.$class()) + " into Integer")
      };
      other = $scope.get('Opal').$coerce_to(other, $scope.get('Integer'), "to_int");
      if ((($a = other < 0) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$raise($scope.get('ArgumentError'), "negative argument")};
      
      var result = [];

      for (var i = 0; i < other; i++) {
        result = result.concat(self);
      }

      return result;
    
    };

    def['$+'] = function(other) {
      var $a, self = this;

      if ((($a = $scope.get('Array')['$==='](other)) !== nil && (!$a.$$is_boolean || $a == true))) {
        other = other.$to_a()
        } else {
        other = $scope.get('Opal').$coerce_to(other, $scope.get('Array'), "to_ary").$to_a()
      };
      return self.concat(other);
    };

    def['$-'] = function(other) {
      var $a, self = this;

      if ((($a = $scope.get('Array')['$==='](other)) !== nil && (!$a.$$is_boolean || $a == true))) {
        other = other.$to_a()
        } else {
        other = $scope.get('Opal').$coerce_to(other, $scope.get('Array'), "to_ary").$to_a()
      };
      if ((($a = self.length === 0) !== nil && (!$a.$$is_boolean || $a == true))) {
        return []};
      if ((($a = other.length === 0) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.$clone()};
      
      var seen   = {},
          result = [];

      for (var i = 0, length = other.length; i < length; i++) {
        seen[other[i]] = true;
      }

      for (var i = 0, length = self.length; i < length; i++) {
        var item = self[i];

        if (!seen[item]) {
          result.push(item);
        }
      }

      return result;
    
    };

    def['$<<'] = function(object) {
      var self = this;

      self.push(object);
      return self;
    };

    def['$<=>'] = function(other) {
      var $a, self = this;

      if ((($a = $scope.get('Array')['$==='](other)) !== nil && (!$a.$$is_boolean || $a == true))) {
        other = other.$to_a()
      } else if ((($a = other['$respond_to?']("to_ary")) !== nil && (!$a.$$is_boolean || $a == true))) {
        other = other.$to_ary().$to_a()
        } else {
        return nil
      };
      
      if (self.$hash() === other.$hash()) {
        return 0;
      }

      if (self.length != other.length) {
        return (self.length > other.length) ? 1 : -1;
      }

      for (var i = 0, length = self.length; i < length; i++) {
        var tmp = (self[i])['$<=>'](other[i]);

        if (tmp !== 0) {
          return tmp;
        }
      }

      return 0;
    ;
    };

    def['$=='] = function(other) {
      var self = this;

      
      var recursed = {};

      function _eqeq(array, other) {
        var i, length, a, b;

        if (!other.$$is_array) {
          if ($scope.get('Opal')['$respond_to?'](other, "to_ary")) {
            return (other)['$=='](array);
          } else {
            return false;
          }
        }

        other = other.$to_a();

        if (array.length !== other.length) {
          return false;
        }

        recursed[(array).$object_id()] = true;

        for (i = 0, length = array.length; i < length; i++) {
          a = array[i];
          b = other[i];
          if (a.$$is_array) {
            if (b.$$is_array && b.length !== a.length) {
              return false;
            }
            if (!recursed.hasOwnProperty((a).$object_id())) {
              if (!_eqeq(a, b)) {
                return false;
              }
            }
          } else {
            if (!(a)['$=='](b)) {
              return false;
            }
          }
        }

        return true;
      }

      return _eqeq(self, other);
    ;
    };

    def['$[]'] = function(index, length) {
      var $a, self = this;

      if ((($a = $scope.get('Range')['$==='](index)) !== nil && (!$a.$$is_boolean || $a == true))) {
        
        var size    = self.length,
            exclude = index.exclude,
            from    = $scope.get('Opal').$coerce_to(index.begin, $scope.get('Integer'), "to_int"),
            to      = $scope.get('Opal').$coerce_to(index.end, $scope.get('Integer'), "to_int");

        if (from < 0) {
          from += size;

          if (from < 0) {
            return nil;
          }
        }

        if (from > size) {
          return nil;
        }

        if (to < 0) {
          to += size;

          if (to < 0) {
            return [];
          }
        }

        if (!exclude) {
          to += 1;
        }

        return self.slice(from, to);
      ;
        } else {
        index = $scope.get('Opal').$coerce_to(index, $scope.get('Integer'), "to_int");
        
        var size = self.length;

        if (index < 0) {
          index += size;

          if (index < 0) {
            return nil;
          }
        }

        if (length === undefined) {
          if (index >= size || index < 0) {
            return nil;
          }

          return self[index];
        }
        else {
          length = $scope.get('Opal').$coerce_to(length, $scope.get('Integer'), "to_int");

          if (length < 0 || index > size || index < 0) {
            return nil;
          }

          return self.slice(index, index + length);
        }
      
      };
    };

    def['$[]='] = function(index, value, extra) {
      var $a, self = this, data = nil, length = nil;

      if ((($a = $scope.get('Range')['$==='](index)) !== nil && (!$a.$$is_boolean || $a == true))) {
        if ((($a = $scope.get('Array')['$==='](value)) !== nil && (!$a.$$is_boolean || $a == true))) {
          data = value.$to_a()
        } else if ((($a = value['$respond_to?']("to_ary")) !== nil && (!$a.$$is_boolean || $a == true))) {
          data = value.$to_ary().$to_a()
          } else {
          data = [value]
        };
        
        var size    = self.length,
            exclude = index.exclude,
            from    = $scope.get('Opal').$coerce_to(index.begin, $scope.get('Integer'), "to_int"),
            to      = $scope.get('Opal').$coerce_to(index.end, $scope.get('Integer'), "to_int");

        if (from < 0) {
          from += size;

          if (from < 0) {
            self.$raise($scope.get('RangeError'), "" + (index.$inspect()) + " out of range");
          }
        }

        if (to < 0) {
          to += size;
        }

        if (!exclude) {
          to += 1;
        }

        if (from > size) {
          for (var i = size; i < from; i++) {
            self[i] = nil;
          }
        }

        if (to < 0) {
          self.splice.apply(self, [from, 0].concat(data));
        }
        else {
          self.splice.apply(self, [from, to - from].concat(data));
        }

        return value;
      ;
        } else {
        if ((($a = extra === undefined) !== nil && (!$a.$$is_boolean || $a == true))) {
          length = 1
          } else {
          length = value;
          value = extra;
          if ((($a = $scope.get('Array')['$==='](value)) !== nil && (!$a.$$is_boolean || $a == true))) {
            data = value.$to_a()
          } else if ((($a = value['$respond_to?']("to_ary")) !== nil && (!$a.$$is_boolean || $a == true))) {
            data = value.$to_ary().$to_a()
            } else {
            data = [value]
          };
        };
        
        var size   = self.length,
            index  = $scope.get('Opal').$coerce_to(index, $scope.get('Integer'), "to_int"),
            length = $scope.get('Opal').$coerce_to(length, $scope.get('Integer'), "to_int"),
            old;

        if (index < 0) {
          old    = index;
          index += size;

          if (index < 0) {
            self.$raise($scope.get('IndexError'), "index " + (old) + " too small for array; minimum " + (-self.length));
          }
        }

        if (length < 0) {
          self.$raise($scope.get('IndexError'), "negative length (" + (length) + ")")
        }

        if (index > size) {
          for (var i = size; i < index; i++) {
            self[i] = nil;
          }
        }

        if (extra === undefined) {
          self[index] = value;
        }
        else {
          self.splice.apply(self, [index, length].concat(data));
        }

        return value;
      ;
      };
    };

    def.$assoc = function(object) {
      var self = this;

      
      for (var i = 0, length = self.length, item; i < length; i++) {
        if (item = self[i], item.length && (item[0])['$=='](object)) {
          return item;
        }
      }

      return nil;
    
    };

    def.$at = function(index) {
      var self = this;

      index = $scope.get('Opal').$coerce_to(index, $scope.get('Integer'), "to_int");
      
      if (index < 0) {
        index += self.length;
      }

      if (index < 0 || index >= self.length) {
        return nil;
      }

      return self[index];
    
    };

    def.$bsearch = TMP_2 = function() {
      var self = this, $iter = TMP_2.$$p, block = $iter || nil;

      TMP_2.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("bsearch")
      };
      
      var min = 0,
          max = self.length,
          mid,
          val,
          ret,
          smaller = false,
          satisfied = nil;

      while (min < max) {
        mid = min + Math.floor((max - min) / 2);
        val = self[mid];
        ret = block(val);

        if (ret === $breaker) {
          return $breaker.$v;
        }
        else if (ret === true) {
          satisfied = val;
          smaller = true;
        }
        else if (ret === false || ret === nil) {
          smaller = false;
        }
        else if (ret.$$is_number) {
          if (ret === 0) { return val; }
          smaller = (ret < 0);
        }
        else {
          self.$raise($scope.get('TypeError'), "wrong argument type " + ((ret).$class()) + " (must be numeric, true, false or nil)")
        }

        if (smaller) { max = mid; } else { min = mid + 1; }
      }

      return satisfied;
    
    };

    def.$cycle = TMP_3 = function(n) {
      var $a, $b, self = this, $iter = TMP_3.$$p, block = $iter || nil;

      if (n == null) {
        n = nil
      }
      TMP_3.$$p = null;
      if ((($a = ((($b = self['$empty?']()) !== false && $b !== nil) ? $b : n['$=='](0))) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil};
      if (block !== false && block !== nil) {
        } else {
        return self.$enum_for("cycle", n)
      };
      if ((($a = n['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        
        while (true) {
          for (var i = 0, length = self.length; i < length; i++) {
            var value = Opal.yield1(block, self[i]);

            if (value === $breaker) {
              return $breaker.$v;
            }
          }
        }
      
        } else {
        n = $scope.get('Opal')['$coerce_to!'](n, $scope.get('Integer'), "to_int");
        
        if (n <= 0) {
          return self;
        }

        while (n > 0) {
          for (var i = 0, length = self.length; i < length; i++) {
            var value = Opal.yield1(block, self[i]);

            if (value === $breaker) {
              return $breaker.$v;
            }
          }

          n--;
        }
      
      };
      return self;
    };

    def.$clear = function() {
      var self = this;

      self.splice(0, self.length);
      return self;
    };

    def.$clone = function() {
      var self = this, copy = nil;

      copy = [];
      copy.$initialize_clone(self);
      return copy;
    };

    def.$dup = function() {
      var self = this, copy = nil;

      copy = [];
      copy.$initialize_dup(self);
      return copy;
    };

    def.$initialize_copy = function(other) {
      var self = this;

      return self.$replace(other);
    };

    def.$collect = TMP_4 = function() {
      var self = this, $iter = TMP_4.$$p, block = $iter || nil;

      TMP_4.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("collect")
      };
      
      var result = [];

      for (var i = 0, length = self.length; i < length; i++) {
        var value = Opal.yield1(block, self[i]);

        if (value === $breaker) {
          return $breaker.$v;
        }

        result.push(value);
      }

      return result;
    
    };

    def['$collect!'] = TMP_5 = function() {
      var self = this, $iter = TMP_5.$$p, block = $iter || nil;

      TMP_5.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("collect!")
      };
      
      for (var i = 0, length = self.length; i < length; i++) {
        var value = Opal.yield1(block, self[i]);

        if (value === $breaker) {
          return $breaker.$v;
        }

        self[i] = value;
      }
    
      return self;
    };

    def.$combination = TMP_6 = function(n) {
      var $a, self = this, $iter = TMP_6.$$p, $yield = $iter || nil, num = nil;

      TMP_6.$$p = null;
      num = $scope.get('Opal')['$coerce_to!'](n, $scope.get('Integer'), "to_int");
      if (($yield !== nil)) {
        } else {
        return self.$enum_for("combination", num)
      };
      
      var i, length, stack, chosen, lev, done, next;

      if (num === 0) {
        ((($a = Opal.yield1($yield, [])) === $breaker) ? $breaker.$v : $a)
      } else if (num === 1) {
        for (i = 0, length = self.length; i < length; i++) {
          ((($a = Opal.yield1($yield, [self[i]])) === $breaker) ? $breaker.$v : $a)
        }
      }
      else if (num === self.length) {
        ((($a = Opal.yield1($yield, self.slice())) === $breaker) ? $breaker.$v : $a)
      }
      else if (num >= 0 && num < self.length) {
        stack = [];
        for (i = 0; i <= num + 1; i++) {
          stack.push(0);
        }

        chosen = [];
        lev = 0;
        done = false;
        stack[0] = -1;

        while (!done) {
          chosen[lev] = self[stack[lev+1]];
          while (lev < num - 1) {
            lev++;
            next = stack[lev+1] = stack[lev] + 1;
            chosen[lev] = self[next];
          }
          ((($a = Opal.yield1($yield, chosen.slice())) === $breaker) ? $breaker.$v : $a)
          lev++;
          do {
            done = (lev === 0);
            stack[lev]++;
            lev--;
          } while ( stack[lev+1] + num === self.length + lev + 1 );
        }
      }
    ;
      return self;
    };

    def.$compact = function() {
      var self = this;

      
      var result = [];

      for (var i = 0, length = self.length, item; i < length; i++) {
        if ((item = self[i]) !== nil) {
          result.push(item);
        }
      }

      return result;
    
    };

    def['$compact!'] = function() {
      var self = this;

      
      var original = self.length;

      for (var i = 0, length = self.length; i < length; i++) {
        if (self[i] === nil) {
          self.splice(i, 1);

          length--;
          i--;
        }
      }

      return self.length === original ? nil : self;
    
    };

    def.$concat = function(other) {
      var $a, self = this;

      if ((($a = $scope.get('Array')['$==='](other)) !== nil && (!$a.$$is_boolean || $a == true))) {
        other = other.$to_a()
        } else {
        other = $scope.get('Opal').$coerce_to(other, $scope.get('Array'), "to_ary").$to_a()
      };
      
      for (var i = 0, length = other.length; i < length; i++) {
        self.push(other[i]);
      }
    
      return self;
    };

    def.$delete = TMP_7 = function(object) {
      var $a, self = this, $iter = TMP_7.$$p, $yield = $iter || nil;

      TMP_7.$$p = null;
      
      var original = self.length;

      for (var i = 0, length = original; i < length; i++) {
        if ((self[i])['$=='](object)) {
          self.splice(i, 1);

          length--;
          i--;
        }
      }

      if (self.length === original) {
        if (($yield !== nil)) {
          return ((($a = Opal.yieldX($yield, [])) === $breaker) ? $breaker.$v : $a);
        }
        return nil;
      }
      return object;
    ;
    };

    def.$delete_at = function(index) {
      var self = this;

      
      index = $scope.get('Opal').$coerce_to(index, $scope.get('Integer'), "to_int");

      if (index < 0) {
        index += self.length;
      }

      if (index < 0 || index >= self.length) {
        return nil;
      }

      var result = self[index];

      self.splice(index, 1);

      return result;
    ;
    };

    def.$delete_if = TMP_8 = function() {
      var self = this, $iter = TMP_8.$$p, block = $iter || nil;

      TMP_8.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("delete_if")
      };
      
      for (var i = 0, length = self.length, value; i < length; i++) {
        if ((value = block(self[i])) === $breaker) {
          return $breaker.$v;
        }

        if (value !== false && value !== nil) {
          self.splice(i, 1);

          length--;
          i--;
        }
      }
    
      return self;
    };

    def.$drop = function(number) {
      var self = this;

      
      if (number < 0) {
        self.$raise($scope.get('ArgumentError'))
      }

      return self.slice(number);
    ;
    };

    Opal.defn(self, '$dup', def.$clone);

    def.$each = TMP_9 = function() {
      var self = this, $iter = TMP_9.$$p, block = $iter || nil;

      TMP_9.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("each")
      };
      
      for (var i = 0, length = self.length; i < length; i++) {
        var value = Opal.yield1(block, self[i]);

        if (value == $breaker) {
          return $breaker.$v;
        }
      }
    
      return self;
    };

    def.$each_index = TMP_10 = function() {
      var self = this, $iter = TMP_10.$$p, block = $iter || nil;

      TMP_10.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("each_index")
      };
      
      for (var i = 0, length = self.length; i < length; i++) {
        var value = Opal.yield1(block, i);

        if (value === $breaker) {
          return $breaker.$v;
        }
      }
    
      return self;
    };

    def['$empty?'] = function() {
      var self = this;

      return self.length === 0;
    };

    def['$eql?'] = function(other) {
      var self = this;

      
      var recursed = {};

      function _eql(array, other) {
        var i, length, a, b;

        if (!other.$$is_array) {
          return false;
        }

        other = other.$to_a();

        if (array.length !== other.length) {
          return false;
        }

        recursed[(array).$object_id()] = true;

        for (i = 0, length = array.length; i < length; i++) {
          a = array[i];
          b = other[i];
          if (a.$$is_array) {
            if (b.$$is_array && b.length !== a.length) {
              return false;
            }
            if (!recursed.hasOwnProperty((a).$object_id())) {
              if (!_eql(a, b)) {
                return false;
              }
            }
          } else {
            if (!(a)['$eql?'](b)) {
              return false;
            }
          }
        }

        return true;
      }

      return _eql(self, other);
    
    };

    def.$fetch = TMP_11 = function(index, defaults) {
      var self = this, $iter = TMP_11.$$p, block = $iter || nil;

      TMP_11.$$p = null;
      
      var original = index;

      index = $scope.get('Opal').$coerce_to(index, $scope.get('Integer'), "to_int");

      if (index < 0) {
        index += self.length;
      }

      if (index >= 0 && index < self.length) {
        return self[index];
      }

      if (block !== nil) {
        return block(original);
      }

      if (defaults != null) {
        return defaults;
      }

      if (self.length === 0) {
        self.$raise($scope.get('IndexError'), "index " + (original) + " outside of array bounds: 0...0")
      }
      else {
        self.$raise($scope.get('IndexError'), "index " + (original) + " outside of array bounds: -" + (self.length) + "..." + (self.length));
      }
    ;
    };

    def.$fill = TMP_12 = function(args) {
      var $a, self = this, $iter = TMP_12.$$p, block = $iter || nil, one = nil, two = nil, obj = nil, left = nil, right = nil;

      args = $slice.call(arguments, 0);
      TMP_12.$$p = null;
      if (block !== false && block !== nil) {
        if ((($a = args.length > 2) !== nil && (!$a.$$is_boolean || $a == true))) {
          self.$raise($scope.get('ArgumentError'), "wrong number of arguments (" + (args.$length()) + " for 0..2)")};
        $a = Opal.to_ary(args), one = ($a[0] == null ? nil : $a[0]), two = ($a[1] == null ? nil : $a[1]);
        } else {
        if ((($a = args.length == 0) !== nil && (!$a.$$is_boolean || $a == true))) {
          self.$raise($scope.get('ArgumentError'), "wrong number of arguments (0 for 1..3)")
        } else if ((($a = args.length > 3) !== nil && (!$a.$$is_boolean || $a == true))) {
          self.$raise($scope.get('ArgumentError'), "wrong number of arguments (" + (args.$length()) + " for 1..3)")};
        $a = Opal.to_ary(args), obj = ($a[0] == null ? nil : $a[0]), one = ($a[1] == null ? nil : $a[1]), two = ($a[2] == null ? nil : $a[2]);
      };
      if ((($a = $scope.get('Range')['$==='](one)) !== nil && (!$a.$$is_boolean || $a == true))) {
        if (two !== false && two !== nil) {
          self.$raise($scope.get('TypeError'), "length invalid with range")};
        left = $scope.get('Opal').$coerce_to(one.$begin(), $scope.get('Integer'), "to_int");
        if ((($a = left < 0) !== nil && (!$a.$$is_boolean || $a == true))) {
          left += self.length;};
        if ((($a = left < 0) !== nil && (!$a.$$is_boolean || $a == true))) {
          self.$raise($scope.get('RangeError'), "" + (one.$inspect()) + " out of range")};
        right = $scope.get('Opal').$coerce_to(one.$end(), $scope.get('Integer'), "to_int");
        if ((($a = right < 0) !== nil && (!$a.$$is_boolean || $a == true))) {
          right += self.length;};
        if ((($a = one['$exclude_end?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
          } else {
          right += 1;
        };
        if ((($a = right <= left) !== nil && (!$a.$$is_boolean || $a == true))) {
          return self};
      } else if (one !== false && one !== nil) {
        left = $scope.get('Opal').$coerce_to(one, $scope.get('Integer'), "to_int");
        if ((($a = left < 0) !== nil && (!$a.$$is_boolean || $a == true))) {
          left += self.length;};
        if ((($a = left < 0) !== nil && (!$a.$$is_boolean || $a == true))) {
          left = 0};
        if (two !== false && two !== nil) {
          right = $scope.get('Opal').$coerce_to(two, $scope.get('Integer'), "to_int");
          if ((($a = right == 0) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self};
          right += left;
          } else {
          right = self.length
        };
        } else {
        left = 0;
        right = self.length;
      };
      if ((($a = left > self.length) !== nil && (!$a.$$is_boolean || $a == true))) {
        
        for (var i = self.length; i < right; i++) {
          self[i] = nil;
        }
      ;};
      if ((($a = right > self.length) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.length = right};
      if (block !== false && block !== nil) {
        
        for (var length = self.length; left < right; left++) {
          var value = block(left);

          if (value === $breaker) {
            return $breaker.$v;
          }

          self[left] = value;
        }
      ;
        } else {
        
        for (var length = self.length; left < right; left++) {
          self[left] = obj;
        }
      ;
      };
      return self;
    };

    def.$first = function(count) {
      var self = this;

      
      if (count == null) {
        return self.length === 0 ? nil : self[0];
      }

      count = $scope.get('Opal').$coerce_to(count, $scope.get('Integer'), "to_int");

      if (count < 0) {
        self.$raise($scope.get('ArgumentError'), "negative array size");
      }

      return self.slice(0, count);
    
    };

    def.$flatten = function(level) {
      var self = this;

      
      var object_id = (self).$object_id();

      function _flatten(array, level) {
        var array = (array).$to_a(),
            result = [],
            i, length,
            item, ary;

        for (i = 0, length = array.length; i < length; i++) {
          item = array[i];

          if (!$scope.get('Opal')['$respond_to?'](item, "to_ary")) {
            result.push(item);
            continue;
          }

          ary = (item).$to_ary();

          if (ary === nil) {
            result.push(item);
            continue;
          }

          if (!ary.$$is_array) {
            self.$raise($scope.get('TypeError'));
          }

          if (object_id === (ary).$object_id()) {
            self.$raise($scope.get('ArgumentError'));
          }

          switch (level) {
          case undefined:
            result.push.apply(result, _flatten(ary));
            break;
          case 0:
            result.push(ary);
            break;
          default:
            result.push.apply(result, _flatten(ary, level - 1));
          }
        }
        return result;
      }

      if (level !== undefined) {
        level = $scope.get('Opal').$coerce_to(level, $scope.get('Integer'), "to_int");
      }

      return _flatten(self, level);
    ;
    };

    def['$flatten!'] = function(level) {
      var self = this;

      
      var flattened = self.$flatten(level);

      if (self.length == flattened.length) {
        for (var i = 0, length = self.length; i < length; i++) {
          if (self[i] !== flattened[i]) {
            break;
          }
        }

        if (i == length) {
          return nil;
        }
      }

      self.$replace(flattened);
    ;
      return self;
    };

    def.$hash = function() {
      var self = this;

      
      var hash = ['A'],
          item;
      for (var i = 0, length = self.length; i < length; i++) {
        item = self[i];
        if (item.$$is_array && (self)['$eql?'](item)) {
          hash.push('self');
        } else {
          hash.push(item.$hash());
        }
      }
      return hash.join(',');
    
    };

    def['$include?'] = function(member) {
      var self = this;

      
      for (var i = 0, length = self.length; i < length; i++) {
        if ((self[i])['$=='](member)) {
          return true;
        }
      }

      return false;
    
    };

    def.$index = TMP_13 = function(object) {
      var self = this, $iter = TMP_13.$$p, block = $iter || nil;

      TMP_13.$$p = null;
      
      if (object != null) {
        for (var i = 0, length = self.length; i < length; i++) {
          if ((self[i])['$=='](object)) {
            return i;
          }
        }
      }
      else if (block !== nil) {
        for (var i = 0, length = self.length, value; i < length; i++) {
          if ((value = block(self[i])) === $breaker) {
            return $breaker.$v;
          }

          if (value !== false && value !== nil) {
            return i;
          }
        }
      }
      else {
        return self.$enum_for("index");
      }

      return nil;
    
    };

    def.$insert = function(index, objects) {
      var self = this;

      objects = $slice.call(arguments, 1);
      
      index = $scope.get('Opal').$coerce_to(index, $scope.get('Integer'), "to_int");

      if (objects.length > 0) {
        if (index < 0) {
          index += self.length + 1;

          if (index < 0) {
            self.$raise($scope.get('IndexError'), "" + (index) + " is out of bounds");
          }
        }
        if (index > self.length) {
          for (var i = self.length; i < index; i++) {
            self.push(nil);
          }
        }

        self.splice.apply(self, [index, 0].concat(objects));
      }
    ;
      return self;
    };

    def.$inspect = function() {
      var self = this;

      
      var result = [],
          id     = self.$__id__();

      for (var i = 0, length = self.length; i < length; i++) {
        var item = self['$[]'](i);

        if ((item).$__id__() === id) {
          result.push('[...]');
        }
        else {
          result.push((item).$inspect());
        }
      }

      return '[' + result.join(', ') + ']';
    ;
    };

    def.$join = function(sep) {
      var $a, self = this;
      if ($gvars[","] == null) $gvars[","] = nil;

      if (sep == null) {
        sep = nil
      }
      if ((($a = self.length === 0) !== nil && (!$a.$$is_boolean || $a == true))) {
        return ""};
      if ((($a = sep === nil) !== nil && (!$a.$$is_boolean || $a == true))) {
        sep = $gvars[","]};
      
      var result = [];
      var object_id = (self).$object_id();

      for (var i = 0, length = self.length; i < length; i++) {
        var item = self[i];

        if ($scope.get('Opal')['$respond_to?'](item, "to_str")) {
          var tmp = (item).$to_str();

          if (tmp !== nil) {
            result.push((tmp).$to_s());

            continue;
          }
        }

        if ($scope.get('Opal')['$respond_to?'](item, "to_ary")) {
          var tmp = (item).$to_ary();

          if (object_id === (tmp).$object_id()) {
            self.$raise($scope.get('ArgumentError'));
          }

          if (tmp !== nil) {
            result.push((tmp).$join(sep));

            continue;
          }
        }

        if ($scope.get('Opal')['$respond_to?'](item, "to_s")) {
          var tmp = (item).$to_s();

          if (tmp !== nil) {
            result.push(tmp);

            continue;
          }
        }

        self.$raise($scope.get('NoMethodError'), "" + ($scope.get('Opal').$inspect(item)) + " doesn't respond to #to_str, #to_ary or #to_s");
      }

      if (sep === nil) {
        return result.join('');
      }
      else {
        return result.join($scope.get('Opal')['$coerce_to!'](sep, $scope.get('String'), "to_str").$to_s());
      }
    ;
    };

    def.$keep_if = TMP_14 = function() {
      var self = this, $iter = TMP_14.$$p, block = $iter || nil;

      TMP_14.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("keep_if")
      };
      
      for (var i = 0, length = self.length, value; i < length; i++) {
        if ((value = block(self[i])) === $breaker) {
          return $breaker.$v;
        }

        if (value === false || value === nil) {
          self.splice(i, 1);

          length--;
          i--;
        }
      }
    
      return self;
    };

    def.$last = function(count) {
      var self = this;

      
      if (count == null) {
        return self.length === 0 ? nil : self[self.length - 1];
      }

      count = $scope.get('Opal').$coerce_to(count, $scope.get('Integer'), "to_int");

      if (count < 0) {
        self.$raise($scope.get('ArgumentError'), "negative array size");
      }

      if (count > self.length) {
        count = self.length;
      }

      return self.slice(self.length - count, self.length);
    
    };

    def.$length = function() {
      var self = this;

      return self.length;
    };

    Opal.defn(self, '$map', def.$collect);

    Opal.defn(self, '$map!', def['$collect!']);

    def.$pop = function(count) {
      var $a, self = this;

      if ((($a = count === undefined) !== nil && (!$a.$$is_boolean || $a == true))) {
        if ((($a = self.length === 0) !== nil && (!$a.$$is_boolean || $a == true))) {
          return nil};
        return self.pop();};
      count = $scope.get('Opal').$coerce_to(count, $scope.get('Integer'), "to_int");
      if ((($a = count < 0) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$raise($scope.get('ArgumentError'), "negative array size")};
      if ((($a = self.length === 0) !== nil && (!$a.$$is_boolean || $a == true))) {
        return []};
      if ((($a = count > self.length) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.splice(0, self.length);
        } else {
        return self.splice(self.length - count, self.length);
      };
    };

    def.$product = TMP_15 = function(args) {
      var $a, self = this, $iter = TMP_15.$$p, block = $iter || nil;

      args = $slice.call(arguments, 0);
      TMP_15.$$p = null;
      
      var result = (block !== nil) ? null : [],
          n = args.length + 1,
          counters = new Array(n),
          lengths  = new Array(n),
          arrays   = new Array(n),
          i, m, subarray, len, resultlen = 1;

      arrays[0] = self;
      for (i = 1; i < n; i++) {
        arrays[i] = $scope.get('Opal').$coerce_to(args[i - 1], $scope.get('Array'), "to_ary");
      }

      for (i = 0; i < n; i++) {
        len = arrays[i].length;
        if (len === 0) {
          return result || self;
        }
        resultlen *= len;
        if (resultlen > 2147483647) {
          self.$raise($scope.get('RangeError'), "too big to product")
        }
        lengths[i] = len;
        counters[i] = 0;
      }

      outer_loop: for (;;) {
        subarray = [];
        for (i = 0; i < n; i++) {
          subarray.push(arrays[i][counters[i]]);
        }
        if (result) {
          result.push(subarray);
        } else {
          ((($a = Opal.yield1(block, subarray)) === $breaker) ? $breaker.$v : $a)
        }
        m = n - 1;
        counters[m]++;
        while (counters[m] === lengths[m]) {
          counters[m] = 0;
          if (--m < 0) break outer_loop;
          counters[m]++;
        }
      }

      return result || self;
    ;
    };

    def.$push = function(objects) {
      var self = this;

      objects = $slice.call(arguments, 0);
      
      for (var i = 0, length = objects.length; i < length; i++) {
        self.push(objects[i]);
      }
    
      return self;
    };

    def.$rassoc = function(object) {
      var self = this;

      
      for (var i = 0, length = self.length, item; i < length; i++) {
        item = self[i];

        if (item.length && item[1] !== undefined) {
          if ((item[1])['$=='](object)) {
            return item;
          }
        }
      }

      return nil;
    
    };

    def.$reject = TMP_16 = function() {
      var self = this, $iter = TMP_16.$$p, block = $iter || nil;

      TMP_16.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("reject")
      };
      
      var result = [];

      for (var i = 0, length = self.length, value; i < length; i++) {
        if ((value = block(self[i])) === $breaker) {
          return $breaker.$v;
        }

        if (value === false || value === nil) {
          result.push(self[i]);
        }
      }
      return result;
    
    };

    def['$reject!'] = TMP_17 = function() {
      var $a, $b, self = this, $iter = TMP_17.$$p, block = $iter || nil, original = nil;

      TMP_17.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("reject!")
      };
      original = self.$length();
      ($a = ($b = self).$delete_if, $a.$$p = block.$to_proc(), $a).call($b);
      if (self.$length()['$=='](original)) {
        return nil
        } else {
        return self
      };
    };

    def.$replace = function(other) {
      var $a, self = this;

      if ((($a = $scope.get('Array')['$==='](other)) !== nil && (!$a.$$is_boolean || $a == true))) {
        other = other.$to_a()
        } else {
        other = $scope.get('Opal').$coerce_to(other, $scope.get('Array'), "to_ary").$to_a()
      };
      
      self.splice(0, self.length);
      self.push.apply(self, other);
    
      return self;
    };

    def.$reverse = function() {
      var self = this;

      return self.slice(0).reverse();
    };

    def['$reverse!'] = function() {
      var self = this;

      return self.reverse();
    };

    def.$reverse_each = TMP_18 = function() {
      var $a, $b, self = this, $iter = TMP_18.$$p, block = $iter || nil;

      TMP_18.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("reverse_each")
      };
      ($a = ($b = self.$reverse()).$each, $a.$$p = block.$to_proc(), $a).call($b);
      return self;
    };

    def.$rindex = TMP_19 = function(object) {
      var self = this, $iter = TMP_19.$$p, block = $iter || nil;

      TMP_19.$$p = null;
      
      if (object != null) {
        for (var i = self.length - 1; i >= 0; i--) {
          if ((self[i])['$=='](object)) {
            return i;
          }
        }
      }
      else if (block !== nil) {
        for (var i = self.length - 1, value; i >= 0; i--) {
          if ((value = block(self[i])) === $breaker) {
            return $breaker.$v;
          }

          if (value !== false && value !== nil) {
            return i;
          }
        }
      }
      else if (object == null) {
        return self.$enum_for("rindex");
      }

      return nil;
    
    };

    def.$rotate = function(n) {
      var self = this;

      if (n == null) {
        n = 1
      }
      n = $scope.get('Opal').$coerce_to(n, $scope.get('Integer'), "to_int");
      
      var ary, idx, firstPart, lastPart;
      
      if (self.length === 1) {
        return self.slice();
      }
      if (self.length === 0) {
        return [];
      }
      
      ary = self.slice();
      idx = n % ary.length;
      
      firstPart = ary.slice(idx);
      lastPart = ary.slice(0, idx);
      return firstPart.concat(lastPart);
    
    };

    def['$rotate!'] = function(cnt) {
      var $a, self = this, ary = nil;

      if (cnt == null) {
        cnt = 1
      }
      if ((($a = self['$frozen?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$raise($scope.get('RuntimeError'), "can't modify frozen Array")};
      
      if (self.length === 0 || self.length === 1) {
        return self;
      }
    
      cnt = $scope.get('Opal').$coerce_to(cnt, $scope.get('Integer'), "to_int");
      ary = self.$rotate(cnt);
      return self.$replace(ary);
    };

    def.$sample = function(n) {
      var $a, $b, TMP_20, self = this;

      if (n == null) {
        n = nil
      }
      if ((($a = ($b = n['$!'](), $b !== false && $b !== nil ?self['$empty?']() : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil};
      if ((($a = (($b = n !== false && n !== nil) ? self['$empty?']() : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return []};
      if (n !== false && n !== nil) {
        return ($a = ($b = ($range(1, n, false))).$map, $a.$$p = (TMP_20 = function(){var self = TMP_20.$$s || this;

        return self['$[]'](self.$rand(self.$length()))}, TMP_20.$$s = self, TMP_20), $a).call($b)
        } else {
        return self['$[]'](self.$rand(self.$length()))
      };
    };

    def.$select = TMP_21 = function() {
      var self = this, $iter = TMP_21.$$p, block = $iter || nil;

      TMP_21.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("select")
      };
      
      var result = [];

      for (var i = 0, length = self.length, item, value; i < length; i++) {
        item = self[i];

        if ((value = Opal.yield1(block, item)) === $breaker) {
          return $breaker.$v;
        }

        if (value !== false && value !== nil) {
          result.push(item);
        }
      }

      return result;
    
    };

    def['$select!'] = TMP_22 = function() {
      var $a, $b, self = this, $iter = TMP_22.$$p, block = $iter || nil;

      TMP_22.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("select!")
      };
      
      var original = self.length;
      ($a = ($b = self).$keep_if, $a.$$p = block.$to_proc(), $a).call($b);
      return self.length === original ? nil : self;
    
    };

    def.$shift = function(count) {
      var $a, self = this;

      if ((($a = count === undefined) !== nil && (!$a.$$is_boolean || $a == true))) {
        if ((($a = self.length === 0) !== nil && (!$a.$$is_boolean || $a == true))) {
          return nil};
        return self.shift();};
      count = $scope.get('Opal').$coerce_to(count, $scope.get('Integer'), "to_int");
      if ((($a = count < 0) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$raise($scope.get('ArgumentError'), "negative array size")};
      if ((($a = self.length === 0) !== nil && (!$a.$$is_boolean || $a == true))) {
        return []};
      return self.splice(0, count);
    };

    Opal.defn(self, '$size', def.$length);

    def.$shuffle = function() {
      var self = this;

      return self.$clone()['$shuffle!']();
    };

    def['$shuffle!'] = function() {
      var self = this;

      
      for (var i = self.length - 1; i > 0; i--) {
        var tmp = self[i],
            j   = Math.floor(Math.random() * (i + 1));

        self[i] = self[j];
        self[j] = tmp;
      }
    
      return self;
    };

    Opal.defn(self, '$slice', def['$[]']);

    def['$slice!'] = function(index, length) {
      var self = this;

      
      if (index < 0) {
        index += self.length;
      }

      if (length != null) {
        return self.splice(index, length);
      }

      if (index < 0 || index >= self.length) {
        return nil;
      }

      return self.splice(index, 1)[0];
    
    };

    def.$sort = TMP_23 = function() {
      var $a, self = this, $iter = TMP_23.$$p, block = $iter || nil;

      TMP_23.$$p = null;
      if ((($a = self.length > 1) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        return self
      };
      
      if (!(block !== nil)) {
        block = function(a, b) {
          return (a)['$<=>'](b);
        };
      }

      try {
        return self.slice().sort(function(x, y) {
          var ret = block(x, y);

          if (ret === $breaker) {
            throw $breaker;
          }
          else if (ret === nil) {
            self.$raise($scope.get('ArgumentError'), "comparison of " + ((x).$inspect()) + " with " + ((y).$inspect()) + " failed");
          }

          return $rb_gt(ret, 0) ? 1 : ($rb_lt(ret, 0) ? -1 : 0);
        });
      }
      catch (e) {
        if (e === $breaker) {
          return $breaker.$v;
        }
        else {
          throw e;
        }
      }
    ;
    };

    def['$sort!'] = TMP_24 = function() {
      var $a, $b, self = this, $iter = TMP_24.$$p, block = $iter || nil;

      TMP_24.$$p = null;
      
      var result;

      if ((block !== nil)) {
        result = ($a = ($b = (self.slice())).$sort, $a.$$p = block.$to_proc(), $a).call($b);
      }
      else {
        result = (self.slice()).$sort();
      }

      self.length = 0;
      for(var i = 0, length = result.length; i < length; i++) {
        self.push(result[i]);
      }

      return self;
    ;
    };

    def.$take = function(count) {
      var self = this;

      
      if (count < 0) {
        self.$raise($scope.get('ArgumentError'));
      }

      return self.slice(0, count);
    ;
    };

    def.$take_while = TMP_25 = function() {
      var self = this, $iter = TMP_25.$$p, block = $iter || nil;

      TMP_25.$$p = null;
      
      var result = [];

      for (var i = 0, length = self.length, item, value; i < length; i++) {
        item = self[i];

        if ((value = block(item)) === $breaker) {
          return $breaker.$v;
        }

        if (value === false || value === nil) {
          return result;
        }

        result.push(item);
      }

      return result;
    
    };

    def.$to_a = function() {
      var self = this;

      return self;
    };

    Opal.defn(self, '$to_ary', def.$to_a);

    def.$to_h = function() {
      var self = this;

      
      var i, len = self.length, ary, key, val, hash = $hash2([], {});

      for (i = 0; i < len; i++) {
        ary = $scope.get('Opal')['$coerce_to?'](self[i], $scope.get('Array'), "to_ary");
        if (!ary.$$is_array) {
          self.$raise($scope.get('TypeError'), "wrong element type " + ((ary).$class()) + " at " + (i) + " (expected array)")
        }
        if (ary.length !== 2) {
          self.$raise($scope.get('ArgumentError'), "wrong array length at " + (i) + " (expected 2, was " + ((ary).$length()) + ")")
        }
        key = ary[0];
        val = ary[1];
        hash.$store(key, val);
      }

      return hash;
    ;
    };

    Opal.defn(self, '$to_s', def.$inspect);

    def.$transpose = function() {
      var $a, $b, TMP_26, self = this, result = nil, max = nil;

      if ((($a = self['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return []};
      result = [];
      max = nil;
      ($a = ($b = self).$each, $a.$$p = (TMP_26 = function(row){var self = TMP_26.$$s || this, $a, $b, TMP_27;
if (row == null) row = nil;
      if ((($a = $scope.get('Array')['$==='](row)) !== nil && (!$a.$$is_boolean || $a == true))) {
          row = row.$to_a()
          } else {
          row = $scope.get('Opal').$coerce_to(row, $scope.get('Array'), "to_ary").$to_a()
        };
        ((($a = max) !== false && $a !== nil) ? $a : max = row.length);
        if ((($a = (row.length)['$=='](max)['$!']()) !== nil && (!$a.$$is_boolean || $a == true))) {
          self.$raise($scope.get('IndexError'), "element size differs (" + (row.length) + " should be " + (max))};
        return ($a = ($b = (row.length)).$times, $a.$$p = (TMP_27 = function(i){var self = TMP_27.$$s || this, $a, $b, $c, entry = nil;
if (i == null) i = nil;
        entry = (($a = i, $b = result, ((($c = $b['$[]']($a)) !== false && $c !== nil) ? $c : $b['$[]=']($a, []))));
          return entry['$<<'](row.$at(i));}, TMP_27.$$s = self, TMP_27), $a).call($b);}, TMP_26.$$s = self, TMP_26), $a).call($b);
      return result;
    };

    def.$uniq = function() {
      var self = this;

      
      var result = [],
          seen   = {};

      for (var i = 0, length = self.length, item, hash; i < length; i++) {
        item = self[i];
        hash = item;

        if (!seen[hash]) {
          seen[hash] = true;

          result.push(item);
        }
      }

      return result;
    
    };

    def['$uniq!'] = function() {
      var self = this;

      
      var original = self.length,
          seen     = {};

      for (var i = 0, length = original, item, hash; i < length; i++) {
        item = self[i];
        hash = item;

        if (!seen[hash]) {
          seen[hash] = true;
        }
        else {
          self.splice(i, 1);

          length--;
          i--;
        }
      }

      return self.length === original ? nil : self;
    
    };

    def.$unshift = function(objects) {
      var self = this;

      objects = $slice.call(arguments, 0);
      
      for (var i = objects.length - 1; i >= 0; i--) {
        self.unshift(objects[i]);
      }
    
      return self;
    };

    def.$values_at = function(args) {
      var $a, $b, TMP_28, self = this, out = nil;

      args = $slice.call(arguments, 0);
      out = [];
      ($a = ($b = args).$each, $a.$$p = (TMP_28 = function(elem){var self = TMP_28.$$s || this, $a, $b, TMP_29, finish = nil, start = nil, i = nil;
if (elem == null) elem = nil;
      if ((($a = elem['$kind_of?']($scope.get('Range'))) !== nil && (!$a.$$is_boolean || $a == true))) {
          finish = $scope.get('Opal').$coerce_to(elem.$last(), $scope.get('Integer'), "to_int");
          start = $scope.get('Opal').$coerce_to(elem.$first(), $scope.get('Integer'), "to_int");
          
          if (start < 0) {
            start = start + self.length;
            return nil;;
          }
        
          
          if (finish < 0) {
            finish = finish + self.length;
          }
          if (elem['$exclude_end?']()) {
            finish--;
          }
          if (finish < start) {
            return nil;;
          }
        
          return ($a = ($b = start).$upto, $a.$$p = (TMP_29 = function(i){var self = TMP_29.$$s || this;
if (i == null) i = nil;
          return out['$<<'](self.$at(i))}, TMP_29.$$s = self, TMP_29), $a).call($b, finish);
          } else {
          i = $scope.get('Opal').$coerce_to(elem, $scope.get('Integer'), "to_int");
          return out['$<<'](self.$at(i));
        }}, TMP_28.$$s = self, TMP_28), $a).call($b);
      return out;
    };

    return (def.$zip = TMP_30 = function(others) {
      var $a, self = this, $iter = TMP_30.$$p, block = $iter || nil;

      others = $slice.call(arguments, 0);
      TMP_30.$$p = null;
      
      var result = [], size = self.length, part, o, i, j, jj;

      for (j = 0, jj = others.length; j < jj; j++) {
        o = others[j];
        if (!o.$$is_array) {
          others[j] = (((($a = $scope.get('Opal')['$coerce_to?'](o, $scope.get('Array'), "to_ary")) !== false && $a !== nil) ? $a : $scope.get('Opal')['$coerce_to!'](o, $scope.get('Enumerator'), "each"))).$to_a();
        }
      }

      for (i = 0; i < size; i++) {
        part = [self[i]];

        for (j = 0, jj = others.length; j < jj; j++) {
          o = others[j][i];

          if (o == null) {
            o = nil;
          }

          part[j + 1] = o;
        }

        result[i] = part;
      }

      if (block !== nil) {
        for (i = 0; i < size; i++) {
          block(result[i]);
        }

        return nil;
      }

      return result;
    
    }, nil) && 'zip';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/array/inheritance"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_times(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs * rhs : lhs['$*'](rhs);
  }
  function $rb_minus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs - rhs : lhs['$-'](rhs);
  }
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$new', '$allocate', '$initialize', '$to_proc', '$__send__', '$clone', '$respond_to?', '$==', '$eql?', '$inspect', '$hash', '$class', '$slice', '$uniq', '$flatten']);
  (function($base, $super) {
    function $Array(){};
    var self = $Array = $klass($base, $super, 'Array', $Array);

    var def = self.$$proto, $scope = self.$$scope;

    return (Opal.defs(self, '$inherited', function(klass) {
      var self = this, replace = nil;

      replace = $scope.get('Class').$new((($scope.get('Array')).$$scope.get('Wrapper')));
      
      klass.$$proto         = replace.$$proto;
      klass.$$proto.$$class = klass;
      klass.$$alloc         = replace.$$alloc;
      klass.$$parent        = (($scope.get('Array')).$$scope.get('Wrapper'));

      klass.$allocate = replace.$allocate;
      klass.$new      = replace.$new;
      klass["$[]"]    = replace["$[]"];
    
    }), nil) && 'inherited'
  })(self, null);
  return (function($base, $super) {
    function $Wrapper(){};
    var self = $Wrapper = $klass($base, $super, 'Wrapper', $Wrapper);

    var def = self.$$proto, $scope = self.$$scope, TMP_1, TMP_2, TMP_3, TMP_4, TMP_5;

    def.literal = nil;
    def.$$is_array = true;

    Opal.defs(self, '$allocate', TMP_1 = function(array) {
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil, obj = nil;

      if (array == null) {
        array = []
      }
      TMP_1.$$p = null;
      obj = Opal.find_super_dispatcher(self, 'allocate', TMP_1, null, $Wrapper).apply(self, []);
      obj.literal = array;
      return obj;
    });

    Opal.defs(self, '$new', TMP_2 = function(args) {
      var $a, $b, self = this, $iter = TMP_2.$$p, block = $iter || nil, obj = nil;

      args = $slice.call(arguments, 0);
      TMP_2.$$p = null;
      obj = self.$allocate();
      ($a = ($b = obj).$initialize, $a.$$p = block.$to_proc(), $a).apply($b, [].concat(args));
      return obj;
    });

    Opal.defs(self, '$[]', function(objects) {
      var self = this;

      objects = $slice.call(arguments, 0);
      return self.$allocate(objects);
    });

    def.$initialize = TMP_3 = function(args) {
      var $a, $b, self = this, $iter = TMP_3.$$p, block = $iter || nil;

      args = $slice.call(arguments, 0);
      TMP_3.$$p = null;
      return self.literal = ($a = ($b = $scope.get('Array')).$new, $a.$$p = block.$to_proc(), $a).apply($b, [].concat(args));
    };

    def.$method_missing = TMP_4 = function(args) {
      var $a, $b, self = this, $iter = TMP_4.$$p, block = $iter || nil, result = nil;

      args = $slice.call(arguments, 0);
      TMP_4.$$p = null;
      result = ($a = ($b = self.literal).$__send__, $a.$$p = block.$to_proc(), $a).apply($b, [].concat(args));
      if ((($a = result === self.literal) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self
        } else {
        return result
      };
    };

    def.$initialize_copy = function(other) {
      var self = this;

      return self.literal = (other.literal).$clone();
    };

    def['$respond_to?'] = TMP_5 = function(name) {var $zuper = $slice.call(arguments, 0);
      var $a, self = this, $iter = TMP_5.$$p, $yield = $iter || nil;

      TMP_5.$$p = null;
      return ((($a = Opal.find_super_dispatcher(self, 'respond_to?', TMP_5, $iter).apply(self, $zuper)) !== false && $a !== nil) ? $a : self.literal['$respond_to?'](name));
    };

    def['$=='] = function(other) {
      var self = this;

      return self.literal['$=='](other);
    };

    def['$eql?'] = function(other) {
      var self = this;

      return self.literal['$eql?'](other);
    };

    def.$to_a = function() {
      var self = this;

      return self.literal;
    };

    def.$to_ary = function() {
      var self = this;

      return self;
    };

    def.$inspect = function() {
      var self = this;

      return self.literal.$inspect();
    };

    def.$hash = function() {
      var self = this;

      return self.literal.$hash();
    };

    def['$*'] = function(other) {
      var self = this;

      
      var result = $rb_times(self.literal, other);

      if (result.$$is_array) {
        return self.$class().$allocate(result)
      }
      else {
        return result;
      }
    ;
    };

    def['$[]'] = function(index, length) {
      var self = this;

      
      var result = self.literal.$slice(index, length);

      if (result.$$is_array && (index.$$is_range || length !== undefined)) {
        return self.$class().$allocate(result)
      }
      else {
        return result;
      }
    ;
    };

    Opal.defn(self, '$slice', def['$[]']);

    def.$uniq = function() {
      var self = this;

      return self.$class().$allocate(self.literal.$uniq());
    };

    def.$flatten = function(level) {
      var self = this;

      return self.$class().$allocate(self.literal.$flatten(level));
    };

    def['$-'] = function(other) {
      var self = this;

      return $rb_minus(self.literal, other);
    };

    return (def['$+'] = function(other) {
      var self = this;

      return $rb_plus(self.literal, other);
    }, nil) && '+';
  })($scope.get('Array'), null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/hash"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$include', '$coerce_to?', '$[]', '$merge!', '$allocate', '$raise', '$!', '$==', '$call', '$coerce_to!', '$lambda?', '$abs', '$arity', '$enum_for', '$inspect', '$flatten', '$eql?', '$===', '$clone', '$to_proc', '$alias_method']);
  self.$require("corelib/enumerable");
  return (function($base, $super) {
    function $Hash(){};
    var self = $Hash = $klass($base, $super, 'Hash', $Hash);

    var def = self.$$proto, $scope = self.$$scope, TMP_1, TMP_2, TMP_3, TMP_4, TMP_5, TMP_6, TMP_7, TMP_8, TMP_9, TMP_10, TMP_11, TMP_12, TMP_13;

    def.proc = def.none = nil;
    self.$include($scope.get('Enumerable'));

    def.$$is_hash = true;

    Opal.defs(self, '$[]', function(argv) {
      var self = this;

      argv = $slice.call(arguments, 0);
      
      var hash, i, argc = argv.length;

      if (argc === 1) {
        hash = $scope.get('Opal')['$coerce_to?'](argv['$[]'](0), $scope.get('Hash'), "to_hash");
        if (hash !== nil) {
          return self.$allocate()['$merge!'](hash);
        }

        argv = $scope.get('Opal')['$coerce_to?'](argv['$[]'](0), $scope.get('Array'), "to_ary");
        if (argv === nil) {
          self.$raise($scope.get('ArgumentError'), "odd number of arguments for Hash")
        }

        argc = argv.length;
        hash = self.$allocate();

        for (i = 0; i < argc; i++) {
          if (!argv[i].$$is_array) continue;
          switch(argv[i].length) {
          case 1:
            hash.$store(argv[i][0], nil);
            break;
          case 2:
            hash.$store(argv[i][0], argv[i][1]);
            break;
          default:
            self.$raise($scope.get('ArgumentError'), "invalid number of elements (" + (argv[i].length) + " for 1..2)")
          }
        }

        return hash;
      }

      if (argc % 2 !== 0) {
        self.$raise($scope.get('ArgumentError'), "odd number of arguments for Hash")
      }

      hash = self.$allocate();

      for (i = 0; i < argc; i += 2) {
        hash.$store(argv[i], argv[i + 1]);
      }

      return hash;
    ;
    });

    Opal.defs(self, '$allocate', function() {
      var self = this;

      
      var hash = new self.$$alloc;

      hash.map  = {};
      hash.smap = {};
      hash.keys = [];
      hash.none = nil;
      hash.proc = nil;

      return hash;
    
    });

    def.$initialize = TMP_1 = function(defaults) {
      var self = this, $iter = TMP_1.$$p, block = $iter || nil;

      TMP_1.$$p = null;
      
      self.none = (defaults === undefined ? nil : defaults);
      self.proc = block;
    
      return self;
    };

    def['$=='] = function(other) {
      var self = this;

      
      if (self === other) {
        return true;
      }

      if (!other.keys || !other.smap || !other.map) {
        return false;
      }

      if (self.keys.length !== other.keys.length) {
        return false;
      }

      var _map  = self.map,
          smap  = self.smap,
          _map2 = other.map,
          smap2 = other.smap,
          map, map2, key, khash, value, value2;

      for (var i = 0, length = self.keys.length; i < length; i++) {
        key = self.keys[i];

        if (key.$$is_string) {
          khash = key;
          map   = smap;
          map2  = smap2;
        } else {
          khash = key.$hash();
          map   = _map;
          map2  = _map2;
        }

        value  = map[khash];
        if (value === undefined) console.log('==', key, self);
        value2 = map2[khash];

        if (value2 === undefined || ((value)['$=='](value2))['$!']()) {
          return false;
        }
      }

      return true;
    
    };

    def['$[]'] = function(key) {
      var self = this;

      
      var map, khash;

      if (key.$$is_string) {
        map = self.smap;
        khash = key;
      } else {
        map = self.map;
        khash = key.$hash();
      }

      if (map === undefined) { console.log(self, '[] --> key:', key, khash, map) }


      if (Opal.hasOwnProperty.call(map, khash)) {
        return map[khash];
      }

      var proc = self.proc;

      if (proc !== nil) {
        return (proc).$call(self, key);
      }

      return self.none;
    
    };

    def['$[]='] = function(key, value) {
      var self = this;

      
      var map, khash, value;

      if (key.$$is_string) {
        map = self.smap;
        khash = key;
      } else {
        map = self.map;
        khash = key.$hash();
      }

      if (!Opal.hasOwnProperty.call(map, khash)) {
        self.keys.push(key);
      }

      map[khash] = value;

      return value;
    
    };

    def.$assoc = function(object) {
      var self = this;

      
      var keys = self.keys,
          map, key, khash;

      for (var i = 0, length = keys.length; i < length; i++) {
        key = keys[i];

        if ((key)['$=='](object)) {
          if (key.$$is_string) {
            map = self.smap;
            khash = key;
          } else {
            map = self.map;
            khash = key.$hash();
          }

          return [key, map[khash]];
        }
      }

      return nil;
    
    };

    def.$clear = function() {
      var self = this;

      
      self.map = {};
      self.smap = {};
      self.keys = [];
      return self;
    
    };

    def.$clone = function() {
      var self = this;

      
      var _map  = {},
          smap  = {},
          _map2 = self.map,
          smap2 = self.smap,
          keys  = [],
          map, map2, key, khash, value;

      for (var i = 0, length = self.keys.length; i < length; i++) {
        key   = self.keys[i];

        if (key.$$is_string) {
          khash = key;
          map = smap;
          map2 = smap2;
        } else {
          khash = key.$hash();
          map = _map;
          map2 = _map2;
        }

        value = map2[khash];

        keys.push(key);
        map[khash] = value;
      }

      var clone = new self.$$class.$$alloc();

      clone.map  = _map;
      clone.smap = smap;
      clone.keys = keys;
      clone.none = self.none;
      clone.proc = self.proc;

      return clone;
    
    };

    def.$default = function(val) {
      var self = this;

      
      if (val !== undefined && self.proc !== nil) {
        return self.proc.$call(self, val);
      }
      return self.none;
    ;
    };

    def['$default='] = function(object) {
      var self = this;

      
      self.proc = nil;
      return (self.none = object);
    
    };

    def.$default_proc = function() {
      var self = this;

      return self.proc;
    };

    def['$default_proc='] = function(proc) {
      var self = this;

      
      if (proc !== nil) {
        proc = $scope.get('Opal')['$coerce_to!'](proc, $scope.get('Proc'), "to_proc");

        if (proc['$lambda?']() && proc.$arity().$abs() != 2) {
          self.$raise($scope.get('TypeError'), "default_proc takes two arguments");
        }
      }
      self.none = nil;
      return (self.proc = proc);
    ;
    };

    def.$delete = TMP_2 = function(key) {
      var self = this, $iter = TMP_2.$$p, block = $iter || nil;

      TMP_2.$$p = null;
      
      var result, map, khash;

      if (key.$$is_string) {
        map = self.smap;
        khash = key;
      } else {
        map = self.map;
        khash = key.$hash();
      }

      result = map[khash];

      if (result != null) {
        delete map[khash];
        self.keys.$delete(key);

        return result;
      }

      if (block !== nil) {
        return block.$call(key);
      }
      return nil;
    
    };

    def.$delete_if = TMP_3 = function() {
      var self = this, $iter = TMP_3.$$p, block = $iter || nil;

      TMP_3.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        return self.$enum_for("delete_if")
      };
      
      var _map = self.map,
          smap = self.smap,
          keys = self.keys,
          map, key, value, obj, khash;

      for (var i = 0, length = keys.length; i < length; i++) {
        key = keys[i];

        if (key.$$is_string) {
          map = smap;
          khash = key;
        } else {
          map = _map;
          khash = key.$hash();
        }
        obj = map[khash];
        value = block(key, obj);

        if (value === $breaker) {
          return $breaker.$v;
        }

        if (value !== false && value !== nil) {
          keys.splice(i, 1);
          delete map[khash];

          length--;
          i--;
        }
      }

      return self;
    
    };

    Opal.defn(self, '$dup', def.$clone);

    def.$each = TMP_4 = function() {
      var self = this, $iter = TMP_4.$$p, block = $iter || nil;

      TMP_4.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        return self.$enum_for("each")
      };
      
      var _map = self.map,
          smap = self.smap,
          keys = self.keys,
          map, key, khash, value;

      for (var i = 0, length = keys.length; i < length; i++) {
        key = keys[i];

        if (key.$$is_string) {
          map = smap;
          khash = key;
        } else {
          map = _map;
          khash = key.$hash();
        }

        value = Opal.yield1(block, [key, map[khash]]);

        if (value === $breaker) {
          return $breaker.$v;
        }
      }

      return self;
    
    };

    def.$each_key = TMP_5 = function() {
      var self = this, $iter = TMP_5.$$p, block = $iter || nil;

      TMP_5.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        return self.$enum_for("each_key")
      };
      
      var keys = self.keys, key;

      for (var i = 0, length = keys.length; i < length; i++) {
        key = keys[i];

        if (block(key) === $breaker) {
          return $breaker.$v;
        }
      }

      return self;
    
    };

    Opal.defn(self, '$each_pair', def.$each);

    def.$each_value = TMP_6 = function() {
      var self = this, $iter = TMP_6.$$p, block = $iter || nil;

      TMP_6.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        return self.$enum_for("each_value")
      };
      
      var _map = self.map,
          smap = self.smap,
          keys = self.keys, key, map, khash;

      for (var i = 0, length = keys.length; i < length; i++) {
        key = keys[i];

        if (key.$$is_string) {
          map = smap;
          khash = key;
        } else {
          map = _map;
          khash = key.$hash();
        }

        if (block(map[khash]) === $breaker) {
          return $breaker.$v;
        }
      }

      return self;
    
    };

    def['$empty?'] = function() {
      var self = this;

      return self.keys.length === 0;
    };

    Opal.defn(self, '$eql?', def['$==']);

    def.$fetch = TMP_7 = function(key, defaults) {
      var self = this, $iter = TMP_7.$$p, block = $iter || nil;

      TMP_7.$$p = null;
      
      var map, khash, value;

      if (key.$$is_string) {
        khash = key;
        map = self.smap;
      } else {
        khash = key.$hash();
        map = self.map;
      }

      value = map[khash];

      if (value != null) {
        return value;
      }

      if (block !== nil) {
        var value;

        if ((value = block(key)) === $breaker) {
          return $breaker.$v;
        }

        return value;
      }

      if (defaults != null) {
        return defaults;
      }

      self.$raise($scope.get('KeyError'), "key not found: " + (key.$inspect()));
    
    };

    def.$flatten = function(level) {
      var self = this;

      
      var _map = self.map,
          smap = self.smap,
          keys = self.keys,
          result = [],
          map, key, khash, value;

      for (var i = 0, length = keys.length; i < length; i++) {
        key = keys[i];

        if (key.$$is_string) {
          khash = key;
          map = smap;
        } else {
          khash = key.$hash();
          map = _map;
        }

        value = map[khash];

        result.push(key);

        if (value.$$is_array) {
          if (level == null || level === 1) {
            result.push(value);
          }
          else {
            result = result.concat((value).$flatten(level - 1));
          }
        }
        else {
          result.push(value);
        }
      }

      return result;
    
    };

    def['$has_key?'] = function(key) {
      var self = this;

      
      var keys = self.keys,
          map, khash;

      if (key.$$is_string) {
        khash = key;
        map = self.smap;
      } else {
        khash = key.$hash();
        map = self.map;
      }

      if (Opal.hasOwnProperty.call(map, khash)) {
        for (var i = 0, length = keys.length; i < length; i++) {
          if (!(key['$eql?'](keys[i]))['$!']()) {
            return true;
          }
        }
      }

      return false;
    
    };

    def['$has_value?'] = function(value) {
      var self = this;

      
      var _map = self.map,
          smap = self.smap,
          keys = self.keys, key, map, khash;

      for (var i = 0, length = keys.length; i < length; i++) {
        key = keys[i];

        if (key.$$is_string) {
          map = smap;
          khash = key;
        } else {
          map = _map;
          khash = key.$hash();
        }

        if ((map[khash])['$=='](value)) {
          return true;
        }
      }

      return false;
    
    };

    var hash_ids = null;

    def.$hash = function() {
      var self = this;

      
      var top = (hash_ids === null);
      try {
        var key, value,
            hash = ['Hash'],
            keys = self.keys,
            id = self.$object_id(),
            counter = 0;

        if (top) {
          hash_ids = {}
        }

        if (hash_ids.hasOwnProperty(id)) {
          return 'self';
        }

        hash_ids[id] = true;

        for (var i = 0, length = keys.length; i < length; i++) {
          key   = keys[i];
          value = key.$$is_string ? self.smap[key] : self.map[key.$hash()];
          key   = key.$hash();
          value = (typeof(value) === 'undefined') ? '' : value.$hash();
          hash.push([key,value]);
        }

        return hash.sort().join();
      } finally {
        if (top) {
          hash_ids = null;
        }
      }
    
    };

    Opal.defn(self, '$include?', def['$has_key?']);

    def.$index = function(object) {
      var self = this;

      
      var _map = self.map,
          smap = self.smap,
          keys = self.keys,
          map, khash, key;

      for (var i = 0, length = keys.length; i < length; i++) {
        key = keys[i];

        if (key.$$is_string) {
          map = smap;
          khash = key;
        } else {
          map = _map;
          khash = key.$hash();
        }

        if ((map[khash])['$=='](object)) {
          return key;
        }
      }

      return nil;
    
    };

    def.$indexes = function(keys) {
      var self = this;

      keys = $slice.call(arguments, 0);
      
      var result = [],
          _map = self.map,
          smap = self.smap,
          map, key, khash, value;

      for (var i = 0, length = keys.length; i < length; i++) {
        key = keys[i];

        if (key.$$is_string) {
          khash = key;
          map = smap;
        } else {
          khash = key.$hash();
          map = _map;
        }

        value = map[khash];

        if (value != null) {
          result.push(value);
        }
        else {
          result.push(self.none);
        }
      }

      return result;
    
    };

    Opal.defn(self, '$indices', def.$indexes);

    var inspect_ids = null;

    def.$inspect = function() {
      var self = this;

      
      var top = (inspect_ids === null);
      try {

        var key, value,
            inspect = [],
            keys = self.keys,
            id = self.$object_id(),
            counter = 0;

        if (top) {
          inspect_ids = {}
        }

        if (inspect_ids.hasOwnProperty(id)) {
          return '{...}';
        }

        inspect_ids[id] = true;

        for (var i = 0, length = keys.length; i < length; i++) {
          key   = keys[i];
          value = key.$$is_string ? self.smap[key] : self.map[key.$hash()];
          key   = key.$inspect();
          value = value.$inspect();
          inspect.push(key + '=>' + value);
        }

        return '{' + inspect.join(', ') + '}';
      } finally {

        if (top) {
          inspect_ids = null;
        }
      }
    
    };

    def.$invert = function() {
      var self = this;

      
      var result = Opal.hash(),
          keys = self.keys,
          _map = self.map,
          smap = self.smap,
          keys2 = result.keys,
          _map2 = result.map,
          smap2 = result.smap,
          map, map2, key, khash, value;

      for (var i = 0, length = keys.length; i < length; i++) {
        key = keys[i];

        if (key.$$is_string) {
          map = smap;
          khash = key;
        } else {
          map = _map;
          khash = key.$hash();
        }

        value = map[khash];
        keys2.push(value);

        if (value.$$is_string) {
          map2 = smap2;
          khash = value;
        } else {
          map2 = _map2;
          khash = value.$hash();
        }

        map2[khash] = key;
      }

      return result;
    
    };

    def.$keep_if = TMP_8 = function() {
      var self = this, $iter = TMP_8.$$p, block = $iter || nil;

      TMP_8.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        return self.$enum_for("keep_if")
      };
      
      var _map = self.map,
          smap = self.smap,
          keys = self.keys,
          map, key, khash, value, keep;

      for (var i = 0, length = keys.length; i < length; i++) {
        key = keys[i];

        if (key.$$is_string) {
          khash = key;
          map = smap;
        } else {
          khash = key.$hash();
          map = _map;
        }

        value = map[khash];
        keep  = block(key, value);

        if (keep === $breaker) {
          return $breaker.$v;
        }

        if (keep === false || keep === nil) {
          keys.splice(i, 1);
          delete map[khash];

          length--;
          i--;
        }
      }

      return self;
    
    };

    Opal.defn(self, '$key', def.$index);

    Opal.defn(self, '$key?', def['$has_key?']);

    def.$keys = function() {
      var self = this;

      return self.keys.slice(0);
    };

    def.$length = function() {
      var self = this;

      return self.keys.length;
    };

    Opal.defn(self, '$member?', def['$has_key?']);

    def.$merge = TMP_9 = function(other) {
      var $a, $b, self = this, $iter = TMP_9.$$p, block = $iter || nil, cloned = nil;

      TMP_9.$$p = null;
      if ((($a = $scope.get('Hash')['$==='](other)) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        other = $scope.get('Opal')['$coerce_to!'](other, $scope.get('Hash'), "to_hash")
      };
      cloned = self.$clone();
      ($a = ($b = cloned)['$merge!'], $a.$$p = block.$to_proc(), $a).call($b, other);
      return cloned;
    };

    def['$merge!'] = TMP_10 = function(other) {
      var self = this, $iter = TMP_10.$$p, block = $iter || nil;

      TMP_10.$$p = null;
      
      if (! $scope.get('Hash')['$==='](other)) {
        other = $scope.get('Opal')['$coerce_to!'](other, $scope.get('Hash'), "to_hash");
      }

      var keys  = self.keys,
          _map  = self.map,
          smap  = self.smap,
          keys2 = other.keys,
          _map2 = other.map,
          smap2 = other.smap,
          map, map2, key, khash, value, value2;

      if (block === nil) {
        for (var i = 0, length = keys2.length; i < length; i++) {
          key = keys2[i];

          if (key.$$is_string) {
            khash = key;
            map = smap;
            map2 = smap2;
          } else {
            khash = key.$hash();
            map = _map;
            map2 = _map2;
          }

          if (map[khash] == null) {
            keys.push(key);
          }

          map[khash] = map2[khash];
        }
      }
      else {
        for (var i = 0, length = keys2.length; i < length; i++) {
          key    = keys2[i];

          if (key.$$is_string) {
            khash = key;
            map = smap;
            map2 = smap2;
          } else {
            khash = key.$hash();
            map = _map;
            map2 = _map2;
          }

          value  = map[khash];
          value2 = map2[khash];

          if (value == null) {
            keys.push(key);
            map[khash] = value2;
          }
          else {
            map[khash] = block(key, value, value2);
          }
        }
      }

      return self;
    ;
    };

    def.$rassoc = function(object) {
      var self = this;

      
      var keys = self.keys,
          _map = self.map,
          smap = self.smap,
          key, khash, value, map;

      for (var i = 0, length = keys.length; i < length; i++) {
        key = keys[i]

        if (key.$$is_string) {
          khash = key;
          map = smap;
        } else {
          khash = key.$hash();
          map = _map;
        }

        value = map[khash];

        if ((value)['$=='](object)) {
          return [key, value];
        }
      }

      return nil;
    
    };

    def.$reject = TMP_11 = function() {
      var self = this, $iter = TMP_11.$$p, block = $iter || nil;

      TMP_11.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        return self.$enum_for("reject")
      };
      
      var keys   = self.keys,
          _map    = self.map,
          smap    = self.smap,
          result = Opal.hash(),
          _map2   = result.map,
          smap2   = result.smap,
          keys2  = result.keys,
          map, map2, key, khash, object, value;

      for (var i = 0, length = keys.length; i < length; i++) {
        key = keys[i];

        if (key.$$is_string) {
          khash = key;
          map = smap;
          map2 = smap2;
        } else {
          khash = key.$hash();
          map = _map;
          map2 = _map2;
        }

        object = map[khash];

        if ((value = block(key, object)) === $breaker) {
          return $breaker.$v;
        }

        if (value === false || value === nil) {
          keys2.push(key);
          map2[khash] = object;
        }
      }

      return result;
    
    };

    def.$replace = function(other) {
      var self = this;

      
      var keys  = self.keys = [],
          _map  = self.map  = {},
          smap  = self.smap = {},
          _map2 = other.map,
          smap2 = other.smap,
          key, khash, map, map2;

      for (var i = 0, length = other.keys.length; i < length; i++) {
        key = other.keys[i];

        if (key.$$is_string) {
          khash = key;
          map = smap;
          map2 = smap2;
        } else {
          khash = key.$hash();
          map = _map;
          map2 = _map2;
        }

        keys.push(key);
        map[khash] = map2[khash];
      }

      return self;
    
    };

    def.$select = TMP_12 = function() {
      var self = this, $iter = TMP_12.$$p, block = $iter || nil;

      TMP_12.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        return self.$enum_for("select")
      };
      
      var keys   = self.keys,
          _map   = self.map,
          smap   = self.smap,
          result = Opal.hash(),
          _map2  = result.map,
          smap2  = result.smap,
          keys2  = result.keys,
          map, map2, key, khash, value, object;

      for (var i = 0, length = keys.length; i < length; i++) {
        key = keys[i];

        if (key.$$is_string) {
          khash = key;
          map = smap;
          map2 = smap2;
        } else {
          khash = key.$hash();
          map = _map;
          map2 = _map2;
        }

        value = map[khash];
        object = block(key, value);

        if (object === $breaker) {
          return $breaker.$v;
        }

        if (object !== false && object !== nil) {
          keys2.push(key);
          map2[khash] = value;
        }
      }

      return result;
    
    };

    def['$select!'] = TMP_13 = function() {
      var self = this, $iter = TMP_13.$$p, block = $iter || nil;

      TMP_13.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        return self.$enum_for("select!")
      };
      
      var _map = self.map,
          smap = self.smap,
          keys = self.keys,
          result = nil,
          key, khash, value, object, map;

      for (var i = 0, length = keys.length; i < length; i++) {
        key = keys[i];

        if (key.$$is_string) {
          khash = key;
          map = smap;
        } else {
          khash = key.$hash();
          map = _map;
        }

        value = map[khash];
        object = block(key, value);

        if (object === $breaker) {
          return $breaker.$v;
        }

        if (object === false || object === nil) {
          keys.splice(i, 1);
          delete map[khash];

          length--;
          i--;
          result = self
        }
      }

      return result;
    
    };

    def.$shift = function() {
      var self = this;

      
      var keys = self.keys,
          _map = self.map,
          smap = self.smap,
          map, key, khash, value;

      if (keys.length) {
        key = keys[0];
        if (key.$$is_string) {
          khash = key;
          map = smap;
        } else {
          khash = key.$hash();
          map = _map;
        }
        value = map[khash];

        delete map[khash];
        keys.splice(0, 1);

        return [key, value];
      }

      return nil;
    
    };

    Opal.defn(self, '$size', def.$length);

    self.$alias_method("store", "[]=");

    def.$to_a = function() {
      var self = this;

      
      var keys = self.keys,
          _map = self.map,
          smap = self.smap,
          result = [],
          map, key, khash;

      for (var i = 0, length = keys.length; i < length; i++) {
        key = keys[i];

        if (key.$$is_string) {
          khash = key;
          map = smap;
        } else {
          khash = key.$hash();
          map = _map;
        }

        result.push([key, map[khash]]);
      }

      return result;
    
    };

    def.$to_h = function() {
      var self = this;

      
      if (self.$$class === Opal.Hash) {
        return self
      }

      var hash   = new Opal.Hash.$$alloc,
          cloned = self.$clone();

      hash.map  = cloned.map;
      hash.smap = cloned.smap;
      hash.keys = cloned.keys;
      hash.none = cloned.none;
      hash.proc = cloned.proc;

      return hash;
    ;
    };

    def.$to_hash = function() {
      var self = this;

      return self;
    };

    Opal.defn(self, '$to_s', def.$inspect);

    Opal.defn(self, '$update', def['$merge!']);

    Opal.defn(self, '$value?', def['$has_value?']);

    Opal.defn(self, '$values_at', def.$indexes);

    return (def.$values = function() {
      var self = this;

      
      var _map = self.map,
          smap = self.smap,
          keys = self.keys,
          result = [],
          map, khash, key;

      for (var i = 0, length = keys.length; i < length; i++) {
        key = keys[i];

        if (key.$$is_string) {
          khash = key;
          map = smap;
        } else {
          khash = key.$hash();
          map = _map;
        }

        result.push(map[khash]);
      }

      return result;
    
    }, nil) && 'values';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/string"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_divide(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs / rhs : lhs['$/'](rhs);
  }
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $gvars = Opal.gvars;

  Opal.add_stubs(['$require', '$include', '$coerce_to?', '$coerce_to', '$raise', '$===', '$format', '$to_s', '$respond_to?', '$to_str', '$<=>', '$==', '$=~', '$new', '$empty?', '$ljust', '$ceil', '$rjust', '$floor', '$to_a', '$each_char', '$to_proc', '$coerce_to!', '$initialize_clone', '$initialize_dup', '$enum_for', '$chomp', '$[]', '$to_i', '$class', '$each_line', '$match', '$captures', '$proc', '$shift', '$__send__', '$succ', '$escape']);
  self.$require("corelib/comparable");
  (function($base, $super) {
    function $String(){};
    var self = $String = $klass($base, $super, 'String', $String);

    var def = self.$$proto, $scope = self.$$scope, TMP_1, TMP_2, TMP_3, TMP_4, TMP_5, TMP_6, TMP_7, TMP_8, TMP_10;

    def.length = nil;
    self.$include($scope.get('Comparable'));

    def.$$is_string = true;

    def.$__id__ = function() {
      var self = this;

      return self.toString();
    };

    Opal.defn(self, '$object_id', def.$__id__);

    Opal.defs(self, '$try_convert', function(what) {
      var self = this;

      return $scope.get('Opal')['$coerce_to?'](what, $scope.get('String'), "to_str");
    });

    Opal.defs(self, '$new', function(str) {
      var self = this;

      if (str == null) {
        str = ""
      }
      str = $scope.get('Opal').$coerce_to(str, $scope.get('String'), "to_str");
      return new String(str);
    });

    def.$initialize = function(str) {
      var self = this;

      
      if (str === undefined) {
        return self;
      }
    
      return self.$raise($scope.get('NotImplementedError'), "Mutable strings are not supported in Opal.");
    };

    def['$%'] = function(data) {
      var $a, self = this;

      if ((($a = $scope.get('Array')['$==='](data)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return ($a = self).$format.apply($a, [self].concat(data))
        } else {
        return self.$format(self, data)
      };
    };

    def['$*'] = function(count) {
      var self = this;

      
      count = $scope.get('Opal').$coerce_to(count, $scope.get('Integer'), "to_int");

      if (count < 0) {
        self.$raise($scope.get('ArgumentError'), "negative argument")
      }

      if (count === 0) {
        return '';
      }

      var result = '',
          string = self.toString();

      // All credit for the bit-twiddling magic code below goes to Mozilla
      // polyfill implementation of String.prototype.repeat() posted here:
      // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String/repeat

      if (string.length * count >= 1 << 28) {
        self.$raise($scope.get('RangeError'), "multiply count must not overflow maximum string size")
      }

      for (;;) {
        if ((count & 1) === 1) {
          result += string;
        }
        count >>>= 1;
        if (count === 0) {
          break;
        }
        string += string;
      }

      return result;
    ;
    };

    def['$+'] = function(other) {
      var self = this;

      other = $scope.get('Opal').$coerce_to(other, $scope.get('String'), "to_str");
      return self + other.$to_s();
    };

    def['$<=>'] = function(other) {
      var $a, self = this;

      if ((($a = other['$respond_to?']("to_str")) !== nil && (!$a.$$is_boolean || $a == true))) {
        other = other.$to_str().$to_s();
        return self > other ? 1 : (self < other ? -1 : 0);
        } else {
        
        var cmp = other['$<=>'](self);

        if (cmp === nil) {
          return nil;
        }
        else {
          return cmp > 0 ? -1 : (cmp < 0 ? 1 : 0);
        }
      ;
      };
    };

    def['$<<'] = function(other) {
      var self = this;

      return self.$raise($scope.get('NotImplementedError'), "#<< not supported. Mutable String methods are not supported in Opal.");
    };

    def['$=='] = function(other) {
      var self = this;

      
      if (other.$$is_string) {
        return self.toString() === other.toString();
      }
      if ($scope.get('Opal')['$respond_to?'](other, "to_str")) {
        return other['$=='](self);
      }
      return false;
    ;
    };

    Opal.defn(self, '$eql?', def['$==']);

    Opal.defn(self, '$===', def['$==']);

    def['$=~'] = function(other) {
      var self = this;

      
      if (other.$$is_string) {
        self.$raise($scope.get('TypeError'), "type mismatch: String given");
      }

      return other['$=~'](self);
    ;
    };

    def['$[]'] = function(index, length) {
      var self = this;

      
      var size = self.length;

      if (index.$$is_range) {
        var exclude = index.exclude,
            length  = $scope.get('Opal').$coerce_to(index.end, $scope.get('Integer'), "to_int"),
            index   = $scope.get('Opal').$coerce_to(index.begin, $scope.get('Integer'), "to_int");

        if (Math.abs(index) > size) {
          return nil;
        }

        if (index < 0) {
          index += size;
        }

        if (length < 0) {
          length += size;
        }

        if (!exclude) {
          length += 1;
        }

        length = length - index;

        if (length < 0) {
          length = 0;
        }

        return self.substr(index, length);
      }


      if (index.$$is_string) {
        if (length != null) {
          self.$raise($scope.get('TypeError'))
        }
        return self.indexOf(index) !== -1 ? index : nil;
      }


      if (index.$$is_regexp) {
        var match = self.match(index);

        if (match === null) {
          $gvars["~"] = nil
          return nil;
        }

        $gvars["~"] = $scope.get('MatchData').$new(index, match)

        if (length == null) {
          return match[0];
        }

        length = $scope.get('Opal').$coerce_to(length, $scope.get('Integer'), "to_int");

        if (length < 0 && -length < match.length) {
          return match[length += match.length];
        }

        if (length >= 0 && length < match.length) {
          return match[length];
        }

        return nil;
      }


      index = $scope.get('Opal').$coerce_to(index, $scope.get('Integer'), "to_int");

      if (index < 0) {
        index += size;
      }

      if (length == null) {
        if (index >= size || index < 0) {
          return nil;
        }
        return self.substr(index, 1);
      }

      length = $scope.get('Opal').$coerce_to(length, $scope.get('Integer'), "to_int");

      if (length < 0) {
        return nil;
      }

      if (index > size || index < 0) {
        return nil;
      }

      return self.substr(index, length);
    ;
    };

    def.$capitalize = function() {
      var self = this;

      return self.charAt(0).toUpperCase() + self.substr(1).toLowerCase();
    };

    Opal.defn(self, '$capitalize!', def['$<<']);

    def.$casecmp = function(other) {
      var self = this;

      other = $scope.get('Opal').$coerce_to(other, $scope.get('String'), "to_str").$to_s();
      
      var ascii_only = /^[\x00-\x7F]*$/;
      if (ascii_only.test(self) && ascii_only.test(other)) {
        self = self.toLowerCase();
        other = other.toLowerCase();
      }
    
      return self['$<=>'](other);
    };

    def.$center = function(width, padstr) {
      var $a, self = this;

      if (padstr == null) {
        padstr = " "
      }
      width = $scope.get('Opal').$coerce_to(width, $scope.get('Integer'), "to_int");
      padstr = $scope.get('Opal').$coerce_to(padstr, $scope.get('String'), "to_str").$to_s();
      if ((($a = padstr['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$raise($scope.get('ArgumentError'), "zero width padding")};
      if ((($a = width <= self.length) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self};
      
      var ljustified = self.$ljust($rb_divide(($rb_plus(width, self.length)), 2).$ceil(), padstr),
          rjustified = self.$rjust($rb_divide(($rb_plus(width, self.length)), 2).$floor(), padstr);

      return rjustified + ljustified.slice(self.length);
    ;
    };

    def.$chars = TMP_1 = function() {
      var $a, $b, self = this, $iter = TMP_1.$$p, block = $iter || nil;

      TMP_1.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        return self.$each_char().$to_a()
      };
      return ($a = ($b = self).$each_char, $a.$$p = block.$to_proc(), $a).call($b);
    };

    def.$chomp = function(separator) {
      var $a, self = this;
      if ($gvars["/"] == null) $gvars["/"] = nil;

      if (separator == null) {
        separator = $gvars["/"]
      }
      if ((($a = separator === nil || self.length === 0) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self};
      separator = $scope.get('Opal')['$coerce_to!'](separator, $scope.get('String'), "to_str").$to_s();
      
      if (separator === "\n") {
        return self.replace(/\r?\n?$/, '');
      }
      else if (separator === "") {
        return self.replace(/(\r?\n)+$/, '');
      }
      else if (self.length > separator.length) {
        var tail = self.substr(self.length - separator.length, separator.length);

        if (tail === separator) {
          return self.substr(0, self.length - separator.length);
        }
      }
    
      return self;
    };

    Opal.defn(self, '$chomp!', def['$<<']);

    def.$chop = function() {
      var self = this;

      
      var length = self.length;

      if (length <= 1) {
        return "";
      }

      if (self.charAt(length - 1) === "\n" && self.charAt(length - 2) === "\r") {
        return self.substr(0, length - 2);
      }
      else {
        return self.substr(0, length - 1);
      }
    
    };

    Opal.defn(self, '$chop!', def['$<<']);

    def.$chr = function() {
      var self = this;

      return self.charAt(0);
    };

    def.$clone = function() {
      var self = this, copy = nil;

      copy = self.slice();
      copy.$initialize_clone(self);
      return copy;
    };

    def.$dup = function() {
      var self = this, copy = nil;

      copy = self.slice();
      copy.$initialize_dup(self);
      return copy;
    };

    def.$count = function(sets) {
      var self = this;

      sets = $slice.call(arguments, 0);
      
      if (sets.length === 0) {
        self.$raise($scope.get('ArgumentError'), "ArgumentError: wrong number of arguments (0 for 1+)")
      }
      var char_class = char_class_from_char_sets(sets);
      if (char_class === null) {
        return 0;
      }
      return self.length - self.replace(new RegExp(char_class, 'g'), '').length;
    ;
    };

    def.$delete = function(sets) {
      var self = this;

      sets = $slice.call(arguments, 0);
      
      if (sets.length === 0) {
        self.$raise($scope.get('ArgumentError'), "ArgumentError: wrong number of arguments (0 for 1+)")
      }
      var char_class = char_class_from_char_sets(sets);
      if (char_class === null) {
        return self;
      }
      return self.replace(new RegExp(char_class, 'g'), '');
    ;
    };

    Opal.defn(self, '$dup', def.$clone);

    def.$downcase = function() {
      var self = this;

      return self.toLowerCase();
    };

    Opal.defn(self, '$downcase!', def['$<<']);

    def.$each_char = TMP_2 = function() {
      var $a, self = this, $iter = TMP_2.$$p, block = $iter || nil;

      TMP_2.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("each_char")
      };
      
      for (var i = 0, length = self.length; i < length; i++) {
        ((($a = Opal.yield1(block, self.charAt(i))) === $breaker) ? $breaker.$v : $a);
      }
    
      return self;
    };

    def.$each_line = TMP_3 = function(separator) {
      var $a, self = this, $iter = TMP_3.$$p, $yield = $iter || nil;
      if ($gvars["/"] == null) $gvars["/"] = nil;

      if (separator == null) {
        separator = $gvars["/"]
      }
      TMP_3.$$p = null;
      if (($yield !== nil)) {
        } else {
        return self.$enum_for("each_line", separator)
      };
      
      if (separator === nil) {
        ((($a = Opal.yield1($yield, self)) === $breaker) ? $breaker.$v : $a);
        return self;
      }

      separator = $scope.get('Opal').$coerce_to(separator, $scope.get('String'), "to_str")

      if (separator.length === 0) {
        for (var a = self.split(/(\n{2,})/), i = 0, n = a.length; i < n; i += 2) {
          if (a[i] || a[i + 1]) {
            ((($a = Opal.yield1($yield, (a[i] || "") + (a[i + 1] || ""))) === $breaker) ? $breaker.$v : $a);
          }
        }
        return self;
      }

      var chomped  = self.$chomp(separator),
          trailing = self.length != chomped.length,
          splitted = chomped.split(separator);

      for (var i = 0, length = splitted.length; i < length; i++) {
        if (i < length - 1 || trailing) {
          ((($a = Opal.yield1($yield, splitted[i] + separator)) === $breaker) ? $breaker.$v : $a);
        }
        else {
          ((($a = Opal.yield1($yield, splitted[i])) === $breaker) ? $breaker.$v : $a);
        }
      }
    ;
      return self;
    };

    def['$empty?'] = function() {
      var self = this;

      return self.length === 0;
    };

    def['$end_with?'] = function(suffixes) {
      var self = this;

      suffixes = $slice.call(arguments, 0);
      
      for (var i = 0, length = suffixes.length; i < length; i++) {
        var suffix = $scope.get('Opal').$coerce_to(suffixes[i], $scope.get('String'), "to_str").$to_s();

        if (self.length >= suffix.length &&
            self.substr(self.length - suffix.length, suffix.length) == suffix) {
          return true;
        }
      }
    
      return false;
    };

    Opal.defn(self, '$eql?', def['$==']);

    Opal.defn(self, '$equal?', def['$===']);

    def.$gsub = TMP_4 = function(pattern, replacement) {
      var self = this, $iter = TMP_4.$$p, block = $iter || nil;

      TMP_4.$$p = null;
      
      var result = '', match_data = nil, index = 0, match, _replacement;

      if (pattern.$$is_regexp) {
        pattern = new RegExp(pattern.source, 'gm' + (pattern.ignoreCase ? 'i' : ''));
      } else {
        pattern = $scope.get('Opal').$coerce_to(pattern, $scope.get('String'), "to_str");
        pattern = new RegExp(pattern.replace(/[.*+?^${}()|[\]\\]/g, '\\$&'), 'gm');
      }

      while (true) {
        match = pattern.exec(self);

        if (match === null) {
          $gvars["~"] = nil
          result += self.slice(index);
          break;
        }

        match_data = $scope.get('MatchData').$new(pattern, match);

        if (replacement === undefined) {
          if (block === nil) {
            self.$raise($scope.get('ArgumentError'), "wrong number of arguments (1 for 2)")
          }
          _replacement = block(match[0]);
        }
        else if (replacement.$$is_hash) {
          _replacement = (replacement)['$[]'](match[0]).$to_s();
        }
        else {
          if (!replacement.$$is_string) {
            replacement = $scope.get('Opal').$coerce_to(replacement, $scope.get('String'), "to_str");
          }
          _replacement = replacement.replace(/([\\]+)([0-9+&`'])/g, function (original, slashes, command) {
            if (slashes.length % 2 === 0) {
              return original;
            }
            switch (command) {
            case "+":
              for (var i = match.length - 1; i > 0; i--) {
                if (match[i] !== undefined) {
                  return slashes.slice(1) + match[i];
                }
              }
              return '';
            case "&": return slashes.slice(1) + match[0];
            case "`": return slashes.slice(1) + self.slice(0, match.index);
            case "'": return slashes.slice(1) + self.slice(match.index + match[0].length);
            default:  return slashes.slice(1) + (match[command] || '');
            }
          }).replace(/\\\\/g, '\\');
        }

        if (pattern.lastIndex === match.index) {
          result += (_replacement + self.slice(index, match.index + 1))
          pattern.lastIndex += 1;
        }
        else {
          result += (self.slice(index, match.index) + _replacement)
        }
        index = pattern.lastIndex;
      }

      $gvars["~"] = match_data
      return result;
    
    };

    Opal.defn(self, '$gsub!', def['$<<']);

    def.$hash = function() {
      var self = this;

      return self.toString();
    };

    def.$hex = function() {
      var self = this;

      return self.$to_i(16);
    };

    def['$include?'] = function(other) {
      var $a, self = this;

      
      if (other.$$is_string) {
        return self.indexOf(other) !== -1;
      }
    
      if ((($a = other['$respond_to?']("to_str")) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise($scope.get('TypeError'), "no implicit conversion of " + (other.$class()) + " into String")
      };
      return self.indexOf(other.$to_str()) !== -1;
    };

    def.$index = function(search, offset) {
      var self = this;

      
      var index,
          match,
          regex;

      if (offset === undefined) {
        offset = 0;
      } else {
        offset = $scope.get('Opal').$coerce_to(offset, $scope.get('Integer'), "to_int");
        if (offset < 0) {
          offset += self.length;
          if (offset < 0) {
            return nil;
          }
        }
      }

      if (search.$$is_regexp) {
        regex = new RegExp(search.source, 'gm' + (search.ignoreCase ? 'i' : ''));
        while (true) {
          match = regex.exec(self);
          if (match === null) {
            $gvars["~"] = nil;
            index = -1;
            break;
          }
          if (match.index >= offset) {
            $gvars["~"] = $scope.get('MatchData').$new(regex, match)
            index = match.index;
            break;
          }
          regex.lastIndex = match.index + 1;
        }
      } else {
        search = $scope.get('Opal').$coerce_to(search, $scope.get('String'), "to_str");
        if (search.length === 0 && offset > self.length) {
          index = -1;
        } else {
          index = self.indexOf(search, offset);
        }
      }

      return index === -1 ? nil : index;
    
    };

    def.$inspect = function() {
      var self = this;

      
      var escapable = /[\\\"\x00-\x1f\x7f-\x9f\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,
          meta = {
            '\u0007': '\\a',
            '\u001b': '\\e',
            '\b': '\\b',
            '\t': '\\t',
            '\n': '\\n',
            '\f': '\\f',
            '\r': '\\r',
            '\v': '\\v',
            '"' : '\\"',
            '\\': '\\\\'
          },
          escaped = self.replace(escapable, function (chr) {
            return meta[chr] || '\\u' + ('0000' + chr.charCodeAt(0).toString(16).toUpperCase()).slice(-4);
          });
      return '"' + escaped.replace(/\#[\$\@\{]/g, '\\$&') + '"';
    
    };

    def.$intern = function() {
      var self = this;

      return self;
    };

    def.$lines = TMP_5 = function(separator) {
      var $a, $b, self = this, $iter = TMP_5.$$p, block = $iter || nil, e = nil;
      if ($gvars["/"] == null) $gvars["/"] = nil;

      if (separator == null) {
        separator = $gvars["/"]
      }
      TMP_5.$$p = null;
      e = ($a = ($b = self).$each_line, $a.$$p = block.$to_proc(), $a).call($b, separator);
      if (block !== false && block !== nil) {
        return self
        } else {
        return e.$to_a()
      };
    };

    def.$length = function() {
      var self = this;

      return self.length;
    };

    def.$ljust = function(width, padstr) {
      var $a, self = this;

      if (padstr == null) {
        padstr = " "
      }
      width = $scope.get('Opal').$coerce_to(width, $scope.get('Integer'), "to_int");
      padstr = $scope.get('Opal').$coerce_to(padstr, $scope.get('String'), "to_str").$to_s();
      if ((($a = padstr['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$raise($scope.get('ArgumentError'), "zero width padding")};
      if ((($a = width <= self.length) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self};
      
      var index  = -1,
          result = "";

      width -= self.length;

      while (++index < width) {
        result += padstr;
      }

      return self + result.slice(0, width);
    
    };

    def.$lstrip = function() {
      var self = this;

      return self.replace(/^\s*/, '');
    };

    Opal.defn(self, '$lstrip!', def['$<<']);

    def.$match = TMP_6 = function(pattern, pos) {
      var $a, $b, self = this, $iter = TMP_6.$$p, block = $iter || nil;

      TMP_6.$$p = null;
      if ((($a = ((($b = $scope.get('String')['$==='](pattern)) !== false && $b !== nil) ? $b : pattern['$respond_to?']("to_str"))) !== nil && (!$a.$$is_boolean || $a == true))) {
        pattern = $scope.get('Regexp').$new(pattern.$to_str())};
      if ((($a = $scope.get('Regexp')['$==='](pattern)) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise($scope.get('TypeError'), "wrong argument type " + (pattern.$class()) + " (expected Regexp)")
      };
      return ($a = ($b = pattern).$match, $a.$$p = block.$to_proc(), $a).call($b, self, pos);
    };

    def.$next = function() {
      var self = this;

      
      var i = self.length;
      if (i === 0) {
        return '';
      }
      var result = self;
      var first_alphanum_char_index = self.search(/[a-zA-Z0-9]/);
      var carry = false;
      var code;
      while (i--) {
        code = self.charCodeAt(i);
        if ((code >= 48 && code <= 57) ||
          (code >= 65 && code <= 90) ||
          (code >= 97 && code <= 122)) {
          switch (code) {
          case 57:
            carry = true;
            code = 48;
            break;
          case 90:
            carry = true;
            code = 65;
            break;
          case 122:
            carry = true;
            code = 97;
            break;
          default:
            carry = false;
            code += 1;
          }
        } else {
          if (first_alphanum_char_index === -1) {
            if (code === 255) {
              carry = true;
              code = 0;
            } else {
              carry = false;
              code += 1;
            }
          } else {
            carry = true;
          }
        }
        result = result.slice(0, i) + String.fromCharCode(code) + result.slice(i + 1);
        if (carry && (i === 0 || i === first_alphanum_char_index)) {
          switch (code) {
          case 65:
            break;
          case 97:
            break;
          default:
            code += 1;
          }
          if (i === 0) {
            result = String.fromCharCode(code) + result;
          } else {
            result = result.slice(0, i) + String.fromCharCode(code) + result.slice(i);
          }
          carry = false;
        }
        if (!carry) {
          break;
        }
      }
      return result;
    
    };

    Opal.defn(self, '$next!', def['$<<']);

    def.$oct = function() {
      var self = this;

      
      var result,
          string = self,
          radix = 8;

      if (/^\s*_/.test(string)) {
        return 0;
      }

      string = string.replace(/^(\s*[+-]?)(0[bodx]?)(.+)$/i, function (original, head, flag, tail) {
        switch (tail.charAt(0)) {
        case '+':
        case '-':
          return original;
        case '0':
          if (tail.charAt(1) === 'x' && flag === '0x') {
            return original;
          }
        }
        switch (flag) {
        case '0b':
          radix = 2;
          break;
        case '0':
        case '0o':
          radix = 8;
          break;
        case '0d':
          radix = 10;
          break;
        case '0x':
          radix = 16;
          break;
        }
        return head + tail;
      });

      result = parseInt(string.replace(/_(?!_)/g, ''), radix);
      return isNaN(result) ? 0 : result;
    
    };

    def.$ord = function() {
      var self = this;

      return self.charCodeAt(0);
    };

    def.$partition = function(sep) {
      var self = this;

      
      var i, m;

      if (sep.$$is_regexp) {
        m = sep.exec(self);
        if (m === null) {
          i = -1;
        } else {
          $scope.get('MatchData').$new(sep, m);
          sep = m[0];
          i = m.index;
        }
      } else {
        sep = $scope.get('Opal').$coerce_to(sep, $scope.get('String'), "to_str");
        i = self.indexOf(sep);
      }

      if (i === -1) {
        return [self, '', ''];
      }

      return [
        self.slice(0, i),
        self.slice(i, i + sep.length),
        self.slice(i + sep.length)
      ];
    
    };

    def.$reverse = function() {
      var self = this;

      return self.split('').reverse().join('');
    };

    Opal.defn(self, '$reverse!', def['$<<']);

    def.$rindex = function(search, offset) {
      var self = this;

      
      var i, m, r, _m;

      if (offset === undefined) {
        offset = self.length;
      } else {
        offset = $scope.get('Opal').$coerce_to(offset, $scope.get('Integer'), "to_int");
        if (offset < 0) {
          offset += self.length;
          if (offset < 0) {
            return nil;
          }
        }
      }

      if (search.$$is_regexp) {
        m = null;
        r = new RegExp(search.source, 'gm' + (search.ignoreCase ? 'i' : ''));
        while (true) {
          _m = r.exec(self);
          if (_m === null || _m.index > offset) {
            break;
          }
          m = _m;
          r.lastIndex = m.index + 1;
        }
        if (m === null) {
          $gvars["~"] = nil
          i = -1;
        } else {
          $scope.get('MatchData').$new(r, m);
          i = m.index;
        }
      } else {
        search = $scope.get('Opal').$coerce_to(search, $scope.get('String'), "to_str");
        i = self.lastIndexOf(search, offset);
      }

      return i === -1 ? nil : i;
    
    };

    def.$rjust = function(width, padstr) {
      var $a, self = this;

      if (padstr == null) {
        padstr = " "
      }
      width = $scope.get('Opal').$coerce_to(width, $scope.get('Integer'), "to_int");
      padstr = $scope.get('Opal').$coerce_to(padstr, $scope.get('String'), "to_str").$to_s();
      if ((($a = padstr['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$raise($scope.get('ArgumentError'), "zero width padding")};
      if ((($a = width <= self.length) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self};
      
      var chars     = Math.floor(width - self.length),
          patterns  = Math.floor(chars / padstr.length),
          result    = Array(patterns + 1).join(padstr),
          remaining = chars - result.length;

      return result + padstr.slice(0, remaining) + self;
    
    };

    def.$rpartition = function(sep) {
      var self = this;

      
      var i, m, r, _m;

      if (sep.$$is_regexp) {
        m = null;
        r = new RegExp(sep.source, 'gm' + (sep.ignoreCase ? 'i' : ''));

        while (true) {
          _m = r.exec(self);
          if (_m === null) {
            break;
          }
          m = _m;
          r.lastIndex = m.index + 1;
        }

        if (m === null) {
          i = -1;
        } else {
          $scope.get('MatchData').$new(r, m);
          sep = m[0];
          i = m.index;
        }

      } else {
        sep = $scope.get('Opal').$coerce_to(sep, $scope.get('String'), "to_str");
        i = self.lastIndexOf(sep);
      }

      if (i === -1) {
        return ['', '', self];
      }

      return [
        self.slice(0, i),
        self.slice(i, i + sep.length),
        self.slice(i + sep.length)
      ];
    
    };

    def.$rstrip = function() {
      var self = this;

      return self.replace(/[\s\u0000]*$/, '');
    };

    def.$scan = TMP_7 = function(pattern) {
      var self = this, $iter = TMP_7.$$p, block = $iter || nil;

      TMP_7.$$p = null;
      
      var result = [],
          match_data = nil,
          match;

      if (pattern.$$is_regexp) {
        pattern = new RegExp(pattern.source, 'gm' + (pattern.ignoreCase ? 'i' : ''));
      } else {
        pattern = $scope.get('Opal').$coerce_to(pattern, $scope.get('String'), "to_str");
        pattern = new RegExp(pattern.replace(/[.*+?^${}()|[\]\\]/g, '\\$&'), 'gm');
      }

      while ((match = pattern.exec(self)) != null) {
        match_data = $scope.get('MatchData').$new(pattern, match);
        if (block === nil) {
          match.length == 1 ? result.push(match[0]) : result.push((match_data).$captures());
        } else {
          match.length == 1 ? block(match[0]) : block.call(self, (match_data).$captures());
        }
        if (pattern.lastIndex === match.index) {
          pattern.lastIndex += 1;
        }
      }

      $gvars["~"] = match_data

      return (block !== nil ? self : result);
    
    };

    Opal.defn(self, '$size', def.$length);

    Opal.defn(self, '$slice', def['$[]']);

    Opal.defn(self, '$slice!', def['$<<']);

    def.$split = function(pattern, limit) {
      var $a, self = this;
      if ($gvars[";"] == null) $gvars[";"] = nil;

      
      if (self.length === 0) {
        return [];
      }

      if (limit === undefined) {
        limit = 0;
      } else {
        limit = $scope.get('Opal')['$coerce_to!'](limit, $scope.get('Integer'), "to_int");
        if (limit === 1) {
          return [self];
        }
      }

      if (pattern === undefined || pattern === nil) {
        pattern = ((($a = $gvars[";"]) !== false && $a !== nil) ? $a : " ");
      }

      var result = [],
          string = self.toString(),
          index = 0,
          match,
          i;

      if (pattern.$$is_regexp) {
        pattern = new RegExp(pattern.source, 'gm' + (pattern.ignoreCase ? 'i' : ''));
      } else {
        pattern = $scope.get('Opal').$coerce_to(pattern, $scope.get('String'), "to_str").$to_s();
        if (pattern === ' ') {
          pattern = /\s+/gm;
          string = string.replace(/^\s+/, '');
        } else {
          pattern = new RegExp(pattern.replace(/[.*+?^${}()|[\]\\]/g, '\\$&'), 'gm');
        }
      }

      result = string.split(pattern);

      if (result.length === 1 && result[0] === string) {
        return result;
      }

      while ((i = result.indexOf(undefined)) !== -1) {
        result.splice(i, 1);
      }

      if (limit === 0) {
        while (result[result.length - 1] === '') {
          result.length -= 1;
        }
        return result;
      }

      match = pattern.exec(string);

      if (limit < 0) {
        if (match !== null && match[0] === '' && pattern.source.indexOf('(?=') === -1) {
          for (i = 0; i < match.length; i++) {
            result.push('');
          }
        }
        return result;
      }

      if (match !== null && match[0] === '') {
        result.splice(limit - 1, result.length - 1, result.slice(limit - 1).join(''));
        return result;
      }

      i = 0;
      while (match !== null) {
        i++;
        index = pattern.lastIndex;
        if (i + 1 === limit) {
          break;
        }
        match = pattern.exec(string);
      }

      result.splice(limit - 1, result.length - 1, string.slice(index));
      return result;
    
    };

    def.$squeeze = function(sets) {
      var self = this;

      sets = $slice.call(arguments, 0);
      
      if (sets.length === 0) {
        return self.replace(/(.)\1+/g, '$1');
      }
      var char_class = char_class_from_char_sets(sets);
      if (char_class === null) {
        return self;
      }
      return self.replace(new RegExp('(' + char_class + ')\\1+', 'g'), '$1');
    
    };

    Opal.defn(self, '$squeeze!', def['$<<']);

    def['$start_with?'] = function(prefixes) {
      var self = this;

      prefixes = $slice.call(arguments, 0);
      
      for (var i = 0, length = prefixes.length; i < length; i++) {
        var prefix = $scope.get('Opal').$coerce_to(prefixes[i], $scope.get('String'), "to_str").$to_s();

        if (self.indexOf(prefix) === 0) {
          return true;
        }
      }

      return false;
    
    };

    def.$strip = function() {
      var self = this;

      return self.replace(/^\s*/, '').replace(/[\s\u0000]*$/, '');
    };

    Opal.defn(self, '$strip!', def['$<<']);

    def.$sub = TMP_8 = function(pattern, replacement) {
      var self = this, $iter = TMP_8.$$p, block = $iter || nil;

      TMP_8.$$p = null;
      
      if (!pattern.$$is_regexp) {
        pattern = $scope.get('Opal').$coerce_to(pattern, $scope.get('String'), "to_str");
        pattern = new RegExp(pattern.replace(/[.*+?^${}()|[\]\\]/g, '\\$&'));
      }

      var result = pattern.exec(self);

      if (result === null) {
        $gvars["~"] = nil
        return self.toString();
      }

      $scope.get('MatchData').$new(pattern, result)

      if (replacement === undefined) {
        if (block === nil) {
          self.$raise($scope.get('ArgumentError'), "wrong number of arguments (1 for 2)")
        }
        return self.slice(0, result.index) + block(result[0]) + self.slice(result.index + result[0].length);
      }

      if (replacement.$$is_hash) {
        return self.slice(0, result.index) + (replacement)['$[]'](result[0]).$to_s() + self.slice(result.index + result[0].length);
      }

      replacement = $scope.get('Opal').$coerce_to(replacement, $scope.get('String'), "to_str");

      replacement = replacement.replace(/([\\]+)([0-9+&`'])/g, function (original, slashes, command) {
        if (slashes.length % 2 === 0) {
          return original;
        }
        switch (command) {
        case "+":
          for (var i = result.length - 1; i > 0; i--) {
            if (result[i] !== undefined) {
              return slashes.slice(1) + result[i];
            }
          }
          return '';
        case "&": return slashes.slice(1) + result[0];
        case "`": return slashes.slice(1) + self.slice(0, result.index);
        case "'": return slashes.slice(1) + self.slice(result.index + result[0].length);
        default:  return slashes.slice(1) + (result[command] || '');
        }
      }).replace(/\\\\/g, '\\');

      return self.slice(0, result.index) + replacement + self.slice(result.index + result[0].length);
    ;
    };

    Opal.defn(self, '$sub!', def['$<<']);

    Opal.defn(self, '$succ', def.$next);

    Opal.defn(self, '$succ!', def['$<<']);

    def.$sum = function(n) {
      var self = this;

      if (n == null) {
        n = 16
      }
      
      n = $scope.get('Opal').$coerce_to(n, $scope.get('Integer'), "to_int");

      var result = 0,
          length = self.length,
          i = 0;

      for (; i < length; i++) {
        result += self.charCodeAt(i);
      }

      if (n <= 0) {
        return result;
      }

      return result & (Math.pow(2, n) - 1);
    ;
    };

    def.$swapcase = function() {
      var self = this;

      
      var str = self.replace(/([a-z]+)|([A-Z]+)/g, function($0,$1,$2) {
        return $1 ? $0.toUpperCase() : $0.toLowerCase();
      });

      if (self.constructor === String) {
        return str;
      }

      return self.$class().$new(str);
    
    };

    Opal.defn(self, '$swapcase!', def['$<<']);

    def.$to_f = function() {
      var self = this;

      
      if (self.charAt(0) === '_') {
        return 0;
      }

      var result = parseFloat(self.replace(/_/g, ''));

      if (isNaN(result) || result == Infinity || result == -Infinity) {
        return 0;
      }
      else {
        return result;
      }
    
    };

    def.$to_i = function(base) {
      var self = this;

      if (base == null) {
        base = 10
      }
      
      var result,
          string = self.toLowerCase(),
          radix = $scope.get('Opal').$coerce_to(base, $scope.get('Integer'), "to_int");

      if (radix === 1 || radix < 0 || radix > 36) {
        self.$raise($scope.get('ArgumentError'), "invalid radix " + (radix))
      }

      if (/^\s*_/.test(string)) {
        return 0;
      }

      string = string.replace(/^(\s*[+-]?)(0[bodx]?)(.+)$/, function (original, head, flag, tail) {
        switch (tail.charAt(0)) {
        case '+':
        case '-':
          return original;
        case '0':
          if (tail.charAt(1) === 'x' && flag === '0x' && (radix === 0 || radix === 16)) {
            return original;
          }
        }
        switch (flag) {
        case '0b':
          if (radix === 0 || radix === 2) {
            radix = 2;
            return head + tail;
          }
          break;
        case '0':
        case '0o':
          if (radix === 0 || radix === 8) {
            radix = 8;
            return head + tail;
          }
          break;
        case '0d':
          if (radix === 0 || radix === 10) {
            radix = 10;
            return head + tail;
          }
          break;
        case '0x':
          if (radix === 0 || radix === 16) {
            radix = 16;
            return head + tail;
          }
          break;
        }
        return original
      });

      result = parseInt(string.replace(/_(?!_)/g, ''), radix);
      return isNaN(result) ? 0 : result;
    ;
    };

    def.$to_proc = function() {
      var $a, $b, TMP_9, self = this, sym = nil;

      sym = self;
      return ($a = ($b = self).$proc, $a.$$p = (TMP_9 = function(args){var self = TMP_9.$$s || this, block, $a, $b, obj = nil;
args = $slice.call(arguments, 0);
        block = TMP_9.$$p || nil, TMP_9.$$p = null;
      if ((($a = args['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
          self.$raise($scope.get('ArgumentError'), "no receiver given")};
        obj = args.$shift();
        return ($a = ($b = obj).$__send__, $a.$$p = block.$to_proc(), $a).apply($b, [sym].concat(args));}, TMP_9.$$s = self, TMP_9), $a).call($b);
    };

    def.$to_s = function() {
      var self = this;

      return self.toString();
    };

    Opal.defn(self, '$to_str', def.$to_s);

    Opal.defn(self, '$to_sym', def.$intern);

    def.$tr = function(from, to) {
      var self = this;

      from = $scope.get('Opal').$coerce_to(from, $scope.get('String'), "to_str").$to_s();
      to = $scope.get('Opal').$coerce_to(to, $scope.get('String'), "to_str").$to_s();
      
      if (from.length == 0 || from === to) {
        return self;
      }

      var subs = {};
      var from_chars = from.split('');
      var from_length = from_chars.length;
      var to_chars = to.split('');
      var to_length = to_chars.length;

      var inverse = false;
      var global_sub = null;
      if (from_chars[0] === '^' && from_chars.length > 1) {
        inverse = true;
        from_chars.shift();
        global_sub = to_chars[to_length - 1]
        from_length -= 1;
      }

      var from_chars_expanded = [];
      var last_from = null;
      var in_range = false;
      for (var i = 0; i < from_length; i++) {
        var ch = from_chars[i];
        if (last_from == null) {
          last_from = ch;
          from_chars_expanded.push(ch);
        }
        else if (ch === '-') {
          if (last_from === '-') {
            from_chars_expanded.push('-');
            from_chars_expanded.push('-');
          }
          else if (i == from_length - 1) {
            from_chars_expanded.push('-');
          }
          else {
            in_range = true;
          }
        }
        else if (in_range) {
          var start = last_from.charCodeAt(0);
          var end = ch.charCodeAt(0);
          if (start > end) {
            self.$raise($scope.get('ArgumentError'), "invalid range \"" + (String.fromCharCode(start)) + "-" + (String.fromCharCode(end)) + "\" in string transliteration")
          }
          for (var c = start + 1; c < end; c++) {
            from_chars_expanded.push(String.fromCharCode(c));
          }
          from_chars_expanded.push(ch);
          in_range = null;
          last_from = null;
        }
        else {
          from_chars_expanded.push(ch);
        }
      }

      from_chars = from_chars_expanded;
      from_length = from_chars.length;

      if (inverse) {
        for (var i = 0; i < from_length; i++) {
          subs[from_chars[i]] = true;
        }
      }
      else {
        if (to_length > 0) {
          var to_chars_expanded = [];
          var last_to = null;
          var in_range = false;
          for (var i = 0; i < to_length; i++) {
            var ch = to_chars[i];
            if (last_from == null) {
              last_from = ch;
              to_chars_expanded.push(ch);
            }
            else if (ch === '-') {
              if (last_to === '-') {
                to_chars_expanded.push('-');
                to_chars_expanded.push('-');
              }
              else if (i == to_length - 1) {
                to_chars_expanded.push('-');
              }
              else {
                in_range = true;
              }
            }
            else if (in_range) {
              var start = last_from.charCodeAt(0);
              var end = ch.charCodeAt(0);
              if (start > end) {
                self.$raise($scope.get('ArgumentError'), "invalid range \"" + (String.fromCharCode(start)) + "-" + (String.fromCharCode(end)) + "\" in string transliteration")
              }
              for (var c = start + 1; c < end; c++) {
                to_chars_expanded.push(String.fromCharCode(c));
              }
              to_chars_expanded.push(ch);
              in_range = null;
              last_from = null;
            }
            else {
              to_chars_expanded.push(ch);
            }
          }

          to_chars = to_chars_expanded;
          to_length = to_chars.length;
        }

        var length_diff = from_length - to_length;
        if (length_diff > 0) {
          var pad_char = (to_length > 0 ? to_chars[to_length - 1] : '');
          for (var i = 0; i < length_diff; i++) {
            to_chars.push(pad_char);
          }
        }

        for (var i = 0; i < from_length; i++) {
          subs[from_chars[i]] = to_chars[i];
        }
      }

      var new_str = ''
      for (var i = 0, length = self.length; i < length; i++) {
        var ch = self.charAt(i);
        var sub = subs[ch];
        if (inverse) {
          new_str += (sub == null ? global_sub : ch);
        }
        else {
          new_str += (sub != null ? sub : ch);
        }
      }
      return new_str;
    
    };

    Opal.defn(self, '$tr!', def['$<<']);

    def.$tr_s = function(from, to) {
      var self = this;

      from = $scope.get('Opal').$coerce_to(from, $scope.get('String'), "to_str").$to_s();
      to = $scope.get('Opal').$coerce_to(to, $scope.get('String'), "to_str").$to_s();
      
      if (from.length == 0) {
        return self;
      }

      var subs = {};
      var from_chars = from.split('');
      var from_length = from_chars.length;
      var to_chars = to.split('');
      var to_length = to_chars.length;

      var inverse = false;
      var global_sub = null;
      if (from_chars[0] === '^' && from_chars.length > 1) {
        inverse = true;
        from_chars.shift();
        global_sub = to_chars[to_length - 1]
        from_length -= 1;
      }

      var from_chars_expanded = [];
      var last_from = null;
      var in_range = false;
      for (var i = 0; i < from_length; i++) {
        var ch = from_chars[i];
        if (last_from == null) {
          last_from = ch;
          from_chars_expanded.push(ch);
        }
        else if (ch === '-') {
          if (last_from === '-') {
            from_chars_expanded.push('-');
            from_chars_expanded.push('-');
          }
          else if (i == from_length - 1) {
            from_chars_expanded.push('-');
          }
          else {
            in_range = true;
          }
        }
        else if (in_range) {
          var start = last_from.charCodeAt(0);
          var end = ch.charCodeAt(0);
          if (start > end) {
            self.$raise($scope.get('ArgumentError'), "invalid range \"" + (String.fromCharCode(start)) + "-" + (String.fromCharCode(end)) + "\" in string transliteration")
          }
          for (var c = start + 1; c < end; c++) {
            from_chars_expanded.push(String.fromCharCode(c));
          }
          from_chars_expanded.push(ch);
          in_range = null;
          last_from = null;
        }
        else {
          from_chars_expanded.push(ch);
        }
      }

      from_chars = from_chars_expanded;
      from_length = from_chars.length;

      if (inverse) {
        for (var i = 0; i < from_length; i++) {
          subs[from_chars[i]] = true;
        }
      }
      else {
        if (to_length > 0) {
          var to_chars_expanded = [];
          var last_to = null;
          var in_range = false;
          for (var i = 0; i < to_length; i++) {
            var ch = to_chars[i];
            if (last_from == null) {
              last_from = ch;
              to_chars_expanded.push(ch);
            }
            else if (ch === '-') {
              if (last_to === '-') {
                to_chars_expanded.push('-');
                to_chars_expanded.push('-');
              }
              else if (i == to_length - 1) {
                to_chars_expanded.push('-');
              }
              else {
                in_range = true;
              }
            }
            else if (in_range) {
              var start = last_from.charCodeAt(0);
              var end = ch.charCodeAt(0);
              if (start > end) {
                self.$raise($scope.get('ArgumentError'), "invalid range \"" + (String.fromCharCode(start)) + "-" + (String.fromCharCode(end)) + "\" in string transliteration")
              }
              for (var c = start + 1; c < end; c++) {
                to_chars_expanded.push(String.fromCharCode(c));
              }
              to_chars_expanded.push(ch);
              in_range = null;
              last_from = null;
            }
            else {
              to_chars_expanded.push(ch);
            }
          }

          to_chars = to_chars_expanded;
          to_length = to_chars.length;
        }

        var length_diff = from_length - to_length;
        if (length_diff > 0) {
          var pad_char = (to_length > 0 ? to_chars[to_length - 1] : '');
          for (var i = 0; i < length_diff; i++) {
            to_chars.push(pad_char);
          }
        }

        for (var i = 0; i < from_length; i++) {
          subs[from_chars[i]] = to_chars[i];
        }
      }
      var new_str = ''
      var last_substitute = null
      for (var i = 0, length = self.length; i < length; i++) {
        var ch = self.charAt(i);
        var sub = subs[ch]
        if (inverse) {
          if (sub == null) {
            if (last_substitute == null) {
              new_str += global_sub;
              last_substitute = true;
            }
          }
          else {
            new_str += ch;
            last_substitute = null;
          }
        }
        else {
          if (sub != null) {
            if (last_substitute == null || last_substitute !== sub) {
              new_str += sub;
              last_substitute = sub;
            }
          }
          else {
            new_str += ch;
            last_substitute = null;
          }
        }
      }
      return new_str;
    
    };

    Opal.defn(self, '$tr_s!', def['$<<']);

    def.$upcase = function() {
      var self = this;

      return self.toUpperCase();
    };

    Opal.defn(self, '$upcase!', def['$<<']);

    def.$freeze = function() {
      var self = this;

      return self;
    };

    def['$frozen?'] = function() {
      var self = this;

      return true;
    };

    def.$upto = TMP_10 = function(stop, excl) {
      var self = this, $iter = TMP_10.$$p, block = $iter || nil;

      if (excl == null) {
        excl = false
      }
      TMP_10.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("upto", stop, excl)
      };
      stop = $scope.get('Opal').$coerce_to(stop, $scope.get('String'), "to_str");
      
      var a, b, s = self.toString();

      if (s.length === 1 && stop.length === 1) {

        a = s.charCodeAt(0);
        b = stop.charCodeAt(0);

        while (a <= b) {
          if (excl && a === b) {
            break;
          }
          block(String.fromCharCode(a));
          a += 1;
        }

      } else if (parseInt(s).toString() === s && parseInt(stop).toString() === stop) {

        a = parseInt(s);
        b = parseInt(stop);

        while (a <= b) {
          if (excl && a === b) {
            break;
          }
          block(a.toString());
          a += 1;
        }

      } else {

        while (s.length <= stop.length && s <= stop) {
          if (excl && s === stop) {
            break;
          }
          block(s);
          s = (s).$succ();
        }

      }
      return self;
    
    };

    
    function char_class_from_char_sets(sets) {
      function explode_sequences_in_character_set(set) {
        var result = '',
            i, len = set.length,
            curr_char,
            skip_next_dash,
            char_code_from,
            char_code_upto,
            char_code;
        for (i = 0; i < len; i++) {
          curr_char = set.charAt(i);
          if (curr_char === '-' && i > 0 && i < (len - 1) && !skip_next_dash) {
            char_code_from = set.charCodeAt(i - 1);
            char_code_upto = set.charCodeAt(i + 1);
            if (char_code_from > char_code_upto) {
              self.$raise($scope.get('ArgumentError'), "invalid range \"" + (char_code_from) + "-" + (char_code_upto) + "\" in string transliteration")
            }
            for (char_code = char_code_from + 1; char_code < char_code_upto + 1; char_code++) {
              result += String.fromCharCode(char_code);
            }
            skip_next_dash = true;
            i++;
          } else {
            skip_next_dash = (curr_char === '\\');
            result += curr_char;
          }
        }
        return result;
      }

      function intersection(setA, setB) {
        if (setA.length === 0) {
          return setB;
        }
        var result = '',
            i, len = setA.length,
            chr;
        for (i = 0; i < len; i++) {
          chr = setA.charAt(i);
          if (setB.indexOf(chr) !== -1) {
            result += chr;
          }
        }
        return result;
      }

      var i, len, set, neg, chr, tmp,
          pos_intersection = '',
          neg_intersection = '';

      for (i = 0, len = sets.length; i < len; i++) {
        set = $scope.get('Opal').$coerce_to(sets[i], $scope.get('String'), "to_str");
        neg = (set.charAt(0) === '^' && set.length > 1);
        set = explode_sequences_in_character_set(neg ? set.slice(1) : set);
        if (neg) {
          neg_intersection = intersection(neg_intersection, set);
        } else {
          pos_intersection = intersection(pos_intersection, set);
        }
      }

      if (pos_intersection.length > 0 && neg_intersection.length > 0) {
        tmp = '';
        for (i = 0, len = pos_intersection.length; i < len; i++) {
          chr = pos_intersection.charAt(i);
          if (neg_intersection.indexOf(chr) === -1) {
            tmp += chr;
          }
        }
        pos_intersection = tmp;
        neg_intersection = '';
      }

      if (pos_intersection.length > 0) {
        return '[' + $scope.get('Regexp').$escape(pos_intersection) + ']';
      }

      if (neg_intersection.length > 0) {
        return '[^' + $scope.get('Regexp').$escape(neg_intersection) + ']';
      }

      return null;
    }
  
  })(self, null);
  return Opal.cdecl($scope, 'Symbol', $scope.get('String'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/string/inheritance"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  function $rb_times(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs * rhs : lhs['$*'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $gvars = Opal.gvars;

  Opal.add_stubs(['$new', '$allocate', '$initialize', '$to_proc', '$__send__', '$class', '$clone', '$respond_to?', '$==', '$inspect', '$map', '$split', '$enum_for', '$each_line', '$to_a', '$%']);
  (function($base, $super) {
    function $String(){};
    var self = $String = $klass($base, $super, 'String', $String);

    var def = self.$$proto, $scope = self.$$scope;

    return (Opal.defs(self, '$inherited', function(klass) {
      var self = this, replace = nil;

      replace = $scope.get('Class').$new((($scope.get('String')).$$scope.get('Wrapper')));
      
      klass.$$proto         = replace.$$proto;
      klass.$$proto.$$class = klass;
      klass.$$alloc         = replace.$$alloc;
      klass.$$parent        = (($scope.get('String')).$$scope.get('Wrapper'));

      klass.$allocate = replace.$allocate;
      klass.$new      = replace.$new;
    
    }), nil) && 'inherited'
  })(self, null);
  return (function($base, $super) {
    function $Wrapper(){};
    var self = $Wrapper = $klass($base, $super, 'Wrapper', $Wrapper);

    var def = self.$$proto, $scope = self.$$scope, TMP_1, TMP_2, TMP_3, TMP_4, TMP_6, TMP_8;

    def.literal = nil;
    def.$$is_string = true;

    Opal.defs(self, '$allocate', TMP_1 = function(string) {
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil, obj = nil;

      if (string == null) {
        string = ""
      }
      TMP_1.$$p = null;
      obj = Opal.find_super_dispatcher(self, 'allocate', TMP_1, null, $Wrapper).apply(self, []);
      obj.literal = string;
      return obj;
    });

    Opal.defs(self, '$new', TMP_2 = function(args) {
      var $a, $b, self = this, $iter = TMP_2.$$p, block = $iter || nil, obj = nil;

      args = $slice.call(arguments, 0);
      TMP_2.$$p = null;
      obj = self.$allocate();
      ($a = ($b = obj).$initialize, $a.$$p = block.$to_proc(), $a).apply($b, [].concat(args));
      return obj;
    });

    Opal.defs(self, '$[]', function(objects) {
      var self = this;

      objects = $slice.call(arguments, 0);
      return self.$allocate(objects);
    });

    def.$initialize = function(string) {
      var self = this;

      if (string == null) {
        string = ""
      }
      return self.literal = string;
    };

    def.$method_missing = TMP_3 = function(args) {
      var $a, $b, self = this, $iter = TMP_3.$$p, block = $iter || nil, result = nil;

      args = $slice.call(arguments, 0);
      TMP_3.$$p = null;
      result = ($a = ($b = self.literal).$__send__, $a.$$p = block.$to_proc(), $a).apply($b, [].concat(args));
      if ((($a = result.$$is_string != null) !== nil && (!$a.$$is_boolean || $a == true))) {
        if ((($a = result == self.literal) !== nil && (!$a.$$is_boolean || $a == true))) {
          return self
          } else {
          return self.$class().$allocate(result)
        }
        } else {
        return result
      };
    };

    def.$initialize_copy = function(other) {
      var self = this;

      return self.literal = (other.literal).$clone();
    };

    def['$respond_to?'] = TMP_4 = function(name) {var $zuper = $slice.call(arguments, 0);
      var $a, self = this, $iter = TMP_4.$$p, $yield = $iter || nil;

      TMP_4.$$p = null;
      return ((($a = Opal.find_super_dispatcher(self, 'respond_to?', TMP_4, $iter).apply(self, $zuper)) !== false && $a !== nil) ? $a : self.literal['$respond_to?'](name));
    };

    def['$=='] = function(other) {
      var self = this;

      return self.literal['$=='](other);
    };

    Opal.defn(self, '$eql?', def['$==']);

    Opal.defn(self, '$===', def['$==']);

    def.$to_s = function() {
      var self = this;

      return self.literal;
    };

    Opal.defn(self, '$to_str', def.$to_s);

    def.$inspect = function() {
      var self = this;

      return self.literal.$inspect();
    };

    def['$+'] = function(other) {
      var self = this;

      return $rb_plus(self.literal, other);
    };

    def['$*'] = function(other) {
      var self = this;

      
      var result = $rb_times(self.literal, other);

      if (result.$$is_string) {
        return self.$class().$allocate(result)
      }
      else {
        return result;
      }
    ;
    };

    def.$split = function(pattern, limit) {
      var $a, $b, TMP_5, self = this;

      return ($a = ($b = self.literal.$split(pattern, limit)).$map, $a.$$p = (TMP_5 = function(str){var self = TMP_5.$$s || this;
if (str == null) str = nil;
      return self.$class().$allocate(str)}, TMP_5.$$s = self, TMP_5), $a).call($b);
    };

    def.$replace = function(string) {
      var self = this;

      return self.literal = string;
    };

    def.$each_line = TMP_6 = function(separator) {
      var $a, $b, TMP_7, self = this, $iter = TMP_6.$$p, $yield = $iter || nil;
      if ($gvars["/"] == null) $gvars["/"] = nil;

      if (separator == null) {
        separator = $gvars["/"]
      }
      TMP_6.$$p = null;
      if (($yield !== nil)) {
        } else {
        return self.$enum_for("each_line", separator)
      };
      return ($a = ($b = self.literal).$each_line, $a.$$p = (TMP_7 = function(str){var self = TMP_7.$$s || this, $a;
if (str == null) str = nil;
      return $a = Opal.yield1($yield, self.$class().$allocate(str)), $a === $breaker ? $a : $a}, TMP_7.$$s = self, TMP_7), $a).call($b, separator);
    };

    def.$lines = TMP_8 = function(separator) {
      var $a, $b, self = this, $iter = TMP_8.$$p, block = $iter || nil, e = nil;
      if ($gvars["/"] == null) $gvars["/"] = nil;

      if (separator == null) {
        separator = $gvars["/"]
      }
      TMP_8.$$p = null;
      e = ($a = ($b = self).$each_line, $a.$$p = block.$to_proc(), $a).call($b, separator);
      if (block !== false && block !== nil) {
        return self
        } else {
        return e.$to_a()
      };
    };

    return (def['$%'] = function(data) {
      var self = this;

      return self.literal['$%'](data);
    }, nil) && '%';
  })($scope.get('String'), null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/match_data"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $gvars = Opal.gvars;

  Opal.add_stubs(['$attr_reader', '$[]', '$raise', '$===', '$inspect', '$to_a', '$coerce_to!']);
  return (function($base, $super) {
    function $MatchData(){};
    var self = $MatchData = $klass($base, $super, 'MatchData', $MatchData);

    var def = self.$$proto, $scope = self.$$scope;

    def.matches = nil;
    self.$attr_reader("post_match", "pre_match", "regexp", "string");

    def.$initialize = function(regexp, match_groups) {
      var self = this;

      $gvars["~"] = self;
      self.regexp = regexp;
      self.begin = match_groups.index;
      self.string = match_groups.input;
      self.pre_match = match_groups.input.slice(0, match_groups.index);
      self.post_match = match_groups.input.slice(match_groups.index + match_groups[0].length);
      self.matches = [];
      
      for (var i = 0, length = match_groups.length; i < length; i++) {
        var group = match_groups[i];

        if (group == null) {
          self.matches.push(nil);
        }
        else {
          self.matches.push(group);
        }
      }
    
    };

    def['$[]'] = function(args) {
      var $a, self = this;

      args = $slice.call(arguments, 0);
      return ($a = self.matches)['$[]'].apply($a, [].concat(args));
    };

    def.$offset = function(n) {
      var self = this;

      
      if (n !== 0) {
        self.$raise($scope.get('ArgumentError'), "MatchData#offset only supports 0th element")
      }
      return [self.begin, self.begin + self.matches[n].length];
    ;
    };

    def['$=='] = function(other) {
      var $a, $b, $c, $d, self = this;

      if ((($a = $scope.get('MatchData')['$==='](other)) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        return false
      };
      return ($a = ($b = ($c = ($d = self.string == other.string, $d !== false && $d !== nil ?self.regexp.toString() == other.regexp.toString() : $d), $c !== false && $c !== nil ?self.pre_match == other.pre_match : $c), $b !== false && $b !== nil ?self.post_match == other.post_match : $b), $a !== false && $a !== nil ?self.begin == other.begin : $a);
    };

    Opal.defn(self, '$eql?', def['$==']);

    def.$begin = function(n) {
      var self = this;

      
      if (n !== 0) {
        self.$raise($scope.get('ArgumentError'), "MatchData#begin only supports 0th element")
      }
      return self.begin;
    ;
    };

    def.$end = function(n) {
      var self = this;

      
      if (n !== 0) {
        self.$raise($scope.get('ArgumentError'), "MatchData#end only supports 0th element")
      }
      return self.begin + self.matches[n].length;
    ;
    };

    def.$captures = function() {
      var self = this;

      return self.matches.slice(1);
    };

    def.$inspect = function() {
      var self = this;

      
      var str = "#<MatchData " + (self.matches[0]).$inspect();

      for (var i = 1, length = self.matches.length; i < length; i++) {
        str += " " + i + ":" + (self.matches[i]).$inspect();
      }

      return str + ">";
    ;
    };

    def.$length = function() {
      var self = this;

      return self.matches.length;
    };

    Opal.defn(self, '$size', def.$length);

    def.$to_a = function() {
      var self = this;

      return self.matches;
    };

    def.$to_s = function() {
      var self = this;

      return self.matches[0];
    };

    return (def.$values_at = function(args) {
      var self = this;

      args = $slice.call(arguments, 0);
      
      var i, a, index, values = [];

      for (i = 0; i < args.length; i++) {

        if (args[i].$$is_range) {
          a = (args[i]).$to_a();
          a.unshift(i, 1);
          Array.prototype.splice.apply(args, a);
        }

        index = $scope.get('Opal')['$coerce_to!'](args[i], $scope.get('Integer'), "to_int");

        if (index < 0) {
          index += self.matches.length;
          if (index < 0) {
            values.push(nil);
            continue;
          }
        }

        values.push(self.matches[index]);
      }

      return values;
    
    }, nil) && 'values_at';
  })(self, null)
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/numeric"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_minus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs - rhs : lhs['$-'](rhs);
  }
  function $rb_lt(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs < rhs : lhs['$<'](rhs);
  }
  function $rb_gt(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs > rhs : lhs['$>'](rhs);
  }
  function $rb_divide(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs / rhs : lhs['$/'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$include', '$coerce', '$===', '$raise', '$class', '$__send__', '$send_coerced', '$coerce_to!', '$-@', '$**', '$respond_to?', '$==', '$enum_for', '$gcd', '$lcm', '$floor', '$%']);
  self.$require("corelib/comparable");
  (function($base, $super) {
    function $Numeric(){};
    var self = $Numeric = $klass($base, $super, 'Numeric', $Numeric);

    var def = self.$$proto, $scope = self.$$scope, TMP_1, TMP_2, TMP_3, TMP_4, TMP_5, TMP_6;

    self.$include($scope.get('Comparable'));

    def.$$is_number = true;

    def.$__id__ = function() {
      var self = this;

      return (self * 2) + 1;
    };

    Opal.defn(self, '$object_id', def.$__id__);

    def.$coerce = function(other, type) {
      var self = this, $case = nil;

      if (type == null) {
        type = "operation"
      }
      try {
      
      if (other.$$is_number) {
        return [self, other];
      }
      else {
        return other.$coerce(self);
      }
    
      } catch ($err) {if (true) {
        return (function() {$case = type;if ("operation"['$===']($case)) {return self.$raise($scope.get('TypeError'), "" + (other.$class()) + " can't be coerced into Numeric")}else if ("comparison"['$===']($case)) {return self.$raise($scope.get('ArgumentError'), "comparison of " + (self.$class()) + " with " + (other.$class()) + " failed")}else { return nil }})()
        }else { throw $err; }
      };
    };

    def.$send_coerced = function(method, other) {
      var $a, self = this, type = nil, $case = nil, a = nil, b = nil;

      type = (function() {$case = method;if ("+"['$===']($case) || "-"['$===']($case) || "*"['$===']($case) || "/"['$===']($case) || "%"['$===']($case) || "&"['$===']($case) || "|"['$===']($case) || "^"['$===']($case) || "**"['$===']($case)) {return "operation"}else if (">"['$===']($case) || ">="['$===']($case) || "<"['$===']($case) || "<="['$===']($case) || "<=>"['$===']($case)) {return "comparison"}else { return nil }})();
      $a = Opal.to_ary(self.$coerce(other, type)), a = ($a[0] == null ? nil : $a[0]), b = ($a[1] == null ? nil : $a[1]);
      return a.$__send__(method, b);
    };

    def['$+'] = function(other) {
      var self = this;

      
      if (other.$$is_number) {
        return self + other;
      }
      else {
        return self.$send_coerced("+", other);
      }
    
    };

    def['$-'] = function(other) {
      var self = this;

      
      if (other.$$is_number) {
        return self - other;
      }
      else {
        return self.$send_coerced("-", other);
      }
    
    };

    def['$*'] = function(other) {
      var self = this;

      
      if (other.$$is_number) {
        return self * other;
      }
      else {
        return self.$send_coerced("*", other);
      }
    
    };

    def['$/'] = function(other) {
      var self = this;

      
      if (other.$$is_number) {
        return self / other;
      }
      else {
        return self.$send_coerced("/", other);
      }
    
    };

    def['$%'] = function(other) {
      var self = this;

      
      if (other.$$is_number) {
        if (other < 0 || self < 0) {
          return (self % other + other) % other;
        }
        else {
          return self % other;
        }
      }
      else {
        return self.$send_coerced("%", other);
      }
    
    };

    def['$&'] = function(other) {
      var self = this;

      
      if (other.$$is_number) {
        return self & other;
      }
      else {
        return self.$send_coerced("&", other);
      }
    
    };

    def['$|'] = function(other) {
      var self = this;

      
      if (other.$$is_number) {
        return self | other;
      }
      else {
        return self.$send_coerced("|", other);
      }
    
    };

    def['$^'] = function(other) {
      var self = this;

      
      if (other.$$is_number) {
        return self ^ other;
      }
      else {
        return self.$send_coerced("^", other);
      }
    
    };

    def['$<'] = function(other) {
      var self = this;

      
      if (other.$$is_number) {
        return self < other;
      }
      else {
        return self.$send_coerced("<", other);
      }
    
    };

    def['$<='] = function(other) {
      var self = this;

      
      if (other.$$is_number) {
        return self <= other;
      }
      else {
        return self.$send_coerced("<=", other);
      }
    
    };

    def['$>'] = function(other) {
      var self = this;

      
      if (other.$$is_number) {
        return self > other;
      }
      else {
        return self.$send_coerced(">", other);
      }
    
    };

    def['$>='] = function(other) {
      var self = this;

      
      if (other.$$is_number) {
        return self >= other;
      }
      else {
        return self.$send_coerced(">=", other);
      }
    
    };

    def['$<=>'] = function(other) {
      var self = this;

      try {
      
      if (other.$$is_number) {
        return self > other ? 1 : (self < other ? -1 : 0);
      }
      else {
        return self.$send_coerced("<=>", other);
      }
    
      } catch ($err) {if (Opal.rescue($err, [$scope.get('ArgumentError')])) {
        return nil
        }else { throw $err; }
      };
    };

    def['$<<'] = function(count) {
      var self = this;

      count = $scope.get('Opal')['$coerce_to!'](count, $scope.get('Integer'), "to_int");
      return count > 0 ? self << count : self >> -count;
    };

    def['$>>'] = function(count) {
      var self = this;

      count = $scope.get('Opal')['$coerce_to!'](count, $scope.get('Integer'), "to_int");
      return count > 0 ? self >> count : self << -count;
    };

    def['$[]'] = function(bit) {
      var self = this, min = nil, max = nil;

      bit = $scope.get('Opal')['$coerce_to!'](bit, $scope.get('Integer'), "to_int");
      min = ((2)['$**'](30))['$-@']();
      max = $rb_minus(((2)['$**'](30)), 1);
      return (bit < min || bit > max) ? 0 : (self >> bit) % 2;
    };

    def['$+@'] = function() {
      var self = this;

      return +self;
    };

    def['$-@'] = function() {
      var self = this;

      return -self;
    };

    def['$~'] = function() {
      var self = this;

      return ~self;
    };

    def['$**'] = function(other) {
      var self = this;

      
      if (other.$$is_number) {
        return Math.pow(self, other);
      }
      else {
        return self.$send_coerced("**", other);
      }
    
    };

    def['$=='] = function(other) {
      var self = this;

      
      if (other.$$is_number) {
        return self == Number(other);
      }
      else if (other['$respond_to?']("==")) {
        return other['$=='](self);
      }
      else {
        return false;
      }
    ;
    };

    def.$abs = function() {
      var self = this;

      return Math.abs(self);
    };

    def.$ceil = function() {
      var self = this;

      return Math.ceil(self);
    };

    def.$chr = function(encoding) {
      var self = this;

      return String.fromCharCode(self);
    };

    def.$conj = function() {
      var self = this;

      return self;
    };

    Opal.defn(self, '$conjugate', def.$conj);

    def.$downto = TMP_1 = function(finish) {
      var self = this, $iter = TMP_1.$$p, block = $iter || nil;

      TMP_1.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        return self.$enum_for("downto", finish)
      };
      
      if (!finish.$$is_number) {
        self.$raise($scope.get('ArgumentError'), "comparison of " + (self.$class()) + " with " + (finish.$class()) + " failed")
      }
      for (var i = self; i >= finish; i--) {
        if (block(i) === $breaker) {
          return $breaker.$v;
        }
      }
    ;
      return self;
    };

    Opal.defn(self, '$eql?', def['$==']);

    def['$equal?'] = function(other) {
      var $a, self = this;

      return ((($a = self['$=='](other)) !== false && $a !== nil) ? $a : isNaN(self) && isNaN(other));
    };

    def['$even?'] = function() {
      var self = this;

      return self % 2 === 0;
    };

    def.$floor = function() {
      var self = this;

      return Math.floor(self);
    };

    def.$gcd = function(other) {
      var $a, self = this;

      if ((($a = $scope.get('Integer')['$==='](other)) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise($scope.get('TypeError'), "not an integer")
      };
      
      var min = Math.abs(self),
          max = Math.abs(other);

      while (min > 0) {
        var tmp = min;

        min = max % min;
        max = tmp;
      }

      return max;
    
    };

    def.$gcdlcm = function(other) {
      var self = this;

      return [self.$gcd(), self.$lcm()];
    };

    def.$hash = function() {
      var self = this;

      return 'Numeric:'+self.toString();
    };

    def['$integer?'] = function() {
      var self = this;

      return self % 1 === 0;
    };

    def['$is_a?'] = TMP_2 = function(klass) {var $zuper = $slice.call(arguments, 0);
      var $a, $b, self = this, $iter = TMP_2.$$p, $yield = $iter || nil;

      TMP_2.$$p = null;
      if ((($a = (($b = klass['$==']($scope.get('Fixnum'))) ? $scope.get('Integer')['$==='](self) : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return true};
      if ((($a = (($b = klass['$==']($scope.get('Integer'))) ? $scope.get('Integer')['$==='](self) : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return true};
      if ((($a = (($b = klass['$==']($scope.get('Float'))) ? $scope.get('Float')['$==='](self) : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return true};
      return Opal.find_super_dispatcher(self, 'is_a?', TMP_2, $iter).apply(self, $zuper);
    };

    Opal.defn(self, '$kind_of?', def['$is_a?']);

    def['$instance_of?'] = TMP_3 = function(klass) {var $zuper = $slice.call(arguments, 0);
      var $a, $b, self = this, $iter = TMP_3.$$p, $yield = $iter || nil;

      TMP_3.$$p = null;
      if ((($a = (($b = klass['$==']($scope.get('Fixnum'))) ? $scope.get('Integer')['$==='](self) : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return true};
      if ((($a = (($b = klass['$==']($scope.get('Integer'))) ? $scope.get('Integer')['$==='](self) : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return true};
      if ((($a = (($b = klass['$==']($scope.get('Float'))) ? $scope.get('Float')['$==='](self) : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return true};
      return Opal.find_super_dispatcher(self, 'instance_of?', TMP_3, $iter).apply(self, $zuper);
    };

    def.$lcm = function(other) {
      var $a, self = this;

      if ((($a = $scope.get('Integer')['$==='](other)) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise($scope.get('TypeError'), "not an integer")
      };
      
      if (self == 0 || other == 0) {
        return 0;
      }
      else {
        return Math.abs(self * other / self.$gcd(other));
      }
    
    };

    Opal.defn(self, '$magnitude', def.$abs);

    Opal.defn(self, '$modulo', def['$%']);

    def.$next = function() {
      var self = this;

      return self + 1;
    };

    def['$nonzero?'] = function() {
      var self = this;

      return self == 0 ? nil : self;
    };

    def['$odd?'] = function() {
      var self = this;

      return self % 2 !== 0;
    };

    def.$ord = function() {
      var self = this;

      return self;
    };

    def.$pred = function() {
      var self = this;

      return self - 1;
    };

    def.$round = function(ndigits) {
      var self = this;

      if (ndigits == null) {
        ndigits = 0
      }
      
      var scale = Math.pow(10, ndigits);
      return Math.round(self * scale) / scale;
    
    };

    def.$step = TMP_4 = function(limit, step) {
      var $a, self = this, $iter = TMP_4.$$p, block = $iter || nil;

      if (step == null) {
        step = 1
      }
      TMP_4.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        return self.$enum_for("step", limit, step)
      };
      if ((($a = step == 0) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$raise($scope.get('ArgumentError'), "step cannot be 0")};
      
      var value = self;

      if (step > 0) {
        while (value <= limit) {
          block(value);
          value += step;
        }
      }
      else {
        while (value >= limit) {
          block(value);
          value += step;
        }
      }
    
      return self;
    };

    Opal.defn(self, '$succ', def.$next);

    def.$times = TMP_5 = function() {
      var self = this, $iter = TMP_5.$$p, block = $iter || nil;

      TMP_5.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        return self.$enum_for("times")
      };
      
      for (var i = 0; i < self; i++) {
        if (block(i) === $breaker) {
          return $breaker.$v;
        }
      }
    
      return self;
    };

    def.$to_f = function() {
      var self = this;

      return self;
    };

    def.$to_i = function() {
      var self = this;

      return parseInt(self);
    };

    Opal.defn(self, '$to_int', def.$to_i);

    def.$to_s = function(base) {
      var $a, $b, self = this;

      if (base == null) {
        base = 10
      }
      if ((($a = ((($b = $rb_lt(base, 2)) !== false && $b !== nil) ? $b : $rb_gt(base, 36))) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$raise($scope.get('ArgumentError'), "base must be between 2 and 36")};
      return self.toString(base);
    };

    Opal.defn(self, '$inspect', def.$to_s);

    def.$divmod = function(rhs) {
      var self = this, q = nil, r = nil;

      q = ($rb_divide(self, rhs)).$floor();
      r = self['$%'](rhs);
      return [q, r];
    };

    def.$upto = TMP_6 = function(finish) {
      var self = this, $iter = TMP_6.$$p, block = $iter || nil;

      TMP_6.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        return self.$enum_for("upto", finish)
      };
      
      if (!finish.$$is_number) {
        self.$raise($scope.get('ArgumentError'), "comparison of " + (self.$class()) + " with " + (finish.$class()) + " failed")
      }
      for (var i = self; i <= finish; i++) {
        if (block(i) === $breaker) {
          return $breaker.$v;
        }
      }
    ;
      return self;
    };

    def['$zero?'] = function() {
      var self = this;

      return self == 0;
    };

    def.$size = function() {
      var self = this;

      return 4;
    };

    def['$nan?'] = function() {
      var self = this;

      return isNaN(self);
    };

    def['$finite?'] = function() {
      var self = this;

      return self != Infinity && self != -Infinity;
    };

    def['$infinite?'] = function() {
      var self = this;

      
      if (self == Infinity) {
        return +1;
      }
      else if (self == -Infinity) {
        return -1;
      }
      else {
        return nil;
      }
    
    };

    def['$positive?'] = function() {
      var self = this;

      return 1 / self > 0;
    };

    return (def['$negative?'] = function() {
      var self = this;

      return 1 / self < 0;
    }, nil) && 'negative?';
  })(self, null);
  Opal.cdecl($scope, 'Fixnum', $scope.get('Numeric'));
  (function($base, $super) {
    function $Integer(){};
    var self = $Integer = $klass($base, $super, 'Integer', $Integer);

    var def = self.$$proto, $scope = self.$$scope;

    return (Opal.defs(self, '$===', function(other) {
      var self = this;

      
      if (!other.$$is_number) {
        return false;
      }

      return (other % 1) === 0;
    
    }), nil) && '==='
  })(self, $scope.get('Numeric'));
  return (function($base, $super) {
    function $Float(){};
    var self = $Float = $klass($base, $super, 'Float', $Float);

    var def = self.$$proto, $scope = self.$$scope, $a;

    Opal.defs(self, '$===', function(other) {
      var self = this;

      return !!other.$$is_number;
    });

    Opal.cdecl($scope, 'INFINITY', Infinity);

    Opal.cdecl($scope, 'NAN', NaN);

    if ((($a = (typeof(Number.EPSILON) !== "undefined")) !== nil && (!$a.$$is_boolean || $a == true))) {
      return Opal.cdecl($scope, 'EPSILON', Number.EPSILON)
      } else {
      return Opal.cdecl($scope, 'EPSILON', 2.2204460492503130808472633361816E-16)
    };
  })(self, $scope.get('Numeric'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/complex"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  return (function($base, $super) {
    function $Complex(){};
    var self = $Complex = $klass($base, $super, 'Complex', $Complex);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('Numeric'))
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/rational"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  return (function($base, $super) {
    function $Rational(){};
    var self = $Rational = $klass($base, $super, 'Rational', $Rational);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('Numeric'))
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/proc"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$raise']);
  return (function($base, $super) {
    function $Proc(){};
    var self = $Proc = $klass($base, $super, 'Proc', $Proc);

    var def = self.$$proto, $scope = self.$$scope, TMP_1, TMP_2;

    def.$$is_proc = true;

    def.$$is_lambda = false;

    Opal.defs(self, '$new', TMP_1 = function() {
      var self = this, $iter = TMP_1.$$p, block = $iter || nil;

      TMP_1.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        self.$raise($scope.get('ArgumentError'), "tried to create a Proc object without a block")
      };
      return block;
    });

    def.$call = TMP_2 = function(args) {
      var self = this, $iter = TMP_2.$$p, block = $iter || nil;

      args = $slice.call(arguments, 0);
      TMP_2.$$p = null;
      
      if (block !== nil) {
        self.$$p = block;
      }

      var result;

      if (self.$$is_lambda) {
        result = self.apply(null, args);
      }
      else {
        result = Opal.yieldX(self, args);
      }

      if (result === $breaker) {
        return $breaker.$v;
      }

      return result;
    
    };

    Opal.defn(self, '$[]', def.$call);

    def.$to_proc = function() {
      var self = this;

      return self;
    };

    def['$lambda?'] = function() {
      var self = this;

      return !!self.$$is_lambda;
    };

    return (def.$arity = function() {
      var self = this;

      return self.length;
    }, nil) && 'arity';
  })(self, null)
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/method"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$attr_reader', '$class', '$arity', '$new', '$name']);
  (function($base, $super) {
    function $Method(){};
    var self = $Method = $klass($base, $super, 'Method', $Method);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    def.method = def.receiver = def.owner = def.name = def.obj = nil;
    self.$attr_reader("owner", "receiver", "name");

    def.$initialize = function(receiver, method, name) {
      var self = this;

      self.receiver = receiver;
      self.owner = receiver.$class();
      self.name = name;
      return self.method = method;
    };

    def.$arity = function() {
      var self = this;

      return self.method.$arity();
    };

    def.$call = TMP_1 = function(args) {
      var self = this, $iter = TMP_1.$$p, block = $iter || nil;

      args = $slice.call(arguments, 0);
      TMP_1.$$p = null;
      
      self.method.$$p = block;

      return self.method.apply(self.receiver, args);
    ;
    };

    Opal.defn(self, '$[]', def.$call);

    def.$unbind = function() {
      var self = this;

      return $scope.get('UnboundMethod').$new(self.owner, self.method, self.name);
    };

    def.$to_proc = function() {
      var self = this;

      return self.method;
    };

    return (def.$inspect = function() {
      var self = this;

      return "#<Method: " + (self.obj.$class()) + "#" + (self.name) + "}>";
    }, nil) && 'inspect';
  })(self, null);
  return (function($base, $super) {
    function $UnboundMethod(){};
    var self = $UnboundMethod = $klass($base, $super, 'UnboundMethod', $UnboundMethod);

    var def = self.$$proto, $scope = self.$$scope;

    def.method = def.name = def.owner = nil;
    self.$attr_reader("owner", "name");

    def.$initialize = function(owner, method, name) {
      var self = this;

      self.owner = owner;
      self.method = method;
      return self.name = name;
    };

    def.$arity = function() {
      var self = this;

      return self.method.$arity();
    };

    def.$bind = function(object) {
      var self = this;

      return $scope.get('Method').$new(object, self.method, self.name);
    };

    return (def.$inspect = function() {
      var self = this;

      return "#<UnboundMethod: " + (self.owner.$name()) + "#" + (self.name) + ">";
    }, nil) && 'inspect';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/range"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_le(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs <= rhs : lhs['$<='](rhs);
  }
  function $rb_lt(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs < rhs : lhs['$<'](rhs);
  }
  function $rb_minus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs - rhs : lhs['$-'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$include', '$attr_reader', '$<=>', '$raise', '$include?', '$enum_for', '$succ', '$!', '$==', '$===', '$exclude_end?', '$eql?', '$begin', '$end', '$abs', '$to_i', '$inspect']);
  self.$require("corelib/enumerable");
  return (function($base, $super) {
    function $Range(){};
    var self = $Range = $klass($base, $super, 'Range', $Range);

    var def = self.$$proto, $scope = self.$$scope, TMP_1, TMP_2, TMP_3;

    def.begin = def.exclude = def.end = nil;
    self.$include($scope.get('Enumerable'));

    def.$$is_range = true;

    self.$attr_reader("begin", "end");

    def.$initialize = function(first, last, exclude) {
      var $a, self = this;

      if (exclude == null) {
        exclude = false
      }
      if ((($a = first['$<=>'](last)) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise($scope.get('ArgumentError'))
      };
      self.begin = first;
      self.end = last;
      return self.exclude = exclude;
    };

    def['$=='] = function(other) {
      var self = this;

      
      if (!other.$$is_range) {
        return false;
      }

      return self.exclude === other.exclude &&
             self.begin   ==  other.begin &&
             self.end     ==  other.end;
    
    };

    def['$==='] = function(value) {
      var self = this;

      return self['$include?'](value);
    };

    def['$cover?'] = function(value) {
      var $a, $b, self = this;

      return (($a = $rb_le(self.begin, value)) ? ((function() {if ((($b = self.exclude) !== nil && (!$b.$$is_boolean || $b == true))) {
        return $rb_lt(value, self.end)
        } else {
        return $rb_le(value, self.end)
      }; return nil; })()) : $a);
    };

    def.$each = TMP_1 = function() {
      var $a, $b, self = this, $iter = TMP_1.$$p, block = $iter || nil, current = nil, last = nil;

      TMP_1.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("each")
      };
      current = self.begin;
      last = self.end;
      while ($rb_lt(current, last)) {
      if (Opal.yield1(block, current) === $breaker) return $breaker.$v;
      current = current.$succ();};
      if ((($a = ($b = self.exclude['$!'](), $b !== false && $b !== nil ?current['$=='](last) : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
        if (Opal.yield1(block, current) === $breaker) return $breaker.$v};
      return self;
    };

    def['$eql?'] = function(other) {
      var $a, $b, self = this;

      if ((($a = $scope.get('Range')['$==='](other)) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        return false
      };
      return ($a = ($b = self.exclude['$==='](other['$exclude_end?']()), $b !== false && $b !== nil ?self.begin['$eql?'](other.$begin()) : $b), $a !== false && $a !== nil ?self.end['$eql?'](other.$end()) : $a);
    };

    def['$exclude_end?'] = function() {
      var self = this;

      return self.exclude;
    };

    Opal.defn(self, '$first', def.$begin);

    Opal.defn(self, '$include?', def['$cover?']);

    Opal.defn(self, '$last', def.$end);

    def.$max = TMP_2 = function() {var $zuper = $slice.call(arguments, 0);
      var self = this, $iter = TMP_2.$$p, $yield = $iter || nil;

      TMP_2.$$p = null;
      if (($yield !== nil)) {
        return Opal.find_super_dispatcher(self, 'max', TMP_2, $iter).apply(self, $zuper)
        } else {
        return self.exclude ? self.end - 1 : self.end;
      };
    };

    Opal.defn(self, '$member?', def['$cover?']);

    def.$min = TMP_3 = function() {var $zuper = $slice.call(arguments, 0);
      var self = this, $iter = TMP_3.$$p, $yield = $iter || nil;

      TMP_3.$$p = null;
      if (($yield !== nil)) {
        return Opal.find_super_dispatcher(self, 'min', TMP_3, $iter).apply(self, $zuper)
        } else {
        return self.begin
      };
    };

    Opal.defn(self, '$member?', def['$include?']);

    def.$size = function() {
      var $a, $b, self = this, _begin = nil, _end = nil, infinity = nil;

      _begin = self.begin;
      _end = self.end;
      if ((($a = self.exclude) !== nil && (!$a.$$is_boolean || $a == true))) {
        _end = $rb_minus(_end, 1)};
      if ((($a = ($b = $scope.get('Numeric')['$==='](_begin), $b !== false && $b !== nil ?$scope.get('Numeric')['$==='](_end) : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        return nil
      };
      if ($rb_lt(_end, _begin)) {
        return 0};
      infinity = (($scope.get('Float')).$$scope.get('INFINITY'));
      if ((($a = ((($b = infinity['$=='](_begin.$abs())) !== false && $b !== nil) ? $b : _end.$abs()['$=='](infinity))) !== nil && (!$a.$$is_boolean || $a == true))) {
        return infinity};
      return ((Math.abs(_end - _begin) + 1)).$to_i();
    };

    def.$step = function(n) {
      var self = this;

      if (n == null) {
        n = 1
      }
      return self.$raise($scope.get('NotImplementedError'));
    };

    def.$to_s = function() {
      var self = this;

      return self.begin.$inspect() + (self.exclude ? '...' : '..') + self.end.$inspect();
    };

    return Opal.defn(self, '$inspect', def.$to_s);
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/time"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_le(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs <= rhs : lhs['$<='](rhs);
  }
  function $rb_minus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs - rhs : lhs['$-'](rhs);
  }
  function $rb_divide(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs / rhs : lhs['$/'](rhs);
  }
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $range = Opal.range;

  Opal.add_stubs(['$require', '$include', '$kind_of?', '$to_i', '$coerce_to', '$between?', '$raise', '$new', '$compact', '$nil?', '$===', '$<=>', '$to_f', '$strftime', '$is_a?', '$zero?', '$wday', '$utc?', '$warn', '$year', '$mon', '$day', '$yday', '$hour', '$min', '$sec', '$rjust', '$ljust', '$zone', '$to_s', '$[]', '$cweek_cyear', '$month', '$isdst', '$private', '$!', '$==', '$ceil']);
  self.$require("corelib/comparable");
  return (function($base, $super) {
    function $Time(){};
    var self = $Time = $klass($base, $super, 'Time', $Time);

    var def = self.$$proto, $scope = self.$$scope;

    def.tz_offset = nil;
    self.$include($scope.get('Comparable'));

    
    var days_of_week = ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"],
        short_days   = ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"],
        short_months = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"],
        long_months  = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"];
  ;

    Opal.defs(self, '$at', function(seconds, frac) {
      var self = this;

      if (frac == null) {
        frac = 0
      }
      return new Date(seconds * 1000 + frac);
    });

    Opal.defs(self, '$new', function(year, month, day, hour, minute, second, utc_offset) {
      var self = this;

      
      switch (arguments.length) {
        case 1:
          return new Date(year, 0);

        case 2:
          return new Date(year, month - 1);

        case 3:
          return new Date(year, month - 1, day);

        case 4:
          return new Date(year, month - 1, day, hour);

        case 5:
          return new Date(year, month - 1, day, hour, minute);

        case 6:
          return new Date(year, month - 1, day, hour, minute, second);

        case 7:
          return new Date(year, month - 1, day, hour, minute, second);

        default:
          return new Date();
      }
    
    });

    Opal.defs(self, '$local', function(year, month, day, hour, minute, second, millisecond) {
      var $a, self = this;

      if (month == null) {
        month = nil
      }
      if (day == null) {
        day = nil
      }
      if (hour == null) {
        hour = nil
      }
      if (minute == null) {
        minute = nil
      }
      if (second == null) {
        second = nil
      }
      if (millisecond == null) {
        millisecond = nil
      }
      if ((($a = arguments.length === 10) !== nil && (!$a.$$is_boolean || $a == true))) {
        
        var args = $slice.call(arguments).reverse();

        second = args[9];
        minute = args[8];
        hour   = args[7];
        day    = args[6];
        month  = args[5];
        year   = args[4];
      };
      year = (function() {if ((($a = year['$kind_of?']($scope.get('String'))) !== nil && (!$a.$$is_boolean || $a == true))) {
        return year.$to_i()
        } else {
        return $scope.get('Opal').$coerce_to(year, $scope.get('Integer'), "to_int")
      }; return nil; })();
      month = (function() {if ((($a = month['$kind_of?']($scope.get('String'))) !== nil && (!$a.$$is_boolean || $a == true))) {
        return month.$to_i()
        } else {
        return $scope.get('Opal').$coerce_to(((($a = month) !== false && $a !== nil) ? $a : 1), $scope.get('Integer'), "to_int")
      }; return nil; })();
      if ((($a = month['$between?'](1, 12)) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise($scope.get('ArgumentError'), "month out of range: " + (month))
      };
      day = (function() {if ((($a = day['$kind_of?']($scope.get('String'))) !== nil && (!$a.$$is_boolean || $a == true))) {
        return day.$to_i()
        } else {
        return $scope.get('Opal').$coerce_to(((($a = day) !== false && $a !== nil) ? $a : 1), $scope.get('Integer'), "to_int")
      }; return nil; })();
      if ((($a = day['$between?'](1, 31)) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise($scope.get('ArgumentError'), "day out of range: " + (day))
      };
      hour = (function() {if ((($a = hour['$kind_of?']($scope.get('String'))) !== nil && (!$a.$$is_boolean || $a == true))) {
        return hour.$to_i()
        } else {
        return $scope.get('Opal').$coerce_to(((($a = hour) !== false && $a !== nil) ? $a : 0), $scope.get('Integer'), "to_int")
      }; return nil; })();
      if ((($a = hour['$between?'](0, 24)) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise($scope.get('ArgumentError'), "hour out of range: " + (hour))
      };
      minute = (function() {if ((($a = minute['$kind_of?']($scope.get('String'))) !== nil && (!$a.$$is_boolean || $a == true))) {
        return minute.$to_i()
        } else {
        return $scope.get('Opal').$coerce_to(((($a = minute) !== false && $a !== nil) ? $a : 0), $scope.get('Integer'), "to_int")
      }; return nil; })();
      if ((($a = minute['$between?'](0, 59)) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise($scope.get('ArgumentError'), "minute out of range: " + (minute))
      };
      second = (function() {if ((($a = second['$kind_of?']($scope.get('String'))) !== nil && (!$a.$$is_boolean || $a == true))) {
        return second.$to_i()
        } else {
        return $scope.get('Opal').$coerce_to(((($a = second) !== false && $a !== nil) ? $a : 0), $scope.get('Integer'), "to_int")
      }; return nil; })();
      if ((($a = second['$between?'](0, 59)) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise($scope.get('ArgumentError'), "second out of range: " + (second))
      };
      return ($a = self).$new.apply($a, [].concat([year, month, day, hour, minute, second].$compact()));
    });

    Opal.defs(self, '$gm', function(year, month, day, hour, minute, second, utc_offset) {
      var $a, self = this;

      if ((($a = year['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$raise($scope.get('TypeError'), "missing year (got nil)")};
      
      if (month > 12 || day > 31 || hour > 24 || minute > 59 || second > 59) {
        self.$raise($scope.get('ArgumentError'));
      }

      var date = new Date(Date.UTC(year, (month || 1) - 1, (day || 1), (hour || 0), (minute || 0), (second || 0)));
      date.tz_offset = 0
      return date;
    ;
    });

    (function(self) {
      var $scope = self.$$scope, def = self.$$proto;

      self.$$proto.$mktime = self.$$proto.$local;
      return self.$$proto.$utc = self.$$proto.$gm;
    })(self.$singleton_class());

    Opal.defs(self, '$now', function() {
      var self = this;

      return new Date();
    });

    def['$+'] = function(other) {
      var $a, self = this;

      if ((($a = $scope.get('Time')['$==='](other)) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$raise($scope.get('TypeError'), "time + time?")};
      other = $scope.get('Opal').$coerce_to(other, $scope.get('Integer'), "to_int");
      
      var result           = new Date(self.getTime() + (other * 1000));
          result.tz_offset = self.tz_offset;

      return result;
    
    };

    def['$-'] = function(other) {
      var $a, self = this;

      if ((($a = $scope.get('Time')['$==='](other)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return (self.getTime() - other.getTime()) / 1000};
      other = $scope.get('Opal').$coerce_to(other, $scope.get('Integer'), "to_int");
      
      var result           = new Date(self.getTime() - (other * 1000));
          result.tz_offset = self.tz_offset;

      return result;
    
    };

    def['$<=>'] = function(other) {
      var self = this;

      return self.$to_f()['$<=>'](other.$to_f());
    };

    def['$=='] = function(other) {
      var self = this;

      return self.$to_f() === other.$to_f();
    };

    def.$asctime = function() {
      var self = this;

      return self.$strftime("%a %b %e %H:%M:%S %Y");
    };

    Opal.defn(self, '$ctime', def.$asctime);

    def.$day = function() {
      var self = this;

      
      if (self.tz_offset === 0) {
        return self.getUTCDate();
      }
      else {
        return self.getDate();
      }
    ;
    };

    def.$yday = function() {
      var self = this;

      
      // http://javascript.about.com/library/bldayyear.htm
      var onejan = new Date(self.getFullYear(), 0, 1);
      return Math.ceil((self - onejan) / 86400000);
    
    };

    def.$isdst = function() {
      var self = this;

      return self.$raise($scope.get('NotImplementedError'));
    };

    def['$eql?'] = function(other) {
      var $a, self = this;

      return ($a = other['$is_a?']($scope.get('Time')), $a !== false && $a !== nil ?(self['$<=>'](other))['$zero?']() : $a);
    };

    def['$friday?'] = function() {
      var self = this;

      return self.$wday() == 5;
    };

    def.$hour = function() {
      var self = this;

      
      if (self.tz_offset === 0) {
        return self.getUTCHours();
      }
      else {
        return self.getHours();
      }
    ;
    };

    def.$inspect = function() {
      var $a, self = this;

      if ((($a = self['$utc?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.$strftime("%Y-%m-%d %H:%M:%S UTC")
        } else {
        return self.$strftime("%Y-%m-%d %H:%M:%S %z")
      };
    };

    Opal.defn(self, '$mday', def.$day);

    def.$min = function() {
      var self = this;

      
      if (self.tz_offset === 0) {
        return self.getUTCMinutes();
      }
      else {
        return self.getMinutes();
      }
    ;
    };

    def.$mon = function() {
      var self = this;

      
      if (self.tz_offset === 0) {
        return self.getUTCMonth() + 1;
      }
      else {
        return self.getMonth() + 1;
      }
    ;
    };

    def['$monday?'] = function() {
      var self = this;

      return self.$wday() == 1;
    };

    Opal.defn(self, '$month', def.$mon);

    def['$saturday?'] = function() {
      var self = this;

      return self.$wday() == 6;
    };

    def.$sec = function() {
      var self = this;

      
      if (self.tz_offset === 0) {
        return self.getUTCSeconds();
      }
      else {
        return self.getSeconds();
      }
    ;
    };

    def.$usec = function() {
      var self = this;

      self.$warn("Microseconds are not supported");
      return 0;
    };

    def.$zone = function() {
      var self = this;

      
      var string = self.toString(),
          result;

      if (string.indexOf('(') == -1) {
        result = string.match(/[A-Z]{3,4}/)[0];
      }
      else {
        result = string.match(/\([^)]+\)/)[0].match(/[A-Z]/g).join('');
      }

      if (result == "GMT" && /(GMT\W*\d{4})/.test(string)) {
        return RegExp.$1;
      }
      else {
        return result;
      }
    
    };

    def.$getgm = function() {
      var self = this;

      
      var result           = new Date(self.getTime());
          result.tz_offset = 0;

      return result;
    
    };

    def['$gmt?'] = function() {
      var self = this;

      return self.tz_offset === 0;
    };

    def.$gmt_offset = function() {
      var self = this;

      return -self.getTimezoneOffset() * 60;
    };

    def.$strftime = function(format) {
      var self = this;

      
      return format.replace(/%([\-_#^0]*:{0,2})(\d+)?([EO]*)(.)/g, function(full, flags, width, _, conv) {
        var result = "",
            width  = parseInt(width),
            zero   = flags.indexOf('0') !== -1,
            pad    = flags.indexOf('-') === -1,
            blank  = flags.indexOf('_') !== -1,
            upcase = flags.indexOf('^') !== -1,
            invert = flags.indexOf('#') !== -1,
            colons = (flags.match(':') || []).length;

        if (zero && blank) {
          if (flags.indexOf('0') < flags.indexOf('_')) {
            zero = false;
          }
          else {
            blank = false;
          }
        }

        switch (conv) {
          case 'Y':
            result += self.$year();
            break;

          case 'C':
            zero    = !blank;
            result += Math.round(self.$year() / 100);
            break;

          case 'y':
            zero    = !blank;
            result += (self.$year() % 100);
            break;

          case 'm':
            zero    = !blank;
            result += self.$mon();
            break;

          case 'B':
            result += long_months[self.$mon() - 1];
            break;

          case 'b':
          case 'h':
            blank   = !zero;
            result += short_months[self.$mon() - 1];
            break;

          case 'd':
            zero    = !blank
            result += self.$day();
            break;

          case 'e':
            blank   = !zero
            result += self.$day();
            break;

          case 'j':
            result += self.$yday();
            break;

          case 'H':
            zero    = !blank;
            result += self.$hour();
            break;

          case 'k':
            blank   = !zero;
            result += self.$hour();
            break;

          case 'I':
            zero    = !blank;
            result += (self.$hour() % 12 || 12);
            break;

          case 'l':
            blank   = !zero;
            result += (self.$hour() % 12 || 12);
            break;

          case 'P':
            result += (self.$hour() >= 12 ? "pm" : "am");
            break;

          case 'p':
            result += (self.$hour() >= 12 ? "PM" : "AM");
            break;

          case 'M':
            zero    = !blank;
            result += self.$min();
            break;

          case 'S':
            zero    = !blank;
            result += self.$sec()
            break;

          case 'L':
            zero    = !blank;
            width   = isNaN(width) ? 3 : width;
            result += self.getMilliseconds();
            break;

          case 'N':
            width   = isNaN(width) ? 9 : width;
            result += (self.getMilliseconds().toString()).$rjust(3, "0");
            result  = (result).$ljust(width, "0");
            break;

          case 'z':
            var offset  = self.getTimezoneOffset(),
                hours   = Math.floor(Math.abs(offset) / 60),
                minutes = Math.abs(offset) % 60;

            result += offset < 0 ? "+" : "-";
            result += hours < 10 ? "0" : "";
            result += hours;

            if (colons > 0) {
              result += ":";
            }

            result += minutes < 10 ? "0" : "";
            result += minutes;

            if (colons > 1) {
              result += ":00";
            }

            break;

          case 'Z':
            result += self.$zone();
            break;

          case 'A':
            result += days_of_week[self.$wday()];
            break;

          case 'a':
            result += short_days[self.$wday()];
            break;

          case 'u':
            result += (self.$wday() + 1);
            break;

          case 'w':
            result += self.$wday();
            break;

          case 'V':
            result += self.$cweek_cyear()['$[]'](0).$to_s().$rjust(2, "0");
            break;

          case 'G':
            result += self.$cweek_cyear()['$[]'](1);
            break;

          case 'g':
            result += self.$cweek_cyear()['$[]'](1)['$[]']($range(-2, -1, false));
            break;

          case 's':
            result += self.$to_i();
            break;

          case 'n':
            result += "\n";
            break;

          case 't':
            result += "\t";
            break;

          case '%':
            result += "%";
            break;

          case 'c':
            result += self.$strftime("%a %b %e %T %Y");
            break;

          case 'D':
          case 'x':
            result += self.$strftime("%m/%d/%y");
            break;

          case 'F':
            result += self.$strftime("%Y-%m-%d");
            break;

          case 'v':
            result += self.$strftime("%e-%^b-%4Y");
            break;

          case 'r':
            result += self.$strftime("%I:%M:%S %p");
            break;

          case 'R':
            result += self.$strftime("%H:%M");
            break;

          case 'T':
          case 'X':
            result += self.$strftime("%H:%M:%S");
            break;

          default:
            return full;
        }

        if (upcase) {
          result = result.toUpperCase();
        }

        if (invert) {
          result = result.replace(/[A-Z]/, function(c) { c.toLowerCase() }).
                          replace(/[a-z]/, function(c) { c.toUpperCase() });
        }

        if (pad && (zero || blank)) {
          result = (result).$rjust(isNaN(width) ? 2 : width, blank ? " " : "0");
        }

        return result;
      });
    
    };

    def['$sunday?'] = function() {
      var self = this;

      return self.$wday() == 0;
    };

    def['$thursday?'] = function() {
      var self = this;

      return self.$wday() == 4;
    };

    def.$to_a = function() {
      var self = this;

      return [self.$sec(), self.$min(), self.$hour(), self.$day(), self.$month(), self.$year(), self.$wday(), self.$yday(), self.$isdst(), self.$zone()];
    };

    def.$to_f = function() {
      var self = this;

      return self.getTime() / 1000;
    };

    def.$to_i = function() {
      var self = this;

      return parseInt(self.getTime() / 1000);
    };

    Opal.defn(self, '$to_s', def.$inspect);

    def['$tuesday?'] = function() {
      var self = this;

      return self.$wday() == 2;
    };

    Opal.defn(self, '$utc?', def['$gmt?']);

    Opal.defn(self, '$utc_offset', def.$gmt_offset);

    def.$wday = function() {
      var self = this;

      
      if (self.tz_offset === 0) {
        return self.getUTCDay();
      }
      else {
        return self.getDay();
      }
    ;
    };

    def['$wednesday?'] = function() {
      var self = this;

      return self.$wday() == 3;
    };

    def.$year = function() {
      var self = this;

      
      if (self.tz_offset === 0) {
        return self.getUTCFullYear();
      }
      else {
        return self.getFullYear();
      }
    ;
    };

    self.$private("cweek_cyear");

    return (def.$cweek_cyear = function() {
      var $a, $b, self = this, jan01 = nil, jan01_wday = nil, first_monday = nil, year = nil, offset = nil, week = nil, dec31 = nil, dec31_wday = nil;

      jan01 = $scope.get('Time').$new(self.$year(), 1, 1);
      jan01_wday = jan01.$wday();
      first_monday = 0;
      year = self.$year();
      if ((($a = (($b = $rb_le(jan01_wday, 4)) ? jan01_wday['$=='](0)['$!']() : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
        offset = $rb_minus(jan01_wday, 1)
        } else {
        offset = $rb_minus($rb_minus(jan01_wday, 7), 1);
        if (offset['$=='](-8)) {
          offset = -1};
      };
      week = ($rb_divide(($rb_plus(self.$yday(), offset)), 7.0)).$ceil();
      if ($rb_le(week, 0)) {
        return $scope.get('Time').$new($rb_minus(self.$year(), 1), 12, 31).$cweek_cyear()
      } else if (week['$=='](53)) {
        dec31 = $scope.get('Time').$new(self.$year(), 12, 31);
        dec31_wday = dec31.$wday();
        if ((($a = (($b = $rb_le(dec31_wday, 3)) ? dec31_wday['$=='](0)['$!']() : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
          week = 1;
          year = $rb_plus(year, 1);};};
      return [week, year];
    }, nil) && 'cweek_cyear';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/struct"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_lt(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs < rhs : lhs['$<'](rhs);
  }
  function $rb_ge(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs >= rhs : lhs['$>='](rhs);
  }
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$include', '$==', '$[]', '$upcase', '$const_set', '$new', '$unshift', '$each', '$define_struct_attribute', '$instance_eval', '$to_proc', '$raise', '$<<', '$members', '$attr_accessor', '$each_with_index', '$instance_variable_set', '$class', '$===', '$-@', '$size', '$include?', '$to_sym', '$instance_variable_get', '$enum_for', '$hash', '$all?', '$length', '$map', '$join', '$inspect', '$each_pair', '$inject', '$[]=', '$flatten', '$to_a']);
  self.$require("corelib/enumerable");
  return (function($base, $super) {
    function $Struct(){};
    var self = $Struct = $klass($base, $super, 'Struct', $Struct);

    var def = self.$$proto, $scope = self.$$scope, TMP_1, TMP_6, TMP_8;

    self.$include($scope.get('Enumerable'));

    Opal.defs(self, '$new', TMP_1 = function(name, args) {var $zuper = $slice.call(arguments, 0);
      var $a, $b, $c, TMP_2, self = this, $iter = TMP_1.$$p, block = $iter || nil;

      args = $slice.call(arguments, 1);
      TMP_1.$$p = null;
      if (self['$==']($scope.get('Struct'))) {
        } else {
        return Opal.find_super_dispatcher(self, 'new', TMP_1, $iter, $Struct).apply(self, $zuper)
      };
      if (name['$[]'](0)['$=='](name['$[]'](0).$upcase())) {
        return $scope.get('Struct').$const_set(name, ($a = self).$new.apply($a, [].concat(args)))
        } else {
        args.$unshift(name);
        return ($b = ($c = $scope.get('Class')).$new, $b.$$p = (TMP_2 = function(){var self = TMP_2.$$s || this, $a, $b, TMP_3, $c;

        ($a = ($b = args).$each, $a.$$p = (TMP_3 = function(arg){var self = TMP_3.$$s || this;
if (arg == null) arg = nil;
          return self.$define_struct_attribute(arg)}, TMP_3.$$s = self, TMP_3), $a).call($b);
          if (block !== false && block !== nil) {
            return ($a = ($c = self).$instance_eval, $a.$$p = block.$to_proc(), $a).call($c)
            } else {
            return nil
          };}, TMP_2.$$s = self, TMP_2), $b).call($c, self);
      };
    });

    Opal.defs(self, '$define_struct_attribute', function(name) {
      var self = this;

      if (self['$==']($scope.get('Struct'))) {
        self.$raise($scope.get('ArgumentError'), "you cannot define attributes to the Struct class")};
      self.$members()['$<<'](name);
      return self.$attr_accessor(name);
    });

    Opal.defs(self, '$members', function() {
      var $a, self = this;
      if (self.members == null) self.members = nil;

      if (self['$==']($scope.get('Struct'))) {
        self.$raise($scope.get('ArgumentError'), "the Struct class has no members")};
      return ((($a = self.members) !== false && $a !== nil) ? $a : self.members = []);
    });

    Opal.defs(self, '$inherited', function(klass) {
      var $a, $b, TMP_4, self = this, members = nil;
      if (self.members == null) self.members = nil;

      members = self.members;
      return ($a = ($b = klass).$instance_eval, $a.$$p = (TMP_4 = function(){var self = TMP_4.$$s || this;

      return self.members = members}, TMP_4.$$s = self, TMP_4), $a).call($b);
    });

    (function(self) {
      var $scope = self.$$scope, def = self.$$proto;

      return self.$$proto['$[]'] = self.$$proto.$new
    })(self.$singleton_class());

    def.$initialize = function(args) {
      var $a, $b, TMP_5, self = this;

      args = $slice.call(arguments, 0);
      return ($a = ($b = self.$members()).$each_with_index, $a.$$p = (TMP_5 = function(name, index){var self = TMP_5.$$s || this;
if (name == null) name = nil;if (index == null) index = nil;
      return self.$instance_variable_set("@" + (name), args['$[]'](index))}, TMP_5.$$s = self, TMP_5), $a).call($b);
    };

    def.$members = function() {
      var self = this;

      return self.$class().$members();
    };

    def['$[]'] = function(name) {
      var $a, self = this;

      if ((($a = $scope.get('Integer')['$==='](name)) !== nil && (!$a.$$is_boolean || $a == true))) {
        if ($rb_lt(name, self.$members().$size()['$-@']())) {
          self.$raise($scope.get('IndexError'), "offset " + (name) + " too small for struct(size:" + (self.$members().$size()) + ")")};
        if ($rb_ge(name, self.$members().$size())) {
          self.$raise($scope.get('IndexError'), "offset " + (name) + " too large for struct(size:" + (self.$members().$size()) + ")")};
        name = self.$members()['$[]'](name);
      } else if ((($a = $scope.get('String')['$==='](name)) !== nil && (!$a.$$is_boolean || $a == true))) {
        if ((($a = self.$members()['$include?'](name.$to_sym())) !== nil && (!$a.$$is_boolean || $a == true))) {
          } else {
          self.$raise($scope.get('NameError'), "no member '" + (name) + "' in struct")
        }
        } else {
        self.$raise($scope.get('TypeError'), "no implicit conversion of " + (name.$class()) + " into Integer")
      };
      return self.$instance_variable_get("@" + (name));
    };

    def['$[]='] = function(name, value) {
      var $a, self = this;

      if ((($a = $scope.get('Integer')['$==='](name)) !== nil && (!$a.$$is_boolean || $a == true))) {
        if ($rb_lt(name, self.$members().$size()['$-@']())) {
          self.$raise($scope.get('IndexError'), "offset " + (name) + " too small for struct(size:" + (self.$members().$size()) + ")")};
        if ($rb_ge(name, self.$members().$size())) {
          self.$raise($scope.get('IndexError'), "offset " + (name) + " too large for struct(size:" + (self.$members().$size()) + ")")};
        name = self.$members()['$[]'](name);
      } else if ((($a = $scope.get('String')['$==='](name)) !== nil && (!$a.$$is_boolean || $a == true))) {
        if ((($a = self.$members()['$include?'](name.$to_sym())) !== nil && (!$a.$$is_boolean || $a == true))) {
          } else {
          self.$raise($scope.get('NameError'), "no member '" + (name) + "' in struct")
        }
        } else {
        self.$raise($scope.get('TypeError'), "no implicit conversion of " + (name.$class()) + " into Integer")
      };
      return self.$instance_variable_set("@" + (name), value);
    };

    def.$each = TMP_6 = function() {
      var $a, $b, TMP_7, self = this, $iter = TMP_6.$$p, $yield = $iter || nil;

      TMP_6.$$p = null;
      if (($yield !== nil)) {
        } else {
        return self.$enum_for("each")
      };
      ($a = ($b = self.$members()).$each, $a.$$p = (TMP_7 = function(name){var self = TMP_7.$$s || this, $a;
if (name == null) name = nil;
      return $a = Opal.yield1($yield, self['$[]'](name)), $a === $breaker ? $a : $a}, TMP_7.$$s = self, TMP_7), $a).call($b);
      return self;
    };

    def.$each_pair = TMP_8 = function() {
      var $a, $b, TMP_9, self = this, $iter = TMP_8.$$p, $yield = $iter || nil;

      TMP_8.$$p = null;
      if (($yield !== nil)) {
        } else {
        return self.$enum_for("each_pair")
      };
      ($a = ($b = self.$members()).$each, $a.$$p = (TMP_9 = function(name){var self = TMP_9.$$s || this, $a;
if (name == null) name = nil;
      return $a = Opal.yieldX($yield, [name, self['$[]'](name)]), $a === $breaker ? $a : $a}, TMP_9.$$s = self, TMP_9), $a).call($b);
      return self;
    };

    def['$eql?'] = function(other) {
      var $a, $b, $c, TMP_10, self = this;

      return ((($a = self.$hash()['$=='](other.$hash())) !== false && $a !== nil) ? $a : ($b = ($c = other.$each_with_index())['$all?'], $b.$$p = (TMP_10 = function(object, index){var self = TMP_10.$$s || this;
if (object == null) object = nil;if (index == null) index = nil;
      return self['$[]'](self.$members()['$[]'](index))['$=='](object)}, TMP_10.$$s = self, TMP_10), $b).call($c));
    };

    def.$length = function() {
      var self = this;

      return self.$members().$length();
    };

    Opal.defn(self, '$size', def.$length);

    def.$to_a = function() {
      var $a, $b, TMP_11, self = this;

      return ($a = ($b = self.$members()).$map, $a.$$p = (TMP_11 = function(name){var self = TMP_11.$$s || this;
if (name == null) name = nil;
      return self['$[]'](name)}, TMP_11.$$s = self, TMP_11), $a).call($b);
    };

    Opal.defn(self, '$values', def.$to_a);

    def.$inspect = function() {
      var $a, $b, TMP_12, self = this, result = nil;

      result = "#<struct ";
      if (self.$class()['$==']($scope.get('Struct'))) {
        result = $rb_plus(result, "" + (self.$class()) + " ")};
      result = $rb_plus(result, ($a = ($b = self.$each_pair()).$map, $a.$$p = (TMP_12 = function(name, value){var self = TMP_12.$$s || this;
if (name == null) name = nil;if (value == null) value = nil;
      return "" + (name) + "=" + (value.$inspect())}, TMP_12.$$s = self, TMP_12), $a).call($b).$join(", "));
      result = $rb_plus(result, ">");
      return result;
    };

    Opal.defn(self, '$to_s', def.$inspect);

    def.$to_h = function() {
      var $a, $b, TMP_13, self = this;

      return ($a = ($b = self.$members()).$inject, $a.$$p = (TMP_13 = function(h, name){var self = TMP_13.$$s || this;
if (h == null) h = nil;if (name == null) name = nil;
      h['$[]='](name, self['$[]'](name));
        return h;}, TMP_13.$$s = self, TMP_13), $a).call($b, $hash2([], {}));
    };

    return (def.$values_at = function(args) {
      var $a, $b, TMP_14, self = this;

      args = $slice.call(arguments, 0);
      args = ($a = ($b = args).$map, $a.$$p = (TMP_14 = function(arg){var self = TMP_14.$$s || this;
if (arg == null) arg = nil;
      return arg.$$is_range ? arg.$to_a() : arg;}, TMP_14.$$s = self, TMP_14), $a).call($b).$flatten();
      
      var result = [];
      for (var i = 0, len = args.length; i < len; i++) {
        if (!args[i].$$is_number) {
          self.$raise($scope.get('TypeError'), "no implicit conversion of " + ((args[i]).$class()) + " into Integer")
        }
        result.push(self['$[]'](args[i]));
      }
      return result;
    ;
    }, nil) && 'values_at';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/io"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var $a, $b, self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $module = Opal.module, $gvars = Opal.gvars;
  if ($gvars.stdout == null) $gvars.stdout = nil;
  if ($gvars.stderr == null) $gvars.stderr = nil;

  Opal.add_stubs(['$attr_accessor', '$size', '$write', '$join', '$map', '$String', '$empty?', '$concat', '$chomp', '$getbyte', '$getc', '$raise', '$new', '$write_proc=', '$extend']);
  (function($base, $super) {
    function $IO(){};
    var self = $IO = $klass($base, $super, 'IO', $IO);

    var def = self.$$proto, $scope = self.$$scope;

    def.tty = def.closed = nil;
    Opal.cdecl($scope, 'SEEK_SET', 0);

    Opal.cdecl($scope, 'SEEK_CUR', 1);

    Opal.cdecl($scope, 'SEEK_END', 2);

    def['$tty?'] = function() {
      var self = this;

      return self.tty;
    };

    def['$closed?'] = function() {
      var self = this;

      return self.closed;
    };

    self.$attr_accessor("write_proc");

    def.$write = function(string) {
      var self = this;

      self.write_proc(string);
      return string.$size();
    };

    self.$attr_accessor("sync");

    (function($base) {
      var self = $module($base, 'Writable');

      var def = self.$$proto, $scope = self.$$scope;

      Opal.defn(self, '$<<', function(string) {
        var self = this;

        self.$write(string);
        return self;
      });

      Opal.defn(self, '$print', function(args) {
        var $a, $b, TMP_1, self = this;
        if ($gvars[","] == null) $gvars[","] = nil;

        args = $slice.call(arguments, 0);
        self.$write(($a = ($b = args).$map, $a.$$p = (TMP_1 = function(arg){var self = TMP_1.$$s || this;
if (arg == null) arg = nil;
        return self.$String(arg)}, TMP_1.$$s = self, TMP_1), $a).call($b).$join($gvars[","]));
        return nil;
      });

      Opal.defn(self, '$puts', function(args) {
        var $a, $b, TMP_2, self = this, newline = nil;
        if ($gvars["/"] == null) $gvars["/"] = nil;

        args = $slice.call(arguments, 0);
        newline = $gvars["/"];
        if ((($a = args['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
          self.$write($gvars["/"])
          } else {
          self.$write(($a = ($b = args).$map, $a.$$p = (TMP_2 = function(arg){var self = TMP_2.$$s || this;
if (arg == null) arg = nil;
          return self.$String(arg).$chomp()}, TMP_2.$$s = self, TMP_2), $a).call($b).$concat([nil]).$join(newline))
        };
        return nil;
      });
    })(self);

    return (function($base) {
      var self = $module($base, 'Readable');

      var def = self.$$proto, $scope = self.$$scope;

      Opal.defn(self, '$readbyte', function() {
        var self = this;

        return self.$getbyte();
      });

      Opal.defn(self, '$readchar', function() {
        var self = this;

        return self.$getc();
      });

      Opal.defn(self, '$readline', function(sep) {
        var self = this;
        if ($gvars["/"] == null) $gvars["/"] = nil;

        if (sep == null) {
          sep = $gvars["/"]
        }
        return self.$raise($scope.get('NotImplementedError'));
      });

      Opal.defn(self, '$readpartial', function(integer, outbuf) {
        var self = this;

        if (outbuf == null) {
          outbuf = nil
        }
        return self.$raise($scope.get('NotImplementedError'));
      });
    })(self);
  })(self, null);
  Opal.cdecl($scope, 'STDERR', $gvars.stderr = $scope.get('IO').$new());
  Opal.cdecl($scope, 'STDIN', $gvars.stdin = $scope.get('IO').$new());
  Opal.cdecl($scope, 'STDOUT', $gvars.stdout = $scope.get('IO').$new());
  (($a = [typeof(process) === 'object' ? function(s){process.stdout.write(s)} : function(s){console.log(s)}]), $b = $gvars.stdout, $b['$write_proc='].apply($b, $a), $a[$a.length-1]);
  (($a = [typeof(process) === 'object' ? function(s){process.stderr.write(s)} : function(s){console.warn(s)}]), $b = $gvars.stderr, $b['$write_proc='].apply($b, $a), $a[$a.length-1]);
  $gvars.stdout.$extend((($scope.get('IO')).$$scope.get('Writable')));
  return $gvars.stderr.$extend((($scope.get('IO')).$$scope.get('Writable')));
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/main"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice;

  Opal.add_stubs(['$include']);
  Opal.defs(self, '$to_s', function() {
    var self = this;

    return "main";
  });
  return (Opal.defs(self, '$include', function(mod) {
    var self = this;

    return $scope.get('Object').$include(mod);
  }), nil) && 'include';
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/variables"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $gvars = Opal.gvars, $hash2 = Opal.hash2;

  Opal.add_stubs(['$new']);
  $gvars["&"] = $gvars["~"] = $gvars["`"] = $gvars["'"] = nil;
  $gvars.LOADED_FEATURES = $gvars["\""] = Opal.loaded_features;
  $gvars.LOAD_PATH = $gvars[":"] = [];
  $gvars["/"] = "\n";
  $gvars[","] = nil;
  Opal.cdecl($scope, 'ARGV', []);
  Opal.cdecl($scope, 'ARGF', $scope.get('Object').$new());
  Opal.cdecl($scope, 'ENV', $hash2([], {}));
  $gvars.VERBOSE = false;
  $gvars.DEBUG = false;
  $gvars.SAFE = 0;
  Opal.cdecl($scope, 'RUBY_PLATFORM', "opal");
  Opal.cdecl($scope, 'RUBY_ENGINE', "opal");
  Opal.cdecl($scope, 'RUBY_VERSION', "2.1.5");
  Opal.cdecl($scope, 'RUBY_ENGINE_VERSION', "0.8.1");
  return Opal.cdecl($scope, 'RUBY_RELEASE_DATE', "2015-10-12");
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/dir"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$[]']);
  return (function($base, $super) {
    function $Dir(){};
    var self = $Dir = $klass($base, $super, 'Dir', $Dir);

    var def = self.$$proto, $scope = self.$$scope;

    return (function(self) {
      var $scope = self.$$scope, def = self.$$proto, TMP_1;

      self.$$proto.$chdir = TMP_1 = function(dir) {
        var $a, self = this, $iter = TMP_1.$$p, $yield = $iter || nil, prev_cwd = nil;

        TMP_1.$$p = null;
        try {
        prev_cwd = Opal.current_dir;
        Opal.current_dir = dir;
        return $a = Opal.yieldX($yield, []), $a === $breaker ? $a : $a;
        } finally {
        Opal.current_dir = prev_cwd;
        };
      };
      self.$$proto.$pwd = function() {
        var self = this;

        return Opal.current_dir || '.';
      };
      self.$$proto.$getwd = self.$$proto.$pwd;
      return (self.$$proto.$home = function() {
        var $a, self = this;

        return ((($a = $scope.get('ENV')['$[]']("HOME")) !== false && $a !== nil) ? $a : ".");
      }, nil) && 'home';
    })(self.$singleton_class())
  })(self, null)
};
/* Generated by Opal 0.8.1 */
Opal.modules["corelib/file"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $range = Opal.range;

  Opal.add_stubs(['$join', '$compact', '$split', '$==', '$first', '$[]=', '$home', '$each', '$pop', '$<<', '$[]', '$gsub', '$find', '$=~']);
  return (function($base, $super) {
    function $File(){};
    var self = $File = $klass($base, $super, 'File', $File);

    var def = self.$$proto, $scope = self.$$scope;

    Opal.cdecl($scope, 'Separator', Opal.cdecl($scope, 'SEPARATOR', "/"));

    Opal.cdecl($scope, 'ALT_SEPARATOR', nil);

    Opal.cdecl($scope, 'PATH_SEPARATOR', ":");

    return (function(self) {
      var $scope = self.$$scope, def = self.$$proto;

      self.$$proto.$expand_path = function(path, basedir) {
        var $a, $b, TMP_1, self = this, parts = nil, new_parts = nil;

        if (basedir == null) {
          basedir = nil
        }
        path = [basedir, path].$compact().$join($scope.get('SEPARATOR'));
        parts = path.$split($scope.get('SEPARATOR'));
        new_parts = [];
        if (parts.$first()['$==']("~")) {
          parts['$[]='](0, $scope.get('Dir').$home())};
        ($a = ($b = parts).$each, $a.$$p = (TMP_1 = function(part){var self = TMP_1.$$s || this;
if (part == null) part = nil;
        if (part['$==']("..")) {
            return new_parts.$pop()
            } else {
            return new_parts['$<<'](part)
          }}, TMP_1.$$s = self, TMP_1), $a).call($b);
        return new_parts.$join($scope.get('SEPARATOR'));
      };
      self.$$proto.$dirname = function(path) {
        var self = this;

        return self.$split(path)['$[]']($range(0, -2, false));
      };
      self.$$proto.$basename = function(path) {
        var self = this;

        return self.$split(path)['$[]'](-1);
      };
      self.$$proto['$exist?'] = function(path) {
        var self = this;

        return Opal.modules[path] != null;
      };
      self.$$proto['$exists?'] = self.$$proto['$exist?'];
      self.$$proto['$directory?'] = function(path) {
        var $a, $b, TMP_2, self = this, files = nil, file = nil;

        files = [];
        
        for (var key in Opal.modules) {
          files.push(key)
        }
      ;
        path = path.$gsub((new RegExp("(^." + $scope.get('SEPARATOR') + "+|" + $scope.get('SEPARATOR') + "+$)")));
        file = ($a = ($b = files).$find, $a.$$p = (TMP_2 = function(file){var self = TMP_2.$$s || this;
if (file == null) file = nil;
        return file['$=~']((new RegExp("^" + path)))}, TMP_2.$$s = self, TMP_2), $a).call($b);
        return file;
      };
      self.$$proto.$join = function(paths) {
        var self = this;

        paths = $slice.call(arguments, 0);
        return paths.$join($scope.get('SEPARATOR')).$gsub((new RegExp("" + $scope.get('SEPARATOR') + "+")), $scope.get('SEPARATOR'));
      };
      return (self.$$proto.$split = function(path) {
        var self = this;

        return path.$split($scope.get('SEPARATOR'));
      }, nil) && 'split';
    })(self.$singleton_class());
  })(self, $scope.get('IO'))
};
/* Generated by Opal 0.8.1 */
(function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice;

  Opal.add_stubs(['$require']);
  self.$require("corelib/runtime");
  self.$require("corelib/helpers");
  self.$require("corelib/module");
  self.$require("corelib/class");
  self.$require("corelib/basic_object");
  self.$require("corelib/kernel");
  self.$require("corelib/nil_class");
  self.$require("corelib/boolean");
  self.$require("corelib/error");
  self.$require("corelib/regexp");
  self.$require("corelib/comparable");
  self.$require("corelib/enumerable");
  self.$require("corelib/enumerator");
  self.$require("corelib/array");
  self.$require("corelib/array/inheritance");
  self.$require("corelib/hash");
  self.$require("corelib/string");
  self.$require("corelib/string/inheritance");
  self.$require("corelib/match_data");
  self.$require("corelib/numeric");
  self.$require("corelib/complex");
  self.$require("corelib/rational");
  self.$require("corelib/proc");
  self.$require("corelib/method");
  self.$require("corelib/range");
  self.$require("corelib/time");
  self.$require("corelib/struct");
  self.$require("corelib/io");
  self.$require("corelib/main");
  self.$require("corelib/variables");
  self.$require("corelib/dir");
  return self.$require("corelib/file");
})(Opal);
(function(){

if (!window.qx) window.qx = {};

qx.$$start = new Date();

if (!qx.$$environment) qx.$$environment = {};
var envinfo = {"qx.application":"qx_lib.Application","qx.debug":false,"qx.debug.databinding":false,"qx.debug.dispose":false,"qx.debug.io":false,"qx.debug.ui.queue":false,"qx.optimization.basecalls":true,"qx.optimization.comments":true,"qx.optimization.privates":true,"qx.optimization.strings":true,"qx.optimization.variables":true,"qx.optimization.variants":true,"qx.optimization.whitespace":true,"qx.revision":"","qx.theme":"qx_lib.theme.Theme","qx.version":"5.0.1"};
for (var k in envinfo) qx.$$environment[k] = envinfo[k];

if (!qx.$$libraries) qx.$$libraries = {};
var libinfo = {"__out__":{"sourceUri":"script"},"qx":{"resourceUri":"resource","sourceUri":"script","sourceViewUri":"https://github.com/qooxdoo/qooxdoo/blob/%{qxGitBranch}/framework/source/class/%{classFilePath}#L%{lineNumber}"},"qx_lib":{"resourceUri":"resource","sourceUri":"script"}};
for (var k in libinfo) qx.$$libraries[k] = libinfo[k];

qx.$$resources = {};
qx.$$translations = {"C":null,"en":null};
qx.$$locales = {"C":null,"en":null};
qx.$$packageData = {};
qx.$$g = {}

qx.$$loader = {
  parts : {"boot":[0]},
  packages : {"0":{"uris":["__out__:qx_lib.06412b5e089f.js"]}},
  urisBefore : [],
  cssBefore : [],
  boot : "boot",
  closureParts : {},
  bootIsInline : true,
  addNoCacheParam : false,

  decodeUris : function(compressedUris)
  {
    var libs = qx.$$libraries;
    var uris = [];
    for (var i=0; i<compressedUris.length; i++)
    {
      var uri = compressedUris[i].split(":");
      var euri;
      if (uri.length==2 && uri[0] in libs) {
        var prefix = libs[uri[0]].sourceUri;
        euri = prefix + "/" + uri[1];
      } else {
        euri = compressedUris[i];
      }
      if (qx.$$loader.addNoCacheParam) {
        euri += "?nocache=" + Math.random();
      }
      
      uris.push(euri);
    }
    return uris;
  }
};

var readyStateValue = {"complete" : true};
if (document.documentMode && document.documentMode < 10 ||
    (typeof window.ActiveXObject !== "undefined" && !document.documentMode)) {
  readyStateValue["loaded"] = true;
}

function loadScript(uri, callback) {
  var elem = document.createElement("script");
  elem.charset = "utf-8";
  elem.src = uri;
  elem.onreadystatechange = elem.onload = function() {
    if (!this.readyState || readyStateValue[this.readyState]) {
      elem.onreadystatechange = elem.onload = null;
      if (typeof callback === "function") {
        callback();
      }
    }
  };

  if (isLoadParallel) {
    elem.async = null;
  }

  var head = document.getElementsByTagName("head")[0];
  head.appendChild(elem);
}

function loadCss(uri) {
  var elem = document.createElement("link");
  elem.rel = "stylesheet";
  elem.type= "text/css";
  elem.href= uri;
  var head = document.getElementsByTagName("head")[0];
  head.appendChild(elem);
}

var isWebkit = /AppleWebKit\/([^ ]+)/.test(navigator.userAgent);
var isLoadParallel = 'async' in document.createElement('script');

function loadScriptList(list, callback) {
  if (list.length == 0) {
    callback();
    return;
  }

  var item;

  if (isLoadParallel) {
    while (list.length) {
      item = list.shift();
      if (list.length) {
        loadScript(item);
      } else {
        loadScript(item, callback);
      }
    }
  } else {
    item = list.shift();
    loadScript(item,  function() {
      if (isWebkit) {
        // force async, else Safari fails with a "maximum recursion depth exceeded"
        window.setTimeout(function() {
          loadScriptList(list, callback);
        }, 0);
      } else {
        loadScriptList(list, callback);
      }
    });
  }
}

var fireContentLoadedEvent = function() {
  qx.$$domReady = true;
  document.removeEventListener('DOMContentLoaded', fireContentLoadedEvent, false);
};
if (document.addEventListener) {
  document.addEventListener('DOMContentLoaded', fireContentLoadedEvent, false);
}

qx.$$loader.importPackageData = function (dataMap, callback) {
  if (dataMap["resources"]){
    var resMap = dataMap["resources"];
    for (var k in resMap) qx.$$resources[k] = resMap[k];
  }
  if (dataMap["locales"]){
    var locMap = dataMap["locales"];
    var qxlocs = qx.$$locales;
    for (var lang in locMap){
      if (!qxlocs[lang]) qxlocs[lang] = locMap[lang];
      else
        for (var k in locMap[lang]) qxlocs[lang][k] = locMap[lang][k];
    }
  }
  if (dataMap["translations"]){
    var trMap   = dataMap["translations"];
    var qxtrans = qx.$$translations;
    for (var lang in trMap){
      if (!qxtrans[lang]) qxtrans[lang] = trMap[lang];
      else
        for (var k in trMap[lang]) qxtrans[lang][k] = trMap[lang][k];
    }
  }
  if (callback){
    callback(dataMap);
  }
}

qx.$$loader.signalStartup = function ()
{
  qx.$$loader.scriptLoaded = true;
  if (window.qx && qx.event && qx.event.handler && qx.event.handler.Application) {
    qx.event.handler.Application.onScriptLoaded();
    qx.$$loader.applicationHandlerReady = true;
  } else {
    qx.$$loader.applicationHandlerReady = false;
  }
}

// Load all stuff
qx.$$loader.init = function(){
  var l=qx.$$loader;
  if (l.cssBefore.length>0) {
    for (var i=0, m=l.cssBefore.length; i<m; i++) {
      loadCss(l.cssBefore[i]);
    }
  }
  if (l.urisBefore.length>0){
    loadScriptList(l.urisBefore, function(){
      l.initUris();
    });
  } else {
    l.initUris();
  }
}

// Load qooxdoo boot stuff
qx.$$loader.initUris = function(){
  var l=qx.$$loader;
  var bootPackageHash=l.parts[l.boot][0];
  if (l.bootIsInline){
    l.importPackageData(qx.$$packageData[bootPackageHash]);
    l.signalStartup();
  } else {
    loadScriptList(l.decodeUris(l.packages[l.parts[l.boot][0]].uris), function(){
      // Opera needs this extra time to parse the scripts
      window.setTimeout(function(){
        l.importPackageData(qx.$$packageData[bootPackageHash] || {});
        l.signalStartup();
      }, 0);
    });
  }
}
})();

qx.$$packageData['0']={"locales":{"C":{"alternateQuotationEnd":"’","alternateQuotationStart":"‘","cldr_am":"AM","cldr_date_format_full":"EEEE, MMMM d, y","cldr_date_format_long":"MMMM d, y","cldr_date_format_medium":"MMM d, y","cldr_date_format_short":"M/d/yy","cldr_date_time_format_EHm":"E HH:mm","cldr_date_time_format_EHms":"E HH:mm:ss","cldr_date_time_format_Ed":"d E","cldr_date_time_format_Ehm":"E h:mm a","cldr_date_time_format_Ehms":"E h:mm:ss a","cldr_date_time_format_Gy":"y G","cldr_date_time_format_GyMMM":"MMM y G","cldr_date_time_format_GyMMMEd":"E, MMM d, y G","cldr_date_time_format_GyMMMd":"MMM d, y G","cldr_date_time_format_H":"HH","cldr_date_time_format_Hm":"HH:mm","cldr_date_time_format_Hms":"HH:mm:ss","cldr_date_time_format_M":"L","cldr_date_time_format_MEd":"E, M/d","cldr_date_time_format_MMM":"LLL","cldr_date_time_format_MMMEd":"E, MMM d","cldr_date_time_format_MMMd":"MMM d","cldr_date_time_format_Md":"M/d","cldr_date_time_format_d":"d","cldr_date_time_format_h":"h a","cldr_date_time_format_hm":"h:mm a","cldr_date_time_format_hms":"h:mm:ss a","cldr_date_time_format_ms":"mm:ss","cldr_date_time_format_y":"y","cldr_date_time_format_yM":"M/y","cldr_date_time_format_yMEd":"E, M/d/y","cldr_date_time_format_yMMM":"MMM y","cldr_date_time_format_yMMMEd":"E, MMM d, y","cldr_date_time_format_yMMMd":"MMM d, y","cldr_date_time_format_yMd":"M/d/y","cldr_date_time_format_yQQQ":"QQQ y","cldr_date_time_format_yQQQQ":"QQQQ y","cldr_day_format_abbreviated_fri":"Fri","cldr_day_format_abbreviated_mon":"Mon","cldr_day_format_abbreviated_sat":"Sat","cldr_day_format_abbreviated_sun":"Sun","cldr_day_format_abbreviated_thu":"Thu","cldr_day_format_abbreviated_tue":"Tue","cldr_day_format_abbreviated_wed":"Wed","cldr_day_format_short_fri":"Fr","cldr_day_format_short_mon":"Mo","cldr_day_format_short_sat":"Sa","cldr_day_format_short_sun":"Su","cldr_day_format_short_thu":"Th","cldr_day_format_short_tue":"Tu","cldr_day_format_short_wed":"We","cldr_day_format_wide_fri":"Friday","cldr_day_format_wide_mon":"Monday","cldr_day_format_wide_sat":"Saturday","cldr_day_format_wide_sun":"Sunday","cldr_day_format_wide_thu":"Thursday","cldr_day_format_wide_tue":"Tuesday","cldr_day_format_wide_wed":"Wednesday","cldr_day_stand-alone_narrow_fri":"F","cldr_day_stand-alone_narrow_mon":"M","cldr_day_stand-alone_narrow_sat":"S","cldr_day_stand-alone_narrow_sun":"S","cldr_day_stand-alone_narrow_thu":"T","cldr_day_stand-alone_narrow_tue":"T","cldr_day_stand-alone_narrow_wed":"W","cldr_month_format_abbreviated_1":"Jan","cldr_month_format_abbreviated_10":"Oct","cldr_month_format_abbreviated_11":"Nov","cldr_month_format_abbreviated_12":"Dec","cldr_month_format_abbreviated_2":"Feb","cldr_month_format_abbreviated_3":"Mar","cldr_month_format_abbreviated_4":"Apr","cldr_month_format_abbreviated_5":"May","cldr_month_format_abbreviated_6":"Jun","cldr_month_format_abbreviated_7":"Jul","cldr_month_format_abbreviated_8":"Aug","cldr_month_format_abbreviated_9":"Sep","cldr_month_format_wide_1":"January","cldr_month_format_wide_10":"October","cldr_month_format_wide_11":"November","cldr_month_format_wide_12":"December","cldr_month_format_wide_2":"February","cldr_month_format_wide_3":"March","cldr_month_format_wide_4":"April","cldr_month_format_wide_5":"May","cldr_month_format_wide_6":"June","cldr_month_format_wide_7":"July","cldr_month_format_wide_8":"August","cldr_month_format_wide_9":"September","cldr_month_stand-alone_narrow_1":"J","cldr_month_stand-alone_narrow_10":"O","cldr_month_stand-alone_narrow_11":"N","cldr_month_stand-alone_narrow_12":"D","cldr_month_stand-alone_narrow_2":"F","cldr_month_stand-alone_narrow_3":"M","cldr_month_stand-alone_narrow_4":"A","cldr_month_stand-alone_narrow_5":"M","cldr_month_stand-alone_narrow_6":"J","cldr_month_stand-alone_narrow_7":"J","cldr_month_stand-alone_narrow_8":"A","cldr_month_stand-alone_narrow_9":"S","cldr_number_decimal_separator":".","cldr_number_group_separator":",","cldr_number_percent_format":"#,##0%","cldr_pm":"PM","cldr_time_format_full":"h:mm:ss a zzzz","cldr_time_format_long":"h:mm:ss a z","cldr_time_format_medium":"h:mm:ss a","cldr_time_format_short":"h:mm a","quotationEnd":"”","quotationStart":"“"},"en":{"alternateQuotationEnd":"’","alternateQuotationStart":"‘","cldr_am":"AM","cldr_date_format_full":"EEEE, MMMM d, y","cldr_date_format_long":"MMMM d, y","cldr_date_format_medium":"MMM d, y","cldr_date_format_short":"M/d/yy","cldr_date_time_format_EHm":"E HH:mm","cldr_date_time_format_EHms":"E HH:mm:ss","cldr_date_time_format_Ed":"d E","cldr_date_time_format_Ehm":"E h:mm a","cldr_date_time_format_Ehms":"E h:mm:ss a","cldr_date_time_format_Gy":"y G","cldr_date_time_format_GyMMM":"MMM y G","cldr_date_time_format_GyMMMEd":"E, MMM d, y G","cldr_date_time_format_GyMMMd":"MMM d, y G","cldr_date_time_format_H":"HH","cldr_date_time_format_Hm":"HH:mm","cldr_date_time_format_Hms":"HH:mm:ss","cldr_date_time_format_M":"L","cldr_date_time_format_MEd":"E, M/d","cldr_date_time_format_MMM":"LLL","cldr_date_time_format_MMMEd":"E, MMM d","cldr_date_time_format_MMMd":"MMM d","cldr_date_time_format_Md":"M/d","cldr_date_time_format_d":"d","cldr_date_time_format_h":"h a","cldr_date_time_format_hm":"h:mm a","cldr_date_time_format_hms":"h:mm:ss a","cldr_date_time_format_ms":"mm:ss","cldr_date_time_format_y":"y","cldr_date_time_format_yM":"M/y","cldr_date_time_format_yMEd":"E, M/d/y","cldr_date_time_format_yMMM":"MMM y","cldr_date_time_format_yMMMEd":"E, MMM d, y","cldr_date_time_format_yMMMd":"MMM d, y","cldr_date_time_format_yMd":"M/d/y","cldr_date_time_format_yQQQ":"QQQ y","cldr_date_time_format_yQQQQ":"QQQQ y","cldr_day_format_abbreviated_fri":"Fri","cldr_day_format_abbreviated_mon":"Mon","cldr_day_format_abbreviated_sat":"Sat","cldr_day_format_abbreviated_sun":"Sun","cldr_day_format_abbreviated_thu":"Thu","cldr_day_format_abbreviated_tue":"Tue","cldr_day_format_abbreviated_wed":"Wed","cldr_day_format_short_fri":"Fr","cldr_day_format_short_mon":"Mo","cldr_day_format_short_sat":"Sa","cldr_day_format_short_sun":"Su","cldr_day_format_short_thu":"Th","cldr_day_format_short_tue":"Tu","cldr_day_format_short_wed":"We","cldr_day_format_wide_fri":"Friday","cldr_day_format_wide_mon":"Monday","cldr_day_format_wide_sat":"Saturday","cldr_day_format_wide_sun":"Sunday","cldr_day_format_wide_thu":"Thursday","cldr_day_format_wide_tue":"Tuesday","cldr_day_format_wide_wed":"Wednesday","cldr_day_stand-alone_narrow_fri":"F","cldr_day_stand-alone_narrow_mon":"M","cldr_day_stand-alone_narrow_sat":"S","cldr_day_stand-alone_narrow_sun":"S","cldr_day_stand-alone_narrow_thu":"T","cldr_day_stand-alone_narrow_tue":"T","cldr_day_stand-alone_narrow_wed":"W","cldr_month_format_abbreviated_1":"Jan","cldr_month_format_abbreviated_10":"Oct","cldr_month_format_abbreviated_11":"Nov","cldr_month_format_abbreviated_12":"Dec","cldr_month_format_abbreviated_2":"Feb","cldr_month_format_abbreviated_3":"Mar","cldr_month_format_abbreviated_4":"Apr","cldr_month_format_abbreviated_5":"May","cldr_month_format_abbreviated_6":"Jun","cldr_month_format_abbreviated_7":"Jul","cldr_month_format_abbreviated_8":"Aug","cldr_month_format_abbreviated_9":"Sep","cldr_month_format_wide_1":"January","cldr_month_format_wide_10":"October","cldr_month_format_wide_11":"November","cldr_month_format_wide_12":"December","cldr_month_format_wide_2":"February","cldr_month_format_wide_3":"March","cldr_month_format_wide_4":"April","cldr_month_format_wide_5":"May","cldr_month_format_wide_6":"June","cldr_month_format_wide_7":"July","cldr_month_format_wide_8":"August","cldr_month_format_wide_9":"September","cldr_month_stand-alone_narrow_1":"J","cldr_month_stand-alone_narrow_10":"O","cldr_month_stand-alone_narrow_11":"N","cldr_month_stand-alone_narrow_12":"D","cldr_month_stand-alone_narrow_2":"F","cldr_month_stand-alone_narrow_3":"M","cldr_month_stand-alone_narrow_4":"A","cldr_month_stand-alone_narrow_5":"M","cldr_month_stand-alone_narrow_6":"J","cldr_month_stand-alone_narrow_7":"J","cldr_month_stand-alone_narrow_8":"A","cldr_month_stand-alone_narrow_9":"S","cldr_number_decimal_separator":".","cldr_number_group_separator":",","cldr_number_percent_format":"#,##0%","cldr_pm":"PM","cldr_time_format_full":"h:mm:ss a zzzz","cldr_time_format_long":"h:mm:ss a z","cldr_time_format_medium":"h:mm:ss a","cldr_time_format_short":"h:mm a","quotationEnd":"”","quotationStart":"“"}},"resources":{"qx/decoration/Classic/arrows-combined.gif":[124,7,"gif","qx"],"qx/decoration/Classic/arrows/down-invert.gif":[7,4,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-38,0],"qx/decoration/Classic/arrows/down-small-invert.gif":[5,3,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-87,0],"qx/decoration/Classic/arrows/down-small.gif":[5,3,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-53,0],"qx/decoration/Classic/arrows/down.gif":[7,4,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-113,0],"qx/decoration/Classic/arrows/forward-invert.gif":[8,7,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-30,0],"qx/decoration/Classic/arrows/forward.gif":[8,7,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-18,0],"qx/decoration/Classic/arrows/left-invert.gif":[4,7,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-92,0],"qx/decoration/Classic/arrows/left-small-invert.gif":[3,5,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-58,0],"qx/decoration/Classic/arrows/left-small.gif":[3,5,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-15,0],"qx/decoration/Classic/arrows/left.gif":[4,7,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-120,0],"qx/decoration/Classic/arrows/next-invert.gif":[4,7,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-80,0],"qx/decoration/Classic/arrows/next.gif":[4,7,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-109,0],"qx/decoration/Classic/arrows/previous-invert.gif":[4,7,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-69,0],"qx/decoration/Classic/arrows/previous.gif":[4,7,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-65,0],"qx/decoration/Classic/arrows/rewind-invert.gif":[8,7,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-45,0],"qx/decoration/Classic/arrows/rewind.gif":[8,7,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-101,0],"qx/decoration/Classic/arrows/right-invert.gif":[4,7,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-61,0],"qx/decoration/Classic/arrows/right-small-invert.gif":[3,5,"gif","qx","qx/decoration/Classic/arrows-combined.gif",0,0],"qx/decoration/Classic/arrows/right-small.gif":[3,5,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-84,0],"qx/decoration/Classic/arrows/right.gif":[4,7,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-26,0],"qx/decoration/Classic/arrows/up-invert.gif":[7,4,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-73,0],"qx/decoration/Classic/arrows/up-small-invert.gif":[5,3,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-96,0],"qx/decoration/Classic/arrows/up-small.gif":[5,3,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-3,0],"qx/decoration/Classic/arrows/up.gif":[7,4,"gif","qx","qx/decoration/Classic/arrows-combined.gif",-8,0],"qx/decoration/Classic/checkbox-radiobutton-combined.png":[504,14,"png","qx"],"qx/decoration/Classic/colorselector-combined.gif":[46,11,"gif","qx"],"qx/decoration/Classic/colorselector/brightness-field.png":[19,256,"png","qx"],"qx/decoration/Classic/colorselector/brightness-handle.gif":[35,11,"gif","qx","qx/decoration/Classic/colorselector-combined.gif",-11,0],"qx/decoration/Classic/colorselector/huesaturation-field.jpg":[256,256,"jpeg","qx"],"qx/decoration/Classic/colorselector/huesaturation-handle.gif":[11,11,"gif","qx","qx/decoration/Classic/colorselector-combined.gif",0,0],"qx/decoration/Classic/cursors-combined.gif":[71,20,"gif","qx"],"qx/decoration/Classic/cursors/alias.gif":[19,15,"gif","qx","qx/decoration/Classic/cursors-combined.gif",-52,0],"qx/decoration/Classic/cursors/copy.gif":[19,15,"gif","qx","qx/decoration/Classic/cursors-combined.gif",-20,0],"qx/decoration/Classic/cursors/move.gif":[13,9,"gif","qx","qx/decoration/Classic/cursors-combined.gif",-39,0],"qx/decoration/Classic/cursors/nodrop.gif":[20,20,"gif","qx","qx/decoration/Classic/cursors-combined.gif",0,0],"qx/decoration/Classic/datechooser/last-month-invert.png":[16,16,"png","qx"],"qx/decoration/Classic/datechooser/last-month.png":[16,16,"png","qx"],"qx/decoration/Classic/datechooser/last-year-invert.png":[16,16,"png","qx"],"qx/decoration/Classic/datechooser/last-year.png":[16,16,"png","qx"],"qx/decoration/Classic/datechooser/next-month-invert.png":[16,16,"png","qx"],"qx/decoration/Classic/datechooser/next-month.png":[16,16,"png","qx"],"qx/decoration/Classic/datechooser/next-year-invert.png":[16,16,"png","qx"],"qx/decoration/Classic/datechooser/next-year.png":[16,16,"png","qx"],"qx/decoration/Classic/form/checkbox-checked-disabled.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-336,0],"qx/decoration/Classic/form/checkbox-checked-focused-invalid.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-28,0],"qx/decoration/Classic/form/checkbox-checked-focused.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-462,0],"qx/decoration/Classic/form/checkbox-checked-hovered-invalid.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-112,0],"qx/decoration/Classic/form/checkbox-checked-hovered.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-140,0],"qx/decoration/Classic/form/checkbox-checked-invalid.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-98,0],"qx/decoration/Classic/form/checkbox-checked-pressed-invalid.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-308,0],"qx/decoration/Classic/form/checkbox-checked-pressed.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",0,0],"qx/decoration/Classic/form/checkbox-checked.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-266,0],"qx/decoration/Classic/form/checkbox-disabled.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-84,0],"qx/decoration/Classic/form/checkbox-focused-invalid.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-476,0],"qx/decoration/Classic/form/checkbox-focused.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-392,0],"qx/decoration/Classic/form/checkbox-hovered-invalid.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-196,0],"qx/decoration/Classic/form/checkbox-hovered.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-154,0],"qx/decoration/Classic/form/checkbox-invalid.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-350,0],"qx/decoration/Classic/form/checkbox-pressed-invalid.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-448,0],"qx/decoration/Classic/form/checkbox-pressed.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-70,0],"qx/decoration/Classic/form/checkbox-undetermined-disabled.png":[14,14,"png","qx"],"qx/decoration/Classic/form/checkbox-undetermined-focused-invalid.png":[14,14,"png","qx"],"qx/decoration/Classic/form/checkbox-undetermined-focused.png":[14,14,"png","qx"],"qx/decoration/Classic/form/checkbox-undetermined-hovered-invalid.png":[14,14,"png","qx"],"qx/decoration/Classic/form/checkbox-undetermined-hovered.png":[14,14,"png","qx"],"qx/decoration/Classic/form/checkbox-undetermined-invalid.png":[14,14,"png","qx"],"qx/decoration/Classic/form/checkbox-undetermined.png":[14,14,"png","qx"],"qx/decoration/Classic/form/checkbox.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-490,0],"qx/decoration/Classic/form/radiobutton-checked-disabled.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-210,0],"qx/decoration/Classic/form/radiobutton-checked-focused-invalid.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-406,0],"qx/decoration/Classic/form/radiobutton-checked-focused.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-378,0],"qx/decoration/Classic/form/radiobutton-checked-hovered-invalid.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-252,0],"qx/decoration/Classic/form/radiobutton-checked-hovered.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-182,0],"qx/decoration/Classic/form/radiobutton-checked-invalid.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-294,0],"qx/decoration/Classic/form/radiobutton-checked-pressed-invalid.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-420,0],"qx/decoration/Classic/form/radiobutton-checked-pressed.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-56,0],"qx/decoration/Classic/form/radiobutton-checked.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-322,0],"qx/decoration/Classic/form/radiobutton-disabled.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-364,0],"qx/decoration/Classic/form/radiobutton-focused-invalid.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-434,0],"qx/decoration/Classic/form/radiobutton-focused.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-168,0],"qx/decoration/Classic/form/radiobutton-hovered-invalid.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-126,0],"qx/decoration/Classic/form/radiobutton-hovered.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-42,0],"qx/decoration/Classic/form/radiobutton-invalid.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-280,0],"qx/decoration/Classic/form/radiobutton-pressed-invalid.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-238,0],"qx/decoration/Classic/form/radiobutton-pressed.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-14,0],"qx/decoration/Classic/form/radiobutton.png":[14,14,"png","qx","qx/decoration/Classic/checkbox-radiobutton-combined.png",-224,0],"qx/decoration/Classic/menu-combined.gif":[64,7,"gif","qx"],"qx/decoration/Classic/menu/checkbox-invert.gif":[16,7,"gif","qx","qx/decoration/Classic/menu-combined.gif",-16,0],"qx/decoration/Classic/menu/checkbox.gif":[16,7,"gif","qx","qx/decoration/Classic/menu-combined.gif",-32,0],"qx/decoration/Classic/menu/radiobutton-invert.gif":[16,5,"gif","qx","qx/decoration/Classic/menu-combined.gif",0,0],"qx/decoration/Classic/menu/radiobutton.gif":[16,5,"gif","qx","qx/decoration/Classic/menu-combined.gif",-48,0],"qx/decoration/Classic/shadow-lr-combined.png":[30,382,"png","qx"],"qx/decoration/Classic/shadow-small-lr-combined.png":[10,136,"png","qx"],"qx/decoration/Classic/shadow-small-tb-combined.png":[5,30,"png","qx"],"qx/decoration/Classic/shadow-tb-combined.png":[15,90,"png","qx"],"qx/decoration/Classic/shadow/shadow-b.png":[15,15,"png","qx","qx/decoration/Classic/shadow-tb-combined.png",0,-30],"qx/decoration/Classic/shadow/shadow-bl.png":[15,15,"png","qx","qx/decoration/Classic/shadow-tb-combined.png",0,0],"qx/decoration/Classic/shadow/shadow-br.png":[15,15,"png","qx","qx/decoration/Classic/shadow-tb-combined.png",0,-60],"qx/decoration/Classic/shadow/shadow-c.png":[40,382,"png","qx"],"qx/decoration/Classic/shadow/shadow-l.png":[15,382,"png","qx","qx/decoration/Classic/shadow-lr-combined.png",-15,0],"qx/decoration/Classic/shadow/shadow-r.png":[15,382,"png","qx","qx/decoration/Classic/shadow-lr-combined.png",0,0],"qx/decoration/Classic/shadow/shadow-small-b.png":[5,5,"png","qx","qx/decoration/Classic/shadow-small-tb-combined.png",0,-25],"qx/decoration/Classic/shadow/shadow-small-bl.png":[5,5,"png","qx","qx/decoration/Classic/shadow-small-tb-combined.png",0,-20],"qx/decoration/Classic/shadow/shadow-small-br.png":[5,5,"png","qx","qx/decoration/Classic/shadow-small-tb-combined.png",0,0],"qx/decoration/Classic/shadow/shadow-small-c.png":[40,136,"png","qx"],"qx/decoration/Classic/shadow/shadow-small-l.png":[5,136,"png","qx","qx/decoration/Classic/shadow-small-lr-combined.png",0,0],"qx/decoration/Classic/shadow/shadow-small-r.png":[5,136,"png","qx","qx/decoration/Classic/shadow-small-lr-combined.png",-5,0],"qx/decoration/Classic/shadow/shadow-small-t.png":[5,5,"png","qx","qx/decoration/Classic/shadow-small-tb-combined.png",0,-5],"qx/decoration/Classic/shadow/shadow-small-tl.png":[5,5,"png","qx","qx/decoration/Classic/shadow-small-tb-combined.png",0,-15],"qx/decoration/Classic/shadow/shadow-small-tr.png":[5,5,"png","qx","qx/decoration/Classic/shadow-small-tb-combined.png",0,-10],"qx/decoration/Classic/shadow/shadow-small.png":[114,146,"png","qx"],"qx/decoration/Classic/shadow/shadow-t.png":[15,15,"png","qx","qx/decoration/Classic/shadow-tb-combined.png",0,-75],"qx/decoration/Classic/shadow/shadow-tl.png":[15,15,"png","qx","qx/decoration/Classic/shadow-tb-combined.png",0,-45],"qx/decoration/Classic/shadow/shadow-tr.png":[15,15,"png","qx","qx/decoration/Classic/shadow-tb-combined.png",0,-15],"qx/decoration/Classic/shadow/shadow.png":[381,412,"png","qx"],"qx/decoration/Classic/splitpane/knob-horizontal.png":[4,15,"png","qx"],"qx/decoration/Classic/splitpane/knob-vertical.png":[15,4,"png","qx"],"qx/decoration/Classic/table-combined.png":[72,11,"png","qx"],"qx/decoration/Classic/table/ascending-invert.png":[10,10,"png","qx","qx/decoration/Classic/table-combined.png",-62,0],"qx/decoration/Classic/table/ascending.png":[10,10,"png","qx","qx/decoration/Classic/table-combined.png",-52,0],"qx/decoration/Classic/table/boolean-false.png":[11,11,"png","qx","qx/decoration/Classic/table-combined.png",-31,0],"qx/decoration/Classic/table/boolean-true.png":[11,11,"png","qx","qx/decoration/Classic/table-combined.png",-10,0],"qx/decoration/Classic/table/descending-invert.png":[10,10,"png","qx","qx/decoration/Classic/table-combined.png",-42,0],"qx/decoration/Classic/table/descending.png":[10,10,"png","qx","qx/decoration/Classic/table-combined.png",0,0],"qx/decoration/Classic/table/select-column-order.png":[10,9,"png","qx","qx/decoration/Classic/table-combined.png",-21,0],"qx/decoration/Classic/tree/minus.gif":[19,16,"gif","qx"],"qx/decoration/Classic/tree/plus.gif":[19,16,"gif","qx"],"qx/decoration/Classic/treevirtual/cross.gif":[19,16,"gif","qx"],"qx/decoration/Classic/treevirtual/cross_minus.gif":[19,16,"gif","qx"],"qx/decoration/Classic/treevirtual/cross_plus.gif":[19,16,"gif","qx"],"qx/decoration/Classic/treevirtual/end.gif":[19,16,"gif","qx"],"qx/decoration/Classic/treevirtual/end_minus.gif":[19,16,"gif","qx"],"qx/decoration/Classic/treevirtual/end_plus.gif":[19,16,"gif","qx"],"qx/decoration/Classic/treevirtual/line.gif":[19,16,"gif","qx"],"qx/decoration/Classic/treevirtual/only_minus.gif":[19,16,"gif","qx"],"qx/decoration/Classic/treevirtual/only_plus.gif":[19,16,"gif","qx"],"qx/decoration/Classic/treevirtual/start.gif":[19,16,"gif","qx"],"qx/decoration/Classic/treevirtual/start_minus.gif":[19,16,"gif","qx"],"qx/decoration/Classic/treevirtual/start_plus.gif":[19,16,"gif","qx"],"qx/decoration/Classic/window-captionbar-buttons-combined.gif":[36,9,"gif","qx"],"qx/decoration/Classic/window/close.gif":[10,9,"gif","qx","qx/decoration/Classic/window-captionbar-buttons-combined.gif",0,0],"qx/decoration/Classic/window/maximize.gif":[9,9,"gif","qx","qx/decoration/Classic/window-captionbar-buttons-combined.gif",-10,0],"qx/decoration/Classic/window/minimize.gif":[9,9,"gif","qx","qx/decoration/Classic/window-captionbar-buttons-combined.gif",-19,0],"qx/decoration/Classic/window/restore.gif":[8,9,"gif","qx","qx/decoration/Classic/window-captionbar-buttons-combined.gif",-28,0],"qx/decoration/Indigo/font/JosefinSlab-SemiBold.ttf":"qx","qx/decoration/Indigo/font/JosefinSlab-SemiBold.woff":"qx","qx/decoration/Modern/arrows-combined.png":[87,8,"png","qx"],"qx/decoration/Modern/arrows/down-invert.png":[8,5,"png","qx","qx/decoration/Modern/arrows-combined.png",-74,0],"qx/decoration/Modern/arrows/down-small.png":[5,3,"png","qx","qx/decoration/Modern/arrows-combined.png",-49,0],"qx/decoration/Modern/arrows/down.png":[8,5,"png","qx","qx/decoration/Modern/arrows-combined.png",-20,0],"qx/decoration/Modern/arrows/forward.png":[10,8,"png","qx","qx/decoration/Modern/arrows-combined.png",-59,0],"qx/decoration/Modern/arrows/left.png":[5,8,"png","qx","qx/decoration/Modern/arrows-combined.png",-44,0],"qx/decoration/Modern/arrows/rewind.png":[10,8,"png","qx","qx/decoration/Modern/arrows-combined.png",-10,0],"qx/decoration/Modern/arrows/right-invert.png":[5,8,"png","qx","qx/decoration/Modern/arrows-combined.png",-5,0],"qx/decoration/Modern/arrows/right.png":[5,8,"png","qx","qx/decoration/Modern/arrows-combined.png",-54,0],"qx/decoration/Modern/arrows/up-invert.png":[8,5,"png","qx","qx/decoration/Modern/arrows-combined.png",-28,0],"qx/decoration/Modern/arrows/up-small.png":[5,3,"png","qx","qx/decoration/Modern/arrows-combined.png",-82,0],"qx/decoration/Modern/arrows/up.png":[8,5,"png","qx","qx/decoration/Modern/arrows-combined.png",-36,0],"qx/decoration/Modern/colorselector-combined.gif":[46,11,"gif","qx"],"qx/decoration/Modern/colorselector/brightness-field.png":[19,256,"png","qx"],"qx/decoration/Modern/colorselector/brightness-handle.gif":[35,11,"gif","qx","qx/decoration/Modern/colorselector-combined.gif",0,0],"qx/decoration/Modern/colorselector/huesaturation-field.jpg":[256,256,"jpeg","qx"],"qx/decoration/Modern/colorselector/huesaturation-handle.gif":[11,11,"gif","qx","qx/decoration/Modern/colorselector-combined.gif",-35,0],"qx/decoration/Modern/cursors-combined.gif":[71,20,"gif","qx"],"qx/decoration/Modern/cursors/alias.gif":[19,15,"gif","qx","qx/decoration/Modern/cursors-combined.gif",-52,0],"qx/decoration/Modern/cursors/copy.gif":[19,15,"gif","qx","qx/decoration/Modern/cursors-combined.gif",-33,0],"qx/decoration/Modern/cursors/move.gif":[13,9,"gif","qx","qx/decoration/Modern/cursors-combined.gif",-20,0],"qx/decoration/Modern/cursors/nodrop.gif":[20,20,"gif","qx","qx/decoration/Modern/cursors-combined.gif",0,0],"qx/decoration/Modern/form/checked.png":[6,6,"png","qx"],"qx/decoration/Modern/form/tooltip-error-arrow-right.png":[11,14,"png","qx"],"qx/decoration/Modern/form/tooltip-error-arrow.png":[11,14,"png","qx"],"qx/decoration/Modern/form/undetermined.png":[6,2,"png","qx"],"qx/decoration/Modern/menu-checkradio-combined.gif":[64,7,"gif","qx"],"qx/decoration/Modern/menu/checkbox-invert.gif":[16,7,"gif","qx","qx/decoration/Modern/menu-checkradio-combined.gif",-16,0],"qx/decoration/Modern/menu/checkbox.gif":[16,7,"gif","qx","qx/decoration/Modern/menu-checkradio-combined.gif",-48,0],"qx/decoration/Modern/menu/radiobutton-invert.gif":[16,5,"gif","qx","qx/decoration/Modern/menu-checkradio-combined.gif",-32,0],"qx/decoration/Modern/menu/radiobutton.gif":[16,5,"gif","qx","qx/decoration/Modern/menu-checkradio-combined.gif",0,0],"qx/decoration/Modern/scrollbar-combined.png":[54,12,"png","qx"],"qx/decoration/Modern/scrollbar/scrollbar-down.png":[6,4,"png","qx","qx/decoration/Modern/scrollbar-combined.png",-28,0],"qx/decoration/Modern/scrollbar/scrollbar-left.png":[4,6,"png","qx","qx/decoration/Modern/scrollbar-combined.png",-50,0],"qx/decoration/Modern/scrollbar/scrollbar-right.png":[4,6,"png","qx","qx/decoration/Modern/scrollbar-combined.png",-46,0],"qx/decoration/Modern/scrollbar/scrollbar-up.png":[6,4,"png","qx","qx/decoration/Modern/scrollbar-combined.png",0,0],"qx/decoration/Modern/splitpane-knobs-combined.png":[8,9,"png","qx"],"qx/decoration/Modern/splitpane/knob-horizontal.png":[1,8,"png","qx","qx/decoration/Modern/splitpane-knobs-combined.png",0,-1],"qx/decoration/Modern/splitpane/knob-vertical.png":[8,1,"png","qx","qx/decoration/Modern/splitpane-knobs-combined.png",0,0],"qx/decoration/Modern/table-combined.png":[94,18,"png","qx"],"qx/decoration/Modern/table/ascending.png":[8,5,"png","qx","qx/decoration/Modern/table-combined.png",0,0],"qx/decoration/Modern/table/boolean-false.png":[14,14,"png","qx","qx/decoration/Modern/table-combined.png",-80,0],"qx/decoration/Modern/table/boolean-true.png":[14,14,"png","qx","qx/decoration/Modern/table-combined.png",-26,0],"qx/decoration/Modern/table/descending.png":[8,5,"png","qx","qx/decoration/Modern/table-combined.png",-18,0],"qx/decoration/Modern/table/select-column-order.png":[10,9,"png","qx","qx/decoration/Modern/table-combined.png",-8,0],"qx/decoration/Modern/toolbar/toolbar-handle-knob.gif":[1,8,"gif","qx"],"qx/decoration/Modern/toolbar/toolbar-part.gif":[7,1,"gif","qx"],"qx/decoration/Modern/tree-combined.png":[32,8,"png","qx"],"qx/decoration/Modern/tree/closed-selected.png":[8,8,"png","qx","qx/decoration/Modern/tree-combined.png",-24,0],"qx/decoration/Modern/tree/closed.png":[8,8,"png","qx","qx/decoration/Modern/tree-combined.png",-16,0],"qx/decoration/Modern/tree/open-selected.png":[8,8,"png","qx","qx/decoration/Modern/tree-combined.png",-8,0],"qx/decoration/Modern/tree/open.png":[8,8,"png","qx","qx/decoration/Modern/tree-combined.png",0,0],"qx/decoration/Modern/window-captionbar-buttons-combined.png":[108,9,"png","qx"],"qx/decoration/Modern/window/close-active-hovered.png":[9,9,"png","qx","qx/decoration/Modern/window-captionbar-buttons-combined.png",-27,0],"qx/decoration/Modern/window/close-active.png":[9,9,"png","qx","qx/decoration/Modern/window-captionbar-buttons-combined.png",-9,0],"qx/decoration/Modern/window/close-inactive.png":[9,9,"png","qx","qx/decoration/Modern/window-captionbar-buttons-combined.png",-90,0],"qx/decoration/Modern/window/maximize-active-hovered.png":[9,9,"png","qx","qx/decoration/Modern/window-captionbar-buttons-combined.png",-18,0],"qx/decoration/Modern/window/maximize-active.png":[9,9,"png","qx","qx/decoration/Modern/window-captionbar-buttons-combined.png",-81,0],"qx/decoration/Modern/window/maximize-inactive.png":[9,9,"png","qx","qx/decoration/Modern/window-captionbar-buttons-combined.png",-54,0],"qx/decoration/Modern/window/minimize-active-hovered.png":[9,9,"png","qx","qx/decoration/Modern/window-captionbar-buttons-combined.png",-63,0],"qx/decoration/Modern/window/minimize-active.png":[9,9,"png","qx","qx/decoration/Modern/window-captionbar-buttons-combined.png",-72,0],"qx/decoration/Modern/window/minimize-inactive.png":[9,9,"png","qx","qx/decoration/Modern/window-captionbar-buttons-combined.png",-36,0],"qx/decoration/Modern/window/restore-active-hovered.png":[9,8,"png","qx","qx/decoration/Modern/window-captionbar-buttons-combined.png",0,0],"qx/decoration/Modern/window/restore-active.png":[9,8,"png","qx","qx/decoration/Modern/window-captionbar-buttons-combined.png",-99,0],"qx/decoration/Modern/window/restore-inactive.png":[9,8,"png","qx","qx/decoration/Modern/window-captionbar-buttons-combined.png",-45,0],"qx/decoration/Simple/arrows/down-invert.gif":[7,4,"gif","qx"],"qx/decoration/Simple/arrows/down-small.gif":[5,3,"gif","qx"],"qx/decoration/Simple/arrows/down.gif":[7,4,"gif","qx"],"qx/decoration/Simple/arrows/forward.gif":[8,7,"gif","qx"],"qx/decoration/Simple/arrows/left-invert.gif":[4,7,"gif","qx"],"qx/decoration/Simple/arrows/left.gif":[4,7,"gif","qx"],"qx/decoration/Simple/arrows/rewind.gif":[8,7,"gif","qx"],"qx/decoration/Simple/arrows/right-invert.gif":[4,7,"gif","qx"],"qx/decoration/Simple/arrows/right.gif":[4,7,"gif","qx"],"qx/decoration/Simple/arrows/up-invert.gif":[7,4,"gif","qx"],"qx/decoration/Simple/arrows/up-small.gif":[5,3,"gif","qx"],"qx/decoration/Simple/arrows/up.gif":[7,4,"gif","qx"],"qx/decoration/Simple/checkbox/checked-disabled.png":[6,6,"png","qx"],"qx/decoration/Simple/checkbox/checked.png":[6,6,"png","qx"],"qx/decoration/Simple/checkbox/undetermined-disabled.png":[6,2,"png","qx"],"qx/decoration/Simple/checkbox/undetermined.png":[6,2,"png","qx"],"qx/decoration/Simple/colorselector/brightness-field.png":[19,256,"png","qx"],"qx/decoration/Simple/colorselector/brightness-handle.gif":[35,11,"gif","qx"],"qx/decoration/Simple/colorselector/huesaturation-field.jpg":[256,256,"jpeg","qx"],"qx/decoration/Simple/colorselector/huesaturation-handle.gif":[11,11,"gif","qx"],"qx/decoration/Simple/cursors/alias.gif":[19,15,"gif","qx"],"qx/decoration/Simple/cursors/copy.gif":[19,15,"gif","qx"],"qx/decoration/Simple/cursors/move.gif":[13,9,"gif","qx"],"qx/decoration/Simple/cursors/nodrop.gif":[20,20,"gif","qx"],"qx/decoration/Simple/menu/checkbox-invert.gif":[16,7,"gif","qx"],"qx/decoration/Simple/menu/checkbox.gif":[16,7,"gif","qx"],"qx/decoration/Simple/menu/radiobutton-invert.gif":[16,5,"gif","qx"],"qx/decoration/Simple/menu/radiobutton.gif":[16,5,"gif","qx"],"qx/decoration/Simple/splitpane/knob-horizontal.png":[1,8,"png","qx"],"qx/decoration/Simple/splitpane/knob-vertical.png":[8,1,"png","qx"],"qx/decoration/Simple/table/ascending-invert.png":[10,10,"png","qx"],"qx/decoration/Simple/table/ascending.png":[10,10,"png","qx"],"qx/decoration/Simple/table/boolean-false.png":[11,11,"png","qx"],"qx/decoration/Simple/table/boolean-true.png":[11,11,"png","qx"],"qx/decoration/Simple/table/descending-invert.png":[10,10,"png","qx"],"qx/decoration/Simple/table/descending.png":[10,10,"png","qx"],"qx/decoration/Simple/table/select-column-order.png":[10,9,"png","qx"],"qx/decoration/Simple/tabview/close.gif":[10,9,"gif","qx"],"qx/decoration/Simple/tree/minus.gif":[19,16,"gif","qx"],"qx/decoration/Simple/tree/plus.gif":[19,16,"gif","qx"],"qx/decoration/Simple/treevirtual/cross.gif":[19,16,"gif","qx"],"qx/decoration/Simple/treevirtual/cross_minus.gif":[19,16,"gif","qx"],"qx/decoration/Simple/treevirtual/cross_plus.gif":[19,16,"gif","qx"],"qx/decoration/Simple/treevirtual/end.gif":[19,16,"gif","qx"],"qx/decoration/Simple/treevirtual/end_minus.gif":[19,16,"gif","qx"],"qx/decoration/Simple/treevirtual/end_plus.gif":[19,16,"gif","qx"],"qx/decoration/Simple/treevirtual/line.gif":[19,16,"gif","qx"],"qx/decoration/Simple/treevirtual/only_minus.gif":[19,16,"gif","qx"],"qx/decoration/Simple/treevirtual/only_plus.gif":[19,16,"gif","qx"],"qx/decoration/Simple/treevirtual/start.gif":[19,16,"gif","qx"],"qx/decoration/Simple/treevirtual/start_minus.gif":[19,16,"gif","qx"],"qx/decoration/Simple/treevirtual/start_plus.gif":[19,16,"gif","qx"],"qx/decoration/Simple/window/close-white.gif":[10,9,"gif","qx"],"qx/decoration/Simple/window/close.gif":[10,9,"gif","qx"],"qx/decoration/Simple/window/maximize-white.gif":[9,9,"gif","qx"],"qx/decoration/Simple/window/maximize.gif":[9,9,"gif","qx"],"qx/decoration/Simple/window/minimize-white.gif":[9,9,"gif","qx"],"qx/decoration/Simple/window/minimize.gif":[9,9,"gif","qx"],"qx/decoration/Simple/window/restore-white.gif":[8,9,"gif","qx"],"qx/decoration/Simple/window/restore.gif":[8,9,"gif","qx"],"qx/icon/Oxygen/16/actions/dialog-cancel.png":[16,16,"png","qx"],"qx/icon/Oxygen/16/actions/dialog-ok.png":[16,16,"png","qx"],"qx/icon/Oxygen/16/actions/view-refresh.png":[16,16,"png","qx"],"qx/icon/Oxygen/16/actions/window-close.png":[16,16,"png","qx"],"qx/icon/Oxygen/16/apps/office-calendar.png":[16,16,"png","qx"],"qx/icon/Oxygen/16/mimetypes/text-plain.png":[16,16,"png","qx"],"qx/icon/Oxygen/16/places/folder-open.png":[16,16,"png","qx"],"qx/icon/Oxygen/16/places/folder.png":[16,16,"png","qx"],"qx/icon/Tango/16/actions/dialog-cancel.png":[16,16,"png","qx"],"qx/icon/Tango/16/actions/dialog-ok.png":[16,16,"png","qx"],"qx/icon/Tango/16/actions/view-refresh.png":[16,16,"png","qx"],"qx/icon/Tango/16/actions/window-close.png":[16,16,"png","qx"],"qx/icon/Tango/16/apps/office-calendar.png":[16,16,"png","qx"],"qx/icon/Tango/16/apps/utilities-color-chooser.png":[16,16,"png","qx"],"qx/icon/Tango/16/mimetypes/office-document.png":[16,16,"png","qx"],"qx/icon/Tango/16/mimetypes/text-plain.png":[16,16,"png","qx"],"qx/icon/Tango/16/places/folder-open.png":[16,16,"png","qx"],"qx/icon/Tango/16/places/folder.png":[16,16,"png","qx"],"qx/icon/Tango/22/mimetypes/office-document.png":[22,22,"png","qx"],"qx/icon/Tango/22/places/folder-open.png":[22,22,"png","qx"],"qx/icon/Tango/22/places/folder.png":[22,22,"png","qx"],"qx/icon/Tango/32/mimetypes/office-document.png":[32,32,"png","qx"],"qx/icon/Tango/32/places/folder-open.png":[32,32,"png","qx"],"qx/icon/Tango/32/places/folder.png":[32,32,"png","qx"],"qx/static/blank.gif":[1,1,"gif","qx"],"qx/static/blank.png":[1,1,"png","qx"]},"translations":{"C":{},"en":{}}};
(function(){var b=".prototype",c="function",d="Boolean",e="Error",f="Object.keys requires an object as argument.",g="constructor",h="warn",j="default",k="Null",m="hasOwnProperty",n="Undefined",o="string",p="Object",q="toLocaleString",r="error",s="toString",t="qx.debug",u="()",v="RegExp",w="String",x="info",y="BROKEN_IE",z="isPrototypeOf",A="Date",B="",C="qx.Bootstrap",D="Function",E="]",F="Cannot call super class. Method is not derived: ",G="Array",H="[Class ",I="valueOf",J="Number",K="Class",L="debug",M="ES5",N=".",O="propertyIsEnumerable",P="object";if(!window.qx){window.qx={};};qx.Bootstrap={genericToString:function(){return H+this.classname+E;},createNamespace:function(name,Q){var T=name.split(N);var S=T[0];var parent=qx.$$namespaceRoot&&qx.$$namespaceRoot[S]?qx.$$namespaceRoot:window;for(var i=0,R=T.length-1;i<R;i++ ,S=T[i]){if(!parent[S]){parent=parent[S]={};}else {parent=parent[S];};};parent[S]=Q;return S;},setDisplayName:function(V,U,name){V.displayName=U+N+name+u;},setDisplayNames:function(X,W){for(var name in X){var Y=X[name];if(Y instanceof Function){Y.displayName=W+N+name+u;};};},base:function(ba,bb){if(qx.Bootstrap.DEBUG){if(!qx.Bootstrap.isFunction(ba.callee.base)){throw new Error(F+ba.callee.displayName);};};if(arguments.length===1){return ba.callee.base.call(this);}else {return ba.callee.base.apply(this,Array.prototype.slice.call(arguments,1));};},define:function(name,bm){if(!bm){bm={statics:{}};};var bi;var be=null;qx.Bootstrap.setDisplayNames(bm.statics,name);if(bm.members||bm.extend){qx.Bootstrap.setDisplayNames(bm.members,name+b);bi=bm.construct||new Function;if(bm.extend){this.extendClass(bi,bi,bm.extend,name,bg);};var bd=bm.statics||{};for(var i=0,bf=qx.Bootstrap.keys(bd),l=bf.length;i<l;i++ ){var bc=bf[i];bi[bc]=bd[bc];};be=bi.prototype;be.base=qx.Bootstrap.base;be.name=be.classname=name;var bk=bm.members||{};var bc,bj;for(var i=0,bf=qx.Bootstrap.keys(bk),l=bf.length;i<l;i++ ){bc=bf[i];bj=bk[bc];if(bj instanceof Function&&be[bc]){bj.base=be[bc];};be[bc]=bj;};}else {bi=bm.statics||{};if(qx.Bootstrap.$$registry&&qx.Bootstrap.$$registry[name]){var bl=qx.Bootstrap.$$registry[name];if(this.keys(bi).length!==0){if(bm.defer){bm.defer(bi,be);};for(var bh in bi){bl[bh]=bi[bh];};return bl;};};};bi.$$type=K;if(!bi.hasOwnProperty(s)){bi.toString=this.genericToString;};var bg=name?this.createNamespace(name,bi):B;bi.name=bi.classname=name;bi.basename=bg;bi.$$events=bm.events;if(bm.defer){bm.defer(bi,be);};if(name!=null){qx.Bootstrap.$$registry[name]=bi;};return bi;}};qx.Bootstrap.define(C,{statics:{LOADSTART:qx.$$start||new Date(),DEBUG:(function(){var bn=true;if(qx.$$environment&&qx.$$environment[t]===false){bn=false;};return bn;})(),getEnvironmentSetting:function(bo){if(qx.$$environment){return qx.$$environment[bo];};},setEnvironmentSetting:function(bp,bq){if(!qx.$$environment){qx.$$environment={};};if(qx.$$environment[bp]===undefined){qx.$$environment[bp]=bq;};},createNamespace:qx.Bootstrap.createNamespace,setRoot:function(br){qx.$$namespaceRoot=br;},base:qx.Bootstrap.base,define:qx.Bootstrap.define,setDisplayName:qx.Bootstrap.setDisplayName,setDisplayNames:qx.Bootstrap.setDisplayNames,genericToString:qx.Bootstrap.genericToString,extendClass:function(clazz,construct,superClass,name,basename){var superproto=superClass.prototype;var helper=new Function();helper.prototype=superproto;var proto=new helper();clazz.prototype=proto;proto.name=proto.classname=name;proto.basename=basename;construct.base=superClass;clazz.superclass=superClass;construct.self=clazz.constructor=proto.constructor=clazz;},getByName:function(name){return qx.Bootstrap.$$registry[name];},$$registry:{},objectGetLength:function(bs){return qx.Bootstrap.keys(bs).length;},objectMergeWith:function(bu,bt,bw){if(bw===undefined){bw=true;};for(var bv in bt){if(bw||bu[bv]===undefined){bu[bv]=bt[bv];};};return bu;},__a:[z,m,q,s,I,O,g],keys:({"ES5":Object.keys,"BROKEN_IE":function(bx){if(bx===null||(typeof bx!=P&&typeof bx!=c)){throw new TypeError(f);};var by=[];var bA=Object.prototype.hasOwnProperty;for(var bB in bx){if(bA.call(bx,bB)){by.push(bB);};};var bz=qx.Bootstrap.__a;for(var i=0,a=bz,l=a.length;i<l;i++ ){if(bA.call(bx,a[i])){by.push(a[i]);};};return by;},"default":function(bC){if(bC===null||(typeof bC!=P&&typeof bC!=c)){throw new TypeError(f);};var bD=[];var bE=Object.prototype.hasOwnProperty;for(var bF in bC){if(bE.call(bC,bF)){bD.push(bF);};};return bD;}})[typeof (Object.keys)==c?M:(function(){for(var bG in {toString:1}){return bG;};})()!==s?y:j],__b:{"[object String]":w,"[object Array]":G,"[object Object]":p,"[object RegExp]":v,"[object Number]":J,"[object Boolean]":d,"[object Date]":A,"[object Function]":D,"[object Error]":e},bind:function(bI,self,bJ){var bH=Array.prototype.slice.call(arguments,2,arguments.length);return function(){var bK=Array.prototype.slice.call(arguments,0,arguments.length);return bI.apply(self,bH.concat(bK));};},firstUp:function(bL){return bL.charAt(0).toUpperCase()+bL.substr(1);},firstLow:function(bM){return bM.charAt(0).toLowerCase()+bM.substr(1);},getClass:function(bO){if(bO===undefined){return n;}else if(bO===null){return k;};var bN=Object.prototype.toString.call(bO);return (qx.Bootstrap.__b[bN]||bN.slice(8,-1));},isString:function(bP){return (bP!==null&&(typeof bP===o||qx.Bootstrap.getClass(bP)==w||bP instanceof String||(!!bP&&!!bP.$$isString)));},isArray:function(bQ){return (bQ!==null&&(bQ instanceof Array||(bQ&&qx.data&&qx.data.IListData&&qx.util.OOUtil.hasInterface(bQ.constructor,qx.data.IListData))||qx.Bootstrap.getClass(bQ)==G||(!!bQ&&!!bQ.$$isArray)));},isObject:function(bR){return (bR!==undefined&&bR!==null&&qx.Bootstrap.getClass(bR)==p);},isFunction:function(bS){return qx.Bootstrap.getClass(bS)==D;},$$logs:[],debug:function(bU,bT){qx.Bootstrap.$$logs.push([L,arguments]);},info:function(bW,bV){qx.Bootstrap.$$logs.push([x,arguments]);},warn:function(bY,bX){qx.Bootstrap.$$logs.push([h,arguments]);},error:function(cb,ca){qx.Bootstrap.$$logs.push([r,arguments]);},trace:function(cc){}}});})();(function(){var a="qx.util.OOUtil";qx.Bootstrap.define(a,{statics:{classIsDefined:function(name){return qx.Bootstrap.getByName(name)!==undefined;},getPropertyDefinition:function(b,name){while(b){if(b.$$properties&&b.$$properties[name]){return b.$$properties[name];};b=b.superclass;};return null;},hasProperty:function(c,name){return !!qx.util.OOUtil.getPropertyDefinition(c,name);},getEventType:function(d,name){var d=d.constructor;while(d.superclass){if(d.$$events&&d.$$events[name]!==undefined){return d.$$events[name];};d=d.superclass;};return null;},supportsEvent:function(e,name){return !!qx.util.OOUtil.getEventType(e,name);},getByInterface:function(h,f){var g,i,l;while(h){if(h.$$implements){g=h.$$flatImplements;for(i=0,l=g.length;i<l;i++ ){if(g[i]===f){return h;};};};h=h.superclass;};return null;},hasInterface:function(k,j){return !!qx.util.OOUtil.getByInterface(k,j);},getMixins:function(n){var m=[];while(n){if(n.$$includes){m.push.apply(m,n.$$flatIncludes);};n=n.superclass;};return m;}}});})();(function(){var a="qx.core.Environment",b="default",c=' type)',d="&",e="qx/static/blank.html",f="true",g="|",h="qx.core.Environment for a list of predefined keys.",j="false",k='] found, and no default ("default") given',l=":",m='" (',n=' in variants [',o=".",p="qx.allowUrlSettings",q='No match for variant "',r=" is not a valid key. Please see the API-doc of ",s="qxenv";qx.Bootstrap.define(a,{statics:{_checks:{},_asyncChecks:{},__c:{},_checksMap:{},_defaults:{"true":true,"qx.allowUrlSettings":false,"qx.allowUrlVariants":false,"qx.debug.property.level":0,"qx.debug":true,"qx.debug.ui.queue":true,"qx.aspects":false,"qx.dynlocale":true,"qx.dyntheme":true,"qx.blankpage":e,"qx.debug.databinding":false,"qx.debug.dispose":false,"qx.optimization.basecalls":false,"qx.optimization.comments":false,"qx.optimization.privates":false,"qx.optimization.strings":false,"qx.optimization.variables":false,"qx.optimization.variants":false,"module.databinding":true,"module.logger":true,"module.property":true,"module.events":true,"qx.nativeScrollBars":false},get:function(w){if(this.__c[w]!=undefined){return this.__c[w];};var y=this._checks[w];if(y){var u=y();this.__c[w]=u;return u;};var t=this._getClassNameFromEnvKey(w);if(t[0]!=undefined){var x=t[0];var v=t[1];var u=x[v]();this.__c[w]=u;return u;};if(qx.Bootstrap.DEBUG){qx.Bootstrap.warn(w+r+h);qx.Bootstrap.trace(this);};},_getClassNameFromEnvKey:function(D){var F=this._checksMap;if(F[D]!=undefined){var A=F[D];var E=A.lastIndexOf(o);if(E>-1){var C=A.slice(0,E);var z=A.slice(E+1);var B=qx.Bootstrap.getByName(C);if(B!=undefined){return [B,z];};};};return [undefined,undefined];},getAsync:function(H,K,self){var L=this;if(this.__c[H]!=undefined){window.setTimeout(function(){K.call(self,L.__c[H]);},0);return;};var I=this._asyncChecks[H];if(I){I(function(N){L.__c[H]=N;K.call(self,N);});return;};var G=this._getClassNameFromEnvKey(H);if(G[0]!=undefined){var J=G[0];var M=G[1];J[M](function(O){L.__c[H]=O;K.call(self,O);});return;};if(qx.Bootstrap.DEBUG){qx.Bootstrap.warn(H+r+h);qx.Bootstrap.trace(this);};},select:function(Q,P){return this.__d(this.get(Q),P);},selectAsync:function(S,R,self){this.getAsync(S,function(T){var U=this.__d(S,R);U.call(self,T);},this);},__d:function(Y,X){var W=X[Y];if(X.hasOwnProperty(Y)){return W;};for(var ba in X){if(ba.indexOf(g)!=-1){var V=ba.split(g);for(var i=0;i<V.length;i++ ){if(V[i]==Y){return X[ba];};};};};if(X[b]!==undefined){return X[b];};if(qx.Bootstrap.DEBUG){throw new Error(q+Y+m+(typeof Y)+c+n+qx.Bootstrap.keys(X)+k);};},filter:function(bb){var bd=[];for(var bc in bb){if(this.get(bc)){bd.push(bb[bc]);};};return bd;},invalidateCacheKey:function(be){delete this.__c[be];},add:function(bg,bf){if(this._checks[bg]==undefined){if(bf instanceof Function){if(!this._checksMap[bg]&&bf.displayName){this._checksMap[bg]=bf.displayName.substr(0,bf.displayName.length-2);};this._checks[bg]=bf;}else {this._checks[bg]=this.__g(bf);};};},addAsync:function(bi,bh){if(this._checks[bi]==undefined){this._asyncChecks[bi]=bh;};},getChecks:function(){return this._checks;},getAsyncChecks:function(){return this._asyncChecks;},_initDefaultQxValues:function(){var bj=function(bl){return function(){return bl;};};for(var bk in this._defaults){this.add(bk,bj(this._defaults[bk]));};},__e:function(){if(qx&&qx.$$environment){for(var bm in qx.$$environment){var bn=qx.$$environment[bm];this._checks[bm]=this.__g(bn);};};},__f:function(){if(window.document&&window.document.location){var bo=window.document.location.search.slice(1).split(d);for(var i=0;i<bo.length;i++ ){var br=bo[i].split(l);if(br.length!=3||br[0]!=s){continue;};var bp=br[1];var bq=decodeURIComponent(br[2]);if(bq==f){bq=true;}else if(bq==j){bq=false;}else if(/^(\d|\.)+$/.test(bq)){bq=parseFloat(bq);};this._checks[bp]=this.__g(bq);};};},__g:function(bs){return qx.Bootstrap.bind(function(bt){return bt;},null,bs);}},defer:function(bu){bu._initDefaultQxValues();bu.__e();if(bu.get(p)===true){bu.__f();};}});})();(function(){var a="ecmascript.array.lastindexof",b="function",c="stack",d="ecmascript.array.map",f="ecmascript.date.now",g="ecmascript.array.reduce",h="e",i="qx.bom.client.EcmaScript",j="ecmascript.object.keys",k="ecmascript.error.stacktrace",l="ecmascript.string.trim",m="ecmascript.array.indexof",n="stacktrace",o="ecmascript.error.toString",p="[object Error]",q="ecmascript.array.foreach",r="ecmascript.function.bind",s="ecmascript.array.reduceright",t="ecmascript.array.some",u="ecmascript.array.filter",v="ecmascript.array.every";qx.Bootstrap.define(i,{statics:{getStackTrace:function(){var w;var e=new Error(h);w=e.stack?c:e.stacktrace?n:null;if(!w){try{throw e;}catch(x){e=x;};};return e.stacktrace?n:e.stack?c:null;},getArrayIndexOf:function(){return !!Array.prototype.indexOf;},getArrayLastIndexOf:function(){return !!Array.prototype.lastIndexOf;},getArrayForEach:function(){return !!Array.prototype.forEach;},getArrayFilter:function(){return !!Array.prototype.filter;},getArrayMap:function(){return !!Array.prototype.map;},getArraySome:function(){return !!Array.prototype.some;},getArrayEvery:function(){return !!Array.prototype.every;},getArrayReduce:function(){return !!Array.prototype.reduce;},getArrayReduceRight:function(){return !!Array.prototype.reduceRight;},getErrorToString:function(){return typeof Error.prototype.toString==b&&Error.prototype.toString()!==p;},getFunctionBind:function(){return typeof Function.prototype.bind===b;},getObjectKeys:function(){return !!Object.keys;},getDateNow:function(){return !!Date.now;},getStringTrim:function(){return typeof String.prototype.trim===b;}},defer:function(y){qx.core.Environment.add(m,y.getArrayIndexOf);qx.core.Environment.add(a,y.getArrayLastIndexOf);qx.core.Environment.add(q,y.getArrayForEach);qx.core.Environment.add(u,y.getArrayFilter);qx.core.Environment.add(d,y.getArrayMap);qx.core.Environment.add(t,y.getArraySome);qx.core.Environment.add(v,y.getArrayEvery);qx.core.Environment.add(g,y.getArrayReduce);qx.core.Environment.add(s,y.getArrayReduceRight);qx.core.Environment.add(f,y.getDateNow);qx.core.Environment.add(o,y.getErrorToString);qx.core.Environment.add(k,y.getStackTrace);qx.core.Environment.add(r,y.getFunctionBind);qx.core.Environment.add(j,y.getObjectKeys);qx.core.Environment.add(l,y.getStringTrim);}});})();(function(){var a="qx.lang.normalize.Function",b="ecmascript.function.bind",c="function",d="Function.prototype.bind called on incompatible ";qx.Bootstrap.define(a,{statics:{bind:function(i){var e=Array.prototype.slice;var h=this;if(typeof h!=c){throw new TypeError(d+h);};var f=e.call(arguments,1);var g=function(){if(this instanceof g){var F=function(){};F.prototype=h.prototype;var self=new F;var j=h.apply(self,f.concat(e.call(arguments)));if(Object(j)===j){return j;};return self;}else {return h.apply(i,f.concat(e.call(arguments)));};};return g;}},defer:function(k){if(!qx.core.Environment.get(b)){Function.prototype.bind=k.bind;};}});})();(function(){var a="function",b="ecmascript.array.lastindexof",c="ecmascript.array.map",d="ecmascript.array.filter",e="Length is 0 and no second argument given",f="qx.lang.normalize.Array",g="ecmascript.array.indexof",h="First argument is not callable",j="ecmascript.array.reduce",k="ecmascript.array.foreach",m="ecmascript.array.reduceright",n="ecmascript.array.some",o="ecmascript.array.every";qx.Bootstrap.define(f,{statics:{indexOf:function(p,q){if(q==null){q=0;}else if(q<0){q=Math.max(0,this.length+q);};for(var i=q;i<this.length;i++ ){if(this[i]===p){return i;};};return -1;},lastIndexOf:function(r,s){if(s==null){s=this.length-1;}else if(s<0){s=Math.max(0,this.length+s);};for(var i=s;i>=0;i-- ){if(this[i]===r){return i;};};return -1;},forEach:function(t,u){var l=this.length;for(var i=0;i<l;i++ ){var v=this[i];if(v!==undefined){t.call(u||window,v,i,this);};};},filter:function(z,w){var x=[];var l=this.length;for(var i=0;i<l;i++ ){var y=this[i];if(y!==undefined){if(z.call(w||window,y,i,this)){x.push(this[i]);};};};return x;},map:function(D,A){var B=[];var l=this.length;for(var i=0;i<l;i++ ){var C=this[i];if(C!==undefined){B[i]=D.call(A||window,C,i,this);};};return B;},some:function(E,F){var l=this.length;for(var i=0;i<l;i++ ){var G=this[i];if(G!==undefined){if(E.call(F||window,G,i,this)){return true;};};};return false;},every:function(H,I){var l=this.length;for(var i=0;i<l;i++ ){var J=this[i];if(J!==undefined){if(!H.call(I||window,J,i,this)){return false;};};};return true;},reduce:function(K,L){if(typeof K!==a){throw new TypeError(h);};if(L===undefined&&this.length===0){throw new TypeError(e);};var M=L===undefined?this[0]:L;for(var i=L===undefined?1:0;i<this.length;i++ ){if(i in this){M=K.call(undefined,M,this[i],i,this);};};return M;},reduceRight:function(N,O){if(typeof N!==a){throw new TypeError(h);};if(O===undefined&&this.length===0){throw new TypeError(e);};var P=O===undefined?this[this.length-1]:O;for(var i=O===undefined?this.length-2:this.length-1;i>=0;i-- ){if(i in this){P=N.call(undefined,P,this[i],i,this);};};return P;}},defer:function(Q){if(!qx.core.Environment.get(g)){Array.prototype.indexOf=Q.indexOf;};if(!qx.core.Environment.get(b)){Array.prototype.lastIndexOf=Q.lastIndexOf;};if(!qx.core.Environment.get(k)){Array.prototype.forEach=Q.forEach;};if(!qx.core.Environment.get(d)){Array.prototype.filter=Q.filter;};if(!qx.core.Environment.get(c)){Array.prototype.map=Q.map;};if(!qx.core.Environment.get(n)){Array.prototype.some=Q.some;};if(!qx.core.Environment.get(o)){Array.prototype.every=Q.every;};if(!qx.core.Environment.get(j)){Array.prototype.reduce=Q.reduce;};if(!qx.core.Environment.get(m)){Array.prototype.reduceRight=Q.reduceRight;};}});})();(function(){var a="qx.Mixin",b=".prototype",c="]",d='Conflict between mixin "',e="constructor",f="Array",g='"!',h='" and "',j="destruct",k='" in property "',m="Mixin",n='" in member "',o="[Mixin ";qx.Bootstrap.define(a,{statics:{define:function(name,q){if(q){if(q.include&&!(qx.Bootstrap.getClass(q.include)===f)){q.include=[q.include];};{};var r=q.statics?q.statics:{};qx.Bootstrap.setDisplayNames(r,name);for(var p in r){if(r[p] instanceof Function){r[p].$$mixin=r;};};if(q.construct){r.$$constructor=q.construct;qx.Bootstrap.setDisplayName(q.construct,name,e);};if(q.include){r.$$includes=q.include;};if(q.properties){r.$$properties=q.properties;};if(q.members){r.$$members=q.members;qx.Bootstrap.setDisplayNames(q.members,name+b);};for(var p in r.$$members){if(r.$$members[p] instanceof Function){r.$$members[p].$$mixin=r;};};if(q.events){r.$$events=q.events;};if(q.destruct){r.$$destructor=q.destruct;qx.Bootstrap.setDisplayName(q.destruct,name,j);};}else {var r={};};r.$$type=m;r.name=name;r.toString=this.genericToString;r.basename=qx.Bootstrap.createNamespace(name,r);this.$$registry[name]=r;return r;},checkCompatibility:function(t){var u=this.flatten(t);var v=u.length;if(v<2){return true;};var w={};var x={};var z={};var y;for(var i=0;i<v;i++ ){y=u[i];for(var s in y.events){if(z[s]){throw new Error(d+y.name+h+z[s]+n+s+g);};z[s]=y.name;};for(var s in y.properties){if(w[s]){throw new Error(d+y.name+h+w[s]+k+s+g);};w[s]=y.name;};for(var s in y.members){if(x[s]){throw new Error(d+y.name+h+x[s]+n+s+g);};x[s]=y.name;};};return true;},isCompatible:function(B,C){var A=qx.util.OOUtil.getMixins(C);A.push(B);return qx.Mixin.checkCompatibility(A);},getByName:function(name){return this.$$registry[name];},isDefined:function(name){return this.getByName(name)!==undefined;},getTotalNumber:function(){return qx.Bootstrap.objectGetLength(this.$$registry);},flatten:function(D){if(!D){return [];};var E=D.concat();for(var i=0,l=D.length;i<l;i++ ){if(D[i].$$includes){E.push.apply(E,this.flatten(D[i].$$includes));};};return E;},genericToString:function(){return o+this.name+c;},$$registry:{},__h:null,__i:function(name,F){}}});})();(function(){var a="qx.core.Aspect",b="before",c="*",d="static";qx.Bootstrap.define(a,{statics:{__j:[],wrap:function(h,l,j){var m=[];var e=[];var k=this.__j;var g;for(var i=0;i<k.length;i++ ){g=k[i];if((g.type==null||j==g.type||g.type==c)&&(g.name==null||h.match(g.name))){g.pos==-1?m.push(g.fcn):e.push(g.fcn);};};if(m.length===0&&e.length===0){return l;};var f=function(){for(var i=0;i<m.length;i++ ){m[i].call(this,h,l,j,arguments);};var n=l.apply(this,arguments);for(var i=0;i<e.length;i++ ){e[i].call(this,h,l,j,arguments,n);};return n;};if(j!==d){f.self=l.self;f.base=l.base;};l.wrapper=f;f.original=l;return f;},addAdvice:function(q,o,p,name){this.__j.push({fcn:q,pos:o===b?-1:1,type:p,name:name});}}});})();(function(){var a='',b="ecmascript.string.trim",c="qx.lang.normalize.String";qx.Bootstrap.define(c,{statics:{trim:function(){return this.replace(/^\s+|\s+$/g,a);}},defer:function(d){if(!qx.core.Environment.get(b)){String.prototype.trim=d.trim;};}});})();(function(){var a="ecmascript.object.keys",b="qx.lang.normalize.Object";qx.Bootstrap.define(b,{statics:{keys:qx.Bootstrap.keys},defer:function(c){if(!qx.core.Environment.get(a)){Object.keys=c.keys;};}});})();(function(){var a='Implementation of method "',b='"',c="function",d='" is not supported by Class "',e="Boolean",f="qx.Interface",g='The event "',h='" required by interface "',j='" is missing in class "',k='"!',m='The property "',n="Interface",o="toggle",p="]",q="[Interface ",r="is",s="Array",t='Implementation of member "';qx.Bootstrap.define(f,{statics:{define:function(name,v){if(v){if(v.extend&&!(qx.Bootstrap.getClass(v.extend)===s)){v.extend=[v.extend];};{};var u=v.statics?v.statics:{};if(v.extend){u.$$extends=v.extend;};if(v.properties){u.$$properties=v.properties;};if(v.members){u.$$members=v.members;};if(v.events){u.$$events=v.events;};}else {var u={};};u.$$type=n;u.name=name;u.toString=this.genericToString;u.basename=qx.Bootstrap.createNamespace(name,u);qx.Interface.$$registry[name]=u;return u;},getByName:function(name){return this.$$registry[name];},isDefined:function(name){return this.getByName(name)!==undefined;},getTotalNumber:function(){return qx.Bootstrap.objectGetLength(this.$$registry);},flatten:function(x){if(!x){return [];};var w=x.concat();for(var i=0,l=x.length;i<l;i++ ){if(x[i].$$extends){w.push.apply(w,this.flatten(x[i].$$extends));};};return w;},__k:function(B,C,y,F,D){var z=y.$$members;if(z){for(var E in z){if(qx.Bootstrap.isFunction(z[E])){var H=this.__l(C,E);var A=H||qx.Bootstrap.isFunction(B[E]);if(!A){if(D){throw new Error(a+E+j+C.classname+h+y.name+b);}else {return false;};};var G=F===true&&!H&&!qx.util.OOUtil.hasInterface(C,y);if(G){B[E]=this.__o(y,B[E],E,z[E]);};}else {if(typeof B[E]===undefined){if(typeof B[E]!==c){if(D){throw new Error(t+E+j+C.classname+h+y.name+b);}else {return false;};};};};};};if(!D){return true;};},__l:function(L,I){var N=I.match(/^(is|toggle|get|set|reset)(.*)$/);if(!N){return false;};var K=qx.Bootstrap.firstLow(N[2]);var M=qx.util.OOUtil.getPropertyDefinition(L,K);if(!M){return false;};var J=N[0]==r||N[0]==o;if(J){return qx.util.OOUtil.getPropertyDefinition(L,K).check==e;};return true;},__m:function(R,O,P){if(O.$$properties){for(var Q in O.$$properties){if(!qx.util.OOUtil.getPropertyDefinition(R,Q)){if(P){throw new Error(m+Q+d+R.classname+k);}else {return false;};};};};if(!P){return true;};},__n:function(V,S,T){if(S.$$events){for(var U in S.$$events){if(!qx.util.OOUtil.supportsEvent(V,U)){if(T){throw new Error(g+U+d+V.classname+k);}else {return false;};};};};if(!T){return true;};},assertObject:function(Y,W){var ba=Y.constructor;this.__k(Y,ba,W,false,true);this.__m(ba,W,true);this.__n(ba,W,true);var X=W.$$extends;if(X){for(var i=0,l=X.length;i<l;i++ ){this.assertObject(Y,X[i]);};};},assert:function(bd,bb,be){this.__k(bd.prototype,bd,bb,be,true);this.__m(bd,bb,true);this.__n(bd,bb,true);var bc=bb.$$extends;if(bc){for(var i=0,l=bc.length;i<l;i++ ){this.assert(bd,bc[i],be);};};},objectImplements:function(bh,bf){var bi=bh.constructor;if(!this.__k(bh,bi,bf)||!this.__m(bi,bf)||!this.__n(bi,bf)){return false;};var bg=bf.$$extends;if(bg){for(var i=0,l=bg.length;i<l;i++ ){if(!this.objectImplements(bh,bg[i])){return false;};};};return true;},classImplements:function(bl,bj){if(!this.__k(bl.prototype,bl,bj)||!this.__m(bl,bj)||!this.__n(bl,bj)){return false;};var bk=bj.$$extends;if(bk){for(var i=0,l=bk.length;i<l;i++ ){if(!this.has(bl,bk[i])){return false;};};};return true;},genericToString:function(){return q+this.name+p;},$$registry:{},__o:function(bo,bn,bp,bm){},__h:null,__i:function(name,bq){}}});})();(function(){var a="ecmascript.error.toString",b="qx.lang.normalize.Error",c=": ",d="Error",e="";qx.Bootstrap.define(b,{statics:{toString:function(){var name=this.name||d;var f=this.message||e;if(name===e&&f===e){return d;};if(name===e){return f;};if(f===e){return name;};return name+c+f;}},defer:function(g){if(!qx.core.Environment.get(a)){Error.prototype.toString=g.toString;};}});})();(function(){var a="qx.lang.normalize.Date",b="ecmascript.date.now";qx.Bootstrap.define(a,{statics:{now:function(){return +new Date();}},defer:function(c){if(!qx.core.Environment.get(b)){Date.now=c.now;};}});})();(function(){var b='!==inherit){',c='qx.lang.Type.isString(value) && qx.util.ColorUtil.isValidPropertyValue(value)',d='value !== null && qx.theme.manager.Font.getInstance().isDynamic(value)',e="set",f=';',g="resetThemed",h='value !== null && value.nodeType === 9 && value.documentElement',j='===value)return value;',k='value !== null && value.$$type === "Mixin"',m='return init;',n='var init=this.',o='value !== null && value.nodeType === 1 && value.attributes',p="var parent = this.getLayoutParent();",q="Error in property ",r='var a=this._getChildren();if(a)for(var i=0,l=a.length;i<l;i++){',s="property",t="();",u='.validate.call(this, value);',v='qx.core.Assert.assertInstance(value, Date, msg) || true',w='else{',x="if (!parent) return;",y=" in method ",z='qx.core.Assert.assertInstance(value, Error, msg) || true',A='=computed;',B='Undefined value is not allowed!',C='(backup);',D='else ',E='=true;',F='if(old===undefined)old=this.',G='if(computed===inherit){',H='old=computed=this.',I="inherit",J='if(this.',K='return this.',L='else if(this.',M='Is invalid!',N='if(value===undefined)prop.error(this,2,"',O='", "',P='var computed, old=this.',Q='else if(computed===undefined)',R='delete this.',S="resetRuntime",T="': ",U=" of class ",V='value !== null && value.nodeType !== undefined',W='===undefined)return;',X='value !== null && qx.theme.manager.Decoration.getInstance().isValidPropertyValue(value)',Y="reset",ba="string",bb="')){",bc="module.events",bd="return this.",be='qx.core.Assert.assertPositiveInteger(value, msg) || true',bf='else this.',bg='value=this.',bh='","',bi='if(init==qx.core.Property.$$inherit)init=null;',bj="get",bk='value !== null && value.$$type === "Interface"',bl='var inherit=prop.$$inherit;',bm="', qx.event.type.Data, [computed, old]",bn="var value = parent.",bo="$$useinit_",bp='computed=undefined;delete this.',bq="(value);",br='this.',bs='Requires exactly one argument!',bt='",value);',bu='computed=value;',bv='}else{',bw="$$runtime_",bx="setThemed",by=';}',bz='(value);',bA="$$user_",bB='!==undefined)',bC='){',bD='qx.core.Assert.assertArray(value, msg) || true',bE='if(computed===undefined||computed===inherit){',bF=";",bG='qx.core.Assert.assertPositiveNumber(value, msg) || true',bH=".prototype",bI="Boolean",bJ=")}",bK="(a[",bL='(computed, old, "',bM="setRuntime",bN='return value;',bO="this.",bP='if(init==qx.core.Property.$$inherit)throw new Error("Inheritable property ',bQ="if(reg.hasListener(this, '",bR='Does not allow any arguments!',bS=')a[i].',bT="()",bU="var a=arguments[0] instanceof Array?arguments[0]:arguments;",bV='.$$properties.',bW='value !== null && value.$$type === "Theme"',bX='old=this.',bY="var reg=qx.event.Registration;",ca="())",cb='=value;',cc='return null;',cd='qx.core.Assert.assertObject(value, msg) || true',ce='");',cf='if(old===computed)return value;',cg='qx.core.Assert.assertString(value, msg) || true',ch='if(old===undefined)old=null;',ci='var pa=this.getLayoutParent();if(pa)computed=pa.',cj="if (value===undefined) value = parent.",ck='value !== null && value.$$type === "Class"',cl='qx.core.Assert.assertFunction(value, msg) || true',cm='!==undefined&&',cn='var computed, old;',co='var backup=computed;',cp=".",cq='}',cr="object",cs="$$init_",ct="$$theme_",cu='!==undefined){',cv='if(computed===undefined)computed=null;',cw="Unknown reason: ",cx="init",cy='qx.core.Assert.assertMap(value, msg) || true',cz="qx.aspects",cA='qx.core.Assert.assertNumber(value, msg) || true',cB='if((computed===undefined||computed===inherit)&&',cC="reg.fireEvent(this, '",cD='Null value is not allowed!',cE='qx.core.Assert.assertInteger(value, msg) || true',cF="value",cG="shorthand",cH='computed=this.',cI='qx.core.Assert.assertInstance(value, RegExp, msg) || true',cJ='value !== null && value.type !== undefined',cK='value !== null && value.document',cL="",cM='throw new Error("Property ',cN="(!this.",cO='qx.core.Assert.assertBoolean(value, msg) || true',cP='if(a[i].',cQ=' of an instance of ',cR="toggle",cS="refresh",cT="$$inherit_",cU='var prop=qx.core.Property;',cV="boolean",cW=" with incoming value '",cX="a=qx.lang.Array.fromShortHand(qx.lang.Array.fromArguments(a));",cY='if(computed===undefined||computed==inherit)computed=null;',da="qx.core.Property",db="is",dc=' is not (yet) ready!");',dd="]);",de='Could not change or apply init value after constructing phase!';qx.Bootstrap.define(da,{statics:{__p:function(){if(qx.core.Environment.get(bc)){qx.event.type.Data;qx.event.dispatch.Direct;};},__q:{"Boolean":cO,"String":cg,"Number":cA,"Integer":cE,"PositiveNumber":bG,"PositiveInteger":be,"Error":z,"RegExp":cI,"Object":cd,"Array":bD,"Map":cy,"Function":cl,"Date":v,"Node":V,"Element":o,"Document":h,"Window":cK,"Event":cJ,"Class":ck,"Mixin":k,"Interface":bk,"Theme":bW,"Color":c,"Decorator":X,"Font":d},__r:{"Node":true,"Element":true,"Document":true,"Window":true,"Event":true},$$inherit:I,$$store:{runtime:{},user:{},theme:{},inherit:{},init:{},useinit:{}},$$method:{get:{},set:{},reset:{},init:{},refresh:{},setRuntime:{},resetRuntime:{},setThemed:{},resetThemed:{}},$$allowedKeys:{name:ba,dereference:cV,inheritable:cV,nullable:cV,themeable:cV,refine:cV,init:null,apply:ba,event:ba,check:null,transform:ba,deferredInit:cV,validate:null},$$allowedGroupKeys:{name:ba,group:cr,mode:ba,themeable:cV},$$inheritable:{},__s:function(dh){var df=this.__t(dh);if(!df.length){var dg=function(){};}else {dg=this.__u(df);};dh.prototype.$$refreshInheritables=dg;},__t:function(di){var dj=[];while(di){var dk=di.$$properties;if(dk){for(var name in this.$$inheritable){if(dk[name]&&dk[name].inheritable){dj.push(name);};};};di=di.superclass;};return dj;},__u:function(inheritables){var inherit=this.$$store.inherit;var init=this.$$store.init;var refresh=this.$$method.refresh;var code=[p,x];for(var i=0,l=inheritables.length;i<l;i++ ){var name=inheritables[i];code.push(bn,inherit[name],bF,cj,init[name],bF,bO,refresh[name],bq);};return new Function(code.join(cL));},attachRefreshInheritables:function(dl){dl.prototype.$$refreshInheritables=function(){qx.core.Property.__s(dl);return this.$$refreshInheritables();};},attachMethods:function(dn,name,dm){dm.group?this.__v(dn,dm,name):this.__w(dn,dm,name);},__v:function(clazz,config,name){var upname=qx.Bootstrap.firstUp(name);var members=clazz.prototype;var themeable=config.themeable===true;{};var setter=[];var resetter=[];if(themeable){var styler=[];var unstyler=[];};var argHandler=bU;setter.push(argHandler);if(themeable){styler.push(argHandler);};if(config.mode==cG){var shorthand=cX;setter.push(shorthand);if(themeable){styler.push(shorthand);};};for(var i=0,a=config.group,l=a.length;i<l;i++ ){{};setter.push(bO,this.$$method.set[a[i]],bK,i,dd);resetter.push(bO,this.$$method.reset[a[i]],t);if(themeable){{};styler.push(bO,this.$$method.setThemed[a[i]],bK,i,dd);unstyler.push(bO,this.$$method.resetThemed[a[i]],t);};};this.$$method.set[name]=e+upname;members[this.$$method.set[name]]=new Function(setter.join(cL));this.$$method.reset[name]=Y+upname;members[this.$$method.reset[name]]=new Function(resetter.join(cL));if(themeable){this.$$method.setThemed[name]=bx+upname;members[this.$$method.setThemed[name]]=new Function(styler.join(cL));this.$$method.resetThemed[name]=g+upname;members[this.$$method.resetThemed[name]]=new Function(unstyler.join(cL));};},__w:function(clazz,config,name){var upname=qx.Bootstrap.firstUp(name);var members=clazz.prototype;{};if(config.dereference===undefined&&typeof config.check===ba){config.dereference=this.__x(config.check);};var method=this.$$method;var store=this.$$store;store.runtime[name]=bw+name;store.user[name]=bA+name;store.theme[name]=ct+name;store.init[name]=cs+name;store.inherit[name]=cT+name;store.useinit[name]=bo+name;method.get[name]=bj+upname;members[method.get[name]]=function(){return qx.core.Property.executeOptimizedGetter(this,clazz,name,bj);};method.set[name]=e+upname;members[method.set[name]]=function(dp){return qx.core.Property.executeOptimizedSetter(this,clazz,name,e,arguments);};method.reset[name]=Y+upname;members[method.reset[name]]=function(){return qx.core.Property.executeOptimizedSetter(this,clazz,name,Y);};if(config.inheritable||config.apply||config.event||config.deferredInit){method.init[name]=cx+upname;members[method.init[name]]=function(dq){return qx.core.Property.executeOptimizedSetter(this,clazz,name,cx,arguments);};{};};if(config.inheritable){method.refresh[name]=cS+upname;members[method.refresh[name]]=function(dr){return qx.core.Property.executeOptimizedSetter(this,clazz,name,cS,arguments);};{};};method.setRuntime[name]=bM+upname;members[method.setRuntime[name]]=function(ds){return qx.core.Property.executeOptimizedSetter(this,clazz,name,bM,arguments);};method.resetRuntime[name]=S+upname;members[method.resetRuntime[name]]=function(){return qx.core.Property.executeOptimizedSetter(this,clazz,name,S);};if(config.themeable){method.setThemed[name]=bx+upname;members[method.setThemed[name]]=function(dt){return qx.core.Property.executeOptimizedSetter(this,clazz,name,bx,arguments);};method.resetThemed[name]=g+upname;members[method.resetThemed[name]]=function(){return qx.core.Property.executeOptimizedSetter(this,clazz,name,g);};{};};if(config.check===bI){members[cR+upname]=new Function(bd+method.set[name]+cN+method.get[name]+ca);members[db+upname]=new Function(bd+method.get[name]+bT);{};};{};},__x:function(du){return !!this.__r[du];},__y:{'0':de,'1':bs,'2':B,'3':bR,'4':cD,'5':M},error:function(dv,dB,dA,dw,dx){var dy=dv.constructor.classname;var dz=q+dA+U+dy+y+this.$$method[dw][dA]+cW+dx+T;throw new Error(dz+(this.__y[dB]||cw+dB));},__z:function(instance,members,name,variant,code,args){var store=this.$$method[variant][name];{members[store]=new Function(cF,code.join(cL));};if(qx.core.Environment.get(cz)){members[store]=qx.core.Aspect.wrap(instance.classname+cp+store,members[store],s);};qx.Bootstrap.setDisplayName(members[store],instance.classname+bH,store);if(args===undefined){return instance[store]();}else {return instance[store](args[0]);};},executeOptimizedGetter:function(dF,dE,name,dD){var dH=dE.$$properties[name];var dG=dE.prototype;var dC=[];var dI=this.$$store;dC.push(J,dI.runtime[name],bB);dC.push(K,dI.runtime[name],f);if(dH.inheritable){dC.push(L,dI.inherit[name],bB);dC.push(K,dI.inherit[name],f);dC.push(D);};dC.push(J,dI.user[name],bB);dC.push(K,dI.user[name],f);if(dH.themeable){dC.push(L,dI.theme[name],bB);dC.push(K,dI.theme[name],f);};if(dH.deferredInit&&dH.init===undefined){dC.push(L,dI.init[name],bB);dC.push(K,dI.init[name],f);};dC.push(D);if(dH.init!==undefined){if(dH.inheritable){dC.push(n,dI.init[name],f);if(dH.nullable){dC.push(bi);}else if(dH.init!==undefined){dC.push(K,dI.init[name],f);}else {dC.push(bP,name,cQ,dE.classname,dc);};dC.push(m);}else {dC.push(K,dI.init[name],f);};}else if(dH.inheritable||dH.nullable){dC.push(cc);}else {dC.push(cM,name,cQ,dE.classname,dc);};return this.__z(dF,dG,name,dD,dC);},executeOptimizedSetter:function(dP,dO,name,dN,dM){var dR=dO.$$properties[name];var dQ=dO.prototype;var dK=[];var dJ=dN===e||dN===bx||dN===bM||(dN===cx&&dR.init===undefined);var dL=dR.apply||dR.event||dR.inheritable;var dS=this.__A(dN,name);this.__B(dK,dR,name,dN,dJ);if(dJ){this.__C(dK,dO,dR,name);};if(dL){this.__D(dK,dJ,dS,dN);};if(dR.inheritable){dK.push(bl);};{};if(!dL){this.__F(dK,name,dN,dJ);}else {this.__G(dK,dR,name,dN,dJ);};if(dR.inheritable){this.__H(dK,dR,name,dN);}else if(dL){this.__I(dK,dR,name,dN);};if(dL){this.__J(dK,dR,name,dN);if(dR.inheritable&&dQ._getChildren){this.__K(dK,name);};};if(dJ){dK.push(bN);};return this.__z(dP,dQ,name,dN,dK,dM);},__A:function(dT,name){if(dT===bM||dT===S){var dU=this.$$store.runtime[name];}else if(dT===bx||dT===g){dU=this.$$store.theme[name];}else if(dT===cx){dU=this.$$store.init[name];}else {dU=this.$$store.user[name];};return dU;},__B:function(dX,dV,name,dY,dW){{if(!dV.nullable||dV.check||dV.inheritable){dX.push(cU);};if(dY===e){dX.push(N,name,bh,dY,bt);};};},__C:function(ea,ec,eb,name){if(eb.transform){ea.push(bg,eb.transform,bz);};if(eb.validate){if(typeof eb.validate===ba){ea.push(br,eb.validate,bz);}else if(eb.validate instanceof Function){ea.push(ec.classname,bV,name);ea.push(u);};};},__D:function(ee,ed,eg,ef){var eh=(ef===Y||ef===g||ef===S);if(ed){ee.push(J,eg,j);}else if(eh){ee.push(J,eg,W);};},__E:undefined,__F:function(ej,name,ek,ei){if(ek===bM){ej.push(br,this.$$store.runtime[name],cb);}else if(ek===S){ej.push(J,this.$$store.runtime[name],bB);ej.push(R,this.$$store.runtime[name],f);}else if(ek===e){ej.push(br,this.$$store.user[name],cb);}else if(ek===Y){ej.push(J,this.$$store.user[name],bB);ej.push(R,this.$$store.user[name],f);}else if(ek===bx){ej.push(br,this.$$store.theme[name],cb);}else if(ek===g){ej.push(J,this.$$store.theme[name],bB);ej.push(R,this.$$store.theme[name],f);}else if(ek===cx&&ei){ej.push(br,this.$$store.init[name],cb);};},__G:function(en,el,name,eo,em){if(el.inheritable){en.push(P,this.$$store.inherit[name],f);}else {en.push(cn);};en.push(J,this.$$store.runtime[name],cu);if(eo===bM){en.push(cH,this.$$store.runtime[name],cb);}else if(eo===S){en.push(R,this.$$store.runtime[name],f);en.push(J,this.$$store.user[name],bB);en.push(cH,this.$$store.user[name],f);en.push(L,this.$$store.theme[name],bB);en.push(cH,this.$$store.theme[name],f);en.push(L,this.$$store.init[name],cu);en.push(cH,this.$$store.init[name],f);en.push(br,this.$$store.useinit[name],E);en.push(cq);}else {en.push(H,this.$$store.runtime[name],f);if(eo===e){en.push(br,this.$$store.user[name],cb);}else if(eo===Y){en.push(R,this.$$store.user[name],f);}else if(eo===bx){en.push(br,this.$$store.theme[name],cb);}else if(eo===g){en.push(R,this.$$store.theme[name],f);}else if(eo===cx&&em){en.push(br,this.$$store.init[name],cb);};};en.push(cq);en.push(L,this.$$store.user[name],cu);if(eo===e){if(!el.inheritable){en.push(bX,this.$$store.user[name],f);};en.push(cH,this.$$store.user[name],cb);}else if(eo===Y){if(!el.inheritable){en.push(bX,this.$$store.user[name],f);};en.push(R,this.$$store.user[name],f);en.push(J,this.$$store.runtime[name],bB);en.push(cH,this.$$store.runtime[name],f);en.push(J,this.$$store.theme[name],bB);en.push(cH,this.$$store.theme[name],f);en.push(L,this.$$store.init[name],cu);en.push(cH,this.$$store.init[name],f);en.push(br,this.$$store.useinit[name],E);en.push(cq);}else {if(eo===bM){en.push(cH,this.$$store.runtime[name],cb);}else if(el.inheritable){en.push(cH,this.$$store.user[name],f);}else {en.push(H,this.$$store.user[name],f);};if(eo===bx){en.push(br,this.$$store.theme[name],cb);}else if(eo===g){en.push(R,this.$$store.theme[name],f);}else if(eo===cx&&em){en.push(br,this.$$store.init[name],cb);};};en.push(cq);if(el.themeable){en.push(L,this.$$store.theme[name],cu);if(!el.inheritable){en.push(bX,this.$$store.theme[name],f);};if(eo===bM){en.push(cH,this.$$store.runtime[name],cb);}else if(eo===e){en.push(cH,this.$$store.user[name],cb);}else if(eo===bx){en.push(cH,this.$$store.theme[name],cb);}else if(eo===g){en.push(R,this.$$store.theme[name],f);en.push(J,this.$$store.init[name],cu);en.push(cH,this.$$store.init[name],f);en.push(br,this.$$store.useinit[name],E);en.push(cq);}else if(eo===cx){if(em){en.push(br,this.$$store.init[name],cb);};en.push(cH,this.$$store.theme[name],f);}else if(eo===cS){en.push(cH,this.$$store.theme[name],f);};en.push(cq);};en.push(L,this.$$store.useinit[name],bC);if(!el.inheritable){en.push(bX,this.$$store.init[name],f);};if(eo===cx){if(em){en.push(cH,this.$$store.init[name],cb);}else {en.push(cH,this.$$store.init[name],f);};}else if(eo===e||eo===bM||eo===bx||eo===cS){en.push(R,this.$$store.useinit[name],f);if(eo===bM){en.push(cH,this.$$store.runtime[name],cb);}else if(eo===e){en.push(cH,this.$$store.user[name],cb);}else if(eo===bx){en.push(cH,this.$$store.theme[name],cb);}else if(eo===cS){en.push(cH,this.$$store.init[name],f);};};en.push(cq);if(eo===e||eo===bM||eo===bx||eo===cx){en.push(w);if(eo===bM){en.push(cH,this.$$store.runtime[name],cb);}else if(eo===e){en.push(cH,this.$$store.user[name],cb);}else if(eo===bx){en.push(cH,this.$$store.theme[name],cb);}else if(eo===cx){if(em){en.push(cH,this.$$store.init[name],cb);}else {en.push(cH,this.$$store.init[name],f);};en.push(br,this.$$store.useinit[name],E);};en.push(cq);};},__H:function(eq,ep,name,er){eq.push(bE);if(er===cS){eq.push(bu);}else {eq.push(ci,this.$$store.inherit[name],f);};eq.push(cB);eq.push(br,this.$$store.init[name],cm);eq.push(br,this.$$store.init[name],b);eq.push(cH,this.$$store.init[name],f);eq.push(br,this.$$store.useinit[name],E);eq.push(bv);eq.push(R,this.$$store.useinit[name],by);eq.push(cq);eq.push(cf);eq.push(G);eq.push(bp,this.$$store.inherit[name],f);eq.push(cq);eq.push(Q);eq.push(R,this.$$store.inherit[name],f);eq.push(bf,this.$$store.inherit[name],A);eq.push(co);if(ep.init!==undefined&&er!==cx){eq.push(F,this.$$store.init[name],bF);}else {eq.push(ch);};eq.push(cY);},__I:function(et,es,name,eu){if(eu!==e&&eu!==bM&&eu!==bx){et.push(cv);};et.push(cf);if(es.init!==undefined&&eu!==cx){et.push(F,this.$$store.init[name],bF);}else {et.push(ch);};},__J:function(ew,ev,name,ex){if(ev.apply){ew.push(br,ev.apply,bL,name,O,ex,ce);};if(ev.event){ew.push(bY,bQ,ev.event,bb,cC,ev.event,bm,bJ);};},__K:function(ey,name){ey.push(r);ey.push(cP,this.$$method.refresh[name],bS,this.$$method.refresh[name],C);ey.push(cq);}}});})();(function(){var b=".prototype",c="$$init_",d="constructor",e="Property module disabled.",f="extend",g="module.property",h="singleton",j="qx.event.type.Data",k="module.events",m="qx.aspects",n="toString",o='extend',p="Array",q="static",r="",s="Events module not enabled.",t="]",u="Class",v="qx.Class",w='"extend" parameter is null or undefined',x="[Class ",y="destructor",z="destruct",A=".",B="member";qx.Bootstrap.define(v,{statics:{__L:qx.core.Environment.get(g)?qx.core.Property:null,define:function(name,F){if(!F){F={};};if(F.include&&!(qx.Bootstrap.getClass(F.include)===p)){F.include=[F.include];};if(F.implement&&!(qx.Bootstrap.getClass(F.implement)===p)){F.implement=[F.implement];};var C=false;if(!F.hasOwnProperty(f)&&!F.type){F.type=q;C=true;};{};var D=this.__O(name,F.type,F.extend,F.statics,F.construct,F.destruct,F.include);if(F.extend){if(F.properties){this.__Q(D,F.properties,true);};if(F.members){this.__S(D,F.members,true,true,false);};if(F.events){this.__P(D,F.events,true);};if(F.include){for(var i=0,l=F.include.length;i<l;i++ ){this.__W(D,F.include[i],false);};};}else if(F.hasOwnProperty(o)&&false){throw new Error(w);};if(F.environment){for(var E in F.environment){qx.core.Environment.add(E,F.environment[E]);};};if(F.implement){for(var i=0,l=F.implement.length;i<l;i++ ){this.__U(D,F.implement[i]);};};{};if(F.defer){F.defer.self=D;F.defer(D,D.prototype,{add:function(name,G){var H={};H[name]=G;qx.Class.__Q(D,H,true);}});};return D;},undefine:function(name){delete this.$$registry[name];var K=name.split(A);var J=[window];for(var i=0;i<K.length;i++ ){J.push(J[i][K[i]]);};for(var i=J.length-1;i>=1;i-- ){var I=J[i];var parent=J[i-1];if(qx.Bootstrap.isFunction(I)||qx.Bootstrap.objectGetLength(I)===0){delete parent[K[i-1]];}else {break;};};},isDefined:qx.util.OOUtil.classIsDefined,getTotalNumber:function(){return qx.Bootstrap.objectGetLength(this.$$registry);},getByName:qx.Bootstrap.getByName,include:function(M,L){{};qx.Class.__W(M,L,false);},patch:function(O,N){{};qx.Class.__W(O,N,true);},isSubClassOf:function(Q,P){if(!Q){return false;};if(Q==P){return true;};if(Q.prototype instanceof P){return true;};return false;},getPropertyDefinition:qx.util.OOUtil.getPropertyDefinition,getProperties:function(S){var R=[];while(S){if(S.$$properties){R.push.apply(R,Object.keys(S.$$properties));};S=S.superclass;};return R;},getByProperty:function(T,name){while(T){if(T.$$properties&&T.$$properties[name]){return T;};T=T.superclass;};return null;},hasProperty:qx.util.OOUtil.hasProperty,getEventType:qx.util.OOUtil.getEventType,supportsEvent:qx.util.OOUtil.supportsEvent,hasOwnMixin:function(V,U){return V.$$includes&&V.$$includes.indexOf(U)!==-1;},getByMixin:function(Y,X){var W,i,l;while(Y){if(Y.$$includes){W=Y.$$flatIncludes;for(i=0,l=W.length;i<l;i++ ){if(W[i]===X){return Y;};};};Y=Y.superclass;};return null;},getMixins:qx.util.OOUtil.getMixins,hasMixin:function(bb,ba){return !!this.getByMixin(bb,ba);},hasOwnInterface:function(bd,bc){return bd.$$implements&&bd.$$implements.indexOf(bc)!==-1;},getByInterface:qx.util.OOUtil.getByInterface,getInterfaces:function(bf){var be=[];while(bf){if(bf.$$implements){be.push.apply(be,bf.$$flatImplements);};bf=bf.superclass;};return be;},hasInterface:qx.util.OOUtil.hasInterface,implementsInterface:function(bh,bg){var bi=bh.constructor;if(this.hasInterface(bi,bg)){return true;};if(qx.Interface.objectImplements(bh,bg)){return true;};if(qx.Interface.classImplements(bi,bg)){return true;};return false;},getInstance:function(){if(!this.$$instance){this.$$allowconstruct=true;this.$$instance=new this();delete this.$$allowconstruct;};return this.$$instance;},genericToString:function(){return x+this.classname+t;},$$registry:qx.Bootstrap.$$registry,__h:null,__M:null,__i:function(name,bj){},__N:function(bk){},__O:function(name,bu,bt,bl,br,bp,bo){var bq;if(!bt&&qx.core.Environment.get(m)==false){bq=bl||{};qx.Bootstrap.setDisplayNames(bq,name);}else {bq={};if(bt){if(!br){br=this.__X();};if(this.__Y(bt,bo)){bq=this.__ba(br,name,bu);}else {bq=br;};if(bu===h){bq.getInstance=this.getInstance;};qx.Bootstrap.setDisplayName(br,name,d);};if(bl){qx.Bootstrap.setDisplayNames(bl,name);var bs;for(var i=0,a=Object.keys(bl),l=a.length;i<l;i++ ){bs=a[i];var bm=bl[bs];if(qx.core.Environment.get(m)){if(bm instanceof Function){bm=qx.core.Aspect.wrap(name+A+bs,bm,q);};bq[bs]=bm;}else {bq[bs]=bm;};};};};var bn=name?qx.Bootstrap.createNamespace(name,bq):r;bq.name=bq.classname=name;bq.basename=bn;bq.$$type=u;if(bu){bq.$$classtype=bu;};if(!bq.hasOwnProperty(n)){bq.toString=this.genericToString;};if(bt){qx.Bootstrap.extendClass(bq,br,bt,name,bn);if(bp){if(qx.core.Environment.get(m)){bp=qx.core.Aspect.wrap(name,bp,y);};bq.$$destructor=bp;qx.Bootstrap.setDisplayName(bp,name,z);};};this.$$registry[name]=bq;return bq;},__P:function(bv,bw,by){{var bx,bx;};if(bv.$$events){for(var bx in bw){bv.$$events[bx]=bw[bx];};}else {bv.$$events=bw;};},__Q:function(bA,bD,bB){if(!qx.core.Environment.get(g)){throw new Error(e);};var bC;if(bB===undefined){bB=false;};var bz=bA.prototype;for(var name in bD){bC=bD[name];{};bC.name=name;if(!bC.refine){if(bA.$$properties===undefined){bA.$$properties={};};bA.$$properties[name]=bC;};if(bC.init!==undefined){bA.prototype[c+name]=bC.init;};if(bC.event!==undefined){if(!qx.core.Environment.get(k)){throw new Error(s);};var event={};event[bC.event]=j;this.__P(bA,event,bB);};if(bC.inheritable){this.__L.$$inheritable[name]=true;if(!bz.$$refreshInheritables){this.__L.attachRefreshInheritables(bA);};};if(!bC.refine){this.__L.attachMethods(bA,name,bC);};};},__R:null,__S:function(bL,bE,bG,bI,bK){var bF=bL.prototype;var bJ,bH;qx.Bootstrap.setDisplayNames(bE,bL.classname+b);for(var i=0,a=Object.keys(bE),l=a.length;i<l;i++ ){bJ=a[i];bH=bE[bJ];{};if(bI!==false&&bH instanceof Function&&bH.$$type==null){if(bK==true){bH=this.__T(bH,bF[bJ]);}else {if(bF[bJ]){bH.base=bF[bJ];};bH.self=bL;};if(qx.core.Environment.get(m)){bH=qx.core.Aspect.wrap(bL.classname+A+bJ,bH,B);};};bF[bJ]=bH;};},__T:function(bM,bN){if(bN){return function(){var bP=bM.base;bM.base=bN;var bO=bM.apply(this,arguments);bM.base=bP;return bO;};}else {return bM;};},__U:function(bS,bQ){{};var bR=qx.Interface.flatten([bQ]);if(bS.$$implements){bS.$$implements.push(bQ);bS.$$flatImplements.push.apply(bS.$$flatImplements,bR);}else {bS.$$implements=[bQ];bS.$$flatImplements=bR;};},__V:function(bU){var name=bU.classname;var bT=this.__ba(bU,name,bU.$$classtype);for(var i=0,a=Object.keys(bU),l=a.length;i<l;i++ ){bV=a[i];bT[bV]=bU[bV];};bT.prototype=bU.prototype;var bX=bU.prototype;for(var i=0,a=Object.keys(bX),l=a.length;i<l;i++ ){bV=a[i];var bY=bX[bV];if(bY&&bY.self==bU){bY.self=bT;};};for(var bV in this.$$registry){var bW=this.$$registry[bV];if(!bW){continue;};if(bW.base==bU){bW.base=bT;};if(bW.superclass==bU){bW.superclass=bT;};if(bW.$$original){if(bW.$$original.base==bU){bW.$$original.base=bT;};if(bW.$$original.superclass==bU){bW.$$original.superclass=bT;};};};qx.Bootstrap.createNamespace(name,bT);this.$$registry[name]=bT;return bT;},__W:function(cf,cd,cc){{};if(this.hasMixin(cf,cd)){return;};var ca=cf.$$original;if(cd.$$constructor&&!ca){cf=this.__V(cf);};var cb=qx.Mixin.flatten([cd]);var ce;for(var i=0,l=cb.length;i<l;i++ ){ce=cb[i];if(ce.$$events){this.__P(cf,ce.$$events,cc);};if(ce.$$properties){this.__Q(cf,ce.$$properties,cc);};if(ce.$$members){this.__S(cf,ce.$$members,cc,cc,cc);};};if(cf.$$includes){cf.$$includes.push(cd);cf.$$flatIncludes.push.apply(cf.$$flatIncludes,cb);}else {cf.$$includes=[cd];cf.$$flatIncludes=cb;};},__X:function(){function cg(){cg.base.apply(this,arguments);};return cg;},__Y:function(ci,ch){{};if(ci&&ci.$$includes){var cj=ci.$$flatIncludes;for(var i=0,l=cj.length;i<l;i++ ){if(cj[i].$$constructor){return true;};};};if(ch){var ck=qx.Mixin.flatten(ch);for(var i=0,l=ck.length;i<l;i++ ){if(ck[i].$$constructor){return true;};};};return false;},__ba:function(cm,name,cl){var co=function(){var cr=co;{};var cp=cr.$$original.apply(this,arguments);if(cr.$$includes){var cq=cr.$$flatIncludes;for(var i=0,l=cq.length;i<l;i++ ){if(cq[i].$$constructor){cq[i].$$constructor.apply(this,arguments);};};};{};return cp;};if(qx.core.Environment.get(m)){var cn=qx.core.Aspect.wrap(name,co,d);co.$$original=cm;co.constructor=cn;co=cn;};co.$$original=cm;cm.wrapper=co;return co;}},defer:function(){if(qx.core.Environment.get(m)){for(var cs in qx.Bootstrap.$$registry){var ct=qx.Bootstrap.$$registry[cs];for(var cu in ct){if(ct[cu] instanceof Function){ct[cu]=qx.core.Aspect.wrap(cs+A+cu,ct[cu],q);};};};};}});})();(function(){var a="qx.data.MBinding";qx.Mixin.define(a,{construct:function(){this.__bb=this.toHashCode();},members:{__bb:null,bind:function(b,e,c,d){return qx.data.SingleValueBinding.bind(this,b,e,c,d);},removeBinding:function(f){qx.data.SingleValueBinding.removeBindingFromObject(this,f);},removeRelatedBindings:function(g){qx.data.SingleValueBinding.removeRelatedBindings(this,g);},removeAllBindings:function(){qx.data.SingleValueBinding.removeAllBindingsForObject(this);},getBindings:function(){return qx.data.SingleValueBinding.getAllBindingsForObject(this);}},destruct:function(){this.$$hash=this.__bb;this.removeAllBindings();delete this.$$hash;}});})();(function(){var a=". Error message: ",b="Boolean",c=").",d="set",f="deepBinding",g=") to the object '",h="item",k="Please use only one array at a time: ",l="Integer",m="reset",n=" of object ",p="qx.data.SingleValueBinding",q="Binding property ",r="Failed so set value ",s="change",t="Binding could not be found!",u="get",v="^",w=" does not work.",x="String",y="Binding from '",z="",A="PositiveNumber",B="]",C="[",D=".",E="PositiveInteger",F='No number or \'last\' value hast been given in an array binding: ',G="' (",H=" on ",I="Binding does not exist!",J="Number",K=".[",L="Date",M=" not possible: No event available. ",N="last";qx.Class.define(p,{statics:{__bc:{},__bd:{},bind:function(bb,U,bg,bd,bj){{};var bf=this.__bf(bb,U,bg,bd,bj);var Q=U.split(D);var P=this.__bn(Q);var S=[];var bh=[];var R=[];var T=[];var Y=bb;try{for(var i=0;i<Q.length;i++ ){if(P[i]!==z){T.push(s);}else {var X=this.__bg(Y,Q[i]);if(!X){if(i==0){throw new qx.core.AssertionError(q+Q[i]+n+Y+M);};this.__bm(undefined,bg,bd,bj,bb);break;};T.push(X);};S[i]=Y;if(i==Q.length-1){if(P[i]!==z){var ba=P[i]===N?Y.length-1:P[i];var O=Y.getItem(ba);this.__bm(O,bg,bd,bj,bb);R[i]=this.__bo(Y,T[i],bg,bd,bj,P[i]);}else {if(Q[i]!=null&&Y[u+qx.lang.String.firstUp(Q[i])]!=null){var O=Y[u+qx.lang.String.firstUp(Q[i])]();this.__bm(O,bg,bd,bj,bb);};R[i]=this.__bo(Y,T[i],bg,bd,bj);};}else {var bi={index:i,propertyNames:Q,sources:S,listenerIds:R,arrayIndexValues:P,targetObject:bg,targetPropertyChain:bd,options:bj,listeners:bh};var be=qx.lang.Function.bind(this.__be,this,bi);bh.push(be);R[i]=Y.addListener(T[i],be);};if(Y[u+qx.lang.String.firstUp(Q[i])]==null){Y=undefined;}else if(P[i]!==z){var ba=P[i]===N?Y.length-1:P[i];Y=Y[u+qx.lang.String.firstUp(Q[i])](ba);}else {Y=Y[u+qx.lang.String.firstUp(Q[i])]();if(Y===null&&(Q.length-1)!=i){Y=undefined;};};if(!Y){this.__bm(Y,bg,bd,bj,bb);break;};};}catch(bk){for(var i=0;i<S.length;i++ ){if(S[i]&&R[i]){S[i].removeListenerById(R[i]);};};var V=bf.targets;var bc=bf.listenerIds;for(var i=0;i<V.length;i++ ){if(V[i]&&bc[i]){V[i].removeListenerById(bc[i]);};};throw bk;};var W={type:f,listenerIds:R,sources:S,targetListenerIds:bf.listenerIds,targets:bf.targets};this.__bp(W,bb,U,bg,bd);return W;},__be:function(br){if(br.options&&br.options.onUpdate){br.options.onUpdate(br.sources[br.index],br.targetObject);};for(var j=br.index+1;j<br.propertyNames.length;j++ ){var bp=br.sources[j];br.sources[j]=null;if(!bp){continue;};bp.removeListenerById(br.listenerIds[j]);};var bp=br.sources[br.index];for(var j=br.index+1;j<br.propertyNames.length;j++ ){if(br.arrayIndexValues[j-1]!==z){bp=bp[u+qx.lang.String.firstUp(br.propertyNames[j-1])](br.arrayIndexValues[j-1]);}else {bp=bp[u+qx.lang.String.firstUp(br.propertyNames[j-1])]();};br.sources[j]=bp;if(!bp){if(br.options&&br.options.converter){var bl=false;if(br.options.ignoreConverter){var bs=br.propertyNames.slice(0,j).join(D);var bq=bs.match(new RegExp(v+br.options.ignoreConverter));bl=bq?bq.length>0:false;};if(!bl){this.__bi(br.targetObject,br.targetPropertyChain,br.options.converter());}else {this.__bh(br.targetObject,br.targetPropertyChain);};}else {this.__bh(br.targetObject,br.targetPropertyChain);};break;};if(j==br.propertyNames.length-1){if(qx.Class.implementsInterface(bp,qx.data.IListData)){var bt=br.arrayIndexValues[j]===N?bp.length-1:br.arrayIndexValues[j];var bm=bp.getItem(bt);this.__bm(bm,br.targetObject,br.targetPropertyChain,br.options,br.sources[br.index]);br.listenerIds[j]=this.__bo(bp,s,br.targetObject,br.targetPropertyChain,br.options,br.arrayIndexValues[j]);}else {if(br.propertyNames[j]!=null&&bp[u+qx.lang.String.firstUp(br.propertyNames[j])]!=null){var bm=bp[u+qx.lang.String.firstUp(br.propertyNames[j])]();this.__bm(bm,br.targetObject,br.targetPropertyChain,br.options,br.sources[br.index]);};var bn=this.__bg(bp,br.propertyNames[j]);if(!bn){this.__bh(br.targetObject,br.targetPropertyChain);break;};br.listenerIds[j]=this.__bo(bp,bn,br.targetObject,br.targetPropertyChain,br.options);};}else {if(br.listeners[j]==null){var bo=qx.lang.Function.bind(this.__be,this,br);br.listeners.push(bo);};if(qx.Class.implementsInterface(bp,qx.data.IListData)){var bn=s;}else {var bn=this.__bg(bp,br.propertyNames[j]);};if(!bn){this.__bh(br.targetObject,br.targetPropertyChain);return;};br.listenerIds[j]=bp.addListener(bn,br.listeners[j]);};};},__bf:function(bw,bE,bI,bA,bC){var bz=bA.split(D);var bx=this.__bn(bz);var bH=[];var bG=[];var bB=[];var bF=[];var by=bI;for(var i=0;i<bz.length-1;i++ ){if(bx[i]!==z){bF.push(s);}else {var bv=this.__bg(by,bz[i]);if(!bv){break;};bF.push(bv);};bH[i]=by;var bD=function(){for(var j=i+1;j<bz.length-1;j++ ){var bL=bH[j];bH[j]=null;if(!bL){continue;};bL.removeListenerById(bB[j]);};var bL=bH[i];for(var j=i+1;j<bz.length-1;j++ ){var bJ=qx.lang.String.firstUp(bz[j-1]);if(bx[j-1]!==z){var bM=bx[j-1]===N?bL.getLength()-1:bx[j-1];bL=bL[u+bJ](bM);}else {bL=bL[u+bJ]();};bH[j]=bL;if(bG[j]==null){bG.push(bD);};if(qx.Class.implementsInterface(bL,qx.data.IListData)){var bK=s;}else {var bK=qx.data.SingleValueBinding.__bg(bL,bz[j]);if(!bK){break;};};bB[j]=bL.addListener(bK,bG[j]);};qx.data.SingleValueBinding.updateTarget(bw,bE,bI,bA,bC);};bG.push(bD);bB[i]=by.addListener(bF[i],bD);var bu=qx.lang.String.firstUp(bz[i]);if(by[u+bu]==null){by=null;}else if(bx[i]!==z){by=by[u+bu](bx[i]);}else {by=by[u+bu]();};if(!by){break;};};return {listenerIds:bB,targets:bH};},updateTarget:function(bN,bQ,bS,bO,bR){var bP=this.resolvePropertyChain(bN,bQ);bP=qx.data.SingleValueBinding.__bq(bP,bS,bO,bR,bN);this.__bi(bS,bO,bP);},resolvePropertyChain:function(o,bT){var bU=this.__bk(bT);return this.__bl(o,bU,bU.length);},__bg:function(bW,bX){var bV=this.__br(bW,bX);if(bV==null){if(qx.Class.supportsEvent(bW.constructor,bX)){bV=bX;}else if(qx.Class.supportsEvent(bW.constructor,s+qx.lang.String.firstUp(bX))){bV=s+qx.lang.String.firstUp(bX);}else {return null;};};return bV;},__bh:function(cd,cb){var cc=this.__bk(cb);var ca=this.__bl(cd,cc);if(ca!=null){var ce=cc[cc.length-1];var bY=this.__bj(ce);if(bY){this.__bi(cd,cb,null);return;};if(ca[m+qx.lang.String.firstUp(ce)]!=undefined){ca[m+qx.lang.String.firstUp(ce)]();}else {ca[d+qx.lang.String.firstUp(ce)](null);};};},__bi:function(ck,ch,ci){var cj=this.__bk(ch);var cg=this.__bl(ck,cj);if(cg){var cl=cj[cj.length-1];var cf=this.__bj(cl);if(cf){if(cf===N){cf=cg.length-1;};cg.setItem(cf,ci);}else {cg[d+qx.lang.String.firstUp(cl)](ci);};};},__bj:function(co){var cm=/^\[(\d+|last)\]$/;var cn=co.match(cm);if(cn){return cn[1];};return null;},__bk:function(cp){return cp.replace(/\[/g,K).split(D).filter(function(cq){return cq!==z;});},__bl:function(cw,cr,cs){cs=cs||cr.length-1;var cu=cw;for(var i=0;i<cs;i++ ){try{var cv=cr[i];var ct=this.__bj(cv);if(ct){if(ct===N){ct=cu.length-1;};cu=cu.getItem(ct);}else {cu=cu[u+qx.lang.String.firstUp(cv)]();};}catch(cx){return null;};};return cu;},__bm:function(cC,cy,cA,cB,cz){cC=this.__bq(cC,cy,cA,cB,cz);if(cC===undefined){this.__bh(cy,cA);};if(cC!==undefined){try{this.__bi(cy,cA,cC);if(cB&&cB.onUpdate){cB.onUpdate(cz,cy,cC);};}catch(e){if(!(e instanceof qx.core.ValidationError)){throw e;};if(cB&&cB.onSetFail){cB.onSetFail(e);}else {qx.log.Logger.warn(r+cC+H+cy+a+e);};};};},__bn:function(cD){var cE=[];for(var i=0;i<cD.length;i++ ){var name=cD[i];if(qx.lang.String.endsWith(name,B)){var cF=name.substring(name.indexOf(C)+1,name.indexOf(B));if(name.indexOf(B)!=name.length-1){throw new Error(k+name+w);};if(cF!==N){if(cF==z||isNaN(parseInt(cF,10))){throw new Error(F+name+w);};};if(name.indexOf(C)!=0){cD[i]=name.substring(0,name.indexOf(C));cE[i]=z;cE[i+1]=cF;cD.splice(i+1,0,h);i++ ;}else {cE[i]=cF;cD.splice(i,1,h);};}else {cE[i]=z;};};return cE;},__bo:function(cG,cJ,cO,cM,cK,cI){{var cH;};var cL=function(cR,e){if(cR!==z){if(cR===N){cR=cG.length-1;};var cS=cG.getItem(cR);if(cS===undefined){qx.data.SingleValueBinding.__bh(cO,cM);};var cQ=e.getData().start;var cP=e.getData().end;if(cR<cQ||cR>cP){return;};}else {var cS=e.getData();};{};cS=qx.data.SingleValueBinding.__bq(cS,cO,cM,cK,cG);{};try{if(cS!==undefined){qx.data.SingleValueBinding.__bi(cO,cM,cS);}else {qx.data.SingleValueBinding.__bh(cO,cM);};if(cK&&cK.onUpdate){cK.onUpdate(cG,cO,cS);};}catch(cT){if(!(cT instanceof qx.core.ValidationError)){throw cT;};if(cK&&cK.onSetFail){cK.onSetFail(cT);}else {qx.log.Logger.warn(r+cS+H+cO+a+cT);};};};if(!cI){cI=z;};cL=qx.lang.Function.bind(cL,cG,cI);var cN=cG.addListener(cJ,cL);return cN;},__bp:function(da,cU,cX,db,cY){var cV;cV=cU.toHashCode();if(this.__bc[cV]===undefined){this.__bc[cV]=[];};var cW=[da,cU,cX,db,cY];this.__bc[cV].push(cW);cV=db.toHashCode();if(this.__bd[cV]===undefined){this.__bd[cV]=[];};this.__bd[cV].push(cW);},__bq:function(df,dl,de,dh,dc){if(dh&&dh.converter){var di;if(dl.getModel){di=dl.getModel();};return dh.converter(df,di,dc,dl);}else {var dg=this.__bk(de);var dd=this.__bl(dl,dg);var dm=de.substring(de.lastIndexOf(D)+1,de.length);if(dd==null){return df;};var dj=qx.Class.getPropertyDefinition(dd.constructor,dm);var dk=dj==null?z:dj.check;return this.__bs(df,dk);};},__br:function(dn,dq){var dp=qx.Class.getPropertyDefinition(dn.constructor,dq);if(dp==null){return null;};return dp.event;},__bs:function(dt,ds){var dr=qx.lang.Type.getClass(dt);if((dr==J||dr==x)&&(ds==l||ds==E)){dt=parseInt(dt,10);};if((dr==b||dr==J||dr==L)&&ds==x){dt=dt+z;};if((dr==J||dr==x)&&(ds==J||ds==A)){dt=parseFloat(dt);};return dt;},removeBindingFromObject:function(du,dy){if(dy.type==f){for(var i=0;i<dy.sources.length;i++ ){if(dy.sources[i]){dy.sources[i].removeListenerById(dy.listenerIds[i]);};};for(var i=0;i<dy.targets.length;i++ ){if(dy.targets[i]){dy.targets[i].removeListenerById(dy.targetListenerIds[i]);};};}else {du.removeListenerById(dy);};var dx=this.getAllBindingsForObject(du);if(dx!=undefined){for(var i=0;i<dx.length;i++ ){if(dx[i][0]==dy){var dv=dx[i][3];if(this.__bd[dv.toHashCode()]){qx.lang.Array.remove(this.__bd[dv.toHashCode()],dx[i]);};var dw=dx[i][1];if(this.__bc[dw.toHashCode()]){qx.lang.Array.remove(this.__bc[dw.toHashCode()],dx[i]);};return;};};};throw new Error(t);},removeAllBindingsForObject:function(dA){{};var dz=this.getAllBindingsForObject(dA);if(dz!=undefined){for(var i=dz.length-1;i>=0;i-- ){this.removeBindingFromObject(dA,dz[i][0]);};};},removeRelatedBindings:function(dC,dD){{};var dF=this.getAllBindingsForObject(dC);if(dF!=undefined){for(var i=dF.length-1;i>=0;i-- ){var dE=dF[i][1];var dB=dF[i][3];if(dE===dD||dB===dD){this.removeBindingFromObject(dC,dF[i][0]);};};};},getAllBindingsForObject:function(dH){var dI=dH.toHashCode();if(this.__bc[dI]===undefined){this.__bc[dI]=[];};var dJ=this.__bc[dI];var dG=this.__bd[dI]?this.__bd[dI]:[];return qx.lang.Array.unique(dJ.concat(dG));},removeAllBindings:function(){for(var dL in this.__bc){var dK=qx.core.ObjectRegistry.fromHashCode(dL);if(dK==null){delete this.__bc[dL];continue;};this.removeAllBindingsForObject(dK);};this.__bc={};},getAllBindings:function(){return this.__bc;},showBindingInLog:function(dN,dP){var dO;for(var i=0;i<this.__bc[dN.toHashCode()].length;i++ ){if(this.__bc[dN.toHashCode()][i][0]==dP){dO=this.__bc[dN.toHashCode()][i];break;};};if(dO===undefined){var dM=I;}else {var dM=y+dO[1]+G+dO[2]+g+dO[3]+G+dO[4]+c;};qx.log.Logger.debug(dM);},showAllBindingsInLog:function(){for(var dR in this.__bc){var dQ=qx.core.ObjectRegistry.fromHashCode(dR);for(var i=0;i<this.__bc[dR].length;i++ ){this.showBindingInLog(dQ,this.__bc[dR][i][0]);};};}}});})();(function(){var a=": ",b="qx.type.BaseError",c="",d="error";qx.Bootstrap.define(b,{extend:Error,construct:function(e,f){var g=Error.call(this,f);if(g.stack){this.stack=g.stack;};if(g.stacktrace){this.stacktrace=g.stacktrace;};this.__bt=e||c;this.message=f||qx.type.BaseError.DEFAULTMESSAGE;},statics:{DEFAULTMESSAGE:d},members:{__bu:null,__bt:null,message:null,getComment:function(){return this.__bt;},toString:function(){return this.__bt+(this.message?a+this.message:c);}}});})();(function(){var a="qx.core.AssertionError";qx.Bootstrap.define(a,{extend:qx.type.BaseError,construct:function(b,c){qx.type.BaseError.call(this,b,c);this.__bv=qx.dev.StackTrace.getStackTrace();},members:{__bv:null,getStackTrace:function(){return this.__bv;}}});})();(function(){var a="anonymous",b="...",c="qx.dev.StackTrace",d="",e="\n",f="?",g="/source/class/",h="Error created at",j="ecmascript.error.stacktrace",k="Backtrace:",l="stack",m=":",n=".",o="function",p="prototype",q="stacktrace";qx.Bootstrap.define(c,{statics:{FILENAME_TO_CLASSNAME:null,FORMAT_STACKTRACE:null,getStackTrace:function(){var t=[];try{throw new Error();}catch(G){if(qx.dev.StackTrace.hasEnvironmentCheck&&qx.core.Environment.get(j)){var y=qx.dev.StackTrace.getStackTraceFromError(G);var B=qx.dev.StackTrace.getStackTraceFromCaller(arguments);qx.lang.Array.removeAt(y,0);t=B.length>y.length?B:y;for(var i=0;i<Math.min(B.length,y.length);i++ ){var w=B[i];if(w.indexOf(a)>=0){continue;};var s=null;var C=w.split(n);var v=/(.*?)\(/.exec(C[C.length-1]);if(v&&v.length==2){s=v[1];C.pop();};if(C[C.length-1]==p){C.pop();};var E=C.join(n);var u=y[i];var F=u.split(m);var A=F[0];var z=F[1];var r;if(F[2]){r=F[2];};var x=null;if(qx.Class&&qx.Class.getByName(A)){x=A;}else {x=E;};var D=x;if(s){D+=n+s;};D+=m+z;if(r){D+=m+r;};t[i]=D;};}else {t=this.getStackTraceFromCaller(arguments);};};return t;},getStackTraceFromCaller:function(K){var J=[];var M=qx.lang.Function.getCaller(K);var H={};while(M){var L=qx.lang.Function.getName(M);J.push(L);try{M=M.caller;}catch(N){break;};if(!M){break;};var I=qx.core.ObjectRegistry.toHashCode(M);if(H[I]){J.push(b);break;};H[I]=M;};return J;},getStackTraceFromError:function(bd){var T=[];var R,S,ba,Q,P,bf,bb;var bc=qx.dev.StackTrace.hasEnvironmentCheck?qx.core.Environment.get(j):null;if(bc===l){if(!bd.stack){return T;};R=/@(.+):(\d+)$/gm;while((S=R.exec(bd.stack))!=null){bb=S[1];Q=S[2];ba=this.__bw(bb);T.push(ba+m+Q);};if(T.length>0){return this.__by(T);};R=/at (.*)/gm;var be=/\((.*?)(:[^\/].*)\)/;var Y=/(.*?)(:[^\/].*)/;while((S=R.exec(bd.stack))!=null){var X=be.exec(S[1]);if(!X){X=Y.exec(S[1]);};if(X){ba=this.__bw(X[1]);T.push(ba+X[2]);}else {T.push(S[1]);};};}else if(bc===q){var U=bd.stacktrace;if(!U){return T;};if(U.indexOf(h)>=0){U=U.split(h)[0];};R=/line\ (\d+?),\ column\ (\d+?)\ in\ (?:.*?)\ in\ (.*?):[^\/]/gm;while((S=R.exec(U))!=null){Q=S[1];P=S[2];bb=S[3];ba=this.__bw(bb);T.push(ba+m+Q+m+P);};if(T.length>0){return this.__by(T);};R=/Line\ (\d+?)\ of\ linked\ script\ (.*?)$/gm;while((S=R.exec(U))!=null){Q=S[1];bb=S[2];ba=this.__bw(bb);T.push(ba+m+Q);};}else if(bd.message&&bd.message.indexOf(k)>=0){var W=bd.message.split(k)[1].trim();var V=W.split(e);for(var i=0;i<V.length;i++ ){var O=V[i].match(/\s*Line ([0-9]+) of.* (\S.*)/);if(O&&O.length>=2){Q=O[1];bf=this.__bw(O[2]);T.push(bf+m+Q);};};}else if(bd.sourceURL&&bd.line){T.push(this.__bw(bd.sourceURL)+m+bd.line);};return this.__by(T);},__bw:function(bh){if(typeof qx.dev.StackTrace.FILENAME_TO_CLASSNAME==o){var bg=qx.dev.StackTrace.FILENAME_TO_CLASSNAME(bh);{};return bg;};return qx.dev.StackTrace.__bx(bh);},__bx:function(bk){var bl=g;var bi=bk.indexOf(bl);var bm=bk.indexOf(f);if(bm>=0){bk=bk.substring(0,bm);};var bj=(bi==-1)?bk:bk.substring(bi+bl.length).replace(/\//g,n).replace(/\.js$/,d);return bj;},__by:function(bn){if(typeof qx.dev.StackTrace.FORMAT_STACKTRACE==o){bn=qx.dev.StackTrace.FORMAT_STACKTRACE(bn);{};};return bn;}},defer:function(bo){bo.hasEnvironmentCheck=qx.bom&&qx.bom.client&&qx.bom.client.EcmaScript&&qx.bom.client.EcmaScript.getStackTrace;}});})();(function(){var a="mshtml",b="engine.name",c="[object Array]",d="qx.lang.Array",e="Cannot clean-up map entry doneObjects[",f="]",g="qx",h="number",j="][",k="string";qx.Bootstrap.define(d,{statics:{cast:function(m,o,p){if(m.constructor===o){return m;};if(qx.data&&qx.data.IListData){if(qx.Class&&qx.Class.hasInterface(m,qx.data.IListData)){var m=m.toArray();};};var n=new o;if((qx.core.Environment.get(b)==a)){if(m.item){for(var i=p||0,l=m.length;i<l;i++ ){n.push(m[i]);};return n;};};if(Object.prototype.toString.call(m)===c&&p==null){n.push.apply(n,m);}else {n.push.apply(n,Array.prototype.slice.call(m,p||0));};return n;},fromArguments:function(q,r){return Array.prototype.slice.call(q,r||0);},fromCollection:function(t){if((qx.core.Environment.get(b)==a)){if(t.item){var s=[];for(var i=0,l=t.length;i<l;i++ ){s[i]=t[i];};return s;};};return Array.prototype.slice.call(t,0);},fromShortHand:function(u){var w=u.length;var v=qx.lang.Array.clone(u);switch(w){case 1:v[1]=v[2]=v[3]=v[0];break;case 2:v[2]=v[0];case 3:v[3]=v[1];};return v;},clone:function(x){return x.concat();},insertAt:function(y,z,i){y.splice(i,0,z);return y;},insertBefore:function(A,C,B){var i=A.indexOf(B);if(i==-1){A.push(C);}else {A.splice(i,0,C);};return A;},insertAfter:function(D,F,E){var i=D.indexOf(E);if(i==-1||i==(D.length-1)){D.push(F);}else {D.splice(i+1,0,F);};return D;},removeAt:function(G,i){return G.splice(i,1)[0];},removeAll:function(H){H.length=0;return this;},append:function(J,I){{};Array.prototype.push.apply(J,I);return J;},exclude:function(M,L){{};for(var i=0,N=L.length,K;i<N;i++ ){K=M.indexOf(L[i]);if(K!=-1){M.splice(K,1);};};return M;},remove:function(O,P){var i=O.indexOf(P);if(i!=-1){O.splice(i,1);return P;};},contains:function(Q,R){return Q.indexOf(R)!==-1;},equals:function(T,S){var length=T.length;if(length!==S.length){return false;};for(var i=0;i<length;i++ ){if(T[i]!==S[i]){return false;};};return true;},sum:function(U){var V=0;for(var i=0,l=U.length;i<l;i++ ){if(U[i]!=undefined){V+=U[i];};};return V;},max:function(W){{};var i,Y=W.length,X=W[0];for(i=1;i<Y;i++ ){if(W[i]>X){X=W[i];};};return X===undefined?null:X;},min:function(ba){{};var i,bc=ba.length,bb=ba[0];for(i=1;i<bc;i++ ){if(ba[i]<bb){bb=ba[i];};};return bb===undefined?null:bb;},unique:function(bf){var bp=[],be={},bi={},bk={};var bj,bd=0;var bn=g+Date.now();var bg=false,bl=false,bo=false;for(var i=0,bm=bf.length;i<bm;i++ ){bj=bf[i];if(bj===null){if(!bg){bg=true;bp.push(bj);};}else if(bj===undefined){}else if(bj===false){if(!bl){bl=true;bp.push(bj);};}else if(bj===true){if(!bo){bo=true;bp.push(bj);};}else if(typeof bj===k){if(!be[bj]){be[bj]=1;bp.push(bj);};}else if(typeof bj===h){if(!bi[bj]){bi[bj]=1;bp.push(bj);};}else {var bh=bj[bn];if(bh==null){bh=bj[bn]=bd++ ;};if(!bk[bh]){bk[bh]=bj;bp.push(bj);};};};for(var bh in bk){try{delete bk[bh][bn];}catch(bq){try{bk[bh][bn]=null;}catch(br){throw new Error(e+bh+j+bn+f);};};};return bp;},range:function(bu,stop,bv){if(arguments.length<=1){stop=bu||0;bu=0;};bv=arguments[2]||1;var length=Math.max(Math.ceil((stop-bu)/bv),0);var bs=0;var bt=Array(length);while(bs<length){bt[bs++ ]=bu;bu+=bv;};return bt;}}});})();(function(){var a="[object Opera]",b="function",c="[^\\.0-9]",d="4.0",e="gecko",f="1.9.0.0",g="Version/",h="9.0",i="8.0",j="engine.version",k="Gecko",l="Maple",m="AppleWebKit/",n="Trident",o="Unsupported client: ",p="",q="opera",r="Windows Phone",s="! Assumed gecko version 1.9.0.0 (Firefox 3.0).",t="mshtml",u="engine.name",v="webkit",w="5.0",x=".",y="qx.bom.client.Engine";qx.Bootstrap.define(y,{statics:{getVersion:function(){var B=window.navigator.userAgent;var C=p;if(qx.bom.client.Engine.__bD()){var A=/Trident\/([^\);]+)(\)|;)/.test(B);if(/MSIE\s+([^\);]+)(\)|;)/.test(B)){C=RegExp.$1;if(C<8&&A){if(RegExp.$1==d){C=i;}else if(RegExp.$1==w){C=h;};};}else if(A){var E=/\brv\:(\d+?\.\d+?)\b/.exec(B);if(E){C=E[1];};};}else if(qx.bom.client.Engine.__bz()){if(/Opera[\s\/]([0-9]+)\.([0-9])([0-9]*)/.test(B)){if(B.indexOf(g)!=-1){var E=B.match(/Version\/(\d+)\.(\d+)/);C=E[1]+x+E[2].charAt(0)+x+E[2].substring(1,E[2].length);}else {C=RegExp.$1+x+RegExp.$2;if(RegExp.$3!=p){C+=x+RegExp.$3;};};};}else if(qx.bom.client.Engine.__bA()){if(/AppleWebKit\/([^ ]+)/.test(B)){C=RegExp.$1;var D=RegExp(c).exec(C);if(D){C=C.slice(0,D.index);};};}else if(qx.bom.client.Engine.__bC()||qx.bom.client.Engine.__bB()){if(/rv\:([^\);]+)(\)|;)/.test(B)){C=RegExp.$1;};}else {var z=window.qxFail;if(z&&typeof z===b){C=z().FULLVERSION;}else {C=f;qx.Bootstrap.warn(o+B+s);};};return C;},getName:function(){var name;if(qx.bom.client.Engine.__bD()){name=t;}else if(qx.bom.client.Engine.__bz()){name=q;}else if(qx.bom.client.Engine.__bA()){name=v;}else if(qx.bom.client.Engine.__bC()||qx.bom.client.Engine.__bB()){name=e;}else {var F=window.qxFail;if(F&&typeof F===b){name=F().NAME;}else {name=e;qx.Bootstrap.warn(o+window.navigator.userAgent+s);};};return name;},__bz:function(){return window.opera&&Object.prototype.toString.call(window.opera)==a;},__bA:function(){return window.navigator.userAgent.indexOf(m)!=-1;},__bB:function(){return window.navigator.userAgent.indexOf(l)!=-1;},__bC:function(){return window.navigator.mozApps&&window.navigator.product===k&&window.navigator.userAgent.indexOf(l)==-1&&window.navigator.userAgent.indexOf(n)==-1;},__bD:function(){if(window.navigator.cpuClass&&(/MSIE\s+([^\);]+)(\)|;)/.test(window.navigator.userAgent)||/Trident\/\d+?\.\d+?/.test(window.navigator.userAgent))){return true;};if(qx.bom.client.Engine.__bE()){return true;};return false;},__bE:function(){return window.navigator.userAgent.indexOf(r)>-1;}},defer:function(G){qx.core.Environment.add(j,G.getVersion);qx.core.Environment.add(u,G.getName);}});})();(function(){var a='anonymous()',b="()",c="qx.globalErrorHandling",d="qx.lang.Function",e=".",f=".prototype.",g=".constructor()";qx.Bootstrap.define(d,{statics:{getCaller:function(h){return h.caller?h.caller.callee:h.callee.caller;},getName:function(i){if(i.displayName){return i.displayName;};if(i.$$original||i.wrapper||i.classname){return i.classname+g;};if(i.$$mixin){for(var j in i.$$mixin.$$members){if(i.$$mixin.$$members[j]==i){return i.$$mixin.name+f+j+b;};};for(var j in i.$$mixin){if(i.$$mixin[j]==i){return i.$$mixin.name+e+j+b;};};};if(i.self){var l=i.self.constructor;if(l){for(var j in l.prototype){if(l.prototype[j]==i){return l.classname+f+j+b;};};for(var j in l){if(l[j]==i){return l.classname+e+j+b;};};};};var k=i.toString().match(/function\s*(\w*)\s*\(.*/);if(k&&k.length>=1&&k[1]){return k[1]+b;};return a;},globalEval:function(data){if(window.execScript){return window.execScript(data);}else {return eval.call(window,data);};},create:function(n,m){{};if(!m){return n;};if(!(m.self||m.args||m.delay!=null||m.periodical!=null||m.attempt)){return n;};return function(event){{};var p=qx.lang.Array.fromArguments(arguments);if(m.args){p=m.args.concat(p);};if(m.delay||m.periodical){var o=function(){return n.apply(m.self||this,p);};if(qx.core.Environment.get(c)){o=qx.event.GlobalError.observeMethod(o);};if(m.delay){return window.setTimeout(o,m.delay);};if(m.periodical){return window.setInterval(o,m.periodical);};}else if(m.attempt){var q=false;try{q=n.apply(m.self||this,p);}catch(r){};return q;}else {return n.apply(m.self||this,p);};};},bind:function(s,self,t){return this.create(s,{self:self,args:arguments.length>2?qx.lang.Array.fromArguments(arguments,2):null});},curry:function(u,v){return this.create(u,{args:arguments.length>1?qx.lang.Array.fromArguments(arguments,1):null});},listener:function(x,self,y){if(arguments.length<3){return function(event){return x.call(self||this,event||window.event);};}else {var w=qx.lang.Array.fromArguments(arguments,2);return function(event){var z=[event||window.event];z.push.apply(z,w);x.apply(self||this,z);};};},attempt:function(A,self,B){return this.create(A,{self:self,attempt:true,args:arguments.length>2?qx.lang.Array.fromArguments(arguments,2):null})();},delay:function(D,C,self,E){return this.create(D,{delay:C,self:self,args:arguments.length>3?qx.lang.Array.fromArguments(arguments,3):null})();},periodical:function(G,F,self,H){return this.create(G,{periodical:F,self:self,args:arguments.length>3?qx.lang.Array.fromArguments(arguments,3):null})();}}});})();(function(){var a="qx.globalErrorHandling",b="qx.event.GlobalError";qx.Bootstrap.define(b,{statics:{__bF:null,__bG:null,__bH:null,__bI:function(){if(qx.core&&qx.core.Environment){return qx.core.Environment.get(a);}else {return !!qx.Bootstrap.getEnvironmentSetting(a);};},setErrorHandler:function(c,d){this.__bF=c||null;this.__bH=d||window;if(this.__bI()){if(c&&window.onerror){var e=qx.Bootstrap.bind(this.__bJ,this);if(this.__bG==null){this.__bG=window.onerror;};var self=this;window.onerror=function(f,g,h){self.__bG(f,g,h);e(f,g,h);};};if(c&&!window.onerror){window.onerror=qx.Bootstrap.bind(this.__bJ,this);};if(this.__bF==null){if(this.__bG!=null){window.onerror=this.__bG;this.__bG=null;}else {window.onerror=null;};};};},__bJ:function(i,j,k){if(this.__bF){this.handleError(new qx.core.WindowError(i,j,k));};},observeMethod:function(l){if(this.__bI()){var self=this;return function(){if(!self.__bF){return l.apply(this,arguments);};try{return l.apply(this,arguments);}catch(m){self.handleError(new qx.core.GlobalError(m,arguments));};};}else {return l;};},handleError:function(n){if(this.__bF){this.__bF.call(this.__bH,n);};}},defer:function(o){if(qx.core&&qx.core.Environment){qx.core.Environment.add(a,true);}else {qx.Bootstrap.setEnvironmentSetting(a,true);};o.setErrorHandler(null,null);}});})();(function(){var a="",b="qx.core.WindowError";qx.Bootstrap.define(b,{extend:Error,construct:function(c,e,f){var d=Error.call(this,c);if(d.stack){this.stack=d.stack;};if(d.stacktrace){this.stacktrace=d.stacktrace;};this.__bK=c;this.__bL=e||a;this.__bM=f===undefined?-1:f;},members:{__bK:null,__bL:null,__bM:null,toString:function(){return this.__bK;},getUri:function(){return this.__bL;},getLineNumber:function(){return this.__bM;}}});})();(function(){var a="GlobalError: ",b="qx.core.GlobalError";qx.Bootstrap.define(b,{extend:Error,construct:function(e,c){if(qx.Bootstrap.DEBUG){qx.core.Assert.assertNotUndefined(e);};this.__bK=a+(e&&e.message?e.message:e);var d=Error.call(this,this.__bK);if(d.stack){this.stack=d.stack;};if(d.stacktrace){this.stacktrace=d.stacktrace;};this.__bN=c;this.__bO=e;},members:{__bO:null,__bN:null,__bK:null,toString:function(){return this.__bK;},getArguments:function(){return this.__bN;},getSourceException:function(){return this.__bO;}},destruct:function(){this.__bO=null;this.__bN=null;this.__bK=null;}});})();(function(){var a="qx.lang.Type",b="Error",c="RegExp",d="Date",e="Number",f="Boolean";qx.Bootstrap.define(a,{statics:{getClass:qx.Bootstrap.getClass,isString:qx.Bootstrap.isString,isArray:qx.Bootstrap.isArray,isObject:qx.Bootstrap.isObject,isFunction:qx.Bootstrap.isFunction,isRegExp:function(g){return this.getClass(g)==c;},isNumber:function(h){return (h!==null&&(this.getClass(h)==e||h instanceof Number));},isBoolean:function(i){return (i!==null&&(this.getClass(i)==f||i instanceof Boolean));},isDate:function(j){return (j!==null&&(this.getClass(j)==d||j instanceof Date));},isError:function(k){return (k!==null&&(this.getClass(k)==b||k instanceof Error));}}});})();(function(){var a=" != ",b="qx.core.Object",c="Expected value to be an array but found ",d="' (rgb(",f=") was fired.",g="Expected value to be an integer >= 0 but found ",h="' to be not equal with '",j="' to '",k="Expected object '",m="Called assertTrue with '",n="Expected value to be a map but found ",o="The function did not raise an exception!",p="Expected value to be undefined but found ",q="Expected value to be a DOM element but found  '",r="Expected value to be a regular expression but found ",s="' to implement the interface '",t="Expected value to be null but found ",u="Invalid argument 'type'",v="Called assert with 'false'",w="Assertion error! ",x="'",y="null",z="' but found '",A="'undefined'",B=",",C="' must must be a key of the map '",D="Expected '",E="The String '",F="Expected value to be a string but found ",G="Event (",H="Expected value to be the CSS color '",I="!",J="Expected value not to be undefined but found undefined!",K="qx.util.ColorUtil",L=": ",M="The raised exception does not have the expected type! ",N=") not fired.",O="'!",P="qx.core.Assert",Q="",R="Expected value to be typeof object but found ",S="' but found ",T="' (identical) but found '",U="' must have any of the values defined in the array '",V="Expected value to be a number but found ",W="Called assertFalse with '",X="qx.ui.core.Widget",Y="]",bJ="Expected value to be a qooxdoo object but found ",bK="' arguments.",bL="Expected value '%1' to be in the range '%2'..'%3'!",bF="Array[",bG="' does not match the regular expression '",bH="' to be not identical with '",bI="Expected [",bP="' arguments but found '",bQ="', which cannot be converted to a CSS color!",bR=", ",cg="qx.core.AssertionError",bM="Expected value to be a boolean but found ",bN="Expected value not to be null but found null!",bO="))!",bD="Expected value to be a qooxdoo widget but found ",bU="The value '",bE="Expected value to be typeof '",bV="\n Stack trace: \n",bW="Expected value to be typeof function but found ",cb="Expected value to be an integer but found ",bS="Called fail().",cf="The parameter 're' must be a string or a regular expression.",bT=")), but found value '",bX="qx.util.ColorUtil not available! Your code must have a dependency on 'qx.util.ColorUtil'",bY="Expected value to be a number >= 0 but found ",ca="Expected value to be instanceof '",cc="], but found [",cd="Wrong number of arguments given. Expected '",ce="object";qx.Bootstrap.define(P,{statics:{__bP:true,__bQ:function(ch,ci){var cm=Q;for(var i=1,l=arguments.length;i<l;i++ ){cm=cm+this.__bR(arguments[i]===undefined?A:arguments[i]);};var cl=Q;if(cm){cl=ch+L+cm;}else {cl=ch;};var ck=w+cl;if(qx.Class&&qx.Class.isDefined(cg)){var cj=new qx.core.AssertionError(ch,cm);if(this.__bP){qx.Bootstrap.error(ck+bV+cj.getStackTrace());};throw cj;}else {if(this.__bP){qx.Bootstrap.error(ck);};throw new Error(ck);};},__bR:function(co){var cn;if(co===null){cn=y;}else if(qx.lang.Type.isArray(co)&&co.length>10){cn=bF+co.length+Y;}else if((co instanceof Object)&&(co.toString==null)){cn=qx.lang.Json.stringify(co,null,2);}else {try{cn=co.toString();}catch(e){cn=Q;};};return cn;},assert:function(cq,cp){cq==true||this.__bQ(cp||Q,v);},fail:function(cr,cs){var ct=cs?Q:bS;this.__bQ(cr||Q,ct);},assertTrue:function(cv,cu){(cv===true)||this.__bQ(cu||Q,m,cv,x);},assertFalse:function(cx,cw){(cx===false)||this.__bQ(cw||Q,W,cx,x);},assertEquals:function(cy,cz,cA){cy==cz||this.__bQ(cA||Q,D,cy,z,cz,O);},assertNotEquals:function(cB,cC,cD){cB!=cC||this.__bQ(cD||Q,D,cB,h,cC,O);},assertIdentical:function(cE,cF,cG){cE===cF||this.__bQ(cG||Q,D,cE,T,cF,O);},assertNotIdentical:function(cH,cI,cJ){cH!==cI||this.__bQ(cJ||Q,D,cH,bH,cI,O);},assertNotUndefined:function(cL,cK){cL!==undefined||this.__bQ(cK||Q,J);},assertUndefined:function(cN,cM){cN===undefined||this.__bQ(cM||Q,p,cN,I);},assertNotNull:function(cP,cO){cP!==null||this.__bQ(cO||Q,bN);},assertNull:function(cR,cQ){cR===null||this.__bQ(cQ||Q,t,cR,I);},assertJsonEquals:function(cS,cT,cU){this.assertEquals(qx.lang.Json.stringify(cS),qx.lang.Json.stringify(cT),cU);},assertMatch:function(cX,cW,cV){this.assertString(cX);this.assert(qx.lang.Type.isRegExp(cW)||qx.lang.Type.isString(cW),cf);cX.search(cW)>=0||this.__bQ(cV||Q,E,cX,bG,cW.toString(),O);},assertArgumentsCount:function(db,dc,dd,cY){var da=db.length;(da>=dc&&da<=dd)||this.__bQ(cY||Q,cd,dc,j,dd,bP,da,bK);},assertEventFired:function(de,event,dh,di,dj){var df=false;var dg=function(e){if(di){di.call(de,e);};df=true;};var dk;try{dk=de.addListener(event,dg,de);dh.call(de);}catch(dl){throw dl;}finally{try{de.removeListenerById(dk);}catch(dm){};};df===true||this.__bQ(dj||Q,G,event,N);},assertEventNotFired:function(dn,event,dr,ds){var dp=false;var dq=function(e){dp=true;};var dt=dn.addListener(event,dq,dn);dr.call();dp===false||this.__bQ(ds||Q,G,event,f);dn.removeListenerById(dt);},assertException:function(dx,dw,dv,du){var dw=dw||Error;var dy;try{this.__bP=false;dx();}catch(dz){dy=dz;}finally{this.__bP=true;};if(dy==null){this.__bQ(du||Q,o);};dy instanceof dw||this.__bQ(du||Q,M,dw,a,dy);if(dv){this.assertMatch(dy.toString(),dv,du);};},assertInArray:function(dC,dB,dA){dB.indexOf(dC)!==-1||this.__bQ(dA||Q,bU,dC,U,dB,x);},assertArrayEquals:function(dD,dE,dF){this.assertArray(dD,dF);this.assertArray(dE,dF);dF=dF||bI+dD.join(bR)+cc+dE.join(bR)+Y;if(dD.length!==dE.length){this.fail(dF,true);};for(var i=0;i<dD.length;i++ ){if(dD[i]!==dE[i]){this.fail(dF,true);};};},assertKeyInMap:function(dI,dH,dG){dH[dI]!==undefined||this.__bQ(dG||Q,bU,dI,C,dH,x);},assertFunction:function(dK,dJ){qx.lang.Type.isFunction(dK)||this.__bQ(dJ||Q,bW,dK,I);},assertString:function(dM,dL){qx.lang.Type.isString(dM)||this.__bQ(dL||Q,F,dM,I);},assertBoolean:function(dO,dN){qx.lang.Type.isBoolean(dO)||this.__bQ(dN||Q,bM,dO,I);},assertNumber:function(dQ,dP){(qx.lang.Type.isNumber(dQ)&&isFinite(dQ))||this.__bQ(dP||Q,V,dQ,I);},assertPositiveNumber:function(dS,dR){(qx.lang.Type.isNumber(dS)&&isFinite(dS)&&dS>=0)||this.__bQ(dR||Q,bY,dS,I);},assertInteger:function(dU,dT){(qx.lang.Type.isNumber(dU)&&isFinite(dU)&&dU%1===0)||this.__bQ(dT||Q,cb,dU,I);},assertPositiveInteger:function(dX,dV){var dW=(qx.lang.Type.isNumber(dX)&&isFinite(dX)&&dX%1===0&&dX>=0);dW||this.__bQ(dV||Q,g,dX,I);},assertInRange:function(eb,ec,ea,dY){(eb>=ec&&eb<=ea)||this.__bQ(dY||Q,qx.lang.String.format(bL,[eb,ec,ea]));},assertObject:function(ee,ed){var ef=ee!==null&&(qx.lang.Type.isObject(ee)||typeof ee===ce);ef||this.__bQ(ed||Q,R,(ee),I);},assertArray:function(eh,eg){qx.lang.Type.isArray(eh)||this.__bQ(eg||Q,c,eh,I);},assertMap:function(ej,ei){qx.lang.Type.isObject(ej)||this.__bQ(ei||Q,n,ej,I);},assertRegExp:function(el,ek){qx.lang.Type.isRegExp(el)||this.__bQ(ek||Q,r,el,I);},assertType:function(eo,en,em){this.assertString(en,u);typeof (eo)===en||this.__bQ(em||Q,bE,en,S,eo,I);},assertInstance:function(er,es,ep){var eq=es.classname||es+Q;er instanceof es||this.__bQ(ep||Q,ca,eq,S,er,I);},assertInterface:function(ev,eu,et){qx.Class&&qx.Class.implementsInterface(ev,eu)||this.__bQ(et||Q,k,ev,s,eu,O);},assertCssColor:function(eC,ez,eB){var ew=qx.Class?qx.Class.getByName(K):null;if(!ew){throw new Error(bX);};var ey=ew.stringToRgb(eC);try{var eA=ew.stringToRgb(ez);}catch(eE){this.__bQ(eB||Q,H,eC,d,ey.join(B),bT,ez,bQ);};var eD=ey[0]==eA[0]&&ey[1]==eA[1]&&ey[2]==eA[2];eD||this.__bQ(eB||Q,H,ey,d,ey.join(B),bT,ez,d,eA.join(B),bO);},assertElement:function(eG,eF){!!(eG&&eG.nodeType===1)||this.__bQ(eF||Q,q,eG,O);},assertQxObject:function(eI,eH){this.__bS(eI,b)||this.__bQ(eH||Q,bJ,eI,I);},assertQxWidget:function(eK,eJ){this.__bS(eK,X)||this.__bQ(eJ||Q,bD,eK,I);},__bS:function(eM,eL){if(!eM){return false;};var eN=eM.constructor;while(eN){if(eN.classname===eL){return true;};eN=eN.superclass;};return false;}}});})();(function(){var a="\x00\b\n\f\r\t",b="-",c="function",d="[null,null,null]",e="T",f="+",g=",\n",h="constructor",i="{\n",j='"+275760-09-13T00:00:00.000Z"',k="true",l="\\n",m="false",n='"-271821-04-20T00:00:00.000Z"',o="json",p='object',q='""',r="qx.lang.Json",s="{}",t="hasOwnProperty",u="@",v="prototype",w='hasOwnProperty',x='"',y="toLocaleString",z="0",A='function',B="",C='\\"',D="\t",E="string",F="}",G="\r",H="toJSON",I=":",J="[\n 1,\n 2\n]",K="\\f",L='"1969-12-31T23:59:59.999Z"',M="/",N="\\b",O="Z",P="\\t",Q="\b",R="[object Number]",S="isPrototypeOf",T="{",U="toString",V="0x",W="[1]",X="\\r",Y="]",bO=",",bP="null",bQ="\\u00",bK="\n",bL="json-stringify",bM="[]",bN="1",bU="000000",bV="[object Boolean]",bW="valueOf",cm="\\\\",bR="[object String]",bS="json-parse",bT="bug-string-char-index",bG="[object Array]",ca="$",bJ="[\n",cb='"-000001-01-01T00:00:00.000Z"',cc="[",bI="[null]",bX="\\",cl="[object Date]",bY='{"a":[1,true,false,null,"\\u0000\\b\\n\\f\\r\\t"]}',cd="a",ce=" ",cf=".",ci="[object Function]",cj="01",ck='"\t"',bH="propertyIsEnumerable",cg="\f",ch="object";qx.Bootstrap.define(r,{statics:{stringify:null,parse:null}});(function(){var co;var cn;var cp;(function(window){var cr={}.toString,cG,cQ,cC;var cy=typeof cp===c&&cp.amd,cx=typeof cn==ch&&cn;if(cx||cy){if(typeof JSON==ch&&JSON){if(cx){cx.stringify=JSON.stringify;cx.parse=JSON.parse;}else {cx=JSON;};}else if(cy){cx=window.JSON={};};}else {cx=window.JSON||(window.JSON={});};var cU=new Date(-3509827334573292);try{cU=cU.getUTCFullYear()==-109252&&cU.getUTCMonth()===0&&cU.getUTCDate()===1&&cU.getUTCHours()==10&&cU.getUTCMinutes()==37&&cU.getUTCSeconds()==6&&cU.getUTCMilliseconds()==708;}catch(da){};function cJ(name){if(name==bT){return cd[0]!=cd;};var de,dd=bY,dh=name==o;if(dh||name==bL||name==bS){if(name==bL||dh){var db=cx.stringify,dg=typeof db==c&&cU;if(dg){(de=function(){return 1;}).toJSON=de;try{dg=db(0)===z&&db(new Number())===z&&db(new String())==q&&db(cr)===cC&&db(cC)===cC&&db()===cC&&db(de)===bN&&db([de])==W&&db([cC])==bI&&db(null)==bP&&db([cC,cr,null])==d&&db({"a":[de,true,false,null,a]})==dd&&db(null,de)===bN&&db([1,2],null,1)==J&&db(new Date(-8.64e15))==n&&db(new Date(8.64e15))==j&&db(new Date(-621987552e5))==cb&&db(new Date(-1))==L;}catch(di){dg=false;};};if(!dh){return dg;};};if(name==bS||dh){var df=cx.parse;if(typeof df==c){try{if(df(z)===0&&!df(false)){de=df(dd);var dc=de[cd].length==5&&de[cd][0]===1;if(dc){try{dc=!df(ck);}catch(dj){};if(dc){try{dc=df(cj)!==1;}catch(dk){};};};};}catch(dl){dc=false;};};if(!dh){return dc;};};return dg&&dc;};};if(!cJ(o)){var cV=ci;var cN=cl;var cv=R;var cY=bR;var cR=bG;var cF=bV;var cE=cJ(bT);if(!cU){var cD=Math.floor;var cM=[0,31,59,90,120,151,181,212,243,273,304,334];var cX=function(dm,dn){return cM[dn]+365*(dm-1970)+cD((dm-1969+(dn=+(dn>1)))/4)-cD((dm-1901+dn)/100)+cD((dm-1601+dn)/400);};};if(!(cG={}.hasOwnProperty)){cG=function(dp){var dq={},dr;if((dq.__bT=null,dq.__bT={"toString":1},dq).toString!=cr){cG=function(ds){var dt=this.__bT,du=ds in (this.__bT=null,this);this.__bT=dt;return du;};}else {dr=dq.constructor;cG=function(dv){var parent=(this.constructor||dr).prototype;return dv in this&&!(dv in parent&&this[dv]===parent[dv]);};};dq=null;return cG.call(this,dp);};};var cH={'boolean':1,'number':1,'string':1,'undefined':1};var cP=function(dy,dw){var dx=typeof dy[dw];return dx==p?!!dy[dw]:!cH[dx];};cQ=function(dz,dA){var dF=0,dE,dC,dD,dB;(dE=function(){this.valueOf=0;}).prototype.valueOf=0;dC=new dE();for(dD in dC){if(cG.call(dC,dD)){dF++ ;};};dE=dC=null;if(!dF){dC=[bW,U,y,bH,S,t,h];dB=function(dH,dI){var dJ=cr.call(dH)==cV,dK,length;var dG=!dJ&&typeof dH.constructor!=A&&cP(dH,w)?dH.hasOwnProperty:cG;for(dK in dH){if(!(dJ&&dK==v)&&dG.call(dH,dK)){dI(dK);};};for(length=dC.length;dK=dC[ --length];dG.call(dH,dK)&&dI(dK));};}else if(dF==2){dB=function(dP,dL){var dO={},dM=cr.call(dP)==cV,dN;for(dN in dP){if(!(dM&&dN==v)&&!cG.call(dO,dN)&&(dO[dN]=1)&&cG.call(dP,dN)){dL(dN);};};};}else {dB=function(dT,dQ){var dR=cr.call(dT)==cV,dS,dU;for(dS in dT){if(!(dR&&dS==v)&&cG.call(dT,dS)&&!(dU=dS===h)){dQ(dS);};};if(dU||cG.call(dT,(dS=h))){dQ(dS);};};};return dB(dz,dA);};if(!cJ(bL)){var cT={'92':cm,'34':C,'8':N,'12':K,'10':l,'13':X,'9':P};var cI=bU;var cW=function(dV,dW){return (cI+(dW||0)).slice(-dV);};var cB=bQ;var cL=function(dY){var eb=x,dX=0,length=dY.length,ec=length>10&&cE,ea;if(ec){ea=dY.split(B);};for(;dX<length;dX++ ){var ed=dY.charCodeAt(dX);switch(ed){case 8:case 9:case 10:case 12:case 13:case 34:case 92:eb+=cT[ed];break;default:if(ed<32){eb+=cB+cW(2,ed.toString(16));break;};eb+=ec?ea[dX]:cE?dY.charAt(dX):dY[dX];};};return eb+x;};var cs=function(ez,eo,ew,el,ek,ex,es){var et=eo[ez],ev,ei,ef,er,ey,ep,eA,en,em,ee,eu,ej,length,eg,eq,eh;try{et=eo[ez];}catch(eB){};if(typeof et==ch&&et){ev=cr.call(et);if(ev==cN&&!cG.call(et,H)){if(et>-1/0&&et<1/0){if(cX){er=cD(et/864e5);for(ei=cD(er/365.2425)+1970-1;cX(ei+1,0)<=er;ei++ );for(ef=cD((er-cX(ei,0))/30.42);cX(ei,ef+1)<=er;ef++ );er=1+er-cX(ei,ef);ey=(et%864e5+864e5)%864e5;ep=cD(ey/36e5)%24;eA=cD(ey/6e4)%60;en=cD(ey/1e3)%60;em=ey%1e3;}else {ei=et.getUTCFullYear();ef=et.getUTCMonth();er=et.getUTCDate();ep=et.getUTCHours();eA=et.getUTCMinutes();en=et.getUTCSeconds();em=et.getUTCMilliseconds();};et=(ei<=0||ei>=1e4?(ei<0?b:f)+cW(6,ei<0?-ei:ei):cW(4,ei))+b+cW(2,ef+1)+b+cW(2,er)+e+cW(2,ep)+I+cW(2,eA)+I+cW(2,en)+cf+cW(3,em)+O;}else {et=null;};}else if(typeof et.toJSON==c&&((ev!=cv&&ev!=cY&&ev!=cR)||cG.call(et,H))){et=et.toJSON(ez);};};if(ew){et=ew.call(eo,ez,et);};if(et===null){return bP;};ev=cr.call(et);if(ev==cF){return B+et;}else if(ev==cv){return et>-1/0&&et<1/0?B+et:bP;}else if(ev==cY){return cL(B+et);};if(typeof et==ch){for(length=es.length;length-- ;){if(es[length]===et){throw TypeError();};};es.push(et);ee=[];eg=ex;ex+=ek;if(ev==cR){for(ej=0,length=et.length;ej<length;eq||(eq=true),ej++ ){eu=cs(ej,et,ew,el,ek,ex,es);ee.push(eu===cC?bP:eu);};eh=eq?(ek?bJ+ex+ee.join(g+ex)+bK+eg+Y:(cc+ee.join(bO)+Y)):bM;}else {cQ(el||et,function(eC){var eD=cs(eC,et,ew,el,ek,ex,es);if(eD!==cC){ee.push(cL(eC)+I+(ek?ce:B)+eD);};eq||(eq=true);});eh=eq?(ek?i+ex+ee.join(g+ex)+bK+eg+F:(T+ee.join(bO)+F)):s;};es.pop();return eh;};};cx.stringify=function(eK,eJ,eL){var eF,eG,eI;if(typeof eJ==c||typeof eJ==ch&&eJ){if(cr.call(eJ)==cV){eG=eJ;}else if(cr.call(eJ)==cR){eI={};for(var eE=0,length=eJ.length,eH;eE<length;eH=eJ[eE++ ],((cr.call(eH)==cY||cr.call(eH)==cv)&&(eI[eH]=1)));};};if(eL){if(cr.call(eL)==cv){if((eL-=eL%1)>0){for(eF=B,eL>10&&(eL=10);eF.length<eL;eF+=ce);};}else if(cr.call(eL)==cY){eF=eL.length<=10?eL:eL.slice(0,10);};};return cs(B,(eH={},eH[B]=eK,eH),eG,eI,eF,B,[]);};};if(!cJ(bS)){var cA=String.fromCharCode;var cz={'92':bX,'34':x,'47':M,'98':Q,'116':D,'110':bK,'102':cg,'114':G};var cq,cu;var cw=function(){cq=cu=null;throw SyntaxError();};var cS=function(){var eO=cu,length=eO.length,eN,eM,eQ,eP,eR;while(cq<length){eR=eO.charCodeAt(cq);switch(eR){case 9:case 10:case 13:case 32:cq++ ;break;case 123:case 125:case 91:case 93:case 58:case 44:eN=cE?eO.charAt(cq):eO[cq];cq++ ;return eN;case 34:for(eN=u,cq++ ;cq<length;){eR=eO.charCodeAt(cq);if(eR<32){cw();}else if(eR==92){eR=eO.charCodeAt( ++cq);switch(eR){case 92:case 34:case 47:case 98:case 116:case 110:case 102:case 114:eN+=cz[eR];cq++ ;break;case 117:eM= ++cq;for(eQ=cq+4;cq<eQ;cq++ ){eR=eO.charCodeAt(cq);if(!(eR>=48&&eR<=57||eR>=97&&eR<=102||eR>=65&&eR<=70)){cw();};};eN+=cA(V+eO.slice(eM,cq));break;default:cw();};}else {if(eR==34){break;};eR=eO.charCodeAt(cq);eM=cq;while(eR>=32&&eR!=92&&eR!=34){eR=eO.charCodeAt( ++cq);};eN+=eO.slice(eM,cq);};};if(eO.charCodeAt(cq)==34){cq++ ;return eN;};cw();default:eM=cq;if(eR==45){eP=true;eR=eO.charCodeAt( ++cq);};if(eR>=48&&eR<=57){if(eR==48&&((eR=eO.charCodeAt(cq+1)),eR>=48&&eR<=57)){cw();};eP=false;for(;cq<length&&((eR=eO.charCodeAt(cq)),eR>=48&&eR<=57);cq++ );if(eO.charCodeAt(cq)==46){eQ= ++cq;for(;eQ<length&&((eR=eO.charCodeAt(eQ)),eR>=48&&eR<=57);eQ++ );if(eQ==cq){cw();};cq=eQ;};eR=eO.charCodeAt(cq);if(eR==101||eR==69){eR=eO.charCodeAt( ++cq);if(eR==43||eR==45){cq++ ;};for(eQ=cq;eQ<length&&((eR=eO.charCodeAt(eQ)),eR>=48&&eR<=57);eQ++ );if(eQ==cq){cw();};cq=eQ;};return +eO.slice(eM,cq);};if(eP){cw();};if(eO.slice(cq,cq+4)==k){cq+=4;return true;}else if(eO.slice(cq,cq+5)==m){cq+=5;return false;}else if(eO.slice(cq,cq+4)==bP){cq+=4;return null;};cw();};};return ca;};var cK=function(eU){var eT,eS;if(eU==ca){cw();};if(typeof eU==E){if((cE?eU.charAt(0):eU[0])==u){return eU.slice(1);};if(eU==cc){eT=[];for(;;eS||(eS=true)){eU=cS();if(eU==Y){break;};if(eS){if(eU==bO){eU=cS();if(eU==Y){cw();};}else {cw();};};if(eU==bO){cw();};eT.push(cK(eU));};return eT;}else if(eU==T){eT={};for(;;eS||(eS=true)){eU=cS();if(eU==F){break;};if(eS){if(eU==bO){eU=cS();if(eU==F){cw();};}else {cw();};};if(eU==bO||typeof eU!=E||(cE?eU.charAt(0):eU[0])!=u||cS()!=I){cw();};eT[eU.slice(1)]=cK(cS());};return eT;};cw();};return eU;};var cO=function(eV,eW,eX){var eY=ct(eV,eW,eX);if(eY===cC){delete eV[eW];}else {eV[eW]=eY;};};var ct=function(fa,fb,fd){var fc=fa[fb],length;if(typeof fc==ch&&fc){if(cr.call(fc)==cR){for(length=fc.length;length-- ;){cO(fc,length,fd);};}else {cQ(fc,function(fe){cO(fc,fe,fd);});};};return fd.call(fa,fb,fc);};cx.parse=function(ff,fi){var fg,fh;cq=0;cu=B+ff;fg=cK(cS());if(cS()!=ca){cw();};cq=cu=null;return fi&&cr.call(fi)==cV?ct((fh={},fh[B]=fg,fh),B,fi):fg;};};};if(cy){cp(function(){return cx;});};}(this));}());qx.lang.Json.stringify=window.JSON.stringify;qx.lang.Json.parse=window.JSON.parse;})();(function(){var a="-",b="]",c='\\u',d="undefined",e="",f='\\$1',g="0041-005A0061-007A00AA00B500BA00C0-00D600D8-00F600F8-02C102C6-02D102E0-02E402EC02EE0370-037403760377037A-037D03860388-038A038C038E-03A103A3-03F503F7-0481048A-05250531-055605590561-058705D0-05EA05F0-05F20621-064A066E066F0671-06D306D506E506E606EE06EF06FA-06FC06FF07100712-072F074D-07A507B107CA-07EA07F407F507FA0800-0815081A082408280904-0939093D09500958-0961097109720979-097F0985-098C098F09900993-09A809AA-09B009B209B6-09B909BD09CE09DC09DD09DF-09E109F009F10A05-0A0A0A0F0A100A13-0A280A2A-0A300A320A330A350A360A380A390A59-0A5C0A5E0A72-0A740A85-0A8D0A8F-0A910A93-0AA80AAA-0AB00AB20AB30AB5-0AB90ABD0AD00AE00AE10B05-0B0C0B0F0B100B13-0B280B2A-0B300B320B330B35-0B390B3D0B5C0B5D0B5F-0B610B710B830B85-0B8A0B8E-0B900B92-0B950B990B9A0B9C0B9E0B9F0BA30BA40BA8-0BAA0BAE-0BB90BD00C05-0C0C0C0E-0C100C12-0C280C2A-0C330C35-0C390C3D0C580C590C600C610C85-0C8C0C8E-0C900C92-0CA80CAA-0CB30CB5-0CB90CBD0CDE0CE00CE10D05-0D0C0D0E-0D100D12-0D280D2A-0D390D3D0D600D610D7A-0D7F0D85-0D960D9A-0DB10DB3-0DBB0DBD0DC0-0DC60E01-0E300E320E330E40-0E460E810E820E840E870E880E8A0E8D0E94-0E970E99-0E9F0EA1-0EA30EA50EA70EAA0EAB0EAD-0EB00EB20EB30EBD0EC0-0EC40EC60EDC0EDD0F000F40-0F470F49-0F6C0F88-0F8B1000-102A103F1050-1055105A-105D106110651066106E-10701075-1081108E10A0-10C510D0-10FA10FC1100-1248124A-124D1250-12561258125A-125D1260-1288128A-128D1290-12B012B2-12B512B8-12BE12C012C2-12C512C8-12D612D8-13101312-13151318-135A1380-138F13A0-13F41401-166C166F-167F1681-169A16A0-16EA1700-170C170E-17111720-17311740-17511760-176C176E-17701780-17B317D717DC1820-18771880-18A818AA18B0-18F51900-191C1950-196D1970-19741980-19AB19C1-19C71A00-1A161A20-1A541AA71B05-1B331B45-1B4B1B83-1BA01BAE1BAF1C00-1C231C4D-1C4F1C5A-1C7D1CE9-1CEC1CEE-1CF11D00-1DBF1E00-1F151F18-1F1D1F20-1F451F48-1F4D1F50-1F571F591F5B1F5D1F5F-1F7D1F80-1FB41FB6-1FBC1FBE1FC2-1FC41FC6-1FCC1FD0-1FD31FD6-1FDB1FE0-1FEC1FF2-1FF41FF6-1FFC2071207F2090-209421022107210A-211321152119-211D212421262128212A-212D212F-2139213C-213F2145-2149214E218321842C00-2C2E2C30-2C5E2C60-2CE42CEB-2CEE2D00-2D252D30-2D652D6F2D80-2D962DA0-2DA62DA8-2DAE2DB0-2DB62DB8-2DBE2DC0-2DC62DC8-2DCE2DD0-2DD62DD8-2DDE2E2F300530063031-3035303B303C3041-3096309D-309F30A1-30FA30FC-30FF3105-312D3131-318E31A0-31B731F0-31FF3400-4DB54E00-9FCBA000-A48CA4D0-A4FDA500-A60CA610-A61FA62AA62BA640-A65FA662-A66EA67F-A697A6A0-A6E5A717-A71FA722-A788A78BA78CA7FB-A801A803-A805A807-A80AA80C-A822A840-A873A882-A8B3A8F2-A8F7A8FBA90A-A925A930-A946A960-A97CA984-A9B2A9CFAA00-AA28AA40-AA42AA44-AA4BAA60-AA76AA7AAA80-AAAFAAB1AAB5AAB6AAB9-AABDAAC0AAC2AADB-AADDABC0-ABE2AC00-D7A3D7B0-D7C6D7CB-D7FBF900-FA2DFA30-FA6DFA70-FAD9FB00-FB06FB13-FB17FB1DFB1F-FB28FB2A-FB36FB38-FB3CFB3EFB40FB41FB43FB44FB46-FBB1FBD3-FD3DFD50-FD8FFD92-FDC7FDF0-FDFBFE70-FE74FE76-FEFCFF21-FF3AFF41-FF5AFF66-FFBEFFC2-FFC7FFCA-FFCFFFD2-FFD7FFDA-FFDC",h="\\\\",j='-',k="g",l="\\\"",m="qx.lang.String",n="(^|[^",o="0",p="%",q='"',r=' ',s='\n',t="])[";qx.Bootstrap.define(m,{statics:{__bU:g,__bV:null,__bW:{},camelCase:function(v){var u=this.__bW[v];if(!u){u=v.replace(/\-([a-z])/g,function(x,w){return w.toUpperCase();});if(v.indexOf(a)>=0){this.__bW[v]=u;};};return u;},hyphenate:function(z){var y=this.__bW[z];if(!y){y=z.replace(/[A-Z]/g,function(A){return (j+A.charAt(0).toLowerCase());});if(z.indexOf(a)==-1){this.__bW[z]=y;};};return y;},capitalize:function(C){if(this.__bV===null){var B=c;this.__bV=new RegExp(n+this.__bU.replace(/[0-9A-F]{4}/g,function(D){return B+D;})+t+this.__bU.replace(/[0-9A-F]{4}/g,function(E){return B+E;})+b,k);};return C.replace(this.__bV,function(F){return F.toUpperCase();});},clean:function(G){return G.replace(/\s+/g,r).trim();},trimLeft:function(H){return H.replace(/^\s+/,e);},trimRight:function(I){return I.replace(/\s+$/,e);},startsWith:function(K,J){return K.indexOf(J)===0;},endsWith:function(M,L){return M.substring(M.length-L.length,M.length)===L;},repeat:function(N,O){return N.length>0?new Array(O+1).join(N):e;},pad:function(Q,length,P){var R=length-Q.length;if(R>0){if(typeof P===d){P=o;};return this.repeat(P,R)+Q;}else {return Q;};},firstUp:qx.Bootstrap.firstUp,firstLow:qx.Bootstrap.firstLow,contains:function(T,S){return T.indexOf(S)!=-1;},format:function(U,V){var W=U;var i=V.length;while(i-- ){W=W.replace(new RegExp(p+(i+1),k),function(){return V[i]+e;});};return W;},escapeRegexpChars:function(X){return X.replace(/([.*+?^${}()|[\]\/\\])/g,f);},toArray:function(Y){return Y.split(/\B|\b/g);},stripTags:function(ba){return ba.replace(/<\/?[^>]+>/gi,e);},stripScripts:function(bd,bc){var be=e;var bb=bd.replace(/<script[^>]*>([\s\S]*?)<\/script>/gi,function(){be+=arguments[1]+s;return e;});if(bc===true){qx.lang.Function.globalEval(be);};return bb;},quote:function(bf){return q+bf.replace(/\\/g,h).replace(/\"/g,l)+q;}}});})();(function(){var c="-",d="",e="qx.core.ObjectRegistry",f="Disposed ",g="$$hash",h="-0",j=" objects",k="Could not dispose object ",m=": ";qx.Bootstrap.define(e,{statics:{inShutDown:false,__j:{},__bX:0,__bY:[],__ca:d,__cb:{},register:function(n){var q=this.__j;if(!q){return;};var p=n.$$hash;if(p==null){var o=this.__bY;if(o.length>0&&true){p=o.pop();}else {p=(this.__bX++ )+this.__ca;};n.$$hash=p;{};};{};q[p]=n;},unregister:function(r){var s=r.$$hash;if(s==null){return;};var t=this.__j;if(t&&t[s]){delete t[s];this.__bY.push(s);};try{delete r.$$hash;}catch(u){if(r.removeAttribute){r.removeAttribute(g);};};},toHashCode:function(v){{};var x=v.$$hash;if(x!=null){return x;};var w=this.__bY;if(w.length>0){x=w.pop();}else {x=(this.__bX++ )+this.__ca;};return v.$$hash=x;},clearHashCode:function(y){{};var z=y.$$hash;if(z!=null){this.__bY.push(z);try{delete y.$$hash;}catch(A){if(y.removeAttribute){y.removeAttribute(g);};};};},fromHashCode:function(B){return this.__j[B]||null;},shutdown:function(){this.inShutDown=true;var D=this.__j;var F=[];for(var C in D){F.push(C);};F.sort(function(a,b){return parseInt(b,10)-parseInt(a,10);});var E,i=0,l=F.length;while(true){try{for(;i<l;i++ ){C=F[i];E=D[C];if(E&&E.dispose){E.dispose();};};}catch(G){qx.Bootstrap.error(this,k+E.toString()+m+G,G);if(i!==l){i++ ;continue;};};break;};qx.Bootstrap.debug(this,f+l+j);delete this.__j;},getRegistry:function(){return this.__j;},getNextHash:function(){return this.__bX;},getPostId:function(){return this.__ca;},getStackTraces:function(){return this.__cb;}},defer:function(H){if(window&&window.top){var frames=window.top.frames;for(var i=0;i<frames.length;i++ ){if(frames[i]===window){H.__ca=c+(i+1);return;};};};H.__ca=h;}});})();(function(){var a="qx.event.type.Data",b="qx.event.type.Event",c="qx.data.IListData";qx.Interface.define(c,{events:{"change":a,"changeLength":b},members:{getItem:function(d){},setItem:function(e,f){},splice:function(g,h,i){},contains:function(j){},getLength:function(){},toArray:function(){}}});})();(function(){var a="qx.core.ValidationError";qx.Class.define(a,{extend:qx.type.BaseError});})();(function(){var a="qx.util.RingBuffer";qx.Bootstrap.define(a,{extend:Object,construct:function(b){this.setMaxEntries(b||50);},members:{__cc:0,__cd:0,__ce:false,__cf:0,__cg:null,__ch:null,setMaxEntries:function(c){this.__ch=c;this.clear();},getMaxEntries:function(){return this.__ch;},addEntry:function(d){this.__cg[this.__cc]=d;this.__cc=this.__ci(this.__cc,1);var e=this.getMaxEntries();if(this.__cd<e){this.__cd++ ;};if(this.__ce&&(this.__cf<e)){this.__cf++ ;};},mark:function(){this.__ce=true;this.__cf=0;},clearMark:function(){this.__ce=false;},getAllEntries:function(){return this.getEntries(this.getMaxEntries(),false);},getEntries:function(f,j){if(f>this.__cd){f=this.__cd;};if(j&&this.__ce&&(f>this.__cf)){f=this.__cf;};if(f>0){var h=this.__ci(this.__cc,-1);var g=this.__ci(h,-f+1);var i;if(g<=h){i=this.__cg.slice(g,h+1);}else {i=this.__cg.slice(g,this.__cd).concat(this.__cg.slice(0,h+1));};}else {i=[];};return i;},clear:function(){this.__cg=new Array(this.getMaxEntries());this.__cd=0;this.__cf=0;this.__cc=0;},__ci:function(n,l){var k=this.getMaxEntries();var m=(n+l)%k;if(m<0){m+=k;};return m;}}});})();(function(){var a="qx.log.appender.RingBuffer";qx.Bootstrap.define(a,{extend:qx.util.RingBuffer,construct:function(b){this.setMaxMessages(b||50);},members:{setMaxMessages:function(c){this.setMaxEntries(c);},getMaxMessages:function(){return this.getMaxEntries();},process:function(d){this.addEntry(d);},getAllLogEvents:function(){return this.getAllEntries();},retrieveLogEvents:function(e,f){return this.getEntries(e,f);},clearHistory:function(){this.clear();}}});})();(function(){var a="qx.log.Logger",b="[",c="...(+",d="array",e=")",f="info",g="node",h="instance",j="string",k="null",m="error",n="#",o="class",p=": ",q="warn",r="document",s="{...(",t="",u="number",v="stringify",w="]",x="date",y="unknown",z="function",A="text[",B="[...(",C="boolean",D="\n",E=")}",F="debug",G=")]",H="map",I="undefined",J="object";qx.Bootstrap.define(a,{statics:{__cj:F,setLevel:function(K){this.__cj=K;},getLevel:function(){return this.__cj;},setTreshold:function(L){this.__cm.setMaxMessages(L);},getTreshold:function(){return this.__cm.getMaxMessages();},__ck:{},__cl:0,register:function(P){if(P.$$id){return;};var M=this.__cl++ ;this.__ck[M]=P;P.$$id=M;var N=this.__cn;var O=this.__cm.getAllLogEvents();for(var i=0,l=O.length;i<l;i++ ){if(N[O[i].level]>=N[this.__cj]){P.process(O[i]);};};},unregister:function(Q){var R=Q.$$id;if(R==null){return;};delete this.__ck[R];delete Q.$$id;},debug:function(T,S){qx.log.Logger.__co(F,arguments);},info:function(V,U){qx.log.Logger.__co(f,arguments);},warn:function(X,W){qx.log.Logger.__co(q,arguments);},error:function(ba,Y){qx.log.Logger.__co(m,arguments);},trace:function(bb){var bc=qx.dev.StackTrace.getStackTrace();qx.log.Logger.__co(f,[(typeof bb!==I?[bb].concat(bc):bc).join(D)]);},deprecatedMethodWarning:function(bf,bd){{var be;};},deprecatedClassWarning:function(bi,bg){{var bh;};},deprecatedEventWarning:function(bl,event,bj){{var bk;};},deprecatedMixinWarning:function(bn,bm){{var bo;};},deprecatedConstantWarning:function(bs,bq,bp){{var self,br;};},deprecateMethodOverriding:function(bv,bu,bw,bt){{var bx;};},clear:function(){this.__cm.clearHistory();},__cm:new qx.log.appender.RingBuffer(50),__cn:{debug:0,info:1,warn:2,error:3},__co:function(bz,bB){var bE=this.__cn;if(bE[bz]<bE[this.__cj]){return;};var by=bB.length<2?null:bB[0];var bD=by?1:0;var bA=[];for(var i=bD,l=bB.length;i<l;i++ ){bA.push(this.__cq(bB[i],true));};var bF=new Date;var bG={time:bF,offset:bF-qx.Bootstrap.LOADSTART,level:bz,items:bA,win:window};if(by){if(by.$$hash!==undefined){bG.object=by.$$hash;}else if(by.$$type){bG.clazz=by;}else if(by.constructor){bG.clazz=by.constructor;};};this.__cm.process(bG);var bC=this.__ck;for(var bH in bC){bC[bH].process(bG);};},__cp:function(bJ){if(bJ===undefined){return I;}else if(bJ===null){return k;};if(bJ.$$type){return o;};var bI=typeof bJ;if(bI===z||bI==j||bI===u||bI===C){return bI;}else if(bI===J){if(bJ.nodeType){return g;}else if(bJ instanceof Error||(bJ.name&&bJ.message)){return m;}else if(bJ.classname){return h;}else if(bJ instanceof Array){return d;}else if(bJ instanceof Date){return x;}else {return H;};};if(bJ.toString){return v;};return y;},__cq:function(bP,bO){var bS=this.__cp(bP);var bM=y;var bL=[];switch(bS){case k:case I:bM=bS;break;case j:case u:case C:case x:bM=bP;break;case g:if(bP.nodeType===9){bM=r;}else if(bP.nodeType===3){bM=A+bP.nodeValue+w;}else if(bP.nodeType===1){bM=bP.nodeName.toLowerCase();if(bP.id){bM+=n+bP.id;};}else {bM=g;};break;case z:bM=qx.lang.Function.getName(bP)||bS;break;case h:bM=bP.basename+b+bP.$$hash+w;break;case o:case v:bM=bP.toString();break;case m:bL=qx.dev.StackTrace.getStackTraceFromError(bP);bM=(bP.basename?bP.basename+p:t)+bP.toString();break;case d:if(bO){bM=[];for(var i=0,l=bP.length;i<l;i++ ){if(bM.length>20){bM.push(c+(l-i)+e);break;};bM.push(this.__cq(bP[i],false));};}else {bM=B+bP.length+G;};break;case H:if(bO){var bK;var bR=[];for(var bQ in bP){bR.push(bQ);};bR.sort();bM=[];for(var i=0,l=bR.length;i<l;i++ ){if(bM.length>20){bM.push(c+(l-i)+e);break;};bQ=bR[i];bK=this.__cq(bP[bQ],false);bK.key=bQ;bM.push(bK);};}else {var bN=0;for(var bQ in bP){bN++ ;};bM=s+bN+E;};break;};return {type:bS,text:bM,trace:bL};}},defer:function(bT){var bU=qx.Bootstrap.$$logs;for(var i=0;i<bU.length;i++ ){bT.__co(bU[i][0],bU[i][1]);};qx.Bootstrap.debug=bT.debug;qx.Bootstrap.info=bT.info;qx.Bootstrap.warn=bT.warn;qx.Bootstrap.error=bT.error;qx.Bootstrap.trace=bT.trace;}});})();(function(){var a="qx.core.MProperty",b="get",c="reset",d="No such property: ",e="set";qx.Mixin.define(a,{members:{set:function(g,h){var f=qx.core.Property.$$method.set;if(qx.Bootstrap.isString(g)){if(!this[f[g]]){if(this[e+qx.Bootstrap.firstUp(g)]!=undefined){this[e+qx.Bootstrap.firstUp(g)](h);return this;};throw new Error(d+g);};return this[f[g]](h);}else {for(var i in g){if(!this[f[i]]){if(this[e+qx.Bootstrap.firstUp(i)]!=undefined){this[e+qx.Bootstrap.firstUp(i)](g[i]);continue;};throw new Error(d+i);};this[f[i]](g[i]);};return this;};},get:function(k){var j=qx.core.Property.$$method.get;if(!this[j[k]]){if(this[b+qx.Bootstrap.firstUp(k)]!=undefined){return this[b+qx.Bootstrap.firstUp(k)]();};throw new Error(d+k);};return this[j[k]]();},reset:function(m){var l=qx.core.Property.$$method.reset;if(!this[l[m]]){if(this[c+qx.Bootstrap.firstUp(m)]!=undefined){this[c+qx.Bootstrap.firstUp(m)]();return;};throw new Error(d+m);};this[l[m]]();}}});})();(function(){var a="info",b="debug",c="warn",d="qx.core.MLogging",e="error";qx.Mixin.define(d,{members:{__cr:qx.log.Logger,debug:function(f){this.__cs(b,arguments);},info:function(g){this.__cs(a,arguments);},warn:function(h){this.__cs(c,arguments);},error:function(i){this.__cs(e,arguments);},trace:function(){this.__cr.trace(this);},__cs:function(j,l){var k=qx.lang.Array.fromArguments(l);k.unshift(this);this.__cr[j].apply(this.__cr,k);}}});})();(function(){var b="qx.dom.Node",c="";qx.Bootstrap.define(b,{statics:{ELEMENT:1,ATTRIBUTE:2,TEXT:3,CDATA_SECTION:4,ENTITY_REFERENCE:5,ENTITY:6,PROCESSING_INSTRUCTION:7,COMMENT:8,DOCUMENT:9,DOCUMENT_TYPE:10,DOCUMENT_FRAGMENT:11,NOTATION:12,getDocument:function(d){return d.nodeType===this.DOCUMENT?d:d.ownerDocument||d.document;},getWindow:function(e){if(e.nodeType==null){return e;};if(e.nodeType!==this.DOCUMENT){e=e.ownerDocument;};return e.defaultView||e.parentWindow;},getDocumentElement:function(f){return this.getDocument(f).documentElement;},getBodyElement:function(g){return this.getDocument(g).body;},isNode:function(h){return !!(h&&h.nodeType!=null);},isElement:function(j){return !!(j&&j.nodeType===this.ELEMENT);},isDocument:function(k){return !!(k&&k.nodeType===this.DOCUMENT);},isDocumentFragment:function(l){return !!(l&&l.nodeType===this.DOCUMENT_FRAGMENT);},isText:function(m){return !!(m&&m.nodeType===this.TEXT);},isWindow:function(n){return !!(n&&n.history&&n.location&&n.document);},isNodeName:function(o,p){if(!p||!o||!o.nodeName){return false;};return p.toLowerCase()==qx.dom.Node.getName(o);},getName:function(q){if(!q||!q.nodeName){return null;};return q.nodeName.toLowerCase();},getText:function(r){if(!r||!r.nodeType){return null;};switch(r.nodeType){case 1:var i,a=[],s=r.childNodes,length=s.length;for(i=0;i<length;i++ ){a[i]=this.getText(s[i]);};return a.join(c);case 2:case 3:case 4:return r.nodeValue;};return null;},isBlockNode:function(t){if(!qx.dom.Node.isElement(t)){return false;};t=qx.dom.Node.getName(t);return /^(body|form|textarea|fieldset|ul|ol|dl|dt|dd|li|div|hr|p|h[1-6]|quote|pre|table|thead|tbody|tfoot|tr|td|th|iframe|address|blockquote)$/.test(t);}}});})();(function(){var a="function",b='loadeddata',c="pointerover",d='pause',f="transitionend",g="gecko",h="browser.name",j='timeupdate',k='canplay',m="HTMLEvents",n='loadedmetadata',o="css.transition",p="mobile safari",q="return;",r="browser.documentmode",s="safari",t='play',u='ended',v="",w="qx.bom.Event",x='playing',y="mouseover",z="end-event",A="mshtml",B="engine.name",C='progress',D="webkit",E='volumechange',F='seeked',G="on",H="undefined";qx.Bootstrap.define(w,{statics:{addNativeListener:function(L,K,I,J){if(L.addEventListener){L.addEventListener(K,I,!!J);}else if(L.attachEvent){L.attachEvent(G+K,I);}else if(typeof L[G+K]!=H){L[G+K]=I;}else {{};};},removeNativeListener:function(P,O,M,N){if(P.removeEventListener){P.removeEventListener(O,M,!!N);}else if(P.detachEvent){try{P.detachEvent(G+O,M);}catch(e){if(e.number!==-2146828218){throw e;};};}else if(typeof P[G+O]!=H){P[G+O]=null;}else {{};};},getTarget:function(e){return e.target||e.srcElement;},getRelatedTarget:function(e){if(e.relatedTarget!==undefined){if((qx.core.Environment.get(B)==g)){try{e.relatedTarget&&e.relatedTarget.nodeType;}catch(Q){return null;};};return e.relatedTarget;}else if(e.fromElement!==undefined&&(e.type===y||e.type===c)){return e.fromElement;}else if(e.toElement!==undefined){return e.toElement;}else {return null;};},preventDefault:function(e){if(e.preventDefault){e.preventDefault();}else {try{e.keyCode=0;}catch(R){};e.returnValue=false;};},stopPropagation:function(e){if(e.stopPropagation){e.stopPropagation();}else {e.cancelBubble=true;};},fire:function(U,S){if(document.createEvent){var T=document.createEvent(m);T.initEvent(S,true,true);return !U.dispatchEvent(T);}else {var T=document.createEventObject();return U.fireEvent(G+S,T);};},supportsEvent:function(V,be){var ba=qx.core.Environment.get(h);var bb=qx.core.Environment.get(B);if(be.toLowerCase().indexOf(f)!=-1&&bb===A&&qx.core.Environment.get(r)>9){return true;};var bc=[p,s];if(bb===D&&bc.indexOf(ba)>-1){var W=[b,C,j,F,k,t,x,d,n,u,E];if(W.indexOf(be.toLowerCase())>-1){return true;};};if(V!=window&&be.toLowerCase().indexOf(f)!=-1){var bd=qx.core.Environment.get(o);return (bd&&bd[z]==be);};var X=G+be.toLowerCase();var Y=(X in V);if(!Y){Y=typeof V[X]==a;if(!Y&&V.setAttribute){V.setAttribute(X,q);Y=typeof V[X]==a;V.removeAttribute(X);};};return Y;},getEventName:function(bf,bi){var bg=[v].concat(qx.bom.Style.VENDOR_PREFIXES);for(var i=0,l=bg.length;i<l;i++ ){var bh=bg[i].toLowerCase();if(qx.bom.Event.supportsEvent(bf,bh+bi)){return bh?bh+qx.lang.String.firstUp(bi):bi;};};return null;}}});})();(function(){var a="-",b="qx.bom.Style",c="",d='-',e="Webkit",f="ms",g=":",h=";",j="Moz",k="O",m="string",n="Khtml";qx.Bootstrap.define(b,{statics:{VENDOR_PREFIXES:[e,j,k,f,n],__ct:{},__cu:null,getPropertyName:function(q){var o=document.documentElement.style;if(o[q]!==undefined){return q;};for(var i=0,l=this.VENDOR_PREFIXES.length;i<l;i++ ){var p=this.VENDOR_PREFIXES[i]+qx.lang.String.firstUp(q);if(o[p]!==undefined){return p;};};return null;},getCssName:function(r){var s=this.__ct[r];if(!s){s=r.replace(/[A-Z]/g,function(t){return (d+t.charAt(0).toLowerCase());});if((/^ms/.test(s))){s=a+s;};this.__ct[r]=s;};return s;},getAppliedStyle:function(A,x,z,v){var C=qx.bom.Style.getCssName(x);var w=qx.dom.Node.getWindow(A);var u=(v!==false)?[null].concat(this.VENDOR_PREFIXES):[null];for(var i=0,l=u.length;i<l;i++ ){var y=false;var B=u[i]?a+u[i].toLowerCase()+a+z:z;if(qx.bom.Style.__cu){y=qx.bom.Style.__cu.call(w,C,B);}else {A.style.cssText+=C+g+B+h;y=(typeof A.style[x]==m&&A.style[x]!==c);};if(y){return B;};};return null;}},defer:function(D){if(window.CSS&&window.CSS.supports){qx.bom.Style.__cu=window.CSS.supports.bind(window.CSS);}else if(window.supportsCSS){qx.bom.Style.__cu=window.supportsCSS.bind(window);};}});})();(function(){var a="rim_tabletos",b="10.1",c="Darwin",d="10.3",e="Windows Phone",f="os.version",g="10.7",h="2003",i=")",j="iPhone",k="android",l="unix",m="ce",n="7",o="SymbianOS",p="10.5",q="os.name",r="10.9",s="|",t="MacPPC",u="95",v="iPod",w="10.8",x="\.",y="Win64",z="linux",A="me",B="10.2",C="Macintosh",D="Android",E="Windows",F="98",G="ios",H="10",I="vista",J="8",K="blackberry",L="2000",M="8.1",N="(",O="",P="win",Q="Linux",R="10.6",S="BSD",T="10.0",U="10.4",V="Mac OS X",W="iPad",X="X11",Y="xp",br="symbian",bs="qx.bom.client.OperatingSystem",bt="g",bn="Win32",bo="10.10",bp="osx",bq="webOS",bw="RIM Tablet OS",bx="BlackBerry",by="nt4",bz=".",bu="MacIntel",bv="webos";qx.Bootstrap.define(bs,{statics:{getName:function(){if(!navigator){return O;};var bA=navigator.platform||O;var bB=navigator.userAgent||O;if(bA.indexOf(E)!=-1||bA.indexOf(bn)!=-1||bA.indexOf(y)!=-1||bB.indexOf(e)!=-1){return P;}else if(bA.indexOf(C)!=-1||bA.indexOf(t)!=-1||bA.indexOf(bu)!=-1||bA.indexOf(V)!=-1){return bp;}else if(bB.indexOf(bw)!=-1){return a;}else if(bB.indexOf(bq)!=-1){return bv;}else if(bA.indexOf(v)!=-1||bA.indexOf(j)!=-1||bA.indexOf(W)!=-1){return G;}else if(bB.indexOf(D)!=-1){return k;}else if(bA.indexOf(Q)!=-1){return z;}else if(bA.indexOf(X)!=-1||bA.indexOf(S)!=-1||bA.indexOf(c)!=-1){return l;}else if(bA.indexOf(o)!=-1){return br;}else if(bA.indexOf(bx)!=-1){return K;};return O;},__cv:{"Windows NT 10.0":H,"Windows NT 6.3":M,"Windows NT 6.2":J,"Windows NT 6.1":n,"Windows NT 6.0":I,"Windows NT 5.2":h,"Windows NT 5.1":Y,"Windows NT 5.0":L,"Windows 2000":L,"Windows NT 4.0":by,"Win 9x 4.90":A,"Windows CE":m,"Windows 98":F,"Win98":F,"Windows 95":u,"Win95":u,"Mac OS X 10_10":bo,"Mac OS X 10.10":bo,"Mac OS X 10_9":r,"Mac OS X 10.9":r,"Mac OS X 10_8":w,"Mac OS X 10.8":w,"Mac OS X 10_7":g,"Mac OS X 10.7":g,"Mac OS X 10_6":R,"Mac OS X 10.6":R,"Mac OS X 10_5":p,"Mac OS X 10.5":p,"Mac OS X 10_4":U,"Mac OS X 10.4":U,"Mac OS X 10_3":d,"Mac OS X 10.3":d,"Mac OS X 10_2":B,"Mac OS X 10.2":B,"Mac OS X 10_1":b,"Mac OS X 10.1":b,"Mac OS X 10_0":T,"Mac OS X 10.0":T},getVersion:function(){var bC=qx.bom.client.OperatingSystem.__cw(navigator.userAgent);if(bC==null){bC=qx.bom.client.OperatingSystem.__cx(navigator.userAgent);};if(bC!=null){return bC;}else {return O;};},__cw:function(bD){var bG=[];for(var bF in qx.bom.client.OperatingSystem.__cv){bG.push(bF);};var bH=new RegExp(N+bG.join(s).replace(/\./g,x)+i,bt);var bE=bH.exec(bD);if(bE&&bE[1]){return qx.bom.client.OperatingSystem.__cv[bE[1]];};return null;},__cx:function(bN){var bI=bN.indexOf(e)!=-1;var bO=bN.indexOf(D)!=-1;var bJ=bN.match(/(iPad|iPhone|iPod)/i)?true:false;if(bI){var bQ=new RegExp(/Windows Phone (\d+(?:\.\d+)+)/i);var bL=bQ.exec(bN);if(bL&&bL[1]){return bL[1];};}else if(bO){var bM=new RegExp(/ Android (\d+(?:\.\d+)+)/i);var bP=bM.exec(bN);if(bP&&bP[1]){return bP[1];};}else if(bJ){var bR=new RegExp(/(CPU|iPhone|iPod) OS (\d+)_(\d+)(?:_(\d+))*\s+/);var bK=bR.exec(bN);if(bK&&bK[2]&&bK[3]){if(bK[4]){return bK[2]+bz+bK[3]+bz+bK[4];}else {return bK[2]+bz+bK[3];};};};return null;}},defer:function(bS){qx.core.Environment.add(q,bS.getName);qx.core.Environment.add(f,bS.getVersion);}});})();(function(){var a="CSS1Compat",b="IEMobile",c=" OPR/",d="msie",e="android",f="operamini",g="gecko",h="maple",i="AdobeAIR|Titanium|Fluid|Chrome|Android|Epiphany|Konqueror|iCab|iPad|iPhone|OmniWeb|Maxthon|Pre|PhantomJS|Mobile Safari|Safari",j="browser.quirksmode",k="browser.name",l="trident",m="mobile chrome",n=")(/| )([0-9]+\.[0-9])",o="iemobile",p="prism|Fennec|Camino|Kmeleon|Galeon|Netscape|SeaMonkey|Namoroka|Firefox",q="IEMobile|Maxthon|MSIE|Trident",r="opera mobi",s="Mobile Safari",t="Maple",u="operamobile",v="ie",w="mobile safari",x="qx.bom.client.Browser",y="(Maple )([0-9]+\.[0-9]+\.[0-9]*)",z="",A="opera mini",B="(",C="browser.version",D="opera",E="ce",F=")(/|)?([0-9]+\.[0-9])?",G="mshtml",H="Opera Mini|Opera Mobi|Opera",I="edge",J="webkit",K="browser.documentmode",L="5.0",M="Mobile/";qx.Bootstrap.define(x,{statics:{getName:function(){var P=navigator.userAgent;var Q=new RegExp(B+qx.bom.client.Browser.__cy+F);var O=P.match(Q);if(!O){return z;};var name=O[1].toLowerCase();var N=qx.bom.client.Engine.getName();if(N===J){if(P.match(/Edge\/\d+\.\d+/)){name=I;}else if(name===e){name=m;}else if(P.indexOf(s)!==-1||P.indexOf(M)!==-1){name=w;}else if(P.indexOf(c)!=-1){name=D;};}else if(N===G){if(name===d||name===l){name=v;if(qx.bom.client.OperatingSystem.getVersion()===E){name=o;};var Q=new RegExp(b);if(P.match(Q)){name=o;};};}else if(N===D){if(name===r){name=u;}else if(name===A){name=f;};}else if(N===g){if(P.indexOf(t)!==-1){name=h;};};return name;},getVersion:function(){var T=navigator.userAgent;var U=new RegExp(B+qx.bom.client.Browser.__cy+n);var R=T.match(U);if(!R){return z;};var name=R[1].toLowerCase();var S=R[3];if(T.match(/Version(\/| )([0-9]+\.[0-9])/)){S=RegExp.$2;};if(qx.bom.client.Engine.getName()==G){S=qx.bom.client.Engine.getVersion();if(name===d&&qx.bom.client.OperatingSystem.getVersion()==E){S=L;};};if(qx.bom.client.Browser.getName()==h){U=new RegExp(y);R=T.match(U);if(!R){return z;};S=R[2];};if(qx.bom.client.Engine.getName()==J||qx.bom.client.Browser.getName()==D){if(T.match(/OPR(\/| )([0-9]+\.[0-9])/)){S=RegExp.$2;};if(T.match(/Edge\/([\d+\.*]+)/)){S=RegExp.$1;};};return S;},getDocumentMode:function(){if(document.documentMode){return document.documentMode;};return 0;},getQuirksMode:function(){if(qx.bom.client.Engine.getName()==G&&parseFloat(qx.bom.client.Engine.getVersion())>=8){return qx.bom.client.Engine.DOCUMENT_MODE===5;}else {return document.compatMode!==a;};},__cy:{"webkit":i,"gecko":p,"mshtml":q,"opera":H}[qx.bom.client.Engine.getName()]},defer:function(V){qx.core.Environment.add(k,V.getName);qx.core.Environment.add(C,V.getVersion);qx.core.Environment.add(K,V.getDocumentMode);qx.core.Environment.add(j,V.getQuirksMode);}});})();(function(){var a="qx.bom.client.CssTransition",b="E",c="transitionEnd",d="e",e="nd",f="transition",g="css.transition",h="Trans";qx.Bootstrap.define(a,{statics:{getTransitionName:function(){return qx.bom.Style.getPropertyName(f);},getSupport:function(){var name=qx.bom.client.CssTransition.getTransitionName();if(!name){return null;};var i=qx.bom.Event.getEventName(window,c);i=i==c?i.toLowerCase():i;if(!i){i=name+(name.indexOf(h)>0?b:d)+e;};return {name:name,"end-event":i};}},defer:function(j){qx.core.Environment.add(g,j.getSupport);}});})();(function(){var a="__cD",b="UNKNOWN_",c="|bubble",d="",e="_",f="__cE",g="c",h="|",j="qx.globalErrorHandling",k="unload",m="|capture",n="DOM_",o="WIN_",p="QX_",q="qx.event.Manager",r="capture",s="DOCUMENT_";qx.Class.define(q,{extend:Object,construct:function(t,v){this.__cz=t;this.__cA=qx.core.ObjectRegistry.toHashCode(t);this.__cB=v;if(t.qx!==qx){var self=this;var u=function(){qx.bom.Event.removeNativeListener(t,k,arguments.callee);self.dispose();};if(qx.core.Environment.get(j)){qx.bom.Event.addNativeListener(t,k,qx.event.GlobalError.observeMethod(u));}else {qx.bom.Event.addNativeListener(t,k,u);};};this.__cC={};this.__cD={};this.__cE={};this.__cF={};},statics:{__cG:0,getNextUniqueId:function(){return (this.__cG++ )+d;}},members:{__cB:null,__cC:null,__cE:null,__cH:null,__cD:null,__cF:null,__cz:null,__cA:null,getWindow:function(){return this.__cz;},getWindowId:function(){return this.__cA;},getHandler:function(x){var w=this.__cD[x.classname];if(w){return w;};return this.__cD[x.classname]=new x(this);},getDispatcher:function(z){var y=this.__cE[z.classname];if(y){return y;};return this.__cE[z.classname]=new z(this,this.__cB);},getListeners:function(B,F,A){var D=B.$$hash||qx.core.ObjectRegistry.toHashCode(B);var G=this.__cC[D];if(!G){return null;};var E=F+(A?m:c);var C=G[E];return C?C.concat():null;},getAllListeners:function(){return this.__cC;},serializeListeners:function(I){var M=I.$$hash||qx.core.ObjectRegistry.toHashCode(I);var Q=this.__cC[M];var L=[];if(Q){var J,P,H,K,N;for(var O in Q){J=O.indexOf(h);P=O.substring(0,J);H=O.charAt(J+1)==g;K=Q[O];for(var i=0,l=K.length;i<l;i++ ){N=K[i];L.push({self:N.context,handler:N.handler,type:P,capture:H});};};};return L;},toggleAttachedEvents:function(T,S){var W=T.$$hash||qx.core.ObjectRegistry.toHashCode(T);var ba=this.__cC[W];if(ba){var U,Y,R,V;for(var X in ba){U=X.indexOf(h);Y=X.substring(0,U);R=X.charCodeAt(U+1)===99;V=ba[X];if(S){this.__cI(T,Y,R);}else {this.__cJ(T,Y,R);};};};},hasListener:function(bc,bg,bb){{};var be=bc.$$hash||qx.core.ObjectRegistry.toHashCode(bc);var bh=this.__cC[be];if(!bh){return false;};var bf=bg+(bb?m:c);var bd=bh[bf];return !!(bd&&bd.length>0);},importListeners:function(bi,bk){{};var bo=bi.$$hash||qx.core.ObjectRegistry.toHashCode(bi);var bq=this.__cC[bo]={};var bm=qx.event.Manager;for(var bj in bk){var bn=bk[bj];var bp=bn.type+(bn.capture?m:c);var bl=bq[bp];if(!bl){bl=bq[bp]=[];this.__cI(bi,bn.type,bn.capture);};bl.push({handler:bn.listener,context:bn.self,unique:bn.unique||(bm.__cG++ )+d});};},addListener:function(bt,bA,bv,self,br){{var bx;};var bs=bt.$$hash||qx.core.ObjectRegistry.toHashCode(bt);var bB=this.__cC[bs];if(!bB){bB=this.__cC[bs]={};};var bw=bA+(br?m:c);var bu=bB[bw];if(!bu){bu=bB[bw]=[];};if(bu.length===0){this.__cI(bt,bA,br);};var bz=(qx.event.Manager.__cG++ )+d;var by={handler:bv,context:self,unique:bz};bu.push(by);return bw+h+bz;},findHandler:function(bG,bP){var bN=false,bF=false,bQ=false,bC=false;var bM;if(bG.nodeType===1){bN=true;bM=n+bG.tagName.toLowerCase()+e+bP;}else if(bG.nodeType===9){bC=true;bM=s+bP;}else if(bG==this.__cz){bF=true;bM=o+bP;}else if(bG.classname){bQ=true;bM=p+bG.classname+e+bP;}else {bM=b+bG+e+bP;};var bE=this.__cF;if(bE[bM]){return bE[bM];};var bL=this.__cB.getHandlers();var bH=qx.event.IEventHandler;var bJ,bK,bI,bD;for(var i=0,l=bL.length;i<l;i++ ){bJ=bL[i];bI=bJ.SUPPORTED_TYPES;if(bI&&!bI[bP]){continue;};bD=bJ.TARGET_CHECK;if(bD){var bO=false;if(bN&&((bD&bH.TARGET_DOMNODE)!=0)){bO=true;}else if(bF&&((bD&bH.TARGET_WINDOW)!=0)){bO=true;}else if(bQ&&((bD&bH.TARGET_OBJECT)!=0)){bO=true;}else if(bC&&((bD&bH.TARGET_DOCUMENT)!=0)){bO=true;};if(!bO){continue;};};bK=this.getHandler(bL[i]);if(bJ.IGNORE_CAN_HANDLE||bK.canHandleEvent(bG,bP)){bE[bM]=bK;return bK;};};return null;},__cI:function(bU,bT,bR){var bS=this.findHandler(bU,bT);if(bS){bS.registerEvent(bU,bT,bR);return;};{};},removeListener:function(bX,ce,ca,self,bV){{var cc;};var bW=bX.$$hash||qx.core.ObjectRegistry.toHashCode(bX);var cf=this.__cC[bW];if(!cf){return false;};var cb=ce+(bV?m:c);var bY=cf[cb];if(!bY){return false;};var cd;for(var i=0,l=bY.length;i<l;i++ ){cd=bY[i];if(cd.handler===ca&&cd.context===self){qx.lang.Array.removeAt(bY,i);if(bY.length==0){this.__cJ(bX,ce,bV);};return true;};};return false;},removeListenerById:function(ci,cq){{var cm;};var ck=cq.split(h);var cp=ck[0];var cg=ck[1].charCodeAt(0)==99;var co=ck[2];var ch=ci.$$hash||qx.core.ObjectRegistry.toHashCode(ci);var cr=this.__cC[ch];if(!cr){return false;};var cl=cp+(cg?m:c);var cj=cr[cl];if(!cj){return false;};var cn;for(var i=0,l=cj.length;i<l;i++ ){cn=cj[i];if(cn.unique===co){qx.lang.Array.removeAt(cj,i);if(cj.length==0){this.__cJ(ci,cp,cg);};return true;};};return false;},removeAllListeners:function(ct){var cv=ct.$$hash||qx.core.ObjectRegistry.toHashCode(ct);var cy=this.__cC[cv];if(!cy){return false;};var cu,cx,cs;for(var cw in cy){if(cy[cw].length>0){cu=cw.split(h);cx=cu[0];cs=cu[1]===r;this.__cJ(ct,cx,cs);};};delete this.__cC[cv];return true;},deleteAllListeners:function(cz){delete this.__cC[cz];},__cJ:function(cD,cC,cA){var cB=this.findHandler(cD,cC);if(cB){cB.unregisterEvent(cD,cC,cA);return;};{};},dispatchEvent:function(cF,event){{var cJ;};var cK=event.getType();if(!event.getBubbles()&&!this.hasListener(cF,cK)){qx.event.Pool.getInstance().poolObject(event);return true;};if(!event.getTarget()){event.setTarget(cF);};var cI=this.__cB.getDispatchers();var cH;var cE=false;for(var i=0,l=cI.length;i<l;i++ ){cH=this.getDispatcher(cI[i]);if(cH.canDispatchEvent(cF,event,cK)){cH.dispatchEvent(cF,event,cK);cE=true;break;};};if(!cE){{};return true;};var cG=event.getDefaultPrevented();qx.event.Pool.getInstance().poolObject(event);return !cG;},dispose:function(){this.__cB.removeManager(this);qx.util.DisposeUtil.disposeMap(this,a);qx.util.DisposeUtil.disposeMap(this,f);this.__cC=this.__cz=this.__cH=null;this.__cB=this.__cF=null;}}});})();(function(){var a="qx.event.IEventHandler";qx.Interface.define(a,{statics:{TARGET_DOMNODE:1,TARGET_WINDOW:2,TARGET_OBJECT:4,TARGET_DOCUMENT:8},members:{canHandleEvent:function(c,b){},registerEvent:function(f,e,d){},unregisterEvent:function(i,h,g){}}});})();(function(){var c="qx.event.Registration";qx.Class.define(c,{statics:{__cK:{},getManager:function(f){if(f==null){{};f=window;}else if(f.nodeType){f=qx.dom.Node.getWindow(f);}else if(!qx.dom.Node.isWindow(f)){f=window;};var e=f.$$hash||qx.core.ObjectRegistry.toHashCode(f);var d=this.__cK[e];if(!d){d=new qx.event.Manager(f,this);this.__cK[e]=d;};return d;},removeManager:function(g){var h=g.getWindowId();delete this.__cK[h];},addListener:function(l,k,i,self,j){return this.getManager(l).addListener(l,k,i,self,j);},removeListener:function(p,o,m,self,n){return this.getManager(p).removeListener(p,o,m,self,n);},removeListenerById:function(q,r){return this.getManager(q).removeListenerById(q,r);},removeAllListeners:function(s){return this.getManager(s).removeAllListeners(s);},deleteAllListeners:function(u){var t=u.$$hash;if(t){this.getManager(u).deleteAllListeners(t);};},hasListener:function(x,w,v){return this.getManager(x).hasListener(x,w,v);},serializeListeners:function(y){return this.getManager(y).serializeListeners(y);},createEvent:function(B,C,A){{};if(C==null){C=qx.event.type.Event;};var z=qx.event.Pool.getInstance().getObject(C);A?z.init.apply(z,A):z.init();if(B){z.setType(B);};return z;},dispatchEvent:function(D,event){return this.getManager(D).dispatchEvent(D,event);},fireEvent:function(E,F,H,G){{var I;};var J=this.createEvent(F,H||null,G);return this.getManager(E).dispatchEvent(E,J);},fireNonBubblingEvent:function(K,P,N,M){{};var O=this.getManager(K);if(!O.hasListener(K,P,false)){return true;};var L=this.createEvent(P,N||null,M);return O.dispatchEvent(K,L);},PRIORITY_FIRST:-32000,PRIORITY_NORMAL:0,PRIORITY_LAST:32000,__cD:[],addHandler:function(Q){{};this.__cD.push(Q);this.__cD.sort(function(a,b){return a.PRIORITY-b.PRIORITY;});},getHandlers:function(){return this.__cD;},__cE:[],addDispatcher:function(S,R){{};this.__cE.push(S);this.__cE.sort(function(a,b){return a.PRIORITY-b.PRIORITY;});},getDispatchers:function(){return this.__cE;}}});})();(function(){var a="qx.core.MEvent";qx.Mixin.define(a,{members:{__cL:qx.event.Registration,addListener:function(d,b,self,c){if(!this.$$disposed){return this.__cL.addListener(this,d,b,self,c);};return null;},addListenerOnce:function(h,f,self,g){var i=function(e){this.removeListener(h,f,this,g);f.call(self||this,e);};if(!f.$$wrapped_callback){f.$$wrapped_callback={};};f.$$wrapped_callback[h+this.$$hash]=i;return this.addListener(h,i,this,g);},removeListener:function(l,j,self,k){if(!this.$$disposed){if(j.$$wrapped_callback&&j.$$wrapped_callback[l+this.$$hash]){var m=j.$$wrapped_callback[l+this.$$hash];delete j.$$wrapped_callback[l+this.$$hash];j=m;};return this.__cL.removeListener(this,l,j,self,k);};return false;},removeListenerById:function(n){if(!this.$$disposed){return this.__cL.removeListenerById(this,n);};return false;},hasListener:function(p,o){return this.__cL.hasListener(this,p,o);},dispatchEvent:function(q){if(!this.$$disposed){return this.__cL.dispatchEvent(this,q);};return true;},fireEvent:function(s,t,r){if(!this.$$disposed){return this.__cL.fireEvent(this,s,t,r);};return true;},fireNonBubblingEvent:function(v,w,u){if(!this.$$disposed){return this.__cL.fireNonBubblingEvent(this,v,w,u);};return true;},fireDataEvent:function(z,A,x,y){if(!this.$$disposed){if(x===undefined){x=null;};return this.__cL.fireNonBubblingEvent(this,z,qx.event.type.Data,[A,x,!!y]);};return true;}}});})();(function(){var a="qx.event.IEventDispatcher";qx.Interface.define(a,{members:{canDispatchEvent:function(c,event,b){this.assertInstance(event,qx.event.type.Event);this.assertString(b);},dispatchEvent:function(e,event,d){this.assertInstance(event,qx.event.type.Event);this.assertString(d);}}});})();(function(){var a="module.events",b="Cloning only possible with properties.",c="qx.core.Object",d="module.property",e="]",f="[",g="Object";qx.Class.define(c,{extend:Object,include:qx.core.Environment.filter({"module.databinding":qx.data.MBinding,"module.logger":qx.core.MLogging,"module.events":qx.core.MEvent,"module.property":qx.core.MProperty}),construct:function(){qx.core.ObjectRegistry.register(this);},statics:{$$type:g},members:{__L:qx.core.Environment.get(d)?qx.core.Property:null,toHashCode:function(){return this.$$hash;},toString:function(){return this.classname+f+this.$$hash+e;},base:function(h,j){{};if(arguments.length===1){return h.callee.base.call(this);}else {return h.callee.base.apply(this,Array.prototype.slice.call(arguments,1));};},self:function(k){return k.callee.self;},clone:function(){if(!qx.core.Environment.get(d)){throw new Error(b);};var n=this.constructor;var m=new n;var p=qx.Class.getProperties(n);var o=this.__L.$$store.user;var q=this.__L.$$method.set;var name;for(var i=0,l=p.length;i<l;i++ ){name=p[i];if(this.hasOwnProperty(o[name])){m[q[name]](this[o[name]]);};};return m;},__cM:null,setUserData:function(r,s){if(!this.__cM){this.__cM={};};this.__cM[r]=s;},getUserData:function(u){if(!this.__cM){return null;};var t=this.__cM[u];return t===undefined?null:t;},isDisposed:function(){return this.$$disposed||false;},dispose:function(){if(this.$$disposed){return;};this.$$disposed=true;this.$$instance=null;this.$$allowconstruct=null;{};var x=this.constructor;var v;while(x.superclass){if(x.$$destructor){x.$$destructor.call(this);};if(x.$$includes){v=x.$$flatIncludes;for(var i=0,l=v.length;i<l;i++ ){if(v[i].$$destructor){v[i].$$destructor.call(this);};};};x=x.superclass;};{var y,w;};},_disposeObjects:function(z){qx.util.DisposeUtil.disposeObjects(this,arguments);},_disposeSingletonObjects:function(A){qx.util.DisposeUtil.disposeObjects(this,arguments,true);},_disposeArray:function(B){qx.util.DisposeUtil.disposeArray(this,B);},_disposeMap:function(C){qx.util.DisposeUtil.disposeMap(this,C);}},environment:{"qx.debug.dispose.level":0},destruct:function(){if(qx.core.Environment.get(a)){if(!qx.core.ObjectRegistry.inShutDown){qx.event.Registration.removeAllListeners(this);}else {qx.event.Registration.deleteAllListeners(this);};};qx.core.ObjectRegistry.unregister(this);this.__cM=null;if(qx.core.Environment.get(d)){var F=this.constructor;var J;var K=this.__L.$$store;var H=K.user;var I=K.theme;var D=K.inherit;var G=K.useinit;var E=K.init;while(F){J=F.$$properties;if(J){for(var name in J){if(J[name].dereference){this[H[name]]=this[I[name]]=this[D[name]]=this[G[name]]=this[E[name]]=undefined;};};};F=F.superclass;};};}});})();(function(){var a=" is a singleton! Please use disposeSingleton instead.",b="undefined",c="qx.util.DisposeUtil",d=" of object: ",e="!",f=" has non disposable entries: ",g="The map field: ",h="The array field: ",j="The object stored in key ",k="Has no disposable object under key: ";qx.Class.define(c,{statics:{disposeObjects:function(n,m,o){var name;for(var i=0,l=m.length;i<l;i++ ){name=m[i];if(n[name]==null||!n.hasOwnProperty(name)){continue;};if(!qx.core.ObjectRegistry.inShutDown){if(n[name].dispose){if(!o&&n[name].constructor.$$instance){throw new Error(j+name+a);}else {n[name].dispose();};}else {throw new Error(k+name+e);};};n[name]=null;};},disposeArray:function(q,p){var r=q[p];if(!r){return;};if(qx.core.ObjectRegistry.inShutDown){q[p]=null;return;};try{var s;for(var i=r.length-1;i>=0;i-- ){s=r[i];if(s){s.dispose();};};}catch(t){throw new Error(h+p+d+q+f+t);};r.length=0;q[p]=null;},disposeMap:function(v,u){var w=v[u];if(!w){return;};if(qx.core.ObjectRegistry.inShutDown){v[u]=null;return;};try{var y;for(var x in w){y=w[x];if(w.hasOwnProperty(x)&&y){y.dispose();};};}catch(z){throw new Error(g+u+d+v+f+z);};v[u]=null;},disposeTriggeredBy:function(A,C){var B=C.dispose;C.dispose=function(){B.call(C);A.dispose();};},destroyContainer:function(E){{};var D=[];this._collectContainerChildren(E,D);var F=D.length;for(var i=F-1;i>=0;i-- ){D[i].destroy();};E.destroy();},_collectContainerChildren:function(I,H){var J=I.getChildren();for(var i=0;i<J.length;i++ ){var G=J[i];H.push(G);if(this.__cN(G)){this._collectContainerChildren(G,H);};};},__cN:function(L){var K=[];if(qx.ui.mobile&&L instanceof qx.ui.mobile.core.Widget){K=[qx.ui.mobile.container.Composite];}else {K=[qx.ui.container.Composite,qx.ui.container.Scroll,qx.ui.container.SlideBar,qx.ui.container.Stack];};for(var i=0,l=K.length;i<l;i++ ){if(typeof K[i]!==b&&qx.Class.isSubClassOf(L.constructor,K[i])){return true;};};return false;}}});})();(function(){var a="qx.event.type.Event";qx.Class.define(a,{extend:qx.core.Object,statics:{CAPTURING_PHASE:1,AT_TARGET:2,BUBBLING_PHASE:3},members:{init:function(c,b){{};this._type=null;this._target=null;this._currentTarget=null;this._relatedTarget=null;this._originalTarget=null;this._stopPropagation=false;this._preventDefault=false;this._bubbles=!!c;this._cancelable=!!b;this._timeStamp=(new Date()).getTime();this._eventPhase=null;return this;},clone:function(d){if(d){var e=d;}else {var e=qx.event.Pool.getInstance().getObject(this.constructor);};e._type=this._type;e._target=this._target;e._currentTarget=this._currentTarget;e._relatedTarget=this._relatedTarget;e._originalTarget=this._originalTarget;e._stopPropagation=this._stopPropagation;e._bubbles=this._bubbles;e._preventDefault=this._preventDefault;e._cancelable=this._cancelable;return e;},stop:function(){if(this._bubbles){this.stopPropagation();};if(this._cancelable){this.preventDefault();};},stopPropagation:function(){{};this._stopPropagation=true;},getPropagationStopped:function(){return !!this._stopPropagation;},preventDefault:function(){{};this._preventDefault=true;},getDefaultPrevented:function(){return !!this._preventDefault;},getType:function(){return this._type;},setType:function(f){this._type=f;},getEventPhase:function(){return this._eventPhase;},setEventPhase:function(g){this._eventPhase=g;},getTimeStamp:function(){return this._timeStamp;},getTarget:function(){return this._target;},setTarget:function(h){this._target=h;},getCurrentTarget:function(){return this._currentTarget||this._target;},setCurrentTarget:function(i){this._currentTarget=i;},getRelatedTarget:function(){return this._relatedTarget;},setRelatedTarget:function(j){this._relatedTarget=j;},getOriginalTarget:function(){return this._originalTarget;},setOriginalTarget:function(k){this._originalTarget=k;},getBubbles:function(){return this._bubbles;},setBubbles:function(l){this._bubbles=l;},isCancelable:function(){return this._cancelable;},setCancelable:function(m){this._cancelable=m;}},destruct:function(){this._target=this._currentTarget=this._relatedTarget=this._originalTarget=null;}});})();(function(){var a="qx.util.ObjectPool",b="Class needs to be defined!",c="Object is already pooled: ",d="Integer";qx.Class.define(a,{extend:qx.core.Object,construct:function(e){qx.core.Object.call(this);this.__cO={};if(e!=null){this.setSize(e);};},properties:{size:{check:d,init:Infinity}},members:{__cO:null,getObject:function(h){if(this.$$disposed){return new h;};if(!h){throw new Error(b);};var f=null;var g=this.__cO[h.classname];if(g){f=g.pop();};if(f){f.$$pooled=false;}else {f=new h;};return f;},poolObject:function(k){if(!this.__cO){return;};var j=k.classname;var m=this.__cO[j];if(k.$$pooled){throw new Error(c+k);};if(!m){this.__cO[j]=m=[];};if(m.length>this.getSize()){if(k.destroy){k.destroy();}else {k.dispose();};return;};k.$$pooled=true;m.push(k);}},destruct:function(){var p=this.__cO;var n,o,i,l;for(n in p){o=p[n];for(i=0,l=o.length;i<l;i++ ){o[i].dispose();};};delete this.__cO;}});})();(function(){var a="singleton",b="qx.event.Pool";qx.Class.define(b,{extend:qx.util.ObjectPool,type:a,construct:function(){qx.util.ObjectPool.call(this,30);}});})();(function(){var a="qx.event.dispatch.Direct";qx.Class.define(a,{extend:qx.core.Object,implement:qx.event.IEventDispatcher,construct:function(b){this._manager=b;},statics:{PRIORITY:qx.event.Registration.PRIORITY_LAST},members:{canDispatchEvent:function(d,event,c){return !event.getBubbles();},dispatchEvent:function(e,event,k){{var j,f;};event.setEventPhase(qx.event.type.Event.AT_TARGET);var g=this._manager.getListeners(e,k,false);if(g){for(var i=0,l=g.length;i<l;i++ ){var h=g[i].context||e;{};g[i].handler.call(h,event);};};}},defer:function(m){qx.event.Registration.addDispatcher(m);}});})();(function(){var a="qx.event.handler.Object";qx.Class.define(a,{extend:qx.core.Object,implement:qx.event.IEventHandler,statics:{PRIORITY:qx.event.Registration.PRIORITY_LAST,SUPPORTED_TYPES:null,TARGET_CHECK:qx.event.IEventHandler.TARGET_OBJECT,IGNORE_CAN_HANDLE:false},members:{canHandleEvent:function(c,b){return qx.Class.supportsEvent(c.constructor,b);},registerEvent:function(f,e,d){},unregisterEvent:function(i,h,g){}},defer:function(j){qx.event.Registration.addHandler(j);}});})();(function(){var a="qx.event.type.Data";qx.Class.define(a,{extend:qx.event.type.Event,members:{__cP:null,__cQ:null,init:function(c,d,b){qx.event.type.Event.prototype.init.call(this,false,b);this.__cP=c;this.__cQ=d;return this;},clone:function(e){var f=qx.event.type.Event.prototype.clone.call(this,e);f.__cP=this.__cP;f.__cQ=this.__cQ;return f;},getData:function(){return this.__cP;},getOldData:function(){return this.__cQ;}},destruct:function(){this.__cP=this.__cQ=null;}});})();(function(){var a="To enable localization please include qx.locale.Manager into your build!",b="qx.locale.MTranslation";qx.Mixin.define(b,{members:{tr:function(c,e){var d=qx.locale.Manager;if(d){return d.tr.apply(d,arguments);};throw new Error(a);},trn:function(g,j,f,h){var i=qx.locale.Manager;if(i){return i.trn.apply(i,arguments);};throw new Error(a);},trc:function(n,m,l){var k=qx.locale.Manager;if(k){return k.trc.apply(k,arguments);};throw new Error(a);},trnc:function(p,q,r,o,s){var t=qx.locale.Manager;if(t){return t.trnc.apply(t,arguments);};throw new Error(a);},marktr:function(v){var u=qx.locale.Manager;if(u){return u.marktr.apply(u,arguments);};throw new Error(a);}}});})();(function(){var a="qx.application.IApplication";qx.Interface.define(a,{members:{main:function(){},finalize:function(){},close:function(){},terminate:function(){}}});})();(function(){var a="qx.core.BaseInit",b="engine.name",c="Main runtime: ",d="",f="os.name",g="engine.version",h="Missing application class: ",i="Load runtime: ",j="ms",k="Could not detect engine!",l="Finalize runtime: ",m='qx_lib.Application',n="Could not detect operating system!",o="Could not detect the version of the engine!";qx.Class.define(a,{statics:{__cR:null,getApplication:function(){return this.__cR||null;},ready:function(){if(this.__cR){return;};if(qx.core.Environment.get(b)==d){qx.log.Logger.warn(k);};if(qx.core.Environment.get(g)==d){qx.log.Logger.warn(o);};if(qx.core.Environment.get(f)==d){qx.log.Logger.warn(n);};qx.log.Logger.debug(this,i+(new Date-qx.Bootstrap.LOADSTART)+j);var q=m;var r=qx.Class.getByName(q);if(r){this.__cR=new r;var p=new Date;this.__cR.main();qx.log.Logger.debug(this,c+(new Date-p)+j);var p=new Date;this.__cR.finalize();qx.log.Logger.debug(this,l+(new Date-p)+j);}else {qx.log.Logger.warn(h+q);};},__cS:function(e){var s=this.__cR;if(s){s.close();};},__cT:function(){var t=this.__cR;if(t){t.terminate();};qx.core.ObjectRegistry.shutdown();}}});})();(function(){var a="qx.event.type.Native";qx.Class.define(a,{extend:qx.event.type.Event,members:{init:function(b,e,f,d,c){qx.event.type.Event.prototype.init.call(this,d,c);this._target=e||qx.bom.Event.getTarget(b);this._relatedTarget=f||qx.bom.Event.getRelatedTarget(b);if(b.timeStamp){this._timeStamp=b.timeStamp;};this._native=b;this._returnValue=null;return this;},clone:function(g){var h=qx.event.type.Event.prototype.clone.call(this,g);var i={};h._native=this._cloneNativeEvent(this._native,i);h._returnValue=this._returnValue;return h;},_cloneNativeEvent:function(j,k){k.preventDefault=(function(){});return k;},preventDefault:function(){qx.event.type.Event.prototype.preventDefault.call(this);qx.bom.Event.preventDefault(this._native);},getNativeEvent:function(){return this._native;},setReturnValue:function(l){this._returnValue=l;},getReturnValue:function(){return this._returnValue;}},destruct:function(){this._native=this._returnValue=null;}});})();(function(){var a="qx.globalErrorHandling",b="qx.event.handler.Window";qx.Class.define(b,{extend:qx.core.Object,implement:qx.event.IEventHandler,construct:function(c){qx.core.Object.call(this);this._manager=c;this._window=c.getWindow();this._initWindowObserver();},statics:{PRIORITY:qx.event.Registration.PRIORITY_NORMAL,SUPPORTED_TYPES:{error:1,load:1,beforeunload:1,unload:1,resize:1,scroll:1,beforeshutdown:1},TARGET_CHECK:qx.event.IEventHandler.TARGET_WINDOW,IGNORE_CAN_HANDLE:true},members:{canHandleEvent:function(f,d){},registerEvent:function(i,h,g){},unregisterEvent:function(l,k,j){},_initWindowObserver:function(){this._onNativeWrapper=qx.lang.Function.listener(this._onNative,this);var m=qx.event.handler.Window.SUPPORTED_TYPES;for(var n in m){qx.bom.Event.addNativeListener(this._window,n,this._onNativeWrapper);};},_stopWindowObserver:function(){var o=qx.event.handler.Window.SUPPORTED_TYPES;for(var p in o){qx.bom.Event.removeNativeListener(this._window,p,this._onNativeWrapper);};},_onNative:function(){var q=qx.core.Environment.select(a,{"true":qx.event.GlobalError.observeMethod(this.__cU),"false":this.__cU});q.apply(this,arguments);},__cU:function(e){if(this.isDisposed()){return;};var v=this._window;var s;try{s=v.document;}catch(w){return;};var t=s.documentElement;var r=qx.bom.Event.getTarget(e);if(r==null||r===v||r===s||r===t){var event=qx.event.Registration.createEvent(e.type,qx.event.type.Native,[e,v]);qx.event.Registration.dispatchEvent(v,event);var u=event.getReturnValue();if(u!=null){e.returnValue=u;return u;};};}},destruct:function(){this._stopWindowObserver();this._manager=this._window=null;},defer:function(x){qx.event.Registration.addHandler(x);}});})();(function(){var a="ready",b="mshtml",c="engine.name",d="qx.event.handler.Application",f="complete",g="webkit",h="gecko",i="qx.globalErrorHandling",j="load",k="unload",l="opera",m="left",n="DOMContentLoaded",o="shutdown",p="browser.documentmode";qx.Class.define(d,{extend:qx.core.Object,implement:qx.event.IEventHandler,construct:function(q){qx.core.Object.call(this);this._window=q.getWindow();this.__cV=false;this.__cW=false;this.__cX=false;this.__cY=false;this._initObserver();qx.event.handler.Application.$$instance=this;},statics:{PRIORITY:qx.event.Registration.PRIORITY_NORMAL,SUPPORTED_TYPES:{ready:1,shutdown:1},TARGET_CHECK:qx.event.IEventHandler.TARGET_WINDOW,IGNORE_CAN_HANDLE:true,onScriptLoaded:function(){var r=qx.event.handler.Application.$$instance;if(r){r.__da();};}},members:{canHandleEvent:function(t,s){},registerEvent:function(w,v,u){},unregisterEvent:function(z,y,x){},__cX:null,__cV:null,__cW:null,__cY:null,__da:function(){if(!this.__cX&&this.__cV&&qx.$$loader.scriptLoaded){if((qx.core.Environment.get(c)==b)){if(qx.event.Registration.hasListener(this._window,a)){this.__cX=true;qx.event.Registration.fireEvent(this._window,a);};}else {this.__cX=true;qx.event.Registration.fireEvent(this._window,a);};};},isApplicationReady:function(){return this.__cX;},_initObserver:function(){if(qx.$$domReady||document.readyState==f||document.readyState==a){this.__cV=true;this.__da();}else {this._onNativeLoadWrapped=qx.lang.Function.bind(this._onNativeLoad,this);if(qx.core.Environment.get(c)==h||qx.core.Environment.get(c)==l||qx.core.Environment.get(c)==g||(qx.core.Environment.get(c)==b&&qx.core.Environment.get(p)>8)){qx.bom.Event.addNativeListener(this._window,n,this._onNativeLoadWrapped);}else {var self=this;var A=function(){try{document.documentElement.doScroll(m);if(document.body){self._onNativeLoadWrapped();};}catch(B){window.setTimeout(A,100);};};A();};qx.bom.Event.addNativeListener(this._window,j,this._onNativeLoadWrapped);};this._onNativeUnloadWrapped=qx.lang.Function.bind(this._onNativeUnload,this);qx.bom.Event.addNativeListener(this._window,k,this._onNativeUnloadWrapped);},_stopObserver:function(){if(this._onNativeLoadWrapped){qx.bom.Event.removeNativeListener(this._window,j,this._onNativeLoadWrapped);};qx.bom.Event.removeNativeListener(this._window,k,this._onNativeUnloadWrapped);this._onNativeLoadWrapped=null;this._onNativeUnloadWrapped=null;},_onNativeLoad:function(){var C=qx.core.Environment.select(i,{"true":qx.event.GlobalError.observeMethod(this.__db),"false":this.__db});C.apply(this,arguments);},__db:function(){this.__cV=true;this.__da();},_onNativeUnload:function(){var D=qx.core.Environment.select(i,{"true":qx.event.GlobalError.observeMethod(this.__dc),"false":this.__dc});D.apply(this,arguments);},__dc:function(){if(!this.__cY){this.__cY=true;try{qx.event.Registration.fireEvent(this._window,o);}catch(e){throw e;}finally{qx.core.ObjectRegistry.shutdown();};};}},destruct:function(){this._stopObserver();this._window=null;},defer:function(E){qx.event.Registration.addHandler(E);}});})();(function(){var a="ready",b="shutdown",c="beforeunload",d="qx.core.Init";qx.Class.define(d,{statics:{getApplication:qx.core.BaseInit.getApplication,ready:qx.core.BaseInit.ready,__cS:function(e){var f=this.getApplication();if(f){e.setReturnValue(f.close());};},__cT:function(){var g=this.getApplication();if(g){g.terminate();};}},defer:function(h){qx.event.Registration.addListener(window,a,h.ready,h);qx.event.Registration.addListener(window,b,h.__cT,h);qx.event.Registration.addListener(window,c,h.__cS,h);}});})();(function(){var a="Abstract method call",b="abstract",c="*",d="",e="-webkit-tap-highlight-color: transparent;",f="-ms-touch-select: none;",g="qx.application.AbstractGui",h="-webkit-touch-callout: none;",i="-webkit-tap-highlight-color: rgba(0,0,0,0);";qx.Class.define(g,{type:b,extend:qx.core.Object,implement:[qx.application.IApplication],include:qx.locale.MTranslation,members:{__dd:null,_createRootWidget:function(){throw new Error(a);},getRoot:function(){return this.__dd;},main:function(){qx.theme.manager.Meta.getInstance().initialize();qx.ui.tooltip.Manager.getInstance();var j=[h,f,i,e].join(d);qx.ui.style.Stylesheet.getInstance().addRule(c,j);this.__dd=this._createRootWidget();window.scrollTo(0,0);},finalize:function(){this.render();},render:function(){qx.ui.core.queue.Manager.flush();},close:function(k){},terminate:function(){}},destruct:function(){this.__dd=null;}});})();(function(){var a="The theme to use is not available: ",b="_applyTheme",c="qx.theme",d="qx.theme.manager.Meta",f="qx.theme.Modern",g="qx.event.type.Event",h="Theme",i="changeTheme",j="singleton";qx.Class.define(d,{type:j,extend:qx.core.Object,events:{"changeTheme":g},properties:{theme:{check:h,nullable:false,apply:b}},members:{_applyTheme:function(k,m){var u=true;var w=true;var o=true;var q=true;var l=true;if(m){u=k.meta.color!==m.meta.color;w=k.meta.decoration!==m.meta.decoration;o=k.meta.font!==m.meta.font;q=k.meta.icon!==m.meta.icon;l=k.meta.appearance!==m.meta.appearance;};var n=qx.theme.manager.Color.getInstance();var t=qx.theme.manager.Decoration.getInstance();var r=qx.theme.manager.Font.getInstance();var p=qx.theme.manager.Icon.getInstance();var s=qx.theme.manager.Appearance.getInstance();this._suspendEvents();if(u){if(!w){var v=t.getTheme();t._applyTheme(v);};n.setTheme(k.meta.color);};t.setTheme(k.meta.decoration);r.setTheme(k.meta.font);p.setTheme(k.meta.icon);s.setTheme(k.meta.appearance);if(u||w||o||q||l){this.fireEvent(i);};this._activateEvents();},__de:null,_fireEvent:function(e){if(e.getTarget()===qx.theme.manager.Color.getInstance()){qx.theme.manager.Decoration.getInstance().refresh();};this.fireEvent(i);},_suspendEvents:function(){var B=qx.theme.manager.Color.getInstance();var A=qx.theme.manager.Decoration.getInstance();var x=qx.theme.manager.Font.getInstance();var z=qx.theme.manager.Icon.getInstance();var y=qx.theme.manager.Appearance.getInstance();if(B.hasListener(i)){B.removeListener(i,this._fireEvent,this);};if(A.hasListener(i)){A.removeListener(i,this._fireEvent,this);};if(x.hasListener(i)){x.removeListener(i,this._fireEvent,this);};if(z.hasListener(i)){z.removeListener(i,this._fireEvent,this);};if(y.hasListener(i)){y.removeListener(i,this._fireEvent,this);};},_activateEvents:function(){var G=qx.theme.manager.Color.getInstance();var F=qx.theme.manager.Decoration.getInstance();var C=qx.theme.manager.Font.getInstance();var E=qx.theme.manager.Icon.getInstance();var D=qx.theme.manager.Appearance.getInstance();if(!G.hasListener(i)){G.addListener(i,this._fireEvent,this);};if(!F.hasListener(i)){F.addListener(i,this._fireEvent,this);};if(!C.hasListener(i)){C.addListener(i,this._fireEvent,this);};if(!E.hasListener(i)){E.addListener(i,this._fireEvent,this);};if(!D.hasListener(i)){D.addListener(i,this._fireEvent,this);};},initialize:function(){var J=qx.core.Environment;var H,I;H=J.get(c);if(H){I=qx.Theme.getByName(H);if(!I){throw new Error(a+H);};this.setTheme(I);};}},environment:{"qx.theme":f}});})();(function(){var a="qx.util.ValueManager",b="abstract";qx.Class.define(a,{type:b,extend:qx.core.Object,construct:function(){qx.core.Object.call(this);this._dynamic={};},members:{_dynamic:null,resolveDynamic:function(c){return this._dynamic[c];},isDynamic:function(d){return !!this._dynamic[d];},resolve:function(e){if(e&&this._dynamic[e]){return this._dynamic[e];};return e;},_setDynamic:function(f){this._dynamic=f;},_getDynamic:function(){return this._dynamic;}},destruct:function(){this._dynamic=null;}});})();(function(){var a="Could not parse color: ",b="_applyTheme",c="qx.theme.manager.Color",d="Theme",e="changeTheme",f="string",g="singleton";qx.Class.define(c,{type:g,extend:qx.util.ValueManager,properties:{theme:{check:d,nullable:true,apply:b,event:e}},members:{_applyTheme:function(j){var h={};if(j){var i=j.colors;for(var name in i){h[name]=this.__df(i,name);};};this._setDynamic(h);},__df:function(l,name){var k=l[name];if(typeof k===f){if(!qx.util.ColorUtil.isCssString(k)){if(l[k]!=undefined){return this.__df(l,k);};throw new Error(a+k);};return k;}else if(k instanceof Array){return qx.util.ColorUtil.rgbToRgbString(k);};throw new Error(a+k);},resolve:function(p){var o=this._dynamic;var m=o[p];if(m){return m;};var n=this.getTheme();if(n!==null&&n.colors[p]){return o[p]=n.colors[p];};return p;},isDynamic:function(s){var r=this._dynamic;if(s&&(r[s]!==undefined)){return true;};var q=this.getTheme();if(q!==null&&s&&(q.colors[s]!==undefined)){r[s]=q.colors[s];return true;};return false;}}});})();(function(){var a="Could not parse color: ",c="",d="Invalid hex value: ",e="Could not convert system colors to RGB: ",h="(",j=")",k="#",l="a",m="Invalid hex3 value: ",n="qx.theme.manager.Color",o="qx.util.ColorUtil",q="Invalid hex6 value: ",s="rgb",u=",";qx.Bootstrap.define(o,{statics:{REGEXP:{hex3:/^#([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/,hex6:/^#([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/,rgb:/^rgb\(\s*([0-9]{1,3}\.{0,1}[0-9]*)\s*,\s*([0-9]{1,3}\.{0,1}[0-9]*)\s*,\s*([0-9]{1,3}\.{0,1}[0-9]*)\s*\)$/,rgba:/^rgba\(\s*([0-9]{1,3}\.{0,1}[0-9]*)\s*,\s*([0-9]{1,3}\.{0,1}[0-9]*)\s*,\s*([0-9]{1,3}\.{0,1}[0-9]*)\s*,\s*([0-9]{1,3}\.{0,1}[0-9]*)\s*\)$/},SYSTEM:{activeborder:true,activecaption:true,appworkspace:true,background:true,buttonface:true,buttonhighlight:true,buttonshadow:true,buttontext:true,captiontext:true,graytext:true,highlight:true,highlighttext:true,inactiveborder:true,inactivecaption:true,inactivecaptiontext:true,infobackground:true,infotext:true,menu:true,menutext:true,scrollbar:true,threeddarkshadow:true,threedface:true,threedhighlight:true,threedlightshadow:true,threedshadow:true,window:true,windowframe:true,windowtext:true},NAMED:{black:[0,0,0],silver:[192,192,192],gray:[128,128,128],white:[255,255,255],maroon:[128,0,0],red:[255,0,0],purple:[128,0,128],fuchsia:[255,0,255],green:[0,128,0],lime:[0,255,0],olive:[128,128,0],yellow:[255,255,0],navy:[0,0,128],blue:[0,0,255],teal:[0,128,128],aqua:[0,255,255],transparent:[-1,-1,-1],magenta:[255,0,255],orange:[255,165,0],brown:[165,42,42]},isNamedColor:function(v){return this.NAMED[v]!==undefined;},isSystemColor:function(w){return this.SYSTEM[w]!==undefined;},supportsThemes:function(){if(qx.Class){return qx.Class.isDefined(n);};return false;},isThemedColor:function(x){if(!this.supportsThemes()){return false;};if(qx.theme&&qx.theme.manager&&qx.theme.manager.Color){return qx.theme.manager.Color.getInstance().isDynamic(x);};return false;},stringToRgb:function(y){if(this.supportsThemes()&&this.isThemedColor(y)){y=qx.theme.manager.Color.getInstance().resolveDynamic(y);};if(this.isNamedColor(y)){return this.NAMED[y].concat();}else if(this.isSystemColor(y)){throw new Error(e+y);}else if(this.isRgbaString(y)){return this.__dh(y);}else if(this.isRgbString(y)){return this.__dg();}else if(this.isHex3String(y)){return this.__di();}else if(this.isHex6String(y)){return this.__dj();};throw new Error(a+y);},cssStringToRgb:function(z){if(this.isNamedColor(z)){return this.NAMED[z];}else if(this.isSystemColor(z)){throw new Error(e+z);}else if(this.isRgbString(z)){return this.__dg();}else if(this.isRgbaString(z)){return this.__dh();}else if(this.isHex3String(z)){return this.__di();}else if(this.isHex6String(z)){return this.__dj();};throw new Error(a+z);},stringToRgbString:function(A){return this.rgbToRgbString(this.stringToRgb(A));},rgbToRgbString:function(B){return s+(B[3]!==undefined?l:c)+h+B.join(u)+j;},rgbToHexString:function(C){return (k+qx.lang.String.pad(C[0].toString(16).toUpperCase(),2)+qx.lang.String.pad(C[1].toString(16).toUpperCase(),2)+qx.lang.String.pad(C[2].toString(16).toUpperCase(),2));},isValidPropertyValue:function(D){return (this.isThemedColor(D)||this.isNamedColor(D)||this.isHex3String(D)||this.isHex6String(D)||this.isRgbString(D)||this.isRgbaString(D));},isCssString:function(E){return (this.isSystemColor(E)||this.isNamedColor(E)||this.isHex3String(E)||this.isHex6String(E)||this.isRgbString(E)||this.isRgbaString(E));},isHex3String:function(F){return this.REGEXP.hex3.test(F);},isHex6String:function(G){return this.REGEXP.hex6.test(G);},isRgbString:function(H){return this.REGEXP.rgb.test(H);},isRgbaString:function(I){return this.REGEXP.rgba.test(I);},__dg:function(){var L=parseInt(RegExp.$1,10);var K=parseInt(RegExp.$2,10);var J=parseInt(RegExp.$3,10);return [L,K,J];},__dh:function(){var P=parseInt(RegExp.$1,10);var O=parseInt(RegExp.$2,10);var M=parseInt(RegExp.$3,10);var N=parseFloat(RegExp.$4,10);if(P===0&&O===0&M===0&&N===0){return [-1,-1,-1];};return [P,O,M];},__di:function(){var S=parseInt(RegExp.$1,16)*17;var R=parseInt(RegExp.$2,16)*17;var Q=parseInt(RegExp.$3,16)*17;return [S,R,Q];},__dj:function(){var V=(parseInt(RegExp.$1,16)*16)+parseInt(RegExp.$2,16);var U=(parseInt(RegExp.$3,16)*16)+parseInt(RegExp.$4,16);var T=(parseInt(RegExp.$5,16)*16)+parseInt(RegExp.$6,16);return [V,U,T];},hex3StringToRgb:function(W){if(this.isHex3String(W)){return this.__di(W);};throw new Error(m+W);},hex3StringToHex6String:function(X){if(this.isHex3String(X)){return this.rgbToHexString(this.hex3StringToRgb(X));};return X;},hex6StringToRgb:function(Y){if(this.isHex6String(Y)){return this.__dj(Y);};throw new Error(q+Y);},hexStringToRgb:function(ba){if(this.isHex3String(ba)){return this.__di(ba);};if(this.isHex6String(ba)){return this.__dj(ba);};throw new Error(d+ba);},rgbToHsb:function(bi){var bc,bd,bf;var bm=bi[0];var bj=bi[1];var bb=bi[2];var bl=(bm>bj)?bm:bj;if(bb>bl){bl=bb;};var be=(bm<bj)?bm:bj;if(bb<be){be=bb;};bf=bl/255.0;if(bl!=0){bd=(bl-be)/bl;}else {bd=0;};if(bd==0){bc=0;}else {var bh=(bl-bm)/(bl-be);var bk=(bl-bj)/(bl-be);var bg=(bl-bb)/(bl-be);if(bm==bl){bc=bg-bk;}else if(bj==bl){bc=2.0+bh-bg;}else {bc=4.0+bk-bh;};bc=bc/6.0;if(bc<0){bc=bc+1.0;};};return [Math.round(bc*360),Math.round(bd*100),Math.round(bf*100)];},hsbToRgb:function(bn){var i,f,p,r,t;var bo=bn[0]/360;var bp=bn[1]/100;var bq=bn[2]/100;if(bo>=1.0){bo%=1.0;};if(bp>1.0){bp=1.0;};if(bq>1.0){bq=1.0;};var br=Math.floor(255*bq);var bs={};if(bp==0.0){bs.red=bs.green=bs.blue=br;}else {bo*=6.0;i=Math.floor(bo);f=bo-i;p=Math.floor(br*(1.0-bp));r=Math.floor(br*(1.0-(bp*f)));t=Math.floor(br*(1.0-(bp*(1.0-f))));switch(i){case 0:bs.red=br;bs.green=t;bs.blue=p;break;case 1:bs.red=r;bs.green=br;bs.blue=p;break;case 2:bs.red=p;bs.green=br;bs.blue=t;break;case 3:bs.red=p;bs.green=r;bs.blue=br;break;case 4:bs.red=t;bs.green=p;bs.blue=br;break;case 5:bs.red=br;bs.green=p;bs.blue=r;break;};};return [bs.red,bs.green,bs.blue];},randomColor:function(){var r=Math.round(Math.random()*255);var g=Math.round(Math.random()*255);var b=Math.round(Math.random()*255);return this.rgbToRgbString([r,g,b]);}}});})();(function(){var a="mshtml",b="engine.name",c="_applyTheme",d="",e="'.",f="qx-",g="__dm",h="Unable to resolve decorator '",j="singleton",k=";",l="qx.theme.manager.Decoration",m=".",n="Theme",o="object",p="changeTheme",q="string",r="browser.documentmode",s=":";qx.Class.define(l,{type:j,extend:qx.core.Object,statics:{CSS_CLASSNAME_PREFIX:f},construct:function(){qx.core.Object.call(this);this.__dk=[];this.__dl=(qx.core.Environment.get(b)==a&&qx.core.Environment.get(r)<9);},properties:{theme:{check:n,nullable:true,apply:c,event:p}},members:{__dm:null,__dk:null,__dl:false,getCssClassName:function(u){var t=qx.theme.manager.Decoration.CSS_CLASSNAME_PREFIX;if(qx.lang.Type.isString(u)){return t+u;}else {return t+u.toHashCode();};},addCssClass:function(z){var w=qx.ui.style.Stylesheet.getInstance();var B=z;z=this.getCssClassName(z);var A=m+z;if(w.hasRule(A)){return z;};if(qx.lang.Type.isString(B)){B=this.resolve(B);};if(!B){throw new Error(h+z+e);};var G=d;var v=B.getStyles(true);for(var D in v){if(qx.Bootstrap.isObject(v[D])){var x=d;var F=v[D];var C=false;for(var y in F){C=true;x+=y+s+F[y]+k;};var E=this.__dl?A:A+(C?s:d);this.__dk.push(E+D);w.addRule(E+D,x);continue;};G+=D+s+v[D]+k;};if(G){w.addRule(A,G);this.__dk.push(A);};return z;},removeAllCssClasses:function(){for(var i=0;i<this.__dk.length;i++ ){var H=this.__dk[i];qx.ui.style.Stylesheet.getInstance().removeRule(H);};this.__dk=[];},resolve:function(L){if(!L){return null;};if(typeof L===o){return L;};var M=this.getTheme();if(!M){return null;};var J=this.__dm;if(!J){J=this.__dm={};};var I=J[L];if(I){return I;};var O=qx.lang.Object.clone(M.decorations[L],true);if(!O){return null;};if(!O.style){O.style={};};var K=O;while(K.include){K=M.decorations[K.include];if(!O.decorator&&K.decorator){O.decorator=qx.lang.Object.clone(K.decorator);};if(K.style){for(var N in K.style){if(O.style[N]===undefined){O.style[N]=qx.lang.Object.clone(K.style[N],true);};};};};return J[L]=(new qx.ui.decoration.Decorator()).set(O.style);},isValidPropertyValue:function(P){if(typeof P===q){return this.isDynamic(P);}else if(typeof P===o){var Q=P.constructor;return qx.Class.hasInterface(Q,qx.ui.decoration.IDecorator);};return false;},isDynamic:function(S){if(!S){return false;};var R=this.getTheme();if(!R){return false;};return !!R.decorations[S];},isCached:function(T){return !this.__dm?false:qx.lang.Object.contains(this.__dm,T);},_applyTheme:function(X,V){var W=qx.util.AliasManager.getInstance();this.removeAllCssClasses();if(V){for(var U in V.aliases){W.remove(U);};};if(X){for(var U in X.aliases){W.add(U,X.aliases[U]);};};this._disposeMap(g);this.__dm={};},clear:function(){var bb=qx.util.AliasManager.getInstance();var ba=this.getTheme();if(!bb.isDisposed()&&ba&&ba.alias){for(var Y in ba.aliases){bb.remove(Y,ba.aliases[Y]);};};this.removeAllCssClasses();this._disposeMap(g);this.__dm={};},refresh:function(){this.clear();var be=qx.util.AliasManager.getInstance();var bd=this.getTheme();if(bd&&bd.alias){for(var bc in bd.aliases){be.add(bc,bd.aliases[bc]);};};}},destruct:function(){this.clear();}});})();(function(){var a="qx.ui.style.Stylesheet",b="singleton";qx.Class.define(a,{type:b,extend:qx.core.Object,construct:function(){qx.core.Object.call(this);this.__dn=qx.bom.Stylesheet.createElement();this.__dk=[];},members:{__dk:null,__dn:null,addRule:function(d,c){if(this.hasRule(d)){return;};qx.bom.Stylesheet.addRule(this.__dn,d,c);this.__dk.push(d);},hasRule:function(e){return this.__dk.indexOf(e)!=-1;},removeRule:function(f){delete this.__dk[this.__dk.indexOf(f)];qx.bom.Stylesheet.removeRule(this.__dn,f);}}});})();(function(){var a="stylesheet",b="head",c="html.stylesheet.addimport",d="html.stylesheet.insertrule",e="}",f="html.stylesheet.createstylesheet",g='@import "',h="text/css",j="{",k='";',l="html.stylesheet.removeimport",m="html.stylesheet.deleterule",n="qx.bom.Stylesheet",o="link",p="style";qx.Bootstrap.define(n,{statics:{includeFile:function(s,q){if(!q){q=document;};var t=q.createElement(o);t.type=h;t.rel=a;t.href=s;var r=q.getElementsByTagName(b)[0];r.appendChild(t);},createElement:function(u){if(qx.core.Environment.get(f)){var v=document.createStyleSheet();if(u){v.cssText=u;};return v;}else {var w=document.createElement(p);w.type=h;if(u){w.appendChild(document.createTextNode(u));};document.getElementsByTagName(b)[0].appendChild(w);return w.sheet;};},addRule:function(z,A,y){{var x;};if(qx.core.Environment.get(d)){z.insertRule(A+j+y+e,z.cssRules.length);}else {z.addRule(A,y);};},removeRule:function(C,E){if(qx.core.Environment.get(m)){var B=C.cssRules;var D=B.length;for(var i=D-1;i>=0; --i){if(B[i].selectorText==E){C.deleteRule(i);};};}else {var B=C.rules;var D=B.length;for(var i=D-1;i>=0; --i){if(B[i].selectorText==E){C.removeRule(i);};};};},removeSheet:function(G){var F=G.ownerNode?G.ownerNode:G.owningElement;qx.dom.Element.removeChild(F,F.parentNode);},removeAllRules:function(I){if(qx.core.Environment.get(m)){var H=I.cssRules;var J=H.length;for(var i=J-1;i>=0;i-- ){I.deleteRule(i);};}else {var H=I.rules;var J=H.length;for(var i=J-1;i>=0;i-- ){I.removeRule(i);};};},addImport:function(L,K){if(qx.core.Environment.get(c)){L.addImport(K);}else {L.insertRule(g+K+k,L.cssRules.length);};},removeImport:function(M,N){if(qx.core.Environment.get(l)){var O=M.imports;var P=O.length;for(var i=P-1;i>=0;i-- ){if(O[i].href==N||O[i].href==qx.util.Uri.getAbsolute(N)){M.removeImport(i);};};}else {var Q=M.cssRules;var P=Q.length;for(var i=P-1;i>=0;i-- ){if(Q[i].href==N){M.deleteRule(i);};};};},removeAllImports:function(S){if(qx.core.Environment.get(l)){var U=S.imports;var T=U.length;for(var i=T-1;i>=0;i-- ){S.removeImport(i);};}else {var R=S.cssRules;var T=R.length;for(var i=T-1;i>=0;i-- ){if(R[i].type==R[i].IMPORT_RULE){S.deleteRule(i);};};};}}});})();(function(){var a="engine.name",b="",c="none",d="qx.dom.Element",e="webkit",f="The tag name is missing!",g="div";qx.Bootstrap.define(d,{statics:{hasChild:function(parent,h){return h.parentNode===parent;},hasChildren:function(j){return !!j.firstChild;},hasChildElements:function(k){k=k.firstChild;while(k){if(k.nodeType===1){return true;};k=k.nextSibling;};return false;},getParentElement:function(m){return m.parentNode;},isInDom:function(p,n){if(!n){n=window;};var o=n.document.getElementsByTagName(p.nodeName);for(var i=0,l=o.length;i<l;i++ ){if(o[i]===p){return true;};};return false;},insertAt:function(q,parent,r){var s=parent.childNodes[r];if(s){parent.insertBefore(q,s);}else {parent.appendChild(q);};return true;},insertBegin:function(t,parent){if(parent.firstChild){this.insertBefore(t,parent.firstChild);}else {parent.appendChild(t);};return true;},insertEnd:function(u,parent){parent.appendChild(u);return true;},insertBefore:function(v,w){w.parentNode.insertBefore(v,w);return true;},insertAfter:function(x,y){var parent=y.parentNode;if(y==parent.lastChild){parent.appendChild(x);}else {return this.insertBefore(x,y.nextSibling);};return true;},remove:function(z){if(!z.parentNode){return false;};z.parentNode.removeChild(z);return true;},removeChild:function(A,parent){if(A.parentNode!==parent){return false;};parent.removeChild(A);return true;},removeChildAt:function(B,parent){var C=parent.childNodes[B];if(!C){return false;};parent.removeChild(C);return true;},replaceChild:function(E,D){if(!D.parentNode){return false;};D.parentNode.replaceChild(E,D);return true;},replaceAt:function(G,H,parent){var F=parent.childNodes[H];if(!F){return false;};parent.replaceChild(G,F);return true;},__do:{},getHelperElement:function(I){if(!I){I=window;};var J=I.location.href;if(!qx.dom.Element.__do[J]){var K=qx.dom.Element.__do[J]=I.document.createElement(g);if(qx.core.Environment.get(a)==e){K.style.display=c;I.document.body.appendChild(K);};};return qx.dom.Element.__do[J];},create:function(name,M,L){if(!L){L=window;};if(!name){throw new Error(f);};var O=L.document.createElement(name);for(var N in M){qx.bom.element.Attribute.set(O,N,M[N]);};return O;},empty:function(P){return P.innerHTML=b;}}});})();(function(){var b="function",c="html.video.h264",d="html.element.contains",f='video/ogg; codecs="theora, vorbis"',g="qxtest",h="html.console",i="html.xul",j="http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul",k="html.video.ogg",l="http://www.w3.org/TR/SVG11/feature#BasicStructure",m="html.storage.local",n="div",o="qx.bom.client.Html",p="getSelection",q='audio',r='video/mp4; codecs="avc1.42E01E, mp4a.40.2"',s="data:image/gif;base64,R0lGODlhAQABAIAAAAAAAP///ywAAAAAAQABAAACAUwAOw==",t="html.audio",u="video",w="url(#default#VML)",x="head",y="audio",z="audio/mpeg",A="org.w3c.dom.svg",B="html.classlist",C="html.svg",D="html.video",E="html.geolocation",F="DOMTokenList",G="html.storage.session",H="1.1",I="html.history.state",J="object",K="html.image.naturaldimensions",L="html.audio.aif",M="audio/x-wav",N='<v:shape id="vml_flag1" adj="1" />',O="html.node.isequalnode",P="html.canvas",Q="audio/ogg",R="",S="html.storage.userdata",T="html.fullscreen",U="number",V="html.element.compareDocumentPosition",W="audio/x-aiff",X="html.audio.au",Y="img",bF="html.selection",bG="selection",bH="html.xpath",bB="$qx_check",bC="test",bD='video',bE="span",bM="html.element.textcontent",bN="geolocation",bW="html.audio.mp3",bA="html.vml",bI="undefined",bJ="html.audio.ogg",bK="none",bL="label",bQ='video/webm; codecs="vp8, vorbis"',ca="html.dataurl",bR="html.webworker",bS="html.dataset",bX="1.0",bO="html.audio.wav",bY="html.filereader",bP="audio/basic",bT="display",bU="html.video.webm",bV="#default#userdata";qx.Bootstrap.define(o,{statics:{getWebWorker:function(){return window.Worker!=null;},getFileReader:function(){return window.FileReader!=null;},getGeoLocation:function(){return bN in navigator;},getAudio:function(){return !!document.createElement(q).canPlayType;},getAudioOgg:function(){if(!qx.bom.client.Html.getAudio()){return R;};var a=document.createElement(y);return a.canPlayType(Q);},getAudioMp3:function(){if(!qx.bom.client.Html.getAudio()){return R;};var a=document.createElement(y);return a.canPlayType(z);},getAudioWav:function(){if(!qx.bom.client.Html.getAudio()){return R;};var a=document.createElement(y);return a.canPlayType(M);},getAudioAu:function(){if(!qx.bom.client.Html.getAudio()){return R;};var a=document.createElement(y);return a.canPlayType(bP);},getAudioAif:function(){if(!qx.bom.client.Html.getAudio()){return R;};var a=document.createElement(y);return a.canPlayType(W);},getVideo:function(){return !!document.createElement(bD).canPlayType;},getVideoOgg:function(){if(!qx.bom.client.Html.getVideo()){return R;};var v=document.createElement(u);return v.canPlayType(f);},getVideoH264:function(){if(!qx.bom.client.Html.getVideo()){return R;};var v=document.createElement(u);return v.canPlayType(r);},getVideoWebm:function(){if(!qx.bom.client.Html.getVideo()){return R;};var v=document.createElement(u);return v.canPlayType(bQ);},getLocalStorage:function(){try{window.localStorage.setItem(bB,bC);window.localStorage.removeItem(bB);return true;}catch(cb){return false;};},getSessionStorage:function(){try{window.sessionStorage.setItem(bB,bC);window.sessionStorage.removeItem(bB);return true;}catch(cc){return false;};},getUserDataStorage:function(){var cd=document.createElement(n);cd.style[bT]=bK;document.getElementsByTagName(x)[0].appendChild(cd);var ce=false;try{cd.addBehavior(bV);cd.load(g);ce=true;}catch(e){};document.getElementsByTagName(x)[0].removeChild(cd);return ce;},getClassList:function(){return !!(document.documentElement.classList&&qx.Bootstrap.getClass(document.documentElement.classList)===F);},getXPath:function(){return !!document.evaluate;},getXul:function(){try{document.createElementNS(j,bL);return true;}catch(e){return false;};},getSvg:function(){return document.implementation&&document.implementation.hasFeature&&(document.implementation.hasFeature(A,bX)||document.implementation.hasFeature(l,H));},getVml:function(){var cf=document.createElement(n);document.body.appendChild(cf);cf.innerHTML=N;cf.firstChild.style.behavior=w;var cg=typeof cf.firstChild.adj==J;document.body.removeChild(cf);return cg;},getCanvas:function(){return !!window.CanvasRenderingContext2D;},getDataUrl:function(ch){var ci=new Image();ci.onload=ci.onerror=function(){window.setTimeout(function(){ch.call(null,(ci.width==1&&ci.height==1));},0);};ci.src=s;},getDataset:function(){return !!document.documentElement.dataset;},getContains:function(){return (typeof document.documentElement.contains!==bI);},getCompareDocumentPosition:function(){return (typeof document.documentElement.compareDocumentPosition===b);},getTextContent:function(){var cj=document.createElement(bE);return (typeof cj.textContent!==bI);},getFullScreen:function(){return document.fullscreenEnabled||document.webkitFullscreenEnabled||document.mozFullScreenEnabled||document.msFullscreenEnabled||false;},getConsole:function(){return typeof window.console!==bI;},getNaturalDimensions:function(){var ck=document.createElement(Y);return typeof ck.naturalHeight===U&&typeof ck.naturalWidth===U;},getHistoryState:function(){return (typeof window.onpopstate!==bI&&typeof window.history.replaceState!==bI&&typeof window.history.pushState!==bI);},getSelection:function(){if(typeof window.getSelection===b){return p;};if(typeof document.selection===J){return bG;};return null;},getIsEqualNode:function(){return typeof document.documentElement.isEqualNode===b;}},defer:function(cl){qx.core.Environment.add(bR,cl.getWebWorker);qx.core.Environment.add(bY,cl.getFileReader);qx.core.Environment.add(E,cl.getGeoLocation);qx.core.Environment.add(t,cl.getAudio);qx.core.Environment.add(bJ,cl.getAudioOgg);qx.core.Environment.add(bW,cl.getAudioMp3);qx.core.Environment.add(bO,cl.getAudioWav);qx.core.Environment.add(X,cl.getAudioAu);qx.core.Environment.add(L,cl.getAudioAif);qx.core.Environment.add(D,cl.getVideo);qx.core.Environment.add(k,cl.getVideoOgg);qx.core.Environment.add(c,cl.getVideoH264);qx.core.Environment.add(bU,cl.getVideoWebm);qx.core.Environment.add(m,cl.getLocalStorage);qx.core.Environment.add(G,cl.getSessionStorage);qx.core.Environment.add(S,cl.getUserDataStorage);qx.core.Environment.add(B,cl.getClassList);qx.core.Environment.add(bH,cl.getXPath);qx.core.Environment.add(i,cl.getXul);qx.core.Environment.add(P,cl.getCanvas);qx.core.Environment.add(C,cl.getSvg);qx.core.Environment.add(bA,cl.getVml);qx.core.Environment.add(bS,cl.getDataset);qx.core.Environment.addAsync(ca,cl.getDataUrl);qx.core.Environment.add(d,cl.getContains);qx.core.Environment.add(V,cl.getCompareDocumentPosition);qx.core.Environment.add(bM,cl.getTextContent);qx.core.Environment.add(h,cl.getConsole);qx.core.Environment.add(K,cl.getNaturalDimensions);qx.core.Environment.add(I,cl.getHistoryState);qx.core.Environment.add(bF,cl.getSelection);qx.core.Environment.add(O,cl.getIsEqualNode);qx.core.Environment.add(T,cl.getFullScreen);}});})();(function(){var a="readOnly",b="data-",c="accessKey",d="qx.bom.element.Attribute",e="rowSpan",f="vAlign",g="className",h="textContent",i="'",j="htmlFor",k="longDesc",l="cellSpacing",m="frameBorder",n="='",o="",p="useMap",q="innerText",r="innerHTML",s="tabIndex",t="dateTime",u="maxLength",v="html.element.textcontent",w="mshtml",x="engine.name",y="cellPadding",z="browser.documentmode",A="colSpan",B="undefined";qx.Bootstrap.define(d,{statics:{__dp:{names:{"class":g,"for":j,html:r,text:qx.core.Environment.get(v)?h:q,colspan:A,rowspan:e,valign:f,datetime:t,accesskey:c,tabindex:s,maxlength:u,readonly:a,longdesc:k,cellpadding:y,cellspacing:l,frameborder:m,usemap:p},runtime:{"html":1,"text":1},bools:{compact:1,nowrap:1,ismap:1,declare:1,noshade:1,checked:1,disabled:1,readOnly:1,multiple:1,selected:1,noresize:1,defer:1,allowTransparency:1},property:{$$html:1,$$widget:1,checked:1,readOnly:1,multiple:1,selected:1,value:1,maxLength:1,className:1,innerHTML:1,innerText:1,textContent:1,htmlFor:1,tabIndex:1},qxProperties:{$$widget:1,$$html:1},propertyDefault:{disabled:false,checked:false,readOnly:false,multiple:false,selected:false,value:o,className:o,innerHTML:o,innerText:o,textContent:o,htmlFor:o,tabIndex:0,maxLength:qx.core.Environment.select(x,{"mshtml":2147483647,"webkit":524288,"default":-1})},removeableProperties:{disabled:1,multiple:1,maxLength:1}},compile:function(C){var D=[];var F=this.__dp.runtime;for(var E in C){if(!F[E]){D.push(E,n,C[E],i);};};return D.join(o);},get:function(I,name){var G=this.__dp;var H;name=G.names[name]||name;if(G.property[name]){H=I[name];if(typeof G.propertyDefault[name]!==B&&H==G.propertyDefault[name]){if(typeof G.bools[name]===B){return null;}else {return H;};};}else {H=I.getAttribute(name);if(G.bools[name]&&!(qx.core.Environment.get(x)==w&&parseInt(qx.core.Environment.get(z),10)<=8)){return qx.Bootstrap.isString(H);};};if(G.bools[name]){return !!H;};return H;},set:function(L,name,K){if(typeof K===B){return;};var J=this.__dp;name=J.names[name]||name;if(J.bools[name]&&!qx.lang.Type.isBoolean(K)){K=qx.lang.Type.isString(K);};if(J.property[name]&&(!(L[name]===undefined)||J.qxProperties[name])){if(K==null){if(J.removeableProperties[name]){L.removeAttribute(name);return;}else if(typeof J.propertyDefault[name]!==B){K=J.propertyDefault[name];};};L[name]=K;}else {if((J.bools[name]||K===null)&&name.indexOf(b)!==0){if(K===true){L.setAttribute(name,name);}else if(K===false||K===null){L.removeAttribute(name);};}else {L.setAttribute(name,K);};};},reset:function(M,name){if(name.indexOf(b)===0){M.removeAttribute(name);}else {this.set(M,name,null);};}}});})();(function(){var a="file",b="+",c="strict",d="anchor",e="div",f="query",g="source",h="password",j="host",k="protocol",l="user",n="directory",p="loose",q="relative",r="queryKey",s="qx.util.Uri",t="",u="path",v="authority",w='">0</a>',x="&",y="port",z='<a href="',A="userInfo",B="?",C="=";qx.Bootstrap.define(s,{statics:{parseUri:function(F,E){var G={key:[g,k,v,A,l,h,j,y,q,u,n,a,f,d],q:{name:r,parser:/(?:^|&)([^&=]*)=?([^&]*)/g},parser:{strict:/^(?:([^:\/?#]+):)?(?:\/\/((?:(([^:@]*)(?::([^:@]*))?)?@)?([^:\/?#]*)(?::(\d*))?))?((((?:[^?#\/]*\/)*)([^?#]*))(?:\?([^#]*))?(?:#(.*))?)/,loose:/^(?:(?![^:@]+:[^:@\/]*@)([^:\/?#.]+):)?(?:\/\/)?((?:(([^:@]*)(?::([^:@]*))?)?@)?([^:\/?#]*)(?::(\d*))?)(((\/(?:[^?#](?![^?#\/]*\.[^?#\/.]+(?:[?#]|$)))*\/?)?([^?#\/]*))(?:\?([^#]*))?(?:#(.*))?)/}};var o=G,m=G.parser[E?c:p].exec(F),D={},i=14;while(i-- ){D[o.key[i]]=m[i]||t;};D[o.q.name]={};D[o.key[12]].replace(o.q.parser,function(I,J,H){if(J){D[o.q.name][J]=H;};});return D;},appendParamsToUrl:function(K,L){if(L===undefined){return K;};{};if(qx.lang.Type.isObject(L)){L=qx.util.Uri.toParameter(L);};if(!L){return K;};return K+=/\?/.test(K)?x+L:B+L;},toParameter:function(M,Q){var P,O=[];for(P in M){if(M.hasOwnProperty(P)){var N=M[P];if(N instanceof Array){for(var i=0;i<N.length;i++ ){this.__dq(P,N[i],O,Q);};}else {this.__dq(P,N,O,Q);};};};return O.join(x);},__dq:function(U,V,T,S){var R=window.encodeURIComponent;if(S){T.push(R(U).replace(/%20/g,b)+C+R(V).replace(/%20/g,b));}else {T.push(R(U)+C+R(V));};},getAbsolute:function(X){var W=document.createElement(e);W.innerHTML=z+X+w;return W.firstChild.href;}}});})();(function(){var a="qx.bom.client.Stylesheet",b="html.stylesheet.deleterule",c="html.stylesheet.insertrule",d="function",e="html.stylesheet.createstylesheet",f="html.stylesheet.addimport",g="html.stylesheet.removeimport",h="object";qx.Bootstrap.define(a,{statics:{__dr:function(){if(!qx.bom.client.Stylesheet.__ds){qx.bom.client.Stylesheet.__ds=qx.bom.Stylesheet.createElement();};return qx.bom.client.Stylesheet.__ds;},getCreateStyleSheet:function(){return typeof document.createStyleSheet===h;},getInsertRule:function(){return typeof qx.bom.client.Stylesheet.__dr().insertRule===d;},getDeleteRule:function(){return typeof qx.bom.client.Stylesheet.__dr().deleteRule===d;},getAddImport:function(){return (typeof qx.bom.client.Stylesheet.__dr().addImport===h);},getRemoveImport:function(){return (typeof qx.bom.client.Stylesheet.__dr().removeImport===h);}},defer:function(i){qx.core.Environment.add(e,i.getCreateStyleSheet);qx.core.Environment.add(c,i.getInsertRule);qx.core.Environment.add(b,i.getDeleteRule);qx.core.Environment.add(f,i.getAddImport);qx.core.Environment.add(g,i.getRemoveImport);}});})();(function(){var a='[object Boolean]',b='[object String]',c='constructor',d='[object Date]',e='[object Number]',f='object',g="qx.lang.Object",h='[object RegExp]',j='[object Array]';qx.Bootstrap.define(g,{statics:{empty:function(k){{};for(var m in k){if(k.hasOwnProperty(m)){delete k[m];};};},isEmpty:function(n){{};for(var o in n){return false;};return true;},getLength:qx.Bootstrap.objectGetLength,getValues:function(q){{};var r=[];var p=Object.keys(q);for(var i=0,l=p.length;i<l;i++ ){r.push(q[p[i]]);};return r;},mergeWith:qx.Bootstrap.objectMergeWith,clone:function(s,v){if(qx.lang.Type.isObject(s)){var t={};for(var u in s){if(v){t[u]=qx.lang.Object.clone(s[u],v);}else {t[u]=s[u];};};return t;}else if(qx.lang.Type.isArray(s)){var t=[];for(var i=0;i<s.length;i++ ){if(v){t[i]=qx.lang.Object.clone(s[i],v);}else {t[i]=s[i];};};return t;};return s;},equals:function(w,x){return qx.lang.Object.__dt(w,x,[],[]);},__dt:function(E,A,y,z){if(E===A){return E!==0||1/E==1/A;};if(E==null||A==null){return E===A;};var D=Object.prototype.toString.call(E);if(D!=Object.prototype.toString.call(A)){return false;};switch(D){case b:return E==String(A);case e:return E!=+E?A!=+A:(E==0?1/E==1/A:E==+A);case d:case a:return +E==+A;case h:return E.source==A.source&&E.global==A.global&&E.multiline==A.multiline&&E.ignoreCase==A.ignoreCase;};if(typeof E!=f||typeof A!=f){return false;};var length=y.length;while(length-- ){if(y[length]==E){return z[length]==A;};};var C=E.constructor,B=A.constructor;if(C!==B&&!(qx.Bootstrap.isFunction(C)&&(C instanceof C)&&qx.Bootstrap.isFunction(B)&&(B instanceof B))&&(c in E&&c in A)){return false;};y.push(E);z.push(A);var H=0,F=true;if(D==j){H=E.length;F=H==A.length;if(F){while(H-- ){if(!(F=qx.lang.Object.__dt(E[H],A[H],y,z))){break;};};};}else {for(var G in E){if(Object.prototype.hasOwnProperty.call(E,G)){H++ ;if(!(F=Object.prototype.hasOwnProperty.call(A,G)&&qx.lang.Object.__dt(E[G],A[G],y,z))){break;};};};if(F){for(G in A){if(Object.prototype.hasOwnProperty.call(A,G)&&!(H-- )){break;};};F=!H;};};y.pop();z.pop();return F;},invert:function(I){{};var J={};for(var K in I){J[I[K].toString()]=K;};return J;},getKeyFromValue:function(L,M){{};for(var N in L){if(L.hasOwnProperty(N)&&L[N]===M){return N;};};return null;},contains:function(O,P){{};return this.getKeyFromValue(O,P)!==null;},fromArray:function(Q){{};var R={};for(var i=0,l=Q.length;i<l;i++ ){{};R[Q[i].toString()]=true;};return R;}}});})();(function(){var a="qx.ui.decoration.MBackgroundColor",b="background-color",c='qx_lib.theme.Theme',d="Color",e="_applyBackgroundColor";qx.Mixin.define(a,{properties:{backgroundColor:{check:d,nullable:true,apply:e}},members:{_styleBackgroundColor:function(f){var g=this.getBackgroundColor();if(g&&c){g=qx.theme.manager.Color.getInstance().resolve(g);};if(g){f[b]=g;};},_applyBackgroundColor:function(){{};}}});})();(function(){var a="qx.ui.decoration.IDecorator";qx.Interface.define(a,{members:{getStyles:function(){},getPadding:function(){},getInsets:function(){}}});})();(function(){var a="abstract",b="Abstract method called.",c="qx.ui.decoration.Abstract";qx.Class.define(c,{extend:qx.core.Object,implement:[qx.ui.decoration.IDecorator],type:a,members:{__du:null,_getDefaultInsets:function(){throw new Error(b);},_isInitialized:function(){throw new Error(b);},_resetInsets:function(){this.__du=null;},getInsets:function(){if(this.__du){return this.__du;};return this._getDefaultInsets();}},destruct:function(){this.__du=null;}});})();(function(){var a="double",b="px ",c="widthTop",d="inset",e="solid",f="dotted",g="styleRight",h="styleBottom",i="_applyWidth",j="border-top",k="border-left",l="ridge",m="border-right",n="qx.ui.decoration.MSingleBorder",o="shorthand",p="Color",q="widthBottom",r="outset",s="widthLeft",t="",u="border-bottom",v="styleTop",w="colorBottom",x="groove",y="styleLeft",z="widthRight",A="dashed",B="Number",C="colorLeft",D="colorRight",E="colorTop",F="_applyStyle",G=" ",H="absolute";qx.Mixin.define(n,{properties:{widthTop:{check:B,init:0,apply:i},widthRight:{check:B,init:0,apply:i},widthBottom:{check:B,init:0,apply:i},widthLeft:{check:B,init:0,apply:i},styleTop:{nullable:true,check:[e,f,A,a,d,r,l,x],init:e,apply:F},styleRight:{nullable:true,check:[e,f,A,a,d,r,l,x],init:e,apply:F},styleBottom:{nullable:true,check:[e,f,A,a,d,r,l,x],init:e,apply:F},styleLeft:{nullable:true,check:[e,f,A,a,d,r,l,x],init:e,apply:F},colorTop:{nullable:true,check:p,apply:F},colorRight:{nullable:true,check:p,apply:F},colorBottom:{nullable:true,check:p,apply:F},colorLeft:{nullable:true,check:p,apply:F},left:{group:[s,y,C]},right:{group:[z,g,D]},top:{group:[c,v,E]},bottom:{group:[q,h,w]},width:{group:[c,z,q,s],mode:o},style:{group:[v,g,h,y],mode:o},color:{group:[E,D,w,C],mode:o}},members:{_styleBorder:function(I){{var K=qx.theme.manager.Color.getInstance();var O=K.resolve(this.getColorTop());var L=K.resolve(this.getColorRight());var J=K.resolve(this.getColorBottom());var N=K.resolve(this.getColorLeft());};var M=this.getWidthTop();if(M>0){I[j]=M+b+this.getStyleTop()+G+(O||t);};var M=this.getWidthRight();if(M>0){I[m]=M+b+this.getStyleRight()+G+(L||t);};var M=this.getWidthBottom();if(M>0){I[u]=M+b+this.getStyleBottom()+G+(J||t);};var M=this.getWidthLeft();if(M>0){I[k]=M+b+this.getStyleLeft()+G+(N||t);};{};I.position=H;},_getDefaultInsetsForBorder:function(){return {top:this.getWidthTop(),right:this.getWidthRight(),bottom:this.getWidthBottom(),left:this.getWidthLeft()};},_applyWidth:function(){this._applyStyle();this._resetInsets();},_applyStyle:function(){{};}}});})();(function(){var a=', url(',b="repeat",c="backgroundPositionX",d="backgroundPositionY",e="px",f="background-position",g=" ",h="background-repeat",i="no-repeat",j=')',k="scale",l="_applyBackgroundPosition",m='url(',n="repeat-x",o="background-image",p="100% 100%",q="repeat-y",r="qx.ui.decoration.MBackgroundImage",s="background-size",t="String",u="_applyBackgroundImage";qx.Mixin.define(r,{properties:{backgroundImage:{check:t,nullable:true,apply:u},backgroundRepeat:{check:[b,n,q,i,k],init:b,apply:u},backgroundPositionX:{nullable:true,apply:l},backgroundPositionY:{nullable:true,apply:l},backgroundPosition:{group:[d,c]}},members:{_styleBackgroundImage:function(v){var x=this.getBackgroundImage();if(!x){return;};var y=qx.util.AliasManager.getInstance().resolve(x);var z=qx.util.ResourceManager.getInstance().toUri(y);if(v[o]){v[o]+=a+z+j;}else {v[o]=m+z+j;};var w=this.getBackgroundRepeat();if(w===k){v[s]=p;}else {v[h]=w;};var top=this.getBackgroundPositionY()||0;var A=this.getBackgroundPositionX()||0;if(!isNaN(top)){top+=e;};if(!isNaN(A)){A+=e;};v[f]=A+g+top;{};},_applyBackgroundImage:function(){{};},_applyBackgroundPosition:function(){{};}}});})();(function(){var a="0",b="qx/static",c="http://",d="https://",e="file://",f="qx.util.AliasManager",g="singleton",h=".",i="/",j="static";qx.Class.define(f,{type:g,extend:qx.util.ValueManager,construct:function(){qx.util.ValueManager.call(this);this.__dv={};this.add(j,b);},members:{__dv:null,_preprocess:function(n){var m=this._getDynamic();if(m[n]===false){return n;}else if(m[n]===undefined){if(n.charAt(0)===i||n.charAt(0)===h||n.indexOf(c)===0||n.indexOf(d)===a||n.indexOf(e)===0){m[n]=false;return n;};if(this.__dv[n]){return this.__dv[n];};var l=n.substring(0,n.indexOf(i));var k=this.__dv[l];if(k!==undefined){m[n]=k+n.substring(l.length);};};return n;},add:function(o,q){this.__dv[o]=q;var p=this._getDynamic();for(var r in p){if(r.substring(0,r.indexOf(i))===o){p[r]=q+r.substring(o.length);};};},remove:function(s){delete this.__dv[s];},resolve:function(t){var u=this._getDynamic();if(t!=null){t=this._preprocess(t);};return u[t]||t;},getAliases:function(){var v={};for(var w in this.__dv){v[w]=this.__dv[w];};return v;}},destruct:function(){this.__dv=null;}});})();(function(){var a="Microsoft.XMLHTTP",b="xhr",c="io.ssl",d="io.xhr",e="",f="file:",g="https:",h="webkit",i="gecko",j="activex",k="opera",l=".",m="io.maxrequests",n="qx.bom.client.Transport";qx.Bootstrap.define(n,{statics:{getMaxConcurrentRequestCount:function(){var p;var r=qx.bom.client.Engine.getVersion().split(l);var o=0;var s=0;var q=0;if(r[0]){o=r[0];};if(r[1]){s=r[1];};if(r[2]){q=r[2];};if(window.maxConnectionsPerServer){p=window.maxConnectionsPerServer;}else if(qx.bom.client.Engine.getName()==k){p=8;}else if(qx.bom.client.Engine.getName()==h){p=4;}else if(qx.bom.client.Engine.getName()==i&&((o>1)||((o==1)&&(s>9))||((o==1)&&(s==9)&&(q>=1)))){p=6;}else {p=2;};return p;},getSsl:function(){return window.location.protocol===g;},getXmlHttpRequest:function(){var t=window.ActiveXObject?(function(){if(window.location.protocol!==f){try{new window.XMLHttpRequest();return b;}catch(u){};};try{new window.ActiveXObject(a);return j;}catch(v){};})():(function(){try{new window.XMLHttpRequest();return b;}catch(w){};})();return t||e;}},defer:function(x){qx.core.Environment.add(m,x.getMaxConcurrentRequestCount);qx.core.Environment.add(c,x.getSsl);qx.core.Environment.add(d,x.getXmlHttpRequest);}});})();(function(){var a="singleton",b="qx.util.LibraryManager";qx.Class.define(b,{extend:qx.core.Object,type:a,statics:{__dw:qx.$$libraries||{}},members:{has:function(c){return !!this.self(arguments).__dw[c];},get:function(d,e){return this.self(arguments).__dw[d][e]?this.self(arguments).__dw[d][e]:null;},set:function(f,g,h){this.self(arguments).__dw[f][g]=h;}}});})();(function(){var a="mshtml",b="engine.name",c="//",d="io.ssl",e="",f="encoding",g="?",h="data",i="string",j="type",k="data:image/",l=";",m="/",n="resourceUri",o="qx.util.ResourceManager",p="singleton",q=",";qx.Class.define(o,{extend:qx.core.Object,type:p,construct:function(){qx.core.Object.call(this);},statics:{__j:qx.$$resources||{},__dx:{}},members:{has:function(r){return !!this.self(arguments).__j[r];},getData:function(s){return this.self(arguments).__j[s]||null;},getImageWidth:function(u){var t=this.self(arguments).__j[u];return t?t[0]:null;},getImageHeight:function(w){var v=this.self(arguments).__j[w];return v?v[1]:null;},getImageFormat:function(y){var x=this.self(arguments).__j[y];return x?x[2]:null;},getCombinedFormat:function(D){var A=e;var C=this.self(arguments).__j[D];var z=C&&C.length>4&&typeof (C[4])==i&&this.constructor.__j[C[4]];if(z){var E=C[4];var B=this.constructor.__j[E];A=B[2];};return A;},toUri:function(I){if(I==null){return I;};var F=this.self(arguments).__j[I];if(!F){return I;};if(typeof F===i){var H=F;}else {var H=F[3];if(!H){return I;};};var G=e;if((qx.core.Environment.get(b)==a)&&qx.core.Environment.get(d)){G=this.self(arguments).__dx[H];};return G+qx.util.LibraryManager.getInstance().get(H,n)+m+I;},toDataUri:function(L){var K=this.constructor.__j[L];var N=this.constructor.__j[K[4]];var M;if(N){var J=N[4][L];M=k+J[j]+l+J[f]+q+J[h];}else {M=this.toUri(L);};return M;}},defer:function(P){if((qx.core.Environment.get(b)==a)){if(qx.core.Environment.get(d)){for(var Q in qx.$$libraries){var O;if(qx.util.LibraryManager.getInstance().get(Q,n)){O=qx.util.LibraryManager.getInstance().get(Q,n);}else {P.__dx[Q]=e;continue;};if(O.match(/^\/\//)!=null){P.__dx[Q]=window.location.protocol;}else if(O.match(/^\//)!=null){P.__dx[Q]=window.location.protocol+c+window.location.host;}else if(O.match(/^\.\//)!=null){var S=document.URL;P.__dx[Q]=S.substring(0,S.lastIndexOf(m)+1);}else if(O.match(/^http/)!=null){P.__dx[Q]=e;}else {var R=window.location.href.indexOf(g);var T;if(R==-1){T=window.location.href;}else {T=window.location.href.substring(0,R);};P.__dx[Q]=T.substring(0,T.lastIndexOf(m)+1);};};};};}});})();(function(){var a="innerWidthRight",b="innerColorBottom",c="css.borderradius",d="px ",e='""',f="_applyDoubleBorder",g="border-top",h="inset 0 -",i="css.boxsizing",j="innerWidthTop",k="100%",l="border-left",m="innerColorRight",n="css.boxshadow",o="innerColorTop",p="innerColorLeft",q="inset ",r="shorthand",s="inset -",t="Color",u="border-box",v="qx.ui.decoration.MDoubleBorder",w="border-bottom",x=":before",y="inset 0 ",z="px solid ",A="innerWidthBottom",B="css.rgba",C="inherit",D="Number",E="innerWidthLeft",F="px 0 ",G="inset 0 0 0 ",H="border-right",I=" ",J=",",K="absolute";qx.Mixin.define(v,{include:[qx.ui.decoration.MSingleBorder,qx.ui.decoration.MBackgroundImage],construct:function(){this._getDefaultInsetsForBorder=this.__dA;this._styleBorder=this.__dy;},properties:{innerWidthTop:{check:D,init:0,apply:f},innerWidthRight:{check:D,init:0,apply:f},innerWidthBottom:{check:D,init:0,apply:f},innerWidthLeft:{check:D,init:0,apply:f},innerWidth:{group:[j,a,A,E],mode:r},innerColorTop:{nullable:true,check:t,apply:f},innerColorRight:{nullable:true,check:t,apply:f},innerColorBottom:{nullable:true,check:t,apply:f},innerColorLeft:{nullable:true,check:t,apply:f},innerColor:{group:[o,m,b,p],mode:r},innerOpacity:{check:D,init:1,apply:f}},members:{__dy:function(L){var U=qx.core.Environment.get(n);var O,Y,innerWidth;{var T=qx.theme.manager.Color.getInstance();O={top:T.resolve(this.getColorTop()),right:T.resolve(this.getColorRight()),bottom:T.resolve(this.getColorBottom()),left:T.resolve(this.getColorLeft())};Y={top:T.resolve(this.getInnerColorTop()),right:T.resolve(this.getInnerColorRight()),bottom:T.resolve(this.getInnerColorBottom()),left:T.resolve(this.getInnerColorLeft())};};innerWidth={top:this.getInnerWidthTop(),right:this.getInnerWidthRight(),bottom:this.getInnerWidthBottom(),left:this.getInnerWidthLeft()};var R=this.getWidthTop();if(R>0){L[g]=R+d+this.getStyleTop()+I+O.top;};R=this.getWidthRight();if(R>0){L[H]=R+d+this.getStyleRight()+I+O.right;};R=this.getWidthBottom();if(R>0){L[w]=R+d+this.getStyleBottom()+I+O.bottom;};R=this.getWidthLeft();if(R>0){L[l]=R+d+this.getStyleLeft()+I+O.left;};var X=this.getInnerOpacity();if(X<1){this.__dz(Y,X);};if(innerWidth.top>0||innerWidth.right>0||innerWidth.bottom>0||innerWidth.left>0){var W=(innerWidth.top||0)+z+Y.top;var V=(innerWidth.right||0)+z+Y.right;var Q=(innerWidth.bottom||0)+z+Y.bottom;var S=(innerWidth.left||0)+z+Y.left;L[x]={"width":k,"height":k,"position":K,"content":e,"border-top":W,"border-right":V,"border-bottom":Q,"border-left":S,"left":0,"top":0};var M=qx.bom.Style.getCssName(qx.core.Environment.get(i));L[x][M]=u;var N=qx.core.Environment.get(c);if(N){N=qx.bom.Style.getCssName(N);L[x][N]=C;};var P=[];if(Y.top&&innerWidth.top&&Y.top==Y.bottom&&Y.top==Y.right&&Y.top==Y.left&&innerWidth.top==innerWidth.bottom&&innerWidth.top==innerWidth.right&&innerWidth.top==innerWidth.left){P.push(G+innerWidth.top+d+Y.top);}else {if(Y.top){P.push(y+(innerWidth.top||0)+d+Y.top);};if(Y.right){P.push(s+(innerWidth.right||0)+F+Y.right);};if(Y.bottom){P.push(h+(innerWidth.bottom||0)+d+Y.bottom);};if(Y.left){P.push(q+(innerWidth.left||0)+F+Y.left);};};if(P.length>0&&U){U=qx.bom.Style.getCssName(U);if(!L[U]){L[U]=P.join(J);}else {L[U]+=J+P.join(J);};};}else {L[x]={border:0};};},__dz:function(bd,ba){if(!qx.core.Environment.get(B)){{};return;};for(var be in bd){var bb=qx.util.ColorUtil.stringToRgb(bd[be]);bb.push(ba);var bc=qx.util.ColorUtil.rgbToRgbString(bb);bd[be]=bc;};},_applyDoubleBorder:function(){{};},__dA:function(){return {top:this.getWidthTop()+this.getInnerWidthTop(),right:this.getWidthRight()+this.getInnerWidthRight(),bottom:this.getWidthBottom()+this.getInnerWidthBottom(),left:this.getWidthLeft()+this.getInnerWidthLeft()};}}});})();(function(){var a="css.float",b="foo",c="css.borderimage.standardsyntax",d="detect",e="borderRadius",f="boxSizing",g="stretch",h="css.borderradius",j="content",k="css.inlineblock",l="css.gradient.filter",m="css.appearance",n="css.opacity",o="div",p="pointerEvents",q="css.gradient.radial",r="css.pointerevents",s="input",t="color",u="string",v="borderImage",w="userSelect",x="styleFloat",y="css.textShadow.filter",z="css.usermodify",A="flexbox",B='url("foo.png") 4 4 4 4 fill stretch',C="css.boxmodel",D="qx.bom.client.Css",E="css.boxshadow",F="appearance",G="-ms-flexbox",H="placeholder",I="-moz-none",J="backgroundImage",K="css.textShadow",L="DXImageTransform.Microsoft.Shadow",M="flex",N="css.alphaimageloaderneeded",O="css.gradient.legacywebkit",P="css.flexboxSyntax",Q="linear-gradient(0deg, #fff, #000)",R="textShadow",S="auto",T="css.borderimage",U="foo.png",V="rgba(1, 2, 3, 0.5)",W="color=#666666,direction=45",X="radial-gradient(0px 0px, cover, red 50%, blue 100%)",Y="rgba",bG="(",bH="-webkit-flex",bI='url("foo.png") 4 4 4 4 stretch',bC="css.gradient.linear",bD="DXImageTransform.Microsoft.Gradient",bE="css.userselect",bF="span",bM="css.boxsizing",bN="-webkit-gradient(linear,0% 0%,100% 100%,from(white), to(red))",bO="mshtml",ca="css.rgba",bJ=");",bK="4 fill",bL="none",bA="startColorStr=#550000FF, endColorStr=#55FFFF00",bR="progid:",bB="css.placeholder",bS="css.userselect.none",bT="css.textoverflow",bX="inline-block",bP="-moz-inline-box",bY="textOverflow",bQ="userModify",bU="boxShadow",bV="cssFloat",bW="border";qx.Bootstrap.define(D,{statics:{__dB:null,getBoxModel:function(){var content=qx.bom.client.Engine.getName()!==bO||!qx.bom.client.Browser.getQuirksMode();return content?j:bW;},getTextOverflow:function(){return qx.bom.Style.getPropertyName(bY);},getPlaceholder:function(){var i=document.createElement(s);return H in i;},getAppearance:function(){return qx.bom.Style.getPropertyName(F);},getBorderRadius:function(){return qx.bom.Style.getPropertyName(e);},getBoxShadow:function(){return qx.bom.Style.getPropertyName(bU);},getBorderImage:function(){return qx.bom.Style.getPropertyName(v);},getBorderImageSyntax:function(){var cc=qx.bom.client.Css.getBorderImage();if(!cc){return null;};var cb=document.createElement(o);if(cc===v){cb.style[cc]=B;if(cb.style.borderImageSource.indexOf(U)>=0&&cb.style.borderImageSlice.indexOf(bK)>=0&&cb.style.borderImageRepeat.indexOf(g)>=0){return true;};}else {cb.style[cc]=bI;if(cb.style[cc].indexOf(U)>=0){return false;};};return null;},getUserSelect:function(){return qx.bom.Style.getPropertyName(w);},getUserSelectNone:function(){var ce=qx.bom.client.Css.getUserSelect();if(ce){var cd=document.createElement(bF);cd.style[ce]=I;return cd.style[ce]===I?I:bL;};return null;},getUserModify:function(){return qx.bom.Style.getPropertyName(bQ);},getFloat:function(){var cf=document.documentElement.style;return cf.cssFloat!==undefined?bV:cf.styleFloat!==undefined?x:null;},getLinearGradient:function(){qx.bom.client.Css.__dB=false;var cj=Q;var cg=document.createElement(o);var ch=qx.bom.Style.getAppliedStyle(cg,J,cj);if(!ch){cj=bN;var ch=qx.bom.Style.getAppliedStyle(cg,J,cj,false);if(ch){qx.bom.client.Css.__dB=true;};};if(!ch){return null;};var ci=/(.*?)\(/.exec(ch);return ci?ci[1]:null;},getFilterGradient:function(){return qx.bom.client.Css.__dC(bD,bA);},getRadialGradient:function(){var cn=X;var ck=document.createElement(o);var cl=qx.bom.Style.getAppliedStyle(ck,J,cn);if(!cl){return null;};var cm=/(.*?)\(/.exec(cl);return cm?cm[1]:null;},getLegacyWebkitGradient:function(){if(qx.bom.client.Css.__dB===null){qx.bom.client.Css.getLinearGradient();};return qx.bom.client.Css.__dB;},getRgba:function(){var co;try{co=document.createElement(o);}catch(cp){co=document.createElement();};try{co.style[t]=V;if(co.style[t].indexOf(Y)!=-1){return true;};}catch(cq){};return false;},getBoxSizing:function(){return qx.bom.Style.getPropertyName(f);},getInlineBlock:function(){var cr=document.createElement(bF);cr.style.display=bX;if(cr.style.display==bX){return bX;};cr.style.display=bP;if(cr.style.display!==bP){return bP;};return null;},getOpacity:function(){return (typeof document.documentElement.style.opacity==u);},getTextShadow:function(){return !!qx.bom.Style.getPropertyName(R);},getFilterTextShadow:function(){return qx.bom.client.Css.__dC(L,W);},__dC:function(cv,ct){var cu=false;var cw=bR+cv+bG+ct+bJ;var cs=document.createElement(o);document.body.appendChild(cs);cs.style.filter=cw;if(cs.filters&&cs.filters.length>0&&cs.filters.item(cv).enabled==true){cu=true;};document.body.removeChild(cs);return cu;},getAlphaImageLoaderNeeded:function(){return qx.bom.client.Engine.getName()==bO&&qx.bom.client.Browser.getDocumentMode()<9;},getPointerEvents:function(){var cx=document.documentElement;if(p in cx.style){var cz=cx.style.pointerEvents;cx.style.pointerEvents=S;cx.style.pointerEvents=b;var cy=cx.style.pointerEvents==S;cx.style.pointerEvents=cz;return cy;};return false;},getFlexboxSyntax:function(){var cB=null;var cA=document.createElement(d);var cC=[{value:M,syntax:M},{value:G,syntax:A},{value:bH,syntax:M}];for(var i=0;i<cC.length;i++ ){try{cA.style.display=cC[i].value;}catch(cD){return null;};if(cA.style.display===cC[i].value){cB=cC[i].syntax;break;};};cA=null;return cB;}},defer:function(cE){qx.core.Environment.add(bT,cE.getTextOverflow);qx.core.Environment.add(bB,cE.getPlaceholder);qx.core.Environment.add(h,cE.getBorderRadius);qx.core.Environment.add(E,cE.getBoxShadow);qx.core.Environment.add(bC,cE.getLinearGradient);qx.core.Environment.add(l,cE.getFilterGradient);qx.core.Environment.add(q,cE.getRadialGradient);qx.core.Environment.add(O,cE.getLegacyWebkitGradient);qx.core.Environment.add(C,cE.getBoxModel);qx.core.Environment.add(ca,cE.getRgba);qx.core.Environment.add(T,cE.getBorderImage);qx.core.Environment.add(c,cE.getBorderImageSyntax);qx.core.Environment.add(z,cE.getUserModify);qx.core.Environment.add(bE,cE.getUserSelect);qx.core.Environment.add(bS,cE.getUserSelectNone);qx.core.Environment.add(m,cE.getAppearance);qx.core.Environment.add(a,cE.getFloat);qx.core.Environment.add(bM,cE.getBoxSizing);qx.core.Environment.add(k,cE.getInlineBlock);qx.core.Environment.add(n,cE.getOpacity);qx.core.Environment.add(K,cE.getTextShadow);qx.core.Environment.add(y,cE.getFilterTextShadow);qx.core.Environment.add(N,cE.getAlphaImageLoaderNeeded);qx.core.Environment.add(r,cE.getPointerEvents);qx.core.Environment.add(P,cE.getFlexboxSyntax);}});})();(function(){var a="radiusTopRight",b="radiusTopLeft",c="px",d="-webkit-border-bottom-left-radius",e="-webkit-background-clip",f="radiusBottomRight",g="Integer",h="-webkit-border-bottom-right-radius",i="background-clip",j="border-top-left-radius",k="border-top-right-radius",l="border-bottom-left-radius",m="radiusBottomLeft",n="-webkit-border-top-left-radius",o="shorthand",p="-moz-border-radius-bottomright",q="padding-box",r="border-bottom-right-radius",s="qx.ui.decoration.MBorderRadius",t="-moz-border-radius-topright",u="engine.name",v="_applyBorderRadius",w="-webkit-border-top-right-radius",x="webkit",y="-moz-border-radius-topleft",z="-moz-border-radius-bottomleft";qx.Mixin.define(s,{properties:{radiusTopLeft:{nullable:true,check:g,apply:v},radiusTopRight:{nullable:true,check:g,apply:v},radiusBottomLeft:{nullable:true,check:g,apply:v},radiusBottomRight:{nullable:true,check:g,apply:v},radius:{group:[b,a,f,m],mode:o}},members:{_styleBorderRadius:function(A){A[e]=q;A[i]=q;var B=false;var C=this.getRadiusTopLeft();if(C>0){B=true;A[y]=C+c;A[n]=C+c;A[j]=C+c;};C=this.getRadiusTopRight();if(C>0){B=true;A[t]=C+c;A[w]=C+c;A[k]=C+c;};C=this.getRadiusBottomLeft();if(C>0){B=true;A[z]=C+c;A[d]=C+c;A[l]=C+c;};C=this.getRadiusBottomRight();if(C>0){B=true;A[p]=C+c;A[h]=C+c;A[r]=C+c;};if(B&&qx.core.Environment.get(u)==x){A[e]=q;}else {A[i]=q;};},_applyBorderRadius:function(){{};}}});})();(function(){var a="border-width",b="css.borderimage.standardsyntax",c="repeat",d="Boolean",e="-l",f="stretch",g="px ",h="sliceBottom",i="-t",j="Integer",k="solid",l="borderImage",m="-r",n="border-style",o="sliceLeft",p="-b",q="sliceRight",r="px",s="repeatX",t=" fill",u="String",v="vertical",w="",x="transparent",y="round",z='") ',A="shorthand",B="qx.ui.decoration.MBorderImage",C="sliceTop",D="horizontal",E="_applyBorderImage",F="border-color",G='url("',H=" ",I="grid",J="repeatY";qx.Mixin.define(B,{properties:{borderImage:{check:u,nullable:true,apply:E},sliceTop:{check:j,nullable:true,init:null,apply:E},sliceRight:{check:j,nullable:true,init:null,apply:E},sliceBottom:{check:j,nullable:true,init:null,apply:E},sliceLeft:{check:j,nullable:true,init:null,apply:E},slice:{group:[C,q,h,o],mode:A},repeatX:{check:[f,c,y],init:f,apply:E},repeatY:{check:[f,c,y],init:f,apply:E},repeat:{group:[s,J],mode:A},fill:{check:d,init:true,apply:E},borderImageMode:{check:[D,v,I],init:I}},members:{_styleBorderImage:function(K){if(!this.getBorderImage()){return;};var M=qx.util.AliasManager.getInstance().resolve(this.getBorderImage());var O=qx.util.ResourceManager.getInstance().toUri(M);var R=this._getDefaultInsetsForBorderImage();var L=[R.top,R.right,R.bottom,R.left];var P=[this.getRepeatX(),this.getRepeatY()].join(H);var S=this.getFill()&&qx.core.Environment.get(b)?t:w;var N=qx.bom.Style.getPropertyName(l);if(N){var Q=qx.bom.Style.getCssName(N);K[Q]=G+O+z+L.join(H)+S+H+P;};K[n]=k;K[F]=x;K[a]=L.join(g)+r;},_getDefaultInsetsForBorderImage:function(){if(!this.getBorderImage()){return {top:0,right:0,bottom:0,left:0};};var T=qx.util.AliasManager.getInstance().resolve(this.getBorderImage());var U=this.__dD(T);return {top:this.getSliceTop()||U[0],right:this.getSliceRight()||U[1],bottom:this.getSliceBottom()||U[2],left:this.getSliceLeft()||U[3]};},_applyBorderImage:function(){{};},__dD:function(bc){var bb=this.getBorderImageMode();var bd=0;var Y=0;var ba=0;var be=0;var bf=/(.*)(\.[a-z]+)$/.exec(bc);var V=bf[1];var X=bf[2];var W=qx.util.ResourceManager.getInstance();if(bb==I||bb==v){bd=W.getImageHeight(V+i+X);ba=W.getImageHeight(V+p+X);};if(bb==I||bb==D){Y=W.getImageWidth(V+m+X);be=W.getImageWidth(V+e+X);};return [bd,Y,ba,be];}}});})();(function(){var a=" 0",b="),to(",c="px",d="css.borderradius",e="from(",f=")",g="background-image",h="background",i="filter",j="background-size",k="', ",l="0",m="_applyLinearBackgroundGradient",n="-webkit-gradient(linear,",o="startColorPosition",p="background-color",q="deg, ",r="url(",s="css.gradient.legacywebkit",t="EndColorStr='#FF",u="startColor",v="shorthand",w="Color",x="px 100%",y="StartColorStr='#FF",z="vertical",A="",B="transparent",C="qx.ui.decoration.MLinearBackgroundGradient",D="% 100%",E="endColorPosition",F="canvas",G="(",H="css.gradient.linear",I="';)",J="endColor",K=", ",L="css.gradient.filter",M="horizontal",N="Number",O="100% ",P='2d',Q="%",R=" ",S="white",T="linear-gradient",U='progid:DXImageTransform.Microsoft.Gradient(GradientType=',V=",";qx.Mixin.define(C,{properties:{startColor:{check:w,nullable:true,apply:m},endColor:{check:w,nullable:true,apply:m},orientation:{check:[M,z],init:z,apply:m},startColorPosition:{check:N,init:0,apply:m},endColorPosition:{check:N,init:100,apply:m},colorPositionUnit:{check:[c,Q],init:Q,apply:m},gradientStart:{group:[u,o],mode:v},gradientEnd:{group:[J,E],mode:v}},members:{__dE:null,_styleLinearBackgroundGradient:function(W){var bj=this.__dF();var bn=bj.start;var bh=bj.end;var bf;if(!bn||!bh){return;};var bq=this.getColorPositionUnit();if(qx.core.Environment.get(s)){bq=bq===c?A:bq;if(this.getOrientation()==M){var bm=this.getStartColorPosition()+bq+a+bq;var bk=this.getEndColorPosition()+bq+a+bq;}else {var bm=l+bq+R+this.getStartColorPosition()+bq;var bk=l+bq+R+this.getEndColorPosition()+bq;};var bb=e+bn+b+bh+f;bf=n+bm+V+bk+V+bb+f;W[h]=bf;}else if(qx.core.Environment.get(L)&&!qx.core.Environment.get(H)&&qx.core.Environment.get(d)){if(!this.__dE){this.__dE=document.createElement(F);};var X=this.getOrientation()==z;var bj=this.__dF();var bg=X?200:1;var ba=X?1:200;var be=Math.max(100,this.getEndColorPosition()-this.getStartColorPosition());if(bq===c){if(X){bg=Math.max(bg,this.getEndColorPosition()-this.getStartColorPosition());}else {ba=Math.max(ba,this.getEndColorPosition()-this.getStartColorPosition());};}else {if(X){bg=Math.max(bg,(this.getEndColorPosition()-this.getStartColorPosition())*2);}else {ba=Math.max(ba,(this.getEndColorPosition()-this.getStartColorPosition())*2);};};this.__dE.width=ba;this.__dE.height=bg;var bd=this.__dE.getContext(P);if(X){var bp=bd.createLinearGradient(0,0,0,bg);}else {var bp=bd.createLinearGradient(0,0,ba,0);};if(bq===Q){bp.addColorStop(Math.max(0,this.getStartColorPosition())/be,bj.start);bp.addColorStop(this.getEndColorPosition()/be,bj.end);}else {var bc=X?bg:ba;bp.addColorStop(Math.max(0,this.getStartColorPosition())/bc,bj.start);bp.addColorStop(this.getEndColorPosition()/bc,bj.end);};bd.fillStyle=bp;bd.fillRect(0,0,ba,bg);var bf=r+this.__dE.toDataURL()+f;W[g]=bf;if(bq===Q){if(X){W[j]=O+be+Q;}else {W[j]=be+D;};}else {W[j]=X?bg+x:O+ba+c;};}else if(qx.core.Environment.get(L)&&!qx.core.Environment.get(H)){var bj=this.__dF();var bo=this.getOrientation()==M?1:0;var bn=bj.start;var bh=bj.end;if(!qx.util.ColorUtil.isHex6String(bn)){bn=qx.util.ColorUtil.stringToRgb(bn);bn=qx.util.ColorUtil.rgbToHexString(bn);};if(!qx.util.ColorUtil.isHex6String(bh)){bh=qx.util.ColorUtil.stringToRgb(bh);bh=qx.util.ColorUtil.rgbToHexString(bh);};bn=bn.substring(1,bn.length);bh=bh.substring(1,bh.length);bf=U+bo+K+y+bn+k+t+bh+I;if(W[i]){W[i]+=K+bf;}else {W[i]=bf;};if(!W[p]||W[p]==B){W[p]=S;};}else {var br=this.getOrientation()==M?0:270;var bi=bn+R+this.getStartColorPosition()+bq;var Y=bh+R+this.getEndColorPosition()+bq;var bl=qx.core.Environment.get(H);if(bl===T){br=this.getOrientation()==M?br+90:br-90;};bf=bl+G+br+q+bi+V+Y+f;if(W[g]){W[g]+=K+bf;}else {W[g]=bf;};};},__dF:function(){{var bs=qx.theme.manager.Color.getInstance();var bu=bs.resolve(this.getStartColor());var bt=bs.resolve(this.getEndColor());};return {start:bu,end:bt};},_applyLinearBackgroundGradient:function(){{};}}});})();(function(){var a="_applyBoxShadow",b="shadowHorizontalLength",c="Boolean",d="",e="px ",f="css.boxshadow",g="shadowVerticalLength",h="inset ",i="shorthand",j="qx.ui.decoration.MBoxShadow",k="Integer",l="Color",m=",";qx.Mixin.define(j,{properties:{shadowHorizontalLength:{nullable:true,check:k,apply:a},shadowVerticalLength:{nullable:true,check:k,apply:a},shadowBlurRadius:{nullable:true,check:k,apply:a},shadowSpreadRadius:{nullable:true,check:k,apply:a},shadowColor:{nullable:true,check:l,apply:a},inset:{init:false,check:c,apply:a},shadowLength:{group:[b,g],mode:i}},members:{_styleBoxShadow:function(n){var v=qx.core.Environment.get(f);if(!v||this.getShadowVerticalLength()==null&&this.getShadowHorizontalLength()==null){return;};{var r=qx.theme.manager.Color.getInstance();var o=r.resolve(this.getShadowColor());};if(o!=null){var u=this.getShadowVerticalLength()||0;var p=this.getShadowHorizontalLength()||0;var blur=this.getShadowBlurRadius()||0;var t=this.getShadowSpreadRadius()||0;var s=this.getInset()?h:d;var q=s+p+e+u+e+blur+e+t+e+o;v=qx.bom.Style.getCssName(v);if(!n[v]){n[v]=q;}else {n[v]+=m+q;};};},_applyBoxShadow:function(){{};}}});})();(function(){var a="qx.ui.decoration.Decorator",b="_style",c="_getDefaultInsetsFor",d="bottom",e="top",f="left",g="right";qx.Class.define(a,{extend:qx.ui.decoration.Abstract,implement:[qx.ui.decoration.IDecorator],include:[qx.ui.decoration.MBackgroundColor,qx.ui.decoration.MBorderRadius,qx.ui.decoration.MBoxShadow,qx.ui.decoration.MDoubleBorder,qx.ui.decoration.MLinearBackgroundGradient,qx.ui.decoration.MBorderImage],members:{__dG:false,getPadding:function(){var k=this.getInset();var h=this._getDefaultInsetsForBorderImage();var n=k.top-(h.top?h.top:this.getWidthTop());var m=k.right-(h.right?h.right:this.getWidthRight());var j=k.bottom-(h.bottom?h.bottom:this.getWidthBottom());var l=k.left-(h.left?h.left:this.getWidthLeft());return {top:k.top?n:this.getInnerWidthTop(),right:k.right?m:this.getInnerWidthRight(),bottom:k.bottom?j:this.getInnerWidthBottom(),left:k.left?l:this.getInnerWidthLeft()};},getStyles:function(r){if(r){return this._getStyles();};var q={};var p=this._getStyles();for(var o in p){q[qx.lang.String.camelCase(o)]=p[o];};return q;},_getStyles:function(){var s={};for(var name in this){if(name.indexOf(b)==0&&this[name] instanceof Function){this[name](s);};};this.__dG=true;return s;},_getDefaultInsets:function(){var w=[e,g,d,f];var u={};for(var name in this){if(name.indexOf(c)==0&&this[name] instanceof Function){var v=this[name]();for(var i=0;i<w.length;i++ ){var t=w[i];if(u[t]==undefined){u[t]=v[t];};if(v[t]>u[t]){u[t]=v[t];};};};};if(u[e]!=undefined){return u;};return {top:0,right:0,bottom:0,left:0};},_isInitialized:function(){return this.__dG;}}});})();(function(){var a="_applyTheme",b="qx.theme.manager.Font",c="_dynamic",d="Theme",e="changeTheme",f="singleton";qx.Class.define(b,{type:f,extend:qx.util.ValueManager,properties:{theme:{check:d,nullable:true,apply:a,event:e}},members:{resolveDynamic:function(h){var g=this._dynamic;return h instanceof qx.bom.Font?h:g[h];},resolve:function(l){var k=this._dynamic;var i=k[l];if(i){return i;};var j=this.getTheme();if(j!==null&&j.fonts[l]){var m=this.__dI(j.fonts[l]);return k[l]=(new m).set(j.fonts[l]);};return l;},isDynamic:function(q){var p=this._dynamic;if(q&&(q instanceof qx.bom.Font||p[q]!==undefined)){return true;};var o=this.getTheme();if(o!==null&&q&&o.fonts[q]){var n=this.__dI(o.fonts[q]);p[q]=(new n).set(o.fonts[q]);return true;};return false;},__dH:function(s,r){if(s[r].include){var t=s[s[r].include];s[r].include=null;delete s[r].include;s[r]=qx.lang.Object.mergeWith(s[r],t,false);this.__dH(s,r);};},_applyTheme:function(y){var u=this._dynamic;for(var x in u){if(u[x].themed){u[x].dispose();delete u[x];};};if(y){var v=y.fonts;for(var x in v){if(v[x].include&&v[v[x].include]){this.__dH(v,x);};var w=this.__dI(v[x]);u[x]=(new w).set(v[x]);u[x].themed=true;};};this._setDynamic(u);},__dI:function(z){if(z.sources){return qx.bom.webfonts.WebFont;};return qx.bom.Font;}},destruct:function(){this._disposeMap(c);}});})();(function(){var a="Boolean",b="px",c="_applyItalic",d="_applyBold",e="underline",f="_applyTextShadow",g="Integer",h="_applyFamily",j="_applyLineHeight",k='"',m="Array",n="line-through",o="overline",p="Color",q="String",r="",s="italic",t="normal",u="qx.bom.Font",v="bold",w="Number",x="_applyDecoration",y=" ",z="_applySize",A=",",B="_applyColor";qx.Class.define(u,{extend:qx.core.Object,construct:function(D,C){qx.core.Object.call(this);this.__dJ={fontFamily:r,fontSize:null,fontWeight:null,fontStyle:null,textDecoration:null,lineHeight:null,color:null,textShadow:null};if(D!==undefined){this.setSize(D);};if(C!==undefined){this.setFamily(C);};},statics:{fromString:function(H){var I=new qx.bom.Font();var F=H.split(/\s+/);var name=[];var G;for(var i=0;i<F.length;i++ ){switch(G=F[i]){case v:I.setBold(true);break;case s:I.setItalic(true);break;case e:I.setDecoration(e);break;default:var E=parseInt(G,10);if(E==G||qx.lang.String.contains(G,b)){I.setSize(E);}else {name.push(G);};break;};};if(name.length>0){I.setFamily(name);};return I;},fromConfig:function(K){var J=new qx.bom.Font;J.set(K);return J;},__dK:{fontFamily:r,fontSize:r,fontWeight:r,fontStyle:r,textDecoration:r,lineHeight:1.2,color:r,textShadow:r},getDefaultStyles:function(){return this.__dK;}},properties:{size:{check:g,nullable:true,apply:z},lineHeight:{check:w,nullable:true,apply:j},family:{check:m,nullable:true,apply:h},bold:{check:a,nullable:true,apply:d},italic:{check:a,nullable:true,apply:c},decoration:{check:[e,n,o],nullable:true,apply:x},color:{check:p,nullable:true,apply:B},textShadow:{nullable:true,check:q,apply:f}},members:{__dJ:null,_applySize:function(M,L){this.__dJ.fontSize=M===null?null:M+b;},_applyLineHeight:function(O,N){this.__dJ.lineHeight=O===null?null:O;},_applyFamily:function(P,Q){var R=r;for(var i=0,l=P.length;i<l;i++ ){if(P[i].indexOf(y)>0){R+=k+P[i]+k;}else {R+=P[i];};if(i!==l-1){R+=A;};};this.__dJ.fontFamily=R;},_applyBold:function(T,S){this.__dJ.fontWeight=T==null?null:T?v:t;},_applyItalic:function(V,U){this.__dJ.fontStyle=V==null?null:V?s:t;},_applyDecoration:function(X,W){this.__dJ.textDecoration=X==null?null:X;},_applyColor:function(ba,Y){this.__dJ.color=null;if(ba){this.__dJ.color=qx.theme.manager.Color.getInstance().resolve(ba);};},_applyTextShadow:function(bc,bb){this.__dJ.textShadow=bc==null?null:bc;},getStyles:function(){return this.__dJ;}}});})();(function(){var a="changeStatus",b="qx.bom.webfonts.WebFont",c="_applySources",d="",e="qx.event.type.Data";qx.Class.define(b,{extend:qx.bom.Font,events:{"changeStatus":e},properties:{sources:{nullable:true,apply:c}},members:{__dL:null,_applySources:function(h,k){var f=[];for(var i=0,l=h.length;i<l;i++ ){var g=this._quoteFontFamily(h[i].family);f.push(g);var j=h[i].source;qx.bom.webfonts.Manager.getInstance().require(g,j,this._onWebFontChangeStatus,this);};this.setFamily(f.concat(this.getFamily()));},_onWebFontChangeStatus:function(m){var n=m.getData();this.fireDataEvent(a,n);{};},_quoteFontFamily:function(o){return o.replace(/["']/g,d);}}});})();(function(){var a="m",b="os.name",c=")",d="os.version",e="qx.bom.webfonts.Manager",f="svg",g="chrome",h="browser.name",k="singleton",n=",\n",o="src: ",p="mobileSafari",q="'eot)",r="');",s="changeStatus",t="interval",u="#",v="firefox",w="!",y="eot",z="ios",A="'eot')",B="\.(",C="}\n",D="font-family: ",E="browser.documentmode",F="mobile safari",G="safari",H="@font-face.*?",I="",J="ttf",K=";\n",L="') format('svg')",M="') format('woff')",N="('embedded-opentype')",O="browser.version",P="opera",Q="engine.version",R="Couldn't create @font-face rule for WebFont ",S="mshtml",T="engine.name",U="url('",V="src: url('",W="('embedded-opentype)",X="\nfont-style: normal;\nfont-weight: normal;",Y="?#iefix') format('embedded-opentype')",bh="woff",bi="ie",bj=";",bf="@font-face {",bg="') format('truetype')";qx.Class.define(e,{extend:qx.core.Object,type:k,construct:function(){qx.core.Object.call(this);this.__dM=[];this.__dN={};this.__dO=[];this.__dP=this.getPreferredFormats();},statics:{FONT_FORMATS:[y,bh,J,f],VALIDATION_TIMEOUT:5000},members:{__dM:null,__dQ:null,__dN:null,__dP:null,__dO:null,__dR:null,require:function(bm,bn,bo,bq){var bl=[];for(var i=0,l=bn.length;i<l;i++ ){var bp=bn[i].split(u);var bk=qx.util.ResourceManager.getInstance().toUri(bp[0]);if(bp.length>1){bk=bk+u+bp[1];};bl.push(bk);};if(qx.core.Environment.get(T)==S&&(parseInt(qx.core.Environment.get(Q))<9||qx.core.Environment.get(E)<9)){if(!this.__dR){this.__dR=new qx.event.Timer(100);this.__dR.addListener(t,this.__dT,this);};if(!this.__dR.isEnabled()){this.__dR.start();};this.__dO.push([bm,bl,bo,bq]);}else {this.__dS(bm,bl,bo,bq);};},remove:function(bs){var br=null;for(var i=0,l=this.__dM.length;i<l;i++ ){if(this.__dM[i]==bs){br=i;this.__ea(bs);break;};};if(br){qx.lang.Array.removeAt(this.__dM,br);};if(bs in this.__dN){this.__dN[bs].dispose();delete this.__dN[bs];};},getPreferredFormats:function(){var bt=[];var bx=qx.core.Environment.get(h);var bu=qx.core.Environment.get(O);var bw=qx.core.Environment.get(b);var bv=qx.core.Environment.get(d);if((bx==bi&&qx.core.Environment.get(E)>=9)||(bx==v&&bu>=3.6)||(bx==g&&bu>=6)){bt.push(bh);};if((bx==P&&bu>=10)||(bx==G&&bu>=3.1)||(bx==v&&bu>=3.5)||(bx==g&&bu>=4)||(bx==F&&bw==z&&bv>=4.2)){bt.push(J);};if(bx==bi&&bu>=4){bt.push(y);};if(bx==p&&bw==z&&bv>=4.1){bt.push(f);};return bt;},removeStyleSheet:function(){this.__dM=[];if(this.__dQ){qx.bom.Stylesheet.removeSheet(this.__dQ);};this.__dQ=null;},__dS:function(bA,bC,bz,bD){if(!qx.lang.Array.contains(this.__dM,bA)){var bE=this.__dV(bC);var bB=this.__dW(bA,bE);if(!bB){throw new Error(R+bA+w);};if(!this.__dQ){this.__dQ=qx.bom.Stylesheet.createElement();};try{this.__dY(bB);}catch(bF){{};};this.__dM.push(bA);};if(!this.__dN[bA]){this.__dN[bA]=new qx.bom.webfonts.Validator(bA);this.__dN[bA].setTimeout(qx.bom.webfonts.Manager.VALIDATION_TIMEOUT);this.__dN[bA].addListenerOnce(s,this.__dU,this);};if(bz){var by=bD||window;this.__dN[bA].addListenerOnce(s,bz,by);};this.__dN[bA].validate();},__dT:function(){if(this.__dO.length==0){this.__dR.stop();return;};var bG=this.__dO.shift();this.__dS.apply(this,bG);},__dU:function(bH){var bI=bH.getData();if(bI.valid===false){qx.event.Timer.once(function(){this.remove(bI.family);},this,250);};},__dV:function(bJ){var bL=qx.bom.webfonts.Manager.FONT_FORMATS;var bK={};for(var i=0,l=bJ.length;i<l;i++ ){var bM=null;for(var x=0;x<bL.length;x++ ){var bN=new RegExp(B+bL[x]+c);var bO=bN.exec(bJ[i]);if(bO){bM=bO[1];};};if(bM){bK[bM]=bJ[i];};};return bK;},__dW:function(bR,bU){var bT=[];var bP=this.__dP.length>0?this.__dP:qx.bom.webfonts.Manager.FONT_FORMATS;for(var i=0,l=bP.length;i<l;i++ ){var bQ=bP[i];if(bU[bQ]){bT.push(this.__dX(bQ,bU[bQ]));};};var bS=o+bT.join(n)+bj;bS=D+bR+K+bS;bS=bS+X;return bS;},__dX:function(bW,bV){switch(bW){case y:return U+bV+r+V+bV+Y;case bh:return U+bV+M;case J:return U+bV+bg;case f:return U+bV+L;default:return null;};},__dY:function(bY){var bX=bf+bY+C;if(qx.core.Environment.get(h)==bi&&qx.core.Environment.get(E)<9){var ca=this.__eb(this.__dQ.cssText);ca+=bX;this.__dQ.cssText=ca;}else {this.__dQ.insertRule(bX,this.__dQ.cssRules.length);};},__ea:function(cb){var ce=new RegExp(H+cb,a);for(var i=0,l=document.styleSheets.length;i<l;i++ ){var cc=document.styleSheets[i];if(cc.cssText){var cd=cc.cssText.replace(/\n/g,I).replace(/\r/g,I);cd=this.__eb(cd);if(ce.exec(cd)){cd=cd.replace(ce,I);};cc.cssText=cd;}else if(cc.cssRules){for(var j=0,m=cc.cssRules.length;j<m;j++ ){var cd=cc.cssRules[j].cssText.replace(/\n/g,I).replace(/\r/g,I);if(ce.exec(cd)){this.__dQ.deleteRule(j);return;};};};};},__eb:function(cf){return cf.replace(q,A).replace(W,N);}},destruct:function(){if(this.__dR){this.__dR.stop();this.__dR.dispose();};delete this.__dM;this.removeStyleSheet();for(var cg in this.__dN){this.__dN[cg].dispose();};qx.bom.webfonts.Validator.removeDefaultHelperElements();}});})();(function(){var a="qx.event.Timer",b="_applyInterval",c="_applyEnabled",d="Boolean",f="interval",g="qx.event.type.Event",h="Integer";qx.Class.define(a,{extend:qx.core.Object,construct:function(i){qx.core.Object.call(this);this.setEnabled(false);if(i!=null){this.setInterval(i);};var self=this;this.__ec=function(){self._oninterval.call(self);};},events:{"interval":g},statics:{once:function(j,k,l){{};var m=new qx.event.Timer(l);m.__ed=j;m.addListener(f,function(e){m.stop();j.call(k,e);m.dispose();k=null;},k);m.start();return m;}},properties:{enabled:{init:true,check:d,apply:c},interval:{check:h,init:1000,apply:b}},members:{__ee:null,__ec:null,_applyInterval:function(o,n){if(this.getEnabled()){this.restart();};},_applyEnabled:function(q,p){if(p){window.clearInterval(this.__ee);this.__ee=null;}else if(q){this.__ee=window.setInterval(this.__ec,this.getInterval());};},start:function(){this.setEnabled(true);},startWith:function(r){this.setInterval(r);this.start();},stop:function(){this.setEnabled(false);},restart:function(){this.stop();this.start();},restartWith:function(s){this.stop();this.startWith(s);},_oninterval:qx.event.GlobalError.observeMethod(function(){if(this.$$disposed){return;};if(this.getEnabled()){this.fireEvent(f);};})},destruct:function(){if(this.__ee){window.clearInterval(this.__ee);};this.__ee=this.__ec=null;}});})();(function(){var a="sans-serif",b="changeStatus",c="Integer",d="auto",e="qx.event.type.Data",f="0",g="qx.bom.webfonts.Validator",h="interval",i="Georgia",j="WEei",k="visible",l="Times New Roman",m="Arial",n="normal",o="Helvetica",p="__ei",q="350px",r="_applyFontFamily",s="-1000px",t="hidden",u="serif",v="span",w="absolute",x=",";qx.Class.define(g,{extend:qx.core.Object,construct:function(y){qx.core.Object.call(this);if(y){this.setFontFamily(y);this.__ef=this._getRequestedHelpers();};},statics:{COMPARISON_FONTS:{sans:[m,o,a],serif:[l,i,u]},HELPER_CSS:{position:w,margin:f,padding:f,top:s,left:s,fontSize:q,width:d,height:d,lineHeight:n,fontVariant:n,visibility:t},COMPARISON_STRING:j,__eg:null,__eh:null,removeDefaultHelperElements:function(){var z=qx.bom.webfonts.Validator.__eh;if(z){for(var A in z){document.body.removeChild(z[A]);};};delete qx.bom.webfonts.Validator.__eh;}},properties:{fontFamily:{nullable:true,init:null,apply:r},timeout:{check:c,init:5000}},events:{"changeStatus":e},members:{__ef:null,__ei:null,__ej:null,validate:function(){this.__ej=new Date().getTime();if(this.__ei){this.__ei.restart();}else {this.__ei=new qx.event.Timer(100);this.__ei.addListener(h,this.__el,this);qx.event.Timer.once(function(){this.__ei.start();},this,0);};},_reset:function(){if(this.__ef){for(var C in this.__ef){var B=this.__ef[C];document.body.removeChild(B);};this.__ef=null;};},_isFontValid:function(){if(!qx.bom.webfonts.Validator.__eg){this.__ek();};if(!this.__ef){this.__ef=this._getRequestedHelpers();};this.__ef.sans.style.visibility=k;this.__ef.sans.style.visibility=t;this.__ef.serif.style.visibility=k;this.__ef.serif.style.visibility=t;var E=qx.bom.element.Dimension.getWidth(this.__ef.sans);var D=qx.bom.element.Dimension.getWidth(this.__ef.serif);var F=qx.bom.webfonts.Validator;if(E!==F.__eg.sans||D!==F.__eg.serif){return true;};return false;},_getRequestedHelpers:function(){var G=[this.getFontFamily()].concat(qx.bom.webfonts.Validator.COMPARISON_FONTS.sans);var H=[this.getFontFamily()].concat(qx.bom.webfonts.Validator.COMPARISON_FONTS.serif);return {sans:this._getHelperElement(G),serif:this._getHelperElement(H)};},_getHelperElement:function(I){var J=qx.lang.Object.clone(qx.bom.webfonts.Validator.HELPER_CSS);if(I){if(J.fontFamily){J.fontFamily+=x+I.join(x);}else {J.fontFamily=I.join(x);};};var K=document.createElement(v);K.innerHTML=qx.bom.webfonts.Validator.COMPARISON_STRING;qx.bom.element.Style.setStyles(K,J);document.body.appendChild(K);return K;},_applyFontFamily:function(M,L){if(M!==L){this._reset();};},__ek:function(){var N=qx.bom.webfonts.Validator;if(!N.__eh){N.__eh={sans:this._getHelperElement(N.COMPARISON_FONTS.sans),serif:this._getHelperElement(N.COMPARISON_FONTS.serif)};};N.__eg={sans:qx.bom.element.Dimension.getWidth(N.__eh.sans),serif:qx.bom.element.Dimension.getWidth(N.__eh.serif)};},__el:function(){if(this._isFontValid()){this.__ei.stop();this._reset();this.fireDataEvent(b,{family:this.getFontFamily(),valid:true});}else {var O=new Date().getTime();if(O-this.__ej>=this.getTimeout()){this.__ei.stop();this._reset();this.fireDataEvent(b,{family:this.getFontFamily(),valid:false});};};}},destruct:function(){this._reset();this.__ei.stop();this.__ei.removeListener(h,this.__el,this);this._disposeObjects(p);}});})();(function(){var a="mshtml",b="engine.name",c="qx.bom.element.Dimension",d="0px",e="paddingRight",f="paddingLeft",g="opera",h="overflowY",i="paddingTop",j="overflowX",k="browser.documentmode",l="paddingBottom";qx.Bootstrap.define(c,{statics:{getWidth:function(n){var m=this._getBoundingClientRect(n);return Math.round(m.right-m.left);},getHeight:function(p){var o=this._getBoundingClientRect(p);return Math.round(o.bottom-o.top);},_getBoundingClientRect:function(t){var s=t.getBoundingClientRect();if(qx.core.Environment.get(k)===11&&!!document.msFullscreenElement&&window!==window.top&&this.__em(t)){var q={};for(var r in s){q[r]=s[r]*100;};s=q;};return s;},__em:function(u){if(document.msFullscreenElement===u){return true;};return qx.dom.Hierarchy.contains(document.msFullscreenElement,u);},getSize:function(v){return {width:this.getWidth(v),height:this.getHeight(v)};},__en:{visible:true,hidden:true},getContentWidth:function(z){var w=qx.bom.element.Style;var x=qx.bom.element.Style.get(z,j);var y=parseInt(w.get(z,f)||d,10);var C=parseInt(w.get(z,e)||d,10);if(this.__en[x]){var B=z.clientWidth;if((qx.core.Environment.get(b)==g)||qx.dom.Node.isBlockNode(z)){B=B-y-C;};if(qx.core.Environment.get(b)==a){if(B===0&&z.offsetHeight===0){return z.offsetWidth;};};return B;}else {if(z.clientWidth>=z.scrollWidth){return Math.max(z.clientWidth,z.scrollWidth)-y-C;}else {var A=z.scrollWidth-y;if(qx.core.Environment.get(b)==a){A-=C;};return A;};};},getContentHeight:function(H){var D=qx.bom.element.Style;var G=qx.bom.element.Style.get(H,h);var F=parseInt(D.get(H,i)||d,10);var E=parseInt(D.get(H,l)||d,10);if(this.__en[G]){return H.clientHeight-F-E;}else {if(H.clientHeight>=H.scrollHeight){return Math.max(H.clientHeight,H.scrollHeight)-F-E;}else {return H.scrollHeight-F;};};},getContentSize:function(I){return {width:this.getContentWidth(I),height:this.getContentHeight(I)};}}});})();(function(){var a="qx.dom.Hierarchy",b="previousSibling",c="html.element.contains",d="html.element.compareDocumentPosition",e="nextSibling",f="parentNode",g="*";qx.Bootstrap.define(a,{statics:{getNodeIndex:function(h){var i=0;while(h&&(h=h.previousSibling)){i++ ;};return i;},getElementIndex:function(l){var j=0;var k=qx.dom.Node.ELEMENT;while(l&&(l=l.previousSibling)){if(l.nodeType==k){j++ ;};};return j;},getNextElementSibling:function(m){while(m&&(m=m.nextSibling)&&!qx.dom.Node.isElement(m)){continue;};return m||null;},getPreviousElementSibling:function(n){while(n&&(n=n.previousSibling)&&!qx.dom.Node.isElement(n)){continue;};return n||null;},contains:function(q,p){if(qx.core.Environment.get(c)){if(qx.dom.Node.isDocument(q)){var o=qx.dom.Node.getDocument(p);return q&&o==q;}else if(qx.dom.Node.isDocument(p)){return false;}else {return q.contains(p);};}else if(qx.core.Environment.get(d)){return !!(q.compareDocumentPosition(p)&16);}else {while(p){if(q==p){return true;};p=p.parentNode;};return false;};},isRendered:function(s){var r=s.ownerDocument||s.document;if(qx.core.Environment.get(c)){if(!s.parentNode){return false;};return r.body.contains(s);}else if(qx.core.Environment.get(d)){return !!(r.compareDocumentPosition(s)&16);}else {while(s){if(s==r.body){return true;};s=s.parentNode;};return false;};},isDescendantOf:function(u,t){return this.contains(t,u);},getCommonParent:function(w,x){if(w===x){return w;};if(qx.core.Environment.get(c)){while(w&&qx.dom.Node.isElement(w)){if(w.contains(x)){return w;};w=w.parentNode;};return null;}else {var v=[];while(w||x){if(w){if(qx.lang.Array.contains(v,w)){return w;};v.push(w);w=w.parentNode;};if(x){if(qx.lang.Array.contains(v,x)){return x;};v.push(x);x=x.parentNode;};};return null;};},getAncestors:function(y){return this._recursivelyCollect(y,f);},getChildElements:function(A){A=A.firstChild;if(!A){return [];};var z=this.getNextSiblings(A);if(A.nodeType===1){z.unshift(A);};return z;},getDescendants:function(B){return qx.lang.Array.fromCollection(B.getElementsByTagName(g));},getFirstDescendant:function(C){C=C.firstChild;while(C&&C.nodeType!=1){C=C.nextSibling;};return C;},getLastDescendant:function(D){D=D.lastChild;while(D&&D.nodeType!=1){D=D.previousSibling;};return D;},getPreviousSiblings:function(E){return this._recursivelyCollect(E,b);},getNextSiblings:function(F){return this._recursivelyCollect(F,e);},_recursivelyCollect:function(I,G){var H=[];while(I=I[G]){if(I.nodeType==1){H.push(I);};};return H;},getSiblings:function(J){return this.getPreviousSiblings(J).reverse().concat(this.getNextSiblings(J));},isEmpty:function(K){K=K.firstChild;while(K){if(K.nodeType===qx.dom.Node.ELEMENT||K.nodeType===qx.dom.Node.TEXT){return false;};K=K.nextSibling;};return true;},cleanWhitespace:function(N){var L=N.firstChild;while(L){var M=L.nextSibling;if(L.nodeType==3&&!/\S/.test(L.nodeValue)){N.removeChild(L);};L=M;};}}});})();(function(){var a="engine.name",b=");",c="",d=")",e="zoom:1;filter:alpha(opacity=",f="qx.bom.element.Opacity",g="css.opacity",h=";",i="opacity:",j="alpha(opacity=",k="opacity",l="filter";qx.Bootstrap.define(f,{statics:{compile:qx.core.Environment.select(a,{"mshtml":function(m){if(m>=1){m=1;};if(m<0.00001){m=0;};if(qx.core.Environment.get(g)){return i+m+h;}else {return e+(m*100)+b;};},"default":function(n){return i+n+h;}}),set:qx.core.Environment.select(a,{"mshtml":function(q,o){if(qx.core.Environment.get(g)){q.style.opacity=o;}else {var p=qx.bom.element.Style.get(q,l,qx.bom.element.Style.COMPUTED_MODE,false);if(o>=1){o=1;};if(o<0.00001){o=0;};if(!q.currentStyle||!q.currentStyle.hasLayout){q.style.zoom=1;};q.style.filter=p.replace(/alpha\([^\)]*\)/gi,c)+j+o*100+d;};},"default":function(s,r){s.style.opacity=r;}}),reset:qx.core.Environment.select(a,{"mshtml":function(u){if(qx.core.Environment.get(g)){u.style.opacity=c;}else {var t=qx.bom.element.Style.get(u,l,qx.bom.element.Style.COMPUTED_MODE,false);u.style.filter=t.replace(/alpha\([^\)]*\)/gi,c);};},"default":function(v){v.style.opacity=c;}}),get:qx.core.Environment.select(a,{"mshtml":function(z,y){if(qx.core.Environment.get(g)){var w=qx.bom.element.Style.get(z,k,y,false);if(w!=null){return parseFloat(w);};return 1.0;}else {var x=qx.bom.element.Style.get(z,l,y,false);if(x){var w=x.match(/alpha\(opacity=(.*)\)/);if(w&&w[1]){return parseFloat(w[1])/100;};};return 1.0;};},"default":function(C,B){var A=qx.bom.element.Style.get(C,k,B,false);if(A!=null){return parseFloat(A);};return 1.0;}})}});})();(function(){var a="cursor:",b="engine.name",c="",d="mshtml",e="nw-resize",f="engine.version",g="nesw-resize",h="browser.documentmode",i=";",j="nwse-resize",k="qx.bom.element.Cursor",l="ne-resize",m="browser.quirksmode",n="cursor";qx.Bootstrap.define(k,{statics:{__eo:{},compile:function(o){return a+(this.__eo[o]||o)+i;},get:function(q,p){return qx.bom.element.Style.get(q,n,p,false);},set:function(s,r){s.style.cursor=this.__eo[r]||r;},reset:function(t){t.style.cursor=c;}},defer:function(u){if(qx.core.Environment.get(b)==d&&((parseFloat(qx.core.Environment.get(f))<9||qx.core.Environment.get(h)<9)&&!qx.core.Environment.get(m))){u.__eo[g]=l;u.__eo[j]=e;};}});})();(function(){var a="border-box",b="qx.bom.element.BoxSizing",c="css.boxsizing",d="",e="boxSizing",f="content-box",g=":",h=";";qx.Bootstrap.define(b,{statics:{__ep:{tags:{button:true,select:true},types:{search:true,button:true,submit:true,reset:true,checkbox:true,radio:true}},__eq:function(j){var i=this.__ep;return i.tags[j.tagName.toLowerCase()]||i.types[j.type];},compile:function(k){if(qx.core.Environment.get(c)){var l=qx.bom.Style.getCssName(qx.core.Environment.get(c));return l+g+k+h;}else {{};};},get:function(m){if(qx.core.Environment.get(c)){return qx.bom.element.Style.get(m,e,null,false)||d;};if(qx.bom.Document.isStandardMode(qx.dom.Node.getWindow(m))){if(!this.__eq(m)){return f;};};return a;},set:function(o,n){if(qx.core.Environment.get(c)){try{o.style[qx.core.Environment.get(c)]=n;}catch(p){{};};}else {{};};},reset:function(q){this.set(q,d);}}});})();(function(){var a="clip:auto;",b="rect(",c=")",d=");",e="",f="px",g="Could not parse clip string: ",h="qx.bom.element.Clip",i="string",j="clip:rect(",k=" ",l="clip",m="rect(auto,auto,auto,auto)",n="rect(auto, auto, auto, auto)",o="auto",p=",";qx.Bootstrap.define(h,{statics:{compile:function(q){if(!q){return a;};var v=q.left;var top=q.top;var u=q.width;var t=q.height;var r,s;if(v==null){r=(u==null?o:u+f);v=o;}else {r=(u==null?o:v+u+f);v=v+f;};if(top==null){s=(t==null?o:t+f);top=o;}else {s=(t==null?o:top+t+f);top=top+f;};return j+top+p+r+p+s+p+v+d;},get:function(z,D){var x=qx.bom.element.Style.get(z,l,D,false);var C,top,A,E;var w,y;if(typeof x===i&&x!==o&&x!==e){x=x.trim();if(/\((.*)\)/.test(x)){var F=RegExp.$1;if(/,/.test(F)){var B=F.split(p);}else {var B=F.split(k);};top=B[0].trim();w=B[1].trim();y=B[2].trim();C=B[3].trim();if(C===o){C=null;};if(top===o){top=null;};if(w===o){w=null;};if(y===o){y=null;};if(top!=null){top=parseInt(top,10);};if(w!=null){w=parseInt(w,10);};if(y!=null){y=parseInt(y,10);};if(C!=null){C=parseInt(C,10);};if(w!=null&&C!=null){A=w-C;}else if(w!=null){A=w;};if(y!=null&&top!=null){E=y-top;}else if(y!=null){E=y;};}else {throw new Error(g+x);};};return {left:C||null,top:top||null,width:A||null,height:E||null};},set:function(L,G){if(!G){L.style.clip=m;return;};var M=G.left;var top=G.top;var K=G.width;var J=G.height;var H,I;if(M==null){H=(K==null?o:K+f);M=o;}else {H=(K==null?o:M+K+f);M=M+f;};if(top==null){I=(J==null?o:J+f);top=o;}else {I=(J==null?o:top+J+f);top=top+f;};L.style.clip=b+top+p+H+p+I+p+M+c;},reset:function(N){N.style.clip=n;}}});})();(function(){var a="css.float",b='cssFloat',c="px",d="Cascaded styles are not supported in this browser!",e="css.appearance",f="pixelRight",g="css.userselect",h="css.boxsizing",i="css.textoverflow",j="pixelHeight",k=":",l="pixelTop",m="css.borderimage",n="pixelLeft",o="css.usermodify",p="qx.bom.element.Style",q="",r="pixelBottom",s="pixelWidth",t='float',u=";",v="\"\"",w="style";qx.Bootstrap.define(p,{statics:{__er:null,__es:null,__et:function(){var y={"appearance":qx.core.Environment.get(e),"userSelect":qx.core.Environment.get(g),"textOverflow":qx.core.Environment.get(i),"borderImage":qx.core.Environment.get(m),"float":qx.core.Environment.get(a),"userModify":qx.core.Environment.get(o),"boxSizing":qx.core.Environment.get(h)};this.__es={};for(var x in qx.lang.Object.clone(y)){if(!y[x]){delete y[x];}else {if(x===t){this.__es[b]=x;}else {this.__es[x]=qx.bom.Style.getCssName(y[x]);};};};this.__er=y;},__eu:function(name){var z=qx.bom.Style.getPropertyName(name);if(z){this.__er[name]=z;};return z;},__ev:{width:s,height:j,left:n,right:f,top:l,bottom:r},__ew:{clip:qx.bom.element.Clip,cursor:qx.bom.element.Cursor,opacity:qx.bom.element.Opacity,boxSizing:qx.bom.element.BoxSizing},compile:function(A){var D=[];var E=this.__ew;var C=this.__es;var name,B;for(name in A){B=A[name];if(B==null){continue;};name=this.__es[name]||name;if(E[name]){D.push(E[name].compile(B));}else {if(!C[name]){C[name]=qx.bom.Style.getCssName(name);};D.push(C[name],k,B===q?v:B,u);};};return D.join(q);},setCss:function(G,F){G.setAttribute(w,F);},getCss:function(H){return H.getAttribute(w);},isPropertySupported:function(I){return (this.__ew[I]||this.__er[I]||I in document.documentElement.style);},COMPUTED_MODE:1,CASCADED_MODE:2,LOCAL_MODE:3,set:function(L,name,J,K){{};name=this.__er[name]||this.__eu(name)||name;if(K!==false&&this.__ew[name]){this.__ew[name].set(L,J);}else {L.style[name]=J!==null?J:q;};},setStyles:function(S,M,T){{};var P=this.__er;var Q=this.__ew;var N=S.style;for(var R in M){var O=M[R];var name=P[R]||this.__eu(R)||R;if(O===undefined){if(T!==false&&Q[name]){Q[name].reset(S);}else {N[name]=q;};}else {if(T!==false&&Q[name]){Q[name].set(S,O);}else {N[name]=O!==null?O:q;};};};},reset:function(V,name,U){name=this.__er[name]||this.__eu(name)||name;if(U!==false&&this.__ew[name]){this.__ew[name].reset(V);}else {V.style[name]=q;};},get:function(ba,name,bc,be){name=this.__er[name]||this.__eu(name)||name;if(be!==false&&this.__ew[name]){return this.__ew[name].get(ba,bc);};switch(bc){case this.LOCAL_MODE:return ba.style[name]||q;case this.CASCADED_MODE:if(ba.currentStyle){return ba.currentStyle[name]||q;};throw new Error(d);default:var X=qx.dom.Node.getDocument(ba);var bb=X.defaultView?X.defaultView.getComputedStyle:undefined;if(bb!==undefined){var W=bb(ba,null);if(W&&W[name]){return W[name];};}else {if(!ba.currentStyle){return ba.style[name]||q;};var bg=ba.currentStyle[name]||ba.style[name]||q;if(/^-?[\.\d]+(px)?$/i.test(bg)){return bg;};var bf=this.__ev[name];if(bf&&(bf in ba.style)){var bd=ba.style[name];ba.style[name]=bg||0;var Y=ba.style[bf]+c;ba.style[name]=bd;return Y;};return bg;};return ba.style[name]||q;};}},defer:function(bh){bh.__et();}});})();(function(){var a="engine.name",b="CSS1Compat",c="position:absolute;width:0;height:0;width:1",d="engine.version",e="qx.bom.Document",f="1px",g="div";qx.Bootstrap.define(e,{statics:{isQuirksMode:qx.core.Environment.select(a,{"mshtml":function(h){if(qx.core.Environment.get(d)>=8){return (h||window).document.documentMode===5;}else {return (h||window).document.compatMode!==b;};},"webkit":function(i){if(document.compatMode===undefined){var j=(i||window).document.createElement(g);j.style.cssText=c;return j.style.width===f?true:false;}else {return (i||window).document.compatMode!==b;};},"default":function(k){return (k||window).document.compatMode!==b;}}),isStandardMode:function(l){return !this.isQuirksMode(l);},getWidth:function(m){var o=(m||window).document;var n=qx.bom.Viewport.getWidth(m);var scroll=this.isStandardMode(m)?o.documentElement.scrollWidth:o.body.scrollWidth;return Math.max(scroll,n);},getHeight:function(p){var r=(p||window).document;var q=qx.bom.Viewport.getHeight(p);var scroll=this.isStandardMode(p)?r.documentElement.scrollHeight:r.body.scrollHeight;return Math.max(scroll,q);}}});})();(function(){var a="ios",b="os.name",c="undefined",d="qx.bom.Viewport";qx.Bootstrap.define(d,{statics:{getWidth:function(e){var e=e||window;var f=e.document;return qx.bom.Document.isStandardMode(e)?f.documentElement.clientWidth:f.body.clientWidth;},getHeight:function(g){var g=g||window;var h=g.document;if(qx.core.Environment.get(b)==a&&window.innerHeight!=h.documentElement.clientHeight){return window.innerHeight;};return qx.bom.Document.isStandardMode(g)?h.documentElement.clientHeight:h.body.clientHeight;},getScrollLeft:function(i){var i=i?i:window;if(typeof i.pageXOffset!==c){return i.pageXOffset;};var j=i.document;return j.documentElement.scrollLeft||j.body.scrollLeft;},getScrollTop:function(k){var k=k?k:window;if(typeof k.pageYOffset!==c){return k.pageYOffset;};var l=k.document;return l.documentElement.scrollTop||l.body.scrollTop;},__ex:function(m){var o=this.getWidth(m)>this.getHeight(m)?90:0;var n=m.orientation;if(n==null||Math.abs(n%180)==o){return {"-270":90,"-180":180,"-90":-90,"0":0,"90":90,"180":180,"270":-90};}else {return {"-270":180,"-180":-90,"-90":0,"0":90,"90":180,"180":-90,"270":0};};},__ey:null,getOrientation:function(p){var p=p||window.top;var q=p.orientation;if(q==null){q=this.getWidth(p)>this.getHeight(p)?90:0;}else {if(this.__ey==null){this.__ey=this.__ex(p);};q=this.__ey[q];};return q;},isLandscape:function(r){var s=this.getOrientation(r);return s===-90||s===90;},isPortrait:function(t){var u=this.getOrientation(t);return u===0||u===180;}}});})();(function(){var a="qx.theme.manager.Icon",b="Theme",c="changeTheme",d="_applyTheme",e="singleton";qx.Class.define(a,{type:e,extend:qx.core.Object,properties:{theme:{check:b,nullable:true,apply:d,event:c}},members:{_applyTheme:function(i,g){var h=qx.util.AliasManager.getInstance();if(g){for(var f in g.aliases){h.remove(f);};};if(i){for(var f in i.aliases){h.add(f,i.aliases[f]);};};}}});})();(function(){var a="Missing appearance: ",b="_applyTheme",c="string",d="qx.theme.manager.Appearance",e=":",f="Theme",g="changeTheme",h="/",j="singleton";qx.Class.define(d,{type:j,extend:qx.core.Object,construct:function(){qx.core.Object.call(this);this.__ez={};this.__eA={};},properties:{theme:{check:f,nullable:true,event:g,apply:b}},members:{__eB:{},__ez:null,__eA:null,_applyTheme:function(){this.__eA={};this.__ez={};},__eC:function(y,u,l,p){var r=u.appearances;var m=r[y];if(!m){var x=h;var n=[];var q=y.split(x);var w=qx.lang.Array.clone(q);var t;while(!m&&q.length>0){n.unshift(q.pop());var o=q.join(x);m=r[o];if(m){t=m.alias||m;if(typeof t===c){var v=t+x+n.join(x);return this.__eC(v,u,l,w);};};};for(var i=0;i<n.length-1;i++ ){n.shift();var s=n.join(x);var k=this.__eC(s,u,null,w);if(k){return k;};};if(l!=null){return this.__eC(l,u,null,w);};{};return null;}else if(typeof m===c){return this.__eC(m,u,l,w);}else if(m.include&&!m.style){return this.__eC(m.include,u,l,w);};return y;},styleFrom:function(R,J,K,A){if(!K){K=this.getTheme();};var H=this.__eA;var z=H[R];if(!z){z=H[R]=this.__eC(R,K,A);};var O=K.appearances[z];if(!O){this.warn(a+R);return null;};if(!O.style){return null;};var P=z;if(J){var D=O.$$bits;if(!D){D=O.$$bits={};O.$$length=0;};var E=0;for(var G in J){if(!J[G]){continue;};if(D[G]==null){D[G]=1<<O.$$length++ ;};E+=D[G];};if(E>0){P+=e+E;};};var F=this.__ez;if(F[P]!==undefined){return F[P];};if(!J){J=this.__eB;};var M;if(O.include||O.base){var Q;if(O.include){Q=this.styleFrom(O.include,J,K,A);};var I=O.style(J,Q);M={};if(O.base){var N=this.styleFrom(z,J,O.base,A);if(O.include){for(var C in N){if(!Q.hasOwnProperty(C)&&!I.hasOwnProperty(C)){M[C]=N[C];};};}else {for(var L in N){if(!I.hasOwnProperty(L)){M[L]=N[L];};};};};if(O.include){for(var B in Q){if(!I.hasOwnProperty(B)){M[B]=Q[B];};};};for(var S in I){M[S]=I[S];};}else {M=O.style(J);};return F[P]=M||null;}},destruct:function(){this.__ez=this.__eA=null;}});})();(function(){var b="'!",c="other",d="widgets",e="undefined",f="fonts",g="appearances",h="qx.Theme",j="]",k="Mixin theme is not a valid theme!",m="[Theme ",n="colors",o="decorations",p="' are not compatible '",q="Theme",r="meta",s="The mixins '",t="borders",u="icons";qx.Bootstrap.define(h,{statics:{define:function(name,w){if(!w){var w={};};w.include=this.__eD(w.include);w.patch=this.__eD(w.patch);{};var v={$$type:q,name:name,title:w.title,toString:this.genericToString};if(w.extend){v.supertheme=w.extend;};v.basename=qx.Bootstrap.createNamespace(name,v);this.__eG(v,w);this.__eE(v,w);this.$$registry[name]=v;for(var i=0,a=w.include,l=a.length;i<l;i++ ){this.include(v,a[i]);};for(var i=0,a=w.patch,l=a.length;i<l;i++ ){this.patch(v,a[i]);};},__eD:function(x){if(!x){return [];};if(qx.Bootstrap.isArray(x)){return x;}else {return [x];};},__eE:function(y,z){var A=z.aliases||{};if(z.extend&&z.extend.aliases){qx.Bootstrap.objectMergeWith(A,z.extend.aliases,false);};y.aliases=A;},getAll:function(){return this.$$registry;},getByName:function(name){return this.$$registry[name];},isDefined:function(name){return this.getByName(name)!==undefined;},getTotalNumber:function(){return qx.Bootstrap.objectGetLength(this.$$registry);},genericToString:function(){return m+this.name+j;},__eF:function(C){for(var i=0,B=this.__eH,l=B.length;i<l;i++ ){if(C[B[i]]){return B[i];};};},__eG:function(H,I){var E=this.__eF(I);if(I.extend&&!E){E=I.extend.type;};H.type=E||c;var F=function(){};if(I.extend){F.prototype=new I.extend.$$clazz;};var D=F.prototype;var G=I[E];for(var J in G){D[J]=G[J];if(D[J].base){{};D[J].base=I.extend;};};H.$$clazz=F;H[E]=new F;},$$registry:{},__eH:[n,t,o,f,u,d,g,r],__h:null,__eI:null,__i:function(){},patch:function(N,L){this.__eJ(L);var P=this.__eF(L);if(P!==this.__eF(N)){throw new Error(s+N.name+p+L.name+b);};var M=L[P];var K=N.$$clazz.prototype;for(var O in M){K[O]=M[O];};},include:function(T,R){this.__eJ(R);var V=R.type;if(V!==T.type){throw new Error(s+T.name+p+R.name+b);};var S=R[V];var Q=T.$$clazz.prototype;for(var U in S){if(Q[U]!==undefined){continue;};Q[U]=S[U];};},__eJ:function(W){if(typeof W===e||W==null){var X=new Error(k);{var Y;};throw X;};}}});})();(function(){var a="__eN",b="qx.ui.tooltip.ToolTip",c="Boolean",d="",f="mouse",g="pointerover",h="interval",i="__eK",j="_applyCurrent",k="widget",l="__eL",m="qx.ui.tooltip.Manager",n="pointermove",o="focusout",p="tooltip-error",q="singleton",r="pointerout";qx.Class.define(m,{type:q,extend:qx.core.Object,construct:function(){qx.core.Object.call(this);qx.event.Registration.addListener(document.body,g,this.__eS,this,true);this.__eK=new qx.event.Timer();this.__eK.addListener(h,this.__eP,this);this.__eL=new qx.event.Timer();this.__eL.addListener(h,this.__eQ,this);this.__eM={left:0,top:0};},properties:{current:{check:b,nullable:true,apply:j},showInvalidToolTips:{check:c,init:true},showToolTips:{check:c,init:true}},members:{__eM:null,__eL:null,__eK:null,__eN:null,__eO:null,getSharedTooltip:function(){if(!this.__eN){this.__eN=new qx.ui.tooltip.ToolTip().set({rich:true});};return this.__eN;},getSharedErrorTooltip:function(){if(!this.__eO){this.__eO=new qx.ui.tooltip.ToolTip().set({appearance:p,rich:true});this.__eO.setLabel(d);this.__eO.syncAppearance();};return this.__eO;},_applyCurrent:function(u,t){if(t&&qx.ui.core.Widget.contains(t,u)){return;};if(t){if(!t.isDisposed()){t.exclude();};this.__eK.stop();this.__eL.stop();};var v=qx.event.Registration;var s=document.body;if(u){this.__eK.startWith(u.getShowTimeout());v.addListener(s,r,this.__eT,this,true);v.addListener(s,o,this.__eU,this,true);v.addListener(s,n,this.__eR,this,true);}else {v.removeListener(s,r,this.__eT,this,true);v.removeListener(s,o,this.__eU,this,true);v.removeListener(s,n,this.__eR,this,true);};},__eP:function(e){var w=this.getCurrent();if(w&&!w.isDisposed()){this.__eL.startWith(w.getHideTimeout());if(w.getPlaceMethod()==k){w.placeToWidget(w.getOpener());}else {w.placeToPoint(this.__eM);};w.show();};this.__eK.stop();},__eQ:function(e){var x=this.getCurrent();if(x&&!x.isDisposed()){x.exclude();};this.__eL.stop();this.resetCurrent();},__eR:function(e){var y=this.__eM;y.left=Math.round(e.getDocumentLeft());y.top=Math.round(e.getDocumentTop());},__eS:function(e){var z=qx.ui.core.Widget.getWidgetByElement(e.getTarget());this.__eR(e);this.showToolTip(z);},showToolTip:function(C){if(!C){return;};var D,B,E,A;while(C!=null){D=C.getToolTip();B=C.getToolTipText()||null;E=C.getToolTipIcon()||null;if(qx.Class.hasInterface(C.constructor,qx.ui.form.IForm)&&!C.isValid()){A=C.getInvalidMessage();};if(D||B||E||A){break;};C=C.getLayoutParent();};if(!C||(!C.getEnabled()&&!C.isShowToolTipWhenDisabled())||C.isBlockToolTip()||(!A&&!this.getShowToolTips())||(A&&!this.getShowInvalidToolTips())){return;};if(A){D=this.getSharedErrorTooltip().set({label:A});};if(!D){D=this.getSharedTooltip().set({label:B,icon:E});};this.setCurrent(D);D.setOpener(C);},__eT:function(e){var F=qx.ui.core.Widget.getWidgetByElement(e.getTarget());if(!F){return;};var G=qx.ui.core.Widget.getWidgetByElement(e.getRelatedTarget());if(!G&&e.getPointerType()==f){return;};var H=this.getCurrent();if(H&&(G==H||qx.ui.core.Widget.contains(H,G))){return;};if(G&&F&&qx.ui.core.Widget.contains(F,G)){return;};if(H&&!G){this.setCurrent(null);}else {this.resetCurrent();};},__eU:function(e){var I=qx.ui.core.Widget.getWidgetByElement(e.getTarget());if(!I){return;};var J=this.getCurrent();if(J&&J==I.getToolTip()){this.setCurrent(null);};}},destruct:function(){qx.event.Registration.removeListener(document.body,g,this.__eS,this,true);this._disposeObjects(i,l,a);this.__eM=null;}});})();(function(){var a="qx.ui.core.MLayoutHandling";qx.Mixin.define(a,{members:{setLayout:function(b){this._setLayout(b);},getLayout:function(){return this._getLayout();}},statics:{remap:function(c){c.getLayout=c._getLayout;c.setLayout=c._setLayout;}}});})();(function(){var a="changeWidth",b="Boolean",c="allowShrinkY",d="_applyAlign",e="_applyStretching",f="bottom",g="Integer",h="changeTheme",i="_applyDimension",j="baseline",k="marginBottom",l="qx.ui.core.LayoutItem",m="center",n="marginTop",o="allowGrowX",p="shorthand",q="middle",r="marginLeft",s="qx.dyntheme",t="allowShrinkX",u="top",v="right",w="marginRight",x="abstract",y="_applyMargin",z="allowGrowY",A="left",B="changeHeight";qx.Class.define(l,{type:x,extend:qx.core.Object,construct:function(){qx.core.Object.call(this);if(qx.core.Environment.get(s)){qx.theme.manager.Meta.getInstance().addListener(h,this._onChangeTheme,this);};},properties:{minWidth:{check:g,nullable:true,apply:i,init:null,themeable:true},width:{check:g,event:a,nullable:true,apply:i,init:null,themeable:true},maxWidth:{check:g,nullable:true,apply:i,init:null,themeable:true},minHeight:{check:g,nullable:true,apply:i,init:null,themeable:true},height:{check:g,event:B,nullable:true,apply:i,init:null,themeable:true},maxHeight:{check:g,nullable:true,apply:i,init:null,themeable:true},allowGrowX:{check:b,apply:e,init:true,themeable:true},allowShrinkX:{check:b,apply:e,init:true,themeable:true},allowGrowY:{check:b,apply:e,init:true,themeable:true},allowShrinkY:{check:b,apply:e,init:true,themeable:true},allowStretchX:{group:[o,t],mode:p,themeable:true},allowStretchY:{group:[z,c],mode:p,themeable:true},marginTop:{check:g,init:0,apply:y,themeable:true},marginRight:{check:g,init:0,apply:y,themeable:true},marginBottom:{check:g,init:0,apply:y,themeable:true},marginLeft:{check:g,init:0,apply:y,themeable:true},margin:{group:[n,w,k,r],mode:p,themeable:true},alignX:{check:[A,m,v],nullable:true,apply:d,themeable:true},alignY:{check:[u,q,f,j],nullable:true,apply:d,themeable:true}},members:{_onChangeTheme:qx.core.Environment.select(s,{"true":function(){var E=qx.util.PropertyUtil.getAllProperties(this.constructor);for(var name in E){var D=E[name];if(D.themeable){var C=qx.util.PropertyUtil.getUserValue(this,name);if(C==null){qx.util.PropertyUtil.resetThemed(this,name);};};};},"false":null}),__eV:null,__eW:null,__eX:null,__eY:null,__fa:null,__fb:null,__fc:null,getBounds:function(){return this.__fb||this.__eW||null;},clearSeparators:function(){},renderSeparator:function(F,G){},renderLayout:function(N,top,K,J){{var L;};var I=null;if(this.getHeight()==null&&this._hasHeightForWidth()){var I=this._getHeightForWidth(K);};if(I!=null&&I!==this.__eV){this.__eV=I;qx.ui.core.queue.Layout.add(this);return null;};var H=this.__eW;if(!H){H=this.__eW={};};var M={};if(N!==H.left||top!==H.top){M.position=true;H.left=N;H.top=top;};if(K!==H.width||J!==H.height){M.size=true;H.width=K;H.height=J;};if(this.__eX){M.local=true;delete this.__eX;};if(this.__fa){M.margin=true;delete this.__fa;};return M;},isExcluded:function(){return false;},hasValidLayout:function(){return !this.__eX;},scheduleLayoutUpdate:function(){qx.ui.core.queue.Layout.add(this);},invalidateLayoutCache:function(){this.__eX=true;this.__eY=null;},getSizeHint:function(O){var P=this.__eY;if(P){return P;};if(O===false){return null;};P=this.__eY=this._computeSizeHint();if(this._hasHeightForWidth()&&this.__eV&&this.getHeight()==null){P.height=this.__eV;};if(P.minWidth>P.width){P.width=P.minWidth;};if(P.maxWidth<P.width){P.width=P.maxWidth;};if(!this.getAllowGrowX()){P.maxWidth=P.width;};if(!this.getAllowShrinkX()){P.minWidth=P.width;};if(P.minHeight>P.height){P.height=P.minHeight;};if(P.maxHeight<P.height){P.height=P.maxHeight;};if(!this.getAllowGrowY()){P.maxHeight=P.height;};if(!this.getAllowShrinkY()){P.minHeight=P.height;};return P;},_computeSizeHint:function(){var U=this.getMinWidth()||0;var R=this.getMinHeight()||0;var V=this.getWidth()||U;var T=this.getHeight()||R;var Q=this.getMaxWidth()||Infinity;var S=this.getMaxHeight()||Infinity;return {minWidth:U,width:V,maxWidth:Q,minHeight:R,height:T,maxHeight:S};},_hasHeightForWidth:function(){var W=this._getLayout();if(W){return W.hasHeightForWidth();};return false;},_getHeightForWidth:function(X){var Y=this._getLayout();if(Y&&Y.hasHeightForWidth()){return Y.getHeightForWidth(X);};return null;},_getLayout:function(){return null;},_applyMargin:function(){this.__fa=true;var parent=this.$$parent;if(parent){parent.updateLayoutProperties();};},_applyAlign:function(){var parent=this.$$parent;if(parent){parent.updateLayoutProperties();};},_applyDimension:function(){qx.ui.core.queue.Layout.add(this);},_applyStretching:function(){qx.ui.core.queue.Layout.add(this);},hasUserBounds:function(){return !!this.__fb;},setUserBounds:function(bb,top,ba,bc){this.__fb={left:bb,top:top,width:ba,height:bc};qx.ui.core.queue.Layout.add(this);},resetUserBounds:function(){delete this.__fb;qx.ui.core.queue.Layout.add(this);},__fd:{},setLayoutProperties:function(bf){if(bf==null){return;};var bd=this.__fc;if(!bd){bd=this.__fc={};};var parent=this.getLayoutParent();if(parent){parent.updateLayoutProperties(bf);};for(var be in bf){if(bf[be]==null){delete bd[be];}else {bd[be]=bf[be];};};},getLayoutProperties:function(){return this.__fc||this.__fd;},clearLayoutProperties:function(){delete this.__fc;},updateLayoutProperties:function(bi){var bg=this._getLayout();if(bg){{var bh;};bg.invalidateChildrenCache();};qx.ui.core.queue.Layout.add(this);},getApplicationRoot:function(){return qx.core.Init.getApplication().getRoot();},getLayoutParent:function(){return this.$$parent||null;},setLayoutParent:function(parent){if(this.$$parent===parent){return;};this.$$parent=parent||null;qx.ui.core.queue.Visibility.add(this);},isRootWidget:function(){return false;},_getRoot:function(){var parent=this;while(parent){if(parent.isRootWidget()){return parent;};parent=parent.$$parent;};return null;},clone:function(){var bj=qx.core.Object.prototype.clone.call(this);var bk=this.__fc;if(bk){bj.__fc=qx.lang.Object.clone(bk);};return bj;}},destruct:function(){if(qx.core.Environment.get(s)){qx.theme.manager.Meta.getInstance().removeListener(h,this._onChangeTheme,this);};this.$$parent=this.$$subparent=this.__fc=this.__eW=this.__fb=this.__eY=null;}});})();(function(){var a="$$theme_",b="$$user_",c="qx.util.PropertyUtil",d="$$init_";qx.Class.define(c,{statics:{getProperties:function(e){return e.$$properties;},getAllProperties:function(j){var g={};var f=j;while(f!=qx.core.Object){var i=this.getProperties(f);for(var h in i){g[h]=i[h];};f=f.superclass;};return g;},getUserValue:function(l,k){return l[b+k];},setUserValue:function(n,m,o){n[b+m]=o;},deleteUserValue:function(q,p){delete (q[b+p]);},getInitValue:function(s,r){return s[d+r];},setInitValue:function(u,t,v){u[d+t]=v;},deleteInitValue:function(x,w){delete (x[d+w]);},getThemeValue:function(z,y){return z[a+y];},setThemeValue:function(B,A,C){B[a+A]=C;},deleteThemeValue:function(E,D){delete (E[a+D]);},setThemed:function(H,G,I){var F=qx.core.Property.$$method.setThemed;H[F[G]](I);},resetThemed:function(K,J){var L=qx.core.Property.$$method.resetThemed;K[L[J]]();}}});})();(function(){var a="qx.ui.core.queue.Layout",b="layout";qx.Class.define(a,{statics:{__dO:{},__fe:{},remove:function(c){delete this.__dO[c.$$hash];},add:function(d){this.__dO[d.$$hash]=d;qx.ui.core.queue.Manager.scheduleFlush(b);},isScheduled:function(e){return !!this.__dO[e.$$hash];},flush:function(){var f=this.__fg();for(var i=f.length-1;i>=0;i-- ){var g=f[i];if(g.hasValidLayout()){continue;};if(g.isRootWidget()&&!g.hasUserBounds()){var j=g.getSizeHint();g.renderLayout(0,0,j.width,j.height);}else {var h=g.getBounds();g.renderLayout(h.left,h.top,h.width,h.height);};};},getNestingLevel:function(l){var k=this.__fe;var n=0;var parent=l;while(true){if(k[parent.$$hash]!=null){n+=k[parent.$$hash];break;};if(!parent.$$parent){break;};parent=parent.$$parent;n+=1;};var m=n;while(l&&l!==parent){k[l.$$hash]=m-- ;l=l.$$parent;};return n;},__ff:function(){var t=qx.ui.core.queue.Visibility;this.__fe={};var s=[];var r=this.__dO;var o,q;for(var p in r){o=r[p];if(t.isVisible(o)){q=this.getNestingLevel(o);if(!s[q]){s[q]={};};s[q][p]=o;delete r[p];};};return s;},__fg:function(){var x=[];var z=this.__ff();for(var w=z.length-1;w>=0;w-- ){if(!z[w]){continue;};for(var v in z[w]){var u=z[w][v];if(w==0||u.isRootWidget()||u.hasUserBounds()){x.push(u);u.invalidateLayoutCache();continue;};var B=u.getSizeHint(false);if(B){u.invalidateLayoutCache();var y=u.getSizeHint();var A=(!u.getBounds()||B.minWidth!==y.minWidth||B.width!==y.width||B.maxWidth!==y.maxWidth||B.minHeight!==y.minHeight||B.height!==y.height||B.maxHeight!==y.maxHeight);}else {A=true;};if(A){var parent=u.getLayoutParent();if(!z[w-1]){z[w-1]={};};z[w-1][parent.$$hash]=parent;}else {x.push(u);};};};return x;}}});})();(function(){var a="mshtml",b="engine.name",c="pop.push.reverse.shift.sort.splice.unshift.join.slice",d="number",e="qx.type.BaseArray",f=".";qx.Bootstrap.define(e,{extend:Array,construct:function(g){},members:{toArray:null,valueOf:null,pop:null,push:null,reverse:null,shift:null,sort:null,splice:null,unshift:null,concat:null,join:null,slice:null,toString:null,indexOf:null,lastIndexOf:null,forEach:null,filter:null,map:null,some:null,every:null}});(function(){function h(p){if((qx.core.Environment.get(b)==a)){j.prototype={length:0,$$isArray:true};var n=c.split(f);for(var length=n.length;length;){j.prototype[n[ --length]]=Array.prototype[n[length]];};};var m=Array.prototype.slice;j.prototype.concat=function(){var r=this.slice(0);for(var i=0,length=arguments.length;i<length;i++ ){var q;if(arguments[i] instanceof j){q=m.call(arguments[i],0);}else if(arguments[i] instanceof Array){q=arguments[i];}else {q=[arguments[i]];};r.push.apply(r,q);};return r;};j.prototype.toString=function(){return m.call(this,0).toString();};j.prototype.toLocaleString=function(){return m.call(this,0).toLocaleString();};j.prototype.constructor=j;j.prototype.indexOf=Array.prototype.indexOf;j.prototype.lastIndexOf=Array.prototype.lastIndexOf;j.prototype.forEach=Array.prototype.forEach;j.prototype.some=Array.prototype.some;j.prototype.every=Array.prototype.every;var o=Array.prototype.filter;var l=Array.prototype.map;j.prototype.filter=function(){var s=new this.constructor;s.push.apply(s,o.apply(this,arguments));return s;};j.prototype.map=function(){var t=new this.constructor;t.push.apply(t,l.apply(this,arguments));return t;};j.prototype.slice=function(){var u=new this.constructor;u.push.apply(u,Array.prototype.slice.apply(this,arguments));return u;};j.prototype.splice=function(){var v=new this.constructor;v.push.apply(v,Array.prototype.splice.apply(this,arguments));return v;};j.prototype.toArray=function(){return Array.prototype.slice.call(this,0);};j.prototype.valueOf=function(){return this.length;};return j;};function j(length){if(arguments.length===1&&typeof length===d){this.length=-1<length&&length===length>>.5?length:this.push(length);}else if(arguments.length){this.push.apply(this,arguments);};};function k(){};k.prototype=[];j.prototype=new k;j.prototype.length=0;qx.type.BaseArray=h(j);})();})();(function(){var a="name",b="qxWeb",c="toString",d="$",e="number",f="_",g="data-qx-class",h="basename",j="classname";qx.Bootstrap.define(b,{extend:qx.type.BaseArray,statics:{__ek:[],$$qx:qx,$init:function(p,n){if(p.length&&p.length==1&&p[0]&&p[0].$widget instanceof qxWeb){return p[0].$widget;};var o=[];for(var i=0;i<p.length;i++ ){var m=!!(p[i]&&(p[i].nodeType===1||p[i].nodeType===9||p[i].nodeType===11));if(m){o.push(p[i]);continue;};var k=!!(p[i]&&p[i].history&&p[i].location&&p[i].document);if(k){o.push(p[i]);};};if(p[0]&&p[0].getAttribute&&p[0].getAttribute(g)&&o.length<2){n=qx.Bootstrap.getByName(p[0].getAttribute(g))||n;};var r=qx.lang.Array.cast(o,n);for(var i=0;i<qxWeb.__ek.length;i++ ){qxWeb.__ek[i].call(r);};return r;},$attach:function(t,s){for(var name in t){if(qxWeb.prototype[name]!=undefined&&Array.prototype[name]==undefined&&s!==true){{};}else {qxWeb.prototype[name]=t[name];};};},$attachStatic:function(v,u){for(var name in v){{};qxWeb[name]=v[name];};},$attachAll:function(y,x){for(var name in y.members){if(name.indexOf(d)!==0&&name.indexOf(f)!==0)qxWeb.prototype[name]=y.members[name];};var w;if(x!=null){qxWeb[x]=qxWeb[x]||{};w=qxWeb[x];}else {w=qxWeb;};for(var name in y.statics){if(name.indexOf(d)!==0&&name.indexOf(f)!==0&&name!==a&&name!==h&&name!==j&&name!==c&&name!==name.toUpperCase())w[name]=y.statics[name];};},$attachInit:function(z){this.__ek.push(z);},define:function(name,A){if(A==undefined){A=name;name=null;};return qx.Bootstrap.define.call(qx.Bootstrap,name,A);}},construct:function(C,B){if(!C&&this instanceof qxWeb){return this;};if(!C){C=[];}else if(qx.Bootstrap.isString(C)){if(B instanceof qxWeb&&B.length!=0){B=B[0];};if(B instanceof qxWeb){C=[];}else {C=qx.bom.Selector.query(C,B);};}else if((C.nodeType===1||C.nodeType===9||C.nodeType===11)||(C.history&&C.location&&C.document)){C=[C];};return qxWeb.$init(C,qxWeb);},members:{filter:function(D){if(qx.lang.Type.isFunction(D)){return qxWeb.$init(Array.prototype.filter.call(this,D),this.constructor);};return qxWeb.$init(qx.bom.Selector.matches(D,this),this.constructor);},unique:function(){var E=qx.lang.Array.unique(this);return qxWeb.$init(E,this.constructor);},slice:function(F,G){if(G!==undefined){return qxWeb.$init(Array.prototype.slice.call(this,F,G),this.constructor);};return qxWeb.$init(Array.prototype.slice.call(this,F),this.constructor);},splice:function(H,I,J){return qxWeb.$init(Array.prototype.splice.apply(this,arguments),this.constructor);},map:function(K,L){return qxWeb.$init(Array.prototype.map.apply(this,arguments),qxWeb);},concat:function(N){var M=Array.prototype.slice.call(this,0);for(var i=0;i<arguments.length;i++ ){if(arguments[i] instanceof qxWeb){M=M.concat(Array.prototype.slice.call(arguments[i],0));}else {M.push(arguments[i]);};};return qxWeb.$init(M,this.constructor);},indexOf:function(O,P){if(!O){return -1;};if(!P){P=0;};if(typeof P!==e){return -1;};if(P<0){P=this.length+P;if(P<0){P=0;};};if(qx.lang.Type.isArray(O)){O=O[0];};for(var i=P,l=this.length;i<l;i++ ){if(this[i]===O){return i;};};return -1;},debug:function(){{};return this;},logThis:function(){{var Q,length;};return this;},_forEachElement:function(S,R){for(var i=0,l=this.length;i<l;i++ ){if(this[i]&&(this[i].nodeType===1||this[i].nodeType===11)){S.apply(R||this,[this[i],i,this]);};};return this;},_forEachElementWrapped:function(U,T){this._forEachElement(function(V,X,W){U.apply(this,[qxWeb(V),X,W]);},T);return this;}},defer:function(Y){if(window.q==undefined){q=Y;};}});})();(function(){var c="-",d="(^|",f="'] ",g="CLASS",h=":disabled",k="div",l="input",n="checked|selected|async|autofocus|autoplay|controls|defer|disabled|hidden|ismap|loop|multiple|open|readonly|required|scoped",o="nth",p="*(?:(['\"])((?:\\\\.|[^\\\\])*?)\\3|(",q="type|href|height|width",r=")*)|.*)\\)|)",s="disabled",t="*(?:value|",u="~=",v="previousSibling",w="*(even|odd|(([+-]|)(\\d*)n|)",x="xml:lang",y="only",z="*",A="unsupported lang: ",B="+|((?:^|[^\\\\])(?:\\\\.)*)",C="i",D="\\\\([\\da-f]{1,6}",E="='$1']",F="w#",G="^=",H="*([>+~]|",I="[t^='']",J="*\\)|)",K="+$",L="=",M="unload",N="id",O="text",P="needsContext",Q="nextSibling",R="$=",S="[s!='']:x",T="string",U=")|.)",V="[\\x20\\t\\r\\n\\f]",W="[name=d]",X="*(?:([+-]|)",Y="*((?:-\\d)?\\d*)",cL="#",cM="[selected]",cN="type",cH="ig",cI="parentNode",cJ="href",cK="0x",cS="(",cT="w",cY="even",cU="<div class='a'></div><div class='a i'></div>",cO="g",cP="*\\]",cQ="*\\)|)(?=[^-]|$)",cR="unsupported pseudo: ",dC="w*",eo="*[*^$|!~]?=",da="<select t=''><option selected=''></option></select>",cV=" ",cW="hidden",el="*(?:([*^$|!~]?=)",cX="*,",db="function",dc="^",dd=")",di=")|)|)",dj=":(",dk="onunload",de="button",df="0",dg="^(",dh="option",dq="odd",dr="class",ds="*(\\d+)|))",dt="lang",dl="|=",dm="\\[",dn="name",dp="D",dx="!=",dy="<input/>",en="*[>+~]|:(even|odd|eq|gt|lt|nth|first|last)(?:\\(",dz="sizzle",du="*=",dv="|",em="Syntax error, unrecognized expression: ",dw=")$",dA="object",dB="?|(",dN="$1",dM=")(?:\\(((['\"])((?:\\\\.|[^\\\\])*?)\\3|((?:\\\\.|[^\\\\()[\\]]|",dL="*([^\\]'\"]*?)",dR="*(?:''|\"\")",dQ="eq",dP="className",dO=":enabled",dG="of-type",dF="TAG",dE="|$)",dD="<a href='#'></a>",dK="empty",dJ="qx.bom.Selector",dI="^(?:",dH="value",dY="[id='",dX="^#(",dW="[*^$]=",dV="*,:x",ed="*(",ec="^\\.(",eb="",ea="CHILD",dU=",.*:",dT="^:(only|first|last|nth|nth-last)-(child|of-type)(?:\\(",dS="(?:\\\\.|[\\w-]|[^\\x00-\\xa0])+",eg="$",ef="\\$&",ee=":checked",ek=",",ej="ID",ei="last",eh="HTML";qx.Bootstrap.define(dJ,{statics:{query:null,matches:null}});(function(window){var i,eM,fM,ew,eB,fC,eG,ep,eE,eF,eD,document,fK,fq,fe,eq,ff,eN,fa=dz+-(new Date()),eH=window.document,eW=0,eA=0,es=ey(),fr=ey(),fI=ey(),eT=function(a,b){if(a===b){eF=true;};return 0;},eR=typeof undefined,ft=1<<31,fA=({}).hasOwnProperty,ev=[],ez=ev.pop,fD=ev.push,fG=ev.push,eI=ev.slice,eS=ev.indexOf||function(fN){var i=0,fO=this.length;for(;i<fO;i++ ){if(this[i]===fN){return i;};};return -1;},fd=n,eK=V,fh=dS,fv=fh.replace(cT,F),fH=dm+eK+ed+fh+dd+eK+el+eK+p+fv+di+eK+cP,fg=dj+fh+dM+fH.replace(3,8)+r,fp=new RegExp(dc+eK+B+eK+K,cO),fx=new RegExp(dc+eK+cX+eK+z),eL=new RegExp(dc+eK+H+eK+dd+eK+z),fj=new RegExp(L+eK+dL+eK+cP,cO),fu=new RegExp(fg),eX=new RegExp(dc+fv+eg),fB={"ID":new RegExp(dX+fh+dd),"CLASS":new RegExp(ec+fh+dd),"TAG":new RegExp(dg+fh.replace(cT,dC)+dd),"ATTR":new RegExp(dc+fH),"PSEUDO":new RegExp(dc+fg),"CHILD":new RegExp(dT+eK+w+eK+X+eK+ds+eK+J,C),"bool":new RegExp(dI+fd+dw,C),"needsContext":new RegExp(dc+eK+en+eK+Y+eK+cQ,C)},fl=/^(?:input|select|textarea|button)$/i,et=/^h\d$/i,fz=/^[^{]+\{\s*\[native \w/,fF=/^(?:#([\w-]+)|(\w+)|\.([\w-]+))$/,fc=/[+~]/,fm=/'|\\/g,eu=new RegExp(D+eK+dB+eK+U,cH),fs=function(_,fP,fQ){var fR=cK+fP-0x10000;return fR!==fR||fQ?fP:fR<0?String.fromCharCode(fR+0x10000):String.fromCharCode(fR>>10|0xD800,fR&0x3FF|0xDC00);};try{fG.apply((ev=eI.call(eH.childNodes)),eH.childNodes);ev[eH.childNodes.length].nodeType;}catch(e){fG={apply:ev.length?function(fT,fS){fD.apply(fT,eI.call(fS));}:function(fV,fU){var j=fV.length,i=0;while((fV[j++ ]=fU[i++ ])){};fV.length=j-1;}};};function fL(gg,fX,gb,gd){var gi,fY,m,fW,i,ge,gh,ga,gf,gc;if((fX?fX.ownerDocument||fX:eH)!==document){eD(fX);};fX=fX||document;gb=gb||[];if(!gg||typeof gg!==T){return gb;};if((fW=fX.nodeType)!==1&&fW!==9){return [];};if(fq&&!gd){if((gi=fF.exec(gg))){if((m=gi[1])){if(fW===9){fY=fX.getElementById(m);if(fY&&fY.parentNode){if(fY.id===m){gb.push(fY);return gb;};}else {return gb;};}else {if(fX.ownerDocument&&(fY=fX.ownerDocument.getElementById(m))&&eN(fX,fY)&&fY.id===m){gb.push(fY);return gb;};};}else if(gi[2]){fG.apply(gb,fX.getElementsByTagName(gg));return gb;}else if((m=gi[3])&&eM.getElementsByClassName&&fX.getElementsByClassName){fG.apply(gb,fX.getElementsByClassName(m));return gb;};};if(eM.qsa&&(!fe||!fe.test(gg))){ga=gh=fa;gf=fX;gc=fW===9&&gg;if(fW===1&&fX.nodeName.toLowerCase()!==dA){ge=eV(gg);if((gh=fX.getAttribute(N))){ga=gh.replace(fm,ef);}else {fX.setAttribute(N,ga);};ga=dY+ga+f;i=ge.length;while(i-- ){ge[i]=ga+eQ(ge[i]);};gf=fc.test(gg)&&eC(fX.parentNode)||fX;gc=ge.join(ek);};if(gc){try{fG.apply(gb,gf.querySelectorAll(gc));return gb;}catch(gj){}finally{if(!gh){fX.removeAttribute(N);};};};};};return eG(gg.replace(fp,dN),fX,gb,gd);};function ey(){var gk=[];function gl(gm,gn){if(gk.push(gm+cV)>fM.cacheLength){delete gl[gk.shift()];};return (gl[gm+cV]=gn);};return gl;};function fy(go){go[fa]=true;return go;};function fk(gq){var gp=document.createElement(k);try{return !!gq(gp);}catch(e){return false;}finally{if(gp.parentNode){gp.parentNode.removeChild(gp);};gp=null;};};function fo(gt,gs){var gr=gt.split(dv),i=gt.length;while(i-- ){fM.attrHandle[gr[i]]=gs;};};function eY(a,b){var gv=b&&a,gu=gv&&a.nodeType===1&&b.nodeType===1&&(~b.sourceIndex||ft)-(~a.sourceIndex||ft);if(gu){return gu;};if(gv){while((gv=gv.nextSibling)){if(gv===b){return -1;};};};return a?1:-1;};function fE(gw){return function(gx){var name=gx.nodeName.toLowerCase();return name===l&&gx.type===gw;};};function er(gy){return function(gz){var name=gz.nodeName.toLowerCase();return (name===l||name===de)&&gz.type===gy;};};function fi(gA){return fy(function(gB){gB=+gB;return fy(function(gE,gC){var j,gD=gA([],gE.length,gB),i=gD.length;while(i-- ){if(gE[(j=gD[i])]){gE[j]=!(gC[j]=gE[j]);};};});});};function eC(gF){return gF&&typeof gF.getElementsByTagName!==eR&&gF;};eM=fL.support={};eB=fL.isXML=function(gG){var gH=gG&&(gG.ownerDocument||gG).documentElement;return gH?gH.nodeName!==eh:false;};eD=fL.setDocument=function(gI){var gK,gJ=gI?gI.ownerDocument||gI:eH,parent=gJ.defaultView;if(gJ===document||gJ.nodeType!==9||!gJ.documentElement){return document;};document=gJ;fK=gJ.documentElement;fq=!eB(gJ);if(parent&&parent!==parent.top){if(parent.addEventListener){parent.addEventListener(M,function(){eD();},false);}else if(parent.attachEvent){parent.attachEvent(dk,function(){eD();});};};eM.attributes=fk(function(gL){gL.className=C;return !gL.getAttribute(dP);});eM.getElementsByTagName=fk(function(gM){gM.appendChild(gJ.createComment(eb));return !gM.getElementsByTagName(z).length;});eM.getElementsByClassName=fz.test(gJ.getElementsByClassName)&&fk(function(gN){gN.innerHTML=cU;gN.firstChild.className=C;return gN.getElementsByClassName(C).length===2;});eM.getById=fk(function(gO){fK.appendChild(gO).id=fa;return !gJ.getElementsByName||!gJ.getElementsByName(fa).length;});if(eM.getById){fM.find[ej]=function(gP,gQ){if(typeof gQ.getElementById!==eR&&fq){var m=gQ.getElementById(gP);return m&&m.parentNode?[m]:[];};};fM.filter[ej]=function(gS){var gR=gS.replace(eu,fs);return function(gT){return gT.getAttribute(N)===gR;};};}else {delete fM.find[ej];fM.filter[ej]=function(gV){var gU=gV.replace(eu,fs);return function(gX){var gW=typeof gX.getAttributeNode!==eR&&gX.getAttributeNode(N);return gW&&gW.value===gU;};};};fM.find[dF]=eM.getElementsByTagName?function(gY,ha){if(typeof ha.getElementsByTagName!==eR){return ha.getElementsByTagName(gY);};}:function(he,hf){var hc,hb=[],i=0,hd=hf.getElementsByTagName(he);if(he===z){while((hc=hd[i++ ])){if(hc.nodeType===1){hb.push(hc);};};return hb;};return hd;};fM.find[g]=eM.getElementsByClassName&&function(hg,hh){if(typeof hh.getElementsByClassName!==eR&&fq){return hh.getElementsByClassName(hg);};};eq=[];fe=[];if((eM.qsa=fz.test(gJ.querySelectorAll))){fk(function(hi){hi.innerHTML=da;if(hi.querySelectorAll(I).length){fe.push(dW+eK+dR);};if(!hi.querySelectorAll(cM).length){fe.push(dm+eK+t+fd+dd);};if(!hi.querySelectorAll(ee).length){fe.push(ee);};});fk(function(hk){var hj=gJ.createElement(l);hj.setAttribute(cN,cW);hk.appendChild(hj).setAttribute(dn,dp);if(hk.querySelectorAll(W).length){fe.push(dn+eK+eo);};if(!hk.querySelectorAll(dO).length){fe.push(dO,h);};hk.querySelectorAll(dV);fe.push(dU);});};if((eM.matchesSelector=fz.test((ff=fK.webkitMatchesSelector||fK.mozMatchesSelector||fK.oMatchesSelector||fK.msMatchesSelector)))){fk(function(hl){eM.disconnectedMatch=ff.call(hl,k);ff.call(hl,S);eq.push(dx,fg);});};fe=fe.length&&new RegExp(fe.join(dv));eq=eq.length&&new RegExp(eq.join(dv));gK=fz.test(fK.compareDocumentPosition);eN=gK||fz.test(fK.contains)?function(a,b){var hm=a.nodeType===9?a.documentElement:a,hn=b&&b.parentNode;return a===hn||!!(hn&&hn.nodeType===1&&(hm.contains?hm.contains(hn):a.compareDocumentPosition&&a.compareDocumentPosition(hn)&16));}:function(a,b){if(b){while((b=b.parentNode)){if(b===a){return true;};};};return false;};eT=gK?function(a,b){if(a===b){eF=true;return 0;};var ho=!a.compareDocumentPosition-!b.compareDocumentPosition;if(ho){return ho;};ho=(a.ownerDocument||a)===(b.ownerDocument||b)?a.compareDocumentPosition(b):1;if(ho&1||(!eM.sortDetached&&b.compareDocumentPosition(a)===ho)){if(a===gJ||a.ownerDocument===eH&&eN(eH,a)){return -1;};if(b===gJ||b.ownerDocument===eH&&eN(eH,b)){return 1;};return eE?(eS.call(eE,a)-eS.call(eE,b)):0;};return ho&4?-1:1;}:function(a,b){if(a===b){eF=true;return 0;};var hp,i=0,hq=a.parentNode,hr=b.parentNode,hs=[a],ht=[b];if(!hq||!hr){return a===gJ?-1:b===gJ?1:hq?-1:hr?1:eE?(eS.call(eE,a)-eS.call(eE,b)):0;}else if(hq===hr){return eY(a,b);};hp=a;while((hp=hp.parentNode)){hs.unshift(hp);};hp=b;while((hp=hp.parentNode)){ht.unshift(hp);};while(hs[i]===ht[i]){i++ ;};return i?eY(hs[i],ht[i]):hs[i]===eH?-1:ht[i]===eH?1:0;};return gJ;};fL.matches=function(hu,hv){return fL(hu,null,null,hv);};fL.matchesSelector=function(hx,hw){if((hx.ownerDocument||hx)!==document){eD(hx);};hw=hw.replace(fj,E);if(eM.matchesSelector&&fq&&(!eq||!eq.test(hw))&&(!fe||!fe.test(hw))){try{var hy=ff.call(hx,hw);if(hy||eM.disconnectedMatch||hx.document&&hx.document.nodeType!==11){return hy;};}catch(e){};};return fL(hw,document,null,[hx]).length>0;};fL.contains=function(hA,hz){if((hA.ownerDocument||hA)!==document){eD(hA);};return eN(hA,hz);};fL.attr=function(hC,name){if((hC.ownerDocument||hC)!==document){eD(hC);};var hB=fM.attrHandle[name.toLowerCase()],hD=hB&&fA.call(fM.attrHandle,name.toLowerCase())?hB(hC,name,!fq):undefined;return hD!==undefined?hD:eM.attributes||!fq?hC.getAttribute(name):(hD=hC.getAttributeNode(name))&&hD.specified?hD.value:null;};fL.error=function(hE){throw new Error(em+hE);};fL.uniqueSort=function(hG){var hH,hF=[],j=0,i=0;eF=!eM.detectDuplicates;eE=!eM.sortStable&&hG.slice(0);hG.sort(eT);if(eF){while((hH=hG[i++ ])){if(hH===hG[i]){j=hF.push(i);};};while(j-- ){hG.splice(hF[j],1);};};eE=null;return hG;};ew=fL.getText=function(hK){var hI,hL=eb,i=0,hJ=hK.nodeType;if(!hJ){while((hI=hK[i++ ])){hL+=ew(hI);};}else if(hJ===1||hJ===9||hJ===11){if(typeof hK.textContent===T){return hK.textContent;}else {for(hK=hK.firstChild;hK;hK=hK.nextSibling){hL+=ew(hK);};};}else if(hJ===3||hJ===4){return hK.nodeValue;};return hL;};fM=fL.selectors={cacheLength:50,createPseudo:fy,match:fB,attrHandle:{},find:{},relative:{">":{dir:cI,first:true}," ":{dir:cI},"+":{dir:v,first:true},"~":{dir:v}},preFilter:{"ATTR":function(hM){hM[1]=hM[1].replace(eu,fs);hM[3]=(hM[4]||hM[5]||eb).replace(eu,fs);if(hM[2]===u){hM[3]=cV+hM[3]+cV;};return hM.slice(0,4);},"CHILD":function(hN){hN[1]=hN[1].toLowerCase();if(hN[1].slice(0,3)===o){if(!hN[3]){fL.error(hN[0]);};hN[4]=+(hN[4]?hN[5]+(hN[6]||1):2*(hN[3]===cY||hN[3]===dq));hN[5]=+((hN[7]+hN[8])||hN[3]===dq);}else if(hN[3]){fL.error(hN[0]);};return hN;},"PSEUDO":function(hP){var hQ,hO=!hP[5]&&hP[2];if(fB[ea].test(hP[0])){return null;};if(hP[3]&&hP[4]!==undefined){hP[2]=hP[4];}else if(hO&&fu.test(hO)&&(hQ=eV(hO,true))&&(hQ=hO.indexOf(dd,hO.length-hQ)-hO.length)){hP[0]=hP[0].slice(0,hQ);hP[2]=hO.slice(0,hQ);};return hP.slice(0,3);}},filter:{"TAG":function(hR){var hS=hR.replace(eu,fs).toLowerCase();return hR===z?function(){return true;}:function(hT){return hT.nodeName&&hT.nodeName.toLowerCase()===hS;};},"CLASS":function(hU){var hV=es[hU+cV];return hV||(hV=new RegExp(d+eK+dd+hU+cS+eK+dE))&&es(hU,function(hW){return hV.test(typeof hW.className===T&&hW.className||typeof hW.getAttribute!==eR&&hW.getAttribute(dr)||eb);});},"ATTR":function(name,hX,hY){return function(ia){var ib=fL.attr(ia,name);if(ib==null){return hX===dx;};if(!hX){return true;};ib+=eb;return hX===L?ib===hY:hX===dx?ib!==hY:hX===G?hY&&ib.indexOf(hY)===0:hX===du?hY&&ib.indexOf(hY)>-1:hX===R?hY&&ib.slice(-hY.length)===hY:hX===u?(cV+ib+cV).indexOf(hY)>-1:hX===dl?ib===hY||ib.slice(0,hY.length+1)===hY+c:false;};},"CHILD":function(ij,ic,ii,ik,ie){var ih=ij.slice(0,3)!==o,forward=ij.slice(-4)!==ei,ig=ic===dG;return ik===1&&ie===0?function(il){return !!il.parentNode;}:function(ir,iu,im){var iq,iv,io,iw,ip,is,ix=ih!==forward?Q:v,parent=ir.parentNode,name=ig&&ir.nodeName.toLowerCase(),it=!im&&!ig;if(parent){if(ih){while(ix){io=ir;while((io=io[ix])){if(ig?io.nodeName.toLowerCase()===name:io.nodeType===1){return false;};};is=ix=ij===y&&!is&&Q;};return true;};is=[forward?parent.firstChild:parent.lastChild];if(forward&&it){iv=parent[fa]||(parent[fa]={});iq=iv[ij]||[];ip=iq[0]===eW&&iq[1];iw=iq[0]===eW&&iq[2];io=ip&&parent.childNodes[ip];while((io= ++ip&&io&&io[ix]||(iw=ip=0)||is.pop())){if(io.nodeType===1&& ++iw&&io===ir){iv[ij]=[eW,ip,iw];break;};};}else if(it&&(iq=(ir[fa]||(ir[fa]={}))[ij])&&iq[0]===eW){iw=iq[1];}else {while((io= ++ip&&io&&io[ix]||(iw=ip=0)||is.pop())){if((ig?io.nodeName.toLowerCase()===name:io.nodeType===1)&& ++iw){if(it){(io[fa]||(io[fa]={}))[ij]=[eW,iw];};if(io===ir){break;};};};};iw-=ie;return iw===ik||(iw%ik===0&&iw/ik>=0);};};},"PSEUDO":function(iz,iA){var iy,iB=fM.pseudos[iz]||fM.setFilters[iz.toLowerCase()]||fL.error(cR+iz);if(iB[fa]){return iB(iA);};if(iB.length>1){iy=[iz,iz,eb,iA];return fM.setFilters.hasOwnProperty(iz.toLowerCase())?fy(function(iD,iC){var iE,iF=iB(iD,iA),i=iF.length;while(i-- ){iE=eS.call(iD,iF[i]);iD[iE]=!(iC[iE]=iF[i]);};}):function(iG){return iB(iG,0,iy);};};return iB;}},pseudos:{"not":fy(function(iI){var iH=[],iJ=[],iK=fC(iI.replace(fp,dN));return iK[fa]?fy(function(iP,iM,iQ,iL){var iN,iO=iK(iP,null,iL,[]),i=iP.length;while(i-- ){if((iN=iO[i])){iP[i]=!(iM[i]=iN);};};}):function(iS,iT,iR){iH[0]=iS;iK(iH,null,iR,iJ);return !iJ.pop();};}),"has":fy(function(iU){return function(iV){return fL(iU,iV).length>0;};}),"contains":fy(function(iW){return function(iX){return (iX.textContent||iX.innerText||ew(iX)).indexOf(iW)>-1;};}),"lang":fy(function(iY){if(!eX.test(iY||eb)){fL.error(A+iY);};iY=iY.replace(eu,fs).toLowerCase();return function(jb){var ja;do {if((ja=fq?jb.lang:jb.getAttribute(x)||jb.getAttribute(dt))){ja=ja.toLowerCase();return ja===iY||ja.indexOf(iY+c)===0;};}while((jb=jb.parentNode)&&jb.nodeType===1);return false;};}),"target":function(jd){var jc=window.location&&window.location.hash;return jc&&jc.slice(1)===jd.id;},"root":function(je){return je===fK;},"focus":function(jf){return jf===document.activeElement&&(!document.hasFocus||document.hasFocus())&&!!(jf.type||jf.href||~jf.tabIndex);},"enabled":function(jg){return jg.disabled===false;},"disabled":function(jh){return jh.disabled===true;},"checked":function(ji){var jj=ji.nodeName.toLowerCase();return (jj===l&&!!ji.checked)||(jj===dh&&!!ji.selected);},"selected":function(jk){if(jk.parentNode){jk.parentNode.selectedIndex;};return jk.selected===true;},"empty":function(jl){for(jl=jl.firstChild;jl;jl=jl.nextSibling){if(jl.nodeType<6){return false;};};return true;},"parent":function(jm){return !fM.pseudos[dK](jm);},"header":function(jn){return et.test(jn.nodeName);},"input":function(jo){return fl.test(jo.nodeName);},"button":function(jp){var name=jp.nodeName.toLowerCase();return name===l&&jp.type===de||name===de;},"text":function(jq){var jr;return jq.nodeName.toLowerCase()===l&&jq.type===O&&((jr=jq.getAttribute(cN))==null||jr.toLowerCase()===O);},"first":fi(function(){return [0];}),"last":fi(function(js,length){return [length-1];}),"eq":fi(function(jt,length,ju){return [ju<0?ju+length:ju];}),"even":fi(function(jv,length){var i=0;for(;i<length;i+=2){jv.push(i);};return jv;}),"odd":fi(function(jw,length){var i=1;for(;i<length;i+=2){jw.push(i);};return jw;}),"lt":fi(function(jx,length,jy){var i=jy<0?jy+length:jy;for(; --i>=0;){jx.push(i);};return jx;}),"gt":fi(function(jz,length,jA){var i=jA<0?jA+length:jA;for(; ++i<length;){jz.push(i);};return jz;})}};fM.pseudos[o]=fM.pseudos[dQ];for(i in {radio:true,checkbox:true,file:true,password:true,image:true}){fM.pseudos[i]=fE(i);};for(i in {submit:true,reset:true}){fM.pseudos[i]=er(i);};function fJ(){};fJ.prototype=fM.filters=fM.pseudos;fM.setFilters=new fJ();function eV(jE,jD){var jK,jJ,jB,jI,jF,jH,jG,jC=fr[jE+cV];if(jC){return jD?0:jC.slice(0);};jF=jE;jH=[];jG=fM.preFilter;while(jF){if(!jK||(jJ=fx.exec(jF))){if(jJ){jF=jF.slice(jJ[0].length)||jF;};jH.push((jB=[]));};jK=false;if((jJ=eL.exec(jF))){jK=jJ.shift();jB.push({value:jK,type:jJ[0].replace(fp,cV)});jF=jF.slice(jK.length);};for(jI in fM.filter){if((jJ=fB[jI].exec(jF))&&(!jG[jI]||(jJ=jG[jI](jJ)))){jK=jJ.shift();jB.push({value:jK,type:jI,matches:jJ});jF=jF.slice(jK.length);};};if(!jK){break;};};return jD?jF.length:jF?fL.error(jE):fr(jE,jH).slice(0);};function eQ(jL){var i=0,jM=jL.length,jN=eb;for(;i<jM;i++ ){jN+=jL[i].value;};return jN;};function eO(jO,jP,jQ){var jR=jP.dir,jT=jQ&&jR===cI,jS=eA++ ;return jP.first?function(jV,jW,jU){while((jV=jV[jR])){if(jV.nodeType===1||jT){return jO(jV,jW,jU);};};}:function(jY,kb,jX){var ka,kc,kd=[eW,jS];if(jX){while((jY=jY[jR])){if(jY.nodeType===1||jT){if(jO(jY,kb,jX)){return true;};};};}else {while((jY=jY[jR])){if(jY.nodeType===1||jT){kc=jY[fa]||(jY[fa]={});if((ka=kc[jR])&&ka[0]===eW&&ka[1]===jS){return (kd[2]=ka[2]);}else {kc[jR]=kd;if((kd[2]=jO(jY,kb,jX))){return true;};};};};};};};function eP(ke){return ke.length>1?function(kg,kh,kf){var i=ke.length;while(i-- ){if(!ke[i](kg,kh,kf)){return false;};};return true;}:ke[0];};function fw(kl,ki,kj){var i=0,kk=ki.length;for(;i<kk;i++ ){fL(kl,ki[i],kj);};return kj;};function ex(kp,kn,kq,ks,km){var ko,ku=[],i=0,kr=kp.length,kt=kn!=null;for(;i<kr;i++ ){if((ko=kp[i])){if(!kq||kq(ko,ks,km)){ku.push(ko);if(kt){kn.push(i);};};};};return ku;};function eJ(kz,ky,kx,kw,kv,kA){if(kw&&!kw[fa]){kw=eJ(kw);};if(kv&&!kv[fa]){kv=eJ(kv,kA);};return fy(function(kJ,kE,kK,kB){var kC,i,kG,kI=[],kM=[],kD=kE.length,kL=kJ||fw(ky||z,kK.nodeType?[kK]:kK,[]),kF=kz&&(kJ||!ky)?ex(kL,kI,kz,kK,kB):kL,kH=kx?kv||(kJ?kz:kD||kw)?[]:kE:kF;if(kx){kx(kF,kH,kK,kB);};if(kw){kC=ex(kH,kM);kw(kC,[],kK,kB);i=kC.length;while(i-- ){if((kG=kC[i])){kH[kM[i]]=!(kF[kM[i]]=kG);};};};if(kJ){if(kv||kz){if(kv){kC=[];i=kH.length;while(i-- ){if((kG=kH[i])){kC.push((kF[i]=kG));};};kv(null,(kH=[]),kC,kB);};i=kH.length;while(i-- ){if((kG=kH[i])&&(kC=kv?eS.call(kJ,kG):kI[i])>-1){kJ[kC]=!(kE[kC]=kG);};};};}else {kH=ex(kH===kE?kH.splice(kD,kH.length):kH);if(kv){kv(null,kE,kH,kB);}else {fG.apply(kE,kH);};};});};function fb(kS){var kN,kP,j,kQ=kS.length,kO=fM.relative[kS[0].type],kV=kO||fM.relative[cV],i=kO?1:0,kU=eO(function(kW){return kW===kN;},kV,true),kR=eO(function(kX){return eS.call(kN,kX)>-1;},kV,true),kT=[function(la,lb,kY){return (!kO&&(kY||lb!==ep))||((kN=lb).nodeType?kU(la,lb,kY):kR(la,lb,kY));}];for(;i<kQ;i++ ){if((kP=fM.relative[kS[i].type])){kT=[eO(eP(kT),kP)];}else {kP=fM.filter[kS[i].type].apply(null,kS[i].matches);if(kP[fa]){j= ++i;for(;j<kQ;j++ ){if(fM.relative[kS[j].type]){break;};};return eJ(i>1&&eP(kT),i>1&&eQ(kS.slice(0,i-1).concat({value:kS[i-2].type===cV?z:eb})).replace(fp,dN),kP,i<j&&fb(kS.slice(i,j)),j<kQ&&fb((kS=kS.slice(j))),j<kQ&&eQ(kS));};kT.push(kP);};};return eP(kT);};function eU(lg,ld){var lc=ld.length>0,le=lg.length>0,lf=function(lp,ls,lh,ll,lk){var ln,j,lt,li=0,i=df,lm=lp&&[],lo=[],lj=ep,lu=lp||le&&fM.find[dF](z,lk),lq=(eW+=lj==null?1:Math.random()||0.1),lr=lu.length;if(lk){ep=ls!==document&&ls;};for(;i!==lr&&(ln=lu[i])!=null;i++ ){if(le&&ln){j=0;while((lt=lg[j++ ])){if(lt(ln,ls,lh)){ll.push(ln);break;};};if(lk){eW=lq;};};if(lc){if((ln=!lt&&ln)){li-- ;};if(lp){lm.push(ln);};};};li+=i;if(lc&&i!==li){j=0;while((lt=ld[j++ ])){lt(lm,lo,ls,lh);};if(lp){if(li>0){while(i-- ){if(!(lm[i]||lo[i])){lo[i]=ez.call(ll);};};};lo=ex(lo);};fG.apply(ll,lo);if(lk&&!lp&&lo.length>0&&(li+ld.length)>1){fL.uniqueSort(ll);};};if(lk){eW=lq;ep=lj;};return lm;};return lc?fy(lf):lf;};fC=fL.compile=function(lw,lz){var i,ly=[],lv=[],lx=fI[lw+cV];if(!lx){if(!lz){lz=eV(lw);};i=lz.length;while(i-- ){lx=fb(lz[i]);if(lx[fa]){ly.push(lx);}else {lv.push(lx);};};lx=fI(lw,eU(lv,ly));lx.selector=lw;};return lx;};eG=fL.select=function(lC,lG,lB,lF){var i,lD,lE,lH,find,lA=typeof lC===db&&lC,lI=!lF&&eV((lC=lA.selector||lC));lB=lB||[];if(lI.length===1){lD=lI[0]=lI[0].slice(0);if(lD.length>2&&(lE=lD[0]).type===ej&&eM.getById&&lG.nodeType===9&&fq&&fM.relative[lD[1].type]){lG=(fM.find[ej](lE.matches[0].replace(eu,fs),lG)||[])[0];if(!lG){return lB;}else if(lA){lG=lG.parentNode;};lC=lC.slice(lD.shift().value.length);};i=fB[P].test(lC)?0:lD.length;while(i-- ){lE=lD[i];if(fM.relative[(lH=lE.type)]){break;};if((find=fM.find[lH])){if((lF=find(lE.matches[0].replace(eu,fs),fc.test(lD[0].type)&&eC(lG.parentNode)||lG))){lD.splice(i,1);lC=lF.length&&eQ(lD);if(!lC){fG.apply(lB,lF);return lB;};break;};};};};(lA||fC(lC,lI))(lF,lG,!fq,lB,fc.test(lC)&&eC(lG.parentNode)||lG);return lB;};eM.sortStable=fa.split(eb).sort(eT).join(eb)===fa;eM.detectDuplicates=!!eF;eD();eM.sortDetached=fk(function(lJ){return lJ.compareDocumentPosition(document.createElement(k))&1;});if(!fk(function(lK){lK.innerHTML=dD;return lK.firstChild.getAttribute(cJ)===cL;})){fo(q,function(lL,name,lM){if(!lM){return lL.getAttribute(name,name.toLowerCase()===cN?1:2);};});};if(!eM.attributes||!fk(function(lN){lN.innerHTML=dy;lN.firstChild.setAttribute(dH,eb);return lN.firstChild.getAttribute(dH)===eb;})){fo(dH,function(lO,name,lP){if(!lP&&lO.nodeName.toLowerCase()===l){return lO.defaultValue;};});};if(!fk(function(lQ){return lQ.getAttribute(s)==null;})){fo(fd,function(lS,name,lR){var lT;if(!lR){return lS[name]===true?name.toLowerCase():(lT=lS.getAttributeNode(name))&&lT.specified?lT.value:null;};});};qx.bom.Selector.query=function(lV,lU){return fL(lV,lU);};qx.bom.Selector.matches=function(lX,lW){return fL(lX,null,null,lW);};})(window);})();(function(){var a="display",b="",c="block",d="none",e="_getHeight",f="_getContentWidth",g="_getContentHeight",h="hidden",j="_getWidth",k="qx.module.Css",m="absolute";qx.Bootstrap.define(k,{statics:{_getHeight:function(p){var q=this[0];if(q){if(qx.dom.Node.isElement(q)){var n;if(p){var o={display:c,position:m,visibility:h};n=qx.module.Css.__fj(q,o,e,this);}else {n=qx.bom.element.Dimension.getHeight(q);};return n;}else if(qx.dom.Node.isDocument(q)){return qx.bom.Document.getHeight(qx.dom.Node.getWindow(q));}else if(qx.dom.Node.isWindow(q)){return qx.bom.Viewport.getHeight(q);};};return null;},_getWidth:function(t){var u=this[0];if(u){if(qx.dom.Node.isElement(u)){var r;if(t){var s={display:c,position:m,visibility:h};r=qx.module.Css.__fj(u,s,j,this);}else {r=qx.bom.element.Dimension.getWidth(u);};return r;}else if(qx.dom.Node.isDocument(u)){return qx.bom.Document.getWidth(qx.dom.Node.getWindow(u));}else if(qx.dom.Node.isWindow(u)){return qx.bom.Viewport.getWidth(u);};};return null;},_getContentHeight:function(w){var y=this[0];if(qx.dom.Node.isElement(y)){var x;if(w){var v={position:m,visibility:h,display:c};x=qx.module.Css.__fj(y,v,g,this);}else {x=qx.bom.element.Dimension.getContentHeight(y);};return x;};return null;},_getContentWidth:function(B){var z=this[0];if(qx.dom.Node.isElement(z)){var C;if(B){var A={position:m,visibility:h,display:c};C=qx.module.Css.__fj(z,A,f,this);}else {C=qx.bom.element.Dimension.getContentWidth(z);};return C;};return null;},__fh:{},__fi:function(G,D){var F=qx.module.Css.__fh;if(!F[G]){var H=D||document;var E=qxWeb(H.createElement(G)).appendTo(D.body);F[G]=E.getStyle(a);E.remove();};return F[G]||b;},__fj:function(L,I,J,O){var M={};for(var N in I){M[N]=L.style[N];L.style[N]=I[N];};var K=O[J]();for(var N in M){L.style[N]=M[N];};return K;},includeStylesheet:function(Q,P){qx.bom.Stylesheet.includeFile(Q,P);}},members:{getHeight:function(R){return this._getHeight(R);},getWidth:function(S){return this._getWidth(S);},getContentHeight:function(T){return this._getContentHeight(T);},getContentWidth:function(U){return this._getContentWidth(U);},show:function(){this._forEachElementWrapped(function(X){var Y=X.getStyle(a);var W=X[0].$$qPrevDisp;var V;if(Y==d){if(W&&W!=d){V=W;}else {var ba=qxWeb.getDocument(X[0]);V=qx.module.Css.__fi(X[0].tagName,ba);};X.setStyle(a,V);X[0].$$qPrevDisp=d;};});return this;},hide:function(){this._forEachElementWrapped(function(bb){var bc=bb.getStyle(a);if(bc!==d){bb[0].$$qPrevDisp=bc;bb.setStyle(a,d);};});return this;},getPosition:function(){var bd=this[0];if(qx.dom.Node.isElement(bd)){return qx.bom.element.Location.getPosition(bd);};return null;},getOffset:function(be){var bf=this[0];if(bf&&qx.dom.Node.isElement(bf)){return qx.bom.element.Location.get(bf,be);};return null;},setStyle:function(name,bg){if(/\w-\w/.test(name)){name=qx.lang.String.camelCase(name);};this._forEachElement(function(bh){qx.bom.element.Style.set(bh,name,bg);});return this;},getStyle:function(name){if(this[0]&&qx.dom.Node.isElement(this[0])){if(/\w-\w/.test(name)){name=qx.lang.String.camelCase(name);};return qx.bom.element.Style.get(this[0],name);};return null;},setStyles:function(bi){for(var name in bi){this.setStyle(name,bi[name]);};return this;},getStyles:function(bk){var bj={};for(var i=0;i<bk.length;i++ ){bj[bk[i]]=this.getStyle(bk[i]);};return bj;},addClass:function(name){this._forEachElement(function(bl){qx.bom.element.Class.add(bl,name);});return this;},addClasses:function(bm){this._forEachElement(function(bn){qx.bom.element.Class.addClasses(bn,bm);});return this;},removeClass:function(name){this._forEachElement(function(bo){qx.bom.element.Class.remove(bo,name);});return this;},removeClasses:function(bp){this._forEachElement(function(bq){qx.bom.element.Class.removeClasses(bq,bp);});return this;},hasClass:function(name){if(!this[0]||!qx.dom.Node.isElement(this[0])){return false;};return qx.bom.element.Class.has(this[0],name);},getClass:function(){if(!this[0]||!qx.dom.Node.isElement(this[0])){return b;};return qx.bom.element.Class.get(this[0]);},toggleClass:function(name){var br=qx.bom.element.Class;this._forEachElement(function(bs){br.has(bs,name)?br.remove(bs,name):br.add(bs,name);});return this;},toggleClasses:function(bt){for(var i=0,l=bt.length;i<l;i++ ){this.toggleClass(bt[i]);};return this;},replaceClass:function(bv,bu){this._forEachElement(function(bw){qx.bom.element.Class.replace(bw,bv,bu);});return this;}},defer:function(bx){qxWeb.$attachAll(this);qxWeb.$attach({"_getWidth":bx._getWidth,"_getHeight":bx._getHeight,"_getContentHeight":bx._getContentHeight,"_getContentWidth":bx._getContentWidth});}});})();(function(){var a="borderBottomWidth",b="scroll",c="qx.bom.element.Location",d="gecko",e="paddingLeft",f="borderRightWidth",g="auto",h="static",i="borderTopWidth",j="borderLeftWidth",k="marginBottom",l="marginTop",m="overflowY",n="marginLeft",o="border-box",p="padding",q="paddingBottom",r="paddingTop",s="marginRight",t="browser.quirksmode",u="engine.name",v="position",w="margin",x="paddingRight",y="BODY",z="overflowX",A="border";qx.Bootstrap.define(c,{statics:{__fk:function(C,B){return qx.bom.element.Style.get(C,B,qx.bom.element.Style.COMPUTED_MODE,false);},__fl:function(E,D){return parseInt(qx.bom.element.Style.get(E,D,qx.bom.element.Style.COMPUTED_MODE,false),10)||0;},__fm:function(G){var H=0,top=0;var F=qx.dom.Node.getWindow(G);H-=qx.bom.Viewport.getScrollLeft(F);top-=qx.bom.Viewport.getScrollTop(F);return {left:H,top:top};},__fn:qx.core.Environment.select(u,{"mshtml":function(K){var J=qx.dom.Node.getDocument(K);var I=J.body;var L=0;var top=0;L-=I.clientLeft+J.documentElement.clientLeft;top-=I.clientTop+J.documentElement.clientTop;if(!qx.core.Environment.get(t)){L+=this.__fl(I,j);top+=this.__fl(I,i);};return {left:L,top:top};},"webkit":function(O){var N=qx.dom.Node.getDocument(O);var M=N.body;var P=M.offsetLeft;var top=M.offsetTop;return {left:P,top:top};},"gecko":function(R){var Q=qx.dom.Node.getDocument(R).body;var S=Q.offsetLeft;var top=Q.offsetTop;if(qx.bom.element.BoxSizing.get(Q)!==o){S+=this.__fl(Q,j);top+=this.__fl(Q,i);};return {left:S,top:top};},"default":function(U){var T=qx.dom.Node.getDocument(U).body;var V=T.offsetLeft;var top=T.offsetTop;return {left:V,top:top};}}),__fo:function(W){var X=W.getBoundingClientRect();return {left:Math.round(X.left),top:Math.round(X.top)};},get:function(bd,be){if(bd.tagName==y){var location=this.__fp(bd);var bh=location.left;var top=location.top;}else {var Y=this.__fn(bd);var bc=this.__fo(bd);var scroll=this.__fm(bd);var bh=bc.left+Y.left-scroll.left;var top=bc.top+Y.top-scroll.top;};var ba=bh+bd.offsetWidth;var bb=top+bd.offsetHeight;if(be){if(be==p||be==b){var bg=qx.bom.element.Style.get(bd,z);if(bg==b||bg==g){ba+=bd.scrollWidth-bd.offsetWidth+this.__fl(bd,j)+this.__fl(bd,f);};var bf=qx.bom.element.Style.get(bd,m);if(bf==b||bf==g){bb+=bd.scrollHeight-bd.offsetHeight+this.__fl(bd,i)+this.__fl(bd,a);};};switch(be){case p:bh+=this.__fl(bd,e);top+=this.__fl(bd,r);ba-=this.__fl(bd,x);bb-=this.__fl(bd,q);case b:bh-=bd.scrollLeft;top-=bd.scrollTop;ba-=bd.scrollLeft;bb-=bd.scrollTop;case A:bh+=this.__fl(bd,j);top+=this.__fl(bd,i);ba-=this.__fl(bd,f);bb-=this.__fl(bd,a);break;case w:bh-=this.__fl(bd,n);top-=this.__fl(bd,l);ba+=this.__fl(bd,s);bb+=this.__fl(bd,k);break;};};return {left:bh,top:top,right:ba,bottom:bb};},__fp:function(bi){var top=bi.offsetTop;var bj=bi.offsetLeft;top+=this.__fl(bi,l);bj+=this.__fl(bi,n);if(qx.core.Environment.get(u)===d){top+=this.__fl(bi,j);bj+=this.__fl(bi,i);};return {left:bj,top:top};},getLeft:function(bk,bl){return this.get(bk,bl).left;},getTop:function(bm,bn){return this.get(bm,bn).top;},getRight:function(bo,bp){return this.get(bo,bp).right;},getBottom:function(bq,br){return this.get(bq,br).bottom;},getRelative:function(bv,bu,bt,bs){var bx=this.get(bv,bt);var bw=this.get(bu,bs);return {left:bx.left-bw.left,top:bx.top-bw.top,right:bx.right-bw.right,bottom:bx.bottom-bw.bottom};},getPosition:function(by){return this.getRelative(by,this.getOffsetParent(by));},getOffsetParent:function(bB){var bA=bB.offsetParent||document.body;var bz=qx.bom.element.Style;while(bA&&(!/^body|html$/i.test(bA.tagName)&&bz.get(bA,v)===h)){bA=bA.offsetParent;};return bA;}}});})();(function(){var a='',b="g",c="(^|\\s)",d='function',e="(\\s|$)",f="",g="\\b|\\b",h="qx.bom.element.Class",j='SVGAnimatedString',k="html.classlist",m="default",n=" ",o='object',p="$2",q="native",r="\\b",s='undefined';qx.Bootstrap.define(h,{statics:{__fq:/\s+/g,__fr:/^\s+|\s+$/g,add:{"native":function(t,name){if(name.length>0){t.classList.add(name);};return name;},"default":function(u,name){if(!this.has(u,name)){u.className+=(u.className?n:f)+name;};return name;}}[qx.core.Environment.get(k)?q:m],addClasses:{"native":function(w,v){for(var i=0;i<v.length;i++ ){if(v[i].length>0){w.classList.add(v[i]);};};return w.className;},"default":function(y,A){var z={};var B;var x=y.className;if(x){B=x.split(this.__fq);for(var i=0,l=B.length;i<l;i++ ){z[B[i]]=true;};for(var i=0,l=A.length;i<l;i++ ){if(!z[A[i]]){B.push(A[i]);};};}else {B=A;};return y.className=B.join(n);}}[qx.core.Environment.get(k)?q:m],get:function(D){var C=D.className;if(typeof C.split!==d){if(typeof C===o){if(qx.Bootstrap.getClass(C)==j){C=C.baseVal;}else {{};C=a;};};if(typeof C===s){{};C=a;};};return C;},has:{"native":function(E,name){return E.classList.contains(name);},"default":function(G,name){var F=new RegExp(c+name+e);return F.test(G.className);}}[qx.core.Environment.get(k)?q:m],remove:{"native":function(H,name){H.classList.remove(name);return name;},"default":function(J,name){var I=new RegExp(c+name+e);J.className=J.className.replace(I,p);return name;}}[qx.core.Environment.get(k)?q:m],removeClasses:{"native":function(L,K){for(var i=0;i<K.length;i++ ){L.classList.remove(K[i]);};return L.className;},"default":function(O,M){var N=new RegExp(r+M.join(g)+r,b);return O.className=O.className.replace(N,f).replace(this.__fr,f).replace(this.__fq,n);}}[qx.core.Environment.get(k)?q:m],replace:function(R,Q,P){if(!this.has(R,Q)){return f;};this.remove(R,Q);return this.add(R,P);},toggle:{"native":function(T,name,S){if(S===undefined){T.classList.toggle(name);}else {S?this.add(T,name):this.remove(T,name);};return name;},"default":function(V,name,U){if(U==null){U=!this.has(V,name);};U?this.add(V,name):this.remove(V,name);return name;}}[qx.core.Environment.get(k)?q:m]}});})();(function(){var a="ipod",b="pc",c="ps3",d=")",e="device.type",f="psp",g="wii",h="xbox",i="\.",j="iemobile",k="ipad",l="ds",m="(",n="mobile",o="tablet",p="ontouchstart",q="g",r="iphone",s="|",t="qx.bom.client.Device",u="desktop",v="device.name",w="device.touch",x="undefined",y="device.pixelRatio";qx.Bootstrap.define(t,{statics:{__cv:{"Windows Phone":j,"iPod":a,"iPad":k,"iPhone":r,"PSP":f,"PLAYSTATION 3":c,"Nintendo Wii":g,"Nintendo DS":l,"XBOX":h,"Xbox":h},getName:function(){var B=[];for(var A in qx.bom.client.Device.__cv){B.push(A);};var C=new RegExp(m+B.join(s).replace(/\./g,i)+d,q);var z=C.exec(navigator.userAgent);if(z&&z[1]){return qx.bom.client.Device.__cv[z[1]];};return b;},getType:function(){return qx.bom.client.Device.detectDeviceType(navigator.userAgent);},detectDeviceType:function(D){if(qx.bom.client.Device.detectTabletDevice(D)){return o;}else if(qx.bom.client.Device.detectMobileDevice(D)){return n;};return u;},detectMobileDevice:function(E){return /android.+mobile|ip(hone|od)|bada\/|blackberry|BB10|maemo|opera m(ob|in)i|fennec|NetFront|phone|psp|symbian|IEMobile|windows (ce|phone)|xda/i.test(E);},detectTabletDevice:function(G){var H=(/MSIE 10/i.test(G))&&(/ARM/i.test(G))&&!(/windows phone/i.test(G));var F=(!(/android.+mobile|Tablet PC/i.test(G))&&(/Android|ipad|tablet|playbook|silk|kindle|psp/i.test(G)));return H||F;},getDevicePixelRatio:function(){if(typeof window.devicePixelRatio!==x){return window.devicePixelRatio;};return 1;},getTouch:function(){return ((p in window)||window.navigator.maxTouchPoints>0||window.navigator.msMaxTouchPoints>0);}},defer:function(I){qx.core.Environment.add(v,I.getName);qx.core.Environment.add(w,I.getTouch);qx.core.Environment.add(e,I.getType);qx.core.Environment.add(y,I.getDevicePixelRatio);}});})();(function(){var a="mshtml",b="function",c="event.mouseevent",d="pointerEnabled",e="onhashchange",f="event.help",g="mousewheel",h="event.customevent",j="event.mousewheel",k="event.touch",l="wheel",m="DOMMouseScroll",n="msPointerEnabled",o="event.hashchange",p="onhelp",q="documentMode",r="qx.bom.client.Event",s="ontouchstart",t="foo",u="event.mspointer",v="event.dispatchevent";qx.Bootstrap.define(r,{statics:{getTouch:function(){return (s in window);},getMsPointer:function(){if(d in window.navigator){return window.navigator.pointerEnabled;}else if(n in window.navigator){return window.navigator.msPointerEnabled;};return false;},getHelp:function(){return (p in document);},getHashChange:function(){var w=qx.bom.client.Engine.getName();var x=e in window;return (w!==a&&x)||(w===a&&q in document&&document.documentMode>=8&&x);},getDispatchEvent:function(){return typeof document.dispatchEvent==b;},getCustomEvent:function(){if(!window.CustomEvent){return false;};try{new window.CustomEvent(t);return true;}catch(y){return false;};},getMouseEvent:function(){if(!window.MouseEvent){return false;};try{new window.MouseEvent(t);return true;}catch(z){return false;};},getMouseWheel:function(A){if(!A){A=window;};var D=[A,A.document,A.document.body];var C=A;var B=m;for(var i=0;i<D.length;i++ ){if(qx.bom.Event.supportsEvent(D[i],l)){B=l;C=D[i];break;};if(qx.bom.Event.supportsEvent(D[i],g)){B=g;C=D[i];break;};};return {type:B,target:C};}},defer:function(E){qx.core.Environment.add(k,E.getTouch);qx.core.Environment.add(c,E.getMouseEvent);qx.core.Environment.add(v,E.getDispatchEvent);qx.core.Environment.add(h,E.getCustomEvent);qx.core.Environment.add(u,E.getMsPointer);qx.core.Environment.add(f,E.getHelp);qx.core.Environment.add(o,E.getHashChange);qx.core.Environment.add(j,E.getMouseWheel);}});})();(function(){var a="engine.name",b="event.mspointer",c="device.type",d="env",e="engine.version",f="qx.module.Environment",g="browser.version",h="event.touch",i="device.name",j="browser.quirksmode",k="browser.name",l="browser.documentmode";qx.Bootstrap.define(f,{statics:{get:function(m){return qx.core.Environment.get(m);},add:function(n,o){qx.core.Environment.add(n,o);return this;}},defer:function(p){qx.core.Environment.get(k);qx.core.Environment.get(g);qx.core.Environment.get(j);qx.core.Environment.get(l);qx.core.Environment.get(a);qx.core.Environment.get(e);qx.core.Environment.get(i);qx.core.Environment.get(c);qx.core.Environment.get(h);qx.core.Environment.get(b);qxWeb.$attachAll(this,d);}});})();(function(){var a="qx.module.Polyfill";qx.Bootstrap.define(a,{});})();(function(){var a="mshtml",b="engine.name",c="complete",d="Array",f="pointerout",g="pointerover",h="string",n="load",o="left",p="qx.module.Event",q="undefined",r="DOMContentLoaded",s="browser.documentmode",t="*";qx.Bootstrap.define(p,{statics:{__fs:{},__ft:{on:{},off:{}},__cX:false,ready:function(u){if(document.readyState===c){window.setTimeout(u,1);return;};var v=function(){qx.module.Event.__cX=true;u();};qxWeb(window).on(n,v);var w=function(){qxWeb(window).off(n,v);u();};if(qxWeb.env.get(b)!==a||qxWeb.env.get(s)>8){qx.bom.Event.addNativeListener(document,r,w);}else {var z=function(){if(qx.module.Event.__cX){return;};try{document.documentElement.doScroll(o);if(document.body){w();};}catch(A){window.setTimeout(z,100);};};z();};},$registerEventNormalization:function(E,B){if(!qx.lang.Type.isArray(E)){E=[E];};var C=qx.module.Event.__fs;for(var i=0,l=E.length;i<l;i++ ){var D=E[i];if(qx.lang.Type.isFunction(B)){if(!C[D]){C[D]=[];};C[D].push(B);};};},$unregisterEventNormalization:function(I,F){if(!qx.lang.Type.isArray(I)){I=[I];};var G=qx.module.Event.__fs;for(var i=0,l=I.length;i<l;i++ ){var H=I[i];if(G[H]){qx.lang.Array.remove(G[H],F);};};},$getEventNormalizationRegistry:function(){return qx.module.Event.__fs;},$registerEventHook:function(O,L,K){if(!qx.lang.Type.isArray(O)){O=[O];};var M=qx.module.Event.__ft.on;for(var i=0,l=O.length;i<l;i++ ){var N=O[i];if(qx.lang.Type.isFunction(L)){if(!M[N]){M[N]=[];};M[N].push(L);};};if(!K){return;};var J=qx.module.Event.__ft.off;for(var i=0,l=O.length;i<l;i++ ){var N=O[i];if(qx.lang.Type.isFunction(K)){if(!J[N]){J[N]=[];};J[N].push(K);};};},$unregisterEventHook:function(U,R,Q){if(!qx.lang.Type.isArray(U)){U=[U];};var S=qx.module.Event.__ft.on;for(var i=0,l=U.length;i<l;i++ ){var T=U[i];if(S[T]){qx.lang.Array.remove(S[T],R);};};if(!Q){return;};var P=qx.module.Event.__ft.off;for(var i=0,l=U.length;i<l;i++ ){var T=U[i];if(P[T]){qx.lang.Array.remove(P[T],Q);};};},$getEventHookRegistry:function(){return qx.module.Event.__ft;}},members:{on:function(bd,bb,bc,W){for(var i=0;i<this.length;i++ ){var V=this[i];var Y=bc||qxWeb(V);var X=qx.module.Event.__ft.on;var be=X[t]||[];if(X[bd]){be=be.concat(X[bd]);};for(var j=0,m=be.length;j<m;j++ ){be[j](V,bd,bb,bc);};var ba=function(bf,event){var bh=qx.module.Event.__fs;var bg=bh[t]||[];if(bh[bd]){bg=bg.concat(bh[bd]);};for(var x=0,y=bg.length;x<y;x++ ){event=bg[x](event,bf,bd);};bb.apply(this,[event]);}.bind(Y,V);ba.original=bb;qx.bom.Event.addNativeListener(V,bd,ba,W);if(!V.$$emitter){V.$$emitter=new qx.event.Emitter();};V.$$lastlistenerId=V.$$emitter.on(bd,ba,Y);V.$$emitter.getEntryById(V.$$lastlistenerId).useCapture=!!W;if(!V.__fu){V.__fu={};};if(!V.__fu[bd]){V.__fu[bd]={};};V.__fu[bd][V.$$lastlistenerId]=ba;if(!bc){if(!V.__fv){V.__fv={};};V.__fv[V.$$lastlistenerId]=Y;};};return this;},off:function(bt,bo,bq,bj){var br=(bo===null&&bq===null);for(var j=0;j<this.length;j++ ){var bi=this[j];if(!bi.__fu){continue;};var bv=[];if(bt!==null){bv.push(bt);}else {for(var bm in bi.__fu){bv.push(bm);};};for(var i=0,l=bv.length;i<l;i++ ){for(var bu in bi.__fu[bv[i]]){var bs=bi.__fu[bv[i]][bu];if(br||bs==bo||bs.original==bo){var bl=typeof bi.__fv!==q&&bi.__fv[bu];var bw;if(!bq&&bl){bw=bi.__fv[bu];};var bp=bi.$$emitter.off(bv[i],bs,bw||bq);if(br||bs.original==bo){qx.bom.Event.removeNativeListener(bi,bv[i],bs,bj);};if(bp!==null){delete bi.__fu[bv[i]][bu];};if(bl){delete bi.__fv[bu];};};};var bk=qx.module.Event.__ft.off;var bn=bk[t]||[];if(bk[bt]){bn=bn.concat(bk[bt]);};for(var k=0,m=bn.length;k<m;k++ ){bn[k](bi,bt,bo,bq);};};};return this;},allOff:function(bx){return this.off(bx||null,null,null);},offById:function(bz){var by=this[0].$$emitter.getEntryById(bz);return this.off(by.name,by.listener.original,by.ctx,by.useCapture);},emit:function(bA,bB){for(var j=0;j<this.length;j++ ){var bC=this[j];if(bC.$$emitter){bC.$$emitter.emit(bA,bB);};};return this;},once:function(bE,bD,bG){var self=this;var bF=function(bH){self.off(bE,bF,bG);bD.call(this,bH);};this.on(bE,bF,bG);return this;},hasListener:function(bL,bJ,bK){if(!this[0]||!this[0].$$emitter||!this[0].$$emitter.getListeners()[bL]){return false;};if(bJ){var bM=this[0].$$emitter.getListeners()[bL];for(var i=0;i<bM.length;i++ ){var bI=false;if(bM[i].listener==bJ){bI=true;};if(bM[i].listener.original&&bM[i].listener.original==bJ){bI=true;};if(bI){if(bK!==undefined){if(bM[i].ctx===bK){return true;};}else {return true;};};};return false;};return this[0].$$emitter.getListeners()[bL].length>0;},copyEventsTo:function(bT){var bR=this.concat();var bS=bT.concat();for(var i=bR.length-1;i>=0;i-- ){var bO=bR[i].getElementsByTagName(t);for(var j=0;j<bO.length;j++ ){bR.push(bO[j]);};};for(var i=bS.length-1;i>=0;i-- ){var bO=bS[i].getElementsByTagName(t);for(var j=0;j<bO.length;j++ ){bS.push(bO[j]);};};bS.forEach(function(bU){bU.$$emitter=null;});for(var i=0;i<bR.length;i++ ){var bN=bR[i];if(!bN.$$emitter){continue;};var bP=bN.$$emitter.getListeners();for(var name in bP){for(var j=bP[name].length-1;j>=0;j-- ){var bQ=bP[name][j].listener;if(bQ.original){bQ=bQ.original;};qxWeb(bS[i]).on(name,bQ,bP[name][j].ctx);};};};},hover:function(bV,bW){this.on(g,bV,this);if(qx.lang.Type.isFunction(bW)){this.on(f,bW,this);};return this;},onMatchTarget:function(bY,ca,cc,cb){cb=cb!==undefined?cb:this;var bX=function(e){var cd=qxWeb(e.getTarget());var ce=typeof ca==h?this.find(ca):qxWeb(ca);if(cd.is(ca)){cc.call(cb,cd,qxWeb.object.clone(e));}else {for(var i=0,l=ce.length;i<l;i++ ){if(cd.isChildOf(qxWeb(ce[i]))){cc.call(cb,cd,qxWeb.object.clone(e));break;};};};};this.forEach(function(cf){var cg={target:ca,type:bY,listener:bX,callback:cc,context:cb};if(!cf.$$matchTargetInfo){cf.$$matchTargetInfo=[];};cf.$$matchTargetInfo.push(cg);});this.on(bY,bX);return this;},offMatchTarget:function(ch,ci,ck,cj){cj=cj!==undefined?cj:this;this.forEach(function(cl){if(cl.$$matchTargetInfo&&qxWeb.type.get(cl.$$matchTargetInfo)==d){var cm=cl.$$matchTargetInfo;for(var i=cm.length-1;i>=0;i-- ){var cn=cm[i];if(cn.type==ch&&cn.callback==ck&&cn.context==cj){this.off(ch,cn.listener);cm.splice(i,1);};};if(cm.length===0){cl.$$matchTargetInfo=null;};};},this);return this;},hasMatchListener:function(ct,co,cp,cq){cq=cq!==undefined?cq:this;for(var j=0,l=this.length;j<l;j++ ){var cs=this[j].$$matchTargetInfo||[];for(var i=cs.length-1;i>=0;i-- ){var cr=cs[i];if(cr.type==ct&&cr.callback==cp&&cr.target==co&&cr.context==cq){return true;};};};return false;}},defer:function(cu){qxWeb.$attachAll(this);qxWeb.$attachStatic({"$registerEventNormalization":cu.$registerEventNormalization,"$unregisterEventNormalization":cu.$unregisterEventNormalization,"$getEventNormalizationRegistry":cu.$getEventNormalizationRegistry,"$registerEventHook":cu.$registerEventHook,"$unregisterEventHook":cu.$unregisterEventHook,"$getEventHookRegistry":cu.$getEventHookRegistry});}});})();(function(){var a="qx.module.event.PointerHandler",b="pointerup",c="event.dispatchevent",d="gesturemove",e="pointerover",f="gesturebegin",g="pointerdown",h="pointermove",i="gesturefinish",j="qx.event.handler.Pointer",k="gesturecancel",l="pointercancel",m="pointerout";qx.Bootstrap.define(a,{statics:{TYPES:[h,e,m,g,b,l,f,d,i,k],register:function(o,n){if(!o.$$pointerHandler){if(!qx.core.Environment.get(c)){if(!o.$$emitter){o.$$emitter=new qx.event.Emitter();};};o.$$pointerHandler=new qx.event.handler.PointerCore(o,o.$$emitter);};},unregister:function(r){if(r.$$pointerHandler){if(r.$$pointerHandler.classname===j){return;};var p=r.$$emitter.getListeners();for(var q in p){if(qx.module.event.PointerHandler.TYPES.indexOf(q)!==-1){if(p[q].length>0){return;};};};r.$$pointerHandler.dispose();r.$$pointerHandler=undefined;};}},defer:function(s){qxWeb.$registerEventHook(s.TYPES,s.register,s.unregister);}});})();(function(){var a="qx.event.Emitter",b="*";qx.Bootstrap.define(a,{extend:Object,statics:{__fw:[]},members:{__fu:null,__fx:null,on:function(name,c,d){var e=qx.event.Emitter.__fw.length;this.__fy(name).push({listener:c,ctx:d,id:e,name:name});qx.event.Emitter.__fw.push({name:name,listener:c,ctx:d});return e;},once:function(name,f,g){var h=qx.event.Emitter.__fw.length;this.__fy(name).push({listener:f,ctx:g,once:true,id:h});qx.event.Emitter.__fw.push({name:name,listener:f,ctx:g});return h;},off:function(name,m,k){var l=this.__fy(name);for(var i=l.length-1;i>=0;i-- ){var n=l[i];if(n.listener==m&&n.ctx==k){l.splice(i,1);qx.event.Emitter.__fw[n.id]=null;return n.id;};};return null;},offById:function(p){var o=qx.event.Emitter.__fw[p];if(o){this.off(o.name,o.listener,o.ctx);};return null;},addListener:function(name,q,r){return this.on(name,q,r);},addListenerOnce:function(name,s,t){return this.once(name,s,t);},removeListener:function(name,u,v){this.off(name,u,v);},removeListenerById:function(w){this.offById(w);},emit:function(name,A){var x=this.__fy(name).concat();var y=[];for(var i=0;i<x.length;i++ ){var z=x[i];z.listener.call(z.ctx,A);if(z.once){y.push(z);};};y.forEach(function(B){var C=this.__fy(name);var D=C.indexOf(B);C.splice(D,1);}.bind(this));x=this.__fy(b);for(var i=x.length-1;i>=0;i-- ){var z=x[i];z.listener.call(z.ctx,A);};},getListeners:function(){return this.__fu;},getEntryById:function(F){for(var name in this.__fu){var E=this.__fu[name];for(var i=0,j=E.length;i<j;i++ ){if(E[i].id===F){return E[i];};};};},__fy:function(name){if(this.__fu==null){this.__fu={};};if(this.__fu[name]==null){this.__fu[name]=[];};return this.__fu[name];}}});})();(function(){var a="touchmove",b="os.name",c="mousedown",d="event.dispatchevent",e="MSPointerDown",f="gesturemove",g="pointerover",h="touch",k="mouseout",m="ms",n="Processed",o="pointercancel",p="pointerleave",q="touchstart",r="pointerenter",s="mouse",t="event.mspointer",u="mousemove",v="MSPointerCancel",w="gesturefinish",z="browser.documentmode",A="pointerup",B="touchend",C="osx",D="mouseover",E="$$qx",F="pointerdown",G="MSPointerUp",H="pointermove",I="MSPointerOver",J="gecko",K="mshtml",L="engine.name",M="mouseup",N="touchcancel",O="contextmenu",P="gesturecancel",Q="MSPointerMove",R="MSPointerOut",S="gesturebegin",T="qx.event.handler.PointerCore",U=".",V="device.touch",W="pointerout";qx.Bootstrap.define(T,{extend:Object,statics:{MOUSE_TO_POINTER_MAPPING:{mousedown:F,mouseup:A,mousemove:H,mouseout:W,mouseover:g},TOUCH_TO_POINTER_MAPPING:{touchstart:F,touchend:A,touchmove:H,touchcancel:o},MSPOINTER_TO_POINTER_MAPPING:{MSPointerDown:F,MSPointerMove:H,MSPointerUp:A,MSPointerCancel:o,MSPointerLeave:p,MSPointerEnter:r,MSPointerOver:g,MSPointerOut:W},POINTER_TO_GESTURE_MAPPING:{pointerdown:S,pointerup:w,pointercancel:P,pointermove:f},LEFT_BUTTON:(qx.core.Environment.get(L)==K&&qx.core.Environment.get(z)<=8)?1:0,SIM_MOUSE_DISTANCE:25,SIM_MOUSE_DELAY:2500,__fz:null},construct:function(ba,bb){this.__fA=ba;this.__fB=bb;this.__fC=[];this.__fD=[];this.__fE=[];this._processedFlag=E+this.classname.substr(this.classname.lastIndexOf(U)+1)+n;var Y=qx.core.Environment.get(L);var X=parseInt(qx.core.Environment.get(z),10);if(Y==K&&X==10){this.__fC=[e,Q,G,v,I,R,F,H,A,o,g,W];this._initPointerObserver();}else {if(qx.core.Environment.get(t)){this.__fF=true;};this.__fC=[F,H,A,o,g,W];this._initPointerObserver();};if(!qx.core.Environment.get(t)){if(qx.core.Environment.get(V)){this.__fC=[q,B,a,N];this._initObserver(this._onTouchEvent);};this.__fC=[c,M,u,D,k,O];this._initObserver(this._onMouseEvent);};},members:{__fA:null,__fB:null,__fC:null,__fF:false,__fG:null,__fH:0,__fD:null,__fI:null,__fE:null,_processedFlag:null,_initPointerObserver:function(){this._initObserver(this._onPointerEvent);},_initObserver:function(bc,bd){this.__fG=qx.lang.Function.listener(bc,this);this.__fC.forEach(function(be){if(bd&&qx.dom.Node.isDocument(this.__fA)){if(!this.__fA.$$emitter){this.__fA.$$emitter=new qx.event.Emitter();};this.__fA.$$emitter.on(be,this.__fG);}else {qx.bom.Event.addNativeListener(this.__fA,be,this.__fG);};}.bind(this));},_onPointerEvent:function(bh){if(!qx.core.Environment.get(t)||(qx.core.Environment.get(z)===10&&bh.type.toLowerCase().indexOf(m)==-1)){return;};if(!this.__fF){bh.stopPropagation();};var bf=qx.event.handler.PointerCore.MSPOINTER_TO_POINTER_MAPPING[bh.type]||bh.type;var bi=qx.bom.Event.getTarget(bh);var bg=new qx.event.type.dom.Pointer(bf,bh);this._fireEvent(bg,bf,bi);},_onTouchEvent:function(bl){if(bl[this._processedFlag]){return;};bl[this._processedFlag]=true;var bm=qx.event.handler.PointerCore.TOUCH_TO_POINTER_MAPPING[bl.type];var bo=bl.changedTouches;this._determineActiveTouches(bl.type,bo);if(bl.touches.length<this.__fE.length){for(var i=this.__fE.length-1;i>=0;i-- ){var bq=new qx.event.type.dom.Pointer(o,bl,{identifier:this.__fE[i].identifier,target:bl.target,pointerType:h,pointerId:this.__fE[i].identifier+2});this._fireEvent(bq,o,bl.target);};this.__fI=null;this.__fE=[];return;};if(bl.type==q&&this.__fI===null){this.__fI=bo[0].identifier;};for(var i=0,l=bo.length;i<l;i++ ){var bp=bo[i];var bn=bl.view.document.elementFromPoint(bp.clientX,bp.clientY)||bl.target;var bk={clientX:bp.clientX,clientY:bp.clientY,pageX:bp.pageX,pageY:bp.pageY,identifier:bp.identifier,screenX:bp.screenX,screenY:bp.screenY,target:bn,pointerType:h,pointerId:bp.identifier+2};if(bl.type==q){var bj=new qx.event.type.dom.Pointer(g,bl,bk);this._fireEvent(bj,g,bk.target);};if(bp.identifier==this.__fI){bk.isPrimary=true;bk.button=0;bk.buttons=1;qx.event.handler.PointerCore.__fz={"x":bp.clientX,"y":bp.clientY,"time":new Date().getTime()};};var br=new qx.event.type.dom.Pointer(bm,bl,bk);this._fireEvent(br,bm,bk.target);if(bl.type==B||bl.type==N){var bs=new qx.event.type.dom.Pointer(W,bl,bk);this._fireEvent(bs,W,bl.target);if(this.__fI==bp.identifier){this.__fI=null;};};};},_onMouseEvent:function(bt){if(bt[this._processedFlag]){return;};bt[this._processedFlag]=true;if(this._isSimulatedMouseEvent(bt.clientX,bt.clientY)){return;};if(bt.type==c){this.__fD[bt.which]=1;}else if(bt.type==M){if(qx.core.Environment.get(b)==C&&qx.core.Environment.get(L)==J){if(this.__fD[bt.which]!=1&&bt.ctrlKey){this.__fD[1]=0;};};this.__fD[bt.which]=0;};var bv=qx.event.handler.PointerCore.MOUSE_TO_POINTER_MAPPING[bt.type];var bu=qx.bom.Event.getTarget(bt);var bw=qx.lang.Array.sum(this.__fD);var bz={pointerType:s,pointerId:1};if(this.__fH!=bw&&bw!==0&&this.__fH!==0){var bx=new qx.event.type.dom.Pointer(H,bt,bz);this._fireEvent(bx,H,bu);};this.__fH=bw;if(bt.type==c&&bw>1){return;};if(bt.type==M&&bw>0){return;};if(bt.type==O){this.__fD[bt.which]=0;return;};var by=new qx.event.type.dom.Pointer(bv,bt,bz);this._fireEvent(by,bv,bu);},_determineActiveTouches:function(bD,bC){if(bD==q){for(var i=0;i<bC.length;i++ ){this.__fE.push(bC[i]);};}else if(bD==B||bD==N){var bA=[];for(var i=0;i<this.__fE.length;i++ ){var bB=true;for(var j=0;j<bC.length;j++ ){if(this.__fE[i].identifier==bC[j].identifier){bB=false;break;};};if(bB){bA.push(this.__fE[i]);};};this.__fE=bA;};},_isSimulatedMouseEvent:function(x,y){var bF=qx.event.handler.PointerCore.__fz;if(bF){var bG=new Date().getTime()-bF.time;var bE=qx.event.handler.PointerCore.SIM_MOUSE_DISTANCE;var bI=Math.abs(x-qx.event.handler.PointerCore.__fz.x);var bH=Math.abs(y-qx.event.handler.PointerCore.__fz.y);if(bG<qx.event.handler.PointerCore.SIM_MOUSE_DELAY){if(bI<bE||bH<bE){return true;};};};return false;},_stopObserver:function(){for(var i=0;i<this.__fC.length;i++ ){qx.bom.Event.removeNativeListener(this.__fA,this.__fC[i],this.__fG);};},_fireEvent:function(bK,bJ,bL){bL=bL||bK.target;bJ=bJ||bK.type;var bM;if((bK.pointerType!==s||bK.button<=qx.event.handler.PointerCore.LEFT_BUTTON)&&(bJ==F||bJ==A||bJ==H)){bM=new qx.event.type.dom.Pointer(qx.event.handler.PointerCore.POINTER_TO_GESTURE_MAPPING[bJ],bK);qx.event.type.dom.Pointer.normalize(bM);bM.srcElement=bL;};if(qx.core.Environment.get(d)){if(!this.__fF){bL.dispatchEvent(bK);};if(bM){bL.dispatchEvent(bM);};}else {bK.srcElement=bL;while(bL){if(bL.$$emitter){bK.currentTarget=bL;if(!bK._stopped){bL.$$emitter.emit(bJ,bK);};if(bM&&!bM._stopped){bM.currentTarget=bL;bL.$$emitter.emit(bM.type,bM);};};bL=bL.parentNode;};};},dispose:function(){this._stopObserver();this.__fA=this.__fB=null;}}});})();(function(){var a="qx.event.type.dom.Custom",b="UIEvents",c="function",d="event.customevent",e="object";qx.Bootstrap.define(a,{extend:Object,statics:{PROPERTIES:{bubbles:false,cancelable:true}},construct:function(f,g,h){this._type=f;this._event=this._createEvent();this._initEvent(g,h);this._event._original=g;this._event.preventDefault=function(){if(this._original.preventDefault){this._original.preventDefault();}else {try{this._original.returnValue=false;}catch(i){};};};if(this._event.stopPropagation){this._event._nativeStopPropagation=this._event.stopPropagation;};this._event.stopPropagation=function(){this._stopped=true;if(this._nativeStopPropagation){this._original.stopPropagation();this._nativeStopPropagation();}else {this._original.cancelBubble=true;};};return this._event;},members:{_type:null,_event:null,_createEvent:function(){var j;if(qx.core.Environment.get(d)){j=new window.CustomEvent(this._type);}else if(typeof document.createEvent==c){j=document.createEvent(b);}else if(typeof document.createEventObject==e){j={};j.type=this._type;};return j;},_initEvent:function(k,m){m=m||{};var l=qx.lang.Object.clone(qx.event.type.dom.Custom.PROPERTIES);for(var n in m){l[n]=m[n];};if(this._event.initEvent){this._event.initEvent(this._type,l.bubbles,l.cancelable);};for(var n in l){this._event[n]=l[n];};}}});})();(function(){var a="bubbles",b="event.mouseevent",c="getScreenLeft",d="getPointerType",e="touch",f="ctrlKey",g="altKey",h="gecko",j="view",k="os.name",m="button",n="string",o="relatedTarget",p="buttons",q="mouse",r="clientX",s="qx.event.type.dom.Pointer",t="UIEvents",u="ios",v="pageY",w="cancelable",x="screenX",y="shiftKey",z="",A="number",B="detail",C="toElement",D="fromElement",E="getViewportLeft",F="function",G="clientY",H="os.version",I="engine.name",J="undefined",K="getViewportTop",L="screenY",M="getScreenTop",N="pen",O="metaKey",P="pageX",Q="object",R="getDocumentTop",S="which",T="getDocumentLeft";qx.Bootstrap.define(s,{extend:qx.event.type.dom.Custom,statics:{MOUSE_PROPERTIES:[a,w,j,B,x,L,r,G,P,v,f,g,y,O,m,S,o,D,C],POINTER_PROPERTIES:{pointerId:1,width:0,height:0,pressure:0.5,tiltX:0,tiltY:0,pointerType:z,isPrimary:false},READONLY_PROPERTIES:[],BIND_METHODS:[d,E,K,T,R,c,M],getPointerType:function(){if(typeof this.pointerType==n){return this.pointerType;};if(typeof this.pointerType==A){if(this.pointerType==this.MSPOINTER_TYPE_MOUSE){return q;};if(this.pointerType==this.MSPOINTER_TYPE_PEN){return N;};if(this.pointerType==this.MSPOINTER_TYPE_TOUCH){return e;};};return z;},getViewportLeft:function(){return this.clientX;},getViewportTop:function(){return this.clientY;},getDocumentLeft:function(){if(this.pageX!==undefined){return this.pageX;}else {var U=qx.dom.Node.getWindow(this.srcElement);return this.clientX+qx.bom.Viewport.getScrollLeft(U);};},getDocumentTop:function(){if(this.pageY!==undefined){return this.pageY;}else {var V=qx.dom.Node.getWindow(this.srcElement);return this.clientY+qx.bom.Viewport.getScrollTop(V);};},getScreenLeft:function(){return this.screenX;},getScreenTop:function(){return this.screenY;},normalize:function(event){var W=qx.event.type.dom.Pointer.BIND_METHODS;for(var i=0,l=W.length;i<l;i++ ){if(typeof event[W[i]]!=F){event[W[i]]=qx.event.type.dom.Pointer[W[i]].bind(event);};};}},construct:function(X,Y,ba){return qx.event.type.dom.Custom.call(this,X,Y,ba);},members:{_createEvent:function(){var bb;if(qx.core.Environment.get(b)){bb=new window.MouseEvent(this._type);}else if(typeof document.createEvent==F){bb=document.createEvent(t);}else if(typeof document.createEventObject==Q){bb={};bb.type=this._type;};return bb;},_initEvent:function(bc,bd){bd=bd||{};var bg=this._event;var bh={};qx.event.type.dom.Pointer.normalize(bc);Object.keys(qx.event.type.dom.Pointer.POINTER_PROPERTIES).concat(qx.event.type.dom.Pointer.MOUSE_PROPERTIES).forEach(function(bi){if(typeof bd[bi]!==J){bh[bi]=bd[bi];}else if(typeof bc[bi]!==J){bh[bi]=bc[bi];}else if(typeof qx.event.type.dom.Pointer.POINTER_PROPERTIES[bi]!==J){bh[bi]=qx.event.type.dom.Pointer.POINTER_PROPERTIES[bi];};});var bf;switch(bc.which){case 1:bf=1;break;case 2:bf=4;break;case 3:bf=2;break;default:bf=0;};if(bf!==undefined){bh.buttons=bf;bh.pressure=bf?0.5:0;};if(bg.initMouseEvent){bg.initMouseEvent(this._type,bh.bubbles,bh.cancelable,bh.view,bh.detail,bh.screenX,bh.screenY,bh.clientX,bh.clientY,bh.ctrlKey,bh.altKey,bh.shiftKey,bh.metaKey,bh.button,bh.relatedTarget);}else if(bg.initUIEvent){bg.initUIEvent(this._type,bh.bubbles,bh.cancelable,bh.view,bh.detail);};for(var be in bh){if(bg[be]!==bh[be]&&qx.event.type.dom.Pointer.READONLY_PROPERTIES.indexOf(be)===-1){bg[be]=bh[be];};};switch(bg.pointerType){case bc.MSPOINTER_TYPE_MOUSE:bg.pointerType=q;break;case bc.MSPOINTER_TYPE_PEN:bg.pointerType=N;break;case bc.MSPOINTER_TYPE_TOUCH:bg.pointerType=e;break;};if(bg.pointerType==q){bg.isPrimary=true;};}},defer:function(bj){if(qx.core.Environment.get(I)==h){bj.READONLY_PROPERTIES.push(p);}else if(qx.core.Environment.get(k)==u&&parseFloat(qx.core.Environment.get(H))>=8){bj.READONLY_PROPERTIES=bj.READONLY_PROPERTIES.concat(bj.MOUSE_PROPERTIES);};}});})();(function(){var a="start",b="animationEnd",c="",d="none",e="browser.name",f="browser.version",g="qx.module.Animation",h="animationIteration",j="end",k="animationStart",l="ease-in",m="iteration",n="ease-out",o="ie",p="display";qx.Bootstrap.define(g,{events:{"animationStart":undefined,"animationIteration":undefined,"animationEnd":undefined},statics:{_fadeOut:{duration:700,timing:n,keep:100,keyFrames:{'0':{opacity:1},'100':{opacity:0,display:d}}},_fadeIn:{duration:700,timing:l,keep:100,keyFrames:{'0':{opacity:0},'100':{opacity:1}}},_animate:function(s,q,r){this._forEachElement(function(t,i){if(t.$$animation){t.$$animation.stop();};var u;if(r){u=qx.bom.element.Animation.animateReverse(t,s,q);}else {u=qx.bom.element.Animation.animate(t,s,q);};var self=this;if(i==0){u.on(a,function(){self.emit(k);},u);u.on(m,function(){self.emit(h);},u);};u.on(j,function(){for(var i=0;i<self.length;i++ ){if(self[i].$$animation){return;};};self.emit(b);},t);});}},members:{getAnimationHandles:function(){var v=[];for(var i=0;i<this.length;i++ ){v[i]=this[i].$$animation;};return v;},animate:function(x,w){qx.module.Animation._animate.bind(this)(x,w,false);return this;},animateReverse:function(z,y){qx.module.Animation._animate.bind(this)(z,y,true);return this;},play:function(){for(var i=0;i<this.length;i++ ){var A=this[i].$$animation;if(A){A.play();};};return this;},pause:function(){for(var i=0;i<this.length;i++ ){var B=this[i].$$animation;if(B){B.pause();};};return this;},stop:function(){for(var i=0;i<this.length;i++ ){var C=this[i].$$animation;if(C){C.stop();};};return this;},isPlaying:function(){for(var i=0;i<this.length;i++ ){var D=this[i].$$animation;if(D&&D.isPlaying()){return true;};};return false;},isEnded:function(){for(var i=0;i<this.length;i++ ){var E=this[i].$$animation;if(E&&!E.isEnded()){return false;};};return true;},fadeIn:function(F){this.setStyle(p,c);return this.animate(qx.module.Animation._fadeIn,F);},fadeOut:function(G){return this.animate(qx.module.Animation._fadeOut,G);}},defer:function(H){qxWeb.$attachAll(this);if(qxWeb.env.get(e)===o&&qxWeb.env.get(f)<=9){H._fadeIn.keyFrames[100].opacity=0.99;};}});})();(function(){var a="css.animation",b="translate",c="rotate",d="skew",e="scale",f="qx.bom.element.Animation";qx.Bootstrap.define(f,{statics:{animate:function(h,k,g){var j=qx.bom.element.Animation.__fJ(h,k.keyFrames);if(qx.core.Environment.get(a)&&j){return qx.bom.element.AnimationCss.animate(h,k,g);}else {return qx.bom.element.AnimationJs.animate(h,k,g);};},animateReverse:function(m,o,l){var n=qx.bom.element.Animation.__fJ(m,o.keyFrames);if(qx.core.Environment.get(a)&&n){return qx.bom.element.AnimationCss.animateReverse(m,o,l);}else {return qx.bom.element.AnimationJs.animateReverse(m,o,l);};},__fJ:function(p,t){var r=[];for(var v in t){var s=t[v];for(var u in s){if(r.indexOf(u)==-1){r.push(u);};};};var q=[e,c,d,b];for(var i=0;i<r.length;i++ ){var u=qx.lang.String.camelCase(r[i]);if(!(u in p.style)){if(q.indexOf(r[i])!=-1){continue;};if(qx.bom.Style.getPropertyName(u)){continue;};return false;};};return true;}}});})();(function(){var a="oAnimationStart",b="animationend",c="MSAnimationStart",d="oRequestAnimationFrame",f="AnimationFillMode",g="webkitAnimationStart",h="MSAnimationEnd",j="requestAnimationFrame",k="mozRequestAnimationFrame",l="webkitanimationend",m="css.animation.requestframe",n="AnimationPlayState",o="",p="MSAnimationIteration",q="animation",r="oAnimationEnd",s="@",t="animationiteration",u="webkitAnimationEnd",v="webkitRequestAnimationFrame",w=" name",x="qx.bom.client.CssAnimation",y="css.animation",z="oAnimationIteration",A="webkitanimationiteration",B="-keyframes",C="msRequestAnimationFrame",D="@keyframes",E="webkitAnimationIteration",F="animationstart",G="webkitanimationstart";qx.Bootstrap.define(x,{statics:{getSupport:function(){var name=qx.bom.client.CssAnimation.getName();if(name!=null){return {"name":name,"play-state":qx.bom.client.CssAnimation.getPlayState(),"start-event":qx.bom.client.CssAnimation.getAnimationStart(),"iteration-event":qx.bom.client.CssAnimation.getAnimationIteration(),"end-event":qx.bom.client.CssAnimation.getAnimationEnd(),"fill-mode":qx.bom.client.CssAnimation.getFillMode(),"keyframes":qx.bom.client.CssAnimation.getKeyFrames()};};return null;},getFillMode:function(){return qx.bom.Style.getPropertyName(f);},getPlayState:function(){return qx.bom.Style.getPropertyName(n);},getName:function(){return qx.bom.Style.getPropertyName(q);},getAnimationStart:function(){if(qx.bom.Event.supportsEvent(window,G)){return g;};var H={"msAnimation":c,"WebkitAnimation":g,"MozAnimation":F,"OAnimation":a,"animation":F};return H[this.getName()];},getAnimationIteration:function(){if(qx.bom.Event.supportsEvent(window,A)){return E;};var I={"msAnimation":p,"WebkitAnimation":E,"MozAnimation":t,"OAnimation":z,"animation":t};return I[this.getName()];},getAnimationEnd:function(){if(qx.bom.Event.supportsEvent(window,l)){return u;};var J={"msAnimation":h,"WebkitAnimation":u,"MozAnimation":b,"OAnimation":r,"animation":b};return J[this.getName()];},getKeyFrames:function(){var K=qx.bom.Style.VENDOR_PREFIXES;var N=[];for(var i=0;i<K.length;i++ ){var M=s+qx.bom.Style.getCssName(K[i])+B;N.push(M);};N.unshift(D);var L=qx.bom.Stylesheet.createElement();for(var i=0;i<N.length;i++ ){try{qx.bom.Stylesheet.addRule(L,N[i]+w,o);return N[i];}catch(e){};};return null;},getRequestAnimationFrame:function(){var O=[j,C,v,k,d];for(var i=0;i<O.length;i++ ){if(window[O[i]]!=undefined){return O[i];};};return null;}},defer:function(P){qx.core.Environment.add(y,P.getSupport);qx.core.Environment.add(m,P.getRequestAnimationFrame);}});})();(function(){var a="fill-mode",b="os.name",c="repeat",d="os.version",f="timing",g="start",h="end",i="Anni",j="alternate",k="keep",l="visibilitychange",m=":",n="ios",o="} ",p="name",q="iteration-event",r="",s="origin",t="forwards",u="start-event",v="iteration",w="end-event",x="css.animation",y="ms ",z="% {",A=" ",B="linear",C=";",D="qx.bom.element.AnimationCss",E="keyframes";qx.Bootstrap.define(D,{statics:{__dn:null,__fK:i,__cl:0,__dk:{},__fL:{"scale":true,"rotate":true,"skew":true,"translate":true},__fM:qx.core.Environment.get(x),animateReverse:function(G,H,F){return this._animate(G,H,F,true);},animate:function(J,K,I){return this._animate(J,K,I,false);},_animate:function(L,S,R,N){this.__fR(S);{};var P=S.keep;if(P!=null&&(N||(S.alternate&&S.repeat%2==0))){P=100-P;};if(!this.__dn){this.__dn=qx.bom.Stylesheet.createElement();};var O=S.keyFrames;if(R==undefined){R=S.duration;};if(this.__fM!=null){var name=this.__fT(O,N);var M=name+A+R+y+S.timing+A+(S.delay?S.delay+y:r)+S.repeat+A+(S.alternate?j:r);qx.bom.Event.addNativeListener(L,this.__fM[u],this.__fN);qx.bom.Event.addNativeListener(L,this.__fM[q],this.__fO);qx.bom.Event.addNativeListener(L,this.__fM[w],this.__fP);{};L.style[qx.lang.String.camelCase(this.__fM[p])]=M;if(P&&P==100&&this.__fM[a]){L.style[this.__fM[a]]=t;};};var Q=new qx.bom.element.AnimationHandle();Q.desc=S;Q.el=L;Q.keep=P;L.$$animation=Q;if(S.origin!=null){qx.bom.element.Transform.setOrigin(L,S.origin);};if(this.__fM==null){window.setTimeout(function(){qx.bom.element.AnimationCss.__fP({target:L});},0);};return Q;},__fN:function(e){if(e.target.$$animation){e.target.$$animation.emit(g,e.target);};},__fO:function(e){if(e.target!=null&&e.target.$$animation!=null){e.target.$$animation.emit(v,e.target);};},__fP:function(e){var T=e.target;var U=T.$$animation;if(!U){return;};var W=U.desc;if(qx.bom.element.AnimationCss.__fM!=null){var V=qx.lang.String.camelCase(qx.bom.element.AnimationCss.__fM[p]);T.style[V]=r;qx.bom.Event.removeNativeListener(T,qx.bom.element.AnimationCss.__fM[p],qx.bom.element.AnimationCss.__fP);};if(W.origin!=null){qx.bom.element.Transform.setOrigin(T,r);};qx.bom.element.AnimationCss.__fQ(T,W.keyFrames[U.keep]);T.$$animation=null;U.el=null;U.ended=true;U.emit(h,T);},__fQ:function(X,Y){var bb;for(var ba in Y){if(ba in qx.bom.element.AnimationCss.__fL){if(!bb){bb={};};bb[ba]=Y[ba];}else {X.style[qx.lang.String.camelCase(ba)]=Y[ba];};};if(bb){qx.bom.element.Transform.transform(X,bb);};},__fR:function(bc){if(!bc.hasOwnProperty(j)){bc.alternate=false;};if(!bc.hasOwnProperty(k)){bc.keep=null;};if(!bc.hasOwnProperty(c)){bc.repeat=1;};if(!bc.hasOwnProperty(f)){bc.timing=B;};if(!bc.hasOwnProperty(s)){bc.origin=null;};},__fS:null,__fT:function(frames,be){var bh=r;for(var bl in frames){bh+=(be?-(bl-100):bl)+z;var bg=frames[bl];var bj;for(var bd in bg){if(bd in this.__fL){if(!bj){bj={};};bj[bd]=bg[bd];}else {var bk=qx.bom.Style.getPropertyName(bd);var bf=(bk!==null)?qx.bom.Style.getCssName(bk):r;bh+=(bf||bd)+m+bg[bd]+C;};};if(bj){bh+=qx.bom.element.Transform.getCss(bj);};bh+=o;};if(this.__dk[bh]){return this.__dk[bh];};var name=this.__fK+this.__cl++ ;var bi=this.__fM[E]+A+name;qx.bom.Stylesheet.addRule(this.__dn,bi,bh);this.__dk[bh]=name;return name;},__fU:function(){this.__cl=0;if(this.__dn){this.__dn.ownerNode.remove();this.__dn=null;this.__dk={};};}},defer:function(bm){if(qx.core.Environment.get(b)===n&&parseInt(qx.core.Environment.get(d))>=8){document.addEventListener(l,function(){if(!document.hidden){bm.__fU();};},false);};}});})();(function(){var a="css.animation",b="Element",c="",d="qx.bom.element.AnimationHandle",e="play-state",f="paused",g="running";qx.Bootstrap.define(d,{extend:qx.event.Emitter,construct:function(){var h=qx.core.Environment.get(a);this.__fV=h&&h[e];this.__fW=true;},events:{"start":b,"end":b,"iteration":b},members:{__fV:null,__fW:false,__fX:false,isPlaying:function(){return this.__fW;},isEnded:function(){return this.__fX;},isPaused:function(){return this.el.style[this.__fV]==f;},pause:function(){if(this.el){this.el.style[this.__fV]=f;this.el.$$animation.__fW=false;if(this.animationId&&qx.bom.element.AnimationJs){qx.bom.element.AnimationJs.pause(this);};};},play:function(){if(this.el){this.el.style[this.__fV]=g;this.el.$$animation.__fW=true;if(this.i!=undefined&&qx.bom.element.AnimationJs){qx.bom.element.AnimationJs.play(this);};};},stop:function(){if(this.el&&qx.core.Environment.get(a)&&!this.jsAnimation){this.el.style[this.__fV]=c;this.el.style[qx.core.Environment.get(a).name]=c;this.el.$$animation.__fW=false;this.el.$$animation.__fX=true;}else if(this.jsAnimation){this.stopped=true;qx.bom.element.AnimationJs.stop(this);};}}});})();(function(){var c="cm",d="mm",e="0",f="pt",g="pc",h="",k="%",l="em",m="qx.bom.element.AnimationJs",n="infinite",o="#",p="in",q="px",r="start",s="end",t="ex",u=";",v="undefined",w="iteration",y="string",z=":";qx.Bootstrap.define(m,{statics:{__fY:30,__ga:[k,p,c,d,l,t,f,g,q],__fL:{"scale":true,"rotate":true,"skew":true,"translate":true},animate:function(B,C,A){return this._animate(B,C,A,false);},animateReverse:function(E,F,D){return this._animate(E,F,D,true);},_animate:function(G,Q,P,I){if(G.$$animation){return G.$$animation;};Q=qx.lang.Object.clone(Q,true);if(P==undefined){P=Q.duration;};var L=Q.keyFrames;var J=this.__gj(L);var K=this.__gi(P,J);var N=parseInt(P/K,10);this.__gb(L,G);var O=this.__gd(N,K,J,L,P,Q.timing);var H=new qx.bom.element.AnimationHandle();H.jsAnimation=true;if(I){O.reverse();H.reverse=true;};H.desc=Q;H.el=G;H.delta=O;H.stepTime=K;H.steps=N;G.$$animation=H;H.i=0;H.initValues={};H.repeatSteps=this.__gg(N,Q.repeat);var M=Q.delay||0;var self=this;H.delayId=window.setTimeout(function(){H.delayId=null;self.play(H);},M);return H;},__gb:function(V,R){var Y={};for(var U in V){for(var name in V[U]){var S=qx.bom.Style.getPropertyName(name);if(S&&S!=name){var X=qx.bom.Style.getCssName(S);V[U][X]=V[U][name];delete V[U][name];name=X;};if(Y[name]==undefined){var W=V[U][name];if(typeof W==y){Y[name]=this.__ge(W);}else {Y[name]=h;};};};};for(var U in V){var T=V[U];for(var name in Y){if(T[name]==undefined){if(name in R.style){if(window.getComputedStyle){T[name]=getComputedStyle(R,null)[name];}else {T[name]=R.style[name];};}else {T[name]=R[name];};if(T[name]===h&&this.__ga.indexOf(Y[name])!=-1){T[name]=e+Y[name];};};};};},__gc:function(bb){bb=qx.lang.Object.clone(bb);var bc;for(var name in bb){if(name in this.__fL){if(!bc){bc={};};bc[name]=bb[name];delete bb[name];};};if(bc){var ba=qx.bom.element.Transform.getCss(bc).split(z);if(ba.length>1){bb[ba[0]]=ba[1].replace(u,h);};};return bb;},__gd:function(bw,bh,bo,bi,be,bq){var bp=new Array(bw);var bm=1;bp[0]=this.__gc(bi[0]);var bt=bi[0];var bj=bi[bo[bm]];var bf=Math.floor(bo[bm]/(bh/be*100));var bs=1;for(var i=1;i<bp.length;i++ ){if(i*bh/be*100>bo[bm]){bt=bj;bm++ ;bj=bi[bo[bm]];bf=Math.floor(bo[bm]/(bh/be*100))-bf;bs=1;};bp[i]={};var bd;for(var name in bj){var br=bj[name]+h;if(name in this.__fL){if(!bd){bd={};};if(qx.Bootstrap.isArray(bt[name])){if(!qx.Bootstrap.isArray(bj[name])){bj[name]=[bj[name]];};bd[name]=[];for(var j=0;j<bj[name].length;j++ ){var bu=bj[name][j]+h;var x=bs/bf;bd[name][j]=this.__gf(bu,bt[name],bq,x);};}else {var x=bs/bf;bd[name]=this.__gf(br,bt[name],bq,x);};}else if(br.charAt(0)==o){var bl=qx.util.ColorUtil.cssStringToRgb(bt[name]);var bk=qx.util.ColorUtil.cssStringToRgb(br);var bg=[];for(var j=0;j<bl.length;j++ ){var bv=bl[j]-bk[j];var x=bs/bf;var bn=qx.bom.AnimationFrame.calculateTiming(bq,x);bg[j]=parseInt(bl[j]-bv*bn,10);};bp[i][name]=qx.util.ColorUtil.rgbToHexString(bg);}else if(!isNaN(parseFloat(br))){var x=bs/bf;bp[i][name]=this.__gf(br,bt[name],bq,x);}else {bp[i][name]=bt[name]+h;};};if(bd){var bx=qx.bom.element.Transform.getCss(bd).split(z);if(bx.length>1){bp[i][bx[0]]=bx[1].replace(u,h);};};bs++ ;};bp[bp.length-1]=this.__gc(bi[100]);return bp;},__ge:function(by){return by.substring((parseFloat(by)+h).length,by.length);},__gf:function(bC,bB,bz,x){var bA=parseFloat(bC)-parseFloat(bB);return (parseFloat(bB)+bA*qx.bom.AnimationFrame.calculateTiming(bz,x))+this.__ge(bC);},play:function(bD){bD.emit(r,bD.el);var bE=window.setInterval(function(){bD.repeatSteps-- ;var bF=bD.delta[bD.i%bD.steps];if(bD.i===0){for(var name in bF){if(bD.initValues[name]===undefined){if(bD.el[name]!==undefined){bD.initValues[name]=bD.el[name];}else if(qx.bom.element.Style){bD.initValues[name]=qx.bom.element.Style.get(bD.el,qx.lang.String.camelCase(name));}else {bD.initValues[name]=bD.el.style[qx.lang.String.camelCase(name)];};};};};qx.bom.element.AnimationJs.__gh(bD.el,bF);bD.i++ ;if(bD.i%bD.steps==0){bD.emit(w,bD.el);if(bD.desc.alternate){bD.delta.reverse();};};if(bD.repeatSteps<0){qx.bom.element.AnimationJs.stop(bD);};},bD.stepTime);bD.animationId=bE;return bD;},pause:function(bG){window.clearInterval(bG.animationId);bG.animationId=null;return bG;},stop:function(bK){var bJ=bK.desc;var bH=bK.el;var bI=bK.initValues;if(bK.animationId){window.clearInterval(bK.animationId);};if(bK.delayId){window.clearTimeout(bK.delayId);};if(bH==undefined){return bK;};var bL=bJ.keep;if(bL!=undefined&&!bK.stopped){if(bK.reverse||(bJ.alternate&&bJ.repeat&&bJ.repeat%2==0)){bL=100-bL;};this.__gh(bH,bJ.keyFrames[bL]);}else {this.__gh(bH,bI);};bH.$$animation=null;bK.el=null;bK.ended=true;bK.animationId=null;bK.emit(s,bH);return bK;},__gg:function(bN,bM){if(bM==undefined){return bN;};if(bM==n){return Number.MAX_VALUE;};return bN*bM;},__gh:function(bP,bO){for(var bQ in bO){if(bO[bQ]===undefined){continue;};if(typeof bP.style[bQ]===v&&bQ in bP){bP[bQ]=bO[bQ];continue;};var name=qx.bom.Style.getPropertyName(bQ)||bQ;if(qx.bom.element.Style){qx.bom.element.Style.set(bP,name,bO[bQ]);}else {bP.style[name]=bO[bQ];};};},__gi:function(bT,bR){var bU=100;for(var i=0;i<bR.length-1;i++ ){bU=Math.min(bU,bR[i+1]-bR[i]);};var bS=bT*bU/100;while(bS>this.__fY){bS=bS/2;};return Math.round(bS);},__gj:function(bW){var bV=Object.keys(bW);for(var i=0;i<bV.length;i++ ){bV[i]=parseInt(bV[i],10);};bV.sort(function(a,b){return a-b;});return bV;}}});})();(function(){var a="css.transform.3d",b="backfaceVisibility",c="transformStyle",d="css.transform",e="transformOrigin",f="qx.bom.client.CssTransform",g="transform",h="perspective",i="perspectiveOrigin";qx.Bootstrap.define(f,{statics:{getSupport:function(){var name=qx.bom.client.CssTransform.getName();if(name!=null){return {"name":name,"style":qx.bom.client.CssTransform.getStyle(),"origin":qx.bom.client.CssTransform.getOrigin(),"3d":qx.bom.client.CssTransform.get3D(),"perspective":qx.bom.client.CssTransform.getPerspective(),"perspective-origin":qx.bom.client.CssTransform.getPerspectiveOrigin(),"backface-visibility":qx.bom.client.CssTransform.getBackFaceVisibility()};};return null;},getStyle:function(){return qx.bom.Style.getPropertyName(c);},getPerspective:function(){return qx.bom.Style.getPropertyName(h);},getPerspectiveOrigin:function(){return qx.bom.Style.getPropertyName(i);},getBackFaceVisibility:function(){return qx.bom.Style.getPropertyName(b);},getOrigin:function(){return qx.bom.Style.getPropertyName(e);},getName:function(){return qx.bom.Style.getPropertyName(g);},get3D:function(){return qx.bom.client.CssTransform.getPerspective()!=null;}},defer:function(j){qx.core.Environment.add(d,j.getSupport);qx.core.Environment.add(a,j.get3D);}});})();(function(){var a="backface-visibility",b="css.transform.3d",c=") ",d="px",e="scale",f="Z",g="X",h=", ",j="visible",k=":",l="3d",m="name",n="",o="origin",p="(",q="qx.bom.element.Transform",r="perspective",s="Y",t="css.transform",u="translate",v="perspective-origin",w="hidden",x=";",y=" ",z="style";qx.Bootstrap.define(q,{statics:{__gk:qx.core.Environment.get(t),transform:function(A,C){var D=this.getTransformValue(C);if(this.__gk!=null){var B=this.__gk[m];A.style[B]=D;};},translate:function(E,F){this.transform(E,{translate:F});},scale:function(G,H){this.transform(G,{scale:H});},rotate:function(I,J){this.transform(I,{rotate:J});},skew:function(K,L){this.transform(K,{skew:L});},getCss:function(N){var O=this.getTransformValue(N);if(this.__gk!=null){var M=this.__gk[m];return qx.bom.Style.getCssName(M)+k+O+x;};return n;},setOrigin:function(P,Q){if(this.__gk!=null){P.style[this.__gk[o]]=Q;};},getOrigin:function(R){if(this.__gk!=null){return R.style[this.__gk[o]];};return n;},setStyle:function(S,T){if(this.__gk!=null){S.style[this.__gk[z]]=T;};},getStyle:function(U){if(this.__gk!=null){return U.style[this.__gk[z]];};return n;},setPerspective:function(V,W){if(this.__gk!=null){V.style[this.__gk[r]]=W+d;};},getPerspective:function(X){if(this.__gk!=null){return X.style[this.__gk[r]];};return n;},setPerspectiveOrigin:function(Y,ba){if(this.__gk!=null){Y.style[this.__gk[v]]=ba;};},getPerspectiveOrigin:function(bb){if(this.__gk!=null){var bc=bb.style[this.__gk[v]];if(bc!=n){return bc;}else {var be=bb.style[this.__gk[v]+g];var bd=bb.style[this.__gk[v]+s];if(be!=n){return be+y+bd;};};};return n;},setBackfaceVisibility:function(bf,bg){if(this.__gk!=null){bf.style[this.__gk[a]]=bg?j:w;};},getBackfaceVisibility:function(bh){if(this.__gk!=null){return bh.style[this.__gk[a]]==j;};return true;},getTransformValue:function(bl){var bm=n;var bi=[u,e];for(var bj in bl){var bk=bl[bj];if(qx.Bootstrap.isArray(bk)){if(bk.length===3&&bi.indexOf(bj)>-1&&qx.core.Environment.get(b)){bm+=this._compute3dProperty(bj,bk);}else {bm+=this._computeAxisProperties(bj,bk);};}else {bm+=bj+p+bk+c;};};return bm.trim();},_compute3dProperty:function(bo,bn){var bp=n;bo+=l;for(var i=0;i<bn.length;i++ ){if(bn[i]==null){bn[i]=0;};};bp+=bo+p+bn.join(h)+c;return bp;},_computeAxisProperties:function(bq,br){var bt=n;var bs=[g,s,f];for(var i=0;i<br.length;i++ ){if(br[i]==null||(i==2&&!qx.core.Environment.get(b))){continue;};bt+=bq+bs[i]+p;bt+=br[i];bt+=c;};return bt;}}});})();(function(){var b="ease-in-out",c="Number",d="css.animation.requestframe",e="qx.bom.AnimationFrame",f="frame",g="end",h="linear",j="ease-in",k="ease-out";qx.Bootstrap.define(e,{extend:qx.event.Emitter,events:{"end":undefined,"frame":c},members:{__gl:false,startSequence:function(l){this.__gl=false;var m=+(new Date());var n=function(p){if(this.__gl){this.id=null;return;};if(p>=m+l){this.emit(g);this.id=null;}else {var o=Math.max(p-m,0);this.emit(f,o);this.id=qx.bom.AnimationFrame.request(n,this);};};this.id=qx.bom.AnimationFrame.request(n,this);},cancelSequence:function(){this.__gl=true;}},statics:{TIMEOUT:30,calculateTiming:function(q,x){if(q==j){var a=[3.1223e-7,0.0757,1.2646,-0.167,-0.4387,0.2654];}else if(q==k){var a=[-7.0198e-8,1.652,-0.551,-0.0458,0.1255,-0.1807];}else if(q==h){return x;}else if(q==b){var a=[2.482e-7,-0.2289,3.3466,-1.0857,-1.7354,0.7034];}else {var a=[-0.0021,0.2472,9.8054,-21.6869,17.7611,-5.1226];};var y=0;for(var i=0;i<a.length;i++ ){y+=a[i]*Math.pow(x,i);};return y;},request:function(r,t){var s=qx.core.Environment.get(d);var u=function(v){if(v<1e10){v=this.__gm+v;};v=v||+(new Date());r.call(t,v);};if(s){return window[s](u);}else {return window.setTimeout(function(){u();},qx.bom.AnimationFrame.TIMEOUT);};}},defer:function(w){w.__gm=window.performance&&performance.timing&&performance.timing.navigationStart;if(!w.__gm){w.__gm=Date.now();};}});})();(function(){var a="qx.util.DeferredCallManager",b="singleton";qx.Class.define(a,{extend:qx.core.Object,type:b,construct:function(){this.__gn={};this.__go=qx.lang.Function.bind(this.__gs,this);this.__gp=false;},members:{__gq:null,__gr:null,__gn:null,__gp:null,__go:null,schedule:function(d){if(this.__gq==null){this.__gq=window.setTimeout(this.__go,0);};var c=d.toHashCode();if(this.__gr&&this.__gr[c]){return;};this.__gn[c]=d;this.__gp=true;},cancel:function(f){var e=f.toHashCode();if(this.__gr&&this.__gr[e]){this.__gr[e]=null;return;};delete this.__gn[e];if(qx.lang.Object.isEmpty(this.__gn)&&this.__gq!=null){window.clearTimeout(this.__gq);this.__gq=null;};},__gs:qx.event.GlobalError.observeMethod(function(){this.__gq=null;while(this.__gp){this.__gr=qx.lang.Object.clone(this.__gn);this.__gn={};this.__gp=false;for(var h in this.__gr){var g=this.__gr[h];if(g){this.__gr[h]=null;g.call();};};};this.__gr=null;})},destruct:function(){if(this.__gq!=null){window.clearTimeout(this.__gq);};this.__go=this.__gn=null;}});})();(function(){var a="qx.util.DeferredCall";qx.Class.define(a,{extend:qx.core.Object,construct:function(b,c){qx.core.Object.call(this);this.__bF=b;this.__bH=c||null;this.__gt=qx.util.DeferredCallManager.getInstance();},members:{__bF:null,__bH:null,__gt:null,cancel:function(){this.__gt.cancel(this);},schedule:function(){this.__gt.schedule(this);},call:function(){{var d;};this.__bH?this.__bF.apply(this.__bH):this.__bF();}},destruct:function(){this.cancel();this.__bH=this.__bF=this.__gt=null;}});})();(function(){var a="Child is already in: ",b="text",c="qx.html.Element",d="|capture|",f="focus",g="blur",h="div",j="class",k="deactivate",m="css.userselect",n="animationEnd",o="capture",p="visible",q="Root elements could not be inserted into other ones.",r="Has no children!",s="|bubble|",t="releaseCapture",u="Could not move to same index!",v="element",w="",z="__gP",A="qxSelectable",B="tabIndex",C="off",D="on",E="qx.html.Iframe",F="activate",G="Has no parent to remove from.",H="mshtml",I="engine.name",J="none",K="Has no child: ",L="scroll",M=" ",N="hidden",O="Has no child at this position!",P="css.userselect.none",Q="Could not overwrite existing element!";qx.Class.define(c,{extend:qx.core.Object,construct:function(T,R,S){qx.core.Object.call(this);this.__gu=T||h;this.__gv=R||null;this.__gw=S||null;},statics:{DEBUG:false,_modified:{},_visibility:{},_scroll:{},_actions:[],__gx:{},__gy:null,__gz:null,_scheduleFlush:function(U){qx.html.Element.__hb.schedule();},flush:function(){var bh;{};var Y=this.__gA();var W=Y.getFocus();if(W&&this.__gC(W)){Y.blur(W);};var bo=Y.getActive();if(bo&&this.__gC(bo)){qx.bom.Element.deactivate(bo);};var bc=this.__gB();if(bc&&this.__gC(bc)){qx.bom.Element.releaseCapture(bc);};var bi=[];var bj=this._modified;for(var bg in bj){bh=bj[bg];if(bh.__gT()||bh.classname==E){if(bh.__gD&&qx.dom.Hierarchy.isRendered(bh.__gD)){bi.push(bh);}else {{};bh.__gS();};delete bj[bg];};};for(var i=0,l=bi.length;i<l;i++ ){bh=bi[i];{};bh.__gS();};var be=this._visibility;for(var bg in be){bh=be[bg];var bk=bh.__gD;if(!bk){delete be[bg];continue;};{};if(!bh.$$disposed){bk.style.display=bh.__gF?w:J;if((qx.core.Environment.get(I)==H)){if(!(document.documentMode>=8)){bk.style.visibility=bh.__gF?p:N;};};};delete be[bg];};var scroll=this._scroll;for(var bg in scroll){bh=scroll[bg];var X=bh.__gD;if(X&&X.offsetWidth){var bb=true;if(bh.__gI!=null){bh.__gD.scrollLeft=bh.__gI;delete bh.__gI;};if(bh.__gJ!=null){bh.__gD.scrollTop=bh.__gJ;delete bh.__gJ;};var bl=bh.__gG;if(bl!=null){var bf=bl.element.getDomElement();if(bf&&bf.offsetWidth){qx.bom.element.Scroll.intoViewX(bf,X,bl.align);delete bh.__gG;}else {bb=false;};};var bm=bh.__gH;if(bm!=null){var bf=bm.element.getDomElement();if(bf&&bf.offsetWidth){qx.bom.element.Scroll.intoViewY(bf,X,bm.align);delete bh.__gH;}else {bb=false;};};if(bb){delete scroll[bg];};};};var ba={"releaseCapture":1,"blur":1,"deactivate":1};for(var i=0;i<this._actions.length;i++ ){var bn=this._actions[i];var bk=bn.element.__gD;if(!bk||!ba[bn.type]&&!bn.element.__gT()){continue;};var bd=bn.args;bd.unshift(bk);qx.bom.Element[bn.type].apply(qx.bom.Element,bd);};this._actions=[];for(var bg in this.__gx){var V=this.__gx[bg];var X=V.element.__gD;if(X){qx.bom.Selection.set(X,V.start,V.end);delete this.__gx[bg];};};qx.event.handler.Appear.refresh();},__gA:function(){if(!this.__gy){var bp=qx.event.Registration.getManager(window);this.__gy=bp.getHandler(qx.event.handler.Focus);};return this.__gy;},__gB:function(){if(!this.__gz){var bq=qx.event.Registration.getManager(window);this.__gz=bq.getDispatcher(qx.event.dispatch.MouseCapture);};return this.__gz.getCaptureElement();},__gC:function(br){var bs=qx.core.ObjectRegistry.fromHashCode(br.$$element);return bs&&!bs.__gT();}},members:{__gu:null,__gD:null,__dd:false,__gE:true,__gF:true,__gG:null,__gH:null,__gI:null,__gJ:null,__gK:null,__gL:null,__gM:null,__gv:null,__gw:null,__gN:null,__gO:null,__gP:null,__gQ:null,__gR:null,_scheduleChildrenUpdate:function(){if(this.__gQ){return;};this.__gQ=true;qx.html.Element._modified[this.$$hash]=this;qx.html.Element._scheduleFlush(v);},_createDomElement:function(){return qx.dom.Element.create(this.__gu);},__gS:function(){{};var length;var bt=this.__gP;if(bt){length=bt.length;var bu;for(var i=0;i<length;i++ ){bu=bt[i];if(bu.__gF&&bu.__gE&&!bu.__gD){bu.__gS();};};};if(!this.__gD){this.__gD=this._createDomElement();this.__gD.$$element=this.$$hash;this._copyData(false);if(bt&&length>0){this._insertChildren();};}else {this._syncData();if(this.__gQ){this._syncChildren();};};delete this.__gQ;},_insertChildren:function(){var bv=this.__gP;var length=bv.length;var bx;if(length>2){var bw=document.createDocumentFragment();for(var i=0;i<length;i++ ){bx=bv[i];if(bx.__gD&&bx.__gE){bw.appendChild(bx.__gD);};};this.__gD.appendChild(bw);}else {var bw=this.__gD;for(var i=0;i<length;i++ ){bx=bv[i];if(bx.__gD&&bx.__gE){bw.appendChild(bx.__gD);};};};},_syncChildren:function(){var bH=qx.core.ObjectRegistry;var by=this.__gP;var bF=by.length;var bz;var bD;var bB=this.__gD;var bG=bB.childNodes;var bA=0;var bE;{var bC;};for(var i=bG.length-1;i>=0;i-- ){bE=bG[i];bD=bH.fromHashCode(bE.$$element);if(!bD||!bD.__gE||bD.__gR!==this){bB.removeChild(bE);{};};};for(var i=0;i<bF;i++ ){bz=by[i];if(bz.__gE){bD=bz.__gD;bE=bG[bA];if(!bD){continue;};if(bD!=bE){if(bE){bB.insertBefore(bD,bE);}else {bB.appendChild(bD);};{};};bA++ ;};};{};},_copyData:function(bJ){var bL=this.__gD;var bN=this.__gw;if(bN){var bK=qx.bom.element.Attribute;for(var bM in bN){bK.set(bL,bM,bN[bM]);};};var bN=this.__gv;if(bN){var bI=qx.bom.element.Style;if(bJ){bI.setStyles(bL,bN);}else {bI.setCss(bL,bI.compile(bN));};};var bN=this.__gN;if(bN){for(var bM in bN){this._applyProperty(bM,bN[bM]);};};var bN=this.__gO;if(bN){qx.event.Registration.getManager(bL).importListeners(bL,bN);delete this.__gO;};},_syncData:function(){var bS=this.__gD;var bR=qx.bom.element.Attribute;var bP=qx.bom.element.Style;var bQ=this.__gL;if(bQ){var bV=this.__gw;if(bV){var bT;for(var bU in bQ){bT=bV[bU];if(bT!==undefined){bR.set(bS,bU,bT);}else {bR.reset(bS,bU);};};};this.__gL=null;};var bQ=this.__gK;if(bQ){var bV=this.__gv;if(bV){var bO={};for(var bU in bQ){bO[bU]=bV[bU];};bP.setStyles(bS,bO);};this.__gK=null;};var bQ=this.__gM;if(bQ){var bV=this.__gN;if(bV){var bT;for(var bU in bQ){this._applyProperty(bU,bV[bU]);};};this.__gM=null;};},__gT:function(){var bW=this;while(bW){if(bW.__dd){return true;};if(!bW.__gE||!bW.__gF){return false;};bW=bW.__gR;};return false;},__gU:function(bX){if(bX.__gR===this){throw new Error(a+bX);};if(bX.__dd){throw new Error(q);};if(bX.__gR){bX.__gR.remove(bX);};bX.__gR=this;if(!this.__gP){this.__gP=[];};if(this.__gD){this._scheduleChildrenUpdate();};},__gV:function(bY){if(bY.__gR!==this){throw new Error(K+bY);};if(this.__gD){this._scheduleChildrenUpdate();};delete bY.__gR;},__gW:function(ca){if(ca.__gR!==this){throw new Error(K+ca);};if(this.__gD){this._scheduleChildrenUpdate();};},getChildren:function(){return this.__gP||null;},getChild:function(cb){var cc=this.__gP;return cc&&cc[cb]||null;},hasChildren:function(){var cd=this.__gP;return cd&&cd[0]!==undefined;},indexOf:function(cf){var ce=this.__gP;return ce?ce.indexOf(cf):-1;},hasChild:function(ch){var cg=this.__gP;return cg&&cg.indexOf(ch)!==-1;},add:function(ci){if(arguments[1]){for(var i=0,l=arguments.length;i<l;i++ ){this.__gU(arguments[i]);};this.__gP.push.apply(this.__gP,arguments);}else {this.__gU(ci);this.__gP.push(ci);};return this;},addAt:function(ck,cj){this.__gU(ck);qx.lang.Array.insertAt(this.__gP,ck,cj);return this;},remove:function(cl){var cm=this.__gP;if(!cm){return this;};if(arguments[1]){var cn;for(var i=0,l=arguments.length;i<l;i++ ){cn=arguments[i];this.__gV(cn);qx.lang.Array.remove(cm,cn);};}else {this.__gV(cl);qx.lang.Array.remove(cm,cl);};return this;},removeAt:function(co){var cp=this.__gP;if(!cp){throw new Error(r);};var cq=cp[co];if(!cq){throw new Error(O);};this.__gV(cq);qx.lang.Array.removeAt(this.__gP,co);return this;},removeAll:function(){var cr=this.__gP;if(cr){for(var i=0,l=cr.length;i<l;i++ ){this.__gV(cr[i]);};cr.length=0;};return this;},getParent:function(){return this.__gR||null;},insertInto:function(parent,cs){parent.__gU(this);if(cs==null){parent.__gP.push(this);}else {qx.lang.Array.insertAt(this.__gP,this,cs);};return this;},insertBefore:function(ct){var parent=ct.__gR;parent.__gU(this);qx.lang.Array.insertBefore(parent.__gP,this,ct);return this;},insertAfter:function(cu){var parent=cu.__gR;parent.__gU(this);qx.lang.Array.insertAfter(parent.__gP,this,cu);return this;},moveTo:function(cv){var parent=this.__gR;parent.__gW(this);var cw=parent.__gP.indexOf(this);if(cw===cv){throw new Error(u);}else if(cw<cv){cv-- ;};qx.lang.Array.removeAt(parent.__gP,cw);qx.lang.Array.insertAt(parent.__gP,this,cv);return this;},moveBefore:function(cx){var parent=this.__gR;return this.moveTo(parent.__gP.indexOf(cx));},moveAfter:function(cy){var parent=this.__gR;return this.moveTo(parent.__gP.indexOf(cy)+1);},free:function(){var parent=this.__gR;if(!parent){throw new Error(G);};if(!parent.__gP){return this;};parent.__gV(this);qx.lang.Array.remove(parent.__gP,this);return this;},getDomElement:function(){return this.__gD||null;},getNodeName:function(){return this.__gu;},setNodeName:function(name){this.__gu=name;},setRoot:function(cz){this.__dd=cz;},useMarkup:function(cA){if(this.__gD){throw new Error(Q);};if(qx.core.Environment.get(I)==H){var cB=document.createElement(h);}else {var cB=qx.dom.Element.getHelperElement();};cB.innerHTML=cA;this.useElement(cB.firstChild);return this.__gD;},useElement:function(cC){if(this.__gD){throw new Error(Q);};this.__gD=cC;this.__gD.$$element=this.$$hash;this._copyData(true);},isFocusable:function(){var cE=this.getAttribute(B);if(cE>=1){return true;};var cD=qx.event.handler.Focus.FOCUSABLE_ELEMENTS;if(cE>=0&&cD[this.__gu]){return true;};return false;},setSelectable:function(cG){this.setAttribute(A,cG?D:C);var cF=qx.core.Environment.get(m);if(cF){this.setStyle(cF,cG?b:qx.core.Environment.get(P));};},isNativelyFocusable:function(){return !!qx.event.handler.Focus.FOCUSABLE_ELEMENTS[this.__gu];},include:function(){if(this.__gE){return this;};delete this.__gE;if(this.__gR){this.__gR._scheduleChildrenUpdate();};return this;},exclude:function(){if(!this.__gE){return this;};this.__gE=false;if(this.__gR){this.__gR._scheduleChildrenUpdate();};return this;},isIncluded:function(){return this.__gE===true;},fadeIn:function(cH){var cI=qxWeb(this.__gD);if(cI.isPlaying()){cI.stop();};if(!this.__gD){this.__gS();cI.push(this.__gD);};if(this.__gD){cI.fadeIn(cH);return cI.getAnimationHandles()[0];};},fadeOut:function(cJ){var cK=qxWeb(this.__gD);if(cK.isPlaying()){cK.stop();};if(this.__gD){cK.fadeOut(cJ).once(n,function(){this.hide();qx.html.Element.flush();},this);return cK.getAnimationHandles()[0];};},show:function(){if(this.__gF){return this;};if(this.__gD){qx.html.Element._visibility[this.$$hash]=this;qx.html.Element._scheduleFlush(v);};if(this.__gR){this.__gR._scheduleChildrenUpdate();};delete this.__gF;return this;},hide:function(){if(!this.__gF){return this;};if(this.__gD){qx.html.Element._visibility[this.$$hash]=this;qx.html.Element._scheduleFlush(v);};this.__gF=false;return this;},isVisible:function(){return this.__gF===true;},scrollChildIntoViewX:function(cO,cM,cP){var cL=this.__gD;var cN=cO.getDomElement();if(cP!==false&&cL&&cL.offsetWidth&&cN&&cN.offsetWidth){qx.bom.element.Scroll.intoViewX(cN,cL,cM);}else {this.__gG={element:cO,align:cM};qx.html.Element._scroll[this.$$hash]=this;qx.html.Element._scheduleFlush(v);};delete this.__gI;},scrollChildIntoViewY:function(cT,cR,cU){var cQ=this.__gD;var cS=cT.getDomElement();if(cU!==false&&cQ&&cQ.offsetWidth&&cS&&cS.offsetWidth){qx.bom.element.Scroll.intoViewY(cS,cQ,cR);}else {this.__gH={element:cT,align:cR};qx.html.Element._scroll[this.$$hash]=this;qx.html.Element._scheduleFlush(v);};delete this.__gJ;},scrollToX:function(x,cV){var cW=this.__gD;if(cV!==true&&cW&&cW.offsetWidth){cW.scrollLeft=x;delete this.__gI;}else {this.__gI=x;qx.html.Element._scroll[this.$$hash]=this;qx.html.Element._scheduleFlush(v);};delete this.__gG;},getScrollX:function(){var cX=this.__gD;if(cX){return cX.scrollLeft;};return this.__gI||0;},scrollToY:function(y,da){var cY=this.__gD;if(da!==true&&cY&&cY.offsetWidth){cY.scrollTop=y;delete this.__gJ;}else {this.__gJ=y;qx.html.Element._scroll[this.$$hash]=this;qx.html.Element._scheduleFlush(v);};delete this.__gH;},getScrollY:function(){var dc=this.__gD;if(dc){return dc.scrollTop;};return this.__gJ||0;},disableScrolling:function(){this.enableScrolling();this.scrollToX(0);this.scrollToY(0);this.addListener(L,this.__gY,this);},enableScrolling:function(){this.removeListener(L,this.__gY,this);},__gX:null,__gY:function(e){if(!this.__gX){this.__gX=true;this.__gD.scrollTop=0;this.__gD.scrollLeft=0;delete this.__gX;};},getTextSelection:function(){var dd=this.__gD;if(dd){return qx.bom.Selection.get(dd);};return null;},getTextSelectionLength:function(){var de=this.__gD;if(de){return qx.bom.Selection.getLength(de);};return null;},getTextSelectionStart:function(){var df=this.__gD;if(df){return qx.bom.Selection.getStart(df);};return null;},getTextSelectionEnd:function(){var dg=this.__gD;if(dg){return qx.bom.Selection.getEnd(dg);};return null;},setTextSelection:function(dh,di){var dj=this.__gD;if(dj){qx.bom.Selection.set(dj,dh,di);return;};qx.html.Element.__gx[this.toHashCode()]={element:this,start:dh,end:di};qx.html.Element._scheduleFlush(v);},clearTextSelection:function(){var dk=this.__gD;if(dk){qx.bom.Selection.clear(dk);};delete qx.html.Element.__gx[this.toHashCode()];},__ha:function(dl,dm){var dn=qx.html.Element._actions;dn.push({type:dl,element:this,args:dm||[]});qx.html.Element._scheduleFlush(v);},focus:function(){this.__ha(f);},blur:function(){this.__ha(g);},activate:function(){this.__ha(F);},deactivate:function(){this.__ha(k);},capture:function(dp){this.__ha(o,[dp!==false]);},releaseCapture:function(){this.__ha(t);},setStyle:function(dq,dr,ds){if(!this.__gv){this.__gv={};};if(this.__gv[dq]==dr){return this;};if(dr==null){delete this.__gv[dq];}else {this.__gv[dq]=dr;};if(this.__gD){if(ds){qx.bom.element.Style.set(this.__gD,dq,dr);return this;};if(!this.__gK){this.__gK={};};this.__gK[dq]=true;qx.html.Element._modified[this.$$hash]=this;qx.html.Element._scheduleFlush(v);};return this;},setStyles:function(du,dw){var dv=qx.bom.element.Style;if(!this.__gv){this.__gv={};};if(this.__gD){if(!this.__gK){this.__gK={};};for(var dt in du){var dx=du[dt];if(this.__gv[dt]==dx){continue;};if(dx==null){delete this.__gv[dt];}else {this.__gv[dt]=dx;};if(dw){dv.set(this.__gD,dt,dx);continue;};this.__gK[dt]=true;};qx.html.Element._modified[this.$$hash]=this;qx.html.Element._scheduleFlush(v);}else {for(var dt in du){var dx=du[dt];if(this.__gv[dt]==dx){continue;};if(dx==null){delete this.__gv[dt];}else {this.__gv[dt]=dx;};};};return this;},removeStyle:function(dz,dy){this.setStyle(dz,null,dy);return this;},getStyle:function(dA){return this.__gv?this.__gv[dA]:null;},getAllStyles:function(){return this.__gv||null;},setAttribute:function(dB,dC,dD){if(!this.__gw){this.__gw={};};if(this.__gw[dB]==dC){return this;};if(dC==null){delete this.__gw[dB];}else {this.__gw[dB]=dC;};if(this.__gD){if(dD){qx.bom.element.Attribute.set(this.__gD,dB,dC);return this;};if(!this.__gL){this.__gL={};};this.__gL[dB]=true;qx.html.Element._modified[this.$$hash]=this;qx.html.Element._scheduleFlush(v);};return this;},setAttributes:function(dE,dF){for(var dG in dE){this.setAttribute(dG,dE[dG],dF);};return this;},removeAttribute:function(dI,dH){return this.setAttribute(dI,null,dH);},getAttribute:function(dJ){return this.__gw?this.__gw[dJ]:null;},addClass:function(name){var dK=((this.getAttribute(j)||w)+M+name).trim();this.setAttribute(j,dK);},removeClass:function(name){var dL=this.getAttribute(j);if(dL){this.setAttribute(j,(dL.replace(name,w)).trim());};},_applyProperty:function(name,dM){},_setProperty:function(dN,dO,dP){if(!this.__gN){this.__gN={};};if(this.__gN[dN]==dO){return this;};if(dO==null){delete this.__gN[dN];}else {this.__gN[dN]=dO;};if(this.__gD){if(dP){this._applyProperty(dN,dO);return this;};if(!this.__gM){this.__gM={};};this.__gM[dN]=true;qx.html.Element._modified[this.$$hash]=this;qx.html.Element._scheduleFlush(v);};return this;},_removeProperty:function(dR,dQ){return this._setProperty(dR,null,dQ);},_getProperty:function(dT){var dS=this.__gN;if(!dS){return null;};var dU=dS[dT];return dU==null?null:dU;},addListener:function(ea,dW,self,dV){if(this.$$disposed){return null;};{var dX;};if(this.__gD){return qx.event.Registration.addListener(this.__gD,ea,dW,self,dV);};if(!this.__gO){this.__gO={};};if(dV==null){dV=false;};var dY=qx.event.Manager.getNextUniqueId();var eb=ea+(dV?d:s)+dY;this.__gO[eb]={type:ea,listener:dW,self:self,capture:dV,unique:dY};return eb;},removeListener:function(ej,ed,self,ec){if(this.$$disposed){return null;};{var eh;};if(this.__gD){if(ed.$$wrapped_callback&&ed.$$wrapped_callback[ej+this.$$hash]){var ee=ed.$$wrapped_callback[ej+this.$$hash];delete ed.$$wrapped_callback[ej+this.$$hash];ed=ee;};qx.event.Registration.removeListener(this.__gD,ej,ed,self,ec);}else {var ef=this.__gO;var ei;if(ec==null){ec=false;};for(var eg in ef){ei=ef[eg];if(ei.listener===ed&&ei.self===self&&ei.capture===ec&&ei.type===ej){delete ef[eg];break;};};};return this;},removeListenerById:function(ek){if(this.$$disposed){return null;};if(this.__gD){qx.event.Registration.removeListenerById(this.__gD,ek);}else {delete this.__gO[ek];};return this;},hasListener:function(en,em){if(this.$$disposed){return false;};if(this.__gD){return qx.event.Registration.hasListener(this.__gD,en,em);};var eo=this.__gO;var eq;if(em==null){em=false;};for(var ep in eo){eq=eo[ep];if(eq.capture===em&&eq.type===en){return true;};};return false;},getListeners:function(){if(this.$$disposed){return null;};if(this.__gD){return qx.event.Registration.getManager(this.__gD).serializeListeners(this.__gD);};var er=[];for(var et in this.__gO){var es=this.__gO[et];er.push({type:es.type,handler:es.listener,self:es.self,capture:es.capture});};return er;}},defer:function(eu){eu.__hb=new qx.util.DeferredCall(eu.flush,eu);},destruct:function(){if(this.$$hash){delete qx.html.Element._modified[this.$$hash];delete qx.html.Element._scroll[this.$$hash];};var ev=this.__gD;if(ev){qx.event.Registration.getManager(ev).removeAllListeners(ev);ev.$$element=w;};if(!qx.core.ObjectRegistry.inShutDown){var parent=this.__gR;if(parent&&!parent.$$disposed){parent.remove(this);};};this._disposeArray(z);this.__gw=this.__gv=this.__gO=this.__gN=this.__gL=this.__gK=this.__gM=this.__gD=this.__gR=this.__gG=this.__gH=null;}});})();(function(){var a="qx.event.handler.Appear",b="engine.name",c="mshtml",d="disappear",e="appear",f="browser.documentmode";qx.Class.define(a,{extend:qx.core.Object,implement:qx.event.IEventHandler,construct:function(g){qx.core.Object.call(this);this.__gt=g;this.__hc={};qx.event.handler.Appear.__hd[this.$$hash]=this;},statics:{PRIORITY:qx.event.Registration.PRIORITY_NORMAL,SUPPORTED_TYPES:{appear:true,disappear:true},TARGET_CHECK:qx.event.IEventHandler.TARGET_DOMNODE,IGNORE_CAN_HANDLE:true,__hd:{},refresh:function(){var h=this.__hd;for(var i in h){h[i].refresh();};}},members:{__gt:null,__hc:null,canHandleEvent:function(k,j){},registerEvent:function(o,p,m){var n=qx.core.ObjectRegistry.toHashCode(o)+p;var l=this.__hc;if(l&&!l[n]){l[n]=o;o.$$displayed=o.offsetWidth>0;};},unregisterEvent:function(t,u,r){var s=qx.core.ObjectRegistry.toHashCode(t)+u;var q=this.__hc;if(!q){return;};if(q[s]){delete q[s];};},refresh:function(){var A=this.__hc;var x;var y=qx.core.Environment.get(b)==c&&qx.core.Environment.get(f)<9;for(var v in A){x=A[v];var w=x.offsetWidth>0;if(!w&&y){w=x.offsetWidth>0;};if((!!x.$$displayed)!==w){x.$$displayed=w;var z=qx.event.Registration.createEvent(w?e:d);this.__gt.dispatchEvent(x,z);};};}},destruct:function(){this.__gt=this.__hc=null;delete qx.event.handler.Appear.__hd[this.$$hash];},defer:function(B){qx.event.Registration.addHandler(B);}});})();(function(){var a="abstract",b="Missing implementation",c="qx.event.dispatch.AbstractBubbling";qx.Class.define(c,{extend:qx.core.Object,implement:qx.event.IEventDispatcher,type:a,construct:function(d){this._manager=d;},members:{_getParent:function(e){throw new Error(b);},canDispatchEvent:function(g,event,f){return event.getBubbles();},dispatchEvent:function(l,event,w){var parent=l;var s=this._manager;var o,x;var n;var v,u;var y;var q=[];o=s.getListeners(l,w,true);x=s.getListeners(l,w,false);if(o){q.push(o);};if(x){q.push(x);};var parent=this._getParent(l);var k=[];var h=[];var m=[];var p=[];while(parent!=null){o=s.getListeners(parent,w,true);if(o){m.push(o);p.push(parent);};x=s.getListeners(parent,w,false);if(x){k.push(x);h.push(parent);};parent=this._getParent(parent);};event.setEventPhase(qx.event.type.Event.CAPTURING_PHASE);for(var i=m.length-1;i>=0;i-- ){y=p[i];event.setCurrentTarget(y);n=m[i];for(var j=0,r=n.length;j<r;j++ ){v=n[j];u=v.context||y;{};v.handler.call(u,event);};if(event.getPropagationStopped()){return;};};event.setEventPhase(qx.event.type.Event.AT_TARGET);event.setCurrentTarget(l);for(var i=0,t=q.length;i<t;i++ ){n=q[i];for(var j=0,r=n.length;j<r;j++ ){v=n[j];u=v.context||l;{};v.handler.call(u,event);};if(event.getPropagationStopped()){return;};};event.setEventPhase(qx.event.type.Event.BUBBLING_PHASE);for(var i=0,t=k.length;i<t;i++ ){y=h[i];event.setCurrentTarget(y);n=k[i];for(var j=0,r=n.length;j<r;j++ ){v=n[j];u=v.context||y;{};v.handler.call(u,event);};if(event.getPropagationStopped()){return;};};}}});})();(function(){var a="qx.event.dispatch.DomBubbling";qx.Class.define(a,{extend:qx.event.dispatch.AbstractBubbling,statics:{PRIORITY:qx.event.Registration.PRIORITY_NORMAL},members:{_getParent:function(b){return b.parentNode;},canDispatchEvent:function(d,event,c){return d.nodeType!==undefined&&event.getBubbles();}},defer:function(e){qx.event.Registration.addDispatcher(e);}});})();(function(){var a="os.name",b="opera",c="engine.name",d="qx.event.type.Dom",e="osx";qx.Class.define(d,{extend:qx.event.type.Native,statics:{SHIFT_MASK:1,CTRL_MASK:2,ALT_MASK:4,META_MASK:8},members:{_cloneNativeEvent:function(f,g){var g=qx.event.type.Native.prototype._cloneNativeEvent.call(this,f,g);g.shiftKey=f.shiftKey;g.ctrlKey=f.ctrlKey;g.altKey=f.altKey;g.metaKey=f.metaKey;return g;},getModifiers:function(){var h=0;var i=this._native;if(i.shiftKey){h|=qx.event.type.Dom.SHIFT_MASK;};if(i.ctrlKey){h|=qx.event.type.Dom.CTRL_MASK;};if(i.altKey){h|=qx.event.type.Dom.ALT_MASK;};if(i.metaKey){h|=qx.event.type.Dom.META_MASK;};return h;},isCtrlPressed:function(){return this._native.ctrlKey;},isShiftPressed:function(){return this._native.shiftKey;},isAltPressed:function(){return this._native.altKey;},isMetaPressed:function(){return this._native.metaKey;},isCtrlOrCommandPressed:function(){if(qx.core.Environment.get(a)==e&&qx.core.Environment.get(c)!=b){return this._native.metaKey;}else {return this._native.ctrlKey;};}}});})();(function(){var a="mshtml",b="engine.name",c="click",d="middle",e="none",f="contextmenu",g="qx.event.type.Mouse",h="browser.documentmode",i="left",j="right",k="browser.name",l="ie";qx.Class.define(g,{extend:qx.event.type.Dom,members:{_cloneNativeEvent:function(m,n){var n=qx.event.type.Dom.prototype._cloneNativeEvent.call(this,m,n);n.button=m.button;n.clientX=Math.round(m.clientX);n.clientY=Math.round(m.clientY);n.pageX=m.pageX?Math.round(m.pageX):undefined;n.pageY=m.pageY?Math.round(m.pageY):undefined;n.screenX=Math.round(m.screenX);n.screenY=Math.round(m.screenY);n.wheelDelta=m.wheelDelta;n.wheelDeltaX=m.wheelDeltaX;n.wheelDeltaY=m.wheelDeltaY;n.delta=m.delta;n.deltaX=m.deltaX;n.deltaY=m.deltaY;n.deltaZ=m.deltaZ;n.detail=m.detail;n.axis=m.axis;n.wheelX=m.wheelX;n.wheelY=m.wheelY;n.HORIZONTAL_AXIS=m.HORIZONTAL_AXIS;n.srcElement=m.srcElement;n.target=m.target;return n;},__he:{'0':i,'2':j,'1':d},__hf:{'0':e,'1':i,'2':j,'4':d},__hg:{'1':i,'2':j,'4':d},stop:function(){this.stopPropagation();},getButton:function(){switch(this._type){case f:return j;case c:if(qx.core.Environment.get(k)===l&&qx.core.Environment.get(h)<9){return i;};default:if(!(qx.core.Environment.get(b)==a&&qx.core.Environment.get(h)<=8)){if(this._native.button===-1){return this.__hf[this._native.buttons]||e;};return this.__he[this._native.button]||e;}else {return this.__hg[this._native.button]||e;};};},isLeftPressed:function(){return this.getButton()===i;},isMiddlePressed:function(){return this.getButton()===d;},isRightPressed:function(){return this.getButton()===j;},getRelatedTarget:function(){return this._relatedTarget;},getViewportLeft:function(){return Math.round(this._native.clientX);},getViewportTop:function(){return Math.round(this._native.clientY);},getDocumentLeft:function(){if(this._native.pageX!==undefined){return Math.round(this._native.pageX);}else if(this._native.srcElement){var o=qx.dom.Node.getWindow(this._native.srcElement);return Math.round(this._native.clientX)+qx.bom.Viewport.getScrollLeft(o);}else {return Math.round(this._native.clientX)+qx.bom.Viewport.getScrollLeft(window);};},getDocumentTop:function(){if(this._native.pageY!==undefined){return Math.round(this._native.pageY);}else if(this._native.srcElement){var p=qx.dom.Node.getWindow(this._native.srcElement);return Math.round(this._native.clientY)+qx.bom.Viewport.getScrollTop(p);}else {return Math.round(this._native.clientY)+qx.bom.Viewport.getScrollTop(window);};},getScreenLeft:function(){return Math.round(this._native.screenX);},getScreenTop:function(){return Math.round(this._native.screenY);}}});})();(function(){var a="",b="mouse",c="number",d="touch",e="qx.event.type.Pointer",f="pen",g="string";qx.Class.define(e,{extend:qx.event.type.Mouse,members:{_cloneNativeEvent:function(h,i){i=qx.event.type.Mouse.prototype._cloneNativeEvent.call(this,h,i);i.pointerId=h.pointerId;i.width=h.width;i.height=h.height;i.pressure=h.pressure;i.tiltX=h.tiltX;i.tiltY=h.tiltY;i.pointerType=h.pointerType;i.isPrimary=h.isPrimary;i._original=h._original;i.MSPOINTER_TYPE_MOUSE=h.MSPOINTER_TYPE_MOUSE;i.MSPOINTER_TYPE_PEN=h.MSPOINTER_TYPE_PEN;i.MSPOINTER_TYPE_TOUCH=h.MSPOINTER_TYPE_TOUCH;return i;},getDocumentLeft:function(){var x=qx.event.type.Mouse.prototype.getDocumentLeft.call(this);if(x==0&&this.getPointerType()==d&&this._native._original!==undefined){x=Math.round(this._native._original.changedTouches[0].pageX)||0;};return x;},getDocumentTop:function(){var y=qx.event.type.Mouse.prototype.getDocumentTop.call(this);if(y==0&&this.getPointerType()==d&&this._native._original!==undefined){y=Math.round(this._native._original.changedTouches[0].pageY)||0;};return y;},getPointerId:function(){return this._native.pointerId||0;},getWidth:function(){return this._native.width||0;},getHeight:function(){return this._native.height||0;},getPressure:function(){return this._native.pressure||0;},getTiltX:function(){return this._native.tiltX||0;},getTiltY:function(){return this._native.tiltY||0;},getOriginalTarget:function(){if(this._native&&this._native._original){var j=this._native._original;try{if(j.type.indexOf(d)==0){if(j.changedTouches[0]){return document.elementFromPoint(j.changedTouches[0].clientX,j.changedTouches[0].clientY);};};}catch(k){return qx.bom.Event.getTarget(this._native);};return qx.bom.Event.getTarget(j);}else if(this._native){return qx.bom.Event.getTarget(this._native);};return qx.event.type.Mouse.prototype.getOriginalTarget.call(this);},getPointerType:function(){if(typeof this._native.pointerType==g){return this._native.pointerType;};if(typeof this._native.pointerType==c){if(this._native.pointerType==this._native.MSPOINTER_TYPE_MOUSE){return b;};if(this._native.pointerType==this._native.MSPOINTER_TYPE_PEN){return f;};if(this._native.pointerType==this._native.MSPOINTER_TYPE_TOUCH){return d;};};return a;},isPrimary:function(){return !!this._native.isPrimary;}}});})();(function(){var a="mshtml",b="engine.name",c="pointerup",d="dispose",e="useraction",f="mouse",g="pointercancel",h="pointerdown",i="pointermove",j="qx.event.handler.Pointer",k="browser.documentmode",l="qxanonymous";qx.Class.define(j,{extend:qx.event.handler.PointerCore,implement:qx.event.IEventHandler,statics:{PRIORITY:qx.event.Registration.PRIORITY_NORMAL,SUPPORTED_TYPES:{pointermove:1,pointerover:1,pointerout:1,pointerdown:1,pointerup:1,pointercancel:1,gesturebegin:1,gesturemove:1,gesturefinish:1,gesturecancel:1},TARGET_CHECK:qx.event.IEventHandler.TARGET_DOMNODE+qx.event.IEventHandler.TARGET_DOCUMENT,IGNORE_CAN_HANDLE:true},construct:function(m){this.__gt=m;this.__cz=m.getWindow();this.__dd=this.__cz.document;qx.event.handler.PointerCore.apply(this,[this.__dd]);},members:{__gt:null,__cz:null,__dd:null,canHandleEvent:function(o,n){},registerEvent:function(r,q,p){},unregisterEvent:function(u,t,s){},_initPointerObserver:function(){var v=false;if(qx.core.Environment.get(b)==a&&qx.core.Environment.get(k)<9){v=true;};this._initObserver(this._onPointerEvent,v);},_fireEvent:function(w,x,y){if(!y){y=qx.bom.Event.getTarget(w);};while(y&&y.getAttribute&&y.getAttribute(l)){y=y.parentNode;};if(!x){x=w.type;};x=qx.event.handler.PointerCore.MSPOINTER_TO_POINTER_MAPPING[x]||x;if(y&&y.nodeType){qx.event.type.dom.Pointer.normalize(w);w.srcElement=y;qx.event.Registration.fireEvent(y,x,qx.event.type.Pointer,[w,y,null,true,true]);if((w.getPointerType()!==f||w.button<=qx.event.handler.PointerCore.LEFT_BUTTON)&&(x==h||x==c||x==i||x==g)){qx.event.Registration.fireEvent(this.__dd,qx.event.handler.PointerCore.POINTER_TO_GESTURE_MAPPING[x],qx.event.type.Pointer,[w,y,null,false,false]);};qx.event.Registration.fireEvent(this.__cz,e,qx.event.type.Data,[x]);};},_onPointerEvent:function(z){if(z._original&&z._original[this._processedFlag]){return;};var A=qx.event.handler.PointerCore.MSPOINTER_TO_POINTER_MAPPING[z.type]||z.type;this._fireEvent(z,A,qx.bom.Event.getTarget(z));},dispose:function(){this.__hh(d);this.__gt=this.__cz=this.__dd=null;},__hh:function(C,B){qx.event.handler.PointerCore.prototype[C].apply(this,B||[]);}},defer:function(D){qx.event.Registration.addHandler(D);qx.event.Registration.getManager(document).getHandler(D);}});})();(function(){var a="qx.event.type.Tap";qx.Class.define(a,{extend:qx.event.type.Pointer});})();(function(){var a="qx.event.type.Track";qx.Class.define(a,{extend:qx.event.type.Pointer,members:{_cloneNativeEvent:function(b,c){var c=qx.event.type.Pointer.prototype._cloneNativeEvent.call(this,b,c);c.delta=b.delta;return c;},getDelta:function(){return this._native.delta;}}});})();(function(){var a="qx.event.type.Swipe";qx.Class.define(a,{extend:qx.event.type.Pointer,members:{_cloneNativeEvent:function(b,c){var c=qx.event.type.Pointer.prototype._cloneNativeEvent.call(this,b,c);c.swipe=b.swipe;return c;},getStartTime:function(){return this._native.swipe.startTime;},getDuration:function(){return this._native.swipe.duration;},getAxis:function(){return this._native.swipe.axis;},getDirection:function(){return this._native.swipe.direction;},getVelocity:function(){return this._native.swipe.velocity;},getDistance:function(){return this._native.swipe.distance;}}});})();(function(){var a="qx.event.type.Rotate";qx.Class.define(a,{extend:qx.event.type.Pointer,members:{_cloneNativeEvent:function(b,c){var c=qx.event.type.Pointer.prototype._cloneNativeEvent.call(this,b,c);c.angle=b.angle;return c;},getAngle:function(){return this._native.angle;}}});})();(function(){var a="qx.event.type.Roll";qx.Class.define(a,{extend:qx.event.type.Pointer,members:{stop:function(){this.stopPropagation();this.preventDefault();},_cloneNativeEvent:function(b,c){var c=qx.event.type.Pointer.prototype._cloneNativeEvent.call(this,b,c);c.delta=b.delta;c.momentum=b.momentum;c.timeoutId=b.timeoutId;return c;},getMomentum:function(){return this._native.momentum;},stopMomentum:function(){if(this._native.timeoutId){qx.event.Registration.getManager(this._originalTarget).getHandler(qx.event.handler.Gesture).stopMomentum(this._native.timeoutId);};},getDelta:function(){return this._native.delta;}}});})();(function(){var a="qx.event.type.Pinch";qx.Class.define(a,{extend:qx.event.type.Pointer,members:{_cloneNativeEvent:function(b,c){var c=qx.event.type.Pointer.prototype._cloneNativeEvent.call(this,b,c);c.scale=b.scale;return c;},getScale:function(){return this._native.scale;}}});})();(function(){var a="swipe",b="pinch",c="event.dispatchevent",d="gesturemove",e="touch",f="longtap",g="event.mousewheel",h="roll",i="dblclick",j="wheel",k="rotate",l="trackstart",m="gesturefinish",n="y",o="browser.documentmode",p="dbltap",q="qx.event.handler.GestureCore",r="right",s="mshtml",t="engine.name",u="gesturecancel",v="gesturebegin",w="track",z="trackend",A="left",B="tap",C="down",D="x",E="up";qx.Bootstrap.define(q,{extend:Object,statics:{TYPES:[B,a,f,p,w,l,z,k,b,h],GESTURE_EVENTS:[v,m,d,u],TAP_MAX_DISTANCE:{"touch":40,"mouse":5,"pen":20},DOUBLETAP_MAX_DISTANCE:{"touch":10,"mouse":4,"pen":10},SWIPE_DIRECTION:{x:[A,r],y:[E,C]},LONGTAP_TIME:500,DOUBLETAP_TIME:500,ROLL_FACTOR:18},construct:function(F,G){this.__fA=F;this.__fB=G;this.__hi={};this.__hj={};this.__hk={};this._initObserver();},members:{__fA:null,__fB:null,__hi:null,__hl:null,__hm:null,__hn:null,__ho:null,__hj:null,__hp:null,__hk:null,__hq:null,_initObserver:function(){qx.event.handler.GestureCore.GESTURE_EVENTS.forEach(function(I){qxWeb(this.__fA).on(I,this.checkAndFireGesture,this);}.bind(this));if(qx.core.Environment.get(t)==s&&qx.core.Environment.get(o)<9){qxWeb(this.__fA).on(i,this._onDblClick,this);};var H=qx.core.Environment.get(g);qxWeb(H.target).on(H.type,this._fireRoll,this);},_stopObserver:function(){qx.event.handler.GestureCore.GESTURE_EVENTS.forEach(function(K){qxWeb(this.__fA).off(K,this.checkAndFireGesture,this);}.bind(this));if(qx.core.Environment.get(t)==s&&qx.core.Environment.get(o)<9){qxWeb(this.__fA).off(i,this._onDblClick,this);};var J=qx.core.Environment.get(g);qxWeb(J.target).off(J.type,this._fireRoll,this);},checkAndFireGesture:function(L,M,N){if(!M){M=L.type;};if(!N){N=qx.bom.Event.getTarget(L);};if(M==v){this.gestureBegin(L,N);}else if(M==d){this.gestureMove(L,N);}else if(M==m){this.gestureFinish(L,N);}else if(M==u){this.gestureCancel(L.pointerId);};},gestureBegin:function(O,P){if(this.__hi[O.pointerId]){this.__hy(this.__hi[O.pointerId]);delete this.__hi[O.pointerId];};if(this._hasIntermediaryHandler(P)){return;};this.__hi[O.pointerId]={"startTime":new Date().getTime(),"lastEventTime":new Date().getTime(),"startX":O.clientX,"startY":O.clientY,"clientX":O.clientX,"clientY":O.clientY,"velocityX":0,"velocityY":0,"target":P,"isTap":true,"isPrimary":O.isPrimary,"longTapTimer":window.setTimeout(this.__hx.bind(this,O,P),qx.event.handler.GestureCore.LONGTAP_TIME)};if(O.isPrimary){this.__hn=false;this.__hm=P;this.__hu(l,O,P);}else {this.__hn=true;if(Object.keys(this.__hi).length===2){this.__ho=this._calcAngle();this.__hq=this._calcDistance();};};},gestureMove:function(R,S){var T=this.__hi[R.pointerId];if(T){var Q=T.clientX;var U=T.clientY;T.clientX=R.clientX;T.clientY=R.clientY;T.lastEventTime=new Date().getTime();if(Q){T.velocityX=T.clientX-Q;};if(U){T.velocityY=T.clientY-U;};if(Object.keys(this.__hi).length===2){this.__hv(R,T.target);this.__hw(R,T.target);};if(!this.__hn){this.__hu(w,R,T.target);this._fireRoll(R,e,T.target);};if(T.isTap){T.isTap=this._isBelowTapMaxDistance(R);if(!T.isTap){this.__hy(T);};};};},_hasIntermediaryHandler:function(V){while(V&&V!==this.__fA){if(V.$$gestureHandler){return true;};V=V.parentNode;};return false;},gestureFinish:function(X,Y){if(!this.__hi[X.pointerId]){return;};var bf=this.__hi[X.pointerId];this.__hy(bf);if(this._hasIntermediaryHandler(Y)){return;};this.__hr(bf.velocityX,bf.velocityY,X,bf.target);this.__hu(z,X,bf.target);if(bf.isTap){if(Y!==bf.target){delete this.__hi[X.pointerId];return;};this._fireEvent(X,B,X.target||Y);var ba=false;if(Object.keys(this.__hj).length>0){var be=Date.now()-qx.event.handler.GestureCore.DOUBLETAP_TIME;for(var bg in this.__hj){if(bg<be){delete this.__hj[bg];}else {var W=this.__hj[bg];var bc=this.__hs(W.x,W.y,X.clientX,X.clientY,X.getPointerType());var bd=W.target===(X.target||Y);var bh=W.button===X.button;if(bc&&bh&&bd){ba=true;delete this.__hj[bg];this._fireEvent(X,p,X.target||Y);};};};};if(!ba){this.__hj[Date.now()]={x:X.clientX,y:X.clientY,target:X.target||Y,button:X.button};};}else if(!this._isBelowTapMaxDistance(X)){var bb=this.__ht(X,Y);if(bb){X.swipe=bb;this._fireEvent(X,a,bf.target||Y);};};delete this.__hi[X.pointerId];},stopMomentum:function(bi){this.__hk[bi]=true;},gestureCancel:function(bj){if(this.__hi[bj]){this.__hy(this.__hi[bj]);delete this.__hi[bj];};},updateGestureTarget:function(bk,bl){this.__hi[bk].target=bl;},__hr:function(bq,br,bn,bo,bt){var bs=bn.timeoutId;if((Math.abs(br)<1&&Math.abs(bq)<1)||this.__hk[bs]){delete this.__hk[bs];return;};if(!bt){bt=1;var bp=2.8;br=br/bp;bq=bq/bp;};bt+=0.0006;br=br/bt;bq=bq/bt;var bm=qx.bom.AnimationFrame.request(qx.lang.Function.bind(function(bu,bv,bw,bx,by){this.__hr(bu,bv,bw,bx,by);},this,bq,br,bn,bo,bt));bq=Math.round(bq*100)/100;br=Math.round(br*100)/100;bn.delta={x:-bq,y:-br};bn.momentum=true;bn.timeoutId=bm;this._fireEvent(bn,h,bn.target||bo);},_calcAngle:function(){var bA=null;var bB=null;for(var bz in this.__hi){var bC=this.__hi[bz];if(bA===null){bA=bC;}else {bB=bC;};};var x=bA.clientX-bB.clientX;var y=bA.clientY-bB.clientY;return (360+Math.atan2(y,x)*(180/Math.PI))%360;},_calcDistance:function(){var bD=null;var bE=null;for(var bG in this.__hi){var bH=this.__hi[bG];if(bD===null){bD=bH;}else {bE=bH;};};var bF=Math.sqrt(Math.pow(bD.clientX-bE.clientX,2)+Math.pow(bD.clientY-bE.clientY,2));return bF;},_isBelowTapMaxDistance:function(bJ){var bK=this._getDeltaCoordinates(bJ);var bI=qx.event.handler.GestureCore.TAP_MAX_DISTANCE[bJ.getPointerType()];if(!bK){return null;};return (Math.abs(bK.x)<=bI&&Math.abs(bK.y)<=bI);},__hs:function(bL,bP,bQ,bR,bS){var bO=qx.event.handler.GestureCore;var bM=Math.abs(bL-bQ)<bO.DOUBLETAP_MAX_DISTANCE[bS];var bN=Math.abs(bP-bR)<bO.DOUBLETAP_MAX_DISTANCE[bS];return bM&&bN;},_getDeltaCoordinates:function(bV){var bW=this.__hi[bV.pointerId];if(!bW){return null;};var bT=bV.clientX-bW.startX;var bU=bV.clientY-bW.startY;var bX=D;if(Math.abs(bT/bU)<1){bX=n;};return {"x":bT,"y":bU,"axis":bX};},_fireEvent:function(ca,cc,cb){if(!this.__fA){return;};var bY;if(qx.core.Environment.get(c)){bY=new qx.event.type.dom.Custom(cc,ca,{bubbles:true,swipe:ca.swipe,scale:ca.scale,angle:ca.angle,delta:ca.delta,pointerType:ca.pointerType,momentum:ca.momentum});cb.dispatchEvent(bY);}else if(this.__fB){bY=new qx.event.type.dom.Custom(cc,ca,{target:this.__fA,currentTarget:this.__fA,srcElement:this.__fA,swipe:ca.swipe,scale:ca.scale,angle:ca.angle,delta:ca.delta,pointerType:ca.pointerType,momentum:ca.momentum});this.__fB.emit(cc,ca);};},_onDblClick:function(cd){var ce=qx.bom.Event.getTarget(cd);this._fireEvent(cd,B,ce);this._fireEvent(cd,p,ce);},__ht:function(ch,ci){var co=this.__hi[ch.pointerId];if(!co){return null;};var ck=qx.event.handler.GestureCore;var cn=this._getDeltaCoordinates(ch);var cl=new Date().getTime()-co.startTime;var cp=(Math.abs(cn.x)>=Math.abs(cn.y))?D:n;var cf=cn[cp];var cg=ck.SWIPE_DIRECTION[cp][cf<0?0:1];var cm=(cl!==0)?cf/cl:0;var cj={startTime:co.startTime,duration:cl,axis:cp,direction:cg,distance:cf,velocity:cm};return cj;},__hu:function(cq,cr,cs){cr.delta=this._getDeltaCoordinates(cr);this._fireEvent(cr,cq,cr.target||cs);},_fireRoll:function(cu,ct,cv){if(cu.type===qx.core.Environment.get(g).type){cu.delta={x:qx.util.Wheel.getDelta(cu,D)*qx.event.handler.GestureCore.ROLL_FACTOR,y:qx.util.Wheel.getDelta(cu,n)*qx.event.handler.GestureCore.ROLL_FACTOR};cu.delta.axis=Math.abs(cu.delta.x/cu.delta.y)<1?n:D;cu.pointerType=j;}else {var cw=this.__hi[cu.pointerId];cu.delta={x:-cw.velocityX,y:-cw.velocityY,axis:Math.abs(cw.velocityX/cw.velocityY)<1?n:D};};this._fireEvent(cu,h,cu.target||cv);},__hv:function(cx,cz){if(!cx.isPrimary){var cy=this._calcAngle();cx.angle=Math.round((cy-this.__ho)%360);this._fireEvent(cx,k,this.__hm);};},__hw:function(cC,cD){if(!cC.isPrimary){var cA=this._calcDistance();var cB=cA/this.__hq;cC.scale=(Math.round(cB*100)/100);this._fireEvent(cC,b,this.__hm);};},__hx:function(cE,cF){var cG=this.__hi[cE.pointerId];if(cG){this._fireEvent(cE,f,cE.target||cF);cG.longTapTimer=null;cG.isTap=false;};},__hy:function(cH){if(cH.longTapTimer){window.clearTimeout(cH.longTapTimer);cH.longTapTimer=null;};},isBelowTapMaxDistance:function(event){var cI=this._calcDelta(event);var cJ=qx.event.handler.GestureCore;return (Math.abs(cI.x)<=cJ.TAP_MAX_DISTANCE&&Math.abs(cI.y)<=cJ.TAP_MAX_DISTANCE);},dispose:function(){for(var cK in this.__hi){this.__hy(cK);};this._stopObserver();this.__fA=this.__fB=null;}}});})();(function(){var a="x",b="y",c="qx.util.Wheel";qx.Bootstrap.define(c,{statics:{MAXSCROLL:null,MINSCROLL:null,FACTOR:1,getDelta:function(e,d){if(d===undefined){var f=0;if(e.wheelDelta!==undefined){f=-e.wheelDelta;}else if(e.detail!==0){f=e.detail;}else if(e.deltaY!==undefined){f=e.deltaY;};return this.__hz(f);};if(d===a){var x=0;if(e.wheelDelta!==undefined){if(e.wheelDeltaX!==undefined){x=e.wheelDeltaX?this.__hz(-e.wheelDeltaX):0;};}else {if(e.axis&&e.axis==e.HORIZONTAL_AXIS&&(e.detail!==undefined)&&(e.detail>0)){x=this.__hz(e.detail);}else if(e.deltaX!==undefined){x=this.__hz(e.deltaX);};};return x;};if(d===b){var y=0;if(e.wheelDelta!==undefined){if(e.wheelDeltaY!==undefined){y=e.wheelDeltaY?this.__hz(-e.wheelDeltaY):0;}else {y=this.__hz(-e.wheelDelta);};}else {if(!(e.axis&&e.axis==e.HORIZONTAL_AXIS)&&(e.detail!==undefined)&&(e.detail>0)){y=this.__hz(e.detail);}else if(e.deltaY!==undefined){y=this.__hz(e.deltaY);};};return y;};return 0;},__hz:function(j){var g=Math.abs(j);if(g===0){return 0;};if(qx.util.Wheel.MINSCROLL==null||qx.util.Wheel.MINSCROLL>g){qx.util.Wheel.MINSCROLL=g;this.__hA();};if(qx.util.Wheel.MAXSCROLL==null||qx.util.Wheel.MAXSCROLL<g){qx.util.Wheel.MAXSCROLL=g;this.__hA();};if(qx.util.Wheel.MAXSCROLL===g&&qx.util.Wheel.MINSCROLL===g){return 2*(j/g);};var h=qx.util.Wheel.MAXSCROLL-qx.util.Wheel.MINSCROLL;var i=(j/h)*Math.log(h)*qx.util.Wheel.FACTOR;return i<0?Math.min(i,-1):Math.max(i,1);},__hA:function(){var k=qx.util.Wheel.MAXSCROLL||0;var n=qx.util.Wheel.MINSCROLL||k;if(k<=n){return;};var l=k-n;var m=(k/l)*Math.log(l);if(m==0){m=1;};qx.util.Wheel.FACTOR=6/m;}}});})();(function(){var a="dblclick",b="mshtml",c="engine.name",d="dispose",e="useraction",f="gesturemove",g="gesturecancel",h="checkAndFireGesture",i="gesturebegin",j="qx.event.handler.Gesture",k="gesturefinish",l="browser.documentmode";qx.Class.define(j,{extend:qx.event.handler.GestureCore,implement:qx.event.IEventHandler,statics:{PRIORITY:qx.event.Registration.PRIORITY_NORMAL,SUPPORTED_TYPES:{tap:1,swipe:1,longtap:1,dbltap:1,rotate:1,pinch:1,track:1,trackstart:1,trackend:1,roll:1},GESTURE_EVENTS:[i,k,f,g],TARGET_CHECK:qx.event.IEventHandler.TARGET_DOMNODE+qx.event.IEventHandler.TARGET_DOCUMENT,IGNORE_CAN_HANDLE:true,EVENT_CLASSES:{"tap":qx.event.type.Tap,"longtap":qx.event.type.Tap,"dbltap":qx.event.type.Tap,"swipe":qx.event.type.Swipe,"rotate":qx.event.type.Rotate,"pinch":qx.event.type.Pinch,"track":qx.event.type.Track,"trackstart":qx.event.type.Track,"trackend":qx.event.type.Track,"roll":qx.event.type.Roll}},construct:function(m){this.__gt=m;this.__cz=m.getWindow();this.__dd=this.__cz.document;qx.event.handler.GestureCore.apply(this,[this.__dd]);},members:{__gt:null,__cz:null,__dd:null,__fu:null,__hB:null,__hC:null,canHandleEvent:function(o,n){},registerEvent:function(r,q,p){},unregisterEvent:function(u,t,s){},_initObserver:function(){this.__fu=qx.lang.Function.listener(this.checkAndFireGesture,this);qx.event.handler.Gesture.GESTURE_EVENTS.forEach(function(w){qx.event.Registration.addListener(this.__dd,w,this.__fu,this);}.bind(this));if(qx.core.Environment.get(c)==b&&qx.core.Environment.get(l)<9){this.__hB=qx.lang.Function.listener(this._onDblClick,this);qx.bom.Event.addNativeListener(this.__dd,a,this.__hB);};var v=qx.bom.client.Event.getMouseWheel(this.__cz);this.__hC=qx.lang.Function.listener(this._fireRoll,this);qx.bom.Event.addNativeListener(v.target,v.type,this.__hC,this);},checkAndFireGesture:function(y,x,z){this.__hh(h,[y.getNativeEvent(),y.getType(),y.getTarget()]);},_stopObserver:function(){qx.event.handler.Gesture.GESTURE_EVENTS.forEach(function(B){qx.event.Registration.removeListener(this.__dd,B,this.__fu);}.bind(this));if(qx.core.Environment.get(c)==b&&qx.core.Environment.get(l)<9){qx.bom.Event.removeNativeListener(this.__dd,a,this.__hB);};var A=qx.bom.client.Event.getMouseWheel(this.__cz);qx.bom.Event.removeNativeListener(A.target,A.type,this.__hC);},_hasIntermediaryHandler:function(C){return false;},_fireEvent:function(E,D,F){if(!F){F=qx.bom.Event.getTarget(E);};if(!D){D=E.type;};var G=qx.event.handler.Gesture.EVENT_CLASSES[D]||qx.event.type.Pointer;if(F&&F.nodeType){qx.event.Registration.fireEvent(F,D,G,[E,F,null,true,true]);};qx.event.Registration.fireEvent(this.__cz,e,qx.event.type.Data,[D]);},dispose:function(){this._stopObserver();this.__hh(d);this.__gt=this.__cz=this.__dd=this.__hB=null;},__hh:function(I,H){qx.event.handler.GestureCore.prototype[I].apply(this,H||[]);}},defer:function(J){qx.event.Registration.addHandler(J);qx.event.Registration.getManager(document).getHandler(J);}});})();(function(){var a="-",b="qx.event.handler.Element",c="load",d="iframe";qx.Class.define(b,{extend:qx.core.Object,implement:qx.event.IEventHandler,construct:function(e){qx.core.Object.call(this);this._manager=e;this._registeredEvents={};},statics:{PRIORITY:qx.event.Registration.PRIORITY_NORMAL,SUPPORTED_TYPES:{abort:true,load:true,scroll:true,select:true,reset:true,submit:true},CANCELABLE:{selectstart:true},TARGET_CHECK:qx.event.IEventHandler.TARGET_DOMNODE,IGNORE_CAN_HANDLE:false},members:{canHandleEvent:function(g,f){if(f===c){return g.tagName.toLowerCase()!==d;}else {return true;};},registerEvent:function(j,l,i){var m=qx.core.ObjectRegistry.toHashCode(j);var h=m+a+l;var k=qx.lang.Function.listener(this._onNative,this,h);qx.bom.Event.addNativeListener(j,l,k);this._registeredEvents[h]={element:j,type:l,listener:k};},unregisterEvent:function(p,r,o){var s=this._registeredEvents;if(!s){return;};var t=qx.core.ObjectRegistry.toHashCode(p);var n=t+a+r;var q=this._registeredEvents[n];if(q){qx.bom.Event.removeNativeListener(p,r,q.listener);};delete this._registeredEvents[n];},_onNative:qx.event.GlobalError.observeMethod(function(v,u){var w=this._registeredEvents;if(!w){return;};var y=w[u];var x=this.constructor.CANCELABLE[y.type];qx.event.Registration.fireNonBubblingEvent(y.element,y.type,qx.event.type.Native,[v,undefined,undefined,undefined,x]);})},destruct:function(){var z;var A=this._registeredEvents;for(var B in A){z=A[B];qx.bom.Event.removeNativeListener(z.element,z.type,z.listener);};this._manager=this._registeredEvents=null;},defer:function(C){qx.event.Registration.addHandler(C);}});})();(function(){var a="qx.event.handler.UserAction";qx.Class.define(a,{extend:qx.core.Object,implement:qx.event.IEventHandler,construct:function(b){qx.core.Object.call(this);this.__gt=b;this.__cz=b.getWindow();},statics:{PRIORITY:qx.event.Registration.PRIORITY_NORMAL,SUPPORTED_TYPES:{useraction:1},TARGET_CHECK:qx.event.IEventHandler.TARGET_WINDOW,IGNORE_CAN_HANDLE:true},members:{__gt:null,__cz:null,canHandleEvent:function(d,c){},registerEvent:function(g,f,e){},unregisterEvent:function(j,i,h){}},destruct:function(){this.__gt=this.__cz=null;},defer:function(k){qx.event.Registration.addHandler(k);}});})();(function(){var a="dblclick",b="os.name",c="mouseup",d="mousedown",e="useraction",f="webkit",g="contextmenu",h="mousewheel",i="engine.name",j="mouseover",k="mouseout",l="gecko",m="ios",n="click",o="mousemove",p="qx.event.handler.Mouse",q="on";qx.Class.define(p,{extend:qx.core.Object,implement:qx.event.IEventHandler,construct:function(r){qx.core.Object.call(this);this.__gt=r;this.__cz=r.getWindow();this.__dd=this.__cz.document;this._initButtonObserver();this._initMoveObserver();this._initWheelObserver();},statics:{PRIORITY:qx.event.Registration.PRIORITY_NORMAL,SUPPORTED_TYPES:{mousemove:1,mouseover:1,mouseout:1,mousedown:1,mouseup:1,click:1,dblclick:1,contextmenu:1,mousewheel:1},TARGET_CHECK:qx.event.IEventHandler.TARGET_DOMNODE+qx.event.IEventHandler.TARGET_DOCUMENT+qx.event.IEventHandler.TARGET_WINDOW,IGNORE_CAN_HANDLE:true},members:{__hD:null,__hE:null,__hF:null,__hG:null,__hH:null,__gt:null,__cz:null,__dd:null,__hI:null,canHandleEvent:function(t,s){},registerEvent:qx.core.Environment.get(b)===m?function(w,v,u){w[q+v]=(function(){return null;});}:(function(){return null;}),unregisterEvent:qx.core.Environment.get(b)===m?function(z,y,x){z[q+y]=undefined;}:(function(){return null;}),__hJ:function(A,B,C){if(!C){C=qx.bom.Event.getTarget(A);};if(C&&C.nodeType){qx.event.Registration.fireEvent(C,B||A.type,B==h?qx.event.type.MouseWheel:qx.event.type.Mouse,[A,C,null,true,true]);};qx.event.Registration.fireEvent(this.__cz,e,qx.event.type.Data,[B||A.type]);},preventNextClick:function(){this.__hI=true;},_initButtonObserver:function(){this.__hD=qx.lang.Function.listener(this._onButtonEvent,this);var Event=qx.bom.Event;Event.addNativeListener(this.__dd,d,this.__hD);Event.addNativeListener(this.__dd,c,this.__hD);Event.addNativeListener(this.__dd,n,this.__hD);Event.addNativeListener(this.__dd,a,this.__hD);Event.addNativeListener(this.__dd,g,this.__hD);},_initMoveObserver:function(){this.__hE=qx.lang.Function.listener(this._onMoveEvent,this);var Event=qx.bom.Event;Event.addNativeListener(this.__dd,o,this.__hE);Event.addNativeListener(this.__dd,j,this.__hE);Event.addNativeListener(this.__dd,k,this.__hE);},_initWheelObserver:function(){this.__hF=qx.lang.Function.listener(this._onWheelEvent,this);var D=qx.bom.client.Event.getMouseWheel(this.__cz);qx.bom.Event.addNativeListener(D.target,D.type,this.__hF);},_stopButtonObserver:function(){var Event=qx.bom.Event;Event.removeNativeListener(this.__dd,d,this.__hD);Event.removeNativeListener(this.__dd,c,this.__hD);Event.removeNativeListener(this.__dd,n,this.__hD);Event.removeNativeListener(this.__dd,a,this.__hD);Event.removeNativeListener(this.__dd,g,this.__hD);},_stopMoveObserver:function(){var Event=qx.bom.Event;Event.removeNativeListener(this.__dd,o,this.__hE);Event.removeNativeListener(this.__dd,j,this.__hE);Event.removeNativeListener(this.__dd,k,this.__hE);},_stopWheelObserver:function(){var E=qx.bom.client.Event.getMouseWheel(this.__cz);qx.bom.Event.removeNativeListener(E.target,E.type,this.__hF);},_onMoveEvent:qx.event.GlobalError.observeMethod(function(F){this.__hJ(F);}),_onButtonEvent:qx.event.GlobalError.observeMethod(function(I){var H=I.type;var J=qx.bom.Event.getTarget(I);if(H==n&&this.__hI){delete this.__hI;return;};if(qx.core.Environment.get(i)==l||qx.core.Environment.get(i)==f){if(J&&J.nodeType==3){J=J.parentNode;};};var G=qx.event.handler.DragDrop&&this.__gt.getHandler(qx.event.handler.DragDrop).isSessionActive();if(G&&H==n){return;};if(this.__hL){this.__hL(I,H,J);};this.__hJ(I,H,J);if(this.__hK){this.__hK(I,H,J);};if(this.__hM&&!G){this.__hM(I,H,J);};this.__hG=H;}),_onWheelEvent:qx.event.GlobalError.observeMethod(function(K){this.__hJ(K,h);}),__hK:qx.core.Environment.select(i,{"opera":function(L,M,N){if(M==c&&L.button==2){this.__hJ(L,g,N);};},"default":null}),__hL:qx.core.Environment.select(i,{"mshtml":function(O,P,Q){if(O.target!==undefined){return;};if(P==c&&this.__hG==n){this.__hJ(O,d,Q);}else if(P==a){this.__hJ(O,n,Q);};},"default":null}),__hM:qx.core.Environment.select(i,{"mshtml":null,"default":function(S,R,T){switch(R){case d:this.__hH=T;break;case c:if(T!==this.__hH){var U=qx.dom.Hierarchy.getCommonParent(T,this.__hH);if(U){this.__hJ(S,n,U);};};};}})},destruct:function(){this._stopButtonObserver();this._stopMoveObserver();this._stopWheelObserver();this.__gt=this.__cz=this.__dd=this.__hH=null;},defer:function(V){qx.event.Registration.addHandler(V);}});})();(function(){var a="qx.event.type.MouseWheel";qx.Class.define(a,{extend:qx.event.type.Mouse,members:{stop:function(){this.stopPropagation();this.preventDefault();},getWheelDelta:function(b){return qx.util.Wheel.getDelta(this._native,b);}}});})();(function(){var a="mshtml",b="engine.name",c="keypress",d="useraction",e="win",f="text",g="keyinput",h="os.name",i="webkit",j="input",k="gecko",l="off",m="keydown",n="autoComplete",o="keyup",p="qx.event.handler.Keyboard";qx.Class.define(p,{extend:qx.core.Object,implement:qx.event.IEventHandler,construct:function(q){qx.core.Object.call(this);this.__gt=q;this.__cz=q.getWindow();if((qx.core.Environment.get(b)==k)){this.__dd=this.__cz;}else {this.__dd=this.__cz.document.documentElement;};this.__hN={};this._initKeyObserver();},statics:{PRIORITY:qx.event.Registration.PRIORITY_NORMAL,SUPPORTED_TYPES:{keyup:1,keydown:1,keypress:1,keyinput:1},TARGET_CHECK:qx.event.IEventHandler.TARGET_DOMNODE,IGNORE_CAN_HANDLE:true},members:{__hO:null,__gt:null,__cz:null,__dd:null,__hN:null,__hP:null,__hQ:null,__hR:null,canHandleEvent:function(s,r){},registerEvent:function(v,u,t){},unregisterEvent:function(y,x,w){},_fireInputEvent:function(A,z){var B=this.__hS();if(B&&B.offsetWidth!=0){var event=qx.event.Registration.createEvent(g,qx.event.type.KeyInput,[A,B,z]);this.__gt.dispatchEvent(B,event);};if(this.__cz){qx.event.Registration.fireEvent(this.__cz,d,qx.event.type.Data,[g]);};},_fireSequenceEvent:function(D,F,C){var E=this.__hS();var G=D.keyCode;var event=qx.event.Registration.createEvent(F,qx.event.type.KeySequence,[D,E,C]);this.__gt.dispatchEvent(E,event);if(qx.core.Environment.get(b)==a||qx.core.Environment.get(b)==i){if(F==m&&event.getDefaultPrevented()){if(!qx.event.util.Keyboard.isNonPrintableKeyCode(G)&&!this._emulateKeyPress[G]){this._fireSequenceEvent(D,c,C);};};};if(this.__cz){qx.event.Registration.fireEvent(this.__cz,d,qx.event.type.Data,[F]);};},__hS:function(){var H=this.__gt.getHandler(qx.event.handler.Focus);var I=H.getActive();if(!I||I.offsetWidth==0){I=H.getFocus();};if(!I||I.offsetWidth==0){I=this.__gt.getWindow().document.body;};return I;},_initKeyObserver:function(){this.__hO=qx.lang.Function.listener(this.__hT,this);this.__hR=qx.lang.Function.listener(this.__hV,this);var Event=qx.bom.Event;Event.addNativeListener(this.__dd,o,this.__hO);Event.addNativeListener(this.__dd,m,this.__hO);Event.addNativeListener(this.__dd,c,this.__hR);},_stopKeyObserver:function(){var Event=qx.bom.Event;Event.removeNativeListener(this.__dd,o,this.__hO);Event.removeNativeListener(this.__dd,m,this.__hO);Event.removeNativeListener(this.__dd,c,this.__hR);for(var K in (this.__hQ||{})){var J=this.__hQ[K];Event.removeNativeListener(J.target,c,J.callback);};delete (this.__hQ);},__hT:qx.event.GlobalError.observeMethod(qx.core.Environment.select(b,{"mshtml":function(N){N=window.event||N;var O=N.keyCode;var M=0;var L=N.type;if(!(this.__hN[O]==m&&L==m)){this._idealKeyHandler(O,M,L,N);};if(L==m){if(qx.event.util.Keyboard.isNonPrintableKeyCode(O)||this._emulateKeyPress[O]){this._idealKeyHandler(O,M,c,N);};};this.__hN[O]=L;},"gecko":function(Q){var S=0;var U=Q.keyCode;var T=Q.type;var R=qx.event.util.Keyboard;if(qx.core.Environment.get(h)==e){var P=U?R.keyCodeToIdentifier(U):R.charCodeToIdentifier(S);if(!(this.__hN[P]==m&&T==m)){this._idealKeyHandler(U,S,T,Q);};this.__hN[P]=T;}else {this._idealKeyHandler(U,S,T,Q);};this.__hU(Q.target,T,U);},"webkit":function(X){var Y=0;var W=0;var V=X.type;Y=X.keyCode;this._idealKeyHandler(Y,W,V,X);if(V==m){if(qx.event.util.Keyboard.isNonPrintableKeyCode(Y)||this._emulateKeyPress[Y]){this._idealKeyHandler(Y,W,c,X);};};this.__hN[Y]=V;},"opera":function(ba){this.__hP=ba.keyCode;this._idealKeyHandler(ba.keyCode,0,ba.type,ba);}})),__hU:qx.core.Environment.select(b,{"gecko":function(bc,be,bf){if(be===m&&(bf==33||bf==34||bf==38||bf==40)&&bc.type==f&&bc.tagName.toLowerCase()===j&&bc.getAttribute(n)!==l){if(!this.__hQ){this.__hQ={};};var bb=qx.core.ObjectRegistry.toHashCode(bc);if(this.__hQ[bb]){return;};var self=this;this.__hQ[bb]={target:bc,callback:function(bg){qx.bom.Event.stopPropagation(bg);self.__hV(bg);}};var bd=qx.event.GlobalError.observeMethod(this.__hQ[bb].callback);qx.bom.Event.addNativeListener(bc,c,bd);};},"default":null}),__hV:qx.event.GlobalError.observeMethod(qx.core.Environment.select(b,{"mshtml":function(bh){bh=window.event||bh;if(this._charCode2KeyCode[bh.keyCode]){this._idealKeyHandler(this._charCode2KeyCode[bh.keyCode],0,bh.type,bh);}else {this._idealKeyHandler(0,bh.keyCode,bh.type,bh);};},"gecko":function(bi){var bj=bi.charCode;var bk=bi.type;this._idealKeyHandler(bi.keyCode,bj,bk,bi);},"webkit":function(bl){if(this._charCode2KeyCode[bl.keyCode]){this._idealKeyHandler(this._charCode2KeyCode[bl.keyCode],0,bl.type,bl);}else {this._idealKeyHandler(0,bl.keyCode,bl.type,bl);};},"opera":function(bm){var bo=bm.keyCode;var bn=bm.type;if(bo!=this.__hP){this._idealKeyHandler(0,this.__hP,bn,bm);}else {if(qx.event.util.Keyboard.keyCodeToIdentifierMap[bm.keyCode]){this._idealKeyHandler(bm.keyCode,0,bm.type,bm);}else {this._idealKeyHandler(0,bm.keyCode,bm.type,bm);};};}})),_idealKeyHandler:function(bs,bq,bt,br){var bp;if(bs||(!bs&&!bq)){bp=qx.event.util.Keyboard.keyCodeToIdentifier(bs);this._fireSequenceEvent(br,bt,bp);}else {bp=qx.event.util.Keyboard.charCodeToIdentifier(bq);this._fireSequenceEvent(br,c,bp);this._fireInputEvent(br,bq);};},_emulateKeyPress:qx.core.Environment.select(b,{"mshtml":{'8':true,'9':true},"webkit":{'8':true,'9':true,'27':true},"default":{}}),_identifierToKeyCode:function(bu){return qx.event.util.Keyboard.identifierToKeyCodeMap[bu]||bu.charCodeAt(0);}},destruct:function(){this._stopKeyObserver();this.__hP=this.__gt=this.__cz=this.__dd=this.__hN=null;},defer:function(bv,bw){qx.event.Registration.addHandler(bv);if((qx.core.Environment.get(b)==a)||qx.core.Environment.get(b)==i){bw._charCode2KeyCode={'13':13,'27':27};};}});})();(function(){var a="qx.event.type.KeyInput";qx.Class.define(a,{extend:qx.event.type.Dom,members:{init:function(c,d,b){qx.event.type.Dom.prototype.init.call(this,c,d,null,true,true);this._charCode=b;return this;},clone:function(e){var f=qx.event.type.Dom.prototype.clone.call(this,e);f._charCode=this._charCode;return f;},getCharCode:function(){return this._charCode;},getChar:function(){return String.fromCharCode(this._charCode);}}});})();(function(){var a="qx.event.type.KeySequence";qx.Class.define(a,{extend:qx.event.type.Dom,members:{init:function(c,d,b){qx.event.type.Dom.prototype.init.call(this,c,d,null,true,true);this._keyCode=c.keyCode;this._identifier=b;return this;},clone:function(e){var f=qx.event.type.Dom.prototype.clone.call(this,e);f._keyCode=this._keyCode;f._identifier=this._identifier;return f;},getKeyIdentifier:function(){return this._identifier;},getKeyCode:function(){return this._keyCode;},isPrintable:function(){return qx.event.util.Keyboard.isPrintableKeyIdentifier(this._identifier);}}});})();(function(){var a="-",b="PageUp",c="Escape",d="Enter",e="+",f="PrintScreen",g="os.name",h="7",i="A",j="Space",k="Left",l="5",m="F5",n="Down",o="Up",p="3",q="Meta",r="F11",s="0",t="F6",u="PageDown",v="osx",w="CapsLock",x="Insert",y="F8",z="Scroll",A="Control",B="Tab",C="Shift",D="End",E="Pause",F="Unidentified",G="/",H="8",I="Z",J="*",K="cmd",L="F1",M="F4",N="Home",O="qx.event.util.Keyboard",P="F2",Q="6",R="F7",S="Apps",T="4",U="F12",V="Alt",W="2",X="NumLock",Y="Delete",bn="1",bo="Win",bp="Backspace",bj="F9",bk="F10",bl="Right",bm="F3",bq="9",br=",";qx.Bootstrap.define(O,{statics:{specialCharCodeMap:{'8':bp,'9':B,'13':d,'27':c,'32':j},numpadToCharCode:{'96':s.charCodeAt(0),'97':bn.charCodeAt(0),'98':W.charCodeAt(0),'99':p.charCodeAt(0),'100':T.charCodeAt(0),'101':l.charCodeAt(0),'102':Q.charCodeAt(0),'103':h.charCodeAt(0),'104':H.charCodeAt(0),'105':bq.charCodeAt(0),'106':J.charCodeAt(0),'107':e.charCodeAt(0),'109':a.charCodeAt(0),'110':br.charCodeAt(0),'111':G.charCodeAt(0)},keyCodeToIdentifierMap:{'16':C,'17':A,'18':V,'20':w,'224':q,'37':k,'38':o,'39':bl,'40':n,'33':b,'34':u,'35':D,'36':N,'45':x,'46':Y,'112':L,'113':P,'114':bm,'115':M,'116':m,'117':t,'118':R,'119':y,'120':bj,'121':bk,'122':r,'123':U,'144':X,'44':f,'145':z,'19':E,'91':qx.core.Environment.get(g)==v?K:bo,'92':bo,'93':qx.core.Environment.get(g)==v?K:S},charCodeA:i.charCodeAt(0),charCodeZ:I.charCodeAt(0),charCode0:s.charCodeAt(0),charCode9:bq.charCodeAt(0),keyCodeToIdentifier:function(bs){if(this.isIdentifiableKeyCode(bs)){var bt=this.numpadToCharCode[bs];if(bt){return String.fromCharCode(bt);};return (this.keyCodeToIdentifierMap[bs]||this.specialCharCodeMap[bs]||String.fromCharCode(bs));}else {return F;};},charCodeToIdentifier:function(bu){return this.specialCharCodeMap[bu]||String.fromCharCode(bu).toUpperCase();},isIdentifiableKeyCode:function(bv){if(bv>=this.charCodeA&&bv<=this.charCodeZ){return true;};if(bv>=this.charCode0&&bv<=this.charCode9){return true;};if(this.specialCharCodeMap[bv]){return true;};if(this.numpadToCharCode[bv]){return true;};if(this.isNonPrintableKeyCode(bv)){return true;};return false;},isNonPrintableKeyCode:function(bw){return this.keyCodeToIdentifierMap[bw]?true:false;},isValidKeyIdentifier:function(bx){if(this.identifierToKeyCodeMap[bx]){return true;};if(bx.length!=1){return false;};if(bx>=s&&bx<=bq){return true;};if(bx>=i&&bx<=I){return true;};switch(bx){case e:case a:case J:case G:case br:return true;default:return false;};},isPrintableKeyIdentifier:function(by){if(by===j){return true;}else {return this.identifierToKeyCodeMap[by]?false:true;};}},defer:function(bz,bA){if(!bz.identifierToKeyCodeMap){bz.identifierToKeyCodeMap={};for(var bB in bz.keyCodeToIdentifierMap){bz.identifierToKeyCodeMap[bz.keyCodeToIdentifierMap[bB]]=parseInt(bB,10);};for(var bB in bz.specialCharCodeMap){bz.identifierToKeyCodeMap[bz.specialCharCodeMap[bB]]=parseInt(bB,10);};};}});})();(function(){var a="selectstart",b="os.name",c="blur",d="mousedown",e="focus",f="os.version",g="qx.event.handler.Focus",h="_applyFocus",i="DOMFocusIn",j="deactivate",k="textarea",l="_applyActive",m='character',n="input",o="ios",p="",q="qxSelectable",r="tabIndex",s="off",t="on",u="activate",v="focusin",w="mshtml",x="engine.name",y="mouseup",z="DOMFocusOut",A="focusout",B="qxKeepFocus",C="draggesture",D="qxKeepActive";qx.Class.define(g,{extend:qx.core.Object,implement:qx.event.IEventHandler,construct:function(E){qx.core.Object.call(this);this._manager=E;this._window=E.getWindow();this._document=this._window.document;this._root=this._document.documentElement;this._body=this._document.body;if((qx.core.Environment.get(b)==o&&parseFloat(qx.core.Environment.get(f))>6)&&(!qx.application.Inline||!qx.core.Init.getApplication() instanceof qx.application.Inline)){this.__hW=true;};this._initObserver();},properties:{active:{apply:l,nullable:true},focus:{apply:h,nullable:true}},statics:{PRIORITY:qx.event.Registration.PRIORITY_NORMAL,SUPPORTED_TYPES:{focus:1,blur:1,focusin:1,focusout:1,activate:1,deactivate:1},IGNORE_CAN_HANDLE:true,FOCUSABLE_ELEMENTS:qx.core.Environment.select(x,{"mshtml":{a:1,body:1,button:1,frame:1,iframe:1,img:1,input:1,object:1,select:1,textarea:1},"gecko":{a:1,body:1,button:1,frame:1,iframe:1,img:1,input:1,object:1,select:1,textarea:1},"opera":{button:1,input:1,select:1,textarea:1},"webkit":{button:1,input:1,select:1,textarea:1}})},members:{__hX:null,__hY:null,__ia:null,__ib:null,__ic:null,__id:null,__ie:null,__if:null,__ig:null,__ih:null,__ii:p,__ij:p,__hW:false,__ik:null,canHandleEvent:function(G,F){},registerEvent:function(J,I,H){},unregisterEvent:function(M,L,K){},focus:function(N){if((qx.core.Environment.get(x)==w)){window.setTimeout(function(){try{N.focus();var O=qx.bom.Selection.get(N);if(O.length==0){var P=N.createTextRange();P.moveStart(m,N.value.length);P.collapse();P.select();};}catch(Q){};},0);}else {try{N.focus();}catch(R){};};this.setFocus(N);this.setActive(N);},activate:function(S){this.setActive(S);},blur:function(T){try{T.blur();}catch(U){};if(this.getActive()===T){this.resetActive();};if(this.getFocus()===T){this.resetFocus();};},deactivate:function(V){if(this.getActive()===V){this.resetActive();};},tryActivate:function(X){var W=this.__iy(X);if(W){this.setActive(W);};},__hJ:function(Y,bb,be,bd){var bc=qx.event.Registration;var ba=bc.createEvent(be,qx.event.type.Focus,[Y,bb,bd]);bc.dispatchEvent(Y,ba);},_windowFocused:true,__il:function(){if(this._windowFocused){this._windowFocused=false;this.__hJ(this._window,null,c,false);};},__im:function(){if(!this._windowFocused){this._windowFocused=true;this.__hJ(this._window,null,e,false);};},_initObserver:qx.core.Environment.select(x,{"gecko":function(){this.__hX=qx.lang.Function.listener(this.__is,this);this.__hY=qx.lang.Function.listener(this.__it,this);this.__ia=qx.lang.Function.listener(this.__ir,this);this.__ib=qx.lang.Function.listener(this.__iq,this);this.__ic=qx.lang.Function.listener(this.__in,this);qx.bom.Event.addNativeListener(this._document,d,this.__hX,true);qx.bom.Event.addNativeListener(this._document,y,this.__hY,true);qx.bom.Event.addNativeListener(this._window,e,this.__ia,true);qx.bom.Event.addNativeListener(this._window,c,this.__ib,true);qx.bom.Event.addNativeListener(this._window,C,this.__ic,true);},"mshtml":function(){this.__hX=qx.lang.Function.listener(this.__is,this);this.__hY=qx.lang.Function.listener(this.__it,this);this.__ie=qx.lang.Function.listener(this.__io,this);this.__if=qx.lang.Function.listener(this.__ip,this);this.__id=qx.lang.Function.listener(this.__iv,this);qx.bom.Event.addNativeListener(this._document,d,this.__hX);qx.bom.Event.addNativeListener(this._document,y,this.__hY);qx.bom.Event.addNativeListener(this._document,v,this.__ie);qx.bom.Event.addNativeListener(this._document,A,this.__if);qx.bom.Event.addNativeListener(this._document,a,this.__id);},"webkit":function(){this.__hX=qx.lang.Function.listener(this.__is,this);this.__hY=qx.lang.Function.listener(this.__it,this);this.__if=qx.lang.Function.listener(this.__ip,this);this.__ia=qx.lang.Function.listener(this.__ir,this);this.__ib=qx.lang.Function.listener(this.__iq,this);this.__id=qx.lang.Function.listener(this.__iv,this);qx.bom.Event.addNativeListener(this._document,d,this.__hX,true);qx.bom.Event.addNativeListener(this._document,y,this.__hY,true);qx.bom.Event.addNativeListener(this._document,a,this.__id,false);qx.bom.Event.addNativeListener(this._window,z,this.__if,true);qx.bom.Event.addNativeListener(this._window,e,this.__ia,true);qx.bom.Event.addNativeListener(this._window,c,this.__ib,true);},"opera":function(){this.__hX=qx.lang.Function.listener(this.__is,this);this.__hY=qx.lang.Function.listener(this.__it,this);this.__ie=qx.lang.Function.listener(this.__io,this);this.__if=qx.lang.Function.listener(this.__ip,this);qx.bom.Event.addNativeListener(this._document,d,this.__hX,true);qx.bom.Event.addNativeListener(this._document,y,this.__hY,true);qx.bom.Event.addNativeListener(this._window,i,this.__ie,true);qx.bom.Event.addNativeListener(this._window,z,this.__if,true);}}),_stopObserver:qx.core.Environment.select(x,{"gecko":function(){qx.bom.Event.removeNativeListener(this._document,d,this.__hX,true);qx.bom.Event.removeNativeListener(this._document,y,this.__hY,true);qx.bom.Event.removeNativeListener(this._window,e,this.__ia,true);qx.bom.Event.removeNativeListener(this._window,c,this.__ib,true);qx.bom.Event.removeNativeListener(this._window,C,this.__ic,true);},"mshtml":function(){qx.bom.Event.removeNativeListener(this._document,d,this.__hX);qx.bom.Event.removeNativeListener(this._document,y,this.__hY);qx.bom.Event.removeNativeListener(this._document,v,this.__ie);qx.bom.Event.removeNativeListener(this._document,A,this.__if);qx.bom.Event.removeNativeListener(this._document,a,this.__id);},"webkit":function(){qx.bom.Event.removeNativeListener(this._document,d,this.__hX,true);qx.bom.Event.removeNativeListener(this._document,y,this.__hY,true);qx.bom.Event.removeNativeListener(this._document,a,this.__id,false);qx.bom.Event.removeNativeListener(this._window,z,this.__if,true);qx.bom.Event.removeNativeListener(this._window,e,this.__ia,true);qx.bom.Event.removeNativeListener(this._window,c,this.__ib,true);},"opera":function(){qx.bom.Event.removeNativeListener(this._document,d,this.__hX,true);qx.bom.Event.removeNativeListener(this._document,y,this.__hY,true);qx.bom.Event.removeNativeListener(this._window,i,this.__ie,true);qx.bom.Event.removeNativeListener(this._window,z,this.__if,true);}}),__in:qx.event.GlobalError.observeMethod(qx.core.Environment.select(x,{"gecko":function(bf){var bg=qx.bom.Event.getTarget(bf);if(!this.__iz(bg)){qx.bom.Event.preventDefault(bf);};},"default":null})),__io:qx.event.GlobalError.observeMethod(qx.core.Environment.select(x,{"mshtml":function(bi){this.__im();var bj=qx.bom.Event.getTarget(bi);var bh=this.__ix(bj);if(bh){this.setFocus(bh);};this.tryActivate(bj);},"opera":function(bk){var bl=qx.bom.Event.getTarget(bk);if(bl==this._document||bl==this._window){this.__im();if(this.__ig){this.setFocus(this.__ig);delete this.__ig;};if(this.__ih){this.setActive(this.__ih);delete this.__ih;};}else {this.setFocus(bl);this.tryActivate(bl);if(!this.__iz(bl)){bl.selectionStart=0;bl.selectionEnd=0;};};},"default":null})),__ip:qx.event.GlobalError.observeMethod(qx.core.Environment.select(x,{"mshtml":function(bm){var bn=qx.bom.Event.getRelatedTarget(bm);if(bn==null){this.__il();this.resetFocus();this.resetActive();};},"webkit":function(bo){var bp=qx.bom.Event.getTarget(bo);if(bp===this.getFocus()){this.resetFocus();};if(bp===this.getActive()){this.resetActive();};},"opera":function(bq){var br=qx.bom.Event.getTarget(bq);if(br==this._document){this.__il();this.__ig=this.getFocus();this.__ih=this.getActive();this.resetFocus();this.resetActive();}else {if(br===this.getFocus()){this.resetFocus();};if(br===this.getActive()){this.resetActive();};};},"default":null})),__iq:qx.event.GlobalError.observeMethod(qx.core.Environment.select(x,{"gecko":function(bs){var bt=qx.bom.Event.getTarget(bs);if(bt===this._window||bt===this._document){this.__il();this.resetActive();this.resetFocus();};},"webkit":function(bu){var bv=qx.bom.Event.getTarget(bu);if(bv===this._window||bv===this._document){this.__il();this.__ig=this.getFocus();this.__ih=this.getActive();this.resetActive();this.resetFocus();};},"default":null})),__ir:qx.event.GlobalError.observeMethod(qx.core.Environment.select(x,{"gecko":function(bw){var bx=qx.bom.Event.getTarget(bw);if(bx===this._window||bx===this._document){this.__im();bx=this._body;};this.setFocus(bx);this.tryActivate(bx);},"webkit":function(by){var bz=qx.bom.Event.getTarget(by);if(bz===this._window||bz===this._document){this.__im();if(this.__ig){this.setFocus(this.__ig);delete this.__ig;};if(this.__ih){this.setActive(this.__ih);delete this.__ih;};}else {this.__ik=by.relatedTarget;this.setFocus(bz);this.__ik=null;this.tryActivate(bz);};},"default":null})),__is:qx.event.GlobalError.observeMethod(qx.core.Environment.select(x,{"mshtml":function(bB){var bC=qx.bom.Event.getTarget(bB);var bA=this.__ix(bC);if(bA){if(!this.__iz(bC)){bC.unselectable=t;try{document.selection.empty();}catch(bD){};try{bA.focus();}catch(bE){};};}else {qx.bom.Event.preventDefault(bB);if(!this.__iz(bC)){bC.unselectable=t;};};},"webkit":function(bG){var bH=qx.bom.Event.getTarget(bG);var bF=this.__ix(bH);if(bF){this.setFocus(bF);}else {qx.bom.Event.preventDefault(bG);};},"gecko":function(bJ){var bK=qx.bom.Event.getTarget(bJ);var bI=this.__ix(bK);if(bI){this.setFocus(bI);}else {qx.bom.Event.preventDefault(bJ);};},"opera":function(bN){var bO=qx.bom.Event.getTarget(bN);var bL=this.__ix(bO);if(!this.__iz(bO)){qx.bom.Event.preventDefault(bN);if(bL){var bM=this.getFocus();if(bM&&bM.selectionEnd){bM.selectionStart=0;bM.selectionEnd=0;bM.blur();};if(bL){this.setFocus(bL);};};}else if(bL){this.setFocus(bL);};},"default":null})),__it:qx.event.GlobalError.observeMethod(qx.core.Environment.select(x,{"mshtml":function(bP){var bQ=qx.bom.Event.getTarget(bP);if(bQ.unselectable){bQ.unselectable=s;};this.tryActivate(this.__iu(bQ));},"gecko":function(bR){var bS=qx.bom.Event.getTarget(bR);while(bS&&bS.offsetWidth===undefined){bS=bS.parentNode;};if(bS){this.tryActivate(bS);};},"webkit":function(bT){var bU=qx.bom.Event.getTarget(bT);this.tryActivate(this.__iu(bU));},"opera":function(bV){var bW=qx.bom.Event.getTarget(bV);this.tryActivate(this.__iu(bW));},"default":null})),__iu:qx.event.GlobalError.observeMethod(qx.core.Environment.select(x,{"mshtml":function(bX){var bY=this.getFocus();if(bY&&bX!=bY&&(bY.nodeName.toLowerCase()===n||bY.nodeName.toLowerCase()===k)){bX=bY;};return bX;},"webkit":function(ca){var cb=this.getFocus();if(cb&&ca!=cb&&(cb.nodeName.toLowerCase()===n||cb.nodeName.toLowerCase()===k)){ca=cb;};return ca;},"default":function(cc){return cc;}})),__iv:qx.event.GlobalError.observeMethod(qx.core.Environment.select(x,{"mshtml":function(cd){var ce=qx.bom.Event.getTarget(cd);if(!this.__iz(ce)){qx.bom.Event.preventDefault(cd);};},"webkit":function(cf){var cg=qx.bom.Event.getTarget(cf);if(!this.__iz(cg)){qx.bom.Event.preventDefault(cf);};},"default":null})),__iw:function(ch){var ci=qx.bom.element.Attribute.get(ch,r);if(ci>=1){return true;};var cj=qx.event.handler.Focus.FOCUSABLE_ELEMENTS;if(ci>=0&&cj[ch.tagName]){return true;};return false;},__ix:function(ck){while(ck&&ck.nodeType===1){if(ck.getAttribute(B)==t){return null;};if(this.__iw(ck)){return ck;};ck=ck.parentNode;};return this._body;},__iy:function(cl){var cm=cl;while(cl&&cl.nodeType===1){if(cl.getAttribute(D)==t){return null;};cl=cl.parentNode;};return cm;},__iz:function(cn){while(cn&&cn.nodeType===1){var co=cn.getAttribute(q);if(co!=null){return co===t;};cn=cn.parentNode;};return true;},_applyActive:function(cq,cp){if(cp){this.__hJ(cp,cq,j,true);};if(cq){this.__hJ(cq,cp,u,true);};if(this.__hW){window.scrollTo(0,0);};},_applyFocus:function(cs,cr){if(cr){this.__hJ(cr,cs,A,true);};if(cs){this.__hJ(cs,cr,v,true);};if(cr){this.__hJ(cr,cs,c,false);};if(cs){this.__hJ(cs,cr||this.__ik,e,false);};}},destruct:function(){this._stopObserver();this._manager=this._window=this._document=this._root=this._body=this.__iA=this.__ik=null;},defer:function(cu){qx.event.Registration.addHandler(cu);var cv=cu.FOCUSABLE_ELEMENTS;for(var ct in cv){cv[ct.toUpperCase()]=1;};}});})();(function(){var a="engine.name",b="qx.bom.Selection",c="character",d="button",e='character',f="#text",g="webkit",h="input",i="gecko",j="EndToEnd",k="opera",l="StartToStart",m="html.selection",n="textarea",o="body";qx.Bootstrap.define(b,{statics:{getSelectionObject:qx.core.Environment.select(m,{"selection":function(p){return p.selection;},"default":function(q){return qx.dom.Node.getWindow(q).getSelection();}}),get:qx.core.Environment.select(m,{"selection":function(r){var s=qx.bom.Range.get(qx.dom.Node.getDocument(r));return s.text;},"default":function(t){if(this.__iB(t)){return t.value.substring(t.selectionStart,t.selectionEnd);}else {return this.getSelectionObject(qx.dom.Node.getDocument(t)).toString();};}}),getLength:qx.core.Environment.select(m,{"selection":function(u){var w=this.get(u);var v=qx.util.StringSplit.split(w,/\r\n/);return w.length-(v.length-1);},"default":function(x){if(qx.core.Environment.get(a)==k){var B,C,A;if(this.__iB(x)){var z=x.selectionStart;var y=x.selectionEnd;B=x.value.substring(z,y);C=y-z;}else {B=qx.bom.Selection.get(x);C=B.length;};A=qx.util.StringSplit.split(B,/\r\n/);return C-(A.length-1);};if(this.__iB(x)){return x.selectionEnd-x.selectionStart;}else {return this.get(x).length;};}}),getStart:qx.core.Environment.select(m,{"selection":function(D){if(this.__iB(D)){var I=qx.bom.Range.get();if(!D.contains(I.parentElement())){return -1;};var J=qx.bom.Range.get(D);var H=D.value.length;J.moveToBookmark(I.getBookmark());J.moveEnd(e,H);return H-J.text.length;}else {var J=qx.bom.Range.get(D);var F=J.parentElement();var K=qx.bom.Range.get();try{K.moveToElementText(F);}catch(M){return 0;};var E=qx.bom.Range.get(qx.dom.Node.getBodyElement(D));E.setEndPoint(l,J);E.setEndPoint(j,K);if(K.compareEndPoints(l,E)==0){return 0;};var G;var L=0;while(true){G=E.moveStart(c,-1);if(K.compareEndPoints(l,E)==0){break;};if(G==0){break;}else {L++ ;};};return  ++L;};},"default":function(N){if(qx.core.Environment.get(a)===i||qx.core.Environment.get(a)===g){if(this.__iB(N)){return N.selectionStart;}else {var P=qx.dom.Node.getDocument(N);var O=this.getSelectionObject(P);if(O.anchorOffset<O.focusOffset){return O.anchorOffset;}else {return O.focusOffset;};};};if(this.__iB(N)){return N.selectionStart;}else {return qx.bom.Selection.getSelectionObject(qx.dom.Node.getDocument(N)).anchorOffset;};}}),getEnd:qx.core.Environment.select(m,{"selection":function(Q){if(this.__iB(Q)){var V=qx.bom.Range.get();if(!Q.contains(V.parentElement())){return -1;};var W=qx.bom.Range.get(Q);var U=Q.value.length;W.moveToBookmark(V.getBookmark());W.moveStart(e,-U);return W.text.length;}else {var W=qx.bom.Range.get(Q);var S=W.parentElement();var X=qx.bom.Range.get();try{X.moveToElementText(S);}catch(ba){return 0;};var U=X.text.length;var R=qx.bom.Range.get(qx.dom.Node.getBodyElement(Q));R.setEndPoint(j,W);R.setEndPoint(l,X);if(X.compareEndPoints(j,R)==0){return U-1;};var T;var Y=0;while(true){T=R.moveEnd(c,1);if(X.compareEndPoints(j,R)==0){break;};if(T==0){break;}else {Y++ ;};};return U-( ++Y);};},"default":function(bb){if(qx.core.Environment.get(a)===i||qx.core.Environment.get(a)===g){if(this.__iB(bb)){return bb.selectionEnd;}else {var bd=qx.dom.Node.getDocument(bb);var bc=this.getSelectionObject(bd);if(bc.focusOffset>bc.anchorOffset){return bc.focusOffset;}else {return bc.anchorOffset;};};};if(this.__iB(bb)){return bb.selectionEnd;}else {return qx.bom.Selection.getSelectionObject(qx.dom.Node.getDocument(bb)).focusOffset;};}}),__iB:function(be){return qx.dom.Node.isElement(be)&&(be.nodeName.toLowerCase()==h||be.nodeName.toLowerCase()==n);},set:qx.core.Environment.select(m,{"selection":function(bf,bi,bh){var bg;if(qx.dom.Node.isDocument(bf)){bf=bf.body;};if(qx.dom.Node.isElement(bf)||qx.dom.Node.isText(bf)){switch(bf.nodeName.toLowerCase()){case h:case n:case d:if(bh===undefined){bh=bf.value.length;};if(bi>=0&&bi<=bf.value.length&&bh>=0&&bh<=bf.value.length){bg=qx.bom.Range.get(bf);bg.collapse(true);bg.moveStart(c,bi);bg.moveEnd(c,bh-bi);bg.select();return true;};break;case f:if(bh===undefined){bh=bf.nodeValue.length;};if(bi>=0&&bi<=bf.nodeValue.length&&bh>=0&&bh<=bf.nodeValue.length){bg=qx.bom.Range.get(qx.dom.Node.getBodyElement(bf));bg.moveToElementText(bf.parentNode);bg.collapse(true);bg.moveStart(c,bi);bg.moveEnd(c,bh-bi);bg.select();return true;};break;default:if(bh===undefined){bh=bf.childNodes.length-1;};if(bf.childNodes[bi]&&bf.childNodes[bh]){bg=qx.bom.Range.get(qx.dom.Node.getBodyElement(bf));bg.moveToElementText(bf.childNodes[bi]);bg.collapse(true);var bj=qx.bom.Range.get(qx.dom.Node.getBodyElement(bf));bj.moveToElementText(bf.childNodes[bh]);bg.setEndPoint(j,bj);bg.select();return true;};};};return false;},"default":function(bk,bp,bm){var bn=bk.nodeName.toLowerCase();if(qx.dom.Node.isElement(bk)&&(bn==h||bn==n)){if(bm===undefined){bm=bk.value.length;};if(bp>=0&&bp<=bk.value.length&&bm>=0&&bm<=bk.value.length){bk.focus();bk.select();bk.setSelectionRange(bp,bm);return true;};}else {var bq=false;var bl=qx.dom.Node.getWindow(bk).getSelection();var bo=qx.bom.Range.get(bk);if(qx.dom.Node.isText(bk)){if(bm===undefined){bm=bk.length;};if(bp>=0&&bp<bk.length&&bm>=0&&bm<=bk.length){bq=true;};}else if(qx.dom.Node.isElement(bk)){if(bm===undefined){bm=bk.childNodes.length-1;};if(bp>=0&&bk.childNodes[bp]&&bm>=0&&bk.childNodes[bm]){bq=true;};}else if(qx.dom.Node.isDocument(bk)){bk=bk.body;if(bm===undefined){bm=bk.childNodes.length-1;};if(bp>=0&&bk.childNodes[bp]&&bm>=0&&bk.childNodes[bm]){bq=true;};};if(bq){if(!bl.isCollapsed){bl.collapseToStart();};bo.setStart(bk,bp);if(qx.dom.Node.isText(bk)){bo.setEnd(bk,bm);}else {bo.setEndAfter(bk.childNodes[bm]);};if(bl.rangeCount>0){bl.removeAllRanges();};bl.addRange(bo);return true;};};return false;}}),setAll:function(br){return qx.bom.Selection.set(br,0);},clear:qx.core.Environment.select(m,{"selection":function(bs){var bu=qx.bom.Range.get(bs);var parent=bu.parentElement();var bv=qx.bom.Range.get(qx.dom.Node.getDocument(bs));if(qx.dom.Node.isText(bs)){bs=bs.parentNode;};if(parent==bv.parentElement()&&parent==bs){var bt=qx.bom.Selection.getSelectionObject(qx.dom.Node.getDocument(bs));bt.empty();};},"default":function(bw){var bB=qx.bom.Selection.getSelectionObject(qx.dom.Node.getDocument(bw));var bx=bw.nodeName.toLowerCase();if(qx.dom.Node.isElement(bw)&&(bx==h||bx==n)){bw.setSelectionRange(0,0);if(qx.bom.Element&&qx.bom.Element.blur){qx.bom.Element.blur(bw);};}else if(qx.dom.Node.isDocument(bw)||bx==o){bB.collapse(bw.body?bw.body:bw,0);}else {var by=qx.bom.Range.get(bw);if(!by.collapsed){var bz;var bA=by.commonAncestorContainer;if(qx.dom.Node.isElement(bw)&&qx.dom.Node.isText(bA)){bz=bA.parentNode;}else {bz=bA;};if(bz==bw){bB.collapse(bw,0);};};};}})}});})();(function(){var a="qx.bom.Range",b="text",c="password",d="file",e="submit",f="reset",g="textarea",h="input",i="hidden",j="html.selection",k="button",l="body";qx.Bootstrap.define(a,{statics:{get:qx.core.Environment.select(j,{"selection":function(m){if(qx.dom.Node.isElement(m)){switch(m.nodeName.toLowerCase()){case h:switch(m.type){case b:case c:case i:case k:case f:case d:case e:return m.createTextRange();default:return qx.bom.Selection.getSelectionObject(qx.dom.Node.getDocument(m)).createRange();};break;case g:case l:case k:return m.createTextRange();default:return qx.bom.Selection.getSelectionObject(qx.dom.Node.getDocument(m)).createRange();};}else {if(m==null){m=window;};return qx.bom.Selection.getSelectionObject(qx.dom.Node.getDocument(m)).createRange();};},"default":function(n){var o=qx.dom.Node.getDocument(n);var p=qx.bom.Selection.getSelectionObject(o);if(p.rangeCount>0){return p.getRangeAt(0);}else {return o.createRange();};}})}});})();(function(){var a="m",b="g",c="^",d="",e="qx.util.StringSplit",f="i",g="$(?!\\s)",h="[object RegExp]",j="y";qx.Bootstrap.define(e,{statics:{split:function(k,p,o){if(Object.prototype.toString.call(p)!==h){return String.prototype.split.call(k,p,o);};var r=[],l=0,m=(p.ignoreCase?f:d)+(p.multiline?a:d)+(p.sticky?j:d),p=RegExp(p.source,m+b),n,t,q,u,s=/()??/.exec(d)[1]===undefined;k=k+d;if(!s){n=RegExp(c+p.source+g,m);};if(o===undefined||+o<0){o=Infinity;}else {o=Math.floor(+o);if(!o){return [];};};while(t=p.exec(k)){q=t.index+t[0].length;if(q>l){r.push(k.slice(l,t.index));if(!s&&t.length>1){t[0].replace(n,function(){for(var i=1;i<arguments.length-2;i++ ){if(arguments[i]===undefined){t[i]=undefined;};};});};if(t.length>1&&t.index<k.length){Array.prototype.push.apply(r,t.slice(1));};u=t[0].length;l=q;if(r.length>=o){break;};};if(p.lastIndex===t.index){p.lastIndex++ ;};};if(l===k.length){if(u||!p.test(d)){r.push(d);};}else {r.push(k.slice(l));};return r.length>o?r.slice(0,o):r;}}});})();(function(){var a="qx.event.type.Focus";qx.Class.define(a,{extend:qx.event.type.Event,members:{init:function(d,b,c){qx.event.type.Event.prototype.init.call(this,c,false);this._target=d;this._relatedTarget=b;return this;}}});})();(function(){var a="touchmove",b="os.name",c="MSPointerDown",d="android",e="engine.version",f="pointercancel",g="qx.event.handler.TouchCore",h="event.mspointer",j="MSPointerCancel",k="y",l="pointer-events",m="pointerup",n="touchend",o="pointerdown",p="MSPointerUp",q="right",r="engine.name",s="undefined",t="touchcancel",u="MSPointerMove",v="webkit",w="none",z="left",A="pointermove",B="down",C="x",D="up",E="touchstart";qx.Bootstrap.define(g,{extend:Object,statics:{TAP_MAX_DISTANCE:qx.core.Environment.get(b)!=d?10:40,SWIPE_DIRECTION:{x:[z,q],y:[D,B]},SWIPE_MIN_DISTANCE:qx.core.Environment.get(b)!=d?11:41,SWIPE_MIN_VELOCITY:0,LONGTAP_TIME:500},construct:function(F,G){this.__iC=F;this.__fB=G;this._initTouchObserver();this.__iD=[];this.__iE={};},members:{__iC:null,__fB:null,__iF:null,__iG:null,__iE:null,__iH:null,__iI:null,__iJ:null,__iD:null,__iK:null,_initTouchObserver:function(){this.__iF=qx.lang.Function.listener(this._onTouchEvent,this);this.__iK=[E,a,n,t];if(qx.core.Environment.get(h)){var H=parseInt(qx.core.Environment.get(e),10);if(H==10){this.__iK=[c,u,p,j];}else {this.__iK=[o,A,m,f];};};for(var i=0;i<this.__iK.length;i++ ){qx.bom.Event.addNativeListener(this.__iC,this.__iK[i],this.__iF);};},_stopTouchObserver:function(){for(var i=0;i<this.__iK.length;i++ ){qx.bom.Event.removeNativeListener(this.__iC,this.__iK[i],this.__iF);};},_onTouchEvent:function(I){this._commonTouchEventHandler(I);},_getScalingDistance:function(K,J){return (Math.sqrt(Math.pow(K.pageX-J.pageX,2)+Math.pow(K.pageY-J.pageY,2)));},_getRotationAngle:function(M,L){var x=M.pageX-L.pageX;var y=M.pageY-L.pageY;return (Math.atan2(y,x)*180/Math.PI);},_calcTouchesDelta:function(N){var O=[];for(var i=0;i<N.length;i++ ){O.push(this._calcSingleTouchDelta(N[i]));};return O;},_calcSingleTouchDelta:function(S){if(this.__iE.hasOwnProperty(S.identifier)){var R=this.__iE[S.identifier];var P=Math.floor(S.clientX-R[0]);var Q=Math.floor(S.clientY-R[1]);var T=C;if(Math.abs(P/Q)<1){T=k;};return {"x":P,"y":Q,"axis":T,"identifier":S.identifier};}else {return {"x":0,"y":0,"axis":null,"identifier":S.identifier};};},_commonTouchEventHandler:function(V,ba){var ba=ba||V.type;if(qx.core.Environment.get(h)){ba=this._mapPointerEvent(ba);var U=this._detectTouchesByPointer(V,ba);V.changedTouches=U;V.targetTouches=U;V.touches=U;};V.delta=[];if(ba==E){this.__iG=this._getTarget(V);if(V.touches&&V.touches.length>1){this.__iI=this._getScalingDistance(V.touches[0],V.touches[1]);this.__iJ=this._getRotationAngle(V.touches[0],V.touches[1]);};for(var i=0;i<V.changedTouches.length;i++ ){var Y=V.changedTouches[i];this.__iE[Y.identifier]=[Y.clientX,Y.clientY];};};if(ba==a){if(typeof V.scale==s&&V.targetTouches.length>1){var W=this._getScalingDistance(V.targetTouches[0],V.targetTouches[1]);V.scale=W/this.__iI;};if((typeof V.rotation==s||qx.core.Environment.get(h))&&V.targetTouches.length>1){var X=this._getRotationAngle(V.targetTouches[0],V.targetTouches[1]);V._rotation=X-this.__iJ;};V.delta=this._calcTouchesDelta(V.targetTouches);};this._fireEvent(V,ba,this.__iG);if(qx.core.Environment.get(h)){if(ba==n||ba==t){delete this.__iD[V.pointerId];};};if((ba==n||ba==t)&&V.changedTouches[0]){delete this.__iE[V.changedTouches[0].identifier];};},_detectTouchesByPointer:function(bd,bf){var bc=[];if(bf==E){this.__iD[bd.pointerId]=bd;}else if(bf==a){this.__iD[bd.pointerId]=bd;};for(var be in this.__iD){var bb=this.__iD[be];bc.push(bb);};return bc;},_mapPointerEvent:function(bg){bg=bg.toLowerCase();if(bg.indexOf(o)!==-1){return E;}else if(bg.indexOf(m)!==-1){return n;}else if(bg.indexOf(A)!==-1){return a;}else if(bg.indexOf(f)!==-1){return t;};return bg;},_getTarget:function(bi){var bj=qx.bom.Event.getTarget(bi);if(qx.core.Environment.get(r)==v){if(bj&&bj.nodeType==3){bj=bj.parentNode;};}else if(qx.core.Environment.get(h)){var bh=this.__iL(bi);if(bh){bj=bh;};};return bj;},__iL:function(bm){var bk=null;var bl=null;if(bm&&bm.touches&&bm.touches.length!==0){bk=bm.touches[0].clientX;bl=bm.touches[0].clientY;};var bo=document.msElementsFromPoint(bk,bl);if(bo){for(var i=0;i<bo.length;i++ ){var bp=bo[i];var bn=qx.bom.element.Style.get(bp,l,3);if(bn!=w){return bp;};};};return null;},_fireEvent:function(bq,br,bs){if(!bs){bs=this._getTarget(bq);};var br=br||bq.type;if(bs&&bs.nodeType&&this.__fB){this.__fB.emit(br,bq);};},dispose:function(){this._stopTouchObserver();this.__iG=this.__iC=this.__iK=this.__iD=this.__fB=this.__iI=this.__iJ=null;}}});})();(function(){var a="resize",b="os.name",c="qx.event.handler.Orientation",d="landscape",e="android",f="portrait",g="orientationchange";qx.Class.define(c,{extend:qx.core.Object,implement:qx.event.IEventHandler,construct:function(h){qx.core.Object.call(this);this.__gt=h;this.__cz=h.getWindow();this._initObserver();},statics:{PRIORITY:qx.event.Registration.PRIORITY_NORMAL,SUPPORTED_TYPES:{orientationchange:1},TARGET_CHECK:qx.event.IEventHandler.TARGET_WINDOW,IGNORE_CAN_HANDLE:true},members:{__gt:null,__cz:null,__iM:null,_currentOrientation:null,__iN:null,canHandleEvent:function(j,i){},registerEvent:function(m,l,k){},unregisterEvent:function(p,o,n){},_initObserver:function(){this.__iN=qx.lang.Function.listener(this._onNative,this);this.__iM=qx.bom.Event.supportsEvent(this.__cz,g)?g:a;var Event=qx.bom.Event;Event.addNativeListener(this.__cz,this.__iM,this.__iN);},_stopObserver:function(){var Event=qx.bom.Event;Event.removeNativeListener(this.__cz,this.__iM,this.__iN);},_onNative:qx.event.GlobalError.observeMethod(function(q){var r=0;if(qx.core.Environment.get(b)==e){r=300;};qx.lang.Function.delay(this._onOrientationChange,r,this,q);}),_onOrientationChange:function(s){var u=qx.bom.Viewport;var t=u.getOrientation(s.target);if(this._currentOrientation!=t){this._currentOrientation=t;var v=u.isLandscape(s.target)?d:f;qx.event.Registration.fireEvent(this.__cz,g,qx.event.type.Orientation,[t,v]);};}},destruct:function(){this._stopObserver();this.__gt=this.__cz=null;},defer:function(w){qx.event.Registration.addHandler(w);}});})();(function(){var a="landscape",b="qx.event.type.Orientation",c="portrait";qx.Class.define(b,{extend:qx.event.type.Event,members:{__iO:null,__iP:null,init:function(d,e){qx.event.type.Event.prototype.init.call(this,false,false);this.__iO=d;this.__iP=e;return this;},clone:function(f){var g=qx.event.type.Event.prototype.clone.call(this,f);g.__iO=this.__iO;g.__iP=this.__iP;return g;},getOrientation:function(){return this.__iO;},isLandscape:function(){return this.__iP==a;},isPortrait:function(){return this.__iP==c;}}});})();(function(){var a="touchmove",b="dispose",c="useraction",d="touchend",e="event.touch",f="touchstart",g="qx.event.handler.Touch";qx.Class.define(g,{extend:qx.event.handler.TouchCore,implement:qx.event.IEventHandler,construct:function(h){this.__gt=h;this.__cz=h.getWindow();this.__dd=this.__cz.document;qx.event.handler.TouchCore.apply(this,[this.__dd]);},statics:{PRIORITY:qx.event.Registration.PRIORITY_NORMAL,SUPPORTED_TYPES:{touchstart:1,touchmove:1,touchend:1,touchcancel:1,tap:1,longtap:1,swipe:1},TARGET_CHECK:qx.event.IEventHandler.TARGET_DOMNODE+qx.event.IEventHandler.TARGET_DOCUMENT,IGNORE_CAN_HANDLE:true,MOUSE_TO_TOUCH_MAPPING:{"mousedown":f,"mousemove":a,"mouseup":d}},members:{__gt:null,__cz:null,__dd:null,__iQ:false,canHandleEvent:function(j,i){},registerEvent:function(m,l,k){},unregisterEvent:function(p,o,n){},_fireEvent:function(r,q,s,t){if(!s){s=this._getTarget(r);};var q=q||r.type;if(s&&s.nodeType){qx.event.Registration.fireEvent(s,q,t||qx.event.type.Touch,[r,s,null,true,true]);};qx.event.Registration.fireEvent(this.__cz,c,qx.event.type.Data,[q]);},_onTouchEvent:qx.event.GlobalError.observeMethod(function(u){this._commonTouchEventHandler(u);}),dispose:function(){this.__hh(b);this.__gt=this.__cz=this.__dd=null;},__hh:function(w,v){qx.event.handler.TouchCore.prototype[w].apply(this,v||[]);}},defer:function(x){qx.event.Registration.addHandler(x);if(qx.core.Environment.get(e)){qx.event.Registration.getManager(document).getHandler(x);};}});})();(function(){var a="touchcancel",b="qx.event.type.Touch",c="touchend",d="undefined";qx.Class.define(b,{extend:qx.event.type.Dom,members:{_cloneNativeEvent:function(e,f){var f=qx.event.type.Dom.prototype._cloneNativeEvent.call(this,e,f);f.pageX=e.pageX;f.pageY=e.pageY;f.offsetX=e.offsetX;f.offsetY=e.offsetY;f.layerX=(e.offsetX||e.layerX);f.layerY=(e.offsetY||e.layerY);f.scale=e.scale;f.rotation=e.rotation;f._rotation=e._rotation;f.delta=e.delta;f.srcElement=e.srcElement;f.targetTouches=[];for(var i=0;i<e.targetTouches.length;i++ ){f.targetTouches[i]=e.targetTouches[i];};f.changedTouches=[];for(i=0;i<e.changedTouches.length;i++ ){f.changedTouches[i]=e.changedTouches[i];};f.touches=[];for(i=0;i<e.touches.length;i++ ){f.touches[i]=e.touches[i];};return f;},stop:function(){this.stopPropagation();},getAllTouches:function(){return this._native.touches;},getTargetTouches:function(){return this._native.targetTouches;},getChangedTargetTouches:function(){return this._native.changedTouches;},isMultiTouch:function(){return this.__iS().length>1;},getScale:function(){return this._native.scale;},getRotation:function(){if(typeof this._native._rotation===d){return this._native.rotation;}else {return this._native._rotation;};},getDelta:function(){return this._native.delta;},getDocumentLeft:function(g){return this.__iR(g).pageX;},getDocumentTop:function(h){return this.__iR(h).pageY;},getScreenLeft:function(j){return this.__iR(j).screenX;},getScreenTop:function(k){return this.__iR(k).screenY;},getViewportLeft:function(l){return this.__iR(l).clientX;},getViewportTop:function(m){return this.__iR(m).clientY;},getIdentifier:function(n){return this.__iR(n).identifier;},__iR:function(o){o=o==null?0:o;return this.__iS()[o];},__iS:function(){var p=(this._isTouchEnd()?this.getChangedTargetTouches():this.getTargetTouches());return p;},_isTouchEnd:function(){return (this.getType()==c||this.getType()==a);}}});})();(function(){var a="text",b="engine.version",c="keydown",d="radio",f="textarea",g="password",h="propertychange",j="select-multiple",k="change",m="input",n="value",p="select",q="browser.documentmode",r="browser.version",s="opera",t="keyup",u="mshtml",v="engine.name",w="keypress",x="checkbox",y="qx.event.handler.Input",z="checked";qx.Class.define(y,{extend:qx.core.Object,implement:qx.event.IEventHandler,construct:function(){qx.core.Object.call(this);this._onChangeCheckedWrapper=qx.lang.Function.listener(this._onChangeChecked,this);this._onChangeValueWrapper=qx.lang.Function.listener(this._onChangeValue,this);this._onInputWrapper=qx.lang.Function.listener(this._onInput,this);this._onPropertyWrapper=qx.lang.Function.listener(this._onProperty,this);if((qx.core.Environment.get(v)==s)){this._onKeyDownWrapper=qx.lang.Function.listener(this._onKeyDown,this);this._onKeyUpWrapper=qx.lang.Function.listener(this._onKeyUp,this);};},statics:{PRIORITY:qx.event.Registration.PRIORITY_NORMAL,SUPPORTED_TYPES:{input:1,change:1},TARGET_CHECK:qx.event.IEventHandler.TARGET_DOMNODE,IGNORE_CAN_HANDLE:false},members:{__iT:false,__iU:null,__iV:null,__iW:null,canHandleEvent:function(C,B){var A=C.tagName.toLowerCase();if(B===m&&(A===m||A===f)){return true;};if(B===k&&(A===m||A===f||A===p)){return true;};return false;},registerEvent:function(H,G,E){if(qx.core.Environment.get(v)==u&&(qx.core.Environment.get(b)<9||(qx.core.Environment.get(b)>=9&&qx.core.Environment.get(q)<9))){if(!H.__iX){var F=H.tagName.toLowerCase();var D=H.type;if(D===a||D===g||F===f||D===x||D===d){qx.bom.Event.addNativeListener(H,h,this._onPropertyWrapper);};if(D!==x&&D!==d){qx.bom.Event.addNativeListener(H,k,this._onChangeValueWrapper);};if(D===a||D===g){this._onKeyPressWrapped=qx.lang.Function.listener(this._onKeyPress,this,H);qx.bom.Event.addNativeListener(H,w,this._onKeyPressWrapped);};H.__iX=true;};}else {if(G===m){this.__iY(H);}else if(G===k){if(H.type===d||H.type===x){qx.bom.Event.addNativeListener(H,k,this._onChangeCheckedWrapper);}else {qx.bom.Event.addNativeListener(H,k,this._onChangeValueWrapper);};if((qx.core.Environment.get(v)==s)||(qx.core.Environment.get(v)==u)){if(H.type===a||H.type===g){this._onKeyPressWrapped=qx.lang.Function.listener(this._onKeyPress,this,H);qx.bom.Event.addNativeListener(H,w,this._onKeyPressWrapped);};};};};},__iY:qx.core.Environment.select(v,{"mshtml":function(I){if(qx.core.Environment.get(b)>=9&&qx.core.Environment.get(q)>=9){qx.bom.Event.addNativeListener(I,m,this._onInputWrapper);if(I.type===a||I.type===g||I.type===f){this._inputFixWrapper=qx.lang.Function.listener(this._inputFix,this,I);qx.bom.Event.addNativeListener(I,t,this._inputFixWrapper);};};},"webkit":function(K){var J=K.tagName.toLowerCase();if(parseFloat(qx.core.Environment.get(b))<532&&J==f){qx.bom.Event.addNativeListener(K,w,this._onInputWrapper);};qx.bom.Event.addNativeListener(K,m,this._onInputWrapper);},"opera":function(L){qx.bom.Event.addNativeListener(L,t,this._onKeyUpWrapper);qx.bom.Event.addNativeListener(L,c,this._onKeyDownWrapper);qx.bom.Event.addNativeListener(L,m,this._onInputWrapper);},"default":function(M){qx.bom.Event.addNativeListener(M,m,this._onInputWrapper);}}),unregisterEvent:function(Q,P){if(qx.core.Environment.get(v)==u&&qx.core.Environment.get(b)<9&&qx.core.Environment.get(q)<9){if(Q.__iX){var O=Q.tagName.toLowerCase();var N=Q.type;if(N===a||N===g||O===f||N===x||N===d){qx.bom.Event.removeNativeListener(Q,h,this._onPropertyWrapper);};if(N!==x&&N!==d){qx.bom.Event.removeNativeListener(Q,k,this._onChangeValueWrapper);};if(N===a||N===g){qx.bom.Event.removeNativeListener(Q,w,this._onKeyPressWrapped);};try{delete Q.__iX;}catch(R){Q.__iX=null;};};}else {if(P===m){this.__ja(Q);}else if(P===k){if(Q.type===d||Q.type===x){qx.bom.Event.removeNativeListener(Q,k,this._onChangeCheckedWrapper);}else {qx.bom.Event.removeNativeListener(Q,k,this._onChangeValueWrapper);};};if((qx.core.Environment.get(v)==s)||(qx.core.Environment.get(v)==u)){if(Q.type===a||Q.type===g){qx.bom.Event.removeNativeListener(Q,w,this._onKeyPressWrapped);};};};},__ja:qx.core.Environment.select(v,{"mshtml":function(S){if(qx.core.Environment.get(b)>=9&&qx.core.Environment.get(q)>=9){qx.bom.Event.removeNativeListener(S,m,this._onInputWrapper);if(S.type===a||S.type===g||S.type===f){qx.bom.Event.removeNativeListener(S,t,this._inputFixWrapper);};};},"webkit":function(U){var T=U.tagName.toLowerCase();if(parseFloat(qx.core.Environment.get(b))<532&&T==f){qx.bom.Event.removeNativeListener(U,w,this._onInputWrapper);};qx.bom.Event.removeNativeListener(U,m,this._onInputWrapper);},"opera":function(V){qx.bom.Event.removeNativeListener(V,t,this._onKeyUpWrapper);qx.bom.Event.removeNativeListener(V,c,this._onKeyDownWrapper);qx.bom.Event.removeNativeListener(V,m,this._onInputWrapper);},"default":function(W){qx.bom.Event.removeNativeListener(W,m,this._onInputWrapper);}}),_onKeyPress:qx.core.Environment.select(v,{"mshtml":function(e,X){if(e.keyCode===13){if(X.value!==this.__iV){this.__iV=X.value;qx.event.Registration.fireEvent(X,k,qx.event.type.Data,[X.value]);};};},"opera":function(e,Y){if(e.keyCode===13){if(Y.value!==this.__iV){this.__iV=Y.value;qx.event.Registration.fireEvent(Y,k,qx.event.type.Data,[Y.value]);};};},"default":null}),_inputFix:qx.core.Environment.select(v,{"mshtml":function(e,ba){if(e.keyCode===46||e.keyCode===8){if(ba.value!==this.__iW){this.__iW=ba.value;qx.event.Registration.fireEvent(ba,m,qx.event.type.Data,[ba.value]);};};},"default":null}),_onKeyDown:qx.core.Environment.select(v,{"opera":function(e){if(e.keyCode===13){this.__iT=true;};},"default":null}),_onKeyUp:qx.core.Environment.select(v,{"opera":function(e){if(e.keyCode===13){this.__iT=false;};},"default":null}),_onInput:qx.event.GlobalError.observeMethod(function(e){var bc=qx.bom.Event.getTarget(e);var bb=bc.tagName.toLowerCase();if(!this.__iT||bb!==m){if((qx.core.Environment.get(v)==s)&&qx.core.Environment.get(r)<10.6){this.__iU=window.setTimeout(function(){qx.event.Registration.fireEvent(bc,m,qx.event.type.Data,[bc.value]);},0);}else {qx.event.Registration.fireEvent(bc,m,qx.event.type.Data,[bc.value]);};};}),_onChangeValue:qx.event.GlobalError.observeMethod(function(e){var bd=qx.bom.Event.getTarget(e);var be=bd.value;if(bd.type===j){var be=[];for(var i=0,o=bd.options,l=o.length;i<l;i++ ){if(o[i].selected){be.push(o[i].value);};};};qx.event.Registration.fireEvent(bd,k,qx.event.type.Data,[be]);}),_onChangeChecked:qx.event.GlobalError.observeMethod(function(e){var bf=qx.bom.Event.getTarget(e);if(bf.type===d){if(bf.checked){qx.event.Registration.fireEvent(bf,k,qx.event.type.Data,[bf.value]);};}else {qx.event.Registration.fireEvent(bf,k,qx.event.type.Data,[bf.checked]);};}),_onProperty:qx.core.Environment.select(v,{"mshtml":qx.event.GlobalError.observeMethod(function(e){var bg=qx.bom.Event.getTarget(e);var bh=e.propertyName;if(bh===n&&(bg.type===a||bg.type===g||bg.tagName.toLowerCase()===f)){if(!bg.$$inValueSet){qx.event.Registration.fireEvent(bg,m,qx.event.type.Data,[bg.value]);};}else if(bh===z){if(bg.type===x){qx.event.Registration.fireEvent(bg,k,qx.event.type.Data,[bg.checked]);}else if(bg.checked){qx.event.Registration.fireEvent(bg,k,qx.event.type.Data,[bg.value]);};};}),"default":function(){}})},defer:function(bi){qx.event.Registration.addHandler(bi);}});})();(function(){var a="offline",b="qx.event.handler.Offline",c="online";qx.Class.define(b,{extend:qx.core.Object,implement:qx.event.IEventHandler,construct:function(d){qx.core.Object.call(this);this.__gt=d;this.__cz=d.getWindow();this._initObserver();},statics:{PRIORITY:qx.event.Registration.PRIORITY_NORMAL,SUPPORTED_TYPES:{online:true,offline:true},TARGET_CHECK:qx.event.IEventHandler.TARGET_WINDOW,IGNORE_CAN_HANDLE:true},members:{__gt:null,__cz:null,__iN:null,canHandleEvent:function(f,e){},registerEvent:function(i,h,g){},unregisterEvent:function(l,k,j){},_initObserver:function(){this.__iN=qx.lang.Function.listener(this._onNative,this);qx.bom.Event.addNativeListener(this.__cz,a,this.__iN);qx.bom.Event.addNativeListener(this.__cz,c,this.__iN);},_stopObserver:function(){qx.bom.Event.removeNativeListener(this.__cz,a,this.__iN);qx.bom.Event.removeNativeListener(this.__cz,c,this.__iN);},_onNative:qx.event.GlobalError.observeMethod(function(m){qx.event.Registration.fireEvent(this.__cz,m.type,qx.event.type.Event,[]);}),isOnline:function(){return !!this.__cz.navigator.onLine;}},destruct:function(){this.__gt=null;this._stopObserver();delete qx.event.handler.Appear.__instances[this.$$hash];},defer:function(n){qx.event.Registration.addHandler(n);}});})();(function(){var a="mshtml",b="engine.name",c="qx.bom.Element";qx.Class.define(c,{statics:{addListener:function(g,f,d,self,e){return qx.event.Registration.addListener(g,f,d,self,e);},removeListener:function(n,m,h,self,k){return qx.event.Registration.removeListener(n,m,h,self,k);},removeListenerById:function(o,p){return qx.event.Registration.removeListenerById(o,p);},hasListener:function(s,r,q){return qx.event.Registration.hasListener(s,r,q);},focus:function(t){qx.event.Registration.getManager(t).getHandler(qx.event.handler.Focus).focus(t);},blur:function(u){qx.event.Registration.getManager(u).getHandler(qx.event.handler.Focus).blur(u);},activate:function(v){qx.event.Registration.getManager(v).getHandler(qx.event.handler.Focus).activate(v);},deactivate:function(w){qx.event.Registration.getManager(w).getHandler(qx.event.handler.Focus).deactivate(w);},capture:function(y,x){qx.event.Registration.getManager(y).getDispatcher(qx.event.dispatch.MouseCapture).activateCapture(y,x);},releaseCapture:function(z){qx.event.Registration.getManager(z).getDispatcher(qx.event.dispatch.MouseCapture).releaseCapture(z);},clone:function(E,L){var C;if(L||((qx.core.Environment.get(b)==a)&&!qx.xml.Document.isXmlDocument(E))){var G=qx.event.Registration.getManager(E);var A=qx.dom.Hierarchy.getDescendants(E);A.push(E);};if((qx.core.Environment.get(b)==a)){for(var i=0,l=A.length;i<l;i++ ){G.toggleAttachedEvents(A[i],false);};};var C=E.cloneNode(true);if((qx.core.Environment.get(b)==a)){for(var i=0,l=A.length;i<l;i++ ){G.toggleAttachedEvents(A[i],true);};};if(L===true){var K=qx.dom.Hierarchy.getDescendants(C);K.push(C);var B,J,I,D;for(var i=0,H=A.length;i<H;i++ ){I=A[i];B=G.serializeListeners(I);if(B.length>0){J=K[i];for(var j=0,F=B.length;j<F;j++ ){D=B[j];G.addListener(J,D.type,D.handler,D.self,D.capture);};};};};return C;}}});})();(function(){var a="mshtml",b="engine.name",c="blur",d="losecapture",e="focus",f="os.version",g="click",h="qx.event.dispatch.MouseCapture",i="capture",j="scroll",k="browser.documentmode";qx.Class.define(h,{extend:qx.event.dispatch.AbstractBubbling,construct:function(l,m){qx.event.dispatch.AbstractBubbling.call(this,l);this.__cz=l.getWindow();this.__cB=m;l.addListener(this.__cz,c,this.releaseCapture,this);l.addListener(this.__cz,e,this.releaseCapture,this);l.addListener(this.__cz,j,this.releaseCapture,this);},statics:{PRIORITY:qx.event.Registration.PRIORITY_FIRST},members:{__cB:null,__jb:null,__jc:true,__cz:null,_getParent:function(n){return n.parentNode;},canDispatchEvent:function(p,event,o){return !!(this.__jb&&this.__jd[o]);},dispatchEvent:function(r,event,q){if(q==g){event.stopPropagation();this.releaseCapture();return;};if(this.__jc||!qx.dom.Hierarchy.contains(this.__jb,r)){r=this.__jb;};qx.event.dispatch.AbstractBubbling.prototype.dispatchEvent.call(this,r,event,q);},__jd:{"mouseup":1,"mousedown":1,"click":1,"dblclick":1,"mousemove":1,"mouseout":1,"mouseover":1,"pointerdown":1,"pointerup":1,"pointermove":1,"pointerover":1,"pointerout":1,"tap":1,"dbltap":1},activateCapture:function(t,s){var s=s!==false;if(this.__jb===t&&this.__jc==s){return;};if(this.__jb){this.releaseCapture();};if(this.hasNativeCapture){this.nativeSetCapture(t,s);var self=this;qx.bom.Event.addNativeListener(t,d,function(){qx.bom.Event.removeNativeListener(t,d,arguments.callee);self.releaseCapture();});};this.__jc=s;this.__jb=t;this.__cB.fireEvent(t,i,qx.event.type.Event,[true,false]);},getCaptureElement:function(){return this.__jb;},releaseCapture:function(){var u=this.__jb;if(!u){return;};this.__jb=null;this.__cB.fireEvent(u,d,qx.event.type.Event,[true,false]);this.nativeReleaseCapture(u);},hasNativeCapture:(qx.core.Environment.get(b)==a&&qx.core.Environment.get(k)<9||(parseInt(qx.core.Environment.get(f),10)>7&&qx.core.Environment.get(k)>9)),nativeSetCapture:qx.core.Environment.select(b,{"mshtml":function(w,v){w.setCapture(v!==false);},"default":(function(){})}),nativeReleaseCapture:qx.core.Environment.select(b,{"mshtml":function(x){x.releaseCapture();},"default":(function(){})})},destruct:function(){this.__jb=this.__cz=this.__cB=null;},defer:function(y){qx.event.Registration.addDispatcher(y);}});})();(function(){var a="qx.event.handler.Capture";qx.Class.define(a,{extend:qx.core.Object,implement:qx.event.IEventHandler,statics:{PRIORITY:qx.event.Registration.PRIORITY_NORMAL,SUPPORTED_TYPES:{capture:true,losecapture:true},TARGET_CHECK:qx.event.IEventHandler.TARGET_DOMNODE,IGNORE_CAN_HANDLE:true},members:{canHandleEvent:function(c,b){},registerEvent:function(f,e,d){},unregisterEvent:function(i,h,g){}},defer:function(j){qx.event.Registration.addHandler(j);}});})();(function(){var a="function",b="plugin.silverlight.version",c="Silverlight",d="Skype.Detection",f="QuickTimeCheckObject.QuickTimeCheck.1",g="Adobe Acrobat",h="plugin.windowsmedia",k="QuickTime",l="plugin.silverlight",m="pdf",n="wmv",o="qx.bom.client.Plugin",p="application/x-skype",q=',',r="plugin.divx",s='=',t="Chrome PDF Viewer",u="divx",v="Windows Media",w="",x="mshtml",y="skype.click2call",z="plugin.skype",A="plugin.gears",B="plugin.quicktime",C="plugin.windowsmedia.version",D="quicktime",E="DivX Web Player",F="AgControl.AgControl",G="Microsoft.XMLHTTP",H="silverlight",I="plugin.pdf",J="plugin.pdf.version",K="MSXML2.DOMDocument.6.0",L="WMPlayer.OCX.7",M="AcroPDF.PDF",N="plugin.activex",O="plugin.quicktime.version",P="plugin.divx.version",Q="npdivx.DivXBrowserPlugin.1",R="object";qx.Bootstrap.define(o,{statics:{getGears:function(){return !!(window.google&&window.google.gears);},getActiveX:function(){if(typeof window.ActiveXObject===a){return true;};try{return (typeof (new window.ActiveXObject(G))===R||typeof (new window.ActiveXObject(K))===R);}catch(S){return false;};},getSkype:function(){if(qx.bom.client.Plugin.getActiveX()){try{new ActiveXObject(d);return true;}catch(e){};};var T=navigator.mimeTypes;if(T){if(p in T){return true;};for(var i=0;i<T.length;i++ ){var U=T[i];if(U.type.indexOf(y)!=-1){return true;};};};return false;},__je:{quicktime:{plugin:[k],control:f},wmv:{plugin:[v],control:L},divx:{plugin:[E],control:Q},silverlight:{plugin:[c],control:F},pdf:{plugin:[t,g],control:M}},getQuicktimeVersion:function(){var V=qx.bom.client.Plugin.__je[D];return qx.bom.client.Plugin.__jf(V.control,V.plugin);},getWindowsMediaVersion:function(){var W=qx.bom.client.Plugin.__je[n];return qx.bom.client.Plugin.__jf(W.control,W.plugin,true);},getDivXVersion:function(){var X=qx.bom.client.Plugin.__je[u];return qx.bom.client.Plugin.__jf(X.control,X.plugin);},getSilverlightVersion:function(){var Y=qx.bom.client.Plugin.__je[H];return qx.bom.client.Plugin.__jf(Y.control,Y.plugin);},getPdfVersion:function(){var ba=qx.bom.client.Plugin.__je[m];return qx.bom.client.Plugin.__jf(ba.control,ba.plugin);},getQuicktime:function(){var bb=qx.bom.client.Plugin.__je[D];return qx.bom.client.Plugin.__jg(bb.control,bb.plugin);},getWindowsMedia:function(){var bc=qx.bom.client.Plugin.__je[n];return qx.bom.client.Plugin.__jg(bc.control,bc.plugin,true);},getDivX:function(){var bd=qx.bom.client.Plugin.__je[u];return qx.bom.client.Plugin.__jg(bd.control,bd.plugin);},getSilverlight:function(){var be=qx.bom.client.Plugin.__je[H];return qx.bom.client.Plugin.__jg(be.control,be.plugin);},getPdf:function(){var bf=qx.bom.client.Plugin.__je[m];return qx.bom.client.Plugin.__jg(bf.control,bf.plugin);},__jf:function(bo,bk,bj){var bg=qx.bom.client.Plugin.__jg(bo,bk,bj);if(!bg){return w;};if(qx.bom.client.Engine.getName()==x&&(qx.bom.client.Browser.getDocumentMode()<11||bj)){try{var bh=new ActiveXObject(bo);var bm;if(bh.GetVersions&&bh.GetVersions()){bm=bh.GetVersions().split(q);if(bm.length>1){bm=bm[0].split(s);if(bm.length===2){return bm[1];};};};bm=bh.versionInfo;if(bm!=undefined){return bm;};bm=bh.version;if(bm!=undefined){return bm;};bm=bh.settings.version;if(bm!=undefined){return bm;};}catch(bp){return w;};return w;}else {var bn=navigator.plugins;var bl=/([0-9]\.[0-9])/g;for(var i=0;i<bn.length;i++ ){var bi=bn[i];for(var j=0;j<bk.length;j++ ){if(bi.name.indexOf(bk[j])!==-1){if(bl.test(bi.name)||bl.test(bi.description)){return RegExp.$1;};};};};return w;};},__jg:function(bt,br,bq){if(qx.bom.client.Engine.getName()==x&&(qx.bom.client.Browser.getDocumentMode()<11||bq)){if(!this.getActiveX()){return false;};try{new ActiveXObject(bt);}catch(bu){return false;};return true;}else {var bs=navigator.plugins;if(!bs){return false;};var name;for(var i=0;i<bs.length;i++ ){name=bs[i].name;for(var j=0;j<br.length;j++ ){if(name.indexOf(br[j])!==-1){return true;};};};return false;};}},defer:function(bv){qx.core.Environment.add(A,bv.getGears);qx.core.Environment.add(B,bv.getQuicktime);qx.core.Environment.add(O,bv.getQuicktimeVersion);qx.core.Environment.add(h,bv.getWindowsMedia);qx.core.Environment.add(C,bv.getWindowsMediaVersion);qx.core.Environment.add(r,bv.getDivX);qx.core.Environment.add(P,bv.getDivXVersion);qx.core.Environment.add(l,bv.getSilverlight);qx.core.Environment.add(b,bv.getSilverlightVersion);qx.core.Environment.add(I,bv.getPdf);qx.core.Environment.add(J,bv.getPdfVersion);qx.core.Environment.add(N,bv.getActiveX);qx.core.Environment.add(z,bv.getSkype);}});})();(function(){var a='<\?xml version="1.0" encoding="utf-8"?>\n<',b="MSXML2.DOMDocument.3.0",c="qx.xml.Document",d="",e=" />",f="xml.domparser",g="SelectionLanguage",h="'",j="MSXML2.XMLHTTP.3.0",k="plugin.activex",m="No XML implementation available!",n="MSXML2.XMLHTTP.6.0",o="xml.implementation",p=" xmlns='",q="text/xml",r="XPath",s="MSXML2.DOMDocument.6.0",t="HTML";qx.Bootstrap.define(c,{statics:{DOMDOC:null,XMLHTTP:null,isXmlDocument:function(u){if(u.nodeType===9){return u.documentElement.nodeName!==t;}else if(u.ownerDocument){return this.isXmlDocument(u.ownerDocument);}else {return false;};},create:function(v,w){if(qx.core.Environment.get(k)){var x=new ActiveXObject(this.DOMDOC);if(this.DOMDOC==b){x.setProperty(g,r);};if(w){var y=a;y+=w;if(v){y+=p+v+h;};y+=e;x.loadXML(y);};return x;};if(qx.core.Environment.get(o)){return document.implementation.createDocument(v||d,w||d,null);};throw new Error(m);},fromString:function(A){if(qx.core.Environment.get(k)){var B=qx.xml.Document.create();B.loadXML(A);return B;};if(qx.core.Environment.get(f)){var z=new DOMParser();return z.parseFromString(A,q);};throw new Error(m);}},defer:function(D){if(qx.core.Environment.get(k)){var C=[s,b];var E=[n,j];for(var i=0,l=C.length;i<l;i++ ){try{new ActiveXObject(C[i]);new ActiveXObject(E[i]);}catch(F){continue;};D.DOMDOC=C[i];D.XMLHTTP=E[i];break;};};}});})();(function(){var a="function",b="xml.implementation",c="xml.attributens",d="xml.selectnodes",e="<a></a>",f="xml.getqualifieditem",g="SelectionLanguage",h="xml.getelementsbytagnamens",i="qx.bom.client.Xml",j="xml.domproperties",k="xml.selectsinglenode",l="1.0",m="xml.createnode",n="xml.domparser",o="getProperty",p="undefined",q="XML",r="string",s="xml.createelementns";qx.Bootstrap.define(i,{statics:{getImplementation:function(){return document.implementation&&document.implementation.hasFeature&&document.implementation.hasFeature(q,l);},getDomParser:function(){return typeof window.DOMParser!==p;},getSelectSingleNode:function(){return typeof qx.xml.Document.create().selectSingleNode!==p;},getSelectNodes:function(){return typeof qx.xml.Document.create().selectNodes!==p;},getElementsByTagNameNS:function(){return typeof qx.xml.Document.create().getElementsByTagNameNS!==p;},getDomProperties:function(){var t=qx.xml.Document.create();return (o in t&&typeof t.getProperty(g)===r);},getAttributeNS:function(){var u=qx.xml.Document.fromString(e).documentElement;return typeof u.getAttributeNS===a&&typeof u.setAttributeNS===a;},getCreateElementNS:function(){return typeof qx.xml.Document.create().createElementNS===a;},getCreateNode:function(){return typeof qx.xml.Document.create().createNode!==p;},getQualifiedItem:function(){var v=qx.xml.Document.fromString(e).documentElement;return typeof v.attributes.getQualifiedItem!==p;}},defer:function(w){qx.core.Environment.add(b,w.getImplementation);qx.core.Environment.add(n,w.getDomParser);qx.core.Environment.add(k,w.getSelectSingleNode);qx.core.Environment.add(d,w.getSelectNodes);qx.core.Environment.add(h,w.getElementsByTagNameNS);qx.core.Environment.add(j,w.getDomProperties);qx.core.Environment.add(c,w.getAttributeNS);qx.core.Environment.add(s,w.getCreateElementNS);qx.core.Environment.add(m,w.getCreateNode);qx.core.Environment.add(f,w.getQualifiedItem);}});})();(function(){var a="borderBottomWidth",b="visible",d="engine.name",e="borderTopWidth",f="top",g="borderLeftStyle",h="none",i="overflow",j="right",k="bottom",l="borderLeftWidth",m="100px",n="-moz-scrollbars-vertical",o="borderRightStyle",p="hidden",q="div",r="left",u="qx.bom.element.Scroll",v="borderRightWidth",w="scroll",x="overflowY";qx.Class.define(u,{statics:{__jh:null,getScrollbarWidth:function(){if(this.__jh!==null){return this.__jh;};var y=qx.bom.element.Style;var A=function(E,F){return parseInt(y.get(E,F),10)||0;};var B=function(G){return (y.get(G,o)==h?0:A(G,v));};var C=function(H){return (y.get(H,g)==h?0:A(H,l));};var D=qx.core.Environment.select(d,{"mshtml":function(I){if(y.get(I,x)==p||I.clientWidth==0){return B(I);};return Math.max(0,I.offsetWidth-I.clientLeft-I.clientWidth);},"default":function(J){if(J.clientWidth==0){var L=y.get(J,i);var K=(L==w||L==n?16:0);return Math.max(0,B(J)+K);};return Math.max(0,(J.offsetWidth-J.clientWidth-C(J)));}});var z=function(M){return D(M)-B(M);};var t=document.createElement(q);var s=t.style;s.height=s.width=m;s.overflow=w;document.body.appendChild(t);var c=z(t);this.__jh=c;document.body.removeChild(t);return this.__jh;},intoViewX:function(bi,stop,bh){var parent=bi.parentNode;var bg=qx.dom.Node.getDocument(bi);var Y=bg.body;var be,Q,V;var R,P,S;var bb,T,O;var X,bc,bd,ba;var bf,U,bj;var N=bh===r;var W=bh===j;stop=stop?stop.parentNode:bg;while(parent&&parent!=stop){if(parent.scrollWidth>parent.clientWidth&&(parent===Y||qx.bom.element.Style.get(parent,x)!=b)){if(parent===Y){Q=parent.scrollLeft;V=Q+qx.bom.Viewport.getWidth();R=qx.bom.Viewport.getWidth();P=parent.clientWidth;S=parent.scrollWidth;bb=0;T=0;O=0;}else {be=qx.bom.element.Location.get(parent);Q=be.left;V=be.right;R=parent.offsetWidth;P=parent.clientWidth;S=parent.scrollWidth;bb=parseInt(qx.bom.element.Style.get(parent,l),10)||0;T=parseInt(qx.bom.element.Style.get(parent,v),10)||0;O=R-P-bb-T;};X=qx.bom.element.Location.get(bi);bc=X.left;bd=X.right;ba=bi.offsetWidth;bf=bc-Q-bb;U=bd-V+T;bj=0;if(N){bj=bf;}else if(W){bj=U+O;}else if(bf<0||ba>P){bj=bf;}else if(U>0){bj=U+O;};parent.scrollLeft+=bj;qx.event.Registration.fireNonBubblingEvent(parent,w);};if(parent===Y){break;};parent=parent.parentNode;};},intoViewY:function(bD,stop,bC){var parent=bD.parentNode;var bB=qx.dom.Node.getDocument(bD);var bk=bB.body;var by,bt,bw;var bE,bx,bu;var bp,bl,bA;var br,bs,bq,bm;var bn,bv,bz;var bo=bC===f;var bF=bC===k;stop=stop?stop.parentNode:bB;while(parent&&parent!=stop){if(parent.scrollHeight>parent.clientHeight&&(parent===bk||qx.bom.element.Style.get(parent,x)!=b)){if(parent===bk){bt=parent.scrollTop;bw=bt+qx.bom.Viewport.getHeight();bE=qx.bom.Viewport.getHeight();bx=parent.clientHeight;bu=parent.scrollHeight;bp=0;bl=0;bA=0;}else {by=qx.bom.element.Location.get(parent);bt=by.top;bw=by.bottom;bE=parent.offsetHeight;bx=parent.clientHeight;bu=parent.scrollHeight;bp=parseInt(qx.bom.element.Style.get(parent,e),10)||0;bl=parseInt(qx.bom.element.Style.get(parent,a),10)||0;bA=bE-bx-bp-bl;};br=qx.bom.element.Location.get(bD);bs=br.top;bq=br.bottom;bm=bD.offsetHeight;bn=bs-bt-bp;bv=bq-bw+bl;bz=0;if(bo){bz=bn;}else if(bF){bz=bv+bA;}else if(bn<0||bm>bx){bz=bn;}else if(bv>0){bz=bv+bA;};parent.scrollTop+=bz;qx.event.Registration.fireNonBubblingEvent(parent,w);};if(parent===bk){break;};parent=parent.parentNode;};},intoView:function(bI,stop,bH,bG){this.intoViewX(bI,stop,bH);this.intoViewY(bI,stop,bG);}}});})();(function(){var a="useraction",b=" due to exceptions in user code. The application has to be reloaded!",c="event.touch",d="qx.ui.core.queue.Manager",f=" times in a row",g="Fatal Error: Flush terminated ";qx.Class.define(d,{statics:{__ji:false,__gl:false,__jj:{},__jk:0,MAX_RETRIES:10,scheduleFlush:function(h){var self=qx.ui.core.queue.Manager;self.__jj[h]=true;if(!self.__ji){self.__gl=false;qx.bom.AnimationFrame.request(function(){if(self.__gl){self.__gl=false;return;};self.flush();},self);self.__ji=true;};},flush:function(){var self=qx.ui.core.queue.Manager;if(self.__jl){return;};self.__jl=true;self.__gl=true;var i=self.__jj;self.__jm(function(){while(i.visibility||i.widget||i.appearance||i.layout||i.element){if(i.widget){delete i.widget;{qx.ui.core.queue.Widget.flush();};};if(i.visibility){delete i.visibility;{qx.ui.core.queue.Visibility.flush();};};if(i.appearance){delete i.appearance;{qx.ui.core.queue.Appearance.flush();};};if(i.widget||i.visibility||i.appearance){continue;};if(i.layout){delete i.layout;{qx.ui.core.queue.Layout.flush();};};if(i.widget||i.visibility||i.appearance||i.layout){continue;};if(i.element){delete i.element;qx.html.Element.flush();};};},function(){self.__ji=false;});self.__jm(function(){if(i.dispose){delete i.dispose;{qx.ui.core.queue.Dispose.flush();};};},function(){self.__jl=false;});self.__jk=0;},__jm:function(j,k){var self=qx.ui.core.queue.Manager;try{j();}catch(e){{};self.__ji=false;self.__jl=false;self.__jk+=1;if(self.__jk<=self.MAX_RETRIES){self.scheduleFlush();}else {throw new Error(g+(self.__jk-1)+f+b);};throw e;}finally{k();};},__jn:function(e){qx.ui.core.queue.Manager.flush();}},defer:function(l){qx.html.Element._scheduleFlush=l.scheduleFlush;qx.event.Registration.addListener(window,a,qx.core.Environment.get(c)?l.__jn:l.flush);}});})();(function(){var a="qx.ui.core.queue.Widget",b="widget",c="$$default";qx.Class.define(a,{statics:{__dO:[],__jj:{},remove:function(e,g){var d=this.__dO;if(!qx.lang.Array.contains(d,e)){return;};var f=e.$$hash;if(g==null){qx.lang.Array.remove(d,e);delete this.__jj[f];return;};if(this.__jj[f]){delete this.__jj[f][g];if(qx.lang.Object.getLength(this.__jj[f])==0){qx.lang.Array.remove(d,e);};};},add:function(j,l){var h=this.__dO;if(!qx.lang.Array.contains(h,j)){h.unshift(j);};if(l==null){l=c;};var k=j.$$hash;if(!this.__jj[k]){this.__jj[k]={};};this.__jj[k][l]=true;qx.ui.core.queue.Manager.scheduleFlush(b);},flush:function(){var m=this.__dO;var n,o;for(var i=m.length-1;i>=0;i-- ){n=m[i];o=this.__jj[n.$$hash];m.splice(i,1);n.syncWidget(o);};if(m.length!=0){return;};this.__dO=[];this.__jj={};}}});})();(function(){var a="qx.ui.core.queue.Visibility",b="visibility";qx.Class.define(a,{statics:{__dO:[],__cP:{},remove:function(c){delete this.__cP[c.$$hash];qx.lang.Array.remove(this.__dO,c);},isVisible:function(d){return this.__cP[d.$$hash]||false;},__jo:function(f){var h=this.__cP;var g=f.$$hash;var e;if(f.isExcluded()){e=false;}else {var parent=f.$$parent;if(parent){e=this.__jo(parent);}else {e=f.isRootWidget();};};return h[g]=e;},add:function(k){var j=this.__dO;if(qx.lang.Array.contains(j,k)){return;};j.unshift(k);qx.ui.core.queue.Manager.scheduleFlush(b);},flush:function(){var o=this.__dO;var p=this.__cP;for(var i=o.length-1;i>=0;i-- ){var n=o[i].$$hash;if(p[n]!=null){o[i].addChildrenToQueue(o);};};var l={};for(var i=o.length-1;i>=0;i-- ){var n=o[i].$$hash;l[n]=p[n];p[n]=null;};for(var i=o.length-1;i>=0;i-- ){var m=o[i];var n=m.$$hash;o.splice(i,1);if(p[n]==null){this.__jo(m);};if(p[n]&&p[n]!=l[n]){m.checkAppearanceNeeds();};};this.__dO=[];}}});})();(function(){var a="appearance",b="qx.ui.core.queue.Appearance";qx.Class.define(b,{statics:{__dO:[],remove:function(c){qx.lang.Array.remove(this.__dO,c);},add:function(e){var d=this.__dO;if(qx.lang.Array.contains(d,e)){return;};d.unshift(e);qx.ui.core.queue.Manager.scheduleFlush(a);},has:function(f){return qx.lang.Array.contains(this.__dO,f);},flush:function(){var j=qx.ui.core.queue.Visibility;var g=this.__dO;var h;for(var i=g.length-1;i>=0;i-- ){h=g[i];g.splice(i,1);if(j.isVisible(h)){h.syncAppearance();}else {h.$$stateChanges=true;};};}}});})();(function(){var a="dispose",b="qx.ui.core.queue.Dispose";qx.Class.define(b,{statics:{__dO:[],add:function(d){var c=this.__dO;if(qx.lang.Array.contains(c,d)){return;};c.unshift(d);qx.ui.core.queue.Manager.scheduleFlush(a);},isEmpty:function(){return this.__dO.length==0;},flush:function(){var e=this.__dO;for(var i=e.length-1;i>=0;i-- ){var f=e[i];e.splice(i,1);f.dispose();};if(e.length!=0){return;};this.__dO=[];}}});})();(function(){var a="backgroundColor",b="drag",c="_applyNativeContextMenu",d="touch",f="div",g="_applyBackgroundColor",h="qx.event.type.Data",j="__ju",k="_applyFocusable",m=" requires a layout, but no one was defined!",n="qx.event.type.KeyInput",o="focused",p="disabled",q="move",r="createChildControl",s="qxanonymous",t="Unsupported control: ",u="dragstart",v="Font",w="qx.dynlocale",x="dragchange",y="_applyEnabled",z="_applySelectable",A="Number",B="_applyKeepActive",C="qx.event.type.Pinch",D="dragend",E="_applyVisibility",F="Child control '",G="qxDraggable",H="qx.event.type.Roll",I="syncAppearance",J="paddingLeft",K="__jp",L="' of widget ",M="qx.event.type.Mouse",N="_applyPadding",O="#",P="At least one child in control ",Q="visible",R="qx.event.type.Event",S="qx.event.type.MouseWheel",T="_applyCursor",U="changeVisibility",V="_applyDraggable",W="resize",X="Decorator",Y="Remove Error: ",cK="zIndex",cL="changeTextColor",cM="$$widget",cG="changeContextMenu",cH="on",cI="paddingTop",cJ="opacity",cR="This widget has no children!",cS="changeSelectable",cT="_applyAnonymous",cU="none",cN="outline",cO="hidden",cP="_applyAppearance",cQ="hovered",cY="_applyOpacity",dB="Boolean",eo="px",da="qx.ui.core.Widget",cV="longtap",cW="default",ej="TabIndex property must be between 1 and 32000",cX="_applyFont",db="cursor",dc="qxDroppable",dd="' already created!",di="changeZIndex",dj=": ",dk="Color",de="changeEnabled",df="Abstract method call: _getContentHeightForWidth()!",dg="changeFont",dh="qx.event.type.Focus",dp="_applyDecorator",dq="_applyZIndex",dr="_applyTextColor",ds="__jw",dl="Widget is not focusable!",dm="qx.ui.menu.Menu",ek="engine.name",dn="qx.event.type.Drag",dw="qx.event.type.KeySequence",dx="excluded",en="DOM element is not yet created!",dy="_applyToolTipText",dt="Exception while creating child control '",du="qx.event.type.Rotate",em="_applyDroppable",dv=" is not a child of this widget!",dz="true",dA="widget",dM="changeDecorator",dL="qx.event.type.Tap",dK="Integer",dQ="_applyTabIndex",dP="changeAppearance",dO="qx.event.type.Track",dN="shorthand",dF="/",dE="String",dD="border-box",dC="",dJ="_applyContextMenu",dI="changeToolTipText",dH="padding",dG="tabIndex",dX="paddingBottom",dW="beforeContextmenuOpen",dV="changeNativeContextMenu",dU="undefined",ec="qx.ui.tooltip.ToolTip",eb="__jt",ea="contextmenu",dY="_applyKeepFocus",dT="paddingRight",dS="changeBackgroundColor",dR="changeLocale",ef="qx.event.type.Pointer",ee="qxKeepFocus",ed="opera",ei="qx.event.type.Touch",eh="qxKeepActive",eg="absolute";qx.Class.define(da,{extend:qx.ui.core.LayoutItem,include:[qx.locale.MTranslation],construct:function(){qx.ui.core.LayoutItem.call(this);this.__jp=this.__jv();this.initFocusable();this.initSelectable();this.initNativeContextMenu();},events:{appear:R,disappear:R,createChildControl:h,resize:h,move:h,syncAppearance:h,mousemove:M,mouseover:M,mouseout:M,mousedown:M,mouseup:M,click:M,dblclick:M,contextmenu:M,beforeContextmenuOpen:h,mousewheel:S,touchstart:ei,touchend:ei,touchmove:ei,touchcancel:ei,tap:dL,longtap:dL,dbltap:dL,swipe:ei,rotate:du,pinch:C,track:dO,roll:H,pointermove:ef,pointerover:ef,pointerout:ef,pointerdown:ef,pointerup:ef,pointercancel:ef,keyup:dw,keydown:dw,keypress:dw,keyinput:n,focus:dh,blur:dh,focusin:dh,focusout:dh,activate:dh,deactivate:dh,capture:R,losecapture:R,drop:dn,dragleave:dn,dragover:dn,drag:dn,dragstart:dn,dragend:dn,dragchange:dn,droprequest:dn},properties:{paddingTop:{check:dK,init:0,apply:N,themeable:true},paddingRight:{check:dK,init:0,apply:N,themeable:true},paddingBottom:{check:dK,init:0,apply:N,themeable:true},paddingLeft:{check:dK,init:0,apply:N,themeable:true},padding:{group:[cI,dT,dX,J],mode:dN,themeable:true},zIndex:{nullable:true,init:10,apply:dq,event:di,check:dK,themeable:true},decorator:{nullable:true,init:null,apply:dp,event:dM,check:X,themeable:true},backgroundColor:{nullable:true,check:dk,apply:g,event:dS,themeable:true},textColor:{nullable:true,check:dk,apply:dr,event:cL,themeable:true,inheritable:true},font:{nullable:true,apply:cX,check:v,event:dg,themeable:true,inheritable:true,dereference:true},opacity:{check:A,apply:cY,themeable:true,nullable:true,init:null},cursor:{check:dE,apply:T,themeable:true,inheritable:true,nullable:true,init:null},toolTip:{check:ec,nullable:true},toolTipText:{check:dE,nullable:true,event:dI,apply:dy},toolTipIcon:{check:dE,nullable:true,event:dI},blockToolTip:{check:dB,init:false},showToolTipWhenDisabled:{check:dB,init:false},visibility:{check:[Q,cO,dx],init:Q,apply:E,event:U},enabled:{init:true,check:dB,inheritable:true,apply:y,event:de},anonymous:{init:false,check:dB,apply:cT},tabIndex:{check:dK,nullable:true,apply:dQ},focusable:{check:dB,init:false,apply:k},keepFocus:{check:dB,init:false,apply:dY},keepActive:{check:dB,init:false,apply:B},draggable:{check:dB,init:false,apply:V},droppable:{check:dB,init:false,apply:em},selectable:{check:dB,init:false,event:cS,apply:z},contextMenu:{check:dm,apply:dJ,nullable:true,event:cG},nativeContextMenu:{check:dB,init:false,themeable:true,event:dV,apply:c},appearance:{check:dE,init:dA,apply:cP,event:dP}},statics:{DEBUG:false,getWidgetByElement:function(es,eq){while(es){var ep=es.$$widget;if(ep!=null){var er=qx.core.ObjectRegistry.fromHashCode(ep);if(!eq||!er.getAnonymous()){return er;};};try{es=es.parentNode;}catch(e){return null;};};return null;},contains:function(parent,et){while(et){et=et.getLayoutParent();if(parent==et){return true;};};return false;},__jq:new qx.util.ObjectPool()},members:{__jp:null,__jr:null,__js:null,__jt:null,_getLayout:function(){return this.__jt;},_setLayout:function(eu){{};if(this.__jt){this.__jt.connectToWidget(null);};if(eu){eu.connectToWidget(this);};this.__jt=eu;qx.ui.core.queue.Layout.add(this);},setLayoutParent:function(parent){if(this.$$parent===parent){return;};var content=this.getContentElement();if(this.$$parent&&!this.$$parent.$$disposed){this.$$parent.getContentElement().remove(content);};this.$$parent=parent||null;if(parent&&!parent.$$disposed){this.$$parent.getContentElement().add(content);};this.$$refreshInheritables();qx.ui.core.queue.Visibility.add(this);},_updateInsets:null,renderLayout:function(eB,top,ey,ew){var eC=qx.ui.core.LayoutItem.prototype.renderLayout.call(this,eB,top,ey,ew);if(!eC){return null;};if(qx.lang.Object.isEmpty(eC)&&!this._updateInsets){return null;};var content=this.getContentElement();var eF=eC.size||this._updateInsets;var eD=eo;var ev={};if(eC.position){ev.left=eB+eD;ev.top=top+eD;};if(eF||eC.margin){ev.width=ey+eD;ev.height=ew+eD;};if(Object.keys(ev).length>0){content.setStyles(ev);};if(eF||eC.local||eC.margin){if(this.__jt&&this.hasLayoutChildren()){var eA=this.getInsets();var innerWidth=ey-eA.left-eA.right;var innerHeight=ew-eA.top-eA.bottom;var eE=this.getDecorator();var ez={left:0,right:0,top:0,bottom:0};if(eE){eE=qx.theme.manager.Decoration.getInstance().resolve(eE);ez=eE.getPadding();};var ex={top:this.getPaddingTop()+ez.top,right:this.getPaddingRight()+ez.right,bottom:this.getPaddingBottom()+ez.bottom,left:this.getPaddingLeft()+ez.left};this.__jt.renderLayout(innerWidth,innerHeight,ex);}else if(this.hasLayoutChildren()){throw new Error(P+this._findTopControl()+m);};};if(eC.position&&this.hasListener(q)){this.fireDataEvent(q,this.getBounds());};if(eC.size&&this.hasListener(W)){this.fireDataEvent(W,this.getBounds());};delete this._updateInsets;return eC;},__ju:null,clearSeparators:function(){var eH=this.__ju;if(!eH){return;};var eI=qx.ui.core.Widget.__jq;var content=this.getContentElement();var eG;for(var i=0,l=eH.length;i<l;i++ ){eG=eH[i];eI.poolObject(eG);content.remove(eG.getContentElement());};eH.length=0;},renderSeparator:function(eK,eL){var eJ=qx.ui.core.Widget.__jq.getObject(qx.ui.core.Widget);eJ.set({decorator:eK});var eN=eJ.getContentElement();this.getContentElement().add(eN);var eM=eN.getDomElement();if(eM){eM.style.top=eL.top+eo;eM.style.left=eL.left+eo;eM.style.width=eL.width+eo;eM.style.height=eL.height+eo;}else {eN.setStyles({left:eL.left+eo,top:eL.top+eo,width:eL.width+eo,height:eL.height+eo});};if(!this.__ju){this.__ju=[];};this.__ju.push(eJ);},_computeSizeHint:function(){var eU=this.getWidth();var eO=this.getMinWidth();var eP=this.getMaxWidth();var eS=this.getHeight();var eQ=this.getMinHeight();var eR=this.getMaxHeight();{};var eV=this._getContentHint();var eT=this.getInsets();var eX=eT.left+eT.right;var eW=eT.top+eT.bottom;if(eU==null){eU=eV.width+eX;};if(eS==null){eS=eV.height+eW;};if(eO==null){eO=eX;if(eV.minWidth!=null){eO+=eV.minWidth;if(eO>eP&&eP!=null){eO=eP;};};};if(eQ==null){eQ=eW;if(eV.minHeight!=null){eQ+=eV.minHeight;if(eQ>eR&&eR!=null){eQ=eR;};};};if(eP==null){if(eV.maxWidth==null){eP=Infinity;}else {eP=eV.maxWidth+eX;if(eP<eO&&eO!=null){eP=eO;};};};if(eR==null){if(eV.maxHeight==null){eR=Infinity;}else {eR=eV.maxHeight+eW;if(eR<eQ&&eQ!=null){eR=eQ;};};};return {width:eU,minWidth:eO,maxWidth:eP,height:eS,minHeight:eQ,maxHeight:eR};},invalidateLayoutCache:function(){qx.ui.core.LayoutItem.prototype.invalidateLayoutCache.call(this);if(this.__jt){this.__jt.invalidateLayoutCache();};},_getContentHint:function(){var fa=this.__jt;if(fa){if(this.hasLayoutChildren()){var fb=fa.getSizeHint();{var eY;};return fb;}else {return {width:0,height:0};};}else {return {width:100,height:50};};},_getHeightForWidth:function(fg){var ff=this.getInsets();var fc=ff.left+ff.right;var fi=ff.top+ff.bottom;var fh=fg-fc;var fd=this._getLayout();if(fd&&fd.hasHeightForWidth()){var fj=fd.getHeightForWidth(fh);}else {fj=this._getContentHeightForWidth(fh);};var fe=fj+fi;return fe;},_getContentHeightForWidth:function(fk){throw new Error(df);},getInsets:function(){var top=this.getPaddingTop();var fl=this.getPaddingRight();var fm=this.getPaddingBottom();var fp=this.getPaddingLeft();if(this.getDecorator()){var fo=qx.theme.manager.Decoration.getInstance().resolve(this.getDecorator());var fn=fo.getInsets();{};top+=fn.top;fl+=fn.right;fm+=fn.bottom;fp+=fn.left;};return {"top":top,"right":fl,"bottom":fm,"left":fp};},getInnerSize:function(){var fr=this.getBounds();if(!fr){return null;};var fq=this.getInsets();return {width:fr.width-fq.left-fq.right,height:fr.height-fq.top-fq.bottom};},fadeOut:function(fs){return this.getContentElement().fadeOut(fs);},fadeIn:function(ft){return this.getContentElement().fadeIn(ft);},_applyAnonymous:function(fu){if(fu){this.getContentElement().setAttribute(s,dz);}else {this.getContentElement().removeAttribute(s);};},show:function(){this.setVisibility(Q);},hide:function(){this.setVisibility(cO);},exclude:function(){this.setVisibility(dx);},isVisible:function(){return this.getVisibility()===Q;},isHidden:function(){return this.getVisibility()!==Q;},isExcluded:function(){return this.getVisibility()===dx;},isSeeable:function(){qx.ui.core.queue.Manager.flush();var fv=this.getContentElement().getDomElement();if(fv){return fv.offsetWidth>0;};return false;},__jv:function(){var fx=this._createContentElement();fx.setAttribute(cM,this.toHashCode());fx.setStyles({"touch-action":cU,"-ms-touch-action":cU});{};var fw={"zIndex":10,"boxSizing":dD};if(!qx.ui.root.Inline||!(this instanceof qx.ui.root.Inline)){fw.position=eg;};fx.setStyles(fw);return fx;},_createContentElement:function(){return new qx.html.Element(f,{overflowX:cO,overflowY:cO});},getContentElement:function(){return this.__jp;},__jw:null,getLayoutChildren:function(){var fz=this.__jw;if(!fz){return this.__jx;};var fA;for(var i=0,l=fz.length;i<l;i++ ){var fy=fz[i];if(fy.hasUserBounds()||fy.isExcluded()){if(fA==null){fA=fz.concat();};qx.lang.Array.remove(fA,fy);};};return fA||fz;},scheduleLayoutUpdate:function(){qx.ui.core.queue.Layout.add(this);},invalidateLayoutChildren:function(){var fB=this.__jt;if(fB){fB.invalidateChildrenCache();};qx.ui.core.queue.Layout.add(this);},hasLayoutChildren:function(){var fD=this.__jw;if(!fD){return false;};var fC;for(var i=0,l=fD.length;i<l;i++ ){fC=fD[i];if(!fC.hasUserBounds()&&!fC.isExcluded()){return true;};};return false;},getChildrenContainer:function(){return this;},__jx:[],_getChildren:function(){return this.__jw||this.__jx;},_indexOf:function(fF){var fE=this.__jw;if(!fE){return -1;};return fE.indexOf(fF);},_hasChildren:function(){var fG=this.__jw;return fG!=null&&(!!fG[0]);},addChildrenToQueue:function(fH){var fI=this.__jw;if(!fI){return;};var fJ;for(var i=0,l=fI.length;i<l;i++ ){fJ=fI[i];fH.push(fJ);fJ.addChildrenToQueue(fH);};},_add:function(fL,fK){{};if(fL.getLayoutParent()==this){qx.lang.Array.remove(this.__jw,fL);};if(this.__jw){this.__jw.push(fL);}else {this.__jw=[fL];};this.__jy(fL,fK);},_addAt:function(fP,fM,fO){if(!this.__jw){this.__jw=[];};if(fP.getLayoutParent()==this){qx.lang.Array.remove(this.__jw,fP);};var fN=this.__jw[fM];if(fN===fP){fP.setLayoutProperties(fO);};if(fN){qx.lang.Array.insertBefore(this.__jw,fP,fN);}else {this.__jw.push(fP);};this.__jy(fP,fO);},_addBefore:function(fQ,fS,fR){{};if(fQ==fS){return;};if(!this.__jw){this.__jw=[];};if(fQ.getLayoutParent()==this){qx.lang.Array.remove(this.__jw,fQ);};qx.lang.Array.insertBefore(this.__jw,fQ,fS);this.__jy(fQ,fR);},_addAfter:function(fV,fT,fU){{};if(fV==fT){return;};if(!this.__jw){this.__jw=[];};if(fV.getLayoutParent()==this){qx.lang.Array.remove(this.__jw,fV);};qx.lang.Array.insertAfter(this.__jw,fV,fT);this.__jy(fV,fU);},_remove:function(fW){if(!this.__jw){throw new Error(cR);};qx.lang.Array.remove(this.__jw,fW);this.__jz(fW);},_removeAt:function(fX){if(!this.__jw){throw new Error(cR);};var fY=this.__jw[fX];qx.lang.Array.removeAt(this.__jw,fX);this.__jz(fY);return fY;},_removeAll:function(){if(!this.__jw){return [];};var ga=this.__jw.concat();this.__jw.length=0;for(var i=ga.length-1;i>=0;i-- ){this.__jz(ga[i]);};qx.ui.core.queue.Layout.add(this);return ga;},_afterAddChild:null,_afterRemoveChild:null,__jy:function(gc,gb){{};var parent=gc.getLayoutParent();if(parent&&parent!=this){parent._remove(gc);};gc.setLayoutParent(this);if(gb){gc.setLayoutProperties(gb);}else {this.updateLayoutProperties();};if(this._afterAddChild){this._afterAddChild(gc);};},__jz:function(gd){{};if(gd.getLayoutParent()!==this){throw new Error(Y+gd+dv);};gd.setLayoutParent(null);if(this.__jt){this.__jt.invalidateChildrenCache();};qx.ui.core.queue.Layout.add(this);if(this._afterRemoveChild){this._afterRemoveChild(gd);};},capture:function(ge){this.getContentElement().capture(ge);},releaseCapture:function(){this.getContentElement().releaseCapture();},isCapturing:function(){var gf=this.getContentElement().getDomElement();if(!gf){return false;};var gg=qx.event.Registration.getManager(gf);var gh=gg.getDispatcher(qx.event.dispatch.MouseCapture);return gf==gh.getCaptureElement();},_applyPadding:function(gj,gi,name){this._updateInsets=true;qx.ui.core.queue.Layout.add(this);this.__jA(name,gj);},__jA:function(gk,gn){var content=this.getContentElement();var gl=this.getDecorator();gl=qx.theme.manager.Decoration.getInstance().resolve(gl);if(gl){var gm=qx.Bootstrap.firstLow(gk.replace(dH,dC));gn+=gl.getPadding()[gm]||0;};content.setStyle(gk,gn+eo);},_applyDecorator:function(gp,go){var content=this.getContentElement();if(go){go=qx.theme.manager.Decoration.getInstance().getCssClassName(go);content.removeClass(go);};if(gp){gp=qx.theme.manager.Decoration.getInstance().addCssClass(gp);content.addClass(gp);};if(gp||go){qx.ui.core.queue.Layout.add(this);};},_applyToolTipText:function(gs,gr){if(qx.core.Environment.get(w)){if(this.__js){return;};var gq=qx.locale.Manager.getInstance();this.__js=gq.addListener(dR,function(){var gt=this.getToolTipText();if(gt&&gt.translate){this.setToolTipText(gt.translate());};},this);};},_applyTextColor:function(gv,gu){},_applyZIndex:function(gx,gw){this.getContentElement().setStyle(cK,gx==null?0:gx);},_applyVisibility:function(gz,gy){var content=this.getContentElement();if(gz===Q){content.show();}else {content.hide();};var parent=this.$$parent;if(parent&&(gy==null||gz==null||gy===dx||gz===dx)){parent.invalidateLayoutChildren();};qx.ui.core.queue.Visibility.add(this);},_applyOpacity:function(gB,gA){this.getContentElement().setStyle(cJ,gB==1?null:gB);},_applyCursor:function(gD,gC){if(gD==null&&!this.isSelectable()){gD=cW;};this.getContentElement().setStyle(db,gD,qx.core.Environment.get(ek)==ed);},_applyBackgroundColor:function(gH,gG){var gF=this.getBackgroundColor();var content=this.getContentElement();var gE=qx.theme.manager.Color.getInstance().resolve(gF);content.setStyle(a,gE);},_applyFont:function(gJ,gI){},_onChangeTheme:function(){if(this.isDisposed()){return;};qx.ui.core.LayoutItem.prototype._onChangeTheme.call(this);this.updateAppearance();var gK=this.getDecorator();this._applyDecorator(null,gK);this._applyDecorator(gK);gK=this.getFont();if(qx.lang.Type.isString(gK)){this._applyFont(gK,gK);};gK=this.getTextColor();if(qx.lang.Type.isString(gK)){this._applyTextColor(gK,gK);};gK=this.getBackgroundColor();if(qx.lang.Type.isString(gK)){this._applyBackgroundColor(gK,gK);};},__jB:null,$$stateChanges:null,_forwardStates:null,hasState:function(gM){var gL=this.__jB;return !!gL&&!!gL[gM];},addState:function(gQ){var gP=this.__jB;if(!gP){gP=this.__jB={};};if(gP[gQ]){return;};this.__jB[gQ]=true;if(gQ===cQ){this.syncAppearance();}else if(!qx.ui.core.queue.Visibility.isVisible(this)){this.$$stateChanges=true;}else {qx.ui.core.queue.Appearance.add(this);};var forward=this._forwardStates;var gO=this.__jE;if(forward&&forward[gQ]&&gO){var gN;for(var gR in gO){gN=gO[gR];if(gN instanceof qx.ui.core.Widget){gO[gR].addState(gQ);};};};},removeState:function(gV){var gU=this.__jB;if(!gU||!gU[gV]){return;};delete this.__jB[gV];if(gV===cQ){this.syncAppearance();}else if(!qx.ui.core.queue.Visibility.isVisible(this)){this.$$stateChanges=true;}else {qx.ui.core.queue.Appearance.add(this);};var forward=this._forwardStates;var gT=this.__jE;if(forward&&forward[gV]&&gT){for(var gW in gT){var gS=gT[gW];if(gS instanceof qx.ui.core.Widget){gS.removeState(gV);};};};},replaceState:function(gY,hc){var hb=this.__jB;if(!hb){hb=this.__jB={};};if(!hb[hc]){hb[hc]=true;};if(hb[gY]){delete hb[gY];};if(!qx.ui.core.queue.Visibility.isVisible(this)){this.$$stateChanges=true;}else {qx.ui.core.queue.Appearance.add(this);};var forward=this._forwardStates;var ha=this.__jE;if(forward&&forward[hc]&&ha){for(var hd in ha){var gX=ha[hd];if(gX instanceof qx.ui.core.Widget){gX.replaceState(gY,hc);};};};},__jC:null,__jD:null,syncAppearance:function(){var hi=this.__jB;var hh=this.__jC;var hj=qx.theme.manager.Appearance.getInstance();var hf=qx.core.Property.$$method.setThemed;var hn=qx.core.Property.$$method.resetThemed;if(this.__jD){delete this.__jD;if(hh){var he=hj.styleFrom(hh,hi,null,this.getAppearance());hh=null;};};if(!hh){var hg=this;var hk=[];do {hk.push(hg.$$subcontrol||hg.getAppearance());}while(hg=hg.$$subparent);hh=hk.reverse().join(dF).replace(/#[0-9]+/g,dC);this.__jC=hh;};var hm=hj.styleFrom(hh,hi,null,this.getAppearance());if(hm){if(he){for(var hl in he){if(hm[hl]===undefined){this[hn[hl]]();};};};{var hl;};for(var hl in hm){hm[hl]===undefined?this[hn[hl]]():this[hf[hl]](hm[hl]);};}else if(he){for(var hl in he){this[hn[hl]]();};};this.fireDataEvent(I,this.__jB);},_applyAppearance:function(hp,ho){this.updateAppearance();},checkAppearanceNeeds:function(){if(!this.__jr){qx.ui.core.queue.Appearance.add(this);this.__jr=true;}else if(this.$$stateChanges){qx.ui.core.queue.Appearance.add(this);delete this.$$stateChanges;};},updateAppearance:function(){this.__jD=true;qx.ui.core.queue.Appearance.add(this);var hs=this.__jE;if(hs){var hq;for(var hr in hs){hq=hs[hr];if(hq instanceof qx.ui.core.Widget){hq.updateAppearance();};};};},syncWidget:function(ht){},getEventTarget:function(){var hu=this;while(hu.getAnonymous()){hu=hu.getLayoutParent();if(!hu){return null;};};return hu;},getFocusTarget:function(){var hv=this;if(!hv.getEnabled()){return null;};while(hv.getAnonymous()||!hv.getFocusable()){hv=hv.getLayoutParent();if(!hv||!hv.getEnabled()){return null;};};return hv;},getFocusElement:function(){return this.getContentElement();},isTabable:function(){return (!!this.getContentElement().getDomElement())&&this.isFocusable();},_applyFocusable:function(hy,hw){var hx=this.getFocusElement();if(hy){var hz=this.getTabIndex();if(hz==null){hz=1;};hx.setAttribute(dG,hz);hx.setStyle(cN,cU);}else {if(hx.isNativelyFocusable()){hx.setAttribute(dG,-1);}else if(hw){hx.setAttribute(dG,null);};};},_applyKeepFocus:function(hB){var hA=this.getFocusElement();hA.setAttribute(ee,hB?cH:null);},_applyKeepActive:function(hD){var hC=this.getContentElement();hC.setAttribute(eh,hD?cH:null);},_applyTabIndex:function(hE){if(hE==null){hE=1;}else if(hE<1||hE>32000){throw new Error(ej);};if(this.getFocusable()&&hE!=null){this.getFocusElement().setAttribute(dG,hE);};},_applySelectable:function(hG,hF){if(hF!==null){this._applyCursor(this.getCursor());};this.getContentElement().setSelectable(hG);},_applyEnabled:function(hI,hH){if(hI===false){this.addState(p);this.removeState(cQ);if(this.isFocusable()){this.removeState(o);this._applyFocusable(false,true);};if(this.isDraggable()){this._applyDraggable(false,true);};if(this.isDroppable()){this._applyDroppable(false,true);};}else {this.removeState(p);if(this.isFocusable()){this._applyFocusable(true,false);};if(this.isDraggable()){this._applyDraggable(true,false);};if(this.isDroppable()){this._applyDroppable(true,false);};};},_applyNativeContextMenu:function(hK,hJ,name){},_applyContextMenu:function(hM,hL){if(hL){hL.removeState(ea);if(hL.getOpener()==this){hL.resetOpener();};if(!hM){this.removeListener(ea,this._onContextMenuOpen);this.removeListener(cV,this._onContextMenuOpen);hL.removeListener(U,this._onBeforeContextMenuOpen,this);};};if(hM){hM.setOpener(this);hM.addState(ea);if(!hL){this.addListener(ea,this._onContextMenuOpen);this.addListener(cV,this._onContextMenuOpen);hM.addListener(U,this._onBeforeContextMenuOpen,this);};};},_onContextMenuOpen:function(e){if(e.getType()==cV){if(e.getPointerType()!==d){return;};};this.getContextMenu().openAtPointer(e);e.stop();},_onBeforeContextMenuOpen:function(e){if(e.getData()==Q&&this.hasListener(dW)){this.fireDataEvent(dW,e);};},_onStopEvent:function(e){e.stopPropagation();},_getDragDropCursor:function(){return qx.ui.core.DragDropCursor.getInstance();},_applyDraggable:function(hO,hN){if(!this.isEnabled()&&hO===true){hO=false;};this._getDragDropCursor();if(hO){this.addListener(u,this._onDragStart);this.addListener(b,this._onDrag);this.addListener(D,this._onDragEnd);this.addListener(x,this._onDragChange);}else {this.removeListener(u,this._onDragStart);this.removeListener(b,this._onDrag);this.removeListener(D,this._onDragEnd);this.removeListener(x,this._onDragChange);};this.getContentElement().setAttribute(G,hO?cH:null);},_applyDroppable:function(hQ,hP){if(!this.isEnabled()&&hQ===true){hQ=false;};this.getContentElement().setAttribute(dc,hQ?cH:null);},_onDragStart:function(e){this._getDragDropCursor().placeToPointer(e);this.getApplicationRoot().setGlobalCursor(cW);},_onDrag:function(e){this._getDragDropCursor().placeToPointer(e);},_onDragEnd:function(e){this._getDragDropCursor().moveTo(-1000,-1000);this.getApplicationRoot().resetGlobalCursor();},_onDragChange:function(e){var hR=this._getDragDropCursor();var hS=e.getCurrentAction();hS?hR.setAction(hS):hR.resetAction();},visualizeFocus:function(){this.addState(o);},visualizeBlur:function(){this.removeState(o);},scrollChildIntoView:function(hX,hW,hV,hU){hU=typeof hU==dU?true:hU;var hT=qx.ui.core.queue.Layout;var parent;if(hU){hU=!hT.isScheduled(hX);parent=hX.getLayoutParent();if(hU&&parent){hU=!hT.isScheduled(parent);if(hU){parent.getChildren().forEach(function(hY){hU=hU&&!hT.isScheduled(hY);});};};};this.scrollChildIntoViewX(hX,hW,hU);this.scrollChildIntoViewY(hX,hV,hU);},scrollChildIntoViewX:function(ic,ia,ib){this.getContentElement().scrollChildIntoViewX(ic.getContentElement(),ia,ib);},scrollChildIntoViewY:function(ih,ie,ig){this.getContentElement().scrollChildIntoViewY(ih.getContentElement(),ie,ig);},focus:function(){if(this.isFocusable()){this.getFocusElement().focus();}else {throw new Error(dl);};},blur:function(){if(this.isFocusable()){this.getFocusElement().blur();}else {throw new Error(dl);};},activate:function(){this.getContentElement().activate();},deactivate:function(){this.getContentElement().deactivate();},tabFocus:function(){this.getFocusElement().focus();},hasChildControl:function(ii){if(!this.__jE){return false;};return !!this.__jE[ii];},__jE:null,_getCreatedChildControls:function(){return this.__jE;},getChildControl:function(il,ik){if(!this.__jE){if(ik){return null;};this.__jE={};};var ij=this.__jE[il];if(ij){return ij;};if(ik===true){return null;};return this._createChildControl(il);},_showChildControl:function(io){var im=this.getChildControl(io);im.show();return im;},_excludeChildControl:function(iq){var ip=this.getChildControl(iq,true);if(ip){ip.exclude();};},_isChildControlVisible:function(is){var ir=this.getChildControl(is,true);if(ir){return ir.isVisible();};return false;},_releaseChildControl:function(iw){var it=this.getChildControl(iw,false);if(!it){throw new Error(t+iw);};delete it.$$subcontrol;delete it.$$subparent;var iu=this.__jB;var forward=this._forwardStates;if(iu&&forward&&it instanceof qx.ui.core.Widget){for(var iv in iu){if(forward[iv]){it.removeState(iv);};};};delete this.__jE[iw];return it;},_createChildControl:function(iB){if(!this.__jE){this.__jE={};}else if(this.__jE[iB]){throw new Error(F+iB+dd);};var iy=iB.indexOf(O);try{if(iy==-1){var ix=this._createChildControlImpl(iB);}else {var ix=this._createChildControlImpl(iB.substring(0,iy),iB.substring(iy+1,iB.length));};}catch(iC){iC.message=dt+iB+L+this.toString()+dj+iC.message;throw iC;};if(!ix){throw new Error(t+iB);};ix.$$subcontrol=iB;ix.$$subparent=this;var iz=this.__jB;var forward=this._forwardStates;if(iz&&forward&&ix instanceof qx.ui.core.Widget){for(var iA in iz){if(forward[iA]){ix.addState(iA);};};};this.fireDataEvent(r,ix);return this.__jE[iB]=ix;},_createChildControlImpl:function(iE,iD){return null;},_disposeChildControls:function(){var iI=this.__jE;if(!iI){return;};var iG=qx.ui.core.Widget;for(var iH in iI){var iF=iI[iH];if(!iG.contains(this,iF)){iF.destroy();}else {iF.dispose();};};delete this.__jE;},_findTopControl:function(){var iJ=this;while(iJ){if(!iJ.$$subparent){return iJ;};iJ=iJ.$$subparent;};return null;},getContentLocation:function(iL){var iK=this.getContentElement().getDomElement();return iK?qx.bom.element.Location.get(iK,iL):null;},setDomLeft:function(iN){var iM=this.getContentElement().getDomElement();if(iM){iM.style.left=iN+eo;}else {throw new Error(en);};},setDomTop:function(iP){var iO=this.getContentElement().getDomElement();if(iO){iO.style.top=iP+eo;}else {throw new Error(en);};},setDomPosition:function(iR,top){var iQ=this.getContentElement().getDomElement();if(iQ){iQ.style.left=iR+eo;iQ.style.top=top+eo;}else {throw new Error(en);};},destroy:function(){if(this.$$disposed){return;};var parent=this.$$parent;if(parent){parent._remove(this);};qx.ui.core.queue.Dispose.add(this);},clone:function(){var iS=qx.ui.core.LayoutItem.prototype.clone.call(this);if(this.getChildren){var iT=this.getChildren();for(var i=0,l=iT.length;i<l;i++ ){iS.add(iT[i].clone());};};return iS;}},destruct:function(){if(!qx.core.ObjectRegistry.inShutDown){if(qx.core.Environment.get(w)){if(this.__js){qx.locale.Manager.getInstance().removeListenerById(this.__js);};};var iU=this.getContentElement();if(iU){iU.setAttribute(cM,null,true);};this._disposeChildControls();qx.ui.core.queue.Appearance.remove(this);qx.ui.core.queue.Layout.remove(this);qx.ui.core.queue.Visibility.remove(this);qx.ui.core.queue.Widget.remove(this);};if(this.getContextMenu()){this.setContextMenu(null);};if(!qx.core.ObjectRegistry.inShutDown){this.clearSeparators();this.__ju=null;}else {this._disposeArray(j);};this._disposeArray(ds);this.__jB=this.__jE=null;this._disposeObjects(eb,K);}});})();(function(){var a="blur",b="qxDraggable",c="touch",d="qx.ui.core.Widget",f="longtap",g="Escape",h="drag",i="keydown",j="Unsupported data type: ",k="roll",l="drop",m="qxDroppable",n="qx.event.handler.DragDrop",o="mouse",p="This method must not be used outside the drop event listener!",q="Control",r="Shift",s="!",t="alias",u="droprequest",v="copy",w="pointerup",x="dragstart",y="move",z="pointerdown",A="dragchange",B="on",C="Alt",D="keyup",E="keypress",F="dragleave",G="dragend",H="dragover",I="left",J="Please use a droprequest listener to the drag source to fill the manager with data!",K="pointermove";qx.Class.define(n,{extend:qx.core.Object,implement:qx.event.IEventHandler,construct:function(L){qx.core.Object.call(this);this.__gt=L;this.__dd=L.getWindow().document.documentElement;this.__gt.addListener(this.__dd,f,this._onLongtap,this);this.__gt.addListener(this.__dd,z,this._onPointerdown,this);qx.event.Registration.addListener(window,a,this._onWindowBlur,this);this.__jR();},statics:{PRIORITY:qx.event.Registration.PRIORITY_NORMAL,SUPPORTED_TYPES:{dragstart:1,dragend:1,dragover:1,dragleave:1,drop:1,drag:1,dragchange:1,droprequest:1},IGNORE_CAN_HANDLE:true,ALLOWED_BUTTONS:[I],MIN_DRAG_DISTANCE:5},properties:{cursor:{check:d,nullable:true,init:null}},members:{__gt:null,__dd:null,__jF:null,__jG:null,__jH:null,__jI:null,__jJ:null,__c:null,__jK:null,__jL:null,__jM:false,__jN:false,__jO:false,__jP:null,__jQ:null,canHandleEvent:function(N,M){},registerEvent:function(Q,P,O){},unregisterEvent:function(T,S,R){},addType:function(U){this.__jH[U]=true;},addAction:function(V){this.__jI[V]=true;},supportsType:function(W){return !!this.__jH[W];},supportsAction:function(X){return !!this.__jI[X];},setDropAllowed:function(Y){this.__jN=Y;this.__jS();},getData:function(ba){if(!this.__jN||!this.__jF){throw new Error(p);};if(!this.__jH[ba]){throw new Error(j+ba+s);};if(!this.__c[ba]){this.__jK=ba;this.__hJ(u,this.__jG,this.__jF,false);};if(!this.__c[ba]){throw new Error(J);};return this.__c[ba]||null;},getCurrentAction:function(){this.__jS();return this.__jL;},getDragTarget:function(){return this.__jP;},addData:function(bb,bc){this.__c[bb]=bc;},getCurrentType:function(){return this.__jK;},isSessionActive:function(){return this.__jM;},__jR:function(){this.__jH={};this.__jI={};this.__jJ={};this.__c={};},__jS:function(){if(this.__jG==null){return;};var bg=this.__jI;var bd=this.__jJ;var be=null;if(this.__jN){if(bd.Shift&&bd.Control&&bg.alias){be=t;}else if(bd.Shift&&bd.Alt&&bg.copy){be=v;}else if(bd.Shift&&bg.move){be=y;}else if(bd.Alt&&bg.alias){be=t;}else if(bd.Control&&bg.copy){be=v;}else if(bg.move){be=y;}else if(bg.copy){be=v;}else if(bg.alias){be=t;};};var bf=this.__jL;if(be!=bf){if(this.__jF){this.__jL=be;this.__jO=this.__hJ(A,this.__jF,this.__jG,true);if(!this.__jO){be=null;};};if(be!=bf){this.__jL=be;this.__hJ(A,this.__jG,this.__jF,false);};};},__hJ:function(bm,bi,bj,bk,bn){var bl=qx.event.Registration;var bh=bl.createEvent(bm,qx.event.type.Drag,[bk,bn]);if(bi!==bj){bh.setRelatedTarget(bj);};return bl.dispatchEvent(bi,bh);},__jT:function(bo){while(bo&&bo.nodeType==1){if(bo.getAttribute(b)==B){return bo;};bo=bo.parentNode;};return null;},__jU:function(bp){while(bp&&bp.nodeType==1){if(bp.getAttribute(m)==B){return bp;};bp=bp.parentNode;};return null;},clearSession:function(){this.__gt.removeListener(this.__dd,K,this._onPointermove,this);this.__gt.removeListener(this.__dd,w,this._onPointerup,this,true);this.__gt.removeListener(this.__dd,i,this._onKeyDown,this,true);this.__gt.removeListener(this.__dd,D,this._onKeyUp,this,true);this.__gt.removeListener(this.__dd,E,this._onKeyPress,this,true);this.__gt.removeListener(this.__dd,k,this._onRoll,this,true);if(this.__jG){this.__hJ(G,this.__jG,this.__jF,false);};this.__jN=false;this.__jF=null;if(this.__jP){this.__jP.removeState(h);this.__jP=null;};this.__jG=null;this.__jM=false;this.__jQ=null;this.__jR();},_onLongtap:function(e){if(e.getPointerType()!=c){return;};this.__gt.addListener(this.__dd,k,this._onRoll,this,true);this._start(e);},_start:function(e){var bq=qx.event.handler.DragDrop.ALLOWED_BUTTONS.indexOf(e.getButton())!==-1;if(!e.isPrimary()||!bq){return false;};var bs=this.__jQ?this.__jQ.target:e.getTarget();var br=this.__jT(bs);if(br){this.__jG=br;var bt=qx.ui.core.Widget.getWidgetByElement(this.__jQ.original);while(bt&&bt.isAnonymous()){bt=bt.getLayoutParent();};if(bt){this.__jP=bt;bt.addState(h);};if(!this.__hJ(x,this.__jG,this.__jF,true,e)){return false;};this.__gt.addListener(this.__dd,i,this._onKeyDown,this,true);this.__gt.addListener(this.__dd,D,this._onKeyUp,this,true);this.__gt.addListener(this.__dd,E,this._onKeyPress,this,true);this.__jM=true;return true;};},_onPointerdown:function(e){if(e.isPrimary()){this.__jQ={target:e.getTarget(),original:e.getOriginalTarget(),left:e.getDocumentLeft(),top:e.getDocumentTop()};this.__gt.addListener(this.__dd,K,this._onPointermove,this);this.__gt.addListener(this.__dd,w,this._onPointerup,this,true);};},_onPointermove:function(e){if(!e.isPrimary()){return;};if(!this.__jM&&e.getPointerType()==o){var bA=this._getDelta(e);var bu=qx.event.handler.DragDrop.MIN_DRAG_DISTANCE;if(bA&&(Math.abs(bA.x)>bu||Math.abs(bA.y)>bu)){if(!this._start(e)){this.clearSession();return;};};};if(!this.__jM){return;};if(!this.__hJ(h,this.__jG,this.__jF,true,e)){this.clearSession();};var bv=e.getTarget();var by=this.getCursor();if(!by){by=qx.ui.core.DragDropCursor.getInstance();};var bx=by.getContentElement().getDomElement();if(bv!==bx){var bw=this.__jU(bv);if(bw&&bw!=this.__jF){if(this.__jF){this.__hJ(F,this.__jF,this.__jG,false,e);};this.__jN=true;this.__jF=bw;this.__jN=this.__hJ(H,bw,this.__jG,true,e);}else if(!bw&&this.__jF){this.__hJ(F,this.__jF,this.__jG,false,e);this.__jF=null;this.__jN=false;qx.event.Timer.once(this.__jS,this,0);};};var bz=this.__jJ;bz.Control=e.isCtrlPressed();bz.Shift=e.isShiftPressed();bz.Alt=e.isAltPressed();this.__jS();},_getDelta:function(e){if(!this.__jQ){return null;};var bB=e.getDocumentLeft()-this.__jQ.left;var bC=e.getDocumentTop()-this.__jQ.top;return {"x":bB,"y":bC};},_onPointerup:function(e){if(!e.isPrimary()){return;};if(this.__jN&&this.__jO){this.__hJ(l,this.__jF,this.__jG,false,e);};if(e.getTarget()==this.__jG){e.stopPropagation();};this.clearSession();},_onRoll:function(e){e.stop();},_onWindowBlur:function(e){this.clearSession();},_onKeyDown:function(e){var bD=e.getKeyIdentifier();switch(bD){case C:case q:case r:if(!this.__jJ[bD]){this.__jJ[bD]=true;this.__jS();};};},_onKeyUp:function(e){var bE=e.getKeyIdentifier();switch(bE){case C:case q:case r:if(this.__jJ[bE]){this.__jJ[bE]=false;this.__jS();};};},_onKeyPress:function(e){var bF=e.getKeyIdentifier();switch(bF){case g:this.clearSession();};}},destruct:function(){qx.event.Registration.removeListener(window,a,this._onWindowBlur,this);this.__jG=this.__jF=this.__gt=this.__dd=this.__jH=this.__jI=this.__jJ=this.__c=null;},defer:function(bG){qx.event.Registration.addHandler(bG);}});})();(function(){var a="qx.event.type.Drag",b="touch";qx.Class.define(a,{extend:qx.event.type.Event,members:{init:function(c,d){qx.event.type.Event.prototype.init.call(this,true,c);if(d){this._native=d.getNativeEvent()||null;this._originalTarget=d.getOriginalTarget()||null;}else {this._native=null;this._originalTarget=null;};return this;},clone:function(e){var f=qx.event.type.Event.prototype.clone.call(this,e);f._native=this._native;return f;},getDocumentLeft:function(){if(this._native==null){return 0;};var x=this._native.pageX;if(x!==undefined){if(x==0&&this._native.pointerType==b){x=this._native._original.changedTouches[0].pageX||0;};return Math.round(x);}else {var g=qx.dom.Node.getWindow(this._native.srcElement);return Math.round(this._native.clientX)+qx.bom.Viewport.getScrollLeft(g);};},getDocumentTop:function(){if(this._native==null){return 0;};var y=this._native.pageY;if(y!==undefined){if(y==0&&this._native.pointerType==b){y=this._native._original.changedTouches[0].pageY||0;};return Math.round(y);}else {var h=qx.dom.Node.getWindow(this._native.srcElement);return Math.round(this._native.clientY)+qx.bom.Viewport.getScrollTop(h);};},getManager:function(){return qx.event.Registration.getManager(this.getTarget()).getHandler(qx.event.handler.DragDrop);},addType:function(i){this.getManager().addType(i);},addAction:function(j){this.getManager().addAction(j);},supportsType:function(k){return this.getManager().supportsType(k);},supportsAction:function(l){return this.getManager().supportsAction(l);},addData:function(m,n){this.getManager().addData(m,n);},getData:function(o){return this.getManager().getData(o);},getCurrentType:function(){return this.getManager().getCurrentType();},getCurrentAction:function(){if(this.getDefaultPrevented()){return null;};return this.getManager().getCurrentAction();},setDropAllowed:function(p){this.getManager().setDropAllowed(p);},getDragTarget:function(){return this.getManager().getDragTarget();},stopSession:function(){this.getManager().clearSession();}}});})();(function(){var a="best-fit",b="placementRight",c="Boolean",d="bottom-right",e="' ",f="widget",g="placementLeft",h="qx.ui.core.MPlacement",i="left-top",j="Integer",k="left-middle",l="right-middle",m="top-center",n="[qx.ui.core.MPlacement.setMoveDirection()], the value was '",o="offsetRight",p="interval",q="keep-align",r="bottom-left",s="pointer",t="direct",u="shorthand",v="Invalid value for the parameter 'direction' ",w="offsetLeft",x="top-left",y="appear",z="offsetBottom",A="top",B="top-right",C="offsetTop",D="but 'top' or 'left' are allowed.",E="right-bottom",F="disappear",G="right-top",H="bottom-center",I="left-bottom",J="left";qx.Mixin.define(h,{statics:{__gF:null,__jV:J,setVisibleElement:function(K){this.__gF=K;},getVisibleElement:function(){return this.__gF;},setMoveDirection:function(L){if(L===A||L===J){this.__jV=L;}else {throw new Error(v+n+L+e+D);};},getMoveDirection:function(){return this.__jV;}},properties:{position:{check:[x,m,B,r,H,d,i,k,I,G,l,E],init:r,themeable:true},placeMethod:{check:[f,s],init:s,themeable:true},domMove:{check:c,init:false},placementModeX:{check:[t,q,a],init:q,themeable:true},placementModeY:{check:[t,q,a],init:q,themeable:true},offsetLeft:{check:j,init:0,themeable:true},offsetTop:{check:j,init:0,themeable:true},offsetRight:{check:j,init:0,themeable:true},offsetBottom:{check:j,init:0,themeable:true},offset:{group:[C,o,z,w],mode:u,themeable:true}},members:{__jW:null,__jX:null,__jY:null,getLayoutLocation:function(N){var P,O,R,top;O=N.getBounds();if(!O){return null;};R=O.left;top=O.top;var Q=O;N=N.getLayoutParent();while(N&&!N.isRootWidget()){O=N.getBounds();R+=O.left;top+=O.top;P=N.getInsets();R+=P.left;top+=P.top;N=N.getLayoutParent();};if(N.isRootWidget()){var M=N.getContentLocation();if(M){R+=M.left;top+=M.top;};};return {left:R,top:top,right:R+Q.width,bottom:top+Q.height};},moveTo:function(Y,top){var X=qx.ui.core.MPlacement.getVisibleElement();if(X){var W=this.getBounds();var V=X.getContentLocation();if(W&&V){var U=top+W.height;var T=Y+W.width;if((T>V.left&&Y<V.right)&&(U>V.top&&top<V.bottom)){var S=qx.ui.core.MPlacement.getMoveDirection();if(S===J){Y=Math.max(V.left-W.width,0);}else {top=Math.max(V.top-W.height,0);};};};};if(this.getDomMove()){this.setDomPosition(Y,top);}else {this.setLayoutProperties({left:Y,top:top});};},placeToWidget:function(bc,ba){if(ba){this.__ka();this.__jW=qx.lang.Function.bind(this.placeToWidget,this,bc,false);qx.event.Idle.getInstance().addListener(p,this.__jW);this.__jY=function(){this.__ka();};this.addListener(F,this.__jY,this);};var bb=bc.getContentLocation()||this.getLayoutLocation(bc);if(bb!=null){this._place(bb);return true;}else {return false;};},__ka:function(){if(this.__jW){qx.event.Idle.getInstance().removeListener(p,this.__jW);this.__jW=null;};if(this.__jY){this.removeListener(F,this.__jY,this);this.__jY=null;};},placeToPointer:function(event){var be=Math.round(event.getDocumentLeft());var top=Math.round(event.getDocumentTop());var bd={left:be,top:top,right:be,bottom:top};this._place(bd);},placeToElement:function(bh,bf){var location=qx.bom.element.Location.get(bh);var bg={left:location.left,top:location.top,right:location.left+bh.offsetWidth,bottom:location.top+bh.offsetHeight};if(bf){this.__jW=qx.lang.Function.bind(this.placeToElement,this,bh,false);qx.event.Idle.getInstance().addListener(p,this.__jW);this.addListener(F,function(){if(this.__jW){qx.event.Idle.getInstance().removeListener(p,this.__jW);this.__jW=null;};},this);};this._place(bg);},placeToPoint:function(bj){var bi={left:bj.left,top:bj.top,right:bj.left,bottom:bj.top};this._place(bi);},_getPlacementOffsets:function(){return {left:this.getOffsetLeft(),top:this.getOffsetTop(),right:this.getOffsetRight(),bottom:this.getOffsetBottom()};},__kb:function(bk){var bl=null;if(this._computePlacementSize){var bl=this._computePlacementSize();}else if(this.isVisible()){var bl=this.getBounds();};if(bl==null){this.addListenerOnce(y,function(){this.__kb(bk);},this);}else {bk.call(this,bl);};},_place:function(bm){this.__kb(function(bo){var bn=qx.util.placement.Placement.compute(bo,this.getLayoutParent().getBounds(),bm,this._getPlacementOffsets(),this.getPosition(),this.getPlacementModeX(),this.getPlacementModeY());this.removeState(g);this.removeState(b);this.addState(bm.left<bn.left?b:g);this.moveTo(bn.left,bn.top);});}},destruct:function(){this.__ka();}});})();(function(){var a="Number",b="interval",c="_applyTimeoutInterval",d="qx.event.type.Event",e="qx.event.Idle",f="singleton";qx.Class.define(e,{extend:qx.core.Object,type:f,construct:function(){qx.core.Object.call(this);var g=new qx.event.Timer(this.getTimeoutInterval());g.addListener(b,this._onInterval,this);g.start();this.__de=g;},events:{"interval":d},properties:{timeoutInterval:{check:a,init:100,apply:c}},members:{__de:null,_applyTimeoutInterval:function(h){this.__de.setInterval(h);},_onInterval:function(){this.fireEvent(b);}},destruct:function(){if(this.__de){this.__de.stop();};this.__de=null;}});})();(function(){var a="-",b="align-start",c="best-fit",d="qx.util.placement.Placement",e='__kc',f="middle",g="bottom",h="keep-align",i="align-end",j="align-center",k="Invalid 'mode' argument!'",l="center",m="edge-start",n="Class",o="direct",p="top",q="left",r="right",s="edge-end";qx.Class.define(d,{extend:qx.core.Object,construct:function(){qx.core.Object.call(this);this.__kc=qx.util.placement.DirectAxis;},properties:{axisX:{check:n},axisY:{check:n},edge:{check:[p,r,g,q],init:p},align:{check:[p,r,g,q,l,f],init:r}},statics:{__kd:null,compute:function(D,w,t,u,C,x,y){this.__kd=this.__kd||new qx.util.placement.Placement();var A=C.split(a);var z=A[0];var v=A[1];{var B;};this.__kd.set({axisX:this.__kh(x),axisY:this.__kh(y),edge:z,align:v});return this.__kd.compute(D,w,t,u);},__ke:null,__kf:null,__kg:null,__kh:function(E){switch(E){case o:this.__ke=this.__ke||qx.util.placement.DirectAxis;return this.__ke;case h:this.__kf=this.__kf||qx.util.placement.KeepAlignAxis;return this.__kf;case c:this.__kg=this.__kg||qx.util.placement.BestFitAxis;return this.__kg;default:throw new Error(k);};}},members:{__kc:null,compute:function(K,H,F,G){{};var I=this.getAxisX()||this.__kc;var L=I.computeStart(K.width,{start:F.left,end:F.right},{start:G.left,end:G.right},H.width,this.__ki());var J=this.getAxisY()||this.__kc;var top=J.computeStart(K.height,{start:F.top,end:F.bottom},{start:G.top,end:G.bottom},H.height,this.__kj());return {left:L,top:top};},__ki:function(){var N=this.getEdge();var M=this.getAlign();if(N==q){return m;}else if(N==r){return s;}else if(M==q){return b;}else if(M==l){return j;}else if(M==r){return i;};},__kj:function(){var P=this.getEdge();var O=this.getAlign();if(P==p){return m;}else if(P==g){return s;}else if(O==p){return b;}else if(O==f){return j;}else if(O==g){return i;};}},destruct:function(){this._disposeObjects(e);}});})();(function(){var a="align-start",b="align-end",c="qx.util.placement.AbstractAxis",d="edge-start",e="align-center",f="abstract method call!",g="edge-end";qx.Bootstrap.define(c,{extend:Object,statics:{computeStart:function(j,k,l,h,i){throw new Error(f);},_moveToEdgeAndAlign:function(n,o,p,m){switch(m){case d:return o.start-p.end-n;case g:return o.end+p.start;case a:return o.start+p.start;case e:return o.start+parseInt((o.end-o.start-n)/2,10)+p.start;case b:return o.end-p.end-n;};},_isInRange:function(r,s,q){return r>=0&&r+s<=q;}}});})();(function(){var a="qx.util.placement.DirectAxis";qx.Bootstrap.define(a,{statics:{_moveToEdgeAndAlign:qx.util.placement.AbstractAxis._moveToEdgeAndAlign,computeStart:function(d,e,f,b,c){return this._moveToEdgeAndAlign(d,e,f,c);}}});})();(function(){var a="qx.util.placement.KeepAlignAxis",b="edge-start",c="edge-end";qx.Bootstrap.define(a,{statics:{_moveToEdgeAndAlign:qx.util.placement.AbstractAxis._moveToEdgeAndAlign,_isInRange:qx.util.placement.AbstractAxis._isInRange,computeStart:function(k,f,g,d,j){var i=this._moveToEdgeAndAlign(k,f,g,j);var e,h;if(this._isInRange(i,k,d)){return i;};if(j==b||j==c){e=f.start-g.end;h=f.end+g.start;}else {e=f.end-g.end;h=f.start+g.start;};if(e>d-h){i=Math.max(0,e-k);}else {i=h;};return i;}}});})();(function(){var a="qx.util.placement.BestFitAxis";qx.Bootstrap.define(a,{statics:{_isInRange:qx.util.placement.AbstractAxis._isInRange,_moveToEdgeAndAlign:qx.util.placement.AbstractAxis._moveToEdgeAndAlign,computeStart:function(g,c,d,b,f){var e=this._moveToEdgeAndAlign(g,c,d,f);if(this._isInRange(e,g,b)){return e;};if(e<0){e=Math.min(0,b-g);};if(e+g>b){e=Math.max(0,b-g);};return e;}}});})();(function(){var a="Image could not be loaded: ",b="Boolean",c="px",d=".png",e="background-image",f="engine.version",g="scale",h="changeSource",j="x",l="div",m="aborted",n="background-size",o="nonScaled",p="3",q="qx.ui.basic.Image",r="top",s="0 0",t=", no-repeat",u="loaded",v="backgroundImage",w="backgroundRepeat",x="-disabled.$1",y="class",z="qx.event.type.Event",A="image",B="loadingFailed",C="css.alphaimageloaderneeded",D="1.5",E="String",F="browser.documentmode",G="backgroundPosition",H="border-box",I="left",J="_applySource",K="__kk",L="$$widget",M="@",N="px, ",O='.',P="scaled",Q=", ",R="2",S="data:image/gif;base64,R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAAALAAAAAABAAEAAAIBRAA7",T="mshtml",U="engine.name",V=", 0 0",W="_applyScale",X="position",Y="replacement",bl="img",bm="no-repeat",bn="background-position",bh="hidden",bi="alphaScaled",bj=",",bk="absolute";qx.Class.define(q,{extend:qx.ui.core.Widget,construct:function(bo){this.__kk={};qx.ui.core.Widget.call(this);if(bo){this.setSource(bo);};},properties:{source:{check:E,init:null,nullable:true,event:h,apply:J,themeable:true},scale:{check:b,init:false,themeable:true,apply:W},appearance:{refine:true,init:A},allowShrinkX:{refine:true,init:false},allowShrinkY:{refine:true,init:false},allowGrowX:{refine:true,init:false},allowGrowY:{refine:true,init:false}},events:{loadingFailed:z,loaded:z,aborted:z},statics:{PLACEHOLDER_IMAGE:S},members:{__kl:null,__km:null,__iP:null,__kk:null,__kn:null,__ko:null,__kp:0,_onChangeTheme:function(){qx.ui.core.Widget.prototype._onChangeTheme.call(this);this._styleSource();},getContentElement:function(){return this.__kt();},_createContentElement:function(){return this.__kt();},_getContentHint:function(){return {width:this.__kl||0,height:this.__km||0};},_applyDecorator:function(br,bq){qx.ui.core.Widget.prototype._applyDecorator.call(this,br,bq);var bs=this.getSource();bs=qx.util.AliasManager.getInstance().resolve(bs);var bp=this.getContentElement();if(this.__ko){bp=bp.getChild(0);};this.__kB(bp,bs);},_applyPadding:function(bu,bt,name){qx.ui.core.Widget.prototype._applyPadding.call(this,bu,bt,name);var bv=this.getContentElement();if(this.__ko){bv.getChild(0).setStyles({top:this.getPaddingTop()||0,left:this.getPaddingLeft()||0});}else {bv.setPadding(this.getPaddingLeft()||0,this.getPaddingTop()||0);};},renderLayout:function(by,top,bw,bz){qx.ui.core.Widget.prototype.renderLayout.call(this,by,top,bw,bz);var bx=this.getContentElement();if(this.__ko){bx.getChild(0).setStyles({width:bw-(this.getPaddingLeft()||0)-(this.getPaddingRight()||0),height:bz-(this.getPaddingTop()||0)-(this.getPaddingBottom()||0),top:this.getPaddingTop()||0,left:this.getPaddingLeft()||0});};},_applyEnabled:function(bB,bA){qx.ui.core.Widget.prototype._applyEnabled.call(this,bB,bA);if(this.getSource()){this._styleSource();};},_applySource:function(bD,bC){if(bC){if(qx.io.ImageLoader.isLoading(bC)){qx.io.ImageLoader.abort(bC);};};this._styleSource();},_applyScale:function(bE){this._styleSource();},__kq:function(bF){this.__iP=bF;},__kr:function(){if(this.__iP==null){var bH=this.getSource();var bG=false;if(bH!=null){bG=qx.lang.String.endsWith(bH,d);};if(this.getScale()&&bG&&qx.core.Environment.get(C)){this.__iP=bi;}else if(this.getScale()){this.__iP=P;}else {this.__iP=o;};};return this.__iP;},__ks:function(bK){var bJ;var bI;if(bK==bi){bJ=true;bI=l;}else if(bK==o){bJ=false;bI=l;}else {bJ=true;bI=bl;};var bM=new qx.html.Image(bI);bM.setAttribute(L,this.toHashCode());bM.setScale(bJ);bM.setStyles({"overflowX":bh,"overflowY":bh,"boxSizing":H});if(qx.core.Environment.get(C)){var bL=this.__ko=new qx.html.Element(l);bL.setAttribute(L,this.toHashCode());bL.setStyle(X,bk);bL.add(bM);return bL;};return bM;},__kt:function(){if(this.$$disposed){return null;};var bN=this.__kr();if(this.__kk[bN]==null){this.__kk[bN]=this.__ks(bN);};var bO=this.__kk[bN];if(!this.__kn){this.__kn=bO;};return bO;},_styleSource:function(){var bT=qx.util.AliasManager.getInstance();var bS=qx.util.ResourceManager.getInstance();var bV=bT.resolve(this.getSource());var bU=this.getContentElement();if(this.__ko){bU=bU.getChild(0);};if(!bV){bU.resetSource();return;};this.__kw(bV);if((qx.core.Environment.get(U)==T)&&(parseInt(qx.core.Environment.get(f),10)<9||qx.core.Environment.get(F)<9)){var bQ=this.getScale()?g:bm;bU.tagNameHint=qx.bom.element.Decoration.getTagName(bQ,bV);};var bR=this.__kv();if(qx.util.ResourceManager.getInstance().has(bV)){var bW=this._findHighResolutionSource(bV);if(bW){var bY=bS.getImageHeight(bV);var bX=bS.getImageWidth(bV);this.setWidth(bY);this.setHeight(bX);var bP=bY+N+bX+c;this.__kn.setStyle(n,bP);this.setSource(bW);bV=bW;};this.__ky(bR,bV);this.__ku();}else if(qx.io.ImageLoader.isLoaded(bV)){this.__kz(bR,bV);this.__ku();}else {this.__kA(bR,bV);};},__ku:function(){this.__kp++ ;qx.bom.AnimationFrame.request(function(ca){if(ca===this.__kp){this.fireEvent(u);}else {this.fireEvent(m);};}.bind(this,this.__kp));},__kv:function(){var cb=this.__kn;if(this.__ko){cb=cb.getChild(0);};return cb;},__kw:qx.core.Environment.select(U,{"mshtml":function(cd){var ce=qx.core.Environment.get(C);var cc=qx.lang.String.endsWith(cd,d);if(ce&&cc){if(this.getScale()&&this.__kr()!=bi){this.__kq(bi);}else if(!this.getScale()&&this.__kr()!=o){this.__kq(o);};}else {if(this.getScale()&&this.__kr()!=P){this.__kq(P);}else if(!this.getScale()&&this.__kr()!=o){this.__kq(o);};};this.__kx(this.__kt());},"default":function(cf){if(this.getScale()&&this.__kr()!=P){this.__kq(P);}else if(!this.getScale()&&this.__kr(o)){this.__kq(o);};this.__kx(this.__kt());}}),__kx:function(cj){var ci=this.__kn;if(ci!=cj){if(ci!=null){var cu=c;var cg={};var co=this.getBounds();if(co!=null){cg.width=co.width+cu;cg.height=co.height+cu;};var cp=this.getInsets();cg.left=parseInt(ci.getStyle(I)||cp.left)+cu;cg.top=parseInt(ci.getStyle(r)||cp.top)+cu;cg.zIndex=10;var cm=this.__ko?cj.getChild(0):cj;cm.setStyles(cg,true);cm.setSelectable(this.getSelectable());if(!ci.isVisible()){cj.hide();};if(!ci.isIncluded()){cj.exclude();};var cr=ci.getParent();if(cr){var ch=cr.getChildren().indexOf(ci);cr.removeAt(ch);cr.addAt(cj,ch);};var cl=cm.getNodeName();cm.setSource(null);var ck=this.__kv();cm.tagNameHint=cl;cm.setAttribute(y,ck.getAttribute(y));qx.html.Element.flush();var ct=ck.getDomElement();var cs=cj.getDomElement();var cq=ci.getListeners()||[];cq.forEach(function(cv){cj.addListener(cv.type,cv.handler,cv.self,cv.capture);});if(ct&&cs){var cn=ct.$$hash;ct.$$hash=cs.$$hash;cs.$$hash=cn;};this.__kn=cj;};};},__ky:function(cx,cz){var cy=qx.util.ResourceManager.getInstance();if(!this.getEnabled()){var cw=cz.replace(/\.([a-z]+)$/,x);if(cy.has(cw)){cz=cw;this.addState(Y);}else {this.removeState(Y);};};if(cx.getSource()===cz){return;};this.__kB(cx,cz);this.__kD(cy.getImageWidth(cz),cy.getImageHeight(cz));},__kz:function(cA,cE){var cC=qx.io.ImageLoader;this.__kB(cA,cE);var cD=cC.getWidth(cE);var cB=cC.getHeight(cE);this.__kD(cD,cB);},__kA:function(cF,cI){var cJ=qx.io.ImageLoader;{var cH,cG,self;};if(!cJ.isFailed(cI)){cJ.load(cI,this.__kC,this);}else {if(cF!=null){cF.resetSource();};};},__kB:function(cK,cO){if(cK.getNodeName()==l){var cS=qx.theme.manager.Decoration.getInstance().resolve(this.getDecorator());if(cS){var cP=(cS.getStartColor()&&cS.getEndColor());var cN=cS.getBackgroundImage();if(cP||cN){var cL=this.getScale()?g:bm;var cM=qx.bom.element.Decoration.getAttributes(cO,cL);var cR=cS.getStyles(true);var cQ={"backgroundImage":cM.style.backgroundImage,"backgroundPosition":(cM.style.backgroundPosition||s),"backgroundRepeat":(cM.style.backgroundRepeat||bm)};if(cN){cQ[G]+=bj+cR[bn]||s;cQ[w]+=Q+cS.getBackgroundRepeat();};if(cP){cQ[G]+=V;cQ[w]+=t;};cQ[v]+=bj+cR[e];cK.setStyles(cQ);return;};}else {cK.setSource(null);};};cK.setSource(cO);},_findHighResolutionSource:function(cT){var cW=[p,R,D];var cV=parseFloat(qx.bom.client.Device.getDevicePixelRatio().toFixed(2));if(cV<=1){return false;};var i=cW.length;while(i>0&&cV>cW[ --i]){};var cU;var k;for(k=i;k>=0;k-- ){cU=this._getHighResolutionSource(cT,cW[k]);if(cU){return cU;};};for(k=i+1;k<cW.length;k++ ){cU=this._getHighResolutionSource(cT,cW[k]);if(cU){return cU;};};return null;},_getHighResolutionSource:function(cX,cY){var db=cX.lastIndexOf(O);if(db>-1){var da=M+cY+j;var dc=cX.slice(0,db)+da+cX.slice(db);if(qx.util.ResourceManager.getInstance().has(dc)){return dc;};};return null;},__kC:function(dd,de){if(this.$$disposed===true){return;};if(dd!==qx.util.AliasManager.getInstance().resolve(this.getSource())){this.fireEvent(m);return;};if(de.failed){this.warn(a+dd);this.fireEvent(B);}else if(de.aborted){this.fireEvent(m);return;}else {this.fireEvent(u);};this.__kz(this.__kv(),dd);},__kD:function(df,dg){if(df!==this.__kl||dg!==this.__km){this.__kl=df;this.__km=dg;qx.ui.core.queue.Layout.add(this);};}},destruct:function(){for(var dh in this.__kk){if(this.__kk.hasOwnProperty(dh)){this.__kk[dh].setAttribute(L,null,true);};};delete this.__kn;if(this.__ko){delete this.__ko;};this._disposeMap(K);}});})();(function(){var a="mshtml",b='img',c="",d="qx.globalErrorHandling",e="load",f="qx.io.ImageLoader";qx.Bootstrap.define(f,{statics:{__cP:{},__kE:{width:null,height:null},__kF:/\.(png|gif|jpg|jpeg|bmp)\b/i,__kG:/^data:image\/(png|gif|jpg|jpeg|bmp)\b/i,isLoaded:function(g){var h=this.__cP[g];return !!(h&&h.loaded);},isFailed:function(j){var k=this.__cP[j];return !!(k&&k.failed);},isLoading:function(m){var n=this.__cP[m];return !!(n&&n.loading);},getFormat:function(r){var q=this.__cP[r];if(!q||!q.format){var o=this.__kG.exec(r);if(o!=null){var p=(q&&qx.lang.Type.isNumber(q.width)?q.width:this.__kE.width);var s=(q&&qx.lang.Type.isNumber(q.height)?q.height:this.__kE.height);q={loaded:true,format:o[1],width:p,height:s};};};return q?q.format:null;},getSize:function(t){var u=this.__cP[t];return u?{width:u.width,height:u.height}:this.__kE;},getWidth:function(v){var w=this.__cP[v];return w?w.width:null;},getHeight:function(x){var y=this.__cP[x];return y?y.height:null;},load:function(B,A,C){var D=this.__cP[B];if(!D){D=this.__cP[B]={};};if(A&&!C){C=window;};if(D.loaded||D.loading||D.failed){if(A){if(D.loading){D.callbacks.push(A,C);}else {A.call(C,B,D);};};}else {D.loading=true;D.callbacks=[];if(A){D.callbacks.push(A,C);};var z=document.createElement(b);var E=qx.lang.Function.listener(this.__kH,this,z,B);z.onload=E;z.onerror=E;z.src=B;D.element=z;};},abort:function(F){var I=this.__cP[F];if(I&&!I.loaded){I.aborted=true;var H=I.callbacks;var G=I.element;G.onload=G.onerror=null;G.src=c;delete I.callbacks;delete I.element;delete I.loading;for(var i=0,l=H.length;i<l;i+=2){H[i].call(H[i+1],F,I);};};this.__cP[F]=null;},__kH:function(){var J=qx.core.Environment.select(d,{"true":qx.event.GlobalError.observeMethod(this.__kI),"false":this.__kI});J.apply(this,arguments);},__kI:function(event,L,K){var P=this.__cP[K];if(qx.bom.client.Engine.getName()==a&&parseFloat(qx.bom.client.Engine.getVersion())===11){document.body.appendChild(L);};var M=function(Q){return (Q&&Q.height!==0);};if(event.type===e&&M(L)){P.loaded=true;P.width=L.width;P.height=L.height;var N=this.__kF.exec(K);if(N!=null){P.format=N[1];};}else {P.failed=true;};if(qx.bom.client.Engine.getName()==a&&parseFloat(qx.bom.client.Engine.getVersion())===11){document.body.removeChild(L);};L.onload=L.onerror=null;var O=P.callbacks;delete P.loading;delete P.callbacks;delete P.element;for(var i=0,l=O.length;i<l;i+=2){O[i].call(O[i+1],K,P);};},dispose:function(){this.__cP={};}}});})();(function(){var a="source",b="engine.name",c="",d="mshtml",e="px",f="px ",g="no-repeat",h="backgroundImage",i="scale",j="webkit",k="div",l="qx.html.Image",m="qx/static/blank.gif",n="backgroundPosition";qx.Class.define(l,{extend:qx.html.Element,members:{__kJ:null,__kK:null,tagNameHint:null,setPadding:function(o,p){this.__kK=o;this.__kJ=p;if(this.getNodeName()==k){this.setStyle(n,o+f+p+e);};},_applyProperty:function(name,t){qx.html.Element.prototype._applyProperty.call(this,name,t);if(name===a){var s=this.getDomElement();var q=this.getAllStyles();if(this.getNodeName()==k&&this.getStyle(h)){q.backgroundRepeat=null;};var u=this._getProperty(a);var r=this._getProperty(i);var v=r?i:g;if(u!=null){u=u||null;q.paddingTop=this.__kJ;q.paddingLeft=this.__kK;qx.bom.element.Decoration.update(s,u,v,q);};};},_removeProperty:function(x,w){if(x==a){this._setProperty(x,c,w);}else {this._setProperty(x,null,w);};},_createDomElement:function(){var z=this._getProperty(i);var A=z?i:g;if((qx.core.Environment.get(b)==d)){var y=this._getProperty(a);if(this.tagNameHint!=null){this.setNodeName(this.tagNameHint);}else {this.setNodeName(qx.bom.element.Decoration.getTagName(A,y));};}else {this.setNodeName(qx.bom.element.Decoration.getTagName(A));};return qx.html.Element.prototype._createDomElement.call(this);},_copyData:function(B){return qx.html.Element.prototype._copyData.call(this,true);},setSource:function(C){this._setProperty(a,C);return this;},getSource:function(){return this._getProperty(a);},resetSource:function(){if((qx.core.Environment.get(b)==j)){this._setProperty(a,m);}else {this._removeProperty(a,true);};return this;},setScale:function(D){this._setProperty(i,D);return this;},getScale:function(){return this._getProperty(i);}}});})();(function(){var a="qx/icon",b="repeat",c="px",d=".png",f="crop",g="px ",h="background-image",i="scale",j="no-repeat",k="div",l="Potential clipped image candidate: ",m="progid:DXImageTransform.Microsoft.AlphaImageLoader(src='",n='<div style="',o="scale-x",p="css.alphaimageloaderneeded",q="repeat-y",r='<img src="',s="qx.bom.element.Decoration",t="Image modification not possible because elements could not be replaced at runtime anymore!",u="', sizingMethod='",v="",w='"/>',x="png",y="img",z="')",A='"></div>',B="mshtml",C="engine.name",D='" style="',E="none",F="b64",G="webkit",H=" ",I="repeat-x",J="background-repeat",K="DXImageTransform.Microsoft.AlphaImageLoader",L="qx/static/blank.gif",M="scale-y",N="absolute";qx.Class.define(s,{statics:{DEBUG:false,__kL:{},__kM:qx.core.Environment.select(C,{"mshtml":{"scale-x":true,"scale-y":true,"scale":true,"no-repeat":true},"default":null}),__kN:{"scale-x":y,"scale-y":y,"scale":y,"repeat":k,"no-repeat":k,"repeat-x":k,"repeat-y":k},update:function(R,S,P,O){var T=this.getTagName(P,S);if(T!=R.tagName.toLowerCase()){throw new Error(t);};var Q=this.getAttributes(S,P,O);if(T===y){R.src=Q.src||qx.util.ResourceManager.getInstance().toUri(L);};if(R.style.backgroundPosition!=v&&Q.style.backgroundPosition===undefined){Q.style.backgroundPosition=null;};if(R.style.clip!=v&&Q.style.clip===undefined){Q.style.clip=null;};qx.bom.element.Style.setStyles(R,Q.style);if(qx.core.Environment.get(p)){try{R.filters[K].apply();}catch(e){};};},create:function(X,V,U){var Y=this.getTagName(V,X);var W=this.getAttributes(X,V,U);var ba=qx.bom.element.Style.compile(W.style);if(Y===y){return r+W.src+D+ba+w;}else {return n+ba+A;};},getTagName:function(bc,bb){if(bb&&qx.core.Environment.get(p)&&this.__kM[bc]&&qx.lang.String.endsWith(bb,d)){return k;};return this.__kN[bc];},getAttributes:function(bh,be,bd){if(!bd){bd={};};if(!bd.position){bd.position=N;};if((qx.core.Environment.get(C)==B)){bd.fontSize=0;bd.lineHeight=0;}else if((qx.core.Environment.get(C)==G)){bd.WebkitUserDrag=E;};var bf=qx.util.ResourceManager.getInstance().getImageFormat(bh)||qx.io.ImageLoader.getFormat(bh);{};var bi;if(qx.core.Environment.get(p)&&this.__kM[be]&&bf===x){var bj=this.__kP(bh);this.__kO(bd,bj.width,bj.height);bi=this.processAlphaFix(bd,be,bh);}else {delete bd.clip;if(be===i){bi=this.__kQ(bd,be,bh);}else if(be===o||be===M){bi=this.__kR(bd,be,bh);}else {bi=this.__kU(bd,be,bh);};};return bi;},__kO:function(bl,bk,bm){if(bl.width==null&&bk!=null){bl.width=bk+c;};if(bl.height==null&&bm!=null){bl.height=bm+c;};},__kP:function(bn){var bo=qx.util.ResourceManager.getInstance().getImageWidth(bn)||qx.io.ImageLoader.getWidth(bn);var bp=qx.util.ResourceManager.getInstance().getImageHeight(bn)||qx.io.ImageLoader.getHeight(bn);return {width:bo,height:bp};},processAlphaFix:function(bs,bt,br){if(bt==b||bt==I||bt==q){return bs;};var bu=bt==j?f:i;var bq=m+qx.util.ResourceManager.getInstance().toUri(br)+u+bu+z;bs.filter=bq;bs.backgroundImage=bs.backgroundRepeat=v;delete bs[h];delete bs[J];return {style:bs};},__kQ:function(bw,bx,bv){var by=qx.util.ResourceManager.getInstance().toUri(bv);var bz=this.__kP(bv);this.__kO(bw,bz.width,bz.height);return {src:by,style:bw};},__kR:function(bA,bB,bD){var bC=qx.util.ResourceManager.getInstance();var bG=bC.getCombinedFormat(bD);var bI=this.__kP(bD);var bE;if(bG){var bH=bC.getData(bD);var bF=bH[4];if(bG==F){bE=bC.toDataUri(bD);}else {bE=bC.toUri(bF);};if(bB===o){bA=this.__kS(bA,bH,bI.height);}else {bA=this.__kT(bA,bH,bI.width);};return {src:bE,style:bA};}else {{};if(bB==o){bA.height=bI.height==null?null:bI.height+c;}else if(bB==M){bA.width=bI.width==null?null:bI.width+c;};bE=bC.toUri(bD);return {src:bE,style:bA};};},__kS:function(bJ,bK,bM){var bL=qx.util.ResourceManager.getInstance().getImageHeight(bK[4]);bJ.clip={top:-bK[6],height:bM};bJ.height=bL+c;if(bJ.top!=null){bJ.top=(parseInt(bJ.top,10)+bK[6])+c;}else if(bJ.bottom!=null){bJ.bottom=(parseInt(bJ.bottom,10)+bM-bL-bK[6])+c;};return bJ;},__kT:function(bO,bP,bN){var bQ=qx.util.ResourceManager.getInstance().getImageWidth(bP[4]);bO.clip={left:-bP[5],width:bN};bO.width=bQ+c;if(bO.left!=null){bO.left=(parseInt(bO.left,10)+bP[5])+c;}else if(bO.right!=null){bO.right=(parseInt(bO.right,10)+bN-bQ-bP[5])+c;};return bO;},__kU:function(bR,bS,bV){var bU=qx.util.ResourceManager.getInstance();var bT=bU.getCombinedFormat(bV);var ce=this.__kP(bV);if(bT&&bS!==b){var cd=bU.getData(bV);var cb=cd[4];if(bT==F){var bX=bU.toDataUri(bV);var bW=0;var bY=0;}else {var bX=bU.toUri(cb);var bW=cd[5];var bY=cd[6];if(bR.paddingTop||bR.paddingLeft||bR.paddingRight||bR.paddingBottom){var top=bR.paddingTop||0;var cf=bR.paddingLeft||0;bW+=bR.paddingLeft||0;bY+=bR.paddingTop||0;bR.clip={left:cf,top:top,width:ce.width,height:ce.height};};};var ca=qx.bom.element.Background.getStyles(bX,bS,bW,bY);for(var cc in ca){bR[cc]=ca[cc];};if(ce.width!=null&&bR.width==null&&(bS==q||bS===j)){bR.width=ce.width+c;};if(ce.height!=null&&bR.height==null&&(bS==I||bS===j)){bR.height=ce.height+c;};return {style:bR};}else {var top=bR.paddingTop||0;var cf=bR.paddingLeft||0;bR.backgroundPosition=cf+g+top+c;{};this.__kO(bR,ce.width,ce.height);this.__kV(bR,bV,bS);return {style:bR};};},__kV:function(cg,cj,ch){var top=null;var cm=null;if(cg.backgroundPosition){var ci=cg.backgroundPosition.split(H);cm=parseInt(ci[0],10);if(isNaN(cm)){cm=ci[0];};top=parseInt(ci[1],10);if(isNaN(top)){top=ci[1];};};var ck=qx.bom.element.Background.getStyles(cj,ch,cm,top);for(var cl in ck){cg[cl]=ck[cl];};if(cg.filter){cg.filter=v;};},__kW:function(cn){if(this.DEBUG&&qx.util.ResourceManager.getInstance().has(cn)&&cn.indexOf(a)==-1){if(!this.__kL[cn]){qx.log.Logger.debug(l+cn);this.__kL[cn]=true;};};}}});})();(function(){var a="')",b="gecko",c="background-image:url(",d="0",e=");",f="",g="px",h="number",i=")",j="background-repeat:",k="engine.version",l="data:",m=" ",n="qx.bom.element.Background",o=";",p="url(",q="background-position:",r="base64",s="url('",t="engine.name",u="'";qx.Class.define(n,{statics:{__kX:[c,null,e,q,null,o,j,null,o],__kY:{backgroundImage:null,backgroundPosition:null,backgroundRepeat:null},__la:function(z,top){var v=qx.core.Environment.get(t);var x=qx.core.Environment.get(k);if(v==b&&x<1.9&&z==top&&typeof z==h){top+=0.01;};if(z){var y=(typeof z==h)?z+g:z;}else {y=d;};if(top){var w=(typeof top==h)?top+g:top;}else {w=d;};return y+m+w;},__lb:function(A){var String=qx.lang.String;var B=A.substr(0,50);return String.startsWith(B,l)&&String.contains(B,r);},compile:function(F,D,H,top){var G=this.__la(H,top);var E=qx.util.ResourceManager.getInstance().toUri(F);if(this.__lb(E)){E=u+E+u;};var C=this.__kX;C[1]=E;C[4]=G;C[7]=D;return C.join(f);},getStyles:function(L,J,N,top){if(!L){return this.__kY;};var M=this.__la(N,top);var K=qx.util.ResourceManager.getInstance().toUri(L);var O;if(this.__lb(K)){O=s+K+a;}else {O=p+K+i;};var I={backgroundPosition:M,backgroundImage:O};if(J!=null){I.backgroundRepeat=J;};return I;},set:function(T,S,Q,U,top){var P=this.getStyles(S,Q,U,top);for(var R in P){T.style[R]=P[R];};}}});})();(function(){var a="dragdrop-cursor",b="_applyAction",c="alias",d="qx.ui.core.DragDropCursor",e="move",f="singleton",g="copy";qx.Class.define(d,{extend:qx.ui.basic.Image,include:qx.ui.core.MPlacement,type:f,construct:function(){qx.ui.basic.Image.call(this);this.setZIndex(1e8);this.setDomMove(true);var h=this.getApplicationRoot();h.add(this,{left:-1000,top:-1000});},properties:{appearance:{refine:true,init:a},action:{check:[c,g,e],apply:b,nullable:true}},members:{_applyAction:function(j,i){if(i){this.removeState(i);};if(j){this.addState(j);};}}});})();(function(){var a="blur",b="activate",c="focus",d="qx.ui.core.EventHandler";qx.Class.define(d,{extend:qx.core.Object,implement:qx.event.IEventHandler,construct:function(){qx.core.Object.call(this);this.__gt=qx.event.Registration.getManager(window);},statics:{PRIORITY:qx.event.Registration.PRIORITY_FIRST,SUPPORTED_TYPES:{mousemove:1,mouseover:1,mouseout:1,mousedown:1,mouseup:1,click:1,dblclick:1,contextmenu:1,mousewheel:1,keyup:1,keydown:1,keypress:1,keyinput:1,capture:1,losecapture:1,focusin:1,focusout:1,focus:1,blur:1,activate:1,deactivate:1,appear:1,disappear:1,dragstart:1,dragend:1,dragover:1,dragleave:1,drop:1,drag:1,dragchange:1,droprequest:1,touchstart:1,touchend:1,touchmove:1,touchcancel:1,tap:1,longtap:1,swipe:1,dbltap:1,track:1,trackend:1,trackstart:1,pinch:1,rotate:1,roll:1,pointermove:1,pointerover:1,pointerout:1,pointerdown:1,pointerup:1,pointercancel:1},IGNORE_CAN_HANDLE:false},members:{__gt:null,__lc:{focusin:1,focusout:1,focus:1,blur:1},__ld:{mouseover:1,mouseout:1,appear:1,disappear:1},canHandleEvent:function(f,e){return f instanceof qx.ui.core.Widget;},_dispatchEvent:function(j){var o=j.getTarget();var n=qx.ui.core.Widget.getWidgetByElement(o);var p=false;while(n&&n.isAnonymous()){var p=true;n=n.getLayoutParent();};if(n&&p&&j.getType()==b){n.getContentElement().activate();};if(this.__lc[j.getType()]){n=n&&n.getFocusTarget();if(!n){return;};};if(j.getRelatedTarget){var w=j.getRelatedTarget();var v=qx.ui.core.Widget.getWidgetByElement(w);while(v&&v.isAnonymous()){v=v.getLayoutParent();};if(v){if(this.__lc[j.getType()]){v=v.getFocusTarget();};if(v===n){return;};};};var r=j.getCurrentTarget();var t=qx.ui.core.Widget.getWidgetByElement(r);if(!t||t.isAnonymous()){return;};if(this.__lc[j.getType()]){t=t.getFocusTarget();};var u=j.getType();if(!t||!(t.isEnabled()||this.__ld[u])){return;};var g=j.getEventPhase()==qx.event.type.Event.CAPTURING_PHASE;var q=this.__gt.getListeners(t,u,g);if(j.getEventPhase()==qx.event.type.Event.AT_TARGET){if(!q){q=[];};var h=this.__gt.getListeners(t,u,!g);if(h){q=q.concat(h);};};if(!q||q.length===0){return;};var k=qx.event.Pool.getInstance().getObject(j.constructor);j.clone(k);k.setTarget(n);k.setRelatedTarget(v||null);k.setCurrentTarget(t);var x=j.getOriginalTarget();if(x){var m=qx.ui.core.Widget.getWidgetByElement(x);while(m&&m.isAnonymous()){m=m.getLayoutParent();};k.setOriginalTarget(m);}else {k.setOriginalTarget(o);};for(var i=0,l=q.length;i<l;i++ ){var s=q[i].context||t;q[i].handler.call(s,k);};if(k.getPropagationStopped()){j.stopPropagation();};if(k.getDefaultPrevented()){j.preventDefault();};qx.event.Pool.getInstance().poolObject(k);},registerEvent:function(A,z,y){var B;if(z===c||z===a){B=A.getFocusElement();}else {B=A.getContentElement();};if(B){B.addListener(z,this._dispatchEvent,this,y);};},unregisterEvent:function(E,D,C){var F;if(D===c||D===a){F=E.getFocusElement();}else {F=E.getContentElement();};if(F){F.removeListener(D,this._dispatchEvent,this,C);};}},destruct:function(){this.__gt=null;},defer:function(G){qx.event.Registration.addHandler(G);}});})();(function(){var a='indexOf',b='slice',c='concat',d='toLocaleLowerCase',e="qx.type.BaseString",f="",g='trim',h='match',j='toLocaleUpperCase',k='search',m='replace',n='toLowerCase',o='charCodeAt',p='split',q='substring',r='lastIndexOf',s='substr',t='toUpperCase',u='charAt';qx.Class.define(e,{extend:Object,construct:function(v){var v=v||f;this.__le=v;this.length=v.length;},members:{$$isString:true,length:0,__le:null,toString:function(){return this.__le;},charAt:null,valueOf:null,charCodeAt:null,concat:null,indexOf:null,lastIndexOf:null,match:null,replace:null,search:null,slice:null,split:null,substr:null,substring:null,toLowerCase:null,toUpperCase:null,toHashCode:function(){return qx.core.ObjectRegistry.toHashCode(this);},toLocaleLowerCase:null,toLocaleUpperCase:null,base:function(x,w){return qx.core.Object.prototype.base.apply(this,arguments);}},defer:function(y,z){{};var A=[u,o,c,a,r,h,m,k,b,p,s,q,n,t,d,j,g];z.valueOf=z.toString;if(new y(f).valueOf()==null){delete z.valueOf;};for(var i=0,l=A.length;i<l;i++ ){z[A[i]]=String.prototype[A[i]];};}});})();(function(){var a="qx.locale.LocalizedString";qx.Class.define(a,{extend:qx.type.BaseString,construct:function(b,d,c){qx.type.BaseString.call(this,b);this.__lf=d;this.__lg=c;},members:{__lf:null,__lg:null,translate:function(){return qx.locale.Manager.getInstance().translate(this.__lf,this.__lg);},getMessageId:function(){return this.__lf;}}});})();(function(){var a="locale",b="_applyLocale",c="",d="changeLocale",e="_",f="C",g="locale.variant",h="qx.dynlocale",j="qx.locale.Manager",k="String",l="singleton";qx.Class.define(j,{type:l,extend:qx.core.Object,construct:function(){qx.core.Object.call(this);this.__lh=qx.$$translations||{};this.__li=qx.$$locales||{};var m=qx.core.Environment.get(a);var n=qx.core.Environment.get(g);if(n!==c){m+=e+n;};this.__lj=m;this.setLocale(m||this.__lk);},statics:{tr:function(p,q){var o=qx.lang.Array.fromArguments(arguments);o.splice(0,1);return qx.locale.Manager.getInstance().translate(p,o);},trn:function(s,v,r,u){var t=qx.lang.Array.fromArguments(arguments);t.splice(0,3);if(r!=1){return qx.locale.Manager.getInstance().translate(v,t);}else {return qx.locale.Manager.getInstance().translate(s,t);};},trc:function(z,x,y){var w=qx.lang.Array.fromArguments(arguments);w.splice(0,2);return qx.locale.Manager.getInstance().translate(x,w);},trnc:function(B,C,F,A,E){var D=qx.lang.Array.fromArguments(arguments);D.splice(0,4);if(A!=1){return qx.locale.Manager.getInstance().translate(F,D);}else {return qx.locale.Manager.getInstance().translate(C,D);};},marktr:function(G){return G;}},properties:{locale:{check:k,nullable:true,apply:b,event:d}},members:{__lk:f,__ll:null,__lm:null,__lh:null,__li:null,__lj:null,getLanguage:function(){return this.__lm;},getTerritory:function(){return this.getLocale().split(e)[1]||c;},getAvailableLocales:function(I){var J=[];for(var H in this.__li){if(H!=this.__lk){if(this.__li[H]===null&&!I){continue;};J.push(H);};};return J;},__ln:function(K){var M;if(K==null){return null;};var L=K.indexOf(e);if(L==-1){M=K;}else {M=K.substring(0,L);};return M;},_applyLocale:function(O,N){{};this.__ll=O;this.__lm=this.__ln(O);},addTranslation:function(P,S){var Q=this.__lh;if(Q[P]){for(var R in S){Q[P][R]=S[R];};}else {Q[P]=S;};},addLocale:function(W,U){var T=this.__li;if(T[W]){for(var V in U){T[W][V]=U[V];};}else {T[W]=U;};},translate:function(bb,ba,X){var Y=this.__lh;return this.__lo(Y,bb,ba,X);},localize:function(bf,be,bc){var bd=this.__li;return this.__lo(bd,bf,be,bc);},__lo:function(bk,bl,bi,bj){{};var bg;if(!bk){return bl;};if(bj){var bh=this.__ln(bj);}else {bj=this.__ll;bh=this.__lm;};if(!bg&&bk[bj]){bg=bk[bj][bl];};if(!bg&&bk[bh]){bg=bk[bh][bl];};if(!bg&&bk[this.__lk]){bg=bk[this.__lk][bl];};if(!bg){bg=bl;};if(bi.length>0){var bm=[];for(var i=0;i<bi.length;i++ ){var bn=bi[i];if(bn&&bn.translate){bm[i]=bn.translate();}else {bm[i]=bn;};};bg=qx.lang.String.format(bg,bm);};if(qx.core.Environment.get(h)){bg=new qx.locale.LocalizedString(bg,bl,bi);};return bg;}},destruct:function(){this.__lh=this.__li=null;}});})();(function(){var a="qx.bom.client.Locale",b="-",c="locale",d="",e="android",f="locale.variant";qx.Bootstrap.define(a,{statics:{getLocale:function(){var g=qx.bom.client.Locale.__lp();var h=g.indexOf(b);if(h!=-1){g=g.substr(0,h);};return g;},getVariant:function(){var i=qx.bom.client.Locale.__lp();var k=d;var j=i.indexOf(b);if(j!=-1){k=i.substr(j+1);};return k;},__lp:function(){var l=(navigator.userLanguage||navigator.language||d);if(qx.bom.client.OperatingSystem.getName()==e){var m=/(\w{2})-(\w{2})/i.exec(navigator.userAgent);if(m){l=m[0];};};return l.toLowerCase();}},defer:function(n){qx.core.Environment.add(c,n.getLocale);qx.core.Environment.add(f,n.getVariant);}});})();(function(){var a="qx.ui.core.MChildrenHandling";qx.Mixin.define(a,{members:{getChildren:function(){return this._getChildren();},hasChildren:function(){return this._hasChildren();},indexOf:function(b){return this._indexOf(b);},add:function(d,c){this._add(d,c);},addAt:function(g,e,f){this._addAt(g,e,f);},addBefore:function(h,j,i){this._addBefore(h,j,i);},addAfter:function(m,k,l){this._addAfter(m,k,l);},remove:function(n){this._remove(n);},removeAt:function(o){return this._removeAt(o);},removeAll:function(){return this._removeAll();}},statics:{remap:function(p){p.getChildren=p._getChildren;p.hasChildren=p._hasChildren;p.indexOf=p._indexOf;p.add=p._add;p.addAt=p._addAt;p.addBefore=p._addBefore;p.addAfter=p._addAfter;p.remove=p._remove;p.removeAt=p._removeAt;p.removeAll=p._removeAll;}}});})();(function(){var a="qx.ui.container.Composite",b="addChildWidget",c="removeChildWidget",d="qx.event.type.Data";qx.Class.define(a,{extend:qx.ui.core.Widget,include:[qx.ui.core.MChildrenHandling,qx.ui.core.MLayoutHandling],construct:function(e){qx.ui.core.Widget.call(this);if(e!=null){this._setLayout(e);};},events:{addChildWidget:d,removeChildWidget:d},members:{_afterAddChild:function(f){this.fireNonBubblingEvent(b,qx.event.type.Data,[f]);},_afterRemoveChild:function(g){this.fireNonBubblingEvent(c,qx.event.type.Data,[g]);}},defer:function(h,i){qx.ui.core.MChildrenHandling.remap(i);qx.ui.core.MLayoutHandling.remap(i);}});})();(function(){var a="qx.ui.popup.Popup",b="visible",c="excluded",d="popup",e="Boolean";qx.Class.define(a,{extend:qx.ui.container.Composite,include:qx.ui.core.MPlacement,construct:function(f){qx.ui.container.Composite.call(this,f);this.initVisibility();},properties:{appearance:{refine:true,init:d},visibility:{refine:true,init:c},autoHide:{check:e,init:true}},members:{show:function(){if(this.getLayoutParent()==null){qx.core.Init.getApplication().getRoot().add(this);};qx.ui.container.Composite.prototype.show.call(this);},_applyVisibility:function(i,h){qx.ui.container.Composite.prototype._applyVisibility.call(this,i,h);var g=qx.ui.popup.Manager.getInstance();i===b?g.add(this):g.remove(this);}},destruct:function(){if(!qx.ui.popup.Manager.getInstance().isDisposed()){qx.ui.popup.Manager.getInstance().remove(this);};}});})();(function(){var a="__lq",b="blur",c="pointerdown",d="singleton",f="qx.ui.popup.Manager";qx.Class.define(f,{type:d,extend:qx.core.Object,construct:function(){qx.core.Object.call(this);this.__lq=[];qx.event.Registration.addListener(document.documentElement,c,this.__ls,this,true);qx.bom.Element.addListener(window,b,this.hideAll,this);},members:{__lq:null,add:function(g){{};this.__lq.push(g);this.__lr();},remove:function(h){{};qx.lang.Array.remove(this.__lq,h);this.__lr();},hideAll:function(){var l=this.__lq.length,j={};while(l-- ){j=this.__lq[l];if(j.getAutoHide()){j.exclude();};};},__lr:function(){var k=1e7;for(var i=0;i<this.__lq.length;i++ ){this.__lq[i].setZIndex(k++ );};},__ls:function(e){var n=qx.ui.core.Widget.getWidgetByElement(e.getTarget());var o=this.__lq;for(var i=0;i<o.length;i++ ){var m=o[i];if(!m.getAutoHide()||n==m||qx.ui.core.Widget.contains(m,n)){continue;};m.exclude();};}},destruct:function(){qx.event.Registration.removeListener(document.documentElement,c,this.__ls,this,true);this._disposeArray(a);}});})();(function(){var a="_applyRich",b="qx.ui.tooltip.ToolTip",c="_applyIcon",d="tooltip",f="pointerover",g="qx.ui.core.Widget",h="arrow",i="Boolean",j="_applyArrowPosition",k="left",l="right",m="_applyLabel",n="Integer",o="String",p="atom";qx.Class.define(b,{extend:qx.ui.popup.Popup,construct:function(q,r){qx.ui.popup.Popup.call(this);this.setLayout(new qx.ui.layout.HBox());this._createChildControl(h);this._createChildControl(p);if(q!=null){this.setLabel(q);};if(r!=null){this.setIcon(r);};this.addListener(f,this._onPointerOver,this);},properties:{appearance:{refine:true,init:d},showTimeout:{check:n,init:700,themeable:true},hideTimeout:{check:n,init:4000,themeable:true},label:{check:o,nullable:true,apply:m},icon:{check:o,nullable:true,apply:c,themeable:true},rich:{check:i,init:false,apply:a},opener:{check:g,nullable:true},arrowPosition:{check:[k,l],init:k,themeable:true,apply:j}},members:{_forwardStates:{placementLeft:true},_createChildControlImpl:function(u,t){var s;switch(u){case p:s=new qx.ui.basic.Atom();this._add(s,{flex:1});break;case h:s=new qx.ui.basic.Image();this._add(s);};return s||qx.ui.popup.Popup.prototype._createChildControlImpl.call(this,u);},_onPointerOver:function(e){},_applyIcon:function(w,v){var x=this.getChildControl(p);w==null?x.resetIcon():x.setIcon(w);},_applyLabel:function(z,y){var A=this.getChildControl(p);z==null?A.resetLabel():A.setLabel(z);},_applyRich:function(C,B){var D=this.getChildControl(p);D.setRich(C);},_applyArrowPosition:function(F,E){this._getLayout().setReversed(F==k);}}});})();(function(){var a="Missing renderLayout() implementation!",b="abstract",c="It is not possible to manually set the connected widget.",d="qx.ui.layout.Abstract",e="Missing getHeightForWidth() implementation!";qx.Class.define(d,{type:b,extend:qx.core.Object,members:{__eY:null,_invalidChildrenCache:null,__lt:null,invalidateLayoutCache:function(){this.__eY=null;},renderLayout:function(g,h,f){this.warn(a);},getSizeHint:function(){if(this.__eY){return this.__eY;};return this.__eY=this._computeSizeHint();},hasHeightForWidth:function(){return false;},getHeightForWidth:function(i){this.warn(e);return null;},_computeSizeHint:function(){return null;},invalidateChildrenCache:function(){this._invalidChildrenCache=true;},verifyLayoutProperty:null,_clearSeparators:function(){var j=this.__lt;if(j instanceof qx.ui.core.LayoutItem){j.clearSeparators();};},_renderSeparator:function(k,l){this.__lt.renderSeparator(k,l);},connectToWidget:function(m){if(m&&this.__lt){throw new Error(c);};this.__lt=m;this.invalidateChildrenCache();},_getWidget:function(){return this.__lt;},_applyLayoutChange:function(){if(this.__lt){this.__lt.scheduleLayoutUpdate();};},_getLayoutChildren:function(){return this.__lt.getLayoutChildren();}},destruct:function(){this.__lt=this.__eY=null;}});})();(function(){var a="Decorator",b="middle",c="_applyLayoutChange",d="_applyReversed",e="bottom",f="center",g="Boolean",h="top",j="left",k="right",m="Integer",n="qx.ui.layout.HBox";qx.Class.define(n,{extend:qx.ui.layout.Abstract,construct:function(o,p,q){qx.ui.layout.Abstract.call(this);if(o){this.setSpacing(o);};if(p){this.setAlignX(p);};if(q){this.setSeparator(q);};},properties:{alignX:{check:[j,f,k],init:j,apply:c},alignY:{check:[h,b,e],init:h,apply:c},spacing:{check:m,init:0,apply:c},separator:{check:a,nullable:true,apply:c},reversed:{check:g,init:false,apply:d}},members:{__lu:null,__lv:null,__lw:null,__gP:null,_applyReversed:function(){this._invalidChildrenCache=true;this._applyLayoutChange();},__lx:function(){var w=this._getLayoutChildren();var length=w.length;var t=false;var r=this.__lu&&this.__lu.length!=length&&this.__lv&&this.__lu;var u;var s=r?this.__lu:new Array(length);var v=r?this.__lv:new Array(length);if(this.getReversed()){w=w.concat().reverse();};for(var i=0;i<length;i++ ){u=w[i].getLayoutProperties();if(u.width!=null){s[i]=parseFloat(u.width)/100;};if(u.flex!=null){v[i]=u.flex;t=true;}else {v[i]=0;};};if(!r){this.__lu=s;this.__lv=v;};this.__lw=t;this.__gP=w;delete this._invalidChildrenCache;},verifyLayoutProperty:null,renderLayout:function(N,H,M){if(this._invalidChildrenCache){this.__lx();};var D=this.__gP;var length=D.length;var P=qx.ui.layout.Util;var L=this.getSpacing();var R=this.getSeparator();if(R){var A=P.computeHorizontalSeparatorGaps(D,L,R);}else {var A=P.computeHorizontalGaps(D,L,true);};var i,O,J,I;var Q=[];var E=A;for(i=0;i<length;i+=1){I=this.__lu[i];J=I!=null?Math.floor((N-A)*I):D[i].getSizeHint().width;Q.push(J);E+=J;};if(this.__lw&&E!=N){var G={};var K,y;for(i=0;i<length;i+=1){K=this.__lv[i];if(K>0){F=D[i].getSizeHint();G[i]={min:F.minWidth,value:Q[i],max:F.maxWidth,flex:K};};};var B=P.computeFlexOffsets(G,N,E);for(i in B){y=B[i].offset;Q[i]+=y;E+=y;};};var V=D[0].getMarginLeft();if(E<N&&this.getAlignX()!=j){V=N-E;if(this.getAlignX()===f){V=Math.round(V/2);};};var F,top,z,J,C,T,x;var L=this.getSpacing();this._clearSeparators();if(R){var S=qx.theme.manager.Decoration.getInstance().resolve(R).getInsets();var U=S.left+S.right;};for(i=0;i<length;i+=1){O=D[i];J=Q[i];F=O.getSizeHint();T=O.getMarginTop();x=O.getMarginBottom();z=Math.max(F.minHeight,Math.min(H-T-x,F.maxHeight));top=P.computeVerticalAlignOffset(O.getAlignY()||this.getAlignY(),z,H,T,x);if(i>0){if(R){V+=C+L;this._renderSeparator(R,{left:V+M.left,top:M.top,width:U,height:H});V+=U+L+O.getMarginLeft();}else {V+=P.collapseMargins(L,C,O.getMarginLeft());};};O.renderLayout(V+M.left,top+M.top,J,z);V+=J;C=O.getMarginRight();};},_computeSizeHint:function(){if(this._invalidChildrenCache){this.__lx();};var bl=qx.ui.layout.Util;var X=this.__gP;var bd=0,be=0,W=0;var bb=0,bc=0;var bi,Y,bk;for(var i=0,l=X.length;i<l;i+=1){bi=X[i];Y=bi.getSizeHint();be+=Y.width;var bh=this.__lv[i];var ba=this.__lu[i];if(bh){bd+=Y.minWidth;}else if(ba){W=Math.max(W,Math.round(Y.minWidth/ba));}else {bd+=Y.width;};bk=bi.getMarginTop()+bi.getMarginBottom();if((Y.height+bk)>bc){bc=Y.height+bk;};if((Y.minHeight+bk)>bb){bb=Y.minHeight+bk;};};bd+=W;var bg=this.getSpacing();var bj=this.getSeparator();if(bj){var bf=bl.computeHorizontalSeparatorGaps(X,bg,bj);}else {var bf=bl.computeHorizontalGaps(X,bg,true);};return {minWidth:bd+bf,width:be+bf,minHeight:bb,height:bc};}},destruct:function(){this.__lu=this.__lv=this.__gP=null;}});})();(function(){var a="middle",b="qx.ui.layout.Util",c="left",d="center",e="top",f="bottom",g="right";qx.Class.define(b,{statics:{PERCENT_VALUE:/[0-9]+(?:\.[0-9]+)?%/,computeFlexOffsets:function(j,n,h){var r,q,s,k;var m=n>h;var t=Math.abs(n-h);var u,o;var p={};for(q in j){r=j[q];p[q]={potential:m?r.max-r.value:r.value-r.min,flex:m?r.flex:1/r.flex,offset:0};};while(t!=0){k=Infinity;s=0;for(q in p){r=p[q];if(r.potential>0){s+=r.flex;k=Math.min(k,r.potential/r.flex);};};if(s==0){break;};k=Math.min(t,k*s)/s;u=0;for(q in p){r=p[q];if(r.potential>0){o=Math.min(t,r.potential,Math.ceil(k*r.flex));u+=o-k*r.flex;if(u>=1){u-=1;o-=1;};r.potential-=o;if(m){r.offset+=o;}else {r.offset-=o;};t-=o;};};};return p;},computeHorizontalAlignOffset:function(w,v,y,z,A){if(z==null){z=0;};if(A==null){A=0;};var x=0;switch(w){case c:x=z;break;case g:x=y-v-A;break;case d:x=Math.round((y-v)/2);if(x<z){x=z;}else if(x<A){x=Math.max(z,y-v-A);};break;};return x;},computeVerticalAlignOffset:function(C,F,B,G,D){if(G==null){G=0;};if(D==null){D=0;};var E=0;switch(C){case e:E=G;break;case f:E=B-F-D;break;case a:E=Math.round((B-F)/2);if(E<G){E=G;}else if(E<D){E=Math.max(G,B-F-D);};break;};return E;},collapseMargins:function(K){var I=0,H=0;for(var i=0,l=arguments.length;i<l;i++ ){var J=arguments[i];if(J<0){H=Math.min(H,J);}else if(J>0){I=Math.max(I,J);};};return I+H;},computeHorizontalGaps:function(O,M,L){if(M==null){M=0;};var N=0;if(L){N+=O[0].getMarginLeft();for(var i=1,l=O.length;i<l;i+=1){N+=this.collapseMargins(M,O[i-1].getMarginRight(),O[i].getMarginLeft());};N+=O[l-1].getMarginRight();}else {for(var i=1,l=O.length;i<l;i+=1){N+=O[i].getMarginLeft()+O[i].getMarginRight();};N+=(M*(l-1));};return N;},computeVerticalGaps:function(S,Q,P){if(Q==null){Q=0;};var R=0;if(P){R+=S[0].getMarginTop();for(var i=1,l=S.length;i<l;i+=1){R+=this.collapseMargins(Q,S[i-1].getMarginBottom(),S[i].getMarginTop());};R+=S[l-1].getMarginBottom();}else {for(var i=1,l=S.length;i<l;i+=1){R+=S[i].getMarginTop()+S[i].getMarginBottom();};R+=(Q*(l-1));};return R;},computeHorizontalSeparatorGaps:function(bb,U,Y){var T=qx.theme.manager.Decoration.getInstance().resolve(Y);var V=T.getInsets();var W=V.left+V.right;var X=0;for(var i=0,l=bb.length;i<l;i++ ){var ba=bb[i];X+=ba.getMarginLeft()+ba.getMarginRight();};X+=(U+W+U)*(l-1);return X;},computeVerticalSeparatorGaps:function(bj,bc,bh){var bf=qx.theme.manager.Decoration.getInstance().resolve(bh);var be=bf.getInsets();var bd=be.top+be.bottom;var bg=0;for(var i=0,l=bj.length;i<l;i++ ){var bi=bj[i];bg+=bi.getMarginTop()+bi.getMarginBottom();};bg+=(bc+bd+bc)*(l-1);return bg;},arrangeIdeals:function(bl,bn,bk,bm,bo,bp){if(bn<bl||bo<bm){if(bn<bl&&bo<bm){bn=bl;bo=bm;}else if(bn<bl){bo-=(bl-bn);bn=bl;if(bo<bm){bo=bm;};}else if(bo<bm){bn-=(bm-bo);bo=bm;if(bn<bl){bn=bl;};};};if(bn>bk||bo>bp){if(bn>bk&&bo>bp){bn=bk;bo=bp;}else if(bn>bk){bo+=(bn-bk);bn=bk;if(bo>bp){bo=bp;};}else if(bo>bp){bn+=(bo-bp);bo=bp;if(bn>bk){bn=bk;};};};return {begin:bn,end:bo};}}});})();(function(){var a="Boolean",b="changeGap",c="changeShow",d="bottom",e="bottom-right",f="_applyCenter",g="changeIcon",h="qx.ui.basic.Atom",i="changeLabel",j="both",k="Integer",l="_applyIconPosition",m="bottom-left",n="String",o="icon",p="top-left",q="top",r="top-right",s="right",t="_applyRich",u="_applyIcon",v="label",w="_applyShow",x="left",y="_applyLabel",z="_applyGap",A="atom";qx.Class.define(h,{extend:qx.ui.core.Widget,construct:function(B,C){{};qx.ui.core.Widget.call(this);this._setLayout(new qx.ui.layout.Atom());if(B!=null){this.setLabel(B);};if(C!==undefined){this.setIcon(C);};},properties:{appearance:{refine:true,init:A},label:{apply:y,nullable:true,check:n,event:i},rich:{check:a,init:false,apply:t},icon:{check:n,apply:u,nullable:true,themeable:true,event:g},gap:{check:k,nullable:false,event:b,apply:z,themeable:true,init:4},show:{init:j,check:[j,v,o],themeable:true,inheritable:true,apply:w,event:c},iconPosition:{init:x,check:[q,s,d,x,p,m,r,e],themeable:true,apply:l},center:{init:false,check:a,themeable:true,apply:f}},members:{_createChildControlImpl:function(F,E){var D;switch(F){case v:D=new qx.ui.basic.Label(this.getLabel());D.setAnonymous(true);D.setRich(this.getRich());this._add(D);if(this.getLabel()==null||this.getShow()===o){D.exclude();};break;case o:D=new qx.ui.basic.Image(this.getIcon());D.setAnonymous(true);this._addAt(D,0);if(this.getIcon()==null||this.getShow()===v){D.exclude();};break;};return D||qx.ui.core.Widget.prototype._createChildControlImpl.call(this,F);},_forwardStates:{focused:true,hovered:true},_handleLabel:function(){if(this.getLabel()==null||this.getShow()===o){this._excludeChildControl(v);}else {this._showChildControl(v);};},_handleIcon:function(){if(this.getIcon()==null||this.getShow()===v){this._excludeChildControl(o);}else {this._showChildControl(o);};},_applyLabel:function(H,G){var I=this.getChildControl(v,true);if(I){I.setValue(H);};this._handleLabel();},_applyRich:function(K,J){var L=this.getChildControl(v,true);if(L){L.setRich(K);};},_applyIcon:function(N,M){var O=this.getChildControl(o,true);if(O){O.setSource(N);};this._handleIcon();},_applyGap:function(Q,P){this._getLayout().setGap(Q);},_applyShow:function(S,R){this._handleLabel();this._handleIcon();},_applyIconPosition:function(U,T){this._getLayout().setIconPosition(U);},_applyCenter:function(W,V){this._getLayout().setCenter(W);},_applySelectable:function(Y,X){qx.ui.core.Widget.prototype._applySelectable.call(this,Y,X);var ba=this.getChildControl(v,true);if(ba){this.getChildControl(v).setSelectable(Y);};}}});})();(function(){var a="middle",b="_applyLayoutChange",c="top-right",d="bottom",e="top-left",f="bottom-left",g="center",h="qx.ui.layout.Atom",j="bottom-right",k="top",l="left",m="right",n="Integer",o="Boolean";qx.Class.define(h,{extend:qx.ui.layout.Abstract,properties:{gap:{check:n,init:4,apply:b},iconPosition:{check:[l,k,m,d,e,f,c,j],init:l,apply:b},center:{check:o,init:false,apply:b}},members:{verifyLayoutProperty:null,renderLayout:function(E,y,D){var N=D.left;var top=D.top;var z=qx.ui.layout.Util;var q=this.getIconPosition();var t=this._getLayoutChildren();var length=t.length;var M,r;var G,x;var C=this.getGap();var J=this.getCenter();var L=[d,m,c,j];if(L.indexOf(q)!=-1){var A=length-1;var v=-1;var s=-1;}else {var A=0;var v=length;var s=1;};if(q==k||q==d){if(J){var F=0;for(var i=A;i!=v;i+=s){r=t[i].getSizeHint().height;if(r>0){F+=r;if(i!=A){F+=C;};};};top+=Math.round((y-F)/2);};var u=top;for(var i=A;i!=v;i+=s){G=t[i];x=G.getSizeHint();M=Math.min(x.maxWidth,Math.max(E,x.minWidth));r=x.height;N=z.computeHorizontalAlignOffset(g,M,E)+D.left;G.renderLayout(N,u,M,r);if(r>0){u=top+r+C;};};}else {var w=E;var p=null;var I=0;for(var i=A;i!=v;i+=s){G=t[i];M=G.getSizeHint().width;if(M>0){if(!p&&G instanceof qx.ui.basic.Label){p=G;}else {w-=M;};I++ ;};};if(I>1){var H=(I-1)*C;w-=H;};if(p){var x=p.getSizeHint();var B=Math.max(x.minWidth,Math.min(w,x.maxWidth));w-=B;};if(J&&w>0){N+=Math.round(w/2);};for(var i=A;i!=v;i+=s){G=t[i];x=G.getSizeHint();r=Math.min(x.maxHeight,Math.max(y,x.minHeight));if(G===p){M=B;}else {M=x.width;};var K=a;if(q==e||q==c){K=k;}else if(q==f||q==j){K=d;};var u=top+z.computeVerticalAlignOffset(K,x.height,y);G.renderLayout(N,u,M,r);if(M>0){N+=M+C;};};};},_computeSizeHint:function(){var Y=this._getLayoutChildren();var length=Y.length;var P,W;if(length===1){var P=Y[0].getSizeHint();W={width:P.width,height:P.height,minWidth:P.minWidth,minHeight:P.minHeight};}else {var U=0,V=0;var R=0,T=0;var S=this.getIconPosition();var Q=this.getGap();if(S===k||S===d){var O=0;for(var i=0;i<length;i++ ){P=Y[i].getSizeHint();V=Math.max(V,P.width);U=Math.max(U,P.minWidth);if(P.height>0){T+=P.height;R+=P.minHeight;O++ ;};};if(O>1){var X=(O-1)*Q;T+=X;R+=X;};}else {var O=0;for(var i=0;i<length;i++ ){P=Y[i].getSizeHint();T=Math.max(T,P.height);R=Math.max(R,P.minHeight);if(P.width>0){V+=P.width;U+=P.minWidth;O++ ;};};if(O>1){var X=(O-1)*Q;V+=X;U+=X;};};W={minWidth:U,width:V,minHeight:R,height:T};};return W;}}});})();(function(){var a="qx.event.type.Data",b="qx.ui.form.IStringForm";qx.Interface.define(b,{events:{"changeValue":a},members:{setValue:function(c){return arguments.length==1;},resetValue:function(){},getValue:function(){}}});})();(function(){var a="safari",b="os.name",c="_applyTextAlign",d="Boolean",f="qx.ui.core.Widget",g="nowrap",h="changeStatus",i="changeTextAlign",j="_applyWrap",k="changeValue",l="browser.name",m="color",n="qx.ui.basic.Label",o="osx",p="css.textoverflow",q="html.xul",r="_applyValue",s="center",t="_applyBuddy",u="enabled",v="String",w="toggleValue",x="whiteSpace",y="textAlign",z="function",A="browser.version",B="qx.dynlocale",C="engine.version",D="right",E="gecko",F="justify",G="changeRich",H="normal",I="_applyRich",J="engine.name",K="label",L="changeLocale",M="left",N="tap",O="A";qx.Class.define(n,{extend:qx.ui.core.Widget,implement:[qx.ui.form.IStringForm],construct:function(P){qx.ui.core.Widget.call(this);if(P!=null){this.setValue(P);};if(qx.core.Environment.get(B)){qx.locale.Manager.getInstance().addListener(L,this._onChangeLocale,this);};},properties:{rich:{check:d,init:false,event:G,apply:I},wrap:{check:d,init:true,apply:j},value:{check:v,apply:r,event:k,nullable:true},buddy:{check:f,apply:t,nullable:true,init:null,dereference:true},textAlign:{check:[M,s,D,F],nullable:true,themeable:true,apply:c,event:i},appearance:{refine:true,init:K},selectable:{refine:true,init:false},allowGrowX:{refine:true,init:false},allowGrowY:{refine:true,init:false},allowShrinkY:{refine:true,init:false}},members:{__ly:null,__lz:null,__lA:null,__lB:null,_getContentHint:function(){if(this.__lz){this.__lC=this.__lD();delete this.__lz;};return {width:this.__lC.width,height:this.__lC.height};},_hasHeightForWidth:function(){return this.getRich()&&this.getWrap();},_applySelectable:function(Q){if(!qx.core.Environment.get(p)&&qx.core.Environment.get(q)){if(Q&&!this.isRich()){{};return;};};qx.ui.core.Widget.prototype._applySelectable.call(this,Q);},_getContentHeightForWidth:function(R){if(!this.getRich()&&!this.getWrap()){return null;};return this.__lD(R).height;},_createContentElement:function(){return new qx.html.Label;},_applyTextAlign:function(T,S){this.getContentElement().setStyle(y,T);},_applyTextColor:function(V,U){if(V){this.getContentElement().setStyle(m,qx.theme.manager.Color.getInstance().resolve(V));}else {this.getContentElement().removeStyle(m);};},__lC:{width:0,height:0},_applyFont:function(Y,X){if(X&&this.__ly&&this.__lB){this.__ly.removeListenerById(this.__lB);this.__lB=null;};var W;if(Y){this.__ly=qx.theme.manager.Font.getInstance().resolve(Y);if(this.__ly instanceof qx.bom.webfonts.WebFont){this.__lB=this.__ly.addListener(h,this._onWebFontStatusChange,this);};W=this.__ly.getStyles();}else {this.__ly=null;W=qx.bom.Font.getDefaultStyles();};if(this.getTextColor()!=null){delete W[m];};this.getContentElement().setStyles(W);this.__lz=true;qx.ui.core.queue.Layout.add(this);},__lD:function(bc){var bb=qx.bom.Label;var be=this.getFont();var ba=be?this.__ly.getStyles():qx.bom.Font.getDefaultStyles();var content=this.getValue()||O;var bd=this.getRich();if(this.__lB){this.__lE();};return bd?bb.getHtmlSize(content,ba,bc):bb.getTextSize(content,ba);},__lE:function(){if(!this.getContentElement()){return;};if(qx.core.Environment.get(b)==o&&qx.core.Environment.get(J)==E&&parseInt(qx.core.Environment.get(C),10)<16&&parseInt(qx.core.Environment.get(C),10)>9){var bf=this.getContentElement().getDomElement();if(bf){bf.innerHTML=bf.innerHTML;};};},_applyBuddy:function(bh,bg){if(bg!=null){this.removeRelatedBindings(bg);this.removeListenerById(this.__lA);this.__lA=null;};if(bh!=null){bh.bind(u,this,u);this.__lA=this.addListener(N,function(){if(bh.isFocusable()){bh.focus.apply(bh);};if(w in bh&&typeof bh.toggleValue===z){bh.toggleValue();};},this);};},_applyRich:function(bi){this.getContentElement().setRich(bi);this.__lz=true;qx.ui.core.queue.Layout.add(this);},_applyWrap:function(bl,bj){if(bl&&!this.isRich()){{};};if(this.isRich()){var bk=bl?H:g;this.getContentElement().setStyle(x,bk);};},_onChangeLocale:qx.core.Environment.select(B,{"true":function(e){var content=this.getValue();if(content&&content.translate){this.setValue(content.translate());};},"false":null}),_onWebFontStatusChange:function(bm){if(bm.getData().valid===true){if(qx.core.Environment.get(l)==a&&parseFloat(qx.core.Environment.get(A))>=8){window.setTimeout(function(){this.__lz=true;qx.ui.core.queue.Layout.add(this);}.bind(this),0);};this.__lz=true;qx.ui.core.queue.Layout.add(this);};},_applyValue:function(bo,bn){this.getContentElement().setValue(bo);this.__lz=true;qx.ui.core.queue.Layout.add(this);}},destruct:function(){if(qx.core.Environment.get(B)){qx.locale.Manager.getInstance().removeListener(L,this._onChangeLocale,this);};if(this.__ly&&this.__lB){this.__ly.removeListenerById(this.__lB);};this.__ly=null;}});})();(function(){var a="value",b="qx.html.Label",c="The label mode cannot be modified after initial creation",d='hidden';qx.Class.define(b,{extend:qx.html.Element,members:{__lF:null,_applyProperty:function(name,e){qx.html.Element.prototype._applyProperty.call(this,name,e);if(name==a){var f=this.getDomElement();qx.bom.Label.setValue(f,e);};},_createDomElement:function(){var h=this.__lF;var g=qx.bom.Label.create(this._content,h);g.style.overflow=d;return g;},_copyData:function(i){return qx.html.Element.prototype._copyData.call(this,true);},setRich:function(j){var k=this.getDomElement();if(k){throw new Error(c);};j=!!j;if(this.__lF==j){return this;};this.__lF=j;return this;},setValue:function(l){this._setProperty(a,l);return this;},getValue:function(){return this._getProperty(a);}}});})();(function(){var a="text",b="function",c="px",d="http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul",e="crop",f="nowrap",g="end",h="div",i="100%",j="auto",k="0",l="css.textoverflow",m="html.xul",n="value",o="visible",p="qx.bom.Label",q="",r="ellipsis",s="normal",t="inherit",u="block",v="label",w="-1000px",x="hidden",y="absolute";qx.Bootstrap.define(p,{statics:{__lG:{fontFamily:1,fontSize:1,fontWeight:1,fontStyle:1,lineHeight:1},__lH:function(){var z=this.__lJ(false);document.body.insertBefore(z,document.body.firstChild);return this._textElement=z;},__lI:function(){var A=this.__lJ(true);document.body.insertBefore(A,document.body.firstChild);return this._htmlElement=A;},__lJ:function(D){var B=qx.dom.Element.create(h);var C=B.style;C.width=C.height=j;C.left=C.top=w;C.visibility=x;C.position=y;C.overflow=o;C.display=u;if(D){C.whiteSpace=s;}else {C.whiteSpace=f;if(!qx.core.Environment.get(l)&&qx.core.Environment.get(m)){var E=document.createElementNS(d,v);var C=E.style;C.padding=k;C.margin=k;C.width=j;for(var F in this.__lG){C[F]=t;};B.appendChild(E);};};return B;},__lK:function(H){var G={};if(H){G.whiteSpace=s;}else if(!qx.core.Environment.get(l)&&qx.core.Environment.get(m)){G.display=u;}else {G.overflow=x;G.whiteSpace=f;G[qx.core.Environment.get(l)]=r;};return G;},create:function(content,L,K){if(!K){K=window;};var I=K.document.createElement(h);if(L){I.useHtml=true;}else if(!qx.core.Environment.get(l)&&qx.core.Environment.get(m)){var M=K.document.createElementNS(d,v);var J=M.style;J.cursor=t;J.color=t;J.overflow=x;J.maxWidth=i;J.padding=k;J.margin=k;J.width=j;for(var N in this.__lG){M.style[N]=t;};M.setAttribute(e,g);I.appendChild(M);}else {qx.bom.element.Style.setStyles(I,this.__lK(L));};if(content){this.setValue(I,content);};return I;},__lL:null,setSanitizer:function(O){{};qx.bom.Label.__lL=O;},setValue:function(Q,P){P=P||q;if(Q.useHtml){if(qx.bom.Label.__lL&&typeof (qx.bom.Label.__lL)===b){P=qx.bom.Label.__lL(P);};Q.innerHTML=P;}else if(!qx.core.Environment.get(l)&&qx.core.Environment.get(m)){Q.firstChild.setAttribute(n,P);}else {qx.bom.element.Attribute.set(Q,a,P);};},getValue:function(R){if(R.useHtml){return R.innerHTML;}else if(!qx.core.Environment.get(l)&&qx.core.Environment.get(m)){return R.firstChild.getAttribute(n)||q;}else {return qx.bom.element.Attribute.get(R,a);};},getHtmlSize:function(content,S,T){var U=this._htmlElement||this.__lI();U.style.width=T!=undefined?T+c:j;U.innerHTML=content;return this.__lM(U,S);},getTextSize:function(W,V){var X=this._textElement||this.__lH();if(!qx.core.Environment.get(l)&&qx.core.Environment.get(m)){X.firstChild.setAttribute(n,W);}else {qx.bom.element.Attribute.set(X,a,W);};return this.__lM(X,V);},__lM:function(bd,Y){var ba=this.__lG;if(!Y){Y={};};for(var bc in ba){bd.style[bc]=Y[bc]||q;};var bb=qx.bom.element.Dimension.getSize(bd);bb.width++ ;return bb;}}});})();(function(){var a="qx.ui.form.IForm",b="qx.event.type.Data";qx.Interface.define(a,{events:{"changeEnabled":b,"changeValid":b,"changeInvalidMessage":b,"changeRequired":b},members:{setEnabled:function(c){return arguments.length==1;},getEnabled:function(){},setRequired:function(d){return arguments.length==1;},getRequired:function(){},setValid:function(e){return arguments.length==1;},getValid:function(){},setInvalidMessage:function(f){return arguments.length==1;},getInvalidMessage:function(){},setRequiredInvalidMessage:function(g){return arguments.length==1;},getRequiredInvalidMessage:function(){}}});})();(function(){var a="qx.application.Standalone";qx.Class.define(a,{extend:qx.application.AbstractGui,members:{_createRootWidget:function(){return new qx.ui.root.Application(document);}}});})();(function(){var a="_applyActiveWindow",b="changeModal",c="__lN",d="windowAdded",f="changeVisibility",g="__gt",h="windowRemoved",i="qx.ui.window.Window",j="changeActive",k="qx.ui.window.MDesktop",l="changeActiveWindow",m="qx.event.type.Data";qx.Mixin.define(k,{properties:{activeWindow:{check:i,apply:a,event:l,init:null,nullable:true}},events:{windowAdded:m,windowRemoved:m},members:{__lN:null,__gt:null,getWindowManager:function(){if(!this.__gt){this.setWindowManager(new qx.ui.window.Window.DEFAULT_MANAGER_CLASS());};return this.__gt;},supportsMaximize:function(){return true;},setWindowManager:function(n){if(this.__gt){this.__gt.setDesktop(null);};n.setDesktop(this);this.__gt=n;},_onChangeActive:function(e){if(e.getData()){this.setActiveWindow(e.getTarget());}else if(this.getActiveWindow()==e.getTarget()){this.setActiveWindow(null);};},_applyActiveWindow:function(p,o){this.getWindowManager().changeActiveWindow(p,o);this.getWindowManager().updateStack();},_onChangeModal:function(e){this.getWindowManager().updateStack();},_onChangeVisibility:function(){this.getWindowManager().updateStack();},_afterAddChild:function(q){if(qx.Class.isDefined(i)&&q instanceof qx.ui.window.Window){this._addWindow(q);};},_addWindow:function(r){if(!qx.lang.Array.contains(this.getWindows(),r)){this.getWindows().push(r);this.fireDataEvent(d,r);r.addListener(j,this._onChangeActive,this);r.addListener(b,this._onChangeModal,this);r.addListener(f,this._onChangeVisibility,this);};if(r.getActive()){this.setActiveWindow(r);};this.getWindowManager().updateStack();},_afterRemoveChild:function(s){if(qx.Class.isDefined(i)&&s instanceof qx.ui.window.Window){this._removeWindow(s);};},_removeWindow:function(t){if(qx.lang.Array.contains(this.getWindows(),t)){qx.lang.Array.remove(this.getWindows(),t);this.fireDataEvent(h,t);t.removeListener(j,this._onChangeActive,this);t.removeListener(b,this._onChangeModal,this);t.removeListener(f,this._onChangeVisibility,this);this.getWindowManager().updateStack();};},getWindows:function(){if(!this.__lN){this.__lN=[];};return this.__lN;}},destruct:function(){this._disposeArray(c);this._disposeObjects(g);}});})();(function(){var a="_applyBlockerColor",b="Number",c="qx.ui.core.MBlocker",d="__lO",e="_applyBlockerOpacity",f="Color";qx.Mixin.define(c,{properties:{blockerColor:{check:f,init:null,nullable:true,apply:a,themeable:true},blockerOpacity:{check:b,init:1,apply:e,themeable:true}},members:{__lO:null,_createBlocker:function(){return new qx.ui.core.Blocker(this);},_applyBlockerColor:function(h,g){this.getBlocker().setColor(h);},_applyBlockerOpacity:function(j,i){this.getBlocker().setOpacity(j);},block:function(){this.getBlocker().block();},isBlocked:function(){return this.__lO&&this.__lO.isBlocked();},unblock:function(){if(this.__lO){this.__lO.unblock();};},forceUnblock:function(){if(this.__lO){this.__lO.forceUnblock();};},blockContent:function(k){this.getBlocker().blockContent(k);},getBlocker:function(){if(!this.__lO){this.__lO=this._createBlocker();};return this.__lO;}},destruct:function(){this._disposeObjects(d);}});})();(function(){var a="qx.dyntheme",b="backgroundColor",c="_applyOpacity",d="Boolean",f="px",g="keydown",h="deactivate",j="changeTheme",k="opacity",l="Tab",m="qx.event.type.Event",n="move",o="__lO",p="Color",q="resize",r="__de",s="zIndex",t="appear",u="qx.ui.root.Abstract",v="keyup",w="keypress",x="Number",y="unblocked",z="qx.ui.core.Blocker",A="disappear",B="blocked",C="_applyColor";qx.Class.define(z,{extend:qx.core.Object,events:{blocked:m,unblocked:m},construct:function(D){qx.core.Object.call(this);this._widget=D;D.addListener(q,this.__lT,this);D.addListener(n,this.__lT,this);D.addListener(A,this.__lV,this);if(qx.Class.isDefined(u)&&D instanceof qx.ui.root.Abstract){this._isRoot=true;this.setKeepBlockerActive(true);};if(qx.core.Environment.get(a)){qx.theme.manager.Meta.getInstance().addListener(j,this._onChangeTheme,this);};this.__lP=[];this.__lQ=[];},properties:{color:{check:p,init:null,nullable:true,apply:C,themeable:true},opacity:{check:x,init:1,apply:c,themeable:true},keepBlockerActive:{check:d,init:false}},members:{__lO:null,__lR:0,__lP:null,__lQ:null,__de:null,_widget:null,_isRoot:false,__lS:null,__lT:function(e){var E=e.getData();if(this.isBlocked()){this._updateBlockerBounds(E);};},__lU:function(){this._updateBlockerBounds(this._widget.getBounds());if(this._widget.isRootWidget()){this._widget.getContentElement().add(this.getBlockerElement());}else {this._widget.getLayoutParent().getContentElement().add(this.getBlockerElement());};},__lV:function(){if(this.isBlocked()){this.getBlockerElement().getParent().remove(this.getBlockerElement());this._widget.addListenerOnce(t,this.__lU,this);};},_updateBlockerBounds:function(F){this.getBlockerElement().setStyles({width:F.width+f,height:F.height+f,left:F.left+f,top:F.top+f});},_applyColor:function(I,H){var G=qx.theme.manager.Color.getInstance().resolve(I);this.__lW(b,G);},_applyOpacity:function(K,J){this.__lW(k,K);},_onChangeTheme:qx.core.Environment.select(a,{"true":function(){this._applyColor(this.getColor());},"false":null}),__lW:function(M,N){var L=[];this.__lO&&L.push(this.__lO);for(var i=0;i<L.length;i++ ){L[i].setStyle(M,N);};},_backupActiveWidget:function(){var O=qx.event.Registration.getManager(window).getHandler(qx.event.handler.Focus);this.__lP.push(qx.ui.core.Widget.getWidgetByElement(O.getActive()));this.__lQ.push(qx.ui.core.Widget.getWidgetByElement(O.getFocus()));if(this._widget.isFocusable()){this._widget.focus();};},_restoreActiveWidget:function(){var Q;var P=this.__lQ.length;if(P>0){Q=this.__lQ.pop();if(Q&&!Q.isDisposed()&&Q.isFocusable()){Q.focus();};};var R=this.__lP.length;if(R>0){Q=this.__lP.pop();if(Q&&!Q.isDisposed()){Q.activate();};};},__lX:function(){return new qx.html.Blocker(this.getColor(),this.getOpacity());},getBlockerElement:function(S){if(!this.__lO){this.__lO=this.__lX();this.__lO.setStyle(s,15);if(!S){if(this._isRoot){S=this._widget;}else {S=this._widget.getLayoutParent();};};S.getContentElement().add(this.__lO);this.__lO.exclude();};return this.__lO;},block:function(){this._block();},_block:function(T,V){if(!this._isRoot&&!this._widget.getLayoutParent()){this.__lS=this._widget.addListenerOnce(t,this._block.bind(this,T));return;};var parent;if(this._isRoot||V){parent=this._widget;}else {parent=this._widget.getLayoutParent();};var U=this.getBlockerElement(parent);if(T!=null){U.setStyle(s,T);};this.__lR++ ;if(this.__lR<2){this._backupActiveWidget();var W=this._widget.getBounds();if(W){this._updateBlockerBounds(W);};U.include();if(!V){U.activate();};U.addListener(h,this.__mb,this);U.addListener(w,this.__ma,this);U.addListener(g,this.__ma,this);U.addListener(v,this.__ma,this);this.fireEvent(B,qx.event.type.Event);};},isBlocked:function(){return this.__lR>0;},unblock:function(){if(this.__lS){this._widget.removeListenerById(this.__lS);};if(!this.isBlocked()){return;};this.__lR-- ;if(this.__lR<1){this.__lY();this.__lR=0;};},forceUnblock:function(){if(!this.isBlocked()){return;};this.__lR=0;this.__lY();},__lY:function(){this._restoreActiveWidget();var X=this.getBlockerElement();X.removeListener(h,this.__mb,this);X.removeListener(w,this.__ma,this);X.removeListener(g,this.__ma,this);X.removeListener(v,this.__ma,this);X.exclude();this.fireEvent(y,qx.event.type.Event);},blockContent:function(Y){this._block(Y,true);},__ma:function(e){if(e.getKeyIdentifier()==l){e.stop();};},__mb:function(){if(this.getKeepBlockerActive()){this.getBlockerElement().activate();};}},destruct:function(){if(qx.core.Environment.get(a)){qx.theme.manager.Meta.getInstance().removeListener(j,this._onChangeTheme,this);};this._widget.removeListener(q,this.__lT,this);this._widget.removeListener(n,this.__lT,this);this._widget.removeListener(t,this.__lU,this);this._widget.removeListener(A,this.__lV,this);if(this.__lS){this._widget.removeListenerById(this.__lS);};this._disposeObjects(o,r);this.__lP=this.__lQ=this._widget=null;}});})();(function(){var a="swipe",b="repeat",c="mousedown",d="url(",f="pointerover",g=")",h="longtap",i="mouseout",j="div",k="roll",l="cursor",m="dblclick",n="mousewheel",o="qx.html.Blocker",p="mousemove",q="dbltap",r="pointerup",s="mouseover",t="appear",u="click",v="pointerdown",w="mshtml",x="engine.name",y="mouseup",z="contextmenu",A="disappear",B="tap",C="pointermove",D="pointerout",E="qx/static/blank.gif",F="absolute";qx.Class.define(o,{extend:qx.html.Element,construct:function(I,G){var I=I?qx.theme.manager.Color.getInstance().resolve(I):null;var H={position:F,opacity:G||0,backgroundColor:I};if((qx.core.Environment.get(x)==w)){H.backgroundImage=d+qx.util.ResourceManager.getInstance().toUri(E)+g;H.backgroundRepeat=b;};qx.html.Element.call(this,j,H);this.addListener(c,this._stopPropagation,this);this.addListener(y,this._stopPropagation,this);this.addListener(u,this._stopPropagation,this);this.addListener(m,this._stopPropagation,this);this.addListener(p,this._stopPropagation,this);this.addListener(s,this._stopPropagation,this);this.addListener(i,this._stopPropagation,this);this.addListener(n,this._stopPropagation,this);this.addListener(k,this._stopPropagation,this);this.addListener(z,this._stopPropagation,this);this.addListener(v,this._stopPropagation,this);this.addListener(r,this._stopPropagation,this);this.addListener(C,this._stopPropagation,this);this.addListener(f,this._stopPropagation,this);this.addListener(D,this._stopPropagation,this);this.addListener(B,this._stopPropagation,this);this.addListener(q,this._stopPropagation,this);this.addListener(a,this._stopPropagation,this);this.addListener(h,this._stopPropagation,this);this.addListener(t,this.__mc,this);this.addListener(A,this.__mc,this);},members:{_stopPropagation:function(e){e.stopPropagation();},__mc:function(){var J=this.getStyle(l);this.setStyle(l,null,true);this.setStyle(l,J,true);}}});})();(function(){var a="changeGlobalCursor",b="engine.name",c="keypress",d="Boolean",f="root",g="help",h="",i="contextmenu",j=" !important",k="input",l="_applyGlobalCursor",m="Space",n="_applyNativeHelp",o=";",p="event.help",q="qx.ui.root.Abstract",r="abstract",s="textarea",t="String",u="*";qx.Class.define(q,{type:r,extend:qx.ui.core.Widget,include:[qx.ui.core.MChildrenHandling,qx.ui.core.MBlocker,qx.ui.window.MDesktop],construct:function(){qx.ui.core.Widget.call(this);qx.ui.core.FocusHandler.getInstance().addRoot(this);qx.ui.core.queue.Visibility.add(this);this.initNativeHelp();this.addListener(c,this.__me,this);},properties:{appearance:{refine:true,init:f},enabled:{refine:true,init:true},focusable:{refine:true,init:true},globalCursor:{check:t,nullable:true,themeable:true,apply:l,event:a},nativeContextMenu:{refine:true,init:false},nativeHelp:{check:d,init:false,apply:n}},members:{__md:null,isRootWidget:function(){return true;},getLayout:function(){return this._getLayout();},_applyGlobalCursor:qx.core.Environment.select(b,{"mshtml":function(w,v){},"default":function(A,z){var y=qx.bom.Stylesheet;var x=this.__md;if(!x){this.__md=x=y.createElement();};y.removeAllRules(x);if(A){y.addRule(x,u,qx.bom.element.Cursor.compile(A).replace(o,h)+j);};}}),_applyNativeContextMenu:function(C,B){if(C){this.removeListener(i,this._onNativeContextMenu,this,true);}else {this.addListener(i,this._onNativeContextMenu,this,true);};},_onNativeContextMenu:function(e){if(e.getTarget().getNativeContextMenu()){return;};e.preventDefault();},__me:function(e){if(e.getKeyIdentifier()!==m){return;};var E=e.getTarget();var D=qx.ui.core.FocusHandler.getInstance();if(!D.isFocused(E)){return;};var F=E.getContentElement().getNodeName();if(F===k||F===s){return;};e.preventDefault();},_applyNativeHelp:function(H,G){if(qx.core.Environment.get(p)){if(G===false){qx.bom.Event.removeNativeListener(document,g,(function(){return false;}));};if(H===false){qx.bom.Event.addNativeListener(document,g,(function(){return false;}));};};}},destruct:function(){this.__md=null;},defer:function(I,J){qx.ui.core.MChildrenHandling.remap(J);}});})();(function(){var a="keypress",b="focusout",c="activate",d="Tab",f="singleton",g="deactivate",h="__mf",j="focusin",k="qx.ui.core.FocusHandler";qx.Class.define(k,{extend:qx.core.Object,type:f,construct:function(){qx.core.Object.call(this);this.__mf={};},members:{__mf:null,__mg:null,__mh:null,__mi:null,connectTo:function(m){m.addListener(a,this.__hV,this);m.addListener(j,this._onFocusIn,this,true);m.addListener(b,this._onFocusOut,this,true);m.addListener(c,this._onActivate,this,true);m.addListener(g,this._onDeactivate,this,true);},addRoot:function(n){this.__mf[n.$$hash]=n;},removeRoot:function(o){delete this.__mf[o.$$hash];},getActiveWidget:function(){return this.__mg;},isActive:function(p){return this.__mg==p;},getFocusedWidget:function(){return this.__mh;},isFocused:function(q){return this.__mh==q;},isFocusRoot:function(r){return !!this.__mf[r.$$hash];},_onActivate:function(e){var t=e.getTarget();this.__mg=t;var s=this.__mj(t);if(s!=this.__mi){this.__mi=s;};},_onDeactivate:function(e){var u=e.getTarget();if(this.__mg==u){this.__mg=null;};},_onFocusIn:function(e){var v=e.getTarget();if(v!=this.__mh){this.__mh=v;v.visualizeFocus();};},_onFocusOut:function(e){var w=e.getTarget();if(w==this.__mh){this.__mh=null;w.visualizeBlur();};},__hV:function(e){if(e.getKeyIdentifier()!=d){return;};if(!this.__mi){return;};e.stopPropagation();e.preventDefault();var x=this.__mh;if(!e.isShiftPressed()){var y=x?this.__mn(x):this.__ml();}else {var y=x?this.__mo(x):this.__mm();};if(y){y.tabFocus();};},__mj:function(z){var A=this.__mf;while(z){if(A[z.$$hash]){return z;};z=z.getLayoutParent();};return null;},__mk:function(I,H){if(I===H){return 0;};var C=I.getTabIndex()||0;var B=H.getTabIndex()||0;if(C!=B){return C-B;};var J=I.getContentElement().getDomElement();var G=H.getContentElement().getDomElement();var F=qx.bom.element.Location;var E=F.get(J);var D=F.get(G);if(E.top!=D.top){return E.top-D.top;};if(E.left!=D.left){return E.left-D.left;};var K=I.getZIndex();var L=H.getZIndex();if(K!=L){return K-L;};return 0;},__ml:function(){return this.__mr(this.__mi,null);},__mm:function(){return this.__ms(this.__mi,null);},__mn:function(M){var N=this.__mi;if(N==M){return this.__ml();};while(M&&M.getAnonymous()){M=M.getLayoutParent();};if(M==null){return [];};var O=[];this.__mp(N,M,O);O.sort(this.__mk);var P=O.length;return P>0?O[0]:this.__ml();},__mo:function(Q){var R=this.__mi;if(R==Q){return this.__mm();};while(Q&&Q.getAnonymous()){Q=Q.getLayoutParent();};if(Q==null){return [];};var S=[];this.__mq(R,Q,S);S.sort(this.__mk);var T=S.length;return T>0?S[T-1]:this.__mm();},__mp:function(parent,U,V){var X=parent.getLayoutChildren();var W;for(var i=0,l=X.length;i<l;i++ ){W=X[i];if(!(W instanceof qx.ui.core.Widget)){continue;};if(!this.isFocusRoot(W)&&W.isEnabled()&&W.isVisible()){if(W.isTabable()&&this.__mk(U,W)<0){V.push(W);};this.__mp(W,U,V);};};},__mq:function(parent,Y,ba){var bc=parent.getLayoutChildren();var bb;for(var i=0,l=bc.length;i<l;i++ ){bb=bc[i];if(!(bb instanceof qx.ui.core.Widget)){continue;};if(!this.isFocusRoot(bb)&&bb.isEnabled()&&bb.isVisible()){if(bb.isTabable()&&this.__mk(Y,bb)>0){ba.push(bb);};this.__mq(bb,Y,ba);};};},__mr:function(parent,bd){var bf=parent.getLayoutChildren();var be;for(var i=0,l=bf.length;i<l;i++ ){be=bf[i];if(!(be instanceof qx.ui.core.Widget)){continue;};if(!this.isFocusRoot(be)&&be.isEnabled()&&be.isVisible()){if(be.isTabable()){if(bd==null||this.__mk(be,bd)<0){bd=be;};};bd=this.__mr(be,bd);};};return bd;},__ms:function(parent,bg){var bi=parent.getLayoutChildren();var bh;for(var i=0,l=bi.length;i<l;i++ ){bh=bi[i];if(!(bh instanceof qx.ui.core.Widget)){continue;};if(!this.isFocusRoot(bh)&&bh.isEnabled()&&bh.isVisible()){if(bh.isTabable()){if(bg==null||this.__mk(bh,bg)>0){bg=bh;};};bg=this.__ms(bh,bg);};};return bg;}},destruct:function(){this._disposeMap(h);this.__mh=this.__mg=this.__mi=null;}});})();(function(){var a="touchmove",b="os.name",c="-webkit-overflow-scrolling",d="touch",f="paddingLeft",g="div",h="100%",i="The root widget does not support 'left', or 'top' paddings!",j="0px",k="The application could not be started due to a missing body tag in the HTML file!",l="ios",m="overflowY",n="resize",o="",p="$$widget",q="paddingTop",r="engine.name",s="none",t="webkit",u="-webkit-backface-visibility",v="touch-action",w="qx.ui.root.Application",x="hidden",y="tap",z="overflowX",A="absolute";qx.Class.define(w,{extend:qx.ui.root.Abstract,construct:function(B){this.__cz=qx.dom.Node.getWindow(B);this.__mt=B;qx.ui.root.Abstract.call(this);qx.event.Registration.addListener(this.__cz,n,this._onResize,this);this._setLayout(new qx.ui.layout.Canvas());qx.ui.core.queue.Layout.add(this);qx.ui.core.FocusHandler.getInstance().connectTo(this);this.getContentElement().disableScrolling();this.getContentElement().setStyle(u,x);this.addListener(a,this.__mu,this);if(qx.core.Environment.get(b)==l){this.getContentElement().addListener(y,function(e){var C=qx.ui.core.Widget.getWidgetByElement(e.getTarget());while(C&&!C.isFocusable()){C=C.getLayoutParent();};if(C&&C.isFocusable()){C.getContentElement().focus();};},this,true);};},members:{__cz:null,__mt:null,_createContentElement:function(){var D=this.__mt;if((qx.core.Environment.get(r)==t)){if(!D.body){alert(k);};};var H=D.documentElement.style;var E=D.body.style;H.overflow=E.overflow=x;H.padding=H.margin=E.padding=E.margin=j;H.width=H.height=E.width=E.height=h;var G=D.createElement(g);D.body.appendChild(G);var F=new qx.html.Root(G);F.setStyles({"position":A,"overflowX":x,"overflowY":x});F.setAttribute(p,this.toHashCode());return F;},_onResize:function(e){qx.ui.core.queue.Layout.add(this);if(qx.ui.popup&&qx.ui.popup.Manager){qx.ui.popup.Manager.getInstance().hideAll();};if(qx.ui.menu&&qx.ui.menu.Manager){qx.ui.menu.Manager.getInstance().hideAll();};},_computeSizeHint:function(){var I=qx.bom.Viewport.getWidth(this.__cz);var J=qx.bom.Viewport.getHeight(this.__cz);return {minWidth:I,width:I,maxWidth:I,minHeight:J,height:J,maxHeight:J};},_applyPadding:function(L,K,name){if(L&&(name==q||name==f)){throw new Error(i);};qx.ui.root.Abstract.prototype._applyPadding.call(this,L,K,name);},__mu:function(e){var M=e.getOriginalTarget();while(M&&M.style){var Q=qx.bom.element.Style.get(M,v)!==s&&qx.bom.element.Style.get(M,v)!==o;var P=qx.bom.element.Style.get(M,c)===d;var O=qx.bom.element.Style.get(M,z)!=x;var N=qx.bom.element.Style.get(M,m)!=x;if(Q||P||N||O){return;};M=M.parentNode;};e.preventDefault();}},destruct:function(){this.__cz=this.__mt=null;}});})();(function(){var a="qx.ui.layout.Canvas",b="number",c="Boolean";qx.Class.define(a,{extend:qx.ui.layout.Abstract,properties:{desktop:{check:c,init:false}},members:{verifyLayoutProperty:null,renderLayout:function(g,j,m){var s=this._getLayoutChildren();var d,u,r;var f,top,e,h,n,k;var q,p,t,o;for(var i=0,l=s.length;i<l;i++ ){d=s[i];u=d.getSizeHint();r=d.getLayoutProperties();q=d.getMarginTop();p=d.getMarginRight();t=d.getMarginBottom();o=d.getMarginLeft();f=r.left!=null?r.left:r.edge;if(qx.lang.Type.isString(f)){f=Math.round(parseFloat(f)*g/100);};e=r.right!=null?r.right:r.edge;if(qx.lang.Type.isString(e)){e=Math.round(parseFloat(e)*g/100);};top=r.top!=null?r.top:r.edge;if(qx.lang.Type.isString(top)){top=Math.round(parseFloat(top)*j/100);};h=r.bottom!=null?r.bottom:r.edge;if(qx.lang.Type.isString(h)){h=Math.round(parseFloat(h)*j/100);};if(f!=null&&e!=null){n=g-f-e-o-p;if(n<u.minWidth){n=u.minWidth;}else if(n>u.maxWidth){n=u.maxWidth;};f+=o;}else {n=r.width;if(n==null){n=u.width;}else {n=Math.round(parseFloat(n)*g/100);if(n<u.minWidth){n=u.minWidth;}else if(n>u.maxWidth){n=u.maxWidth;};};if(e!=null){f=g-n-e-p-o;}else if(f==null){f=o;}else {f+=o;};};if(top!=null&&h!=null){k=j-top-h-q-t;if(k<u.minHeight){k=u.minHeight;}else if(k>u.maxHeight){k=u.maxHeight;};top+=q;}else {k=r.height;if(k==null){k=u.height;}else {k=Math.round(parseFloat(k)*j/100);if(k<u.minHeight){k=u.minHeight;}else if(k>u.maxHeight){k=u.maxHeight;};};if(h!=null){top=j-k-h-t-q;}else if(top==null){top=q;}else {top+=q;};};f+=m.left;top+=m.top;d.renderLayout(f,top,n,k);};},_computeSizeHint:function(){var M=0,y=0;var J=0,I=0;var H,v;var E,C;var L=this._getLayoutChildren();var w,B,z;var K=this.isDesktop();var A,top,x,D;for(var i=0,l=L.length;i<l;i++ ){w=L[i];B=w.getLayoutProperties();z=w.getSizeHint();var G=w.getMarginLeft()+w.getMarginRight();var F=w.getMarginTop()+w.getMarginBottom();H=z.width+G;v=z.minWidth+G;A=B.left!=null?B.left:B.edge;if(A&&typeof A===b){H+=A;v+=A;};x=B.right!=null?B.right:B.edge;if(x&&typeof x===b){H+=x;v+=x;};M=Math.max(M,H);y=K?0:Math.max(y,v);E=z.height+F;C=z.minHeight+F;top=B.top!=null?B.top:B.edge;if(top&&typeof top===b){E+=top;C+=top;};D=B.bottom!=null?B.bottom:B.edge;if(D&&typeof D===b){E+=D;C+=D;};J=Math.max(J,E);I=K?0:Math.max(I,C);};return {width:M,minWidth:y,height:J,minHeight:I};}}});})();(function(){var a="qx.html.Root";qx.Class.define(a,{extend:qx.html.Element,construct:function(b){qx.html.Element.call(this);if(b!=null){this.useElement(b);};},members:{useElement:function(c){qx.html.Element.prototype.useElement.call(this,c);this.setRoot(true);qx.html.Element._modified[this.$$hash]=this;}}});})();(function(){var a='loaders/desktop_loader',b="qx_lib.Application",c='Application';qx.Class.define(b,{extend:qx.application.Standalone,properties:{appInterface:{nullable:true}},members:{main:function(){qx.application.Standalone.prototype.main.call(this);this.collect_all();window.Opal.require(a);window.Opal.get(c).$start(this.getRoot());},collect_all:function(){this.collect_data();this.collect_html();this.collect_lib();this.collect_ui();this.collect_util();},collect_data:function(){this.collect_data_controller();this.collect_data_marshal();},collect_data_controller:function(){var d;d=qx.data.controller.List;},collect_data_marshal:function(){var e;e=qx.data.marshal.Json;},collect_html:function(){var f;f=qx.html.Element;},collect_lib:function(){this.collect_lib_io();this.collect_lib_ui();},collect_lib_io:function(){var g;g=qx_lib.io.Server;},collect_lib_ui:function(){var h;h=qx_lib.data.ListModel;h=qx_lib.ui.grid.Grid;},collect_ui:function(){this.collect_ui_basic();this.collect_ui_container();this.collect_ui_core();this.collect_ui_embed();this.collect_ui_form();this.collect_ui_layout();this.collect_ui_menu();this.collect_ui_menubar();this.collect_ui_splitpane();this.collect_ui_style();this.collect_ui_tabview();this.collect_ui_tree();this.collect_ui_window();},collect_ui_basic:function(){var i;i=qx.ui.basic.Atom;i=qx.ui.basic.Image;i=qx.ui.basic.Label;},collect_ui_container:function(){var j;j=qx.ui.container.Composite;j=qx.ui.container.Resizer;j=qx.ui.container.Scroll;j=qx.ui.container.SlideBar;j=qx.ui.container.Stack;},collect_ui_core:function(){var k;k=qx.ui.core.Widget;},collect_ui_embed:function(){var l;l=qx.ui.embed.Html;},collect_ui_form:function(){var m;m=qx.ui.form.Button;m=qx.ui.form.CheckBox;m=qx.ui.form.ComboBox;m=qx.ui.form.DateField;m=qx.ui.form.Form;m=qx.ui.form.List;m=qx.ui.form.ListItem;m=qx.ui.form.MenuButton;m=qx.ui.form.PasswordField;m=qx.ui.form.RadioButton;m=qx.ui.form.RadioButtonGroup;m=qx.ui.form.RadioGroup;m=qx.ui.form.RepeatButton;m=qx.ui.form.Resetter;m=qx.ui.form.SelectBox;m=qx.ui.form.Slider;m=qx.ui.form.Spinner;m=qx.ui.form.SplitButton;m=qx.ui.form.TextArea;m=qx.ui.form.TextField;m=qx.ui.form.ToggleButton;m=qx.ui.form.VirtualComboBox;m=qx.ui.form.VirtualSelectBox;this.collect_ui_form_renderer();},collect_ui_form_renderer:function(){var n;n=qx.ui.form.renderer.Double;n=qx.ui.form.renderer.Single;n=qx.ui.form.renderer.SinglePlaceholder;},collect_ui_layout:function(){var o;o=qx.ui.layout.Basic;o=qx.ui.layout.Canvas;o=qx.ui.layout.HBox;o=qx.ui.layout.VBox;},collect_ui_menu:function(){var p;p=qx.ui.menu.Button;p=qx.ui.menu.Menu;},collect_ui_menubar:function(){var q;q=qx.ui.menubar.Button;q=qx.ui.menubar.MenuBar;},collect_ui_splitpane:function(){var r;r=qx.ui.splitpane.Pane;},collect_ui_style:function(){var s;s=qx.ui.style.Stylesheet;},collect_ui_tabview:function(){var t;t=qx.ui.tabview.Page;t=qx.ui.tabview.TabButton;t=qx.ui.tabview.TabView;},collect_ui_tree:function(){var u;u=qx.ui.tree.Tree;u=qx.ui.tree.TreeFile;u=qx.ui.tree.TreeFolder;u=qx.ui.tree.VirtualTree;u=qx.ui.tree.VirtualTreeItem;},collect_ui_window:function(){var v;v=qx.ui.window.Window;},collect_util:function(){var w;w=qx.util.Serializer;}}});})();(function(){var a="change",b="qx.data.Array",c="qx.data.controller.MSelection",d="_applySelection",e="Target property is needed.",f="changeSelection",g="target",h="qx.event.type.Data";qx.Mixin.define(c,{construct:function(){if(!qx.Class.hasProperty(this.constructor,g)){throw new Error(e);};if(this.getSelection()==null){this.__mv=new qx.data.Array();this.setSelection(this.__mv);};},properties:{selection:{check:b,event:f,apply:d,init:null}},events:{"changeSelection":h},members:{_modifingSelection:0,__mw:null,__mx:null,__mv:null,_applySelection:function(k,j){if(this.__mx!=undefined&&j!=undefined){j.removeListenerById(this.__mx);};this.__mx=k.addListener(a,this.__my,this);this._updateSelection();},__my:function(){this._updateSelection();},_changeTargetSelection:function(){if(this.getTarget()==null){return;};if(!this.__mz()&&!this.__mA()){return;};if(this._inSelectionModification()){return;};var m=this.getTarget().getSelection();var l=this.getSelection();if(l==null){l=new qx.data.Array();this.__mv=l;this.setSelection(l);};var n=[0,l.getLength()];for(var i=0;i<m.length;i++ ){n.push(m[i].getModel());};l.splice.apply(l,n).dispose();this.fireDataEvent(f,this.getSelection());},_addChangeTargetListener:function(p,o){if(this.__mw!=undefined&&o!=undefined){o.removeListenerById(this.__mw);};if(p!=null){if(this.__mz()||this.__mA()){this.__mw=p.addListener(f,this._changeTargetSelection,this);};};},_updateSelection:function(){if(!this.getTarget()){return;};this._startSelectionModification();if(this.__mz()){var s=[];for(var i=0;i<this.getSelection().length;i++ ){var t=this.getSelection().getItem(i);var u=this.__mC(t);if(u!=null){s.push(u);};};this.getTarget().setSelection(s);s=this.getTarget().getSelection();var q=[];for(var i=0;i<s.length;i++ ){q[i]=s[i].getModel();};for(var i=this.getSelection().length-1;i>=0;i-- ){if(!qx.lang.Array.contains(q,this.getSelection().getItem(i))){this.getSelection().splice(i,1).dispose();};};}else if(this.__mA()){var r=this.getSelection().getItem(this.getSelection().length-1);if(r!==undefined){this.__mB(r);this.getSelection().splice(0,this.getSelection().getLength()-1).dispose();}else {this.getTarget().resetSelection();};};this._endSelectionModification();},__mz:function(){var v=this.getTarget().constructor;return qx.Class.implementsInterface(v,qx.ui.core.IMultiSelection);},__mA:function(){var w=this.getTarget().constructor;return qx.Class.implementsInterface(w,qx.ui.core.ISingleSelection);},__mB:function(x){var y=this.__mC(x);if(y==null){return;};if(this.__mz()){this.getTarget().addToSelection(y);}else if(this.__mA()){this.getTarget().setSelection([y]);};},__mC:function(z){var A=this.getTarget().getSelectables(true);for(var i=0;i<A.length;i++ ){if(A[i].getModel()==z){return A[i];};};return null;},_startSelectionModification:function(){this._modifingSelection++ ;},_endSelectionModification:function(){this._modifingSelection>0?this._modifingSelection-- :null;},_inSelectionModification:function(){return this._modifingSelection>0;}},destruct:function(){if(this.__mv){this.__mv.dispose();};}});})();(function(){var a="qx.data.marshal.MEventBubbling",b="",c="]",d="idBubble-",f="[",g="changeBubble",h=".",j="qx.event.type.Data";qx.Mixin.define(a,{events:{"changeBubble":j},members:{_applyEventPropagation:function(l,k,name){this.fireDataEvent(g,{value:l,name:name,old:k,item:this});this._registerEventChaining(l,k,name);},_registerEventChaining:function(n,m,name){if(m!=null&&m.getUserData&&m.getUserData(d+this.$$hash)!=null){var p=m.getUserData(d+this.$$hash);for(var i=0;i<p.length;i++ ){m.removeListenerById(p[i]);};m.setUserData(d+this.$$hash,null);};if((n instanceof qx.core.Object)&&qx.Class.hasMixin(n.constructor,qx.data.marshal.MEventBubbling)){var o=qx.lang.Function.bind(this.__mD,this,name);var q=n.addListener(g,o,this);var p=n.getUserData(d+this.$$hash);if(p==null){p=[];n.setUserData(d+this.$$hash,p);};p.push(q);};},__mD:function(name,e){var y=e.getData();var u=y.value;var s=y.old;if(qx.Class.hasInterface(e.getTarget().constructor,qx.data.IListData)){if(y.name.indexOf){var x=y.name.indexOf(h)!=-1?y.name.indexOf(h):y.name.length;var v=y.name.indexOf(f)!=-1?y.name.indexOf(f):y.name.length;if(v==0){var t=name+y.name;}else if(x<v){var r=y.name.substring(0,x);var w=y.name.substring(x+1,y.name.length);if(w[0]!=f){w=h+w;};var t=name+f+r+c+w;}else if(v<x){var r=y.name.substring(0,v);var w=y.name.substring(v,y.name.length);var t=name+f+r+c+w;}else {var t=name+f+y.name+c;};}else {var t=name+f+y.name+c;};}else {if(parseInt(name)==name&&name!==b){name=f+name+c;};var t=name+h+y.name;};this.fireDataEvent(g,{value:u,name:t,old:s,item:y.item||e.getTarget()});}}});})();(function(){var a="-",b="add",c="order",d="add/remove",e="Boolean",f="",g="change",h="qx.data.Array",j="Type of the parameter not supported!",k="0-",l="remove",m="0",n="number",o="changeBubble",p="changeLength",q="qx.event.type.Data";qx.Class.define(h,{extend:qx.core.Object,include:qx.data.marshal.MEventBubbling,implement:[qx.data.IListData],construct:function(r){qx.core.Object.call(this);if(r==undefined){this.__mE=[];}else if(arguments.length>1){this.__mE=[];for(var i=0;i<arguments.length;i++ ){this.__mE.push(arguments[i]);};}else if(typeof r==n){this.__mE=new Array(r);}else if(r instanceof Array){this.__mE=qx.lang.Array.clone(r);}else {this.__mE=[];this.dispose();throw new Error(j);};for(var i=0;i<this.__mE.length;i++ ){this._applyEventPropagation(this.__mE[i],null,i);};this.__mF();{};},properties:{autoDisposeItems:{check:e,init:false}},events:{"change":q,"changeLength":q},members:{__mE:null,concat:function(s){if(s){var t=this.__mE.concat(s);}else {var t=this.__mE.concat();};return new qx.data.Array(t);},join:function(u){return this.__mE.join(u);},pop:function(){var v=this.__mE.pop();this.__mF();this._registerEventChaining(null,v,this.length-1);this.fireDataEvent(o,{value:[],name:this.length+f,old:[v],item:this});this.fireDataEvent(g,{start:this.length-1,end:this.length-1,type:l,removed:[v],added:[]},null);return v;},push:function(w){for(var i=0;i<arguments.length;i++ ){this.__mE.push(arguments[i]);this.__mF();this._registerEventChaining(arguments[i],null,this.length-1);this.fireDataEvent(o,{value:[arguments[i]],name:(this.length-1)+f,old:[],item:this});this.fireDataEvent(g,{start:this.length-1,end:this.length-1,type:b,added:[arguments[i]],removed:[]},null);};return this.length;},reverse:function(){if(this.length==0){return;};var x=this.__mE.concat();this.__mE.reverse();this.__mG(0,this.length);this.fireDataEvent(g,{start:0,end:this.length-1,type:c,added:[],removed:[]},null);this.fireDataEvent(o,{value:this.__mE,name:k+(this.__mE.length-1),old:x,item:this});},shift:function(){if(this.length==0){return;};var y=this.__mE.shift();this.__mF();this._registerEventChaining(null,y,this.length-1);this.__mG(0,this.length);this.fireDataEvent(o,{value:[],name:m,old:[y],item:this});this.fireDataEvent(g,{start:0,end:this.length-1,type:l,removed:[y],added:[]},null);return y;},slice:function(A,z){return new qx.data.Array(this.__mE.slice(A,z));},splice:function(F,H,J){var N=this.__mE.length;var I=this.__mE.splice.apply(this.__mE,arguments);if(this.__mE.length!=N){this.__mF();}else if(H==arguments.length-2){var B=qx.lang.Array.fromArguments(arguments,2);for(var i=0;i<B.length;i++ ){if(B[i]!==I[i]){break;};if(i==B.length-1){return new qx.data.Array();};};};var L=H>0;var D=arguments.length>2;if(L||D){var B=qx.lang.Array.fromArguments(arguments,2);if(I.length==0){var M=b;var E=F+B.length;}else if(B.length==0){var M=l;var E=this.length-1;}else {var M=d;var E=F+Math.max(B.length,I.length)-1;};this.fireDataEvent(g,{start:F,end:E,type:M,added:B,removed:I},null);};for(var i=0;i<I.length;i++ ){this._registerEventChaining(null,I[i],i);};for(var i=2;i<arguments.length;i++ ){this._registerEventChaining(arguments[i],null,F+(i-2));};this.__mG(F+(arguments.length-2)-H,this.length);if(L||D){var G=[];for(var i=2;i<arguments.length;i++ ){G[i-2]=arguments[i];};var C=(F+Math.max(arguments.length-3,H-1));var name=F==C?C:F+a+C;var K={value:G,name:name+f,old:I,item:this};this.fireDataEvent(o,K);};return (new qx.data.Array(I));},sort:function(P){if(this.length==0){return;};var O=this.__mE.concat();this.__mE.sort.apply(this.__mE,arguments);if(qx.lang.Array.equals(this.__mE,O)===true){return;};this.__mG(0,this.length);this.fireDataEvent(g,{start:0,end:this.length-1,type:c,added:[],removed:[]},null);this.fireDataEvent(o,{value:this.__mE,name:k+(this.length-1),old:O,item:this});},unshift:function(Q){for(var i=arguments.length-1;i>=0;i-- ){this.__mE.unshift(arguments[i]);this.__mF();this.__mG(0,this.length);this.fireDataEvent(o,{value:[this.__mE[0]],name:m,old:[this.__mE[1]],item:this});this.fireDataEvent(g,{start:0,end:this.length-1,type:b,added:[arguments[i]],removed:[]},null);};return this.length;},toArray:function(){return this.__mE;},getItem:function(R){return this.__mE[R];},setItem:function(S,U){var T=this.__mE[S];if(T===U){return;};this.__mE[S]=U;this._registerEventChaining(U,T,S);if(this.length!=this.__mE.length){this.__mF();};this.fireDataEvent(o,{value:[U],name:S+f,old:[T],item:this});this.fireDataEvent(g,{start:S,end:S,type:d,added:[U],removed:[T]},null);},getLength:function(){return this.length;},indexOf:function(V){return this.__mE.indexOf(V);},lastIndexOf:function(W){return this.__mE.lastIndexOf(W);},toString:function(){if(this.__mE!=null){return this.__mE.toString();};return f;},contains:function(X){return this.__mE.indexOf(X)!==-1;},copy:function(){return this.concat();},insertAt:function(Y,ba){this.splice(Y,0,ba).dispose();},insertBefore:function(bc,bb){var bd=this.indexOf(bc);if(bd==-1){this.push(bb);}else {this.splice(bd,0,bb).dispose();};},insertAfter:function(bf,be){var bg=this.indexOf(bf);if(bg==-1||bg==(this.length-1)){this.push(be);}else {this.splice(bg+1,0,be).dispose();};},removeAt:function(bh){var bi=this.splice(bh,1);var bj=bi.getItem(0);bi.dispose();return bj;},removeAll:function(){for(var i=0;i<this.__mE.length;i++ ){this._registerEventChaining(null,this.__mE[i],i);};if(this.getLength()==0){return [];};var bl=this.getLength();var bk=this.__mE.concat();this.__mE.length=0;this.__mF();this.fireDataEvent(o,{value:[],name:k+(bl-1),old:bk,item:this});this.fireDataEvent(g,{start:0,end:bl-1,type:l,removed:bk,added:[]},null);return bk;},append:function(bm){if(bm instanceof qx.data.Array){bm=bm.toArray();};{};var bn=this.__mE.length;Array.prototype.push.apply(this.__mE,bm);for(var i=0;i<bm.length;i++ ){this._registerEventChaining(bm[i],null,bn+i);};var bn=this.length;this.__mF();var name=bn==(this.length-1)?bn:bn+a+(this.length-1);this.fireDataEvent(o,{value:bm,name:name+f,old:[],item:this});this.fireDataEvent(g,{start:bn,end:this.length-1,type:b,added:bm,removed:[]},null);},remove:function(bo){var bp=this.indexOf(bo);if(bp!=-1){this.splice(bp,1).dispose();return bo;};},equals:function(bq){if(this.length!==bq.length){return false;};for(var i=0;i<this.length;i++ ){if(this.getItem(i)!==bq.getItem(i)){return false;};};return true;},sum:function(){var br=0;for(var i=0;i<this.length;i++ ){br+=this.getItem(i);};return br;},max:function(){var bs=this.getItem(0);for(var i=1;i<this.length;i++ ){if(this.getItem(i)>bs){bs=this.getItem(i);};};return bs===undefined?null:bs;},min:function(){var bt=this.getItem(0);for(var i=1;i<this.length;i++ ){if(this.getItem(i)<bt){bt=this.getItem(i);};};return bt===undefined?null:bt;},forEach:function(bu,bv){for(var i=0;i<this.__mE.length;i++ ){bu.call(bv,this.__mE[i],i,this);};},filter:function(bw,self){return new qx.data.Array(this.__mE.filter(bw,self));},map:function(bx,self){return new qx.data.Array(this.__mE.map(bx,self));},some:function(by,self){return this.__mE.some(by,self);},every:function(bz,self){return this.__mE.every(bz,self);},reduce:function(bB,bA){return this.__mE.reduce(bB,bA);},reduceRight:function(bD,bC){return this.__mE.reduceRight(bD,bC);},__mF:function(){var bE=this.length;this.length=this.__mE.length;this.fireDataEvent(p,this.length,bE);},__mG:function(bG,bF){for(var i=bG;i<bF;i++ ){this._registerEventChaining(this.__mE[i],this.__mE[i],i);};}},destruct:function(){for(var i=0;i<this.__mE.length;i++ ){var bH=this.__mE[i];this._applyEventPropagation(null,bH,i);if(this.isAutoDisposeItems()&&bH&&bH instanceof qx.core.Object){bH.dispose();};};this.__mE=null;}});})();(function(){var a="qx.ui.core.ISingleSelection",b="qx.event.type.Data";qx.Interface.define(a,{events:{"changeSelection":b},members:{getSelection:function(){return true;},setSelection:function(c){return arguments.length==1;},resetSelection:function(){return true;},isSelected:function(d){return arguments.length==1;},isSelectionEmpty:function(){return true;},getSelectables:function(e){return arguments.length==1;}}});})();(function(){var a="qx.ui.core.IMultiSelection";qx.Interface.define(a,{extend:qx.ui.core.ISingleSelection,members:{selectAll:function(){return true;},addToSelection:function(b){return arguments.length==1;},removeFromSelection:function(c){return arguments.length==1;}}});})();(function(){var a="qx.data.controller.ISelection";qx.Interface.define(a,{members:{setSelection:function(b){},getSelection:function(){},resetSelection:function(){}}});})();(function(){var a="changeModel",b="_applyLabelOptions",c="_applyLabelPath",d="changeTarget",e="_applyModel",f="change",g="icon",h="ReverseBindingId",k="qx.data.controller.List",m="_applyIconPath",n="BindingId",o="_applyDelegate",p="",q="changeDelegate",r="_applyTarget",s="]",t="qx.data.IListData",u="model[",v="model",w="label",x="String",y="_applyIconOptions",z=".";qx.Class.define(k,{extend:qx.core.Object,include:qx.data.controller.MSelection,implement:qx.data.controller.ISelection,construct:function(A,B,C){qx.core.Object.call(this);this.__mH=[];this.__mI=[];this.__mJ=[];this.__mK={};if(C!=null){this.setLabelPath(C);};if(A!=null){this.setModel(A);};if(B!=null){this.setTarget(B);};},properties:{model:{check:t,apply:e,event:a,nullable:true,dereference:true},target:{apply:r,event:d,nullable:true,init:null,dereference:true},labelPath:{check:x,apply:c,nullable:true},iconPath:{check:x,apply:m,nullable:true},labelOptions:{apply:b,nullable:true},iconOptions:{apply:y,nullable:true},delegate:{apply:o,event:q,init:null,nullable:true}},members:{__mL:null,__mH:null,__mK:null,__mI:null,__mJ:null,__mM:null,__mN:null,update:function(){this.__mP();this.__mT();this._updateSelection();},_applyDelegate:function(E,D){this._setConfigureItem(E,D);this._setFilter(E,D);this._setCreateItem(E,D);this._setBindItem(E,D);},_applyIconOptions:function(G,F){this.__mT();},_applyLabelOptions:function(I,H){this.__mT();},_applyIconPath:function(K,J){this.__mT();},_applyLabelPath:function(M,L){this.__mT();},_applyModel:function(P,N){if(N!=undefined){if(this.__mL!=undefined){N.removeListenerById(this.__mL);};};if(this.getSelection()!=undefined&&this.getSelection().length>0){this.getSelection().splice(0,this.getSelection().length).dispose();};if(P!=null){this.__mL=P.addListener(f,this.__mO,this);this.__mV();this.__mP();if(N==null){this._changeTargetSelection();}else {this.__mM=true;qx.ui.core.queue.Widget.add(this);};}else {var O=this.getTarget();if(O!=null){var length=O.getChildren().length;for(var i=0;i<length;i++ ){this.__mS();};};};},_applyTarget:function(S,R){this._addChangeTargetListener(S,R);if(R!=undefined){var Q=R.removeAll();for(var i=0;i<Q.length;i++ ){Q[i].destroy();};this.removeAllBindings();};if(S!=null){if(this.getModel()!=null){for(var i=0;i<this.__mH.length;i++ ){this.__mR(this.__mW(i));};};};},__mO:function(){this.__mN=true;qx.ui.core.queue.Widget.add(this);if(this.__mH.length!=this.getModel().getLength()){this.update();};},syncWidget:function(){if(this.__mM){this._changeTargetSelection();};if(this.__mN){this._updateSelection();};this.__mN=this.__mM=null;},__mP:function(){if(this.getTarget()==null){return;};this.__mV();var U=this.__mH.length;var T=this.getTarget().getChildren().length;if(U>T){for(var j=T;j<U;j++ ){this.__mR(this.__mW(j));};}else if(U<T){for(var j=T;j>U;j-- ){this.__mS();};};this.__mM=true;qx.ui.core.queue.Widget.add(this);},__mQ:function(){var V=this.getModel();if(V!=null){V.removeListenerById(this.__mL);this.__mL=V.addListener(f,this.__mO,this);};},_createItem:function(){var X=this.getDelegate();if(X!=null&&X.createItem!=null){var W=X.createItem();}else {var W=new qx.ui.form.ListItem();};if(X!=null&&X.configureItem!=null){X.configureItem(W);};return W;},__mR:function(Y){var ba=this._createItem();this._bindListItem(ba,Y);this.getTarget().add(ba);},__mS:function(){this._startSelectionModification();var bc=this.getTarget().getChildren();var bb=bc.length-1;var bd=bc[bb];this._removeBindingsFrom(bd);this.getTarget().removeAt(bb);bd.destroy();this._endSelectionModification();},getVisibleModels:function(){var be=[];var bf=this.getTarget();if(bf!=null){var bg=bf.getChildren();for(var i=0;i<bg.length;i++ ){be.push(bg[i].getModel());};};return new qx.data.Array(be);},_bindListItem:function(bh,bj){var bi=this.getDelegate();if(bi!=null&&bi.bindItem!=null){bi.bindItem(this,bh,bj);}else {this.bindDefaultProperties(bh,bj);};},bindDefaultProperties:function(bk,bl){this.bindProperty(p,v,null,bk,bl);this.bindProperty(this.getLabelPath(),w,this.getLabelOptions(),bk,bl);if(this.getIconPath()!=null){this.bindProperty(this.getIconPath(),g,this.getIconOptions(),bk,bl);};},bindProperty:function(bo,br,bp,bn,bm){if(bp!=null){var bp=qx.lang.Object.clone(bp);this.__mK[br]=bp.onUpdate;delete bp.onUpdate;}else {bp={};this.__mK[br]=null;};bp.onUpdate=qx.lang.Function.bind(this._onBindingSet,this,bm);bp.ignoreConverter=v;var bq=u+bm+s;if(bo!=null&&bo!=p){bq+=z+bo;};var bs=this.bind(bq,bn,br,bp);bn.setUserData(br+n,bs);if(!qx.lang.Array.contains(this.__mI,br)){this.__mI.push(br);};},bindPropertyReverse:function(by,bv,bw,bx,bt){var bu=u+bt+s;if(by!=null&&by!=p){bu+=z+by;};var bz=bx.bind(bv,this,bu,bw);bx.setUserData(by+h,bz);if(!qx.lang.Array.contains(this.__mJ,by)){this.__mJ.push(by);};},_onBindingSet:function(bA,bB,bC){if(this.getModel()==null||this._inSelectionModification()){return;};for(var i=0;i<this.__mI.length;i++ ){if(this.__mK[this.__mI[i]]!=null){this.__mK[this.__mI[i]]();};};},_removeBindingsFrom:function(bD){for(var i=0;i<this.__mI.length;i++ ){var bE=bD.getUserData(this.__mI[i]+n);if(bE!=null){this.removeBinding(bE);};};for(var i=0;i<this.__mJ.length;i++ ){var bE=bD.getUserData(this.__mJ[i]+h);if(bE!=null){bD.removeBinding(bE);};};},__mT:function(){if(this.getTarget()==null||this.getModel()==null){return;};var bF=this.getTarget().getChildren();for(var i=0;i<bF.length;i++ ){this._removeBindingsFrom(bF[i]);this._bindListItem(bF[i],this.__mW(i));};this.__mQ();},_setConfigureItem:function(bI,bG){if(bI!=null&&bI.configureItem!=null&&this.getTarget()!=null){var bH=this.getTarget().getChildren();for(var i=0;i<bH.length;i++ ){bI.configureItem(bH[i]);};};},_setBindItem:function(bK,bJ){if(bK!=null&&bK.bindItem!=null){if(bJ!=null&&bJ.bindItem!=null&&bK.bindItem==bJ.bindItem){return;};this.__mT();};},_setCreateItem:function(bM,bL){if(this.getTarget()==null||this.getModel()==null||bM==null||bM.createItem==null){return;};this._startSelectionModification();var bO=this.getTarget().getChildren();for(var i=0,l=bO.length;i<l;i++ ){this._removeBindingsFrom(bO[i]);};var bN=this.getTarget().removeAll();for(var i=0;i<bN.length;i++ ){bN[i].destroy();};this.update();this._endSelectionModification();this._updateSelection();},_setFilter:function(bR,bP){if((bR==null||bR.filter==null)&&(bP!=null&&bP.filter!=null)){this.__mU();};if(this.getTarget()==null||this.getModel()==null||bR==null||bR.filter==null){return;};this._startSelectionModification();var bU=this.getTarget().getChildren();for(var i=0,l=bU.length;i<l;i++ ){this._removeBindingsFrom(bU[i]);};var bQ=this.__mH;this.__mV();if(bQ.length>this.__mH.length){for(var j=bQ.length;j>this.__mH.length;j-- ){this.getTarget().removeAt(j-1).destroy();};}else if(bQ.length<this.__mH.length){for(var j=bQ.length;j<this.__mH.length;j++ ){var bT=this._createItem();this.getTarget().add(bT);};};var bS=this.getTarget().getChildren();for(var i=0;i<bS.length;i++ ){this._bindListItem(bS[i],this.__mW(i));};this.__mQ();this._endSelectionModification();this._updateSelection();},__mU:function(){this.__mV();this.__mP();this.__mT();this.__mN=true;qx.ui.core.queue.Widget.add(this);},__mV:function(){var bW=this.getModel();if(bW==null){return;};var bX=this.getDelegate();if(bX!=null){var bV=bX.filter;};this.__mH=[];for(var i=0;i<bW.getLength();i++ ){if(bV==null||bV(bW.getItem(i))){this.__mH.push(i);};};},__mW:function(bY){return this.__mH[bY];}},destruct:function(){this.__mH=this.__mK=this.__mI=null;this.__mJ=null;qx.ui.core.queue.Widget.remove(this);}});})();(function(){var a="changeModel",b="_applyModel",c="qx.ui.form.MModelProperty";qx.Mixin.define(c,{properties:{model:{nullable:true,event:a,apply:b,dereference:true}},members:{_applyModel:function(e,d){}}});})();(function(){var a="qx.ui.form.IModel",b="qx.event.type.Data";qx.Interface.define(a,{events:{"changeModel":b},members:{setModel:function(c){},getModel:function(){},resetModel:function(){}}});})();(function(){var a="listitem",b="hovered",c="pointerover",d="qx.ui.form.ListItem",e="qx.event.type.Event",f="pointerout";qx.Class.define(d,{extend:qx.ui.basic.Atom,implement:[qx.ui.form.IModel],include:[qx.ui.form.MModelProperty],construct:function(h,i,g){qx.ui.basic.Atom.call(this,h,i);if(g!=null){this.setModel(g);};this.addListener(c,this._onPointerOver,this);this.addListener(f,this._onPointerOut,this);},events:{"action":e},properties:{appearance:{refine:true,init:a}},members:{_forwardStates:{focused:true,hovered:true,selected:true,dragover:true},_onPointerOver:function(){this.addState(b);},_onPointerOut:function(){this.removeState(b);}},destruct:function(){this.removeListener(c,this._onPointerOver,this);this.removeListener(f,this._onPointerOut,this);}});})();(function(){var a="qx.data.marshal.IMarshaler";qx.Interface.define(a,{members:{toClass:function(b,c){},toModel:function(d){}}});})();(function(){var a='"',b="Class '",c="",d="Unsupported type!",e="change",f="qx.data.marshal.Json",g="Array",h="_validate",j="' could not be found.",k="]",l="[",m="set",n="_applyEventPropagation",o="qx.data.model.";qx.Class.define(f,{extend:qx.core.Object,implement:[qx.data.marshal.IMarshaler],construct:function(p){qx.core.Object.call(this);this.__mX=p;},statics:{$$instance:null,createModel:function(q,r){if(this.$$instance===null){this.$$instance=new qx.data.marshal.Json();};this.$$instance.toClass(q,r);return this.$$instance.toModel(q);}},members:{__mX:null,__mY:function(s){return Object.keys(s).sort().join(a);},toClass:function(t,u){this.__na(t,u,null,0);},__na:function(G,B,v,D){if(!qx.lang.Type.isObject(G)||!!G.$$isString||G instanceof qx.core.Object){if(G instanceof Array||qx.Bootstrap.getClass(G)==g){for(var i=0;i<G.length;i++ ){this.__na(G[i],B,v+l+i+k,D+1);};};return;};var x=this.__mY(G);if(this.__ne(x,v,D)){return;};for(var F in G){this.__na(G[F],B,F,D+1);};if(qx.Class.isDefined(o+x)){return;};if(this.__mX&&this.__mX.getModelClass&&this.__mX.getModelClass(x,G,v,D)!=null){return;};var H={};var y={__nb:this.__nb};for(var F in G){if(this.__mX&&this.__mX.getPropertyMapping){F=this.__mX.getPropertyMapping(F,x);};F=F.replace(/-|\.|\s+/g,c);{};H[F]={};H[F].nullable=true;H[F].event=e+qx.lang.String.firstUp(F);if(B){H[F].apply=n;};if(this.__mX&&this.__mX.getValidationRule){var A=this.__mX.getValidationRule(x,F);if(A){H[F].validate=h+F;y[h+F]=A;};};};if(this.__mX&&this.__mX.getModelSuperClass){var E=this.__mX.getModelSuperClass(x,v,D)||qx.core.Object;}else {var E=qx.core.Object;};var z=[];if(this.__mX&&this.__mX.getModelMixins){var C=this.__mX.getModelMixins(x,v,D);if(!qx.lang.Type.isArray(C)){if(C!=null){z=[C];};}else {z=C;};};if(B){z.push(qx.data.marshal.MEventBubbling);};var w={extend:E,include:z,properties:H,members:y,destruct:this.__nc};qx.Class.define(o+x,w);},__nc:function(){var I=qx.util.PropertyUtil.getAllProperties(this.constructor);for(var J in I){this.__nb(this.get(I[J].name));};},__nb:function(K){if(!(K instanceof qx.core.Object)){return;};if(K.isDisposed()){return;};K.dispose();},__nd:function(L,P,Q,O){var R;if(this.__mX&&this.__mX.getModelClass){R=this.__mX.getModelClass(L,P,Q,O);};if(R!=null){return (new R());}else {var N=o+L;var M=qx.Class.getByName(N);if(!M){throw new Error(b+N+j);};return (new M());};},__ne:function(S,V,T){var U=this.__mX;return U&&U.ignore&&U.ignore(S,V,T);},toModel:function(W){return this.__nf(W,null,0);},__nf:function(bj,Y,bf){var bd=qx.lang.Type.isObject(bj);var X=bj instanceof Array||qx.Bootstrap.getClass(bj)==g;if((!bd&&!X)||!!bj.$$isString||bj instanceof qx.core.Object){return bj;}else if(this.__ne(this.__mY(bj),Y,bf)){return bj;}else if(X){var bh=qx.data.Array;if(this.__mX&&this.__mX.getArrayClass){var be=this.__mX.getArrayClass(Y,bf);bh=be||bh;};var bk=new bh();bk.setAutoDisposeItems(true);for(var i=0;i<bj.length;i++ ){bk.push(this.__nf(bj[i],Y+l+i+k,bf+1));};return bk;}else if(bd){var ba=this.__mY(bj);var bi=this.__nd(ba,bj,Y,bf);for(var bg in bj){var bb=bg;if(this.__mX&&this.__mX.getPropertyMapping){bb=this.__mX.getPropertyMapping(bg,ba);};var bl=bb.replace(/-|\.|\s+/g,c);{};bb=bl;var bc=m+qx.lang.String.firstUp(bb);if(bi[bc]){bi[bc](this.__nf(bj[bg],bg,bf+1));};};return bi;};throw new Error(d);}},destruct:function(){this.__mX=null;}});})();(function(){var a='qx_lib.io.Server',b='delete',c='post',d='/',f='put',g='success',h='/api/',i='singleton',j='get';qx.Class.define(a,{extend:qx.core.Object,type:i,statics:{apiDelete:function(k,n,m,l){this.getInstance().apiDelete(k,n,m,l);},apiGet:function(o,r,q,p){this.getInstance().apiGet(o,r,q,p);},apiPost:function(s,v,u,t){this.getInstance().apiPost(s,v,u,t);},apiPut:function(w,z,y,x){this.getInstance().apiPut(w,z,y,x);}},members:{apiDelete:function(A,D,C,B){this.sendApi(b,A,D,C,B);},apiGet:function(E,H,G,F){this.sendApi(j,E,H,G,F);},apiPost:function(I,L,K,J){this.sendApi(c,I,L,K,J);},apiPut:function(M,P,O,N){this.sendApi(f,M,P,O,N);},sendApi:function(W,T,S,V,U){var Q=h+T;if(S&&S.length)Q+=d+S;var R=new qx.io.request.Xhr(Q,W);if(V&&Object.keys(V).length)R.setRequestData(V);R.addListener(g,function(e){var Y=e.getTarget();var X=Y.getResponseText();U.apply(null,[X]);},this);R.send();}}});})();(function(){var a="loadEnd",b="qx.io.request.AbstractRequest",c="changePhase",d="GET",f="sent",g="qx.event.type.Data",h="qx.io.request.authentication.IAuthentication",i="error",j="Blob",k="fail",l="loading",m="load",n="qx.event.type.Event",o="abort",p="success",q="String",r="",s="opened",t="POST",u="timeout",v="statusError",w="readyStateChange",x="Abstract method call",y="abstract",z="unsent",A="changeResponse",B="Number",C="FormData",D="Content-Type",E="ArrayBuffer",F="undefined";qx.Class.define(b,{type:y,extend:qx.core.Object,construct:function(G){qx.core.Object.call(this);if(G!==undefined){this.setUrl(G);};this.__ng={};var H=this._transport=this._createTransport();this._setPhase(z);this.__nh=qx.lang.Function.bind(this._onReadyStateChange,this);this.__ni=qx.lang.Function.bind(this._onLoad,this);this.__nj=qx.lang.Function.bind(this._onLoadEnd,this);this.__nk=qx.lang.Function.bind(this._onAbort,this);this.__nl=qx.lang.Function.bind(this._onTimeout,this);this.__nm=qx.lang.Function.bind(this._onError,this);H.onreadystatechange=this.__nh;H.onload=this.__ni;H.onloadend=this.__nj;H.onabort=this.__nk;H.ontimeout=this.__nl;H.onerror=this.__nm;},events:{"readyStateChange":n,"success":n,"load":n,"loadEnd":n,"abort":n,"timeout":n,"error":n,"statusError":n,"fail":n,"changeResponse":g,"changePhase":g},properties:{url:{check:q},timeout:{check:B,nullable:true,init:0},requestData:{check:function(I){return qx.lang.Type.isString(I)||qx.Class.isSubClassOf(I.constructor,qx.core.Object)||qx.lang.Type.isObject(I)||qx.lang.Type.isArray(I)||qx.Bootstrap.getClass(I)==j||qx.Bootstrap.getClass(I)==E||qx.Bootstrap.getClass(I)==C;},nullable:true},authentication:{check:h,nullable:true}},members:{__nh:null,__ni:null,__nj:null,__nk:null,__nl:null,__nm:null,__nn:null,__no:null,__np:null,__ng:null,__nq:null,_transport:null,_createTransport:function(){throw new Error(x);},_getConfiguredUrl:function(){},_getConfiguredRequestHeaders:function(){},_getParsedResponse:function(){throw new Error(x);},_getMethod:function(){return d;},_isAsync:function(){return true;},send:function(){var M=this._transport,K,L,N,J;K=this._getConfiguredUrl();if(/\#/.test(K)){K=K.replace(/\#.*/,r);};M.timeout=this.getTimeout();L=this._getMethod();N=this._isAsync();{};M.open(L,K,N);this._setPhase(s);J=this.getRequestData();if([E,j,C].indexOf(qx.Bootstrap.getClass(J))==-1){J=this._serializeData(J);};this._setRequestHeaders();{};L==d?M.send():M.send(J);this._setPhase(f);},abort:function(){{};this.__no=true;this.__np=o;this._transport.abort();},_setRequestHeaders:function(){var P=this._transport,O=this._getAllRequestHeaders();for(var Q in O){P.setRequestHeader(Q,O[Q]);};},_getAllRequestHeaders:function(){var R={};qx.lang.Object.mergeWith(R,this._getConfiguredRequestHeaders());qx.lang.Object.mergeWith(R,this.__nr());qx.lang.Object.mergeWith(R,this.__nq);qx.lang.Object.mergeWith(R,this.__ng);return R;},__nr:function(){var T=this.getAuthentication(),S={};if(T){T.getAuthHeaders().forEach(function(U){S[U.key]=U.value;});return S;};},setRequestHeader:function(V,W){this.__ng[V]=W;},getRequestHeader:function(X){return this.__ng[X];},removeRequestHeader:function(Y){if(this.__ng[Y]){delete this.__ng[Y];};},getTransport:function(){return this._transport;},getReadyState:function(){return this._transport.readyState;},getPhase:function(){return this.__np;},getStatus:function(){return this._transport.status;},getStatusText:function(){return this._transport.statusText;},getResponseText:function(){return this._transport.responseText;},getAllResponseHeaders:function(){return this._transport.getAllResponseHeaders();},getResponseHeader:function(ba){return this._transport.getResponseHeader(ba);},overrideResponseContentType:function(bb){return this._transport.overrideMimeType(bb);},getResponseContentType:function(){return this.getResponseHeader(D);},isDone:function(){return this.getReadyState()===4;},getResponse:function(){return this.__nn;},_setResponse:function(bd){var bc=bd;if(this.__nn!==bd){this.__nn=bd;this.fireEvent(A,qx.event.type.Data,[this.__nn,bc]);};},_onReadyStateChange:function(){var be=this.getReadyState();{};this.fireEvent(w);if(this.__no){return;};if(be===3){this._setPhase(l);};if(this.isDone()){this.__ns();};},__ns:function(){{};this._setPhase(m);if(qx.util.Request.isSuccessful(this.getStatus())){{};this._setResponse(this._getParsedResponse());this._fireStatefulEvent(p);}else {try{this._setResponse(this._getParsedResponse());}catch(e){};if(this.getStatus()!==0){this._fireStatefulEvent(v);this.fireEvent(k);};};},_onLoad:function(){this.fireEvent(m);},_onLoadEnd:function(){this.fireEvent(a);},_onAbort:function(){this._fireStatefulEvent(o);},_onTimeout:function(){this._fireStatefulEvent(u);this.fireEvent(k);},_onError:function(){this.fireEvent(i);this.fireEvent(k);},_fireStatefulEvent:function(bf){{};this._setPhase(bf);this.fireEvent(bf);},_setPhase:function(bg){var bh=this.__np;{};this.__np=bg;this.fireDataEvent(c,bg,bh);},_serializeData:function(bk){var bi=typeof this.getMethod!==F&&this.getMethod()==t,bj=/application\/.*\+?json/.test(this.getRequestHeader(D));if(!bk){return null;};if(qx.lang.Type.isString(bk)){return bk;};if(qx.Class.isSubClassOf(bk.constructor,qx.core.Object)){return qx.util.Serializer.toUriParameter(bk);};if(bj&&(qx.lang.Type.isObject(bk)||qx.lang.Type.isArray(bk))){return qx.lang.Json.stringify(bk);};if(qx.lang.Type.isObject(bk)){return qx.util.Uri.toParameter(bk,bi);};return null;}},environment:{"qx.debug.io":false},destruct:function(){var bm=this._transport,bl=function(){};if(this._transport){bm.onreadystatechange=bm.onload=bm.onloadend=bm.onabort=bm.ontimeout=bm.onerror=bl;window.setTimeout(function(){bm.dispose();},0);};}});})();(function(){var a="HEAD",b="CONNECT",c="OPTIONS",d="PUT",e="GET",f="PATCH",g="//",h="DELETE",i="POST",j="TRACE",k="qx.util.Request";qx.Bootstrap.define(k,{statics:{isCrossDomain:function(l){var n=qx.util.Uri.parseUri(l),location=window.location;if(!location){return false;};var m=location.protocol;if(!(l.indexOf(g)!==-1)){return false;};if(m.substr(0,m.length-1)==n.protocol&&location.host===n.host&&location.port===n.port){return false;};return true;},isSuccessful:function(status){return (status>=200&&status<300||status===304);},isMethod:function(p){var o=[e,i,d,h,a,c,j,b,f];return (o.indexOf(p)!==-1)?true:false;},methodAllowsRequestBody:function(q){return !((/^(GET|HEAD)$/).test(q));}}});})();(function(){var a="qx.util.Serializer",b='\\\\',c='\\f',d='"',e="null",f='\\"',g="}",h="get",j="{",k='\\r',l="",m='\\t',n="]",o="Class",p="Interface",q="[",r="Mixin",s='":',t="&",u='\\b',v="=",w='\\n',x=",";qx.Class.define(a,{statics:{toUriParameter:function(z,C,y){var E=l;var B=qx.util.PropertyUtil.getAllProperties(z.constructor);for(var name in B){if(B[name].group!=undefined){continue;};var A=z[h+qx.lang.String.firstUp(name)]();if(qx.lang.Type.isArray(A)){var D=qx.data&&qx.data.IListData&&qx.Class.hasInterface(A&&A.constructor,qx.data.IListData);for(var i=0;i<A.length;i++ ){var F=D?A.getItem(i):A[i];E+=this.__nt(name,F,C);};}else if(qx.lang.Type.isDate(A)&&y!=null){E+=this.__nt(name,y.format(A),C);}else {E+=this.__nt(name,A,C);};};return E.substring(0,E.length-1);},__nt:function(name,I,G){if(I&&I.$$type==o){I=I.classname;};if(I&&(I.$$type==p||I.$$type==r)){I=I.name;};if(I instanceof qx.core.Object&&G!=null){var H=encodeURIComponent(G(I));if(H===undefined){var H=encodeURIComponent(I);};}else {var H=encodeURIComponent(I);};return encodeURIComponent(name)+v+H+t;},toNativeObject:function(L,N,K){var O;if(L==null){return null;};if(qx.data&&qx.data.IListData&&qx.Class.hasInterface(L.constructor,qx.data.IListData)){O=[];for(var i=0;i<L.getLength();i++ ){O.push(qx.util.Serializer.toNativeObject(L.getItem(i),N,K));};return O;};if(qx.lang.Type.isArray(L)){O=[];for(var i=0;i<L.length;i++ ){O.push(qx.util.Serializer.toNativeObject(L[i],N,K));};return O;};if(L.$$type==o){return L.classname;};if(L.$$type==p||L.$$type==r){return L.name;};if(L instanceof qx.core.Object){if(N!=null){var J=N(L);if(J!=undefined){return J;};};O={};var Q=qx.util.PropertyUtil.getAllProperties(L.constructor);for(var name in Q){if(Q[name].group!=undefined){continue;};var M=L[h+qx.lang.String.firstUp(name)]();O[name]=qx.util.Serializer.toNativeObject(M,N,K);};return O;};if(qx.lang.Type.isDate(L)&&K!=null){return K.format(L);};if(qx.locale&&qx.locale.LocalizedString&&L instanceof qx.locale.LocalizedString){return L.toString();};if(qx.lang.Type.isObject(L)){O={};for(var P in L){O[P]=qx.util.Serializer.toNativeObject(L[P],N,K);};return O;};return L;},toJson:function(T,V,S){var W=l;if(T==null){return e;};if(qx.data&&qx.data.IListData&&qx.Class.hasInterface(T.constructor,qx.data.IListData)){W+=q;for(var i=0;i<T.getLength();i++ ){W+=qx.util.Serializer.toJson(T.getItem(i),V,S)+x;};if(W!=q){W=W.substring(0,W.length-1);};return W+n;};if(qx.lang.Type.isArray(T)){W+=q;for(var i=0;i<T.length;i++ ){W+=qx.util.Serializer.toJson(T[i],V,S)+x;};if(W!=q){W=W.substring(0,W.length-1);};return W+n;};if(T.$$type==o){return d+T.classname+d;};if(T.$$type==p||T.$$type==r){return d+T.name+d;};if(T instanceof qx.core.Object){if(V!=null){var R=V(T);if(R!=undefined){return d+R+d;};};W+=j;var Y=qx.util.PropertyUtil.getAllProperties(T.constructor);for(var name in Y){if(Y[name].group!=undefined){continue;};var U=T[h+qx.lang.String.firstUp(name)]();W+=d+name+s+qx.util.Serializer.toJson(U,V,S)+x;};if(W!=j){W=W.substring(0,W.length-1);};return W+g;};if(qx.locale&&qx.locale.LocalizedString&&T instanceof qx.locale.LocalizedString){T=T.toString();};if(qx.lang.Type.isDate(T)&&S!=null){return d+S.format(T)+d;};if(qx.lang.Type.isObject(T)){W+=j;for(var X in T){W+=d+X+s+qx.util.Serializer.toJson(T[X],V,S)+x;};if(W!=j){W=W.substring(0,W.length-1);};return W+g;};if(qx.lang.Type.isString(T)){T=T.replace(/([\\])/g,b);T=T.replace(/(["])/g,f);T=T.replace(/([\r])/g,k);T=T.replace(/([\f])/g,c);T=T.replace(/([\n])/g,w);T=T.replace(/([\t])/g,m);T=T.replace(/([\b])/g,u);return d+T+d;};if(qx.lang.Type.isDate(T)||qx.lang.Type.isRegExp(T)){return d+T+d;};return T+l;}}});})();(function(){var a="null",b="XMLHttpRequest",c="Boolean",d="X-Requested-With",e="",f="application/x-www-form-urlencoded",g="Cache-Control",h="Content-Type",i="qx.event.type.Event",j="GET",k="qx.io.request.Xhr",l="Accept",m="String";qx.Class.define(k,{extend:qx.io.request.AbstractRequest,construct:function(n,o){if(o!==undefined){this.setMethod(o);};qx.io.request.AbstractRequest.call(this,n);this._parser=this._createResponseParser();},events:{"readyStateChange":i,"success":i,"load":i,"statusError":i},properties:{method:{init:j},async:{check:c,init:true},accept:{check:m,nullable:true},cache:{check:function(p){return qx.lang.Type.isBoolean(p)||qx.lang.Type.isString(p);},init:true}},members:{_parser:null,_createTransport:function(){return new qx.bom.request.Xhr();},_getConfiguredUrl:function(){var q=this.getUrl(),r;if(this.getMethod()===j&&this.getRequestData()){r=this._serializeData(this.getRequestData());q=qx.util.Uri.appendParamsToUrl(q,r);};if(this.getCache()===false){q=qx.util.Uri.appendParamsToUrl(q,{nocache:new Date().valueOf()});};return q;},_getConfiguredRequestHeaders:function(){var s={},t=qx.util.Request.methodAllowsRequestBody(this.getMethod());if(!qx.util.Request.isCrossDomain(this.getUrl())){s[d]=b;};if(qx.lang.Type.isString(this.getCache())){s[g]=this.getCache();};if(this.getRequestData()!==a&&t){s[h]=f;};if(this.getAccept()){{};s[l]=this.getAccept();};return s;},_getMethod:function(){return this.getMethod();},_isAsync:function(){return this.isAsync();},_createResponseParser:function(){return new qx.util.ResponseParser();},_getParsedResponse:function(){var v=this._transport.responseText,u=this.getResponseContentType()||e;return this._parser.parse(v,u);},setParser:function(w){return this._parser.setParser(w);}}});})();(function(){var a="activex",b="No XHR support available.",c="If-None-Match",d="xhr",f="If-Modified-Since",g="engine.version",h="onunload",i="GET",j="-1",k="qx.debug.io",l="HTMLDocument",m="error",n="loadend",o="Blob",p="load",q="abort",r="String",s="browser.documentmode",t="",u="engine.name",v="Microsoft.XMLHTTP",w="Already disposed",x="browser.version",y="opera",z="qx.bom.request.Xhr",A="Not enough arguments",B="timeout",C="gecko",D="If-Match",E="mshtml",F="readystatechange",G="Microsoft.XMLDOM",H="file:",I="FormData",J="If-Range",K="Content-Type",L="io.xhr",M="on",N="ArrayBuffer",O="undefined",P="Native XHR object doesn't support overrideMimeType.";qx.Bootstrap.define(z,{extend:Object,construct:function(){var Q=qx.Bootstrap.bind(this.__nI,this);if(qx.event&&qx.event.GlobalError&&qx.event.GlobalError.observeMethod){this.__nu=qx.event.GlobalError.observeMethod(Q);}else {this.__nu=Q;};this.__nv=qx.Bootstrap.bind(this.__nH,this);this.__nl=qx.Bootstrap.bind(this.__nM,this);this.__nG();this._emitter=new qx.event.Emitter();if(window.attachEvent){this.__nw=qx.Bootstrap.bind(this.__nP,this);window.attachEvent(h,this.__nw);};},statics:{UNSENT:0,OPENED:1,HEADERS_RECEIVED:2,LOADING:3,DONE:4},events:{"readystatechange":z,"error":z,"loadend":z,"timeout":z,"abort":z,"load":z},members:{readyState:0,responseText:t,responseXML:null,status:0,statusText:t,timeout:0,open:function(V,R,S,U,T){this.__nR();if(typeof R===O){throw new Error(A);}else if(typeof V===O){V=i;};this.__no=false;this.__nx=false;this.__ny=false;this.__nz=R;if(typeof S==O){S=true;};this.__nA=S;if(!this.__nQ()&&this.readyState>qx.bom.request.Xhr.UNSENT){this.dispose();this.__nG();};this.__nC.onreadystatechange=this.__nu;try{{};this.__nC.open(V,R,S,U,T);}catch(W){if(!qx.util.Request.isCrossDomain(R)){throw W;};if(!this.__nA){this.__nB=W;};if(this.__nA){if(window.XDomainRequest){this.readyState=4;this.__nC=new XDomainRequest();this.__nC.onerror=qx.Bootstrap.bind(function(){this._emit(F);this._emit(m);this._emit(n);},this);{};this.__nC.open(V,R,S,U,T);return;};window.setTimeout(qx.Bootstrap.bind(function(){if(this.__nE){return;};this.readyState=4;this._emit(F);this._emit(m);this._emit(n);},this));};};if(qx.core.Environment.get(u)===E&&qx.core.Environment.get(s)<9&&this.__nC.readyState>0){this.__nC.setRequestHeader(f,j);};if(qx.core.Environment.get(u)===C&&parseInt(qx.core.Environment.get(g),10)<2&&!this.__nA){this.readyState=qx.bom.request.Xhr.OPENED;this._emit(F);};},setRequestHeader:function(X,Y){this.__nR();if(X==D||X==f||X==c||X==J){this.__ny=true;};this.__nC.setRequestHeader(X,Y);return this;},send:function(bb){this.__nR();if(!this.__nA&&this.__nB){throw this.__nB;};if(qx.core.Environment.get(u)===y&&this.timeout===0){this.timeout=10000;};if(this.timeout>0){this.__nD=window.setTimeout(this.__nl,this.timeout);};bb=typeof bb==O?null:bb;var ba=qx.Bootstrap.getClass(bb);bb=(bb!==null&&this.__nF.indexOf(ba)===-1)?bb.toString():bb;try{{};this.__nC.send(bb);}catch(bd){if(!this.__nA){throw bd;};if(this._getProtocol()===H){this.readyState=2;this.__nJ();var bc=this;window.setTimeout(function(){if(bc.__nE){return;};bc.readyState=3;bc.__nJ();bc.readyState=4;bc.__nJ();});};};if(qx.core.Environment.get(u)===C&&!this.__nA){this.__nI();};this.__nx=true;return this;},abort:function(){this.__nR();this.__no=true;this.__nC.abort();if(this.__nC){this.readyState=this.__nC.readyState;};return this;},_emit:function(event){if(this[M+event]){this[M+event]();};this._emitter.emit(event,this);},onreadystatechange:function(){},onload:function(){},onloadend:function(){},onerror:function(){},onabort:function(){},ontimeout:function(){},on:function(name,be,bf){this._emitter.on(name,be,bf);return this;},getResponseHeader:function(bg){this.__nR();if(qx.core.Environment.get(s)===9&&this.__nC.aborted){return t;};return this.__nC.getResponseHeader(bg);},getAllResponseHeaders:function(){this.__nR();if(qx.core.Environment.get(s)===9&&this.__nC.aborted){return t;};return this.__nC.getAllResponseHeaders();},overrideMimeType:function(bh){this.__nR();if(this.__nC.overrideMimeType){this.__nC.overrideMimeType(bh);}else {throw new Error(P);};return this;},getRequest:function(){return this.__nC;},dispose:function(){if(this.__nE){return false;};window.clearTimeout(this.__nD);if(window.detachEvent){window.detachEvent(h,this.__nw);};try{this.__nC.onreadystatechange;}catch(bj){return false;};var bi=function(){};this.__nC.onreadystatechange=bi;this.__nC.onload=bi;this.__nC.onerror=bi;this.abort();this.__nC=null;this.__nE=true;return true;},isDisposed:function(){return !!this.__nE;},_createNativeXhr:function(){var bk=qx.core.Environment.get(L);if(bk===d){return new XMLHttpRequest();};if(bk==a){return new window.ActiveXObject(v);};qx.Bootstrap.error(this,b);},_getProtocol:function(){var bl=this.__nz;var bm=/^(\w+:)\/\//;if(bl!==null&&bl.match){var bn=bl.match(bm);if(bn&&bn[1]){return bn[1];};};return window.location.protocol;},__nC:null,__nA:null,__nu:null,__nv:null,__nw:null,__nl:null,__nx:null,__nz:null,__no:null,__gs:null,__nE:null,__nD:null,__nB:null,__ny:null,__nF:null,__nG:function(){this.__nC=this._createNativeXhr();this.__nC.onreadystatechange=this.__nu;if(this.__nC.onabort){this.__nC.onabort=this.__nv;};this.__nE=this.__nx=this.__no=false;this.__nF=[N,o,l,r,I];},__nH:function(){if(!this.__no){this.abort();};},__nI:function(){var bo=this.__nC,bp=true;{};if(this.readyState==bo.readyState){return;};this.readyState=bo.readyState;if(this.readyState===qx.bom.request.Xhr.DONE&&this.__no&&!this.__nx){return;};if(!this.__nA&&(bo.readyState==2||bo.readyState==3)){return;};this.status=0;this.statusText=this.responseText=t;this.responseXML=null;if(this.readyState>=qx.bom.request.Xhr.HEADERS_RECEIVED){try{this.status=bo.status;this.statusText=bo.statusText;this.responseText=bo.responseText;this.responseXML=bo.responseXML;}catch(bq){bp=false;};if(bp){this.__nN();this.__nO();};};this.__nJ();if(this.readyState==qx.bom.request.Xhr.DONE){if(bo){bo.onreadystatechange=function(){};};};},__nJ:function(){if(this.readyState===qx.bom.request.Xhr.DONE){window.clearTimeout(this.__nD);};this._emit(F);if(this.readyState===qx.bom.request.Xhr.DONE){this.__nK();};},__nK:function(){if(this.__gs){this._emit(B);if(qx.core.Environment.get(u)===y){this._emit(m);};this.__gs=false;}else {if(this.__no){this._emit(q);}else {if(this.__nL()){this._emit(m);}else {this._emit(p);};};};this._emit(n);},__nL:function(){var br;if(this._getProtocol()===H){br=!this.responseText;}else {br=!this.statusText&&this.status!==204;};return br;},__nM:function(){var bs=this.__nC;this.readyState=qx.bom.request.Xhr.DONE;this.__gs=true;bs.aborted=true;bs.abort();this.responseText=t;this.responseXML=null;this.__nJ();},__nN:function(){var bt=this.readyState===qx.bom.request.Xhr.DONE;if(this._getProtocol()===H&&this.status===0&&bt){if(!this.__nL()){this.status=200;};};if(this.status===1223){this.status=204;};if(qx.core.Environment.get(u)===y){if(bt&&this.__ny&&!this.__no&&this.status===0){this.status=304;};};},__nO:function(){if(qx.core.Environment.get(u)==E&&(this.getResponseHeader(K)||t).match(/[^\/]+\/[^\+]+\+xml/)&&this.responseXML&&!this.responseXML.documentElement){var bu=new window.ActiveXObject(G);bu.async=false;bu.validateOnParse=false;bu.loadXML(this.responseText);this.responseXML=bu;};},__nP:function(){try{if(this){this.dispose();};}catch(e){};},__nQ:function(){var name=qx.core.Environment.get(u);var bv=qx.core.Environment.get(x);return !(name==E&&bv<9||name==C&&bv<3.5);},__nR:function(){if(this.__nE){throw new Error(w);};}},defer:function(){qx.core.Environment.add(k,false);}});})();(function(){var a="function",b="qx.util.ResponseParser",c="";qx.Bootstrap.define(b,{construct:function(d){if(d!==undefined){this.setParser(d);};},statics:{PARSER:{json:qx.lang.Json.parse,xml:qx.xml.Document.fromString}},members:{__nS:null,parse:function(g,f){var e=this._getParser(f);if(typeof e===a){if(g!==c){return e.call(this,g);};};return g;},setParser:function(h){if(typeof qx.util.ResponseParser.PARSER[h]===a){return this.__nS=qx.util.ResponseParser.PARSER[h];};{};return this.__nS=h;},_getParser:function(j){var i=this.__nS,l=c,k=c;if(i){return i;};l=j||c;k=l.replace(/;.*$/,c);if(/^application\/(\w|\.)*\+?json$/.test(k)){i=qx.util.ResponseParser.PARSER.json;};if(/^application\/xml$/.test(k)){i=qx.util.ResponseParser.PARSER.xml;};if(/[^\/]+\/[^\+]+\+xml$/.test(l)){i=qx.util.ResponseParser.PARSER.xml;};return i;}}});})();(function(){var a="qx.data.Array",b="qx_lib.data.ListModel",c="changeData",d="String",e="changeHtml";qx.Class.define(b,{extend:qx.core.Object,properties:{html:{check:d,event:e},data:{check:a,event:c}},construct:function(f,g){qx.core.Object.call(this);this.setHtml(f);this.setData(g);}});})();(function(){var a="x",b="Float",c="qx.ui.core.MDragDropScrolling",d="",f="). Must be: 'left', 'right', 'top' or 'bottom'",g="bottom",h="interval",i="dragend",j="scrollbar-",k="drag",l="Invalid edge type given (",m="top",n="left",o="right",p="Integer",q="y";qx.Mixin.define(c,{construct:function(){var r=this;if(this instanceof qx.ui.core.DragDropScrolling){r=this._getWidget();};r.addListener(k,this.__nW,this);r.addListener(i,this.__nX,this);this.__nT=[n,o];this.__nU=[m,g];},properties:{dragScrollThresholdX:{check:p,init:30},dragScrollThresholdY:{check:p,init:30},dragScrollSlowDownFactor:{check:b,init:0.1}},members:{__nV:null,__nT:null,__nU:null,_findScrollableParent:function(s){var t=s;if(t===null){return null;};while(t.getLayoutParent()){t=t.getLayoutParent();if(this._isScrollable(t)){return t;};};return null;},_isScrollable:function(u){return qx.Class.hasMixin(u.constructor,qx.ui.core.scroll.MScrollBarFactory);},_getBounds:function(v){var w=v.getContentLocation();if(v.getScrollAreaContainer){w=v.getScrollAreaContainer().getContentLocation();};return w;},_getEdgeType:function(y,z,x){if((y.left*-1)<=z&&y.left<0){return n;}else if((y.top*-1)<=x&&y.top<0){return m;}else if(y.right<=z&&y.right>0){return o;}else if(y.bottom<=x&&y.bottom>0){return g;}else {return null;};},_getAxis:function(A){if(this.__nT.indexOf(A)!==-1){return a;}else if(this.__nU.indexOf(A)!==-1){return q;}else {throw new Error(l+A+f);};},_getThresholdByEdgeType:function(B){if(this.__nT.indexOf(B)!==-1){return this.getDragScrollThresholdX();}else if(this.__nU.indexOf(B)!==-1){return this.getDragScrollThresholdY();};},_isScrollbarVisible:function(C,D){if(C&&C._isChildControlVisible){return C._isChildControlVisible(j+D);}else {return false;};},_isScrollbarExceedingMaxPos:function(F,H,E){var G=0;if(!F){return true;};G=F.getPosition()+E;return (G>F.getMaximum()||G<0);},_calculateThresholdExceedance:function(J,I){var K=I-Math.abs(J);return J<0?(K*-1):K;},_calculateScrollAmount:function(L,M){return Math.floor(((L/100)*M)*this.getDragScrollSlowDownFactor());},_scrollBy:function(N,T,Q){var S=N.getChildControl(j+T,true);if(!S){return;};var P=S.getBounds(),O=T===a?P.width:P.height,R=this._calculateScrollAmount(O,Q);if(this._isScrollbarExceedingMaxPos(S,T,R)){this.__nV.stop();};S.scrollBy(R);},__nW:function(e){if(this.__nV){this.__nV.stop();};var W=e.getOriginalTarget();if(!W){return;};var U;if(this._isScrollable(W)){U=W;}else {U=this._findScrollableParent(W);};while(U){var Y=this._getBounds(U),V=e.getDocumentLeft(),ba=e.getDocumentTop(),bc={"left":Y.left-V,"right":Y.right-V,"top":Y.top-ba,"bottom":Y.bottom-ba},X=null,bd=d,bb=0;X=this._getEdgeType(bc,this.getDragScrollThresholdX(),this.getDragScrollThresholdY());if(!X){U=this._findScrollableParent(U);continue;};bd=this._getAxis(X);if(this._isScrollbarVisible(U,bd)){bb=this._calculateThresholdExceedance(bc[X],this._getThresholdByEdgeType(X));if(this.__nV){this.__nV.dispose();};this.__nV=new qx.event.Timer(50);this.__nV.addListener(h,function(be,bg,bf){this._scrollBy(be,bg,bf);}.bind(this,U,bd,bb));this.__nV.start();e.stopPropagation();return;}else {U=this._findScrollableParent(U);};};},__nX:function(e){if(this.__nV){this.__nV.stop();};}},destruct:function(){if(this.__nV){this.__nV.dispose();};}});})();(function(){var a="qx.ui.core.DragDropScrolling";qx.Class.define(a,{extend:qx.core.Object,include:[qx.ui.core.MDragDropScrolling],construct:function(b){qx.core.Object.call(this);this._widget=b;},members:{_widget:null,_getWidget:function(){return this._widget||qx.core.Init.getApplication().getRoot();}}});})();(function(){var a="qx.nativeScrollBars",b="qx.ui.core.scroll.MScrollBarFactory";qx.core.Environment.add(a,false);qx.Mixin.define(b,{members:{_createScrollBar:function(c){if(qx.core.Environment.get(a)){return new qx.ui.core.scroll.NativeScrollBar(c);}else {return new qx.ui.core.scroll.ScrollBar(c);};}}});})();(function(){var a='qx.event.type.Event',b="qx.ui.core.scroll.IScrollBar",c="qx.event.type.Data";qx.Interface.define(b,{events:{"scroll":c,"scrollAnimationEnd":a},properties:{orientation:{},maximum:{},position:{},knobFactor:{}},members:{scrollTo:function(e,d){this.assertNumber(e);},scrollBy:function(g,f){this.assertNumber(g);},scrollBySteps:function(i,h){this.assertNumber(i);}}});})();(function(){var a="qx.ui.core.scroll.NativeScrollBar",b="px",c="PositiveNumber",d="frame",f="end",g="box-sizing",h="Integer",i="track",j="browser.name",k="hidden",l="content-box",m="pointerup",n="vertical",o="_applyOrientation",p="scrollAnimationEnd",q="__oa",r="appear",s="pointerdown",t="PositiveInteger",u='qx.event.type.Event',v="mshtml",w="engine.name",x="horizontal",y="Number",z="edge",A="_applyPosition",B="scrollbar",C="_applyMaximum",D="pointermove",E="native",F="scroll";qx.Class.define(a,{extend:qx.ui.core.Widget,implement:qx.ui.core.scroll.IScrollBar,construct:function(G){qx.ui.core.Widget.call(this);this.addState(E);this.getContentElement().addListener(F,this._onScroll,this);this.addListener(s,this._stopPropagation,this);this.addListener(m,this._stopPropagation,this);this.addListener(D,this._stopPropagation,this);this.addListener(r,this._onAppear,this);this.getContentElement().add(this._getScrollPaneElement());this.getContentElement().setStyle(g,l);if(G!=null){this.setOrientation(G);}else {this.initOrientation();};this.addListener(i,function(e){e.stopPropagation();},this);},events:{scrollAnimationEnd:u},properties:{appearance:{refine:true,init:B},orientation:{check:[x,n],init:x,apply:o},maximum:{check:t,apply:C,init:100},position:{check:y,init:0,apply:A,event:F},singleStep:{check:h,init:20},knobFactor:{check:c,nullable:true}},members:{__nY:null,__oa:null,__kp:null,__ob:null,_getScrollPaneElement:function(){if(!this.__oa){this.__oa=new qx.html.Element();};return this.__oa;},renderLayout:function(I,top,H,K){var J=qx.ui.core.Widget.prototype.renderLayout.call(this,I,top,H,K);this._updateScrollBar();return J;},_getContentHint:function(){var L=qx.bom.element.Scroll.getScrollbarWidth();return {width:this.__nY?100:L,maxWidth:this.__nY?null:L,minWidth:this.__nY?null:L,height:this.__nY?L:100,maxHeight:this.__nY?L:null,minHeight:this.__nY?L:null};},_applyEnabled:function(N,M){qx.ui.core.Widget.prototype._applyEnabled.call(this,N,M);this._updateScrollBar();},_applyMaximum:function(O){this._updateScrollBar();},_applyPosition:function(P){var content=this.getContentElement();if(this.__nY){content.scrollToX(P);}else {content.scrollToY(P);};},_applyOrientation:function(S,Q){var R=this.__nY=S===x;this.set({allowGrowX:R,allowShrinkX:R,allowGrowY:!R,allowShrinkY:!R});if(R){this.replaceState(n,x);}else {this.replaceState(x,n);};this.getContentElement().setStyles({overflowX:R?F:k,overflowY:R?k:F});qx.ui.core.queue.Layout.add(this);},_updateScrollBar:function(){var U=this.__nY;var V=this.getBounds();if(!V){return;};if(this.isEnabled()){var W=U?V.width:V.height;var T=this.getMaximum()+W;}else {T=0;};if(qx.core.Environment.get(w)==v||qx.core.Environment.get(j)==z){var V=this.getBounds();this.getContentElement().setStyles({left:(U?V.left:(V.left-1))+b,top:(U?(V.top-1):V.top)+b,width:(U?V.width:V.width+1)+b,height:(U?V.height+1:V.height)+b});};this._getScrollPaneElement().setStyles({left:0,top:0,width:(U?T:1)+b,height:(U?1:T)+b});this.updatePosition(this.getPosition());},scrollTo:function(Y,X){this.stopScrollAnimation();if(X){var ba=this.getPosition();this.__ob=new qx.bom.AnimationFrame();this.__ob.on(d,function(bc){var bb=parseInt(bc/X*(Y-ba)+ba);this.updatePosition(bb);},this);this.__ob.on(f,function(){this.setPosition(Math.max(0,Math.min(this.getMaximum(),Y)));this.__ob=null;this.fireEvent(p);},this);this.__ob.startSequence(X);}else {this.updatePosition(Y);};},updatePosition:function(bd){this.setPosition(Math.max(0,Math.min(this.getMaximum(),bd)));},scrollBy:function(bf,be){this.scrollTo(this.getPosition()+bf,be);},scrollBySteps:function(bh,bg){var bi=this.getSingleStep();this.scrollBy(bh*bi,bg);},stopScrollAnimation:function(){if(this.__ob){this.__ob.cancelSequence();this.__ob=null;};},_onScroll:function(e){var bj=this.getContentElement();var bk=this.__nY?bj.getScrollX():bj.getScrollY();this.setPosition(bk);},_onAppear:function(e){this._applyPosition(this.getPosition());},_stopPropagation:function(e){e.stopPropagation();}},destruct:function(){this._disposeObjects(q);}});})();(function(){var a="slider",b="slideAnimationEnd",c="PositiveNumber",d="changeValue",f="Integer",g="execute",h="track",i="qx.lang.Type.isNumber(value)&&value>=0&&value<=this.getMaximum()",j="_applyKnobFactor",k="visible",l="qx.event.type.Event",m="knob",n="button-begin",o="hidden",p="qx.ui.core.scroll.ScrollBar",q="resize",r="vertical",s="_applyOrientation",t="scrollAnimationEnd",u="_applyPageStep",v="right",w="PositiveInteger",x="horizontal",y="up",z="_applyPosition",A="scrollbar",B="_applyMaximum",C="left",D="button-end",E="down",F="scroll";qx.Class.define(p,{extend:qx.ui.core.Widget,implement:qx.ui.core.scroll.IScrollBar,construct:function(G){qx.ui.core.Widget.call(this);this._createChildControl(n);this._createChildControl(a).addListener(q,this._onResizeSlider,this);this._createChildControl(D);if(G!=null){this.setOrientation(G);}else {this.initOrientation();};this.addListener(h,function(e){e.stopPropagation();},this);},events:{"scrollAnimationEnd":l},properties:{appearance:{refine:true,init:A},orientation:{check:[x,r],init:x,apply:s},maximum:{check:w,apply:B,init:100},position:{check:i,init:0,apply:z,event:F},singleStep:{check:f,init:20},pageStep:{check:f,init:10,apply:u},knobFactor:{check:c,apply:j,nullable:true}},members:{__oc:2,__od:0,_computeSizeHint:function(){var H=qx.ui.core.Widget.prototype._computeSizeHint.call(this);if(this.getOrientation()===x){this.__od=H.minWidth;H.minWidth=0;}else {this.__od=H.minHeight;H.minHeight=0;};return H;},renderLayout:function(M,top,J,I){var L=qx.ui.core.Widget.prototype.renderLayout.call(this,M,top,J,I);var K=this.getOrientation()===x;if(this.__od>=(K?J:I)){this.getChildControl(n).setVisibility(o);this.getChildControl(D).setVisibility(o);}else {this.getChildControl(n).setVisibility(k);this.getChildControl(D).setVisibility(k);};return L;},_createChildControlImpl:function(P,O){var N;switch(P){case a:N=new qx.ui.core.scroll.ScrollSlider();N.setPageStep(100);N.setFocusable(false);N.addListener(d,this._onChangeSliderValue,this);N.addListener(b,this._onSlideAnimationEnd,this);this._add(N,{flex:1});break;case n:N=new qx.ui.form.RepeatButton();N.setFocusable(false);N.addListener(g,this._onExecuteBegin,this);this._add(N);break;case D:N=new qx.ui.form.RepeatButton();N.setFocusable(false);N.addListener(g,this._onExecuteEnd,this);this._add(N);break;};return N||qx.ui.core.Widget.prototype._createChildControlImpl.call(this,P);},_applyMaximum:function(Q){this.getChildControl(a).setMaximum(Q);},_applyPosition:function(R){this.getChildControl(a).setValue(R);},_applyKnobFactor:function(S){this.getChildControl(a).setKnobFactor(S);},_applyPageStep:function(T){this.getChildControl(a).setPageStep(T);},_applyOrientation:function(W,U){var V=this._getLayout();if(V){V.dispose();};if(W===x){this._setLayout(new qx.ui.layout.HBox());this.setAllowStretchX(true);this.setAllowStretchY(false);this.replaceState(r,x);this.getChildControl(n).replaceState(y,C);this.getChildControl(D).replaceState(E,v);}else {this._setLayout(new qx.ui.layout.VBox());this.setAllowStretchX(false);this.setAllowStretchY(true);this.replaceState(x,r);this.getChildControl(n).replaceState(C,y);this.getChildControl(D).replaceState(v,E);};this.getChildControl(a).setOrientation(W);},scrollTo:function(Y,X){this.getChildControl(a).slideTo(Y,X);},scrollBy:function(bb,ba){this.getChildControl(a).slideBy(bb,ba);},scrollBySteps:function(bd,bc){var be=this.getSingleStep();this.getChildControl(a).slideBy(bd*be,bc);},updatePosition:function(bf){this.getChildControl(a).updatePosition(bf);},stopScrollAnimation:function(){this.getChildControl(a).stopSlideAnimation();},_onExecuteBegin:function(e){this.scrollBy(-this.getSingleStep(),50);},_onExecuteEnd:function(e){this.scrollBy(this.getSingleStep(),50);},_onSlideAnimationEnd:function(){this.fireEvent(t);},_onChangeSliderValue:function(e){this.setPosition(e.getData());},_onResizeSlider:function(e){var bg=this.getChildControl(a).getChildControl(m);var bj=bg.getSizeHint();var bh=false;var bi=this.getChildControl(a).getInnerSize();if(this.getOrientation()==r){if(bi.height<bj.minHeight+this.__oc){bh=true;};}else {if(bi.width<bj.minWidth+this.__oc){bh=true;};};if(bh){bg.exclude();}else {bg.show();};}}});})();(function(){var a="",b="qx.ui.form.MForm",c="Boolean",d="_applyValid",f="changeLocale",g="changeRequired",h="changeValid",i="qx.dynlocale",j="changeInvalidMessage",k="String",l="invalid";qx.Mixin.define(b,{construct:function(){if(qx.core.Environment.get(i)){qx.locale.Manager.getInstance().addListener(f,this.__oe,this);};},properties:{valid:{check:c,init:true,apply:d,event:h},required:{check:c,init:false,event:g},invalidMessage:{check:k,init:a,event:j},requiredInvalidMessage:{check:k,nullable:true,event:j}},members:{_applyValid:function(n,m){n?this.removeState(l):this.addState(l);},__oe:qx.core.Environment.select(i,{"true":function(e){var o=this.getInvalidMessage();if(o&&o.translate){this.setInvalidMessage(o.translate());};var p=this.getRequiredInvalidMessage();if(p&&p.translate){this.setRequiredInvalidMessage(p.translate());};},"false":null})},destruct:function(){if(qx.core.Environment.get(i)){qx.locale.Manager.getInstance().removeListener(f,this.__oe,this);};}});})();(function(){var a="qx.ui.form.IRange";qx.Interface.define(a,{members:{setMinimum:function(b){return arguments.length==1;},getMinimum:function(){},setMaximum:function(c){return arguments.length==1;},getMaximum:function(){},setSingleStep:function(d){return arguments.length==1;},getSingleStep:function(){},setPageStep:function(e){return arguments.length==1;},getPageStep:function(){}}});})();(function(){var a="qx.ui.form.INumberForm",b="qx.event.type.Data";qx.Interface.define(a,{events:{"changeValue":b},members:{setValue:function(c){return arguments.length==1;},resetValue:function(){},getValue:function(){}}});})();(function(){var a="slider",b="hovered",c="pressed",d="px",f="pointerover",g="PageUp",h="changeValue",i="frame",j="end",k='qx.event.type.Data',l="Left",m="Down",n="Integer",o="roll",p="qx.ui.form.Slider",q="PageDown",r="interval",s="_applyValue",t="Up",u="_applyKnobFactor",v="End",w="pointermove",x="height",y="y",z="dbltap",A="resize",B="pointerup",C="vertical",D="Right",E="width",F="_applyOrientation",G="left",H="Home",I="floor",J="_applyMinimum",K="pointerdown",L="top",M="changeMaximum",N="horizontal",O="slideAnimationEnd",P="knob",Q="ceil",R='qx.event.type.Event',S="x",T="keypress",U="losecapture",V="contextmenu",W="wheel",X="_applyMaximum",Y="Number",bf="tap",bg="typeof value==='number'&&value>=this.getMinimum()&&value<=this.getMaximum()",bh="changeMinimum",be="pointerout";qx.Class.define(p,{extend:qx.ui.core.Widget,implement:[qx.ui.form.IForm,qx.ui.form.INumberForm,qx.ui.form.IRange],include:[qx.ui.form.MForm],construct:function(bi){qx.ui.core.Widget.call(this);this._setLayout(new qx.ui.layout.Canvas());this.addListener(T,this._onKeyPress);this.addListener(o,this._onRoll);this.addListener(K,this._onPointerDown);this.addListener(B,this._onPointerUp);this.addListener(U,this._onPointerUp);this.addListener(A,this._onUpdate);this.addListener(V,this._onStopEvent);this.addListener(bf,this._onStopEvent);this.addListener(z,this._onStopEvent);if(bi!=null){this.setOrientation(bi);}else {this.initOrientation();};},events:{changeValue:k,slideAnimationEnd:R},properties:{appearance:{refine:true,init:a},focusable:{refine:true,init:true},orientation:{check:[N,C],init:N,apply:F},value:{check:bg,init:0,apply:s,nullable:true},minimum:{check:n,init:0,apply:J,event:bh},maximum:{check:n,init:100,apply:X,event:M},singleStep:{check:n,init:1},pageStep:{check:n,init:10},knobFactor:{check:Y,apply:u,nullable:true}},members:{__of:null,__og:null,__oh:null,__oi:null,__oj:null,__ok:null,__ol:null,__om:null,__de:null,__on:null,__oo:null,__op:null,__ob:null,_forwardStates:{invalid:true},renderLayout:function(bk,top,bj,bl){qx.ui.core.Widget.prototype.renderLayout.call(this,bk,top,bj,bl);this._updateKnobPosition();},_createChildControlImpl:function(bo,bn){var bm;switch(bo){case P:bm=new qx.ui.core.Widget();bm.addListener(A,this._onUpdate,this);bm.addListener(f,this._onPointerOver);bm.addListener(be,this._onPointerOut);this._add(bm);break;};return bm||qx.ui.core.Widget.prototype._createChildControlImpl.call(this,bo);},_onPointerOver:function(e){this.addState(b);},_onPointerOut:function(e){this.removeState(b);},_onRoll:function(e){if(e.getPointerType()!=W){return;};var bq=this.getOrientation()===N?S:y;var br=e.getDelta()[bq];var bp=br>0?1:br<0?-1:0;this.slideBy(bp*this.getSingleStep());e.stop();},_onKeyPress:function(e){var bt=this.getOrientation()===N;var bs=bt?l:t;var forward=bt?D:m;switch(e.getKeyIdentifier()){case forward:this.slideForward();break;case bs:this.slideBack();break;case q:this.slidePageForward(100);break;case g:this.slidePageBack(100);break;case H:this.slideToBegin(200);break;case v:this.slideToEnd(200);break;default:return;};e.stop();},_onPointerDown:function(e){if(this.__oi){return;};var bw=this.__nY;var bu=this.getChildControl(P);var bv=bw?G:L;var bA=bw?e.getDocumentLeft():e.getDocumentTop();var bC=this.getDecorator();bC=qx.theme.manager.Decoration.getInstance().resolve(bC);if(bw){var bz=bC?bC.getInsets().left:0;var by=(this.getPaddingLeft()||0)+bz;}else {var bz=bC?bC.getInsets().top:0;var by=(this.getPaddingTop()||0)+bz;};var bB=this.__of=qx.bom.element.Location.get(this.getContentElement().getDomElement())[bv];bB+=by;var bx=this.__og=qx.bom.element.Location.get(bu.getContentElement().getDomElement())[bv];if(e.getTarget()===bu){this.__oi=true;if(!this.__on){this.__on=new qx.event.Timer(100);this.__on.addListener(r,this._fireValue,this);};this.__on.start();this.__oj=bA+bB-bx;bu.addState(c);}else {this.__ok=true;this.__ol=bA<=bx?-1:1;this.__or(e);this._onInterval();if(!this.__de){this.__de=new qx.event.Timer(100);this.__de.addListener(r,this._onInterval,this);};this.__de.start();};this.addListener(w,this._onPointerMove);this.capture();e.stopPropagation();},_onPointerUp:function(e){if(this.__oi){this.releaseCapture();delete this.__oi;this.__on.stop();this._fireValue();delete this.__oj;this.getChildControl(P).removeState(c);if(e.getType()===B){var bE;var bF;var bD;if(this.__nY){bE=e.getDocumentLeft()-(this._valueToPosition(this.getValue())+this.__of);bD=qx.bom.element.Location.get(this.getContentElement().getDomElement())[L];bF=e.getDocumentTop()-(bD+this.getChildControl(P).getBounds().top);}else {bE=e.getDocumentTop()-(this._valueToPosition(this.getValue())+this.__of);bD=qx.bom.element.Location.get(this.getContentElement().getDomElement())[G];bF=e.getDocumentLeft()-(bD+this.getChildControl(P).getBounds().left);};if(bF<0||bF>this.__oh||bE<0||bE>this.__oh){this.getChildControl(P).removeState(b);};};}else if(this.__ok){this.__de.stop();this.releaseCapture();delete this.__ok;delete this.__ol;delete this.__om;};this.removeListener(w,this._onPointerMove);if(e.getType()===B){e.stopPropagation();};},_onPointerMove:function(e){if(this.__oi){var bH=this.__nY?e.getDocumentLeft():e.getDocumentTop();var bG=bH-this.__oj;this.slideTo(this._positionToValue(bG));}else if(this.__ok){this.__or(e);};e.stopPropagation();},_onInterval:function(e){var bI=this.getValue()+(this.__ol*this.getPageStep());if(bI<this.getMinimum()){bI=this.getMinimum();}else if(bI>this.getMaximum()){bI=this.getMaximum();};var bJ=this.__ol==-1;if((bJ&&bI<=this.__om)||(!bJ&&bI>=this.__om)){bI=this.__om;};this.slideTo(bI);},_onUpdate:function(e){var bL=this.getInnerSize();var bM=this.getChildControl(P).getBounds();var bK=this.__nY?E:x;this._updateKnobSize();this.__oq=bL[bK]-bM[bK];this.__oh=bM[bK];this._updateKnobPosition();},__nY:false,__oq:0,__or:function(e){var bN=this.__nY;var bU=bN?e.getDocumentLeft():e.getDocumentTop();var bX=this.__of;var bO=this.__og;var bY=this.__oh;var bV=bU-bX;if(bU>=bO){bV-=bY;};var bR=this._positionToValue(bV);var bP=this.getMinimum();var bQ=this.getMaximum();if(bR<bP){bR=bP;}else if(bR>bQ){bR=bQ;}else {var bW=this.getValue();var bT=this.getPageStep();var bS=this.__ol<0?I:Q;bR=bW+(Math[bS]((bR-bW)/bT)*bT);};if(this.__om==null||(this.__ol==-1&&bR<=this.__om)||(this.__ol==1&&bR>=this.__om)){this.__om=bR;};},_positionToValue:function(cb){var ca=this.__oq;if(ca==null||ca==0){return 0;};var cd=cb/ca;if(cd<0){cd=0;}else if(cd>1){cd=1;};var cc=this.getMaximum()-this.getMinimum();return this.getMinimum()+Math.round(cc*cd);},_valueToPosition:function(ch){var ce=this.__oq;if(ce==null){return 0;};var cf=this.getMaximum()-this.getMinimum();if(cf==0){return 0;};var ch=ch-this.getMinimum();var cg=ch/cf;if(cg<0){cg=0;}else if(cg>1){cg=1;};return Math.round(ce*cg);},_updateKnobPosition:function(){this._setKnobPosition(this._valueToPosition(this.getValue()));},_setKnobPosition:function(ck){var ci=this.getChildControl(P);var cj=this.getDecorator();cj=qx.theme.manager.Decoration.getInstance().resolve(cj);var content=ci.getContentElement();if(this.__nY){if(cj&&cj.getPadding()){ck+=cj.getPadding().left;};ck+=this.getPaddingLeft()||0;content.setStyle(G,ck+d,true);}else {if(cj&&cj.getPadding()){ck+=cj.getPadding().top;};ck+=this.getPaddingTop()||0;content.setStyle(L,ck+d,true);};},_updateKnobSize:function(){var cm=this.getKnobFactor();if(cm==null){return;};var cl=this.getInnerSize();if(cl==null){return;};if(this.__nY){this.getChildControl(P).setWidth(Math.round(cm*cl.width));}else {this.getChildControl(P).setHeight(Math.round(cm*cl.height));};},slideToBegin:function(cn){this.slideTo(this.getMinimum(),cn);},slideToEnd:function(co){this.slideTo(this.getMaximum(),co);},slideForward:function(){this.slideBy(this.getSingleStep());},slideBack:function(){this.slideBy(-this.getSingleStep());},slidePageForward:function(cp){this.slideBy(this.getPageStep(),cp);},slidePageBack:function(cq){this.slideBy(-this.getPageStep(),cq);},slideBy:function(cs,cr){this.slideTo(this.getValue()+cs,cr);},slideTo:function(cu,ct){this.stopSlideAnimation();if(ct){this.__ot(cu,ct);}else {this.updatePosition(cu);};},updatePosition:function(cv){this.setValue(this.__os(cv));},stopSlideAnimation:function(){if(this.__ob){this.__ob.cancelSequence();this.__ob=null;};},__os:function(cw){if(cw<this.getMinimum()){cw=this.getMinimum();}else if(cw>this.getMaximum()){cw=this.getMaximum();}else {cw=this.getMinimum()+Math.round((cw-this.getMinimum())/this.getSingleStep())*this.getSingleStep();};return cw;},__ot:function(cy,cx){cy=this.__os(cy);var cz=this.getValue();this.__ob=new qx.bom.AnimationFrame();this.__ob.on(i,function(cA){this.setValue(parseInt(cA/cx*(cy-cz)+cz));},this);this.__ob.on(j,function(){this.setValue(cy);this.__ob=null;this.fireEvent(O);},this);this.__ob.startSequence(cx);},_applyOrientation:function(cD,cC){var cB=this.getChildControl(P);this.__nY=cD===N;if(this.__nY){this.removeState(C);cB.removeState(C);this.addState(N);cB.addState(N);cB.setLayoutProperties({top:0,right:null,bottom:0});}else {this.removeState(N);cB.removeState(N);this.addState(C);cB.addState(C);cB.setLayoutProperties({right:0,bottom:null,left:0});};this._updateKnobPosition();},_applyKnobFactor:function(cF,cE){if(cF!=null){this._updateKnobSize();}else {if(this.__nY){this.getChildControl(P).resetWidth();}else {this.getChildControl(P).resetHeight();};};},_applyValue:function(cH,cG){if(cH!=null){this._updateKnobPosition();if(this.__oi){this.__op=[cH,cG];}else {this.fireEvent(h,qx.event.type.Data,[cH,cG]);};}else {this.resetValue();};},_fireValue:function(){if(!this.__op){return;};var cI=this.__op;this.__op=null;this.fireEvent(h,qx.event.type.Data,cI);},_applyMinimum:function(cK,cJ){if(this.getValue()<cK){this.setValue(cK);};this._updateKnobPosition();},_applyMaximum:function(cM,cL){if(this.getValue()>cM){this.setValue(cM);};this._updateKnobPosition();}}});})();(function(){var a="dblclick",b="qx.ui.core.scroll.ScrollSlider",c="horizontal",d="keypress",f="roll",g="knob";qx.Class.define(b,{extend:qx.ui.form.Slider,construct:function(h){qx.ui.form.Slider.call(this,h);this.removeListener(d,this._onKeyPress);this.removeListener(f,this._onRoll);},members:{_createChildControlImpl:function(k,j){var i;switch(k){case g:i=qx.ui.form.Slider.prototype._createChildControlImpl.call(this,k);i.addListener(a,function(e){e.stopPropagation();});};return i||qx.ui.form.Slider.prototype._createChildControlImpl.call(this,k);},getSizeHint:function(l){var m=qx.ui.form.Slider.prototype.getSizeHint.call(this);if(this.getOrientation()===c){m.width=0;}else {m.height=0;};return m;}}});})();(function(){var a="toolTipText",b="icon",c="label",d="qx.ui.core.MExecutable",f="enabled",g="value",h="qx.event.type.Event",j="execute",k="_applyCommand",l="qx.ui.command.Command",m="menu",n="changeCommand";qx.Mixin.define(d,{events:{"execute":h},properties:{command:{check:l,apply:k,event:n,nullable:true}},members:{__ou:null,__ov:false,__ow:null,_bindableProperties:[f,c,b,a,g,m],execute:function(){var o=this.getCommand();if(o){if(this.__ov){this.__ov=false;}else {this.__ov=true;o.execute(this);};};this.fireEvent(j);},__ox:function(e){if(this.__ov){this.__ov=false;return;};if(this.isEnabled()){this.__ov=true;this.execute();};},_applyCommand:function(r,p){if(p!=null){p.removeListenerById(this.__ow);};if(r!=null){this.__ow=r.addListener(j,this.__ox,this);};var q=this.__ou;if(q==null){this.__ou=q={};};var u;for(var i=0;i<this._bindableProperties.length;i++ ){var t=this._bindableProperties[i];if(p!=null&&!p.isDisposed()&&q[t]!=null){p.removeBinding(q[t]);q[t]=null;};if(r!=null&&qx.Class.hasProperty(this.constructor,t)){var s=r.get(t);if(s==null){u=this.get(t);if(u==null){this.syncAppearance();u=qx.util.PropertyUtil.getThemeValue(this,t);};}else {u=null;};q[t]=r.bind(t,this,t);if(u){this.set(t,u);};};};}},destruct:function(){this._applyCommand(null,this.getCommand());this.__ou=null;}});})();(function(){var a="qx.ui.form.IExecutable",b="qx.event.type.Data";qx.Interface.define(a,{events:{"execute":b},members:{setCommand:function(c){return arguments.length==1;},getCommand:function(){},execute:function(){}}});})();(function(){var a="dblclick",b="qx.ui.form.Button",c="pointerup",d="Enter",f="pressed",g="pointerover",h="hovered",i="pointerdown",j="Space",k="keydown",l="abandoned",m="tap",n="button",o="keyup",p="pointerout";qx.Class.define(b,{extend:qx.ui.basic.Atom,include:[qx.ui.core.MExecutable],implement:[qx.ui.form.IExecutable],construct:function(q,s,r){qx.ui.basic.Atom.call(this,q,s);if(r!=null){this.setCommand(r);};this.addListener(g,this._onPointerOver);this.addListener(p,this._onPointerOut);this.addListener(i,this._onPointerDown);this.addListener(c,this._onPointerUp);this.addListener(m,this._onTap);this.addListener(k,this._onKeyDown);this.addListener(o,this._onKeyUp);this.addListener(a,function(e){e.stopPropagation();});},properties:{appearance:{refine:true,init:n},focusable:{refine:true,init:true}},members:{_forwardStates:{focused:true,hovered:true,pressed:true,disabled:true},press:function(){if(this.hasState(l)){return;};this.addState(f);},release:function(){if(this.hasState(f)){this.removeState(f);};},reset:function(){this.removeState(f);this.removeState(l);this.removeState(h);},_onPointerOver:function(e){if(!this.isEnabled()||e.getTarget()!==this){return;};if(this.hasState(l)){this.removeState(l);this.addState(f);};this.addState(h);},_onPointerOut:function(e){if(!this.isEnabled()||e.getTarget()!==this){return;};this.removeState(h);if(this.hasState(f)){this.removeState(f);this.addState(l);};},_onPointerDown:function(e){if(!e.isLeftPressed()){return;};e.stopPropagation();this.capture();this.removeState(l);this.addState(f);},_onPointerUp:function(e){this.releaseCapture();var t=this.hasState(f);var u=this.hasState(l);if(t){this.removeState(f);};if(u){this.removeState(l);};e.stopPropagation();},_onTap:function(e){this.execute();e.stopPropagation();},_onKeyDown:function(e){switch(e.getKeyIdentifier()){case d:case j:this.removeState(l);this.addState(f);e.stopPropagation();};},_onKeyUp:function(e){switch(e.getKeyIdentifier()){case d:case j:if(this.hasState(f)){this.removeState(l);this.removeState(f);this.execute();e.stopPropagation();};};}}});})();(function(){var a="press",b="hovered",c="qx.ui.form.RepeatButton",d="release",f="Enter",g="pressed",h="interval",i="__de",j="qx.event.type.Event",k="Space",l="abandoned",m="Integer",n="execute";qx.Class.define(c,{extend:qx.ui.form.Button,construct:function(o,p){qx.ui.form.Button.call(this,o,p);this.__de=new qx.event.AcceleratingTimer();this.__de.addListener(h,this._onInterval,this);},events:{"execute":j,"press":j,"release":j},properties:{interval:{check:m,init:100},firstInterval:{check:m,init:500},minTimer:{check:m,init:20},timerDecrease:{check:m,init:2}},members:{__oy:null,__de:null,press:function(){if(this.isEnabled()){if(!this.hasState(g)){this.__oz();};this.removeState(l);this.addState(g);};},release:function(q){if(!this.isEnabled()){return;};if(this.hasState(g)){if(!this.__oy){this.execute();};};this.removeState(g);this.removeState(l);this.__oA();},_applyEnabled:function(s,r){qx.ui.form.Button.prototype._applyEnabled.call(this,s,r);if(!s){if(this.isCapturing()){this.releaseCapture();};this.removeState(g);this.removeState(l);this.__oA();};},_onPointerOver:function(e){if(!this.isEnabled()||e.getTarget()!==this){return;};if(this.hasState(l)){this.removeState(l);this.addState(g);this.__de.start();};this.addState(b);},_onPointerOut:function(e){if(!this.isEnabled()||e.getTarget()!==this){return;};this.removeState(b);if(this.hasState(g)){this.removeState(g);this.addState(l);this.__de.stop();};},_onPointerDown:function(e){if(!e.isLeftPressed()){return;};this.capture();this.__oz();e.stopPropagation();},_onPointerUp:function(e){this.releaseCapture();if(!this.hasState(l)){this.addState(b);if(this.hasState(g)&&!this.__oy){this.execute();};};this.__oA();e.stopPropagation();},_onTap:function(e){},_onKeyUp:function(e){switch(e.getKeyIdentifier()){case f:case k:if(this.hasState(g)){if(!this.__oy){this.execute();};this.removeState(g);this.removeState(l);e.stopPropagation();this.__oA();};};},_onKeyDown:function(e){switch(e.getKeyIdentifier()){case f:case k:this.removeState(l);this.addState(g);e.stopPropagation();this.__oz();};},_onInterval:function(e){this.__oy=true;this.fireEvent(n);},__oz:function(){this.fireEvent(a);this.__oy=false;this.__de.set({interval:this.getInterval(),firstInterval:this.getFirstInterval(),minimum:this.getMinTimer(),decrease:this.getTimerDecrease()}).start();this.removeState(l);this.addState(g);},__oA:function(){this.fireEvent(d);this.__de.stop();this.removeState(l);this.removeState(g);}},destruct:function(){this._disposeObjects(i);}});})();(function(){var a="Integer",b="__de",c="interval",d="qx.event.type.Event",e="qx.event.AcceleratingTimer";qx.Class.define(e,{extend:qx.core.Object,construct:function(){qx.core.Object.call(this);this.__de=new qx.event.Timer(this.getInterval());this.__de.addListener(c,this._onInterval,this);},events:{"interval":d},properties:{interval:{check:a,init:100},firstInterval:{check:a,init:500},minimum:{check:a,init:20},decrease:{check:a,init:2}},members:{__de:null,__oB:null,start:function(){this.__de.setInterval(this.getFirstInterval());this.__de.start();},stop:function(){this.__de.stop();this.__oB=null;},_onInterval:function(){this.__de.stop();if(this.__oB==null){this.__oB=this.getInterval();};this.__oB=Math.max(this.getMinimum(),this.__oB-this.getDecrease());this.__de.setInterval(this.__oB);this.__de.start();this.fireEvent(c);}},destruct:function(){this._disposeObjects(b);}});})();(function(){var a="Decorator",b="_applyLayoutChange",c="center",d="_applyReversed",e="bottom",f="qx.ui.layout.VBox",g="top",h="left",j="middle",k="Integer",m="right",n="Boolean";qx.Class.define(f,{extend:qx.ui.layout.Abstract,construct:function(o,p,q){qx.ui.layout.Abstract.call(this);if(o){this.setSpacing(o);};if(p){this.setAlignY(p);};if(q){this.setSeparator(q);};},properties:{alignY:{check:[g,j,e],init:g,apply:b},alignX:{check:[h,c,m],init:h,apply:b},spacing:{check:k,init:0,apply:b},separator:{check:a,nullable:true,apply:b},reversed:{check:n,init:false,apply:d}},members:{__oC:null,__lv:null,__lw:null,__gP:null,_applyReversed:function(){this._invalidChildrenCache=true;this._applyLayoutChange();},__lx:function(){var w=this._getLayoutChildren();var length=w.length;var s=false;var r=this.__oC&&this.__oC.length!=length&&this.__lv&&this.__oC;var u;var t=r?this.__oC:new Array(length);var v=r?this.__lv:new Array(length);if(this.getReversed()){w=w.concat().reverse();};for(var i=0;i<length;i++ ){u=w[i].getLayoutProperties();if(u.height!=null){t[i]=parseFloat(u.height)/100;};if(u.flex!=null){v[i]=u.flex;s=true;}else {v[i]=0;};};if(!r){this.__oC=t;this.__lv=v;};this.__lw=s;this.__gP=w;delete this._invalidChildrenCache;},verifyLayoutProperty:null,renderLayout:function(O,G,R){if(this._invalidChildrenCache){this.__lx();};var D=this.__gP;var length=D.length;var N=qx.ui.layout.Util;var M=this.getSpacing();var T=this.getSeparator();if(T){var A=N.computeVerticalSeparatorGaps(D,M,T);}else {var A=N.computeVerticalGaps(D,M,true);};var i,S,z,H;var I=[];var P=A;for(i=0;i<length;i+=1){H=this.__oC[i];z=H!=null?Math.floor((G-A)*H):D[i].getSizeHint().height;I.push(z);P+=z;};if(this.__lw&&P!=G){var F={};var L,y;for(i=0;i<length;i+=1){L=this.__lv[i];if(L>0){E=D[i].getSizeHint();F[i]={min:E.minHeight,value:I[i],max:E.maxHeight,flex:L};};};var B=N.computeFlexOffsets(F,G,P);for(i in B){y=B[i].offset;I[i]+=y;P+=y;};};var top=D[0].getMarginTop();if(P<G&&this.getAlignY()!=g){top=G-P;if(this.getAlignY()===j){top=Math.round(top/2);};};var E,V,J,z,x,K,C;this._clearSeparators();if(T){var U=qx.theme.manager.Decoration.getInstance().resolve(T).getInsets();var Q=U.top+U.bottom;};for(i=0;i<length;i+=1){S=D[i];z=I[i];E=S.getSizeHint();K=S.getMarginLeft();C=S.getMarginRight();J=Math.max(E.minWidth,Math.min(O-K-C,E.maxWidth));V=N.computeHorizontalAlignOffset(S.getAlignX()||this.getAlignX(),J,O,K,C);if(i>0){if(T){top+=x+M;this._renderSeparator(T,{top:top+R.top,left:R.left,height:Q,width:O});top+=Q+M+S.getMarginTop();}else {top+=N.collapseMargins(M,x,S.getMarginTop());};};S.renderLayout(V+R.left,top+R.top,J,z);top+=z;x=S.getMarginBottom();};},_computeSizeHint:function(){if(this._invalidChildrenCache){this.__lx();};var W=qx.ui.layout.Util;var bl=this.__gP;var ba=0,bb=0,bj=0;var bc=0,bd=0;var bh,X,bk;for(var i=0,l=bl.length;i<l;i+=1){bh=bl[i];X=bh.getSizeHint();bb+=X.height;var bg=this.__lv[i];var Y=this.__oC[i];if(bg){ba+=X.minHeight;}else if(Y){bj=Math.max(bj,Math.round(X.minHeight/Y));}else {ba+=X.height;};bk=bh.getMarginLeft()+bh.getMarginRight();if((X.width+bk)>bd){bd=X.width+bk;};if((X.minWidth+bk)>bc){bc=X.minWidth+bk;};};ba+=bj;var bf=this.getSpacing();var bi=this.getSeparator();if(bi){var be=W.computeVerticalSeparatorGaps(bl,bf,bi);}else {var be=W.computeVerticalGaps(bl,bf,true);};return {minHeight:ba+be,height:bb+be,minWidth:bc,width:bd};}},destruct:function(){this.__oC=this.__lv=this.__gP=null;}});})();(function(){var a="_applyStatusBarVisible",b="columnVisibilityMenuCreateStart",c="function",d="blur",f="qx.ui.table.Table",g="Boolean",h="_applyFocusCellOnPointerMove",k="columnVisibilityMenuCreateEnd",m="__oM",n="changeVisible",o="verticalScrollBarChanged",p="_applyMetaColumnCounts",q="PageUp",r="focus",s="changeDataRowRenderer",t="dataChanged",u="changeHeaderCellHeight",v="Escape",w="A",y="changeSelectionModel",z="__de",A="Left",B="Down",C="Integer",D="_applyHeaderCellHeight",E="qx.event.type.Data",F="statusbar",G="Function",H="menu",I="qx.ui.table.ITableModel",J="column-button",K="_applySelectionModel",L="changeLocale",M="menu-button",N="_applyResetSelectionOnHeaderTap",O="_applyAdditionalStatusBarText",P="table",Q="_applyColumnVisibilityButtonVisible",R="changeTableModel",S="qx.event.type.Event",T="tableWidthChanged",U="changeSelection",V="_applyHeaderCellsVisible",W="Object",X="_applyShowCellFocusIndicator",Y="resize",bQ="changeScrollY",bR="_applyTableModel",bS="End",bM="_applyKeepFirstVisibleRowComplete",bN="widthChanged",bO="one of one row",bP="Home",bW="_applyRowHeight",bX="F2",bY="appear",cs="visibilityChanged",bT="qx.dynlocale",bU="%1 rows",bV="qx.ui.table.selection.Model",bJ='"',cq="one row",bL="PageDown",cc="%1 of %2 rows",cd="__oN",bK="__oL",ca="keypress",ck="changeRowHeight",cb="Up",ce="Enter",cf="Number",cg="metaDataChanged",cl="__oE",cm="header",cp="__oD",cr="_applyContextMenuFromDataCellsOnly",ch="qx.ui.table.IRowRenderer",ci="Right",cj="orderChanged",cn="qx.ui.table.pane.CellEvent",co="Space";qx.Class.define(f,{extend:qx.ui.core.Widget,include:qx.ui.core.MDragDropScrolling,construct:function(ct,cv){qx.ui.core.Widget.call(this);if(!cv){cv={};};if(cv.initiallyHiddenColumns){this.setInitiallyHiddenColumns(cv.initiallyHiddenColumns);};if(cv.selectionManager){this.setNewSelectionManager(cv.selectionManager);};if(cv.selectionModel){this.setNewSelectionModel(cv.selectionModel);};if(cv.tableColumnModel){this.setNewTableColumnModel(cv.tableColumnModel);};if(cv.tablePane){this.setNewTablePane(cv.tablePane);};if(cv.tablePaneHeader){this.setNewTablePaneHeader(cv.tablePaneHeader);};if(cv.tablePaneScroller){this.setNewTablePaneScroller(cv.tablePaneScroller);};if(cv.tablePaneModel){this.setNewTablePaneModel(cv.tablePaneModel);};if(cv.columnMenu){this.setNewColumnMenu(cv.columnMenu);};this._setLayout(new qx.ui.layout.VBox());this.__oD=new qx.ui.container.Composite(new qx.ui.layout.HBox());this._add(this.__oD,{flex:1});this.setDataRowRenderer(new qx.ui.table.rowrenderer.Default(this));this.__oE=this.getNewSelectionManager()(this);this.setSelectionModel(this.getNewSelectionModel()(this));this.setTableModel(ct||this.getEmptyTableModel());this.setMetaColumnCounts([-1]);this.setTabIndex(1);this.addListener(ca,this._onKeyPress);this.addListener(r,this._onFocusChanged);this.addListener(d,this._onFocusChanged);var cu=new qx.ui.core.Widget().set({height:0});this._add(cu);cu.addListener(Y,this._onResize,this);this.__oF=null;this.__oG=null;if(qx.core.Environment.get(bT)){qx.locale.Manager.getInstance().addListener(L,this._onChangeLocale,this);};this.initStatusBarVisible();ct=this.getTableModel();if(ct.init&&typeof (ct.init)==c){ct.init(this);};},events:{"columnVisibilityMenuCreateStart":E,"columnVisibilityMenuCreateEnd":E,"tableWidthChanged":S,"verticalScrollBarChanged":E,"cellTap":cn,"cellDbltap":cn,"cellContextmenu":cn,"dataEdited":E},statics:{__oH:{cellTap:1,cellDbltap:1,cellContextmenu:1}},properties:{appearance:{refine:true,init:P},focusable:{refine:true,init:true},minWidth:{refine:true,init:50},initiallyHiddenColumns:{init:null},selectable:{refine:true,init:false},selectionModel:{check:bV,apply:K,event:y},tableModel:{check:I,apply:bR,event:R},rowHeight:{check:cf,init:20,apply:bW,event:ck,themeable:true},forceLineHeight:{check:g,init:true},headerCellsVisible:{check:g,init:true,apply:V,themeable:true},headerCellHeight:{check:C,init:16,apply:D,event:u,nullable:true,themeable:true},statusBarVisible:{check:g,init:true,apply:a},additionalStatusBarText:{nullable:true,init:null,apply:O},columnVisibilityButtonVisible:{check:g,init:true,apply:Q,themeable:true},metaColumnCounts:{check:W,apply:p},focusCellOnPointerMove:{check:g,init:false,apply:h},rowFocusChangeModifiesSelection:{check:g,init:true},showCellFocusIndicator:{check:g,init:true,apply:X},contextMenuFromDataCellsOnly:{check:g,init:true,apply:cr},keepFirstVisibleRowComplete:{check:g,init:true,apply:bM},alwaysUpdateCells:{check:g,init:false},resetSelectionOnHeaderTap:{check:g,init:true,apply:N},dataRowRenderer:{check:ch,init:null,nullable:true,event:s},modalCellEditorPreOpenFunction:{check:G,init:null,nullable:true},newColumnMenu:{check:G,init:function(){return new qx.ui.table.columnmenu.Button();}},newSelectionManager:{check:G,init:function(cw){return new qx.ui.table.selection.Manager(cw);}},newSelectionModel:{check:G,init:function(cx){return new qx.ui.table.selection.Model(cx);}},newTableColumnModel:{check:G,init:function(cy){return new qx.ui.table.columnmodel.Basic(cy);}},newTablePane:{check:G,init:function(cz){return new qx.ui.table.pane.Pane(cz);}},newTablePaneHeader:{check:G,init:function(cA){return new qx.ui.table.pane.Header(cA);}},newTablePaneScroller:{check:G,init:function(cB){return new qx.ui.table.pane.Scroller(cB);}},newTablePaneModel:{check:G,init:function(cC){return new qx.ui.table.pane.Model(cC);}}},members:{__oF:null,__oG:null,__oD:null,__oE:null,__oI:null,__oJ:null,__oK:null,__oL:null,__oM:null,__oN:null,__oO:null,__de:null,_createChildControlImpl:function(cG,cF){var cD;switch(cG){case F:cD=new qx.ui.basic.Label();cD.set({allowGrowX:true});this._add(cD);break;case J:cD=this.getNewColumnMenu()();cD.set({focusable:false});var cE=cD.factory(H,{table:this});cE.addListener(bY,this._initColumnMenu,this);break;};return cD||qx.ui.core.Widget.prototype._createChildControlImpl.call(this,cG);},_applySelectionModel:function(cI,cH){this.__oE.setSelectionModel(cI);if(cH!=null){cH.removeListener(U,this._onSelectionChanged,this);};cI.addListener(U,this._onSelectionChanged,this);},_applyRowHeight:function(cL,cK){var cJ=this._getPaneScrollerArr();for(var i=0;i<cJ.length;i++ ){cJ[i].updateVerScrollBarMaximum();};},_applyHeaderCellsVisible:function(cO,cN){var cM=this._getPaneScrollerArr();for(var i=0;i<cM.length;i++ ){if(cO){cM[i]._showChildControl(cm);}else {cM[i]._excludeChildControl(cm);};};if(this.getColumnVisibilityButtonVisible()){this._applyColumnVisibilityButtonVisible(cO);};},_applyHeaderCellHeight:function(cR,cQ){var cP=this._getPaneScrollerArr();for(var i=0;i<cP.length;i++ ){cP[i].getHeader().setHeight(cR);};},getEmptyTableModel:function(){if(!this.__oN){this.__oN=new qx.ui.table.model.Simple();this.__oN.setColumns([]);this.__oN.setData([]);};return this.__oN;},_applyTableModel:function(cT,cS){this.getTableColumnModel().init(cT.getColumnCount(),this);if(cS!=null){cS.removeListener(cg,this._onTableModelMetaDataChanged,this);cS.removeListener(t,this._onTableModelDataChanged,this);};cT.addListener(cg,this._onTableModelMetaDataChanged,this);cT.addListener(t,this._onTableModelDataChanged,this);this._updateStatusBar();this._updateTableData(0,cT.getRowCount(),0,cT.getColumnCount());this._onTableModelMetaDataChanged();if(cS&&cT.init&&typeof (cT.init)==c){cT.init(this);};},getTableColumnModel:function(){if(!this.__oM){var cX=this.__oM=this.getNewTableColumnModel()(this);cX.addListener(cs,this._onColVisibilityChanged,this);cX.addListener(bN,this._onColWidthChanged,this);cX.addListener(cj,this._onColOrderChanged,this);var cV=this.getTableModel();cX.init(cV.getColumnCount(),this);var cU=this._getPaneScrollerArr();for(var i=0;i<cU.length;i++ ){var cY=cU[i];var cW=cY.getTablePaneModel();cW.setTableColumnModel(cX);};};return this.__oM;},_applyStatusBarVisible:function(db,da){if(db){this._showChildControl(F);}else {this._excludeChildControl(F);};if(db){this._updateStatusBar();};},_applyAdditionalStatusBarText:function(dd,dc){this.__oI=dd;this._updateStatusBar();},_applyColumnVisibilityButtonVisible:function(df,de){if(df){this._showChildControl(J);}else {this._excludeChildControl(J);};},_applyMetaColumnCounts:function(dp,dj){var dn=dp;var dg=this._getPaneScrollerArr();var dl={};if(dp>dj){var dr=qx.event.Registration.getManager(dg[0]);for(var di in qx.ui.table.Table.__oH){dl[di]={};dl[di].capture=dr.getListeners(dg[0],di,true);dl[di].bubble=dr.getListeners(dg[0],di,false);};};this._cleanUpMetaColumns(dn.length);var dm=0;for(var i=0;i<dg.length;i++ ){var dt=dg[i];var dq=dt.getTablePaneModel();dq.setFirstColumnX(dm);dq.setMaxColumnCount(dn[i]);dm+=dn[i];};if(dn.length>dg.length){var ds=this.getTableColumnModel();for(var i=dg.length;i<dn.length;i++ ){var dq=this.getNewTablePaneModel()(ds);dq.setFirstColumnX(dm);dq.setMaxColumnCount(dn[i]);dm+=dn[i];var dt=this.getNewTablePaneScroller()(this);dt.setTablePaneModel(dq);dt.addListener(bQ,this._onScrollY,this);for(di in qx.ui.table.Table.__oH){if(!dl[di]){break;};if(dl[di].capture&&dl[di].capture.length>0){var dh=dl[di].capture;for(var j=0;j<dh.length;j++ ){var dk=dh[j].context;if(!dk){dk=this;}else if(dk==dg[0]){dk=dt;};dt.addListener(di,dh[j].handler,dk,true);};};if(dl[di].bubble&&dl[di].bubble.length>0){var dv=dl[di].bubble;for(var j=0;j<dv.length;j++ ){var dk=dv[j].context;if(!dk){dk=this;}else if(dk==dg[0]){dk=dt;};dt.addListener(di,dv[j].handler,dk,false);};};};var du=(i==dn.length-1)?1:0;this.__oD.add(dt,{flex:du});dg=this._getPaneScrollerArr();};};for(var i=0;i<dg.length;i++ ){var dt=dg[i];var dw=(i==(dg.length-1));dt.getHeader().setHeight(this.getHeaderCellHeight());dt.setTopRightWidget(dw?this.getChildControl(J):null);};if(!this.isColumnVisibilityButtonVisible()){this._excludeChildControl(J);};this._updateScrollerWidths();this._updateScrollBarVisibility();},_applyFocusCellOnPointerMove:function(dz,dy){var dx=this._getPaneScrollerArr();for(var i=0;i<dx.length;i++ ){dx[i].setFocusCellOnPointerMove(dz);};},_applyShowCellFocusIndicator:function(dC,dB){var dA=this._getPaneScrollerArr();for(var i=0;i<dA.length;i++ ){dA[i].setShowCellFocusIndicator(dC);};},_applyContextMenuFromDataCellsOnly:function(dF,dE){var dD=this._getPaneScrollerArr();for(var i=0;i<dD.length;i++ ){dD[i].setContextMenuFromDataCellsOnly(dF);};},_applyKeepFirstVisibleRowComplete:function(dI,dH){var dG=this._getPaneScrollerArr();for(var i=0;i<dG.length;i++ ){dG[i].onKeepFirstVisibleRowCompleteChanged();};},_applyResetSelectionOnHeaderTap:function(dL,dK){var dJ=this._getPaneScrollerArr();for(var i=0;i<dJ.length;i++ ){dJ[i].setResetSelectionOnHeaderTap(dL);};},getSelectionManager:function(){return this.__oE;},_getPaneScrollerArr:function(){return this.__oD.getChildren();},getPaneScroller:function(dM){return this._getPaneScrollerArr()[dM];},_cleanUpMetaColumns:function(dO){var dN=this._getPaneScrollerArr();if(dN!=null){for(var i=dN.length-1;i>=dO;i-- ){dN[i].destroy();};};},_onChangeLocale:function(dP){this.updateContent();this._updateStatusBar();},_onChangeTheme:function(){qx.ui.core.Widget.prototype._onChangeTheme.call(this);this.getDataRowRenderer().initThemeValues();this.updateContent();this._updateStatusBar();},_onSelectionChanged:function(dR){var dQ=this._getPaneScrollerArr();for(var i=0;i<dQ.length;i++ ){dQ[i].onSelectionChanged();};this._updateStatusBar();},_onTableModelMetaDataChanged:function(dT){var dS=this._getPaneScrollerArr();for(var i=0;i<dS.length;i++ ){dS[i].onTableModelMetaDataChanged();};this._updateStatusBar();},_onTableModelDataChanged:function(dU){var dV=dU.getData();this._updateTableData(dV.firstRow,dV.lastRow,dV.firstColumn,dV.lastColumn,dV.removeStart,dV.removeCount);},_onContextMenuOpen:function(e){},_updateTableData:function(eb,ee,dY,dX,ea,ed){var dW=this._getPaneScrollerArr();if(ed){this.getSelectionModel().removeSelectionInterval(ea,ea+ed);if(this.__oG>=ea&&this.__oG<(ea+ed)){this.setFocusedCell();};};for(var i=0;i<dW.length;i++ ){dW[i].onTableModelDataChanged(eb,ee,dY,dX);};var ec=this.getTableModel().getRowCount();if(ec!=this.__oJ){this.__oJ=ec;this._updateScrollBarVisibility();this._updateStatusBar();};},_onScrollY:function(eg){if(!this.__oK){this.__oK=true;var ef=this._getPaneScrollerArr();for(var i=0;i<ef.length;i++ ){ef[i].setScrollY(eg.getData());};this.__oK=false;};},_onKeyPress:function(ep){if(!this.getEnabled()){return;};var en=this.__oG;var ek=true;var eo=ep.getKeyIdentifier();if(this.isEditing()){if(ep.getModifiers()==0){switch(eo){case ce:this.stopEditing();var en=this.__oG;this.moveFocusedCell(0,1);if(this.__oG!=en){ek=this.startEditing();};break;case v:this.cancelEditing();this.focus();break;default:ek=false;break;};};}else {if(ep.isCtrlPressed()){ek=true;switch(eo){case w:var el=this.getTableModel().getRowCount();if(el>0){this.getSelectionModel().setSelectionInterval(0,el-1);};break;default:ek=false;break;};}else {switch(eo){case co:this.__oE.handleSelectKeyDown(this.__oG,ep);break;case bX:case ce:this.startEditing();ek=true;break;case bP:this.setFocusedCell(this.__oF,0,true);break;case bS:var el=this.getTableModel().getRowCount();this.setFocusedCell(this.__oF,el-1,true);break;case A:this.moveFocusedCell(-1,0);break;case ci:this.moveFocusedCell(1,0);break;case cb:this.moveFocusedCell(0,-1);break;case B:this.moveFocusedCell(0,1);break;case q:case bL:var ej=this.getPaneScroller(0);var em=ej.getTablePane();var ei=this.getRowHeight();var eh=(eo==q)?-1:1;el=em.getVisibleRowCount()-1;ej.setScrollY(ej.getScrollY()+eh*el*ei);this.moveFocusedCell(0,eh*el);break;default:ek=false;};};};if(en!=this.__oG&&this.getRowFocusChangeModifiesSelection()){this.__oE.handleMoveKeyDown(this.__oG,ep);};if(ek){ep.preventDefault();ep.stopPropagation();};},_onFocusChanged:function(er){var eq=this._getPaneScrollerArr();for(var i=0;i<eq.length;i++ ){eq[i].onFocusChanged();};},_onColVisibilityChanged:function(et){var es=this._getPaneScrollerArr();for(var i=0;i<es.length;i++ ){es[i].onColVisibilityChanged();};var eu=et.getData();if(this.__oL!=null&&eu.col!=null&&eu.visible!=null){this.__oL[eu.col].setVisible(eu.visible);};this._updateScrollerWidths();this._updateScrollBarVisibility();},_onColWidthChanged:function(ew){var ev=this._getPaneScrollerArr();for(var i=0;i<ev.length;i++ ){var ex=ew.getData();ev[i].setColumnWidth(ex.col,ex.newWidth);};this._updateScrollerWidths();this._updateScrollBarVisibility();},_onColOrderChanged:function(ez){var ey=this._getPaneScrollerArr();for(var i=0;i<ey.length;i++ ){ey[i].onColOrderChanged();};this._updateScrollerWidths();this._updateScrollBarVisibility();},getTablePaneScrollerAtPageX:function(eB){var eA=this._getMetaColumnAtPageX(eB);return (eA!=-1)?this.getPaneScroller(eA):null;},setFocusedCell:function(eE,eF,eD){if(!this.isEditing()&&(eE!=this.__oF||eF!=this.__oG)){if(eE===null){eE=0;};this.__oF=eE;this.__oG=eF;var eC=this._getPaneScrollerArr();for(var i=0;i<eC.length;i++ ){eC[i].setFocusedCell(eE,eF);};if(eE!=null&&eD){this.scrollCellVisible(eE,eF);};};},resetSelection:function(){this.getSelectionModel().resetSelection();},resetCellFocus:function(){this.setFocusedCell(null,null,false);},getFocusedColumn:function(){return this.__oF;},getFocusedRow:function(){return this.__oG;},highlightFocusedRow:function(eG){this.getDataRowRenderer().setHighlightFocusRow(eG);},clearFocusedRowHighlight:function(eI){if(eI){var eJ=eI.getRelatedTarget();if(eJ instanceof qx.ui.table.pane.Pane||eJ instanceof qx.ui.table.pane.FocusIndicator){return;};};this.resetCellFocus();var eH=this._getPaneScrollerArr();for(var i=0;i<eH.length;i++ ){eH[i].onFocusChanged();};},moveFocusedCell:function(eK,eL){var eP=this.__oF;var eQ=this.__oG;if(eP==null||eQ==null){return;};if(eK!=0){var eO=this.getTableColumnModel();var x=eO.getVisibleX(eP);var eN=eO.getVisibleColumnCount();x=qx.lang.Number.limit(x+eK,0,eN-1);eP=eO.getVisibleColumnAtX(x);};if(eL!=0){var eM=this.getTableModel();eQ=qx.lang.Number.limit(eQ+eL,0,eM.getRowCount()-1);};this.setFocusedCell(eP,eQ,true);},scrollCellVisible:function(eU,eV){var eS=this.getContentElement().getDomElement();if(!eS){this.addListenerOnce(bY,function(){this.scrollCellVisible(eU,eV);},this);};var eT=this.getTableColumnModel();var x=eT.getVisibleX(eU);var eR=this._getMetaColumnAtColumnX(x);if(eR!=-1){this.getPaneScroller(eR).scrollCellVisible(eU,eV);};},isEditing:function(){if(this.__oF!=null){var x=this.getTableColumnModel().getVisibleX(this.__oF);var eW=this._getMetaColumnAtColumnX(x);return this.getPaneScroller(eW).isEditing();};return false;},startEditing:function(){if(this.__oF!=null){var x=this.getTableColumnModel().getVisibleX(this.__oF);var eY=this._getMetaColumnAtColumnX(x);var eX=this.getPaneScroller(eY).startEditing();return eX;};return false;},stopEditing:function(){if(this.__oF!=null){var x=this.getTableColumnModel().getVisibleX(this.__oF);var fa=this._getMetaColumnAtColumnX(x);this.getPaneScroller(fa).stopEditing();};},cancelEditing:function(){if(this.__oF!=null){var x=this.getTableColumnModel().getVisibleX(this.__oF);var fb=this._getMetaColumnAtColumnX(x);this.getPaneScroller(fb).cancelEditing();};},updateContent:function(){var fc=this._getPaneScrollerArr();for(var i=0;i<fc.length;i++ ){fc[i].getTablePane().updateContent(true);};},blockHeaderElements:function(){var fd=this._getPaneScrollerArr();for(var i=0;i<fd.length;i++ ){fd[i].getHeader().getBlocker().blockContent(20);};this.getChildControl(J).getBlocker().blockContent(20);},unblockHeaderElements:function(){var fe=this._getPaneScrollerArr();for(var i=0;i<fe.length;i++ ){fe[i].getHeader().getBlocker().unblock();};this.getChildControl(J).getBlocker().unblock();},_getMetaColumnAtPageX:function(fh){var ff=this._getPaneScrollerArr();for(var i=0;i<ff.length;i++ ){var fg=ff[i].getContentLocation();if(fh>=fg.left&&fh<=fg.right){return i;};};return -1;},_getMetaColumnAtColumnX:function(fj){var fk=this.getMetaColumnCounts();var fl=0;for(var i=0;i<fk.length;i++ ){var fi=fk[i];fl+=fi;if(fi==-1||fj<fl){return i;};};return -1;},_updateStatusBar:function(){var fm=this.getTableModel();if(this.getStatusBarVisible()){var fn=this.getSelectionModel().getSelectedCount();var fo=fm.getRowCount();var fp;if(fo>=0){if(fn==0){fp=this.trn(cq,bU,fo,fo);}else {fp=this.trn(bO,cc,fo,fn,fo);};};if(this.__oI){if(fp){fp+=this.__oI;}else {fp=this.__oI;};};if(fp){this.getChildControl(F).setValue(fp);};};},_updateScrollerWidths:function(){var fq=this._getPaneScrollerArr();for(var i=0;i<fq.length;i++ ){var fs=(i==(fq.length-1));var ft=fq[i].getTablePaneModel().getTotalWidth();fq[i].setPaneWidth(ft);var fr=fs?1:0;fq[i].setLayoutProperties({flex:fr});};},_updateScrollBarVisibility:function(){if(!this.getBounds()){return;};var fx=qx.ui.table.pane.Scroller.HORIZONTAL_SCROLLBAR;var fz=qx.ui.table.pane.Scroller.VERTICAL_SCROLLBAR;var fu=this._getPaneScrollerArr();var fw=false;var fy=false;for(var i=0;i<fu.length;i++ ){var fA=(i==(fu.length-1));var fv=fu[i].getNeededScrollBars(fw,!fA);if(fv&fx){fw=true;};if(fA&&(fv&fz)){fy=true;};};for(var i=0;i<fu.length;i++ ){var fA=(i==(fu.length-1));fu[i].setHorizontalScrollBarVisible(fw);if(fA){if(this.__oO==null){this.__oO=fu[i].getVerticalScrollBarVisible();this.__de=qx.event.Timer.once(function(){this.__oO=null;this.__de=null;},this,0);};};fu[i].setVerticalScrollBarVisible(fA&&fy);if(fA&&fy!=this.__oO){this.fireDataEvent(o,fy);};};},_initColumnMenu:function(){var fD=this.getTableModel();var fE=this.getTableColumnModel();var fF=this.getChildControl(J);fF.empty();var fC=fF.getMenu();var fG={table:this,menu:fC,columnButton:fF};this.fireDataEvent(b,fG);this.__oL={};for(var fH=0,l=fD.getColumnCount();fH<l;fH++ ){var fB=fF.factory(M,{text:fD.getColumnName(fH),column:fH,bVisible:fE.isColumnVisible(fH)});qx.core.Assert.assertInterface(fB,qx.ui.table.IColumnMenuItem);fB.addListener(n,this._createColumnVisibilityCheckBoxHandler(fH),this);this.__oL[fH]=fB;};fG={table:this,menu:fC,columnButton:fF};this.fireDataEvent(k,fG);},_createColumnVisibilityCheckBoxHandler:function(fI){return function(fJ){var fK=this.getTableColumnModel();fK.setColumnVisible(fI,fJ.getData());};},setColumnWidth:function(fM,fL){this.getTableColumnModel().setColumnWidth(fM,fL);},_onResize:function(){this.fireEvent(T);this._updateScrollerWidths();this._updateScrollBarVisibility();},addListener:function(fQ,fP,self,fN){if(this.self(arguments).__oH[fQ]){var fR=[fQ];for(var i=0,fO=this._getPaneScrollerArr();i<fO.length;i++ ){fR.push(fO[i].addListener.apply(fO[i],arguments));};return fR.join(bJ);}else {return qx.ui.core.Widget.prototype.addListener.call(this,fQ,fP,self,fN);};},removeListener:function(fV,fU,self,fS){if(this.self(arguments).__oH[fV]){for(var i=0,fT=this._getPaneScrollerArr();i<fT.length;i++ ){fT[i].removeListener.apply(fT[i],arguments);};}else {qx.ui.core.Widget.prototype.removeListener.call(this,fV,fU,self,fS);};},removeListenerById:function(gb){var fX=gb.split(bJ);var ga=fX.shift();if(this.self(arguments).__oH[ga]){var fY=true;for(var i=0,fW=this._getPaneScrollerArr();i<fW.length;i++ ){fY=fW[i].removeListenerById.call(fW[i],fX[i])&&fY;};return fY;}else {return qx.ui.core.Widget.prototype.removeListenerById.call(this,gb);};},destroy:function(){this.getChildControl(J).getMenu().destroy();qx.ui.core.Widget.prototype.destroy.call(this);}},destruct:function(){if(qx.core.Environment.get(bT)){qx.locale.Manager.getInstance().removeListener(L,this._onChangeLocale,this);};var gd=this.getSelectionModel();if(gd){gd.dispose();};var gc=this.getDataRowRenderer();if(gc){gc.dispose();};this._cleanUpMetaColumns(0);this.getTableColumnModel().dispose();this._disposeObjects(cl,cp,cd,cd,m,z);this._disposeMap(bK);}});})();(function(){var a="qx.ui.table.IRowRenderer";qx.Interface.define(a,{members:{updateDataRowElement:function(c,b){},getRowHeightStyle:function(d){},createRowStyle:function(e){},getRowClass:function(f){}}});})();(function(){var a="table-row-background-even",b="Boolean",c="content",d="default",e="height:",f="changeTheme",g="'",h="table-row",i="table-row-background-focused",j="css.boxmodel",k=';color:',l="table-row-background-odd",m="1px solid ",n="table-row-line",o="qx.dyntheme",p="table-row-background-selected",q="background-color:",r="",s=';border-bottom: 1px solid ',t="table-row-selected",u="table-row-background-focused-selected",v="px;",w="qx.ui.table.rowrenderer.Default",x=";";qx.Class.define(w,{extend:qx.core.Object,implement:qx.ui.table.IRowRenderer,construct:function(){qx.core.Object.call(this);this.initThemeValues();if(qx.core.Environment.get(o)){qx.theme.manager.Meta.getInstance().addListener(f,this.initThemeValues,this);};},properties:{highlightFocusRow:{check:b,init:true}},members:{_colors:null,__oP:null,__oQ:null,initThemeValues:function(){this.__oQ=r;this.__oP={};this._colors={};this._renderFont(qx.theme.manager.Font.getInstance().resolve(d));var y=qx.theme.manager.Color.getInstance();this._colors.bgcolFocusedSelected=y.resolve(u);this._colors.bgcolFocused=y.resolve(i);this._colors.bgcolSelected=y.resolve(p);this._colors.bgcolEven=y.resolve(a);this._colors.bgcolOdd=y.resolve(l);this._colors.colSelected=y.resolve(t);this._colors.colNormal=y.resolve(h);this._colors.horLine=y.resolve(n);},_insetY:1,_renderFont:function(z){if(z){this.__oP=z.getStyles();this.__oQ=qx.bom.element.Style.compile(this.__oP);this.__oQ=this.__oQ.replace(/"/g,g);}else {this.__oQ=r;this.__oP=qx.bom.Font.getDefaultStyles();};},updateDataRowElement:function(D,A){var B=this.__oP;var C=A.style;qx.bom.element.Style.setStyles(A,B);if(D.focusedRow&&this.getHighlightFocusRow()){C.backgroundColor=D.selected?this._colors.bgcolFocusedSelected:this._colors.bgcolFocused;}else {if(D.selected){C.backgroundColor=this._colors.bgcolSelected;}else {C.backgroundColor=(D.row%2==0)?this._colors.bgcolEven:this._colors.bgcolOdd;};};C.color=D.selected?this._colors.colSelected:this._colors.colNormal;C.borderBottom=m+this._colors.horLine;},getRowHeightStyle:function(E){if(qx.core.Environment.get(j)==c){E-=this._insetY;};return e+E+v;},createRowStyle:function(G){var F=[];F.push(x);F.push(this.__oQ);F.push(q);if(G.focusedRow&&this.getHighlightFocusRow()){F.push(G.selected?this._colors.bgcolFocusedSelected:this._colors.bgcolFocused);}else {if(G.selected){F.push(this._colors.bgcolSelected);}else {F.push((G.row%2==0)?this._colors.bgcolEven:this._colors.bgcolOdd);};};F.push(k);F.push(G.selected?this._colors.colSelected:this._colors.colNormal);F.push(s,this._colors.horLine);return F.join(r);},getRowClass:function(H){return r;},getRowAttributes:function(I){return r;}},destruct:function(){this._colors=this.__oP=this.__oQ=null;if(qx.core.Environment.get(o)){qx.theme.manager.Meta.getInstance().removeListener(f,this.initThemeValues,this);};}});})();(function(){var a="qx.ui.table.IColumnMenuButton";qx.Interface.define(a,{properties:{menu:{}},members:{factory:function(b,c){return true;},empty:function(){return true;}}});})();(function(){var a="qx.ui.menu.Menu",b="submenu",c="hovered",d="Enter",f="pressed",g="abandoned",h="contextmenu",i="changeVisibility",j="changeMenu",k="qx.ui.form.MenuButton",l="menubutton",m="visible",n="left",o="_applyMenu";qx.Class.define(k,{extend:qx.ui.form.Button,construct:function(q,r,p){qx.ui.form.Button.call(this,q,r);if(p!=null){this.setMenu(p);};},properties:{menu:{check:a,nullable:true,apply:o,event:j},appearance:{refine:true,init:l}},members:{_applyVisibility:function(u,t){qx.ui.form.Button.prototype._applyVisibility.call(this,u,t);var s=this.getMenu();if(u!=m&&s){s.hide();};},_applyMenu:function(w,v){if(v){v.removeListener(i,this._onMenuChange,this);v.resetOpener();};if(w){w.addListener(i,this._onMenuChange,this);w.setOpener(this);w.removeState(b);w.removeState(h);};},open:function(x){var y=this.getMenu();if(y){qx.ui.menu.Manager.getInstance().hideAll();y.setOpener(this);y.open();if(x){var z=y.getSelectables()[0];if(z){y.setSelectedButton(z);};};};},_onMenuChange:function(e){var A=this.getMenu();if(A.isVisible()){this.addState(f);}else {this.removeState(f);};},_onPointerDown:function(e){qx.ui.form.Button.prototype._onPointerDown.call(this,e);if(e.getButton()!=n){return;};var B=this.getMenu();if(B){if(!B.isVisible()){this.open();}else {B.exclude();};e.stopPropagation();};},_onPointerUp:function(e){qx.ui.form.Button.prototype._onPointerUp.call(this,e);e.stopPropagation();},_onPointerOver:function(e){this.addState(c);},_onPointerOut:function(e){this.removeState(c);},_onKeyDown:function(e){switch(e.getKeyIdentifier()){case d:this.removeState(g);this.addState(f);var C=this.getMenu();if(C){if(!C.isVisible()){this.open();}else {C.exclude();};};e.stopPropagation();};},_onKeyUp:function(e){}}});})();(function(){var a="blur",b="keypress",c="Enter",d="Up",f="__lq",g="contextmenu",h="interval",j="__oR",k="roll",l="__oS",m="Escape",n="pointerdown",o="qx.ui.menu.Manager",p="keydown",q="Left",r="event.touch",s="Down",t="Right",u="keyup",v="singleton",w="Space";qx.Class.define(o,{type:v,extend:qx.core.Object,construct:function(){qx.core.Object.call(this);this.__lq=[];var x=document.body;var y=qx.event.Registration;y.addListener(window.document.documentElement,n,this._onPointerDown,this,true);y.addListener(x,k,this._onRoll,this,true);y.addListener(x,p,this._onKeyUpDown,this,true);y.addListener(x,u,this._onKeyUpDown,this,true);y.addListener(x,b,this._onKeyPress,this,true);if(!qx.core.Environment.get(r)){qx.bom.Element.addListener(window,a,this.hideAll,this);};this.__oR=new qx.event.Timer();this.__oR.addListener(h,this._onOpenInterval,this);this.__oS=new qx.event.Timer();this.__oS.addListener(h,this._onCloseInterval,this);},members:{__oT:null,__oU:null,__oR:null,__oS:null,__lq:null,_getChild:function(z,B,A,E){var D=z.getChildren();var length=D.length;var C;for(var i=B;i<length&&i>=0;i+=A){C=D[i];if(C.isEnabled()&&!C.isAnonymous()&&C.isVisible()){return C;};};if(E){i=i==length?0:length-1;for(;i!=B;i+=A){C=D[i];if(C.isEnabled()&&!C.isAnonymous()&&C.isVisible()){return C;};};};return null;},_isInMenu:function(F){while(F){if(F instanceof qx.ui.menu.Menu){return true;};F=F.getLayoutParent();};return false;},_isMenuOpener:function(H){var G=this.__lq;for(var i=0;i<G.length;i++ ){if(G[i].getOpener()===H){return true;};};return false;},_getMenuButton:function(I){while(I){if(I instanceof qx.ui.menu.AbstractButton){return I;};I=I.getLayoutParent();};return null;},add:function(J){{};var K=this.__lq;K.push(J);J.setZIndex(1e6+K.length);},remove:function(L){{};var M=this.__lq;if(M){qx.lang.Array.remove(M,L);};},hideAll:function(){var N=this.__lq;if(N){for(var i=N.length-1;i>=0;i-- ){N[i].exclude();};};},getActiveMenu:function(){var O=this.__lq;return O.length>0?O[O.length-1]:null;},scheduleOpen:function(P){this.cancelClose(P);if(P.isVisible()){if(this.__oT){this.cancelOpen(this.__oT);};}else if(this.__oT!=P){this.__oT=P;this.__oR.restartWith(P.getOpenInterval());};},scheduleClose:function(Q){this.cancelOpen(Q);if(!Q.isVisible()){if(this.__oU){this.cancelClose(this.__oU);};}else if(this.__oU!=Q){this.__oU=Q;this.__oS.restartWith(Q.getCloseInterval());};},cancelOpen:function(R){if(this.__oT==R){this.__oR.stop();this.__oT=null;};},cancelClose:function(S){if(this.__oU==S){this.__oS.stop();this.__oU=null;};},_onOpenInterval:function(e){this.__oR.stop();this.__oT.open();this.__oT=null;},_onCloseInterval:function(e){this.__oS.stop();this.__oU.exclude();this.__oU=null;},preventContextMenuOnce:function(){qx.event.Registration.addListener(document.body,g,this.__oV,this,true);},__oV:function(e){var T=e.getTarget();T=qx.ui.core.Widget.getWidgetByElement(T,true);if(this._isInMenu(T)){e.stopPropagation();e.preventDefault();};qx.event.Registration.removeListener(document.body,g,this.__oV,this,true);},_onPointerDown:function(e){var U=e.getTarget();U=qx.ui.core.Widget.getWidgetByElement(U,true);if(U==null){this.hideAll();return;};if(U.getMenu&&U.getMenu()&&U.getMenu().isVisible()){return;};if(this.__lq.length>0&&!this._isInMenu(U)){this.hideAll();};},__oW:{"Enter":1,"Space":1},__oX:{"Escape":1,"Up":1,"Down":1,"Left":1,"Right":1},_onKeyUpDown:function(e){var V=this.getActiveMenu();if(!V){return;};var W=e.getKeyIdentifier();if(this.__oX[W]||(this.__oW[W]&&V.getSelectedButton())){e.stopPropagation();};},_onKeyPress:function(e){var ba=this.getActiveMenu();if(!ba){return;};var X=e.getKeyIdentifier();var bc=this.__oX[X];var Y=this.__oW[X];if(bc){switch(X){case d:this._onKeyPressUp(ba);break;case s:this._onKeyPressDown(ba);break;case q:this._onKeyPressLeft(ba);break;case t:this._onKeyPressRight(ba);break;case m:this.hideAll();break;};e.stopPropagation();e.preventDefault();}else if(Y){var bb=ba.getSelectedButton();if(bb){switch(X){case c:this._onKeyPressEnter(ba,bb,e);break;case w:this._onKeyPressSpace(ba,bb,e);break;};e.stopPropagation();e.preventDefault();};};},_onKeyPressUp:function(bh){var bd=bh.getSelectedButton();var be=bh.getChildren();var bf=bd?bh.indexOf(bd)-1:be.length-1;var bg=this._getChild(bh,bf,-1,true);if(bg){bh.setSelectedButton(bg);}else {bh.resetSelectedButton();};},_onKeyPressDown:function(bi){var bj=bi.getSelectedButton();var bl=bj?bi.indexOf(bj)+1:0;var bk=this._getChild(bi,bl,1,true);if(bk){bi.setSelectedButton(bk);}else {bi.resetSelectedButton();};},_onKeyPressLeft:function(bo){var bq=bo.getOpener();if(!bq){return;};if(bq instanceof qx.ui.menu.AbstractButton){var bn=bq.getLayoutParent();bn.resetOpenedButton();bn.setSelectedButton(bq);}else if(bq instanceof qx.ui.menubar.Button){var br=bq.getMenuBar().getMenuButtons();var bm=br.indexOf(bq);if(bm===-1){return;};var bs=null;var length=br.length;for(var i=1;i<=length;i++ ){var bp=br[(bm-i+length)%length];if(bp.isEnabled()&&bp.isVisible()){bs=bp;break;};};if(bs&&bs!=bq){bs.open(true);};};},_onKeyPressRight:function(bw){var bu=bw.getSelectedButton();if(bu){var bt=bu.getMenu();if(bt){bw.setOpenedButton(bu);var bB=this._getChild(bt,0,1);if(bB){bt.setSelectedButton(bB);};return;};}else if(!bw.getOpenedButton()){var bB=this._getChild(bw,0,1);if(bB){bw.setSelectedButton(bB);if(bB.getMenu()){bw.setOpenedButton(bB);};return;};};var by=bw.getOpener();if(by instanceof qx.ui.menu.Button&&bu){while(by){by=by.getLayoutParent();if(by instanceof qx.ui.menu.Menu){by=by.getOpener();if(by instanceof qx.ui.menubar.Button){break;};}else {break;};};if(!by){return;};};if(by instanceof qx.ui.menubar.Button){var bz=by.getMenuBar().getMenuButtons();var bv=bz.indexOf(by);if(bv===-1){return;};var bA=null;var length=bz.length;for(var i=1;i<=length;i++ ){var bx=bz[(bv+i)%length];if(bx.isEnabled()&&bx.isVisible()){bA=bx;break;};};if(bA&&bA!=by){bA.open(true);};};},_onKeyPressEnter:function(bC,bD,e){if(bD.hasListener(b)){var bE=e.clone();bE.setBubbles(false);bE.setTarget(bD);bD.dispatchEvent(bE);};this.hideAll();},_onKeyPressSpace:function(bF,bG,e){if(bG.hasListener(b)){var bH=e.clone();bH.setBubbles(false);bH.setTarget(bG);bG.dispatchEvent(bH);};},_onRoll:function(e){var bI=e.getTarget();bI=qx.ui.core.Widget.getWidgetByElement(bI,true);if(this.__lq.length>0&&!this._isInMenu(bI)&&!this._isMenuOpener(bI)&&!e.getMomentum()){this.hideAll();};}},destruct:function(){var bK=qx.event.Registration;var bJ=document.body;bK.removeListener(window.document.documentElement,n,this._onPointerDown,this,true);bK.removeListener(bJ,p,this._onKeyUpDown,this,true);bK.removeListener(bJ,u,this._onKeyUpDown,this,true);bK.removeListener(bJ,b,this._onKeyPress,this,true);this._disposeObjects(j,l);this._disposeArray(f);}});})();(function(){var a="indexOf",b="addAfter",c="add",d="addBefore",e="_",f="addAt",g="hasChildren",h="removeAt",i="removeAll",j="getChildren",k="remove",l="qx.ui.core.MRemoteChildrenHandling";qx.Mixin.define(l,{members:{__oY:function(q,m,o,n){var p=this.getChildrenContainer();if(p===this){q=e+q;};return (p[q])(m,o,n);},getChildren:function(){return this.__oY(j);},hasChildren:function(){return this.__oY(g);},add:function(s,r){return this.__oY(c,s,r);},remove:function(t){return this.__oY(k,t);},removeAll:function(){return this.__oY(i);},indexOf:function(u){return this.__oY(a,u);},addAt:function(x,v,w){this.__oY(f,x,v,w);},addBefore:function(y,A,z){this.__oY(d,y,A,z);},addAfter:function(D,B,C){this.__oY(b,D,B,C);},removeAt:function(E){return this.__oY(h,E);}}});})();(function(){var a="Boolean",b="pointerover",c="qx.ui.core.Widget",d="excluded",f="selected",g="Integer",h="slidebar",j="menu",k="visible",l="_applySelectedButton",m="_applySpacingY",n="resize",o="_blocker",p="_applyCloseInterval",q="_applyBlockerColor",r="The menu instance needs a configured 'opener' widget!",s="_applyIconColumnWidth",t="qx.ui.menu.Menu",u="_applyOpenedButton",v="Color",w="Number",x="_applyArrowColumnWidth",y="Unknown target: ",z="_applySpacingX",A="_applyBlockerOpacity",B="Could not open menu instance because 'opener' widget is not visible",C="_applyOpenInterval",D="pointerout";qx.Class.define(t,{extend:qx.ui.core.Widget,include:[qx.ui.core.MPlacement,qx.ui.core.MRemoteChildrenHandling],construct:function(){qx.ui.core.Widget.call(this);this._setLayout(new qx.ui.menu.Layout);var E=this.getApplicationRoot();E.add(this);this.addListener(b,this._onPointerOver);this.addListener(D,this._onPointerOut);this.addListener(n,this._onResize,this);E.addListener(n,this._onResize,this);this._blocker=new qx.ui.core.Blocker(E);this.initVisibility();this.initKeepFocus();this.initKeepActive();},properties:{appearance:{refine:true,init:j},allowGrowX:{refine:true,init:false},allowGrowY:{refine:true,init:false},visibility:{refine:true,init:d},keepFocus:{refine:true,init:true},keepActive:{refine:true,init:true},spacingX:{check:g,apply:z,init:0,themeable:true},spacingY:{check:g,apply:m,init:0,themeable:true},iconColumnWidth:{check:g,init:0,themeable:true,apply:s},arrowColumnWidth:{check:g,init:0,themeable:true,apply:x},blockerColor:{check:v,init:null,nullable:true,apply:q,themeable:true},blockerOpacity:{check:w,init:1,apply:A,themeable:true},selectedButton:{check:c,nullable:true,apply:l},openedButton:{check:c,nullable:true,apply:u},opener:{check:c,nullable:true},openInterval:{check:g,themeable:true,init:250,apply:C},closeInterval:{check:g,themeable:true,init:250,apply:p},blockBackground:{check:a,themeable:true,init:false}},members:{__pa:null,__pb:null,_blocker:null,open:function(){if(this.getOpener()!=null){var F=this.placeToWidget(this.getOpener(),true);if(F){this.__pd();this.show();this._placementTarget=this.getOpener();}else {this.warn(B);};}else {this.warn(r);};},openAtPointer:function(e){this.placeToPointer(e);this.__pd();this.show();this._placementTarget={left:e.getDocumentLeft(),top:e.getDocumentTop()};},openAtPoint:function(G){this.placeToPoint(G);this.__pd();this.show();this._placementTarget=G;},addSeparator:function(){this.add(new qx.ui.menu.Separator);},getColumnSizes:function(){return this._getMenuLayout().getColumnSizes();},getSelectables:function(){var H=[];var I=this.getChildren();for(var i=0;i<I.length;i++ ){if(I[i].isEnabled()){H.push(I[i]);};};return H;},_applyIconColumnWidth:function(K,J){this._getMenuLayout().setIconColumnWidth(K);},_applyArrowColumnWidth:function(M,L){this._getMenuLayout().setArrowColumnWidth(M);},_applySpacingX:function(O,N){this._getMenuLayout().setColumnSpacing(O);},_applySpacingY:function(Q,P){this._getMenuLayout().setSpacing(Q);},_applyVisibility:function(T,S){qx.ui.core.Widget.prototype._applyVisibility.call(this,T,S);var R=qx.ui.menu.Manager.getInstance();if(T===k){R.add(this);var U=this.getParentMenu();if(U){U.setOpenedButton(this.getOpener());};}else if(S===k){R.remove(this);var U=this.getParentMenu();if(U&&U.getOpenedButton()==this.getOpener()){U.resetOpenedButton();};this.resetOpenedButton();this.resetSelectedButton();};this.__pc();},__pc:function(){if(this.isVisible()){if(this.getBlockBackground()){var V=this.getZIndex();this._blocker.blockContent(V-1);};}else {if(this._blocker.isBlocked()){this._blocker.unblock();};};},getParentMenu:function(){var W=this.getOpener();if(!W||!(W instanceof qx.ui.menu.AbstractButton)){return null;};if(W&&W.getContextMenu()===this){return null;};while(W&&!(W instanceof qx.ui.menu.Menu)){W=W.getLayoutParent();};return W;},_applySelectedButton:function(Y,X){if(X){X.removeState(f);};if(Y){Y.addState(f);};},_applyOpenedButton:function(bb,ba){if(ba&&ba.getMenu()){ba.getMenu().exclude();};if(bb){bb.getMenu().open();};},_applyBlockerColor:function(bd,bc){this._blocker.setColor(bd);},_applyBlockerOpacity:function(bf,be){this._blocker.setOpacity(bf);},getChildrenContainer:function(){return this.getChildControl(h,true)||this;},_createChildControlImpl:function(bi,bh){var bg;switch(bi){case h:var bg=new qx.ui.menu.MenuSlideBar();var bk=this._getLayout();this._setLayout(new qx.ui.layout.Grow());var bj=bg.getLayout();bg.setLayout(bk);bj.dispose();var bl=qx.lang.Array.clone(this.getChildren());for(var i=0;i<bl.length;i++ ){bg.add(bl[i]);};this.removeListener(n,this._onResize,this);bg.getChildrenContainer().addListener(n,this._onResize,this);this._add(bg);break;};return bg||qx.ui.core.Widget.prototype._createChildControlImpl.call(this,bi);},_getMenuLayout:function(){if(this.hasChildControl(h)){return this.getChildControl(h).getChildrenContainer().getLayout();}else {return this._getLayout();};},_getMenuBounds:function(){if(this.hasChildControl(h)){return this.getChildControl(h).getChildrenContainer().getBounds();}else {return this.getBounds();};},_computePlacementSize:function(){return this._getMenuBounds();},__pd:function(){var bn=this._getMenuBounds();if(!bn){this.addListenerOnce(n,this.__pd,this);return;};var bm=this.getLayoutParent().getBounds().height;var top=this.getLayoutProperties().top;var bo=this.getLayoutProperties().left;if(top<0){this._assertSlideBar(function(){this.setHeight(bn.height+top);this.moveTo(bo,0);});}else if(top+bn.height>bm){this._assertSlideBar(function(){this.setHeight(bm-top);});}else {this.setHeight(null);};},_assertSlideBar:function(bp){if(this.hasChildControl(h)){return bp.call(this);};this.__pb=bp;qx.ui.core.queue.Widget.add(this);},syncWidget:function(bq){this.getChildControl(h);if(this.__pb){this.__pb.call(this);delete this.__pb;};},_onResize:function(){if(this.isVisible()){var br=this._placementTarget;if(!br){return;}else if(br instanceof qx.ui.core.Widget){this.placeToWidget(br,true);}else if(br.top!==undefined){this.placeToPoint(br);}else {throw new Error(y+br);};this.__pd();};},_onPointerOver:function(e){var bt=qx.ui.menu.Manager.getInstance();bt.cancelClose(this);var bu=e.getTarget();if(bu.isEnabled()&&bu instanceof qx.ui.menu.AbstractButton){this.setSelectedButton(bu);var bs=bu.getMenu&&bu.getMenu();if(bs){bs.setOpener(bu);bt.scheduleOpen(bs);this.__pa=bs;}else {var bv=this.getOpenedButton();if(bv){bt.scheduleClose(bv.getMenu());};if(this.__pa){bt.cancelOpen(this.__pa);this.__pa=null;};};}else if(!this.getOpenedButton()){this.resetSelectedButton();};},_onPointerOut:function(e){var bw=qx.ui.menu.Manager.getInstance();if(!qx.ui.core.Widget.contains(this,e.getRelatedTarget())){var bx=this.getOpenedButton();bx?this.setSelectedButton(bx):this.resetSelectedButton();if(bx){bw.cancelClose(bx.getMenu());};if(this.__pa){bw.cancelOpen(this.__pa);};};}},destruct:function(){if(!qx.core.ObjectRegistry.inShutDown){qx.ui.menu.Manager.getInstance().remove(this);};this.getApplicationRoot().removeListener(n,this._onResize,this);this._placementTarget=null;this._disposeObjects(o);}});})();(function(){var a="qx.ui.menu.Layout",b="Integer",c="_applyLayoutChange";qx.Class.define(a,{extend:qx.ui.layout.VBox,properties:{columnSpacing:{check:b,init:0,apply:c},spanColumn:{check:b,init:1,nullable:true,apply:c},iconColumnWidth:{check:b,init:0,themeable:true,apply:c},arrowColumnWidth:{check:b,init:0,themeable:true,apply:c}},members:{__pe:null,_computeSizeHint:function(){var q=this._getLayoutChildren();var o,g,k;var e=this.getSpanColumn();var j=this.__pe=[0,0,0,0];var n=this.getColumnSpacing();var m=0;var f=0;for(var i=0,l=q.length;i<l;i++ ){o=q[i];if(o.isAnonymous()){continue;};g=o.getChildrenSizes();for(var h=0;h<g.length;h++ ){if(e!=null&&h==e&&g[e+1]==0){m=Math.max(m,g[h]);}else {j[h]=Math.max(j[h],g[h]);};};var d=q[i].getInsets();f=Math.max(f,d.left+d.right);};if(e!=null&&j[e]+n+j[e+1]<m){j[e]=m-j[e+1]-n;};if(m==0){k=n*2;}else {k=n*3;};if(j[0]==0){j[0]=this.getIconColumnWidth();};if(j[3]==0){j[3]=this.getArrowColumnWidth();};var p=qx.ui.layout.VBox.prototype._computeSizeHint.call(this).height;return {minHeight:p,height:p,width:qx.lang.Array.sum(j)+f+k};},getColumnSizes:function(){return this.__pe||null;}},destruct:function(){this.__pe=null;}});})();(function(){var a="menu-separator",b="qx.ui.menu.Separator";qx.Class.define(b,{extend:qx.ui.core.Widget,properties:{appearance:{refine:true,init:a},anonymous:{refine:true,init:true}}});})();(function(){var a="Boolean",b="changeIcon",c="changeLabel",d="shortcut",f="_applyMenu",g="submenu",h="changeLocale",i="arrow",j="_applyShowCommandLabel",k="String",l="qx.ui.menu.AbstractButton",m="",n="icon",o="qx.dynlocale",p="abstract",q="qx.ui.menu.Menu",r="keypress",s="_applyIcon",t="changeShowCommandLabel",u="label",v="changeMenu",w="tap",x="_applyLabel",y="changeCommand";qx.Class.define(l,{extend:qx.ui.core.Widget,include:[qx.ui.core.MExecutable],implement:[qx.ui.form.IExecutable],type:p,construct:function(){qx.ui.core.Widget.call(this);this._setLayout(new qx.ui.menu.ButtonLayout);this.addListener(w,this._onTap);this.addListener(r,this._onKeyPress);this.addListener(y,this._onChangeCommand,this);},properties:{blockToolTip:{refine:true,init:true},label:{check:k,apply:x,nullable:true,event:c},menu:{check:q,apply:f,nullable:true,dereference:true,event:v},icon:{check:k,apply:s,themeable:true,nullable:true,event:b},showCommandLabel:{check:a,apply:j,themeable:true,init:true,event:t}},members:{_createChildControlImpl:function(B,A){var z;switch(B){case n:z=new qx.ui.basic.Image;z.setAnonymous(true);this._add(z,{column:0});break;case u:z=new qx.ui.basic.Label;z.setAnonymous(true);this._add(z,{column:1});break;case d:z=new qx.ui.basic.Label;z.setAnonymous(true);if(!this.getShowCommandLabel()){z.exclude();};this._add(z,{column:2});break;case i:z=new qx.ui.basic.Image;z.setAnonymous(true);this._add(z,{column:3});break;};return z||qx.ui.core.Widget.prototype._createChildControlImpl.call(this,B);},_forwardStates:{selected:1},getChildrenSizes:function(){var C=0,D=0,E=0,I=0;if(this._isChildControlVisible(n)){var J=this.getChildControl(n);C=J.getMarginLeft()+J.getSizeHint().width+J.getMarginRight();};if(this._isChildControlVisible(u)){var G=this.getChildControl(u);D=G.getMarginLeft()+G.getSizeHint().width+G.getMarginRight();};if(this._isChildControlVisible(d)){var F=this.getChildControl(d);E=F.getMarginLeft()+F.getSizeHint().width+F.getMarginRight();};if(this._isChildControlVisible(i)){var H=this.getChildControl(i);I=H.getMarginLeft()+H.getSizeHint().width+H.getMarginRight();};return [C,D,E,I];},_onTap:function(e){if(e.isLeftPressed()){this.execute();qx.ui.menu.Manager.getInstance().hideAll();}else {if(!this.getContextMenu()){qx.ui.menu.Manager.getInstance().preventContextMenuOnce();};};},_onKeyPress:function(e){this.execute();},_onChangeCommand:function(e){var K=e.getData();if(K==null){return;};if(qx.core.Environment.get(o)){var M=e.getOldData();if(!M){qx.locale.Manager.getInstance().addListener(h,this._onChangeLocale,this);};if(!K){qx.locale.Manager.getInstance().removeListener(h,this._onChangeLocale,this);};};var L=K!=null?K.toString():m;this.getChildControl(d).setValue(L);},_onChangeLocale:qx.core.Environment.select(o,{"true":function(e){var N=this.getCommand();if(N!=null){this.getChildControl(d).setValue(N.toString());};},"false":null}),_applyIcon:function(P,O){if(P){this._showChildControl(n).setSource(P);}else {this._excludeChildControl(n);};},_applyLabel:function(R,Q){if(R){this._showChildControl(u).setValue(R);}else {this._excludeChildControl(u);};},_applyMenu:function(T,S){if(S){S.resetOpener();S.removeState(g);};if(T){this._showChildControl(i);T.setOpener(this);T.addState(g);}else {this._excludeChildControl(i);};},_applyShowCommandLabel:function(V,U){if(V){this._showChildControl(d);}else {this._excludeChildControl(d);};}},destruct:function(){this.removeListener(y,this._onChangeCommand,this);if(this.getMenu()){if(!qx.core.ObjectRegistry.inShutDown){this.getMenu().destroy();};};if(qx.core.Environment.get(o)){qx.locale.Manager.getInstance().removeListener(h,this._onChangeLocale,this);};}});})();(function(){var a="middle",b="qx.ui.menu.ButtonLayout",c="left";qx.Class.define(b,{extend:qx.ui.layout.Abstract,members:{verifyLayoutProperty:null,renderLayout:function(g,k,d){var s=this._getLayoutChildren();var e;var o;var j=[];for(var i=0,l=s.length;i<l;i++ ){e=s[i];o=e.getLayoutProperties().column;j[o]=e;};var r=this.__pf(s[0]);var q=r.getColumnSizes();var m=r.getSpacingX();var t=qx.lang.Array.sum(q)+m*(q.length-1);if(t<g){q[1]+=g-t;};var u=d.left,top=d.top;var n=qx.ui.layout.Util;for(var i=0,l=q.length;i<l;i++ ){e=j[i];if(e){var h=e.getSizeHint();var f=top+n.computeVerticalAlignOffset(e.getAlignY()||a,h.height,k,0,0);var p=n.computeHorizontalAlignOffset(e.getAlignX()||c,h.width,q[i],e.getMarginLeft(),e.getMarginRight());e.renderLayout(u+p,f,h.width,h.height);};if(q[i]>0){u+=q[i]+m;};};},__pf:function(v){while(!(v instanceof qx.ui.menu.Menu)){v=v.getLayoutParent();};return v;},_computeSizeHint:function(){var y=this._getLayoutChildren();var x=0;var z=0;for(var i=0,l=y.length;i<l;i++ ){var w=y[i].getSizeHint();z+=w.width;x=Math.max(x,w.height);};return {width:z,height:x};}}});})();(function(){var a="qx.ui.core.MRemoteLayoutHandling";qx.Mixin.define(a,{members:{setLayout:function(b){this.getChildrenContainer().setLayout(b);},getLayout:function(){return this.getChildrenContainer().getLayout();}}});})();(function(){var a="qx.ui.container.SlideBar",b="scrollY",c="button-backward",d="horizontal",f="update",g="scrollX",h="_applyOrientation",i="button-forward",j="scrollAnimationEnd",k="content",l="mouse",m="qx.event.type.Event",n="scrollpane",o="vertical",p="Integer",q="slidebar",r="roll",s="execute";qx.Class.define(a,{extend:qx.ui.core.Widget,include:[qx.ui.core.MRemoteChildrenHandling,qx.ui.core.MRemoteLayoutHandling],construct:function(t){qx.ui.core.Widget.call(this);var u=this.getChildControl(n);this._add(u,{flex:1});if(t!=null){this.setOrientation(t);}else {this.initOrientation();};this.addListener(r,this._onRoll,this);},properties:{appearance:{refine:true,init:q},orientation:{check:[d,o],init:d,apply:h},scrollStep:{check:p,init:15,themeable:true}},events:{scrollAnimationEnd:m},members:{getChildrenContainer:function(){return this.getChildControl(k);},_createChildControlImpl:function(x,w){var v;switch(x){case i:v=new qx.ui.form.RepeatButton;v.addListener(s,this._onExecuteForward,this);v.setFocusable(false);this._addAt(v,2);break;case c:v=new qx.ui.form.RepeatButton;v.addListener(s,this._onExecuteBackward,this);v.setFocusable(false);this._addAt(v,0);break;case k:v=new qx.ui.container.Composite();this.getChildControl(n).add(v);break;case n:v=new qx.ui.core.scroll.ScrollPane();v.addListener(f,this._onResize,this);v.addListener(g,this._onScroll,this);v.addListener(b,this._onScroll,this);v.addListener(j,this._onScrollAnimationEnd,this);break;};return v||qx.ui.core.Widget.prototype._createChildControlImpl.call(this,x);},_forwardStates:{barLeft:true,barTop:true,barRight:true,barBottom:true},scrollBy:function(A,y){var z=this.getChildControl(n);if(this.getOrientation()===d){z.scrollByX(A,y);}else {z.scrollByY(A,y);};},scrollTo:function(C,B){var D=this.getChildControl(n);if(this.getOrientation()===d){D.scrollToX(C,B);}else {D.scrollToY(C,B);};},_applyEnabled:function(F,E,name){qx.ui.core.Widget.prototype._applyEnabled.call(this,F,E,name);this._updateArrowsEnabled();},_applyOrientation:function(J,H){var I=[this.getLayout(),this._getLayout()];var K=this.getChildControl(i);var G=this.getChildControl(c);if(H==o&&J==d){K.removeState(o);G.removeState(o);K.addState(d);G.addState(d);}else if(H==d&&J==o){K.removeState(d);G.removeState(d);K.addState(o);G.addState(o);};if(J==d){this._setLayout(new qx.ui.layout.HBox());this.setLayout(new qx.ui.layout.HBox());}else {this._setLayout(new qx.ui.layout.VBox());this.setLayout(new qx.ui.layout.VBox());};if(I[0]){I[0].dispose();};if(I[1]){I[1].dispose();};},_onRoll:function(e){if(e.getPointerType()==l){return;};var O=0;var N=this.getChildControl(n);if(this.getOrientation()===d){O=e.getDelta().x;var P=N.getScrollX();var L=N.getScrollMaxX();var M=parseInt(O);if(!(M<0&&P<=0||M>0&&P>=L||O==0)){e.stop();}else {e.stopMomentum();};}else {O=e.getDelta().y;var P=N.getScrollY();var L=N.getScrollMaxY();var M=parseInt(O);if(!(M<0&&P<=0||M>0&&P>=L||O==0)){e.stop();}else {e.stopMomentum();};};this.scrollBy(parseInt(O,10));if(e.getMomentum()){e.stop();};},_onScroll:function(){this._updateArrowsEnabled();},_onScrollAnimationEnd:function(){this.fireEvent(j);},_onResize:function(e){var content=this.getChildControl(n).getChildren()[0];if(!content){return;};var Q=this.getInnerSize();var S=content.getBounds();var R=(this.getOrientation()===d)?S.width>Q.width:S.height>Q.height;if(R){this._showArrows();this._updateArrowsEnabled();}else {this._hideArrows();};},_onExecuteBackward:function(){this.scrollBy(-this.getScrollStep());},_onExecuteForward:function(){this.scrollBy(this.getScrollStep());},_updateArrowsEnabled:function(){if(!this.getEnabled()){this.getChildControl(c).setEnabled(false);this.getChildControl(i).setEnabled(false);return;};var U=this.getChildControl(n);if(this.getOrientation()===d){var T=U.getScrollX();var V=U.getScrollMaxX();}else {var T=U.getScrollY();var V=U.getScrollMaxY();};this.getChildControl(c).setEnabled(T>0);this.getChildControl(i).setEnabled(T<V);},_showArrows:function(){this._showChildControl(i);this._showChildControl(c);},_hideArrows:function(){this._excludeChildControl(i);this._excludeChildControl(c);this.scrollTo(0);}}});})();(function(){var a="resize",b="scrollY",c="scrollAnimationEnd",d="update",f="scrollX",g="_applyScrollX",h="_applyScrollY",i="frame",j="qx.lang.Type.isNumber(value)&&value>=0&&value<=this.getScrollMaxX()",k="appear",l="qx.lang.Type.isNumber(value)&&value>=0&&value<=this.getScrollMaxY()",m="qx.event.type.Event",n="qx.ui.core.scroll.ScrollPane",o="end",p="scroll";qx.Class.define(n,{extend:qx.ui.core.Widget,construct:function(){qx.ui.core.Widget.call(this);this.set({minWidth:0,minHeight:0});this._setLayout(new qx.ui.layout.Grow());this.addListener(a,this._onUpdate);var q=this.getContentElement();q.addListener(p,this._onScroll,this);q.addListener(k,this._onAppear,this);},events:{update:m,scrollAnimationEnd:m},properties:{scrollX:{check:j,apply:g,event:f,init:0},scrollY:{check:l,apply:h,event:b,init:0}},members:{__pg:null,add:function(r){var s=this._getChildren()[0];if(s){this._remove(s);s.removeListener(a,this._onUpdate,this);};if(r){this._add(r);r.addListener(a,this._onUpdate,this);};},remove:function(t){if(t){this._remove(t);t.removeListener(a,this._onUpdate,this);};},getChildren:function(){return this._getChildren();},_onUpdate:function(e){this.fireEvent(d);},_onScroll:function(e){var u=this.getContentElement();this.setScrollX(u.getScrollX());this.setScrollY(u.getScrollY());},_onAppear:function(e){var z=this.getContentElement();var v=this.getScrollX();var A=z.getScrollX();if(v!=A){z.scrollToX(v);};var w=this.getScrollY();var B=z.getScrollY();if(w!=B){z.scrollToY(w);};},getItemTop:function(C){var top=0;do {top+=C.getBounds().top;C=C.getLayoutParent();}while(C&&C!==this);return top;},getItemBottom:function(D){return this.getItemTop(D)+D.getBounds().height;},getItemLeft:function(E){var F=0;var parent;do {F+=E.getBounds().left;parent=E.getLayoutParent();if(parent){F+=parent.getInsets().left;};E=parent;}while(E&&E!==this);return F;},getItemRight:function(G){return this.getItemLeft(G)+G.getBounds().width;},getScrollSize:function(){return this.getChildren()[0].getBounds();},getScrollMaxX:function(){var I=this.getInnerSize();var H=this.getScrollSize();if(I&&H){return Math.max(0,H.width-I.width);};return 0;},getScrollMaxY:function(){var K=this.getInnerSize();var J=this.getScrollSize();if(K&&J){return Math.max(0,J.height-K.height);};return 0;},scrollToX:function(O,L){var M=this.getScrollMaxX();if(O<0){O=0;}else if(O>M){O=M;};this.stopScrollAnimation();if(L){var N=this.getScrollX();this.__pg=new qx.bom.AnimationFrame();this.__pg.on(o,function(){this.setScrollX(O);this.__pg=null;this.fireEvent(c);},this);this.__pg.on(i,function(Q){var P=parseInt(Q/L*(O-N)+N);this.setScrollX(P);},this);this.__pg.startSequence(L);}else {this.setScrollX(O);};},scrollToY:function(U,R){var S=this.getScrollMaxY();if(U<0){U=0;}else if(U>S){U=S;};this.stopScrollAnimation();if(R){var T=this.getScrollY();this.__pg=new qx.bom.AnimationFrame();this.__pg.on(o,function(){this.setScrollY(U);this.__pg=null;this.fireEvent(c);},this);this.__pg.on(i,function(W){var V=parseInt(W/R*(U-T)+T);this.setScrollY(V);},this);this.__pg.startSequence(R);}else {this.setScrollY(U);};},scrollByX:function(x,X){this.scrollToX(this.getScrollX()+x,X);},scrollByY:function(y,Y){this.scrollToY(this.getScrollY()+y,Y);},stopScrollAnimation:function(){if(this.__pg){this.__pg.cancelSequence();this.__pg=null;};},_applyScrollX:function(ba){this.getContentElement().scrollToX(ba);},_applyScrollY:function(bb){this.getContentElement().scrollToY(bb);}}});})();(function(){var a="qx.ui.layout.Grow";qx.Class.define(a,{extend:qx.ui.layout.Abstract,members:{verifyLayoutProperty:null,renderLayout:function(b,c,e){var h=this._getLayoutChildren();var g,j,f,d;for(var i=0,l=h.length;i<l;i++ ){g=h[i];j=g.getSizeHint();f=b;if(f<j.minWidth){f=j.minWidth;}else if(f>j.maxWidth){f=j.maxWidth;};d=c;if(d<j.minHeight){d=j.minHeight;}else if(d>j.maxHeight){d=j.maxHeight;};g.renderLayout(e.left,e.top,f,d);};},_computeSizeHint:function(){var r=this._getLayoutChildren();var p,t;var s=0,q=0;var o=0,m=0;var k=Infinity,n=Infinity;for(var i=0,l=r.length;i<l;i++ ){p=r[i];t=p.getSizeHint();s=Math.max(s,t.width);q=Math.max(q,t.height);o=Math.max(o,t.minWidth);m=Math.max(m,t.minHeight);k=Math.min(k,t.maxWidth);n=Math.min(n,t.maxHeight);};return {width:s,height:q,minWidth:o,minHeight:m,maxWidth:k,maxHeight:n};}}});})();(function(){var a="button-backward",b="vertical",c="button-forward",d="menu-slidebar",e="qx.ui.menu.MenuSlideBar",f="execute";qx.Class.define(e,{extend:qx.ui.container.SlideBar,construct:function(){qx.ui.container.SlideBar.call(this,b);},properties:{appearance:{refine:true,init:d}},members:{_createChildControlImpl:function(i,h){var g;switch(i){case c:g=new qx.ui.form.HoverButton();g.addListener(f,this._onExecuteForward,this);this._addAt(g,2);break;case a:g=new qx.ui.form.HoverButton();g.addListener(f,this._onExecuteBackward,this);this._addAt(g,0);break;};return g||qx.ui.container.SlideBar.prototype._createChildControlImpl.call(this,i);}}});})();(function(){var a="hovered",b="hover-button",c="pointerover",d="interval",f="__de",g="qx.ui.form.HoverButton",h="Integer",i="pointerout";qx.Class.define(g,{extend:qx.ui.basic.Atom,include:[qx.ui.core.MExecutable],implement:[qx.ui.form.IExecutable],construct:function(j,k){qx.ui.basic.Atom.call(this,j,k);this.addListener(c,this._onPointerOver,this);this.addListener(i,this._onPointerOut,this);this.__de=new qx.event.AcceleratingTimer();this.__de.addListener(d,this._onInterval,this);},properties:{appearance:{refine:true,init:b},interval:{check:h,init:80},firstInterval:{check:h,init:200},minTimer:{check:h,init:20},timerDecrease:{check:h,init:2}},members:{__de:null,_onPointerOver:function(e){if(!this.isEnabled()||e.getTarget()!==this){return;};this.__de.set({interval:this.getInterval(),firstInterval:this.getFirstInterval(),minimum:this.getMinTimer(),decrease:this.getTimerDecrease()}).start();this.addState(a);},_onPointerOut:function(e){this.__de.stop();this.removeState(a);if(!this.isEnabled()||e.getTarget()!==this){return;};},_onInterval:function(){if(this.isEnabled()){this.execute();}else {this.__de.stop();};}},destruct:function(){this._disposeObjects(f);}});})();(function(){var a="hovered",b="inherit",c="pressed",d="mouse",f="qx.ui.menubar.Button",g="keydown",h="menubar-button",i="keyup";qx.Class.define(f,{extend:qx.ui.form.MenuButton,construct:function(k,l,j){qx.ui.form.MenuButton.call(this,k,l,j);this.removeListener(g,this._onKeyDown);this.removeListener(i,this._onKeyUp);},properties:{appearance:{refine:true,init:h},show:{refine:true,init:b},focusable:{refine:true,init:false}},members:{getMenuBar:function(){var parent=this;while(parent){if(parent instanceof qx.ui.toolbar.ToolBar){return parent;};parent=parent.getLayoutParent();};return null;},open:function(m){qx.ui.form.MenuButton.prototype.open.call(this,m);var menubar=this.getMenuBar();if(menubar){menubar._setAllowMenuOpenHover(true);};},_onMenuChange:function(e){var n=this.getMenu();var menubar=this.getMenuBar();if(n.isVisible()){this.addState(c);if(menubar){menubar.setOpenMenu(n);};}else {this.removeState(c);if(menubar&&menubar.getOpenMenu()==n){menubar.resetOpenMenu();menubar._setAllowMenuOpenHover(false);};};},_onPointerUp:function(e){qx.ui.form.MenuButton.prototype._onPointerUp.call(this,e);var o=this.getMenu();if(o&&o.isVisible()&&!this.hasState(c)){this.addState(c);};},_onPointerOver:function(e){this.addState(a);if(this.getMenu()&&e.getPointerType()==d){var menubar=this.getMenuBar();if(menubar&&menubar._isAllowMenuOpenHover()){qx.ui.menu.Manager.getInstance().hideAll();menubar._setAllowMenuOpenHover(true);if(this.isEnabled()){this.open();};};};}}});})();(function(){var a="Boolean",b="changeShow",c="qx.ui.core.Widget",d="_applyOverflowHandling",f="both",g="_applySpacing",h="qx.ui.toolbar.ToolBar",j="qx.event.type.Data",k="Integer",m="showItem",n="Widget must be child of the toolbar.",o="visible",p="changeOpenMenu",q="resize",r="icon",s="_applyOverflowIndicator",t="toolbar",u="qx.ui.menu.Menu",v="Priority already in use!",w="label",x="_applyShow",y="excluded",z="show",A="hideItem";qx.Class.define(h,{extend:qx.ui.core.Widget,include:qx.ui.core.MChildrenHandling,construct:function(){qx.ui.core.Widget.call(this);this._setLayout(new qx.ui.layout.HBox());this.__ph=[];this.__pi=[];},properties:{appearance:{refine:true,init:t},openMenu:{check:u,event:p,nullable:true},show:{init:f,check:[f,w,r],inheritable:true,apply:x,event:b},spacing:{nullable:true,check:k,themeable:true,apply:g},overflowIndicator:{check:c,nullable:true,apply:s},overflowHandling:{init:false,check:a,apply:d}},events:{"hideItem":j,"showItem":j},members:{__ph:null,__pi:null,_computeSizeHint:function(){var D=qx.ui.core.Widget.prototype._computeSizeHint.call(this);if(true&&this.getOverflowHandling()){var B=0;var C=this.getOverflowIndicator();if(C){B=C.getSizeHint().width+this.getSpacing();};D.minWidth=B;};return D;},_onResize:function(e){this._recalculateOverflow(e.getData().width);},_recalculateOverflow:function(F,G){if(!this.getOverflowHandling()){return;};G=G||this.getSizeHint().width;var E=this.getOverflowIndicator();var L=0;if(E){L=E.getSizeHint().width;};if(F==undefined&&this.getBounds()!=null){F=this.getBounds().width;};if(F==undefined){return;};if(F<G){do {var K=this._getNextToHide();if(!K){return;};var P=K.getMarginLeft()+K.getMarginRight();P=Math.max(P,this.getSpacing());var J=K.getSizeHint().width+P;this.__pk(K);G-=J;if(E&&E.getVisibility()!=o){E.setVisibility(o);G+=L;var H=E.getMarginLeft()+E.getMarginRight();G+=Math.max(H,this.getSpacing());};}while(G>F);}else if(this.__ph.length>0){do {var Q=this.__ph[0];if(Q){var P=Q.getMarginLeft()+Q.getMarginRight();P=Math.max(P,this.getSpacing());if(Q.getContentElement().getDomElement()==null){Q.syncAppearance();Q.invalidateLayoutCache();};var I=Q.getSizeHint().width;var O=false;if(this.__ph.length==1&&L>0){var N=P-this.getSpacing();var M=G-L+I+N;O=F>M;};if(F>G+I+P||O){this.__pj(Q);G+=I;if(E&&this.__ph.length==0){E.setVisibility(y);};}else {return;};};}while(F>=G&&this.__ph.length>0);};},__pj:function(R){R.setVisibility(o);this.__ph.shift();this.fireDataEvent(m,R);},__pk:function(S){if(!S){return;};this.__ph.unshift(S);S.setVisibility(y);this.fireDataEvent(A,S);},_getNextToHide:function(){for(var i=this.__pi.length-1;i>=0;i-- ){var T=this.__pi[i];if(T&&T.getVisibility&&T.getVisibility()==o){return T;};};var U=this._getChildren();for(var i=U.length-1;i>=0;i-- ){var V=U[i];if(V==this.getOverflowIndicator()){continue;};if(V.getVisibility&&V.getVisibility()==o){return V;};};},setRemovePriority:function(X,W,Y){if(!Y&&this.__pi[W]!=undefined){throw new Error(v);};this.__pi[W]=X;},_applyOverflowHandling:function(bc,ba){this.invalidateLayoutCache();var parent=this.getLayoutParent();if(parent){parent.invalidateLayoutCache();};var bb=this.getBounds();if(bb&&bb.width){this._recalculateOverflow(bb.width);};if(bc){this.addListener(q,this._onResize,this);}else {this.removeListener(q,this._onResize,this);var bd=this.getOverflowIndicator();if(bd){bd.setVisibility(y);};for(var i=0;i<this.__ph.length;i++ ){this.__ph[i].setVisibility(o);};this.__ph=[];};},_applyOverflowIndicator:function(bf,be){if(be){this._remove(be);};if(bf){if(this._indexOf(bf)==-1){throw new Error(n);};bf.setVisibility(y);};},__pl:false,_setAllowMenuOpenHover:function(bg){this.__pl=bg;},_isAllowMenuOpenHover:function(){return this.__pl;},_applySpacing:function(bj,bh){var bi=this._getLayout();bj==null?bi.resetSpacing():bi.setSpacing(bj);},_applyShow:function(bl){var bk=this._getChildren();for(var i=0;i<bk.length;i++ ){if(bk[i].setShow){bk[i].setShow(bl);};};},_add:function(bo,bm){qx.ui.core.Widget.prototype._add.call(this,bo,bm);if(bo.setShow&&!qx.util.PropertyUtil.getUserValue(bo,z)){bo.setShow(this.getShow());};var bn=this.getSizeHint().width+bo.getSizeHint().width+2*this.getSpacing();this._recalculateOverflow(null,bn);},_addAt:function(bs,bp,bq){qx.ui.core.Widget.prototype._addAt.call(this,bs,bp,bq);if(bs.setShow&&!qx.util.PropertyUtil.getUserValue(bs,z)){bs.setShow(this.getShow());};var br=this.getSizeHint().width+bs.getSizeHint().width+2*this.getSpacing();this._recalculateOverflow(null,br);},_addBefore:function(bw,bt,bu){qx.ui.core.Widget.prototype._addBefore.call(this,bw,bt,bu);if(bw.setShow&&!qx.util.PropertyUtil.getUserValue(bw,z)){bw.setShow(this.getShow());};var bv=this.getSizeHint().width+bw.getSizeHint().width+2*this.getSpacing();this._recalculateOverflow(null,bv);},_addAfter:function(bA,bx,by){qx.ui.core.Widget.prototype._addAfter.call(this,bA,bx,by);if(bA.setShow&&!qx.util.PropertyUtil.getUserValue(bA,z)){bA.setShow(this.getShow());};var bz=this.getSizeHint().width+bA.getSizeHint().width+2*this.getSpacing();this._recalculateOverflow(null,bz);},_remove:function(bC){qx.ui.core.Widget.prototype._remove.call(this,bC);var bB=this.getSizeHint().width-bC.getSizeHint().width-2*this.getSpacing();this._recalculateOverflow(null,bB);},_removeAt:function(bD){var bF=this._getChildren()[bD];qx.ui.core.Widget.prototype._removeAt.call(this,bD);var bE=this.getSizeHint().width-bF.getSizeHint().width-2*this.getSpacing();this._recalculateOverflow(null,bE);},_removeAll:function(){qx.ui.core.Widget.prototype._removeAll.call(this);this._recalculateOverflow(null,0);},addSpacer:function(){var bG=new qx.ui.core.Spacer;this._add(bG,{flex:1});return bG;},addSeparator:function(){this.add(new qx.ui.toolbar.Separator);},getMenuButtons:function(){var bI=this.getChildren();var bH=[];var bJ;for(var i=0,l=bI.length;i<l;i++ ){bJ=bI[i];if(bJ instanceof qx.ui.menubar.Button){bH.push(bJ);}else if(bJ instanceof qx.ui.toolbar.Part){bH.push.apply(bH,bJ.getMenuButtons());};};return bH;}},destruct:function(){if(this.hasListener(q)){this.removeListener(q,this._onResize,this);};}});})();(function(){var a="qx.ui.core.Spacer";qx.Class.define(a,{extend:qx.ui.core.LayoutItem,construct:function(b,c){qx.ui.core.LayoutItem.call(this);this.setWidth(b!=null?b:0);this.setHeight(c!=null?c:0);},members:{checkAppearanceNeeds:function(){},addChildrenToQueue:function(d){},destroy:function(){if(this.$$disposed){return;};var parent=this.$$parent;if(parent){parent._remove(this);};qx.ui.core.queue.Dispose.add(this);}}});})();(function(){var a="toolbar-separator",b="qx.ui.toolbar.Separator";qx.Class.define(b,{extend:qx.ui.core.Widget,properties:{appearance:{refine:true,init:a},anonymous:{refine:true,init:true},width:{refine:true,init:0},height:{refine:true,init:0}}});})();(function(){var a="Integer",b="middle",c="qx.ui.toolbar.Part",d="icon",e="label",f="handle",g="left",h="container",j="syncAppearance",k="visible",m="changeShow",n="_applySpacing",o="both",p="toolbar/part",q="right",r="changeChildren";qx.Class.define(c,{extend:qx.ui.core.Widget,include:[qx.ui.core.MRemoteChildrenHandling],construct:function(){qx.ui.core.Widget.call(this);this._setLayout(new qx.ui.layout.HBox);this._createChildControl(f);},properties:{appearance:{refine:true,init:p},show:{init:o,check:[o,e,d],inheritable:true,event:m},spacing:{nullable:true,check:a,themeable:true,apply:n}},members:{_createChildControlImpl:function(u,t){var s;switch(u){case f:s=new qx.ui.basic.Image();s.setAlignY(b);this._add(s);break;case h:s=new qx.ui.toolbar.PartContainer();s.addListener(j,this.__pm,this);this._add(s);s.addListener(r,function(){this.__pm();},this);break;};return s||qx.ui.core.Widget.prototype._createChildControlImpl.call(this,u);},getChildrenContainer:function(){return this.getChildControl(h);},_applySpacing:function(x,v){var w=this.getChildControl(h).getLayout();x==null?w.resetSpacing():w.setSpacing(x);},__pm:function(){var y=this.getChildrenContainer().getChildren();y=y.filter(function(z){return z.getVisibility()==k;});for(var i=0;i<y.length;i++ ){if(i==0&&i!=y.length-1){y[i].addState(g);y[i].removeState(q);y[i].removeState(b);}else if(i==y.length-1&&i!=0){y[i].addState(q);y[i].removeState(g);y[i].removeState(b);}else if(i==0&&i==y.length-1){y[i].removeState(g);y[i].removeState(b);y[i].removeState(q);}else {y[i].addState(b);y[i].removeState(q);y[i].removeState(g);};};},addSeparator:function(){this.add(new qx.ui.toolbar.Separator);},getMenuButtons:function(){var B=this.getChildren();var A=[];var C;for(var i=0,l=B.length;i<l;i++ ){C=B[i];if(C instanceof qx.ui.menubar.Button){A.push(C);};};return A;}}});})();(function(){var a="toolbar/part/container",b="icon",c="label",d="qx.ui.toolbar.PartContainer",e="qx.event.type.Event",f="changeShow",g="both",h="changeChildren";qx.Class.define(d,{extend:qx.ui.container.Composite,construct:function(){qx.ui.container.Composite.call(this);this._setLayout(new qx.ui.layout.HBox);},events:{changeChildren:e},properties:{appearance:{refine:true,init:a},show:{init:g,check:[g,c,b],inheritable:true,event:f}},members:{_afterAddChild:function(i){this.fireEvent(h);},_afterRemoveChild:function(j){this.fireEvent(h);}}});})();(function(){var a="qx.ui.menu.Button",b="menu-button";qx.Class.define(a,{extend:qx.ui.menu.AbstractButton,construct:function(g,d,f,c){qx.ui.menu.AbstractButton.call(this);if(g!=null){this.setLabel(g);};if(d!=null){this.setIcon(d);};if(f!=null){this.setCommand(f);};if(c!=null){this.setMenu(c);};},properties:{appearance:{refine:true,init:b}},members:{_onTap:function(e){if(e.isLeftPressed()&&this.getMenu()){this.execute();this.getMenu().open();return;};qx.ui.menu.AbstractButton.prototype._onTap.call(this,e);}}});})();(function(){var a="Unrecognized factory request: ",b="menu-button",c="table-column-reset-button",d="separator",e="user-button",f="qx.ui.table.columnmenu.Button",g="menu";qx.Class.define(f,{extend:qx.ui.form.MenuButton,implement:qx.ui.table.IColumnMenuButton,construct:function(){qx.ui.form.MenuButton.call(this);this.__lO=new qx.ui.core.Blocker(this);},members:{__oL:null,__lO:null,factory:function(h,k){switch(h){case g:var m=new qx.ui.menu.Menu();this.setMenu(m);return m;case b:var n=new qx.ui.table.columnmenu.MenuItem(k.text);n.setVisible(k.bVisible);this.getMenu().add(n);return n;case e:var j=new qx.ui.menu.Button(k.text);j.set({appearance:c});return j;case d:return new qx.ui.menu.Separator();default:throw new Error(a+h);};},getBlocker:function(){return this.__lO;},empty:function(){var o=this.getMenu();var p=o.getChildren();for(var i=0,l=p.length;i<l;i++ ){p[0].destroy();};}},destruct:function(){this.__lO.dispose();}});})();(function(){var a="qx.ui.table.IColumnMenuItem",b="qx.event.type.Data";qx.Interface.define(a,{properties:{visible:{}},events:{changeVisible:b}});})();(function(){var a="qx.ui.form.IBooleanForm",b="qx.event.type.Data";qx.Interface.define(a,{events:{"changeValue":b},members:{setValue:function(c){return arguments.length==1;},resetValue:function(){},getValue:function(){}}});})();(function(){var a="menu-checkbox",b="Boolean",c="label",d="_applyValue",f="value",g="changeValue",h="toolTipText",i="enabled",j="qx.ui.menu.CheckBox",k="checked",l="menu",m="execute";qx.Class.define(j,{extend:qx.ui.menu.AbstractButton,implement:[qx.ui.form.IBooleanForm],construct:function(o,n){qx.ui.menu.AbstractButton.call(this);if(o!=null){if(o.translate){this.setLabel(o.translate());}else {this.setLabel(o);};};if(n!=null){this.setMenu(n);};this.addListener(m,this._onExecute,this);},properties:{appearance:{refine:true,init:a},value:{check:b,init:false,apply:d,event:g,nullable:true}},members:{_bindableProperties:[i,c,h,f,l],_applyValue:function(q,p){q?this.addState(k):this.removeState(k);},_onExecute:function(e){this.toggleValue();}}});})();(function(){var a="changeVisible",b="qx.ui.table.columnmenu.MenuItem",c="_applyVisible",d="Boolean",f="changeValue";qx.Class.define(b,{extend:qx.ui.menu.CheckBox,implement:qx.ui.table.IColumnMenuItem,properties:{visible:{check:d,init:true,apply:c,event:a}},construct:function(g){qx.ui.menu.CheckBox.call(this,g);this.addListener(f,function(e){this.bInListener=true;this.setVisible(e.getData());this.bInListener=false;});},members:{_applyVisible:function(i,h){if(!this.bInListener){this.setValue(i);};}}});})();(function(){var a="qx.ui.table.selection.Model",b="qx.ui.table.selection.Manager";qx.Class.define(b,{extend:qx.core.Object,construct:function(){qx.core.Object.call(this);},properties:{selectionModel:{check:a}},members:{__pn:null,handleTap:function(c,d){if(d.isLeftPressed()){var e=this.getSelectionModel();if(!e.isSelectedIndex(c)){this._handleSelectEvent(c,d);this.__pn=true;}else {this.__pn=false;};}else if(d.isRightPressed()&&d.getModifiers()==0){var e=this.getSelectionModel();if(!e.isSelectedIndex(c)){e.setSelectionInterval(c,c);};};if(d.isLeftPressed()&&!this.__pn){this._handleSelectEvent(c,d);};},handleSelectKeyDown:function(f,g){this._handleSelectEvent(f,g);},handleMoveKeyDown:function(h,i){var k=this.getSelectionModel();switch(i.getModifiers()){case 0:k.setSelectionInterval(h,h);break;case qx.event.type.Dom.SHIFT_MASK:var j=k.getAnchorSelectionIndex();if(j==-1){k.setSelectionInterval(h,h);}else {k.setSelectionInterval(j,h);};break;};},_handleSelectEvent:function(l,n){var p=this.getSelectionModel();var m=p.getLeadSelectionIndex();var o=p.getAnchorSelectionIndex();if(n.isShiftPressed()){if(l!=m||p.isSelectionEmpty()){if(o==-1){o=l;};if(n.isCtrlOrCommandPressed()){p.addSelectionInterval(o,l);}else {p.setSelectionInterval(o,l);};};}else if(n.isCtrlOrCommandPressed()){if(p.isSelectedIndex(l)){p.removeSelectionInterval(l,l);}else {p.addSelectionInterval(l,l);};}else {p.setSelectionInterval(l,l);};}}});})();(function(){var a="..",b="changeSelection",c="Try to turn off batch mode althoug it was not turned on.",d="_applySelectionMode",f="]",g="qx.event.type.Event",h="Ranges:",k="qx.ui.table.selection.Model",l=" [";qx.Class.define(k,{extend:qx.core.Object,construct:function(){qx.core.Object.call(this);this.__po=[];this.__pp=-1;this.__pq=-1;this.hasBatchModeRefCount=0;this.__pr=false;},events:{"changeSelection":g},statics:{NO_SELECTION:1,SINGLE_SELECTION:2,SINGLE_INTERVAL_SELECTION:3,MULTIPLE_INTERVAL_SELECTION:4,MULTIPLE_INTERVAL_SELECTION_TOGGLE:5},properties:{selectionMode:{init:2,check:[1,2,3,4,5],apply:d}},members:{__pr:null,__pp:null,__pq:null,__po:null,_applySelectionMode:function(m){this.resetSelection();},setBatchMode:function(n){if(n){this.hasBatchModeRefCount+=1;}else {if(this.hasBatchModeRefCount==0){throw new Error(c);};this.hasBatchModeRefCount-=1;if(this.__pr){this.__pr=false;this._fireChangeSelection();};};return this.hasBatchMode();},hasBatchMode:function(){return this.hasBatchModeRefCount>0;},getAnchorSelectionIndex:function(){return this.__pp;},_setAnchorSelectionIndex:function(o){this.__pp=o;},getLeadSelectionIndex:function(){return this.__pq;},_setLeadSelectionIndex:function(p){this.__pq=p;},_getSelectedRangeArr:function(){return this.__po;},resetSelection:function(){if(!this.isSelectionEmpty()){this._resetSelection();this._fireChangeSelection();};},isSelectionEmpty:function(){return this.__po.length==0;},getSelectedCount:function(){var r=0;for(var i=0;i<this.__po.length;i++ ){var q=this.__po[i];r+=q.maxIndex-q.minIndex+1;};return r;},isSelectedIndex:function(s){for(var i=0;i<this.__po.length;i++ ){var t=this.__po[i];if(s>=t.minIndex&&s<=t.maxIndex){return true;};};return false;},getSelectedRanges:function(){var u=[];for(var i=0;i<this.__po.length;i++ ){u.push({minIndex:this.__po[i].minIndex,maxIndex:this.__po[i].maxIndex});};return u;},iterateSelection:function(w,v){for(var i=0;i<this.__po.length;i++ ){for(var j=this.__po[i].minIndex;j<=this.__po[i].maxIndex;j++ ){w.call(v,j);};};},setSelectionInterval:function(z,y){var x=this.self(arguments);switch(this.getSelectionMode()){case x.NO_SELECTION:return;case x.SINGLE_SELECTION:if(this.isSelectedIndex(y)){return;};z=y;break;case x.MULTIPLE_INTERVAL_SELECTION_TOGGLE:this.setBatchMode(true);try{for(var i=z;i<=y;i++ ){if(!this.isSelectedIndex(i)){this._addSelectionInterval(i,i);}else {this.removeSelectionInterval(i,i);};};}catch(e){throw e;}finally{this.setBatchMode(false);};this._fireChangeSelection();return;};this._resetSelection();this._addSelectionInterval(z,y);this._fireChangeSelection();},addSelectionInterval:function(B,A){var C=qx.ui.table.selection.Model;switch(this.getSelectionMode()){case C.NO_SELECTION:return;case C.MULTIPLE_INTERVAL_SELECTION:case C.MULTIPLE_INTERVAL_SELECTION_TOGGLE:this._addSelectionInterval(B,A);this._fireChangeSelection();break;default:this.setSelectionInterval(B,A);break;};},removeSelectionInterval:function(G,E){this.__pp=G;this.__pq=E;var D=Math.min(G,E);var H=Math.max(G,E);for(var i=0;i<this.__po.length;i++ ){var J=this.__po[i];if(J.minIndex>H){break;}else if(J.maxIndex>=D){var K=(J.minIndex>=D)&&(J.minIndex<=H);var F=(J.maxIndex>=D)&&(J.maxIndex<=H);if(K&&F){this.__po.splice(i,1);i-- ;}else if(K){J.minIndex=H+1;}else if(F){J.maxIndex=D-1;}else {var I={minIndex:H+1,maxIndex:J.maxIndex};this.__po.splice(i+1,0,I);J.maxIndex=D-1;break;};};};this._fireChangeSelection();},_resetSelection:function(){this.__po=[];this.__pp=-1;this.__pq=-1;},_addSelectionInterval:function(O,M){this.__pp=O;this.__pq=M;var L=Math.min(O,M);var P=Math.max(O,M);var N=0;for(;N<this.__po.length;N++ ){var Q=this.__po[N];if(Q.minIndex>L){break;};};this.__po.splice(N,0,{minIndex:L,maxIndex:P});var R=this.__po[0];for(var i=1;i<this.__po.length;i++ ){var Q=this.__po[i];if(R.maxIndex+1>=Q.minIndex){R.maxIndex=Math.max(R.maxIndex,Q.maxIndex);this.__po.splice(i,1);i-- ;}else {R=Q;};};},_dumpRanges:function(){var S=h;for(var i=0;i<this.__po.length;i++ ){var T=this.__po[i];S+=l+T.minIndex+a+T.maxIndex+f;};this.debug(S);},_fireChangeSelection:function(){if(this.hasBatchMode()){this.__pr=true;}else {this.fireEvent(b);};}},destruct:function(){this.__po=null;}});})();(function(){var a="qx.ui.table.ICellRenderer";qx.Interface.define(a,{members:{createDataCellHtml:function(b,c){return true;}}});})();(function(){var a="qooxdoo-table-cell",b="content",c='</div>',d="nowrap",e="default",f="changeTheme",g="}",h="width:",i=".qooxdoo-table-cell-right { text-align:right } ",j="css.boxsizing",k="css.boxmodel",l="0px 6px",m='<div class="',n="0px",o="height:",p="1px solid ",q=".qooxdoo-table-cell-bold { font-weight:bold } ",r="String",s="} ",t="qx.dyntheme",u='>',v="",w='" ',x="ellipsis",y="content-box",z="px;",A='left:',B="qx.ui.table.cellrenderer.Abstract",C='" style="',D="abstract",E="none",F=".qooxdoo-table-cell {",G="hidden",H="table-column-line",I='px;',J=".qooxdoo-table-cell-italic { font-style:italic} ",K="absolute";qx.Class.define(B,{type:D,implement:qx.ui.table.ICellRenderer,extend:qx.core.Object,construct:function(){qx.core.Object.call(this);var L=qx.ui.table.cellrenderer.Abstract;if(!L.__ps){L.__ps=this.self(arguments);this._createStyleSheet();if(qx.core.Environment.get(t)){qx.theme.manager.Meta.getInstance().addListener(f,this._onChangeTheme,this);};};},properties:{defaultCellStyle:{init:null,check:r,nullable:true}},members:{_onChangeTheme:qx.core.Environment.select(t,{"true":function(){qx.bom.Stylesheet.removeAllRules(qx.ui.table.cellrenderer.Abstract.__ps.stylesheet);this._createStyleSheet();},"false":null}),_insetX:6+6+1,_insetY:0,_createStyleSheet:function(){var M=qx.theme.manager.Color.getInstance();var N=F+qx.bom.element.Style.compile({position:K,top:n,overflow:G,whiteSpace:d,borderRight:p+M.resolve(H),padding:l,cursor:e,textOverflow:x,userSelect:E})+s+i+J+q;if(qx.core.Environment.get(j)){N+=F+qx.bom.element.BoxSizing.compile(y)+g;};qx.ui.table.cellrenderer.Abstract.__ps.stylesheet=qx.bom.Stylesheet.createElement(N);},_getCellClass:function(O){return a;},_getCellStyle:function(P){return P.style||v;},_getCellAttributes:function(Q){return v;},_getContentHtml:function(R){return R.value||v;},_getCellSizeStyle:function(S,W,U,V){var T=v;if(qx.core.Environment.get(k)==b){S-=U;W-=V;};T+=h+Math.max(S,0)+z;T+=o+Math.max(W,0)+z;return T;},createDataCellHtml:function(X,Y){Y.push(m,this._getCellClass(X),C,A,X.styleLeft,I,this._getCellSizeStyle(X.styleWidth,X.styleHeight,this._insetX,this._insetY),this._getCellStyle(X),w,this._getCellAttributes(X),u+this._getContentHtml(X),c);}},destruct:function(){if(qx.core.Environment.get(t)){qx.theme.manager.Meta.getInstance().removeListener(f,this._onChangeTheme,this);};}});})();(function(){var a="Boolean",b="",c="qx.ui.table.cellrenderer.Default",d="number",e=" qooxdoo-table-cell-bold",f=" qooxdoo-table-cell-right",g=" qooxdoo-table-cell-italic",h="string";qx.Class.define(c,{extend:qx.ui.table.cellrenderer.Abstract,statics:{STYLEFLAG_ALIGN_RIGHT:1,STYLEFLAG_BOLD:2,STYLEFLAG_ITALIC:4,_numberFormat:null},properties:{useAutoAlign:{check:a,init:true}},members:{_getStyleFlags:function(i){if(this.getUseAutoAlign()){if(typeof i.value==d){return qx.ui.table.cellrenderer.Default.STYLEFLAG_ALIGN_RIGHT;};};return 0;},_getCellClass:function(j){var k=qx.ui.table.cellrenderer.Abstract.prototype._getCellClass.call(this,j);if(!k){return b;};var l=this._getStyleFlags(j);if(l&qx.ui.table.cellrenderer.Default.STYLEFLAG_ALIGN_RIGHT){k+=f;};if(l&qx.ui.table.cellrenderer.Default.STYLEFLAG_BOLD){k+=e;};if(l&qx.ui.table.cellrenderer.Default.STYLEFLAG_ITALIC){k+=g;};return k;},_getContentHtml:function(m){return qx.bom.String.escape(this._formatValue(m));},_formatValue:function(n){var p=n.value;var o;if(p==null){return b;};if(typeof p==h){return p;}else if(typeof p==d){if(!qx.ui.table.cellrenderer.Default._numberFormat){qx.ui.table.cellrenderer.Default._numberFormat=new qx.util.format.NumberFormat();qx.ui.table.cellrenderer.Default._numberFormat.setMaximumFractionDigits(2);};o=qx.ui.table.cellrenderer.Default._numberFormat.format(p);}else if(p instanceof Date){o=qx.util.format.DateFormat.getDateInstance().format(p);}else {o=p.toString();};return o;}}});})();(function(){var a="<br",b="",c=" &nbsp;",d="<br>",e=" ",f="\n",g="qx.bom.String";qx.Bootstrap.define(g,{statics:{TO_CHARCODE:{"quot":34,"amp":38,"lt":60,"gt":62,"nbsp":160,"iexcl":161,"cent":162,"pound":163,"curren":164,"yen":165,"brvbar":166,"sect":167,"uml":168,"copy":169,"ordf":170,"laquo":171,"not":172,"shy":173,"reg":174,"macr":175,"deg":176,"plusmn":177,"sup2":178,"sup3":179,"acute":180,"micro":181,"para":182,"middot":183,"cedil":184,"sup1":185,"ordm":186,"raquo":187,"frac14":188,"frac12":189,"frac34":190,"iquest":191,"Agrave":192,"Aacute":193,"Acirc":194,"Atilde":195,"Auml":196,"Aring":197,"AElig":198,"Ccedil":199,"Egrave":200,"Eacute":201,"Ecirc":202,"Euml":203,"Igrave":204,"Iacute":205,"Icirc":206,"Iuml":207,"ETH":208,"Ntilde":209,"Ograve":210,"Oacute":211,"Ocirc":212,"Otilde":213,"Ouml":214,"times":215,"Oslash":216,"Ugrave":217,"Uacute":218,"Ucirc":219,"Uuml":220,"Yacute":221,"THORN":222,"szlig":223,"agrave":224,"aacute":225,"acirc":226,"atilde":227,"auml":228,"aring":229,"aelig":230,"ccedil":231,"egrave":232,"eacute":233,"ecirc":234,"euml":235,"igrave":236,"iacute":237,"icirc":238,"iuml":239,"eth":240,"ntilde":241,"ograve":242,"oacute":243,"ocirc":244,"otilde":245,"ouml":246,"divide":247,"oslash":248,"ugrave":249,"uacute":250,"ucirc":251,"uuml":252,"yacute":253,"thorn":254,"yuml":255,"fnof":402,"Alpha":913,"Beta":914,"Gamma":915,"Delta":916,"Epsilon":917,"Zeta":918,"Eta":919,"Theta":920,"Iota":921,"Kappa":922,"Lambda":923,"Mu":924,"Nu":925,"Xi":926,"Omicron":927,"Pi":928,"Rho":929,"Sigma":931,"Tau":932,"Upsilon":933,"Phi":934,"Chi":935,"Psi":936,"Omega":937,"alpha":945,"beta":946,"gamma":947,"delta":948,"epsilon":949,"zeta":950,"eta":951,"theta":952,"iota":953,"kappa":954,"lambda":955,"mu":956,"nu":957,"xi":958,"omicron":959,"pi":960,"rho":961,"sigmaf":962,"sigma":963,"tau":964,"upsilon":965,"phi":966,"chi":967,"psi":968,"omega":969,"thetasym":977,"upsih":978,"piv":982,"bull":8226,"hellip":8230,"prime":8242,"Prime":8243,"oline":8254,"frasl":8260,"weierp":8472,"image":8465,"real":8476,"trade":8482,"alefsym":8501,"larr":8592,"uarr":8593,"rarr":8594,"darr":8595,"harr":8596,"crarr":8629,"lArr":8656,"uArr":8657,"rArr":8658,"dArr":8659,"hArr":8660,"forall":8704,"part":8706,"exist":8707,"empty":8709,"nabla":8711,"isin":8712,"notin":8713,"ni":8715,"prod":8719,"sum":8721,"minus":8722,"lowast":8727,"radic":8730,"prop":8733,"infin":8734,"ang":8736,"and":8743,"or":8744,"cap":8745,"cup":8746,"int":8747,"there4":8756,"sim":8764,"cong":8773,"asymp":8776,"ne":8800,"equiv":8801,"le":8804,"ge":8805,"sub":8834,"sup":8835,"sube":8838,"supe":8839,"oplus":8853,"otimes":8855,"perp":8869,"sdot":8901,"lceil":8968,"rceil":8969,"lfloor":8970,"rfloor":8971,"lang":9001,"rang":9002,"loz":9674,"spades":9824,"clubs":9827,"hearts":9829,"diams":9830,"OElig":338,"oelig":339,"Scaron":352,"scaron":353,"Yuml":376,"circ":710,"tilde":732,"ensp":8194,"emsp":8195,"thinsp":8201,"zwnj":8204,"zwj":8205,"lrm":8206,"rlm":8207,"ndash":8211,"mdash":8212,"lsquo":8216,"rsquo":8217,"sbquo":8218,"ldquo":8220,"rdquo":8221,"bdquo":8222,"dagger":8224,"Dagger":8225,"permil":8240,"lsaquo":8249,"rsaquo":8250,"euro":8364},escape:function(h){return qx.util.StringEscape.escape(h,qx.bom.String.FROM_CHARCODE);},unescape:function(i){return qx.util.StringEscape.unescape(i,qx.bom.String.TO_CHARCODE);},fromText:function(j){return qx.bom.String.escape(j).replace(/(  |\n)/g,function(l){var k={"  ":c,"\n":d};return k[l]||l;});},toText:function(m){return qx.bom.String.unescape(m.replace(/\s+|<([^>])+>/gi,function(n){if(n.indexOf(a)===0){return f;}else if(n.length>0&&n.replace(/^\s*/,b).replace(/\s*$/,b)==b){return e;}else {return b;};}));}},defer:function(o){o.FROM_CHARCODE=qx.lang.Object.invert(o.TO_CHARCODE);}});})();(function(){var a="&",b='X',c="",d='#',e="&#",f=";",g="qx.util.StringEscape";qx.Bootstrap.define(g,{statics:{escape:function(o,m){var j,n=c;for(var i=0,l=o.length;i<l;i++ ){var k=o.charAt(i);var h=k.charCodeAt(0);if(m[h]){j=a+m[h]+f;}else {if(h>0x7F){j=e+h+f;}else {j=k;};};n+=j;};return n;},unescape:function(p,q){return p.replace(/&[#\w]+;/gi,function(t){var r=t;var t=t.substring(1,t.length-1);var s=q[t];if(s){r=String.fromCharCode(s);}else {if(t.charAt(0)==d){if(t.charAt(1).toUpperCase()==b){s=t.substring(2);if(s.match(/^[0-9A-Fa-f]+$/gi)){r=String.fromCharCode(parseInt(s,16));};}else {s=t.substring(1);if(s.match(/^\d+$/gi)){r=String.fromCharCode(parseInt(s,10));};};};};return r;});}}});})();(function(){var a="qx.util.format.IFormat";qx.Interface.define(a,{members:{format:function(b){},parse:function(c){}}});})();(function(){var a="-",b="Boolean",c='{0,1}[0-9]{3}){0,}){0,1}',d=")?$",e="Number string '",f="' does not match the number format",g="Wrong argument type. String is expected.",h="0",i="^(",j="changeLocale",k=')?([-+]){0,1}',l='(',m="-Infinity",n="String",o="",p='\\d+){0,1}(',q='([0-9]{1,3}(?:',r="g",s="qx.util.format.NumberFormat",t="qx.dynlocale",u="locale",v="Wrong number of arguments.",w="Number",x="NaN",y="changeNumberFormat",z=".",A="Infinity";qx.Class.define(s,{extend:qx.core.Object,implement:qx.util.format.IFormat,construct:function(B){qx.core.Object.call(this);if(arguments.length>0){if(arguments.length===1){if(qx.lang.Type.isString(B)){this.setLocale(B);}else {throw new Error(g);};}else {throw new Error(v);};};if(!B){this.setLocale(qx.locale.Manager.getInstance().getLocale());if(qx.core.Environment.get(t)){qx.locale.Manager.getInstance().bind(u,this,u);};};},properties:{minimumIntegerDigits:{check:w,init:0},maximumIntegerDigits:{check:w,nullable:true},minimumFractionDigits:{check:w,init:0},maximumFractionDigits:{check:w,nullable:true},groupingUsed:{check:b,init:true},prefix:{check:n,init:o,event:y},postfix:{check:n,init:o,event:y},locale:{check:n,init:null,event:j}},members:{format:function(H){if(isNaN(H)){return x;};switch(H){case Infinity:return A;case -Infinity:return m;};var F=(H<0);if(F){H=-H;};if(this.getMaximumFractionDigits()!=null){var N=Math.pow(10,this.getMaximumFractionDigits());H=Math.round(H*N)/N;};var L=String(Math.floor(H)).length;var C=o+H;var J=C.substring(0,L);while(J.length<this.getMinimumIntegerDigits()){J=h+J;};if(this.getMaximumIntegerDigits()!=null&&J.length>this.getMaximumIntegerDigits()){J=J.substring(J.length-this.getMaximumIntegerDigits());};var I=C.substring(L+1);while(I.length<this.getMinimumFractionDigits()){I+=h;};if(this.getMaximumFractionDigits()!=null&&I.length>this.getMaximumFractionDigits()){I=I.substring(0,this.getMaximumFractionDigits());};if(this.getGroupingUsed()){var E=J;J=o;var M;for(M=E.length;M>3;M-=3){J=o+qx.locale.Number.getGroupSeparator(this.getLocale())+E.substring(M-3,M)+J;};J=E.substring(0,M)+J;};var G=this.getPrefix()?this.getPrefix():o;var D=this.getPostfix()?this.getPostfix():o;var K=G+(F?a:o)+J;if(I.length>0){K+=o+qx.locale.Number.getDecimalSeparator(this.getLocale())+I;};K+=D;return K;},parse:function(W){var S=qx.lang.String.escapeRegexpChars(qx.locale.Number.getGroupSeparator(this.getLocale())+o);var Q=qx.lang.String.escapeRegexpChars(qx.locale.Number.getDecimalSeparator(this.getLocale())+o);var O=new RegExp(i+qx.lang.String.escapeRegexpChars(this.getPrefix())+k+q+S+c+l+Q+p+qx.lang.String.escapeRegexpChars(this.getPostfix())+d);var R=O.exec(W);if(R==null){throw new Error(e+W+f);};var T=(R[2]==a);var V=R[3]||h;var U=R[4];V=V.replace(new RegExp(S,r),o);var P=(T?a:o)+V;if(U!=null&&U.length!=0){U=U.replace(new RegExp(Q),o);P+=z+U;};return parseFloat(P);}},destruct:function(){if(qx.core.Environment.get(t)){qx.locale.Manager.getInstance().removeRelatedBindings(this);};}});})();(function(){var a="cldr_number_decimal_separator",b="cldr_number_percent_format",c="qx.locale.Number",d="cldr_number_group_separator";qx.Class.define(c,{statics:{getDecimalSeparator:function(e){return qx.locale.Manager.getInstance().localize(a,[],e);},getGroupSeparator:function(f){return qx.locale.Manager.getInstance().localize(d,[],f);},getPercentFormat:function(g){return qx.locale.Manager.getInstance().localize(b,[],g);}}});})();(function(){var a="-",b="QQQ",c='W',d="qqqq",e="h",f="KK",g='L',h=" is not supported",m="Z",n="Date string '",o="GGGGG",p='abbreviated',q="Y+",r=". Wildcard ",s="eeeee",t="(\\d\\d?)",u="LL",v="abbreviated",w="EEEE",x="^",y=":",z='y',A="quoted_literal",B="weekOfYear",C="locale",D="SSS",E='a',F="day",G="(-*",H="qqq",I="(\\d\\d)",J='3rd quarter',K="K",L='q',M="pm",N="Error parsing date '",O='Anno Domini',P='G',Q="a",R='2nd quarter',S="yyyy-MM-dd'T'HH:mm:ss",T="GMT",U="HH:mm:ss",V="dd",W="+",X="qx.util.format.DateFormat",Y='weekDay',el="am",eg="eee",em="quarter",ei="ee",ej="q",ef="QQQQ",ek="HHmmss",eq="ww",er="H",es='Y+',et="wide",en='wide',eo="(\\d\\d\\d?)",eh="(GMT[\\+\\-]\\d\\d:\\d\\d)",ep="_applyLocale",ex="Q",fb="c",ey="(",ez="weekOfMonth",eu="ccc",ev="long",gc="Malformed date format: ",ew="EEEEE",eA='Y',eB="EE",eC="GGG",eH='-',eI="w",eJ="mm",eD='h',eE="S",eF='s',eG='A',eO="yyyy-MM-dd",eP="G",eQ='z',eR="GG",eK=")",eL="kk",eM="ss",eN="([\\+\\-]\\d\\d\\d\\d)",eV="default",eW='BC',gg='H',eX='S',eS='1st quarter',eT="stand-alone",ge="0",eU="MMMM",eY='c',fa="d",fm='Q',fl='Q1',fk="eeee",fq="QQ",fp="LLLLL",fo="HH",fn='k',ff="m",fe="DDD",fd="D",fc='D',fj="LLLL",fi="(\\d\\d*?)",fh='e',fg="': the value for day or month is too large",fx="literal",fw='K',fv="L",fu="hh",fB="ms",fA="(\\d\\d?\\d?)",fz="cccc",fy="e",ft='y+',fs="|",fr="W",fM="dayOfYear",fL="GGGG",fK='Q2',fQ='B',fP="MM",fO='Q4',fN="y+",fF='narrow',fE="ccccc",fD="E",fC="wildcard",fJ='',fI="' does not match the date format: ",fH="short",fG='d',fW="unkown",fV="narrow",fU="\\d?",fT="'",gb="k",ga='Before Christ',fY='Z',fX='m',fS=" ",fR="yyyy-MM-dd'T'HH:mm:ss'Z'",dP="String",dO='Q3',gh="z",dM="DD",dN="min",dL="",gf='M',dJ="sec",dK='isoUtcDateTime',dI='4th quarter',gd="(\\d?)",dG="SS",dH="MMM",dF="s",dY="M",ea='w',dW="EEE",dX="$",dU="?",dV='AD',dT="\\d",dE="+?",dR="hour",dS="MMMMM",dQ='E',ee='month',ec="LLL",ed="qq",eb="format";qx.Class.define(X,{extend:qx.core.Object,implement:qx.util.format.IFormat,construct:function(gj,gi){qx.core.Object.call(this);if(!gi){this.__ll=qx.locale.Manager.getInstance().getLocale();this.__pt=qx.locale.Manager.getInstance().bind(C,this,C);}else {this.__ll=gi;this.setLocale(gi);};this.__pu=this.__ll;if(gj!=null){this.__pv=gj.toString();if(this.__pv in qx.util.format.DateFormat.ISO_MASKS){if(this.__pv===dK){this.__pw=true;};this.__pv=qx.util.format.DateFormat.ISO_MASKS[this.__pv];};}else {this.__pv=qx.locale.Date.getDateFormat(ev,this.__ll)+fS+qx.locale.Date.getDateTimeFormat(ek,U,this.__ll);};},properties:{locale:{apply:ep,nullable:true,check:dP}},statics:{getDateInstance:function(){var gk=qx.util.format.DateFormat;var gl=qx.locale.Date.getDateFormat(fH)+dL;if(gk._dateInstance==null||gk._dateInstance.__pv!=gl){gk._dateInstance=new gk(gl);};return gk._dateInstance;},getDateTimeInstance:function(){var gm=qx.util.format.DateFormat;var gn=qx.locale.Date.getDateFormat(ev)+fS+qx.locale.Date.getDateTimeFormat(ek,U);if(gm._dateTimeInstance==null||gm._dateTimeInstance.__pv!=gn){gm._dateTimeInstance=new gm(gn);};return gm._dateTimeInstance;},ASSUME_YEAR_2000_THRESHOLD:30,ISO_MASKS:{isoDate:eO,isoTime:U,isoDateTime:S,isoUtcDateTime:fR},AM_MARKER:el,PM_MARKER:M},members:{__pt:null,__ll:null,__pu:null,__pv:null,__px:null,__py:null,__pz:null,__pw:null,__pA:function(gp,go){var gq=dL+(gp<0?(-1*gp):gp);while(gq.length<go){gq=ge+gq;};return gp<0?a+gq:gq;},__pB:function(gr){var gs=new Date(gr.getTime());var gt=gs.getDate();while(gs.getMonth()!=0){gs.setDate(-1);gt+=gs.getDate()+1;};return gt;},__pC:function(gu){return new Date(gu.getTime()+(3-((gu.getDay()+6)%7))*86400000);},__pD:function(gv){var gx=this.__pC(gv);var gy=gx.getFullYear();var gw=this.__pC(new Date(gy,0,4));return Math.floor(1.5+(gx.getTime()-gw.getTime())/86400000/7);},__pE:function(gz){var gB=this.__pC(gz);var gA=this.__pC(new Date(gz.getFullYear(),gz.getMonth(),4));return Math.floor(1.5+(gB.getTime()-gA.getTime())/86400000/7);},__pF:function(gC){var gD=this.__pC(gC);return gD.getFullYear();},__pG:function(gF){var gE=new Date(gF,2,1);gE.setDate(-1);return gE.getDate()+1===29;},__pH:function(gK,gL){var gJ=0;var gM=0;if(!gL){gL=1971;};var gG=0;for(var i=1;i<=12;i++ ){var gI=new Date(gL,i,1);gI.setDate(-1);var gH=gI.getDate()+1;gG+=gH;if(gG<gK){gJ++ ;gM+=gH;}else {gM=gK-(gG-gH);break;};};return {month:gJ,day:gM};},__pI:function(gQ,gO,gN){var gP;switch(gO){case 11:gP=gQ-1;if(gQ!=this.__pF(new Date(gP,gO,gN))){gP=gQ;};break;case 0:gP=gQ+1;if(gQ!=this.__pF(new Date(gP,gO,gN))){gP=gQ;};break;default:gP=gQ;};return gP;},_applyLocale:function(gS,gR){this.__ll=gS===null?this.setLocale(this.__pu):gS;},format:function(hj){if(hj==null){return null;};if(isNaN(hj.getTime())){{};return null;};if(this.__pw){hj=new Date(hj.getUTCFullYear(),hj.getUTCMonth(),hj.getUTCDate(),hj.getUTCHours(),hj.getUTCMinutes(),hj.getUTCSeconds(),hj.getUTCMilliseconds());};var gV=this.__ll;var hb=hj.getFullYear();var gW=hj.getMonth();var ha=hj.getDate();var he=hj.getDay();var hh=hj.getHours();var ho=hj.getMinutes();var hf=hj.getSeconds();var hm=hj.getMilliseconds();var hd=hj.getTimezoneOffset();var hi=hd>0?1:-1;var gT=Math.floor(Math.abs(hd)/60);var hl=Math.abs(hd)%60;this.__pJ();var hn=dL;for(var i=0;i<this.__pz.length;i++ ){var hc=this.__pz[i];if(hc.type==fx){hn+=hc.text;}else {var gU=hc.character;var hg=hc.size;var hp=dU;switch(gU){case z:if(hg==2){hp=this.__pA(hb%100,2);}else {var gX=Math.abs(hb);hp=gX+dL;if(hg>hp.length){for(var j=hp.length;j<hg;j++ ){hp=ge+hp;};};if(hb<0){hp=a+hp;};};break;case eA:hp=this.__pF(hj)+dL;var gX=hp.replace(eH,fJ);if(hg>hp.length){for(var j=gX.length;j<hg;j++ ){gX=ge+gX;};};hp=hp.indexOf(a)!=-1?a+gX:gX;break;case P:if(hg>=1&&hg<=3){hp=hb>0?dV:eW;}else if(hg==4){hp=hb>0?O:ga;}else if(hg==5){hp=hb>0?eG:fQ;};break;case fm:if(hg==1||hg==2){hp=this.__pA(parseInt(gW/4)+1,hg);};if(hg==3){hp=fm+(parseInt(gW/4)+1);};break;case L:if(hg==1||hg==2){hp=this.__pA(parseInt(gW/4)+1,hg);};if(hg==3){hp=fm+(parseInt(gW/4)+1);};break;case fc:hp=this.__pA(this.__pB(hj),hg);break;case fG:hp=this.__pA(ha,hg);break;case ea:hp=this.__pA(this.__pD(hj),hg);break;case c:hp=this.__pE(hj);break;case dQ:if(hg>=1&&hg<=3){hp=qx.locale.Date.getDayName(v,he,gV,eb,true);}else if(hg==4){hp=qx.locale.Date.getDayName(et,he,gV,eb,true);}else if(hg==5){hp=qx.locale.Date.getDayName(fV,he,gV,eb,true);};break;case fh:var gY=qx.locale.Date.getWeekStart(gV);var hk=1+((he-gY>=0)?(he-gY):7+(he-gY));if(hg>=1&&hg<=2){hp=this.__pA(hk,hg);}else if(hg==3){hp=qx.locale.Date.getDayName(v,he,gV,eb,true);}else if(hg==4){hp=qx.locale.Date.getDayName(et,he,gV,eb,true);}else if(hg==5){hp=qx.locale.Date.getDayName(fV,he,gV,eb,true);};break;case eY:var gY=qx.locale.Date.getWeekStart(gV);var hk=1+((he-gY>=0)?(he-gY):7+(he-gY));if(hg==1){hp=fJ+hk;}else if(hg==3){hp=qx.locale.Date.getDayName(v,he,gV,eT,true);}else if(hg==4){hp=qx.locale.Date.getDayName(et,he,gV,eT,true);}else if(hg==5){hp=qx.locale.Date.getDayName(fV,he,gV,eT,true);};break;case gf:if(hg==1||hg==2){hp=this.__pA(gW+1,hg);}else if(hg==3){hp=qx.locale.Date.getMonthName(v,gW,gV,eb,true);}else if(hg==4){hp=qx.locale.Date.getMonthName(et,gW,gV,eb,true);}else if(hg==5){hp=qx.locale.Date.getMonthName(fV,gW,gV,eb,true);};break;case g:if(hg==1||hg==2){hp=this.__pA(gW+1,hg);}else if(hg==3){hp=qx.locale.Date.getMonthName(v,gW,gV,eT,true);}else if(hg==4){hp=qx.locale.Date.getMonthName(et,gW,gV,eT,true);}else if(hg==5){hp=qx.locale.Date.getMonthName(fV,gW,gV,eT,true);};break;case E:hp=(hh<12)?qx.locale.Date.getAmMarker(gV):qx.locale.Date.getPmMarker(gV);break;case gg:hp=this.__pA(hh,hg);break;case fn:hp=this.__pA((hh==0)?24:hh,hg);break;case fw:hp=this.__pA(hh%12,hg);break;case eD:hp=this.__pA(((hh%12)==0)?12:(hh%12),hg);break;case fX:hp=this.__pA(ho,hg);break;case eF:hp=this.__pA(hf,hg);break;case eX:hp=this.__pA(hm,3);if(hg<hp.length){hp=hp.substr(0,hg);}else {while(hg>hp.length){hp+=ge;};};break;case eQ:if(hg>=1&&hg<=4){hp=T+((hi>0)?a:W)+this.__pA(Math.abs(gT),2)+y+this.__pA(hl,2);};break;case fY:if(hg>=1&&hg<=3){hp=((hi>0)?a:W)+this.__pA(Math.abs(gT),2)+this.__pA(hl,2);}else {hp=T+((hi>0)?a:W)+this.__pA(Math.abs(gT),2)+y+this.__pA(hl,2);};break;};hn+=hp;};};return hn;},parse:function(hw){this.__pK();var hz=this.__px.regex.exec(hw);if(hz==null){throw new Error(n+hw+fI+this.__pv);};var hq={era:1,year:1970,quarter:1,month:0,day:1,dayOfYear:1,hour:0,ispm:false,weekDay:4,weekYear:1970,weekOfMonth:1,weekOfYear:1,min:0,sec:0,ms:0};var hu=1;var hr=false;var hs=false;for(var i=0;i<this.__px.usedRules.length;i++ ){var ht=this.__px.usedRules[i];var hv=hz[hu];if(ht.field!=null){hq[ht.field]=parseInt(hv,10);}else {ht.manipulator(hq,hv,ht.pattern);};if(ht.pattern==q){var hx=false;for(var k=0;k<this.__px.usedRules.length;k++ ){if(this.__px.usedRules[k].pattern==ft){hx=true;break;};};if(!hx){hr=true;};};if(ht.pattern.indexOf(fd)!=-1){var hB=false;for(var k=0;k<this.__px.usedRules.length;k++ ){if(this.__px.usedRules[k].pattern.indexOf(fa)!=-1){hB=true;break;};};if(!hB){hs=true;};};hu+=(ht.groups==null)?1:ht.groups;};if(hr){hq.year=this.__pI(hq.weekYear,hq.month,hq.day);};if(hs){var hA=this.__pH(hq.dayOfYear,hq.year);hq.month=hA.month;hq.day=hA.day;};if(hq.era<0&&(hq.year*hq.era<0)){hq.year=hq.year*hq.era;};var hy=new Date(hq.year,hq.month,hq.day,(hq.ispm)?(hq.hour+12):hq.hour,hq.min,hq.sec,hq.ms);if(this.__pw){hy=new Date(hy.getUTCFullYear(),hy.getUTCMonth(),hy.getUTCDate(),hy.getUTCHours(),hy.getUTCMinutes(),hy.getUTCSeconds(),hy.getUTCMilliseconds());};if(hq.month!=hy.getMonth()||hq.year!=hy.getFullYear()){throw new Error(N+hw+fg);};return hy;},__pJ:function(){if(this.__pz!=null){return;};this.__pz=[];var hG;var hE=0;var hI=dL;var hC=this.__pv;var hF=eV;var i=0;while(i<hC.length){var hH=hC.charAt(i);switch(hF){case A:if(hH==fT){if(i+1>=hC.length){i++ ;break;};var hD=hC.charAt(i+1);if(hD==fT){hI+=hH;i++ ;}else {i++ ;hF=fW;};}else {hI+=hH;i++ ;};break;case fC:if(hH==hG){hE++ ;i++ ;}else {this.__pz.push({type:fC,character:hG,size:hE});hG=null;hE=0;hF=eV;};break;default:if((hH>=E&&hH<=eQ)||(hH>=eG&&hH<=fY)){hG=hH;hF=fC;}else if(hH==fT){if(i+1>=hC.length){hI+=hH;i++ ;break;};var hD=hC.charAt(i+1);if(hD==fT){hI+=hH;i++ ;};i++ ;hF=A;}else {hF=eV;};if(hF!=eV){if(hI.length>0){this.__pz.push({type:fx,text:hI});hI=dL;};}else {hI+=hH;i++ ;};break;};};if(hG!=null){this.__pz.push({type:fC,character:hG,size:hE});}else if(hI.length>0){this.__pz.push({type:fx,text:hI});};},__pK:function(){if(this.__px!=null){return;};var hM=this.__pv;this.__pM();this.__pJ();var hS=[];var hN=x;for(var hK=0;hK<this.__pz.length;hK++ ){var hT=this.__pz[hK];if(hT.type==fx){hN+=qx.lang.String.escapeRegexpChars(hT.text);}else {var hL=hT.character;var hO=hT.size;var hQ;for(var hU=0;hU<this.__py.length;hU++ ){var hP=this.__py[hU];if(this.__pL(hP,hL,hO)){hQ=hP;break;};};if(hQ==null){var hR=dL;for(var i=0;i<hO;i++ ){hR+=hL;};throw new Error(gc+hM+r+hR+h);}else {hS.push(hQ);hN+=hQ.regex;};};};hN+=dX;var hJ;try{hJ=new RegExp(hN);}catch(hV){throw new Error(gc+hM);};this.__px={regex:hJ,"usedRules":hS,pattern:hN};},__pL:function(hW,hX,hY){if(hX===z&&hW.pattern===ft){hW.regex=hW.regexFunc(hY);return true;}else if(hX===eA&&hW.pattern===es){hW.regex=hW.regexFunc(hY);return true;}else {return hX==hW.pattern.charAt(0)&&hY==hW.pattern.length;};},__pM:function(){var ip=qx.util.format.DateFormat;var is=qx.lang.String;if(this.__py!=null){return;};var iq=this.__py=[];var ik=qx.locale.Date.getAmMarker(this.__ll).toString()||ip.AM_MARKER;var iz=qx.locale.Date.getPmMarker(this.__ll).toString()||ip.PM_MARKER;var ib=this.__ll;var ih=function(iC,iD){iD=parseInt(iD,10);if(iD>=0){if(iD<ip.ASSUME_YEAR_2000_THRESHOLD){iD+=2000;}else if(iD<100){iD+=1900;};};iC.year=iD;};var iB=function(iE,iF){iF=parseInt(iF,10);if(iF>=0){if(iF<ip.ASSUME_YEAR_2000_THRESHOLD){iF+=2000;}else if(iF<100){iF+=1900;};};iE.weekYear=iF;};var ij=function(iG,iH){iG.month=parseInt(iH,10)-1;};var ie=function(iK,iL){var iJ=qx.locale.Date.getWeekStart(ib);var iI=(parseInt(iL,10)-1+iJ)<=6?parseInt(iL,10)-1+iJ:(parseInt(iL,10)-1+iJ)-7;iK.weekDay=iI;};var id=function(iM,iN){var iO=qx.locale.Date.getPmMarker(ib).toString()||ip.PM_MARKER;iM.ispm=(iN==iO);};var ir=function(iP,iQ){iP.hour=parseInt(iQ,10)%24;};var ic=function(iR,iS){iR.hour=parseInt(iS,10)%12;};var iu=function(iT,iU){return;};var iv=[eG,fQ];var iw=function(iV,iW){iV.era=iW==eG?1:-1;};var io=[dV,eW];var im=function(iX,iY){iX.era=iY==dV?1:-1;};var iy=[O,ga];var il=function(ja,jb){ja.era=jb==O?1:-1;};var iA=[fl,fK,dO,fO];var ia=function(jc,jd){jc.quarter=iA.indexOf(jd);};var ix=[eS,R,J,dI];var it=function(je,jf){je.quarter=ix.indexOf(jf);};var ii={};var ig=function(jj){var jl=[g,gf];var jh=[eY,fh,dQ];var jo=jj.charAt(0);var ji=jl.indexOf(jo)>=0;var jk=function(){var jq=ji?jl:jh;var jr=jo===jq[0]?eT:eb;var jp=jj.length;var js=p;switch(jp){case 4:js=en;break;case 5:js=fF;break;default:js=p;};return [jr,js];};if(!ii[jj]){ii[jj]={};var jg=jk();var jn=ji?qx.locale.Date.getMonthNames:qx.locale.Date.getDayNames;var jm=jn.call(qx.locale.Date,jg[1],ib,jg[0],true);for(var i=0,l=jm.length;i<l;i++ ){jm[i]=is.escapeRegexpChars(jm[i].toString());};ii[jj].data=jm;ii[jj].func=function(jt,ju){ju=is.escapeRegexpChars(ju);jt[ji?ee:Y]=jm.indexOf(ju);};};return ii[jj];};iq.push({pattern:fN,regexFunc:function(jw){var jv=G;for(var i=0;i<jw;i++ ){jv+=dT;if(i===jw-1&&i!==1){jv+=dE;};};jv+=eK;return jv;},manipulator:ih});iq.push({pattern:q,regexFunc:function(jy){var jx=G;for(var i=0;i<jy;i++ ){jx+=dT;if(i===jy-1){jx+=dE;};};jx+=eK;return jx;},manipulator:iB});iq.push({pattern:eP,regex:ey+io.join(fs)+eK,manipulator:im});iq.push({pattern:eR,regex:ey+io.join(fs)+eK,manipulator:im});iq.push({pattern:eC,regex:ey+io.join(fs)+eK,manipulator:im});iq.push({pattern:fL,regex:ey+iy.join(fs)+eK,manipulator:il});iq.push({pattern:o,regex:ey+iv.join(fs)+eK,manipulator:iw});iq.push({pattern:ex,regex:fi,field:em});iq.push({pattern:fq,regex:t,field:em});iq.push({pattern:b,regex:ey+iA.join(fs)+eK,manipulator:ia});iq.push({pattern:ef,regex:ey+ix.join(fs)+eK,manipulator:it});iq.push({pattern:ej,regex:fi,field:em});iq.push({pattern:ed,regex:t,field:em});iq.push({pattern:H,regex:ey+iA.join(fs)+eK,manipulator:ia});iq.push({pattern:d,regex:ey+ix.join(fs)+eK,manipulator:it});iq.push({pattern:dY,regex:fi,manipulator:ij});iq.push({pattern:fP,regex:t,manipulator:ij});iq.push({pattern:dH,regex:ey+ig(dH).data.join(fs)+eK,manipulator:ig(dH).func});iq.push({pattern:eU,regex:ey+ig(eU).data.join(fs)+eK,manipulator:ig(eU).func});iq.push({pattern:dS,regex:ey+ig(dS).data.join(fs)+eK,manipulator:ig(dS).func});iq.push({pattern:fv,regex:fi,manipulator:ij});iq.push({pattern:u,regex:t,manipulator:ij});iq.push({pattern:ec,regex:ey+ig(ec).data.join(fs)+eK,manipulator:ig(ec).func});iq.push({pattern:fj,regex:ey+ig(fj).data.join(fs)+eK,manipulator:ig(fj).func});iq.push({pattern:fp,regex:ey+ig(fp).data.join(fs)+eK,manipulator:ig(fp).func});iq.push({pattern:V,regex:t,field:F});iq.push({pattern:fa,regex:fi,field:F});iq.push({pattern:fd,regex:gd,field:fM});iq.push({pattern:dM,regex:t,field:fM});iq.push({pattern:fe,regex:eo,field:fM});iq.push({pattern:fD,regex:ey+ig(fD).data.join(fs)+eK,manipulator:ig(fD).func});iq.push({pattern:eB,regex:ey+ig(eB).data.join(fs)+eK,manipulator:ig(eB).func});iq.push({pattern:dW,regex:ey+ig(dW).data.join(fs)+eK,manipulator:ig(dW).func});iq.push({pattern:w,regex:ey+ig(w).data.join(fs)+eK,manipulator:ig(w).func});iq.push({pattern:ew,regex:ey+ig(ew).data.join(fs)+eK,manipulator:ig(ew).func});iq.push({pattern:fy,regex:gd,manipulator:ie});iq.push({pattern:ei,regex:t,manipulator:ie});iq.push({pattern:eg,regex:ey+ig(eg).data.join(fs)+eK,manipulator:ig(eg).func});iq.push({pattern:fk,regex:ey+ig(fk).data.join(fs)+eK,manipulator:ig(fk).func});iq.push({pattern:s,regex:ey+ig(s).data.join(fs)+eK,manipulator:ig(s).func});iq.push({pattern:fb,regex:fU,manipulator:ie});iq.push({pattern:eu,regex:ey+ig(eu).data.join(fs)+eK,manipulator:ig(eu).func});iq.push({pattern:fz,regex:ey+ig(fz).data.join(fs)+eK,manipulator:ig(fz).func});iq.push({pattern:fE,regex:ey+ig(fE).data.join(fs)+eK,manipulator:ig(fE).func});iq.push({pattern:Q,regex:ey+ik+fs+iz+eK,manipulator:id});iq.push({pattern:fr,regex:gd,field:ez});iq.push({pattern:eI,regex:t,field:B});iq.push({pattern:eq,regex:I,field:B});iq.push({pattern:fo,regex:t,field:dR});iq.push({pattern:er,regex:t,field:dR});iq.push({pattern:eL,regex:t,manipulator:ir});iq.push({pattern:gb,regex:t,manipulator:ir});iq.push({pattern:f,regex:t,field:dR});iq.push({pattern:K,regex:t,field:dR});iq.push({pattern:fu,regex:t,manipulator:ic});iq.push({pattern:e,regex:t,manipulator:ic});iq.push({pattern:eJ,regex:t,field:dN});iq.push({pattern:ff,regex:t,field:dN});iq.push({pattern:eM,regex:t,field:dJ});iq.push({pattern:dF,regex:t,field:dJ});iq.push({pattern:D,regex:fA,field:fB});iq.push({pattern:dG,regex:fA,field:fB});iq.push({pattern:eE,regex:fA,field:fB});iq.push({pattern:m,regex:eN,manipulator:iu});iq.push({pattern:gh,regex:eh,manipulator:iu});}},destruct:function(){if(this.__pt!=null){qx.locale.Manager.getInstance().removeBinding(this.__pt);};this.__pz=this.__px=this.__py=null;}});})();(function(){var a="format",b='format',c="short",d="HH:mm",e='_',f="HHmmsszz",g="HHmm",h="HHmmss",j="thu",k="This case should never happen.",l="sat",m="cldr_date_format_",n="HH:mm:ss zz",o="full",p="cldr_pm",q="cldr_day_",r="_",s="cldr_month_",t="long",u="wed",v="medium",w="fri",x="cldr_am",y="tue",z="qx.locale.Date",A="mon",B="cldr_date_time_format_",C="sun",D="cldr_time_format_",E="HH:mm:ss",F='stand-alone';qx.Class.define(z,{statics:{__pN:qx.locale.Manager.getInstance(),getAmMarker:function(G){return this.__pN.localize(x,[],G);},getPmMarker:function(H){return this.__pN.localize(p,[],H);},getDayNames:function(length,J,N,K){var N=N?N:a;{};var L=[C,A,y,u,j,w,l];var M=[];for(var i=0;i<L.length;i++ ){var I=q+N+r+length+r+L[i];M.push(K?this.__pO(N,N===b?F:b,I,J):this.__pN.localize(I,[],J));};return M;},getDayName:function(length,T,Q,P,S){var P=P?P:a;{};var R=[C,A,y,u,j,w,l];var O=q+P+r+length+r+R[T];return S?this.__pO(P,P===b?F:b,O,Q):this.__pN.localize(O,[],Q);},getMonthNames:function(length,V,Y,W){var Y=Y?Y:a;{};var X=[];for(var i=0;i<12;i++ ){var U=s+Y+r+length+r+(i+1);X.push(W?this.__pO(Y,Y===b?F:b,U,V):this.__pN.localize(U,[],V));};return X;},getMonthName:function(length,bd,bc,ba,bb){var ba=ba?ba:a;{};var be=s+ba+r+length+r+(bd+1);return bb?this.__pO(ba,ba===b?F:b,be,bc):this.__pN.localize(be,[],bc);},getDateFormat:function(bh,bf){{};var bg=m+bh;return this.__pN.localize(bg,[],bf);},getDateTimeFormat:function(bm,bk,bi){var bl=B+bm;var bj=this.__pN.localize(bl,[],bi);if(bj==bl){bj=bk;};return bj;},getTimeFormat:function(bq,bn){{};var bp=D+bq;var bo=this.__pN.localize(bp,[],bn);if(bo!=bp){return bo;};switch(bq){case c:case v:return qx.locale.Date.getDateTimeFormat(g,d);case t:return qx.locale.Date.getDateTimeFormat(h,E);case o:return qx.locale.Date.getDateTimeFormat(f,n);default:throw new Error(k);};},getWeekStart:function(br){var bs={"MV":5,"AE":6,"AF":6,"BH":6,"DJ":6,"DZ":6,"EG":6,"ER":6,"ET":6,"IQ":6,"IR":6,"JO":6,"KE":6,"KW":6,"LB":6,"LY":6,"MA":6,"OM":6,"QA":6,"SA":6,"SD":6,"SO":6,"TN":6,"YE":6,"AS":0,"AU":0,"AZ":0,"BW":0,"CA":0,"CN":0,"FO":0,"GE":0,"GL":0,"GU":0,"HK":0,"IE":0,"IL":0,"IS":0,"JM":0,"JP":0,"KG":0,"KR":0,"LA":0,"MH":0,"MN":0,"MO":0,"MP":0,"MT":0,"NZ":0,"PH":0,"PK":0,"SG":0,"TH":0,"TT":0,"TW":0,"UM":0,"US":0,"UZ":0,"VI":0,"ZA":0,"ZW":0,"MW":0,"NG":0,"TJ":0};var bt=qx.locale.Date._getTerritory(br);return bs[bt]!=null?bs[bt]:1;},getWeekendStart:function(bu){var bw={"EG":5,"IL":5,"SY":5,"IN":0,"AE":4,"BH":4,"DZ":4,"IQ":4,"JO":4,"KW":4,"LB":4,"LY":4,"MA":4,"OM":4,"QA":4,"SA":4,"SD":4,"TN":4,"YE":4};var bv=qx.locale.Date._getTerritory(bu);return bw[bv]!=null?bw[bv]:6;},getWeekendEnd:function(bx){var by={"AE":5,"BH":5,"DZ":5,"IQ":5,"JO":5,"KW":5,"LB":5,"LY":5,"MA":5,"OM":5,"QA":5,"SA":5,"SD":5,"TN":5,"YE":5,"AF":5,"IR":5,"EG":6,"IL":6,"SY":6};var bz=qx.locale.Date._getTerritory(bx);return by[bz]!=null?by[bz]:0;},isWeekend:function(bC,bA){var bD=qx.locale.Date.getWeekendStart(bA);var bB=qx.locale.Date.getWeekendEnd(bA);if(bB>bD){return ((bC>=bD)&&(bC<=bB));}else {return ((bC>=bD)||(bC<=bB));};},_getTerritory:function(bE){if(bE){var bF=bE.split(r)[1]||bE;}else {bF=this.__pN.getTerritory()||this.__pN.getLanguage();};return bF.toUpperCase();},__pO:function(bI,bH,bL,bK){var bJ=this.__pN.localize(bL,[],bK);if(bJ==bL){var bG=bL.replace(e+bI+e,e+bH+e);return this.__pN.localize(bG,[],bK);}else {return bJ;};}}});})();(function(){var a="qx.ui.table.ICellEditorFactory";qx.Interface.define(a,{members:{createCellEditor:function(b){return true;},getCellEditorValue:function(c){return true;}}});})();(function(){var a="Function",b="abstract",c="",d="number",e="appear",f="Abstract method call!",g="qx.ui.table.celleditor.AbstractField";qx.Class.define(g,{extend:qx.core.Object,implement:qx.ui.table.ICellEditorFactory,type:b,properties:{validationFunction:{check:a,nullable:true,init:null}},members:{_createEditor:function(){throw new Error(f);},createCellEditor:function(h){var i=this._createEditor();i.originalValue=h.value;if(h.value===null||h.value===undefined){h.value=c;};i.setValue(c+h.value);i.addListener(e,function(){i.selectAllText();});return i;},getCellEditorValue:function(j){var k=j.getValue();var l=this.getValidationFunction();if(l){k=l(k,j.originalValue);};if(typeof j.originalValue==d){k=parseFloat(k);};return k;}}});})();(function(){var a="number",b="qx.ui.table.celleditor.TextField",c="table-editor-textfield";qx.Class.define(b,{extend:qx.ui.table.celleditor.AbstractField,members:{getCellEditorValue:function(d){var e=d.getValue();var f=this.getValidationFunction();if(f){e=f(e,d.originalValue);};if(typeof d.originalValue==a){if(e!=null){e=parseFloat(e);};};return e;},_createEditor:function(){var g=new qx.ui.form.TextField();g.setAppearance(c);return g;}}});})();(function(){var a="-ms-input-placeholder",b="engine.version",c="changeValue",d="autoComplete: off;",f=".qx-abstract-field",g="browser.name",h="textarea",i="qx.event.type.Data",j="showingPlaceholder",k="focused",l="qx-placeholder-color",m=" !important",n="border-radius: 0;",o="browser.documentmode",p="qx.ui.form.AbstractField",q="-ms-input-placeholder, textarea.qx-placeholder-color",r="input",s="qx.dynlocale",t="maxLength",u="normal",v="qx-abstract-field",w="position: absolute;",x="String",y="margin: 0;",z="readOnly",A="text",B="syncAppearance",C="changeStatus",D="A",E="input::-moz-placeholder, textarea::-moz-placeholder",F="color",G="readonly",H="resize: none;",I="display : block;",J="Invalid value type: ",K="input.qx-placeholder-color::-webkit-input-placeholder, textarea.qx-placeholder-color::-webkit-input-placeholder",L="firefox",M="visible",N=":",O="input:-moz-placeholder, textarea:-moz-placeholder",P="_applyMaxLength",Q="outline: none;",R="background : transparent;",S="browser.version",T="pointerdown",U="spellcheck",V="focusin",W="mshtml",X="css.placeholder",Y="hidden",bP="border: none;",bQ="Boolean",bR="px",bL="url(",bM=")",bN="gecko",bO="change",bW="placeholder",bX="center",bY="input.qx-placeholder-color",ca="disabled",bS="textAlign",bT="appearance: none;",bU="color: ",bV="right",ce="PositiveInteger",cr="engine.name",cs="padding: 0;",cf="_applyPlaceholder",cb="left",cc="text-placeholder",cu="_applyTextAlign",cd="RegExp",cg="changeLocale",ch="::",ci="_applyReadOnly",cm="",cv="visibility",cn="appear",cj="false",ck="abstract",ct="focusout",cl="edge",co="webkit",cp="qx/static/blank.gif",cw="changeReadOnly",cq="absolute";qx.Class.define(p,{extend:qx.ui.core.Widget,implement:[qx.ui.form.IStringForm,qx.ui.form.IForm],include:[qx.ui.form.MForm],type:ck,statics:{__ds:null,__pP:function(){var cx=qx.core.Environment.get(cr);var cC=qx.core.Environment.get(g);var cy=qx.theme.manager.Color.getInstance();var cz=cy.resolve(cc);var cA;if(cx==bN){if(parseFloat(qx.core.Environment.get(b))>=19){cA=E;}else {cA=O;};qx.ui.style.Stylesheet.getInstance().addRule(cA,bU+cz+m);}else if(cx==co&&cC!=cl){cA=K;qx.ui.style.Stylesheet.getInstance().addRule(cA,bU+cz);}else if(cx==W||cC==cl){var cB=cC==cl?ch:N;cA=[bY,q,a].join(cB);qx.ui.style.Stylesheet.getInstance().addRule(cA,bU+cz+m);};}},construct:function(cD){qx.ui.core.Widget.call(this);this.__pQ=!qx.core.Environment.get(X);if(cD!=null){this.setValue(cD);};this.getContentElement().addListener(bO,this._onChangeContent,this);if(this.__pQ){this.addListener(B,this._syncPlaceholder,this);}else {qx.ui.form.AbstractField.__pP();this.getContentElement().addClass(l);};if(qx.core.Environment.get(s)){qx.locale.Manager.getInstance().addListener(cg,this._onChangeLocale,this);};},events:{"input":i,"changeValue":i},properties:{textAlign:{check:[cb,bX,bV],nullable:true,themeable:true,apply:cu},readOnly:{check:bQ,apply:ci,event:cw,init:false},selectable:{refine:true,init:true},focusable:{refine:true,init:true},maxLength:{apply:P,check:ce,init:Infinity},liveUpdate:{check:bQ,init:false},placeholder:{check:x,nullable:true,apply:cf},filter:{check:cd,nullable:true,init:null}},members:{__pR:true,_placeholder:null,__iV:null,__iW:null,__pQ:true,__ly:null,__lB:null,getFocusElement:function(){var cE=this.getContentElement();if(cE){return cE;};},_createInputElement:function(){return new qx.html.Input(A);},renderLayout:function(cN,top,cI,cG){var cF=this._updateInsets;var cL=qx.ui.core.Widget.prototype.renderLayout.call(this,cN,top,cI,cG);if(!cL){return;};var cJ=cL.size||cF;var cM=bR;if(cJ||cL.local||cL.margin){var innerWidth=cI;var innerHeight=cG;};var cK=this.getContentElement();if(cF&&this.__pQ){if(this.__pQ){var cH=this.getInsets();this._getPlaceholderElement().setStyles({paddingTop:cH.top+cM,paddingRight:cH.right+cM,paddingBottom:cH.bottom+cM,paddingLeft:cH.left+cM});};};if(cJ||cL.margin){if(this.__pQ){var cH=this.getInsets();this._getPlaceholderElement().setStyles({"width":(innerWidth-cH.left-cH.right)+cM,"height":(innerHeight-cH.top-cH.bottom)+cM});};cK.setStyles({"width":innerWidth+cM,"height":innerHeight+cM});this._renderContentElement(innerHeight,cK);};if(cL.position){if(this.__pQ){this._getPlaceholderElement().setStyles({"left":cN+cM,"top":top+cM});};};},_renderContentElement:function(innerHeight,cO){},_createContentElement:function(){var cP=this._createInputElement();cP.setSelectable(this.getSelectable());cP.setEnabled(this.getEnabled());cP.addListener(r,this._onHtmlInput,this);cP.setAttribute(U,cj);cP.addClass(v);if((qx.core.Environment.get(cr)==W)&&(qx.core.Environment.get(o)==8)){cP.setStyles({backgroundImage:bL+qx.util.ResourceManager.getInstance().toUri(cp)+bM});};return cP;},_applyEnabled:function(cS,cR){qx.ui.core.Widget.prototype._applyEnabled.call(this,cS,cR);this.getContentElement().setEnabled(cS);if(this.__pQ){if(cS){this._showPlaceholder();}else {this._removePlaceholder();};}else {var cQ=this.getContentElement();cQ.setAttribute(bW,cS?this.getPlaceholder():cm);};},__pS:{width:16,height:16},_getContentHint:function(){return {width:this.__pS.width*10,height:this.__pS.height||16};},_applyFont:function(cV,cU){if(cU&&this.__ly&&this.__lB){this.__ly.removeListenerById(this.__lB);this.__lB=null;};var cT;if(cV){this.__ly=qx.theme.manager.Font.getInstance().resolve(cV);if(this.__ly instanceof qx.bom.webfonts.WebFont){this.__lB=this.__ly.addListener(C,this._onWebFontStatusChange,this);};cT=this.__ly.getStyles();}else {cT=qx.bom.Font.getDefaultStyles();};if(this.getTextColor()!=null){delete cT[F];};if(qx.core.Environment.get(cr)==W&&qx.core.Environment.get(o)<11){qx.html.Element.flush();this.getContentElement().setStyles(cT,true);}else {this.getContentElement().setStyles(cT);};if(this.__pQ){delete cT[F];this._getPlaceholderElement().setStyles(cT);};if(cV){this.__pS=qx.bom.Label.getTextSize(D,cT);}else {delete this.__pS;};qx.ui.core.queue.Layout.add(this);},_applyTextColor:function(cX,cW){if(cX){this.getContentElement().setStyle(F,qx.theme.manager.Color.getInstance().resolve(cX));}else {this.getContentElement().removeStyle(F);};},_applyMaxLength:function(da,cY){if(da){this.getContentElement().setAttribute(t,da);}else {this.getContentElement().removeAttribute(t);};},tabFocus:function(){qx.ui.core.Widget.prototype.tabFocus.call(this);this.selectAllText();},_getTextSize:function(){return this.__pS;},_onHtmlInput:function(e){var dd=e.getData();var dc=true;this.__pR=false;if(this.__iW&&this.__iW===dd){dc=false;};if(this.getFilter()!=null){var df=cm;var db=dd.search(this.getFilter());var de=dd;while(db>=0){df=df+(de.charAt(db));de=de.substring(db+1,de.length);db=de.search(this.getFilter());};if(df!=dd){dc=false;dd=df;this.getContentElement().setValue(dd);};};if(dc){this.fireDataEvent(r,dd,this.__iW);this.__iW=dd;if(this.getLiveUpdate()){this.__pT(dd);};};},_onWebFontStatusChange:function(dh){if(dh.getData().valid===true){var dg=this.__ly.getStyles();this.__pS=qx.bom.Label.getTextSize(D,dg);qx.ui.core.queue.Layout.add(this);};},__pT:function(dj){var di=this.__iV;this.__iV=dj;if(di!=dj){this.fireNonBubblingEvent(c,qx.event.type.Data,[dj,di]);};},setValue:function(dn){if(dn===null){if(this.__pR){return dn;};dn=cm;this.__pR=true;}else {this.__pR=false;if(this.__pQ){this._removePlaceholder();};};if(qx.lang.Type.isString(dn)){var dm=this.getContentElement();if(dm.getValue()!=dn){var dk=dm.getValue();dm.setValue(dn);var dl=this.__pR?null:dn;this.__iV=dk;this.__pT(dl);this.__iW=this.__iV;};if(this.__pQ){this._showPlaceholder();};return dn;};throw new Error(J+dn);},getValue:function(){var dp=this.getContentElement().getValue();return this.__pR?null:dp;},resetValue:function(){this.setValue(null);},_onChangeContent:function(e){this.__pR=e.getData()===null;this.__pT(e.getData());},getTextSelection:function(){return this.getContentElement().getTextSelection();},getTextSelectionLength:function(){return this.getContentElement().getTextSelectionLength();},getTextSelectionStart:function(){return this.getContentElement().getTextSelectionStart();},getTextSelectionEnd:function(){return this.getContentElement().getTextSelectionEnd();},setTextSelection:function(dq,dr){this.getContentElement().setTextSelection(dq,dr);},clearTextSelection:function(){this.getContentElement().clearTextSelection();},selectAllText:function(){this.setTextSelection(0);},setLayoutParent:function(parent){qx.ui.core.Widget.prototype.setLayoutParent.call(this,parent);if(this.__pQ){if(parent){this.getLayoutParent().getContentElement().add(this._getPlaceholderElement());}else {var ds=this._getPlaceholderElement();ds.getParent().remove(ds);};};},_showPlaceholder:function(){var du=this.getValue()||cm;var dt=this.getPlaceholder();if(dt!=null&&du==cm&&!this.hasState(k)&&!this.hasState(ca)){if(this.hasState(j)){this._syncPlaceholder();}else {this.addState(j);};};},_onPointerDownPlaceholder:function(){window.setTimeout(function(){this.focus();}.bind(this),0);},_removePlaceholder:function(){if(this.hasState(j)){if(this.__pQ){this._getPlaceholderElement().setStyle(cv,Y);};this.removeState(j);};},_syncPlaceholder:function(){if(this.hasState(j)&&this.__pQ){this._getPlaceholderElement().setStyle(cv,M);};},_getPlaceholderElement:function(){if(this._placeholder==null){this._placeholder=new qx.html.Label();var dv=qx.theme.manager.Color.getInstance();this._placeholder.setStyles({"zIndex":11,"position":cq,"color":dv.resolve(cc),"whiteSpace":u,"cursor":A,"visibility":Y});this._placeholder.addListener(T,this._onPointerDownPlaceholder,this);};return this._placeholder;},_onChangeLocale:qx.core.Environment.select(s,{"true":function(e){var content=this.getPlaceholder();if(content&&content.translate){this.setPlaceholder(content.translate());};},"false":null}),_onChangeTheme:function(){qx.ui.core.Widget.prototype._onChangeTheme.call(this);if(this._placeholder){this._placeholder.dispose();this._placeholder=null;};if(!this.__pQ&&qx.ui.form.AbstractField.__ds){qx.bom.Stylesheet.removeSheet(qx.ui.form.AbstractField.__ds);qx.ui.form.AbstractField.__ds=null;qx.ui.form.AbstractField.__pP();};},_applyPlaceholder:function(dx,dw){if(this.__pQ){this._getPlaceholderElement().setValue(dx);if(dx!=null){this.addListener(V,this._removePlaceholder,this);this.addListener(ct,this._showPlaceholder,this);this._showPlaceholder();}else {this.removeListener(V,this._removePlaceholder,this);this.removeListener(ct,this._showPlaceholder,this);this._removePlaceholder();};}else {if(this.getEnabled()){this.getContentElement().setAttribute(bW,dx);if(qx.core.Environment.get(g)===L&&parseFloat(qx.core.Environment.get(S))<36&&this.getContentElement().getNodeName()===h&&!this.getContentElement().getDomElement()){this.addListenerOnce(cn,function(){this.getContentElement().getDomElement().removeAttribute(bW);this.getContentElement().getDomElement().setAttribute(bW,dx);},this);};};};},_applyTextAlign:function(dz,dy){this.getContentElement().setStyle(bS,dz);},_applyReadOnly:function(dB,dA){var dC=this.getContentElement();dC.setAttribute(z,dB);if(dB){this.addState(G);this.setFocusable(false);}else {this.removeState(G);this.setFocusable(true);};}},defer:function(dD){var dE=bP+cs+y+I+R+Q+bT+w+d+H+n;qx.ui.style.Stylesheet.getInstance().addRule(f,dE);},destruct:function(){if(this._placeholder){this._placeholder.removeListener(T,this._onPointerDownPlaceholder,this);var parent=this._placeholder.getParent();if(parent){parent.remove(this._placeholder);};this._placeholder.dispose();};this._placeholder=this.__ly=null;if(qx.core.Environment.get(s)){qx.locale.Manager.getInstance().removeListener(cg,this._onChangeLocale,this);};if(this.__ly&&this.__lB){this.__ly.removeListenerById(this.__lB);};this.getContentElement().removeListener(r,this._onHtmlInput,this);}});})();(function(){var a="engine.name",b="",c="wrap",d="none",e="Text wrapping is only support by textareas!",f="overflow",g="webkit",h="input",i="qx.html.Input",j="value",k="select",l="disabled",m="read-only",n="textarea",o="overflowX",p="overflowY";qx.Class.define(i,{extend:qx.html.Element,construct:function(s,q,r){if(s===k||s===n){var t=s;}else {t=h;};qx.html.Element.call(this,t,q,r);this.__pU=s;},members:{__pU:null,__pV:null,__pW:null,_createDomElement:function(){return qx.bom.Input.create(this.__pU);},_applyProperty:function(name,u){qx.html.Element.prototype._applyProperty.call(this,name,u);var v=this.getDomElement();if(name===j){qx.bom.Input.setValue(v,u);}else if(name===c){qx.bom.Input.setWrap(v,u);this.setStyle(f,v.style.overflow,true);this.setStyle(o,v.style.overflowX,true);this.setStyle(p,v.style.overflowY,true);};},setEnabled:function(w){this.__pW=w;this.setAttribute(l,w===false);if(qx.core.Environment.get(a)==g){if(!w){this.setStyles({"userModify":m,"userSelect":d});}else {this.setStyles({"userModify":null,"userSelect":this.__pV?null:d});};};},setSelectable:qx.core.Environment.select(a,{"webkit":function(x){this.__pV=x;qx.html.Element.prototype.setSelectable.call(this,this.__pW&&x);},"default":function(y){qx.html.Element.prototype.setSelectable.call(this,y);}}),setValue:function(z){var A=this.getDomElement();if(A){if(A.value!=z){qx.bom.Input.setValue(A,z);};}else {this._setProperty(j,z);};return this;},getValue:function(){var B=this.getDomElement();if(B){return qx.bom.Input.getValue(B);};return this._getProperty(j)||b;},setWrap:function(C,D){if(this.__pU===n){this._setProperty(c,C,D);}else {throw new Error(e);};return this;},getWrap:function(){if(this.__pU===n){return this._getProperty(c);}else {throw new Error(e);};}}});})();(function(){var a="wrap",b="text",c="engine.name",d="soft",e="",f="mshtml",g="number",h="checkbox",j="select-one",k="input",m="option",n="value",o="off",p="select",q="radio",r="qx.bom.Input",s="nowrap",t="textarea",u="auto",v="normal";qx.Bootstrap.define(r,{statics:{__jH:{text:1,textarea:1,select:1,checkbox:1,radio:1,password:1,hidden:1,submit:1,image:1,file:1,search:1,reset:1,button:1},create:function(y,w,z){{};var w=w?qx.lang.Object.clone(w):{};var x;if(y===t||y===p){x=y;}else {x=k;w.type=y;};return qx.dom.Element.create(x,w,z);},setValue:function(F,E){var G=F.nodeName.toLowerCase();var B=F.type;var Array=qx.lang.Array;var H=qx.lang.Type;if(typeof E===g){E+=e;};if((B===h||B===q)){if(H.isArray(E)){F.checked=Array.contains(E,F.value);}else {F.checked=F.value==E;};}else if(G===p){var A=H.isArray(E);var I=F.options;var C,D;for(var i=0,l=I.length;i<l;i++ ){C=I[i];D=C.getAttribute(n);if(D==null){D=C.text;};C.selected=A?Array.contains(E,D):E==D;};if(A&&E.length==0){F.selectedIndex=-1;};}else if((B===b||B===t)&&(qx.core.Environment.get(c)==f)){F.$$inValueSet=true;F.value=E;F.$$inValueSet=null;}else {F.value=E;};},getValue:function(Q){var O=Q.nodeName.toLowerCase();if(O===m){return (Q.attributes.value||{}).specified?Q.value:Q.text;};if(O===p){var J=Q.selectedIndex;if(J<0){return null;};var P=[];var S=Q.options;var N=Q.type==j;var R=qx.bom.Input;var M;for(var i=N?J:0,L=N?J+1:S.length;i<L;i++ ){var K=S[i];if(K.selected){M=R.getValue(K);if(N){return M;};P.push(M);};};return P;}else {return (Q.value||e).replace(/\r/g,e);};},setWrap:qx.core.Environment.select(c,{"mshtml":function(W,T){var V=T?d:o;var U=T?u:e;W.wrap=V;W.style.overflowY=U;},"gecko":function(bb,X){var ba=X?d:o;var Y=X?e:u;bb.setAttribute(a,ba);bb.style.overflow=Y;},"webkit":function(bf,bc){var be=bc?d:o;var bd=bc?e:u;bf.setAttribute(a,be);bf.style.overflow=bd;},"default":function(bh,bg){bh.style.whiteSpace=bg?v:s;}})}});})();(function(){var a="mshtml",b="engine.name",c="qx.ui.form.TextField",d="Enter",e='px',f="mobile",g="tablet",h="device.type",i="textfield",j="engine.version",k="keypress",l="browser.documentmode";qx.Class.define(c,{extend:qx.ui.form.AbstractField,properties:{appearance:{refine:true,init:i},allowGrowY:{refine:true,init:false},allowShrinkY:{refine:true,init:false}},members:{_renderContentElement:function(innerHeight,m){if((qx.core.Environment.get(b)==a)&&(parseInt(qx.core.Environment.get(j),10)<9||qx.core.Environment.get(l)<9)){m.setStyles({"line-height":innerHeight+e});};},_createContentElement:function(){var n=qx.ui.form.AbstractField.prototype._createContentElement.call(this);var o=qx.core.Environment.get(h);if(o==g||o==f){n.addListener(k,this._onKeyPress,this);};return n;},_onKeyPress:function(p){if(p.getKeyIdentifier()==d){this.blur();};}},destruct:function(){this.getContentElement().removeListener(k,this._onKeyPress,this);}});})();(function(){var a="qx.ui.table.IHeaderRenderer";qx.Interface.define(a,{members:{createHeaderCell:function(b){return true;},updateHeaderCell:function(c,d){return true;}}});})();(function(){var a="qx.ui.table.headerrenderer.Default",b="sortedAscending",c="sorted",d="String";qx.Class.define(a,{extend:qx.core.Object,implement:qx.ui.table.IHeaderRenderer,statics:{STATE_SORTED:c,STATE_SORTED_ASCENDING:b},properties:{toolTip:{check:d,init:null,nullable:true}},members:{createHeaderCell:function(e){var f=new qx.ui.table.headerrenderer.HeaderCell();this.updateHeaderCell(e,f);return f;},updateHeaderCell:function(j,i){var h=qx.ui.table.headerrenderer.Default;if(j.name&&j.name.translate){i.setLabel(j.name.translate());}else {i.setLabel(j.name);};var g=i.getToolTip();if(this.getToolTip()!=null){if(g==null){g=new qx.ui.tooltip.ToolTip(this.getToolTip());i.setToolTip(g);qx.util.DisposeUtil.disposeTriggeredBy(g,i);}else {g.setLabel(this.getToolTip());};};j.sorted?i.addState(h.STATE_SORTED):i.removeState(h.STATE_SORTED);j.sortedAscending?i.addState(h.STATE_SORTED_ASCENDING):i.removeState(h.STATE_SORTED_ASCENDING);}}});})();(function(){var a="_applySortIcon",b="_applyIcon",c="icon",d="label",e="table-header-cell",f="qx.ui.table.headerrenderer.HeaderCell",g="_applyLabel",h="String",i="sort-icon";qx.Class.define(f,{extend:qx.ui.container.Composite,construct:function(){qx.ui.container.Composite.call(this);var j=new qx.ui.layout.Grid();j.setRowFlex(0,1);j.setColumnFlex(1,1);j.setColumnFlex(2,1);this.setLayout(j);},properties:{appearance:{refine:true,init:e},label:{check:h,init:null,nullable:true,apply:g},sortIcon:{check:h,init:null,nullable:true,apply:a,themeable:true},icon:{check:h,init:null,nullable:true,apply:b}},members:{_applyLabel:function(l,k){if(l){this._showChildControl(d).setValue(l);}else {this._excludeChildControl(d);};},_applySortIcon:function(n,m){if(n){this._showChildControl(i).setSource(n);}else {this._excludeChildControl(i);};},_applyIcon:function(p,o){if(p){this._showChildControl(c).setSource(p);}else {this._excludeChildControl(c);};},_createChildControlImpl:function(s,r){var q;switch(s){case d:q=new qx.ui.basic.Label(this.getLabel()).set({anonymous:true,allowShrinkX:true});this._add(q,{row:0,column:1});break;case i:q=new qx.ui.basic.Image(this.getSortIcon());q.setAnonymous(true);this._add(q,{row:0,column:2});break;case c:q=new qx.ui.basic.Image(this.getIcon()).set({anonymous:true,allowShrinkX:true});this._add(q,{row:0,column:0});break;};return q||qx.ui.container.Composite.prototype._createChildControlImpl.call(this,s);}}});})();(function(){var a="' must be defined!",b="height",c="hAlign",d="vAlign",e="Integer",f="'",g="_applyLayoutChange",h="qx.ui.layout.Grid",m="maxHeight",n="Cannot add widget '",o="width",p=") for '",q="'!. ",r="top",s="minHeight",t="' in this cell (",u=", ",v="The layout properties 'row' and 'column' of the child widget '",w="minWidth",z="flex",A="left",B="maxWidth",C="There is already a widget '";qx.Class.define(h,{extend:qx.ui.layout.Abstract,construct:function(E,D){qx.ui.layout.Abstract.call(this);this.__pX=[];this.__pY=[];if(E){this.setSpacingX(E);};if(D){this.setSpacingY(D);};},properties:{spacingX:{check:e,init:0,apply:g},spacingY:{check:e,init:0,apply:g}},members:{__qa:null,__pX:null,__pY:null,__qb:null,__qc:null,__qd:null,__qe:null,__qf:null,__qg:null,verifyLayoutProperty:null,__qh:function(){var L=[];var G=[];var J=[];var I=-1;var F=-1;var K=this._getLayoutChildren();for(var i=0,l=K.length;i<l;i++ ){var M=K[i];var N=M.getLayoutProperties();var O=N.row;var H=N.column;N.colSpan=N.colSpan||1;N.rowSpan=N.rowSpan||1;if(O==null||H==null){throw new Error(v+M+a);};if(L[O]&&L[O][H]){throw new Error(n+M+q+C+L[O][H]+t+O+u+H+p+this+f);};for(var x=H;x<H+N.colSpan;x++ ){for(var y=O;y<O+N.rowSpan;y++ ){if(L[y]==undefined){L[y]=[];};L[y][x]=M;F=Math.max(F,x);I=Math.max(I,y);};};if(N.rowSpan>1){J.push(M);};if(N.colSpan>1){G.push(M);};};for(var y=0;y<=I;y++ ){if(L[y]==undefined){L[y]=[];};};this.__qa=L;this.__qb=G;this.__qc=J;this.__qd=I;this.__qe=F;this.__qf=null;this.__qg=null;delete this._invalidChildrenCache;},_setRowData:function(S,Q,R){var P=this.__pX[S];if(!P){this.__pX[S]={};this.__pX[S][Q]=R;}else {P[Q]=R;};},_setColumnData:function(T,V,W){var U=this.__pY[T];if(!U){this.__pY[T]={};this.__pY[T][V]=W;}else {U[V]=W;};},setSpacing:function(X){this.setSpacingY(X);this.setSpacingX(X);return this;},setColumnAlign:function(Y,ba,bb){{};this._setColumnData(Y,c,ba);this._setColumnData(Y,d,bb);this._applyLayoutChange();return this;},getColumnAlign:function(bc){var bd=this.__pY[bc]||{};return {vAlign:bd.vAlign||r,hAlign:bd.hAlign||A};},setRowAlign:function(bf,be,bg){{};this._setRowData(bf,c,be);this._setRowData(bf,d,bg);this._applyLayoutChange();return this;},getRowAlign:function(bi){var bh=this.__pX[bi]||{};return {vAlign:bh.vAlign||r,hAlign:bh.hAlign||A};},getCellWidget:function(bk,bj){if(this._invalidChildrenCache){this.__qh();};var bk=this.__qa[bk]||{};return bk[bj]||null;},getRowCount:function(){if(this._invalidChildrenCache){this.__qh();};return this.__qd+1;},getColumnCount:function(){if(this._invalidChildrenCache){this.__qh();};return this.__qe+1;},getCellAlign:function(bs,bm){var br=r;var bp=A;var bq=this.__pX[bs];var bn=this.__pY[bm];var bl=this.__qa[bs][bm];if(bl){var bo={vAlign:bl.getAlignY(),hAlign:bl.getAlignX()};}else {bo={};};if(bo.vAlign){br=bo.vAlign;}else if(bq&&bq.vAlign){br=bq.vAlign;}else if(bn&&bn.vAlign){br=bn.vAlign;};if(bo.hAlign){bp=bo.hAlign;}else if(bn&&bn.hAlign){bp=bn.hAlign;}else if(bq&&bq.hAlign){bp=bq.hAlign;};return {vAlign:br,hAlign:bp};},setColumnFlex:function(bt,bu){this._setColumnData(bt,z,bu);this._applyLayoutChange();return this;},getColumnFlex:function(bv){var bw=this.__pY[bv]||{};return bw.flex!==undefined?bw.flex:0;},setRowFlex:function(by,bx){this._setRowData(by,z,bx);this._applyLayoutChange();return this;},getRowFlex:function(bB){var bz=this.__pX[bB]||{};var bA=bz.flex!==undefined?bz.flex:0;return bA;},setColumnMaxWidth:function(bC,bD){this._setColumnData(bC,B,bD);this._applyLayoutChange();return this;},getColumnMaxWidth:function(bE){var bF=this.__pY[bE]||{};return bF.maxWidth!==undefined?bF.maxWidth:Infinity;},setColumnWidth:function(bG,bH){this._setColumnData(bG,o,bH);this._applyLayoutChange();return this;},getColumnWidth:function(bI){var bJ=this.__pY[bI]||{};return bJ.width!==undefined?bJ.width:null;},setColumnMinWidth:function(bK,bL){this._setColumnData(bK,w,bL);this._applyLayoutChange();return this;},getColumnMinWidth:function(bM){var bN=this.__pY[bM]||{};return bN.minWidth||0;},setRowMaxHeight:function(bP,bO){this._setRowData(bP,m,bO);this._applyLayoutChange();return this;},getRowMaxHeight:function(bR){var bQ=this.__pX[bR]||{};return bQ.maxHeight||Infinity;},setRowHeight:function(bS,bT){this._setRowData(bS,b,bT);this._applyLayoutChange();return this;},getRowHeight:function(bV){var bU=this.__pX[bV]||{};return bU.height!==undefined?bU.height:null;},setRowMinHeight:function(bX,bW){this._setRowData(bX,s,bW);this._applyLayoutChange();return this;},getRowMinHeight:function(ca){var bY=this.__pX[ca]||{};return bY.minHeight||0;},__qi:function(cc){var cb=cc.getSizeHint();var ce=cc.getMarginLeft()+cc.getMarginRight();var cd=cc.getMarginTop()+cc.getMarginBottom();var cf={height:cb.height+cd,width:cb.width+ce,minHeight:cb.minHeight+cd,minWidth:cb.minWidth+ce,maxHeight:cb.maxHeight+cd,maxWidth:cb.maxWidth+ce};return cf;},_fixHeightsRowSpan:function(cA){var cm=this.getSpacingY();for(var i=0,l=this.__qc.length;i<l;i++ ){var cs=this.__qc[i];var cp=this.__qi(cs);var ci=cs.getLayoutProperties();var co=ci.row;var cx=cm*(ci.rowSpan-1);var cg=cx;var cj={};for(var j=0;j<ci.rowSpan;j++ ){var cn=ci.row+j;var cw=cA[cn];var cy=this.getRowFlex(cn);if(cy>0){cj[cn]={min:cw.minHeight,value:cw.height,max:cw.maxHeight,flex:cy};};cx+=cw.height;cg+=cw.minHeight;};if(cx<cp.height){if(!qx.lang.Object.isEmpty(cj)){var cz=qx.ui.layout.Util.computeFlexOffsets(cj,cp.height,cx);for(var k=0;k<ci.rowSpan;k++ ){var cu=cz[co+k]?cz[co+k].offset:0;cA[co+k].height+=cu;};}else {var cr=cm*(ci.rowSpan-1);var cq=cp.height-cr;var cv=Math.floor(cq/ci.rowSpan);var ct=0;var ch=0;for(var k=0;k<ci.rowSpan;k++ ){var cl=cA[co+k].height;ct+=cl;if(cl<cv){ch++ ;};};var ck=Math.floor((cq-ct)/ch);for(var k=0;k<ci.rowSpan;k++ ){if(cA[co+k].height<cv){cA[co+k].height+=ck;};};};};if(cg<cp.minHeight){var cz=qx.ui.layout.Util.computeFlexOffsets(cj,cp.minHeight,cg);for(var j=0;j<ci.rowSpan;j++ ){var cu=cz[co+j]?cz[co+j].offset:0;cA[co+j].minHeight+=cu;};};};},_fixWidthsColSpan:function(cE){var cF=this.getSpacingX();for(var i=0,l=this.__qb.length;i<l;i++ ){var cB=this.__qb[i];var cD=this.__qi(cB);var cH=cB.getLayoutProperties();var cC=cH.column;var cN=cF*(cH.colSpan-1);var cG=cN;var cI={};var cK;for(var j=0;j<cH.colSpan;j++ ){var cJ=cH.column+j;var cM=cE[cJ];var cL=this.getColumnFlex(cJ);if(cL>0){cI[cJ]={min:cM.minWidth,value:cM.width,max:cM.maxWidth,flex:cL};};cN+=cM.width;cG+=cM.minWidth;};if(cN<cD.width){var cO=qx.ui.layout.Util.computeFlexOffsets(cI,cD.width,cN);for(var j=0;j<cH.colSpan;j++ ){cK=cO[cC+j]?cO[cC+j].offset:0;cE[cC+j].width+=cK;};};if(cG<cD.minWidth){var cO=qx.ui.layout.Util.computeFlexOffsets(cI,cD.minWidth,cG);for(var j=0;j<cH.colSpan;j++ ){cK=cO[cC+j]?cO[cC+j].offset:0;cE[cC+j].minWidth+=cK;};};};},_getRowHeights:function(){if(this.__qf!=null){return this.__qf;};var cY=[];var cR=this.__qd;var cQ=this.__qe;for(var da=0;da<=cR;da++ ){var cS=0;var cU=0;var cT=0;for(var cX=0;cX<=cQ;cX++ ){var cP=this.__qa[da][cX];if(!cP){continue;};var cV=cP.getLayoutProperties().rowSpan||0;if(cV>1){continue;};var cW=this.__qi(cP);if(this.getRowFlex(da)>0){cS=Math.max(cS,cW.minHeight);}else {cS=Math.max(cS,cW.height);};cU=Math.max(cU,cW.height);};var cS=Math.max(cS,this.getRowMinHeight(da));var cT=this.getRowMaxHeight(da);if(this.getRowHeight(da)!==null){var cU=this.getRowHeight(da);}else {var cU=Math.max(cS,Math.min(cU,cT));};cY[da]={minHeight:cS,height:cU,maxHeight:cT};};if(this.__qc.length>0){this._fixHeightsRowSpan(cY);};this.__qf=cY;return cY;},_getColWidths:function(){if(this.__qg!=null){return this.__qg;};var df=[];var dc=this.__qe;var de=this.__qd;for(var dk=0;dk<=dc;dk++ ){var di=0;var dh=0;var dd=Infinity;for(var dl=0;dl<=de;dl++ ){var db=this.__qa[dl][dk];if(!db){continue;};var dg=db.getLayoutProperties().colSpan||0;if(dg>1){continue;};var dj=this.__qi(db);if(this.getColumnFlex(dk)>0){dh=Math.max(dh,dj.minWidth);}else {dh=Math.max(dh,dj.width);};di=Math.max(di,dj.width);};dh=Math.max(dh,this.getColumnMinWidth(dk));dd=this.getColumnMaxWidth(dk);if(this.getColumnWidth(dk)!==null){var di=this.getColumnWidth(dk);}else {var di=Math.max(dh,Math.min(di,dd));};df[dk]={minWidth:dh,width:di,maxWidth:dd};};if(this.__qb.length>0){this._fixWidthsColSpan(df);};this.__qg=df;return df;},_getColumnFlexOffsets:function(dq){var dm=this.getSizeHint();var ds=dq-dm.width;if(ds==0){return {};};var dp=this._getColWidths();var dn={};for(var i=0,l=dp.length;i<l;i++ ){var dt=dp[i];var dr=this.getColumnFlex(i);if((dr<=0)||(dt.width==dt.maxWidth&&ds>0)||(dt.width==dt.minWidth&&ds<0)){continue;};dn[i]={min:dt.minWidth,value:dt.width,max:dt.maxWidth,flex:dr};};return qx.ui.layout.Util.computeFlexOffsets(dn,dq,dm.width);},_getRowFlexOffsets:function(dw){var du=this.getSizeHint();var dy=dw-du.height;if(dy==0){return {};};var dx=this._getRowHeights();var dv={};for(var i=0,l=dx.length;i<l;i++ ){var dA=dx[i];var dz=this.getRowFlex(i);if((dz<=0)||(dA.height==dA.maxHeight&&dy>0)||(dA.height==dA.minHeight&&dy<0)){continue;};dv[i]={min:dA.minHeight,value:dA.height,max:dA.maxHeight,flex:dz};};return qx.ui.layout.Util.computeFlexOffsets(dv,dw,du.height);},renderLayout:function(dV,dB,dU){if(this._invalidChildrenCache){this.__qh();};var dP=qx.ui.layout.Util;var dD=this.getSpacingX();var dJ=this.getSpacingY();var dT=this._getColWidths();var dW=this._getColumnFlexOffsets(dV);var dE=[];var dY=this.__qe;var dC=this.__qd;var dX;for(var ea=0;ea<=dY;ea++ ){dX=dW[ea]?dW[ea].offset:0;dE[ea]=dT[ea].width+dX;};var dM=this._getRowHeights();var dO=this._getRowFlexOffsets(dB);var eg=[];for(var dK=0;dK<=dC;dK++ ){dX=dO[dK]?dO[dK].offset:0;eg[dK]=dM[dK].height+dX;};var ee=0;for(var ea=0;ea<=dY;ea++ ){var top=0;for(var dK=0;dK<=dC;dK++ ){var dR=this.__qa[dK][ea];if(!dR){top+=eg[dK]+dJ;continue;};var dF=dR.getLayoutProperties();if(dF.row!==dK||dF.column!==ea){top+=eg[dK]+dJ;continue;};var ef=dD*(dF.colSpan-1);for(var i=0;i<dF.colSpan;i++ ){ef+=dE[ea+i];};var dS=dJ*(dF.rowSpan-1);for(var i=0;i<dF.rowSpan;i++ ){dS+=eg[dK+i];};var dG=dR.getSizeHint();var ed=dR.getMarginTop();var dQ=dR.getMarginLeft();var dN=dR.getMarginBottom();var dI=dR.getMarginRight();var dL=Math.max(dG.minWidth,Math.min(ef-dQ-dI,dG.maxWidth));var eh=Math.max(dG.minHeight,Math.min(dS-ed-dN,dG.maxHeight));var eb=this.getCellAlign(dK,ea);var ec=ee+dP.computeHorizontalAlignOffset(eb.hAlign,dL,ef,dQ,dI);var dH=top+dP.computeVerticalAlignOffset(eb.vAlign,eh,dS,ed,dN);dR.renderLayout(ec+dU.left,dH+dU.top,dL,eh);top+=eg[dK]+dJ;};ee+=dE[ea]+dD;};},invalidateLayoutCache:function(){qx.ui.layout.Abstract.prototype.invalidateLayoutCache.call(this);this.__qg=null;this.__qf=null;},_computeSizeHint:function(){if(this._invalidChildrenCache){this.__qh();};var ek=this._getColWidths();var ei=0,eq=0;for(var i=0,l=ek.length;i<l;i++ ){var ep=ek[i];if(this.getColumnFlex(i)>0){ei+=ep.minWidth;}else {ei+=ep.width;};eq+=ep.width;};var er=this._getRowHeights();var el=0,em=0;for(var i=0,l=er.length;i<l;i++ ){var es=er[i];if(this.getRowFlex(i)>0){el+=es.minHeight;}else {el+=es.height;};em+=es.height;};var eo=this.getSpacingX()*(ek.length-1);var en=this.getSpacingY()*(er.length-1);var ej={minWidth:ei+eo,width:eq+eo,minHeight:el+en,height:em+en};return ej;}},destruct:function(){this.__qa=this.__pX=this.__pY=this.__qb=this.__qc=this.__qg=this.__qf=null;}});})();(function(){var a="__qp",b="visibilityChanged",c="orderChanged",d="widthChanged",e="qx.ui.table.columnmodel.Basic",f=". The column is not added to this TablePaneModel.",g="__qn",h="Showing column failed: ",j=", got ",k="setColumnsOrder: Invalid number of column positions given, expected ",l="headerCellRendererChanged",m="qx.event.type.Data",n="__qo",o="visibilityChangedPre";qx.Class.define(e,{extend:qx.core.Object,construct:function(){qx.core.Object.call(this);this.__qj=[];this.__qk=[];},events:{"widthChanged":m,"visibilityChangedPre":m,"visibilityChanged":m,"orderChanged":m,"headerCellRendererChanged":m},statics:{DEFAULT_WIDTH:100,DEFAULT_HEADER_RENDERER:qx.ui.table.headerrenderer.Default,DEFAULT_DATA_RENDERER:qx.ui.table.cellrenderer.Default,DEFAULT_EDITOR_FACTORY:qx.ui.table.celleditor.TextField},members:{__oK:null,__ql:null,__qk:null,__qj:null,__qm:null,__qn:null,__qo:null,__qp:null,init:function(s,v){{};this.__qm=[];var r=qx.ui.table.columnmodel.Basic.DEFAULT_WIDTH;var t=this.__qn||(this.__qn=new qx.ui.table.columnmodel.Basic.DEFAULT_HEADER_RENDERER());var q=this.__qo||(this.__qo=new qx.ui.table.columnmodel.Basic.DEFAULT_DATA_RENDERER());var p=this.__qp||(this.__qp=new qx.ui.table.columnmodel.Basic.DEFAULT_EDITOR_FACTORY());this.__qj=[];this.__qk=[];var u;if(v){u=v.getInitiallyHiddenColumns();};u=u||[];for(var z=0;z<s;z++ ){this.__qm[z]={width:r,headerRenderer:t,dataRenderer:q,editorFactory:p};this.__qj[z]=z;this.__qk[z]=z;};this.__ql=null;this.__oK=true;for(var w=0;w<u.length;w++ ){this.setColumnVisible(u[w],false);};this.__oK=false;for(z=0;z<s;z++ ){var y={col:z,visible:this.isColumnVisible(z)};this.fireDataEvent(o,y);this.fireDataEvent(b,y);};},getVisibleColumns:function(){return this.__qk!=null?this.__qk:[];},setColumnWidth:function(E,A,D){{};var C=this.__qm[E].width;if(C!=A){this.__qm[E].width=A;var B={col:E,newWidth:A,oldWidth:C,isPointerAction:D||false};this.fireDataEvent(d,B);};},getColumnWidth:function(F){{};return this.__qm[F].width;},setHeaderCellRenderer:function(H,I){{};var G=this.__qm[H].headerRenderer;if(G!==this.__qn){G.dispose();};this.__qm[H].headerRenderer=I;this.fireDataEvent(l,{col:H});},getHeaderCellRenderer:function(J){{};return this.__qm[J].headerRenderer;},setDataCellRenderer:function(L,M){{};var K=this.__qm[L].dataRenderer;this.__qm[L].dataRenderer=M;if(K!==this.__qo){return K;};return null;},getDataCellRenderer:function(N){{};return this.__qm[N].dataRenderer;},setCellEditorFactory:function(P,O){{};var Q=this.__qm[P].editorFactory;if(Q!==this.__qp){Q.dispose();};this.__qm[P].editorFactory=O;},getCellEditorFactory:function(R){{};return this.__qm[R].editorFactory;},_getColToXPosMap:function(){if(this.__ql==null){this.__ql={};for(var U=0;U<this.__qj.length;U++ ){var T=this.__qj[U];this.__ql[T]={overX:U};};for(var S=0;S<this.__qk.length;S++ ){var T=this.__qk[S];this.__ql[T].visX=S;};};return this.__ql;},getVisibleColumnCount:function(){return this.__qk!=null?this.__qk.length:0;},getVisibleColumnAtX:function(V){{};return this.__qk[V];},getVisibleX:function(W){{};return this._getColToXPosMap()[W].visX;},getOverallColumnCount:function(){return this.__qj.length;},getOverallColumnAtX:function(X){{};return this.__qj[X];},getOverallX:function(Y){{};return this._getColToXPosMap()[Y].overX;},isColumnVisible:function(ba){{};return (this._getColToXPosMap()[ba].visX!=null);},setColumnVisible:function(bj,bd){{};if(bd!=this.isColumnVisible(bj)){if(bd){var bh=this._getColToXPosMap();var bf=bh[bj].overX;if(bf==null){throw new Error(h+bj+f);};var be;for(var x=bf+1;x<this.__qj.length;x++ ){var bg=this.__qj[x];var bb=bh[bg].visX;if(bb!=null){be=bb;break;};};if(be==null){be=this.__qk.length;};this.__qk.splice(be,0,bj);}else {var bc=this.getVisibleX(bj);this.__qk.splice(bc,1);};this.__ql=null;if(!this.__oK){var bi={col:bj,visible:bd};this.fireDataEvent(o,bi);this.fireDataEvent(b,bi);};};},moveColumn:function(bk,bn){{};this.__oK=true;var bo=this.__qj[bk];var bl=this.isColumnVisible(bo);if(bl){this.setColumnVisible(bo,false);};this.__qj.splice(bk,1);this.__qj.splice(bn,0,bo);this.__ql=null;if(bl){this.setColumnVisible(bo,true);};this.__oK=false;var bm={col:bo,fromOverXPos:bk,toOverXPos:bn};this.fireDataEvent(c,bm);},setColumnsOrder:function(br){{};if(br.length==this.__qj.length){this.__oK=true;var bq=new Array(br.length);for(var bp=0;bp<this.__qj.length;bp++ ){var bs=this.isColumnVisible(bp);bq[bp]=bs;if(bs){this.setColumnVisible(bp,false);};};this.__qj=qx.lang.Array.clone(br);this.__ql=null;for(var bp=0;bp<this.__qj.length;bp++ ){if(bq[bp]){this.setColumnVisible(bp,true);};};this.__oK=false;this.fireDataEvent(c);}else {throw new Error(k+this.__qj.length+j+br.length);};}},destruct:function(){for(var i=0;i<this.__qm.length;i++ ){this.__qm[i].headerRenderer.dispose();this.__qm[i].dataRenderer.dispose();this.__qm[i].editorFactory.dispose();};this.__qj=this.__qk=this.__qm=this.__ql=null;this._disposeObjects(g,n,a);}});})();(function(){var a='<div>',b="</div>",c="overflow: hidden;",d="qx.event.type.Data",e='</div>',f="paneReloadsData",g="div",h='style="',j="_applyMaxCacheLines",k="track",l="qx.ui.table.pane.Pane",m="width: 100%;",n="qx.event.type.Event",o="_applyVisibleRowCount",p='>',q="",r='" ',s="line-height: ",t="paneUpdated",u="appear",v='class="',w="width:100%;",z="px;",A='<div ',B="'>",C="Number",D="_applyFirstVisibleRow",E="<div style='",F=";position:relative;";qx.Class.define(l,{extend:qx.ui.core.Widget,construct:function(G){qx.ui.core.Widget.call(this);this.__qq=G;this.__qr=0;this.__oJ=0;this.__qs=[];},events:{"paneReloadsData":d,"paneUpdated":n},properties:{firstVisibleRow:{check:C,init:0,apply:D},visibleRowCount:{check:C,init:0,apply:o},maxCacheLines:{check:C,init:1000,apply:j},allowShrinkX:{refine:true,init:false}},members:{__oJ:null,__qr:null,__qq:null,__qt:null,__oG:null,__oF:null,__qs:null,__qu:0,_applyFirstVisibleRow:function(I,H){this.updateContent(false,I-H);},_applyVisibleRowCount:function(K,J){this.updateContent(true);},_getContentHint:function(){return {width:this.getPaneScroller().getTablePaneModel().getTotalWidth(),height:400};},getPaneScroller:function(){return this.__qq;},getTable:function(){return this.__qq.getTable();},setFocusedCell:function(N,M,L){if(N!=this.__oF||M!=this.__oG){var O=this.__oG;this.__oF=N;this.__oG=M;if(M!=O&&!L){if(O!==null){this.updateContent(false,null,O,true);};if(M!==null){this.updateContent(false,null,M,true);};};};},onSelectionChanged:function(){this.updateContent(false,null,null,true);},onFocusChanged:function(){this.updateContent(false,null,null,true);},setColumnWidth:function(Q,P){this.updateContent(true);},onColOrderChanged:function(){this.updateContent(true);},onPaneModelChanged:function(){this.updateContent(true);},onTableModelDataChanged:function(U,W,S,R){this.__qv();var T=this.getFirstVisibleRow();var V=this.getVisibleRowCount();if(W==-1||W>=T&&U<T+V){this.updateContent();};},onTableModelMetaDataChanged:function(){this.updateContent(true);},_applyMaxCacheLines:function(Y,X){if(this.__qu>=Y&&Y!==-1){this.__qv();};},__qv:function(){this.__qs=[];this.__qu=0;},__qw:function(bc,ba,bb){if(!ba&&!bb&&this.__qs[bc]){return this.__qs[bc];}else {return null;};},__qx:function(bh,bg,be,bf){var bd=this.getMaxCacheLines();if(!be&&!bf&&!this.__qs[bh]&&bd>0){this._applyMaxCacheLines(bd);this.__qs[bh]=bg;this.__qu+=1;};},updateContent:function(bi,bl,bk,bj){if(bi){this.__qv();};if(bl&&Math.abs(bl)<=Math.min(10,this.getVisibleRowCount())){this._scrollContent(bl);}else if(bj&&!this.getTable().getAlwaysUpdateCells()){this._updateRowStyles(bk);}else {this._updateAllRows();};},_updateRowStyles:function(br){var bq=this.getContentElement().getDomElement();if(!bq||!bq.firstChild){this._updateAllRows();return;};var bu=this.getTable();var bn=bu.getSelectionModel();var bs=bu.getTableModel();var bv=bu.getDataRowRenderer();var bp=bq.firstChild.childNodes;var bo={table:bu};var bw=this.getFirstVisibleRow();var y=0;var bm=bp.length;if(br!=null){var bt=br-bw;if(bt>=0&&bt<bm){bw=br;y=bt;bm=bt+1;}else {return;};};for(;y<bm;y++ ,bw++ ){bo.row=bw;bo.selected=bn.isSelectedIndex(bw);bo.focusedRow=(this.__oG==bw);bo.rowData=bs.getRowData(bw);bv.updateDataRowElement(bo,bp[y]);};},_getRowsHtml:function(bI,bR){var bC=this.getTable();var bF=bC.getSelectionModel();var bA=bC.getTableModel();var bB=bC.getTableColumnModel();var bW=this.getPaneScroller().getTablePaneModel();var bL=bC.getDataRowRenderer();bA.prefetchRows(bI,bI+bR-1);var bT=bC.getRowHeight();var bV=bW.getColumnCount();var cb=0;var bz=[];for(var x=0;x<bV;x++ ){var ca=bW.getColumnAtX(x);var bE=bB.getColumnWidth(ca);bz.push({col:ca,xPos:x,editable:bA.isColumnEditable(ca),focusedCol:this.__oF==ca,styleLeft:cb,styleWidth:bE});cb+=bE;};var bY=[];var bx=false;for(var bD=bI;bD<bI+bR;bD++ ){var bG=bF.isSelectedIndex(bD);var bK=(this.__oG==bD);var bM=this.__qw(bD,bG,bK);if(bM){bY.push(bM);continue;};var bN=[];var bS={table:bC};bS.styleHeight=bT;bS.row=bD;bS.selected=bG;bS.focusedRow=bK;bS.rowData=bA.getRowData(bD);if(!bS.rowData){bx=true;};bN.push(A);var bO=bL.getRowAttributes(bS);if(bO){bN.push(bO);};var by=bL.getRowClass(bS);if(by){bN.push(v,by,r);};var bP=bL.createRowStyle(bS);bP+=F+bL.getRowHeightStyle(bT)+w;if(bP){bN.push(h,bP,r);};bN.push(p);var bX=false;for(x=0;x<bV&&!bX;x++ ){var bH=bz[x];for(var bU in bH){bS[bU]=bH[bU];};var ca=bS.col;bS.value=bA.getValue(ca,bD);var bJ=bB.getDataCellRenderer(ca);bS.style=bJ.getDefaultCellStyle();bX=bJ.createDataCellHtml(bS,bN)||false;};bN.push(e);var bQ=bN.join(q);this.__qx(bD,bQ,bG,bK);bY.push(bQ);};this.fireDataEvent(f,bx);return bY.join(q);},_scrollContent:function(cn){var cc=this.getContentElement().getDomElement();if(!(cc&&cc.firstChild)){this._updateAllRows();return;};var cl=cc.firstChild;var cd=cl.childNodes;var cf=this.getVisibleRowCount();var ci=this.getFirstVisibleRow();var cg=this.getTable().getTableModel();var cm=0;cm=cg.getRowCount();if(ci+cf>cm){this._updateAllRows();return;};var co=cn<0?cf+cn:0;var ce=cn<0?0:cf-cn;for(var i=Math.abs(cn)-1;i>=0;i-- ){var ch=cd[co];try{cl.removeChild(ch);}catch(cp){break;};};if(!this.__qt){this.__qt=document.createElement(g);};var ck=a;ck+=this._getRowsHtml(ci+ce,Math.abs(cn));ck+=e;this.__qt.innerHTML=ck;var cj=this.__qt.firstChild.childNodes;if(cn>0){for(var i=cj.length-1;i>=0;i-- ){var ch=cj[0];cl.appendChild(ch);};}else {for(var i=cj.length-1;i>=0;i-- ){var ch=cj[cj.length-1];cl.insertBefore(ch,cl.firstChild);};};if(this.__oG!==null){this._updateRowStyles(this.__oG-cn);this._updateRowStyles(this.__oG);};this.fireEvent(t);},_updateAllRows:function(){var ct=this.getContentElement().getDomElement();if(!ct){this.addListenerOnce(u,arguments.callee,this);return;};var cz=this.getTable();var cw=cz.getTableModel();var cy=this.getPaneScroller().getTablePaneModel();var cx=cy.getColumnCount();var cq=cz.getRowHeight();var cu=this.getFirstVisibleRow();var cr=this.getVisibleRowCount();var cA=cw.getRowCount();if(cu+cr>cA){cr=Math.max(0,cA-cu);};var cs=cy.getTotalWidth();var cv;if(cr>0){cv=[E,m,(cz.getForceLineHeight()?s+cq+z:q),c,B,this._getRowsHtml(cu,cr),b];}else {cv=[];};var cB=cv.join(q);ct.innerHTML=cB;this.setWidth(cs);this.__qr=cx;this.__oJ=cr;this.fireEvent(t);}},destruct:function(){this.__qt=this.__qq=this.__qs=null;this.removeListener(k,this._onTrack,this);}});})();(function(){var a="first",b="hovered",c="__qq",d="last",e="qx.ui.table.pane.Header";qx.Class.define(e,{extend:qx.ui.core.Widget,construct:function(f){qx.ui.core.Widget.call(this);this._setLayout(new qx.ui.layout.HBox());this.__lO=new qx.ui.core.Blocker(this);this.__qq=f;},members:{__qq:null,__qy:null,__qz:null,__lO:null,getPaneScroller:function(){return this.__qq;},getTable:function(){return this.__qq.getTable();},getBlocker:function(){return this.__lO;},onColOrderChanged:function(){this._updateContent(true);},onPaneModelChanged:function(){this._updateContent(true);},onTableModelMetaDataChanged:function(){this._updateContent();},setColumnWidth:function(h,g,i){var j=this.getHeaderWidgetAtColumn(h);if(j!=null){j.setWidth(g);};},setPointerOverColumn:function(l){if(l!=this.__qz){if(this.__qz!=null){var k=this.getHeaderWidgetAtColumn(this.__qz);if(k!=null){k.removeState(b);};};if(l!=null){this.getHeaderWidgetAtColumn(l).addState(b);};this.__qz=l;};},getHeaderWidgetAtColumn:function(n){var m=this.getPaneScroller().getTablePaneModel().getX(n);return this._getChildren()[m];},showColumnMoveFeedback:function(y,x){var r=this.getContentLocation();if(this.__qy==null){var w=this.getTable();var o=this.getPaneScroller().getTablePaneModel().getX(y);var q=this._getChildren()[o];var s=w.getTableModel();var u=w.getTableColumnModel();var v={xPos:o,col:y,name:s.getColumnName(y),table:w};var t=u.getHeaderCellRenderer(y);var p=t.createHeaderCell(v);var z=q.getBounds();p.setWidth(z.width);p.setHeight(z.height);p.setZIndex(1000000);p.setOpacity(0.8);p.setLayoutProperties({top:r.top});this.getApplicationRoot().add(p);this.__qy=p;};this.__qy.setLayoutProperties({left:r.left+x});this.__qy.show();},hideColumnMoveFeedback:function(){if(this.__qy!=null){this.__qy.destroy();this.__qy=null;};},isShowingColumnMoveFeedback:function(){return this.__qy!=null;},_updateContent:function(B){var L=this.getTable();var E=L.getTableModel();var H=L.getTableColumnModel();var A=this.getPaneScroller().getTablePaneModel();var D=this._getChildren();var F=A.getColumnCount();var I=E.getSortColumnIndex();if(B){this._cleanUpCells();};var J={};J.sortedAscending=E.isSortAscending();for(var x=0;x<F;x++ ){var M=A.getColumnAtX(x);if(M===undefined){continue;};var K=H.getColumnWidth(M);var G=H.getHeaderCellRenderer(M);J.xPos=x;J.col=M;J.name=E.getColumnName(M);J.editable=E.isColumnEditable(M);J.sorted=(M==I);J.table=L;var C=D[x];if(C==null){C=G.createHeaderCell(J);C.set({width:K});this._add(C);}else {G.updateHeaderCell(J,C);};if(x===0){C.addState(a);C.removeState(d);}else if(x===F-1){C.removeState(a);C.addState(d);}else {C.removeState(a);C.removeState(d);};};},_cleanUpCells:function(){var O=this._getChildren();for(var x=O.length-1;x>=0;x-- ){var N=O[x];N.destroy();};}},destruct:function(){this.__lO.dispose();this._disposeObjects(c);}});})();(function(){var a="__qB",b="changeScrollX",c="Boolean",d="__qH",g="scroll",h="focus-indicator",i="_paneClipper",j="changeHorizontalScrollBarVisible",k="cellDbltap",l="excluded",m="__de",n="bottom",o="_applyScrollTimeout",p="_applyTablePaneModel",q="Integer",r="cellTap",s="roll",t="qx.event.type.Data",u="dblclick",v="dataEdited",w="__qF",y="mouse",z="changeVerticalScrollBarVisible",A="interval",B="__qD",C="qx.ui.table.pane.Scroller",D="scrollbar-y",E="table-scroller-focus-indicator",F="_headerClipper",G="visible",H="__qC",I="resize-line",J="_applyShowCellFocusIndicator",K="header",L="dbltap",M="resize",N="pointerup",O="vertical",P="changeScrollY",Q="device.type",R="os.scrollBarOverlayed",S="pointerdown",T="modelChanged",U="changeTablePaneModel",V="beforeSort",W="__qG",X="scrollbar-x",Y="cellContextmenu",bz="__qE",bA="horizontal",bB="table-scroller",bv="appear",bw="losecapture",bx="contextmenu",by="col-resize",bF="disappear",bG="_applyVerticalScrollBarVisible",bH="_applyHorizontalScrollBarVisible",bL="editing",bC="pane",bD="desktop",bE="close",bt="tap",bK="pointermove",bu="qx.ui.table.pane.Model",bI="qx.ui.table.pane.CellEvent",bJ="pointerout";qx.Class.define(C,{extend:qx.ui.core.Widget,include:[qx.ui.core.scroll.MScrollBarFactory],construct:function(bM){qx.ui.core.Widget.call(this);this.__qA=bM;var bN=new qx.ui.layout.Grid();bN.setColumnFlex(0,1);bN.setRowFlex(1,1);this._setLayout(bN);this.__qB=this._showChildControl(K);this.__qC=this._showChildControl(bC);this.__qD=new qx.ui.container.Composite(new qx.ui.layout.HBox()).set({minWidth:0});this._add(this.__qD,{row:0,column:0,colSpan:2});this._headerClipper=this._createHeaderClipper();this._headerClipper.add(this.__qB);this._headerClipper.addListener(bw,this._onChangeCaptureHeader,this);this._headerClipper.addListener(bK,this._onPointermoveHeader,this);this._headerClipper.addListener(S,this._onPointerdownHeader,this);this._headerClipper.addListener(N,this._onPointerupHeader,this);this._headerClipper.addListener(bt,this._onTapHeader,this);this.__qD.add(this._headerClipper,{flex:1});this._paneClipper=this._createPaneClipper();this._paneClipper.add(this.__qC);this._paneClipper.addListener(s,this._onRoll,this);this._paneClipper.addListener(bK,this._onPointermovePane,this);this._paneClipper.addListener(S,this._onPointerdownPane,this);this._paneClipper.addListener(bt,this._onTapPane,this);this._paneClipper.addListener(bx,this._onTapPane,this);this._paneClipper.addListener(bx,this._onContextMenu,this);if(qx.core.Environment.get(Q)===bD){this._paneClipper.addListener(u,this._onDbltapPane,this);}else {this._paneClipper.addListener(L,this._onDbltapPane,this);};this._paneClipper.addListener(M,this._onResizePane,this);if(qx.core.Environment.get(R)){this.__qE=new qx.ui.container.Composite();this.__qE.setLayout(new qx.ui.layout.Canvas());this.__qE.add(this._paneClipper,{edge:0});this._add(this.__qE,{row:1,column:0});}else {this._add(this._paneClipper,{row:1,column:0});};this.__qF=this._showChildControl(X);this.__qG=this._showChildControl(D);this.__qH=this.getChildControl(h);this.initShowCellFocusIndicator();this.getChildControl(I).hide();this.addListener(bJ,this._onPointerout,this);this.addListener(bv,this._onAppear,this);this.addListener(bF,this._onDisappear,this);this.__de=new qx.event.Timer();this.__de.addListener(A,this._oninterval,this);this.initScrollTimeout();},statics:{MIN_COLUMN_WIDTH:10,RESIZE_REGION_RADIUS:5,TAP_TOLERANCE:5,HORIZONTAL_SCROLLBAR:1,VERTICAL_SCROLLBAR:2},events:{"changeScrollY":t,"changeScrollX":t,"cellTap":bI,"cellDbltap":bI,"cellContextmenu":bI,"beforeSort":t},properties:{horizontalScrollBarVisible:{check:c,init:false,apply:bH,event:j},verticalScrollBarVisible:{check:c,init:false,apply:bG,event:z},tablePaneModel:{check:bu,apply:p,event:U},liveResize:{check:c,init:false},focusCellOnPointerMove:{check:c,init:false},selectBeforeFocus:{check:c,init:false},showCellFocusIndicator:{check:c,init:true,apply:J},contextMenuFromDataCellsOnly:{check:c,init:true},resetSelectionOnHeaderTap:{check:c,init:true},scrollTimeout:{check:q,init:100,apply:o},appearance:{refine:true,init:bB}},members:{__oJ:null,__qA:null,__qI:null,__qJ:null,__qK:null,_moveColumn:null,__qL:null,_lastMoveTargetX:null,_lastMoveTargetScroller:null,__qM:null,__qN:null,__qO:null,__qP:null,__qQ:null,__qR:false,__qS:null,__qT:null,__qU:null,__oF:null,__oG:null,_cellEditor:null,__qV:null,__qW:null,__qF:null,__qG:null,__qB:null,_headerClipper:null,__qC:null,_paneClipper:null,__qE:null,__qH:null,__qD:null,__de:null,getPaneInsetRight:function(){var bQ=this.getTopRightWidget();var bR=bQ&&bQ.isVisible()&&bQ.getBounds()?bQ.getBounds().width+bQ.getMarginLeft()+bQ.getMarginRight():0;var bP=this.__qG;var bO=this.getVerticalScrollBarVisible()?this.getVerticalScrollBarWidth()+bP.getMarginLeft()+bP.getMarginRight():0;return Math.max(bR,bO);},setPaneWidth:function(bS){if(this.isVerticalScrollBarVisible()){bS+=this.getPaneInsetRight();};this.setWidth(bS);},_createChildControlImpl:function(bV,bU){var bT;switch(bV){case K:bT=(this.getTable().getNewTablePaneHeader())(this);break;case bC:bT=(this.getTable().getNewTablePane())(this);break;case h:bT=new qx.ui.table.pane.FocusIndicator(this);bT.setUserBounds(0,0,0,0);bT.setZIndex(1000);bT.addListener(N,this._onPointerupFocusIndicator,this);this._paneClipper.add(bT);bT.show();bT.setDecorator(null);break;case I:bT=new qx.ui.core.Widget();bT.setUserBounds(0,0,0,0);bT.setZIndex(1000);this._paneClipper.add(bT);break;case X:bT=this._createScrollBar(bA).set({alignY:n});bT.addListener(g,this._onScrollX,this);if(this.__qE!=null){bT.setMinHeight(qx.ui.core.scroll.AbstractScrollArea.DEFAULT_SCROLLBAR_WIDTH);this.__qE.add(bT,{bottom:0,right:0,left:0});}else {this._add(bT,{row:2,column:0});};break;case D:bT=this._createScrollBar(O);bT.addListener(g,this._onScrollY,this);if(this.__qE!=null){this.__qE.add(bT,{right:0,bottom:0,top:0});}else {this._add(bT,{row:1,column:1});};break;};return bT||qx.ui.core.Widget.prototype._createChildControlImpl.call(this,bV);},_applyHorizontalScrollBarVisible:function(bX,bW){this.__qF.setVisibility(bX?G:l);},_applyVerticalScrollBarVisible:function(ca,bY){this.__qG.setVisibility(ca?G:l);},_applyTablePaneModel:function(cc,cb){if(cb!=null){cb.removeListener(T,this._onPaneModelChanged,this);};cc.addListener(T,this._onPaneModelChanged,this);},_applyShowCellFocusIndicator:function(ce,cd){if(ce){this.__qH.setDecorator(E);this._updateFocusIndicator();}else {if(this.__qH){this.__qH.setDecorator(null);};};},getScrollY:function(){return this.__qG.getPosition();},setScrollY:function(scrollY,cf){this.__qG.scrollTo(scrollY);if(cf){this._updateContent();};},getScrollX:function(){return this.__qF.getPosition();},setScrollX:function(scrollX){this.__qF.scrollTo(scrollX);},getTable:function(){return this.__qA;},_createPaneClipper:function(){return new qx.ui.table.pane.Clipper();},_createHeaderClipper:function(){return new qx.ui.table.pane.Clipper();},onColVisibilityChanged:function(){this.updateHorScrollBarMaximum();this._updateFocusIndicator();},setColumnWidth:function(ci,ch){this.__qB.setColumnWidth(ci,ch);this.__qC.setColumnWidth(ci,ch);var cg=this.getTablePaneModel();var x=cg.getX(ci);if(x!=-1){this.updateHorScrollBarMaximum();this._updateFocusIndicator();};},onColOrderChanged:function(){this.__qB.onColOrderChanged();this.__qC.onColOrderChanged();this.updateHorScrollBarMaximum();},onTableModelDataChanged:function(cm,ck,cn,cl){this.__qC.onTableModelDataChanged(cm,ck,cn,cl);var cj=this.getTable().getTableModel().getRowCount();if(cj!=this.__oJ){this.updateVerScrollBarMaximum();if(this.getFocusedRow()>=cj){if(cj==0){this.setFocusedCell(null,null);}else {this.setFocusedCell(this.getFocusedColumn(),cj-1);};};this.__oJ=cj;};},onSelectionChanged:function(){this.__qC.onSelectionChanged();},onFocusChanged:function(){this.__qC.onFocusChanged();},onTableModelMetaDataChanged:function(){this.__qB.onTableModelMetaDataChanged();this.__qC.onTableModelMetaDataChanged();},_onPaneModelChanged:function(){this.__qB.onPaneModelChanged();this.__qC.onPaneModelChanged();},_onResizePane:function(){this.updateHorScrollBarMaximum();this.updateVerScrollBarMaximum();this._updateContent();this.__qB._updateContent();this.__qA._updateScrollBarVisibility();},updateHorScrollBarMaximum:function(){var cs=this._paneClipper.getInnerSize();if(!cs){return;};var cp=this.getTablePaneModel().getTotalWidth();var cq=this.__qF;if(cs.width<cp){var co=Math.max(0,cp-cs.width);cq.setMaximum(co);cq.setKnobFactor(cs.width/cp);var cr=cq.getPosition();cq.setPosition(Math.min(cr,co));}else {cq.setMaximum(0);cq.setKnobFactor(1);cq.setPosition(0);};},updateVerScrollBarMaximum:function(){var cA=this._paneClipper.getInnerSize();if(!cA){return;};var cy=this.getTable().getTableModel();var cu=cy.getRowCount();if(this.getTable().getKeepFirstVisibleRowComplete()){cu+=1;};var ct=this.getTable().getRowHeight();var cw=cu*ct;var cz=this.__qG;if(cA.height<cw){var cv=Math.max(0,cw-cA.height);cz.setMaximum(cv);cz.setKnobFactor(cA.height/cw);var cx=cz.getPosition();cz.setPosition(Math.min(cx,cv));}else {cz.setMaximum(0);cz.setKnobFactor(1);cz.setPosition(0);};},onKeepFirstVisibleRowCompleteChanged:function(){this.updateVerScrollBarMaximum();this._updateContent();},_onAppear:function(){this._startInterval(this.getScrollTimeout());},_onDisappear:function(){this._stopInterval();},_onScrollX:function(e){var cB=e.getData();this.fireDataEvent(b,cB,e.getOldData());this._headerClipper.scrollToX(cB);this._paneClipper.scrollToX(cB);},_onScrollY:function(e){this.fireDataEvent(P,e.getData(),e.getOldData());this._postponedUpdateContent();},_onRoll:function(e){var cD=this.getTable();if(e.getPointerType()==y||!cD.getEnabled()){return;};var cE=e.getDelta();if(cE.y>0&&cE.y<1){cE.y=1;}else if(cE.y<0&&cE.y>-1){cE.y=-1;};this.__qG.scrollBy(parseInt(cE.y,10));var cC=cE.y!=0&&!this.__qX(this.__qG,cE.y);if(cE.x>0&&cE.x<1){cE.x=1;}else if(cE.x<0&&cE.x>-1){cE.x=-1;};this.__qF.scrollBy(parseInt(cE.x,10));if(this.__qT&&this.getFocusCellOnPointerMove()){this._focusCellAtPagePos(this.__qT,this.__qU);};cC=cC||(cE.x!=0&&!this.__qX(this.__qF,cE.x));if(cC){e.stop();}else {e.stopMomentum();};},__qX:function(cG,cH){var cF=cG.getPosition();return (cH<0&&cF<=0)||(cH>0&&cF>=cG.getMaximum());},__qY:function(cN){var cL=this.getTable();var cO=this.__qB.getHeaderWidgetAtColumn(this.__qN);var cI=cO.getSizeHint().minWidth;var cK=Math.max(cI,this.__qP+cN-this.__qO);if(this.getLiveResize()){var cJ=cL.getTableColumnModel();cJ.setColumnWidth(this.__qN,cK,true);}else {this.__qB.setColumnWidth(this.__qN,cK,true);var cM=this.getTablePaneModel();this._showResizeLine(cM.getColumnLeft(this.__qN)+cK);};this.__qO+=cK-this.__qP;this.__qP=cK;},__ra:function(cR){var cQ=qx.ui.table.pane.Scroller.TAP_TOLERANCE;if(this.__qB.isShowingColumnMoveFeedback()||cR>this.__qM+cQ||cR<this.__qM-cQ){this.__qL+=cR-this.__qM;this.__qB.showColumnMoveFeedback(this._moveColumn,this.__qL);var cP=this.__qA.getTablePaneScrollerAtPageX(cR);if(this._lastMoveTargetScroller&&this._lastMoveTargetScroller!=cP){this._lastMoveTargetScroller.hideColumnMoveFeedback();};if(cP!=null){this._lastMoveTargetX=cP.showColumnMoveFeedback(cR);}else {this._lastMoveTargetX=null;};this._lastMoveTargetScroller=cP;this.__qM=cR;};},_onPointermoveHeader:function(e){var cY=this.getTable();if(!cY.getEnabled()){return;};var da=false;var cS=null;var cW=e.getDocumentLeft();var cX=e.getDocumentTop();this.__qT=cW;this.__qU=cX;if(this.__qN!=null){this.__qY(cW);da=true;e.stopPropagation();}else if(this._moveColumn!=null){this.__ra(cW);e.stopPropagation();}else {var cT=this._getResizeColumnForPageX(cW);if(cT!=-1){da=true;}else {var cV=cY.getTableModel();var db=this._getColumnForPageX(cW);if(db!=null&&cV.isColumnSortable(db)){cS=db;};};};var cU=da?by:null;this.getApplicationRoot().setGlobalCursor(cU);this.setCursor(cU);this.__qB.setPointerOverColumn(cS);},_onPointermovePane:function(e){var dc=this.getTable();if(!dc.getEnabled()){return;};var de=e.getDocumentLeft();var df=e.getDocumentTop();this.__qT=de;this.__qU=df;var dd=this._getRowForPagePos(de,df);if(dd!=null&&this._getColumnForPageX(de)!=null){if(this.getFocusCellOnPointerMove()){this._focusCellAtPagePos(de,df);};};this.__qB.setPointerOverColumn(null);},_onPointerdownHeader:function(e){if(!this.getTable().getEnabled()){return;};var dh=e.getDocumentLeft();var di=this._getResizeColumnForPageX(dh);if(di!=-1){this._startResizeHeader(di,dh);e.stop();}else {var dg=this._getColumnForPageX(dh);if(dg!=null){this._startMoveHeader(dg,dh);e.stop();};};},_startResizeHeader:function(dl,dj){var dk=this.getTable().getTableColumnModel();this.__qN=dl;this.__qO=dj;this.__qP=dk.getColumnWidth(this.__qN);this._headerClipper.capture();},_startMoveHeader:function(dm,dn){this._moveColumn=dm;this.__qM=dn;this.__qL=this.getTablePaneModel().getColumnLeft(dm);this._headerClipper.capture();},_onPointerdownPane:function(e){var dr=this.getTable();if(!dr.getEnabled()){return;};if(dr.isEditing()){dr.stopEditing();};var dp=e.getDocumentLeft();var dq=e.getDocumentTop();var dt=this._getRowForPagePos(dp,dq);var ds=this._getColumnForPageX(dp);if(dt!==null){this.__qQ={row:dt,col:ds};this.__qR=false;};},_onPointerupFocusIndicator:function(e){if(this.__qQ&&!this.__qR&&!this.isEditing()&&this.__qH.getRow()==this.__qQ.row&&this.__qH.getColumn()==this.__qQ.col){this.fireEvent(r,qx.ui.table.pane.CellEvent,[this,e,this.__qQ.row,this.__qQ.col],true);this.__qR=true;}else if(!this.isEditing()){this._onPointerdownPane(e);};},_onChangeCaptureHeader:function(e){if(this.__qN!=null){this._stopResizeHeader();};if(this._moveColumn!=null){this._stopMoveHeader();};},_stopResizeHeader:function(){var du=this.getTable().getTableColumnModel();if(!this.getLiveResize()){this._hideResizeLine();du.setColumnWidth(this.__qN,this.__qP,true);};this.__qN=null;this._headerClipper.releaseCapture();this.getApplicationRoot().setGlobalCursor(null);this.setCursor(null);if(this.isEditing()){var dv=this._cellEditor.getBounds().height;this._cellEditor.setUserBounds(0,0,this.__qP,dv);};},_stopMoveHeader:function(){var dA=this.getTable().getTableColumnModel();var dB=this.getTablePaneModel();this.__qB.hideColumnMoveFeedback();if(this._lastMoveTargetScroller){this._lastMoveTargetScroller.hideColumnMoveFeedback();};if(this._lastMoveTargetX!=null){var dD=dB.getFirstColumnX()+dB.getX(this._moveColumn);var dz=this._lastMoveTargetX;if(dz!=dD&&dz!=dD+1){var dC=dA.getVisibleColumnAtX(dD);var dy=dA.getVisibleColumnAtX(dz);var dx=dA.getOverallX(dC);var dw=(dy!=null)?dA.getOverallX(dy):dA.getOverallColumnCount();if(dw>dx){dw-- ;};dA.moveColumn(dx,dw);this._updateFocusIndicator();};};this._moveColumn=null;this._lastMoveTargetX=null;this._headerClipper.releaseCapture();},_onPointerupHeader:function(e){var dE=this.getTable();if(!dE.getEnabled()){return;};if(this.__qN!=null){this._stopResizeHeader();this.__qS=true;e.stop();}else if(this._moveColumn!=null){this._stopMoveHeader();e.stop();};},_onTapHeader:function(e){if(this.__qS){this.__qS=false;return;};var dJ=this.getTable();if(!dJ.getEnabled()){return;};var dH=dJ.getTableModel();var dI=e.getDocumentLeft();var dG=this._getResizeColumnForPageX(dI);if(dG==-1){var dM=this._getColumnForPageX(dI);if(dM!=null&&dH.isColumnSortable(dM)){var dF=dH.getSortColumnIndex();var dK=(dM!=dF)?true:!dH.isSortAscending();var dL={column:dM,ascending:dK,tapEvent:e};if(this.fireDataEvent(V,dL,null,true)){if(dJ.isEditing()){dJ.stopEditing();};dH.sortByColumn(dM,dK);if(this.getResetSelectionOnHeaderTap()){dJ.getSelectionModel().resetSelection();};};};};e.stop();},_onTapPane:function(e){var dQ=this.getTable();if(!dQ.getEnabled()){return;};var dO=e.getDocumentLeft();var dP=e.getDocumentTop();var dS=this._getRowForPagePos(dO,dP);var dR=this._getColumnForPageX(dO);if(dS!=null&&dR!=null){var dN=this.getSelectBeforeFocus();if(dN){dQ.getSelectionManager().handleTap(dS,e);};if(!this.getFocusCellOnPointerMove()){this._focusCellAtPagePos(dO,dP);};if(!dN){dQ.getSelectionManager().handleTap(dS,e);};if(this.__qH.isHidden()||(this.__qQ&&!this.__qR&&!this.isEditing()&&dS==this.__qQ.row&&dR==this.__qQ.col)){this.fireEvent(r,qx.ui.table.pane.CellEvent,[this,e,dS,dR],true);this.__qR=true;};};},_onContextMenu:function(e){var dU=e.getDocumentLeft();var dV=e.getDocumentTop();var dX=this._getRowForPagePos(dU,dV);var dW=this._getColumnForPageX(dU);if(dX===null&&this.getContextMenuFromDataCellsOnly()){return;};if(!this.getShowCellFocusIndicator()||dX===null||(this.__qQ&&dX==this.__qQ.row&&dW==this.__qQ.col)){this.fireEvent(Y,qx.ui.table.pane.CellEvent,[this,e,dX,dW],true);var dT=this.getTable().getContextMenu();if(dT){if(dT.getChildren().length>0){dT.openAtPointer(e);}else {dT.exclude();};e.preventDefault();};};},_onContextMenuOpen:function(e){},_onDbltapPane:function(e){var eb=e.getDocumentLeft();var ec=e.getDocumentTop();var ea=this._getColumnForPageX(eb);if(ea!==null){this._focusCellAtPagePos(eb,ec);this.startEditing();var dY=this._getRowForPagePos(eb,ec);if(dY!=-1&&dY!=null){this.fireEvent(k,qx.ui.table.pane.CellEvent,[this,e,dY],true);};};},_onPointerout:function(e){var ed=this.getTable();if(!ed.getEnabled()){return;};if(this.__qN==null){this.setCursor(null);this.getApplicationRoot().setGlobalCursor(null);};this.__qB.setPointerOverColumn(null);if(this.getFocusCellOnPointerMove()){this.__qA.setFocusedCell();};},_showResizeLine:function(x){var ef=this._showChildControl(I);var ee=ef.getWidth();var eg=this._paneClipper.getBounds();ef.setUserBounds(x-Math.round(ee/2),0,ee,eg.height);},_hideResizeLine:function(){this._excludeChildControl(I);},showColumnMoveFeedback:function(ep){var eq=this.getTablePaneModel();var ej=this.getTable().getTableColumnModel();var ek=this.__qC.getContentLocation().left;var eo=eq.getColumnCount();var el=0;var et=0;var ei=ek;for(var eh=0;eh<eo;eh++ ){var es=eq.getColumnAtX(eh);var er=ej.getColumnWidth(es);if(ep<ei+er/2){break;};ei+=er;el=eh+1;et=ei-ek;};var en=this._paneClipper.getContentLocation().left;var em=this._paneClipper.getBounds().width;var scrollX=en-ek;et=qx.lang.Number.limit(et,scrollX+2,scrollX+em-1);this._showResizeLine(et);return eq.getFirstColumnX()+el;},hideColumnMoveFeedback:function(){this._hideResizeLine();},_focusCellAtPagePos:function(ew,eu){var ex=this._getRowForPagePos(ew,eu);if(ex!=-1&&ex!=null){var ev=this._getColumnForPageX(ew);this.__qA.setFocusedCell(ev,ex);};},setFocusedCell:function(ey,ez){if(!this.isEditing()){this.__qC.setFocusedCell(ey,ez,this.__qJ);this.__oF=ey;this.__oG=ez;this._updateFocusIndicator();};},getFocusedColumn:function(){return this.__oF;},getFocusedRow:function(){return this.__oG;},scrollCellVisible:function(eM,eN){var eK=this.getTablePaneModel();var eA=eK.getX(eM);if(eA!=-1){var eH=this._paneClipper.getInnerSize();if(!eH){return;};var eD=this.getTable().getTableColumnModel();var eE=eK.getColumnLeft(eM);var eL=eD.getColumnWidth(eM);var eC=this.getTable().getRowHeight();var eB=eN*eC;var scrollX=this.getScrollX();var scrollY=this.getScrollY();var eJ=Math.min(eE,eE+eL-eH.width);var eG=eE;this.setScrollX(Math.max(eJ,Math.min(eG,scrollX)));var eI=eB+eC-eH.height;if(this.getTable().getKeepFirstVisibleRowComplete()){eI+=eC;};var eF=eB;this.setScrollY(Math.max(eI,Math.min(eF,scrollY)),true);};},isEditing:function(){return this._cellEditor!=null;},startEditing:function(){var eS=this.getTable();var eQ=eS.getTableModel();var eU=this.__oF;if(!this.isEditing()&&(eU!=null)&&eQ.isColumnEditable(eU)){var eT=this.__oG;var eO=this.getTablePaneModel().getX(eU);var eP=eQ.getValue(eU,eT);this.scrollCellVisible(eO,eT);this.__qV=eS.getTableColumnModel().getCellEditorFactory(eU);var eR={col:eU,row:eT,xPos:eO,value:eP,table:eS};this._cellEditor=this.__qV.createCellEditor(eR);if(this._cellEditor===null){return false;}else if(this._cellEditor instanceof qx.ui.window.Window){this._cellEditor.setModal(true);this._cellEditor.setShowClose(false);this._cellEditor.addListener(bE,this._onCellEditorModalWindowClose,this);var f=eS.getModalCellEditorPreOpenFunction();if(f!=null){f(this._cellEditor,eR);};this._cellEditor.open();}else {var eV=this.__qH.getInnerSize();this._cellEditor.setUserBounds(0,0,eV.width,eV.height);this.__qH.addListener(S,function(e){this.__qQ={row:this.__oG,col:this.__oF};e.stopPropagation();},this);this.__qH.add(this._cellEditor);this.__qH.addState(bL);this.__qH.setKeepActive(false);this.__qH.setDecorator(E);this._cellEditor.focus();this._cellEditor.activate();};return true;};return false;},stopEditing:function(){if(!this.getShowCellFocusIndicator()){this.__qH.setDecorator(null);};this.flushEditor();this.cancelEditing();},flushEditor:function(){if(this.isEditing()){var eW=this.__qV.getCellEditorValue(this._cellEditor);var eX=this.getTable().getTableModel().getValue(this.__oF,this.__oG);this.getTable().getTableModel().setValue(this.__oF,this.__oG,eW);this.__qA.focus();this.__qA.fireDataEvent(v,{row:this.__oG,col:this.__oF,oldValue:eX,value:eW});};},cancelEditing:function(){if(this.isEditing()){if(!(this._cellEditor instanceof qx.ui.window.Window)){this.__qH.removeState(bL);this.__qH.setKeepActive(true);};this._cellEditor.destroy();this._cellEditor=null;this.__qV=null;};},_onCellEditorModalWindowClose:function(e){this.stopEditing();},_getColumnForPageX:function(eY){var fc=this.getTable().getTableColumnModel();var fd=this.getTablePaneModel();var fb=fd.getColumnCount();var fa=this.__qC.getContentLocation().left;for(var x=0;x<fb;x++ ){var ff=fd.getColumnAtX(x);var fe=fc.getColumnWidth(ff);fa+=fe;if(eY<fa){return ff;};};return null;},_getResizeColumnForPageX:function(fh){var fk=this.getTable().getTableColumnModel();var fl=this.getTablePaneModel();var fj=fl.getColumnCount();var fi=this.__qB.getContentLocation().left;var fg=qx.ui.table.pane.Scroller.RESIZE_REGION_RADIUS;for(var x=0;x<fj;x++ ){var fn=fl.getColumnAtX(x);var fm=fk.getColumnWidth(fn);fi+=fm;if(fh>=(fi-fg)&&fh<=(fi+fg)){return fn;};};return -1;},_getRowForPagePos:function(fu,fv){var fo=this.__qC.getContentLocation();if(fu<fo.left||fu>fo.right){return null;};if(fv>=fo.top&&fv<=fo.bottom){var fp=this.getTable().getRowHeight();var scrollY=this.__qG.getPosition();if(this.getTable().getKeepFirstVisibleRowComplete()){scrollY=Math.floor(scrollY/fp)*fp;};var fs=scrollY+fv-fo.top;var fw=Math.floor(fs/fp);var fq=this.getTable().getTableModel();var ft=fq.getRowCount();return (fw<ft)?fw:null;};var fr=this.__qB.getContentLocation();if(fv>=fr.top&&fv<=fr.bottom&&fu<=fr.right){return -1;};return null;},setTopRightWidget:function(fx){var fy=this.__qW;if(fy!=null){this.__qD.remove(fy);};if(fx!=null){this.__qD.add(fx);};this.__qW=fx;},getTopRightWidget:function(){return this.__qW;},getHeader:function(){return this.__qB;},getTablePane:function(){return this.__qC;},getVerticalScrollBarWidth:function(){var fz=this.__qG;return fz.isVisible()?(fz.getSizeHint().width||0):0;},getNeededScrollBars:function(fD,fE){var fP=this.__qG;var fO=fP.getSizeHint().width+fP.getMarginLeft()+fP.getMarginRight();var fJ=this.__qF;var fM=fJ.getSizeHint().height+fJ.getMarginTop()+fJ.getMarginBottom();var fG=this._paneClipper.getInnerSize();var fA=fG?fG.width:0;if(this.getVerticalScrollBarVisible()){fA+=fO;};var fK=fG?fG.height:0;if(this.getHorizontalScrollBarVisible()){fK+=fM;};var fH=this.getTable().getTableModel();var fI=fH.getRowCount();var fB=this.getTablePaneModel().getTotalWidth();var fL=this.getTable().getRowHeight()*fI;var fC=false;var fQ=false;if(fB>fA){fC=true;if(fL>fK-fM){fQ=true;};}else if(fL>fK){fQ=true;if(!fE&&(fB>fA-fO)){fC=true;};};var fF=qx.ui.table.pane.Scroller.HORIZONTAL_SCROLLBAR;var fN=qx.ui.table.pane.Scroller.VERTICAL_SCROLLBAR;return ((fD||fC)?fF:0)|((fE||!fQ)?0:fN);},getPaneClipper:function(){return this._paneClipper;},getScrollAreaContainer:function(){return this.getPaneClipper();},_applyScrollTimeout:function(fS,fR){this._startInterval(fS);},_startInterval:function(fT){this.__de.setInterval(fT);this.__de.start();},_stopInterval:function(){this.__de.stop();},_postponedUpdateContent:function(){this._updateContent();},_oninterval:qx.event.GlobalError.observeMethod(function(){if(this.__qJ&&!this.__qC._layoutPending){this.__qJ=false;this._updateContent();};}),_updateContent:function(){var gc=this._paneClipper.getInnerSize();if(!gc){return;};var ga=gc.height;var scrollX=this.__qF.getPosition();var scrollY=this.__qG.getPosition();var fV=this.getTable().getRowHeight();var fX=Math.floor(scrollY/fV);var gb=this.__qC.getFirstVisibleRow();this.__qC.setFirstVisibleRow(fX);var fW=Math.ceil(ga/fV);var fU=0;var fY=this.getTable().getKeepFirstVisibleRowComplete();if(!fY){fW++ ;fU=scrollY%fV;};this.__qC.setVisibleRowCount(fW);if(fX!=gb){this._updateFocusIndicator();};this._paneClipper.scrollToX(scrollX);if(!fY){this._paneClipper.scrollToY(fU);};},_updateFocusIndicator:function(){var gd=this.getTable();if(!gd.getEnabled()){return;};this.__qH.moveToCell(this.__oF,this.__oG);}},destruct:function(){this._stopInterval();var ge=this.getTablePaneModel();if(ge){ge.dispose();};this.__qQ=this.__qW=this.__qA=null;this._disposeObjects(w,W,F,i,d,a,H,B,m,bz);}});})();(function(){var a="Escape",b="keypress",c="Enter",d="excluded",f="Integer",g="qx.ui.table.pane.FocusIndicator";qx.Class.define(g,{extend:qx.ui.container.Composite,construct:function(h){qx.ui.container.Composite.call(this);this.__rb=h;this.setKeepActive(true);this.addListener(b,this._onKeyPress,this);},properties:{visibility:{refine:true,init:d},row:{check:f,nullable:true},column:{check:f,nullable:true}},members:{__rb:null,_onKeyPress:function(e){var i=e.getKeyIdentifier();if(i!==a&&i!==c){e.stopPropagation();};},moveToCell:function(p,q){if(!this.__rb.getShowCellFocusIndicator()&&!this.__rb.getTable().getTableModel().isColumnEditable(p)){this.exclude();return;}else {this.show();};if(p==null){this.hide();this.setRow(null);this.setColumn(null);}else {var j=this.__rb.getTablePaneModel().getX(p);if(j==-1){this.hide();this.setRow(null);this.setColumn(null);}else {var o=this.__rb.getTable();var m=o.getTableColumnModel();var n=this.__rb.getTablePaneModel();var l=this.__rb.getTablePane().getFirstVisibleRow();var k=o.getRowHeight();this.setUserBounds(n.getColumnLeft(p)-2,(q-l)*k-2,m.getColumnWidth(p)+3,k+3);this.show();this.setRow(q);this.setColumn(p);};};}},destruct:function(){this.__rb=null;}});})();(function(){var a="mouse",b="scrollbar-y",c="qx.ui.core.scroll.MRoll",d="pointerdown",f="scrollbar-x",g="roll";qx.Mixin.define(c,{members:{__rc:null,_addRollHandling:function(){this.addListener(g,this._onRoll,this);this.addListener(d,this._onPointerDownForRoll,this);},_removeRollHandling:function(){this.removeListener(g,this._onRoll,this);this.removeListener(d,this._onPointerDownForRoll,this);},_onPointerDownForRoll:function(e){this.__rc=e.getPointerId();},_onRoll:function(e){if(e.getPointerType()==a){return;};if(this.__rc&&e.getMomentum()){qx.event.Registration.getManager(e.getOriginalTarget()).getHandler(qx.event.handler.Gesture).gestureCancel(this.__rc);e.stopMomentum();this.__rc=null;return;};this.__rc=null;var k=this._isChildControlVisible(f);var l=this._isChildControlVisible(b);var q=l?this.getChildControl(b,true):null;var p=k?this.getChildControl(f,true):null;var m=e.getDelta().y;var h=e.getDelta().x;var j=!l;var o=!k;if(q){if(m!==0){q.scrollBy(parseInt(m,10));};var n=q.getPosition();var i=q.getMaximum();if(m<0&&n<=0||m>0&&n>=i){j=true;};};if(p){if(h!==0){p.scrollBy(parseInt(h,10));};var n=p.getPosition();var i=p.getMaximum();if(h<0&&n<=0||h>0&&n>=i){o=true;};};if(o&&j){e.stopMomentum();};if((!j&&h===0)||(!o&&m===0)||((!o||!j)&&h!==0&&m!==0)){e.stop();};}}});})();(function(){var a="scrollY",b="_computeScrollbars",c="X",d="scrollbar-y",f="scrollAnimation",g="scrollbarX",h="auto",i="End",j="corner",k="os.scrollBarOverlayed",l="scrollarea",m="changeVisibility",n="vertical",o="scrollX",p="scrollAnimationEnd",q="off",r="horizontal",s="scrollbar-x",t="Y",u='qx.event.type.Event',v="qx.ui.core.scroll.AbstractScrollArea",w="abstract",x="update",y="scrollbarY",z="pane",A="on",B="scroll";qx.Class.define(v,{extend:qx.ui.core.Widget,include:[qx.ui.core.scroll.MScrollBarFactory,qx.ui.core.scroll.MRoll,qx.ui.core.MDragDropScrolling],type:w,statics:{DEFAULT_SCROLLBAR_WIDTH:14},construct:function(){qx.ui.core.Widget.call(this);if(qx.core.Environment.get(k)){this._setLayout(new qx.ui.layout.Canvas());}else {var C=new qx.ui.layout.Grid();C.setColumnFlex(0,1);C.setRowFlex(0,1);this._setLayout(C);};this._addRollHandling();},events:{scrollAnimationXEnd:u,scrollAnimationYEnd:u},properties:{appearance:{refine:true,init:l},width:{refine:true,init:100},height:{refine:true,init:200},scrollbarX:{check:[h,A,q],init:h,themeable:true,apply:b},scrollbarY:{check:[h,A,q],init:h,themeable:true,apply:b},scrollbar:{group:[g,y]}},members:{_createChildControlImpl:function(F,E){var D;switch(F){case z:D=new qx.ui.core.scroll.ScrollPane();D.addListener(x,this._computeScrollbars,this);D.addListener(o,this._onScrollPaneX,this);D.addListener(a,this._onScrollPaneY,this);if(qx.core.Environment.get(k)){this._add(D,{edge:0});}else {this._add(D,{row:0,column:0});};break;case s:D=this._createScrollBar(r);D.setMinWidth(0);D.exclude();D.addListener(B,this._onScrollBarX,this);D.addListener(m,this._onChangeScrollbarXVisibility,this);D.addListener(p,this._onScrollAnimationEnd.bind(this,c));if(qx.core.Environment.get(k)){D.setMinHeight(qx.ui.core.scroll.AbstractScrollArea.DEFAULT_SCROLLBAR_WIDTH);this._add(D,{bottom:0,right:0,left:0});}else {this._add(D,{row:1,column:0});};break;case d:D=this._createScrollBar(n);D.setMinHeight(0);D.exclude();D.addListener(B,this._onScrollBarY,this);D.addListener(m,this._onChangeScrollbarYVisibility,this);D.addListener(p,this._onScrollAnimationEnd.bind(this,t));if(qx.core.Environment.get(k)){D.setMinWidth(qx.ui.core.scroll.AbstractScrollArea.DEFAULT_SCROLLBAR_WIDTH);this._add(D,{right:0,bottom:0,top:0});}else {this._add(D,{row:0,column:1});};break;case j:D=new qx.ui.core.Widget();D.setWidth(0);D.setHeight(0);D.exclude();if(!qx.core.Environment.get(k)){this._add(D,{row:1,column:1});};break;};return D||qx.ui.core.Widget.prototype._createChildControlImpl.call(this,F);},getPaneSize:function(){return this.getChildControl(z).getInnerSize();},getItemTop:function(G){return this.getChildControl(z).getItemTop(G);},getItemBottom:function(H){return this.getChildControl(z).getItemBottom(H);},getItemLeft:function(I){return this.getChildControl(z).getItemLeft(I);},getItemRight:function(J){return this.getChildControl(z).getItemRight(J);},scrollToX:function(L,K){qx.ui.core.queue.Manager.flush();this.getChildControl(s).scrollTo(L,K);},scrollByX:function(N,M){qx.ui.core.queue.Manager.flush();this.getChildControl(s).scrollBy(N,M);},getScrollX:function(){var O=this.getChildControl(s,true);return O?O.getPosition():0;},scrollToY:function(Q,P){qx.ui.core.queue.Manager.flush();this.getChildControl(d).scrollTo(Q,P);},scrollByY:function(S,R){qx.ui.core.queue.Manager.flush();this.getChildControl(d).scrollBy(S,R);},getScrollY:function(){var T=this.getChildControl(d,true);return T?T.getPosition():0;},stopScrollAnimationX:function(){var U=this.getChildControl(s,true);if(U){U.stopScrollAnimation();};},stopScrollAnimationY:function(){var V=this.getChildControl(d,true);if(V){V.stopScrollAnimation();};},_onScrollAnimationEnd:function(W){this.fireEvent(f+W+i);},_onScrollBarX:function(e){this.getChildControl(z).scrollToX(e.getData());},_onScrollBarY:function(e){this.getChildControl(z).scrollToY(e.getData());},_onScrollPaneX:function(e){var X=this.getChildControl(s);if(X){X.updatePosition(e.getData());};},_onScrollPaneY:function(e){var Y=this.getChildControl(d);if(Y){Y.updatePosition(e.getData());};},_onChangeScrollbarXVisibility:function(e){var ba=this._isChildControlVisible(s);var bb=this._isChildControlVisible(d);if(!ba){this.scrollToX(0);};ba&&bb?this._showChildControl(j):this._excludeChildControl(j);},_onChangeScrollbarYVisibility:function(e){var bc=this._isChildControlVisible(s);var bd=this._isChildControlVisible(d);if(!bd){this.scrollToY(0);};bc&&bd?this._showChildControl(j):this._excludeChildControl(j);},_computeScrollbars:function(){var bk=this.getChildControl(z);var content=bk.getChildren()[0];if(!content){this._excludeChildControl(s);this._excludeChildControl(d);return;};var be=this.getInnerSize();var bj=bk.getInnerSize();var bh=bk.getScrollSize();if(!bj||!bh){return;};var bl=this.getScrollbarX();var bm=this.getScrollbarY();if(bl===h&&bm===h){var bi=bh.width>be.width;var bn=bh.height>be.height;if((bi||bn)&&!(bi&&bn)){if(bi){bn=bh.height>bj.height;}else if(bn){bi=bh.width>bj.width;};};}else {var bi=bl===A;var bn=bm===A;if(bh.width>(bi?bj.width:be.width)&&bl===h){bi=true;};if(bh.height>(bi?bj.height:be.height)&&bm===h){bn=true;};};if(bi){var bg=this.getChildControl(s);bg.show();bg.setMaximum(Math.max(0,bh.width-bj.width));bg.setKnobFactor((bh.width===0)?0:bj.width/bh.width);}else {this._excludeChildControl(s);};if(bn){var bf=this.getChildControl(d);bf.show();bf.setMaximum(Math.max(0,bh.height-bj.height));bf.setKnobFactor((bh.height===0)?0:bj.height/bh.height);}else {this._excludeChildControl(d);};}}});})();(function(){var a="os.version",b="os.name",c="qx.mobile.nativescroll",d="osx",e="qx.nativeScrollBars",f="event.mspointer",g="android",h="firefox",i="os.scrollBarOverlayed",j="browser.version",k="ios",l=".",m="browser.name",n="qx.bom.client.Scroll";qx.Bootstrap.define(n,{statics:{scrollBarOverlayed:function(){var o=qx.bom.element.Scroll.getScrollbarWidth();var q=qx.bom.client.OperatingSystem.getName()===d;var p=qx.core.Environment.get(e);return o===0&&q&&p;},getNativeScroll:function(){if(qx.core.Environment.get(b)==k&&parseInt(qx.core.Environment.get(j),10)>7){return true;};if(qx.core.Environment.get(m)==h){return true;};if(qx.core.Environment.get(b)==g){var s=qx.core.Environment.get(a);var r=s.split(l);if(r[0]>4||(r.length>1&&r[0]>3&&r[1]>3)){return true;};};if(qx.core.Environment.get(f)){return true;};return false;}},defer:function(t){qx.core.Environment.add(i,t.scrollBarOverlayed);qx.core.Environment.add(c,t.getNativeScroll);}});})();(function(){var a="qx.ui.table.pane.Clipper";qx.Class.define(a,{extend:qx.ui.container.Composite,construct:function(){qx.ui.container.Composite.call(this,new qx.ui.layout.Grow());this.setMinWidth(0);},members:{scrollToX:function(b){this.getContentElement().scrollToX(b,false);},scrollToY:function(c){this.getContentElement().scrollToY(c,true);}}});})();(function(){var a="Integer",b="qx.ui.table.pane.CellEvent";qx.Class.define(b,{extend:qx.event.type.Pointer,properties:{row:{check:a,nullable:true},column:{check:a,nullable:true}},members:{init:function(e,c,f,d){c.clone(this);this.setBubbles(false);if(f!=null){this.setRow(f);}else {this.setRow(e._getRowForPagePos(this.getDocumentLeft(),this.getDocumentTop()));};if(d!=null){this.setColumn(d);}else {this.setColumn(e._getColumnForPageX(this.getDocumentLeft()));};},clone:function(g){var h=qx.event.type.Pointer.prototype.clone.call(this,g);h.set({row:this.getRow(),column:this.getColumn()});return h;}}});})();(function(){var a="qx.lang.Number";qx.Class.define(a,{statics:{isInRange:function(c,d,b){return c>=d&&c<=b;},isBetweenRange:function(f,g,e){return f>g&&f<e;},limit:function(i,j,h){if(h!=null&&i>h){return h;}else if(j!=null&&i<j){return j;}else {return i;};}}});})();(function(){var a="qx.ui.window.IWindowManager";qx.Interface.define(a,{members:{setDesktop:function(b){this.assertInterface(b,qx.ui.window.IDesktop);},changeActiveWindow:function(c,d){},updateStack:function(){},bringToFront:function(e){this.assertInstance(e,qx.ui.window.Window);},sendToBack:function(f){this.assertInstance(f,qx.ui.window.Window);}}});})();(function(){var a="qx.ui.window.IDesktop";qx.Interface.define(a,{members:{setWindowManager:function(b){this.assertInterface(b,qx.ui.window.IWindowManager);},getWindows:function(){},supportsMaximize:function(){},blockContent:function(c){this.assertInteger(c);},unblock:function(){},isBlocked:function(){}}});})();(function(){var a="qx.ui.window.Manager",b="__rd";qx.Class.define(a,{extend:qx.core.Object,implement:qx.ui.window.IWindowManager,members:{__rd:null,setDesktop:function(c){this.__rd=c;this.updateStack();},getDesktop:function(){return this.__rd;},changeActiveWindow:function(d,e){if(d){this.bringToFront(d);d.setActive(true);};if(e){e.resetActive();};},_minZIndex:1e5,updateStack:function(){qx.ui.core.queue.Widget.add(this);},syncWidget:function(){this.__rd.forceUnblock();var h=this.__rd.getWindows();var g=this._minZIndex;var m=g+h.length*2;var j=g+h.length*4;var k=null;for(var i=0,l=h.length;i<l;i++ ){var f=h[i];if(!f.isVisible()){continue;};k=k||f;if(f.isModal()){f.setZIndex(j);this.__rd.blockContent(j-1);j+=2;k=f;}else if(f.isAlwaysOnTop()){f.setZIndex(m);m+=2;}else {f.setZIndex(g);g+=2;};if(!k.isModal()&&f.isActive()||f.getZIndex()>k.getZIndex()){k=f;};};this.__rd.setActiveWindow(k);},bringToFront:function(o){var n=this.__rd.getWindows();var p=qx.lang.Array.remove(n,o);if(p){n.push(o);this.updateStack();};},sendToBack:function(r){var q=this.__rd.getWindows();var s=qx.lang.Array.remove(q,r);if(s){q.unshift(r);this.updateStack();};}},destruct:function(){this._disposeObjects(b);}});})();(function(){var a="__rf",b="pointerup",c="Boolean",d="losecapture",f="__re",g="move",h="qx.ui.core.MMovable",i="The move handle could not be redefined!",j="pointerdown",k="maximized",l="pointermove",m="roll",n="move-frame";qx.Mixin.define(h,{properties:{movable:{check:c,init:true},useMoveFrame:{check:c,init:false}},members:{__re:null,__rf:null,__rg:null,__rh:null,__ri:null,__rj:null,__rk:null,__rl:false,__rm:null,__rn:0,_activateMoveHandle:function(o){if(this.__re){throw new Error(i);};this.__re=o;o.addListener(j,this._onMovePointerDown,this);o.addListener(b,this._onMovePointerUp,this);o.addListener(l,this._onMovePointerMove,this);o.addListener(d,this.__rr,this);},__ro:function(){var p=this.__rf;if(!p){p=this.__rf=new qx.ui.core.Widget();p.setAppearance(n);p.exclude();qx.core.Init.getApplication().getRoot().add(p);};return p;},__rp:function(){var location=this.getContentLocation();var r=this.getBounds();var q=this.__ro();q.setUserBounds(location.left,location.top,r.width,r.height);q.show();q.setZIndex(this.getZIndex()+1);},__rq:function(e){var t=this.__rg;var s=Math.max(t.left,Math.min(t.right,e.getDocumentLeft()));var v=Math.max(t.top,Math.min(t.bottom,e.getDocumentTop()));var u=this.__rh+s;var w=this.__ri+v;return {viewportLeft:parseInt(u,10),viewportTop:parseInt(w,10),parentLeft:parseInt(u-this.__rj,10),parentTop:parseInt(w-this.__rk,10)};},_onMoveRoll:function(e){e.stop();},_onMovePointerDown:function(e){if(!this.getMovable()||this.hasState(k)){return;};this.addListener(m,this._onMoveRoll,this);var parent=this.getLayoutParent();var x=parent.getContentLocation();var z=parent.getBounds();if(qx.Class.implementsInterface(parent,qx.ui.window.IDesktop)){if(!parent.isBlocked()){this.__rm=parent.getBlockerColor();this.__rn=parent.getBlockerOpacity();parent.setBlockerColor(null);parent.setBlockerOpacity(1);parent.blockContent(this.getZIndex()-1);this.__rl=true;};};this.__rg={left:x.left,top:x.top,right:x.left+z.width,bottom:x.top+z.height};var y=this.getContentLocation();this.__rj=x.left;this.__rk=x.top;this.__rh=y.left-e.getDocumentLeft();this.__ri=y.top-e.getDocumentTop();this.addState(g);this.__re.capture();if(this.getUseMoveFrame()){this.__rp();};e.stop();},_onMovePointerMove:function(e){if(!this.hasState(g)){return;};var B=this.__rq(e);if(this.getUseMoveFrame()){this.__ro().setDomPosition(B.viewportLeft,B.viewportTop);}else {var A=this.getLayoutParent().getInsets();this.setDomPosition(B.parentLeft-(A.left||0),B.parentTop-(A.top||0));};e.stopPropagation();},_onMovePointerUp:function(e){if(this.hasListener(m,this._onMoveRoll,this)){this.removeListener(m,this._onMoveRoll,this);};if(!this.hasState(g)){return;};this.removeState(g);var parent=this.getLayoutParent();if(qx.Class.implementsInterface(parent,qx.ui.window.IDesktop)){if(this.__rl){parent.unblock();parent.setBlockerColor(this.__rm);parent.setBlockerOpacity(this.__rn);this.__rm=null;this.__rn=0;this.__rl=false;};};this.__re.releaseCapture();var D=this.__rq(e);var C=this.getLayoutParent().getInsets();this.setLayoutProperties({left:D.parentLeft-(C.left||0),top:D.parentTop-(C.top||0)});if(this.getUseMoveFrame()){this.__ro().exclude();};e.stopPropagation();},__rr:function(e){if(!this.hasState(g)){return;};this.removeState(g);if(this.getUseMoveFrame()){this.__ro().exclude();};}},destruct:function(){this._disposeObjects(a,f);this.__rg=null;}});})();(function(){var a="resetPaddingRight",b="setPaddingTop",c="_applyContentPadding",d="setPaddingBottom",e="resetThemed",f="contentPaddingRight",g="Integer",h="contentPaddingLeft",i="setThemedPaddingLeft",j="resetPaddingTop",k="shorthand",l="setThemedPaddingRight",m="setThemed",n="setPaddingRight",o="contentPaddingBottom",p="resetPaddingBottom",q="qx.ui.core.MContentPadding",r="resetPaddingLeft",s="setThemedPaddingTop",t="setPaddingLeft",u="setThemedPaddingBottom",v="contentPaddingTop";qx.Mixin.define(q,{properties:{contentPaddingTop:{check:g,init:0,apply:c,themeable:true},contentPaddingRight:{check:g,init:0,apply:c,themeable:true},contentPaddingBottom:{check:g,init:0,apply:c,themeable:true},contentPaddingLeft:{check:g,init:0,apply:c,themeable:true},contentPadding:{group:[v,f,o,h],mode:k,themeable:true}},members:{__rs:{contentPaddingTop:b,contentPaddingRight:n,contentPaddingBottom:d,contentPaddingLeft:t},__rt:{contentPaddingTop:s,contentPaddingRight:l,contentPaddingBottom:u,contentPaddingLeft:i},__ru:{contentPaddingTop:j,contentPaddingRight:a,contentPaddingBottom:p,contentPaddingLeft:r},_applyContentPadding:function(z,w,name,y){var A=this._getContentPaddingTarget();if(z==null){var x=this.__ru[name];A[x]();}else {if(y==m||y==e){var B=this.__rt[name];A[B](z);}else {var B=this.__rs[name];A[B](z);};};}}});})();(function(){var a="Boolean",b="w-resize",c="touch",d="sw-resize",f="n-resize",g="resizableRight",h="ne-resize",i="se-resize",j="Integer",k="e-resize",l="resizableLeft",m="move",n="shorthand",o="maximized",p="resize",q="nw-resize",r="pointerup",s="pointerdown",t="qx.ui.core.MResizable",u="losecapture",v="resize-frame",w="resizableBottom",x="s-resize",y="pointermove",z="resizableTop",A="pointerout";qx.Mixin.define(t,{construct:function(){var content=this.getContentElement();content.addListener(s,this.__rH,this,true);content.addListener(r,this.__rI,this);content.addListener(y,this.__rK,this);content.addListener(A,this.__rL,this);content.addListener(u,this.__rJ,this);var B=content.getDomElement();if(B==null){B=window;};this.__rv=qx.event.Registration.getManager(B).getHandler(qx.event.handler.DragDrop);},properties:{resizableTop:{check:a,init:true},resizableRight:{check:a,init:true},resizableBottom:{check:a,init:true},resizableLeft:{check:a,init:true},resizable:{group:[z,g,w,l],mode:n},resizeSensitivity:{check:j,init:5},useResizeFrame:{check:a,init:true}},members:{__rv:null,__rw:null,__rx:null,__ry:null,__rz:null,__rA:null,__rB:null,RESIZE_TOP:1,RESIZE_BOTTOM:2,RESIZE_LEFT:4,RESIZE_RIGHT:8,_getResizeFrame:function(){var C=this.__rw;if(!C){C=this.__rw=new qx.ui.core.Widget();C.setAppearance(v);C.exclude();qx.core.Init.getApplication().getRoot().add(C);};return C;},__rC:function(){var location=this.getContentLocation();var D=this._getResizeFrame();D.setUserBounds(location.left,location.top,location.right-location.left,location.bottom-location.top);D.show();D.setZIndex(this.getZIndex()+1);},__rD:function(e){var F=this.__rx;var E=this.getSizeHint();var I=this.__rB;var H=this.__rA;var J=H.width;var G=H.height;var L=H.left;var top=H.top;var K;if((F&this.RESIZE_TOP)||(F&this.RESIZE_BOTTOM)){K=Math.max(I.top,Math.min(I.bottom,e.getDocumentTop()))-this.__rz;if(F&this.RESIZE_TOP){G-=K;}else {G+=K;};if(G<E.minHeight){G=E.minHeight;}else if(G>E.maxHeight){G=E.maxHeight;};if(F&this.RESIZE_TOP){top+=H.height-G;};};if((F&this.RESIZE_LEFT)||(F&this.RESIZE_RIGHT)){K=Math.max(I.left,Math.min(I.right,e.getDocumentLeft()))-this.__ry;if(F&this.RESIZE_LEFT){J-=K;}else {J+=K;};if(J<E.minWidth){J=E.minWidth;}else if(J>E.maxWidth){J=E.maxWidth;};if(F&this.RESIZE_LEFT){L+=H.width-J;};};return {viewportLeft:L,viewportTop:top,parentLeft:H.bounds.left+L-H.left,parentTop:H.bounds.top+top-H.top,width:J,height:G};},__rE:{'1':f,'2':x,'4':b,'8':k,'5':q,'6':d,'9':h,'10':i},__rF:function(e){var location=this.getContentLocation();var O=this.getResizeSensitivity();var N=e.getDocumentLeft();var P=e.getDocumentTop();var M=this.__rG(location,N,P,O);if(M>0){M=M|this.__rG(location,N,P,O*2);};this.__rx=M;},__rG:function(location,T,R,S){var Q=0;if(this.getResizableTop()&&Math.abs(location.top-R)<S&&T>location.left-S&&T<location.right+S){Q+=this.RESIZE_TOP;}else if(this.getResizableBottom()&&Math.abs(location.bottom-R)<S&&T>location.left-S&&T<location.right+S){Q+=this.RESIZE_BOTTOM;};if(this.getResizableLeft()&&Math.abs(location.left-T)<S&&R>location.top-S&&R<location.bottom+S){Q+=this.RESIZE_LEFT;}else if(this.getResizableRight()&&Math.abs(location.right-T)<S&&R>location.top-S&&R<location.bottom+S){Q+=this.RESIZE_RIGHT;};return Q;},__rH:function(e){if(!this.__rx||!this.getEnabled()||e.getPointerType()==c){return;};this.addState(p);this.__ry=e.getDocumentLeft();this.__rz=e.getDocumentTop();var location=this.getContentLocation();var V=this.getBounds();this.__rA={top:location.top,left:location.left,width:location.right-location.left,height:location.bottom-location.top,bounds:qx.lang.Object.clone(V)};var parent=this.getLayoutParent();var W=parent.getContentLocation();var U=parent.getBounds();this.__rB={left:W.left,top:W.top,right:W.left+U.width,bottom:W.top+U.height};if(this.getUseResizeFrame()){this.__rC();};this.capture();e.stop();},__rI:function(e){if(!this.hasState(p)||!this.getEnabled()||e.getPointerType()==c){return;};if(this.getUseResizeFrame()){this._getResizeFrame().exclude();};var X=this.__rD(e);this.setWidth(X.width);this.setHeight(X.height);if(this.getResizableLeft()||this.getResizableTop()){this.setLayoutProperties({left:X.parentLeft,top:X.parentTop});};this.__rx=0;this.removeState(p);this.resetCursor();this.getApplicationRoot().resetGlobalCursor();this.releaseCapture();e.stopPropagation();},__rJ:function(e){if(!this.__rx){return;};this.resetCursor();this.getApplicationRoot().resetGlobalCursor();this.removeState(m);if(this.getUseResizeFrame()){this._getResizeFrame().exclude();};},__rK:function(e){if(!this.getEnabled()||e.getPointerType()==c){return;};if(this.hasState(p)){var ba=this.__rD(e);if(this.getUseResizeFrame()){var Y=this._getResizeFrame();Y.setUserBounds(ba.viewportLeft,ba.viewportTop,ba.width,ba.height);}else {this.setWidth(ba.width);this.setHeight(ba.height);if(this.getResizableLeft()||this.getResizableTop()){this.setLayoutProperties({left:ba.parentLeft,top:ba.parentTop});};};e.stopPropagation();}else if(!this.hasState(o)&&!this.__rv.isSessionActive()){this.__rF(e);var bd=this.__rx;var bc=this.getApplicationRoot();if(bd){var bb=this.__rE[bd];this.setCursor(bb);bc.setGlobalCursor(bb);}else if(this.getCursor()){this.resetCursor();bc.resetGlobalCursor();};};},__rL:function(e){if(e.getPointerType()==c){return;};if(this.getCursor()&&!this.hasState(p)){this.resetCursor();this.getApplicationRoot().resetGlobalCursor();};}},destruct:function(){if(this.__rw!=null&&!qx.core.ObjectRegistry.inShutDown){this.__rw.destroy();this.__rw=null;};this.__rv=null;}});})();(function(){var a="beforeClose",b="beforeMinimize",c="Boolean",d="window-resize-frame",f="changeStatus",g="changeIcon",h="excluded",i="_applyModal",j="execute",k="restore-button",l="_applyActive",m="minimize-button",n="qx.event.type.Event",o="close-button",p="beforeRestore",q="statusbar",r="captionbar",s="String",t="minimize",u="dbltap",v="modal",w="changeModal",x="title",y="icon",z="showStatusbar",A="changeAlwaysOnTop",B="_applyShowStatusbar",C="pointerdown",D="maximized",E="_applyStatus",F="qx.ui.window.Window",G="normal",H="changeCaption",I="engine.name",J="statusbar-text",K="focusout",L="beforeMaximize",M="maximize",N="maximize-button",O="restore",P="window",Q="pane",R="close",S="changeActive",T="mshtml",U="_applyCaptionBarChange",V="active",W="minimized";qx.Class.define(F,{extend:qx.ui.core.Widget,include:[qx.ui.core.MRemoteChildrenHandling,qx.ui.core.MRemoteLayoutHandling,qx.ui.core.MResizable,qx.ui.core.MMovable,qx.ui.core.MContentPadding],construct:function(X,Y){qx.ui.core.Widget.call(this);this._setLayout(new qx.ui.layout.VBox());this._createChildControl(r);this._createChildControl(Q);if(Y!=null){this.setIcon(Y);};if(X!=null){this.setCaption(X);};this._updateCaptionBar();this.addListener(C,this._onWindowPointerDown,this,true);this.addListener(K,this._onWindowFocusOut,this);qx.core.Init.getApplication().getRoot().add(this);this.initVisibility();qx.ui.core.FocusHandler.getInstance().addRoot(this);this._getResizeFrame().setAppearance(d);},statics:{DEFAULT_MANAGER_CLASS:qx.ui.window.Manager},events:{"beforeClose":n,"close":n,"beforeMinimize":n,"minimize":n,"beforeMaximize":n,"maximize":n,"beforeRestore":n,"restore":n},properties:{appearance:{refine:true,init:P},visibility:{refine:true,init:h},focusable:{refine:true,init:true},active:{check:c,init:false,apply:l,event:S},alwaysOnTop:{check:c,init:false,event:A},modal:{check:c,init:false,event:w,apply:i},caption:{apply:U,event:H,nullable:true},icon:{check:s,nullable:true,apply:U,event:g,themeable:true},status:{check:s,nullable:true,apply:E,event:f},showClose:{check:c,init:true,apply:U,themeable:true},showMaximize:{check:c,init:true,apply:U,themeable:true},showMinimize:{check:c,init:true,apply:U,themeable:true},allowClose:{check:c,init:true,apply:U},allowMaximize:{check:c,init:true,apply:U},allowMinimize:{check:c,init:true,apply:U},showStatusbar:{check:c,init:false,apply:B}},members:{__rM:null,__rN:null,getChildrenContainer:function(){return this.getChildControl(Q);},_forwardStates:{active:true,maximized:true,showStatusbar:true,modal:true},setLayoutParent:function(parent){{};qx.ui.core.Widget.prototype.setLayoutParent.call(this,parent);},_createChildControlImpl:function(be,bd){var ba;switch(be){case q:ba=new qx.ui.container.Composite(new qx.ui.layout.HBox());this._add(ba);ba.add(this.getChildControl(J));break;case J:ba=new qx.ui.basic.Label();ba.setValue(this.getStatus());break;case Q:ba=new qx.ui.container.Composite();this._add(ba,{flex:1});break;case r:var bb=new qx.ui.layout.Grid();bb.setRowFlex(0,1);bb.setColumnFlex(1,1);ba=new qx.ui.container.Composite(bb);this._add(ba);ba.addListener(u,this._onCaptionPointerDblTap,this);this._activateMoveHandle(ba);break;case y:ba=new qx.ui.basic.Image(this.getIcon());this.getChildControl(r).add(ba,{row:0,column:0});break;case x:ba=new qx.ui.basic.Label(this.getCaption());ba.setWidth(0);ba.setAllowGrowX(true);var bc=this.getChildControl(r);bc.add(ba,{row:0,column:1});break;case m:ba=new qx.ui.form.Button();ba.setFocusable(false);ba.addListener(j,this._onMinimizeButtonTap,this);this.getChildControl(r).add(ba,{row:0,column:2});break;case k:ba=new qx.ui.form.Button();ba.setFocusable(false);ba.addListener(j,this._onRestoreButtonTap,this);this.getChildControl(r).add(ba,{row:0,column:3});break;case N:ba=new qx.ui.form.Button();ba.setFocusable(false);ba.addListener(j,this._onMaximizeButtonTap,this);this.getChildControl(r).add(ba,{row:0,column:4});break;case o:ba=new qx.ui.form.Button();ba.setFocusable(false);ba.addListener(j,this._onCloseButtonTap,this);this.getChildControl(r).add(ba,{row:0,column:6});break;};return ba||qx.ui.core.Widget.prototype._createChildControlImpl.call(this,be);},_updateCaptionBar:function(){var bg;var bh=this.getIcon();if(bh){this.getChildControl(y).setSource(bh);this._showChildControl(y);}else {this._excludeChildControl(y);};var bf=this.getCaption();if(bf){this.getChildControl(x).setValue(bf);this._showChildControl(x);}else {this._excludeChildControl(x);};if(this.getShowMinimize()){this._showChildControl(m);bg=this.getChildControl(m);this.getAllowMinimize()?bg.resetEnabled():bg.setEnabled(false);}else {this._excludeChildControl(m);};if(this.getShowMaximize()){if(this.isMaximized()){this._showChildControl(k);this._excludeChildControl(N);}else {this._showChildControl(N);this._excludeChildControl(k);};bg=this.getChildControl(N);this.getAllowMaximize()?bg.resetEnabled():bg.setEnabled(false);}else {this._excludeChildControl(N);this._excludeChildControl(k);};if(this.getShowClose()){this._showChildControl(o);bg=this.getChildControl(o);this.getAllowClose()?bg.resetEnabled():bg.setEnabled(false);}else {this._excludeChildControl(o);};},close:function(){if(!this.isVisible()){return;};if(this.fireNonBubblingEvent(a,qx.event.type.Event,[false,true])){this.hide();this.fireEvent(R);};},open:function(){this.show();this.setActive(true);this.focus();},center:function(){var parent=this.getLayoutParent();if(parent){var bj=parent.getBounds();if(bj){var bk=this.getSizeHint();var bi=Math.round((bj.width-bk.width)/2);var top=Math.round((bj.height-bk.height)/2);if(top<0){top=0;};this.moveTo(bi,top);return;};};{};},maximize:function(){if(this.isMaximized()){return;};var parent=this.getLayoutParent();if(parent!=null&&parent.supportsMaximize()){if(this.fireNonBubblingEvent(L,qx.event.type.Event,[false,true])){if(!this.isVisible()){this.open();};var bl=this.getLayoutProperties();this.__rN=bl.left===undefined?0:bl.left;this.__rM=bl.top===undefined?0:bl.top;this.setLayoutProperties({left:null,top:null,edge:0});this.addState(D);this._updateCaptionBar();this.fireEvent(M);};};},minimize:function(){if(!this.isVisible()){return;};if(this.fireNonBubblingEvent(b,qx.event.type.Event,[false,true])){var bm=this.getLayoutProperties();this.__rN=bm.left===undefined?0:bm.left;this.__rM=bm.top===undefined?0:bm.top;this.removeState(D);this.hide();this.fireEvent(t);};},restore:function(){if(this.getMode()===G){return;};if(this.fireNonBubblingEvent(p,qx.event.type.Event,[false,true])){if(!this.isVisible()){this.open();};var bn=this.__rN;var top=this.__rM;this.setLayoutProperties({edge:null,left:bn,top:top});this.removeState(D);this._updateCaptionBar();this.fireEvent(O);};},moveTo:function(bo,top){if(this.isMaximized()){return;};this.setLayoutProperties({left:bo,top:top});},isMaximized:function(){return this.hasState(D);},getMode:function(){if(!this.isVisible()){return W;}else {if(this.isMaximized()){return D;}else {return G;};};},_applyActive:function(bq,bp){if(bp){this.removeState(V);}else {this.addState(V);};},_applyModal:function(bs,br){if(br){this.removeState(v);}else {this.addState(v);};},_getContentPaddingTarget:function(){return this.getChildControl(Q);},_applyShowStatusbar:function(bv,bt){var bu=this._getResizeFrame();if(bv){this.addState(z);bu.addState(z);}else {this.removeState(z);bu.removeState(z);};if(bv){this._showChildControl(q);}else {this._excludeChildControl(q);};},_applyCaptionBarChange:function(bx,bw){this._updateCaptionBar();},_applyStatus:function(bz,by){var bA=this.getChildControl(J,true);if(bA){bA.setValue(bz);};},_applyFocusable:function(bC,bB){if(qx.core.Environment.get(I)!==T){qx.ui.core.Widget.prototype._applyFocusable.call(this,bC,bB);};},_onWindowEventStop:function(e){e.stopPropagation();},_onWindowPointerDown:function(e){this.setActive(true);},_onWindowFocusOut:function(e){if(this.getModal()){return;};var bD=e.getRelatedTarget();if(bD!=null&&!qx.ui.core.Widget.contains(this,bD)){this.setActive(false);};},_onCaptionPointerDblTap:function(e){if(this.getAllowMaximize()){this.isMaximized()?this.restore():this.maximize();};},_onMinimizeButtonTap:function(e){this.minimize();this.getChildControl(m).reset();},_onRestoreButtonTap:function(e){this.restore();this.getChildControl(k).reset();},_onMaximizeButtonTap:function(e){this.maximize();this.getChildControl(N).reset();},_onCloseButtonTap:function(e){this.close();this.getChildControl(o).reset();}}});})();(function(){var a="Number",b="qx.event.type.Event",c="modelChanged",d="_applyFirstColumnX",e="headerCellRendererChanged",f="Integer",g="qx.ui.table.pane.Model",h="_applyMaxColumnCount",i="visibilityChangedPre";qx.Class.define(g,{extend:qx.core.Object,construct:function(j){qx.core.Object.call(this);this.setTableColumnModel(j);},events:{"modelChanged":b},statics:{EVENT_TYPE_MODEL_CHANGED:c},properties:{firstColumnX:{check:f,init:0,apply:d},maxColumnCount:{check:a,init:-1,apply:h}},members:{__rO:null,__rP:null,_applyFirstColumnX:function(l,k){this.__rO=null;this.fireEvent(qx.ui.table.pane.Model.EVENT_TYPE_MODEL_CHANGED);},_applyMaxColumnCount:function(n,m){this.__rO=null;this.fireEvent(qx.ui.table.pane.Model.EVENT_TYPE_MODEL_CHANGED);},setTableColumnModel:function(o){if(this.__rP){this.__rP.removeListener(i,this._onColVisibilityChanged,this);this.__rP.removeListener(e,this._onHeaderCellRendererChanged,this);};this.__rP=o;this.__rP.addListener(i,this._onColVisibilityChanged,this);this.__rP.addListener(e,this._onHeaderCellRendererChanged,this);this.__rO=null;},_onColVisibilityChanged:function(p){this.__rO=null;this.fireEvent(qx.ui.table.pane.Model.EVENT_TYPE_MODEL_CHANGED);},_onHeaderCellRendererChanged:function(q){this.fireEvent(qx.ui.table.pane.Model.EVENT_TYPE_MODEL_CHANGED);},getColumnCount:function(){if(this.__rO==null){var r=this.getFirstColumnX();var t=this.getMaxColumnCount();var s=this.__rP.getVisibleColumnCount();if(t==-1||(r+t)>s){this.__rO=s-r;}else {this.__rO=t;};};return this.__rO;},getColumnAtX:function(v){var u=this.getFirstColumnX();return this.__rP.getVisibleColumnAtX(u+v);},getX:function(z){var w=this.getFirstColumnX();var y=this.getMaxColumnCount();var x=this.__rP.getVisibleX(z)-w;if(x>=0&&(y==-1||x<y)){return x;}else {return -1;};},getColumnLeft:function(D){var B=0;var C=this.getColumnCount();for(var x=0;x<C;x++ ){var A=this.getColumnAtX(x);if(A==D){return B;};B+=this.__rP.getColumnWidth(A);};return -1;},getTotalWidth:function(){var E=0;var F=this.getColumnCount();for(var x=0;x<F;x++ ){var G=this.getColumnAtX(x);E+=this.__rP.getColumnWidth(G);};return E;}},destruct:function(){if(this.__rP){this.__rP.removeListener(i,this._onColVisibilityChanged,this);this.__rP.removeListener(e,this._onHeaderCellRendererChanged,this);};this.__rP=null;}});})();(function(){var a="qx.event.type.Event",b="qx.ui.table.ITableModel",c="qx.event.type.Data";qx.Interface.define(b,{events:{"dataChanged":c,"metaDataChanged":a,"sorted":c},members:{getRowCount:function(){},getRowData:function(d){},getColumnCount:function(){},getColumnId:function(e){},getColumnIndexById:function(f){},getColumnName:function(g){},isColumnEditable:function(h){},isColumnSortable:function(i){},sortByColumn:function(j,k){},getSortColumnIndex:function(){},isSortAscending:function(){},prefetchRows:function(m,l){},getValue:function(o,n){},getValueById:function(q,p){},setValue:function(s,r,t){},setValueById:function(v,u,w){}}});})();(function(){var a=" != ",b="abstract",c="metaDataChanged",d="columnIdArr and columnNameArr have different length: ",e="getValue is abstract",f="qx.ui.table.model.Abstract",g="qx.event.type.Event",h="this.__columnIdArr and columnNameArr have different length: ",j="setValue is abstract",k="getRowCount is abstract",l="qx.event.type.Data";qx.Class.define(f,{type:b,extend:qx.core.Object,implement:qx.ui.table.ITableModel,events:{"dataChanged":l,"metaDataChanged":g,"sorted":l},construct:function(){qx.core.Object.call(this);this.__rQ=[];this.__rR=[];this.__rS={};},members:{__rQ:null,__rR:null,__rS:null,__oK:null,init:function(m){},getRowCount:function(){throw new Error(k);},getRowData:function(n){return null;},isColumnEditable:function(o){return false;},isColumnSortable:function(p){return false;},sortByColumn:function(q,r){},getSortColumnIndex:function(){return -1;},isSortAscending:function(){return true;},prefetchRows:function(t,s){},getValue:function(v,u){throw new Error(e);},getValueById:function(x,w){return this.getValue(this.getColumnIndexById(x),w);},setValue:function(z,y,A){throw new Error(j);},setValueById:function(C,B,D){this.setValue(this.getColumnIndexById(C),B,D);},getColumnCount:function(){return this.__rQ.length;},getColumnIndexById:function(E){return this.__rS[E];},getColumnId:function(F){return this.__rQ[F];},getColumnName:function(G){return this.__rR[G];},setColumnIds:function(H){this.__rQ=H;this.__rS={};for(var i=0;i<H.length;i++ ){this.__rS[H[i]]=i;};this.__rR=new Array(H.length);if(!this.__oK){this.fireEvent(c);};},setColumnNamesByIndex:function(I){if(this.__rQ.length!=I.length){throw new Error(h+this.__rQ.length+a+I.length);};this.__rR=I;this.fireEvent(c);},setColumnNamesById:function(J){this.__rR=new Array(this.__rQ.length);for(var i=0;i<this.__rQ.length; ++i){this.__rR[i]=J[this.__rQ[i]];};},setColumns:function(K,L){var M=this.__rQ.length==0||L;if(L==null){if(this.__rQ.length==0){L=K;}else {L=this.__rQ;};};if(L.length!=K.length){throw new Error(d+L.length+a+K.length);};if(M){this.__oK=true;this.setColumnIds(L);this.__oK=false;};this.setColumnNamesByIndex(K);}},destruct:function(){this.__rQ=this.__rR=this.__rS=null;}});})();(function(){var a="qx.ui.table.model.Simple",b="Boolean",c="this.__rowArr out of bounds: ",d="metaDataChanged",e=")",f="dataChanged",g=" (0..",h="sorted";qx.Class.define(a,{extend:qx.ui.table.model.Abstract,construct:function(){qx.ui.table.model.Abstract.call(this);this.__rT=[];this.__rU=-1;this.__rV=[];this.__rW=null;},properties:{caseSensitiveSorting:{check:b,init:true}},statics:{_defaultSortComparatorAscending:function(l,m){var k=l[arguments.callee.columnIndex];var n=m[arguments.callee.columnIndex];if(qx.lang.Type.isNumber(k)&&qx.lang.Type.isNumber(n)){var o=isNaN(k)?isNaN(n)?0:1:isNaN(n)?-1:null;if(o!=null){return o;};};return (k>n)?1:((k==n)?0:-1);},_defaultSortComparatorInsensitiveAscending:function(q,r){var p=(q[arguments.callee.columnIndex].toLowerCase?q[arguments.callee.columnIndex].toLowerCase():q[arguments.callee.columnIndex]);var s=(r[arguments.callee.columnIndex].toLowerCase?r[arguments.callee.columnIndex].toLowerCase():r[arguments.callee.columnIndex]);if(qx.lang.Type.isNumber(p)&&qx.lang.Type.isNumber(s)){var t=isNaN(p)?isNaN(s)?0:1:isNaN(s)?-1:null;if(t!=null){return t;};};return (p>s)?1:((p==s)?0:-1);},_defaultSortComparatorDescending:function(v,w){var u=v[arguments.callee.columnIndex];var x=w[arguments.callee.columnIndex];if(qx.lang.Type.isNumber(u)&&qx.lang.Type.isNumber(x)){var y=isNaN(u)?isNaN(x)?0:1:isNaN(x)?-1:null;if(y!=null){return y;};};return (u<x)?1:((u==x)?0:-1);},_defaultSortComparatorInsensitiveDescending:function(A,B){var z=(A[arguments.callee.columnIndex].toLowerCase?A[arguments.callee.columnIndex].toLowerCase():A[arguments.callee.columnIndex]);var C=(B[arguments.callee.columnIndex].toLowerCase?B[arguments.callee.columnIndex].toLowerCase():B[arguments.callee.columnIndex]);if(qx.lang.Type.isNumber(z)&&qx.lang.Type.isNumber(C)){var D=isNaN(z)?isNaN(C)?0:1:isNaN(C)?-1:null;if(D!=null){return D;};};return (z<C)?1:((z==C)?0:-1);}},members:{__rT:null,__rW:null,__rX:null,__rV:null,__rU:null,__rY:null,getRowData:function(E){var F=this.__rT[E];if(F==null||F.originalData==null){return F;}else {return F.originalData;};},getRowDataAsMap:function(G){var I=this.__rT[G];if(I!=null){var H={};for(var J=0;J<this.getColumnCount();J++ ){H[this.getColumnId(J)]=I[J];};if(I.originalData!=null){for(var K in I.originalData){if(H[K]==undefined){H[K]=I.originalData[K];};};};return H;};return (I&&I.originalData)?I.originalData:null;},getDataAsMapArray:function(){var M=this.getRowCount();var L=[];for(var i=0;i<M;i++ ){L.push(this.getRowDataAsMap(i));};return L;},setEditable:function(N){this.__rW=[];for(var O=0;O<this.getColumnCount();O++ ){this.__rW[O]=N;};this.fireEvent(d);},setColumnEditable:function(P,Q){if(Q!=this.isColumnEditable(P)){if(this.__rW==null){this.__rW=[];};this.__rW[P]=Q;this.fireEvent(d);};},isColumnEditable:function(R){return this.__rW?(this.__rW[R]==true):false;},setColumnSortable:function(S,T){if(T!=this.isColumnSortable(S)){if(this.__rX==null){this.__rX=[];};this.__rX[S]=T;this.fireEvent(d);};},isColumnSortable:function(U){return (this.__rX?(this.__rX[U]!==false):true);},sortByColumn:function(V,ba){var Y;var X=this.__rV[V];if(X){Y=(ba?X.ascending:X.descending);}else {if(this.getCaseSensitiveSorting()){Y=(ba?qx.ui.table.model.Simple._defaultSortComparatorAscending:qx.ui.table.model.Simple._defaultSortComparatorDescending);}else {Y=(ba?qx.ui.table.model.Simple._defaultSortComparatorInsensitiveAscending:qx.ui.table.model.Simple._defaultSortComparatorInsensitiveDescending);};};Y.columnIndex=V;this.__rT.sort(Y);this.__rU=V;this.__rY=ba;var W={columnIndex:V,ascending:ba};this.fireDataEvent(h,W);this.fireEvent(d);},setSortMethods:function(bb,bc){var bd;if(qx.lang.Type.isFunction(bc)){bd={ascending:bc,descending:function(be,bf){return bc(bf,be);}};}else {bd=bc;};this.__rV[bb]=bd;},getSortMethods:function(bg){return this.__rV[bg];},clearSorting:function(){if(this.__rU!=-1){this.__rU=-1;this.__rY=true;this.fireEvent(d);};},getSortColumnIndex:function(){return this.__rU;},_setSortColumnIndex:function(bh){this.__rU=bh;},isSortAscending:function(){return this.__rY;},_setSortAscending:function(bi){this.__rY=bi;},getRowCount:function(){return this.__rT.length;},getValue:function(bk,bj){if(bj<0||bj>=this.__rT.length){throw new Error(c+bj+g+this.__rT.length+e);};return this.__rT[bj][bk];},setValue:function(bm,bl,bo){if(this.__rT[bl][bm]!=bo){this.__rT[bl][bm]=bo;if(this.hasListener(f)){var bn={firstRow:bl,lastRow:bl,firstColumn:bm,lastColumn:bm};this.fireDataEvent(f,bn);};if(bm==this.__rU){this.clearSorting();};};},setData:function(bq,bp){this.__rT=bq;if(this.hasListener(f)){var br={firstRow:0,lastRow:bq.length-1,firstColumn:0,lastColumn:this.getColumnCount()-1};this.fireDataEvent(f,br);};if(bp!==false){this.clearSorting();};},getData:function(){return this.__rT;},setDataAsMapArray:function(bt,bs,bu){this.setData(this._mapArray2RowArr(bt,bs),bu);},addRows:function(bx,bw,bv){if(bw==null){bw=this.__rT.length;};bx.splice(0,0,bw,0);Array.prototype.splice.apply(this.__rT,bx);var by={firstRow:bw,lastRow:this.__rT.length-1,firstColumn:0,lastColumn:this.getColumnCount()-1};this.fireDataEvent(f,by);if(bv!==false){this.clearSorting();};},addRowsAsMapArray:function(bA,bB,bz,bC){this.addRows(this._mapArray2RowArr(bA,bz),bB,bC);},setRows:function(bF,bE,bD){if(bE==null){bE=0;};bF.splice(0,0,bE,bF.length);Array.prototype.splice.apply(this.__rT,bF);var bG={firstRow:bE,lastRow:this.__rT.length-1,firstColumn:0,lastColumn:this.getColumnCount()-1};this.fireDataEvent(f,bG);if(bD!==false){this.clearSorting();};},setRowsAsMapArray:function(bI,bJ,bH,bK){this.setRows(this._mapArray2RowArr(bI,bH),bJ,bK);},removeRows:function(bM,bN,bL){this.__rT.splice(bM,bN);var bO={firstRow:bM,lastRow:this.__rT.length-1,firstColumn:0,lastColumn:this.getColumnCount()-1,removeStart:bM,removeCount:bN};this.fireDataEvent(f,bO);if(bL!==false){this.clearSorting();};},_mapArray2RowArr:function(bQ,bU){var bT=bQ.length;var bP=this.getColumnCount();var bS=new Array(bT);var bR;for(var i=0;i<bT; ++i){bR=[];if(bU){bR.originalData=bQ[i];};for(var j=0;j<bP; ++j){bR[j]=bQ[i][this.getColumnId(j)];};bS[i]=bR;};return bS;}},destruct:function(){this.__rT=this.__rW=this.__rV=this.__rX=null;}});})();(function(){var a='Column 2',b='cellTap',c='Column 3',d='Column 1',e='qx_lib.ui.grid.Grid',f='click selection handler';qx.Class.define(e,{extend:qx.ui.table.Table,properties:{clickSelectionHandler:{nullable:true}},construct:function(){var g=new qx.ui.table.columnmodel.Resize();var h={tableColumnModel:function(i){return (new qx.ui.table.columnmodel.Resize(i));}};qx.ui.table.Table.call(this,null,h);this.init();},members:{getData:function(){return this.getTableModel().getData();},init:function(){this.setClickSelectionHandler(null);this.setColumns([d,a,c]);this.setData([]);},onClickSelection:function(j){this.setClickSelectionHandler(j);this.addListener(b,this.onClickSelectionHandler,this);},onClickSelectionHandler:function(l){var m=l.getRow();var k=this.getTableModel().getRowData(m);console.log(f,k);},setColumnFlex:function(o,n){var p=this.getTableColumnModel();var q=p.getBehavior();q.setWidth(o,0,n);},setColumnWidth:function(s,r){var t=this.getTableColumnModel();var u=t.getBehavior();u.setWidth(s,r);},setColumns:function(x,w){var v=new qx.ui.table.model.Simple();v.setColumns(x,w);this.setTableModel(v);},setData:function(y){this.getTableModel().setData(y);},setDataAsMapArray:function(z){this.getTableModel().setDataAsMapArray(z);}}});})();(function(){var a="columnVisibilityMenuCreateEnd",b="tableWidthChanged",c="verticalScrollBarChanged",d="qx.ui.table.columnmodel.resizebehavior.Abstract",e="qx.ui.table.columnmodel.Resize",f="_applyBehavior",g="separator",h="appear",i="visibilityChanged",j="Reset column widths",k="changeBehavior",l="user-button",m="widthChanged",n="execute";qx.Class.define(e,{extend:qx.ui.table.columnmodel.Basic,include:qx.locale.MTranslation,construct:function(){qx.ui.table.columnmodel.Basic.call(this);this.__sa=false;this.__sb=false;},properties:{behavior:{check:d,init:null,nullable:true,apply:f,event:k}},members:{__sb:null,__sa:null,__qA:null,_applyBehavior:function(q,p){if(p!=null){p.dispose();p=null;};q._setNumColumns(this.getOverallColumnCount());q.setTableColumnModel(this);},init:function(s,r){qx.ui.table.columnmodel.Basic.prototype.init.call(this,s,r);if(this.__qA==null){this.__qA=r;r.addListener(h,this._onappear,this);r.addListener(b,this._onTableWidthChanged,this);r.addListener(c,this._onverticalscrollbarchanged,this);r.addListener(a,this._addResetColumnWidthButton,this);this.addListener(m,this._oncolumnwidthchanged,this);this.addListener(i,this._onvisibilitychanged,this);};if(this.getBehavior()==null){this.setBehavior(new qx.ui.table.columnmodel.resizebehavior.Default());};this.getBehavior()._setNumColumns(s);},getTable:function(){return this.__qA;},_addResetColumnWidthButton:function(event){var v=event.getData();var u=v.columnButton;var t=v.menu;var o;o=u.factory(g);t.add(o);o=u.factory(l,{text:this.tr(j)});t.add(o);o.addListener(n,this._onappear,this);},_onappear:function(event){if(this.__sa){return;};this.__sa=true;{};this.getBehavior().onAppear(event,event.getType()!==h);this.__qA._updateScrollerWidths();this.__qA._updateScrollBarVisibility();this.__sa=false;this.__sb=true;},_onTableWidthChanged:function(event){if(this.__sa||!this.__sb){return;};this.__sa=true;{};this.getBehavior().onTableWidthChanged(event);this.__sa=false;},_onverticalscrollbarchanged:function(event){if(this.__sa||!this.__sb){return;};this.__sa=true;{};this.getBehavior().onVerticalScrollBarChanged(event);qx.event.Timer.once(function(){if(this.__qA&&!this.__qA.isDisposed()){this.__qA._updateScrollerWidths();this.__qA._updateScrollBarVisibility();};},this,0);this.__sa=false;},_oncolumnwidthchanged:function(event){if(this.__sa||!this.__sb){return;};this.__sa=true;{};this.getBehavior().onColumnWidthChanged(event);this.__sa=false;},_onvisibilitychanged:function(event){if(this.__sa||!this.__sb){return;};this.__sa=true;{};this.getBehavior().onVisibilityChanged(event);this.__sa=false;}},destruct:function(){this.__qA=null;}});})();(function(){var a="string",b="number",c="auto",d="*",e="qx.ui.core.ColumnData";qx.Class.define(e,{extend:qx.ui.core.LayoutItem,construct:function(){qx.ui.core.LayoutItem.call(this);this.setColumnWidth(c);},members:{__sc:null,renderLayout:function(g,top,f,h){this.__sc=f;},getComputedWidth:function(){return this.__sc;},getFlex:function(){return this.getLayoutProperties().flex||0;},setColumnWidth:function(l,i){var i=i||0;var j=null;if(typeof l==b){this.setWidth(l);}else if(typeof l==a){if(l==c){i=1;}else {var k=l.match(/^[0-9]+(?:\.[0-9]+)?([%\*])$/);if(k){if(k[1]==d){i=parseFloat(l);}else {j=l;};};};};this.setLayoutProperties({flex:i,width:j});}},environment:{"qx.tableResizeDebug":false}});})();(function(){var a="onAppear is abstract",b="onVisibilityChanged is abstract",c="onVerticalScrollBarChanged is abstract",d="abstract",e="qx.ui.table.columnmodel.resizebehavior.Abstract",f="_setNumColumns is abstract",g="onTableWidthChanged is abstract",h="onColumnWidthChanged is abstract";qx.Class.define(e,{type:d,extend:qx.core.Object,members:{_setNumColumns:function(i){throw new Error(f);},onAppear:function(event,j){throw new Error(a);},onTableWidthChanged:function(event){throw new Error(g);},onVerticalScrollBarChanged:function(event){throw new Error(c);},onColumnWidthChanged:function(event){throw new Error(h);},onVisibilityChanged:function(event){throw new Error(b);},_getAvailableWidth:function(){var l=this.getTableColumnModel();var o=l.getTable();var k=o._getPaneScrollerArr();if(!k[0]||!k[0].getLayoutParent().getBounds()){return null;};var n=k[0].getLayoutParent().getBounds().width;var m=k[k.length-1];n-=m.getPaneInsetRight();return n;}}});})();(function(){var a="Column number out of range",b="Function",c="Boolean",d="minWidth",e="width",f="__se",g="qx.ui.table.columnmodel.Resize",h="qx.ui.table.columnmodel.resizebehavior.Default",j="Unknown property: ",k="maxWidth",m="__sf";qx.Class.define(h,{extend:qx.ui.table.columnmodel.resizebehavior.Abstract,construct:function(){qx.ui.table.columnmodel.resizebehavior.Abstract.call(this);this.__sd=[];this.__se=new qx.ui.layout.HBox();this.__se.connectToWidget(this);this.__sf=new qx.util.DeferredCall(this._computeColumnsFlexWidth,this);},properties:{newResizeBehaviorColumnData:{check:b,init:function(n){return new qx.ui.core.ColumnData();}},initializeWidthsOnEveryAppear:{check:c,init:false},tableColumnModel:{check:g}},members:{__se:null,__sg:null,__sd:null,__sf:null,__sh:false,setWidth:function(p,o,q){if(p>=this.__sd.length){throw new Error(a);};this.__sd[p].setColumnWidth(o,q);this.__sf.schedule();},setMinWidth:function(s,r){if(s>=this.__sd.length){throw new Error(a);};this.__sd[s].setMinWidth(r);this.__sf.schedule();},setMaxWidth:function(u,t){if(u>=this.__sd.length){throw new Error(a);};this.__sd[u].setMaxWidth(t);this.__sf.schedule();},set:function(w,v){for(var x in v){switch(x){case e:this.setWidth(w,v[x]);break;case d:this.setMinWidth(w,v[x]);break;case k:this.setMaxWidth(w,v[x]);break;default:throw new Error(j+x);};};},onAppear:function(event,y){if(y===true||!this.__sh||this.getInitializeWidthsOnEveryAppear()){this._computeColumnsFlexWidth();this.__sh=true;};},onTableWidthChanged:function(event){this._computeColumnsFlexWidth();},onVerticalScrollBarChanged:function(event){this._computeColumnsFlexWidth();},onColumnWidthChanged:function(event){this._extendNextColumn(event);},onVisibilityChanged:function(event){var z=event.getData();if(z.visible){this._computeColumnsFlexWidth();return;};this._extendLastColumn(event);},_setNumColumns:function(B){var A=this.__sd;if(B<=A.length){A.splice(B,A.length);return;};for(var i=A.length;i<B;i++ ){A[i]=this.getNewResizeBehaviorColumnData()();A[i].columnNumber=i;};},getLayoutChildren:function(){return this.__sg;},_computeColumnsFlexWidth:function(){this.__sf.cancel();var C=this._getAvailableWidth();if(C===null){return;};var G=this.getTableColumnModel();var E=G.getVisibleColumns();var F=E.length;var D=this.__sd;var i,l;if(F===0){return;};var I=[];for(i=0;i<F;i++ ){I.push(D[E[i]]);};this.__sg=I;this.__si();this.__se.renderLayout(C,100,{top:0,right:0,bottom:0,left:0});for(i=0,l=I.length;i<l;i++ ){var H=I[i].getComputedWidth();G.setColumnWidth(E[i],H);};},__si:function(){this.__se.invalidateChildrenCache();var J=this.__sg;for(var i=0,l=J.length;i<l;i++ ){J[i].invalidateLayoutCache();};},_extendNextColumn:function(event){var L=this.getTableColumnModel();var R=event.getData();var M=L.getVisibleColumns();var O=this._getAvailableWidth();var K=M.length;if(R.newWidth>R.oldWidth){return;};var i;var N;var Q=0;for(i=0;i<K;i++ ){Q+=L.getColumnWidth(M[i]);};if(Q<O){for(i=0;i<M.length;i++ ){if(M[i]==R.col){N=M[i+1];break;};};if(N){var P=(O-(Q-L.getColumnWidth(N)));L.setColumnWidth(N,P);};};},_extendLastColumn:function(event){var T=this.getTableColumnModel();var ba=event.getData();if(ba.visible){return;};var U=T.getVisibleColumns();if(U.length==0){return;};var V=this._getAvailableWidth(T);var S=U.length;var i;var X;var Y=0;for(i=0;i<S;i++ ){Y+=T.getColumnWidth(U[i]);};if(Y<V){X=U[U.length-1];var W=(V-(Y-T.getColumnWidth(X)));T.setColumnWidth(X,W);};},_getResizeColumnData:function(){return this.__sd;}},destruct:function(){this.__sd=this.__sg=null;this._disposeObjects(f,m);}});})();(function(){var a="resizer",b="qx.ui.container.Resizer";qx.Class.define(b,{extend:qx.ui.container.Composite,include:qx.ui.core.MResizable,properties:{appearance:{refine:true,init:a}}});})();(function(){var a="qx.ui.container.Scroll",b="pane";qx.Class.define(a,{extend:qx.ui.core.scroll.AbstractScrollArea,include:[qx.ui.core.MContentPadding],construct:function(content){qx.ui.core.scroll.AbstractScrollArea.call(this);if(content){this.add(content);};},members:{add:function(c){this.getChildControl(b).add(c);},remove:function(d){this.getChildControl(b).remove(d);},getChildren:function(){return this.getChildControl(b).getChildren();},_getContentPaddingTarget:function(){return this.getChildControl(b);}}});})();(function(){var a=" array contains ",b="__gt",c="qx.ui.core.MSingleSelectionHandling",d=" items!",f="changeSelection",g="changeSelected",h="Could only select one item, but the selection",i="qx.event.type.Data";qx.Mixin.define(c,{events:{"changeSelection":i},members:{__gt:null,getSelection:function(){var j=this.__sj().getSelected();if(j){return [j];}else {return [];};},setSelection:function(k){switch(k.length){case 0:this.resetSelection();break;case 1:this.__sj().setSelected(k[0]);break;default:throw new Error(h+a+k.length+d);};},resetSelection:function(){this.__sj().resetSelected();},isSelected:function(l){return this.__sj().isSelected(l);},isSelectionEmpty:function(){return this.__sj().isSelectionEmpty();},getSelectables:function(m){return this.__sj().getSelectables(m);},_onChangeSelected:function(e){var o=e.getData();var n=e.getOldData();o==null?o=[]:o=[o];n==null?n=[]:n=[n];this.fireDataEvent(f,o,n);},__sj:function(){if(this.__gt==null){var p=this;this.__gt=new qx.ui.core.SingleSelectionManager({getItems:function(){return p._getItems();},isItemSelectable:function(q){if(p._isItemSelectable){return p._isItemSelectable(q);}else {return q.isVisible();};}});this.__gt.addListener(g,this._onChangeSelected,this);};this.__gt.setAllowEmptySelection(this._isAllowEmptySelection());return this.__gt;}},destruct:function(){this._disposeObjects(b);}});})();(function(){var a=", because it is not a child element!",b="__sm",c="Boolean",d="__sl",e="qx.ui.core.SingleSelectionManager",f="Could not check if ",g=" is selected,",h="Could not select ",j="__sk",k="changeSelected",l=" because it is not a child element!",m="qx.event.type.Data";qx.Class.define(e,{extend:qx.core.Object,construct:function(n){qx.core.Object.call(this);{};this.__sk=n;},events:{"changeSelected":m},properties:{allowEmptySelection:{check:c,init:true,apply:b}},members:{__sl:null,__sk:null,getSelected:function(){return this.__sl;},setSelected:function(o){if(!this.__so(o)){throw new Error(h+o+a);};this.__sn(o);},resetSelected:function(){this.__sn(null);},isSelected:function(p){if(!this.__so(p)){throw new Error(f+p+g+l);};return this.__sl===p;},isSelectionEmpty:function(){return this.__sl==null;},getSelectables:function(r){var q=this.__sk.getItems();var s=[];for(var i=0;i<q.length;i++ ){if(this.__sk.isItemSelectable(q[i])){s.push(q[i]);};};if(!r){for(var i=s.length-1;i>=0;i-- ){if(!s[i].getEnabled()){s.splice(i,1);};};};return s;},__sm:function(u,t){if(!u){this.__sn(this.__sl);};},__sn:function(v){var y=this.__sl;var w=v;if(w!=null&&y===w){return;};if(!this.isAllowEmptySelection()&&w==null){var x=this.getSelectables(true)[0];if(x){w=x;};};this.__sl=w;this.fireDataEvent(k,w,y);},__so:function(z){var A=this.__sk.getItems();for(var i=0;i<A.length;i++ ){if(A[i]===z){return true;};};return false;}},destruct:function(){if(this.__sk.toHashCode){this._disposeObjects(j);}else {this.__sk=null;};this._disposeObjects(d);}});})();(function(){var a="_applyDynamic",b="changeSelection",c="Boolean",d="qx.ui.container.Stack";qx.Class.define(d,{extend:qx.ui.core.Widget,implement:qx.ui.core.ISingleSelection,include:[qx.ui.core.MSingleSelectionHandling,qx.ui.core.MChildrenHandling],construct:function(){qx.ui.core.Widget.call(this);this._setLayout(new qx.ui.layout.Grow);this.addListener(b,this.__sp,this);},properties:{dynamic:{check:c,init:false,apply:a}},members:{_applyDynamic:function(g){var j=this._getChildren();var f=this.getSelection()[0];var h;for(var i=0,l=j.length;i<l;i++ ){h=j[i];if(h!=f){if(g){j[i].exclude();}else {j[i].hide();};};};},_getItems:function(){return this.getChildren();},_isAllowEmptySelection:function(){return true;},_isItemSelectable:function(k){return true;},__sp:function(e){var m=e.getOldData()[0];var n=e.getData()[0];if(m){if(this.isDynamic()){m.exclude();}else {m.hide();};};if(n){n.show();};},_afterAddChild:function(p){var o=this.getSelection()[0];if(!o){this.setSelection([p]);}else if(o!==p){if(this.isDynamic()){p.exclude();}else {p.hide();};};},_afterRemoveChild:function(q){if(this.getSelection()[0]===q){var r=this._getChildren()[0];if(r){this.setSelection([r]);}else {this.resetSelection();};};},previous:function(){var u=this.getSelection()[0];var s=this._indexOf(u)-1;var v=this._getChildren();if(s<0){s=v.length-1;};var t=v[s];this.setSelection([t]);},next:function(){var x=this.getSelection()[0];var w=this._indexOf(x)+1;var y=this._getChildren();var z=y[w]||y[0];this.setSelection([z]);}}});})();(function(){var a="_applyOverflowX",b="_applyOverflowY",c="auto",d="overflowX",e="visible",f="hidden",g="qx.ui.core.MNativeOverflow",h="scroll",i="overflowY";qx.Mixin.define(g,{properties:{overflowX:{check:[f,e,h,c],nullable:true,apply:a},overflowY:{check:[f,e,h,c],nullable:true,apply:b},overflow:{group:[d,i]}},members:{_applyOverflowX:function(j){this.getContentElement().setStyle(d,j);},_applyOverflowY:function(k){this.getContentElement().setStyle(i,k);}}});})();(function(){var a="mshtml",b="engine.name",c="relative",d="",e="text",f="none",g="position",h="webkit",i="changeHtml",j="_applyCssClass",k="qx.ui.embed.Html",l="_applyHtml",m="userSelect",n="color",o="String",p="browser.documentmode",q="html";qx.Class.define(k,{extend:qx.ui.core.Widget,include:[qx.ui.core.MNativeOverflow],construct:function(r){qx.ui.core.Widget.call(this);if(r!=null){this.setHtml(r);};},properties:{html:{check:o,apply:l,event:i,nullable:true},cssClass:{check:o,init:d,apply:j},selectable:{refine:true,init:true},focusable:{refine:true,init:true}},members:{getFocusElement:function(){return this.getContentElement();},_applyHtml:function(u,s){var t=this.getContentElement();if(qx.core.Environment.get(b)==a&&qx.core.Environment.get(p)==9){t.setStyle(g,c);};t.setAttribute(q,u||d);},_applyCssClass:function(w,v){this.getContentElement().removeClass(v);this.getContentElement().addClass(w);},_applySelectable:function(x){qx.ui.core.Widget.prototype._applySelectable.call(this,x);if((qx.core.Environment.get(b)==h)){this.getContentElement().setStyle(m,x?e:f);};},_applyFont:function(A,z){var y=A?qx.theme.manager.Font.getInstance().resolve(A).getStyles():qx.bom.Font.getDefaultStyles();if(this.getTextColor()!=null){delete y[n];};this.getContentElement().setStyles(y);},_applyTextColor:function(C,B){if(C){this.getContentElement().setStyle(n,qx.theme.manager.Color.getInstance().resolve(C));}else {this.getContentElement().removeStyle(n);};}}});})();(function(){var a="qx.ui.form.IRadioItem",b="qx.event.type.Data";qx.Interface.define(a,{events:{"changeValue":b},members:{setValue:function(c){},getValue:function(){},setGroup:function(d){this.assertInstance(d,qx.ui.form.RadioGroup);},getGroup:function(){}}});})();(function(){var a="Could not set the model selection. Maybe your models are not unique? ",b="qx.ui.form.MModelSelection",c="change",d="__sq",f="changeSelection",g="qx.event.type.Data";qx.Mixin.define(b,{construct:function(){this.__sq=new qx.data.Array();this.__sq.addListener(c,this.__st,this);this.addListener(f,this.__ss,this);},events:{changeModelSelection:g},members:{__sq:null,__sr:false,__ss:function(){if(this.__sr){return;};var l=this.getSelection();var m=[];for(var i=0;i<l.length;i++ ){var h=l[i];var k=h.getModel?h.getModel():null;if(k!==null){m.push(k);};};if(m.length===l.length){try{this.setModelSelection(m);}catch(e){throw new Error(a+e);};};},__st:function(){this.__sr=true;var p=this.getSelectables(true);var q=[];var o=this.__sq.toArray();for(var i=0;i<o.length;i++ ){var s=o[i];for(var j=0;j<p.length;j++ ){var t=p[j];var n=t.getModel?t.getModel():null;if(s===n){q.push(t);break;};};};this.setSelection(q);this.__sr=false;var r=this.getSelection();if(!qx.lang.Array.equals(r,q)){this.__ss();};},getModelSelection:function(){return this.__sq;},setModelSelection:function(u){if(!u){this.__sq.removeAll();return;};{};u.unshift(this.__sq.getLength());u.unshift(0);var v=this.__sq.splice.apply(this.__sq,u);v.dispose();}},destruct:function(){this._disposeObjects(d);}});})();(function(){var a="qx.ui.form.IModelSelection";qx.Interface.define(a,{members:{setModelSelection:function(b){},getModelSelection:function(){}}});})();(function(){var a="_applyAllowEmptySelection",b="_applyInvalidMessage",c="qx.ui.form.RadioGroup",d="Boolean",f="_applyValid",g="",h="changeRequired",j="changeValid",k="__su",m="changeEnabled",n="changeInvalidMessage",o="changeSelection",p="changeValue",q="_applyEnabled",r="String";qx.Class.define(c,{extend:qx.core.Object,implement:[qx.ui.core.ISingleSelection,qx.ui.form.IForm,qx.ui.form.IModelSelection],include:[qx.ui.core.MSingleSelectionHandling,qx.ui.form.MModelSelection],construct:function(s){qx.core.Object.call(this);this.__su=[];this.addListener(o,this.__sp,this);if(s!=null){this.add.apply(this,arguments);};},properties:{enabled:{check:d,apply:q,event:m,init:true},wrap:{check:d,init:true},allowEmptySelection:{check:d,init:false,apply:a},valid:{check:d,init:true,apply:f,event:j},required:{check:d,init:false,event:h},invalidMessage:{check:r,init:g,event:n,apply:b},requiredInvalidMessage:{check:r,nullable:true,event:n}},members:{__su:null,getItems:function(){return this.__su;},add:function(u){var v=this.__su;var t;for(var i=0,l=arguments.length;i<l;i++ ){t=arguments[i];if(qx.lang.Array.contains(v,t)){continue;};t.addListener(p,this._onItemChangeChecked,this);v.push(t);t.setGroup(this);if(t.getValue()){this.setSelection([t]);};};if(!this.isAllowEmptySelection()&&v.length>0&&!this.getSelection()[0]){this.setSelection([v[0]]);};},remove:function(w){var x=this.__su;if(qx.lang.Array.contains(x,w)){qx.lang.Array.remove(x,w);if(w.getGroup()===this){w.resetGroup();};w.removeListener(p,this._onItemChangeChecked,this);if(w.getValue()){this.resetSelection();};};},getChildren:function(){return this.__su;},_onItemChangeChecked:function(e){var y=e.getTarget();if(y.getValue()){this.setSelection([y]);}else if(this.getSelection()[0]==y){this.resetSelection();};},_applyInvalidMessage:function(A,z){for(var i=0;i<this.__su.length;i++ ){this.__su[i].setInvalidMessage(A);};},_applyValid:function(C,B){for(var i=0;i<this.__su.length;i++ ){this.__su[i].setValid(C);};},_applyEnabled:function(F,E){var D=this.__su;if(F==null){for(var i=0,l=D.length;i<l;i++ ){D[i].resetEnabled();};}else {for(var i=0,l=D.length;i<l;i++ ){D[i].setEnabled(F);};};},_applyAllowEmptySelection:function(H,G){if(!H&&this.isSelectionEmpty()){this.resetSelection();};},selectNext:function(){var J=this.getSelection()[0];var K=this.__su;var I=K.indexOf(J);if(I==-1){return;};var i=0;var length=K.length;if(this.getWrap()){I=(I+1)%length;}else {I=Math.min(I+1,length-1);};while(i<length&&!K[I].getEnabled()){I=(I+1)%length;i++ ;};this.setSelection([K[I]]);},selectPrevious:function(){var M=this.getSelection()[0];var N=this.__su;var L=N.indexOf(M);if(L==-1){return;};var i=0;var length=N.length;if(this.getWrap()){L=(L-1+length)%length;}else {L=Math.max(L-1,0);};while(i<length&&!N[L].getEnabled()){L=(L-1+length)%length;i++ ;};this.setSelection([N[L]]);},_getItems:function(){return this.getItems();},_isAllowEmptySelection:function(){return this.isAllowEmptySelection();},_isItemSelectable:function(O){return this.__su.indexOf(O)!=-1;},__sp:function(e){var Q=e.getData()[0];var P=e.getOldData()[0];if(P){P.setValue(false);};if(Q){Q.setValue(true);};}},destruct:function(){this._disposeArray(k);}});})();(function(){var a="hovered",b="Boolean",c="pressed",d="_applyTriState",f="pointerover",g="changeValue",h="Space",i="keydown",j="abandoned",k="undetermined",l="_applyGroup",m="button",n="execute",o="Enter",p="qx.ui.form.RadioGroup",q="_applyValue",r="qx.ui.form.ToggleButton",s="pointerup",t="pointerdown",u="keyup",v="checked",w="pointerout";qx.Class.define(r,{extend:qx.ui.basic.Atom,include:[qx.ui.core.MExecutable],implement:[qx.ui.form.IBooleanForm,qx.ui.form.IExecutable,qx.ui.form.IRadioItem],construct:function(x,y){qx.ui.basic.Atom.call(this,x,y);this.addListener(f,this._onPointerOver);this.addListener(w,this._onPointerOut);this.addListener(t,this._onPointerDown);this.addListener(s,this._onPointerUp);this.addListener(i,this._onKeyDown);this.addListener(u,this._onKeyUp);this.addListener(n,this._onExecute,this);},properties:{appearance:{refine:true,init:m},focusable:{refine:true,init:true},value:{check:b,nullable:true,event:g,apply:q,init:false},group:{check:p,nullable:true,apply:l},triState:{check:b,apply:d,nullable:true,init:null}},members:{_applyGroup:function(A,z){if(z){z.remove(this);};if(A){A.add(this);};},_applyValue:function(C,B){C?this.addState(v):this.removeState(v);if(this.isTriState()){if(C===null){this.addState(k);}else if(B===null){this.removeState(k);};};},_applyTriState:function(E,D){this._applyValue(this.getValue());},_onExecute:function(e){this.toggleValue();},_onPointerOver:function(e){if(e.getTarget()!==this){return;};this.addState(a);if(this.hasState(j)){this.removeState(j);this.addState(c);};},_onPointerOut:function(e){if(e.getTarget()!==this){return;};this.removeState(a);if(this.hasState(c)){if(!this.getValue()){this.removeState(c);};this.addState(j);};},_onPointerDown:function(e){if(!e.isLeftPressed()){return;};this.capture();this.removeState(j);this.addState(c);e.stopPropagation();},_onPointerUp:function(e){this.releaseCapture();if(this.hasState(j)){this.removeState(j);}else if(this.hasState(c)){this.execute();};this.removeState(c);e.stopPropagation();},_onKeyDown:function(e){switch(e.getKeyIdentifier()){case o:case h:this.removeState(j);this.addState(c);e.stopPropagation();};},_onKeyUp:function(e){if(!this.hasState(c)){return;};switch(e.getKeyIdentifier()){case o:case h:this.removeState(j);this.execute();this.removeState(c);e.stopPropagation();};}}});})();(function(){var a="label",b="checkbox",c="qx.ui.form.CheckBox",d="value",e="toolTipText",f="enabled",g="menu";qx.Class.define(c,{extend:qx.ui.form.ToggleButton,include:[qx.ui.form.MForm,qx.ui.form.MModelProperty],implement:[qx.ui.form.IForm,qx.ui.form.IModel],construct:function(h){{};qx.ui.form.ToggleButton.call(this,h);this.setValue(false);},properties:{appearance:{refine:true,init:b},allowGrowX:{refine:true,init:false}},members:{_forwardStates:{invalid:true,focused:true,undetermined:true,checked:true,hovered:true},_bindableProperties:[f,a,e,d,g]}});})();(function(){var a="popupOpen",b="PageUp",c="blur",d="one",f="Escape",g="_applyMaxListHeight",h="Function",i="",j="Abstract method: _onListPointerDown()",k="visible",l="changeSelection",m="middle",n="Abstract method: _onListChangeSelection()",o="changeVisibility",p="resize",q="list",r="qx.ui.form.AbstractSelectBox",s="pointerdown",t="keypress",u="PageDown",v="abstract",w="popup",x="Number",y="pane",z="tap";qx.Class.define(r,{extend:qx.ui.core.Widget,include:[qx.ui.core.MRemoteChildrenHandling,qx.ui.form.MForm],implement:[qx.ui.form.IForm],type:v,construct:function(){qx.ui.core.Widget.call(this);var A=new qx.ui.layout.HBox();this._setLayout(A);A.setAlignY(m);this.addListener(t,this._onKeyPress);this.addListener(c,this._onBlur,this);this.addListener(p,this._onResize,this);},properties:{focusable:{refine:true,init:true},width:{refine:true,init:120},maxListHeight:{check:x,apply:g,nullable:true,init:200},format:{check:h,init:function(B){return this._defaultFormat(B);},nullable:true}},members:{_createChildControlImpl:function(E,D){var C;switch(E){case q:C=new qx.ui.form.List().set({focusable:false,keepFocus:true,height:null,width:null,maxHeight:this.getMaxListHeight(),selectionMode:d,quickSelection:true});C.addListener(l,this._onListChangeSelection,this);C.addListener(s,this._onListPointerDown,this);C.getChildControl(y).addListener(z,this.close,this);break;case w:C=new qx.ui.popup.Popup(new qx.ui.layout.VBox());C.setAutoHide(false);C.setKeepActive(true);C.add(this.getChildControl(q));C.addListener(o,this._onPopupChangeVisibility,this);break;};return C||qx.ui.core.Widget.prototype._createChildControlImpl.call(this,E);},_applyMaxListHeight:function(G,F){this.getChildControl(q).setMaxHeight(G);},getChildrenContainer:function(){return this.getChildControl(q);},open:function(){var H=this.getChildControl(w);H.placeToWidget(this,true);H.show();},close:function(){this.getChildControl(w).hide();},toggle:function(){var I=this.getChildControl(w).isVisible();if(I){this.close();}else {this.open();};},_defaultFormat:function(J){var L=J?J.getLabel():i;var K=J?J.getRich():false;if(K){L=L.replace(/<[^>]+?>/g,i);L=qx.bom.String.unescape(L);};return L;},_onBlur:function(e){this.close();},_onKeyPress:function(e){var M=e.getKeyIdentifier();var N=this.getChildControl(w);if(N.isHidden()&&(M==u||M==b)){e.stopPropagation();}else if(!N.isHidden()&&M==f){this.close();e.stop();}else {this.getChildControl(q).handleKeyPress(e);};},_onResize:function(e){this.getChildControl(w).setMinWidth(e.getData().width);},_onListChangeSelection:function(e){throw new Error(n);},_onListPointerDown:function(e){throw new Error(j);},_onPopupChangeVisibility:function(e){e.getData()==k?this.addState(a):this.removeState(a);}}});})();(function(){var c="os.name",d="Abstract method call: _getLastSelectable()",f="Could not select more than one items in mode: ",g="Boolean",h="one",j="Abstract method call: _getLocation()",k="Left",m="Abstract method call: _scrollBy()",n="The method getSelectedItem() is only supported in 'single' and 'one' selection mode!",o="Abstract method call: _scrollItemIntoView()",p="Escape",q="Abstract method call: _isSelectable()",r="Space",s="Abstract method call: _getPage()",t="right",u="lead",v="Down",w="Abstract method call: _getFirstSelectable()",x="Up",y="qx.event.type.Data",z="under",A="A",B="_applyMode",C="PageDown",D="Can not select multiple items in selection mode: ",E="osx",F="mouse",G="interval",H="Abstract method call: _styleSelectable()",I="anchor",J="Abstract method call: _capture()",K="Abstract method call: _getDimension()",L="The method invertSelection() is only supported in 'multi' and 'additive' selection mode!",M="!",N="Abstract method call: _getScroll()",O="Abstract method call: _getRelatedSelectable()",P="End",Q="__sx",R="quick",S="selected",T="key",U="Abstract method call: _getSelectableLocationX()",V="multi",W="left",X="Abstract method call: _selectableToHashCode()",Y="Home",bt="Can not select all items in selection mode: ",bu="single",bv="Right",bp="Abstract method call: getSelectables()",bq="Abstract method call: _getSelectableLocationY()",br="additive",bs="abstract",bA="changeSelection",bB="above",bC="Abstract method call: _getSelectableRange()",bD="tap",bw="drag",bx="PageUp",by="qx.ui.core.selection.Abstract",bz="Abstract method call: _releaseCapture()";qx.Class.define(by,{type:bs,extend:qx.core.Object,construct:function(){qx.core.Object.call(this);this.__gx={};},events:{"changeSelection":y},properties:{mode:{check:[bu,V,br,h],init:bu,apply:B},drag:{check:g,init:false},quick:{check:g,init:false}},members:{__sv:0,__sw:0,__sx:null,__sy:null,__sz:null,__sA:null,__sB:null,__sC:null,__sD:null,__sE:null,__sF:null,__sG:null,__sH:null,__sI:null,__sJ:null,__sK:null,__sL:null,__gx:null,__sM:null,__sN:null,_userInteraction:false,__sO:null,getSelectionContext:function(){return this.__sK;},selectAll:function(){var bE=this.getMode();if(bE==bu||bE==h){throw new Error(bt+bE);};this._selectAllItems();this._fireChange();},selectItem:function(bF){this._setSelectedItem(bF);var bG=this.getMode();if(bG!==bu&&bG!==h){this._setLeadItem(bF);this._setAnchorItem(bF);};this._scrollItemIntoView(bF);this._fireChange();},addItem:function(bH){var bI=this.getMode();if(bI===bu||bI===h){this._setSelectedItem(bH);}else {if(this._getAnchorItem()==null){this._setAnchorItem(bH);};this._setLeadItem(bH);this._addToSelection(bH);};this._scrollItemIntoView(bH);this._fireChange();},removeItem:function(bJ){this._removeFromSelection(bJ);if(this.getMode()===h&&this.isSelectionEmpty()){var bK=this._applyDefaultSelection();if(bK==bJ){return;};};if(this.getLeadItem()==bJ){this._setLeadItem(null);};if(this._getAnchorItem()==bJ){this._setAnchorItem(null);};this._fireChange();},selectItemRange:function(bL,bM){var bN=this.getMode();if(bN==bu||bN==h){throw new Error(D+bN);};this._selectItemRange(bL,bM);this._setAnchorItem(bL);this._setLeadItem(bM);this._scrollItemIntoView(bM);this._fireChange();},clearSelection:function(){if(this.getMode()==h){var bO=this._applyDefaultSelection(true);if(bO!=null){return;};};this._clearSelection();this._setLeadItem(null);this._setAnchorItem(null);this._fireChange();},replaceSelection:function(bP){var bQ=this.getMode();if(bQ==h||bQ===bu){if(bP.length>1){throw new Error(f+bQ+M);};if(bP.length==1){this.selectItem(bP[0]);}else {this.clearSelection();};return;}else {this._replaceMultiSelection(bP);};},getSelectedItem:function(){var bR=this.getMode();if(bR===bu||bR===h){var bS=this._getSelectedItem();return bS!=undefined?bS:null;};throw new Error(n);},getSelection:function(){return qx.lang.Object.getValues(this.__gx);},getSortedSelection:function(){var bU=this.getSelectables();var bT=qx.lang.Object.getValues(this.__gx);bT.sort(function(a,b){return bU.indexOf(a)-bU.indexOf(b);});return bT;},isItemSelected:function(bV){var bW=this._selectableToHashCode(bV);return this.__gx[bW]!==undefined;},isSelectionEmpty:function(){return qx.lang.Object.isEmpty(this.__gx);},invertSelection:function(){var bY=this.getMode();if(bY===bu||bY===h){throw new Error(L);};var bX=this.getSelectables();for(var i=0;i<bX.length;i++ ){this._toggleInSelection(bX[i]);};this._fireChange();},_setLeadItem:function(cb){var ca=this.__sL;if(ca!==null){this._styleSelectable(ca,u,false);};if(cb!==null){this._styleSelectable(cb,u,true);};this.__sL=cb;},getLeadItem:function(){return this.__sL;},_setAnchorItem:function(cd){var cc=this.__sM;if(cc!=null){this._styleSelectable(cc,I,false);};if(cd!=null){this._styleSelectable(cd,I,true);};this.__sM=cd;},_getAnchorItem:function(){return this.__sM!==null?this.__sM:null;},_isSelectable:function(ce){throw new Error(q);},_getSelectableFromPointerEvent:function(event){var cf=event.getTarget();if(cf&&this._isSelectable(cf)){return cf;};return null;},_selectableToHashCode:function(cg){throw new Error(X);},_styleSelectable:function(ch,ci,cj){throw new Error(H);},_capture:function(){throw new Error(J);},_releaseCapture:function(){throw new Error(bz);},_getLocation:function(){throw new Error(j);},_getDimension:function(){throw new Error(K);},_getSelectableLocationX:function(ck){throw new Error(U);},_getSelectableLocationY:function(cl){throw new Error(bq);},_getScroll:function(){throw new Error(N);},_scrollBy:function(cm,cn){throw new Error(m);},_scrollItemIntoView:function(co){throw new Error(o);},getSelectables:function(cp){throw new Error(bp);},_getSelectableRange:function(cr,cq){throw new Error(bC);},_getFirstSelectable:function(){throw new Error(w);},_getLastSelectable:function(){throw new Error(d);},_getRelatedSelectable:function(cs,ct){throw new Error(O);},_getPage:function(cv,cu){throw new Error(s);},_applyMode:function(cx,cw){this._setLeadItem(null);this._setAnchorItem(null);this._clearSelection();if(cx===h){this._applyDefaultSelection(true);};this._fireChange();},handlePointerOver:function(event){if(this.__sO!=null&&this.__sO!=this._getScroll().top){this.__sO=null;return;};if(event.getPointerType()!=F){return;};this._userInteraction=true;if(!this.getQuick()){this._userInteraction=false;return;};var cz=this.getMode();if(cz!==h&&cz!==bu){this._userInteraction=false;return;};var cy=this._getSelectableFromPointerEvent(event);if(cy===null){this._userInteraction=false;return;};this._setSelectedItem(cy);this._fireChange(R);this._userInteraction=false;},handlePointerDown:function(event){this._userInteraction=true;var cB=this._getSelectableFromPointerEvent(event);if(cB===null){this._userInteraction=false;return;};var cD=event.isCtrlPressed()||(qx.core.Environment.get(c)==E&&event.isMetaPressed());var cA=event.isShiftPressed();if(this.isItemSelected(cB)&&!cA&&!cD&&!this.getDrag()){this.__sN=cB;this._userInteraction=false;return;}else {this.__sN=null;};this._scrollItemIntoView(cB);var cC=this.getMode();if(this.getDrag()&&cC!==bu&&cC!==h&&!cA&&!cD&&event.getPointerType()==F){this._setAnchorItem(cB);this._setLeadItem(cB);this.__sB=this._getLocation();this.__sy=this._getScroll();this.__sC=event.getDocumentLeft()+this.__sy.left;this.__sD=event.getDocumentTop()+this.__sy.top;this.__sE=true;this._capture();};this._fireChange(bD);this._userInteraction=false;},handleTap:function(event){this._userInteraction=true;var cH=event.isCtrlPressed()||(qx.core.Environment.get(c)==E&&event.isMetaPressed());var cE=event.isShiftPressed();if(!cH&&!cE&&this.__sN!=null){this._userInteraction=false;var cF=this._getSelectableFromPointerEvent(event);if(cF===null||!this.isItemSelected(cF)){return;};};var cF=this._getSelectableFromPointerEvent(event);if(cF===null){this._userInteraction=false;return;};switch(this.getMode()){case bu:case h:this._setSelectedItem(cF);break;case br:this._setLeadItem(cF);this._setAnchorItem(cF);this._toggleInSelection(cF);break;case V:this._setLeadItem(cF);if(cE){var cG=this._getAnchorItem();if(cG===null){cG=this._getFirstSelectable();this._setAnchorItem(cG);};this._selectItemRange(cG,cF,cH);}else if(cH){this._setAnchorItem(cF);this._toggleInSelection(cF);}else {this._setAnchorItem(cF);this._setSelectedItem(cF);};break;};this._cleanup();},handleLoseCapture:function(event){this._cleanup();},handlePointerMove:function(event){if(!this.__sE){return;};this.__sF=event.getDocumentLeft();this.__sG=event.getDocumentTop();this._userInteraction=true;var cI=this.__sF+this.__sy.left;if(cI>this.__sC){this.__sH=1;}else if(cI<this.__sC){this.__sH=-1;}else {this.__sH=0;};var cJ=this.__sG+this.__sy.top;if(cJ>this.__sD){this.__sI=1;}else if(cJ<this.__sD){this.__sI=-1;}else {this.__sI=0;};var location=this.__sB;if(this.__sF<location.left){this.__sv=this.__sF-location.left;}else if(this.__sF>location.right){this.__sv=this.__sF-location.right;}else {this.__sv=0;};if(this.__sG<location.top){this.__sw=this.__sG-location.top;}else if(this.__sG>location.bottom){this.__sw=this.__sG-location.bottom;}else {this.__sw=0;};if(!this.__sx){this.__sx=new qx.event.Timer(100);this.__sx.addListener(G,this._onInterval,this);};this.__sx.start();this._autoSelect();event.stopPropagation();this._userInteraction=false;},handleAddItem:function(e){var cK=e.getData();if(this.getMode()===h&&this.isSelectionEmpty()){this.addItem(cK);};},handleRemoveItem:function(e){this.removeItem(e.getData());},_cleanup:function(){if(!this.getDrag()&&this.__sE){return;};if(this.__sJ){this._fireChange(bD);};delete this.__sE;delete this.__sz;delete this.__sA;this._releaseCapture();if(this.__sx){this.__sx.stop();};},_onInterval:function(e){this._scrollBy(this.__sv,this.__sw);this.__sy=this._getScroll();this._autoSelect();},_autoSelect:function(){var cT=this._getDimension();var cM=Math.max(0,Math.min(this.__sF-this.__sB.left,cT.width))+this.__sy.left;var cL=Math.max(0,Math.min(this.__sG-this.__sB.top,cT.height))+this.__sy.top;if(this.__sz===cM&&this.__sA===cL){return;};this.__sz=cM;this.__sA=cL;var cQ=this._getAnchorItem();var cO=cQ;var cR=this.__sH;var cU,cN;while(cR!==0){cU=cR>0?this._getRelatedSelectable(cO,t):this._getRelatedSelectable(cO,W);if(cU!==null){cN=this._getSelectableLocationX(cU);if((cR>0&&cN.left<=cM)||(cR<0&&cN.right>=cM)){cO=cU;continue;};};break;};var cS=this.__sI;var cV,cP;while(cS!==0){cV=cS>0?this._getRelatedSelectable(cO,z):this._getRelatedSelectable(cO,bB);if(cV!==null){cP=this._getSelectableLocationY(cV);if((cS>0&&cP.top<=cL)||(cS<0&&cP.bottom>=cL)){cO=cV;continue;};};break;};var cW=this.getMode();if(cW===V){this._selectItemRange(cQ,cO);}else if(cW===br){if(this.isItemSelected(cQ)){this._selectItemRange(cQ,cO,true);}else {this._deselectItemRange(cQ,cO);};this._setAnchorItem(cO);};this._fireChange(bw);},__oX:{Home:1,Down:1,Right:1,PageDown:1,End:1,Up:1,Left:1,PageUp:1},handleKeyPress:function(event){this._userInteraction=true;var dd,dc;var df=event.getKeyIdentifier();var de=this.getMode();var cY=event.isCtrlPressed()||(qx.core.Environment.get(c)==E&&event.isMetaPressed());var da=event.isShiftPressed();var db=false;if(df===A&&cY){if(de!==bu&&de!==h){this._selectAllItems();db=true;};}else if(df===p){if(de!==bu&&de!==h){this._clearSelection();db=true;};}else if(df===r){var cX=this.getLeadItem();if(cX!=null&&!da){if(cY||de===br){this._toggleInSelection(cX);}else {this._setSelectedItem(cX);};db=true;};}else if(this.__oX[df]){db=true;if(de===bu||de==h){dd=this._getSelectedItem();}else {dd=this.getLeadItem();};if(dd!==null){switch(df){case Y:dc=this._getFirstSelectable();break;case P:dc=this._getLastSelectable();break;case x:dc=this._getRelatedSelectable(dd,bB);break;case v:dc=this._getRelatedSelectable(dd,z);break;case k:dc=this._getRelatedSelectable(dd,W);break;case bv:dc=this._getRelatedSelectable(dd,t);break;case bx:dc=this._getPage(dd,true);break;case C:dc=this._getPage(dd,false);break;};}else {switch(df){case Y:case v:case bv:case C:dc=this._getFirstSelectable();break;case P:case x:case k:case bx:dc=this._getLastSelectable();break;};};if(dc!==null){switch(de){case bu:case h:this._setSelectedItem(dc);break;case br:this._setLeadItem(dc);break;case V:if(da){var dg=this._getAnchorItem();if(dg===null){this._setAnchorItem(dg=this._getFirstSelectable());};this._setLeadItem(dc);this._selectItemRange(dg,dc,cY);}else {this._setAnchorItem(dc);this._setLeadItem(dc);if(!cY){this._setSelectedItem(dc);};};break;};this.__sO=this._getScroll().top;this._scrollItemIntoView(dc);};};if(db){event.stop();this._fireChange(T);};this._userInteraction=false;},_selectAllItems:function(){var dh=this.getSelectables();for(var i=0,l=dh.length;i<l;i++ ){this._addToSelection(dh[i]);};},_clearSelection:function(){var di=this.__gx;for(var dj in di){this._removeFromSelection(di[dj]);};this.__gx={};},_selectItemRange:function(dk,dp,dm){var dq=this._getSelectableRange(dk,dp);if(!dm){var dn=this.__gx;var dr=this.__sP(dq);for(var dl in dn){if(!dr[dl]){this._removeFromSelection(dn[dl]);};};};for(var i=0,l=dq.length;i<l;i++ ){this._addToSelection(dq[i]);};},_deselectItemRange:function(du,ds){var dt=this._getSelectableRange(du,ds);for(var i=0,l=dt.length;i<l;i++ ){this._removeFromSelection(dt[i]);};},__sP:function(dw){var dx={};var dv;for(var i=0,l=dw.length;i<l;i++ ){dv=dw[i];dx[this._selectableToHashCode(dv)]=dv;};return dx;},_getSelectedItem:function(){for(var dy in this.__gx){return this.__gx[dy];};return null;},_setSelectedItem:function(dA){if(this._isSelectable(dA)){var dz=this.__gx;var dB=this._selectableToHashCode(dA);if(!dz[dB]||(dz.length>=2)){this._clearSelection();this._addToSelection(dA);};};},_addToSelection:function(dC){var dD=this._selectableToHashCode(dC);if(this.__gx[dD]==null&&this._isSelectable(dC)){this.__gx[dD]=dC;this._styleSelectable(dC,S,true);this.__sJ=true;};},_toggleInSelection:function(dE){var dF=this._selectableToHashCode(dE);if(this.__gx[dF]==null){this.__gx[dF]=dE;this._styleSelectable(dE,S,true);}else {delete this.__gx[dF];this._styleSelectable(dE,S,false);};this.__sJ=true;},_removeFromSelection:function(dG){var dH=this._selectableToHashCode(dG);if(this.__gx[dH]!=null){delete this.__gx[dH];this._styleSelectable(dG,S,false);this.__sJ=true;};},_replaceMultiSelection:function(dK){if(dK.length===0){this.clearSelection();return;};var dL=false;var dO,dN;var dJ={};for(var i=0,l=dK.length;i<l;i++ ){dO=dK[i];if(this._isSelectable(dO)){dN=this._selectableToHashCode(dO);dJ[dN]=dO;};};var dP=dK[0];var dI=dO;var dM=this.__gx;for(var dN in dM){if(dJ[dN]){delete dJ[dN];}else {dO=dM[dN];delete dM[dN];this._styleSelectable(dO,S,false);dL=true;};};for(var dN in dJ){dO=dM[dN]=dJ[dN];this._styleSelectable(dO,S,true);dL=true;};if(!dL){return false;};this._scrollItemIntoView(dI);this._setLeadItem(dP);this._setAnchorItem(dP);this.__sJ=true;this._fireChange();},_fireChange:function(dQ){if(this.__sJ){this.__sK=dQ||null;this.fireDataEvent(bA,this.getSelection());delete this.__sJ;};},_applyDefaultSelection:function(dR){if(dR===true||this.getMode()===h&&this.isSelectionEmpty()){var dS=this._getFirstSelectable();if(dS!=null){this.selectItem(dS);};return dS;};return null;}},destruct:function(){this._disposeObjects(Q);this.__gx=this.__sN=this.__sM=null;this.__sL=null;}});})();(function(){var a="vertical",b="under",c="above",d="qx.ui.core.selection.Widget",e="left",f="right";qx.Class.define(d,{extend:qx.ui.core.selection.Abstract,construct:function(g){qx.ui.core.selection.Abstract.call(this);this.__lt=g;},members:{__lt:null,_isSelectable:function(h){return this._isItemSelectable(h)&&h.getLayoutParent()===this.__lt;},_selectableToHashCode:function(j){return j.$$hash;},_styleSelectable:function(k,m,n){n?k.addState(m):k.removeState(m);},_capture:function(){this.__lt.capture();},_releaseCapture:function(){this.__lt.releaseCapture();},_isItemSelectable:function(o){if(this._userInteraction){return o.isVisible()&&o.isEnabled();}else {return o.isVisible();};},_getWidget:function(){return this.__lt;},_getLocation:function(){var p=this.__lt.getContentElement().getDomElement();return p?qx.bom.element.Location.get(p):null;},_getDimension:function(){return this.__lt.getInnerSize();},_getSelectableLocationX:function(q){var r=q.getBounds();if(r){return {left:r.left,right:r.left+r.width};};},_getSelectableLocationY:function(s){var t=s.getBounds();if(t){return {top:t.top,bottom:t.top+t.height};};},_getScroll:function(){return {left:0,top:0};},_scrollBy:function(u,v){},_scrollItemIntoView:function(w){this.__lt.scrollChildIntoView(w);},getSelectables:function(x){var y=false;if(!x){y=this._userInteraction;this._userInteraction=true;};var B=this.__lt.getChildren();var z=[];var A;for(var i=0,l=B.length;i<l;i++ ){A=B[i];if(this._isItemSelectable(A)){z.push(A);};};this._userInteraction=y;return z;},_getSelectableRange:function(C,D){if(C===D){return [C];};var H=this.__lt.getChildren();var E=[];var G=false;var F;for(var i=0,l=H.length;i<l;i++ ){F=H[i];if(F===C||F===D){if(G){E.push(F);break;}else {G=true;};};if(G&&this._isItemSelectable(F)){E.push(F);};};return E;},_getFirstSelectable:function(){var I=this.__lt.getChildren();for(var i=0,l=I.length;i<l;i++ ){if(this._isItemSelectable(I[i])){return I[i];};};return null;},_getLastSelectable:function(){var J=this.__lt.getChildren();for(var i=J.length-1;i>0;i-- ){if(this._isItemSelectable(J[i])){return J[i];};};return null;},_getRelatedSelectable:function(N,O){var L=this.__lt.getOrientation()===a;var P=this.__lt.getChildren();var K=P.indexOf(N);var M;if((L&&O===c)||(!L&&O===e)){for(var i=K-1;i>=0;i-- ){M=P[i];if(this._isItemSelectable(M)){return M;};};}else if((L&&O===b)||(!L&&O===f)){for(var i=K+1;i<P.length;i++ ){M=P[i];if(this._isItemSelectable(M)){return M;};};};return null;},_getPage:function(R,Q){if(Q){return this._getFirstSelectable();}else {return this._getLastSelectable();};}},destruct:function(){this.__lt=null;}});})();(function(){var a="Invalid lead item: ",b="qx.ui.core.selection.ScrollArea";qx.Class.define(b,{extend:qx.ui.core.selection.Widget,members:{_isSelectable:function(c){return this._isItemSelectable(c)&&c.getLayoutParent()===this._getWidget().getChildrenContainer();},_getDimension:function(){return this._getWidget().getPaneSize();},_getScroll:function(){var d=this._getWidget();return {left:d.getScrollX(),top:d.getScrollY()};},_scrollBy:function(f,g){var e=this._getWidget();e.scrollByX(f);e.scrollByY(g);},_getPage:function(k,o){var q=this.getSelectables();var length=q.length;var p=q.indexOf(k);if(p===-1){throw new Error(a+k);};var h=this._getWidget();var r=h.getScrollY();var innerHeight=h.getInnerSize().height;var top,l,s;if(o){var n=r;var i=p;while(1){for(;i>=0;i-- ){top=h.getItemTop(q[i]);if(top<n){s=i+1;break;};};if(s==null){var t=this._getFirstSelectable();return t==k?null:t;};if(s>=p){n-=innerHeight+r-h.getItemBottom(k);s=null;continue;};return q[s];};}else {var m=innerHeight+r;var i=p;while(1){for(;i<length;i++ ){l=h.getItemBottom(q[i]);if(l>m){s=i-1;break;};};if(s==null){var j=this._getLastSelectable();return j==k?null:j;};if(s<=p){m+=h.getItemTop(k)-r;s=null;continue;};return q[s];};};}}});})();(function(){var a="Boolean",b="one",c="pointerover",d="qx.event.type.Data",f=", because it is not a child element!",g="_applyDragSelection",h="__gt",j="Could not add + ",k=" from selection, because it is not a child element!",l="changeSelection",m="Could not test if ",n="Could not remove ",o="addItem",p="multi",q="_applyQuickSelection",r="single",s="qx.ui.core.MMultiSelectionHandling",t="pointermove",u=" is selected, because it is not a child element!",v="_applySelectionMode",w="additive",x="keypress",y="losecapture",z="Could not select ",A="removeItem",B="tap",C="pointerdown",D=" to selection, because it is not a child element!";qx.Mixin.define(s,{construct:function(){var F=this.SELECTION_MANAGER;var E=this.__gt=new F(this);this.addListener(C,E.handlePointerDown,E);this.addListener(B,E.handleTap,E);this.addListener(c,E.handlePointerOver,E);this.addListener(t,E.handlePointerMove,E);this.addListener(y,E.handleLoseCapture,E);this.addListener(x,E.handleKeyPress,E);this.addListener(o,E.handleAddItem,E);this.addListener(A,E.handleRemoveItem,E);E.addListener(l,this._onSelectionChange,this);},events:{"changeSelection":d},properties:{selectionMode:{check:[r,p,w,b],init:r,apply:v},dragSelection:{check:a,init:false,apply:g},quickSelection:{check:a,init:false,apply:q}},members:{__gt:null,selectAll:function(){this.__gt.selectAll();},isSelected:function(G){if(!qx.ui.core.Widget.contains(this,G)){throw new Error(m+G+u);};return this.__gt.isItemSelected(G);},addToSelection:function(H){if(!qx.ui.core.Widget.contains(this,H)){throw new Error(j+H+D);};this.__gt.addItem(H);},removeFromSelection:function(I){if(!qx.ui.core.Widget.contains(this,I)){throw new Error(n+I+k);};this.__gt.removeItem(I);},selectRange:function(J,K){this.__gt.selectItemRange(J,K);},resetSelection:function(){this.__gt.clearSelection();},setSelection:function(L){for(var i=0;i<L.length;i++ ){if(!qx.ui.core.Widget.contains(this,L[i])){throw new Error(z+L[i]+f);};};if(L.length===0){this.resetSelection();}else {var M=this.getSelection();if(!qx.lang.Array.equals(M,L)){this.__gt.replaceSelection(L);};};},getSelection:function(){return this.__gt.getSelection();},getSortedSelection:function(){return this.__gt.getSortedSelection();},isSelectionEmpty:function(){return this.__gt.isSelectionEmpty();},getSelectionContext:function(){return this.__gt.getSelectionContext();},_getManager:function(){return this.__gt;},getSelectables:function(N){return this.__gt.getSelectables(N);},invertSelection:function(){this.__gt.invertSelection();},_getLeadItem:function(){var O=this.__gt.getMode();if(O===r||O===b){return this.__gt.getSelectedItem();}else {return this.__gt.getLeadItem();};},_applySelectionMode:function(Q,P){this.__gt.setMode(Q);},_applyDragSelection:function(S,R){this.__gt.setDrag(S);},_applyQuickSelection:function(U,T){this.__gt.setQuick(U);},_onSelectionChange:function(e){this.fireDataEvent(l,e.getData());}},destruct:function(){this._disposeObjects(h);}});})();(function(){var a="qx.ui.form.List",b="text",c="Boolean",d="one",f="addChildWidget",g="_applySpacing",h="Enter",j="Integer",k="action",m="keyinput",n="__sQ",o="qx.event.type.Data",p="addItem",q="vertical",r="removeChildWidget",s="",t="_applyOrientation",u="single",v="keypress",w="list",x="horizontal",y="label",z="pane",A="removeItem";qx.Class.define(a,{extend:qx.ui.core.scroll.AbstractScrollArea,implement:[qx.ui.core.IMultiSelection,qx.ui.form.IForm,qx.ui.form.IModelSelection],include:[qx.ui.core.MRemoteChildrenHandling,qx.ui.core.MMultiSelectionHandling,qx.ui.form.MForm,qx.ui.form.MModelSelection],construct:function(B){qx.ui.core.scroll.AbstractScrollArea.call(this);this.__sQ=this._createListItemContainer();this.__sQ.addListener(f,this._onAddChild,this);this.__sQ.addListener(r,this._onRemoveChild,this);this.getChildControl(z).add(this.__sQ);if(B){this.setOrientation(x);}else {this.initOrientation();};this.addListener(v,this._onKeyPress);this.addListener(m,this._onKeyInput);this.__sR=s;},events:{addItem:o,removeItem:o},properties:{appearance:{refine:true,init:w},focusable:{refine:true,init:true},orientation:{check:[x,q],init:q,apply:t},spacing:{check:j,init:0,apply:g,themeable:true},enableInlineFind:{check:c,init:true}},members:{__sR:null,__sS:null,__sQ:null,SELECTION_MANAGER:qx.ui.core.selection.ScrollArea,getChildrenContainer:function(){return this.__sQ;},_onAddChild:function(e){this.fireDataEvent(p,e.getData());},_onRemoveChild:function(e){this.fireDataEvent(A,e.getData());},handleKeyPress:function(e){if(!this._onKeyPress(e)){this._getManager().handleKeyPress(e);};},_createListItemContainer:function(){return new qx.ui.container.Composite;},_applyOrientation:function(F,C){var D=F===x;var E=D?new qx.ui.layout.HBox():new qx.ui.layout.VBox();var content=this.__sQ;content.setLayout(E);content.setAllowGrowX(!D);content.setAllowGrowY(D);this._applySpacing(this.getSpacing());},_applySpacing:function(H,G){this.__sQ.getLayout().setSpacing(H);},_onKeyPress:function(e){if(e.getKeyIdentifier()==h&&!e.isAltPressed()){var I=this.getSelection();for(var i=0;i<I.length;i++ ){I[i].fireEvent(k);};return true;};return false;},_onKeyInput:function(e){if(!this.getEnableInlineFind()){return;};var J=this.getSelectionMode();if(!(J===u||J===d)){return;};if(((new Date).valueOf()-this.__sS)>1000){this.__sR=s;};this.__sR+=e.getChar();var K=this.findItemByLabelFuzzy(this.__sR);if(K){this.setSelection([K]);};this.__sS=(new Date).valueOf();},findItemByLabelFuzzy:function(M){M=M.toLowerCase();var L=this.getChildren();for(var i=0,l=L.length;i<l;i++ ){var N=L[i].getLabel();if(N&&N.toLowerCase().indexOf(M)==0){return L[i];};};return null;},findItem:function(P,T){if(T!==false){P=P.toLowerCase();};var R=this.getChildren();var U;for(var i=0,l=R.length;i<l;i++ ){U=R[i];var S;if(U.isRich()){var O=U.getChildControl(y,true);if(O){var Q=O.getContentElement().getDomElement();if(Q){S=qx.bom.element.Attribute.get(Q,b);};};}else {S=U.getLabel();};if(S!=null){if(S.translate){S=S.translate();};if(T!==false){S=S.toLowerCase();};if(S.toString()==P.toString()){return U;};};};return null;}},destruct:function(){this._disposeObjects(n);}});})();(function(){var a="blur",b="Enter",c="changeValue",d="selected",f="key",g="button",h="Down",i="execute",j="qx.event.type.Data",k="inner",l="visible",m="quick",n="String",o="list",p="textfield",q="qx.ui.form.ComboBox",r="single",s="focusin",t="combobox",u="popup",v="focusout",w="_applyPlaceholder",x="tap";qx.Class.define(q,{extend:qx.ui.form.AbstractSelectBox,implement:[qx.ui.form.IStringForm],construct:function(){qx.ui.form.AbstractSelectBox.call(this);var y=this._createChildControl(p);this._createChildControl(g);this.addListener(x,this._onTap);this.addListener(s,function(e){y.fireNonBubblingEvent(s,qx.event.type.Focus);},this);this.addListener(v,function(e){y.fireNonBubblingEvent(v,qx.event.type.Focus);},this);},properties:{appearance:{refine:true,init:t},placeholder:{check:n,nullable:true,apply:w}},events:{"changeValue":j},members:{__sT:null,__sU:null,_applyPlaceholder:function(A,z){this.getChildControl(p).setPlaceholder(A);},_createChildControlImpl:function(D,C){var B;switch(D){case p:B=new qx.ui.form.TextField();B.setFocusable(false);B.addState(k);B.addListener(c,this._onTextFieldChangeValue,this);B.addListener(a,this.close,this);this._add(B,{flex:1});break;case g:B=new qx.ui.form.Button();B.setFocusable(false);B.setKeepActive(true);B.addState(k);B.addListener(i,this.toggle,this);this._add(B);break;case o:B=qx.ui.form.AbstractSelectBox.prototype._createChildControlImpl.call(this,D);B.setSelectionMode(r);break;};return B||qx.ui.form.AbstractSelectBox.prototype._createChildControlImpl.call(this,D);},_forwardStates:{focused:true,invalid:true},tabFocus:function(){var E=this.getChildControl(p);E.getFocusElement().focus();E.selectAllText();},focus:function(){qx.ui.form.AbstractSelectBox.prototype.focus.call(this);this.getChildControl(p).getFocusElement().focus();},setValue:function(G){var F=this.getChildControl(p);if(F.getValue()==G){return;};F.setValue(G);},getValue:function(){return this.getChildControl(p).getValue();},resetValue:function(){this.getChildControl(p).setValue(null);},_onKeyPress:function(e){var I=this.getChildControl(u);var H=e.getKeyIdentifier();if(H==h&&e.isAltPressed()){this.getChildControl(g).addState(d);this.toggle();e.stopPropagation();}else if(H==b){if(I.isVisible()){this._setPreselectedItem();this.resetAllTextSelection();this.close();e.stop();};}else if(I.isVisible()){qx.ui.form.AbstractSelectBox.prototype._onKeyPress.call(this,e);};},_onTap:function(e){this.close();},_onListPointerDown:function(e){this._setPreselectedItem();},_setPreselectedItem:function(){if(this.__sT){var J=this.__sT.getLabel();if(this.getFormat()!=null){J=this.getFormat().call(this,this.__sT);};if(J&&J.translate){J=J.translate();};this.setValue(J);this.__sT=null;};},_onListChangeSelection:function(e){var K=e.getData();if(K.length>0){var M=this.getChildControl(o);var L=M.getSelectionContext();if(L==m||L==f){this.__sT=K[0];}else {var N=K[0].getLabel();if(this.getFormat()!=null){N=this.getFormat().call(this,K[0]);};if(N&&N.translate){N=N.translate();};this.setValue(N);this.__sT=null;};};},_onPopupChangeVisibility:function(e){qx.ui.form.AbstractSelectBox.prototype._onPopupChangeVisibility.call(this,e);var P=this.getChildControl(u);if(P.isVisible()){var O=this.getChildControl(o);var Q=this.getValue();var R=null;if(Q){R=O.findItem(Q);};if(R){O.setSelection([R]);}else {O.resetSelection();};}else {if(e.getOldData()==l){this.tabFocus();};};this.getChildControl(g).removeState(d);},_onTextFieldChangeValue:function(e){var T=e.getData();var S=this.getChildControl(o);if(T!=null){var U=S.findItem(T,false);if(U){S.setSelection([U]);}else {S.resetSelection();};}else {S.resetSelection();};this.fireDataEvent(c,T,e.getOldData());},getTextSelection:function(){return this.getChildControl(p).getTextSelection();},getTextSelectionLength:function(){return this.getChildControl(p).getTextSelectionLength();},setTextSelection:function(V,W){this.getChildControl(p).setTextSelection(V,W);},clearTextSelection:function(){this.getChildControl(p).clearTextSelection();},selectAllText:function(){this.getChildControl(p).selectAllText();},resetAllTextSelection:function(){this.clearTextSelection();this.selectAllText();}}});})();(function(){var a="qx.ui.form.IDateForm",b="qx.event.type.Data";qx.Interface.define(a,{events:{"changeValue":b},members:{setValue:function(c){return arguments.length==1;},resetValue:function(){},getValue:function(){}}});})();(function(){var a="popupOpen",b="blur",c="qx.util.format.DateFormat",d="Escape",f="changeValue",g="Left",h="Down",i="button",j="Up",k="execute",l="qx.event.type.Data",m="inner",n="list",o="changeLocale",p="visible",q="middle",r="String",s="_applyDateFormat",t="pointerup",u="",v="changeVisibility",w="textfield",x="qx.dynlocale",y="medium",z="focusin",A="popup",B="qx.ui.form.DateField",C="focusout",D="datefield",E="_applyPlaceholder",F="hidden",G="tap",H="Right";qx.Class.define(B,{extend:qx.ui.core.Widget,include:[qx.ui.core.MRemoteChildrenHandling,qx.ui.form.MForm],implement:[qx.ui.form.IForm,qx.ui.form.IDateForm],construct:function(){qx.ui.core.Widget.call(this);var J=new qx.ui.layout.HBox();this._setLayout(J);J.setAlignY(q);var I=this._createChildControl(w);this._createChildControl(i);this.addListener(G,this._onTap,this);this.addListener(b,this._onBlur,this);this.addListener(z,function(e){I.fireNonBubblingEvent(z,qx.event.type.Focus);I.setTextSelection(0,0);},this);this.addListener(C,function(e){I.fireNonBubblingEvent(C,qx.event.type.Focus);},this);this._setDefaultDateFormat();this._addLocaleChangeListener();},events:{"changeValue":l},properties:{dateFormat:{check:c,apply:s},placeholder:{check:r,nullable:true,apply:E},appearance:{refine:true,init:D},focusable:{refine:true,init:true},width:{refine:true,init:120}},statics:{__sV:null,__sW:null,getDefaultDateFormatter:function(){var K=qx.locale.Date.getDateFormat(y).toString();if(K==this.__sV){return this.__sW;};if(this.__sW){this.__sW.dispose();};this.__sW=new qx.util.format.DateFormat(K,qx.locale.Manager.getInstance().getLocale());this.__sV=K;return this.__sW;}},members:{__sX:null,_forwardStates:{focused:true,invalid:true},_setDefaultDateFormat:function(){this.setDateFormat(qx.ui.form.DateField.getDefaultDateFormatter());},_addLocaleChangeListener:function(){if(qx.core.Environment.get(x)){this.__sX=qx.locale.Manager.getInstance().addListener(o,function(){this._setDefaultDateFormat();},this);};},setValue:function(N){var L=this.getChildControl(w);L.setValue(this.getDateFormat().format(N));var M=this.getChildControl(n);M.setValue(N);},getValue:function(){var O=this.getChildControl(w).getValue();try{return this.getDateFormat().parse(O);}catch(P){return null;};},resetValue:function(){var Q=this.getChildControl(w);Q.setValue(u);var R=this.getChildControl(n);R.setValue(null);},open:function(){var S=this.getChildControl(A);S.placeToWidget(this,true);S.show();},close:function(){this.getChildControl(A).hide();},toggle:function(){var T=this.getChildControl(A).isVisible();if(T){this.close();}else {this.open();};},_applyDateFormat:function(Y,W){if(!W){return;};try{var V=this.getChildControl(w);var X=V.getValue();var U=W.parse(X);V.setValue(Y.format(U));}catch(ba){};},_applyPlaceholder:function(bc,bb){this.getChildControl(w).setPlaceholder(bc);},_createChildControlImpl:function(bf,be){var bd;switch(bf){case w:bd=new qx.ui.form.TextField();bd.setFocusable(false);bd.addState(m);bd.addListener(f,this._onTextFieldChangeValue,this);bd.addListener(b,this.close,this);this._add(bd,{flex:1});break;case i:bd=new qx.ui.form.Button();bd.setFocusable(false);bd.setKeepActive(true);bd.addState(m);bd.addListener(k,this.toggle,this);this._add(bd);break;case n:bd=new qx.ui.control.DateChooser();bd.setFocusable(false);bd.setKeepFocus(true);bd.addListener(k,this._onChangeDate,this);break;case A:bd=new qx.ui.popup.Popup(new qx.ui.layout.VBox);bd.setAutoHide(false);bd.add(this.getChildControl(n));bd.addListener(t,this._onChangeDate,this);bd.addListener(v,this._onPopupChangeVisibility,this);break;};return bd||qx.ui.core.Widget.prototype._createChildControlImpl.call(this,bf);},_onChangeDate:function(e){var bg=this.getChildControl(w);var bh=this.getChildControl(n).getValue();bg.setValue(this.getDateFormat().format(bh));this.close();},_onTap:function(e){this.close();},_onBlur:function(e){this.close();},_onKeyPress:function(e){var bi=e.getKeyIdentifier();if(bi==h&&e.isAltPressed()){this.toggle();e.stopPropagation();return;};var bj=this.getChildControl(A);if(bj.getVisibility()==F){return;};if(bi==d){this.close();e.stopPropagation();return;};if(bi===g||bi===H||bi===h||bi===j){e.preventDefault();};this.getChildControl(n).handleKeyPress(e);},_onPopupChangeVisibility:function(e){e.getData()==p?this.addState(a):this.removeState(a);var bl=this.getChildControl(A);if(bl.isVisible()){var bm=this.getChildControl(n);var bk=this.getValue();bm.setValue(bk);};},_onTextFieldChangeValue:function(e){var bn=this.getValue();if(bn!=null){var bo=this.getChildControl(n);bo.setValue(bn);};this.fireDataEvent(f,this.getValue());},isEmpty:function(){var bp=this.getChildControl(w).getValue();return bp==null||bp==u;}},destruct:function(){if(qx.core.Environment.get(x)){if(this.__sX){qx.locale.Manager.getInstance().removeListenerById(this.__sX);};};}});})();(function(){var a="PageUp",b="day",c="lastMonth",d="yyyyMMMM",f="Next month",g="Escape",h="Left",j="last-month-button",k="default",l="today",m="weekday",n="changeValue",o="Space",p="selected",q="Down",r="Integer",s="qx.ui.control.DateChooser",t="Up",u="Enter",v="PageDown",w="day#",z="changeLocale",A="Next year",B="ww",C="_applyValue",D="next-month-button",E="next-month-button-tooltip",F="weekday#",G="last-month-button-tooltip",H="datechooser",I="header",J="week",K="lastYear",L="pointerup",M="otherMonth",N="month-year-label",O="nextYear",P="last-year-button",Q="changeShownYear",R="week#",S="qx.dynlocale",T="next-year-button",U="Last month",V="Right",W="Last year",X="MMMM yyyy",Y="EE",bs="week#0",bt="keypress",bu="",bo="navigation-bar",bp="nextMonth",bq="last-year-button-tooltip",br="next-year-button-tooltip",by="Date",bz="tap",bA="pointerdown",bB="date-pane",bv="dbltap",bw="weekend",bx="changeShownMonth";qx.Class.define(s,{extend:qx.ui.core.Widget,include:[qx.ui.core.MExecutable,qx.ui.form.MForm],implement:[qx.ui.form.IExecutable,qx.ui.form.IForm,qx.ui.form.IDateForm],construct:function(bC){qx.ui.core.Widget.call(this);var bF=new qx.ui.layout.VBox();this._setLayout(bF);this._createChildControl(bo);this._createChildControl(bB);this.addListener(bt,this._onKeyPress);var bD=qx.ui.control.DateChooser;if(!bD.MONTH_YEAR_FORMAT){bD.MONTH_YEAR_FORMAT=qx.locale.Date.getDateTimeFormat(d,X);};var bE=(bC!=null)?bC:new Date();this.showMonth(bE.getMonth(),bE.getFullYear());if(qx.core.Environment.get(S)){qx.locale.Manager.getInstance().addListener(z,this._updateDatePane,this);};this.addListener(bA,this._onPointerUpDown,this);this.addListener(L,this._onPointerUpDown,this);},statics:{MONTH_YEAR_FORMAT:null,WEEKDAY_FORMAT:Y,WEEK_FORMAT:B},properties:{appearance:{refine:true,init:H},width:{refine:true,init:200},height:{refine:true,init:150},shownMonth:{check:r,init:null,nullable:true,event:bx},shownYear:{check:r,init:null,nullable:true,event:Q},value:{check:by,init:null,nullable:true,event:n,apply:C}},members:{__sY:null,__ta:null,__tb:null,_forwardStates:{invalid:true},_createChildControlImpl:function(bK,bH){var bG;switch(bK){case bo:bG=new qx.ui.container.Composite(new qx.ui.layout.HBox());bG.add(this.getChildControl(P));bG.add(this.getChildControl(j));bG.add(this.getChildControl(N),{flex:1});bG.add(this.getChildControl(D));bG.add(this.getChildControl(T));this._add(bG);break;case bq:bG=new qx.ui.tooltip.ToolTip(this.tr(W));break;case P:bG=new qx.ui.toolbar.Button();bG.addState(K);bG.setFocusable(false);bG.setToolTip(this.getChildControl(bq));bG.addListener(bz,this._onNavButtonTap,this);break;case G:bG=new qx.ui.tooltip.ToolTip(this.tr(U));break;case j:bG=new qx.ui.toolbar.Button();bG.addState(c);bG.setFocusable(false);bG.setToolTip(this.getChildControl(G));bG.addListener(bz,this._onNavButtonTap,this);break;case E:bG=new qx.ui.tooltip.ToolTip(this.tr(f));break;case D:bG=new qx.ui.toolbar.Button();bG.addState(bp);bG.setFocusable(false);bG.setToolTip(this.getChildControl(E));bG.addListener(bz,this._onNavButtonTap,this);break;case br:bG=new qx.ui.tooltip.ToolTip(this.tr(A));break;case T:bG=new qx.ui.toolbar.Button();bG.addState(O);bG.setFocusable(false);bG.setToolTip(this.getChildControl(br));bG.addListener(bz,this._onNavButtonTap,this);break;case N:bG=new qx.ui.basic.Label();bG.setAllowGrowX(true);bG.setAnonymous(true);break;case J:bG=new qx.ui.basic.Label();bG.setAllowGrowX(true);bG.setAllowGrowY(true);bG.setSelectable(false);bG.setAnonymous(true);bG.setCursor(k);break;case m:bG=new qx.ui.basic.Label();bG.setAllowGrowX(true);bG.setAllowGrowY(true);bG.setSelectable(false);bG.setAnonymous(true);bG.setCursor(k);break;case b:bG=new qx.ui.basic.Label();bG.setAllowGrowX(true);bG.setAllowGrowY(true);bG.setCursor(k);bG.addListener(bA,this._onDayTap,this);bG.addListener(bv,this._onDayDblTap,this);break;case bB:var bJ=new qx.ui.layout.Grid();bG=new qx.ui.container.Composite(bJ);for(var i=0;i<8;i++ ){bJ.setColumnFlex(i,1);};for(var i=0;i<7;i++ ){bJ.setRowFlex(i,1);};var bI=this.getChildControl(bs);bI.addState(I);bG.add(bI,{column:0,row:0});this.__sY=[];for(var i=0;i<7;i++ ){bI=this.getChildControl(F+i);bG.add(bI,{column:i+1,row:0});this.__sY.push(bI);};this.__ta=[];this.__tb=[];for(var y=0;y<6;y++ ){var bI=this.getChildControl(R+(y+1));bG.add(bI,{column:0,row:y+1});this.__tb.push(bI);for(var x=0;x<7;x++ ){var bI=this.getChildControl(w+((y*7)+x));bG.add(bI,{column:x+1,row:y+1});this.__ta.push(bI);};};this._add(bG);break;};return bG||qx.ui.core.Widget.prototype._createChildControlImpl.call(this,bK);},_applyValue:function(bN,bM){if((bN!=null)&&(this.getShownMonth()!=bN.getMonth()||this.getShownYear()!=bN.getFullYear())){this.showMonth(bN.getMonth(),bN.getFullYear());}else {var bL=(bN==null)?-1:bN.getDate();for(var i=0;i<6*7;i++ ){var bO=this.__ta[i];if(bO.hasState(M)){if(bO.hasState(p)){bO.removeState(p);};}else {var bP=parseInt(bO.getValue(),10);if(bP==bL){bO.addState(p);}else if(bO.hasState(p)){bO.removeState(p);};};};};},_onPointerUpDown:function(e){var bQ=e.getTarget();if(bQ==this.getChildControl(bo)||bQ==this.getChildControl(bB)){e.stopPropagation();return;};},_onNavButtonTap:function(bS){var bR=this.getShownYear();var bT=this.getShownMonth();switch(bS.getCurrentTarget()){case this.getChildControl(P):bR-- ;break;case this.getChildControl(j):bT-- ;if(bT<0){bT=11;bR-- ;};break;case this.getChildControl(D):bT++ ;if(bT>=12){bT=0;bR++ ;};break;case this.getChildControl(T):bR++ ;break;};this.showMonth(bT,bR);},_onDayTap:function(bU){var bV=bU.getCurrentTarget().dateTime;this.setValue(new Date(bV));},_onDayDblTap:function(){this.execute();},_onKeyPress:function(bY){var ca=null;var cb=null;var bX=null;if(bY.getModifiers()==0){switch(bY.getKeyIdentifier()){case h:ca=-1;break;case V:ca=1;break;case t:ca=-7;break;case q:ca=7;break;case a:cb=-1;break;case v:cb=1;break;case g:if(this.getValue()!=null){this.setValue(null);return;};break;case u:case o:if(this.getValue()!=null){this.execute();};return;};}else if(bY.isShiftPressed()){switch(bY.getKeyIdentifier()){case a:bX=-1;break;case v:bX=1;break;};};if(ca!=null||cb!=null||bX!=null){var bW=this.getValue();if(bW!=null){bW=new Date(bW.getTime());};if(bW==null){bW=new Date();}else {if(ca!=null){bW.setDate(bW.getDate()+ca);};if(cb!=null){bW.setMonth(bW.getMonth()+cb);};if(bX!=null){bW.setFullYear(bW.getFullYear()+bX);};};this.setValue(bW);};},showMonth:function(cc,cd){if((cc!=null&&cc!=this.getShownMonth())||(cd!=null&&cd!=this.getShownYear())){if(cc!=null){this.setShownMonth(cc);};if(cd!=null){this.setShownYear(cd);};this._updateDatePane();};},handleKeyPress:function(e){this._onKeyPress(e);},_updateDatePane:function(){var ct=qx.ui.control.DateChooser;var cq=new Date();var ci=cq.getFullYear();var co=cq.getMonth();var cl=cq.getDate();var cu=this.getValue();var cx=(cu==null)?-1:cu.getFullYear();var cE=(cu==null)?-1:cu.getMonth();var cn=(cu==null)?-1:cu.getDate();var cm=this.getShownMonth();var cB=this.getShownYear();var cj=qx.locale.Date.getWeekStart();var cv=new Date(this.getShownYear(),this.getShownMonth(),1);var cs=new qx.util.format.DateFormat(ct.MONTH_YEAR_FORMAT);this.getChildControl(N).setValue(cs.format(cv));var cD=cv.getDay();var cp=1+((7-cD)%7);var cw=new qx.util.format.DateFormat(ct.WEEKDAY_FORMAT);for(var i=0;i<7;i++ ){var cy=(i+cj)%7;var cA=this.__sY[i];cv.setDate(cp+cy);cA.setValue(cw.format(cv));if(qx.locale.Date.isWeekend(cy)){cA.addState(bw);}else {cA.removeState(bw);};};cv=new Date(cB,cm,1,12,0,0);var cf=(7+cD-cj)%7;cv.setDate(cv.getDate()-cf);var cz=new qx.util.format.DateFormat(ct.WEEK_FORMAT);for(var cr=0;cr<6;cr++ ){this.__tb[cr].setValue(cz.format(cv));for(var i=0;i<7;i++ ){var cA=this.__ta[cr*7+i];var ch=cv.getFullYear();var cg=cv.getMonth();var ck=cv.getDate();var ce=(cx==ch&&cE==cg&&cn==ck);if(ce){cA.addState(p);}else {cA.removeState(p);};if(cg!=cm){cA.addState(M);}else {cA.removeState(M);};var cC=(ch==ci&&cg==co&&ck==cl);if(cC){cA.addState(l);}else {cA.removeState(l);};cA.setValue(bu+ck);cA.dateTime=cv.getTime();cv.setDate(cv.getDate()+1);};};cs.dispose();cw.dispose();cz.dispose();}},destruct:function(){if(qx.core.Environment.get(S)){qx.locale.Manager.getInstance().removeListener(z,this._updateDatePane,this);};this.__sY=this.__ta=this.__tb=null;}});})();(function(){var a="inherit",b="toolbar-button",c="keydown",d="qx.ui.toolbar.Button",e="keyup";qx.Class.define(d,{extend:qx.ui.form.Button,construct:function(f,h,g){qx.ui.form.Button.call(this,f,h,g);this.removeListener(c,this._onKeyDown);this.removeListener(e,this._onKeyUp);},properties:{appearance:{refine:true,init:b},show:{refine:true,init:a},focusable:{refine:true,init:false}},members:{_applyVisibility:function(j,i){qx.ui.form.Button.prototype._applyVisibility.call(this,j,i);var parent=this.getLayoutParent();if(parent&&parent instanceof qx.ui.toolbar.PartContainer){qx.ui.core.queue.Appearance.add(parent);};}}});})();(function(){var a="change",b="qx.ui.form.Form",c="qx.event.type.Event",d="";qx.Class.define(b,{extend:qx.core.Object,construct:function(){qx.core.Object.call(this);this.__tc=[];this._buttons=[];this._buttonOptions=[];this._validationManager=this._createValidationManager();this._resetter=this._createResetter();},events:{"change":c},members:{__tc:null,_validationManager:null,_groupCounter:0,_buttons:null,_buttonOptions:null,_resetter:null,add:function(g,f,h,name,e,k){if(this.__td()){this.__tc.push({title:null,items:[],labels:[],names:[],options:[],headerOptions:{}});};this.__tc[this._groupCounter].items.push(g);this.__tc[this._groupCounter].labels.push(f);this.__tc[this._groupCounter].options.push(k);if(name==null){name=f.replace(/\s+|&|-|\+|\*|\/|\||!|\.|,|:|\?|;|~|%|\{|\}|\(|\)|\[|\]|<|>|=|\^|@|\\/g,d);};this.__tc[this._groupCounter].names.push(name);this._validationManager.add(g,h,e);this._resetter.add(g);this.fireEvent(a);},addGroupHeader:function(m,l){if(!this.__td()){this._groupCounter++ ;};this.__tc.push({title:m,items:[],labels:[],names:[],options:[],headerOptions:l});this.fireEvent(a);},addButton:function(n,o){this._buttons.push(n);this._buttonOptions.push(o||null);this.fireEvent(a);},__td:function(){return this.__tc.length===0;},remove:function(p){for(var i=0;i<this.__tc.length;i++ ){var r=this.__tc[i];for(var j=0;j<r.items.length;j++ ){var q=r.items[j];if(q===p){r.items.splice(j,1);r.labels.splice(j,1);r.names.splice(j,1);r.options.splice(j,1);this._validationManager.remove(p);this._resetter.remove(p);this.fireEvent(a);return true;};};};return false;},removeGroupHeader:function(u){for(var i=0;i<this.__tc.length;i++ ){var t=this.__tc[i];if(t.title===u){var s;if(i==0){if(this.__tc.length==1){t.title=null;t.headerOptions={};this.fireEvent(a);return true;}else {s=this.__tc[i+1];};}else {s=this.__tc[i-1];};s.items=s.items.concat(t.items);s.labels=s.labels.concat(t.labels);s.names=s.names.concat(t.names);s.options=s.options.concat(t.options);this.__tc.splice(i,1);this._groupCounter-- ;this.fireEvent(a);return true;};};return false;},removeButton:function(w){for(var i=0;i<this._buttons.length;i++ ){var v=this._buttons[i];if(v===w){this._buttons.splice(i,1);this._buttonOptions.splice(i,1);this.fireEvent(a);return true;};};return false;},getItems:function(){var x={};for(var i=0;i<this.__tc.length;i++ ){var y=this.__tc[i];for(var j=0;j<y.names.length;j++ ){var name=y.names[j];x[name]=y.items[j];};};return x;},reset:function(){this._resetter.reset();this._validationManager.reset();},redefineResetter:function(){this._resetter.redefine();},redefineResetterItem:function(z){this._resetter.redefineItem(z);},validate:function(){return this._validationManager.validate();},getValidationManager:function(){return this._validationManager;},getGroups:function(){return this.__tc;},getButtons:function(){return this._buttons;},getButtonOptions:function(){return this._buttonOptions;},_createValidationManager:function(){return new qx.ui.form.validation.Manager();},_createResetter:function(){return new qx.ui.form.Resetter();}},destruct:function(){this.__tc=this._buttons=this._buttonOptions=null;this._validationManager.dispose();this._resetter.dispose();}});})();(function(){var a="Widgets supporting selection can only be validated ",b="Added widget not supported.",c="",d="in the form validator",f="mouse",g="changeValid",h="qx.event.type.Event",j="complete",k="value instanceof Function || qx.Class.isSubClassOf(value.constructor, qx.ui.form.validation.AsyncValidator)",l="qx.ui.form.validation.Manager",m="This field is required",n="String",o="qx.event.type.Data";qx.Class.define(l,{extend:qx.core.Object,construct:function(){qx.core.Object.call(this);this.__te=[];this.__tf={};this.setRequiredFieldMessage(qx.locale.Manager.tr(m));},events:{"changeValid":o,"complete":h},properties:{validator:{check:k,init:null,nullable:true},invalidMessage:{check:n,init:c},requiredFieldMessage:{check:n,init:c},context:{nullable:true}},members:{__te:null,__tg:null,__tf:null,__th:null,add:function(s,p,r){if(!this.__tm(s)){throw new Error(b);};if(this.__tn(s)&&!s.getValue){if(p!=null){throw new Error(a+d);};};var q={item:s,validator:p,valid:null,context:r};this.__te.push(q);},remove:function(v){var t=this.__te;for(var i=0,u=t.length;i<u;i++ ){if(v===t[i].item){t.splice(i,1);return v;};};return null;},getItems:function(){var w=[];for(var i=0;i<this.__te.length;i++ ){w.push(this.__te[i].item);};return w;},validate:function(){var B=true;this.__th=true;var y=[];for(var i=0;i<this.__te.length;i++ ){var z=this.__te[i].item;var C=this.__te[i].validator;y.push(z);if(C==null){var x=this.__ti(z);B=B&&x;this.__th=x&&this.__th;continue;};var x=this.__tj(this.__te[i],z.getValue());B=x&&B;if(x!=null){this.__th=x&&this.__th;};};var A=this.__tk(y);if(qx.lang.Type.isBoolean(A)){this.__th=A&&this.__th;};B=A&&B;this._setValid(B);if(qx.lang.Object.isEmpty(this.__tf)){this.fireEvent(j);};return B;},__ti:function(E){if(E.getRequired()){if(this.__tn(E)){var D=!!E.getSelection()[0];}else {var G=E.getValue();var D=!!G||G===0;};E.setValid(D);var H=E.getRequiredInvalidMessage();var F=H?H:this.getRequiredFieldMessage();E.setInvalidMessage(F);return D;};return true;},__tj:function(K,L){var M=K.item;var O=K.context;var N=K.validator;if(this.__tl(N)){this.__tf[M.toHashCode()]=null;N.validate(M,M.getValue(),this,O);return null;};var I=null;try{var I=N.call(O||this,L,M);if(I===undefined){I=true;};}catch(e){if(e instanceof qx.core.ValidationError){I=false;if(e.message&&e.message!=qx.type.BaseError.DEFAULTMESSAGE){var J=e.message;}else {var J=e.getComment();};M.setInvalidMessage(J);}else {throw e;};};M.setValid(I);K.valid=I;return I;},__tk:function(Q){var R=this.getValidator();var S=this.getContext()||this;if(R==null){return true;};this.setInvalidMessage(c);if(this.__tl(R)){this.__tf[this.toHashCode()]=null;R.validateForm(Q,this,S);return null;};try{var T=R.call(S,Q,this);if(T===undefined){T=true;};}catch(e){if(e instanceof qx.core.ValidationError){T=false;if(e.message&&e.message!=qx.type.BaseError.DEFAULTMESSAGE){var P=e.message;}else {var P=e.getComment();};this.setInvalidMessage(P);}else {throw e;};};return T;},__tl:function(V){var U=false;if(!qx.lang.Type.isFunction(V)){U=qx.Class.isSubClassOf(V.constructor,qx.ui.form.validation.AsyncValidator);};return U;},__tm:function(W){var X=W.constructor;return qx.Class.hasInterface(X,qx.ui.form.IForm);},__tn:function(Y){var ba=Y.constructor;return qx.Class.hasInterface(ba,qx.ui.core.ISingleSelection);},_setValid:function(bb){this._showToolTip(bb);var bc=this.__tg;this.__tg=bb;if(bc!=bb){this.fireDataEvent(g,bb,bc);};},_showToolTip:function(bg){if(!qx.ui.tooltip||!qx.ui.tooltip.Manager){return;};var be=qx.ui.tooltip.Manager.getInstance().getSharedErrorTooltip();if(!bg){var bd;for(var i=0;i<this.__te.length;i++ ){var bf=this.__te[i].item;if(!bf.isValid()){bd=bf;if(!(bf.getContentLocation)){return;};if(bf.isSeeable()===false){continue;};be.setLabel(bf.getInvalidMessage());if(be.getPlaceMethod()==f){var location=bf.getContentLocation();var top=location.top-be.getOffsetTop();be.placeToPoint({left:location.right,top:top});}else {be.placeToWidget(bf);};be.show();return;};};}else {be.exclude();};},getValid:function(){return this.__tg;},isValid:function(){return this.getValid();},getInvalidMessages:function(){var bh=[];for(var i=0;i<this.__te.length;i++ ){var bi=this.__te[i].item;if(!bi.getValid()){bh.push(bi.getInvalidMessage());};};if(this.getInvalidMessage()!=c){bh.push(this.getInvalidMessage());};return bh;},getInvalidFormItems:function(){var bj=[];for(var i=0;i<this.__te.length;i++ ){var bk=this.__te[i].item;if(!bk.getValid()){bj.push(bk);};};return bj;},reset:function(){for(var i=0;i<this.__te.length;i++ ){var bl=this.__te[i];bl.item.setValid(true);};this.__tg=null;this._showToolTip(true);},setItemValid:function(bn,bm){this.__tf[bn.toHashCode()]=bm;bn.setValid(bm);this.__to();},setFormValid:function(bo){this.__tf[this.toHashCode()]=bo;this.__to();},__to:function(){var bq=this.__th;for(var br in this.__tf){var bp=this.__tf[br];bq=bp&&bq;if(bp==null){return;};};this._setValid(bq);this.__tf={};this.fireEvent(j);}},destruct:function(){this.__te=null;}});})();(function(){var a="qx.ui.form.validation.AsyncValidator";qx.Class.define(a,{extend:qx.core.Object,construct:function(b){qx.core.Object.call(this);this.__tp=b;},members:{__tp:null,__tq:null,__gt:null,__tr:null,validate:function(c,e,d,f){this.__tr=false;this.__tq=c;this.__gt=d;this.__tp.call(f||this,this,e);},validateForm:function(g,h,i){this.__tr=true;this.__gt=h;this.__tp.call(i,g,this);},setValid:function(k,j){if(this.__tr){if(j!==undefined){this.__gt.setInvalidMessage(j);};this.__gt.setFormValid(k);}else {if(j!==undefined){this.__tq.setInvalidMessage(j);};this.__gt.setItemValid(this.__tq,k);};}},destruct:function(){this.__gt=this.__tq=null;}});})();(function(){var a="The given item has not been added.",b="qx.ui.form.Resetter",c=" not supported for reseting.",d="Item ";qx.Class.define(b,{extend:qx.core.Object,construct:function(){qx.core.Object.call(this);this.__su=[];},members:{__su:null,add:function(e){if(this._supportsValue(e)){var f=e.getValue();}else if(this.__tn(e)){var f=e.getSelection();}else if(this.__tu(e)){var f=e.getSelection().concat();}else {throw new Error(d+e+c);};this.__su.push({item:e,init:f});},remove:function(g){for(var i=0;i<this.__su.length;i++ ){var h=this.__su[i];if(h.item===g){this.__su.splice(i,1);return true;};};return false;},reset:function(){for(var i=0;i<this.__su.length;i++ ){var j=this.__su[i];this.__ts(j.item,j.init);};},resetItem:function(k){var l;for(var i=0;i<this.__su.length;i++ ){var m=this.__su[i];if(m.item===k){l=m.init;break;};};if(l===undefined){throw new Error(a);};this.__ts(k,l);},__ts:function(n,o){if(this._supportsValue(n)){n.setValue(o);}else if(this.__tn(n)||this.__tu(n)){n.setSelection(o);};},redefine:function(){for(var i=0;i<this.__su.length;i++ ){var p=this.__su[i].item;this.__su[i].init=this.__tt(p);};},redefineItem:function(q){var r;for(var i=0;i<this.__su.length;i++ ){if(this.__su[i].item===q){r=this.__su[i];break;};};if(r===undefined){throw new Error(a);};r.init=this.__tt(r.item);},__tt:function(s){if(this._supportsValue(s)){return s.getValue();}else if(this.__tn(s)||this.__tu(s)){return s.getSelection();};},__tn:function(t){var u=t.constructor;return qx.Class.hasInterface(u,qx.ui.core.ISingleSelection);},__tu:function(v){var w=v.constructor;return qx.Class.hasInterface(w,qx.data.controller.ISelection);},_supportsValue:function(x){var y=x.constructor;return (qx.Class.hasInterface(y,qx.ui.form.IBooleanForm)||qx.Class.hasInterface(y,qx.ui.form.IColorForm)||qx.Class.hasInterface(y,qx.ui.form.IDateForm)||qx.Class.hasInterface(y,qx.ui.form.INumberForm)||qx.Class.hasInterface(y,qx.ui.form.IStringForm));}},destruct:function(){this.__su=null;}});})();(function(){var a="qx.ui.form.IColorForm",b="qx.event.type.Data";qx.Interface.define(a,{events:{"changeValue":b},members:{setValue:function(c){return arguments.length==1;},resetValue:function(){},getValue:function(){}}});})();(function(){var a="password",b="qx.ui.form.PasswordField";qx.Class.define(b,{extend:qx.ui.form.TextField,members:{_createInputElement:function(){return new qx.html.Input(a);}}});})();(function(){var a="keypress",b="Boolean",c="Right",d="label",f="Left",g="_applyValue",h="changeValue",i="Up",j="value",k="qx.ui.form.RadioButton",l="radiobutton",m="toolTipText",n="enabled",o="qx.ui.form.RadioGroup",p="Down",q="_applyGroup",r="checked",s="menu",t="execute";qx.Class.define(k,{extend:qx.ui.form.Button,include:[qx.ui.form.MForm,qx.ui.form.MModelProperty],implement:[qx.ui.form.IRadioItem,qx.ui.form.IForm,qx.ui.form.IBooleanForm,qx.ui.form.IModel],construct:function(u){{};qx.ui.form.Button.call(this,u);this.addListener(t,this._onExecute);this.addListener(a,this._onKeyPress);},properties:{group:{check:o,nullable:true,apply:q},value:{check:b,nullable:true,event:h,apply:g,init:false},appearance:{refine:true,init:l},allowGrowX:{refine:true,init:false}},members:{_forwardStates:{checked:true,focused:true,invalid:true,hovered:true},_bindableProperties:[n,d,m,j,s],_applyValue:function(w,v){w?this.addState(r):this.removeState(r);},_applyGroup:function(y,x){if(x){x.remove(this);};if(y){y.add(this);};},_onExecute:function(e){var z=this.getGroup();if(z&&z.getAllowEmptySelection()){this.toggleValue();}else {this.setValue(true);};},_onKeyPress:function(e){var A=this.getGroup();if(!A){return;};switch(e.getKeyIdentifier()){case f:case i:A.selectPrevious();break;case c:case p:A.selectNext();break;};}}});})();(function(){var a="Boolean",b="_applyValid",c="",d="changeRequired",f="changeValid",g="__tv",h="changeInvalidMessage",j="changeSelection",k="_applyInvalidMessage",l="String",m="qx.ui.form.RadioButtonGroup",n="qx.event.type.Data";qx.Class.define(m,{extend:qx.ui.core.Widget,include:[qx.ui.core.MLayoutHandling,qx.ui.form.MModelSelection],implement:[qx.ui.form.IForm,qx.ui.core.ISingleSelection,qx.ui.form.IModelSelection],construct:function(o){qx.ui.core.Widget.call(this);if(o==null){this.setLayout(new qx.ui.layout.VBox(4));}else {this.setLayout(o);};this.__tv=new qx.ui.form.RadioGroup();this.__tv.addListener(j,function(e){this.fireDataEvent(j,e.getData(),e.getOldData());},this);},properties:{valid:{check:a,init:true,apply:b,event:f},required:{check:a,init:false,event:d},invalidMessage:{check:l,init:c,event:h,apply:k},requiredInvalidMessage:{check:l,nullable:true,event:h}},events:{"changeSelection":n},members:{__tv:null,_applyInvalidMessage:function(r,p){var q=this._getChildren();for(var i=0;i<q.length;i++ ){q[i].setInvalidMessage(r);};},_applyValid:function(u,s){var t=this._getChildren();for(var i=0;i<t.length;i++ ){t[i].setValid(u);};},getRadioGroup:function(){return this.__tv;},getChildren:function(){return this._getChildren();},add:function(w,v){this.__tv.add(w);this._add(w,v);},remove:function(x){this.__tv.remove(x);this._remove(x);},removeAll:function(){var y=this.__tv.getItems();for(var i=y.length-1;i>=0;i-- ){this.__tv.remove(y[i]);};return this._removeAll();},getSelection:function(){return this.__tv.getSelection();},setSelection:function(z){this.__tv.setSelection(z);},resetSelection:function(){this.__tv.resetSelection();},isSelected:function(A){return this.__tv.isSelected(A);},isSelectionEmpty:function(){return this.__tv.isSelectionEmpty();},getSelectables:function(B){return this.__tv.getSelectables(B);}},destruct:function(){this._disposeObjects(g);}});})();(function(){var a="hovered",b="Enter",c="pressed",d="one",f="pointerover",g="changeLabel",h="changeIcon",i="Space",j="abandoned",k="key",l="list",m="keyinput",n="arrow",o="changeSelection",p="quick",q="",r="qx.ui.form.SelectBox",s="spacer",t="selectbox",u="popup",v=" ",w="tap",x="pointerout",y="atom";qx.Class.define(r,{extend:qx.ui.form.AbstractSelectBox,implement:[qx.ui.core.ISingleSelection,qx.ui.form.IModelSelection],include:[qx.ui.core.MSingleSelectionHandling,qx.ui.form.MModelSelection],construct:function(){qx.ui.form.AbstractSelectBox.call(this);this._createChildControl(y);this._createChildControl(s);this._createChildControl(n);this.addListener(f,this._onPointerOver,this);this.addListener(x,this._onPointerOut,this);this.addListener(w,this._onTap,this);this.addListener(m,this._onKeyInput,this);this.addListener(o,this.__sp,this);},properties:{appearance:{refine:true,init:t}},members:{__sT:null,_createChildControlImpl:function(B,A){var z;switch(B){case s:z=new qx.ui.core.Spacer();this._add(z,{flex:1});break;case y:z=new qx.ui.basic.Atom(v);z.setCenter(false);z.setAnonymous(true);this._add(z,{flex:1});break;case n:z=new qx.ui.basic.Image();z.setAnonymous(true);this._add(z);break;};return z||qx.ui.form.AbstractSelectBox.prototype._createChildControlImpl.call(this,B);},_forwardStates:{focused:true},_getItems:function(){return this.getChildrenContainer().getChildren();},_isAllowEmptySelection:function(){return this.getChildrenContainer().getSelectionMode()!==d;},__sp:function(e){var D=e.getData()[0];var C=this.getChildControl(l);if(C.getSelection()[0]!=D){if(D){C.setSelection([D]);}else {C.resetSelection();};};this.__tw();this.__tx();},__tw:function(){var F=this.getChildControl(l).getSelection()[0];var E=this.getChildControl(y);var G=F?F.getIcon():q;G==null?E.resetIcon():E.setIcon(G);},__tx:function(){var J=this.getChildControl(l).getSelection()[0];var H=this.getChildControl(y);var I=J?J.getLabel():q;var K=this.getFormat();if(K!=null){I=K.call(this,J);};if(I&&I.translate){I=I.translate();};I==null?H.resetLabel():H.setLabel(I);},_onPointerOver:function(e){if(!this.isEnabled()||e.getTarget()!==this){return;};if(this.hasState(j)){this.removeState(j);this.addState(c);};this.addState(a);},_onPointerOut:function(e){if(!this.isEnabled()||e.getTarget()!==this){return;};this.removeState(a);if(this.hasState(c)){this.removeState(c);this.addState(j);};},_onTap:function(e){this.toggle();},_onKeyPress:function(e){var L=e.getKeyIdentifier();if(L==b||L==i){if(this.__sT){this.setSelection([this.__sT]);this.__sT=null;};this.toggle();}else {qx.ui.form.AbstractSelectBox.prototype._onKeyPress.call(this,e);};},_onKeyInput:function(e){var M=e.clone();M.setTarget(this._list);M.setBubbles(false);this.getChildControl(l).dispatchEvent(M);},_onListPointerDown:function(e){if(this.__sT){this.setSelection([this.__sT]);this.__sT=null;};},_onListChangeSelection:function(e){var Q=e.getData();var O=e.getOldData();if(O&&O.length>0){O[0].removeListener(h,this.__tw,this);O[0].removeListener(g,this.__tx,this);};if(Q.length>0){var N=this.getChildControl(u);var P=this.getChildControl(l);var R=P.getSelectionContext();if(N.isVisible()&&(R==p||R==k)){this.__sT=Q[0];}else {this.setSelection([Q[0]]);this.__sT=null;};Q[0].addListener(h,this.__tw,this);Q[0].addListener(g,this.__tx,this);}else {this.resetSelection();};},_onPopupChangeVisibility:function(e){qx.ui.form.AbstractSelectBox.prototype._onPopupChangeVisibility.call(this,e);var T=this.getChildControl(u);if(!T.isVisible()){var V=this.getChildControl(l);if(V.hasChildren()){V.setSelection(this.getSelection());};}else {var S=T.getLayoutLocation(this);var X=qx.bom.Viewport.getHeight();var W=S.top;var Y=X-S.bottom;var U=W>Y?W:Y;var ba=this.getMaxListHeight();var V=this.getChildControl(l);if(ba==null||ba>U){V.setMaxHeight(U);}else if(ba<U){V.setMaxHeight(ba);};};}},destruct:function(){this.__sT=null;}});})();(function(){var a="PageUp",b="Boolean",c="_applyEditable",d="_applyWrap",f="changeValue",g="keydown",h="Down",i="\-]",j="roll",k="execute",l="inner",m="PageDown",n="changeLocale",o="_applyValue",p="Up",q="downbutton",r="",s="number",t="textfield",u="_applyMinimum",v="qx.util.format.NumberFormat",w="qx.dynlocale",x="[0-9",y="upbutton",z="keyup",A="spinner",B="this._checkValue(value)",C="Number",D="wheel",E="_applyMaximum",F="changeNumberFormat",G="changeMaximum",H="changeMinimum",I="_applyNumberFormat",J="qx.ui.form.Spinner";qx.Class.define(J,{extend:qx.ui.core.Widget,implement:[qx.ui.form.INumberForm,qx.ui.form.IRange,qx.ui.form.IForm],include:[qx.ui.core.MContentPadding,qx.ui.form.MForm],construct:function(N,M,K){qx.ui.core.Widget.call(this);var L=new qx.ui.layout.Grid();L.setColumnFlex(0,1);L.setRowFlex(0,1);L.setRowFlex(1,1);this._setLayout(L);this.addListener(g,this._onKeyDown,this);this.addListener(z,this._onKeyUp,this);this.addListener(j,this._onRoll,this);if(qx.core.Environment.get(w)){qx.locale.Manager.getInstance().addListener(n,this._onChangeLocale,this);};this._createChildControl(t);this._createChildControl(y);this._createChildControl(q);if(N!=null){this.setMinimum(N);};if(K!=null){this.setMaximum(K);};if(M!==undefined){this.setValue(M);}else {this.initValue();};},properties:{appearance:{refine:true,init:A},focusable:{refine:true,init:true},singleStep:{check:C,init:1},pageStep:{check:C,init:10},minimum:{check:C,apply:u,init:0,event:H},value:{check:B,nullable:true,apply:o,init:0,event:f},maximum:{check:C,apply:E,init:100,event:G},wrap:{check:b,init:false,apply:d},editable:{check:b,init:true,apply:c},numberFormat:{check:v,apply:I,nullable:true},allowShrinkY:{refine:true,init:false}},members:{__ty:null,__tz:false,__tA:false,_createChildControlImpl:function(Q,P){var O;switch(Q){case t:O=new qx.ui.form.TextField();O.setFilter(this._getFilterRegExp());O.addState(l);O.setWidth(40);O.setFocusable(false);O.addListener(f,this._onTextChange,this);this._add(O,{column:0,row:0,rowSpan:2});break;case y:O=new qx.ui.form.RepeatButton();O.addState(l);O.setFocusable(false);O.addListener(k,this._countUp,this);this._add(O,{column:1,row:0});break;case q:O=new qx.ui.form.RepeatButton();O.addState(l);O.setFocusable(false);O.addListener(k,this._countDown,this);this._add(O,{column:1,row:1});break;};return O||qx.ui.core.Widget.prototype._createChildControlImpl.call(this,Q);},_getFilterRegExp:function(){var S,W,U;if(this.getNumberFormat()!==null){U=this.getNumberFormat().getLocale();}else {U=qx.locale.Manager.getInstance().getLocale();};S=qx.locale.Number.getDecimalSeparator(U);W=qx.locale.Number.getGroupSeparator(U);var V=r;var T=r;if(this.getNumberFormat()!==null){V=this.getNumberFormat().getPrefix()||r;T=this.getNumberFormat().getPostfix()||r;};var R=new RegExp(x+qx.lang.String.escapeRegexpChars(S)+qx.lang.String.escapeRegexpChars(W)+qx.lang.String.escapeRegexpChars(V)+qx.lang.String.escapeRegexpChars(T)+i);return R;},_forwardStates:{focused:true,invalid:true},tabFocus:function(){var X=this.getChildControl(t);X.getFocusElement().focus();X.selectAllText();},_applyMinimum:function(ba,Y){if(this.getMaximum()<ba){this.setMaximum(ba);};if(this.getValue()<ba){this.setValue(ba);}else {this._updateButtons();};},_applyMaximum:function(bc,bb){if(this.getMinimum()>bc){this.setMinimum(bc);};if(this.getValue()>bc){this.setValue(bc);}else {this._updateButtons();};},_applyEnabled:function(be,bd){qx.ui.core.Widget.prototype._applyEnabled.call(this,be,bd);this._updateButtons();},_checkValue:function(bf){return typeof bf===s&&bf>=this.getMinimum()&&bf<=this.getMaximum();},_applyValue:function(bi,bh){var bg=this.getChildControl(t);this._updateButtons();this.__ty=bi;if(bi!==null){if(this.getNumberFormat()){bg.setValue(this.getNumberFormat().format(bi));}else {bg.setValue(bi+r);};}else {bg.setValue(r);};},_applyEditable:function(bl,bk){var bj=this.getChildControl(t);if(bj){bj.setReadOnly(!bl);};},_applyWrap:function(bn,bm){this._updateButtons();},_applyNumberFormat:function(bq,bp){var bo=this.getChildControl(t);bo.setFilter(this._getFilterRegExp());var br=this.getNumberFormat();if(br!==null){br.addListener(F,this._onChangeNumberFormat,this);};this._applyValue(this.__ty,undefined);},_getContentPaddingTarget:function(){return this.getChildControl(t);},_updateButtons:function(){var bt=this.getChildControl(y);var bs=this.getChildControl(q);var bu=this.getValue();if(!this.getEnabled()){bt.setEnabled(false);bs.setEnabled(false);}else {if(this.getWrap()){bt.setEnabled(true);bs.setEnabled(true);}else {if(bu!==null&&bu<this.getMaximum()){bt.setEnabled(true);}else {bt.setEnabled(false);};if(bu!==null&&bu>this.getMinimum()){bs.setEnabled(true);}else {bs.setEnabled(false);};};};},_onKeyDown:function(e){switch(e.getKeyIdentifier()){case a:this.__tz=true;case p:this.getChildControl(y).press();break;case m:this.__tA=true;case h:this.getChildControl(q).press();break;default:return;};e.stopPropagation();e.preventDefault();},_onKeyUp:function(e){switch(e.getKeyIdentifier()){case a:this.getChildControl(y).release();this.__tz=false;break;case p:this.getChildControl(y).release();break;case m:this.getChildControl(q).release();this.__tA=false;break;case h:this.getChildControl(q).release();break;};},_onRoll:function(e){if(e.getPointerType()!=D){return;};var bv=e.getDelta().y;if(bv<0){this._countUp();}else if(bv>0){this._countDown();};e.stop();},_onTextChange:function(e){var bw=this.getChildControl(t);var bx;if(this.getNumberFormat()){try{bx=this.getNumberFormat().parse(bw.getValue());}catch(by){};};if(bx===undefined){bx=parseFloat(bw.getValue());};if(!isNaN(bx)){if(bx>this.getMaximum()){bx=this.getMaximum();}else if(bx<this.getMinimum()){bx=this.getMinimum();};if(bx===this.__ty){this._applyValue(this.__ty);}else {this.setValue(bx);};}else {this._applyValue(this.__ty,undefined);};},_onChangeLocale:function(bA){if(this.getNumberFormat()!==null){this.setNumberFormat(this.getNumberFormat());var bz=this.getChildControl(t);bz.setFilter(this._getFilterRegExp());bz.setValue(this.getNumberFormat().format(this.getValue()));};},_onChangeNumberFormat:function(bC){var bB=this.getChildControl(t);bB.setFilter(this._getFilterRegExp());bB.setValue(this.getNumberFormat().format(this.getValue()));},_countUp:function(){if(this.__tz){var bE=this.getValue()+this.getPageStep();}else {var bE=this.getValue()+this.getSingleStep();};if(this.getWrap()){if(bE>this.getMaximum()){var bD=this.getMaximum()-bE;bE=this.getMinimum()-bD-1;};};this.gotoValue(bE);},_countDown:function(){if(this.__tA){var bG=this.getValue()-this.getPageStep();}else {var bG=this.getValue()-this.getSingleStep();};if(this.getWrap()){if(bG<this.getMinimum()){var bF=this.getMinimum()+bG;bG=this.getMaximum()+bF+1;};};this.gotoValue(bG);},gotoValue:function(bH){return this.setValue(Math.min(this.getMaximum(),Math.max(this.getMinimum(),bH)));}},destruct:function(){if(qx.core.Environment.get(w)){qx.locale.Manager.getInstance().removeListener(n,this._onChangeLocale,this);};}});})();(function(){var a="splitbutton",b="hovered",c="Enter",d="pressed",f="changeShow",g="pointerover",h="Space",i="keydown",j="abandoned",k="both",l="button",m="execute",n="_applyMenu",o="arrow",p="String",q="icon",r="changeVisibility",s="keyup",t="qx.ui.menu.Menu",u="_applyIcon",v="label",w="_applyShow",x="changeMenu",y="_applyLabel",z="pointerout",A="qx.ui.form.SplitButton";qx.Class.define(A,{extend:qx.ui.core.Widget,include:[qx.ui.core.MExecutable],implement:[qx.ui.form.IExecutable],construct:function(E,C,B,D){qx.ui.core.Widget.call(this);this._setLayout(new qx.ui.layout.HBox);this._createChildControl(o);this.addListener(g,this._onPointerOver,this,true);this.addListener(z,this._onPointerOut,this,true);this.addListener(i,this._onKeyDown);this.addListener(s,this._onKeyUp);if(E!=null){this.setLabel(E);};if(C!=null){this.setIcon(C);};if(B!=null){this.setMenu(B);};if(D!=null){this.setCommand(D);};},properties:{appearance:{refine:true,init:a},focusable:{refine:true,init:true},label:{apply:y,nullable:true,check:p},icon:{check:p,apply:u,nullable:true,themeable:true},show:{init:k,check:[k,v,q],themeable:true,inheritable:true,apply:w,event:f},menu:{check:t,nullable:true,apply:n,event:x}},members:{__tB:null,_createChildControlImpl:function(H,G){var F;switch(H){case l:F=new qx.ui.form.Button;F.addListener(m,this._onButtonExecute,this);F.setFocusable(false);this._addAt(F,0,{flex:1});break;case o:F=new qx.ui.form.MenuButton();F.setFocusable(false);F.setShow(k);this._addAt(F,1);break;};return F||qx.ui.core.Widget.prototype._createChildControlImpl.call(this,H);},_forwardStates:{hovered:1,focused:1},_applyLabel:function(K,J){var I=this.getChildControl(l);K==null?I.resetLabel():I.setLabel(K);},_applyIcon:function(N,M){var L=this.getChildControl(l);N==null?L.resetIcon():L.setIcon(N);},_applyMenu:function(P,O){var Q=this.getChildControl(o);if(P){Q.resetEnabled();Q.setMenu(P);P.setOpener(this);P.addListener(r,this._onChangeMenuVisibility,this);}else {Q.setEnabled(false);Q.resetMenu();};if(O){O.removeListener(r,this._onChangeMenuVisibility,this);O.resetOpener();};},_applyShow:function(S,R){},_onPointerOver:function(e){e.stopPropagation();this.addState(b);delete this.__tB;},_onPointerOut:function(e){e.stopPropagation();if(!this.hasState(b)){return;};var U=e.getRelatedTarget();if(qx.ui.core.Widget.contains(this,U)){return;};var T=this.getMenu();if(T&&T.isVisible()){this.__tB=true;return;};this.removeState(b);},_onKeyDown:function(e){var V=this.getChildControl(l);switch(e.getKeyIdentifier()){case c:case h:V.removeState(j);V.addState(d);};},_onKeyUp:function(e){var W=this.getChildControl(l);switch(e.getKeyIdentifier()){case c:case h:if(W.hasState(d)){W.removeState(j);W.removeState(d);W.execute();};};},_onButtonExecute:function(e){this.execute();},_onChangeMenuVisibility:function(e){if(!this.getMenu().isVisible()&&this.__tB){this.removeState(b);};}}});})();(function(){var a="input",b="Boolean",c="px",d="nowrap",f="_applyWrap",g="-1",h="_applyMinimalLineHeight",i="Integer",j="textarea",k="auto",l="0",m="-9999px",n="roll",o="whiteSpace",p="visible",q="browser.documentmode",r="",s="width",t="_applyAutoSize",u="appear",v="tabIndex",w='name',x='autoSize is ignored when the height property is set. If you want to set an initial height, use the minHeight property instead.',y="normal",z="mshtml",A="engine.name",B="qx.ui.form.TextArea",C="webkit",D="wheel",E='id',F="hidden",G="overflowY",H="absolute";qx.Class.define(B,{extend:qx.ui.form.AbstractField,construct:function(I){qx.ui.form.AbstractField.call(this,I);this.initWrap();this.addListener(n,this._onRoll,this);},properties:{wrap:{check:b,init:true,apply:f},appearance:{refine:true,init:j},singleStep:{check:i,init:20},minimalLineHeight:{check:i,apply:h,init:4},autoSize:{check:b,apply:t,init:false}},members:{__tC:null,__tD:null,__tE:null,setValue:function(J){J=qx.ui.form.AbstractField.prototype.setValue.call(this,J);this.__tF();return J;},_onRoll:function(e){if(e.getPointerType()!=D){return;};var K=this.getContentElement();var scrollY=K.getScrollY();K.scrollToY(scrollY+(e.getDelta().y/30)*this.getSingleStep());var L=K.getScrollY();if(L!=scrollY){e.stop();};},__tF:function(){if(this.isAutoSize()){var O=this.__tG();if(O&&this.getBounds()){this.__tE=this.__tE||this._getAreaHeight();var Q=this._getScrolledAreaHeight();if(this.getMaxHeight()){var M=this.getInsets();var N=-M.top+this.getMaxHeight()-M.bottom;if(Q>N){this.getContentElement().setStyle(G,k);}else {this.getContentElement().setStyle(G,F);};};var P=Math.max(Q,this.__tE);this._setAreaHeight(P);}else {this.getContentElement().addListenerOnce(u,function(){this.__tF();},this);};};},_getAreaHeight:function(){return this.getInnerSize().height;},_setAreaHeight:function(R){if(this._getAreaHeight()!==R){this.__tD=R;qx.ui.core.queue.Layout.add(this);qx.ui.core.queue.Manager.flush();this.__tJ();};},_getScrolledAreaHeight:function(){var T=this.__tG();var S=T.getDomElement();if(S){if(!S.parentNode){qx.html.Element.flush();return this._getScrolledAreaHeight();};if(qx.core.Environment.get(A)===C||(qx.core.Environment.get(A)==z)){T.setWrap(!this.getWrap(),true);};T.setWrap(this.getWrap(),true);if(qx.core.Environment.get(A)===C||(qx.core.Environment.get(A)==z)){S.style.overflow=F;};if(qx.core.Environment.get(A)==z&&qx.core.Environment.get(q)>=8){S.style.overflow=p;S.style.overflowX=F;};T.setValue(this.getValue()||r);if(qx.core.Environment.get(A)==z){S.style.height=k;qx.html.Element.flush();S.style.height=l;};this.__tI(T);if(qx.core.Environment.get(A)==z&&qx.core.Environment.get(q)==8){if(!S.scrollTop){qx.html.Element.flush();};};return S.scrollTop;};},__tG:function(){this.__tC=this.__tC||this.__tH();return this.__tC;},__tH:function(){var W,V,X,U;W=this.getContentElement();if(!W.getDomElement()){return null;};X=qx.bom.Element.clone(W.getDomElement());U=new qx.html.Input(j);U.useElement(X);V=U;V.setStyles({position:H,top:0,left:m,height:0,overflow:F},true);V.removeAttribute(E);V.removeAttribute(w);V.setAttribute(v,g);V.setValue(W.getValue()||r);V.insertBefore(W);this.__tI(V);return V;},__tI:function(Y){Y=Y.getDomElement();if(Y){Y.scrollTop=10000;};},_createInputElement:function(){return new qx.html.Input(j,{overflowX:k,overflowY:k});},_applyWrap:function(bc,ba){this.getContentElement().setWrap(bc);if(this._placeholder){var bb=bc?y:d;this._placeholder.setStyle(o,bb);};this.__tF();},_applyMinimalLineHeight:function(){qx.ui.core.queue.Layout.add(this);},_applyAutoSize:function(be,bd){{};if(be){this.__tF();this.addListener(a,this.__tF,this);if(!this.getBounds()){this.addListenerOnce(u,function(){this.getContentElement().setStyle(G,F);});}else {this.getContentElement().setStyle(G,F);};}else {this.removeListener(a,this.__tF);this.getContentElement().setStyle(G,k);};},_applyDimension:function(bf){qx.ui.form.AbstractField.prototype._applyDimension.call(this);{};if(bf===this.getMaxHeight()){this.__tF();};},__tJ:function(){var content=this.getContentElement();var bh=content.getDomElement();var bg=content.getStyle(s);content.setStyle(s,parseInt(bg,10)+1000+c,true);if(bh){qx.bom.element.Dimension.getWidth(bh);};content.setStyle(s,bg,true);},__tK:function(){if(this.isAutoSize()&&this.getHeight()){this.warn(x);};},_getContentHint:function(){var bi=qx.ui.form.AbstractField.prototype._getContentHint.call(this);bi.height=bi.height*this.getMinimalLineHeight();bi.width=this._getTextSize().width*20;if(this.isAutoSize()){bi.height=this.__tD||bi.height;};return bi;}},destruct:function(){this.setAutoSize(false);if(this.__tC){this.__tC.dispose();};}});})();(function(){var a="changeModel",b="changeLabelOptions",c="blur",d="_applyDelegate",e="_applyLabelPath",f="popupOpen",g="qx.data.Array",h="qx.ui.form.core.AbstractVirtualBox",i="changeIconOptions",j="_applyLabelOptions",k="Down",l="Integer",m="_applyModel",n="open",o="Up",p="",q="_applyMaxListHeight",r="visible",s="middle",t="_applyIconPath",u="resize",v="list",w="changeDelegate",x="changeVisibility",y="close",z="changeLabelPath",A="dropdown",B="keypress",C="changeIconPath",D="_applyRowHeight",E="abstract",F="Number",G="String",H="Escape",I="_applyIconOptions",J="tap",K=".",L="[0]";qx.Class.define(h,{extend:qx.ui.core.Widget,include:qx.ui.form.MForm,implement:qx.ui.form.IForm,type:E,construct:function(M){qx.ui.core.Widget.call(this);var N=new qx.ui.layout.HBox();this._setLayout(N);N.setAlignY(s);this.addListener(B,this._handleKeyboard,this);this.addListener(J,this._handlePointer,this);this.addListener(c,this._onBlur,this);this.addListener(u,this._onResize,this);this._createChildControl(A);if(M!=null){this.initModel(M);}else {this.__tL=new qx.data.Array();this.initModel(this.__tL);};},properties:{focusable:{refine:true,init:true},width:{refine:true,init:120},model:{check:g,apply:m,event:a,nullable:false,deferredInit:true},delegate:{apply:d,event:w,init:null,nullable:true},labelPath:{check:G,apply:e,event:z,nullable:true},labelOptions:{apply:j,event:b,nullable:true},iconPath:{check:G,event:C,apply:t,nullable:true},iconOptions:{apply:I,event:i,nullable:true},itemHeight:{check:l,init:25,apply:D,themeable:true},maxListHeight:{check:F,apply:q,nullable:true,init:200}},members:{__tL:null,_forwardStates:{focused:true,invalid:true},refresh:function(){this.getChildControl(A).getChildControl(v).refresh();qx.ui.core.queue.Widget.add(this);},open:function(){this._beforeOpen();this.getChildControl(A).open();},close:function(){this._beforeClose();this.getChildControl(A).close();},toggle:function(){var O=this.getChildControl(A);if(O.isVisible()){this.close();}else {this.open();};},_createChildControlImpl:function(R,Q){var P;switch(R){case A:P=new qx.ui.form.core.VirtualDropDownList(this);P.addListener(x,this._onPopupChangeVisibility,this);break;};return P||qx.ui.core.Widget.prototype._createChildControlImpl.call(this,R,Q);},_beforeOpen:function(){},_beforeClose:function(){},_getAction:function(event){var S=event.getKeyIdentifier();var U=this.getChildControl(A).isVisible();var T=this._isModifierPressed(event);if(!U&&!T&&(S===k||S===o)){return n;}else if(U&&!T&&S===H){return y;}else {return null;};},_getBindPath:function(W,X){var V=W+L;if(X!=null&&X!=p){V+=K+X;};return V;},_isModifierPressed:function(event){var bc=event.isAltPressed();var bb=event.isCtrlOrCommandPressed();var Y=event.isShiftPressed();var ba=event.isMetaPressed();return (bc||bb||Y||ba);},_onBlur:function(event){this.close();},_handleKeyboard:function(event){var bd=this._getAction(event);var be=this.getChildControl(A).isVisible();switch(bd){case n:this.open();break;case y:this.close();break;default:if(be){this.getChildControl(A)._handleKeyboard(event);};break;};},_handlePointer:function(event){},_onResize:function(event){this.getChildControl(A).setMinWidth(event.getData().width);},_onPopupChangeVisibility:function(event){event.getData()==r?this.addState(f):this.removeState(f);},_applyModel:function(bg,bf){this.getChildControl(A).getChildControl(v).setModel(bg);qx.ui.core.queue.Widget.add(this);},_applyDelegate:function(bi,bh){this.getChildControl(A).getChildControl(v).setDelegate(bi);},_applyLabelPath:function(bk,bj){this.getChildControl(A).getChildControl(v).setLabelPath(bk);qx.ui.core.queue.Widget.add(this);},_applyLabelOptions:function(bm,bl){this.getChildControl(A).getChildControl(v).setLabelOptions(bm);qx.ui.core.queue.Widget.add(this);},_applyIconPath:function(bo,bn){this.getChildControl(A).getChildControl(v).setIconPath(bo);qx.ui.core.queue.Widget.add(this);},_applyIconOptions:function(bq,bp){this.getChildControl(A).getChildControl(v).setIconOptions(bq);qx.ui.core.queue.Widget.add(this);},_applyRowHeight:function(bs,br){this.getChildControl(A).getChildControl(v).setItemHeight(bs);},_applyMaxListHeight:function(bu,bt){this.getChildControl(A).getChildControl(v).setMaxHeight(bu);}},destruct:function(){if(this.__tL){this.__tL.dispose();};}});})();(function(){var a="changeModel",b="one",c="Enter",d="list",e="changeDelegate",f="change",g="qx.data.Array",h="changeVisibility",j="qx.ui.form.core.VirtualDropDownList",k="_applySelection",l="changeSelection",m="tap",n="Invalid parameter 'target'!",o="qx.event.type.Data";qx.Class.define(j,{extend:qx.ui.popup.Popup,construct:function(p){qx.core.Assert.assertNotNull(p,n);qx.core.Assert.assertNotUndefined(p,n);qx.core.Assert.assertInterface(p,qx.ui.form.core.AbstractVirtualBox,n);qx.ui.popup.Popup.call(this,new qx.ui.layout.VBox());this._target=p;this._createChildControl(d);this.addListener(h,this.__tP,this);this.__tM=new qx.data.Array();this.initSelection(this.__tM);},properties:{autoHide:{refine:true,init:false},keepActive:{refine:true,init:true},selection:{check:g,event:l,apply:k,nullable:false,deferredInit:true}},events:{"changeSelection":o},members:{_target:null,_preselected:null,__tN:false,__tO:false,__tM:null,open:function(){this.placeToWidget(this._target,true);this.show();},close:function(){this.hide();},setPreselected:function(q){this._preselected=q;this.__tO=true;var s=this.getChildControl(d).getSelection();var r=new qx.data.Array([q]);this.__tR(r,s);r.dispose();this.__tO=false;},_createChildControlImpl:function(v,u){var t;switch(v){case d:t=new qx.ui.list.List().set({focusable:false,keepFocus:true,height:null,width:null,maxHeight:this._target.getMaxListHeight(),selectionMode:b,quickSelection:true});t.getSelection().addListener(f,this._onListChangeSelection,this);t.addListener(m,this._handlePointer,this);t.addListener(a,this._onChangeModel,this);t.addListener(e,this._onChangeDelegate,this);this.add(t,{flex:1});break;};return t||qx.ui.popup.Popup.prototype._createChildControlImpl.call(this,v,u);},_handleKeyboard:function(event){if(this.isVisible()&&event.getKeyIdentifier()===c){this.__tQ();return;};var w=event.clone();w.setTarget(this.getChildControl(d));w.setBubbles(false);this.getChildControl(d).dispatchEvent(w);},_handlePointer:function(event){this.__tQ();},__sp:function(event){if(this.__tN){return;};var x=this.getSelection();var y=this.getChildControl(d).getSelection();this.__tO=true;this.__tR(x,y);this.__tO=false;this.__tN=true;this.__tR(y,x);this.__tN=false;},_onListChangeSelection:function(event){if(this.__tO){return;};var z=this.getChildControl(d).getSelection();if(this.isVisible()){this.setPreselected(z.getItem(0));}else {this.__tN=true;this.__tR(z,this.getSelection());this.__tN=false;};},__tP:function(event){if(this.isVisible()){if(this._preselected==null){var A=this.getSelection();var B=this.getChildControl(d).getSelection();this.__tR(A,B);};this.__tS();}else {this.setPreselected(null);};},_onChangeModel:function(event){this.getSelection().removeAll();},_onChangeDelegate:function(event){this.getSelection().removeAll();},_applySelection:function(D,C){D.addListener(f,this.__sp,this);if(C!=null){C.removeListener(f,this.__sp,this);};this.__tR(D,this.getChildControl(d).getSelection(D));},__tQ:function(){if(this._preselected!=null){var E=this.getSelection();E.splice(0,1,this._preselected);this._preselected=null;this.close();};},__tR:function(F,H){if(F.equals(H)){return;};if(F.getLength()<=0){H.removeAll();}else {var G=H.toArray();qx.lang.Array.removeAll(G);for(var i=0;i<F.getLength();i++ ){G.push(F.getItem(i));};H.length=G.length;H.fireDataEvent(f,{});};},__tS:function(){this.__tT();this.__tU();},__tT:function(){var I=this._target.getBounds().width;this.setWidth(I);},__tU:function(){var J=this.__tV();var L=this._target.getMaxListHeight();var K=this.getChildControl(d);var M=K.getPane().getRowConfig().getTotalSize();if(L==null||M<L){L=M;};if(L>J){K.setMaxHeight(J);}else if(L<J){K.setMaxHeight(L);};},__tV:function(){var N=this.getLayoutLocation(this._target);var Q=qx.bom.Viewport.getHeight();var P=N.top;var O=Q-N.bottom;return P>O?P:O;}},destruct:function(){if(this.__tM){this.__tM.dispose();};}});})();(function(){var a="scrollY",b="The method 'getItemRight' is not implemented!",c="update",d="scrollX",f="The method 'getItemLeft' is not implemented!",g="The method 'getItemBottom' is not implemented!",h="The method 'getItemTop' is not implemented!",i="pane",j="os.scrollBarOverlayed",k="qx.ui.virtual.core.Scroller";qx.Class.define(k,{extend:qx.ui.core.scroll.AbstractScrollArea,construct:function(o,l,n,m){qx.ui.core.scroll.AbstractScrollArea.call(this);this.__tW=new qx.ui.virtual.core.Pane(o,l,n,m);this.__tW.addListener(c,this._computeScrollbars,this);this.__tW.addListener(d,this._onScrollPaneX,this);this.__tW.addListener(a,this._onScrollPaneY,this);if(qx.core.Environment.get(j)){this._add(this.__tW,{edge:0});}else {this._add(this.__tW,{row:0,column:0});};},properties:{width:{refine:true,init:null},height:{refine:true,init:null}},members:{__tW:null,getPane:function(){return this.__tW;},_createChildControlImpl:function(q,p){if(q==i){return this.__tW;}else {return qx.ui.core.scroll.AbstractScrollArea.prototype._createChildControlImpl.call(this,q);};},getItemTop:function(r){throw new Error(h);},getItemBottom:function(s){throw new Error(g);},getItemLeft:function(t){throw new Error(f);},getItemRight:function(u){throw new Error(b);},_onScrollBarX:function(e){this.__tW.setScrollX(e.getData());},_onScrollBarY:function(e){this.__tW.setScrollY(e.getData());}},destruct:function(){this.__tW.dispose();this.__tW=null;}});})();(function(){var a="qx.ui.virtual.core.Pane",b="resize",c="qx.ui.virtual.core.CellEvent",d="__tX",f="update",g="scrollX",h="scrollY",j="change",k="contextmenu",l="__uf",m="__tY",n="__ug",o="appear",p="cellDbltap",q="pointerdown",r="tap",s="qx.event.type.Event",t="qx.event.type.Data",u="cellTap",v="cellContextmenu",w="dbltap";qx.Class.define(a,{extend:qx.ui.core.Widget,construct:function(A,x,z,y){qx.ui.core.Widget.call(this);this.__tX=new qx.ui.virtual.core.Axis(z,A);this.__tY=new qx.ui.virtual.core.Axis(y,x);this.__ua=0;this.__ub=0;this.__uc=0;this.__ud=0;this.__ue={};this.__jj={};this.__uf=new qx.ui.container.Composite();this.__uf.setUserBounds(0,0,0,0);this._add(this.__uf);this.__ug=[];this.__tX.addListener(j,this.fullUpdate,this);this.__tY.addListener(j,this.fullUpdate,this);this.addListener(b,this._onResize,this);this.addListenerOnce(o,this._onAppear,this);this.addListener(q,this._onPointerDown,this);this.addListener(r,this._onTap,this);this.addListener(w,this._onDbltap,this);this.addListener(k,this._onContextmenu,this);},events:{cellTap:c,cellContextmenu:c,cellDbltap:c,update:s,scrollX:t,scrollY:t},properties:{width:{refine:true,init:400},height:{refine:true,init:300}},members:{__tX:null,__tY:null,__ua:null,__ub:null,__uc:null,__ud:null,__ue:null,__jj:null,__uf:null,__ug:null,__uh:null,__pe:null,__ui:null,__uj:null,getRowConfig:function(){return this.__tX;},getColumnConfig:function(){return this.__tY;},getChildren:function(){return [this.__uf];},addLayer:function(B){{};this.__ug.push(B);B.setUserBounds(0,0,0,0);this.__uf.add(B);},getLayers:function(){return this.__ug;},getVisibleLayers:function(){var C=[];for(var i=0;i<this.__ug.length;i++ ){var D=this.__ug[i];if(D.isVisible()){C.push(D);};};return C;},getScrollMaxX:function(){var E=this.getInnerSize();if(E){return Math.max(0,this.__tY.getTotalSize()-E.width);};return 0;},getScrollMaxY:function(){var F=this.getInnerSize();if(F){return Math.max(0,this.__tX.getTotalSize()-F.height);};return 0;},setScrollY:function(I){var G=this.getScrollMaxY();if(I<0){I=0;}else if(I>G){I=G;};if(this.__ua!==I){var H=this.__ua;this.__ua=I;this._deferredUpdateScrollPosition();this.fireDataEvent(h,I,H);};},getScrollY:function(){return this.__ua;},setScrollX:function(L){var J=this.getScrollMaxX();if(L<0){L=0;}else if(L>J){L=J;};if(L!==this.__ub){var K=this.__ub;this.__ub=L;this._deferredUpdateScrollPosition();this.fireDataEvent(g,L,K);};},getScrollX:function(){return this.__ub;},getScrollSize:function(){return {width:this.__tY.getTotalSize(),height:this.__tX.getTotalSize()};},scrollRowIntoView:function(O){var P=this.getBounds();if(!P){this.addListenerOnce(o,function(){qx.event.Timer.once(function(){this.scrollRowIntoView(O);},this,0);},this);return;};var Q=this.__tX.getItemPosition(O);var N=Q+this.__tX.getItemSize(O);var M=this.getScrollY();if(Q<M){this.setScrollY(Q);}else if(N>M+P.height){this.setScrollY(N-P.height);};},scrollColumnIntoView:function(R){var U=this.getBounds();if(!U){this.addListenerOnce(o,function(){qx.event.Timer.once(function(){this.scrollColumnIntoView(R);},this,0);},this);return;};var T=this.__tY.getItemPosition(R);var S=T+this.__tY.getItemSize(R);var V=this.getScrollX();if(T<V){this.setScrollX(T);}else if(S>V+U.width){this.setScrollX(S-U.width);};},scrollCellIntoView:function(W,Y){var X=this.getBounds();if(!X){this.addListenerOnce(o,function(){qx.event.Timer.once(function(){this.scrollCellIntoView(W,Y);},this,0);},this);return;};this.scrollColumnIntoView(W);this.scrollRowIntoView(Y);},getCellAtPosition:function(ba,bb){var be,bd;var bc=this.getContentLocation();if(!bc||bb<bc.top||bb>=bc.bottom||ba<bc.left||ba>=bc.right){return null;};be=this.__tX.getItemAtPosition(this.getScrollY()+bb-bc.top);bd=this.__tY.getItemAtPosition(this.getScrollX()+ba-bc.left);if(!be||!bd){return null;};return {row:be.index,column:bd.index};},prefetchX:function(bi,bl,bj,bh){var bf=this.getVisibleLayers();if(bf.length==0){return;};var bk=this.getBounds();if(!bk){return;};var bm=this.__ub+bk.width;var bn=this.__ud-bm;if(this.__ub-this.__ue.left<Math.min(this.__ub,bi)||this.__ue.right-bm<Math.min(bn,bj)){var bo=Math.min(this.__ub,bl);var bg=Math.min(bn,bh);this._setLayerWindow(bf,this.__ub-bo,this.__ua,bk.width+bo+bg,bk.height,false);};},prefetchY:function(by,bu,bq,bt){var bp=this.getVisibleLayers();if(bp.length==0){return;};var bv=this.getBounds();if(!bv){return;};var br=this.__ua+bv.height;var bs=this.__uc-br;if(this.__ua-this.__ue.top<Math.min(this.__ua,by)||this.__ue.bottom-br<Math.min(bs,bq)){var bx=Math.min(this.__ua,bu);var bw=Math.min(bs,bt);this._setLayerWindow(bp,this.__ub,this.__ua-bx,bv.width,bv.height+bx+bw,false);};},_onResize:function(){if(this.getContentElement().getDomElement()){this.__uh=true;this._updateScrollPosition();this.__uh=null;this.fireEvent(f);};},_onAppear:function(){this.fullUpdate();},_onPointerDown:function(e){this.__uj=this.getCellAtPosition(e.getDocumentLeft(),e.getDocumentTop());},_onTap:function(e){this.__uk(e,u);},_onContextmenu:function(e){this.__uk(e,v);},_onDbltap:function(e){this.__uk(e,p);},__uk:function(e,bB){var bA=this.getCellAtPosition(e.getDocumentLeft(),e.getDocumentTop());if(!bA){return;};var bz=this.__uj;if(bz==null||bz.row!==bA.row||bz.column!==bA.column){return;};this.fireNonBubblingEvent(bB,qx.ui.virtual.core.CellEvent,[this,e,bA.row,bA.column]);},syncWidget:function(bC){if(this.__jj._fullUpdate){this._fullUpdate();}else if(this.__jj._updateScrollPosition){this._updateScrollPosition();};this.__jj={};},_setLayerWindow:function(bD,bS,top,bO,bK,bU){var bJ=this.__tX.getItemAtPosition(top);if(bJ){var bM=bJ.index;var bQ=this.__tX.getItemSizes(bM,bK+bJ.offset);var bL=qx.lang.Array.sum(bQ);var bT=top-bJ.offset;var bP=top-bJ.offset+bL;}else {var bM=0;var bQ=[];var bL=0;var bT=0;var bP=0;};var bE=this.__tY.getItemAtPosition(bS);if(bE){var bI=bE.index;var bG=this.__tY.getItemSizes(bI,bO+bE.offset);var bN=qx.lang.Array.sum(bG);var bR=bS-bE.offset;var bH=bS-bE.offset+bN;}else {var bI=0;var bG=[];var bN=0;var bR=0;var bH=0;};this.__ue={top:bT,bottom:bP,left:bR,right:bH};this.__uf.setUserBounds((this.getPaddingLeft()||0)+(this.__ue.left-this.__ub),(this.getPaddingTop()||0)+(this.__ue.top-this.__ua),bN,bL);this.__pe=bG;this.__ui=bQ;for(var i=0;i<this.__ug.length;i++ ){var bF=this.__ug[i];bF.setUserBounds(0,0,bN,bL);if(bU){bF.fullUpdate(bM,bI,bQ,bG);}else {bF.updateLayerWindow(bM,bI,bQ,bG);};};},__ul:function(){if(this.__uh){return;};var bV=this.getScrollSize();if(this.__uc!==bV.height||this.__ud!==bV.width){this.__uc=bV.height;this.__ud=bV.width;this.fireEvent(f);};},fullUpdate:function(){this.__jj._fullUpdate=1;qx.ui.core.queue.Widget.add(this);},isUpdatePending:function(){return !!this.__jj._fullUpdate;},_fullUpdate:function(){var bW=this.getVisibleLayers();if(bW.length==0){this.__ul();return;};var bX=this.getBounds();if(!bX){return;};this._setLayerWindow(bW,this.__ub,this.__ua,bX.width,bX.height,true);this.__ul();},_deferredUpdateScrollPosition:function(){this.__jj._updateScrollPosition=1;qx.ui.core.queue.Widget.add(this);},_updateScrollPosition:function(){var bY=this.getVisibleLayers();if(bY.length==0){this.__ul();return;};var cb=this.getBounds();if(!cb){return;};var ca={top:this.__ua,bottom:this.__ua+cb.height,left:this.__ub,right:this.__ub+cb.width};if(this.__ue.top<=ca.top&&this.__ue.bottom>=ca.bottom&&this.__ue.left<=ca.left&&this.__ue.right>=ca.right){this.__uf.setUserBounds((this.getPaddingLeft()||0)+(this.__ue.left-ca.left),(this.getPaddingTop()||0)+(this.__ue.top-ca.top),this.__ue.right-this.__ue.left,this.__ue.bottom-this.__ue.top);}else {this._setLayerWindow(bY,this.__ub,this.__ua,cb.width,cb.height,false);};this.__ul();}},destruct:function(){this._disposeArray(n);this._disposeObjects(d,m,l);this.__ue=this.__jj=this.__pe=this.__ui=null;}});})();(function(){var c="change",d="qx.event.type.Event",e="qx.ui.virtual.core.Axis";qx.Class.define(e,{extend:qx.core.Object,construct:function(f,g){qx.core.Object.call(this);this.itemCount=g;this.defaultItemSize=f;this.customSizes={};},events:{"change":d},members:{__um:null,getDefaultItemSize:function(){return this.defaultItemSize;},setDefaultItemSize:function(h){if(this.defaultItemSize!==h){this.defaultItemSize=h;this.__um=null;this.fireNonBubblingEvent(c);};},getItemCount:function(){return this.itemCount;},setItemCount:function(j){if(this.itemCount!==j){this.itemCount=j;this.__um=null;this.fireNonBubblingEvent(c);};},setItemSize:function(k,l){{};if(this.customSizes[k]==l){return;};if(l===null){delete this.customSizes[k];}else {this.customSizes[k]=l;};this.__um=null;this.fireNonBubblingEvent(c);},getItemSize:function(m){return this.customSizes[m]||this.defaultItemSize;},resetItemSizes:function(){this.customSizes={};this.__um=null;this.fireNonBubblingEvent(c);},__un:function(){if(this.__um){return this.__um;};var q=this.defaultItemSize;var o=this.itemCount;var w=[];for(var t in this.customSizes){var n=parseInt(t,10);if(n<o){w.push(n);};};if(w.length==0){var s=[{startIndex:0,endIndex:o-1,firstItemSize:q,rangeStart:0,rangeEnd:o*q-1}];this.__um=s;return s;};w.sort(function(a,b){return a>b?1:-1;});var s=[];var p=0;for(var i=0;i<w.length;i++ ){var n=w[i];if(n>=o){break;};var v=this.customSizes[n];var r=n*q+p;p+=v-q;s[i]={startIndex:n,firstItemSize:v,rangeStart:r};if(i>0){s[i-1].rangeEnd=r-1;s[i-1].endIndex=n-1;};};if(s[0].rangeStart>0){s.unshift({startIndex:0,endIndex:s[0].startIndex-1,firstItemSize:q,rangeStart:0,rangeEnd:s[0].rangeStart-1});};var x=s[s.length-1];var u=(o-x.startIndex-1)*q;x.rangeEnd=x.rangeStart+x.firstItemSize+u-1;x.endIndex=o-1;this.__um=s;return s;},__uo:function(D){var z=this.__um||this.__un();var A=0;var y=z.length-1;while(true){var C=A+((y-A)>>1);var B=z[C];if(B.rangeEnd<D){A=C+1;}else if(B.rangeStart>D){y=C-1;}else {return B;};};},getItemAtPosition:function(I){if(I<0||I>=this.getTotalSize()){return null;};var H=this.__uo(I);var J=H.rangeStart;var E=H.startIndex;var G=H.firstItemSize;if(J+G>I){return {index:E,offset:I-J};}else {var F=this.defaultItemSize;return {index:E+1+Math.floor((I-J-G)/F),offset:(I-J-G)%F};};},__up:function(K){var M=this.__um||this.__un();var N=0;var L=M.length-1;while(true){var P=N+((L-N)>>1);var O=M[P];if(O.endIndex<K){N=P+1;}else if(O.startIndex>K){L=P-1;}else {return O;};};},getItemPosition:function(Q){if(Q<0||Q>=this.itemCount){return null;};var R=this.__up(Q);if(R.startIndex==Q){return R.rangeStart;}else {return R.rangeStart+R.firstItemSize+(Q-R.startIndex-1)*this.defaultItemSize;};},getTotalSize:function(){var S=this.__um||this.__un();return S[S.length-1].rangeEnd+1;},getItemSizes:function(Y,T){var X=this.customSizes;var W=this.defaultItemSize;var V=0;var U=[];var i=0;while(V<T){var ba=X[Y]!=null?X[Y]:W;Y++ ;V+=ba;U[i++ ]=ba;if(Y>=this.itemCount){break;};};return U;}},destruct:function(){this.customSizes=this.__um=null;}});})();(function(){var a="qx.ui.virtual.core.CellEvent",b="Integer";qx.Class.define(a,{extend:qx.event.type.Pointer,properties:{row:{check:b,nullable:true},column:{check:b,nullable:true}},members:{init:function(e,c,f,d){c.clone(this);this.setBubbles(false);this.setRow(f);this.setColumn(d);}}});})();(function(){var a="one",b="Boolean",c="qx.ui.virtual.selection.MModel",d="qx.data.Array",f="multi",g="selected",h="_applySelection",j="change",k="_applyDragSelection",l="single",m="_applyQuickSelection",n="changeSelection",o="_applySelectionMode",p="additive",q="qx.event.type.Data";qx.Mixin.define(c,{construct:function(){this._initSelectionManager();this.__tM=new qx.data.Array();this.initSelection(this.__tM);},properties:{selection:{check:d,event:n,apply:h,nullable:false,deferredInit:true},selectionMode:{check:[l,f,p,a],init:l,apply:o},dragSelection:{check:b,init:false,apply:k},quickSelection:{check:b,init:false,apply:m}},events:{"changeSelection":q},members:{_manager:null,__uq:false,__ur:false,__tM:null,_initSelectionManager:function(){var self=this;var r={isItemSelectable:function(s){return self._provider.isSelectable(s);},styleSelectable:function(t,u,v){if(u!=g){return;};if(v){self._provider.styleSelectabled(t);}else {self._provider.styleUnselectabled(t);};}};this._manager=new qx.ui.virtual.selection.Row(this.getPane(),r);this._manager.attachPointerEvents(this.getPane());this._manager.attachKeyEvents(this);this._manager.addListener(n,this._onManagerChangeSelection,this);},setAutoScrollIntoView:function(w){this._manager._autoScrollIntoView=w;},getAutoScrollIntoView:function(){return this._manager._autoScrollIntoView;},_updateSelection:function(){if(this._manager==null){return;};this._onChangeSelection();},_applySelection:function(y,x){y.addListener(j,this._onChangeSelection,this);if(x!=null){x.removeListener(j,this._onChangeSelection,this);};this._onChangeSelection();},_applySelectionMode:function(A,z){this._manager.setMode(A);},_applyDragSelection:function(C,B){this._manager.setDrag(C);},_applyQuickSelection:function(E,D){this._manager.setQuick(E);},_onChangeSelection:function(e){if(this.__ur==true){return;};this.__uq=true;var G=this.getSelection();var I=[];for(var i=0;i<G.getLength();i++ ){var H=G.getItem(i);var J=this._getSelectables();var F=-1;if(J!=null){F=J.indexOf(H);};var K=this._reverseLookup(F);if(K>=0){I.push(K);};};if(this._beforeApplySelection!=null&&qx.lang.Type.isFunction(this._beforeApplySelection)){this._beforeApplySelection(I);};try{if(!qx.lang.Array.equals(I,this._manager.getSelection())){this._manager.replaceSelection(I);};}catch(L){this._manager.selectItem(I[I.length-1]);};this.__tR();if(this._afterApplySelection!=null&&qx.lang.Type.isFunction(this._afterApplySelection)){this._afterApplySelection();};this.__uq=false;},_onManagerChangeSelection:function(e){if(this.__uq==true){return;};this.__ur=true;this.__tR();this.__ur=false;},__tR:function(){if(this.__ut()){return;};var M=this._manager.getSelection();var N=[];for(var i=0;i<M.length;i++ ){var O=this._getDataFromRow(M[i]);if(O!=null){N.push(O);};};this.__us(N);},__us:function(R){var Q=this.getSelection();if(R.length>0){var P=[0,Q.getLength()];P=P.concat(R);var S=Q.splice.apply(Q,P);S.dispose();}else {Q.removeAll();};},__ut:function(){var U=this.getSelection();var W=this._manager.getSelection();if(U.getLength()!==W.length){return false;};for(var i=0;i<U.getLength();i++ ){var V=U.getItem(i);var X=this._getSelectables();var T=-1;if(X!=null){T=X.indexOf(V);};var Y=this._reverseLookup(T);if(Y!==W[i]){return false;};};return true;},_applyDefaultSelection:function(){if(this._manager!=null){this._manager._applyDefaultSelection();};}},destruct:function(){this._manager.dispose();this._manager=null;if(this.__tM){this.__tM.dispose();};}});})();(function(){var a="qx.ui.virtual.selection.Abstract",b="keypress",c="losecapture",d="pointerover",e="pointerdown",f="removeItem",g="tap",h="pointermove",i="addItem";qx.Class.define(a,{extend:qx.ui.core.selection.Abstract,construct:function(k,j){qx.ui.core.selection.Abstract.call(this);{};this._pane=k;this._delegate=j||{};},members:{_autoScrollIntoView:true,_isSelectable:function(l){return this._delegate.isItemSelectable?this._delegate.isItemSelectable(l):true;},_styleSelectable:function(m,n,o){if(this._delegate.styleSelectable){this._delegate.styleSelectable(m,n,o);};},attachPointerEvents:function(){var p=this._pane.getContentElement();p.addListener(e,this.handlePointerDown,this);p.addListener(g,this.handleTap,this);p.addListener(d,this.handlePointerOver,this);p.addListener(h,this.handlePointerMove,this);p.addListener(c,this.handleLoseCapture,this);},detatchPointerEvents:function(){var q=this._pane.getContentElement();q.removeListener(e,this.handlePointerDown,this);q.removeListener(g,this.handleTap,this);q.removeListener(d,this.handlePointerOver,this);q.removeListener(h,this.handlePointerMove,this);q.removeListener(c,this.handleLoseCapture,this);},attachKeyEvents:function(r){r.addListener(b,this.handleKeyPress,this);},detachKeyEvents:function(s){s.removeListener(b,this.handleKeyPress,this);},attachListEvents:function(t){t.addListener(i,this.handleAddItem,this);t.addListener(f,this.handleRemoveItem,this);},detachListEvents:function(u){u.removeListener(i,this.handleAddItem,this);u.removeListener(f,this.handleRemoveItem,this);},_capture:function(){this._pane.capture();},_releaseCapture:function(){this._pane.releaseCapture();},_getScroll:function(){return {left:this._pane.getScrollX(),top:this._pane.getScrollY()};},_scrollBy:function(v,w){this._pane.setScrollX(this._pane.getScrollX()+v);this._pane.setScrollY(this._pane.getScrollY()+w);},_getLocation:function(){var x=this._pane.getContentElement().getDomElement();return x?qx.bom.element.Location.get(x):null;},_getDimension:function(){return this._pane.getInnerSize();}},destruct:function(){this._pane=this._delegate=null;}});})();(function(){var a="qx.ui.virtual.selection.Row",b="above",c="under";qx.Class.define(a,{extend:qx.ui.virtual.selection.Abstract,members:{_getItemCount:function(){return this._pane.getRowConfig().getItemCount();},_getSelectableFromPointerEvent:function(event){var d=this._pane.getCellAtPosition(event.getDocumentLeft(),event.getDocumentTop());if(!d){return null;};return this._isSelectable(d.row)?d.row:null;},getSelectables:function(f){var e=[];for(var i=0,l=this._getItemCount();i<l;i++ ){if(this._isSelectable(i)){e.push(i);};};return e;},_getSelectableRange:function(j,k){var m=[];var g=Math.min(j,k);var h=Math.max(j,k);for(var i=g;i<=h;i++ ){if(this._isSelectable(i)){m.push(i);};};return m;},_getFirstSelectable:function(){var n=this._getItemCount();for(var i=0;i<n;i++ ){if(this._isSelectable(i)){return i;};};return null;},_getLastSelectable:function(){var o=this._getItemCount();for(var i=o-1;i>=0;i-- ){if(this._isSelectable(i)){return i;};};return null;},_getRelatedSelectable:function(q,s){if(s==b){var r=q-1;var p=0;var t=-1;}else if(s==c){var r=q+1;var p=this._getItemCount()-1;var t=1;}else {return null;};for(var i=r;i!==p+t;i+=t){if(this._isSelectable(i)){return i;};};return null;},_getPage:function(v,u){if(u){return this._getFirstSelectable();}else {return this._getLastSelectable();};},_selectableToHashCode:function(w){return w;},_scrollItemIntoView:function(x){if(this._autoScrollIntoView){this._pane.scrollRowIntoView(x);};},_getSelectableLocationX:function(y){return {left:0,right:this._pane.getColumnConfig().getTotalSize()-1};},_getSelectableLocationY:function(z){var C=this._pane.getRowConfig();var B=C.getItemPosition(z);var A=B+C.getItemSize(z)-1;return {top:B,bottom:A};}}});})();(function(){var c="changeModel",d="resize",f="_applyLabelOptions",g="_applyLabelPath",h="qx.data.Array",j="_applyGroupLabelOptions",m="changeLength",n="row-layer",o="_applyModel",p="changeGroups",q="Boolean",r="GroupingTypeError: You can't mix 'Objects' and 'Strings' as",s="Integer",t="sorter",u="_applyGroupRowHeight",v="group",w="_applyIconPath",x="_applyDelegate",y="changeDelegate",z=" group identifier!",A="???",B="_applyRowHeight",C="qx.data.IListData",D="filter",E="virtual-list",F="String",G="_applyGroupLabelPath",H="_applyIconOptions",I="qx.ui.list.List";qx.Class.define(I,{extend:qx.ui.virtual.core.Scroller,include:[qx.ui.virtual.selection.MModel],implement:qx.data.controller.ISelection,construct:function(J){qx.ui.virtual.core.Scroller.call(this,0,1,20,100);this._init();this.__uu=new qx.data.Array();this.initGroups(this.__uu);if(J!=null){this.initModel(J);};this.initItemHeight();},properties:{appearance:{refine:true,init:E},focusable:{refine:true,init:true},width:{refine:true,init:100},height:{refine:true,init:200},model:{check:C,apply:o,event:c,nullable:true,deferredInit:true},itemHeight:{check:s,init:25,apply:B,themeable:true},groupItemHeight:{check:s,init:null,nullable:true,apply:u,themeable:true},labelPath:{check:F,apply:g,nullable:true},iconPath:{check:F,apply:w,nullable:true},groupLabelPath:{check:F,apply:G,nullable:true},labelOptions:{apply:f,nullable:true},iconOptions:{apply:H,nullable:true},groupLabelOptions:{apply:j,nullable:true},delegate:{apply:x,event:y,init:null,nullable:true},autoGrouping:{check:q,init:true},groups:{check:h,event:p,nullable:false,deferredInit:true}},members:{_background:null,_provider:null,_layer:null,__mH:null,__uv:null,__uw:null,__ux:false,__uy:false,__uz:false,__uu:null,refresh:function(){this.__mV();},_createChildControlImpl:function(M,L){var K;switch(M){case n:K=new qx.ui.virtual.layer.Row(null,null);break;};return K||qx.ui.virtual.core.Scroller.prototype._createChildControlImpl.call(this,M);},_init:function(){this._provider=new qx.ui.list.provider.WidgetProvider(this);this.__mH=[];this.__uv=[];this.__uw={};this.__ux=false;this.__uy=false;this.__uz=false;this.getPane().addListener(d,this._onResize,this);this._initBackground();this._initLayer();},_initBackground:function(){this._background=this.getChildControl(n);this.getPane().addLayer(this._background);},_initLayer:function(){this._layer=this._provider.createLayer();this.getPane().addLayer(this._layer);},_getDataFromRow:function(P){var O=null;var N=this.getModel();if(N==null){return null;};if(this._isGroup(P)){O=this.getGroups().getItem(this._lookupGroup(P));}else {O=N.getItem(this._lookup(P));};if(O!=null){return O;}else {return null;};},_getLookupTable:function(){return this.__mH;},_lookup:function(Q){return this.__mH[Q];},_lookupGroup:function(R){return this.__uv.indexOf(R);},_reverseLookup:function(S){if(S<0){return -1;};return this.__mH.indexOf(S);},_isGroup:function(T){return this._lookup(T)==-1;},_getSelectables:function(){return this.getModel();},_applyModel:function(V,U){if(V!=null){V.addListener(m,this._onModelChange,this);};if(U!=null){U.removeListener(m,this._onModelChange,this);};this._provider.removeBindings();this._onModelChange();},_applyRowHeight:function(X,W){this.getPane().getRowConfig().setDefaultItemSize(X);},_applyGroupRowHeight:function(ba,Y){this.__uB();},_applyLabelPath:function(bc,bb){this._provider.setLabelPath(bc);},_applyIconPath:function(be,bd){this._provider.setIconPath(be);},_applyGroupLabelPath:function(bg,bf){this._provider.setGroupLabelPath(bg);},_applyLabelOptions:function(bi,bh){this._provider.setLabelOptions(bi);},_applyIconOptions:function(bk,bj){this._provider.setIconOptions(bk);},_applyGroupLabelOptions:function(bm,bl){this._provider.setGroupLabelOptions(bm);},_applyDelegate:function(bo,bn){this._provider.setDelegate(bo);this.__mV();},_onResize:function(e){this.getPane().getColumnConfig().setItemSize(0,e.getData().width);},_onModelChange:function(e){this.__mV();this._applyDefaultSelection();},__uA:function(){this.getPane().getRowConfig().setItemCount(this.__mH.length);this.getPane().fullUpdate();},__uB:function(){var bp=this.getPane().getRowConfig();var bq=this.getGroupItemHeight();bp.resetItemSizes();if(bq){for(var i=0,l=this.__mH.length;i<l; ++i){if(this.__mH[i]==-1){bp.setItemSize(i,bq);};};};},__mV:function(){this.__mH=[];this.__uv=[];this.__uw={};if(this.isAutoGrouping()){this.getGroups().removeAll();};var br=this.getModel();if(br!=null){this._runDelegateFilter(br);this._runDelegateSorter(br);this._runDelegateGroup(br);};this._updateSelection();this.__uB();this.__uA();},_runDelegateFilter:function(bt){var bs=qx.util.Delegate.getMethod(this.getDelegate(),D);for(var i=0,l=bt.length;i<l; ++i){if(bs==null||bs(bt.getItem(i))){this.__mH.push(i);};};},_runDelegateSorter:function(bu){if(this.__mH.length==0){return;};var bv=qx.util.Delegate.getMethod(this.getDelegate(),t);if(bv!=null){this.__mH.sort(function(a,b){return bv(bu.getItem(a),bu.getItem(b));});};},_runDelegateGroup:function(bz){var bA=qx.util.Delegate.getMethod(this.getDelegate(),v);if(bA!=null){for(var i=0,l=this.__mH.length;i<l; ++i){var bw=this.__mH[i];var by=this.getModel().getItem(bw);var bx=bA(by);this.__uC(bx,bw);};this.__mH=this.__uD();};},__uC:function(bC,bB){if(bC==null){this.__uz=true;bC=A;};var name=this.__uE(bC);if(this.__uw[name]==null){this.__uw[name]=[];if(this.isAutoGrouping()){this.getGroups().push(bC);};};this.__uw[name].push(bB);},__uD:function(){this.__uF();var bG=[];var bI=0;var bE=this.getGroups();for(var i=0;i<bE.getLength();i++ ){var bD=bE.getItem(i);bG.push(-1);this.__uv.push(bI);bI++ ;var bH=this.__uE(bD);var bF=this.__uw[bH];if(bF!=null){for(var k=0;k<bF.length;k++ ){bG.push(bF[k]);bI++ ;};};};return bG;},__uE:function(bK){var name=null;if(!qx.lang.Type.isString(bK)){var bJ=this.getGroups().indexOf(bK);this.__uy=true;name=v;if(bJ==-1){name+=this.getGroups().getLength();}else {name+=bJ;};}else {this.__ux=true;var name=bK;};return name;},__uF:function(){if(this.__uy&&this.__uz||this.__uy&&this.__ux){throw new Error(r+z);};}},destruct:function(){var bL=this.getModel();if(bL!=null){bL.removeListener(m,this._onModelChange,this);};var bM=this.getPane();if(bM!=null){bM.removeListener(d,this._onResize,this);};this._background.dispose();this._provider.dispose();this._layer.dispose();this._background=this._provider=this._layer=this.__mH=this.__uv=this.__uw=null;if(this.__uu){this.__uu.dispose();};}});})();(function(){var a="qx.ui.virtual.core.ILayer";qx.Interface.define(a,{members:{fullUpdate:function(d,e,c,b){this.assertArgumentsCount(arguments,6,6);this.assertPositiveInteger(d);this.assertPositiveInteger(e);this.assertArray(c);this.assertArray(b);},updateLayerWindow:function(h,i,g,f){this.assertArgumentsCount(arguments,6,6);this.assertPositiveInteger(h);this.assertPositiveInteger(i);this.assertArray(g);this.assertArray(f);},updateLayerData:function(){}}});})();(function(){var a="qx.ui.virtual.layer.Abstract",b="abstract",c="Abstract method '_fullUpdate' called!";qx.Class.define(a,{extend:qx.ui.core.Widget,type:b,implement:[qx.ui.virtual.core.ILayer],construct:function(){qx.ui.core.Widget.call(this);this.__jj={};},properties:{anonymous:{refine:true,init:true}},members:{__jj:null,__bN:null,__uG:null,__uH:null,__ui:null,__pe:null,getFirstRow:function(){return this.__uG;},getFirstColumn:function(){return this.__uH;},getRowSizes:function(){return this.__ui||[];},getColumnSizes:function(){return this.__pe||[];},syncWidget:function(e){if(!this.getContentElement().getDomElement()){return;};if(this.__jj.fullUpdate||this.__jj.updateLayerWindow&&this.__jj.updateLayerData){this._fullUpdate.apply(this,this.__bN);}else if(this.__jj.updateLayerWindow){this._updateLayerWindow.apply(this,this.__bN);}else if(this.__jj.updateLayerData&&this.__ui){this._updateLayerData();};if(this.__jj.fullUpdate||this.__jj.updateLayerWindow){var d=this.__bN;this.__uG=d[0];this.__uH=d[1];this.__ui=d[2];this.__pe=d[3];};this.__jj={};},_updateLayerData:function(){this._fullUpdate(this.__uG,this.__uH,this.__ui,this.__pe);},_fullUpdate:function(g,i,h,f){throw new Error(c);},_updateLayerWindow:function(k,m,l,j){this._fullUpdate(k,m,l,j);},updateLayerData:function(){this.__jj.updateLayerData=true;qx.ui.core.queue.Widget.add(this);},fullUpdate:function(p,q,o,n){this.__bN=arguments;this.__jj.fullUpdate=true;qx.ui.core.queue.Widget.add(this);},updateLayerWindow:function(t,u,s,r){this.__bN=arguments;this.__jj.updateLayerWindow=true;qx.ui.core.queue.Widget.add(this);}},destruct:function(){this.__jj=this.__bN=this.__ui=this.__pe=null;}});})();(function(){var a="_applyColorOdd",b="_applyColorEven",c="Color",d="qx.ui.virtual.layer.AbstractBackground";qx.Class.define(d,{extend:qx.ui.virtual.layer.Abstract,construct:function(e,f){qx.ui.virtual.layer.Abstract.call(this);if(e){this.setColorEven(e);};if(f){this.setColorOdd(f);};this.__uI={};this.__uJ={};},properties:{colorEven:{nullable:true,check:c,apply:b,themeable:true},colorOdd:{nullable:true,check:c,apply:a,themeable:true}},members:{__uK:null,__uL:null,__uI:null,__uJ:null,setColor:function(h,g){if(g){this.__uI[h]=qx.theme.manager.Color.getInstance().resolve(g);}else {delete (this.__uI[h]);};},clearCustomColors:function(){this.__uI={};this.updateLayerData();},getColor:function(i){var j=this.__uI[i];if(j){return j;}else {return i%2==0?this.__uK:this.__uL;};},_applyColorEven:function(l,k){if(l){this.__uK=qx.theme.manager.Color.getInstance().resolve(l);}else {this.__uK=null;};this.updateLayerData();},_applyColorOdd:function(n,m){if(n){this.__uL=qx.theme.manager.Color.getInstance().resolve(n);}else {this.__uL=null;};this.updateLayerData();},setBackground:function(o,p){if(p){this.__uJ[o]=qx.theme.manager.Decoration.getInstance().resolve(p);}else {delete (this.__uJ[o]);};this.updateLayerData();},getBackground:function(q){return this.__uJ[q];}},destruct:function(){this.__uI=this.__uJ=null;}});})();(function(){var a="left: 0;",b="width:",c="background-color:",d="qx.ui.virtual.layer.Row",e="",f="block",g="position: absolute;",h="</div>",i="height:",j="row-layer",k="<div style='",l="'>",m=";",n="px;",o="none",p="top:";qx.Class.define(d,{extend:qx.ui.virtual.layer.AbstractBackground,properties:{appearance:{refine:true,init:j}},members:{_fullUpdate:function(x,u,t,w){var A=[];var z=qx.lang.Array.sum(w);var top=0;var D=x;var s=0;for(var y=0;y<t.length;y++ ){var v=this.getColor(D);var B=v?c+v+m:e;var C=this.getBackground(D);var q=C?qx.bom.element.Style.compile(C.getStyles()):e;A.push(k,g,a,p,top,n,i,t[y],n,b,z,n,B,q,l,h);s++ ;top+=t[y];D+=1;};var r=this.getContentElement().getDomElement();r.style.display=o;r.innerHTML=A.join(e);r.style.display=f;this._width=z;},_updateLayerWindow:function(G,H,F,E){if(G!==this.getFirstRow()||F.length!==this.getRowSizes().length||this._width<qx.lang.Array.sum(E)){this._fullUpdate(G,H,F,E);};},setColor:function(J,I){qx.ui.virtual.layer.AbstractBackground.prototype.setColor.call(this,J,I);if(this.__uM(J)){this.updateLayerData();};},setBackground:function(K,L){qx.ui.virtual.layer.AbstractBackground.prototype.setBackground.call(this,K,L);if(this.__uM(K)){this.updateLayerData();};},__uM:function(M){var O=this.getFirstRow();var N=O+this.getRowSizes().length-1;return M>=O&&M<=N;}}});})();(function(){var a="qx.ui.list.provider.IListProvider";qx.Interface.define(a,{members:{createLayer:function(){},createItemRenderer:function(){},createGroupRenderer:function(){},styleSelectabled:function(b){},styleUnselectabled:function(c){},isSelectable:function(d){},setLabelPath:function(e){},setIconPath:function(f){},setLabelOptions:function(g){},setIconOptions:function(h){},setDelegate:function(i){},removeBindings:function(){}}});})();(function(){var a="qx.ui.virtual.core.IWidgetCellProvider";qx.Interface.define(a,{members:{getCellWidget:function(c,b){},poolCellWidget:function(d){}}});})();(function(){var a="cell.type",b="model[",c="groups[",d="model",f="",g="changeDelegate",h="label",i="qx.ui.list.core.MWidgetController",j="icon",k="]",l="value",m="BindingIds",n=".",o="group",p="String";qx.Mixin.define(i,{construct:function(){this.__uN=[];},properties:{labelPath:{check:p,nullable:true},iconPath:{check:p,nullable:true},groupLabelPath:{check:p,nullable:true},labelOptions:{nullable:true},iconOptions:{nullable:true},groupLabelOptions:{nullable:true},delegate:{event:g,init:null,nullable:true}},members:{__uN:null,bindDefaultProperties:function(q,r){if(q.getUserData(a)!=o){this.bindProperty(f,d,null,q,r);this.bindProperty(this.getLabelPath(),h,this.getLabelOptions(),q,r);if(this.getIconPath()!=null){this.bindProperty(this.getIconPath(),j,this.getIconOptions(),q,r);};}else {this.bindProperty(this.getGroupLabelPath(),l,this.getGroupLabelOptions(),q,r);};},bindProperty:function(u,x,v,t,s){var y=t.getUserData(a);var w=this.__uO(s,u,y);if(v){v.ignoreConverter=d;};var z=this._list.bind(w,t,x,v);this.__uP(t,z);},bindPropertyReverse:function(D,B,E,F,A){var G=F.getUserData(a);var C=this.__uO(A,D,G);var H=F.bind(B,this._list,C,E);this.__uP(F,H);},removeBindings:function(){while(this.__uN.length>0){var I=this.__uN.pop();this._removeBindingsFrom(I);};},_configureItem:function(J){var K=this.getDelegate();if(K!=null&&K.configureItem!=null){K.configureItem(J);};},_configureGroupItem:function(L){var M=this.getDelegate();if(M!=null&&M.configureGroupItem!=null){M.configureGroupItem(L);};},_bindItem:function(N,P){var O=this.getDelegate();if(O!=null&&O.bindItem!=null){O.bindItem(this,N,P);}else {this.bindDefaultProperties(N,P);};},_bindGroupItem:function(Q,S){var R=this.getDelegate();if(R!=null&&R.bindGroupItem!=null){R.bindGroupItem(this,Q,S);}else {this.bindDefaultProperties(Q,S);};},_removeBindingsFrom:function(T){var U=this.__uQ(T);while(U.length>0){var V=U.pop();try{this._list.removeBinding(V);}catch(e){T.removeBinding(V);};};if(qx.lang.Array.contains(this.__uN,T)){qx.lang.Array.remove(this.__uN,T);};},__uO:function(X,ba,Y){var W=b+X+k;if(Y==o){W=c+X+k;};if(ba!=null&&ba!=f){W+=n+ba;};return W;},__uP:function(bc,bd){var bb=this.__uQ(bc);if(!qx.lang.Array.contains(bb,bd)){bb.push(bd);};if(!qx.lang.Array.contains(this.__uN,bc)){this.__uN.push(bc);};},__uQ:function(be){var bf=be.getUserData(m);if(bf==null){bf=[];be.setUserData(m,bf);};return bf;}},destruct:function(){this.__uN=null;}});})();(function(){var a="cell.type",b="changeDelegate",c="qx.ui.list.provider.WidgetProvider",d="createItem",e="group-item",f="onPool",g="item",h="createGroupItem",i="created",j="group";qx.Class.define(c,{extend:qx.core.Object,implement:[qx.ui.virtual.core.IWidgetCellProvider,qx.ui.list.provider.IListProvider],include:[qx.ui.list.core.MWidgetController],construct:function(k){qx.core.Object.call(this);this._list=k;this._itemRenderer=this.createItemRenderer();this._groupRenderer=this.createGroupRenderer();this._itemRenderer.addListener(i,this._onItemCreated,this);this._groupRenderer.addListener(i,this._onGroupItemCreated,this);this._list.addListener(b,this._onChangeDelegate,this);},members:{_itemRenderer:null,_groupRenderer:null,getCellWidget:function(n,l){var m=null;if(!this._list._isGroup(n)){m=this._itemRenderer.getCellWidget();m.setUserData(a,g);this._bindItem(m,this._list._lookup(n));if(this._list._manager.isItemSelected(n)){this._styleSelectabled(m);}else {this._styleUnselectabled(m);};}else {m=this._groupRenderer.getCellWidget();m.setUserData(a,j);this._bindGroupItem(m,this._list._lookupGroup(n));};return m;},poolCellWidget:function(o){this._removeBindingsFrom(o);if(o.getUserData(a)==g){this._itemRenderer.pool(o);}else if(o.getUserData(a)==j){this._groupRenderer.pool(o);};this._onPool(o);},createLayer:function(){return new qx.ui.virtual.layer.WidgetCell(this);},createItemRenderer:function(){var p=qx.util.Delegate.getMethod(this.getDelegate(),d);if(p==null){p=function(){return new qx.ui.form.ListItem();};};var q=new qx.ui.virtual.cell.WidgetCell();q.setDelegate({createWidget:p});return q;},createGroupRenderer:function(){var r=qx.util.Delegate.getMethod(this.getDelegate(),h);if(r==null){r=function(){var t=new qx.ui.basic.Label();t.setAppearance(e);return t;};};var s=new qx.ui.virtual.cell.WidgetCell();s.setDelegate({createWidget:r});return s;},styleSelectabled:function(v){var u=this.__uR(v);this._styleSelectabled(u);},styleUnselectabled:function(x){var w=this.__uR(x);this._styleUnselectabled(w);},isSelectable:function(z){if(this._list._isGroup(z)){return false;};var y=this._list._layer.getRenderedCellWidget(z,0);if(y!=null){return y.isEnabled();}else {return true;};},_styleSelectabled:function(A){this.__uS(A,{selected:1});},_styleUnselectabled:function(B){this.__uS(B,{});},_onPool:function(C){var D=qx.util.Delegate.getMethod(this.getDelegate(),f);if(D!=null){D(C);};},_onItemCreated:function(event){var E=event.getData();this._configureItem(E);},_onGroupItemCreated:function(event){var F=event.getData();this._configureGroupItem(F);},_onChangeDelegate:function(event){this._itemRenderer.dispose();this._itemRenderer=this.createItemRenderer();this._itemRenderer.addListener(i,this._onItemCreated,this);this._groupRenderer.dispose();this._groupRenderer=this.createGroupRenderer();this._groupRenderer.addListener(i,this._onGroupItemCreated,this);this.removeBindings();this._list.getPane().fullUpdate();},__uR:function(G){return this._list._layer.getRenderedCellWidget(G,0);},__uS:function(I,H){if(I==null){return;};this._itemRenderer.updateStates(I,H);}},destruct:function(){this._itemRenderer.dispose();this._groupRenderer.dispose();this._itemRenderer=this._groupRenderer=null;}});})();(function(){var a="cell.column",b="updated",c="qx.event.type.Event",d="qx.ui.virtual.layer.WidgetCell",e="cell.row",f="cell.empty";qx.Class.define(d,{extend:qx.ui.virtual.layer.Abstract,include:[qx.ui.core.MChildrenHandling],construct:function(g){qx.ui.virtual.layer.Abstract.call(this);this.setZIndex(12);{};this._cellProvider=g;this.__uT=[];},properties:{anonymous:{refine:true,init:false}},events:{updated:c},members:{__uT:null,getRenderedCellWidget:function(p,m){if(this._getChildren().length===0){return null;};var h=this.getColumnSizes().length;var o=this.getRowSizes().length;var n=this.getFirstRow();var l=this.getFirstColumn();if(p<n||p>=n+o||m<l||m>=l+h){return null;};var k=(m-l)+(p-n)*h;var j=this._getChildren()[k];if(!j||j.getUserData(f)){return null;}else {return j;};},_getSpacer:function(){var q=this.__uT.pop();if(!q){q=new qx.ui.core.Spacer();q.setUserData(f,1);};return q;},_activateNotEmptyChild:function(r){var s=qx.ui.core.FocusHandler.getInstance().getActiveWidget();if(s==r||qx.ui.core.Widget.contains(r,s)){var t=this._getChildren();for(var i=t.length-1;i>=0;i-- ){if(!t[i].getUserData(f)){t[i].activate();break;};};};},_fullUpdate:function(A,u,z,w){var B=this._cellProvider;var G=this._getChildren().concat();for(var i=0;i<G.length;i++ ){var E=G[i];if(E.getUserData(f)){this.__uT.push(E);}else {this._activateNotEmptyChild(E);B.poolCellWidget(E);};};var top=0;var F=0;var D=[];for(var y=0;y<z.length;y++ ){for(var x=0;x<w.length;x++ ){var H=A+y;var v=u+x;var C=B.getCellWidget(H,v)||this._getSpacer();D.push(C);C.setUserBounds(F,top,w[x],z[y]);C.setUserData(e,H);C.setUserData(a,v);this._add(C);F+=w[x];};top+=z[y];F=0;};G.forEach(function(I){if(D.indexOf(I)===-1){this._remove(I);};}.bind(this));this.fireEvent(b);},_updateLayerWindow:function(W,J,M,bb){{};var N=W+M.length-1;var X=J+bb.length-1;var L={firstRow:Math.max(W,this.getFirstRow()),lastRow:Math.min(N,this._lastRow),firstColumn:Math.max(J,this.getFirstColumn()),lastColumn:Math.min(X,this._lastColumn)};this._lastColumn=X;this._lastRow=N;if(L.firstRow>L.lastRow||L.firstColumn>L.lastColumn){return this._fullUpdate(W,J,M,bb);};var O=this._getChildren();var Q=this.getColumnSizes().length;var T=[];var ba={};for(var P=W;P<=N;P++ ){T[P]=[];for(var Y=J;Y<=X;Y++ ){if(P>=L.firstRow&&P<=L.lastRow&&Y>=L.firstColumn&&Y<=L.lastColumn){var x=Y-this.getFirstColumn();var y=P-this.getFirstRow();var R=y*Q+x;T[P][Y]=O[R];ba[R]=true;};};};var K=this._cellProvider;var O=this._getChildren().concat();for(var i=0;i<O.length;i++ ){if(!ba[i]){var V=O[i];if(V.getUserData(f)){this.__uT.push(V);}else {this._activateNotEmptyChild(V);K.poolCellWidget(V);};};};var top=0;var bc=0;var U=[];for(var y=0;y<M.length;y++ ){for(var x=0;x<bb.length;x++ ){var P=W+y;var Y=J+x;var S=T[P][Y]||K.getCellWidget(P,Y)||this._getSpacer();U.push(S);S.setUserBounds(bc,top,bb[x],M[y]);S.setUserData(e,P);S.setUserData(a,Y);this._add(S);bc+=bb[x];};top+=M[y];bc=0;};O.forEach(function(bd){if(U.indexOf(bd)===-1){this._remove(bd);};}.bind(this));this.fireEvent(b);}},destruct:function(){var be=this._getChildren();for(var i=0;i<be.length;i++ ){be[i].dispose();};this._cellProvider=this.__uT=null;}});})();(function(){var a="qx.util.Delegate";qx.Class.define(a,{statics:{getMethod:function(b,c){if(qx.util.Delegate.containsMethod(b,c)){return qx.lang.Function.bind(b[c],b);};return null;},containsMethod:function(e,f){var d=qx.lang.Type;if(d.isObject(e)){return d.isFunction(e[f]);};return false;}}});})();(function(){var a="qx.ui.virtual.cell.IWidgetCell";qx.Interface.define(a,{members:{getCellWidget:function(c,b){},pool:function(d){},updateStates:function(f,e){},updateData:function(g,h){}}});})();(function(){var a="created",b="qx.ui.virtual.cell.AbstractWidget",c="cell.states",d="abstract method call",e="qx.event.type.Data";qx.Class.define(b,{extend:qx.core.Object,implement:[qx.ui.virtual.cell.IWidgetCell],construct:function(){qx.core.Object.call(this);this.__cO=[];},events:{"created":e},members:{__cO:null,_createWidget:function(){throw new Error(d);},updateData:function(f,g){throw new Error(d);},updateStates:function(j,h){var k=j.getUserData(c);if(k){var i=h||{};for(var l in k){if(!i[l]){j.removeState(l);};};}else {k={};};if(h){for(var l in h){if(!k.state){j.addState(l);};};};j.setUserData(c,h);},getCellWidget:function(o,m){var n=this.__uU();this.updateStates(n,m);this.updateData(n,o);return n;},pool:function(p){this.__cO.push(p);},_cleanupPool:function(){var q=this.__cO.pop();while(q){q.destroy();q=this.__cO.pop();};},__uU:function(){var r=this.__cO.shift();if(r==null){r=this._createWidget();this.fireDataEvent(a,r);};return r;}},destruct:function(){this._cleanupPool();this.__cO=null;}});})();(function(){var a="_applyDelegate",b="qx.ui.virtual.cell.WidgetCell",c="Can't update data! The key '",d="' is not a Property!";qx.Class.define(b,{extend:qx.ui.virtual.cell.AbstractWidget,properties:{delegate:{apply:a,init:null,nullable:true}},members:{_applyDelegate:function(f,e){this._cleanupPool();},_createWidget:function(){var g=this.getDelegate();if(g!=null&&g.createWidget!=null){return g.createWidget();}else {return new qx.ui.core.Widget();};},updateData:function(h,i){for(var j in i){if(qx.Class.hasProperty(h.constructor,j)){qx.util.PropertyUtil.setUserValue(h,j,i[j]);}else {throw new Error(c+j+d);};};}}});})();(function(){var a="Enter",b="qx.ui.form.VirtualComboBox",c="changeValue",d="button",f="execute",g="Function",h="inner",j="list",k="change",m="virtual-combobox",n="value",o="select",p="String",q="",r="textfield",s="dropdown",t="single",u="focusin",v="converter",w="focusout",x="_applyPlaceholder",y="tap";qx.Class.define(b,{extend:qx.ui.form.core.AbstractVirtualBox,implement:[qx.ui.form.IStringForm],construct:function(z){qx.ui.form.core.AbstractVirtualBox.call(this,z);var A=this._createChildControl(r);this._createChildControl(d);var B=this.getChildControl(s);B.getChildControl(j).setSelectionMode(t);this.__gx=B.getSelection();this.__gx.addListener(k,this.__uX,this);this.bind(n,A,n);A.bind(n,this,n);this.addListener(u,function(e){A.fireNonBubblingEvent(u,qx.event.type.Focus);},this);this.addListener(w,function(e){A.fireNonBubblingEvent(w,qx.event.type.Focus);},this);},properties:{appearance:{refine:true,init:m},width:{refine:true,init:120},value:{nullable:true,event:c},placeholder:{check:p,nullable:true,apply:x},defaultFormat:{check:g,init:null,nullable:true}},members:{__uV:null,__uW:null,__gx:null,__uq:null,getTextSelection:function(){return this.getChildControl(r).getTextSelection();},getTextSelectionLength:function(){return this.getChildControl(r).getTextSelectionLength();},setTextSelection:function(C,D){this.getChildControl(r).setTextSelection(C,D);},clearTextSelection:function(){this.getChildControl(r).clearTextSelection();},selectAllText:function(){this.getChildControl(r).selectAllText();},resetAllTextSelection:function(){this.clearTextSelection();this.selectAllText();},tabFocus:function(){var E=this.getChildControl(r);E.getFocusElement().focus();E.selectAllText();},focus:function(){qx.ui.form.core.AbstractVirtualBox.prototype.focus.call(this);this.getChildControl(r).getFocusElement().focus();},_createChildControlImpl:function(H,G){var F;switch(H){case r:F=new qx.ui.form.TextField();F.setFocusable(false);F.addState(h);this._add(F,{flex:1});break;case d:F=new qx.ui.form.Button();F.setFocusable(false);F.setKeepActive(true);F.addState(h);F.addListener(f,this.toggle,this);this._add(F);break;};return F||qx.ui.form.core.AbstractVirtualBox.prototype._createChildControlImpl.call(this,H,G);},_beforeOpen:function(){this.__uY();},_handleKeyboard:function(event){var I=this._getAction(event);switch(I){case o:this.setValue(this.getChildControl(r).getValue());break;default:qx.ui.form.core.AbstractVirtualBox.prototype._handleKeyboard.call(this,event);break;};},_getAction:function(event){var J=event.getKeyIdentifier();var L=this.getChildControl(s).isVisible();var K=this._isModifierPressed(event);if(!L&&!K&&J===a){return o;}else {return qx.ui.form.core.AbstractVirtualBox.prototype._getAction.call(this,event);};},_handlePointer:function(event){qx.ui.form.core.AbstractVirtualBox.prototype._handlePointer.call(this,event);var M=event.getType();if(M!==y){return;};this.close();},__uX:function(event){if(this.__uq==true){return;};var N=this.__gx.getItem(0);if(N){N=this.__bq(N);this.setValue(N);};},_applyPlaceholder:function(P,O){this.getChildControl(r).setPlaceholder(P);},__uY:function(){var S=this.getValue();var R=this.getChildControl(s);var Q=R.getSelection();if(this.__bq(Q.getItem(0))!==S){this.__uq=true;Q.removeAll();this.__uq=false;if(S==null||S==q){return;};var W=this.getModel();var V=R.getChildControl(j)._getLookupTable();for(var i=0,l=V.length;i<l;i++ ){var T=W.getItem(V[i]);var U=this.__bq(T);if(U&&U.indexOf(S)==0){R.setPreselected(T);break;};};};},__bq:function(ba){var Y=this.getLabelOptions();var bc=this.getDefaultFormat();var bd=this.getLabelPath();var bb=null;if(bd!=null){bb=qx.data.SingleValueBinding.resolvePropertyChain(ba,bd);}else if(qx.lang.Type.isString(ba)){bb=ba;};var X=qx.util.Delegate.getMethod(Y,v);if(X!=null){bb=X(bb);};if(bb!=null&&bc!=null){bb=bc(qx.lang.String.stripTags(bb));};return bb;}},destruct:function(){var be=this.getChildControl(r);this.removeAllBindings();be.removeAllBindings();this.__gx.removeListener(k,this.__uX,this);this.__gx=null;}});})();(function(){var a="hovered",b="Enter",c="pressed",d="qx.data.Array",e="pointerover",f="Space",g="abandoned",h="qx.event.type.Data",j="open",k="list",l="interval",m="qx.ui.form.VirtualSelectBox",n="arrow",o="changeSelection",p="selection",q="search",r="_applySelection",s="",t="icon",u="spacer",v="dropdown",w="tap",x="converter",y="model",z="label",A=" ",B="virtual-selectbox",C="pointerout",D="atom";qx.Class.define(m,{extend:qx.ui.form.core.AbstractVirtualBox,implement:qx.data.controller.ISelection,construct:function(E){qx.ui.form.core.AbstractVirtualBox.call(this,E);this._createChildControl(D);this._createChildControl(u);this._createChildControl(n);this.addListener(e,this._onPointerOver,this);this.addListener(C,this._onPointerOut,this);this.__bc=[];this.initSelection(this.getChildControl(v).getSelection());this.__va=new qx.event.Timer(500);this.__va.addListener(l,this.__vc,this);},properties:{appearance:{refine:true,init:B},width:{refine:true,init:120},selection:{check:d,event:o,apply:r,nullable:false,deferredInit:true}},events:{"changeSelection":h},members:{__vb:s,__va:null,__bc:null,syncWidget:function(F){this._removeBindings();this._addBindings();},_createChildControlImpl:function(I,H){var G;switch(I){case u:G=new qx.ui.core.Spacer();this._add(G,{flex:1});break;case D:G=new qx.ui.form.ListItem(s);G.setCenter(false);G.setAnonymous(true);this._add(G,{flex:1});break;case n:G=new qx.ui.basic.Image();G.setAnonymous(true);this._add(G);break;};return G||qx.ui.form.core.AbstractVirtualBox.prototype._createChildControlImpl.call(this,I,H);},_getAction:function(event){var J=event.getKeyIdentifier();var L=this.getChildControl(v).isVisible();var K=this._isModifierPressed(event);if(!L&&!K&&(J===b||J===f)){return j;}else if(L&&event.isPrintable()){return q;}else {return qx.ui.form.core.AbstractVirtualBox.prototype._getAction.call(this,event);};},_addBindings:function(){var N=this.getChildControl(D);var P=this._getBindPath(p,s);var M=this.bind(P,N,y,null);this.__bc.push(M);var O=this._getBindPath(p,this.getLabelPath());M=this.bind(O,N,z,this.getLabelOptions());this.__bc.push(M);if(this.getIconPath()!=null){var Q=this._getBindPath(p,this.getIconPath());M=this.bind(Q,N,t,this.getIconOptions());this.__bc.push(M);};},_removeBindings:function(){while(this.__bc.length>0){var R=this.__bc.pop();this.removeBinding(R);};},_handlePointer:function(event){qx.ui.form.core.AbstractVirtualBox.prototype._handlePointer.call(this,event);var S=event.getType();if(S===w){this.toggle();};},_handleKeyboard:function(event){var T=this._getAction(event);switch(T){case q:this.__vb+=this.__vd(event.getKeyIdentifier());this.__va.restart();break;default:qx.ui.form.core.AbstractVirtualBox.prototype._handleKeyboard.call(this,event);break;};},_onPointerOver:function(event){if(!this.isEnabled()||event.getTarget()!==this){return;};if(this.hasState(g)){this.removeState(g);this.addState(c);};this.addState(a);},_onPointerOut:function(event){if(!this.isEnabled()||event.getTarget()!==this){return;};this.removeState(a);if(this.hasState(c)){this.removeState(c);this.addState(g);};},_applySelection:function(V,U){this.getChildControl(v).setSelection(V);qx.ui.core.queue.Widget.add(this);},__vc:function(){this.__va.stop();var Y=this.__vb;if(Y===null||Y===s){return;};var bg=this.getModel();var bb=this.getChildControl(v).getChildControl(k);var W=bb.getSelection();var length=bb._getLookupTable().length;var bf=bg.indexOf(W.getItem(0));var bd=bb._reverseLookup(bf);for(var i=1;i<=length;i++ ){var bh=(i+bd)%length;var be=bg.getItem(bb._lookup(bh));if(!be){continue;};var bc=be;if(this.getLabelPath()){bc=qx.data.SingleValueBinding.resolvePropertyChain(be,this.getLabelPath());var ba=this.getLabelOptions();if(ba){var X=qx.util.Delegate.getMethod(ba,x);if(X){bc=X(bc,be);};};};if(qx.lang.String.startsWith(bc.toLowerCase(),Y.toLowerCase())){W.push(be);break;};};this.__vb=s;},__vd:function(bi){if(bi===f){return A;}else {return bi;};}},destruct:function(){this._removeBindings();this.__va.removeListener(l,this.__vc,this);this.__va.dispose();this.__va=null;}});})();(function(){var a="qx.ui.form.renderer.IFormRenderer";qx.Interface.define(a,{members:{addItems:function(b,f,d,c,e){},addButton:function(g,h){}}});})();(function(){var a=" <span style='color:red'>*</span> ",b="Abstract method call",c="abstract",d="",f="changeLocale",g="change",h="visibility",j="qx.dynlocale",k="qx.ui.form.renderer.AbstractRenderer",l=" :";qx.Class.define(k,{type:c,extend:qx.ui.core.Widget,implement:qx.ui.form.renderer.IFormRenderer,construct:function(m){qx.ui.core.Widget.call(this);this._labels=[];if(qx.core.Environment.get(j)){qx.locale.Manager.getInstance().addListener(f,this._onChangeLocale,this);this._names=[];};this._form=m;this._render();m.addListener(g,this._onFormChange,this);},members:{_names:null,_form:null,_labels:null,_render:function(){var q=this._form.getGroups();for(var i=0;i<q.length;i++ ){var p=q[i];this.addItems(p.items,p.labels,p.title,p.options,p.headerOptions);};var n=this._form.getButtons();var o=this._form.getButtonOptions();for(var i=0;i<n.length;i++ ){this.addButton(n[i],o[i]);};},_onFormChange:function(){this._removeAll();for(var i=0;i<this._labels.length;i++ ){this._labels[i].dispose();};this._labels=[];this._render();},_connectVisibility:function(r,s){r.bind(h,s,h);},_onChangeLocale:qx.core.Environment.select(j,{"true":function(e){for(var i=0;i<this._names.length;i++ ){var u=this._names[i];if(u.name&&u.name.translate){u.name=u.name.translate();};var t=this._createLabelText(u.name,u.item);u.label.setValue(t);};},"false":null}),_createLabelText:function(name,v){var w=d;if(v.getRequired()){w=a;};var x=name.length>0||v.getRequired()?l:d;return name+w+x;},addItems:function(y,z,A){throw new Error(b);},addButton:function(B){throw new Error(b);}},destruct:function(){if(qx.core.Environment.get(j)){qx.locale.Manager.getInstance().removeListener(f,this._onChangeLocale,this);};this._names=null;}});})();(function(){var a="bold",b="_buttonRow",c="qx.ui.form.renderer.Double",d="qx.dynlocale",e="top",f="left",g="right";qx.Class.define(c,{extend:qx.ui.form.renderer.AbstractRenderer,construct:function(j){var h=new qx.ui.layout.Grid();h.setSpacing(6);h.setColumnAlign(0,g,e);h.setColumnAlign(1,f,e);h.setColumnAlign(2,g,e);h.setColumnAlign(3,f,e);this._setLayout(h);qx.ui.form.renderer.AbstractRenderer.call(this,j);},members:{_row:0,_buttonRow:null,_onFormChange:function(){this._buttonRow.destroy();this._buttonRow=null;this._row=0;qx.ui.form.renderer.AbstractRenderer.prototype._onFormChange.call(this);},addItems:function(l,o,k){if(k!=null){this._add(this._createHeader(k),{row:this._row,column:0,colSpan:4});this._row++ ;};for(var i=0;i<l.length;i++ ){var m=this._createLabel(o[i],l[i]);this._add(m,{row:this._row,column:(i*2)%4});var n=l[i];m.setBuddy(n);this._connectVisibility(n,m);this._add(n,{row:this._row,column:((i*2)%4)+1});if(i%2==1){this._row++ ;};if(qx.core.Environment.get(d)){this._names.push({name:o[i],label:m,item:l[i]});};};if(i%2==1){this._row++ ;};},addButton:function(p){if(this._buttonRow==null){this._buttonRow=new qx.ui.container.Composite();this._buttonRow.setMarginTop(5);var q=new qx.ui.layout.HBox();q.setAlignX(g);q.setSpacing(5);this._buttonRow.setLayout(q);this._add(this._buttonRow,{row:this._row,column:0,colSpan:4});this._row++ ;};this._buttonRow.add(p);},getLayout:function(){return this._getLayout();},_createLabel:function(name,r){var s=new qx.ui.basic.Label(this._createLabelText(name,r));this._labels.push(s);s.setRich(true);return s;},_createHeader:function(u){var t=new qx.ui.basic.Label(u);this._labels.push(t);t.setFont(a);if(this._row!=0){t.setMarginTop(10);};t.setAlignX(f);return t;}},destruct:function(){if(this._buttonRow){this._buttonRow.removeAll();this._disposeObjects(b);};}});})();(function(){var a="bold",b="_buttonRow",c="qx.ui.form.renderer.Single",d="form-renderer-label",e="qx.dynlocale",f="top",g="left",h="right";qx.Class.define(c,{extend:qx.ui.form.renderer.AbstractRenderer,construct:function(k){var j=new qx.ui.layout.Grid();j.setSpacing(6);j.setColumnFlex(0,1);j.setColumnAlign(0,h,f);this._setLayout(j);qx.ui.form.renderer.AbstractRenderer.call(this,k);},members:{_row:0,_buttonRow:null,_onFormChange:function(){if(this._buttonRow){this._buttonRow.destroy();this._buttonRow=null;};this._row=0;qx.ui.form.renderer.AbstractRenderer.prototype._onFormChange.call(this);},addItems:function(m,p,l){if(l!=null){this._add(this._createHeader(l),{row:this._row,column:0,colSpan:2});this._row++ ;};for(var i=0;i<m.length;i++ ){var n=this._createLabel(p[i],m[i]);this._add(n,{row:this._row,column:0});var o=m[i];n.setBuddy(o);this._add(o,{row:this._row,column:1});this._row++ ;this._connectVisibility(o,n);if(qx.core.Environment.get(e)){this._names.push({name:p[i],label:n,item:m[i]});};};},addButton:function(q){if(this._buttonRow==null){this._buttonRow=new qx.ui.container.Composite();this._buttonRow.setMarginTop(5);var r=new qx.ui.layout.HBox();r.setAlignX(h);r.setSpacing(5);this._buttonRow.setLayout(r);this._add(this._buttonRow,{row:this._row,column:0,colSpan:2});this._row++ ;};this._buttonRow.add(q);},getLayout:function(){return this._getLayout();},_createLabel:function(name,s){var t=new qx.ui.basic.Label(this._createLabelText(name,s));this._labels.push(t);t.setRich(true);t.setAppearance(d);return t;},_createHeader:function(v){var u=new qx.ui.basic.Label(v);this._labels.push(u);u.setFont(a);if(this._row!=0){u.setMarginTop(10);};u.setAlignX(g);return u;}},destruct:function(){if(this._buttonRow){this._buttonRow.removeAll();this._disposeObjects(b);};}});})();(function(){var a="Only widgets with placeholders supported.",b="qx.ui.form.renderer.SinglePlaceholder";qx.Class.define(b,{extend:qx.ui.form.renderer.Single,implement:qx.ui.form.renderer.IFormRenderer,members:{addItems:function(c,d,e){if(e!=null){this._add(this._createHeader(e),{row:this._row,column:0,colSpan:2});this._row++ ;};for(var i=0;i<c.length;i++ ){if(c[i].setPlaceholder===undefined){throw new Error(a);};c[i].setPlaceholder(d[i]);this._add(c[i],{row:this._row,column:0});this._row++ ;};}}});})();(function(){var a="qx.ui.layout.Basic";qx.Class.define(a,{extend:qx.ui.layout.Abstract,members:{verifyLayoutProperty:null,renderLayout:function(e,b,d){var c=this._getLayoutChildren();var f,g,h,j,top;for(var i=0,l=c.length;i<l;i++ ){f=c[i];g=f.getSizeHint();h=f.getLayoutProperties();j=d.left+(h.left||0)+f.getMarginLeft();top=d.top+(h.top||0)+f.getMarginTop();f.renderLayout(j,top,g.width,g.height);};},_computeSizeHint:function(){var q=this._getLayoutChildren();var n,s,o;var r=0,p=0;var k,m;for(var i=0,l=q.length;i<l;i++ ){n=q[i];s=n.getSizeHint();o=n.getLayoutProperties();k=s.width+(o.left||0)+n.getMarginLeft()+n.getMarginRight();m=s.height+(o.top||0)+n.getMarginTop()+n.getMarginBottom();if(k>r){r=k;};if(m>p){p=m;};};return {width:r,height:p};}}});})();(function(){var a="qx.ui.menubar.MenuBar",b="menubar";qx.Class.define(a,{extend:qx.ui.toolbar.ToolBar,properties:{appearance:{refine:true,init:b}}});})();(function(){var a="slider",b="splitter",c="px",d="Integer",f="height",g="move",h="maxHeight",i="resize",j="pointerup",k="vertical",l="width",m="_applyOrientation",n="_applyOffset",o="splitpane",p="qx.ui.splitpane.Pane",q="pointerdown",r="top",s="minHeight",t="knob",u="horizontal",v="minWidth",w="appear",z="losecapture",A="left",B="pointermove",C="maxWidth",D="pointerout";qx.Class.define(p,{extend:qx.ui.core.Widget,construct:function(E){qx.ui.core.Widget.call(this);this.__gP=[];if(E){this.setOrientation(E);}else {this.initOrientation();};this.__lO.addListener(q,this._onPointerDown,this);this.__lO.addListener(j,this._onPointerUp,this);this.__lO.addListener(B,this._onPointerMove,this);this.__lO.addListener(D,this._onPointerOut,this);this.__lO.addListener(z,this._onPointerUp,this);},properties:{appearance:{refine:true,init:o},offset:{check:d,init:6,apply:n},orientation:{init:u,check:[u,k],apply:m}},members:{__ve:null,__vf:false,__vg:null,__vh:null,__nY:null,__vi:null,__vj:null,__gP:null,__lO:null,_createChildControlImpl:function(H,G){var F;switch(H){case a:F=new qx.ui.splitpane.Slider(this);F.exclude();this._add(F,{type:H});break;case b:F=new qx.ui.splitpane.Splitter(this);this._add(F,{type:H});F.addListener(g,this.__vk,this);break;};return F||qx.ui.core.Widget.prototype._createChildControlImpl.call(this,H);},__vk:function(e){this.__vm(e.getData());},__vl:function(K){this.__lO=new qx.ui.splitpane.Blocker(K);this.getContentElement().add(this.__lO);var I=this.getChildControl(b);var J=I.getWidth();if(!J){I.addListenerOnce(w,function(){this.__vm();},this);};I.addListener(i,function(e){var L=e.getData();if(L.height==0||L.width==0){this.__lO.hide();}else {this.__lO.show();};},this);},getBlocker:function(){return this.__lO;},_applyOrientation:function(Q,N){var R=this.getChildControl(a);var M=this.getChildControl(b);this.__nY=Q===u;if(!this.__lO){this.__vl(Q);};this.__lO.setOrientation(Q);var P=this._getLayout();if(P){P.dispose();};var O=Q===k?new qx.ui.splitpane.VLayout:new qx.ui.splitpane.HLayout;this._setLayout(O);M.removeState(N);M.addState(Q);M.getChildControl(t).removeState(N);M.getChildControl(t).addState(Q);R.removeState(N);R.addState(Q);qx.ui.core.queue.Manager.flush();this.__vm();},_applyOffset:function(T,S){this.__vm();},__vm:function(V){var U=this.getChildControl(b);var ba=this.getOffset();var bb=U.getBounds();var Y=U.getContentElement().getDomElement();if(!Y){return;};if(this.__nY){var X=null;if(V){X=V.width;}else if(bb){X=bb.width;};var bc=V&&V.left;if(X){if(isNaN(bc)){bc=qx.bom.element.Location.getPosition(Y).left;};this.__lO.setWidth(ba,X);this.__lO.setLeft(ba,bc);};}else {var W=null;if(V){W=V.height;}else if(bb){W=bb.height;};var top=V&&V.top;if(W){if(isNaN(top)){top=qx.bom.element.Location.getPosition(Y).top;};this.__lO.setHeight(ba,W);this.__lO.setTop(ba,top);};};},add:function(be,bd){if(bd==null){this._add(be);}else {this._add(be,{flex:bd});};this.__gP.push(be);},remove:function(bf){this._remove(bf);qx.lang.Array.remove(this.__gP,bf);},getChildren:function(){return this.__gP;},_onPointerDown:function(e){if(!e.isLeftPressed()){return;};var bg=this.getChildControl(b);var bj=bg.getContentLocation();var bh=this.getContentLocation();this.__ve=this.__nY?e.getDocumentLeft()-bj.left+bh.left:e.getDocumentTop()-bj.top+bh.top;var bi=this.getChildControl(a);var bk=bg.getBounds();bi.setUserBounds(bk.left,bk.top,bk.width,bk.height);bi.setZIndex(bg.getZIndex()+1);bi.show();this.__vf=true;this.__lO.capture();e.stop();},_onPointerMove:function(e){this._setLastPointerPosition(e.getDocumentLeft(),e.getDocumentTop());if(this.__vf){this.__vn();var bl=this.getChildControl(a);var bm=this.__vi;if(this.__nY){bl.setDomLeft(bm);this.__lO.setStyle(A,(bm-this.getOffset())+c);}else {bl.setDomTop(bm);this.__lO.setStyle(r,(bm-this.getOffset())+c);};e.stop();};},_onPointerOut:function(e){this._setLastPointerPosition(e.getDocumentLeft(),e.getDocumentTop());},_onPointerUp:function(e){if(!this.__vf){return;};this._finalizeSizes();var bn=this.getChildControl(a);bn.exclude();this.__vf=false;this.releaseCapture();e.stop();},_finalizeSizes:function(){var br=this.__vi;var bo=this.__vj;if(br==null){return;};var bt=this._getChildren();var bs=bt[2];var bq=bt[3];var bp=bs.getLayoutProperties().flex;var bu=bq.getLayoutProperties().flex;if((bp!=0)&&(bu!=0)){bs.setLayoutProperties({flex:br});bq.setLayoutProperties({flex:bo});}else {if(this.__nY){bs.setWidth(br);bq.setWidth(bo);}else {bs.setHeight(br);bq.setHeight(bo);};};},__vn:function(){if(this.__nY){var bx=v,bE=l,by=C,bC=this.__vg;}else {var bx=s,bE=f,by=h,bC=this.__vh;};var bD=this._getChildren();var bv=bD[2].getSizeHint();var bA=bD[3].getSizeHint();var bB=bD[2].getBounds()[bE]+bD[3].getBounds()[bE];var bz=bC-this.__ve;var bw=bB-bz;if(bz<bv[bx]){bw-=bv[bx]-bz;bz=bv[bx];}else if(bw<bA[bx]){bz-=bA[bx]-bw;bw=bA[bx];};if(bz>bv[by]){bw+=bz-bv[by];bz=bv[by];}else if(bw>bA[by]){bz+=bw-bA[by];bw=bA[by];};this.__vi=bz;this.__vj=bw;},_isActiveDragSession:function(){return this.__vf;},_setLastPointerPosition:function(x,y){this.__vg=x;this.__vh=y;}},destruct:function(){this.__gP=null;}});})();(function(){var a="qx.ui.splitpane.Slider";qx.Class.define(a,{extend:qx.ui.core.Widget,properties:{allowShrinkX:{refine:true,init:false},allowShrinkY:{refine:true,init:false}}});})();(function(){var a="center",b="qx.ui.splitpane.Splitter",c="vertical",d="knob",e="middle";qx.Class.define(b,{extend:qx.ui.core.Widget,construct:function(f){qx.ui.core.Widget.call(this);if(f.getOrientation()==c){this._setLayout(new qx.ui.layout.HBox(0,a));this._getLayout().setAlignY(e);}else {this._setLayout(new qx.ui.layout.VBox(0,e));this._getLayout().setAlignX(a);};this._createChildControl(d);},properties:{allowShrinkX:{refine:true,init:false},allowShrinkY:{refine:true,init:false}},members:{_createChildControlImpl:function(i,h){var g;switch(i){case d:g=new qx.ui.basic.Image;this._add(g);break;};return g||qx.ui.core.Widget.prototype._createChildControlImpl.call(this,i);}}});})();(function(){var a="mshtml",b="engine.name",c="repeat",d="horizontal",e="top",f="height",g="_applyOrientation",h="url(",i="qx.ui.splitpane.Blocker",j="width",k=")",l="col-resize",m="row-resize",n="px",o="100%",p="div",q="left",r="vertical",s="qx/static/blank.gif",t="cursor",u="absolute";qx.Class.define(i,{extend:qx.html.Element,construct:function(w){var v={position:u,zIndex:11};if((qx.core.Environment.get(b)==a)){v.backgroundImage=h+qx.util.ResourceManager.getInstance().toUri(s)+k;v.backgroundRepeat=c;};qx.html.Element.call(this,p,v);if(w){this.setOrientation(w);}else {this.initOrientation();};},properties:{orientation:{init:d,check:[d,r],apply:g}},members:{_applyOrientation:function(y,x){if(y==d){this.setStyle(f,o);this.setStyle(t,l);this.setStyle(e,null);}else {this.setStyle(j,o);this.setStyle(q,null);this.setStyle(t,m);};},setWidth:function(A,z){var B=z+2*A;this.setStyle(j,B+n);},setHeight:function(D,C){var E=C+2*D;this.setStyle(f,E+n);},setLeft:function(G,F){var H=F-G;this.setStyle(q,H+n);},setTop:function(J,I){var top=I-J;this.setStyle(e,top+n);}}});})();(function(){var a="splitter",b="qx.ui.splitpane.VLayout",c="slider";qx.Class.define(b,{extend:qx.ui.layout.Abstract,members:{verifyLayoutProperty:null,renderLayout:function(t,l,s){var j=this._getLayoutChildren();var length=j.length;var u,o;var p,w,v,k;var y=s.left||0;var m=s.top||0;for(var i=0;i<length;i++ ){u=j[i];o=u.getLayoutProperties().type;if(o===a){w=u;}else if(o===c){v=u;}else if(!p){p=u;}else {k=u;};};if(p&&k){var A=p.getLayoutProperties().flex;var g=k.getLayoutProperties().flex;if(A==null){A=1;};if(g==null){g=1;};var d=p.getSizeHint();var e=w.getSizeHint();var n=k.getSizeHint();var r=d.height;var h=e.height;var z=n.height;if(A>0&&g>0){var f=A+g;var q=l-h;var r=Math.round((q/f)*A);var z=q-r;var x=qx.ui.layout.Util.arrangeIdeals(d.minHeight,r,d.maxHeight,n.minHeight,z,n.maxHeight);r=x.begin;z=x.end;}else if(A>0){r=l-h-z;if(r<d.minHeight){r=d.minHeight;};if(r>d.maxHeight){r=d.maxHeight;};}else if(g>0){z=l-r-h;if(z<n.minHeight){z=n.minHeight;};if(z>n.maxHeight){z=n.maxHeight;};};p.renderLayout(y,m,t,r);w.renderLayout(y,r+m,t,h);k.renderLayout(y,r+h+m,t,z);}else {w.renderLayout(0,0,0,0);if(p){p.renderLayout(y,m,t,l);}else if(k){k.renderLayout(y,m,t,l);};};},_computeSizeHint:function(){var K=this._getLayoutChildren();var length=K.length;var J,C,D;var E=0,G=0,F=0;var H=0,I=0,B=0;for(var i=0;i<length;i++ ){J=K[i];D=J.getLayoutProperties();if(D.type===c){continue;};C=J.getSizeHint();E+=C.minHeight;G+=C.height;F+=C.maxHeight;if(C.minWidth>H){H=C.minWidth;};if(C.width>I){I=C.width;};if(C.maxWidth>B){B=C.maxWidth;};};return {minHeight:E,height:G,maxHeight:F,minWidth:H,width:I,maxWidth:B};}}});})();(function(){var a="splitter",b="qx.ui.splitpane.HLayout",c="slider";qx.Class.define(b,{extend:qx.ui.layout.Abstract,members:{verifyLayoutProperty:null,renderLayout:function(s,k,r){var h=this._getLayoutChildren();var length=h.length;var t,n;var o,v,u,j;var x=r.left||0;var l=r.top||0;for(var i=0;i<length;i++ ){t=h[i];n=t.getLayoutProperties().type;if(n===a){v=t;}else if(n===c){u=t;}else if(!o){o=t;}else {j=t;};};if(o&&j){var A=o.getLayoutProperties().flex;var g=j.getLayoutProperties().flex;if(A==null){A=1;};if(g==null){g=1;};var d=o.getSizeHint();var e=v.getSizeHint();var m=j.getSizeHint();var q=d.width;var z=e.width;var y=m.width;if(A>0&&g>0){var f=A+g;var p=s-z;var q=Math.round((p/f)*A);var y=p-q;var w=qx.ui.layout.Util.arrangeIdeals(d.minWidth,q,d.maxWidth,m.minWidth,y,m.maxWidth);q=w.begin;y=w.end;}else if(A>0){q=s-z-y;if(q<d.minWidth){q=d.minWidth;};if(q>d.maxWidth){q=d.maxWidth;};}else if(g>0){y=s-q-z;if(y<m.minWidth){y=m.minWidth;};if(y>m.maxWidth){y=m.maxWidth;};};o.renderLayout(x,l,q,k);v.renderLayout(q+x,l,z,k);j.renderLayout(q+z+x,l,y,k);}else {v.renderLayout(0,0,0,0);if(o){o.renderLayout(x,l,s,k);}else if(j){j.renderLayout(x,l,s,k);};};},_computeSizeHint:function(){var K=this._getLayoutChildren();var length=K.length;var J,C,D;var H=0,I=0,B=0;var E=0,G=0,F=0;for(var i=0;i<length;i++ ){J=K[i];D=J.getLayoutProperties();if(D.type===c){continue;};C=J.getSizeHint();H+=C.minWidth;I+=C.width;B+=C.maxWidth;if(C.minHeight>E){E=C.minHeight;};if(C.height>G){G=C.height;};if(C.maxHeight>F){F=C.maxHeight;};};return {minWidth:H,width:I,maxWidth:B,minHeight:E,height:G,maxHeight:F};}}});})();(function(){var a="_applyIcon",b="",c="icon",d="page",e="close",f="qx.event.type.Event",g="_applyShowCloseButton",h="tabview-page",i="qx.ui.tabview.Page",j="_applyLabel",k="button",l="String",m="Boolean";qx.Class.define(i,{extend:qx.ui.container.Composite,construct:function(n,o){qx.ui.container.Composite.call(this);this._createChildControl(k);if(n!=null){this.setLabel(n);};if(o!=null){this.setIcon(o);};},events:{"close":f},properties:{appearance:{refine:true,init:h},label:{check:l,init:b,apply:j},icon:{check:l,init:b,apply:a,nullable:true},showCloseButton:{check:m,init:false,apply:g}},members:{_forwardStates:{barTop:1,barRight:1,barBottom:1,barLeft:1,firstTab:1,lastTab:1},_applyIcon:function(r,q){var p=this.getChildControl(k);if(r){p.setIcon(r);p._showChildControl(c);}else {p._excludeChildControl(c);};},_applyLabel:function(t,s){this.getChildControl(k).setLabel(t);},_applyEnabled:function(w,v){qx.ui.container.Composite.prototype._applyEnabled.call(this,w,v);var u=this.getChildControl(k);w==null?u.resetEnabled():u.setEnabled(w);},_createChildControlImpl:function(z,y){var x;switch(z){case k:x=new qx.ui.tabview.TabButton;x.setAllowGrowX(true);x.setAllowGrowY(true);x.setUserData(d,this);x.addListener(e,this._onButtonClose,this);break;};return x||qx.ui.container.Composite.prototype._createChildControlImpl.call(this,z);},_applyShowCloseButton:function(B,A){this.getChildControl(k).setShowCloseButton(B);},_onButtonClose:function(){this.fireEvent(e);},getButton:function(){return this.getChildControl(k);}}});})();(function(){var a="Boolean",b="icon",c="label",d="bottom",e="qx.ui.tabview.TabButton",f="center",g="_applyShowCloseButton",h="close-button",i="top",j="left",k="tap",l="middle",m="close",n="right",o="qx.event.type.Data";qx.Class.define(e,{extend:qx.ui.form.RadioButton,implement:qx.ui.form.IRadioItem,construct:function(){qx.ui.form.RadioButton.call(this);var p=new qx.ui.layout.Grid(2,0);p.setRowAlign(0,j,l);p.setColumnAlign(0,n,l);this._getLayout().dispose();this._setLayout(p);this.initShowCloseButton();},events:{"close":o},properties:{showCloseButton:{check:a,init:false,apply:g}},members:{_forwardStates:{focused:true,checked:true},_applyIconPosition:function(s,q){var r={icon:this.getChildControl(b),label:this.getChildControl(c),closeButton:this.getShowCloseButton()?this.getChildControl(h):null};for(var t in r){if(r[t]){this._remove(r[t]);};};switch(s){case i:this._add(r.label,{row:3,column:2});this._add(r.icon,{row:1,column:2});if(r.closeButton){this._add(r.closeButton,{row:0,column:4});};break;case d:this._add(r.label,{row:1,column:2});this._add(r.icon,{row:3,column:2});if(r.closeButton){this._add(r.closeButton,{row:0,column:4});};break;case j:this._add(r.label,{row:0,column:2});this._add(r.icon,{row:0,column:0});if(r.closeButton){this._add(r.closeButton,{row:0,column:4});};break;case n:this._add(r.label,{row:0,column:0});this._add(r.icon,{row:0,column:2});if(r.closeButton){this._add(r.closeButton,{row:0,column:4});};break;};},_createChildControlImpl:function(w,v){var u;switch(w){case c:var u=new qx.ui.basic.Label(this.getLabel());u.setAnonymous(true);this._add(u,{row:0,column:2});this._getLayout().setColumnFlex(2,1);break;case b:u=new qx.ui.basic.Image(this.getIcon());u.setAnonymous(true);this._add(u,{row:0,column:0});break;case h:u=new qx.ui.form.Button();u.setFocusable(false);u.setKeepActive(true);u.addListener(k,this._onCloseButtonTap,this);this._add(u,{row:0,column:4});if(!this.getShowCloseButton()){u.exclude();};break;};return u||qx.ui.form.RadioButton.prototype._createChildControlImpl.call(this,w);},_onCloseButtonTap:function(){this.fireDataEvent(m,this);},_applyShowCloseButton:function(y,x){if(y){this._showChildControl(h);}else {this._excludeChildControl(h);};},_applyCenter:function(A){var z=this._getLayout();if(A){z.setColumnAlign(2,f,l);}else {z.setColumnAlign(2,j,l);};}}});})();(function(){var a="bottom",b="button",c="qx.event.type.Data",d="lastTab",f="barRight",g="beforeChangeSelection",h="close-button",j="changeSelection",k="bar",m="Index should be less than : ",n="tabview",o="vertical",p="page",q="close",r="_applyBarPosition",s="barLeft",t="__tv",u="top",v="horizontal",w="qx.ui.tabview.TabView",x="right",y="barTop",z="barBottom",A="pane",B="left",C="firstTab";qx.Class.define(w,{extend:qx.ui.core.Widget,implement:qx.ui.core.ISingleSelection,include:[qx.ui.core.MContentPadding],construct:function(E){qx.ui.core.Widget.call(this);this.__vo={top:y,right:f,bottom:z,left:s};this._createChildControl(k);this._createChildControl(A);var D=this.__tv=new qx.ui.form.RadioGroup;D.setWrap(false);D.addListener(j,this._onChangeSelection,this);if(E!=null){this.setBarPosition(E);}else {this.initBarPosition();};},events:{"changeSelection":c},properties:{appearance:{refine:true,init:n},barPosition:{check:[B,x,u,a],init:u,apply:r}},members:{__tv:null,_createChildControlImpl:function(H,G){var F;switch(H){case k:F=new qx.ui.container.SlideBar();F.setZIndex(10);this._add(F);break;case A:F=new qx.ui.container.Stack;F.setZIndex(5);this._add(F,{flex:1});break;};return F||qx.ui.core.Widget.prototype._createChildControlImpl.call(this,H);},_getContentPaddingTarget:function(){return this.getChildControl(A);},add:function(M){{};var I=M.getButton();var J=this.getChildControl(k);var L=this.getChildControl(A);M.exclude();J.add(I);L.add(M);this.__tv.add(I);M.addState(this.__vo[this.getBarPosition()]);M.addState(d);var K=this.getChildren();if(K[0]==M){M.addState(C);}else {K[K.length-2].removeState(d);};M.addListener(q,this._onPageClose,this);},addAt:function(Q,N){{};var S=this.getChildren();if(!(N==null)&&N>S.length){throw new Error(m+S.length);};if(N==null){N=S.length;};var P=Q.getButton();var O=this.getChildControl(k);var R=this.getChildControl(A);Q.exclude();O.addAt(P,N);R.addAt(Q,N);this.__tv.add(P);Q.addState(this.__vo[this.getBarPosition()]);S=this.getChildren();if(N==S.length-1){Q.addState(d);};if(S[0]==Q){Q.addState(C);}else {S[S.length-2].removeState(d);};Q.addListener(q,this._onPageClose,this);},remove:function(W){var X=this.getChildControl(A);var U=this.getChildControl(k);var V=W.getButton();var Y=X.getChildren();if(this.getSelection()[0]==W){var T=Y.indexOf(W);if(T==0){if(Y[1]){this.setSelection([Y[1]]);}else {this.resetSelection();};}else {this.setSelection([Y[T-1]]);};};U.remove(V);X.remove(W);this.__tv.remove(V);W.removeState(this.__vo[this.getBarPosition()]);if(W.hasState(C)){W.removeState(C);if(Y[0]){Y[0].addState(C);};};if(W.hasState(d)){W.removeState(d);if(Y.length>0){Y[Y.length-1].addState(d);};};W.removeListener(q,this._onPageClose,this);},getChildren:function(){return this.getChildControl(A).getChildren();},indexOf:function(ba){return this.getChildControl(A).indexOf(ba);},__vo:null,_applyBarPosition:function(be,bg){var bb=this.getChildControl(k);var bh=this.getChildControl(A);var bj=be==B||be==x;var bk=be==x||be==a;var bi=bj?qx.ui.layout.HBox:qx.ui.layout.VBox;var bl=this._getLayout();if(bl&&bl instanceof bi){}else {this._setLayout(bl=new bi);};bl.setReversed(bk);bb.setOrientation(bj?o:v);var bf=this.getChildren();var i,l;if(bg){var bd=this.__vo[bg];bb.removeState(bd);bh.removeState(bd);for(i=0,l=bf.length;i<l;i++ ){bf[i].removeState(bd);};};if(be){var bc=this.__vo[be];bb.addState(bc);bh.addState(bc);for(i=0,l=bf.length;i<l;i++ ){bf[i].addState(bc);};};},getSelection:function(){var bm=this.__tv.getSelection();var bn=[];for(var i=0;i<bm.length;i++ ){bn.push(bm[i].getUserData(p));};return bn;},setSelection:function(bp){var bo=[];for(var i=0;i<bp.length;i++ ){bo.push(bp[i].getChildControl(b));};this.__tv.setSelection(bo);},resetSelection:function(){this.__tv.resetSelection();},isSelected:function(bq){var br=bq.getChildControl(b);return this.__tv.isSelected(br);},isSelectionEmpty:function(){return this.__tv.isSelectionEmpty();},getSelectables:function(bt){var bs=this.__tv.getSelectables(bt);var bu=[];for(var i=0;i<bs.length;i++ ){bu.push(bs[i].getUserData(p));};return bu;},_onChangeSelection:function(e){var bz=this.getChildControl(A);var bw=e.getData()[0];var by=e.getOldData()[0];var bx=[];var bv=[];if(bw){bx=[bw.getUserData(p)];bz.setSelection(bx);bw.focus();this.scrollChildIntoView(bw,null,null,false);}else {bz.resetSelection();};if(by){bv=[by.getUserData(p)];};this.fireDataEvent(j,bx,bv);},_onBeforeChangeSelection:function(e){if(!this.fireNonBubblingEvent(g,qx.event.type.Event,[false,true])){e.preventDefault();};},_onRadioChangeSelection:function(e){var bA=e.getData()[0];if(bA){this.setSelection([bA.getUserData(p)]);}else {this.resetSelection();};},_onPageClose:function(e){var bC=e.getTarget();var bB=bC.getButton().getChildControl(h);bB.reset();this.remove(bC);}},destruct:function(){this._disposeObjects(t);this.__vo=null;}});})();(function(){var a="qx.ui.tree.selection.SelectionManager",b="above",c="under",d="right",e="left";qx.Class.define(a,{extend:qx.ui.core.selection.ScrollArea,members:{_getSelectableLocationY:function(f){var g=f.getBounds();if(g){var top=this._getWidget().getItemTop(f);return {top:top,bottom:top+g.height};};},_isSelectable:function(h){return this._isItemSelectable(h)&&h instanceof qx.ui.tree.core.AbstractTreeItem;},_getSelectableFromPointerEvent:function(event){return this._getWidget().getTreeItem(event.getTarget());},getSelectables:function(j){var k=false;if(!j){k=this._userInteraction;this._userInteraction=true;};var m=this._getWidget();var n=[];if(m.getRoot()!=null){var l=m.getRoot().getItems(true,!!j,m.getHideRoot());for(var i=0;i<l.length;i++ ){if(this._isSelectable(l[i])){n.push(l[i]);};};};this._userInteraction=k;return n;},_getSelectableRange:function(q,o){if(q===o){return [q];};var s=this.getSelectables();var p=s.indexOf(q);var r=s.indexOf(o);if(p<0||r<0){return [];};if(p<r){return s.slice(p,r+1);}else {return s.slice(r,p+1);};},_getFirstSelectable:function(){return this.getSelectables()[0]||null;},_getLastSelectable:function(){var t=this.getSelectables();if(t.length>0){return t[t.length-1];}else {return null;};},_getRelatedSelectable:function(u,w){var v=this._getWidget();var x=null;switch(w){case b:x=v.getPreviousNodeOf(u,false);break;case c:x=v.getNextNodeOf(u,false);break;case e:case d:break;};if(!x){return null;};if(this._isSelectable(x)){return x;}else {return this._getRelatedSelectable(x,w);};}}});})();(function(){var a="_applyIconOpened",b="changeOpen",c="Boolean",d="changeIcon",f="changeIconOpened",g="changeLabel",h="Integer",i="auto",j="Abstract method call.",k="_applyIndent",l="open",m="changeOpenSymbolMode",n="_applyOpenSymbolMode",o="middle",p="__vr",q="String",r="resize",s="",t="icon",u="opened",v="iconOpened",w="abstract",x="never",y="_applyIcon",z="always",A="_applyOpen",B="changeIndent",C="qx.ui.tree.core.AbstractItem",D="label",E="_applyLabel";qx.Class.define(C,{extend:qx.ui.core.Widget,type:w,include:[qx.ui.form.MModelProperty],implement:[qx.ui.form.IModel],construct:function(F){qx.ui.core.Widget.call(this);if(F!=null){this.setLabel(F);};this._setLayout(new qx.ui.layout.HBox());this._addWidgets();this.initOpen();},properties:{open:{check:c,init:false,event:b,apply:A},openSymbolMode:{check:[z,x,i],init:i,event:m,apply:n},indent:{check:h,init:19,apply:k,event:B,themeable:true},icon:{check:q,apply:y,event:d,nullable:true,themeable:true},iconOpened:{check:q,apply:a,event:f,nullable:true,themeable:true},label:{check:q,apply:E,event:g,init:s}},members:{__vp:null,__vq:null,__vr:null,_addWidgets:function(){throw new Error(j);},_createChildControlImpl:function(I,H){var G;switch(I){case D:G=new qx.ui.basic.Label().set({alignY:o,anonymous:true,value:this.getLabel()});break;case t:G=new qx.ui.basic.Image().set({alignY:o,anonymous:true,source:this.getIcon()});break;case l:G=new qx.ui.tree.core.FolderOpenButton().set({alignY:o});G.addListener(b,this._onChangeOpen,this);G.addListener(r,this._updateIndent,this);break;};return G||qx.ui.core.Widget.prototype._createChildControlImpl.call(this,I);},addWidget:function(J,K){this._add(J,K);},addSpacer:function(){if(!this.__vr){this.__vr=new qx.ui.core.Spacer();}else {this._remove(this.__vr);};this._add(this.__vr);},addOpenButton:function(){this._add(this.getChildControl(l));},_onChangeOpen:function(e){if(this.isOpenable()){this.setOpen(e.getData());};},addIcon:function(){var L=this.getChildControl(t);if(this.__vq){this._remove(L);};this._add(L);this.__vq=true;},addLabel:function(M){var N=this.getChildControl(D);if(this.__vp){this._remove(N);};if(M){this.setLabel(M);}else {N.setValue(this.getLabel());};this._add(N);this.__vp=true;},_applyIcon:function(P,O){if(!this.__vt()){this.__vu(P);}else if(!this.isOpen()){this.__vu(P);};},_applyIconOpened:function(R,Q){if(this.isOpen()){if(this.__vs()&&this.__vt()){this.__vu(R);}else if(!this.__vs()&&this.__vt()){this.__vu(R);};};},_applyLabel:function(T,S){var U=this.getChildControl(D,true);if(U){U.setValue(T);};},_applyOpen:function(W,V){var open=this.getChildControl(l,true);if(open){open.setOpen(W);};var X;if(W){X=this.__vt()?this.getIconOpened():null;}else {X=this.getIcon();};if(X){this.__vu(X);};W?this.addState(u):this.removeState(u);},__vs:function(){return qx.util.PropertyUtil.getUserValue(this,t);},__vt:function(){return qx.util.PropertyUtil.getUserValue(this,v);},__vu:function(Y){var ba=this.getChildControl(t,true);if(ba){ba.setSource(Y);};},isOpenable:function(){var bb=this.getOpenSymbolMode();return (bb===z||bb===i&&this.hasChildren());},_shouldShowOpenSymbol:function(){throw new Error(j);},_applyOpenSymbolMode:function(bd,bc){this._updateIndent();},_updateIndent:function(){var bf=0;var open=this.getChildControl(l,true);if(open){if(this._shouldShowOpenSymbol()){open.show();var be=open.getBounds();if(be){bf=be.width;}else {return;};}else {open.exclude();};};if(this.__vr){this.__vr.setWidth((this.getLevel()+1)*this.getIndent()-bf);};},_applyIndent:function(bh,bg){this._updateIndent();},getLevel:function(){throw new Error(j);},syncWidget:function(bi){this._updateIndent();},hasChildren:function(){throw new Error(j);}},destruct:function(){this._disposeObjects(p);}});})();(function(){var a="pointerup",b="qx.ui.tree.core.FolderOpenButton",c="Boolean",d="opened",f="_applyOpen",g="pointerdown",h="tap",i="changeOpen";qx.Class.define(b,{extend:qx.ui.basic.Image,include:qx.ui.core.MExecutable,construct:function(){qx.ui.basic.Image.call(this);this.initOpen();this.addListener(h,this._onTap);this.addListener(g,this._stopPropagation,this);this.addListener(a,this._stopPropagation,this);},properties:{open:{check:c,init:false,event:i,apply:f}},members:{_applyOpen:function(k,j){k?this.addState(d):this.removeState(d);this.execute();},_stopPropagation:function(e){e.stopPropagation();},_onTap:function(e){this.toggleOpen();e.stopPropagation();}}});})();(function(){var a="open",b="abstract",c="__gP",d="'. It is not a child of this tree item.",e="addItem",f="visible",g="excluded",h="removeItem",j="Cannot remove treeitem '",k="__vv",m="qx.ui.tree.core.AbstractTreeItem";qx.Class.define(m,{extend:qx.ui.tree.core.AbstractItem,type:b,construct:function(n){qx.ui.tree.core.AbstractItem.call(this,n);this.__gP=[];},properties:{parent:{check:m,nullable:true}},members:{__gP:null,__vv:null,getTree:function(){var p=this;while(p.getParent()){p=p.getParent();};var o=p.getLayoutParent()?p.getLayoutParent().getLayoutParent():0;if(o&&o instanceof qx.ui.core.scroll.ScrollPane){return o.getLayoutParent();};return null;},_applyOpen:function(r,q){if(this.hasChildren()){this.getChildrenContainer().setVisibility(r?f:g);};qx.ui.tree.core.AbstractItem.prototype._applyOpen.call(this,r,q);},_shouldShowOpenSymbol:function(){var open=this.getChildControl(a,true);if(!open){return false;};var s=this.getTree();if(!s.getRootOpenClose()){if(s.getHideRoot()){if(s.getRoot()==this.getParent()){return false;};}else {if(s.getRoot()==this){return false;};};};return this.isOpenable();},_updateIndent:function(){if(!this.getTree()){return;};qx.ui.tree.core.AbstractItem.prototype._updateIndent.call(this);},getLevel:function(){var t=this.getTree();if(!t){return;};var u=this;var v=-1;while(u){u=u.getParent();v+=1;};if(t.getHideRoot()){v-=1;};if(!t.getRootOpenClose()){v-=1;};return v;},addState:function(w){qx.ui.tree.core.AbstractItem.prototype.addState.call(this,w);var x=this._getChildren();for(var i=0,l=x.length;i<l;i++ ){var y=x[i];if(y.addState){x[i].addState(w);};};},removeState:function(z){qx.ui.tree.core.AbstractItem.prototype.removeState.call(this,z);var A=this._getChildren();for(var i=0,l=A.length;i<l;i++ ){var B=A[i];if(B.removeState){A[i].removeState(z);};};},getChildrenContainer:function(){if(!this.__vv){this.__vv=new qx.ui.container.Composite(new qx.ui.layout.VBox()).set({visibility:this.isOpen()?f:g});};return this.__vv;},hasChildrenContainer:function(){return this.__vv;},getParentChildrenContainer:function(){if(this.getParent()){return this.getParent().getChildrenContainer();}else if(this.getLayoutParent()){return this.getLayoutParent();}else {return null;};},getChildren:function(){return this.__gP;},hasChildren:function(){return this.__gP?this.__gP.length>0:false;},getItems:function(C,G,F){if(F!==false){var D=[];}else {var D=[this];};var I=this.hasChildren()&&(G!==false||this.isOpen());if(I){var H=this.getChildren();if(C===false){D=D.concat(H);}else {for(var i=0,E=H.length;i<E;i++ ){D=D.concat(H[i].getItems(C,G,false));};};};return D;},recursiveAddToWidgetQueue:function(){var J=this.getItems(true,true,false);for(var i=0,l=J.length;i<l;i++ ){qx.ui.core.queue.Widget.add(J[i]);};},__vw:function(){if(this.getParentChildrenContainer()){this.getParentChildrenContainer()._addAfter(this.getChildrenContainer(),this);};},add:function(P){var K=this.getChildrenContainer();var N=this.getTree();for(var i=0,l=arguments.length;i<l;i++ ){var O=arguments[i];var M=O.getParent();if(M){M.remove(O);};O.setParent(this);var L=this.hasChildren();K.add(O);if(O.hasChildren()){K.add(O.getChildrenContainer());};this.__gP.push(O);if(!L){this.__vw();};if(N){O.recursiveAddToWidgetQueue();N.fireNonBubblingEvent(e,qx.event.type.Data,[O]);};};if(N){qx.ui.core.queue.Widget.add(this);};},addAt:function(V,R){{};if(R==this.__gP.length){this.add(V);return;};var U=V.getParent();if(U){U.remove(V);};var S=this.getChildrenContainer();V.setParent(this);var T=this.hasChildren();var Q=this.__gP[R];S.addBefore(V,Q);if(V.hasChildren()){S.addAfter(V.getChildrenContainer(),V);};qx.lang.Array.insertAt(this.__gP,V,R);if(!T){this.__vw();};if(this.getTree()){V.recursiveAddToWidgetQueue();qx.ui.core.queue.Widget.add(this);};},addBefore:function(W,Y){{};var X=W.getParent();if(X){X.remove(W);};this.addAt(W,this.__gP.indexOf(Y));},addAfter:function(bb,ba){{};var bc=bb.getParent();if(bc){bc.remove(bb);};this.addAt(bb,this.__gP.indexOf(ba)+1);},addAtBegin:function(bd){this.addAt(bd,0);},remove:function(bi){for(var i=0,l=arguments.length;i<l;i++ ){var bh=arguments[i];if(this.__gP.indexOf(bh)==-1){this.warn(j+bh+d);return;};var be=this.getChildrenContainer();if(bh.hasChildrenContainer()){var bg=bh.getChildrenContainer();if(be.getChildren().indexOf(bg)>=0){be.remove(bg);};};qx.lang.Array.remove(this.__gP,bh);bh.setParent(null);be.remove(bh);};var bf=this.getTree();if(bf){bf.fireNonBubblingEvent(h,qx.event.type.Data,[bh]);};qx.ui.core.queue.Widget.add(this);},removeAt:function(bj){var bk=this.__gP[bj];if(bk){this.remove(bk);};},removeAll:function(){var bl=this.__gP.concat();for(var i=this.__gP.length-1;i>=0;i-- ){this.remove(this.__gP[i]);};return bl;}},destruct:function(){this._disposeArray(c);this._disposeObjects(k);}});})();(function(){var a="Enter",b="_applyOpenMode",c="excluded",d="Space",f="Left",g="Boolean",h="changeOpenMode",j="_applyRootOpenClose",k="__sQ",l="visible",m="changeSelection",n="qx.ui.tree.Tree",o="qx.event.type.Data",p="qx.ui.tree.core.AbstractTreeItem",q="tree",r="dbltap",s="_applyHideRoot",t="changeRoot",u="_applyRoot",v="keypress",w="none",x="pane",y="tap",z="Right";qx.Class.define(n,{extend:qx.ui.core.scroll.AbstractScrollArea,implement:[qx.ui.core.IMultiSelection,qx.ui.form.IModelSelection,qx.ui.form.IForm],include:[qx.ui.core.MMultiSelectionHandling,qx.ui.core.MContentPadding,qx.ui.form.MModelSelection,qx.ui.form.MForm],construct:function(){qx.ui.core.scroll.AbstractScrollArea.call(this);this.__sQ=new qx.ui.container.Composite(new qx.ui.layout.VBox()).set({allowShrinkY:false,allowGrowX:true});this.getChildControl(x).add(this.__sQ);this.initOpenMode();this.initRootOpenClose();this.addListener(m,this._onChangeSelection,this);this.addListener(v,this._onKeyPress,this);},events:{addItem:o,removeItem:o},properties:{openMode:{check:[y,r,w],init:r,apply:b,event:h,themeable:true},root:{check:p,init:null,nullable:true,event:t,apply:u},hideRoot:{check:g,init:false,apply:s},rootOpenClose:{check:g,init:false,apply:j},appearance:{refine:true,init:q},focusable:{refine:true,init:true}},members:{__sQ:null,SELECTION_MANAGER:qx.ui.tree.selection.SelectionManager,getChildrenContainer:function(){return this.__sQ;},_applyRoot:function(C,A){var B=this.getChildrenContainer();if(A&&!A.isDisposed()){B.remove(A);if(A.hasChildren()){B.remove(A.getChildrenContainer());};};if(C){B.add(C);if(C.hasChildren()){B.add(C.getChildrenContainer());};C.setVisibility(this.getHideRoot()?c:l);C.recursiveAddToWidgetQueue();};},_applyHideRoot:function(F,D){var E=this.getRoot();if(!E){return;};E.setVisibility(F?c:l);E.recursiveAddToWidgetQueue();},_applyRootOpenClose:function(I,G){var H=this.getRoot();if(!H){return;};H.recursiveAddToWidgetQueue();},_getContentPaddingTarget:function(){return this.__sQ;},getNextNodeOf:function(M,J){if((J!==false||M.isOpen())&&M.hasChildren()){return M.getChildren()[0];};while(M){var parent=M.getParent();if(!parent){return null;};var L=parent.getChildren();var K=L.indexOf(M);if(K>-1&&K<L.length-1){return L[K+1];};M=parent;};return null;},getPreviousNodeOf:function(O,P){var parent=O.getParent();if(!parent){return null;};if(this.getHideRoot()){if(parent==this.getRoot()){if(parent.getChildren()[0]==O){return null;};};}else {if(O==this.getRoot()){return null;};};var R=parent.getChildren();var N=R.indexOf(O);if(N>0){var Q=R[N-1];while((P!==false||Q.isOpen())&&Q.hasChildren()){var S=Q.getChildren();Q=S[S.length-1];};return Q;}else {return parent;};},getNextSiblingOf:function(U){if(U==this.getRoot()){return null;};var parent=U.getParent();var T=parent.getChildren();var V=T.indexOf(U);if(V<T.length-1){return T[V+1];};return null;},getPreviousSiblingOf:function(X){if(X==this.getRoot()){return null;};var parent=X.getParent();var W=parent.getChildren();var Y=W.indexOf(X);if(Y>0){return W[Y-1];};return null;},getItems:function(bb,ba){if(this.getRoot()!=null){return this.getRoot().getItems(bb,ba,this.getHideRoot());}else {return [];};},getChildren:function(){if(this.getRoot()!=null){return [this.getRoot()];}else {return [];};},getTreeItem:function(bc){while(bc){if(bc==this){return null;};if(bc instanceof qx.ui.tree.core.AbstractTreeItem){return bc;};bc=bc.getLayoutParent();};return null;},_applyOpenMode:function(be,bd){if(bd==y){this.removeListener(y,this._onOpen,this);}else if(bd==r){this.removeListener(r,this._onOpen,this);};if(be==y){this.addListener(y,this._onOpen,this);}else if(be==r){this.addListener(r,this._onOpen,this);};},_onOpen:function(e){var bf=this.getTreeItem(e.getTarget());if(!bf||!bf.isOpenable()){return;};bf.setOpen(!bf.isOpen());e.stopPropagation();},_onChangeSelection:function(e){var bh=e.getData();for(var i=0;i<bh.length;i++ ){var bg=bh[i];while(bg.getParent()!=null){bg=bg.getParent();bg.setOpen(true);};};},_onKeyPress:function(e){var bi=this._getLeadItem();if(bi!==null){switch(e.getKeyIdentifier()){case f:if(bi.isOpenable()&&bi.isOpen()){bi.setOpen(false);}else if(bi.getParent()){this.setSelection([bi.getParent()]);};break;case z:if(bi.isOpenable()&&!bi.isOpen()){bi.setOpen(true);};break;case a:case d:if(bi.isOpenable()){bi.toggleOpen();};break;};};}},destruct:function(){this._disposeObjects(k);}});})();(function(){var a="qx.ui.tree.TreeFile",b="tree-file";qx.Class.define(a,{extend:qx.ui.tree.core.AbstractTreeItem,properties:{appearance:{refine:true,init:b}},members:{_addWidgets:function(){this.addSpacer();this.addIcon();this.addLabel();}}});})();(function(){var a="tree-folder",b="qx.ui.tree.TreeFolder";qx.Class.define(b,{extend:qx.ui.tree.core.AbstractTreeItem,properties:{appearance:{refine:true,init:a}},members:{_addWidgets:function(){this.addSpacer();this.addOpenButton();this.addIcon();this.addLabel();}}});})();(function(){var a="qx.ui.tree.core.IVirtualTree";qx.Interface.define(a,{members:{isShowTopLevelOpenCloseIcons:function(){},getLookupTable:function(){},isNode:function(b){this.assertArgumentsCount(arguments,1,1);this.assertInterface(b,qx.core.Object);},isNodeOpen:function(c){this.assertArgumentsCount(arguments,1,1);this.assertInterface(c,qx.core.Object);},getLevel:function(d){this.assertArgumentsCount(arguments,1,1);this.assertInteger(d);},hasChildren:function(e){this.assertArgumentsCount(arguments,1,1);this.assertInterface(e,qx.core.Object);},openNode:function(f){this.assertArgumentsCount(arguments,1,1);this.assertInterface(f,qx.core.Object);},openNodeWithoutScrolling:function(g){this.assertArgumentsCount(arguments,1,1);this.assertInterface(g,qx.core.Object);},closeNode:function(h){this.assertArgumentsCount(arguments,1,1);this.assertInterface(h,qx.core.Object);},closeNodeWithoutScrolling:function(i){this.assertArgumentsCount(arguments,1,1);this.assertInterface(i,qx.core.Object);},getSelection:function(){}}});})();(function(){var a="changeModel",b="resize",c="_applyLabelOptions",d="_applyLabelPath",f='Could not build tree, because \'childProperty\' and/or \'labelPath\' is \'null\'!',g="one",h="_applyOpenMode",j="qx.core.Object",k="Space",l="Left",m="filter",n="Integer",o="_applyModel",p="_applyShowTopLevelOpenCloseIcons",q="Boolean",r="open",s="changeOpenMode",t="cellTap",u="changeDelegate",v="virtual-tree",w="_applyChildProperty",x="qx.event.type.Data",y="_applyIconPath",z="dbltap",A="_applyDelegate",B="cellDbltap",C="_applyHideRoot",D="close",E="_applyRowHeight",F="changeBubble",G="sorter",H="Right",I="Enter",J="qx.ui.tree.VirtualTree",K="keypress",L="none",M="updated",N="_applyShowLeafs",O="String",P="_applyIconOptions",Q="tap",R=".";qx.Class.define(J,{extend:qx.ui.virtual.core.Scroller,implement:[qx.ui.tree.core.IVirtualTree,qx.data.controller.ISelection],include:[qx.ui.virtual.selection.MModel,qx.ui.core.MContentPadding],construct:function(S,U,T){qx.ui.virtual.core.Scroller.call(this,0,1,20,100);this._init();if(U!=null){this.setLabelPath(U);};if(T!=null){this.setChildProperty(T);};if(S!=null){this.initModel(S);};this.initItemHeight();this.initOpenMode();this.addListener(K,this._onKeyPress,this);},events:{open:x,close:x},properties:{appearance:{refine:true,init:v},focusable:{refine:true,init:true},width:{refine:true,init:100},height:{refine:true,init:200},itemHeight:{check:n,init:25,apply:E,themeable:true},openMode:{check:[Q,z,L],init:z,apply:h,event:s,themeable:true},hideRoot:{check:q,init:false,apply:C},showTopLevelOpenCloseIcons:{check:q,init:false,apply:p},showLeafs:{check:q,init:true,apply:N},childProperty:{check:O,apply:w,nullable:true},labelPath:{check:O,apply:d,nullable:true},iconPath:{check:O,apply:y,nullable:true},labelOptions:{apply:c,nullable:true},iconOptions:{apply:P,nullable:true},model:{check:j,apply:o,event:a,nullable:true,deferredInit:true},delegate:{event:u,apply:A,init:null,nullable:true}},members:{_provider:null,_layer:null,__mH:null,__vx:null,__vy:null,__hb:null,_itemWidth:0,__vz:null,syncWidget:function(W){var Y=this._layer.getFirstRow();var X=this._layer.getRowSizes().length;for(var bb=Y;bb<Y+X;bb++ ){var V=this._layer.getRenderedCellWidget(bb,0);if(V!=null){this._itemWidth=Math.max(this._itemWidth,V.getSizeHint().width);};};var ba=this.getPane().getInnerSize().width;this.getPane().getColumnConfig().setItemSize(0,Math.max(this._itemWidth,ba));},openNode:function(bc){this.__vC(bc);this.buildLookupTable();},openNodeWithoutScrolling:function(bd){var be=this.getAutoScrollIntoView();this.setAutoScrollIntoView(false);this.openNode(bd);this.setAutoScrollIntoView(be);},refresh:function(){this.buildLookupTable();},openNodeAndParents:function(bf){this.__vD(this.getModel(),bf);this.buildLookupTable();},closeNode:function(bg){if(qx.lang.Array.contains(this.__vx,bg)){qx.lang.Array.remove(this.__vx,bg);this.fireDataEvent(D,bg);this.buildLookupTable();};},closeNodeWithoutScrolling:function(bh){var bi=this.getAutoScrollIntoView();this.setAutoScrollIntoView(false);this.closeNode(bh);this.setAutoScrollIntoView(bi);},isNodeOpen:function(bj){return qx.lang.Array.contains(this.__vx,bj);},_init:function(){this.__mH=new qx.data.Array();this.__vx=[];this.__vy=[];this._initLayer();},_initLayer:function(){this._provider=new qx.ui.tree.provider.WidgetProvider(this);this._layer=this._provider.createLayer();this._layer.addListener(M,this._onUpdated,this);this.getPane().addLayer(this._layer);this.getPane().addListenerOnce(b,function(e){this.getPane().getColumnConfig().setItemSize(0,e.getData().width);},this);},getLookupTable:function(){return this.__mH;},_reverseLookup:function(bk){return bk;},_getDataFromRow:function(bl){return this.__mH.getItem(bl);},_getSelectables:function(){return this.__mH;},getOpenNodes:function(){return this.__vx;},isNode:function(bm){return qx.ui.tree.core.Util.isNode(bm,this.getChildProperty());},getLevel:function(bn){return this.__vy[bn];},hasChildren:function(bo){return qx.ui.tree.core.Util.hasChildren(bo,this.getChildProperty(),!this.isShowLeafs());},_getContentPaddingTarget:function(){return this.getPane();},_applyRowHeight:function(bq,bp){this.getPane().getRowConfig().setDefaultItemSize(bq);},_applyOpenMode:function(bs,br){var bt=this.getPane();if(bs===z){bt.addListener(B,this._onOpen,this);}else if(bs===Q){bt.addListener(t,this._onOpen,this);};if(br===z){bt.removeListener(B,this._onOpen,this);}else if(br===Q){bt.removeListener(t,this._onOpen,this);};},_applyHideRoot:function(bv,bu){this.buildLookupTable();},_applyShowTopLevelOpenCloseIcons:function(bx,bw){this.buildLookupTable();},_applyShowLeafs:function(bz,by){this.buildLookupTable();},_applyChildProperty:function(bB,bA){this._provider.setChildProperty(bB);},_applyLabelPath:function(bD,bC){this._provider.setLabelPath(bD);},_applyIconPath:function(bF,bE){this._provider.setIconPath(bF);},_applyLabelOptions:function(bH,bG){this._provider.setLabelOptions(bH);},_applyIconOptions:function(bJ,bI){this._provider.setIconOptions(bJ);},_applyModel:function(bL,bK){this.__vx=[];if(bL!=null){{};bL.addListener(F,this._onChangeBubble,this);this.__vC(bL);};if(bK!=null){bK.removeListener(F,this._onChangeBubble,this);};this.__vA();},_applyDelegate:function(bN,bM){this._provider.setDelegate(bN);this.buildLookupTable();},_onChangeBubble:function(event){var bS=event.getData();var bP=bS.name;var bO=bP.lastIndexOf(R);if(bO!=-1){bP=bP.substr(bO+1,bP.length);};if(qx.lang.String.startsWith(bP,this.getChildProperty())){var bQ=bS.item;if(qx.Class.isSubClassOf(bQ.constructor,qx.data.Array)){if(bO===-1){bQ=this.getModel();}else {var bR=bS.name.substr(0,bO);bQ=qx.data.SingleValueBinding.resolvePropertyChain(this.getModel(),bR);};};if(this.__mH.indexOf(bQ)!=-1){this.__vA();};};},_onUpdated:function(event){if(this.__hb==null){this.__hb=new qx.util.DeferredCall(function(){qx.ui.core.queue.Widget.add(this);},this);};this.__hb.schedule();},_onOpen:function(event){var bU=event.getRow();var bT=this.__mH.getItem(bU);if(this.isNode(bT)){if(this.isNodeOpen(bT)){this.closeNode(bT);}else {this.openNode(bT);};};},_onKeyPress:function(e){var bV=this.getSelection();if(bV.getLength()>0){var bX=bV.getItem(0);var bW=this.isNode(bX);switch(e.getKeyIdentifier()){case l:if(bW&&this.isNodeOpen(bX)){this.closeNode(bX);}else {var parent=this.getParent(bX);if(parent!=null){bV.splice(0,1,parent);};};break;case H:if(bW&&!this.isNodeOpen(bX)){this.openNode(bX);}else {if(bW){var bY=bX.get(this.getChildProperty());if(bY!=null&&bY.getLength()>0){bV.splice(0,1,bY.getItem(0));};};};break;case I:case k:if(!bW){return;};if(this.isNodeOpen(bX)){this.closeNode(bX);}else {this.openNode(bX);};break;};};},_beforeApplySelection:function(cb){if(cb.length===0&&this.getSelectionMode()===g){var ca=this.__vF();var cc=this.getLookupTable().indexOf(ca);if(cc>=0){cb.push(cc);};};},_afterApplySelection:function(){var cd=this.getSelection();if(cd.getLength()>0&&this.getSelectionMode()===g){this.__vE(cd.getItem(0));}else {this.__vz=[];};},__vA:function(){this.buildLookupTable();this._applyDefaultSelection();},buildLookupTable:function(){if(this.getModel()!=null&&(this.getChildProperty()==null||this.getLabelPath()==null)){throw new Error(f);};this._itemWidth=0;var ch=[];this.__vy=[];var cf=-1;var cg=this.getModel();if(cg!=null){if(!this.isHideRoot()){cf++ ;ch.push(cg);this.__vy.push(cf);};if(this.isNodeOpen(cg)){var ce=this.__vB(cg,cf);ch=ch.concat(ce);};};if(!qx.lang.Array.equals(this.__mH.toArray(),ch)){this._provider.removeBindings();this.__mH.removeAll();this.__mH.append(ch);this.__uA();this._updateSelection();};},__vB:function(ci,cn){var cm=[];cn++ ;if(!this.isNode(ci)){return cm;};var cq=ci.get(this.getChildProperty());if(cq==null){return cm;};cq=cq.copy();var co=this.getDelegate();var cl=qx.util.Delegate.getMethod(co,m);var ck=qx.util.Delegate.getMethod(co,G);if(ck!=null){cq.sort(ck);};for(var i=0;i<cq.getLength();i++ ){var cp=cq.getItem(i);if(cl&&!cl(cp)){continue;};if(this.isNode(cp)){this.__vy.push(cn);cm.push(cp);if(this.isNodeOpen(cp)){var cj=this.__vB(cp,cn);cm=cm.concat(cj);};}else {if(this.isShowLeafs()){this.__vy.push(cn);cm.push(cp);};};};cq.dispose();return cm;},__vC:function(cr){if(!qx.lang.Array.contains(this.__vx,cr)){this.__vx.push(cr);this.fireDataEvent(r,cr);};},__vD:function(cs,ct){if(cs===ct){this.__vC(ct);return true;};if(!this.isNode(cs)){return false;};var cw=cs.get(this.getChildProperty());if(cw==null){return false;};for(var i=0;i<cw.getLength();i++ ){var cv=cw.getItem(i);var cu=this.__vD(cv,ct);if(cu===true){this.__vC(cv);return true;};};return false;},__uA:function(){this.getPane().getRowConfig().setItemCount(this.__mH.getLength());this.getPane().fullUpdate();},getParent:function(cy){var cz=this.__mH.indexOf(cy);if(cz<0){return null;};var cA=this.__vy[cz];while(cz>0){cz-- ;var cx=this.__vy[cz];if(cx<cA){return this.__mH.getItem(cz);};};return null;},__vE:function(cB){this.__vz=[];var parent=this.getParent(cB);while(parent!=null){this.__vz.unshift(parent);parent=this.getParent(parent);};},__vF:function(){if(this.__vz==null){return this.getModel();};var cC=this.getLookupTable();var parent=this.__vz.pop();while(parent!=null){if(cC.contains(parent)){return parent;};parent=this.__vz.pop();};return this.getModel();}},destruct:function(){var cE=this.getPane();if(cE!=null){if(cE.hasListener(B)){cE.removeListener(B,this._onOpen,this);};if(cE.hasListener(t)){cE.removeListener(t,this._onOpen,this);};};if(!qx.core.ObjectRegistry.inShutDown&&this.__hb!=null){this.__hb.cancel();this.__hb.dispose();};var cD=this.getModel();if(cD!=null){cD.removeListener(F,this._onChangeBubble,this);};this._layer.removeListener(M,this._onUpdated,this);this._layer.destroy();this._provider.dispose();this.__mH.dispose();this._layer=this._provider=this.__mH=this.__vx=this.__hb=null;}});})();(function(){var a="qx.ui.tree.provider.IVirtualTreeProvider";qx.Interface.define(a,{members:{createLayer:function(){},createRenderer:function(){},setChildProperty:function(b){this.assertArgumentsCount(arguments,1,1);this.assertString(b);},setLabelPath:function(c){this.assertArgumentsCount(arguments,1,1);this.assertString(c);},styleSelectabled:function(d){this.assertArgumentsCount(arguments,1,1);this.assertInteger(d);},styleUnselectabled:function(e){this.assertArgumentsCount(arguments,1,1);this.assertInteger(e);},isSelectable:function(f){this.assertArgumentsCount(arguments,1,1);this.assertInteger(f);}}});})();(function(){var a="BindingIds",b="model",c="",d="changeDelegate",f="virtual-tree-file",g="label",h="appearance",i="icon",j="bindItem",k=".length",l="]",m="virtual-tree-folder",n="[",o=".",p="String",q="qx.ui.tree.core.MWidgetController";qx.Mixin.define(q,{construct:function(){this.__uN=[];},properties:{labelPath:{check:p,nullable:true},iconPath:{check:p,nullable:true},labelOptions:{nullable:true},iconOptions:{nullable:true},childProperty:{check:p,nullable:true},delegate:{event:d,init:null,nullable:true}},members:{__uN:null,bindDefaultProperties:function(u,s){this.bindProperty(c,b,null,u,s);this.bindProperty(this.getLabelPath(),g,this.getLabelOptions(),u,s);var r=this.__uO(s);var t=this._tree.getLookupTable();t=qx.data.SingleValueBinding.resolvePropertyChain(t,r);if(qx.util.OOUtil.hasProperty(t.constructor,this.getChildProperty())){this.bindProperty(this.getChildProperty()+k,h,{converter:function(){return m;}},u,s);}else {u.setAppearance(f);};if(this.getIconPath()!=null){this.bindProperty(this.getIconPath(),i,this.getIconOptions(),u,s);};},bindProperty:function(y,B,z,x,v){var A=this.__uO(v,y);var w=this._tree.getLookupTable();var C=w.bind(A,x,B,z);this.__uP(x,C);},bindPropertyReverse:function(J,F,H,I,D){var G=this.__uO(D,J);var E=this._tree.getLookupTable();var K=I.bind(F,E,G,H);this.__uP(I,K);},removeBindings:function(){while(this.__uN.length>0){var L=this.__uN.pop();this._removeBindingsFrom(L);};},_bindItem:function(N,O){var M=qx.util.Delegate.getMethod(this.getDelegate(),j);if(M!=null){M(this,N,O);}else {this.bindDefaultProperties(N,O);};},_removeBindingsFrom:function(P){var Q=this.__uQ(P);while(Q.length>0){var R=Q.pop();try{this._tree.getLookupTable().removeBinding(R);}catch(e){P.removeBinding(R);};};if(qx.lang.Array.contains(this.__uN,P)){qx.lang.Array.remove(this.__uN,P);};},__uO:function(T,U){var S=n+T+l;if(U!=null&&U!=c){S+=o+U;};return S;},__uP:function(W,X){var V=this.__uQ(W);if(!qx.lang.Array.contains(V,X)){V.push(X);};if(!qx.lang.Array.contains(this.__uN,W)){this.__uN.push(W);};},__uQ:function(Y){var ba=Y.getUserData(a);if(ba==null){ba=[];Y.setUserData(a,ba);};return ba;}},destruct:function(){this.__uN=null;}});})();(function(){var a="changeOpen",b="cell.childProperty",c="changeDelegate",d="cell.level",e="createItem",f="auto",g="cell.showLeafs",h="configureItem",i="created",j="onPool",k="cell.row",l="qx.ui.tree.provider.WidgetProvider",m="never";qx.Class.define(l,{extend:qx.core.Object,implement:[qx.ui.virtual.core.IWidgetCellProvider,qx.ui.tree.provider.IVirtualTreeProvider],include:[qx.ui.tree.core.MWidgetController],construct:function(n){qx.core.Object.call(this);this._tree=n;this.addListener(c,this._onChangeDelegate,this);this._onChangeDelegate();},members:{_tree:null,_renderer:null,getCellWidget:function(t,r){var s=this._tree.getLookupTable().getItem(t);var p=false;if(this._tree.isNode(s)){p=this._tree.hasChildren(s);};var o=this._renderer.getCellWidget();o.setOpen(p&&this._tree.isNodeOpen(s));o.addListener(a,this.__vG,this);o.setUserData(b,this.getChildProperty());o.setUserData(g,this._tree.isShowLeafs());if(this._tree.getSelection().contains(s)){this._styleSelectabled(o);}else {this._styleUnselectabled(o);};var q=this._tree.getLevel(t);if(!this._tree.isShowTopLevelOpenCloseIcons()){q-=1;};o.setUserData(d,q);if(!this._tree.isShowTopLevelOpenCloseIcons()&&q==-1){o.setOpenSymbolMode(m);}else {o.setOpenSymbolMode(f);};this._bindItem(o,t);qx.ui.core.queue.Widget.add(o);return o;},poolCellWidget:function(u){u.removeListener(a,this.__vG,this);this._removeBindingsFrom(u);this._renderer.pool(u);this._onPool(u);},createLayer:function(){return new qx.ui.virtual.layer.WidgetCell(this);},createRenderer:function(){var v=qx.util.Delegate.getMethod(this.getDelegate(),e);if(v==null){v=function(){return new qx.ui.tree.VirtualTreeItem();};};var w=new qx.ui.virtual.cell.WidgetCell();w.setDelegate({createWidget:v});return w;},styleSelectabled:function(y){var x=this._tree._layer.getRenderedCellWidget(y,0);this._styleSelectabled(x);},styleUnselectabled:function(A){var z=this._tree._layer.getRenderedCellWidget(A,0);this._styleUnselectabled(z);},isSelectable:function(C){var B=this._tree._layer.getRenderedCellWidget(C,0);if(B!=null){return B.isEnabled();}else {return true;};},_styleSelectabled:function(D){if(D==null){return;};this._renderer.updateStates(D,{selected:1});},_styleUnselectabled:function(E){if(E==null){return;};this._renderer.updateStates(E,{});},_onPool:function(F){var G=qx.util.Delegate.getMethod(this.getDelegate(),j);if(G!=null){G(F);};},_onItemCreated:function(event){var H=qx.util.Delegate.getMethod(this.getDelegate(),h);if(H!=null){var I=event.getData();H(I);};},_onChangeDelegate:function(event){if(this._renderer!=null){this._renderer.dispose();this.removeBindings();};this._renderer=this.createRenderer();this._renderer.addListener(i,this._onItemCreated,this);},__vG:function(event){var K=event.getTarget();var L=K.getUserData(k);var J=this._tree.getLookupTable().getItem(L);if(event.getData()){this._tree.openNodeWithoutScrolling(J);}else {this._tree.closeNodeWithoutScrolling(J);};}},destruct:function(){this.removeBindings();this._renderer.dispose();this._tree=this._renderer=null;}});})();(function(){var a="open",b="cell.childProperty",c="cell.level",d="qx.ui.tree.VirtualTreeItem",f="change",g="virtual-tree-folder",h="cell.showLeafs",i="changeLength";qx.Class.define(d,{extend:qx.ui.tree.core.AbstractItem,properties:{appearance:{refine:true,init:g}},members:{_forwardStates:{selected:true},_addWidgets:function(){this.addSpacer();this.addOpenButton();this.addIcon();this.addLabel();},_shouldShowOpenSymbol:function(){var open=this.getChildControl(a,true);if(open==null){return false;};return this.isOpenable();},getLevel:function(){return this.getUserData(c);},hasChildren:function(){var j=this.getModel();var l=this.getUserData(b);var k=this.getUserData(h);return qx.ui.tree.core.Util.hasChildren(j,l,!k);},_applyModel:function(p,m){var q=this.getUserData(b);var o=this.getUserData(h);if(p!=null&&qx.ui.tree.core.Util.isNode(p,q)){var n=f+qx.lang.String.firstUp(q);if(qx.Class.hasProperty(p.constructor,q)){p.addListener(n,this._onChangeChildProperty,this);};if(qx.ui.tree.core.Util.hasChildren(p,q,!o)){p.get(q).addListener(i,this._onChangeLength,this);this._updateIndent();};};if(m!=null&&qx.ui.tree.core.Util.isNode(m,q)){var n=f+qx.lang.String.firstUp(q);m.removeListener(n,this._onChangeChildProperty,this);var r=m.get(q);if(r){r.removeListener(i,this._onChangeLength,this);};};},_onChangeLength:function(){this._updateIndent();},_onChangeChildProperty:function(e){var t=e.getData();var s=e.getOldData();if(t){this._updateIndent();t.addListener(i,this._onChangeLength,this);};if(s){s.removeListener(i,this._onChangeLength,this);};}}});})();(function(){var a="qx.ui.tree.core.Util";qx.Class.define(a,{statics:{isNode:function(b,c){if(b==null||c==null){return false;};return qx.Class.hasProperty(b.constructor,c);},hasChildren:function(d,f,h){if(d==null||f==null||!this.isNode(d,f)){return false;};var g=d.get(f);if(g==null){return false;};if(!h){return g.length>0;}else {for(var i=0;i<g.getLength();i++ ){var e=g.getItem(i);if(this.isNode(e,f)){return true;};};};return false;}}});})();(function(){var a="Tango",b="qx/icon/Tango",c="qx.theme.icon.Tango";qx.Theme.define(c,{title:a,aliases:{"icon":b}});})();(function(){var a="Liberation Sans",b="Tahoma",c="os.name",d="sans-serif",e="monospace",f="win",g="Arial",h="Lucida Grande",i="osx",j="Courier New",k="os.version",l="Lucida Console",m="7",n="Monaco",o="Candara",p="Segoe UI",q="Consolas",r="vista",s="qx.theme.modern.Font",t="DejaVu Sans Mono";qx.Theme.define(s,{fonts:{"default":{size:(qx.core.Environment.get(c)==f&&(qx.core.Environment.get(k)==m||qx.core.Environment.get(k)==r))?12:11,lineHeight:1.4,family:qx.core.Environment.get(c)==i?[h]:((qx.core.Environment.get(c)==f&&(qx.core.Environment.get(k)==m||qx.core.Environment.get(k)==r)))?[p,o]:[b,a,g,d]},"bold":{size:(qx.core.Environment.get(c)==f&&(qx.core.Environment.get(k)==m||qx.core.Environment.get(k)==r))?12:11,lineHeight:1.4,family:qx.core.Environment.get(c)==i?[h]:((qx.core.Environment.get(c)==f&&(qx.core.Environment.get(k)==m||qx.core.Environment.get(k)==r)))?[p,o]:[b,a,g,d],bold:true},"small":{size:(qx.core.Environment.get(c)==f&&(qx.core.Environment.get(k)==m||qx.core.Environment.get(k)==r))?11:10,lineHeight:1.4,family:qx.core.Environment.get(c)==i?[h]:((qx.core.Environment.get(c)==f&&(qx.core.Environment.get(k)==m||qx.core.Environment.get(k)==r)))?[p,o]:[b,a,g,d]},"monospace":{size:11,lineHeight:1.4,family:qx.core.Environment.get(c)==i?[l,n]:((qx.core.Environment.get(c)==f&&(qx.core.Environment.get(k)==m||qx.core.Environment.get(k)==r)))?[q]:[q,t,j,e]}}});})();(function(){var a="qx_lib.theme.Font";qx.Theme.define(a,{extend:qx.theme.modern.Font,fonts:{}});})();(function(){var a="button-checked",b="window-resize-frame",c="decoration/window/maximize-active-hovered.png",d="radiobutton-hovered",e="decoration/arrows/right.png",f="background-application",g="keyboard-focus",h="group-item",i="scrollbar/button",j="decoration/cursors/",k="icon/16/actions/dialog-ok.png",l="border-invalid",m="combobox/button",n="input",o="slidebar",p="menu",q="table-scroller-focus-indicator",r="move-frame",s="nodrop",t="decoration/table/boolean-true.png",u="table-header-cell",v="app-header",w="row-layer",x="icon/16/places/folder.png",y="text-inactive",z="image",A="radiobutton",B="move",C="window-resize-frame-incl-statusbar",D="radiobutton-checked-focused",E="decoration/window/restore-active-hovered.png",F="window-captionbar-inactive",G="list",H="text-label",I="tree-folder",J="right.png",K="tabview-page-button-bottom-inactive",L="tooltip-error",M="decoration/tree/closed.png",N="window-statusbar",O="button-hovered",P="bold",Q="decoration/scrollbar/scrollbar-",R="background-tip",S="scrollbar-slider-horizontal-disabled",T="text-disabled",U="table-scroller-header",V="radiobutton-disabled",W="scrollbar-slider-horizontal",X="button-pressed",Y="table-pane",fD="decoration/window/close-active.png",fz="native",fE="checkbox-hovered",fA="decoration/window/minimize-active-hovered.png",fB="input-disabled",fw="virtual-list",fC="menubar",fJ="groupbox",fK="icon/16/actions/dialog-cancel.png",fL="tabview-page-button-top-inactive",fM="tabview-page-button-left-inactive",fF="menu-slidebar",fG="toolbar-button-checked",fH="decoration/arrows/left.png",fI="decoration/tree/open-selected.png",fQ="tree-item",gs="radiobutton-checked",fR="decoration/window/minimize-inactive.png",fS="menu-button",fN="button-focused",fO="icon/16/apps/office-calendar.png",ht="text-light",fP="menu-slidebar-button",fT="decoration/arrows/down.png",fU="middle",fV="group",gb="tree",gc="tabview-page-button-right-inactive",gd="decoration/window/minimize-active.png",fW="decoration/window/restore-inactive.png",fX="input-focused-invalid",fY="text-active",ga="splitpane",gh="text-input",gi="combobox/textfield",hy="decoration/window/close-active-hovered.png",gj="invalid",ge="qx/icon/Tango/16/actions/window-close.png",gf="combobox",hx="button-disabled",gg="tabview-page-button-left-active",gn="slidebar/button-forward",go="border-separator",hD="treevirtual-contract",gp="decoration/window/maximize-inactive.png",gk="scrollbar",gl="icon/22/places/folder-open.png",hB="right-top",gm="scrollarea",gq="background-splitpane",gr="datechooser/nav-button",gD="scrollbar-vertical",gC="decoration/toolbar/toolbar-handle-knob.gif",gB="icon/22/mimetypes/office-document.png",gH="text-selected",gG="cell",gF="button-checked-focused",gE="up.png",gw="best-fit",gv="decoration/tree/closed-selected.png",gu="text-hovered",gt="qx.theme.modern.Appearance",gA="decoration/tree/open.png",gz="default",gy="decoration/arrows/up-invert.png",gx="checkbox-disabled",gO="selected",gN="toolbar-button-hovered",gM="decoration/form/checked.png",gL="button",gS="progressive-table-header",gR="decoration/menu/radiobutton.gif",gQ="window-incl-statusbar",gP="decoration/arrows/down-small.png",gK="decoration/arrows/forward.png",gJ="decoration/table/descending.png",gI="decoration/form/undetermined.png",he="tree-file",hd="decoration/form/tooltip-error-arrow-right.png",hc="keep-align",hi="scrollbar-slider-vertical",hh="center",hg="toolbar",hf="alias",gW="decoration/window/restore-active.png",gV="datechooser",gU="toolbar-button",gT="decoration/table/boolean-false.png",hb="qx/static/blank.png",ha="window-pane",gY="icon/32/mimetypes/office-document.png",gX="slidebar/button-backward",ho="radiobutton-checked-disabled",hn="tabview-pane",hm="decoration/arrows/rewind.png",hl="checkbox-focused",hs="selectbox",hr="background-light",hq="top",hp="right",hk="main",hj="button-frame",eB="progressbar-background",eA="radiobutton-checked-hovered",hE="popup",ey="treevirtual-folder",ez="checkbox",ex="table-header-cell-hovered",hC="window",ev="icon/16/mimetypes/office-document.png",ew="treevirtual-expand",eu="text-gray",hz="left",es="decoration/menu/radiobutton-invert.gif",et="text-placeholder",er="atom",eK="text-title",eL="slider",eI="background-medium",eJ="decoration/table/select-column-order.png",eG="down.png",eH="widget",eF="groupitem-text",eq="tabview-page-button-top-active",eD="icon/32/places/folder-open.png",eE="icon/22/places/folder.png",eC="decoration/window/maximize-active.png",eY="decoration/window/close-inactive.png",eW="toolbar-part",eX="decoration/splitpane/knob-vertical.png",eU="left.png",eV="decoration/menu/checkbox-invert.gif",eT="table",hw="decoration/arrows/up.png",eR="table-statusbar",eS="decoration/form/tooltip-error-arrow.png",eQ="window-captionbar-active",hA="copy",eO="radiobutton-focused",eP="decoration/arrows/down-invert.png",eM="decoration/menu/checkbox.gif",eN="",fh="window-caption-active-text",fi="decoration/splitpane/knob-horizontal.png",ff="textfield",fg="icon/32/places/folder.png",fd="toolbar-separator",fe="tabview-page-button-bottom-active",fc="decoration/arrows/up-small.png",hv="decoration/table/ascending.png",fa="small",fb="tabview-page-button-right-active",fv="spinner",hu="tooltip",fx="-disabled",fs="label",fr="scrollbar-horizontal",fu="-invalid",ft="progressbar",fo="progressive-table-header-cell",fn="menu-separator",fq="pane",fp="decoration/arrows/right-invert.png",fk="icon/16/places/folder-open.png",fj="qx/static/blank.gif",fm=".gif",fl="icon/16/actions/view-refresh.png",fy="input-focused";qx.Theme.define(gt,{appearances:{"widget":{},"root":{style:function(hF){return {backgroundColor:f,textColor:H,font:gz};}},"label":{style:function(hG){return {textColor:hG.disabled?T:undefined};}},"move-frame":{style:function(hH){return {decorator:hk};}},"resize-frame":r,"dragdrop-cursor":{style:function(hI){var hJ=s;if(hI.copy){hJ=hA;}else if(hI.move){hJ=B;}else if(hI.alias){hJ=hf;};return {source:j+hJ+fm,position:hB,offset:[2,16,2,6]};}},"image":{style:function(hK){return {opacity:!hK.replacement&&hK.disabled?0.3:1};}},"atom":{},"atom/label":fs,"atom/icon":z,"popup":{style:function(hL){return {decorator:hE,backgroundColor:hr};}},"button-frame":{alias:er,style:function(hM){var hP,hO;var hN=[3,9];if(hM.checked&&hM.focused&&!hM.inner){hP=gF;hO=undefined;hN=[1,7];}else if(hM.disabled){hP=hx;hO=undefined;}else if(hM.pressed){hP=X;hO=gu;}else if(hM.checked){hP=a;hO=undefined;}else if(hM.hovered){hP=O;hO=gu;}else if(hM.focused&&!hM.inner){hP=fN;hO=undefined;hN=[1,7];}else {hP=gL;hO=undefined;};if(hM.invalid&&!hM.disabled){hP+=fu;};return {decorator:hP,textColor:hO,padding:hN,margin:[1,0]};}},"button-frame/image":{style:function(hQ){return {opacity:!hQ.replacement&&hQ.disabled?0.5:1};}},"button":{alias:hj,include:hj,style:function(hR){return {center:true};}},"hover-button":{alias:er,include:er,style:function(hS){var hT=hS.hovered?gO:undefined;return {decorator:hT,textColor:hS.hovered?gH:undefined};}},"menubutton":{include:gL,alias:gL,style:function(hU){return {icon:fT,iconPosition:hp};}},"splitbutton":{},"splitbutton/button":gL,"splitbutton/arrow":{alias:gL,include:gL,style:function(hV,hW){return {icon:fT,padding:[hW.padding[0],hW.padding[1]-6],marginLeft:1};}},"form-renderer-label":{include:fs,style:function(){return {paddingTop:4};}},"checkbox":{alias:er,style:function(hX){var hY;if(hX.checked){hY=gM;}else if(hX.undetermined){hY=gI;}else {hY=fj;};return {icon:hY,minWidth:14,gap:8,paddingLeft:2};}},"checkbox/icon":{style:function(ia){var ic;if(ia.disabled){ic=gx;}else if(ia.focused){ic=hl;}else if(ia.hovered){ic=fE;}else {ic=ez;};ic+=ia.invalid&&!ia.disabled?fu:eN;var ib=ia.undetermined?[3,1]:1;return {decorator:ic,padding:ib,width:10,height:10};}},"radiobutton":{alias:er,style:function(id){return {icon:hb,gap:8,paddingLeft:2};}},"radiobutton/icon":{style:function(ie){var ig;if(ie.disabled&&!ie.checked){ig=V;}else if(ie.checked&&ie.focused){ig=D;}else if(ie.checked&&ie.disabled){ig=ho;}else if(ie.checked&&ie.hovered){ig=eA;}else if(ie.checked){ig=gs;}else if(ie.focused){ig=eO;}else if(ie.hovered){ig=d;}else {ig=A;};ig+=ie.invalid&&!ie.disabled?fu:eN;return {decorator:ig,width:10,height:10};}},"textfield":{style:function(ih){var im;var ik=!!ih.focused;var ii=!!ih.invalid;var ij=!!ih.disabled;if(ik&&ii&&!ij){im=fX;}else if(ik&&!ii&&!ij){im=fy;}else if(ij){im=fB;}else if(!ik&&ii&&!ij){im=l;}else {im=n;};var il;if(ih.disabled){il=T;}else if(ih.showingPlaceholder){il=et;}else {il=gh;};return {decorator:im,padding:[2,4,1],textColor:il};}},"textarea":{include:ff,style:function(io){return {padding:4};}},"spinner":{style:function(ip){var is;var ir=!!ip.focused;var it=!!ip.invalid;var iq=!!ip.disabled;if(ir&&it&&!iq){is=fX;}else if(ir&&!it&&!iq){is=fy;}else if(iq){is=fB;}else if(!ir&&it&&!iq){is=l;}else {is=n;};return {decorator:is};}},"spinner/textfield":{style:function(iu){return {marginRight:2,padding:[2,4,1],textColor:iu.disabled?T:gh};}},"spinner/upbutton":{alias:hj,include:hj,style:function(iv,iw){return {icon:fc,padding:[iw.padding[0]-1,iw.padding[1]-5],margin:0};}},"spinner/downbutton":{alias:hj,include:hj,style:function(ix,iy){return {icon:gP,padding:[iy.padding[0]-1,iy.padding[1]-5],margin:0};}},"datefield":gf,"datefield/button":{alias:m,include:m,style:function(iz){return {icon:fO,padding:[0,3],decorator:undefined};}},"datefield/textfield":gi,"datefield/list":{alias:gV,include:gV,style:function(iA){return {decorator:undefined};}},"groupbox":{style:function(iB){return {legendPosition:hq};}},"groupbox/legend":{alias:er,style:function(iC){return {padding:[1,0,1,4],textColor:iC.invalid?gj:eK,font:P};}},"groupbox/frame":{style:function(iD){return {padding:10,margin:1,decorator:fV};}},"check-groupbox":fJ,"check-groupbox/legend":{alias:ez,include:ez,style:function(iE){return {padding:[1,0,1,4],textColor:iE.invalid?gj:eK,font:P};}},"radio-groupbox":fJ,"radio-groupbox/legend":{alias:A,include:A,style:function(iF){return {padding:[1,0,1,4],textColor:iF.invalid?gj:eK,font:P};}},"scrollarea":{style:function(iG){return {minWidth:50,minHeight:50};}},"scrollarea/corner":{style:function(iH){return {backgroundColor:f};}},"scrollarea/pane":eH,"scrollarea/scrollbar-x":gk,"scrollarea/scrollbar-y":gk,"scrollbar":{style:function(iI){if(iI[fz]){return {};};return {width:iI.horizontal?undefined:16,height:iI.horizontal?16:undefined,decorator:(iI.horizontal?fr:gD),padding:1};}},"scrollbar/slider":{alias:eL,style:function(iJ){return {padding:iJ.horizontal?[0,1,0,1]:[1,0,1,0]};}},"scrollbar/slider/knob":{include:hj,style:function(iK){var iL=iK.horizontal?W:hi;if(iK.disabled){iL+=fx;};return {decorator:iL,minHeight:iK.horizontal?undefined:9,minWidth:iK.horizontal?9:undefined,padding:undefined,margin:0};}},"scrollbar/button":{alias:hj,include:hj,style:function(iM){var iO=Q;if(iM.left){iO+=eU;}else if(iM.right){iO+=J;}else if(iM.up){iO+=gE;}else {iO+=eG;};if(iM.left||iM.right){var iN=iM.left?3:4;return {padding:[3,0,3,iN],icon:iO,width:15,height:14,margin:0};}else {return {padding:3,icon:iO,width:14,height:15,margin:0};};}},"scrollbar/button-begin":i,"scrollbar/button-end":i,"slider":{style:function(iP){var iS;var iR=!!iP.focused;var iT=!!iP.invalid;var iQ=!!iP.disabled;if(iR&&iT&&!iQ){iS=fX;}else if(iR&&!iT&&!iQ){iS=fy;}else if(iQ){iS=fB;}else if(!iR&&iT&&!iQ){iS=l;}else {iS=n;};return {decorator:iS};}},"slider/knob":{include:hj,style:function(iU){return {decorator:iU.disabled?S:W,height:14,width:14,padding:0,margin:0};}},"list":{alias:gm,style:function(iV){var iY;var iX=!!iV.focused;var ja=!!iV.invalid;var iW=!!iV.disabled;if(iX&&ja&&!iW){iY=fX;}else if(iX&&!ja&&!iW){iY=fy;}else if(iW){iY=fB;}else if(!iX&&ja&&!iW){iY=l;}else {iY=n;};return {backgroundColor:hr,decorator:iY};}},"list/pane":eH,"listitem":{alias:er,style:function(jb){return {padding:jb.dragover?[4,4,2,4]:4,textColor:jb.selected?gH:undefined,decorator:jb.selected?gO:undefined,opacity:jb.drag?0.5:undefined};}},"slidebar":{},"slidebar/scrollpane":{},"slidebar/content":{},"slidebar/button-forward":{alias:hj,include:hj,style:function(jc){return {padding:5,center:true,icon:jc.vertical?fT:e};}},"slidebar/button-backward":{alias:hj,include:hj,style:function(jd){return {padding:5,center:true,icon:jd.vertical?hw:fH};}},"tabview":{style:function(je){return {contentPadding:16};}},"tabview/bar":{alias:o,style:function(jf){var jg={marginBottom:jf.barTop?-1:0,marginTop:jf.barBottom?-4:0,marginLeft:jf.barRight?-3:0,marginRight:jf.barLeft?-1:0,paddingTop:0,paddingRight:0,paddingBottom:0,paddingLeft:0};if(jf.barTop||jf.barBottom){jg.paddingLeft=5;jg.paddingRight=7;}else {jg.paddingTop=5;jg.paddingBottom=7;};return jg;}},"tabview/bar/button-forward":{include:gn,alias:gn,style:function(jh){if(jh.barTop||jh.barBottom){return {marginTop:2,marginBottom:2};}else {return {marginLeft:2,marginRight:2};};}},"tabview/bar/button-backward":{include:gX,alias:gX,style:function(ji){if(ji.barTop||ji.barBottom){return {marginTop:2,marginBottom:2};}else {return {marginLeft:2,marginRight:2};};}},"tabview/bar/scrollpane":{},"tabview/pane":{style:function(jj){return {decorator:hn,marginBottom:jj.barBottom?-1:0,marginTop:jj.barTop?-1:0,marginLeft:jj.barLeft?-1:0,marginRight:jj.barRight?-1:0};}},"tabview-page":{alias:eH,include:eH,style:function(jk){return {padding:[4,3]};}},"tabview-page/button":{alias:er,style:function(jl){var jr,jn=0;var jq=0,jm=0,jo=0,jp=0;if(jl.checked){if(jl.barTop){jr=eq;jn=[5,11];jo=jl.firstTab?0:-5;jp=jl.lastTab?0:-5;}else if(jl.barBottom){jr=fe;jn=[5,11];jo=jl.firstTab?0:-5;jp=jl.lastTab?0:-5;jq=3;}else if(jl.barRight){jr=fb;jn=[5,10];jq=jl.firstTab?0:-5;jm=jl.lastTab?0:-5;jo=2;}else {jr=gg;jn=[5,10];jq=jl.firstTab?0:-5;jm=jl.lastTab?0:-5;};}else {if(jl.barTop){jr=fL;jn=[3,9];jq=4;jo=jl.firstTab?5:1;jp=1;}else if(jl.barBottom){jr=K;jn=[3,9];jm=4;jo=jl.firstTab?5:1;jp=1;jq=3;}else if(jl.barRight){jr=gc;jn=[3,9];jp=5;jq=jl.firstTab?5:1;jm=1;jo=3;}else {jr=fM;jn=[3,9];jo=5;jq=jl.firstTab?5:1;jm=1;jp=1;};};return {zIndex:jl.checked?10:5,decorator:jr,padding:jn,marginTop:jq,marginBottom:jm,marginLeft:jo,marginRight:jp,textColor:jl.disabled?T:jl.checked?fY:y};}},"tabview-page/button/label":{alias:fs,style:function(js){return {padding:js.focused?[0,1,0,1]:[1,2,1,2],decorator:js.focused?g:undefined};}},"tabview-page/button/close-button":{alias:er,style:function(jt){return {icon:ge};}},"toolbar":{style:function(ju){return {decorator:hg,spacing:2};}},"toolbar/part":{style:function(jv){return {decorator:eW,spacing:2};}},"toolbar/part/container":{style:function(jw){return {paddingLeft:2,paddingRight:2};}},"toolbar/part/handle":{style:function(jx){return {source:gC,marginLeft:3,marginRight:3};}},"toolbar-button":{alias:er,style:function(jy){var jz;if(jy.pressed||(jy.checked&&!jy.hovered)||(jy.checked&&jy.disabled)){jz=fG;}else if(jy.hovered&&!jy.disabled){jz=gN;};return {marginTop:2,marginBottom:2,padding:(jy.pressed||jy.checked||jy.hovered)&&!jy.disabled||(jy.disabled&&jy.checked)?3:5,decorator:jz};}},"toolbar-menubutton":{alias:gU,include:gU,style:function(jA){return {showArrow:true};}},"toolbar-menubutton/arrow":{alias:z,include:z,style:function(jB){return {source:gP};}},"toolbar-splitbutton":{style:function(jC){return {marginTop:2,marginBottom:2};}},"toolbar-splitbutton/button":{alias:gU,include:gU,style:function(jD){return {icon:fT,marginTop:undefined,marginBottom:undefined};}},"toolbar-splitbutton/arrow":{alias:gU,include:gU,style:function(jE){if(jE.pressed||jE.checked||(jE.hovered&&!jE.disabled)){var jF=1;}else {var jF=3;};return {padding:jF,icon:fT,marginTop:undefined,marginBottom:undefined};}},"toolbar-separator":{style:function(jG){return {decorator:fd,margin:7};}},"tree":G,"tree-item":{style:function(jH){var jI=jH.selected?gO:undefined;return {padding:[2,6],textColor:jH.selected?gH:undefined,decorator:jI,opacity:jH.drag?0.5:undefined};}},"tree-item/icon":{include:z,style:function(jJ){return {paddingRight:5};}},"tree-item/label":fs,"tree-item/open":{include:z,style:function(jK){var jL;if(jK.selected&&jK.opened){jL=fI;}else if(jK.selected&&!jK.opened){jL=gv;}else if(jK.opened){jL=gA;}else {jL=M;};return {padding:[0,5,0,2],source:jL};}},"tree-folder":{include:fQ,alias:fQ,style:function(jM){var jO,jN;if(jM.small){jO=jM.opened?fk:x;jN=fk;}else if(jM.large){jO=jM.opened?eD:fg;jN=eD;}else {jO=jM.opened?gl:eE;jN=gl;};return {icon:jO,iconOpened:jN};}},"tree-file":{include:fQ,alias:fQ,style:function(jP){return {icon:jP.small?ev:jP.large?gY:gB};}},"treevirtual":eT,"treevirtual-folder":{style:function(jQ){return {icon:jQ.opened?fk:x};}},"treevirtual-file":{include:ey,alias:ey,style:function(jR){return {icon:ev};}},"treevirtual-line":{style:function(jS){return {icon:fj};}},"treevirtual-contract":{style:function(jT){return {icon:gA,paddingLeft:5,paddingTop:2};}},"treevirtual-expand":{style:function(jU){return {icon:M,paddingLeft:5,paddingTop:2};}},"treevirtual-only-contract":hD,"treevirtual-only-expand":ew,"treevirtual-start-contract":hD,"treevirtual-start-expand":ew,"treevirtual-end-contract":hD,"treevirtual-end-expand":ew,"treevirtual-cross-contract":hD,"treevirtual-cross-expand":ew,"treevirtual-end":{style:function(jV){return {icon:fj};}},"treevirtual-cross":{style:function(jW){return {icon:fj};}},"tooltip":{include:hE,style:function(jX){return {backgroundColor:R,padding:[1,3,2,3],offset:[15,5,5,5]};}},"tooltip/atom":er,"tooltip-error":{style:function(jY){return {placeMethod:eH,offset:[-3,1,0,0],arrowPosition:jY.placementLeft?hz:hp,position:hB,showTimeout:100,hideTimeout:10000,padding:[0,4,4,0]};}},"tooltip-error/arrow":{include:z,style:function(ka){var kb=ka.placementLeft?hd:eS;return {source:kb,padding:[6,0,0,0],zIndex:10000001};}},"tooltip-error/atom":{include:hE,style:function(kc){return {textColor:gH,backgroundColor:undefined,decorator:L,font:P,padding:[3,4,4,4],margin:[1,0,0,0],maxWidth:333};}},"window":{style:function(kd){return {decorator:kd.showStatusbar?gQ:hC,contentPadding:[10,10,10,10],margin:kd.maximized?0:[0,5,5,0]};}},"window-resize-frame":{style:function(ke){return {decorator:ke.showStatusbar?C:b};}},"window/pane":{style:function(kf){return {decorator:ha};}},"window/captionbar":{style:function(kg){return {decorator:(kg.active?eQ:F),textColor:kg.active?fh:eu,minHeight:26,paddingRight:2};}},"window/icon":{style:function(kh){return {margin:[5,0,3,6]};}},"window/title":{style:function(ki){return {alignY:fU,font:P,marginLeft:6,marginRight:12};}},"window/minimize-button":{alias:er,style:function(kj){return {icon:kj.active?kj.hovered?fA:gd:fR,margin:[4,8,2,0]};}},"window/restore-button":{alias:er,style:function(kk){return {icon:kk.active?kk.hovered?E:gW:fW,margin:[5,8,2,0]};}},"window/maximize-button":{alias:er,style:function(kl){return {icon:kl.active?kl.hovered?c:eC:gp,margin:[4,8,2,0]};}},"window/close-button":{alias:er,style:function(km){return {icon:km.active?km.hovered?hy:fD:eY,margin:[4,8,2,0]};}},"window/statusbar":{style:function(kn){return {padding:[2,6],decorator:N,minHeight:18};}},"window/statusbar-text":{style:function(ko){return {font:fa};}},"iframe":{style:function(kp){return {decorator:hk};}},"resizer":{style:function(kq){return {decorator:fq};}},"splitpane":{style:function(kr){return {decorator:ga};}},"splitpane/splitter":{style:function(ks){return {width:ks.horizontal?3:undefined,height:ks.vertical?3:undefined,backgroundColor:gq};}},"splitpane/splitter/knob":{style:function(kt){return {source:kt.horizontal?fi:eX};}},"splitpane/slider":{style:function(ku){return {width:ku.horizontal?3:undefined,height:ku.vertical?3:undefined,backgroundColor:gq};}},"selectbox":hj,"selectbox/atom":er,"selectbox/popup":hE,"selectbox/list":{alias:G},"selectbox/arrow":{include:z,style:function(kv){return {source:fT,paddingLeft:5};}},"datechooser":{style:function(kw){var kz;var ky=!!kw.focused;var kA=!!kw.invalid;var kx=!!kw.disabled;if(ky&&kA&&!kx){kz=fX;}else if(ky&&!kA&&!kx){kz=fy;}else if(kx){kz=fB;}else if(!ky&&kA&&!kx){kz=l;}else {kz=n;};return {padding:2,decorator:kz,backgroundColor:hr};}},"datechooser/navigation-bar":{},"datechooser/nav-button":{include:hj,alias:hj,style:function(kB){var kC={padding:[2,4]};if(kB.lastYear){kC.icon=hm;kC.marginRight=1;}else if(kB.lastMonth){kC.icon=fH;}else if(kB.nextYear){kC.icon=gK;kC.marginLeft=1;}else if(kB.nextMonth){kC.icon=e;};return kC;}},"datechooser/last-year-button-tooltip":hu,"datechooser/last-month-button-tooltip":hu,"datechooser/next-year-button-tooltip":hu,"datechooser/next-month-button-tooltip":hu,"datechooser/last-year-button":gr,"datechooser/last-month-button":gr,"datechooser/next-month-button":gr,"datechooser/next-year-button":gr,"datechooser/month-year-label":{style:function(kD){return {font:P,textAlign:hh,textColor:kD.disabled?T:undefined};}},"datechooser/date-pane":{style:function(kE){return {textColor:kE.disabled?T:undefined,marginTop:2};}},"datechooser/weekday":{style:function(kF){return {textColor:kF.disabled?T:kF.weekend?ht:undefined,textAlign:hh,paddingTop:2,backgroundColor:eI};}},"datechooser/week":{style:function(kG){return {textAlign:hh,padding:[2,4],backgroundColor:eI};}},"datechooser/day":{style:function(kH){var kI=kH.disabled?undefined:kH.selected?gO:undefined;return {textAlign:hh,decorator:kI,textColor:kH.disabled?T:kH.selected?gH:kH.otherMonth?ht:undefined,font:kH.today?P:undefined,padding:[2,4]};}},"combobox":{style:function(kJ){var kM;var kL=!!kJ.focused;var kN=!!kJ.invalid;var kK=!!kJ.disabled;if(kL&&kN&&!kK){kM=fX;}else if(kL&&!kN&&!kK){kM=fy;}else if(kK){kM=fB;}else if(!kL&&kN&&!kK){kM=l;}else {kM=n;};return {decorator:kM};}},"combobox/popup":hE,"combobox/list":{alias:G},"combobox/button":{include:hj,alias:hj,style:function(kO,kP){var kQ={icon:fT,padding:[kP.padding[0],kP.padding[1]-6],margin:undefined};if(kO.selected){kQ.decorator=fN;};return kQ;}},"combobox/textfield":{include:ff,style:function(kR){return {decorator:undefined};}},"menu":{style:function(kS){var kT={decorator:p,spacingX:6,spacingY:1,iconColumnWidth:16,arrowColumnWidth:4,placementModeY:kS.submenu||kS.contextmenu?gw:hc};if(kS.submenu){kT.position=hB;kT.offset=[-2,-3];};return kT;}},"menu/slidebar":fF,"menu-slidebar":eH,"menu-slidebar-button":{style:function(kU){var kV=kU.hovered?gO:undefined;return {decorator:kV,padding:7,center:true};}},"menu-slidebar/button-backward":{include:fP,style:function(kW){return {icon:kW.hovered?gy:hw};}},"menu-slidebar/button-forward":{include:fP,style:function(kX){return {icon:kX.hovered?eP:fT};}},"menu-separator":{style:function(kY){return {height:0,decorator:fn,margin:[4,2]};}},"menu-button":{alias:er,style:function(la){var lb=la.selected?gO:undefined;return {decorator:lb,textColor:la.selected?gH:undefined,padding:[4,6]};}},"menu-button/icon":{include:z,style:function(lc){return {alignY:fU};}},"menu-button/label":{include:fs,style:function(ld){return {alignY:fU,padding:1};}},"menu-button/shortcut":{include:fs,style:function(le){return {alignY:fU,marginLeft:14,padding:1};}},"menu-button/arrow":{include:z,style:function(lf){return {source:lf.selected?fp:e,alignY:fU};}},"menu-checkbox":{alias:fS,include:fS,style:function(lg){return {icon:!lg.checked?undefined:lg.selected?eV:eM};}},"menu-radiobutton":{alias:fS,include:fS,style:function(lh){return {icon:!lh.checked?undefined:lh.selected?es:gR};}},"menubar":{style:function(li){return {decorator:fC};}},"menubar-button":{alias:er,style:function(lj){var lk=(lj.pressed||lj.hovered)&&!lj.disabled?gO:undefined;return {decorator:lk,textColor:lj.pressed||lj.hovered?gH:undefined,padding:[3,8]};}},"colorselector":eH,"colorselector/control-bar":eH,"colorselector/control-pane":eH,"colorselector/visual-pane":fJ,"colorselector/preset-grid":eH,"colorselector/colorbucket":{style:function(ll){return {decorator:hk,width:16,height:16};}},"colorselector/preset-field-set":fJ,"colorselector/input-field-set":{include:fJ,alias:fJ,style:function(){return {paddingTop:20};}},"colorselector/preview-field-set":{include:fJ,alias:fJ,style:function(){return {paddingTop:20};}},"colorselector/hex-field-composite":eH,"colorselector/hex-field":ff,"colorselector/rgb-spinner-composite":eH,"colorselector/rgb-spinner-red":fv,"colorselector/rgb-spinner-green":fv,"colorselector/rgb-spinner-blue":fv,"colorselector/hsb-spinner-composite":eH,"colorselector/hsb-spinner-hue":fv,"colorselector/hsb-spinner-saturation":fv,"colorselector/hsb-spinner-brightness":fv,"colorselector/preview-content-old":{style:function(lm){return {decorator:hk,width:50,height:10};}},"colorselector/preview-content-new":{style:function(ln){return {decorator:hk,backgroundColor:hr,width:50,height:10};}},"colorselector/hue-saturation-field":{style:function(lo){return {decorator:hk,margin:5};}},"colorselector/brightness-field":{style:function(lp){return {decorator:hk,margin:[5,7]};}},"colorselector/hue-saturation-pane":eH,"colorselector/hue-saturation-handle":eH,"colorselector/brightness-pane":eH,"colorselector/brightness-handle":eH,"colorpopup":{alias:hE,include:hE,style:function(lq){return {padding:5,backgroundColor:f};}},"colorpopup/field":{style:function(lr){return {decorator:hk,margin:2,width:14,height:14,backgroundColor:hr};}},"colorpopup/selector-button":gL,"colorpopup/auto-button":gL,"colorpopup/preview-pane":fJ,"colorpopup/current-preview":{style:function(ls){return {height:20,padding:4,marginLeft:4,decorator:hk,allowGrowX:true};}},"colorpopup/selected-preview":{style:function(lt){return {height:20,padding:4,marginRight:4,decorator:hk,allowGrowX:true};}},"colorpopup/colorselector-okbutton":{alias:gL,include:gL,style:function(lu){return {icon:k};}},"colorpopup/colorselector-cancelbutton":{alias:gL,include:gL,style:function(lv){return {icon:fK};}},"table":{alias:eH,style:function(lw){return {decorator:eT};}},"table/statusbar":{style:function(lx){return {decorator:eR,padding:[0,2]};}},"table/column-button":{alias:hj,style:function(ly){return {decorator:U,padding:3,icon:eJ};}},"table-column-reset-button":{include:fS,alias:fS,style:function(){return {icon:fl};}},"table-scroller":eH,"table-scroller/scrollbar-x":gk,"table-scroller/scrollbar-y":gk,"table-scroller/header":{style:function(lz){return {decorator:U,textColor:lz.disabled?T:undefined};}},"table-scroller/pane":{style:function(lA){return {backgroundColor:Y};}},"table-scroller/focus-indicator":{style:function(lB){return {decorator:q};}},"table-scroller/resize-line":{style:function(lC){return {backgroundColor:go,width:2};}},"table-header-cell":{alias:er,style:function(lD){return {minWidth:13,minHeight:20,padding:lD.hovered?[3,4,2,4]:[3,4],decorator:lD.hovered?ex:u,sortIcon:lD.sorted?(lD.sortedAscending?hv:gJ):undefined};}},"table-header-cell/label":{style:function(lE){return {minWidth:0,alignY:fU,paddingRight:5};}},"table-header-cell/sort-icon":{style:function(lF){return {alignY:fU,alignX:hp,opacity:lF.disabled?0.3:1};}},"table-header-cell/icon":{style:function(lG){return {minWidth:0,alignY:fU,paddingRight:5,opacity:lG.disabled?0.3:1};}},"table-editor-textfield":{include:ff,style:function(lH){return {decorator:undefined,padding:[2,2],backgroundColor:hr};}},"table-editor-selectbox":{include:hs,alias:hs,style:function(lI){return {padding:[0,2],backgroundColor:hr};}},"table-editor-combobox":{include:gf,alias:gf,style:function(lJ){return {decorator:undefined,backgroundColor:hr};}},"progressive-table-header":{alias:eH,style:function(lK){return {decorator:gS};}},"progressive-table-header-cell":{alias:er,style:function(lL){return {minWidth:40,minHeight:25,paddingLeft:6,decorator:fo};}},"app-header":{style:function(lM){return {font:P,textColor:gH,padding:[8,12],decorator:v};}},"app-header-label":fs,"app-splitpane":{alias:ga,style:function(lN){return {padding:0};}},"virtual-list":G,"virtual-list/row-layer":w,"row-layer":eH,"group-item":{include:fs,alias:fs,style:function(lO){return {padding:4,decorator:h,textColor:eF,font:P};}},"virtual-selectbox":hs,"virtual-selectbox/dropdown":hE,"virtual-selectbox/dropdown/list":{alias:fw},"virtual-combobox":gf,"virtual-combobox/dropdown":hE,"virtual-combobox/dropdown/list":{alias:fw},"virtual-tree":{include:gb,alias:gb,style:function(lP){return {itemHeight:26};}},"virtual-tree-folder":I,"virtual-tree-file":he,"column-layer":eH,"cell":{style:function(lQ){return {textColor:lQ.selected?gH:H,padding:[3,6],font:gz};}},"cell-string":gG,"cell-number":{include:gG,style:function(lR){return {textAlign:hp};}},"cell-image":gG,"cell-boolean":{include:gG,style:function(lS){return {iconTrue:t,iconFalse:gT};}},"cell-atom":gG,"cell-date":gG,"cell-html":gG,"progressbar":{style:function(lT){return {decorator:ft,padding:[1],backgroundColor:eB,width:200,height:20};}},"progressbar/progress":{style:function(lU){return {decorator:(lU.disabled?h:gO)};}}}});})();(function(){var a="qx_lib.theme.Appearance";qx.Theme.define(a,{extend:qx.theme.modern.Appearance,appearances:{}});})();(function(){var a="button-checked-focused",b="window-resize-frame",c="checkbox-disabled-border",d="group-background",e="menu-end",f="keyboard-focus",g="button-disabled-start",h="selected-end",i="table-header-hovered",j="border-invalid",k="decoration/toolbar/toolbar-part.gif",l="border-separator",m="window-border-caption",n="radiobutton-hovered",o="button-hovered-end",p="border-input",q="radiobutton",r="repeat-y",s="border-dragover",t="border-inner-input",u="radiobutton-checked-focused",v="groupitem-end",w="group-border",x="input-start",y="button-hovered-start",z="tooltip-error",A="button-hovered",B="selected-start",C="progressive-table-header-border-right",D="button-border-disabled",E="scrollbar-slider-horizontal",F="button-pressed",G="window-statusbar-background",H="tabview-end",I="radiobutton-hovered-invalid",J="checkbox-hovered",K="radiobutton-background",L="window-captionbar-active",M="checkbox-hovered-inner",N="toolbar-button-hovered",O="window-caption-active-end",P="solid",Q="button-start",R="dotted",S="radiobutton-disabled",T="radiobutton-checked",U="checkbox-disabled-end",V="window-caption-active-start",W="window-border",X="button-focused",Y="input",cv="tabview-inactive",cw="qx/decoration/Modern",cx="border-toolbar-separator-left",cr="invalid",cs="button-disabled",ct="horizontal",cu="table-header-start",cC="background-splitpane",cD="button-end",cE="button-checked",cF="border-toolbar-border-inner",cy="px",cz="input-border-disabled",cA="scrollbar-slider-vertical",cB="checkbox-inner",cJ="button",dk="button-disabled-end",dI="toolbar-end",cK="groupitem-start",cG="menu-start",cH="input-focused-start",dD="scrollbar-start",cI="scrollbar-slider-start",cL="radiobutton-checked-disabled",cM="checkbox-focused",cN="border-toolbar-button-outer",cS="background-light",cT="qx.theme.modern.Decoration",cU="checkbox-hovered-invalid",cO="radiobutton-checked-hovered",cP="tabview-page-button-top-inactive",cQ="#243B58",cR="checkbox",cY="checkbox-focus",da="window",dF="checkbox-disabled-inner",db="border-toolbar-separator-right",cV="tabview-inactive-start",cW="scrollbar-end",dE="table-header-end",cX="tabview-background",df="checkbox-end",dg="border-button",dH="tabview-inactive-end",dh="input-end",dc="tabview-page-button-top-active",dd="input-focused-inner-invalid",dG="menu-separator-top",de="shadow",di="window-caption-inactive-start",dj="scrollbar-slider-end",dw="background-pane",dv="pane-end",du="input-focused-end",dA="menubar-start",dz="toolbar-start",dy="radiobutton-focused",dx="pane-start",dp="table-focus-indicator",dn="menu-separator-bottom",dm="#1D2D45",dl="border-main",dt="scrollbar-horizontal",ds="window-caption-inactive-end",dr="checkbox-border",dq="tabview-start",dC="checkbox-hovered-inner-invalid",dB="input-focused";qx.Theme.define(cT,{aliases:{decoration:cw},decorations:{"main":{style:{width:1,color:dl}},"selected":{style:{startColorPosition:0,endColorPosition:100,startColor:B,endColor:h}},"dragover":{style:{bottom:[2,P,s]}},"pane":{style:{width:1,color:cX,radius:3,shadowColor:de,shadowBlurRadius:2,shadowLength:0,gradientStart:[dx,0],gradientEnd:[dv,100]}},"group":{style:{backgroundColor:d,radius:4,color:w,width:1}},"keyboard-focus":{style:{width:1,color:f,style:R}},"radiobutton":{style:{backgroundColor:K,radius:5,width:1,innerWidth:2,color:dr,innerColor:K,shadowLength:0,shadowBlurRadius:0,shadowColor:cY}},"radiobutton-checked":{include:q,style:{backgroundColor:T}},"radiobutton-checked-focused":{include:T,style:{shadowBlurRadius:4}},"radiobutton-checked-hovered":{include:T,style:{innerColor:J}},"radiobutton-focused":{include:q,style:{shadowBlurRadius:4}},"radiobutton-hovered":{include:q,style:{backgroundColor:J,innerColor:J}},"radiobutton-disabled":{include:q,style:{innerColor:S,backgroundColor:S,color:c}},"radiobutton-checked-disabled":{include:S,style:{backgroundColor:cL}},"radiobutton-invalid":{include:q,style:{color:cr}},"radiobutton-checked-invalid":{include:T,style:{color:cr}},"radiobutton-checked-focused-invalid":{include:u,style:{color:cr,shadowColor:cr}},"radiobutton-checked-hovered-invalid":{include:cO,style:{color:cr,innerColor:I}},"radiobutton-focused-invalid":{include:dy,style:{color:cr,shadowColor:cr}},"radiobutton-hovered-invalid":{include:n,style:{color:cr,innerColor:I,backgroundColor:I}},"separator-horizontal":{style:{widthLeft:1,colorLeft:l}},"separator-vertical":{style:{widthTop:1,colorTop:l}},"tooltip-error":{style:{backgroundColor:z,radius:4,shadowColor:de,shadowBlurRadius:2,shadowLength:1}},"popup":{style:{width:1,color:dl,shadowColor:de,shadowBlurRadius:3,shadowLength:1}},"scrollbar-horizontal":{style:{gradientStart:[dD,0],gradientEnd:[cW,100]}},"scrollbar-vertical":{include:dt,style:{orientation:ct}},"scrollbar-slider-horizontal":{style:{gradientStart:[cI,0],gradientEnd:[dj,100],color:dl,width:1,radius:3}},"scrollbar-slider-vertical":{include:E,style:{orientation:ct}},"scrollbar-slider-horizontal-disabled":{include:E,style:{color:D}},"scrollbar-slider-vertical-disabled":{include:cA,style:{color:D}},"button":{style:{radius:3,color:dg,width:1,startColor:Q,endColor:cD,startColorPosition:35,endColorPosition:100}},"button-disabled":{include:cJ,style:{color:D,startColor:g,endColor:dk}},"button-hovered":{include:cJ,style:{startColor:y,endColor:o}},"button-checked":{include:cJ,style:{endColor:Q,startColor:cD}},"button-pressed":{include:cJ,style:{endColor:y,startColor:o}},"button-focused":{style:{radius:3,color:dg,width:1,innerColor:X,innerWidth:2,startColor:Q,endColor:cD,startColorPosition:30,endColorPosition:100}},"button-checked-focused":{include:X,style:{endColor:Q,startColor:cD}},"button-invalid":{include:cJ,style:{color:j}},"button-disabled-invalid":{include:cs,style:{color:j}},"button-hovered-invalid":{include:A,style:{color:j}},"button-checked-invalid":{include:cE,style:{color:j}},"button-pressed-invalid":{include:F,style:{color:j}},"button-focused-invalid":{include:X,style:{color:j}},"button-checked-focused-invalid":{include:a,style:{color:j}},"checkbox":{style:{width:1,color:dr,innerWidth:1,innerColor:cB,backgroundColor:df,shadowLength:0,shadowBlurRadius:0,shadowColor:cY}},"checkbox-hovered":{include:cR,style:{innerColor:M,backgroundColor:J}},"checkbox-focused":{include:cR,style:{shadowBlurRadius:4}},"checkbox-disabled":{include:cR,style:{color:c,innerColor:dF,backgroundColor:U}},"checkbox-invalid":{include:cR,style:{color:cr}},"checkbox-hovered-invalid":{include:J,style:{color:cr,innerColor:dC,backgroundColor:cU}},"checkbox-focused-invalid":{include:cM,style:{color:cr,shadowColor:cr}},"input":{style:{color:p,innerColor:t,innerWidth:1,width:1,backgroundColor:cS,startColor:x,endColor:dh,startColorPosition:0,endColorPosition:12,colorPositionUnit:cy}},"border-invalid":{include:Y,style:{color:j}},"input-focused":{include:Y,style:{startColor:cH,innerColor:du,endColorPosition:4}},"input-focused-invalid":{include:dB,style:{innerColor:dd,color:j}},"input-disabled":{include:Y,style:{color:cz}},"toolbar":{style:{startColorPosition:40,endColorPosition:60,startColor:dz,endColor:dI}},"toolbar-button-hovered":{style:{color:cN,width:1,innerWidth:1,innerColor:cF,radius:2,gradientStart:[Q,30],gradientEnd:[cD,100]}},"toolbar-button-checked":{include:N,style:{gradientStart:[cD,30],gradientEnd:[Q,100]}},"toolbar-separator":{style:{widthLeft:1,widthRight:1,colorLeft:cx,colorRight:db,styleLeft:P,styleRight:P}},"toolbar-part":{style:{backgroundImage:k,backgroundRepeat:r}},"tabview-pane":{style:{width:1,color:W,radius:3,gradientStart:[dq,90],gradientEnd:[H,100]}},"tabview-page-button-top-active":{style:{radius:[3,3,0,0],width:[1,1,0,1],color:cX,backgroundColor:dq,shadowLength:1,shadowColor:de,shadowBlurRadius:2}},"tabview-page-button-top-inactive":{style:{radius:[3,3,0,0],color:cv,colorBottom:cX,width:1,gradientStart:[cV,0],gradientEnd:[dH,100]}},"tabview-page-button-bottom-active":{include:dc,style:{radius:[0,0,3,3],width:[0,1,1,1],backgroundColor:cV,shadowLength:0,shadowBlurRadius:0}},"tabview-page-button-bottom-inactive":{include:cP,style:{radius:[0,0,3,3],width:[0,1,1,1],colorBottom:cv,colorTop:cX}},"tabview-page-button-left-active":{include:dc,style:{radius:[3,0,0,3],width:[1,0,1,1],shadowLength:0,shadowBlurRadius:0}},"tabview-page-button-left-inactive":{include:cP,style:{radius:[3,0,0,3],width:[1,0,1,1],colorBottom:cv,colorRight:cX}},"tabview-page-button-right-active":{include:dc,style:{radius:[0,3,3,0],width:[1,1,1,0],shadowLength:0,shadowBlurRadius:0}},"tabview-page-button-right-inactive":{include:cP,style:{radius:[0,3,3,0],width:[1,1,1,0],colorBottom:cv,colorLeft:cX}},"splitpane":{style:{backgroundColor:dw,width:3,color:cC,style:P}},"window":{style:{radius:[5,5,0,0],shadowBlurRadius:4,shadowLength:2,shadowColor:de}},"window-incl-statusbar":{include:da,style:{radius:[5,5,5,5]}},"window-resize-frame":{style:{radius:[5,5,0,0],width:1,color:dl}},"window-resize-frame-incl-statusbar":{include:b,style:{radius:[5,5,5,5]}},"window-captionbar-active":{style:{width:1,color:W,colorBottom:m,radius:[5,5,0,0],gradientStart:[V,30],gradientEnd:[O,70]}},"window-captionbar-inactive":{include:L,style:{gradientStart:[di,30],gradientEnd:[ds,70]}},"window-statusbar":{style:{backgroundColor:G,width:[0,1,1,1],color:W,radius:[0,0,5,5]}},"window-pane":{style:{backgroundColor:dw,width:1,color:W,widthTop:0}},"table":{style:{width:1,color:dl,style:P}},"table-statusbar":{style:{widthTop:1,colorTop:dl,style:P}},"table-scroller-header":{style:{gradientStart:[cu,10],gradientEnd:[dE,90],widthBottom:1,colorBottom:dl}},"table-header-cell":{style:{widthRight:1,colorRight:l,styleRight:P}},"table-header-cell-hovered":{style:{widthRight:1,colorRight:l,styleRight:P,widthBottom:1,colorBottom:i,styleBottom:P}},"table-scroller-focus-indicator":{style:{width:2,color:dp,style:P}},"progressive-table-header":{style:{width:1,color:dl,style:P}},"progressive-table-header-cell":{style:{gradientStart:[cu,10],gradientEnd:[dE,90],widthRight:1,colorRight:C}},"menu":{style:{gradientStart:[cG,0],gradientEnd:[e,100],shadowColor:de,shadowBlurRadius:2,shadowLength:1,width:1,color:dl}},"menu-separator":{style:{widthTop:1,colorTop:dG,widthBottom:1,colorBottom:dn}},"menubar":{style:{gradientStart:[dA,0],gradientEnd:[e,100],width:1,color:l}},"app-header":{style:{gradientStart:[cQ,0],gradientEnd:[dm,100]}},"progressbar":{style:{width:1,color:p}},"group-item":{style:{startColorPosition:0,endColorPosition:100,startColor:cK,endColor:v}}}});})();(function(){var a="qx_lib.theme.Decoration";qx.Theme.define(a,{extend:qx.theme.modern.Decoration,decorations:{}});})();(function(){var a="black",b="#EEEEEE",c="#1a1a1a",d="#ffffdd",e="#b6b6b6",f="#004DAD",g="#BABABA",h="#005BC3",i="#334866",j="#00204D",k="#CECECE",l="gray",m="#D9D9D9",n="#D8D8D8",o="#99C3FE",p="#001533",q="#B3B3B3",r="#F4F4F4",s="#D5D5D5",t="#fffefe",u="#C3C3C3",v="#E4E4E4",w="#DDDDDD",x="#FF9999",y="css.rgba",z="#E8E8E9",A="#084FAA",B="#AFAFAF",C="white",D="#C5C5C5",E="rgba(0, 0, 0, 0.4)",F="#DBDBDB",G="#4a4a4a",H="#83BAEA",I="#D7E7F4",J="#07125A",K="#084FAB",L="#FAF2F2",M="#87AFE7",N="#F7EAEA",O="#777D8D",P="#FBFBFB",Q="#CACACA",R="#909090",S="#9B9B9B",T="#F0F9FE",U="#314a6e",V="#B4B4B4",W="#787878",X="qx.theme.modern.Color",Y="#000000",cb="#26364D",cc="#A7A7A7",cd="#D1E4FF",bW="#5CB0FD",bX="#FCFCFC",bY="#EAEAEA",ca="#003B91",ci="#80B4EF",cj="#FF6B78",ck="#949494",cl="#808080",ce="#F3F3F3",cf="#930000",cg="#7B7B7B",ch="#F0F0F0",cp="#C82C2C",cM="#DFDFDF",cN="#B6B6B6",cq="#0880EF",cm="#4d4d4d",cn="#f4f4f4",cP="#7B7A7E",co="#D0D0D0",cr="#f8f8f8",cs="#404955",ct="#959595",cx="#AAAAAA",cQ="#F7E9E9",cy="#314A6E",cu="#C72B2B",cv="#FAFAFA",cO="#FBFCFB",cw="#B2D2FF",cC="#666666",cD="#CBC8CD",cE="#999999",cF="#8EB8D6",cz="#b8b8b8",cA="#727272",cR="#33508D",cB="#E8E8E8",cJ="#CCCCCC",cK="#CCC",cS="#EFEFEF",cL="#F2F2F2",cG="#F1F1F1",cH="#990000",cI="#00368A";qx.Theme.define(X,{colors:{"background-application":cM,"background-pane":ce,"background-light":bX,"background-medium":b,"background-splitpane":B,"background-tip":d,"background-tip-error":cu,"background-odd":v,"progressbar-background":C,"text-light":R,"text-gray":G,"text-label":c,"text-title":U,"text-input":Y,"text-hovered":p,"text-disabled":cP,"text-selected":t,"text-active":cb,"text-inactive":cs,"text-placeholder":cD,"border-inner-scrollbar":C,"border-main":cm,"menu-separator-top":D,"menu-separator-bottom":cv,"border-separator":cl,"border-toolbar-button-outer":e,"border-toolbar-border-inner":cr,"border-toolbar-separator-right":cn,"border-toolbar-separator-left":cz,"border-input":i,"border-inner-input":C,"border-disabled":cN,"border-pane":j,"border-button":cC,"border-column":cJ,"border-focused":o,"invalid":cH,"border-focused-invalid":x,"border-dragover":cR,"keyboard-focus":a,"table-pane":ce,"table-focus-indicator":cq,"table-row-background-focused-selected":K,"table-row-background-focused":ci,"table-row-background-selected":K,"table-row-background-even":ce,"table-row-background-odd":v,"table-row-selected":t,"table-row":c,"table-row-line":cK,"table-column-line":cK,"table-header-hovered":C,"progressive-table-header":cx,"progressive-table-header-border-right":cL,"progressive-table-row-background-even":r,"progressive-table-row-background-odd":v,"progressive-progressbar-background":l,"progressive-progressbar-indicator-done":cJ,"progressive-progressbar-indicator-undone":C,"progressive-progressbar-percent-background":l,"progressive-progressbar-percent-text":C,"selected-start":f,"selected-end":cI,"background-selected":cI,"tabview-background":J,"shadow":qx.core.Environment.get(y)?E:cE,"pane-start":P,"pane-end":ch,"group-background":cB,"group-border":V,"radiobutton-background":cS,"checkbox-border":cy,"checkbox-focus":M,"checkbox-hovered":cw,"checkbox-hovered-inner":cd,"checkbox-inner":b,"checkbox-start":v,"checkbox-end":ce,"checkbox-disabled-border":W,"checkbox-disabled-inner":Q,"checkbox-disabled-start":co,"checkbox-disabled-end":n,"checkbox-hovered-inner-invalid":L,"checkbox-hovered-invalid":cQ,"radiobutton-checked":h,"radiobutton-disabled":s,"radiobutton-checked-disabled":cg,"radiobutton-hovered-invalid":N,"tooltip-error":cp,"scrollbar-start":cJ,"scrollbar-end":cG,"scrollbar-slider-start":b,"scrollbar-slider-end":u,"button-border-disabled":ct,"button-start":ch,"button-end":B,"button-disabled-start":r,"button-disabled-end":g,"button-hovered-start":T,"button-hovered-end":cF,"button-focused":H,"border-invalid":cf,"input-start":ch,"input-end":cO,"input-focused-start":I,"input-focused-end":bW,"input-focused-inner-invalid":cj,"input-border-disabled":S,"input-border-inner":C,"toolbar-start":cS,"toolbar-end":w,"window-border":j,"window-border-caption":cA,"window-caption-active-text":C,"window-caption-active-start":A,"window-caption-active-end":ca,"window-caption-inactive-start":cL,"window-caption-inactive-end":F,"window-statusbar-background":cS,"tabview-start":bX,"tabview-end":b,"tabview-inactive":O,"tabview-inactive-start":bY,"tabview-inactive-end":k,"table-header-start":cB,"table-header-end":q,"menu-start":z,"menu-end":m,"menubar-start":cB,"groupitem-start":cc,"groupitem-end":ck,"groupitem-text":C,"virtual-row-layer-background-even":C,"virtual-row-layer-background-odd":C}});})();(function(){var a="qx_lib.theme.Color";qx.Theme.define(a,{extend:qx.theme.modern.Color,colors:{}});})();(function(){var a="qx_lib.theme.Theme";qx.Theme.define(a,{meta:{color:qx_lib.theme.Color,decoration:qx_lib.theme.Decoration,font:qx_lib.theme.Font,icon:qx.theme.icon.Tango,appearance:qx_lib.theme.Appearance}});})();(function(){var a="black",b="border-focused-light",c="border-dark",d="solid",e="dotted",f="effect",g="border-separator",h="border-focused-dark",i="tooltip-text",j="border-focused-light-shadow",k="table-header-border",l="table-focus-indicator",m="outset",n="border-focused-dark-shadow",o="qx/decoration/Classic",p="border-lead",q="border-dark-shadow",r="#FFF",s="main",t="invalid",u="shadow",v="border-light-shadow",w="qx.theme.classic.Decoration",x="white",y="gray",z="border-light";qx.Theme.define(w,{aliases:{decoration:o},decorations:{"main":{style:{width:1,color:c}},"keyboard-focus":{style:{width:1,color:a,style:e}},"inset":{style:{width:1,innerWidth:1,color:[q,z,z,q],innerColor:[c,v,v,c]}},"outset":{style:{width:1,innerWidth:1,color:[v,c,c,v],innerColor:[z,q,q,z]}},"groove":{style:{width:1,innerWidth:1,color:[q,z,z,q],innerColor:[z,q,q,z]}},"ridge":{style:{width:1,innerWidth:1,color:[z,q,q,z],innerColor:[q,z,z,q]}},"inset-thin":{style:{width:1,color:[q,z,z,q]}},"outset-thin":{style:{width:1,color:[z,q,q,z]}},"focused-inset":{style:{width:1,innerWidth:1,color:[n,b,b,n],innerColor:[h,j,j,h]}},"focused-outset":{style:{width:1,innerWidth:1,color:[j,h,h,j],innerColor:[b,n,n,b]}},"border-invalid":{style:{width:1,innerWidth:1,color:[q,z,z,q],innerColor:t}},"separator-horizontal":{style:{widthLeft:1,colorLeft:g}},"separator-vertical":{style:{widthTop:1,colorTop:g}},"window":{include:m,style:{shadowLength:1,shadowBlurRadius:2,shadowColor:u}},"lead-item":{style:{width:1,style:e,color:p}},"tooltip":{style:{width:1,color:i,shadowLength:1,shadowBlurRadius:5,shadowColor:u}},"tooltip-error":{style:{width:1,color:i,shadowLength:1,shadowBlurRadius:5,shadowColor:u}},"popup":{include:s,style:{shadowLength:2,shadowBlurRadius:2,shadowColor:u}},"toolbar-separator":{style:{widthLeft:1,colorLeft:q}},"toolbar-part-handle":{style:{width:1,style:d,colorTop:x,colorLeft:x,colorRight:q,colorBottom:q}},"menu":{include:m,style:{shadowLength:1,shadowBlurRadius:3,shadowColor:u}},"menu-separator":{style:{widthTop:1,widthBottom:1,colorTop:c,colorBottom:z}},"datechooser-date-pane":{style:{widthTop:1,colorTop:y,style:d}},"datechooser-weekday":{style:{widthBottom:1,colorBottom:y,style:d}},"datechooser-week":{style:{widthRight:1,colorRight:y,style:d}},"datechooser-week-header":{style:{widthBottom:1,colorBottom:y,widthRight:1,colorRight:y,style:d}},"tabview-page-button-top":{style:{width:1,color:[v,c,c,v],innerWidth:1,innerColor:[z,q,q,z],widthBottom:0,innerWidthBottom:0}},"tabview-page-button-bottom":{style:{width:1,color:[v,c,c,v],innerWidth:1,innerColor:[z,q,q,z],widthTop:0,innerWidthTop:0}},"tabview-page-button-left":{style:{width:1,color:[v,c,c,v],innerWidth:1,innerColor:[z,q,q,z],widthRight:0,innerWidthRight:0}},"tabview-page-button-right":{style:{width:1,color:[v,c,c,v],innerWidth:1,innerColor:[z,q,q,z],widthLeft:0,innerWidthLeft:0}},"table-statusbar":{style:{widthTop:1,colorTop:q,styleTop:d}},"table-scroller-header":{style:{widthBottom:1,colorBottom:k,styleBottom:d}},"table-scroller-focus-indicator":{style:{width:2,color:l,style:d}},"table-header-cell":{style:{widthRight:1,colorRight:k,styleRight:d}},"table-header-cell-hovered":{style:{widthRight:1,colorRight:k,styleRight:d,widthBottom:2,colorBottom:f,styleBottom:d}},"progressbar":{style:{backgroundColor:r,width:1,color:g}}}});})();(function(){var a="table-row-background-even",b="decoration/treevirtual/cross_minus.gif",c="radiobutton-hovered",d="menu-slidebar-button",e="keyboard-focus",f="decoration/treevirtual/start_plus.gif",g="background-disabled",h="background",i="scrollbar/button",j="date-chooser-title",k="decoration/cursors/",l="icon/16/actions/dialog-ok.png",m="combobox/button",n="slidebar",o="menu",p="table-scroller-focus-indicator",q="move-frame",r="nodrop",s="date-chooser-selected",t="tabview-page-button-left",u="decoration/arrows/up-small.gif",v="image",w="radiobutton",x="move",y="radiobutton-checked-focused",z="list",A="decoration/arrows/right.gif",B="qx.theme.classic.Appearance",C="decoration/menu/checkbox.gif",D="tooltip-error",E="decoration/arrows/up.gif",F="default",G="button-hovered",H="bold",I="resize-frame",J="decoration/arrows/rewind.gif",K="text-disabled",L="table-scroller-header",M="table-pane",N="white",O="table-header-cell-hover",P="focused-outset",Q="checkbox-hovered",R="text",S="checkbox",T="virtual-list",U="groupbox",V="icon/16/actions/dialog-cancel.png",W="menu-slidebar",X="border-dark",Y="datechooser-date-pane",eJ="background-pane",eF="decoration/treevirtual/cross_plus.gif",eK="decoration/arrows/down-small.gif",eG="qx/icon/Oxygen/16/actions/window-close.png",eH="menu-button",eE="datechooser-week",eI="icon/16/apps/office-calendar.png",eP="datechooser-weekday",eQ="table-header-border",eW="middle",eR="#BABABA",eL="tree",eM="checkbox-undetermined",eN="window-active-caption-text",eO="window-active-caption",eV="icon",fy="checkbox-checked-focused",eX="splitpane",eY="toolbar-separator",eS="groove",eT="invalid",gz="icon/16/places/folder.png",eU="checkbox-pressed",fa="combobox",fb="tree-folder",fc="cell",fh="slidebar/button-forward",fi="tooltip-invalid",fj="icon/16/mimetypes/text-plain.png",fd="decoration/window/restore.gif",fe="scrollbar",ff="decoration/menu/checkbox-invert.gif",fg="right-top",fn="scrollarea",fo="window-inactive-caption-text",gB="inset-thin",fp="text-selected",fk="table-header-cell",fl="button-checked",gA="best-fit",fm="up.gif",ft="checkbox-undetermined-hovered",fu=".png",gG="keep-align",fv="background-focused",fq="tabview-page-button-right",fr="tabview-page-button-top",gE="tabview-page-button-bottom",fs="inset",fw="row-layer",fx="button",fJ="decoration/menu/radiobutton.gif",fI="decoration/arrows/",fH="decoration/table/descending.png",fN="progressbar",fM="tree-file",fL="tooltip-text",fK="checkbox-checked-hovered",fC="left.gif",fB="center",fA="decoration/arrows/up-invert.gif",fz="datechooser/button",fG="alias",fF="datechooser",fE="toolbar-button",fD="outset",fU="decoration/arrows/right-invert.gif",fT="slidebar/button-backward",fS="button-abandoned",fR="radiobutton-checked-disabled",fY="lead-item",fX="checkbox-focused",fW="selectbox",fV="background-light",fQ="decoration/arrows/down.gif",fP="right",fO="decoration/treevirtual/start_minus.gif",gk="main",gj="spinner",gi="button-frame",go="background-field",gn="radiobutton-checked-hovered",gm="popup",gl="treevirtual-folder",gd="decoration/window/minimize.gif",gc="checkbox-checked",gb="table-header-cell-hovered",ga="down.gif",gh="background-selected",gg="window",gf="decoration/treevirtual/end.gif",ge="decoration/treevirtual/end_minus.gif",gu="window-inactive-caption",gt="decoration/menu/radiobutton-invert.gif",gs="text-placeholder",gr="atom",gy="slider",gx="decoration/table/select-column-order.png",gw="decoration/arrows/next.gif",gv="table-header",gq="decoration/treevirtual/only_minus.gif",gp="datechooser-week-header",ed="widget",ec="decoration/window/maximize.gif",gH="decoration/treevirtual/only_plus.gif",ea="checkbox-checked-pressed",eb="date-chooser",dY="decoration/arrows/down-invert.gif",gF="menu-separator",dW="decoration/splitpane/knob-vertical.png",dX=".gif",dV="decoration/arrows/forward.gif",gC="radiobutton-checked-pressed",dT="table-statusbar",dU="radiobutton-pressed",dS="focused-inset",em="decoration/form/",en="light-background",ek="copy",el="table-row-background-selected",ei="radiobutton-focused",ej="decoration/tree/minus.gif",eh="",dR="decoration/splitpane/knob-horizontal.png",ef="outset-thin",eg="textfield",ee="right.gif",eA="radiobutton-checked",ey="decoration/treevirtual/cross.gif",ez="table-scroller/header",ew="decoration/table/ascending.png",ex="decoration/treevirtual/line.gif",ev="tooltip",eB="label",et="decoration/tree/plus.gif",eu="-invalid",es="decoration/treevirtual/end_plus.gif",gD="checkbox-undetermined-focused",eq="toolbar-part-handle",er="decoration/arrows/left.gif",eo="background-invalid",ep="icon/16/places/folder-open.png",eC="decoration/window/close.gif",eD="icon/16/actions/view-refresh.png";qx.Theme.define(B,{appearances:{"widget":{},"label":{style:function(gI){return {textColor:gI.disabled?K:undefined};}},"image":{style:function(gJ){return {opacity:!gJ.replacement&&gJ.disabled?0.3:undefined};}},"atom":{},"atom/label":eB,"atom/icon":v,"root":{style:function(gK){return {backgroundColor:h,textColor:R,font:F};}},"popup":{style:function(gL){return {decorator:gm,backgroundColor:eJ};}},"tooltip":{include:gm,style:function(gM){return {backgroundColor:ev,textColor:fL,decorator:ev,padding:[1,3,2,3],offset:[15,5,5,5]};}},"tooltip/atom":gr,"tooltip-error":{include:ev,style:function(gN){return {textColor:fp,showTimeout:100,hideTimeout:10000,decorator:D,font:H,backgroundColor:fi};}},"tooltip-error/atom":gr,"iframe":{style:function(gO){return {backgroundColor:N,decorator:fs};}},"move-frame":{style:function(gP){return {decorator:gk};}},"resize-frame":q,"dragdrop-cursor":{style:function(gQ){var gR=r;if(gQ.copy){gR=ek;}else if(gQ.move){gR=x;}else if(gQ.alias){gR=fG;};return {source:k+gR+dX,position:fg,offset:[2,16,2,6]};}},"button-frame":{alias:gr,style:function(gS){if(gS.pressed||gS.abandoned||gS.checked){var gU=!gS.inner&&gS.focused?dS:fs;var gT=[4,3,2,5];}else {var gU=!gS.inner&&gS.focused?P:fD;var gT=[3,4];};return {backgroundColor:gS.abandoned?fS:gS.hovered?G:gS.checked?fl:fx,decorator:gU,padding:gT};}},"button":{alias:gi,include:gi,style:function(gV){return {center:true};}},"hover-button":{alias:gr,include:gr,style:function(gW){return {backgroundColor:gW.hovered?gh:undefined,textColor:gW.hovered?fp:undefined};}},"menubutton":{include:fx,alias:fx,style:function(gX){return {icon:fQ,iconPosition:fP};}},"splitbutton":{},"splitbutton/button":fx,"splitbutton/arrow":{alias:fx,include:fx,style:function(gY){return {icon:fQ};}},"scrollarea/corner":{style:function(){return {backgroundColor:h};}},"scrollarea":ed,"scrollarea/pane":ed,"scrollarea/scrollbar-x":fe,"scrollarea/scrollbar-y":fe,"list":{alias:fn,style:function(ha){var hd;var hc=!!ha.focused;var he=!!ha.invalid;var hb=!!ha.disabled;if(he&&!hb){hd=eo;}else if(hc&&!he&&!hb){hd=fv;}else if(hb){hd=g;}else {hd=N;};return {decorator:ha.focused?dS:fs,backgroundColor:hd};}},"listitem":{alias:gr,style:function(hf){return {gap:4,padding:hf.lead?[2,4]:[3,5],backgroundColor:hf.selected?gh:undefined,textColor:hf.selected?fp:undefined,decorator:hf.lead?fY:undefined,opacity:hf.drag?0.5:undefined};}},"form-renderer-label":{include:eB,style:function(){return {paddingTop:4};}},"textfield":{style:function(hg){var hk;var hj=!!hg.focused;var hh=!!hg.invalid;var hi=!!hg.disabled;if(hh&&!hi){hk=eo;}else if(hj&&!hh&&!hi){hk=fv;}else if(hi){hk=g;}else {hk=go;};var hl;if(hg.disabled){hl=K;}else if(hg.showingPlaceholder){hl=gs;}else {hl=undefined;};return {decorator:hg.focused?dS:fs,padding:[2,3],textColor:hl,backgroundColor:hk};}},"textarea":eg,"checkbox":{alias:gr,style:function(hm){var ho;if(hm.checked){if(hm.disabled){ho=gc;}else if(hm.focused){ho=fy;}else if(hm.pressed){ho=ea;}else if(hm.hovered){ho=fK;}else {ho=gc;};}else if(hm.undetermined){if(hm.disabled){ho=eM;}else if(hm.focused){ho=gD;}else if(hm.hovered){ho=ft;}else {ho=eM;};}else if(!hm.disabled){if(hm.focused){ho=fX;}else if(hm.pressed){ho=eU;}else if(hm.hovered){ho=Q;};};ho=ho||S;var hn=hm.invalid&&!hm.disabled?eu:eh;return {icon:em+ho+hn+fu,gap:6};}},"radiobutton":{alias:S,include:S,style:function(hp){var hr;if(hp.checked&&hp.focused){hr=y;}else if(hp.checked&&hp.disabled){hr=fR;}else if(hp.checked&&hp.pressed){hr=gC;}else if(hp.checked&&hp.hovered){hr=gn;}else if(hp.checked){hr=eA;}else if(hp.focused){hr=ei;}else if(hp.pressed){hr=dU;}else if(hp.hovered){hr=c;}else {hr=w;};var hq=hp.invalid&&!hp.disabled?eu:eh;return {icon:em+hr+hq+fu};}},"spinner":{style:function(hs){return {decorator:hs.focused?dS:fs,textColor:hs.disabled?K:undefined};}},"spinner/textfield":{include:eg,style:function(ht){return {decorator:undefined,padding:[2,3]};}},"spinner/upbutton":{alias:fx,include:fx,style:function(hu){return {icon:u,padding:hu.pressed?[2,2,0,4]:[1,3,1,3],backgroundColor:hu.hovered?G:fx};}},"spinner/downbutton":{alias:fx,include:fx,style:function(hv){return {icon:eK,padding:hv.pressed?[2,2,0,4]:[1,3,1,3],backgroundColor:hv.hovered?G:fx};}},"datefield":fa,"datefield/button":{alias:m,include:m,style:function(hw){return {icon:eI,padding:[0,3],backgroundColor:undefined,decorator:undefined};}},"datefield/list":{alias:fF,include:fF,style:function(hx){return {decorator:hx.focused?dS:fs};}},"groupbox":{style:function(hy){return {backgroundColor:h};}},"groupbox/legend":{alias:gr,style:function(hz){return {backgroundColor:h,textColor:hz.invalid?eT:undefined,padding:[1,0,1,4]};}},"groupbox/frame":{style:function(hA){return {padding:[12,9],marginTop:10,decorator:eS};}},"check-groupbox":U,"check-groupbox/legend":{alias:S,include:S,style:function(hB){return {backgroundColor:h,textColor:hB.invalid?eT:undefined,padding:[1,0,1,4]};}},"radio-groupbox":U,"radio-groupbox/legend":{alias:w,include:w,style:function(hC){return {backgroundColor:h,textColor:hC.invalid?eT:undefined,padding:[1,0,1,4]};}},"toolbar":{style:function(hD){return {backgroundColor:h};}},"toolbar/part":{},"toolbar/part/container":{},"toolbar/part/handle":{style:function(hE){return {decorator:eq,backgroundColor:h,padding:[0,1],margin:[3,2],allowGrowY:true};}},"toolbar-separator":{style:function(hF){return {margin:[3,2],decorator:eY};}},"toolbar-button":{alias:gr,style:function(hG){if(hG.pressed||hG.checked||hG.abandoned){var hI=gB;var hH=[3,2,1,4];}else if(hG.hovered&&!hG.disabled){var hI=ef;var hH=[2,3];}else {var hI=undefined;var hH=[3,4];};return {cursor:F,decorator:hI,padding:hH,backgroundColor:hG.abandoned?fS:hG.checked?fV:fx};}},"toolbar-menubutton":{alias:fE,include:fE,style:function(hJ){return {showArrow:true};}},"toolbar-menubutton/arrow":{alias:v,include:v,style:function(hK){return {source:eK};}},"toolbar-splitbutton":{},"toolbar-splitbutton/button":fE,"toolbar-splitbutton/arrow":{alias:fE,include:fE,style:function(hL){return {icon:fQ};}},"slidebar":{},"slidebar/scrollpane":{},"slidebar/content":{},"slidebar/button-forward":{alias:fx,include:fx,style:function(hM){return {icon:hM.vertical?fQ:gw};}},"slidebar/button-backward":{alias:fx,include:fx,style:function(hN){return {icon:hN.vertical?E:er};}},"tabview":{},"tabview/bar":{alias:n,style:function(hO){var hP=0,hS=0,hQ=0,hR=0;if(hO.barTop){hQ=-2;}else if(hO.barBottom){hP=-2;}else if(hO.barRight){hR=-2;}else {hS=-2;};return {marginBottom:hQ,marginTop:hP,marginLeft:hR,marginRight:hS};}},"tabview/bar/button-forward":{include:fh,alias:fh,style:function(hT){if(hT.barTop||hT.barBottom){return {marginTop:2,marginBottom:2};}else {return {marginLeft:2,marginRight:2};};}},"tabview/bar/button-backward":{include:fT,alias:fT,style:function(hU){if(hU.barTop||hU.barBottom){return {marginTop:2,marginBottom:2};}else {return {marginLeft:2,marginRight:2};};}},"tabview/pane":{style:function(hV){return {backgroundColor:h,decorator:fD,padding:10};}},"tabview-page":ed,"tabview-page/button":{style:function(ia){var ih;var ie=0,ic=0,hX=0,ib=0;if(ia.barTop||ia.barBottom){var hY=2,hW=2,id=6,ig=6;}else {var hY=6,hW=6,id=6,ig=6;};if(ia.barTop){ih=fr;}else if(ia.barRight){ih=fq;}else if(ia.barBottom){ih=gE;}else {ih=t;};if(ia.checked){if(ia.barTop||ia.barBottom){id+=2;ig+=2;}else {hY+=2;hW+=2;};}else {if(ia.barTop||ia.barBottom){hX+=2;ie+=2;}else if(ia.barLeft||ia.barRight){ic+=2;ib+=2;};};if(ia.checked){if(!ia.firstTab){if(ia.barTop||ia.barBottom){ib=-4;}else {ie=-4;};};if(!ia.lastTab){if(ia.barTop||ia.barBottom){ic=-4;}else {hX=-4;};};};return {zIndex:ia.checked?10:5,decorator:ih,backgroundColor:h,padding:[hY,ig,hW,id],margin:[ie,ic,hX,ib],textColor:ia.disabled?K:undefined};}},"tabview-page/button/label":{alias:eB,style:function(ii){return {padding:ii.focused?[0,1,0,1]:[1,2,1,2],decorator:ii.focused?e:undefined};}},"tabview-page/button/icon":v,"tabview-page/button/close-button":{alias:gr,style:function(ij){return {icon:eG};}},"scrollbar":{},"scrollbar/slider":{alias:gy,style:function(ik){return {backgroundColor:fV};}},"scrollbar/slider/knob":{include:gi,style:function(il){return {height:14,width:14,minHeight:il.horizontal?undefined:9,minWidth:il.horizontal?9:undefined};}},"scrollbar/button":{alias:fx,include:fx,style:function(im){var io;if(im.up||im.down){if(im.pressed||im.abandoned||im.checked){io=[5,2,3,4];}else {io=[4,3];};}else {if(im.pressed||im.abandoned||im.checked){io=[4,3,2,5];}else {io=[3,4];};};var ip=fI;if(im.left){ip+=fC;}else if(im.right){ip+=ee;}else if(im.up){ip+=fm;}else {ip+=ga;};return {padding:io,icon:ip};}},"scrollbar/button-begin":i,"scrollbar/button-end":i,"slider":{style:function(iq){var ir;if(iq.disabled){ir=g;}else if(iq.invalid){ir=eo;}else if(iq.focused){ir=fV;}else {ir=go;};return {backgroundColor:ir,decorator:iq.focused?dS:fs};}},"slider/knob":{include:gi,style:function(is){return {width:14,height:14,decorator:fD};}},"tree-folder/open":{style:function(it){return {source:it.opened?ej:et};}},"tree-folder":{style:function(iu){return {padding:[2,3,2,0],icon:iu.opened?ep:gz,iconOpened:ep,opacity:iu.drag?0.5:undefined};}},"tree-folder/icon":{style:function(iv){return {padding:[0,4,0,0]};}},"tree-folder/label":{style:function(iw){return {padding:[1,2],backgroundColor:iw.selected?gh:undefined,textColor:iw.selected?fp:undefined};}},"tree-file":{include:fb,alias:fb,style:function(ix){return {icon:fj,opacity:ix.drag?0.5:undefined};}},"tree":{include:z,alias:z,style:function(iy){return {contentPadding:[4,4,4,4]};}},"treevirtual":{style:function(iz){return {decorator:gk};}},"treevirtual-folder":{style:function(iA){return {icon:(iA.opened?ep:gz),opacity:iA.drag?0.5:undefined};}},"treevirtual-file":{include:gl,alias:gl,style:function(iB){return {icon:fj,opacity:iB.drag?0.5:undefined};}},"treevirtual-line":{style:function(iC){return {icon:ex};}},"treevirtual-contract":{style:function(iD){return {icon:ej};}},"treevirtual-expand":{style:function(iE){return {icon:et};}},"treevirtual-only-contract":{style:function(iF){return {icon:gq};}},"treevirtual-only-expand":{style:function(iG){return {icon:gH};}},"treevirtual-start-contract":{style:function(iH){return {icon:fO};}},"treevirtual-start-expand":{style:function(iI){return {icon:f};}},"treevirtual-end-contract":{style:function(iJ){return {icon:ge};}},"treevirtual-end-expand":{style:function(iK){return {icon:es};}},"treevirtual-cross-contract":{style:function(iL){return {icon:b};}},"treevirtual-cross-expand":{style:function(iM){return {icon:eF};}},"treevirtual-end":{style:function(iN){return {icon:gf};}},"treevirtual-cross":{style:function(iO){return {icon:ey};}},"window":{style:function(iP){return {contentPadding:[10,10,10,10],backgroundColor:h,decorator:iP.maximized?undefined:gg};}},"window-resize-frame":I,"window/pane":{},"window/captionbar":{style:function(iQ){return {padding:1,backgroundColor:iQ.active?eO:gu,textColor:iQ.active?eN:fo};}},"window/icon":{style:function(iR){return {marginRight:4};}},"window/title":{style:function(iS){return {cursor:F,font:H,marginRight:20,alignY:eW};}},"window/minimize-button":{include:fx,alias:fx,style:function(iT){return {icon:gd,padding:iT.pressed||iT.abandoned?[2,1,0,3]:[1,2]};}},"window/restore-button":{include:fx,alias:fx,style:function(iU){return {icon:fd,padding:iU.pressed||iU.abandoned?[2,1,0,3]:[1,2]};}},"window/maximize-button":{include:fx,alias:fx,style:function(iV){return {icon:ec,padding:iV.pressed||iV.abandoned?[2,1,0,3]:[1,2]};}},"window/close-button":{include:fx,alias:fx,style:function(iW){return {marginLeft:2,icon:eC,padding:iW.pressed||iW.abandoned?[2,1,0,3]:[1,2]};}},"window/statusbar":{style:function(iX){return {decorator:gB,padding:[2,6]};}},"window/statusbar-text":eB,"resizer":{style:function(iY){return {decorator:fD};}},"splitpane":{},"splitpane/splitter":{style:function(ja){return {backgroundColor:h};}},"splitpane/splitter/knob":{style:function(jb){return {source:jb.horizontal?dR:dW,padding:2};}},"splitpane/slider":{style:function(jc){return {backgroundColor:X,opacity:0.3};}},"selectbox":{include:gi,style:function(jd){var je=fx;if(jd.invalid&&!jd.disabled){je=eo;}else if(jd.abandoned){je=fS;}else if(!jd.abandoned&&jd.hovered){je=G;}else if(!jd.abandoned&&!jd.hovered&&jd.checked){je=fl;};return {backgroundColor:je};}},"selectbox/atom":gr,"selectbox/popup":gm,"selectbox/list":z,"selectbox/arrow":{include:v,style:function(jf){return {source:fQ,paddingRight:4,paddingLeft:5};}},"datechooser":{style:function(jg){return {decorator:fD};}},"datechooser/navigation-bar":{style:function(jh){return {backgroundColor:eb,textColor:jh.disabled?K:jh.invalid?eT:undefined,padding:[2,10]};}},"datechooser/last-year-button-tooltip":ev,"datechooser/last-month-button-tooltip":ev,"datechooser/next-year-button-tooltip":ev,"datechooser/next-month-button-tooltip":ev,"datechooser/last-year-button":fz,"datechooser/last-month-button":fz,"datechooser/next-year-button":fz,"datechooser/next-month-button":fz,"datechooser/button/icon":{},"datechooser/button":{style:function(ji){var jj={width:17,show:eV};if(ji.lastYear){jj.icon=J;}else if(ji.lastMonth){jj.icon=er;}else if(ji.nextYear){jj.icon=dV;}else if(ji.nextMonth){jj.icon=A;};if(ji.pressed||ji.checked||ji.abandoned){jj.decorator=gB;}else if(ji.hovered){jj.decorator=ef;}else {jj.decorator=undefined;};if(ji.pressed||ji.checked||ji.abandoned){jj.padding=[2,0,0,2];}else if(ji.hovered){jj.padding=1;}else {jj.padding=2;};return jj;}},"datechooser/month-year-label":{style:function(jk){return {font:H,textAlign:fB};}},"datechooser/date-pane":{style:function(jl){return {decorator:Y,backgroundColor:eb};}},"datechooser/weekday":{style:function(jm){return {decorator:eP,font:H,textAlign:fB,textColor:jm.disabled?K:jm.weekend?j:eb,backgroundColor:jm.weekend?eb:j};}},"datechooser/day":{style:function(jn){return {textAlign:fB,decorator:jn.today?gk:undefined,textColor:jn.disabled?K:jn.selected?fp:jn.otherMonth?K:undefined,backgroundColor:jn.disabled?undefined:jn.selected?s:undefined,padding:[2,4]};}},"datechooser/week":{style:function(jo){return {textAlign:fB,textColor:j,padding:[2,4],decorator:jo.header?gp:eE};}},"combobox":{style:function(jp){var jq;if(jp.disabled){jq=g;}else if(jp.invalid){jq=eo;}else if(jp.focused){jq=fv;}else {jq=go;};return {decorator:jp.focused?dS:fs,textColor:jp.disabled?K:undefined,backgroundColor:jq};}},"combobox/button":{alias:fx,include:fx,style:function(jr){return {icon:fQ,backgroundColor:jr.hovered?G:fx};}},"combobox/popup":gm,"combobox/list":z,"combobox/textfield":{include:eg,style:function(js){return {decorator:undefined,padding:[2,3],backgroundColor:undefined};}},"menu":{style:function(jt){var ju={backgroundColor:h,decorator:o,spacingX:6,spacingY:1,iconColumnWidth:16,arrowColumnWidth:4,padding:1,placementModeY:jt.submenu||jt.contextmenu?gA:gG};if(jt.submenu){ju.position=fg;ju.offset=[-2,-3];};if(jt.contextmenu){ju.offset=4;};return ju;}},"menu/slidebar":W,"menu-slidebar":ed,"menu-slidebar-button":{style:function(jv){return {backgroundColor:jv.hovered?gh:undefined,padding:6,center:true};}},"menu-slidebar/button-backward":{include:d,style:function(jw){return {icon:jw.hovered?fA:E};}},"menu-slidebar/button-forward":{include:d,style:function(jx){return {icon:jx.hovered?dY:fQ};}},"menu-separator":{style:function(jy){return {height:0,decorator:gF,marginTop:4,marginBottom:4,marginLeft:2,marginRight:2};}},"menu-button":{alias:gr,style:function(jz){return {backgroundColor:jz.selected?gh:undefined,textColor:jz.selected?fp:undefined,padding:[2,6]};}},"menu-button/icon":{include:v,style:function(jA){return {alignY:eW};}},"menu-button/label":{include:eB,style:function(jB){return {alignY:eW,padding:1};}},"menu-button/shortcut":{include:eB,style:function(jC){return {alignY:eW,marginLeft:14,padding:1};}},"menu-button/arrow":{include:v,style:function(jD){return {source:jD.selected?fU:A,alignY:eW};}},"menu-checkbox":{alias:eH,include:eH,style:function(jE){return {icon:!jE.checked?undefined:jE.selected?ff:C};}},"menu-radiobutton":{alias:eH,include:eH,style:function(jF){return {icon:!jF.checked?undefined:jF.selected?gt:fJ};}},"menubar":{style:function(jG){return {backgroundColor:h,decorator:fD};}},"menubar-button":{alias:gr,style:function(jH){return {padding:[2,6],backgroundColor:jH.pressed||jH.hovered&&!jH.disabled?gh:undefined,textColor:jH.pressed||jH.hovered?fp:undefined};}},"colorselector":ed,"colorselector/control-bar":ed,"colorselector/visual-pane":U,"colorselector/control-pane":ed,"colorselector/preset-grid":ed,"colorselector/colorbucket":{style:function(jI){return {decorator:gB,width:16,height:16};}},"colorselector/preset-field-set":U,"colorselector/input-field-set":{include:U,alias:U,style:function(){return {paddingTop:12};}},"colorselector/preview-field-set":{include:U,alias:U,style:function(){return {paddingTop:12};}},"colorselector/hex-field-composite":ed,"colorselector/hex-field":eg,"colorselector/rgb-spinner-composite":ed,"colorselector/rgb-spinner-red":gj,"colorselector/rgb-spinner-green":gj,"colorselector/rgb-spinner-blue":gj,"colorselector/hsb-spinner-composite":ed,"colorselector/hsb-spinner-hue":gj,"colorselector/hsb-spinner-saturation":gj,"colorselector/hsb-spinner-brightness":gj,"colorselector/preview-content-old":{style:function(jJ){return {decorator:gB,width:50,height:10};}},"colorselector/preview-content-new":{style:function(jK){return {decorator:gB,backgroundColor:N,width:50,height:10};}},"colorselector/hue-saturation-field":{style:function(jL){return {decorator:gB,margin:5};}},"colorselector/brightness-field":{style:function(jM){return {decorator:gB,margin:[5,7]};}},"colorselector/hue-saturation-pane":ed,"colorselector/hue-saturation-handle":ed,"colorselector/brightness-pane":ed,"colorselector/brightness-handle":ed,"table":ed,"table/statusbar":{style:function(jN){return {decorator:dT,paddingLeft:2,paddingRight:2};}},"table/column-button":{alias:fx,style:function(jO){var jQ,jP;if(jO.pressed||jO.checked||jO.abandoned){jQ=gB;jP=[3,2,1,4];}else if(jO.hovered){jQ=ef;jP=[2,3];}else {jQ=undefined;jP=[3,4];};return {decorator:jQ,padding:jP,backgroundColor:jO.abandoned?fS:fx,icon:gx};}},"table-column-reset-button":{extend:eH,alias:eH,style:function(){return {icon:eD};}},"table-scroller/scrollbar-x":fe,"table-scroller/scrollbar-y":fe,"table-scroller":ed,"table-scroller/header":{style:function(jR){return {decorator:L,backgroundColor:gv};}},"table-scroller/pane":{style:function(jS){return {backgroundColor:M};}},"table-scroller/focus-indicator":{style:function(jT){return {decorator:p};}},"table-scroller/resize-line":{style:function(jU){return {backgroundColor:eQ,width:3};}},"table-header-cell":{alias:gr,style:function(jV){return {minWidth:13,paddingLeft:2,paddingRight:2,paddingBottom:jV.hovered?0:2,decorator:jV.hovered?gb:fk,backgroundColor:jV.hovered?O:fk,sortIcon:jV.sorted?(jV.sortedAscending?ew:fH):undefined};}},"table-header-cell/icon":{style:function(jW){return {marginRight:4,opacity:jW.disabled?0.3:1};}},"table-header-cell/sort-icon":{style:function(jX){return {alignY:eW,opacity:jX.disabled?0.3:1};}},"table-editor-textfield":{include:eg,style:function(jY){return {decorator:undefined,padding:[2,2]};}},"table-editor-selectbox":{include:fW,alias:fW,style:function(ka){return {padding:[0,2]};}},"table-editor-combobox":{include:fa,alias:fa,style:function(kb){return {decorator:undefined};}},"progressive-table-header":{alias:ez},"progressive-table-header-cell":{style:function(kc){return {decorator:fk,backgroundColor:fk,padding:[0,6,0,6]};}},"colorpopup":{alias:gm,include:gm,style:function(kd){return {decorator:fD,padding:5,backgroundColor:h};}},"colorpopup/field":{style:function(ke){return {decorator:gB,margin:2,width:14,height:14,backgroundColor:h};}},"colorpopup/selector-button":fx,"colorpopup/auto-button":fx,"colorpopup/preview-pane":U,"colorpopup/current-preview":{style:function(kf){return {height:20,padding:4,marginLeft:4,decorator:gB,allowGrowX:true};}},"colorpopup/selected-preview":{style:function(kg){return {height:20,padding:4,marginRight:4,decorator:gB,allowGrowX:true};}},"colorpopup/colorselector-okbutton":{alias:fx,include:fx,style:function(kh){return {icon:l};}},"colorpopup/colorselector-cancelbutton":{alias:fx,include:fx,style:function(ki){return {icon:V};}},"virtual-list":z,"virtual-list/row-layer":fw,"row-layer":ed,"column-layer":ed,"group-item":{include:eB,alias:eB,style:function(kj){return {padding:4,backgroundColor:eR,textColor:N,font:H};}},"virtual-selectbox":fW,"virtual-selectbox/dropdown":gm,"virtual-selectbox/dropdown/list":{alias:T},"virtual-combobox":fa,"virtual-combobox/dropdown":gm,"virtual-combobox/dropdown/list":{alias:T},"virtual-tree":{include:eL,alias:eL,style:function(kk){return {itemHeight:21};}},"virtual-tree-folder":fb,"virtual-tree-file":fM,"cell":{style:function(kl){return {backgroundColor:kl.selected?el:a,textColor:kl.selected?fp:R,padding:[3,6]};}},"cell-string":fc,"cell-number":{include:fc,style:function(km){return {textAlign:fP};}},"cell-image":fc,"cell-boolean":fc,"cell-atom":fc,"cell-date":fc,"cell-html":fc,"progressbar":{style:function(kn){return {decorator:fN,padding:[1],backgroundColor:N,width:200,height:20};}},"progressbar/progress":{style:function(ko){return {backgroundColor:ko.disabled?g:gh};}},"app-header":{style:function(kp){return {textColor:fp,backgroundColor:gh,padding:[8,12]};}},"app-header-label":eB,"app-splitpane":{alias:eX,style:function(kq){return {padding:[0,10,10,10],backgroundColor:en};}}}});})();(function(){var a="Liberation Sans",b="Verdana",c="Bitstream Vera Sans",d="Lucida Grande",e="Tahoma",f="monospace",g="qx.theme.classic.Font",h="Courier New",i="DejaVu Sans Mono";qx.Theme.define(g,{fonts:{"default":{size:11,lineHeight:1.4,family:[d,e,b,c,a]},"bold":{size:11,lineHeight:1.4,family:[d,e,b,c,a],bold:true},"small":{size:10,lineHeight:1.4,family:[d,e,b,c,a]},"monospace":{size:11,lineHeight:1.4,family:[i,h,f]}}});})();(function(){var a="Oxygen",b="qx.theme.icon.Oxygen",c="qx/icon/Oxygen";qx.Theme.define(b,{title:a,aliases:{"icon":c}});})();(function(){var a="black",b="#888888",c="#3E6CA8",d="#3E5B97",e="#EBE9ED",f="#FFFFE1",g="#F3F8FD",h="#A7A6AA",i="#666666",j="#CBC8CD",k="#FFE0E0",l="#F4F4F4",m="#808080",n="#CCCCCC",o="#C82C2C",p="#DBEAF9",q="#BCCEE5",r="#A5BDDE",s="#7CA0CF",t="#EEE",u="#F3F0F5",v="#F6F5F7",w="#FF9999",x="qx.theme.classic.Color",y="css.rgba",z="#990000",A="#F9F8E9",B="white",C="gray",D="#DCDFE4",E="rgba(0, 0, 0, 0.4)",F="#FAFBFE",G="#AAAAAA",H="#85878C";qx.Theme.define(x,{colors:{"background":e,"background-light":u,"light-background":e,"background-focused":g,"background-focused-inner":p,"background-disabled":l,"background-selected":c,"background-field":B,"background-pane":F,"background-invalid":k,"border-lead":b,"border-light":B,"border-light-shadow":D,"border-dark-shadow":h,"border-dark":H,"border-main":H,"border-focused-light":q,"border-focused-light-shadow":r,"border-focused-dark-shadow":s,"border-focused-dark":c,"border-separator":m,"shadow":qx.core.Environment.get(y)?E:i,"invalid":z,"border-focused-invalid":w,"text":a,"text-disabled":h,"text-selected":B,"text-focused":d,"text-placeholder":j,"tooltip":f,"tooltip-text":a,"tooltip-invalid":o,"button":e,"button-hovered":v,"button-abandoned":A,"button-checked":u,"window-active-caption-text":[255,255,255],"window-inactive-caption-text":[255,255,255],"window-active-caption":[51,94,168],"window-inactive-caption":[111,161,217],"date-chooser":B,"date-chooser-title":[116,116,116],"date-chooser-selected":[52,52,52],"effect":[254,200,60],"table-pane":B,"table-header":[242,242,242],"table-header-border":[214,213,217],"table-header-cell":[235,234,219],"table-header-cell-hover":[255,255,255],"table-focus-indicator":[179,217,255],"table-row-background-focused-selected":[90,138,211],"table-row-background-focused":[221,238,255],"table-row-background-selected":[51,94,168],"table-row-background-even":[250,248,243],"table-row-background-odd":[255,255,255],"table-row-selected":[255,255,255],"table-row":[0,0,0],"table-row-line":t,"table-column-line":t,"progressive-table-header":G,"progressive-table-row-background-even":[250,248,243],"progressive-table-row-background-odd":[255,255,255],"progressive-progressbar-background":C,"progressive-progressbar-indicator-done":n,"progressive-progressbar-indicator-undone":B,"progressive-progressbar-percent-background":C,"progressive-progressbar-percent-text":B}});})();(function(){var a="Classic Windows",b="qx.theme.Classic";qx.Theme.define(b,{title:a,meta:{color:qx.theme.classic.Color,decoration:qx.theme.classic.Decoration,font:qx.theme.classic.Font,appearance:qx.theme.classic.Appearance,icon:qx.theme.icon.Oxygen}});})();(function(){var a="JosefinSlab",b="Verdana",c="qx/decoration/Indigo/font/JosefinSlab-SemiBold.ttf",d="qx/decoration/Indigo/font/JosefinSlab-SemiBold.woff",e="Lucida Grande",f="sans-serif",g="qx.theme.indigo.Font",h="monospace",i="font",j="serif",k="DejaVu Sans",l="Courier New",m="DejaVu Sans Mono";qx.Theme.define(g,{fonts:{"default":{size:12,family:[e,k,b,f],color:i,lineHeight:1.8},"bold":{size:12,family:[e,k,b,f],bold:true,color:i,lineHeight:1.8},"headline":{size:22,family:[j],sources:[{family:a,source:[d,c]}]},"small":{size:11,family:[e,k,b,f],color:i,lineHeight:1.8},"monospace":{size:11,family:[m,l,h],color:i,lineHeight:1.8}}});})();(function(){var a="table-row-background-even",b="button-box-pressed-top-right",c="arrow-left",d="datechooser-weekday",e="arrow-up",f="menu-slidebar-button",g="background-disabled",h="background",j="scrollbar/button",k="icon/16/actions/dialog-ok.png",l="border-invalid",m="combobox/button",n="button-box-top-right",o="slidebar",p="#BABABA",q="button-box-hovered-bottom-right",r="move-frame",s="nodrop",t="window-caption",u="table-header-cell",v="button-box-hovered-top-right",w="row-layer",x="treevirtual-plus-only",y="-right",z="button-frame",A="radiobutton",B="move",C="treevirtual-plus-end",D="background-selected-dark",E="vertical",F="list",G="arrow-down-small",H="arrow-down",I="arrow-",J="-pressed",K="tooltip-error",L="button-box",M="window-restore",N="bold",O="resize-frame",P="text-disabled",Q="scroll-knob",R="tree-minus",S="statusbar",T="white",U="tabview-close",V="down",W="text",X="checkbox",Y="atom/label",eJ="button-box-pressed-bottom-right",eF="button-box-pressed-hovered-bottom-right",eK="background-disabled-checked",eG="groupbox",eH="icon/16/actions/dialog-cancel.png",eE="qx.theme.simple.Appearance",eI="menu-slidebar",eP="-left",eQ="treevirtual-minus-cross",eW="arrow-right",eR="background-pane",eL="table-",eM="scroll-knob-pressed",eN="icon",eO="arrow-rewind",eV="icon/16/apps/office-calendar.png",fz="headline",eX="treevirtual-plus-start",eY="treevirtual-minus-end",eS="middle",eT="-middle",gA="tree",eU="checkbox-undetermined",fa="button-box-bottom-right",fb="datechooser-week",fc="menu-button",fh="descending",fi="splitpane",fj="slidebar/button-forward",fd="toolbar-separator",fe="arrow-up-small",ff="progressive-table-header",fg="invalid",fn="icon/16/places/folder.png",fo="combobox",fp="tree-folder",fq="horizontal",fk="icon/16/mimetypes/text-plain.png",fl="border-light-shadow",gB="tree-plus",fm="text-placeholder",fu="scrollbar",fv="dragover",gG="treevirtual-plus-cross",fw="scrollarea",fr="treevirtual-line",fs="text-selected",gE="cell",ft="menu-checkbox",fx="best-fit",fy="button-border",fK="treevirtual-cross",fJ="default",fI="tabview-page-button-right",fO="button-hover",fN="tabview-page-button-top",fM="tabview-page-button-bottom",fL="inset",fD="tabview-page-button-left",fC="button",fB="menubar-button-pressed",fA="progressbar",fH="tree-file",fG="tooltip-text",fF="keep-align",fE="center",fV="datechooser/button",fU="alias",fT="datechooser",fS="toolbar-button",ga="ascending",fY="button-box-hovered-right-borderless",fX="button-box-right-borderless",fW="lead-item",fR="checkbox-focused",fQ="selectbox",fP="window-minimize",gl="right",gk="button-box-pressed-hovered-top-right",gj="main",gp="image",go="knob-",gn="blank",gm="popup",ge="treevirtual-folder",gd="treevirtual-minus-only",gc="treevirtual-minus-start",gb="checkbox-checked",gi="virtual-list",gh="background-selected",gg="window",gf="-hovered",gv="window-active",gu="table-header-cell-first",gt="left",gs="button-box-pressed-right-borderless",gz="scroll-knob-hovered",gy="up",gx="atom",gw="main-dark",gr="select-column-order",gq="button-box-pressed-hovered-right-borderless",ed="-invalid",ec="scroll-knob-pressed-hovered",gH="white-box",ea="datechooser-week-header",eb="widget",dY="menubar-button-hovered",gF="table-header-column-button",dW="window-close",dX="datechooser-date-pane",dV="cursor-",gC="-focused",dT="menu-radiobutton",dU="window-maximize",dS="treevirtual-end",em="button-box-hovered",en="table",ek="arrow-forward",el="right-top",ei="pointer",ej="focused-inset",eh="slidebar/button-backward",dR="light-background",ef="copy",eg="table-row-background-selected",ee="radiobutton-focused",eA="",ey="textfield",ez="scrollbar/slider/knob",ew="button-box-pressed-hovered",ex="atom/icon",ev="spinner",eB="tooltip",et="-disabled",eu="label",es="table-header",gD="progressive-table-header-cell",eq="menu-separator",er="-invert",eo="link",ep="icon/16/places/folder-open.png",eC="icon/16/actions/view-refresh.png",eD="button-box-pressed";qx.Theme.define(eE,{appearances:{"widget":{},"label":{style:function(gI){return {textColor:gI.disabled?P:undefined};}},"image":{style:function(gJ){return {opacity:!gJ.replacement&&gJ.disabled?0.3:undefined};}},"atom":{},"atom/label":eu,"atom/icon":gp,"root":{style:function(gK){return {backgroundColor:h,textColor:W,font:fJ};}},"popup":{style:function(gL){return {decorator:gm,backgroundColor:eR};}},"tooltip":{include:gm,style:function(gM){return {backgroundColor:eB,textColor:fG,decorator:eB,padding:[1,3,2,3],offset:[10,5,5,5]};}},"tooltip/atom":gx,"tooltip-error":{include:eB,style:function(gN){return {textColor:fs,showTimeout:100,hideTimeout:10000,decorator:K,font:N,backgroundColor:undefined};}},"tooltip-error/atom":gx,"iframe":{style:function(gO){return {backgroundColor:T,decorator:gw};}},"move-frame":{style:function(gP){return {decorator:gw};}},"resize-frame":r,"dragdrop-cursor":{style:function(gQ){var gR=s;if(gQ.copy){gR=ef;}else if(gQ.move){gR=B;}else if(gQ.alias){gR=fU;};return {source:qx.theme.simple.Image.URLS[dV+gR],position:el,offset:[2,16,2,6]};}},"slidebar":{},"slidebar/scrollpane":{},"slidebar/content":{},"slidebar/button-forward":{alias:fC,include:fC,style:function(gS){return {icon:qx.theme.simple.Image.URLS[I+(gS.vertical?V:gl)]};}},"slidebar/button-backward":{alias:fC,include:fC,style:function(gT){return {icon:qx.theme.simple.Image.URLS[I+(gT.vertical?gy:gt)]};}},"table":eb,"table/statusbar":{style:function(gU){return {decorator:S,padding:[2,5]};}},"table/column-button":{alias:fC,style:function(gV){return {decorator:gF,padding:3,icon:qx.theme.simple.Image.URLS[gr]};}},"table-column-reset-button":{include:fc,alias:fc,style:function(){return {icon:eC};}},"table-scroller/scrollbar-x":fu,"table-scroller/scrollbar-y":fu,"table-scroller":eb,"table-scroller/header":{style:function(){return {decorator:es};}},"table-scroller/pane":{},"table-scroller/focus-indicator":{style:function(gW){return {decorator:gj};}},"table-scroller/resize-line":{style:function(gX){return {backgroundColor:fy,width:3};}},"table-header-cell":{alias:gx,style:function(gY){return {decorator:gY.first?gu:u,minWidth:13,font:N,paddingTop:3,paddingLeft:5,cursor:gY.disabled?undefined:ei,sortIcon:gY.sorted?(qx.theme.simple.Image.URLS[eL+(gY.sortedAscending?ga:fh)]):undefined};}},"table-header-cell/icon":{include:ex,style:function(ha){return {paddingRight:5};}},"table-header-cell/sort-icon":{style:function(hb){return {alignY:eS,alignX:gl,paddingRight:5};}},"table-editor-textfield":{include:ey,style:function(hc){return {decorator:undefined,padding:[2,2]};}},"table-editor-selectbox":{include:fQ,alias:fQ,style:function(hd){return {padding:[0,2]};}},"table-editor-combobox":{include:fo,alias:fo,style:function(he){return {decorator:undefined};}},"progressive-table-header":{style:function(hf){return {decorator:ff};}},"progressive-table-header-cell":{style:function(hg){return {decorator:gD,padding:[5,6,5,6]};}},"treevirtual":{include:ey,alias:en,style:function(hh,hi){return {padding:[hi.padding[0]+2,hi.padding[1]+1]};}},"treevirtual-folder":{style:function(hj){return {icon:(hj.opened?ep:fn),opacity:hj.drag?0.5:undefined};}},"treevirtual-file":{include:ge,alias:ge,style:function(hk){return {icon:fk,opacity:hk.drag?0.5:undefined};}},"treevirtual-line":{style:function(hl){return {icon:qx.theme.simple.Image.URLS[fr]};}},"treevirtual-contract":{style:function(hm){return {icon:qx.theme.simple.Image.URLS[R]};}},"treevirtual-expand":{style:function(hn){return {icon:qx.theme.simple.Image.URLS[gB]};}},"treevirtual-only-contract":{style:function(ho){return {icon:qx.theme.simple.Image.URLS[gd]};}},"treevirtual-only-expand":{style:function(hp){return {icon:qx.theme.simple.Image.URLS[x]};}},"treevirtual-start-contract":{style:function(hq){return {icon:qx.theme.simple.Image.URLS[gc]};}},"treevirtual-start-expand":{style:function(hr){return {icon:qx.theme.simple.Image.URLS[eX]};}},"treevirtual-end-contract":{style:function(hs){return {icon:qx.theme.simple.Image.URLS[eY]};}},"treevirtual-end-expand":{style:function(ht){return {icon:qx.theme.simple.Image.URLS[C]};}},"treevirtual-cross-contract":{style:function(hu){return {icon:qx.theme.simple.Image.URLS[eQ]};}},"treevirtual-cross-expand":{style:function(hv){return {icon:qx.theme.simple.Image.URLS[gG]};}},"treevirtual-end":{style:function(hw){return {icon:qx.theme.simple.Image.URLS[dS]};}},"treevirtual-cross":{style:function(hx){return {icon:qx.theme.simple.Image.URLS[fK]};}},"resizer":{style:function(hy){return {decorator:gw};}},"splitpane":{},"splitpane/splitter":{style:function(hz){return {backgroundColor:dR};}},"splitpane/splitter/knob":{style:function(hA){return {source:qx.theme.simple.Image.URLS[go+(hA.horizontal?fq:E)],padding:2};}},"splitpane/slider":{style:function(hB){return {backgroundColor:fl,opacity:0.3};}},"menu":{style:function(hC){var hD={backgroundColor:h,decorator:gj,spacingX:6,spacingY:1,iconColumnWidth:16,arrowColumnWidth:4,padding:1,placementModeY:hC.submenu||hC.contextmenu?fx:fF};if(hC.submenu){hD.position=el;hD.offset=[-2,-3];};if(hC.contextmenu){hD.offset=4;};return hD;}},"menu/slidebar":eI,"menu-slidebar":eb,"menu-slidebar-button":{style:function(hE){return {backgroundColor:hE.hovered?gh:undefined,padding:6,center:true};}},"menu-slidebar/button-backward":{include:f,style:function(hF){return {icon:qx.theme.simple.Image.URLS[e+(hF.hovered?er:eA)]};}},"menu-slidebar/button-forward":{include:f,style:function(hG){return {icon:qx.theme.simple.Image.URLS[H+(hG.hovered?er:eA)]};}},"menu-separator":{style:function(hH){return {height:0,decorator:eq,marginTop:4,marginBottom:4,marginLeft:2,marginRight:2};}},"menu-button":{alias:gx,style:function(hI){return {backgroundColor:hI.selected?gh:undefined,textColor:hI.selected?fs:undefined,padding:[2,6]};}},"menu-button/icon":{include:gp,style:function(hJ){return {alignY:eS};}},"menu-button/label":{include:eu,style:function(hK){return {alignY:eS,padding:1};}},"menu-button/shortcut":{include:eu,style:function(hL){return {alignY:eS,marginLeft:14,padding:1};}},"menu-button/arrow":{include:gp,style:function(hM){return {source:qx.theme.simple.Image.URLS[eW+(hM.selected?er:eA)],alignY:eS};}},"menu-checkbox":{alias:fc,include:fc,style:function(hN){return {icon:!hN.checked?undefined:qx.theme.simple.Image.URLS[ft+(hN.selected?er:eA)]};}},"menu-radiobutton":{alias:fc,include:fc,style:function(hO){return {icon:!hO.checked?undefined:qx.theme.simple.Image.URLS[dT+(hO.selected?er:eA)]};}},"menubar":{style:function(hP){return {backgroundColor:dR,padding:[4,2]};}},"menubar-button":{style:function(hQ){var hS;var hR=[2,6];if(!hQ.disabled){if(hQ.pressed){hS=fB;hR=[1,5,2,5];}else if(hQ.hovered){hS=dY;hR=[1,5];};};return {padding:hR,cursor:hQ.disabled?undefined:ei,textColor:eo,decorator:hS};}},"virtual-list":F,"virtual-list/row-layer":w,"row-layer":eb,"column-layer":eb,"group-item":{include:eu,alias:eu,style:function(hT){return {padding:4,backgroundColor:p,textColor:T,font:N};}},"virtual-selectbox":fQ,"virtual-selectbox/dropdown":gm,"virtual-selectbox/dropdown/list":{alias:gi},"virtual-combobox":fo,"virtual-combobox/dropdown":gm,"virtual-combobox/dropdown/list":{alias:gi},"virtual-tree":{include:gA,alias:gA,style:function(hU){return {itemHeight:21};}},"virtual-tree-folder":fp,"virtual-tree-file":fH,"cell":{style:function(hV){return {backgroundColor:hV.selected?eg:a,textColor:hV.selected?fs:W,padding:[3,6]};}},"cell-string":gE,"cell-number":{include:gE,style:function(hW){return {textAlign:gl};}},"cell-image":gE,"cell-boolean":gE,"cell-atom":gE,"cell-date":gE,"cell-html":gE,"scrollbar":{},"scrollbar/slider":{},"scrollbar/slider/knob":{style:function(hX){var hY=Q;if(!hX.disabled){if(hX.hovered&&!hX.pressed&&!hX.checked){hY=gz;}else if(hX.hovered&&(hX.pressed||hX.checked)){hY=ec;}else if(hX.pressed||hX.checked){hY=eM;};};return {height:14,width:14,cursor:hX.disabled?undefined:ei,decorator:hY,minHeight:hX.horizontal?undefined:20,minWidth:hX.horizontal?20:undefined};}},"scrollbar/button":{style:function(ia){var ib={};ib.padding=4;var ic=eA;if(ia.left){ic=gt;ib.marginRight=2;}else if(ia.right){ic+=gl;ib.marginLeft=2;}else if(ia.up){ic+=gy;ib.marginBottom=2;}else {ic+=V;ib.marginTop=2;};ib.icon=qx.theme.simple.Image.URLS[I+ic];ib.cursor=ei;ib.decorator=L;return ib;}},"scrollbar/button-begin":j,"scrollbar/button-end":j,"scrollarea/corner":{style:function(id){return {backgroundColor:h};}},"scrollarea":eb,"scrollarea/pane":eb,"scrollarea/scrollbar-x":fu,"scrollarea/scrollbar-y":fu,"textfield":{style:function(ie){var ih;if(ie.disabled){ih=P;}else if(ie.showingPlaceholder){ih=fm;}else {ih=undefined;};var ii;var ig;if(ie.disabled){ii=fL;ig=[2,3];}else if(ie.invalid){ii=l;ig=[1,2];}else if(ie.focused){ii=ej;ig=[1,2];}else {ig=[2,3];ii=fL;};return {decorator:ii,padding:ig,textColor:ih,backgroundColor:ie.disabled?g:T};}},"textarea":ey,"radiobutton/icon":{style:function(ij){var ik=A;if(ij.focused&&!ij.invalid){ik=ee;};ik+=ij.invalid&&!ij.disabled?ed:eA;var il;if(ij.disabled&&ij.checked){il=eK;}else if(ij.disabled){il=g;}else if(ij.checked){il=gh;};return {decorator:ik,width:12,height:12,backgroundColor:il};}},"radiobutton":{style:function(im){return {icon:qx.theme.simple.Image.URLS[gn]};}},"form-renderer-label":{include:eu,style:function(){return {paddingTop:3};}},"checkbox":{alias:gx,style:function(io){var ip;if(io.checked){ip=qx.theme.simple.Image.URLS[gb];}else if(io.undetermined){ip=qx.theme.simple.Image.URLS[eU];}else {ip=qx.theme.simple.Image.URLS[gn];};return {icon:ip,gap:6};}},"checkbox/icon":{style:function(iq){var is=X;if(iq.focused&&!iq.invalid){is=fR;};is+=iq.invalid&&!iq.disabled?ed:eA;var ir;if(iq.checked){ir=2;}else if(iq.undetermined){ir=[4,2];};return {decorator:is,width:12,height:12,padding:ir,backgroundColor:T};}},"spinner":{style:function(it){return {textColor:it.disabled?P:undefined};}},"spinner/textfield":ey,"spinner/upbutton":{alias:m,include:m,style:function(iu){var iv=n;if(iu.hovered&&!iu.pressed&&!iu.checked){iv=v;}else if(iu.hovered&&(iu.pressed||iu.checked)){iv=gk;}else if(iu.pressed||iu.checked){iv=b;};return {icon:qx.theme.simple.Image.URLS[fe],decorator:iv,width:17};}},"spinner/downbutton":{alias:m,include:m,style:function(iw){var ix=fa;if(iw.hovered&&!iw.pressed&&!iw.checked){ix=q;}else if(iw.hovered&&(iw.pressed||iw.checked)){ix=eF;}else if(iw.pressed||iw.checked){ix=eJ;};return {icon:qx.theme.simple.Image.URLS[G],decorator:ix,width:17};}},"selectbox":z,"selectbox/atom":gx,"selectbox/popup":gm,"selectbox/list":{alias:F,include:F,style:function(){return {decorator:undefined};}},"selectbox/arrow":{include:gp,style:function(iy){return {source:qx.theme.simple.Image.URLS[H],paddingRight:4,paddingLeft:5};}},"combobox":{},"combobox/button":{alias:z,include:z,style:function(iz){var iA=fX;if(iz.hovered&&!iz.pressed&&!iz.checked){iA=fY;}else if(iz.hovered&&(iz.pressed||iz.checked)){iA=gq;}else if(iz.pressed||iz.checked){iA=gs;};return {icon:qx.theme.simple.Image.URLS[H],decorator:iA,padding:[0,5],width:19};}},"combobox/popup":gm,"combobox/list":{alias:F},"combobox/textfield":ey,"datefield":ey,"datefield/button":{alias:m,include:m,style:function(iB){return {icon:eV,padding:[0,0,0,3],backgroundColor:undefined,decorator:undefined,width:19};}},"datefield/textfield":{alias:ey,include:ey,style:function(iC){return {decorator:undefined,padding:0};}},"datefield/list":{alias:fT,include:fT,style:function(iD){return {decorator:undefined};}},"list":{alias:fw,include:ey},"listitem":{alias:gx,style:function(iE){var iF=[3,5,3,5];if(iE.lead){iF=[2,4,2,4];};if(iE.dragover){iF[2]-=2;};var iG;if(iE.selected){iG=gh;if(iE.disabled){iG+=et;};};return {gap:4,padding:iF,backgroundColor:iG,textColor:iE.selected?fs:undefined,decorator:iE.lead?fW:iE.dragover?fv:undefined,opacity:iE.drag?0.5:undefined};}},"slider":{style:function(iH){var iJ;var iI;if(iH.disabled){iJ=fL;iI=[2,3];}else if(iH.invalid){iJ=l;iI=[1,2];}else if(iH.focused){iJ=ej;iI=[1,2];}else {iI=[2,3];iJ=fL;};return {decorator:iJ,padding:iI};}},"slider/knob":ez,"button-frame":{alias:gx,style:function(iK){var iL=L;if(!iK.disabled){if(iK.hovered&&!iK.pressed&&!iK.checked){iL=em;}else if(iK.hovered&&(iK.pressed||iK.checked)){iL=ew;}else if(iK.pressed||iK.checked){iL=eD;};};if(iK.invalid&&!iK.disabled){iL+=ed;}else if(iK.focused){iL+=gC;};return {decorator:iL,padding:[3,8],cursor:iK.disabled?undefined:ei,minWidth:5,minHeight:5};}},"button-frame/label":{alias:Y,style:function(iM){return {textColor:iM.disabled?P:undefined};}},"button":{alias:z,include:z,style:function(iN){return {center:true};}},"hover-button":{alias:fC,include:fC,style:function(iO){return {decorator:iO.hovered?fO:undefined};}},"menubutton":{include:fC,alias:fC,style:function(iP){return {icon:qx.theme.simple.Image.URLS[H],iconPosition:gl};}},"splitbutton":{},"splitbutton/button":{alias:gx,style:function(iQ){var iR=L;if(!iQ.disabled){if(iQ.pressed||iQ.checked){iR+=J;};if(iQ.hovered){iR+=gf;};};if(iQ.focused){iR+=gC;};iR+=eP;return {decorator:iR,padding:[3,8],cursor:iQ.disabled?undefined:ei};}},"splitbutton/arrow":{style:function(iS){var iT=L;if(!iS.disabled){if(iS.pressed||iS.checked){iT+=J;};if(iS.hovered){iT+=gf;};};if(iS.focused){iT+=gC;};iT+=y;return {icon:qx.theme.simple.Image.URLS[H],decorator:iT,cursor:iS.disabled?undefined:ei,padding:[3,4]};}},"groupbox":{},"groupbox/legend":{alias:gx,style:function(iU){return {textColor:iU.invalid?fg:undefined,padding:5,margin:4,font:N};}},"groupbox/frame":{style:function(iV){return {backgroundColor:h,padding:[6,9],margin:[18,2,2,2],decorator:gH};}},"check-groupbox":eG,"check-groupbox/legend":{alias:X,include:X,style:function(iW){return {textColor:iW.invalid?fg:undefined,padding:5,margin:4,font:N};}},"radio-groupbox":eG,"radio-groupbox/legend":{alias:A,include:A,style:function(iX){return {textColor:iX.invalid?fg:undefined,padding:5,margin:4,font:N};}},"tree-folder/open":{include:gp,style:function(iY){return {source:iY.opened?qx.theme.simple.Image.URLS[R]:qx.theme.simple.Image.URLS[gB]};}},"tree-folder":{style:function(ja){var jb;if(ja.selected){jb=gh;if(ja.disabled){jb+=et;};};return {padding:[2,8,2,5],icon:ja.opened?ep:fn,backgroundColor:jb,iconOpened:ep,opacity:ja.drag?0.5:undefined};}},"tree-folder/icon":{include:gp,style:function(jc){return {padding:[0,4,0,0]};}},"tree-folder/label":{style:function(jd){return {padding:[1,2],textColor:jd.selected&&!jd.disabled?fs:undefined};}},"tree-file":{include:fp,alias:fp,style:function(je){return {icon:fk,opacity:je.drag?0.5:undefined};}},"tree":{include:F,alias:F,style:function(jf){return {contentPadding:jf.invalid&&!jf.disabled?[3,0]:[4,1],padding:jf.focused?0:1};}},"window":{style:function(jg){return {contentPadding:[10,10,10,10],backgroundColor:h,decorator:jg.maximized?undefined:jg.active?gv:gg};}},"window-resize-frame":O,"window/pane":{},"window/captionbar":{style:function(jh){return {backgroundColor:jh.active?dR:g,padding:8,font:N,decorator:t};}},"window/icon":{style:function(ji){return {marginRight:4};}},"window/title":{style:function(jj){return {cursor:fJ,font:N,marginRight:20,alignY:eS};}},"window/minimize-button":{alias:fC,style:function(jk){return {icon:qx.theme.simple.Image.URLS[fP],padding:[1,2],cursor:jk.disabled?undefined:ei};}},"window/restore-button":{alias:fC,style:function(jl){return {icon:qx.theme.simple.Image.URLS[M],padding:[1,2],cursor:jl.disabled?undefined:ei};}},"window/maximize-button":{alias:fC,style:function(jm){return {icon:qx.theme.simple.Image.URLS[dU],padding:[1,2],cursor:jm.disabled?undefined:ei};}},"window/close-button":{alias:fC,style:function(jn){return {marginLeft:2,icon:qx.theme.simple.Image.URLS[dW],padding:[1,2],cursor:jn.disabled?undefined:ei};}},"window/statusbar":{style:function(jo){return {decorator:S,padding:[2,6]};}},"window/statusbar-text":eu,"datechooser":{style:function(jp){return {decorator:gj,minWidth:220};}},"datechooser/navigation-bar":{style:function(jq){return {backgroundColor:h,textColor:jq.disabled?P:jq.invalid?fg:undefined,padding:[2,10]};}},"datechooser/last-year-button-tooltip":eB,"datechooser/last-month-button-tooltip":eB,"datechooser/next-year-button-tooltip":eB,"datechooser/next-month-button-tooltip":eB,"datechooser/last-year-button":fV,"datechooser/last-month-button":fV,"datechooser/next-year-button":fV,"datechooser/next-month-button":fV,"datechooser/button/icon":{},"datechooser/button":{style:function(jr){var js={width:17,show:eN,cursor:jr.disabled?undefined:ei};if(jr.lastYear){js.icon=qx.theme.simple.Image.URLS[eO];}else if(jr.lastMonth){js.icon=qx.theme.simple.Image.URLS[c];}else if(jr.nextYear){js.icon=qx.theme.simple.Image.URLS[ek];}else if(jr.nextMonth){js.icon=qx.theme.simple.Image.URLS[eW];};return js;}},"datechooser/month-year-label":{style:function(jt){return {font:N,textAlign:fE};}},"datechooser/date-pane":{style:function(ju){return {decorator:dX,backgroundColor:h};}},"datechooser/weekday":{style:function(jv){return {decorator:d,font:N,textAlign:fE,textColor:jv.disabled?P:jv.weekend?D:h,backgroundColor:jv.weekend?h:D,paddingTop:2};}},"datechooser/day":{style:function(jw){return {textAlign:fE,decorator:jw.today?gj:undefined,textColor:jw.disabled?P:jw.selected?fs:jw.otherMonth?P:undefined,backgroundColor:jw.disabled?undefined:jw.selected?gh:undefined,padding:jw.today?[1,3]:[2,4]};}},"datechooser/week":{style:function(jx){return {textAlign:fE,textColor:D,padding:[2,4],decorator:jx.header?ea:fb};}},"progressbar":{style:function(jy){return {decorator:fA,padding:1,backgroundColor:T,width:200,height:20};}},"progressbar/progress":{style:function(jz){return {backgroundColor:jz.disabled?eK:gh};}},"toolbar":{style:function(jA){return {backgroundColor:dR,padding:0};}},"toolbar/part":{style:function(jB){return {margin:[0,15]};}},"toolbar/part/container":{},"toolbar/part/handle":{},"toolbar-separator":{style:function(jC){return {decorator:fd,margin:[7,0],width:4};}},"toolbar-button":{alias:gx,style:function(jD){var jF=L;if(jD.disabled){jF=L;}else if(jD.hovered&&!jD.pressed&&!jD.checked){jF=em;}else if(jD.hovered&&(jD.pressed||jD.checked)){jF=ew;}else if(jD.pressed||jD.checked){jF=eD;};if(jD.left){jF+=eP;}else if(jD.right){jF+=y;}else if(jD.middle){jF+=eT;};var jE=[7,10];if(jD.left||jD.middle||jD.right){jE=[7,0];};return {cursor:jD.disabled?undefined:ei,decorator:jF,margin:jE,padding:[3,5]};}},"toolbar-menubutton":{alias:fS,include:fS,style:function(jG){return {showArrow:true};}},"toolbar-menubutton/arrow":{alias:gp,include:gp,style:function(jH){return {source:qx.theme.simple.Image.URLS[H],cursor:jH.disabled?undefined:ei,padding:[0,5],marginLeft:2};}},"toolbar-splitbutton":{},"toolbar-splitbutton/button":{alias:fS,include:fS,style:function(jI){var jJ=L;if(jI.disabled){jJ=L;}else if(jI.hovered&&!jI.pressed&&!jI.checked){jJ=em;}else if(jI.hovered&&(jI.pressed||jI.checked)){jJ=ew;}else if(jI.pressed||jI.checked){jJ=eD;};if(jI.left){jJ+=eP;}else if(jI.right){jJ+=eT;}else if(jI.middle){jJ+=eT;};return {icon:qx.theme.simple.Image.URLS[H],decorator:jJ};}},"toolbar-splitbutton/arrow":{alias:fS,include:fS,style:function(jK){var jL=L;if(jK.disabled){jL=L;}else if(jK.hovered&&!jK.pressed&&!jK.checked){jL=em;}else if(jK.hovered&&(jK.pressed||jK.checked)){jL=ew;}else if(jK.pressed||jK.checked){jL=eD;};if(jK.left){jL+=eT;}else if(jK.right){jL+=y;}else if(jK.middle){jL+=eT;};return {icon:qx.theme.simple.Image.URLS[H],decorator:jL};}},"tabview":{},"tabview/bar":{alias:o,style:function(jM){var jN=0,jQ=0,jO=0,jP=0;if(jM.barTop){jO-=1;}else if(jM.barBottom){jN-=1;}else if(jM.barRight){jP-=1;}else {jQ-=1;};return {marginBottom:jO,marginTop:jN,marginLeft:jP,marginRight:jQ};}},"tabview/bar/button-forward":{include:fj,alias:fj,style:function(jR){if(jR.barTop){return {marginTop:4,marginBottom:2,decorator:null};}else if(jR.barBottom){return {marginTop:2,marginBottom:4,decorator:null};}else if(jR.barLeft){return {marginLeft:4,marginRight:2,decorator:null};}else {return {marginLeft:2,marginRight:4,decorator:null};};}},"tabview/bar/button-backward":{include:eh,alias:eh,style:function(jS){if(jS.barTop){return {marginTop:4,marginBottom:2,decorator:null};}else if(jS.barBottom){return {marginTop:2,marginBottom:4,decorator:null};}else if(jS.barLeft){return {marginLeft:4,marginRight:2,decorator:null};}else {return {marginLeft:2,marginRight:4,decorator:null};};}},"tabview/pane":{style:function(jT){return {backgroundColor:h,decorator:gj,padding:10};}},"tabview-page":eb,"tabview-page/button":{style:function(jU){var jW;if(jU.barTop||jU.barBottom){var jV=[8,16,8,13];}else {var jV=[8,4,8,4];};if(jU.checked){if(jU.barTop){jW=fN;}else if(jU.barBottom){jW=fM;}else if(jU.barRight){jW=fI;}else if(jU.barLeft){jW=fD;};}else {for(var i=0;i<jV.length;i++ ){jV[i]+=1;};if(jU.barTop){jV[2]-=1;}else if(jU.barBottom){jV[0]-=1;}else if(jU.barRight){jV[3]-=1;}else if(jU.barLeft){jV[1]-=1;};};return {zIndex:jU.checked?10:5,decorator:jW,textColor:jU.disabled?P:jU.checked?null:eo,padding:jV,cursor:ei};}},"tabview-page/button/label":{alias:eu,style:function(jX){return {padding:[0,1,0,1]};}},"tabview-page/button/icon":gp,"tabview-page/button/close-button":{alias:gx,style:function(jY){return {cursor:jY.disabled?undefined:ei,icon:qx.theme.simple.Image.URLS[U]};}},"colorpopup":{alias:gm,include:gm,style:function(ka){return {padding:5};}},"colorpopup/field":{style:function(kb){return {margin:2,width:14,height:14,backgroundColor:h,decorator:gw};}},"colorpopup/selector-button":fC,"colorpopup/auto-button":fC,"colorpopup/preview-pane":eG,"colorpopup/current-preview":{style:function(kc){return {height:20,padding:4,marginLeft:4,decorator:gw,allowGrowX:true};}},"colorpopup/selected-preview":{style:function(kd){return {height:20,padding:4,marginRight:4,decorator:gw,allowGrowX:true};}},"colorpopup/colorselector-okbutton":{alias:fC,include:fC,style:function(ke){return {icon:k};}},"colorpopup/colorselector-cancelbutton":{alias:fC,include:fC,style:function(kf){return {icon:eH};}},"colorselector":eb,"colorselector/control-bar":eb,"colorselector/visual-pane":eG,"colorselector/control-pane":eb,"colorselector/preset-grid":eb,"colorselector/colorbucket":{style:function(kg){return {decorator:gw,width:16,height:16};}},"colorselector/preset-field-set":eG,"colorselector/input-field-set":{include:eG,alias:eG,style:function(){return {paddingTop:12};}},"colorselector/preview-field-set":{include:eG,alias:eG,style:function(){return {paddingTop:12};}},"colorselector/hex-field-composite":eb,"colorselector/hex-field":ey,"colorselector/rgb-spinner-composite":eb,"colorselector/rgb-spinner-red":ev,"colorselector/rgb-spinner-green":ev,"colorselector/rgb-spinner-blue":ev,"colorselector/hsb-spinner-composite":eb,"colorselector/hsb-spinner-hue":ev,"colorselector/hsb-spinner-saturation":ev,"colorselector/hsb-spinner-brightness":ev,"colorselector/preview-content-old":{style:function(kh){return {decorator:gw,width:50,height:25};}},"colorselector/preview-content-new":{style:function(ki){return {decorator:gw,backgroundColor:T,width:50,height:25};}},"colorselector/hue-saturation-field":{style:function(kj){return {decorator:gw,margin:5};}},"colorselector/brightness-field":{style:function(kk){return {decorator:gw,margin:[5,7]};}},"colorselector/hue-saturation-pane":eb,"colorselector/hue-saturation-handle":eb,"colorselector/brightness-pane":eb,"colorselector/brightness-handle":eb,"app-header":{style:function(kl){return {font:fz,textColor:fs,backgroundColor:D,padding:[8,12]};}},"app-header-label":{style:function(km){return {paddingTop:5};}},"app-splitpane":{alias:fi,style:function(kn){return {padding:[0,10,10,10],backgroundColor:dR};}}}});})();(function(){var a="decoration/table/select-column-order.png",b="decoration/treevirtual/end.gif",c="decoration/checkbox/checked.png",d="decoration/arrows/right.gif",e="decoration/window/maximize.gif",f="decoration/treevirtual/only_plus.gif",g="qx.theme.simple.Image",h="decoration/cursors/move.gif",i="decoration/menu/checkbox.gif",j="decoration/table/ascending.png",k="decoration/arrows/down-small.gif",l="decoration/checkbox/undetermined.png",m="decoration/splitpane/knob-vertical.png",n="decoration/arrows/forward.gif",o="decoration/arrows/up-small.gif",p="decoration/arrows/up-invert.gif",q="decoration/treevirtual/cross_plus.gif",r="decoration/window/minimize.gif",s="qx/static/blank.png",t="decoration/tree/minus.gif",u="decoration/arrows/down-invert.gif",v="decoration/arrows/right-invert.gif",w="decoration/cursors/alias.gif",x="decoration/splitpane/knob-horizontal.png",y="decoration/treevirtual/only_minus.gif",z="decoration/treevirtual/start_plus.gif",A="decoration/cursors/nodrop.gif",B="decoration/cursors/copy.gif",C="decoration/arrows/down.gif",D="decoration/treevirtual/end_plus.gif",E="decoration/treevirtual/start_minus.gif",F="decoration/treevirtual/cross.gif",G="decoration/menu/radiobutton.gif",H="decoration/treevirtual/line.gif",I="decoration/arrows/up.gif",J="decoration/tabview/close.gif",K="decoration/tree/plus.gif",L="decoration/arrows/rewind.gif",M="decoration/window/restore.gif",N="decoration/table/descending.png",O="decoration/menu/checkbox-invert.gif",P="decoration/treevirtual/cross_minus.gif",Q="decoration/treevirtual/end_minus.gif",R="decoration/arrows/left.gif",S="decoration/menu/radiobutton-invert.gif",T="decoration/window/close.gif";qx.Class.define(g,{extend:qx.core.Object,statics:{URLS:{"blank":s,"checkbox-checked":c,"checkbox-undetermined":l,"window-minimize":r,"window-maximize":e,"window-restore":M,"window-close":T,"cursor-copy":B,"cursor-move":h,"cursor-alias":w,"cursor-nodrop":A,"arrow-right":d,"arrow-left":R,"arrow-up":I,"arrow-down":C,"arrow-forward":n,"arrow-rewind":L,"arrow-down-small":k,"arrow-up-small":o,"arrow-up-invert":p,"arrow-down-invert":u,"arrow-right-invert":v,"knob-horizontal":x,"knob-vertical":m,"tree-minus":t,"tree-plus":K,"select-column-order":a,"table-ascending":j,"table-descending":N,"treevirtual-line":H,"treevirtual-minus-only":y,"treevirtual-plus-only":f,"treevirtual-minus-start":E,"treevirtual-plus-start":z,"treevirtual-minus-end":Q,"treevirtual-plus-end":D,"treevirtual-minus-cross":P,"treevirtual-plus-cross":q,"treevirtual-end":b,"treevirtual-cross":F,"menu-checkbox":i,"menu-checkbox-invert":O,"menu-radiobutton-invert":S,"menu-radiobutton":G,"tabview-close":J}}});})();(function(){var a="knob-",b="window",c="vertical",d="font",e="window-caption-active",f="window-caption",g="headline",h="groupbox",i="background",j="splitpane",k="window-active",l="highlight",m="default",n="tree",o="middle",p="horizontal",q="app-header",r="text-selected",s="light-background",t="qx.theme.indigo.Appearance";qx.Theme.define(t,{extend:qx.theme.simple.Appearance,appearances:{"colorselector/input-field-set":{include:h,alias:h,style:function(){return {paddingTop:0};}},"colorselector/preview-field-set":{include:h,alias:h,style:function(){return {paddingTop:0};}},"toolbar":{style:function(u){return {backgroundColor:s,padding:[4,0]};}},"splitpane/splitter/knob":{style:function(v){return {source:qx.theme.simple.Image.URLS[a+(v.horizontal?p:c)],padding:3};}},"window":{style:function(w){return {contentPadding:[10,10,10,10],backgroundColor:w.maximized?i:undefined,decorator:w.maximized?undefined:w.active?k:b};}},"window/captionbar":{style:function(x){var y=x.active&&!x.disabled;return {padding:[3,8,y?1:3,8],textColor:y?l:d,decorator:y?e:f};}},"window/title":{style:function(z){return {cursor:m,font:m,marginRight:20,alignY:o};}},"virtual-tree":{include:n,alias:n,style:function(A){return {itemHeight:27};}},"app-header":{style:function(B){return {font:g,textColor:r,decorator:q,padding:10};}},"app-header-label":{style:function(C){return {paddingTop:5};}},"app-splitpane":{alias:j,style:function(D){return {padding:[0,10,10,10],backgroundColor:s};}}}});})();(function(){var a="button-box-dark-pressed",b="checkbox",c="tabview-page-button-top",d="button-border",e="table-header",f="button-box-invalid",g="button-border-hovered",h="menubar-button-hovered",i="button-box-dark",j="#999999",k="button-box-hovered-focused",l="solid",m="qx/decoration/Simple",n="dotted",o="border-separator",p="shadow",q="window-border",r="tooltip-text",s="button-box-hovered",t="table-focus-indicator",u="button-box-pressed-invalid",v="dark-blue",w="scrollbar-dark",x="radiobutton",y="scroll-knob",z="qx.theme.simple.Decoration",A="button-box-focused",B="table-header-cell",C="button",D="scroll-knob-pressed",E="border-lead",F="button-box-pressed-hovered",G="border-main",H="#FFF",I="button-box-pressed-focused",J="invalid",K="button-box",L="background",M="scrollbar-bright",N="button-box-bright",O="window-border-inner",P="border-light-shadow",Q="white-box-border",R="background-selected",S="window",T="white",U="gray",V="border-light",W="button-box-bright-pressed",X="button-box-pressed-hovered-focused",Y="button-box-pressed";qx.Theme.define(z,{aliases:{decoration:m},decorations:{"border-blue":{style:{width:4,color:R}},"main":{style:{width:1,color:G}},"main-dark":{style:{width:1,color:d}},"popup":{style:{width:1,color:q,shadowLength:2,shadowBlurRadius:5,shadowColor:p}},"dragover":{style:{bottom:[2,l,v]}},"button-box":{style:{radius:3,width:1,color:d,gradientStart:[N,40],gradientEnd:[i,70],backgroundColor:N}},"button-box-pressed":{include:K,style:{gradientStart:[W,40],gradientEnd:[a,70],backgroundColor:W}},"button-box-pressed-hovered":{include:Y,style:{color:g}},"button-box-hovered":{include:K,style:{color:g}},"button-box-invalid":{include:K,style:{color:J}},"button-box-pressed-invalid":{include:Y,style:{color:J}},"button-box-hovered-invalid":{include:f},"button-box-pressed-hovered-invalid":{include:u},"button-box-focused":{include:K,style:{color:R}},"button-box-pressed-focused":{include:Y,style:{color:R}},"button-box-hovered-focused":{include:A},"button-box-pressed-hovered-focused":{include:I},"button-box-right":{include:K,style:{radius:[0,3,3,0]}},"button-box-pressed-right":{include:Y,style:{radius:[0,3,3,0]}},"button-box-pressed-hovered-right":{include:F,style:{radius:[0,3,3,0]}},"button-box-hovered-right":{include:s,style:{radius:[0,3,3,0]}},"button-box-focused-right":{include:A,style:{radius:[0,3,3,0]}},"button-box-hovered-focused-right":{include:k,style:{radius:[0,3,3,0]}},"button-box-pressed-focused-right":{include:I,style:{radius:[0,3,3,0]}},"button-box-pressed-hovered-focused-right":{include:X,style:{radius:[0,3,3,0]}},"button-box-right-borderless":{include:K,style:{radius:[0,3,3,0],width:[1,1,1,0]}},"button-box-pressed-right-borderless":{include:Y,style:{radius:[0,3,3,0],width:[1,1,1,0]}},"button-box-pressed-hovered-right-borderless":{include:F,style:{radius:[0,3,3,0],width:[1,1,1,0]}},"button-box-hovered-right-borderless":{include:s,style:{radius:[0,3,3,0],width:[1,1,1,0]}},"button-box-top-right":{include:K,style:{radius:[0,3,0,0],width:[1,1,1,0]}},"button-box-pressed-top-right":{include:Y,style:{radius:[0,3,0,0],width:[1,1,1,0]}},"button-box-pressed-hovered-top-right":{include:F,style:{radius:[0,3,0,0],width:[1,1,1,0]}},"button-box-hovered-top-right":{include:s,style:{radius:[0,3,0,0],width:[1,1,1,0]}},"button-box-bottom-right":{include:K,style:{radius:[0,0,3,0],width:[0,1,1,0]}},"button-box-pressed-bottom-right":{include:Y,style:{radius:[0,0,3,0],width:[0,1,1,0]}},"button-box-pressed-hovered-bottom-right":{include:F,style:{radius:[0,0,3,0],width:[0,1,1,0]}},"button-box-hovered-bottom-right":{include:s,style:{radius:[0,0,3,0],width:[0,1,1,0]}},"button-box-bottom-left":{include:K,style:{radius:[0,0,0,3],width:[0,0,1,1]}},"button-box-pressed-bottom-left":{include:Y,style:{radius:[0,0,0,3],width:[0,0,1,1]}},"button-box-pressed-hovered-bottom-left":{include:F,style:{radius:[0,0,0,3],width:[0,0,1,1]}},"button-box-hovered-bottom-left":{include:s,style:{radius:[0,0,0,3],width:[0,0,1,1]}},"button-box-top-left":{include:K,style:{radius:[3,0,0,0],width:[1,0,0,1]}},"button-box-pressed-top-left":{include:Y,style:{radius:[3,0,0,0],width:[1,0,0,1]}},"button-box-pressed-hovered-top-left":{include:F,style:{radius:[3,0,0,0],width:[1,0,0,1]}},"button-box-hovered-top-left":{include:s,style:{radius:[3,0,0,0],width:[1,0,0,1]}},"button-box-middle":{include:K,style:{radius:0,width:[1,0,1,1]}},"button-box-pressed-middle":{include:Y,style:{radius:0,width:[1,0,1,1]}},"button-box-pressed-hovered-middle":{include:F,style:{radius:0,width:[1,0,1,1]}},"button-box-hovered-middle":{include:s,style:{radius:0,width:[1,0,1,1]}},"button-box-left":{include:K,style:{radius:[3,0,0,3],width:[1,0,1,1]}},"button-box-pressed-left":{include:Y,style:{radius:[3,0,0,3],width:[1,0,1,1]}},"button-box-pressed-hovered-left":{include:F,style:{radius:[3,0,0,3],width:[1,0,1,1]}},"button-box-hovered-left":{include:s,style:{radius:[3,0,0,3],width:[1,0,1,1]}},"button-box-focused-left":{include:A,style:{radius:[3,0,0,3],width:[1,0,1,1]}},"button-box-hovered-focused-left":{include:k,style:{radius:[3,0,0,3],width:[1,0,1,1]}},"button-box-pressed-hovered-focused-left":{include:X,style:{radius:[3,0,0,3],width:[1,0,1,1]}},"button-box-pressed-focused-left":{include:I,style:{radius:[3,0,0,3],width:[1,0,1,1]}},"separator-horizontal":{style:{widthLeft:1,colorLeft:o}},"separator-vertical":{style:{widthTop:1,colorTop:o}},"scroll-knob":{style:{radius:3,width:1,color:d,backgroundColor:M}},"scroll-knob-pressed":{include:y,style:{backgroundColor:w}},"scroll-knob-hovered":{include:y,style:{color:g}},"scroll-knob-pressed-hovered":{include:D,style:{color:g}},"button-hover":{style:{backgroundColor:C,radius:3}},"window":{style:{width:1,color:q,innerWidth:4,innerColor:O,shadowLength:1,shadowBlurRadius:3,shadowColor:p,backgroundColor:L}},"window-active":{include:S,style:{shadowLength:2,shadowBlurRadius:5}},"window-caption":{style:{width:[0,0,2,0],color:O}},"white-box":{style:{width:1,color:Q,shadowBlurRadius:2,shadowColor:j,radius:7,backgroundColor:T,shadowLength:0}},"inset":{style:{width:1,color:[P,V,V,V]}},"focused-inset":{style:{width:2,color:R}},"border-invalid":{style:{width:2,color:J}},"lead-item":{style:{width:1,style:n,color:E}},"tooltip":{style:{width:1,color:r,shadowLength:1,shadowBlurRadius:2,shadowColor:p}},"tooltip-error":{style:{radius:5,backgroundColor:J}},"toolbar-separator":{style:{widthLeft:1,colorLeft:d}},"menu-separator":{style:{widthTop:1,colorTop:R}},"menubar-button-hovered":{style:{width:1,color:G,radius:3,backgroundColor:T}},"menubar-button-pressed":{include:h,style:{radius:[3,3,0,0],width:[1,1,0,1]}},"datechooser-date-pane":{style:{widthTop:1,colorTop:U,style:l}},"datechooser-weekday":{style:{widthBottom:1,colorBottom:U,style:l}},"datechooser-week":{style:{widthRight:1,colorRight:U,style:l}},"datechooser-week-header":{style:{widthBottom:1,colorBottom:U,widthRight:1,colorRight:U,style:l}},"tabview-page-button-top":{style:{width:[1,1,0,1],backgroundColor:L,color:G,radius:[3,3,0,0]}},"tabview-page-button-bottom":{include:c,style:{radius:[0,0,3,3],width:[0,1,1,1]}},"tabview-page-button-left":{include:c,style:{radius:[3,0,0,3],width:[1,0,1,1]}},"tabview-page-button-right":{include:c,style:{radius:[0,3,3,0],width:[1,1,1,0]}},"statusbar":{style:{widthTop:1,colorTop:R,styleTop:l}},"table-scroller-focus-indicator":{style:{width:2,color:t,style:l}},"table-header":{include:K,style:{radius:0,width:[1,0,1,0]}},"table-header-column-button":{include:e,style:{width:1,color:d}},"table-header-cell":{style:{widthRight:1,color:d}},"table-header-cell-first":{include:B,style:{widthLeft:1}},"progressive-table-header":{include:K,style:{radius:0,width:[1,0,1,1]}},"progressive-table-header-cell":{style:{widthRight:1,color:d}},"progressbar":{style:{backgroundColor:H,width:1,color:o}},"radiobutton":{style:{radius:10,width:1,color:d,innerColor:L,innerWidth:2}},"radiobutton-focused":{include:x,style:{color:R}},"radiobutton-invalid":{include:x,style:{color:J}},"checkbox":{style:{width:1,color:d}},"checkbox-focused":{include:b,style:{color:R}},"checkbox-invalid":{include:b,style:{color:J}}}});})();(function(){var a="qx.theme.indigo.Decoration",b="solid",c="window-border",d="white-box-border",e="#505154",f="background",g="highlight",h="border-main",i="white",j="highlight-shade",k="shadow",l="qx/decoration/Simple",m="#323335";qx.Theme.define(a,{extend:qx.theme.simple.Decoration,aliases:{decoration:l},decorations:{"window":{style:{width:1,color:c,shadowLength:1,shadowBlurRadius:3,shadowColor:k,backgroundColor:f,radius:3}},"window-caption":{style:{radius:[3,3,0,0],color:c,widthBottom:1}},"window-caption-active":{style:{radius:[3,3,0,0],color:g,widthBottom:3}},"white-box":{style:{width:1,color:d,backgroundColor:i}},"statusbar":{style:{widthTop:1,colorTop:h,styleTop:b}},"app-header":{style:{innerWidthBottom:1,innerColorBottom:j,widthBottom:9,colorBottom:g,gradientStart:[e,0],gradientEnd:[m,100],backgroundColor:m}}}});})();(function(){var a="#D9D9D9",b="#BBBBBB",c="#24B",d="qx.theme.indigo.Color",e="#dddddd",f="#888888",g="#CCCCCC",h="rgba(0, 0, 0, 0.4)",i="#B7B7B7",j="#1866B5",k="#BABABA",l="black",m="#F7F7F7",n="#A7A6AA",o="#EBEBEB",p="#666666",q="#CBC8CD",r="#F9F9F9",s="#CDCDCD",t="#808080",u="#F4F4F4",v="#C00F00",w="#686868",x="white",y="#5583D0",z="#262626",A="css.rgba",B="#EEE",C="#3D72C9",D="#E3E3E3",E="#323335",F="#BBB",G="#FE0",H="#F1F1F1",I="#939393",J="#134983",K="gray",L="#E8F0E3",M="#AAAAAA";qx.Theme.define(d,{colors:{"background":x,"dark-blue":E,"light-background":u,"font":z,"highlight":C,"highlight-shade":y,"background-selected":C,"background-selected-disabled":s,"background-selected-dark":E,"background-disabled":m,"background-disabled-checked":b,"background-pane":x,"tabview-unselected":j,"tabview-button-border":J,"tabview-label-active-disabled":a,"link":c,"scrollbar-bright":H,"scrollbar-dark":o,"button":L,"button-border":F,"button-border-hovered":I,"invalid":v,"button-box-bright":r,"button-box-dark":D,"button-box-bright-pressed":k,"button-box-dark-pressed":o,"border-lead":f,"window-border":e,"window-border-inner":u,"white-box-border":e,"shadow":qx.core.Environment.get(A)?h:p,"border-main":e,"border-light":i,"border-light-shadow":w,"border-separator":t,"text":z,"text-disabled":n,"text-selected":x,"text-placeholder":q,"tooltip":G,"tooltip-text":l,"table-header":[242,242,242],"table-focus-indicator":C,"table-header-cell":[235,234,219],"table-row-background-focused-selected":C,"table-row-background-focused":u,"table-row-background-selected":[51,94,168],"table-row-background-even":x,"table-row-background-odd":x,"table-row-selected":[255,255,255],"table-row":[0,0,0],"table-row-line":B,"table-column-line":B,"progressive-table-header":M,"progressive-table-row-background-even":[250,248,243],"progressive-table-row-background-odd":[255,255,255],"progressive-progressbar-background":K,"progressive-progressbar-indicator-done":g,"progressive-progressbar-indicator-undone":x,"progressive-progressbar-percent-background":K,"progressive-progressbar-percent-text":x}});})();(function(){var a="Indigo",b="qx.theme.Indigo";qx.Theme.define(b,{title:a,meta:{color:qx.theme.indigo.Color,decoration:qx.theme.indigo.Decoration,font:qx.theme.indigo.Font,appearance:qx.theme.indigo.Appearance,icon:qx.theme.icon.Tango}});})();

qx.$$loader.init();

/* Generated by Opal 0.8.1 */
Opal.modules["core/dimensions"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module;

  return (function($base) {
    var self = $module($base, 'Dimensions');

    var def = self.$$proto, $scope = self.$$scope;

    Opal.cdecl($scope, 'FOOTER_HEIGHT', 23);

    Opal.cdecl($scope, 'FOOTER_PADDING', 3);

    Opal.cdecl($scope, 'NAV_PANEL_WIDTH', 135);
  })(self)
};
/* Generated by Opal 0.8.1 */
Opal.modules["core/icons"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module;

  return (function($base) {
    var self = $module($base, 'Icons');

    var def = self.$$proto, $scope = self.$$scope;

    Opal.cdecl($scope, 'WINDOW_ICON', "resource/images/screen54.png")
  })(self)
};
/* Generated by Opal 0.8.1 */
Opal.modules["native"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_minus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs - rhs : lhs['$-'](rhs);
  }
  function $rb_ge(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs >= rhs : lhs['$>='](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $range = Opal.range, $hash2 = Opal.hash2, $klass = Opal.klass, $gvars = Opal.gvars;

  Opal.add_stubs(['$try_convert', '$native?', '$respond_to?', '$to_n', '$raise', '$inspect', '$Native', '$end_with?', '$define_method', '$[]', '$convert', '$call', '$to_proc', '$new', '$each', '$native_reader', '$native_writer', '$extend', '$alias_method', '$to_a', '$_Array', '$include', '$method_missing', '$bind', '$instance_method', '$[]=', '$slice', '$length', '$enum_for', '$===', '$<<', '$==', '$instance_variable_set', '$members', '$each_with_index', '$each_pair', '$name', '$native_module']);
  (function($base) {
    var self = $module($base, 'Native');

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    Opal.defs(self, '$is_a?', function(object, klass) {
      var self = this;

      
      try {
        return object instanceof self.$try_convert(klass);
      }
      catch (e) {
        return false;
      }
    ;
    });

    Opal.defs(self, '$try_convert', function(value) {
      var self = this;

      
      if (self['$native?'](value)) {
        return value;
      }
      else if (value['$respond_to?']("to_n")) {
        return value.$to_n();
      }
      else {
        return nil;
      }
    ;
    });

    Opal.defs(self, '$convert', function(value) {
      var self = this;

      
      if (self['$native?'](value)) {
        return value;
      }
      else if (value['$respond_to?']("to_n")) {
        return value.$to_n();
      }
      else {
        self.$raise($scope.get('ArgumentError'), "" + (value.$inspect()) + " isn't native");
      }
    ;
    });

    Opal.defs(self, '$call', TMP_1 = function(obj, key, args) {
      var self = this, $iter = TMP_1.$$p, block = $iter || nil;

      args = $slice.call(arguments, 2);
      TMP_1.$$p = null;
      
      var prop = obj[key];

      if (prop instanceof Function) {
        var converted = new Array(args.length);

        for (var i = 0, length = args.length; i < length; i++) {
          var item = args[i],
              conv = self.$try_convert(item);

          converted[i] = conv === nil ? item : conv;
        }

        if (block !== nil) {
          converted.push(block);
        }

        return self.$Native(prop.apply(obj, converted));
      }
      else {
        return self.$Native(prop);
      }
    ;
    });

    (function($base) {
      var self = $module($base, 'Helpers');

      var def = self.$$proto, $scope = self.$$scope;

      Opal.defn(self, '$alias_native', function(new$, old, options) {
        var $a, $b, TMP_2, $c, TMP_3, $d, TMP_4, self = this, as = nil;

        if (old == null) {
          old = new$
        }
        if (options == null) {
          options = $hash2([], {})
        }
        if ((($a = old['$end_with?']("=")) !== nil && (!$a.$$is_boolean || $a == true))) {
          return ($a = ($b = self).$define_method, $a.$$p = (TMP_2 = function(value){var self = TMP_2.$$s || this;
            if (self["native"] == null) self["native"] = nil;
if (value == null) value = nil;
          self["native"][old['$[]']($range(0, -2, false))] = $scope.get('Native').$convert(value);
            return value;}, TMP_2.$$s = self, TMP_2), $a).call($b, new$)
        } else if ((($a = as = options['$[]']("as")) !== nil && (!$a.$$is_boolean || $a == true))) {
          return ($a = ($c = self).$define_method, $a.$$p = (TMP_3 = function(args){var self = TMP_3.$$s || this, block, $a, $b, $c, value = nil;
            if (self["native"] == null) self["native"] = nil;
args = $slice.call(arguments, 0);
            block = TMP_3.$$p || nil, TMP_3.$$p = null;
          if ((($a = value = ($b = ($c = $scope.get('Native')).$call, $b.$$p = block.$to_proc(), $b).apply($c, [self["native"], old].concat(args))) !== nil && (!$a.$$is_boolean || $a == true))) {
              return as.$new(value.$to_n())
              } else {
              return nil
            }}, TMP_3.$$s = self, TMP_3), $a).call($c, new$)
          } else {
          return ($a = ($d = self).$define_method, $a.$$p = (TMP_4 = function(args){var self = TMP_4.$$s || this, block, $a, $b;
            if (self["native"] == null) self["native"] = nil;
args = $slice.call(arguments, 0);
            block = TMP_4.$$p || nil, TMP_4.$$p = null;
          return ($a = ($b = $scope.get('Native')).$call, $a.$$p = block.$to_proc(), $a).apply($b, [self["native"], old].concat(args))}, TMP_4.$$s = self, TMP_4), $a).call($d, new$)
        };
      });

      Opal.defn(self, '$native_reader', function(names) {
        var $a, $b, TMP_5, self = this;

        names = $slice.call(arguments, 0);
        return ($a = ($b = names).$each, $a.$$p = (TMP_5 = function(name){var self = TMP_5.$$s || this, $a, $b, TMP_6;
if (name == null) name = nil;
        return ($a = ($b = self).$define_method, $a.$$p = (TMP_6 = function(){var self = TMP_6.$$s || this;
            if (self["native"] == null) self["native"] = nil;

          return self.$Native(self["native"][name])}, TMP_6.$$s = self, TMP_6), $a).call($b, name)}, TMP_5.$$s = self, TMP_5), $a).call($b);
      });

      Opal.defn(self, '$native_writer', function(names) {
        var $a, $b, TMP_7, self = this;

        names = $slice.call(arguments, 0);
        return ($a = ($b = names).$each, $a.$$p = (TMP_7 = function(name){var self = TMP_7.$$s || this, $a, $b, TMP_8;
if (name == null) name = nil;
        return ($a = ($b = self).$define_method, $a.$$p = (TMP_8 = function(value){var self = TMP_8.$$s || this;
            if (self["native"] == null) self["native"] = nil;
if (value == null) value = nil;
          return self.$Native(self["native"][name] = value)}, TMP_8.$$s = self, TMP_8), $a).call($b, "" + (name) + "=")}, TMP_7.$$s = self, TMP_7), $a).call($b);
      });

      Opal.defn(self, '$native_accessor', function(names) {
        var $a, $b, self = this;

        names = $slice.call(arguments, 0);
        ($a = self).$native_reader.apply($a, [].concat(names));
        return ($b = self).$native_writer.apply($b, [].concat(names));
      });
    })(self);

    Opal.defs(self, '$included', function(klass) {
      var self = this;

      return klass.$extend($scope.get('Helpers'));
    });

    Opal.defn(self, '$initialize', function(native$) {
      var $a, self = this;

      if ((($a = $scope.get('Kernel')['$native?'](native$)) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        $scope.get('Kernel').$raise($scope.get('ArgumentError'), "" + (native$.$inspect()) + " isn't native")
      };
      return self["native"] = native$;
    });

    Opal.defn(self, '$to_n', function() {
      var self = this;
      if (self["native"] == null) self["native"] = nil;

      return self["native"];
    });
  })(self);
  (function($base) {
    var self = $module($base, 'Kernel');

    var def = self.$$proto, $scope = self.$$scope, TMP_9;

    Opal.defn(self, '$native?', function(value) {
      var self = this;

      return value == null || !value.$$class;
    });

    Opal.defn(self, '$Native', function(obj) {
      var $a, self = this;

      if ((($a = obj == null) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
      } else if ((($a = self['$native?'](obj)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return (($scope.get('Native')).$$scope.get('Object')).$new(obj)
        } else {
        return obj
      };
    });

    self.$alias_method("_Array", "Array");

    Opal.defn(self, '$Array', TMP_9 = function(object, args) {
      var $a, $b, self = this, $iter = TMP_9.$$p, block = $iter || nil;

      args = $slice.call(arguments, 1);
      TMP_9.$$p = null;
      if ((($a = self['$native?'](object)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return ($a = ($b = (($scope.get('Native')).$$scope.get('Array'))).$new, $a.$$p = block.$to_proc(), $a).apply($b, [object].concat(args)).$to_a()};
      return self.$_Array(object);
    });
  })(self);
  (function($base, $super) {
    function $Object(){};
    var self = $Object = $klass($base, $super, 'Object', $Object);

    var def = self.$$proto, $scope = self.$$scope, TMP_10, TMP_11, TMP_12;

    def["native"] = nil;
    self.$include(Opal.get('Native'));

    Opal.defn(self, '$==', function(other) {
      var self = this;

      return self["native"] === $scope.get('Native').$try_convert(other);
    });

    Opal.defn(self, '$has_key?', function(name) {
      var self = this;

      return Opal.hasOwnProperty.call(self["native"], name);
    });

    Opal.defn(self, '$key?', def['$has_key?']);

    Opal.defn(self, '$include?', def['$has_key?']);

    Opal.defn(self, '$member?', def['$has_key?']);

    Opal.defn(self, '$each', TMP_10 = function(args) {
      var $a, self = this, $iter = TMP_10.$$p, $yield = $iter || nil;

      args = $slice.call(arguments, 0);
      TMP_10.$$p = null;
      if (($yield !== nil)) {
        
        for (var key in self["native"]) {
          ((($a = Opal.yieldX($yield, [key, self["native"][key]])) === $breaker) ? $breaker.$v : $a)
        }
      ;
        return self;
        } else {
        return ($a = self).$method_missing.apply($a, ["each"].concat(args))
      };
    });

    Opal.defn(self, '$[]', function(key) {
      var self = this;

      
      var prop = self["native"][key];

      if (prop instanceof Function) {
        return prop;
      }
      else {
        return Opal.get('Native').$call(self["native"], key)
      }
    ;
    });

    Opal.defn(self, '$[]=', function(key, value) {
      var $a, self = this, native$ = nil;

      native$ = $scope.get('Native').$try_convert(value);
      if ((($a = native$ === nil) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self["native"][key] = value;
        } else {
        return self["native"][key] = native$;
      };
    });

    Opal.defn(self, '$merge!', function(other) {
      var self = this;

      
      var other = $scope.get('Native').$convert(other);

      for (var prop in other) {
        self["native"][prop] = other[prop];
      }
    ;
      return self;
    });

    Opal.defn(self, '$respond_to?', function(name, include_all) {
      var self = this;

      if (include_all == null) {
        include_all = false
      }
      return $scope.get('Kernel').$instance_method("respond_to?").$bind(self).$call(name, include_all);
    });

    Opal.defn(self, '$respond_to_missing?', function(name) {
      var self = this;

      return Opal.hasOwnProperty.call(self["native"], name);
    });

    Opal.defn(self, '$method_missing', TMP_11 = function(mid, args) {
      var $a, $b, self = this, $iter = TMP_11.$$p, block = $iter || nil;

      args = $slice.call(arguments, 1);
      TMP_11.$$p = null;
      
      if (mid.charAt(mid.length - 1) === '=') {
        return self['$[]='](mid.$slice(0, $rb_minus(mid.$length(), 1)), args['$[]'](0));
      }
      else {
        return ($a = ($b = Opal.get('Native')).$call, $a.$$p = block.$to_proc(), $a).apply($b, [self["native"], mid].concat(args));
      }
    ;
    });

    Opal.defn(self, '$nil?', function() {
      var self = this;

      return false;
    });

    Opal.defn(self, '$is_a?', function(klass) {
      var self = this;

      return Opal.is_a(self, klass);
    });

    Opal.defn(self, '$kind_of?', def['$is_a?']);

    Opal.defn(self, '$instance_of?', function(klass) {
      var self = this;

      return self.$$class === klass;
    });

    Opal.defn(self, '$class', function() {
      var self = this;

      return self.$$class;
    });

    Opal.defn(self, '$to_a', TMP_12 = function(options) {
      var $a, $b, self = this, $iter = TMP_12.$$p, block = $iter || nil;

      if (options == null) {
        options = $hash2([], {})
      }
      TMP_12.$$p = null;
      return ($a = ($b = (($scope.get('Native')).$$scope.get('Array'))).$new, $a.$$p = block.$to_proc(), $a).call($b, self["native"], options).$to_a();
    });

    return (Opal.defn(self, '$inspect', function() {
      var self = this;

      return "#<Native:" + (String(self["native"])) + ">";
    }), nil) && 'inspect';
  })($scope.get('Native'), $scope.get('BasicObject'));
  (function($base, $super) {
    function $Array(){};
    var self = $Array = $klass($base, $super, 'Array', $Array);

    var def = self.$$proto, $scope = self.$$scope, TMP_13, TMP_14;

    def.named = def["native"] = def.get = def.block = def.set = def.length = nil;
    self.$include($scope.get('Native'));

    self.$include($scope.get('Enumerable'));

    def.$initialize = TMP_13 = function(native$, options) {
      var $a, self = this, $iter = TMP_13.$$p, block = $iter || nil;

      if (options == null) {
        options = $hash2([], {})
      }
      TMP_13.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_13, null).apply(self, [native$]);
      self.get = ((($a = options['$[]']("get")) !== false && $a !== nil) ? $a : options['$[]']("access"));
      self.named = options['$[]']("named");
      self.set = ((($a = options['$[]']("set")) !== false && $a !== nil) ? $a : options['$[]']("access"));
      self.length = ((($a = options['$[]']("length")) !== false && $a !== nil) ? $a : "length");
      self.block = block;
      if ((($a = self.$length() == null) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.$raise($scope.get('ArgumentError'), "no length found on the array-like object")
        } else {
        return nil
      };
    };

    def.$each = TMP_14 = function() {
      var self = this, $iter = TMP_14.$$p, block = $iter || nil;

      TMP_14.$$p = null;
      if (block !== false && block !== nil) {
        } else {
        return self.$enum_for("each")
      };
      
      for (var i = 0, length = self.$length(); i < length; i++) {
        var value = Opal.yield1(block, self['$[]'](i));

        if (value === $breaker) {
          return $breaker.$v;
        }
      }
    ;
      return self;
    };

    def['$[]'] = function(index) {
      var $a, self = this, result = nil, $case = nil;

      result = (function() {$case = index;if ($scope.get('String')['$===']($case) || $scope.get('Symbol')['$===']($case)) {if ((($a = self.named) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self["native"][self.named](index);
        } else {
        return self["native"][index];
      }}else if ($scope.get('Integer')['$===']($case)) {if ((($a = self.get) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self["native"][self.get](index);
        } else {
        return self["native"][index];
      }}else { return nil }})();
      if (result !== false && result !== nil) {
        if ((($a = self.block) !== nil && (!$a.$$is_boolean || $a == true))) {
          return self.block.$call(result)
          } else {
          return self.$Native(result)
        }
        } else {
        return nil
      };
    };

    def['$[]='] = function(index, value) {
      var $a, self = this;

      if ((($a = self.set) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self["native"][self.set](index, $scope.get('Native').$convert(value));
        } else {
        return self["native"][index] = $scope.get('Native').$convert(value);
      };
    };

    def.$last = function(count) {
      var $a, self = this, index = nil, result = nil;

      if (count == null) {
        count = nil
      }
      if (count !== false && count !== nil) {
        index = $rb_minus(self.$length(), 1);
        result = [];
        while ($rb_ge(index, 0)) {
        result['$<<'](self['$[]'](index));
        index = $rb_minus(index, 1);};
        return result;
        } else {
        return self['$[]']($rb_minus(self.$length(), 1))
      };
    };

    def.$length = function() {
      var self = this;

      return self["native"][self.length];
    };

    Opal.defn(self, '$to_ary', def.$to_a);

    return (def.$inspect = function() {
      var self = this;

      return self.$to_a().$inspect();
    }, nil) && 'inspect';
  })($scope.get('Native'), null);
  (function($base, $super) {
    function $Numeric(){};
    var self = $Numeric = $klass($base, $super, 'Numeric', $Numeric);

    var def = self.$$proto, $scope = self.$$scope;

    return (def.$to_n = function() {
      var self = this;

      return self.valueOf();
    }, nil) && 'to_n'
  })(self, null);
  (function($base, $super) {
    function $Proc(){};
    var self = $Proc = $klass($base, $super, 'Proc', $Proc);

    var def = self.$$proto, $scope = self.$$scope;

    return (def.$to_n = function() {
      var self = this;

      return self;
    }, nil) && 'to_n'
  })(self, null);
  (function($base, $super) {
    function $String(){};
    var self = $String = $klass($base, $super, 'String', $String);

    var def = self.$$proto, $scope = self.$$scope;

    return (def.$to_n = function() {
      var self = this;

      return self.valueOf();
    }, nil) && 'to_n'
  })(self, null);
  (function($base, $super) {
    function $Regexp(){};
    var self = $Regexp = $klass($base, $super, 'Regexp', $Regexp);

    var def = self.$$proto, $scope = self.$$scope;

    return (def.$to_n = function() {
      var self = this;

      return self.valueOf();
    }, nil) && 'to_n'
  })(self, null);
  (function($base, $super) {
    function $MatchData(){};
    var self = $MatchData = $klass($base, $super, 'MatchData', $MatchData);

    var def = self.$$proto, $scope = self.$$scope;

    def.matches = nil;
    return (def.$to_n = function() {
      var self = this;

      return self.matches;
    }, nil) && 'to_n'
  })(self, null);
  (function($base, $super) {
    function $Struct(){};
    var self = $Struct = $klass($base, $super, 'Struct', $Struct);

    var def = self.$$proto, $scope = self.$$scope;

    def.$initialize = function(args) {
      var $a, $b, TMP_15, $c, TMP_16, self = this, object = nil;

      args = $slice.call(arguments, 0);
      if ((($a = (($b = args.$length()['$=='](1)) ? self['$native?'](args['$[]'](0)) : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
        object = args['$[]'](0);
        return ($a = ($b = self.$members()).$each, $a.$$p = (TMP_15 = function(name){var self = TMP_15.$$s || this;
if (name == null) name = nil;
        return self.$instance_variable_set("@" + (name), self.$Native(object[name]))}, TMP_15.$$s = self, TMP_15), $a).call($b);
        } else {
        return ($a = ($c = self.$members()).$each_with_index, $a.$$p = (TMP_16 = function(name, index){var self = TMP_16.$$s || this;
if (name == null) name = nil;if (index == null) index = nil;
        return self.$instance_variable_set("@" + (name), args['$[]'](index))}, TMP_16.$$s = self, TMP_16), $a).call($c)
      };
    };

    return (def.$to_n = function() {
      var $a, $b, TMP_17, self = this, result = nil;

      result = {};
      ($a = ($b = self).$each_pair, $a.$$p = (TMP_17 = function(name, value){var self = TMP_17.$$s || this;
if (name == null) name = nil;if (value == null) value = nil;
      return result[name] = value.$to_n();}, TMP_17.$$s = self, TMP_17), $a).call($b);
      return result;
    }, nil) && 'to_n';
  })(self, null);
  (function($base, $super) {
    function $Array(){};
    var self = $Array = $klass($base, $super, 'Array', $Array);

    var def = self.$$proto, $scope = self.$$scope;

    return (def.$to_n = function() {
      var self = this;

      
      var result = [];

      for (var i = 0, length = self.length; i < length; i++) {
        var obj = self[i];

        if ((obj)['$respond_to?']("to_n")) {
          result.push((obj).$to_n());
        }
        else {
          result.push(obj);
        }
      }

      return result;
    ;
    }, nil) && 'to_n'
  })(self, null);
  (function($base, $super) {
    function $Boolean(){};
    var self = $Boolean = $klass($base, $super, 'Boolean', $Boolean);

    var def = self.$$proto, $scope = self.$$scope;

    return (def.$to_n = function() {
      var self = this;

      return self.valueOf();
    }, nil) && 'to_n'
  })(self, null);
  (function($base, $super) {
    function $Time(){};
    var self = $Time = $klass($base, $super, 'Time', $Time);

    var def = self.$$proto, $scope = self.$$scope;

    return (def.$to_n = function() {
      var self = this;

      return self;
    }, nil) && 'to_n'
  })(self, null);
  (function($base, $super) {
    function $NilClass(){};
    var self = $NilClass = $klass($base, $super, 'NilClass', $NilClass);

    var def = self.$$proto, $scope = self.$$scope;

    return (def.$to_n = function() {
      var self = this;

      return null;
    }, nil) && 'to_n'
  })(self, null);
  (function($base, $super) {
    function $Hash(){};
    var self = $Hash = $klass($base, $super, 'Hash', $Hash);

    var def = self.$$proto, $scope = self.$$scope, TMP_18;

    def.$initialize = TMP_18 = function(defaults) {
      var self = this, $iter = TMP_18.$$p, block = $iter || nil;

      TMP_18.$$p = null;
      
      if (defaults != null) {
        if (defaults.constructor === Object) {
          var _map = self.map,
              smap = self.smap,
              keys = self.keys,
              map, khash, value;

          for (var key in defaults) {
            value = defaults[key];

            if (key.$$is_string) {
              map = smap;
              khash = key;
            } else {
              map = _map;
              khash = key.$hash();
            }

            if (value && value.constructor === Object) {
              map[khash] = $scope.get('Hash').$new(value);
            }
            else {
              map[khash] = self.$Native(value);
            }

            keys.push(key);
          }
        }
        else {
          self.none = defaults;
        }
      }
      else if (block !== nil) {
        self.proc = block;
      }

      return self;
    
    };

    return (def.$to_n = function() {
      var self = this;

      
      var result = {},
          keys   = self.keys,
          _map   = self.map,
          smap   = self.smap,
          map, khash, value, key;

      for (var i = 0, length = keys.length; i < length; i++) {
        key   = keys[i];

        if (key.$$is_string) {
          map = smap;
          khash = key;
        } else {
          map = _map;
          khash = key.$hash();
        }

        value = map[khash];

        if ((value)['$respond_to?']("to_n")) {
          result[key] = (value).$to_n();
        }
        else {
          result[key] = value;
        }
      }

      return result;
    ;
    }, nil) && 'to_n';
  })(self, null);
  (function($base, $super) {
    function $Module(){};
    var self = $Module = $klass($base, $super, 'Module', $Module);

    var def = self.$$proto, $scope = self.$$scope;

    return (def.$native_module = function() {
      var self = this;

      return Opal.global[self.$name()] = self;
    }, nil) && 'native_module'
  })(self, null);
  (function($base, $super) {
    function $Class(){};
    var self = $Class = $klass($base, $super, 'Class', $Class);

    var def = self.$$proto, $scope = self.$$scope;

    def.$native_alias = function(new_jsid, existing_mid) {
      var self = this;

      
      var aliased = self.$$proto['$' + existing_mid];
      if (!aliased) {
        self.$raise($scope.get('NameError'), "undefined method `" + (existing_mid) + "' for class `" + (self.$inspect()) + "'");
      }
      self.$$proto[new_jsid] = aliased;
    ;
    };

    return (def.$native_class = function() {
      var self = this;

      self.$native_module();
      self["new"] = self.$new;
    }, nil) && 'native_class';
  })(self, null);
  return $gvars.$ = $gvars.global = self.$Native(Opal.global);
};
/* Generated by Opal 0.8.1 */
Opal.modules["singleton"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module;

  Opal.add_stubs(['$raise', '$class', '$__init__', '$instance_eval', '$new', '$extend']);
  return (function($base) {
    var self = $module($base, 'Singleton');

    var def = self.$$proto, $scope = self.$$scope;

    Opal.defn(self, '$clone', function() {
      var self = this;

      return self.$raise($scope.get('TypeError'), "can't clone instance of singleton " + (self.$class()));
    });

    Opal.defn(self, '$dup', function() {
      var self = this;

      return self.$raise($scope.get('TypeError'), "can't dup instance of singleton " + (self.$class()));
    });

    (function($base) {
      var self = $module($base, 'SingletonClassMethods');

      var def = self.$$proto, $scope = self.$$scope, TMP_1, TMP_2;

      Opal.defn(self, '$clone', TMP_1 = function() {var $zuper = $slice.call(arguments, 0);
        var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

        TMP_1.$$p = null;
        return $scope.get('Singleton').$__init__(Opal.find_super_dispatcher(self, 'clone', TMP_1, $iter).apply(self, $zuper));
      });

      Opal.defn(self, '$inherited', TMP_2 = function(sub_klass) {var $zuper = $slice.call(arguments, 0);
        var self = this, $iter = TMP_2.$$p, $yield = $iter || nil;

        TMP_2.$$p = null;
        Opal.find_super_dispatcher(self, 'inherited', TMP_2, $iter).apply(self, $zuper);
        return $scope.get('Singleton').$__init__(sub_klass);
      });
    })(self);

    (function(self) {
      var $scope = self.$$scope, def = self.$$proto, TMP_4;

      self.$$proto.$__init__ = function(klass) {
        var $a, $b, TMP_3, self = this;

        ($a = ($b = klass).$instance_eval, $a.$$p = (TMP_3 = function(){var self = TMP_3.$$s || this;

        return self.singleton__instance__ = nil}, TMP_3.$$s = self, TMP_3), $a).call($b);
        Opal.defs(klass, '$instance', function() {
          var $a, self = this;
          if (self.singleton__instance__ == null) self.singleton__instance__ = nil;

          if ((($a = self.singleton__instance__) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.singleton__instance__};
          return self.singleton__instance__ = self.$new();
        });
        return klass;
      };
      return (self.$$proto.$included = TMP_4 = function(klass) {var $zuper = $slice.call(arguments, 0);
        var self = this, $iter = TMP_4.$$p, $yield = $iter || nil;

        TMP_4.$$p = null;
        Opal.find_super_dispatcher(self, 'included', TMP_4, $iter).apply(self, $zuper);
        klass.$extend($scope.get('SingletonClassMethods'));
        return $scope.get('Singleton').$__init__(klass);
      }, nil) && 'included';
    })($scope.get('Singleton').$singleton_class());
  })(self)
};
/* Generated by Opal 0.8.1 */
Opal.modules["core/object_space"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$include', '$attr_reader', '$sort', '$keys', '$classes', '$instance', '$global_constants', '$globals', '$Native', '$each', '$const_get', '$<<', '$is_a?', '$[]=', '$name']);
  self.$require("native");
  self.$require("singleton");
  return (function($base, $super) {
    function $ObjectSpace(){};
    var self = $ObjectSpace = $klass($base, $super, 'ObjectSpace', $ObjectSpace);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$include($scope.get('Singleton'));

    self.$attr_reader("classes");

    self.$attr_reader("global_constants");

    self.$attr_reader("globals");

    Opal.defs(self, '$class_names', function() {
      var self = this;

      return self.$classes().$keys().$sort();
    });

    Opal.defs(self, '$classes', function() {
      var self = this;

      return self.$instance().$classes();
    });

    Opal.defs(self, '$global_constants', function() {
      var self = this;

      return self.$instance().$global_constants();
    });

    Opal.defs(self, '$globals', function() {
      var self = this;

      return self.$instance().$globals();
    });

    return (def.$initialize = TMP_1 = function() {var $zuper = $slice.call(arguments, 0);
      var $a, $b, TMP_2, self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, $iter).apply(self, $zuper);
      self.classes = $hash2([], {});
      self.globals = [];
      self.global_constants = self.$Native(Opal.top.$$class.$constants()).$sort();
      return ($a = ($b = self.$global_constants()).$each, $a.$$p = (TMP_2 = function(c){var self = TMP_2.$$s || this, $a, o = nil;
if (c == null) c = nil;
      o = $scope.get('Opal').$const_get(c);
        self.$globals()['$<<'](o);
        if ((($a = o['$is_a?']($scope.get('Class'))) !== nil && (!$a.$$is_boolean || $a == true))) {
          return self.$classes()['$[]='](o.$name(), o)
          } else {
          return nil
        };}, TMP_2.$$s = self, TMP_2), $a).call($b);
    }, nil) && 'initialize';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["core/util"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module;

  Opal.add_stubs(['$require', '$as_qx', '$respond_to?', '$to_n', '$widget', '$to_sym', '$gsub', '$downcase', '$lambda', '$puts', '$split', '$[]=', '$capitalize', '$[]', '$join']);
  self.$require("core/object_space");
  self.$require("core/qx_wrap");
  self.$require("qx/ui/style_sheet");
  return (function($base) {
    var self = $module($base, 'Util');

    var def = self.$$proto, $scope = self.$$scope;

    Opal.defn(self, '$as_native', function(x) {
      var $a, self = this, n = nil;

      n = self.$as_qx(x);
      if ((($a = (n['$respond_to?']("to_n"))) !== nil && (!$a.$$is_boolean || $a == true))) {
        return n.$to_n()
        } else {
        return n
      };
    });

    Opal.defn(self, '$as_qx', function(x) {
      var $a, self = this;

      if ((($a = (x['$respond_to?']("widget"))) !== nil && (!$a.$$is_boolean || $a == true))) {
        return x.$widget()
        } else {
        return x
      };
    });

    Opal.defn(self, '$id_sym', function(s) {
      var self = this;

      return (s.$downcase().$gsub(" ", "_")).$to_sym();
    });

    Opal.defn(self, '$label_sym', function(s) {
      var self = this;

      return (s.$gsub(" ", "_")).$to_sym();
    });

    Opal.defn(self, '$nil_handler', function() {
      var $a, $b, TMP_1, self = this;

      return ($a = ($b = self).$lambda, $a.$$p = (TMP_1 = function(msg){var self = TMP_1.$$s || this, $a, $b, TMP_2;
if (msg == null) msg = nil;
      return ($a = ($b = self).$lambda, $a.$$p = (TMP_2 = function(){var self = TMP_2.$$s || this;

        return self.$puts(msg)}, TMP_2.$$s = self, TMP_2), $a).call($b)}, TMP_1.$$s = self, TMP_1), $a).call($b);
    });

    Opal.defn(self, '$text_from_id', function(s) {
      var self = this, words = nil;

      words = s.$split("_");
      words['$[]='](0, words['$[]'](0).$capitalize());
      return words.$join(" ");
    });
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["core/qx_wrap"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$include', '$attr_reader', '$lib_io_server', '$instance', '$Native', '$ui_style_stylesheet', '$core_object', '$data_array', '$data_controller_list', '$to_n', '$lib_data_list_model', '$lib_ui_grid_grid', '$ui_basic_atom', '$ui_basic_image', '$ui_basic_label', '$ui_container_composite', '$ui_container_scroll', '$ui_container_stack', '$ui_core_widget', '$ui_embed_html', '$ui_form_button', '$ui_form_combo_box', '$ui_form_form', '$ui_form_list', '$ui_form_list_item', '$ui_form_password_field', '$ui_form_radio_button', '$ui_form_radio_group', '$ui_form_radio_button_group', '$ui_form_renderer_double', '$ui_form_renderer_single', '$ui_form_renderer_single_placeholder', '$ui_form_select_box', '$ui_form_text_area', '$ui_form_text_field', '$ui_layout_basic', '$ui_layout_canvas', '$ui_layout_hbox', '$ui_layout_vbox', '$ui_menu_button', '$ui_menu_menu', '$ui_menubar_button', '$ui_menubar_menubar', '$ui_splitpane_pane', '$ui_tabview_page', '$ui_tabview_tab_button', '$ui_tabview_tab_view', '$ui_tree_file', '$ui_tree_folder', '$ui_tree_tree', '$ui_tree_virtual_tree', '$ui_tree_virtual_tree_item', '$ui_window_window']);
  self.$require("native");
  self.$require("singleton");
  self.$require("core/util");
  return (function($base, $super) {
    function $QxWrap(){};
    var self = $QxWrap = $klass($base, $super, 'QxWrap', $QxWrap);

    var def = self.$$proto, $scope = self.$$scope;

    self.$include($scope.get('Singleton'));

    self.$include($scope.get('Util'));

    self.$attr_reader("core_object");

    self.$attr_reader("data_array");

    self.$attr_reader("data_controller_list");

    self.$attr_reader("lib_data_list_model");

    self.$attr_reader("lib_io_server");

    self.$attr_reader("lib_ui_grid_grid");

    self.$attr_reader("ui_basic_atom");

    self.$attr_reader("ui_basic_image");

    self.$attr_reader("ui_basic_label");

    self.$attr_reader("ui_container_composite");

    self.$attr_reader("ui_container_scroll");

    self.$attr_reader("ui_container_stack");

    self.$attr_reader("ui_core_widget");

    self.$attr_reader("ui_embed_html");

    self.$attr_reader("ui_form_button");

    self.$attr_reader("ui_form_combo_box");

    self.$attr_reader("ui_form_form");

    self.$attr_reader("ui_form_list");

    self.$attr_reader("ui_form_list_item");

    self.$attr_reader("ui_form_password_field");

    self.$attr_reader("ui_form_radio_button");

    self.$attr_reader("ui_form_radio_button_group");

    self.$attr_reader("ui_form_radio_group");

    self.$attr_reader("ui_form_renderer_double");

    self.$attr_reader("ui_form_renderer_single");

    self.$attr_reader("ui_form_renderer_single_placeholder");

    self.$attr_reader("ui_form_select_box");

    self.$attr_reader("ui_form_text_area");

    self.$attr_reader("ui_form_text_field");

    self.$attr_reader("ui_layout_basic");

    self.$attr_reader("ui_layout_canvas");

    self.$attr_reader("ui_layout_hbox");

    self.$attr_reader("ui_layout_vbox");

    self.$attr_reader("ui_menu_button");

    self.$attr_reader("ui_menu_menu");

    self.$attr_reader("ui_menubar_button");

    self.$attr_reader("ui_menubar_menubar");

    self.$attr_reader("ui_splitpane_pane");

    self.$attr_reader("ui_style_stylesheet");

    self.$attr_reader("ui_tabview_page");

    self.$attr_reader("ui_tabview_tab_button");

    self.$attr_reader("ui_tabview_tab_view");

    self.$attr_reader("ui_tree_file");

    self.$attr_reader("ui_tree_folder");

    self.$attr_reader("ui_tree_tree");

    self.$attr_reader("ui_tree_virtual_tree");

    self.$attr_reader("ui_tree_virtual_tree_item");

    self.$attr_reader("ui_window_window");

    Opal.defs(self, '$lib_io_delete_server_api', function(controller, selector, params, handler) {
      var self = this, lib_io_server = nil;

      lib_io_server = self.$instance().$lib_io_server();
      return self.$Native(lib_io_server.apiDelete(controller, selector, params, handler));
    });

    Opal.defs(self, '$lib_io_get_server_api', function(controller, selector, params, handler) {
      var self = this, lib_io_server = nil;

      lib_io_server = self.$instance().$lib_io_server();
      return self.$Native(lib_io_server.apiGet(controller, selector, params, handler));
    });

    Opal.defs(self, '$lib_io_post_server_api', function(controller, selector, params, handler) {
      var self = this, lib_io_server = nil;

      lib_io_server = self.$instance().$lib_io_server();
      return self.$Native(lib_io_server.apiPost(controller, selector, params, handler));
    });

    Opal.defs(self, '$lib_io_put_server_api', function(controller, selector, params, handler) {
      var self = this, lib_io_server = nil;

      lib_io_server = self.$instance().$lib_io_server();
      return self.$Native(lib_io_server.apiPut(controller, selector, params, handler));
    });

    Opal.defs(self, '$instance_ui_style_stylesheet', function() {
      var self = this, ui_style_stylesheet = nil;

      ui_style_stylesheet = self.$instance().$ui_style_stylesheet();
      return self.$Native(ui_style_stylesheet.getInstance());
    });

    Opal.defs(self, '$new_core_object', function() {
      var self = this, core_object = nil;

      core_object = self.$instance().$core_object();
      return self.$Native(new core_object());
    });

    Opal.defs(self, '$new_data_array', function() {
      var self = this, data_array = nil;

      data_array = self.$instance().$data_array();
      return self.$Native(new data_array());
    });

    Opal.defs(self, '$new_data_controller_list', function(items, widget, tag) {
      var self = this, data_controller_list = nil;

      if (tag == null) {
        tag = "html"
      }
      data_controller_list = self.$instance().$data_controller_list();
      return self.$Native(new data_controller_list(items.$to_n(), widget.$to_n(), tag));
    });

    Opal.defs(self, '$new_lib_data_list_model', function(html, data) {
      var self = this, lib_data_list_model = nil;

      lib_data_list_model = self.$instance().$lib_data_list_model();
      return new lib_data_list_model(html, data);
    });

    Opal.defs(self, '$new_lib_ui_grid_grid', function() {
      var self = this, lib_ui_grid_grid = nil;

      lib_ui_grid_grid = self.$instance().$lib_ui_grid_grid();
      return self.$Native(new lib_ui_grid_grid());
    });

    Opal.defs(self, '$new_ui_basic_atom', function(label, icon) {
      var self = this, ui_basic_atom = nil;

      ui_basic_atom = self.$instance().$ui_basic_atom();
      return self.$Native(new ui_basic_atom(label, icon));
    });

    Opal.defs(self, '$new_ui_basic_image', function(source) {
      var self = this, ui_basic_image = nil;

      ui_basic_image = self.$instance().$ui_basic_image();
      return self.$Native(new ui_basic_image(source));
    });

    Opal.defs(self, '$new_ui_basic_label', function() {
      var self = this, ui_basic_label = nil;

      ui_basic_label = self.$instance().$ui_basic_label();
      return self.$Native(new ui_basic_label());
    });

    Opal.defs(self, '$new_ui_container_composite', function() {
      var self = this, ui_container_composite = nil;

      ui_container_composite = self.$instance().$ui_container_composite();
      return self.$Native(new ui_container_composite());
    });

    Opal.defs(self, '$new_ui_container_scroll', function() {
      var self = this, ui_container_scroll = nil;

      ui_container_scroll = self.$instance().$ui_container_scroll();
      return self.$Native(new ui_container_scroll());
    });

    Opal.defs(self, '$new_ui_container_stack', function() {
      var self = this, ui_container_stack = nil;

      ui_container_stack = self.$instance().$ui_container_stack();
      return self.$Native(new ui_container_stack());
    });

    Opal.defs(self, '$new_ui_core_widget', function() {
      var self = this, ui_core_widget = nil;

      ui_core_widget = self.$instance().$ui_core_widget();
      return self.$Native(new ui_core_widget());
    });

    Opal.defs(self, '$new_ui_embed_html', function() {
      var self = this, ui_embed_html = nil;

      ui_embed_html = self.$instance().$ui_embed_html();
      return self.$Native(new ui_embed_html());
    });

    Opal.defs(self, '$new_ui_form_button', function(label) {
      var self = this, ui_form_button = nil;

      if (label == null) {
        label = "Button"
      }
      ui_form_button = self.$instance().$ui_form_button();
      return self.$Native(new ui_form_button(label, null));
    });

    Opal.defs(self, '$new_ui_form_combo_box', function() {
      var self = this, ui_form_combo_box = nil;

      ui_form_combo_box = self.$instance().$ui_form_combo_box();
      return self.$Native(new ui_form_combo_box());
    });

    Opal.defs(self, '$new_ui_form_form', function() {
      var self = this, ui_form_form = nil;

      ui_form_form = self.$instance().$ui_form_form();
      return self.$Native(new ui_form_form());
    });

    Opal.defs(self, '$new_ui_form_list', function() {
      var self = this, ui_form_list = nil;

      ui_form_list = self.$instance().$ui_form_list();
      return self.$Native(new ui_form_list());
    });

    Opal.defs(self, '$new_ui_form_list_item', function(label) {
      var self = this, ui_form_list_item = nil;

      ui_form_list_item = self.$instance().$ui_form_list_item();
      return new ui_form_list_item(label);
    });

    Opal.defs(self, '$new_ui_form_password_field', function(value) {
      var self = this, ui_form_password_field = nil;

      if (value == null) {
        value = ""
      }
      ui_form_password_field = self.$instance().$ui_form_password_field();
      return self.$Native(new ui_form_password_field(value));
    });

    Opal.defs(self, '$new_ui_form_radio_button', function(label) {
      var self = this, ui_form_radio_button = nil;

      if (label == null) {
        label = nil
      }
      ui_form_radio_button = self.$instance().$ui_form_radio_button();
      return self.$Native(new ui_form_radio_button(label));
    });

    Opal.defs(self, '$new_ui_form_radio_button_group', function(items) {
      var self = this, ui_form_radio_group = nil;

      items = $slice.call(arguments, 0);
      ui_form_radio_group = self.$instance().$ui_form_radio_group();
      return self.$Native(new ui_form_radio_group(items));
    });

    Opal.defs(self, '$new_ui_form_radio_group', function(layout) {
      var self = this, ui_form_radio_button_group = nil;

      if (layout == null) {
        layout = nil
      }
      ui_form_radio_button_group = self.$instance().$ui_form_radio_button_group();
      return self.$Native(new ui_form_radio_button_group(layout));
    });

    Opal.defs(self, '$new_ui_form_renderer_double', function(nativeForm) {
      var self = this, ui_form_renderer_double = nil;

      ui_form_renderer_double = self.$instance().$ui_form_renderer_double();
      return self.$Native(new ui_form_renderer_double(nativeForm));
    });

    Opal.defs(self, '$new_ui_form_renderer_single', function(nativeForm) {
      var self = this, ui_form_renderer_single = nil;

      ui_form_renderer_single = self.$instance().$ui_form_renderer_single();
      return self.$Native(new ui_form_renderer_single(nativeForm));
    });

    Opal.defs(self, '$new_ui_form_renderer_single_placeholder', function(nativeForm) {
      var self = this, ui_form_renderer_single_placeholder = nil;

      ui_form_renderer_single_placeholder = self.$instance().$ui_form_renderer_single_placeholder();
      return self.$Native(new ui_form_renderer_single_placeholder(nativeForm));
    });

    Opal.defs(self, '$new_ui_form_select_box', function() {
      var self = this, ui_form_select_box = nil;

      ui_form_select_box = self.$instance().$ui_form_select_box();
      return self.$Native(new ui_form_select_box());
    });

    Opal.defs(self, '$new_ui_form_text_area', function() {
      var self = this, ui_form_text_area = nil;

      ui_form_text_area = self.$instance().$ui_form_text_area();
      return self.$Native(new ui_form_text_area(''));
    });

    Opal.defs(self, '$new_ui_form_text_field', function(value) {
      var self = this, ui_form_text_field = nil;

      if (value == null) {
        value = ""
      }
      ui_form_text_field = self.$instance().$ui_form_text_field();
      return self.$Native(new ui_form_text_field(value));
    });

    Opal.defs(self, '$new_ui_layout_basic', function() {
      var self = this, ui_layout_basic = nil;

      ui_layout_basic = self.$instance().$ui_layout_basic();
      return self.$Native(new ui_layout_basic());
    });

    Opal.defs(self, '$new_ui_layout_canvas', function() {
      var self = this, ui_layout_canvas = nil;

      ui_layout_canvas = self.$instance().$ui_layout_canvas();
      return self.$Native(new ui_layout_canvas());
    });

    Opal.defs(self, '$new_ui_layout_hbox', function() {
      var self = this, ui_layout_hbox = nil;

      ui_layout_hbox = self.$instance().$ui_layout_hbox();
      return self.$Native(new ui_layout_hbox());
    });

    Opal.defs(self, '$new_ui_layout_vbox', function() {
      var self = this, ui_layout_vbox = nil;

      ui_layout_vbox = self.$instance().$ui_layout_vbox();
      return self.$Native(new ui_layout_vbox());
    });

    Opal.defs(self, '$new_ui_menu_button', function() {
      var self = this, ui_menu_button = nil;

      ui_menu_button = self.$instance().$ui_menu_button();
      return self.$Native(new ui_menu_button());
    });

    Opal.defs(self, '$new_ui_menu_menu', function() {
      var self = this, ui_menu_menu = nil;

      ui_menu_menu = self.$instance().$ui_menu_menu();
      return self.$Native(new ui_menu_menu());
    });

    Opal.defs(self, '$new_ui_menubar_button', function() {
      var self = this, ui_menubar_button = nil;

      ui_menubar_button = self.$instance().$ui_menubar_button();
      return self.$Native(new ui_menubar_button());
    });

    Opal.defs(self, '$new_ui_menubar_menubar', function() {
      var self = this, ui_menubar_menubar = nil;

      ui_menubar_menubar = self.$instance().$ui_menubar_menubar();
      return self.$Native(new ui_menubar_menubar());
    });

    Opal.defs(self, '$new_ui_splitpane_pane', function() {
      var self = this, ui_splitpane_pane = nil;

      ui_splitpane_pane = self.$instance().$ui_splitpane_pane();
      return self.$Native(new ui_splitpane_pane());
    });

    Opal.defs(self, '$new_ui_tabview_page', function(label) {
      var self = this, ui_tabview_page = nil;

      ui_tabview_page = self.$instance().$ui_tabview_page();
      return self.$Native(new ui_tabview_page(label));
    });

    Opal.defs(self, '$new_ui_tabview_tab_button', function() {
      var self = this, ui_tabview_tab_button = nil;

      ui_tabview_tab_button = self.$instance().$ui_tabview_tab_button();
      return self.$Native(new ui_tabview_tab_button());
    });

    Opal.defs(self, '$new_ui_tabview_tab_view', function() {
      var self = this, new_ui_tabview_tab_view = nil;

      new_ui_tabview_tab_view = self.$instance().$ui_tabview_tab_view();
      return self.$Native(new new_ui_tabview_tab_view());
    });

    Opal.defs(self, '$new_ui_tree_file', function(label) {
      var self = this, ui_tree_file = nil;

      if (label == null) {
        label = nil
      }
      ui_tree_file = self.$instance().$ui_tree_file();
      return self.$Native(new ui_tree_file(label));
    });

    Opal.defs(self, '$new_ui_tree_folder', function(label) {
      var self = this, ui_tree_folder = nil;

      if (label == null) {
        label = nil
      }
      ui_tree_folder = self.$instance().$ui_tree_folder();
      return self.$Native(new ui_tree_folder(label));
    });

    Opal.defs(self, '$new_ui_tree_tree', function() {
      var self = this, ui_tree_tree = nil;

      ui_tree_tree = self.$instance().$ui_tree_tree();
      return self.$Native(new ui_tree_tree());
    });

    Opal.defs(self, '$new_ui_tree_virtual_tree', function(model) {
      var self = this, ui_tree_virtual_tree = nil;

      ui_tree_virtual_tree = self.$instance().$ui_tree_virtual_tree();
      return self.$Native(new ui_tree_virtual_tree(model, 'name', 'children'));
    });

    Opal.defs(self, '$new_ui_tree_virtual_tree_item', function(label) {
      var self = this, ui_tree_virtual_tree_item = nil;

      if (label == null) {
        label = nil
      }
      ui_tree_virtual_tree_item = self.$instance().$ui_tree_virtual_tree_item();
      return self.$Native(new ui_tree_virtual_tree_item(label));
    });

    Opal.defs(self, '$new_ui_window_window', function() {
      var self = this, ui_window_window = nil;

      ui_window_window = self.$instance().$ui_window_window();
      return self.$Native(new ui_window_window());
    });

    return (def.$initialize = function() {
      var self = this;

      self.core_object = self.$Native(qx.core.Object);
      self.data_array = self.$Native(qx.data.Array);
      self.data_controller_list = self.$Native(qx.data.controller.List);
      self.lib_data_list_model = self.$Native(qx_lib.data.ListModel);
      self.lib_io_server = self.$Native(qx_lib.io.Server);
      self.lib_ui_grid_grid = self.$Native(qx_lib.ui.grid.Grid);
      self.ui_basic_atom = self.$Native(qx.ui.basic.Atom);
      self.ui_basic_image = self.$Native(qx.ui.basic.Image);
      self.ui_basic_label = self.$Native(qx.ui.basic.Label);
      self.ui_container_composite = self.$Native(qx.ui.container.Composite);
      self.ui_container_scroll = self.$Native(qx.ui.container.Scroll);
      self.ui_container_stack = self.$Native(qx.ui.container.Stack);
      self.ui_core_widget = self.$Native(qx.ui.core.Widget);
      self.ui_embed_html = self.$Native(qx.ui.embed.Html);
      self.ui_form_button = self.$Native(qx.ui.form.Button);
      self.ui_form_combo_box = self.$Native(qx.ui.form.ComboBox);
      self.ui_form_form = self.$Native(qx.ui.form.Form);
      self.ui_form_list = self.$Native(qx.ui.form.List);
      self.ui_form_list_item = self.$Native(qx.ui.form.ListItem);
      self.ui_form_password_field = self.$Native(qx.ui.form.PasswordField);
      self.ui_form_radio_button = self.$Native(qx.ui.form.RadioButton);
      self.ui_form_radio_button_group = self.$Native(qx.ui.form.RadioButtonGroup);
      self.ui_form_radio_group = self.$Native(qx.ui.form.RadioGroup);
      self.ui_form_renderer_double = self.$Native(qx.ui.form.renderer.Double);
      self.ui_form_renderer_single = self.$Native(qx.ui.form.renderer.Single);
      self.ui_form_renderer_single_placeholder = self.$Native(qx.ui.form.renderer.SinglePlaceholder);
      self.ui_form_select_box = self.$Native(qx.ui.form.SelectBox);
      self.ui_form_text_area = self.$Native(qx.ui.form.TextArea);
      self.ui_form_text_field = self.$Native(qx.ui.form.TextField);
      self.ui_layout_basic = self.$Native(qx.ui.layout.Basic);
      self.ui_layout_canvas = self.$Native(qx.ui.layout.Canvas);
      self.ui_layout_hbox = self.$Native(qx.ui.layout.HBox);
      self.ui_layout_vbox = self.$Native(qx.ui.layout.VBox);
      self.ui_menu_button = self.$Native(qx.ui.menu.Button);
      self.ui_menu_menu = self.$Native(qx.ui.menu.Menu);
      self.ui_menubar_button = self.$Native(qx.ui.menubar.Button);
      self.ui_menubar_menubar = self.$Native(qx.ui.menubar.MenuBar);
      self.ui_splitpane_pane = self.$Native(qx.ui.splitpane.Pane);
      self.ui_style_stylesheet = self.$Native(qx.ui.style.Stylesheet);
      self.ui_tabview_page = self.$Native(qx.ui.tabview.Page);
      self.ui_tabview_tab_button = self.$Native(qx.ui.tabview.TabButton);
      self.ui_tabview_tab_view = self.$Native(qx.ui.tabview.TabView);
      self.ui_tree_file = self.$Native(qx.ui.tree.TreeFile);
      self.ui_tree_folder = self.$Native(qx.ui.tree.TreeFolder);
      self.ui_tree_tree = self.$Native(qx.ui.tree.Tree);
      self.ui_tree_virtual_tree = self.$Native(qx.ui.tree.VirtualTree);
      self.ui_tree_virtual_tree_item = self.$Native(qx.ui.tree.VirtualTreeItem);
      return self.ui_window_window = self.$Native(qx.ui.window.Window);
    }, nil) && 'initialize';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["qx/ui/qx_object"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$attr_reader']);
  self.$require("core/qx_wrap");
  return (function($base, $super) {
    function $QxObject(){};
    var self = $QxObject = $klass($base, $super, 'QxObject', $QxObject);

    var def = self.$$proto, $scope = self.$$scope;

    self.$attr_reader("object");

    return (def.$initialize = function(qxObject) {
      var self = this;

      return self.object = qxObject;
    }, nil) && 'initialize';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["qx/ui/element"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$object', '$Native', '$to_s', '$new']);
  self.$require("qx/ui/qx_object");
  return (function($base, $super) {
    function $Element(){};
    var self = $Element = $klass($base, $super, 'Element', $Element);

    var def = self.$$proto, $scope = self.$$scope;

    def.$add_class = function(name) {
      var self = this;

      return self.$object().addClass(name);
    };

    def.$get_style = function(name) {
      var self = this;

      return self.$Native(self.$object().getStyle(name));
    };

    def.$remove_class = function(name) {
      var self = this;

      return self.$object().removeClass(name);
    };

    def.$set_style = function(name, value) {
      var self = this;

      return self.$object().setStyle(name.$to_s(), value.$to_s());
    };

    return (def.$styles = function() {
      var self = this;

      return $scope.get('Hash').$new(self.$object().getAllStyles());
    }, nil) && 'styles';
  })(self, $scope.get('QxObject'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["qx/ui/widget"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$attr_reader', '$initial_widget', '$add_handlers', '$add_listeners', '$add_class', '$get_element', '$widget', '$initial_appear_handler', '$has_appear_listener', '$add_listener', '$lambda', '$on_appear', '$nil?', '$appear_handler', '$add_listener_appear', '$element', '$new', '$get_style', '$new_ui_core_widget', '$collect', '$as_native', '$class', '$appear_handled', '$on_appear_action', '$call', '$set_styles', '$remove_class', '$set_style', '$push', '$styles', '$!', '$empty?', '$pop', '$[]']);
  self.$require("core/qx_wrap");
  self.$require("qx/ui/element");
  return (function($base, $super) {
    function $Widget(){};
    var self = $Widget = $klass($base, $super, 'Widget', $Widget);

    var def = self.$$proto, $scope = self.$$scope, TMP_1, TMP_3;

    self.$attr_reader("appear_handled");

    self.$attr_reader("appear_handler");

    self.$attr_reader("element");

    self.$attr_reader("has_appear_listener");

    self.$attr_reader("styles");

    self.$attr_reader("widget");

    def.$initialize = TMP_1 = function() {var $zuper = $slice.call(arguments, 0);
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      TMP_1.$$p = null;
      self.styles = [];
      self.widget = self.$initial_widget();
      self.$add_handlers();
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, $iter).apply(self, $zuper);
      self.$add_listeners();
      return self.element = nil;
    };

    def.$add_class = function(name) {
      var self = this;

      return self.$get_element().$add_class(name);
    };

    def.$add_listener = function(name, fn) {
      var self = this;

      return self.$widget().native.addListener(name, fn, this);
    };

    def.$add_handlers = function() {
      var self = this;

      self.appear_handled = false;
      self.appear_handler = self.$initial_appear_handler();
      self.has_appear_listener = false;
      return self.style_handler = nil;
    };

    def.$add_listener_appear = function() {
      var $a, $b, TMP_2, self = this;

      if ((($a = self.$has_appear_listener()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        self.$add_listener("appear", ($a = ($b = self).$lambda, $a.$$p = (TMP_2 = function(){var self = TMP_2.$$s || this;

        return self.$on_appear()}, TMP_2.$$s = self, TMP_2), $a).call($b));
        return self.has_appear_listener = true;
      };
    };

    def.$add_listeners = function() {
      var $a, self = this;

      if ((($a = self.$appear_handler()['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        return self.$add_listener_appear()
      };
    };

    def.$get_element = function() {
      var $a, self = this;

      if ((($a = self.$element()['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.element = $scope.get('Element').$new(self.$widget().native.getContentElement())};
      return self.$element();
    };

    def.$get_style = function(name) {
      var self = this;

      return self.$get_element().$get_style(name);
    };

    def.$initial_appear_handler = function() {
      var self = this;

      return nil;
    };

    def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_core_widget();
    };

    def.$method_missing = TMP_3 = function(m, args) {
      var $a, $b, TMP_4, self = this, $iter = TMP_3.$$p, block = $iter || nil, fn = nil, nargs = nil;

      args = $slice.call(arguments, 1);
      TMP_3.$$p = null;
      fn = self.$widget().native[m];
      nargs = ($a = ($b = args).$collect, $a.$$p = (TMP_4 = function(arg){var self = TMP_4.$$s || this;
if (arg == null) arg = nil;
      return self.$as_native(arg)}, TMP_4.$$s = self, TMP_4), $a).call($b);
      if ((($a = fn['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return "method " + (m) + " not found in " + (self.$class())
        } else {
        return fn.apply(self.$widget().native, nargs);
      };
    };

    def.$on_appear = function() {
      var $a, self = this;

      if ((($a = self.$appear_handled()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        self.appear_handled = true;
        return self.$on_appear_action();
      };
    };

    def.$on_appear_action = function() {
      var $a, self = this;

      if ((($a = self.$appear_handler()['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$appear_handler().$call()
      };
      return self.$set_styles();
    };

    def.$on_click = function(fn) {
      var self = this;

      return self.$add_listener("click", fn);
    };

    def.$remove_class = function(name) {
      var self = this;

      return self.$get_element().$remove_class(name);
    };

    def.$set_border_color = function(value) {
      var self = this;

      return self.$set_style("border-color", value);
    };

    def.$set_color = function(value) {
      var self = this;

      return self.$set_style("color", value);
    };

    def.$set_style = function(name, value) {
      var $a, self = this;

      self.$add_listener_appear();
      self.$styles().$push($hash2(["name", "value"], {"name": name, "value": value}));
      if ((($a = self.$appear_handled()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.$set_styles()
        } else {
        return nil
      };
    };

    def.$set_styles = function() {
      var $a, $b, self = this, style = nil;

      while ((($b = self.$styles()['$empty?']()['$!']()) !== nil && (!$b.$$is_boolean || $b == true))) {
      style = self.$styles().$pop();
      self.$get_element().$set_style(style['$[]']("name"), style['$[]']("value"));};
    };

    return (def.$get_styles = function() {
      var self = this;

      return self.$get_element().$styles();
    }, nil) && 'get_styles';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["qx/ui/style_sheet"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$include', '$add_rule', '$instance', '$has_rule', '$remove_rule', '$replace_rule', '$addRule', '$widget', '$hasRule', '$instance_ui_style_stylesheet', '$removeRule']);
  self.$require("core/qx_wrap");
  self.$require("qx/ui/widget");
  self.$require("singleton");
  return (function($base, $super) {
    function $StyleSheet(){};
    var self = $StyleSheet = $klass($base, $super, 'StyleSheet', $StyleSheet);

    var def = self.$$proto, $scope = self.$$scope;

    self.$include($scope.get('Singleton'));

    Opal.defs(self, '$add_rule', function(selector, css) {
      var self = this;

      return self.$instance().$add_rule(selector, css);
    });

    Opal.defs(self, '$has_rule', function(selector) {
      var self = this;

      return self.$instance().$has_rule(selector);
    });

    Opal.defs(self, '$remove_rule', function(selector) {
      var self = this;

      return self.$instance().$remove_rule(selector);
    });

    Opal.defs(self, '$replace_rule', function(selector, css) {
      var self = this;

      return self.$instance().$replace_rule(selector, css);
    });

    def.$add_rule = function(selector, css) {
      var self = this;

      return self.$widget().$addRule(selector, css);
    };

    def.$has_rule = function(selector) {
      var self = this;

      return self.$widget().$hasRule(selector);
    };

    def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$instance_ui_style_stylesheet();
    };

    def.$remove_rule = function(selector) {
      var self = this;

      return self.$widget().$removeRule(selector);
    };

    return (def.$replace_rule = function(selector, css) {
      var $a, self = this;

      if ((($a = self.$has_rule(selector)) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$remove_rule(selector)};
      return self.$add_rule(selector, css);
    }, nil) && 'replace_rule';
  })(self, $scope.get('Widget'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["core/marshal"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module;

  Opal.add_stubs(['$require', '$include', '$as_native']);
  self.$require("core/util");
  return (function($base) {
    var self = $module($base, 'Marshal');

    var def = self.$$proto, $scope = self.$$scope;

    self.$include($scope.get('Util'));

    Opal.defn(self, '$create_model_from_object', function(obj) {
      var self = this, native_obj = nil;

      native_obj = self.$as_native(obj);
      return qx.data.marshal.Json.createModel(native_obj, true);
    });
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["data/model"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  return (function($base, $super) {
    function $Model(){};
    var self = $Model = $klass($base, $super, 'Model', $Model);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, null)
};
/* Generated by Opal 0.8.1 */
Opal.modules["json"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $hash2 = Opal.hash2, $klass = Opal.klass;

  Opal.add_stubs(['$new', '$push', '$[]=', '$[]', '$create_id', '$json_create', '$attr_accessor', '$create_id=', '$===', '$parse', '$generate', '$from_object', '$to_json', '$responds_to?', '$to_io', '$write', '$to_s', '$to_a', '$strftime']);
  (function($base) {
    var self = $module($base, 'JSON');

    var def = self.$$proto, $scope = self.$$scope, $a, $b;

    
    var $parse  = JSON.parse,
        $hasOwn = Opal.hasOwnProperty;

    function to_opal(value, options) {
      switch (typeof value) {
        case 'string':
          return value;

        case 'number':
          return value;

        case 'boolean':
          return !!value;

        case 'null':
          return nil;

        case 'object':
          if (!value) return nil;

          if (value.$$is_array) {
            var arr = (options.array_class).$new();

            for (var i = 0, ii = value.length; i < ii; i++) {
              (arr).$push(to_opal(value[i], options));
            }

            return arr;
          }
          else {
            var hash = (options.object_class).$new();

            for (var k in value) {
              if ($hasOwn.call(value, k)) {
                (hash)['$[]='](k, to_opal(value[k], options));
              }
            }

            var klass;
            if ((klass = (hash)['$[]']($scope.get('JSON').$create_id())) != nil) {
              klass = Opal.cget(klass);
              return (klass).$json_create(hash);
            }
            else {
              return hash;
            }
          }
      }
    };
  

    (function(self) {
      var $scope = self.$$scope, def = self.$$proto;

      return self.$attr_accessor("create_id")
    })(self.$singleton_class());

    (($a = ["json_class"]), $b = self, $b['$create_id='].apply($b, $a), $a[$a.length-1]);

    Opal.defs(self, '$[]', function(value, options) {
      var $a, self = this;

      if (options == null) {
        options = $hash2([], {})
      }
      if ((($a = $scope.get('String')['$==='](value)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.$parse(value, options)
        } else {
        return self.$generate(value, options)
      };
    });

    Opal.defs(self, '$parse', function(source, options) {
      var self = this;

      if (options == null) {
        options = $hash2([], {})
      }
      return self.$from_object($parse(source), options);
    });

    Opal.defs(self, '$parse!', function(source, options) {
      var self = this;

      if (options == null) {
        options = $hash2([], {})
      }
      return self.$parse(source, options);
    });

    Opal.defs(self, '$from_object', function(js_object, options) {
      var $a, $b, $c, self = this;

      if (options == null) {
        options = $hash2([], {})
      }
      ($a = "object_class", $b = options, ((($c = $b['$[]']($a)) !== false && $c !== nil) ? $c : $b['$[]=']($a, $scope.get('Hash'))));
      ($a = "array_class", $b = options, ((($c = $b['$[]']($a)) !== false && $c !== nil) ? $c : $b['$[]=']($a, $scope.get('Array'))));
      return to_opal(js_object, options.smap);
    });

    Opal.defs(self, '$generate', function(obj, options) {
      var self = this;

      if (options == null) {
        options = $hash2([], {})
      }
      return obj.$to_json(options);
    });

    Opal.defs(self, '$dump', function(obj, io, limit) {
      var $a, self = this, string = nil;

      if (io == null) {
        io = nil
      }
      if (limit == null) {
        limit = nil
      }
      string = self.$generate(obj);
      if (io !== false && io !== nil) {
        if ((($a = io['$responds_to?']("to_io")) !== nil && (!$a.$$is_boolean || $a == true))) {
          io = io.$to_io()};
        io.$write(string);
        return io;
        } else {
        return string
      };
    });
  })(self);
  (function($base, $super) {
    function $Object(){};
    var self = $Object = $klass($base, $super, 'Object', $Object);

    var def = self.$$proto, $scope = self.$$scope;

    return (Opal.defn(self, '$to_json', function() {
      var self = this;

      return self.$to_s().$to_json();
    }), nil) && 'to_json'
  })(self, null);
  (function($base) {
    var self = $module($base, 'Enumerable');

    var def = self.$$proto, $scope = self.$$scope;

    Opal.defn(self, '$to_json', function() {
      var self = this;

      return self.$to_a().$to_json();
    })
  })(self);
  (function($base, $super) {
    function $Array(){};
    var self = $Array = $klass($base, $super, 'Array', $Array);

    var def = self.$$proto, $scope = self.$$scope;

    return (def.$to_json = function() {
      var self = this;

      
      var result = [];

      for (var i = 0, length = self.length; i < length; i++) {
        result.push((self[i]).$to_json());
      }

      return '[' + result.join(', ') + ']';
    
    }, nil) && 'to_json'
  })(self, null);
  (function($base, $super) {
    function $Boolean(){};
    var self = $Boolean = $klass($base, $super, 'Boolean', $Boolean);

    var def = self.$$proto, $scope = self.$$scope;

    return (def.$to_json = function() {
      var self = this;

      return (self == true) ? 'true' : 'false';
    }, nil) && 'to_json'
  })(self, null);
  (function($base, $super) {
    function $Hash(){};
    var self = $Hash = $klass($base, $super, 'Hash', $Hash);

    var def = self.$$proto, $scope = self.$$scope;

    return (def.$to_json = function() {
      var self = this;

      
      var inspect = [],
          keys = self.keys,
          _map = self.map,
          smap = self.smap,
          map, khash;

      for (var i = 0, length = keys.length; i < length; i++) {
        var key = keys[i];

        if (key.$$is_string) {
          map = smap;
          khash = key;
        } else {
          map = _map;
          khash = key.$hash();
        }

        inspect.push((key).$to_s().$to_json() + ':' + (map[khash]).$to_json());
      }

      return '{' + inspect.join(', ') + '}';
    ;
    }, nil) && 'to_json'
  })(self, null);
  (function($base, $super) {
    function $NilClass(){};
    var self = $NilClass = $klass($base, $super, 'NilClass', $NilClass);

    var def = self.$$proto, $scope = self.$$scope;

    return (def.$to_json = function() {
      var self = this;

      return "null";
    }, nil) && 'to_json'
  })(self, null);
  (function($base, $super) {
    function $Numeric(){};
    var self = $Numeric = $klass($base, $super, 'Numeric', $Numeric);

    var def = self.$$proto, $scope = self.$$scope;

    return (def.$to_json = function() {
      var self = this;

      return self.toString();
    }, nil) && 'to_json'
  })(self, null);
  (function($base, $super) {
    function $String(){};
    var self = $String = $klass($base, $super, 'String', $String);

    var def = self.$$proto, $scope = self.$$scope;

    return Opal.defn(self, '$to_json', def.$inspect)
  })(self, null);
  (function($base, $super) {
    function $Time(){};
    var self = $Time = $klass($base, $super, 'Time', $Time);

    var def = self.$$proto, $scope = self.$$scope;

    return (def.$to_json = function() {
      var self = this;

      return self.$strftime("%FT%T%z").$to_json();
    }, nil) && 'to_json'
  })(self, null);
  return (function($base, $super) {
    function $Date(){};
    var self = $Date = $klass($base, $super, 'Date', $Date);

    var def = self.$$proto, $scope = self.$$scope;

    def.$to_json = function() {
      var self = this;

      return self.$to_s().$to_json();
    };

    return (def.$as_json = function() {
      var self = this;

      return self.$to_s();
    }, nil) && 'as_json';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["qx/io/server"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$include', '$nil?', '$to_s', '$lib_io_delete_server_api', '$handler_fn', '$lib_io_get_server_api', '$lib_io_post_server_api', '$to_n', '$to_json', '$lib_io_put_server_api', '$private', '$lambda', '$parse', '$call']);
  self.$require("json");
  self.$require("singleton");
  self.$require("core/qx_wrap");
  return (function($base, $super) {
    function $Server(){};
    var self = $Server = $klass($base, $super, 'Server', $Server);

    var def = self.$$proto, $scope = self.$$scope;

    self.$include($scope.get('Singleton'));

    Opal.defs(self, '$deleteApi', function(controller, selector, params, fn) {
      var $a, self = this;

      if ((($a = selector['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        selector = selector.$to_s()
      };
      return $scope.get('QxWrap').$lib_io_delete_server_api(controller, selector, params, self.$handler_fn(fn));
    });

    Opal.defs(self, '$getApi', function(controller, selector, params, fn) {
      var $a, self = this;

      if ((($a = selector['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        selector = selector.$to_s()
      };
      return $scope.get('QxWrap').$lib_io_get_server_api(controller, selector, params, self.$handler_fn(fn));
    });

    Opal.defs(self, '$postApi', function(controller, selector, params, fn) {
      var $a, self = this;

      if ((($a = selector['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        selector = selector.$to_s()
      };
      return $scope.get('QxWrap').$lib_io_post_server_api(controller, selector, params, self.$handler_fn(fn));
    });

    Opal.defs(self, '$putApi', function(controller, selector, params, fn) {
      var $a, self = this;

      if ((($a = selector['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        selector = selector.$to_s()
      };
      if ((($a = params['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        params = $hash2(["data"], {"data": params.$to_json()}).$to_n()
      };
      return $scope.get('QxWrap').$lib_io_put_server_api(controller, selector, params, self.$handler_fn(fn));
    });

    self.$private();

    return (Opal.defs(self, '$handler_fn', function(fn) {
      var $a, $b, TMP_1, self = this;

      return ($a = ($b = self).$lambda, $a.$$p = (TMP_1 = function(responseText){var self = TMP_1.$$s || this, reply = nil;
if (responseText == null) responseText = nil;
      reply = $scope.get('JSON').$parse(responseText);
        return fn.$call(reply);}, TMP_1.$$s = self, TMP_1), $a).call($b);
    }), nil) && 'handler_fn';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["data/store"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$attr_reader', '$initial_widget', '$add_change_listener', '$autoload_data?', '$data_load', '$addListener', '$widget', '$lambda', '$sync_changed', '$<<', '$sync_changed_consumer', '$data', '$[]=', '$data_records', '$[]', '$postApi', '$api_resource_name', '$model', '$handle_create_reply', '$deleteApi', '$handle_delete_reply', '$getApi', '$handle_load_reply', '$putApi', '$handle_update_reply', '$fireDataEvent', '$add_data_record', '$fire_event', '$clear_changed', '$remove_data_record', '$clear_data', '$each', '$update_data_record', '$new_core_object', '$data_consumers', '$store_changed', '$changed_id', '$store_changed_id', '$delete', '$index']);
  self.$require("core/qx_wrap");
  self.$require("qx/io/server");
  return (function($base, $super) {
    function $Store(){};
    var self = $Store = $klass($base, $super, 'Store', $Store);

    var def = self.$$proto, $scope = self.$$scope;

    def.data_consumers = nil;
    self.$attr_reader("changed_id");

    self.$attr_reader("data");

    self.$attr_reader("data_consumers");

    self.$attr_reader("data_records");

    self.$attr_reader("model");

    self.$attr_reader("widget");

    def.$initialize = function(model) {
      var $a, self = this;

      self.changed_id = nil;
      self.data = [];
      self.data_consumers = [];
      self.data_records = $hash2([], {});
      self.model = model;
      self.widget = self.$initial_widget();
      self.$add_change_listener();
      if ((($a = self['$autoload_data?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.$data_load()
        } else {
        return nil
      };
    };

    def.$add_change_listener = function() {
      var $a, $b, TMP_1, self = this;

      return self.$widget().$addListener("changed", ($a = ($b = self).$lambda, $a.$$p = (TMP_1 = function(){var self = TMP_1.$$s || this;

      return self.$sync_changed()}, TMP_1.$$s = self, TMP_1), $a).call($b));
    };

    def.$add_data_consumer = function(data_consumer) {
      var self = this;

      self.data_consumers['$<<'](data_consumer);
      return self.$sync_changed_consumer(data_consumer);
    };

    def.$add_data_record = function(record) {
      var self = this;

      self.$data()['$<<'](record);
      return self.$data_records()['$[]='](record['$[]']("id"), record);
    };

    def['$autoload_data?'] = function() {
      var self = this;

      return true;
    };

    def.$clear_changed = function() {
      var self = this;

      return self.changed_id = nil;
    };

    def.$clear_data = function() {
      var self = this;

      self.data = [];
      return self.data_records = $hash2([], {});
    };

    def.$data_create = function() {
      var $a, $b, TMP_2, self = this;

      return $scope.get('Server').$postApi(self.$model().$api_resource_name(), "", nil, ($a = ($b = self).$lambda, $a.$$p = (TMP_2 = function(record){var self = TMP_2.$$s || this;
if (record == null) record = nil;
      return self.$handle_create_reply(record)}, TMP_2.$$s = self, TMP_2), $a).call($b));
    };

    def.$data_delete = function(record_id) {
      var $a, $b, TMP_3, self = this;

      return $scope.get('Server').$deleteApi(self.$model().$api_resource_name(), record_id, nil, ($a = ($b = self).$lambda, $a.$$p = (TMP_3 = function(record){var self = TMP_3.$$s || this;
if (record == null) record = nil;
      return self.$handle_delete_reply(record)}, TMP_3.$$s = self, TMP_3), $a).call($b));
    };

    def.$data_load = function() {
      var $a, $b, TMP_4, self = this;

      return $scope.get('Server').$getApi(self.$model().$api_resource_name(), "", nil, ($a = ($b = self).$lambda, $a.$$p = (TMP_4 = function(data){var self = TMP_4.$$s || this;
if (data == null) data = nil;
      return self.$handle_load_reply(data)}, TMP_4.$$s = self, TMP_4), $a).call($b));
    };

    def.$data_update = function(record) {
      var $a, $b, TMP_5, self = this;

      return $scope.get('Server').$putApi(self.$model().$api_resource_name(), record['$[]']("id"), record, ($a = ($b = self).$lambda, $a.$$p = (TMP_5 = function(record){var self = TMP_5.$$s || this;
if (record == null) record = nil;
      return self.$handle_update_reply(record)}, TMP_5.$$s = self, TMP_5), $a).call($b));
    };

    def.$fire_event = function(name) {
      var self = this;

      return self.$widget().$fireDataEvent(name);
    };

    def.$get_data_record = function(record_id) {
      var self = this;

      return self.$data_records()['$[]'](record_id);
    };

    def.$handle_create_reply = function(record) {
      var self = this;

      self.changed_id = record['$[]']("id");
      self.$add_data_record(record);
      return self.$fire_event("changed");
    };

    def.$handle_delete_reply = function(record) {
      var self = this;

      self.$clear_changed();
      self.$remove_data_record(record);
      return self.$fire_event("changed");
    };

    def.$handle_load_reply = function(reply) {
      var $a, $b, TMP_6, self = this;

      self.$clear_changed();
      self.$clear_data();
      ($a = ($b = reply).$each, $a.$$p = (TMP_6 = function(record){var self = TMP_6.$$s || this;
if (record == null) record = nil;
      return self.$add_data_record(record)}, TMP_6.$$s = self, TMP_6), $a).call($b);
      return self.$fire_event("changed");
    };

    def.$handle_update_reply = function(record) {
      var self = this;

      self.$clear_changed();
      self.$update_data_record(record);
      return self.$fire_event("changed");
    };

    def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_core_object();
    };

    def.$sync_changed = function() {
      var $a, $b, TMP_7, self = this;

      return ($a = ($b = self.$data_consumers()).$each, $a.$$p = (TMP_7 = function(data_consumer){var self = TMP_7.$$s || this;
if (data_consumer == null) data_consumer = nil;
      return self.$sync_changed_consumer(data_consumer)}, TMP_7.$$s = self, TMP_7), $a).call($b);
    };

    def.$sync_changed_consumer = function(data_consumer) {
      var $a, self = this;

      data_consumer.$store_changed();
      if ((($a = self.$changed_id()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return data_consumer.$store_changed_id(self.$changed_id())
        } else {
        return nil
      };
    };

    def.$remove_data_record = function(record) {
      var self = this;

      self.$data().$delete(record);
      return self.$data_records().$delete(record['$[]']("id"));
    };

    return (def.$update_data_record = function(record) {
      var self = this, old_record = nil, index = nil;

      old_record = self.$data_records()['$[]'](record['$[]']("id"));
      index = self.$data().$index(old_record);
      self.$data()['$[]='](index, record);
      return self.$data_records()['$[]='](record['$[]']("id"), record);
    }, nil) && 'update_data_record';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["dynamic_themes/theme_base"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  return (function($base, $super) {
    function $ThemeBase(){};
    var self = $ThemeBase = $klass($base, $super, 'ThemeBase', $ThemeBase);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, null)
};
/* Generated by Opal 0.8.1 */
Opal.modules["dynamic_themes/modern_theme"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$theme_colors']);
  self.$require("dynamic_themes/theme_base");
  return (function($base, $super) {
    function $ModernTheme(){};
    var self = $ModernTheme = $klass($base, $super, 'ModernTheme', $ModernTheme);

    var def = self.$$proto, $scope = self.$$scope;

    Opal.defs(self, '$colors', function() {
      var self = this;

      return self.$theme_colors();
    });

    Opal.defs(self, '$name', function() {
      var self = this;

      return "modern_theme";
    });

    return (Opal.defs(self, '$theme_colors', function() {
      var self = this;

      return $hash2(["background-application", "background-light", "background-medium", "background-odd", "background-pane", "background-selected", "background-splitpane", "background-tip", "background-tip-error", "border-button", "border-column", "border-disabled", "border-dragover", "border-focused", "border-focused-invalid", "border-inner-input", "border-inner-scrollbar", "border-input", "border-invalid", "border-main", "border-pane", "border-separator", "border-toolbar-border-inner", "border-toolbar-button-outer", "border-toolbar-separator-left", "border-toolbar-separator-right", "button-border-disabled", "button-disabled-end", "button-disabled-start", "button-end", "button-focused", "button-hovered-end", "button-hovered-start", "button-start", "checkbox-border", "checkbox-disabled-border", "checkbox-disabled-end", "checkbox-disabled-inner", "checkbox-disabled-start", "checkbox-end", "checkbox-focus", "checkbox-hovered", "checkbox-hovered-inner", "checkbox-hovered-inner-invalid", "checkbox-hovered-invalid", "checkbox-inner", "checkbox-start", "group-background", "group-border", "groupitem-end", "groupitem-start", "groupitem-text", "input-border-disabled", "input-border-inner", "input-end", "input-focused-end", "input-focused-inner-invalid", "input-focused-start", "input-start", "invalid", "keyboard-focus", "menu-end", "menu-separator-bottom", "menu-separator-top", "menu-start", "menubar-start", "pane-end", "pane-start", "progressbar-background", "progressive-progressbar-background", "progressive-progressbar-indicator-done", "progressive-progressbar-indicator-undone", "progressive-progressbar-percent-background", "progressive-progressbar-percent-text", "progressive-table-header", "progressive-table-header-border-right", "progressive-table-row-background-even", "progressive-table-row-background-odd", "radiobutton-background", "radiobutton-checked", "radiobutton-checked-disabled", "radiobutton-disabled", "radiobutton-hovered-invalid", "scrollbar-end", "scrollbar-slider-end", "scrollbar-slider-start", "scrollbar-start", "selected-end", "selected-start", "shadow", "table-column-line", "table-focus-indicator", "table-header-end", "table-header-hovered", "table-header-start", "table-pane", "table-row", "table-row-background-even", "table-row-background-focused", "table-row-background-focused-selected", "table-row-background-odd", "table-row-background-selected", "table-row-line", "table-row-selected", "tabview-background", "tabview-end", "tabview-inactive", "tabview-inactive-end", "tabview-inactive-start", "tabview-start", "text-active", "text-disabled", "text-gray", "text-hovered", "text-inactive", "text-input", "text-label", "text-light", "text-placeholder", "text-selected", "text-title", "toolbar-end", "toolbar-start", "tooltip-error", "virtual-row-layer-background-even", "virtual-row-layer-background-odd", "window-border", "window-border-caption", "window-caption-active-end", "window-caption-active-start", "window-caption-active-text", "window-caption-inactive-end", "window-caption-inactive-start", "window-statusbar-background"], {"background-application": "#DFDFDF", "background-light": "#FCFCFC", "background-medium": "#EEEEEE", "background-odd": "#E4E4E4", "background-pane": "#F3F3F3", "background-selected": "#00368A", "background-splitpane": "#AFAFAF", "background-tip": "#ffffdd", "background-tip-error": "#C72B2B", "border-button": "#666666", "border-column": "#CCCCCC", "border-disabled": "#B6B6B6", "border-dragover": "#33508D", "border-focused": "#99C3FE", "border-focused-invalid": "#FF9999", "border-inner-input": "white", "border-inner-scrollbar": "white", "border-input": "#334866", "border-invalid": "#930000", "border-main": "#4d4d4d", "border-pane": "#00204D", "border-separator": "#808080", "border-toolbar-border-inner": "#f8f8f8", "border-toolbar-button-outer": "#b6b6b6", "border-toolbar-separator-left": "#b8b8b8", "border-toolbar-separator-right": "#f4f4f4", "button-border-disabled": "#959595", "button-disabled-end": "#BABABA", "button-disabled-start": "#F4F4F4", "button-end": "#AFAFAF", "button-focused": "#83BAEA", "button-hovered-end": "#8EB8D6", "button-hovered-start": "#F0F9FE", "button-start": "#F0F0F0", "checkbox-border": "#314A6E", "checkbox-disabled-border": "#787878", "checkbox-disabled-end": "#D8D8D8", "checkbox-disabled-inner": "#CACACA", "checkbox-disabled-start": "#D0D0D0", "checkbox-end": "#F3F3F3", "checkbox-focus": "#87AFE7", "checkbox-hovered": "#B2D2FF", "checkbox-hovered-inner": "#D1E4FF", "checkbox-hovered-inner-invalid": "#FAF2F2", "checkbox-hovered-invalid": "#F7E9E9", "checkbox-inner": "#EEEEEE", "checkbox-start": "#E4E4E4", "group-background": "#E8E8E8", "group-border": "#B4B4B4", "groupitem-end": "#949494", "groupitem-start": "#A7A7A7", "groupitem-text": "white", "input-border-disabled": "#9B9B9B", "input-border-inner": "white", "input-end": "#FBFCFB", "input-focused-end": "#5CB0FD", "input-focused-inner-invalid": "#FF6B78", "input-focused-start": "#D7E7F4", "input-start": "#F0F0F0", "invalid": "#990000", "keyboard-focus": "black", "menu-end": "#D9D9D9", "menu-separator-bottom": "#FAFAFA", "menu-separator-top": "#C5C5C5", "menu-start": "#E8E8E9", "menubar-start": "#E8E8E8", "pane-end": "#F0F0F0", "pane-start": "#FBFBFB", "progressbar-background": "white", "progressive-progressbar-background": "gray", "progressive-progressbar-indicator-done": "#CCCCCC", "progressive-progressbar-indicator-undone": "white", "progressive-progressbar-percent-background": "gray", "progressive-progressbar-percent-text": "white", "progressive-table-header": "#AAAAAA", "progressive-table-header-border-right": "#F2F2F2", "progressive-table-row-background-even": "#F4F4F4", "progressive-table-row-background-odd": "#E4E4E4", "radiobutton-background": "#EFEFEF", "radiobutton-checked": "#005BC3", "radiobutton-checked-disabled": "#7B7B7B", "radiobutton-disabled": "#D5D5D5", "radiobutton-hovered-invalid": "#F7EAEA", "scrollbar-end": "#F1F1F1", "scrollbar-slider-end": "#C3C3C3", "scrollbar-slider-start": "#EEEEEE", "scrollbar-start": "#CCCCCC", "selected-end": "#00368A", "selected-start": "#004DAD", "shadow": "rgba(0, 0, 0, 0.4)", "table-column-line": "#CCC", "table-focus-indicator": "#0880EF", "table-header-end": "#B3B3B3", "table-header-hovered": "white", "table-header-start": "#E8E8E8", "table-pane": "#F3F3F3", "table-row": "#1a1a1a", "table-row-background-even": "#F3F3F3", "table-row-background-focused": "#80B4EF", "table-row-background-focused-selected": "#084FAB", "table-row-background-odd": "#E4E4E4", "table-row-background-selected": "#084FAB", "table-row-line": "#CCC", "table-row-selected": "#fffefe", "tabview-background": "#07125A", "tabview-end": "#EEEEEE", "tabview-inactive": "#777D8D", "tabview-inactive-end": "#CECECE", "tabview-inactive-start": "#EAEAEA", "tabview-start": "#FCFCFC", "text-active": "#26364D", "text-disabled": "#7B7A7E", "text-gray": "#4a4a4a", "text-hovered": "#001533", "text-inactive": "#404955", "text-input": "#000000", "text-label": "#1a1a1a", "text-light": "#909090", "text-placeholder": "#CBC8CD", "text-selected": "#fffefe", "text-title": "#314a6e", "toolbar-end": "#DDDDDD", "toolbar-start": "#EFEFEF", "tooltip-error": "#C82C2C", "virtual-row-layer-background-even": "white", "virtual-row-layer-background-odd": "white", "window-border": "#00204D", "window-border-caption": "#727272", "window-caption-active-end": "#003B91", "window-caption-active-start": "#084FAA", "window-caption-active-text": "white", "window-caption-inactive-end": "#DBDBDB", "window-caption-inactive-start": "#F2F2F2", "window-statusbar-background": "#EFEFEF"});
    }), nil) && 'theme_colors';
  })(self, $scope.get('ThemeBase'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["dynamic_themes/default_theme"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$merge', '$colors', '$theme_colors']);
  self.$require("dynamic_themes/modern_theme");
  return (function($base, $super) {
    function $DefaultTheme(){};
    var self = $DefaultTheme = $klass($base, $super, 'DefaultTheme', $DefaultTheme);

    var def = self.$$proto, $scope = self.$$scope;

    Opal.defs(self, '$colors', function() {
      var self = this;

      return $scope.get('ModernTheme').$colors().$merge(self.$theme_colors());
    });

    Opal.defs(self, '$name', function() {
      var self = this;

      return "default_theme";
    });

    return (Opal.defs(self, '$theme_colors', function() {
      var self = this;

      return $hash2(["background-application", "background-pane", "background-splitpane", "button-end", "button-start", "text-label", "window-border", "window-border-caption", "window-caption-active-end", "window-caption-active-start", "window-caption-active-text", "window-caption-inactive-end", "window-caption-inactive-start", "window-statusbar-background"], {"background-application": "#ffffff", "background-pane": "#F4F4F4", "background-splitpane": "#E0ECFF", "button-end": "#215291", "button-start": "#215291", "text-label": "#fff", "window-border": "#00204D", "window-border-caption": "#727272", "window-caption-active-end": "#215291", "window-caption-active-start": "#215291", "window-caption-active-text": "white", "window-caption-inactive-end": "#DBDBDB", "window-caption-inactive-start": "#E0ECFF", "window-statusbar-background": "#EFEFEF"});
    }), nil) && 'theme_colors';
  })(self, $scope.get('ModernTheme'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["managers/palettes/palette"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$attr_reader', '$add_theme_overrides', '$initialize_color_map', '$[]=', '$color_map', '$to_sym', '$theme_overrides', '$to_s', '$add_style_colors', '$add_widget_colors', '$[]']);
  return (function($base, $super) {
    function $Palette(){};
    var self = $Palette = $klass($base, $super, 'Palette', $Palette);

    var def = self.$$proto, $scope = self.$$scope;

    self.$attr_reader("color_map");

    self.$attr_reader("theme_overrides");

    Opal.defs(self, '$name', function() {
      var self = this;

      return "dummy";
    });

    def.$initialize = function() {
      var self = this;

      self.color_map = $hash2([], {});
      self.theme_overrides = $hash2([], {});
      self.$add_theme_overrides();
      return self.$initialize_color_map();
    };

    def.$add_color = function(tag, value) {
      var self = this;

      return self.$color_map()['$[]='](tag.$to_sym(), value);
    };

    def.$add_override = function(tag, value) {
      var self = this;

      return self.$theme_overrides()['$[]='](tag.$to_s(), value);
    };

    def.$add_style_colors = function() {
      var self = this;

      return nil;
    };

    def.$add_theme_overrides = function() {
      var self = this;

      return nil;
    };

    def.$add_widget_colors = function() {
      var self = this;

      return nil;
    };

    def.$initialize_color_map = function() {
      var self = this;

      self.$add_style_colors();
      return self.$add_widget_colors();
    };

    return (def.$resolve_color = function(tag) {
      var self = this;

      return self.$color_map()['$[]'](tag.$to_sym());
    }, nil) && 'resolve_color';
  })(self, null)
};
/* Generated by Opal 0.8.1 */
Opal.modules["managers/palettes/default_palette"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$add_color', '$add_override']);
  self.$require("managers/palettes/palette");
  return (function($base, $super) {
    function $DefaultPalette(){};
    var self = $DefaultPalette = $klass($base, $super, 'DefaultPalette', $DefaultPalette);

    var def = self.$$proto, $scope = self.$$scope;

    Opal.defs(self, '$name', function() {
      var self = this;

      return "default";
    });

    def.$add_style_colors = function() {
      var self = this;

      self.$add_color("button", $hash2(["background_image", "border"], {"background_image": ["#F0F0F0", "35%", "#AFAFAF", "100%"], "border": ["#666666"]}));
      self.$add_color("input", $hash2(["background_color", "background_image", "border"], {"background_color": ["#FCFCFC"], "background_image": ["#F0F0F0", "#FBFCFB"], "border": ["#334866"]}));
      self.$add_color("input_placeholder", $hash2(["color"], {"color": ["#CBC8CD"]}));
      self.$add_color("selected", $hash2(["background_image"], {"background_image": ["#004DAD", "0%", "#00368A", "100%"]}));
      self.$add_color("split_pane", $hash2(["background_color", "border"], {"background_color": ["#F3F3F3"], "border": ["#AFAFAF"]}));
      self.$add_color("split_pane", $hash2(["background_color", "border"], {"background_color": ["#F3F3F3"], "border": ["#AFAFAF"]}));
      self.$add_color("toolbar_separator", $hash2(["border_left_color", "border_right_color"], {"border_left_color": ["#B8B8B8"], "border_right_color": ["#F4F4F4"]}));
      self.$add_color("window_captionbar_active", $hash2(["background_image", "border_color"], {"background_image": ["#AA4F08", "30%", "#913B00", "70%"], "border_color": ["#00204D", "#00204D", "#727272"]}));
      return self.$add_color("window_pane", $hash2(["background_color", "border_bottom_color", "border_left_color", "border_right_color"], {"background_color": ["#FFFFFF"], "border_bottom_color": ["#00204D"], "border_left_color": ["#00204D"], "border_right_color": ["#00204D"]}));
    };

    def.$add_widget_colors = function() {
      var self = this;

      self.$add_color("desktop_background", "#FFFFFF");
      self.$add_color("desktop_icon", "#aaa");
      self.$add_color("footer_background", "slategray");
      self.$add_color("footer_text", "#aaa");
      self.$add_color("navbar_background", "slategray");
      self.$add_color("navbar_label", "ivory");
      self.$add_color("navpanel_background", "#8c99a6");
      return self.$add_color("workspace_background", "#FFFFFF");
    };

    return (def.$add_theme_overrides = function() {
      var self = this;

      return self.$add_override("background-application", "white");
    }, nil) && 'add_theme_overrides';
  })(self, $scope.get('Palette'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["managers/palettes/neptune_palette"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$add_color', '$add_override']);
  self.$require("managers/palettes/palette");
  return (function($base, $super) {
    function $NeptunePalette(){};
    var self = $NeptunePalette = $klass($base, $super, 'NeptunePalette', $NeptunePalette);

    var def = self.$$proto, $scope = self.$$scope;

    Opal.defs(self, '$name', function() {
      var self = this;

      return "neptune";
    });

    def.$add_style_colors = function() {
      var self = this;

      self.$add_color("button", $hash2(["background_image", "border"], {"background_image": ["#F0F0F0", "35%", "#AFAFAF", "100%"], "border": ["#666666"]}));
      self.$add_color("input", $hash2(["background_color", "background_image", "border"], {"background_color": ["#FCFCFC"], "background_image": ["#F0F0F0", "#FBFCFB"], "border": ["#334866"]}));
      self.$add_color("input_placeholder", $hash2(["color"], {"color": ["#CBC8CD"]}));
      self.$add_color("selected", $hash2(["background_image"], {"background_image": ["#004DAD", "0%", "#00368A", "100%"]}));
      self.$add_color("split_pane", $hash2(["background_color", "border"], {"background_color": ["#F3F3F3"], "border": ["#AFAFAF"]}));
      self.$add_color("split_pane", $hash2(["background_color", "border"], {"background_color": ["#F3F3F3"], "border": ["#AFAFAF"]}));
      self.$add_color("toolbar_separator", $hash2(["border_left_color", "border_right_color"], {"border_left_color": ["#B8B8B8"], "border_right_color": ["#F4F4F4"]}));
      self.$add_color("window_captionbar_active", $hash2(["background_image", "border_color"], {"background_image": ["#004F08", "30%", "#003B00", "70%"], "border_color": ["#00204D", "#00204D", "#727272"]}));
      return self.$add_color("window_pane", $hash2(["background_color", "border_bottom_color", "border_left_color", "border_right_color"], {"background_color": ["#F3F3F3"], "border_bottom_color": ["#00204D"], "border_left_color": ["#00204D"], "border_right_color": ["#00204D"]}));
    };

    def.$add_widget_colors = function() {
      var self = this;

      self.$add_color("desktop_background", "#c5ccd3");
      self.$add_color("desktop_icon", "#aaa");
      self.$add_color("footer_background", "slategray");
      self.$add_color("footer_text", "#aaa");
      self.$add_color("navbar_background", "slategray");
      self.$add_color("navbar_label", "ivory");
      self.$add_color("navpanel_background", "#8c99a6");
      return self.$add_color("workspace_background", "#c5ccd3");
    };

    return (def.$add_theme_overrides = function() {
      var self = this;

      return self.$add_override("background-application", "white");
    }, nil) && 'add_theme_overrides';
  })(self, $scope.get('Palette'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["managers/theme_manager"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$include', '$attr_reader', '$get_dynamic_theme_colors', '$instance', '$get_theme_colors', '$get_theme_names', '$set_dynamic_theme', '$set_theme', '$set_theme_color', '$update', '$load_dynamic_themes', '$[]', '$dynamic_themes', '$to_sym', '$nil?', '$colors', '$theme_colors', '$colors_obj', '$each', '$[]=', '$sort', '$new', '$==', '$<<', '$values', '$name', '$Native', '$to_n', '$to_s', '$first', '$last']);
  self.$require("dynamic_themes/default_theme");
  self.$require("dynamic_themes/modern_theme");
  self.$require("singleton");
  return (function($base, $super) {
    function $ThemeManager(){};
    var self = $ThemeManager = $klass($base, $super, 'ThemeManager', $ThemeManager);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$include($scope.get('Singleton'));

    self.$attr_reader("colors_obj");

    self.$attr_reader("dynamic_themes");

    self.$attr_reader("theme_colors");

    Opal.defs(self, '$get_dynamic_theme_colors', function(name) {
      var self = this;

      return self.$instance().$get_dynamic_theme_colors(name);
    });

    Opal.defs(self, '$get_theme_colors', function() {
      var self = this;

      return self.$instance().$get_theme_colors();
    });

    Opal.defs(self, '$get_theme_names', function() {
      var self = this;

      return self.$instance().$get_theme_names();
    });

    Opal.defs(self, '$set_dynamic_theme', function(name) {
      var self = this;

      return self.$instance().$set_dynamic_theme(name);
    });

    Opal.defs(self, '$set_theme', function(theme_name) {
      var self = this;

      return self.$instance().$set_theme(theme_name);
    });

    Opal.defs(self, '$set_theme_color', function(name, value) {
      var self = this;

      return self.$instance().$set_theme_color(name, value);
    });

    Opal.defs(self, '$update', function() {
      var self = this;

      return self.$instance().$update();
    });

    def.$initialize = TMP_1 = function() {var $zuper = $slice.call(arguments, 0);
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, $iter).apply(self, $zuper);
      self.colors_obj = nil;
      self.dynamic_themes = $hash2([], {});
      self.theme_colors = nil;
      return self.$load_dynamic_themes();
    };

    def.$get_dynamic_theme_colors = function(name) {
      var $a, self = this, dynamic_theme = nil;

      dynamic_theme = self.$dynamic_themes()['$[]'](name.$to_sym());
      if ((($a = dynamic_theme['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        return dynamic_theme.$colors()
      };
    };

    def.$get_theme_colors = function() {
      var $a, $b, TMP_2, self = this, color_keys = nil;

      if ((($a = self.$theme_colors()['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        return self.$theme_colors()
      };
      self.theme_colors = $hash2([], {});
      self.colors_obj = qx.theme.manager.Meta.getInstance().getTheme().meta.color.colors.__proto__.__proto__;
      color_keys = Object.keys(self.$colors_obj());
      ($a = ($b = color_keys.$sort()).$each, $a.$$p = (TMP_2 = function(color_key){var self = TMP_2.$$s || this;
if (color_key == null) color_key = nil;
      return self.$theme_colors()['$[]='](color_key, self.$colors_obj()[color_key])}, TMP_2.$$s = self, TMP_2), $a).call($b);
      return self.$theme_colors();
    };

    def.$get_theme_names = function() {
      var $a, $b, TMP_3, self = this, themes = nil, theme_names = nil;

      themes = $scope.get('Hash').$new(qx.Theme.getAll());
      theme_names = [];
      ($a = ($b = themes.$values()).$each, $a.$$p = (TMP_3 = function(theme){var self = TMP_3.$$s || this;
if (theme == null) theme = nil;
      if (theme['$[]']("type")['$==']("meta")) {
          return theme_names['$<<'](theme['$[]']("name"))
          } else {
          return nil
        }}, TMP_3.$$s = self, TMP_3), $a).call($b);
      return theme_names;
    };

    def.$load_dynamic_themes = function() {
      var $a, $b, TMP_4, self = this;

      return ($a = ($b = [$scope.get('DefaultTheme'), $scope.get('ModernTheme')]).$each, $a.$$p = (TMP_4 = function(theme){var self = TMP_4.$$s || this;
if (theme == null) theme = nil;
      return self.$dynamic_themes()['$[]='](theme.$name(), theme)}, TMP_4.$$s = self, TMP_4), $a).call($b);
    };

    def.$set_dynamic_theme = function(name) {
      var $a, $b, TMP_5, self = this, colors = nil;

      colors = self.$get_dynamic_theme_colors(name);
      if ((($a = colors['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return "" + (name) + " not found"
        } else {
        ($a = ($b = colors).$each, $a.$$p = (TMP_5 = function(name, color){var self = TMP_5.$$s || this;
if (name == null) name = nil;if (color == null) color = nil;
        return self.$set_theme_color(name, color)}, TMP_5.$$s = self, TMP_5), $a).call($b);
        return self.$update();
      };
    };

    def.$set_theme = function(theme_name) {
      var $a, self = this, theme = nil;

      theme = self.$Native(qx.Theme.getByName(theme_name));
      if ((($a = theme['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        return qx.theme.manager.Meta.getInstance().setTheme(theme.$to_n());
      };
    };

    def.$set_theme_color = function(name, value) {
      var $a, self = this;

      if ((($a = self.$theme_colors()['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$get_theme_colors()};
      self.$theme_colors()['$[]='](name.$to_s(), value.$to_s());
      self.$colors_obj()[name.$to_s()] = value.$to_s();
      return value;
    };

    return (def.$update = function() {
      var self = this, theme_names = nil, current = nil, alternate = nil;

      theme_names = self.$get_theme_names();
      current = theme_names.$first();
      alternate = theme_names.$last();
      self.$set_theme(alternate);
      self.$set_theme(current);
      return "update complete";
    }, nil) && 'update';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["managers/color_manager"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$include', '$attr_reader', '$resolve_color', '$instance', '$set_color_palette', '$load_palettes', '$empty?', '$theme_overrides', '$current_palette', '$each', '$set_theme_color', '$update', '$[]=', '$palettes', '$name', '$new', '$[]', '$to_sym', '$apply_theme_overrides']);
  self.$require("singleton");
  self.$require("managers/palettes/default_palette");
  self.$require("managers/palettes/neptune_palette");
  self.$require("managers/theme_manager");
  return (function($base, $super) {
    function $ColorManager(){};
    var self = $ColorManager = $klass($base, $super, 'ColorManager', $ColorManager);

    var def = self.$$proto, $scope = self.$$scope;

    self.$include($scope.get('Singleton'));

    self.$attr_reader("current_palette");

    self.$attr_reader("palettes");

    Opal.defs(self, '$resolve_color', function(tag) {
      var self = this;

      return self.$instance().$resolve_color(tag);
    });

    Opal.defs(self, '$set_color_palette', function(name) {
      var self = this;

      return self.$instance().$set_color_palette(name);
    });

    def.$initialize = function() {
      var self = this;

      self.palettes = $hash2([], {});
      return self.$load_palettes();
    };

    def.$apply_theme_overrides = function() {
      var $a, $b, TMP_1, self = this;

      if ((($a = self.$current_palette().$theme_overrides()['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        ($a = ($b = self.$current_palette().$theme_overrides()).$each, $a.$$p = (TMP_1 = function(k, v){var self = TMP_1.$$s || this;
if (k == null) k = nil;if (v == null) v = nil;
        return $scope.get('ThemeManager').$set_theme_color(k, v)}, TMP_1.$$s = self, TMP_1), $a).call($b);
        return $scope.get('ThemeManager').$update();
      };
    };

    def.$load_palettes = function() {
      var $a, $b, TMP_2, self = this;

      return ($a = ($b = [$scope.get('DefaultPalette'), $scope.get('NeptunePalette')]).$each, $a.$$p = (TMP_2 = function(palette_class){var self = TMP_2.$$s || this;
if (palette_class == null) palette_class = nil;
      return self.$palettes()['$[]='](palette_class.$name(), palette_class.$new())}, TMP_2.$$s = self, TMP_2), $a).call($b);
    };

    def.$resolve_color = function(tag) {
      var self = this;

      return self.$current_palette().$resolve_color(tag);
    };

    return (def.$set_color_palette = function(name) {
      var self = this;

      self.current_palette = self.$palettes()['$[]'](name.$to_sym());
      return self.$apply_theme_overrides();
    }, nil) && 'set_color_palette';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["managers/rules/css_rule"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$resolve_color', '$name', '$nil?', '$%', '$keys', '$css_color_templates', '$each', '$[]', '$css_color_values', '$<<', '$gen_color_part', '$css_base_parts', '$css_gen_color_parts', '$join', '$css_parts']);
  self.$require("managers/color_manager");
  return (function($base, $super) {
    function $CssRule(){};
    var self = $CssRule = $klass($base, $super, 'CssRule', $CssRule);

    var def = self.$$proto, $scope = self.$$scope;

    def.$css_base_parts = function() {
      var self = this;

      return [];
    };

    def.$css_color_templates = function() {
      var self = this;

      return $hash2([], {});
    };

    def.$css_color_values = function() {
      var self = this;

      return $scope.get('ColorManager').$resolve_color(self.$name());
    };

    def.$gen_color_part = function(template, values) {
      var $a, self = this;

      if ((($a = values['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return template
        } else {
        return template['$%'](values)
      };
    };

    def.$css_gen_color_parts = function() {
      var $a, $b, TMP_1, self = this, color_parts = nil, keys = nil;

      color_parts = [];
      keys = self.$css_color_templates().$keys();
      ($a = ($b = keys).$each, $a.$$p = (TMP_1 = function(key){var self = TMP_1.$$s || this, template = nil, values = nil;
if (key == null) key = nil;
      template = self.$css_color_templates()['$[]'](key);
        values = self.$css_color_values()['$[]'](key);
        return color_parts['$<<'](self.$gen_color_part(template, values));}, TMP_1.$$s = self, TMP_1), $a).call($b);
      return color_parts;
    };

    def.$css_parts = function() {
      var self = this;

      return $rb_plus(self.$css_base_parts(), self.$css_gen_color_parts());
    };

    def.$css_text = function() {
      var self = this;

      return self.$css_parts().$join(";");
    };

    def.$name = function() {
      var self = this;

      return "dummy";
    };

    return (def.$selector = function() {
      var self = this;

      return "dummy";
    }, nil) && 'selector';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["managers/rules/css_button"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require']);
  self.$require("managers/rules/css_rule");
  return (function($base, $super) {
    function $CssButton(){};
    var self = $CssButton = $klass($base, $super, 'CssButton', $CssButton);

    var def = self.$$proto, $scope = self.$$scope;

    def.$css_base_parts = function() {
      var self = this;

      return ["-webkit-background-clip: padding-box", "border-radius: 3px", "background-clip: padding-box"];
    };

    def.$css_color_templates = function() {
      var self = this;

      return $hash2(["background_image", "border"], {"background_image": "background-image: linear-gradient(%s %s, %s %s)", "border": "border: 1px solid %s"});
    };

    def.$name = function() {
      var self = this;

      return "button";
    };

    return (def.$selector = function() {
      var self = this;

      return ".qx-button";
    }, nil) && 'selector';
  })(self, $scope.get('CssRule'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["managers/rules/css_input"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require']);
  self.$require("managers/rules/css_rule");
  return (function($base, $super) {
    function $CssInput(){};
    var self = $CssInput = $klass($base, $super, 'CssInput', $CssInput);

    var def = self.$$proto, $scope = self.$$scope;

    def.$css_base_parts = function() {
      var self = this;

      return ["box-shadow: white 0px 0px 0px 1px inset", "-webkit-background-clip: padding-box", "background-clip: padding-box"];
    };

    def.$css_color_templates = function() {
      var self = this;

      return $hash2(["background_color", "background_image", "border"], {"background_color": "background-color: %s", "background_image": "background-image: linear-gradient(%s 0px, %s 12px)", "border": "border: 1px solid %s"});
    };

    def.$name = function() {
      var self = this;

      return "input";
    };

    return (def.$selector = function() {
      var self = this;

      return ".qx-input";
    }, nil) && 'selector';
  })(self, $scope.get('CssRule'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["managers/rules/css_input_placeholder"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require']);
  self.$require("managers/rules/css_rule");
  return (function($base, $super) {
    function $CssInputPlaceholder(){};
    var self = $CssInputPlaceholder = $klass($base, $super, 'CssInputPlaceholder', $CssInputPlaceholder);

    var def = self.$$proto, $scope = self.$$scope;

    def.$css_base_parts = function() {
      var self = this;

      return ["border-width: 1px"];
    };

    def.$css_color_templates = function() {
      var self = this;

      return $hash2(["color"], {"color": "color: %s"});
    };

    def.$name = function() {
      var self = this;

      return "input_placeholder";
    };

    return (def.$selector = function() {
      var self = this;

      return ".input.qx-placeholder-color, a.qx-placeholder-color";
    }, nil) && 'selector';
  })(self, $scope.get('CssRule'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["managers/rules/css_selected"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require']);
  self.$require("managers/rules/css_rule");
  return (function($base, $super) {
    function $CssSelected(){};
    var self = $CssSelected = $klass($base, $super, 'CssSelected', $CssSelected);

    var def = self.$$proto, $scope = self.$$scope;

    def.$css_base_parts = function() {
      var self = this;

      return ["-webkit-background-clip: padding-box", "background-clip: padding-box"];
    };

    def.$css_color_templates = function() {
      var self = this;

      return $hash2(["background_image"], {"background_image": "background-image: linear-gradient(%s %s, %s %s)"});
    };

    def.$name = function() {
      var self = this;

      return "selected";
    };

    return (def.$selector = function() {
      var self = this;

      return ".qx-selected";
    }, nil) && 'selector';
  })(self, $scope.get('CssRule'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["managers/rules/css_split_pane"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require']);
  self.$require("managers/rules/css_rule");
  return (function($base, $super) {
    function $CssSplitPane(){};
    var self = $CssSplitPane = $klass($base, $super, 'CssSplitPane', $CssSplitPane);

    var def = self.$$proto, $scope = self.$$scope;

    def.$css_base_parts = function() {
      var self = this;

      return ["-webkit-background-clip: padding-box", "background-clip: padding-box"];
    };

    def.$css_color_templates = function() {
      var self = this;

      return $hash2(["background_color", "border"], {"background_color": "background-color: %s", "border": "border: 3px solid %s"});
    };

    def.$name = function() {
      var self = this;

      return "split_pane";
    };

    return (def.$selector = function() {
      var self = this;

      return ".qx-splitpane";
    }, nil) && 'selector';
  })(self, $scope.get('CssRule'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["managers/rules/css_window_captionbar_active"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require']);
  self.$require("managers/rules/css_rule");
  return (function($base, $super) {
    function $CssWindowCaptionbarActive(){};
    var self = $CssWindowCaptionbarActive = $klass($base, $super, 'CssWindowCaptionbarActive', $CssWindowCaptionbarActive);

    var def = self.$$proto, $scope = self.$$scope;

    def.$css_base_parts = function() {
      var self = this;

      return ["border-width: 1px", "border-style: solid", "-webkit-background-clip: padding-box", "border-top-left-radius: 5px", "border-top-right-radius: 5px", "background-clip: padding-box"];
    };

    def.$css_color_templates = function() {
      var self = this;

      return $hash2(["background_image", "border_color"], {"background_image": "background-image: linear-gradient(%s %s, %s %s)", "border_color": "border-color: %s %s %s"});
    };

    def.$name = function() {
      var self = this;

      return "window_captionbar_active";
    };

    return (def.$selector = function() {
      var self = this;

      return ".qx-window-captionbar-active";
    }, nil) && 'selector';
  })(self, $scope.get('CssRule'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["managers/rules/css_window_pane"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require']);
  self.$require("managers/rules/css_rule");
  return (function($base, $super) {
    function $CssWindowPane(){};
    var self = $CssWindowPane = $klass($base, $super, 'CssWindowPane', $CssWindowPane);

    var def = self.$$proto, $scope = self.$$scope;

    def.$css_base_parts = function() {
      var self = this;

      return ["border-right-width: 1px", "border-right-style: solid", "border-bottom-width: 1px", "border-bottom-style: solid", "border-left-width: 1px", "border-left-style: solid", "-webkit-background-clip: padding-box", "background-clip: padding-box"];
    };

    def.$css_color_templates = function() {
      var self = this;

      return $hash2(["background_color", "border_bottom_color", "border_left_color", "border_right_color"], {"background_color": "background-color: %s", "border_bottom_color": "border-bottom-color: %s", "border_left_color": "border-left-color: %s", "border_right_color": "border-right-color: %s"});
    };

    def.$name = function() {
      var self = this;

      return "window_pane";
    };

    return (def.$selector = function() {
      var self = this;

      return ".qx-window-pane";
    }, nil) && 'selector';
  })(self, $scope.get('CssRule'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["managers/css_manager"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$include', '$attr_reader', '$apply_rules', '$instance', '$add_rules', '$each', '$new', '$[]=', '$css_rules', '$name', '$replace_rule', '$selector', '$css_text', '$puts', '$%', '$apply_rule', '$values']);
  self.$require("singleton");
  self.$require("managers/rules/css_button");
  self.$require("managers/rules/css_input");
  self.$require("managers/rules/css_input_placeholder");
  self.$require("managers/rules/css_selected");
  self.$require("managers/rules/css_split_pane");
  self.$require("managers/rules/css_window_captionbar_active");
  self.$require("managers/rules/css_window_pane");
  self.$require("qx/ui/style_sheet");
  return (function($base, $super) {
    function $CssManager(){};
    var self = $CssManager = $klass($base, $super, 'CssManager', $CssManager);

    var def = self.$$proto, $scope = self.$$scope;

    self.$include($scope.get('Singleton'));

    self.$attr_reader("css_rules");

    Opal.defs(self, '$apply_rules', function() {
      var self = this;

      return self.$instance().$apply_rules();
    });

    def.$initialize = function() {
      var self = this;

      self.css_rules = $hash2([], {});
      return self.$add_rules();
    };

    def.$add_rules = function() {
      var $a, $b, TMP_1, self = this;

      return ($a = ($b = [$scope.get('CssButton'), $scope.get('CssInput'), $scope.get('CssInputPlaceholder'), $scope.get('CssSelected'), $scope.get('CssSplitPane'), $scope.get('CssWindowCaptionbarActive'), $scope.get('CssWindowPane')]).$each, $a.$$p = (TMP_1 = function(rule_class){var self = TMP_1.$$s || this, rule = nil;
if (rule_class == null) rule_class = nil;
      rule = rule_class.$new();
        return self.$css_rules()['$[]='](rule.$name(), rule);}, TMP_1.$$s = self, TMP_1), $a).call($b);
    };

    def.$apply_rule = function(rule) {
      var self = this;

      try {
      return $scope.get('StyleSheet').$replace_rule(rule.$selector(), rule.$css_text())
      } catch ($err) {if (true) {
        return self.$puts("Stylesheet error: [%s] [%s]"['$%']([rule.$selector(), rule.$css_text()]))
        }else { throw $err; }
      };
    };

    return (def.$apply_rules = function() {
      var $a, $b, TMP_2, self = this;

      return ($a = ($b = self.$css_rules().$values()).$each, $a.$$p = (TMP_2 = function(rule){var self = TMP_2.$$s || this;
if (rule == null) rule = nil;
      return self.$apply_rule(rule)}, TMP_2.$$s = self, TMP_2), $a).call($b);
    }, nil) && 'apply_rules';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["template"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$[]', '$[]=', '$keys', '$attr_reader', '$instance_exec', '$to_proc', '$new', '$<<', '$join']);
  return (function($base, $super) {
    function $Template(){};
    var self = $Template = $klass($base, $super, 'Template', $Template);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    def.name = def.body = nil;
    self._cache = $hash2([], {});

    Opal.defs(self, '$[]', function(name) {
      var $a, self = this;
      if (self._cache == null) self._cache = nil;

      return ((($a = self._cache['$[]'](name)) !== false && $a !== nil) ? $a : self._cache['$[]']("templates/" + (name)));
    });

    Opal.defs(self, '$[]=', function(name, instance) {
      var self = this;
      if (self._cache == null) self._cache = nil;

      return self._cache['$[]='](name, instance);
    });

    Opal.defs(self, '$paths', function() {
      var self = this;
      if (self._cache == null) self._cache = nil;

      return self._cache.$keys();
    });

    self.$attr_reader("body");

    def.$initialize = TMP_1 = function(name) {
      var $a, self = this, $iter = TMP_1.$$p, body = $iter || nil;

      TMP_1.$$p = null;
      $a = [name, body], self.name = $a[0], self.body = $a[1];
      return $scope.get('Template')['$[]='](name, self);
    };

    def.$inspect = function() {
      var self = this;

      return "#<Template: '" + (self.name) + "'>";
    };

    def.$render = function(ctx) {
      var $a, $b, self = this;

      if (ctx == null) {
        ctx = self
      }
      return ($a = ($b = ctx).$instance_exec, $a.$$p = self.body.$to_proc(), $a).call($b, $scope.get('OutputBuffer').$new());
    };

    return (function($base, $super) {
      function $OutputBuffer(){};
      var self = $OutputBuffer = $klass($base, $super, 'OutputBuffer', $OutputBuffer);

      var def = self.$$proto, $scope = self.$$scope;

      def.buffer = nil;
      def.$initialize = function() {
        var self = this;

        return self.buffer = [];
      };

      def.$append = function(str) {
        var self = this;

        return self.buffer['$<<'](str);
      };

      Opal.defn(self, '$append=', def.$append);

      return (def.$join = function() {
        var self = this;

        return self.buffer.$join();
      }, nil) && 'join';
    })(self, null);
  })(self, null)
};
/* Generated by Opal 0.8.1 */
Opal.modules["managers/store_manager"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$include', '$attr_reader', '$add_store', '$instance', '$find_store', '$api_resource_name', '$new', '$[]=', '$stores', '$to_sym', '$[]']);
  self.$require("singleton");
  self.$require("data/store");
  return (function($base, $super) {
    function $StoreManager(){};
    var self = $StoreManager = $klass($base, $super, 'StoreManager', $StoreManager);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$include($scope.get('Singleton'));

    self.$attr_reader("stores");

    Opal.defs(self, '$add_store', function(model) {
      var self = this;

      return self.$instance().$add_store(model);
    });

    Opal.defs(self, '$find_store', function(tag) {
      var self = this;

      return self.$instance().$find_store(tag);
    });

    def.$initialize = TMP_1 = function() {var $zuper = $slice.call(arguments, 0);
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, $iter).apply(self, $zuper);
      return self.stores = $hash2([], {});
    };

    def.$add_store = function(model) {
      var self = this, tag = nil, store = nil;

      tag = model.$api_resource_name();
      store = $scope.get('Store').$new(model);
      return self.$stores()['$[]='](tag.$to_sym(), store);
    };

    return (def.$find_store = function(tag) {
      var self = this;

      return self.$stores()['$[]'](tag.$to_sym());
    }, nil) && 'find_store';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["managers/model_manager"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$include', '$attr_reader', '$add_model', '$instance', '$find_model', '$[]=', '$models', '$to_sym', '$add_store', '$[]']);
  self.$require("singleton");
  self.$require("managers/store_manager");
  return (function($base, $super) {
    function $ModelManager(){};
    var self = $ModelManager = $klass($base, $super, 'ModelManager', $ModelManager);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$include($scope.get('Singleton'));

    self.$attr_reader("models");

    Opal.defs(self, '$add_model', function(tag, model) {
      var self = this;

      return self.$instance().$add_model(tag, model);
    });

    Opal.defs(self, '$find_model', function(tag) {
      var self = this;

      return self.$instance().$find_model(tag);
    });

    def.$initialize = TMP_1 = function() {var $zuper = $slice.call(arguments, 0);
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, $iter).apply(self, $zuper);
      return self.models = $hash2([], {});
    };

    def.$add_model = function(tag, model) {
      var self = this;

      self.$models()['$[]='](tag.$to_sym(), model);
      return $scope.get('StoreManager').$add_store(model);
    };

    return (def.$find_model = function(tag) {
      var self = this;

      return self.$models()['$[]'](tag.$to_sym());
    }, nil) && 'find_model';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["managers/template_manager"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$include', '$attr_reader', '$add_template', '$instance', '$find_template', '$render', '$to_s', '$[]=', '$models', '$to_sym', '$[]', '$templates', '$nil?', '$find_model', '$new']);
  self.$require("singleton");
  self.$require("template");
  self.$require("managers/model_manager");
  return (function($base, $super) {
    function $TemplateManager(){};
    var self = $TemplateManager = $klass($base, $super, 'TemplateManager', $TemplateManager);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$include($scope.get('Singleton'));

    self.$attr_reader("models");

    self.$attr_reader("templates");

    Opal.defs(self, '$add_template', function(tag, model_tag) {
      var self = this;

      if (model_tag == null) {
        model_tag = nil
      }
      return self.$instance().$add_template(tag, model_tag);
    });

    Opal.defs(self, '$find_template', function(tag) {
      var self = this;

      return self.$instance().$find_template(tag);
    });

    Opal.defs(self, '$render', function(tag, data) {
      var self = this;

      if (data == null) {
        data = $hash2([], {})
      }
      return self.$instance().$render(tag, data);
    });

    def.$initialize = TMP_1 = function() {var $zuper = $slice.call(arguments, 0);
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, $iter).apply(self, $zuper);
      self.models = $hash2([], {});
      return self.templates = $hash2([], {});
    };

    def.$add_template = function(tag, model_tag) {
      var self = this, template = nil;

      self.$require("templates/" + (tag.$to_s()));
      self.$models()['$[]='](tag.$to_sym(), model_tag);
      template = $scope.get('Template')['$[]'](tag.$to_s());
      return self.$templates()['$[]='](tag.$to_sym(), template);
    };

    def.$find_template = function(tag) {
      var self = this;

      return self.$templates()['$[]'](tag.$to_sym());
    };

    return (def.$render = function(tag, data) {
      var $a, $b, self = this, template = nil, model_tag = nil, record = nil, model_class = nil;

      template = self.$find_template(tag);
      model_tag = self.$models()['$[]'](tag.$to_sym());
      record = nil;
      if ((($a = ((($b = model_tag['$nil?']()) !== false && $b !== nil) ? $b : data['$nil?']())) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        model_class = $scope.get('ModelManager').$find_model(model_tag);
        if ((($a = model_class['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
          } else {
          record = model_class.$new(data)
        };
      };
      if ((($a = template['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        return template.$render(record)
      };
    }, nil) && 'render';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["qx/ui/component"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$add_classes', '$nil?', '$initial_height', '$set_height', '$initial_layout', '$set_layout', '$initial_width', '$set_width', '$as_qx', '$add', '$widget', '$each', '$add_class', '$initial_classes', '$as_native', '$setBackgroundColor', '$setHeight', '$setWidth']);
  self.$require("core/util");
  self.$require("qx/ui/widget");
  return (function($base, $super) {
    function $Component(){};
    var self = $Component = $klass($base, $super, 'Component', $Component);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    def.$initialize = TMP_1 = function() {var $zuper = $slice.call(arguments, 0);
      var $a, self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, $iter).apply(self, $zuper);
      self.$add_classes();
      if ((($a = self.$initial_height()['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$set_height(self.$initial_height())
      };
      if ((($a = self.$initial_layout()['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$set_layout(self.$initial_layout())
      };
      if ((($a = self.$initial_width()['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        return self.$set_width(self.$initial_width())
      };
    };

    def.$add = function(aComponent, arg) {
      var $a, self = this, component = nil;

      if (arg == null) {
        arg = nil
      }
      component = self.$as_qx(aComponent);
      if ((($a = arg['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.$widget().$add(component)
        } else {
        return self.$widget().$add(component, arg)
      };
    };

    def.$add_classes = function() {
      var $a, $b, TMP_2, self = this;

      return ($a = ($b = self.$initial_classes()).$each, $a.$$p = (TMP_2 = function(clsname){var self = TMP_2.$$s || this;
if (clsname == null) clsname = nil;
      return self.$add_class(clsname)}, TMP_2.$$s = self, TMP_2), $a).call($b);
    };

    def.$add_native = function(item) {
      var self = this;

      return self.$widget().$add(self.$as_native(item));
    };

    def.$initial_classes = function() {
      var self = this;

      return [];
    };

    def.$initial_height = function() {
      var self = this;

      return nil;
    };

    def.$initial_layout = function() {
      var self = this;

      return nil;
    };

    def.$initial_width = function() {
      var self = this;

      return nil;
    };

    def.$set_background_color = function(aColor) {
      var self = this;

      return self.$widget().$setBackgroundColor(aColor);
    };

    def.$set_height = function(n) {
      var self = this;

      return self.$widget().$setHeight(n);
    };

    return (def.$set_width = function(n) {
      var self = this;

      return self.$widget().$setWidth(n);
    }, nil) && 'set_width';
  })(self, $scope.get('Widget'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/html"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$setOverflowX', '$widget', '$setOverflowY', '$set_value', '$new_ui_embed_html', '$setHtml']);
  self.$require("core/qx_wrap");
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $Html(){};
    var self = $Html = $klass($base, $super, 'Html', $Html);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    def.$initialize = TMP_1 = function(html) {
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      if (html == null) {
        html = ""
      }
      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      self.$widget().$setOverflowX("auto");
      self.$widget().$setOverflowY("auto");
      return self.$set_value(html);
    };

    def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_embed_html();
    };

    return (def.$set_value = function(html) {
      var self = this;

      return self.$widget().$setHtml(html);
    }, nil) && 'set_value';
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["managers/html_manager"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$include', '$attr_reader', '$add_panel', '$instance', '$get_panel', '$update_panel', '$new', '$[]=', '$panels', '$to_sym', '$[]', '$find_panel_rec', '$nil?', '$set_value', '$render']);
  self.$require("singleton");
  self.$require("managers/template_manager");
  self.$require("widgets/html");
  return (function($base, $super) {
    function $HtmlManager(){};
    var self = $HtmlManager = $klass($base, $super, 'HtmlManager', $HtmlManager);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$include($scope.get('Singleton'));

    self.$attr_reader("panels");

    Opal.defs(self, '$add_panel', function(panel_id, template_id, model_id, init_data) {
      var self = this;

      if (model_id == null) {
        model_id = nil
      }
      if (init_data == null) {
        init_data = $hash2([], {})
      }
      return self.$instance().$add_panel(panel_id, template_id, model_id, init_data);
    });

    Opal.defs(self, '$get_panel', function(panel_id) {
      var self = this;

      return self.$instance().$get_panel(panel_id);
    });

    Opal.defs(self, '$update_panel', function(panel_id, data) {
      var self = this;

      if (data == null) {
        data = $hash2([], {})
      }
      return self.$instance().$update_panel(panel_id, data);
    });

    def.$initialize = TMP_1 = function() {var $zuper = $slice.call(arguments, 0);
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, $iter).apply(self, $zuper);
      return self.panels = $hash2([], {});
    };

    def.$add_panel = function(panel_id, template_id, model_id, init_data) {
      var self = this, panel_rec = nil;

      panel_rec = $hash2(["html", "model_id", "template_id"], {"html": $scope.get('Html').$new(), "model_id": model_id, "template_id": template_id});
      self.$panels()['$[]='](panel_id.$to_sym(), panel_rec);
      return self.$update_panel(panel_id, init_data);
    };

    def.$find_panel_rec = function(panel_id) {
      var self = this;

      return self.$panels()['$[]'](panel_id.$to_sym());
    };

    def.$get_panel = function(panel_id) {
      var $a, self = this, panel_rec = nil;

      panel_rec = self.$find_panel_rec(panel_id);
      if ((($a = panel_rec['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return $scope.get('Html').$new()
        } else {
        return panel_rec['$[]']("html")
      };
    };

    return (def.$update_panel = function(panel_id, data) {
      var $a, self = this, panel_rec = nil, html = nil;

      panel_rec = self.$find_panel_rec(panel_id);
      if ((($a = panel_rec['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        html = panel_rec['$[]']("html");
        return html.$set_value($scope.get('TemplateManager').$render(panel_rec['$[]']("template_id"), data));
      };
    }, nil) && 'update_panel';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["loaders/desktop_loader"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$include', '$attr_reader', '$build', '$instance', '$set_color_palette', '$set_model_configs', '$set_panel_configs', '$set_root', '$set_stylesheet_configs', '$set_template_configs', '$set_theme', '$set_viewport_class', '$build_css', '$build_managers', '$build_stylesheets', '$build_viewport', '$apply_rules', '$build_models', '$build_templates', '$build_panels', '$[]', '$nil?', '$add_model', '$each', '$build_model_from_config', '$model_configs', '$add_panel', '$build_panel_from_config', '$panel_configs', '$add_rule', '$build_stylesheet_from_config', '$stylesheet_configs', '$add_template', '$build_template_from_config', '$template_configs', '$viewport_class', '$add', '$root', '$widget', '$Native', '$set_dynamic_theme', '$update']);
  self.$require("native");
  self.$require("singleton");
  self.$require("managers/color_manager");
  self.$require("managers/css_manager");
  self.$require("managers/html_manager");
  self.$require("managers/model_manager");
  self.$require("managers/template_manager");
  self.$require("managers/theme_manager");
  self.$require("qx/ui/style_sheet");
  return (function($base, $super) {
    function $DesktopLoader(){};
    var self = $DesktopLoader = $klass($base, $super, 'DesktopLoader', $DesktopLoader);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$include($scope.get('Singleton'));

    self.$attr_reader("model_configs");

    self.$attr_reader("panel_configs");

    self.$attr_reader("root");

    self.$attr_reader("stylesheet_configs");

    self.$attr_reader("template_configs");

    self.$attr_reader("viewport_class");

    Opal.defs(self, '$build', function() {
      var self = this;

      return self.$instance().$build();
    });

    Opal.defs(self, '$set_color_palette', function(palette_name) {
      var self = this;

      return self.$instance().$set_color_palette(palette_name);
    });

    Opal.defs(self, '$set_model_configs', function(cfgs) {
      var self = this;

      return self.$instance().$set_model_configs(cfgs);
    });

    Opal.defs(self, '$set_panel_configs', function(cfgs) {
      var self = this;

      return self.$instance().$set_panel_configs(cfgs);
    });

    Opal.defs(self, '$set_root', function(root) {
      var self = this;

      return self.$instance().$set_root(root);
    });

    Opal.defs(self, '$set_stylesheet_configs', function(cfgs) {
      var self = this;

      return self.$instance().$set_stylesheet_configs(cfgs);
    });

    Opal.defs(self, '$set_template_configs', function(cfgs) {
      var self = this;

      return self.$instance().$set_template_configs(cfgs);
    });

    Opal.defs(self, '$set_theme', function(theme_name) {
      var self = this;

      return self.$instance().$set_theme(theme_name);
    });

    Opal.defs(self, '$set_viewport_class', function(cls) {
      var self = this;

      return self.$instance().$set_viewport_class(cls);
    });

    def.$initialize = TMP_1 = function() {var $zuper = $slice.call(arguments, 0);
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, $iter).apply(self, $zuper);
      self.$set_theme("default");
      self.$set_color_palette("default");
      self.$set_model_configs([]);
      self.$set_panel_configs([]);
      self.$set_stylesheet_configs([]);
      return self.$set_template_configs([]);
    };

    def.$build = function() {
      var self = this;

      self.$build_css();
      self.$build_managers();
      self.$build_stylesheets();
      return self.$build_viewport();
    };

    def.$build_css = function() {
      var self = this;

      return $scope.get('CssManager').$apply_rules();
    };

    def.$build_managers = function() {
      var self = this;

      self.$build_models();
      self.$build_templates();
      return self.$build_panels();
    };

    def.$build_model_from_config = function(cfg) {
      var $a, $b, self = this, name = nil, model_class = nil;

      name = cfg['$[]']("name");
      model_class = cfg['$[]']("model_class");
      if ((($a = ((($b = name['$nil?']()) !== false && $b !== nil) ? $b : model_class['$nil?']())) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        return $scope.get('ModelManager').$add_model(name, model_class)
      };
    };

    def.$build_models = function() {
      var $a, $b, TMP_2, self = this;

      return ($a = ($b = self.$model_configs()).$each, $a.$$p = (TMP_2 = function(cfg){var self = TMP_2.$$s || this;
if (cfg == null) cfg = nil;
      return self.$build_model_from_config(cfg)}, TMP_2.$$s = self, TMP_2), $a).call($b);
    };

    def.$build_panel_from_config = function(cfg) {
      var $a, $b, self = this, name = nil, template = nil, model = nil, init_data = nil;

      name = cfg['$[]']("name");
      template = cfg['$[]']("template");
      model = cfg['$[]']("model");
      init_data = ((($a = cfg['$[]']("init_data")) !== false && $a !== nil) ? $a : $hash2([], {}));
      if ((($a = ((($b = name['$nil?']()) !== false && $b !== nil) ? $b : template['$nil?']())) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        return $scope.get('HtmlManager').$add_panel(name, template, model, init_data)
      };
    };

    def.$build_panels = function() {
      var $a, $b, TMP_3, self = this;

      return ($a = ($b = self.$panel_configs()).$each, $a.$$p = (TMP_3 = function(cfg){var self = TMP_3.$$s || this;
if (cfg == null) cfg = nil;
      return self.$build_panel_from_config(cfg)}, TMP_3.$$s = self, TMP_3), $a).call($b);
    };

    def.$build_stylesheet_from_config = function(cfg) {
      var self = this, selector = nil, css = nil;

      selector = cfg['$[]']("selector");
      css = cfg['$[]']("css");
      return $scope.get('StyleSheet').$add_rule(selector, css);
    };

    def.$build_stylesheets = function() {
      var $a, $b, TMP_4, self = this;

      return ($a = ($b = self.$stylesheet_configs()).$each, $a.$$p = (TMP_4 = function(cfg){var self = TMP_4.$$s || this;
if (cfg == null) cfg = nil;
      return self.$build_stylesheet_from_config(cfg)}, TMP_4.$$s = self, TMP_4), $a).call($b);
    };

    def.$build_template_from_config = function(cfg) {
      var $a, self = this, name = nil, model_name = nil;

      name = cfg['$[]']("name");
      model_name = cfg['$[]']("model_name");
      if ((($a = name['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        return $scope.get('TemplateManager').$add_template(name, model_name)
      };
    };

    def.$build_templates = function() {
      var $a, $b, TMP_5, self = this;

      return ($a = ($b = self.$template_configs()).$each, $a.$$p = (TMP_5 = function(cfg){var self = TMP_5.$$s || this;
if (cfg == null) cfg = nil;
      return self.$build_template_from_config(cfg)}, TMP_5.$$s = self, TMP_5), $a).call($b);
    };

    def.$build_viewport = function() {
      var $a, self = this, vp = nil;

      if ((($a = self.$viewport_class()['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        vp = self.$viewport_class().$instance();
        return self.$root().$add(vp.$widget(), $hash2(["left", "top", "width", "height"], {"left": 0, "top": 0, "width": "100%", "height": "100%"}));
      };
    };

    def.$set_color_palette = function(palette_name) {
      var self = this;

      return $scope.get('ColorManager').$set_color_palette(palette_name);
    };

    def.$set_model_configs = function(cfgs) {
      var self = this;

      return self.model_configs = cfgs;
    };

    def.$set_panel_configs = function(cfgs) {
      var self = this;

      return self.panel_configs = cfgs;
    };

    def.$set_root = function(qx_root) {
      var self = this;

      return self.root = self.$Native(qx_root);
    };

    def.$set_stylesheet_configs = function(cfgs) {
      var self = this;

      return self.stylesheet_configs = cfgs;
    };

    def.$set_template_configs = function(cfgs) {
      var self = this;

      return self.template_configs = cfgs;
    };

    def.$set_theme = function(theme_name) {
      var self = this;

      $scope.get('ThemeManager').$set_dynamic_theme(theme_name);
      return $scope.get('ThemeManager').$update();
    };

    return (def.$set_viewport_class = function(cls) {
      var self = this;

      return self.viewport_class = cls;
    }, nil) && 'set_viewport_class';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["managers/rules/css_toolbar_separator"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require']);
  self.$require("managers/rules/css_rule");
  return (function($base, $super) {
    function $CssToolbarSeparator(){};
    var self = $CssToolbarSeparator = $klass($base, $super, 'CssToolbarSeparator', $CssToolbarSeparator);

    var def = self.$$proto, $scope = self.$$scope;

    def.$css_base_parts = function() {
      var self = this;

      return ["border-right-width: 1px", "border-right-style: solid", "border-left-width: 1px", "border-left-style: solid", "-webkit-background-clip: padding-box", "background-clip: padding-box"];
    };

    def.$css_color_templates = function() {
      var self = this;

      return $hash2(["border_left_color", "border_right_color"], {"border_left_color": "border-left-color: %s", "border_right_color": "border-right-color: %s"});
    };

    def.$name = function() {
      var self = this;

      return "toolbar_separator";
    };

    return (def.$selector = function() {
      var self = this;

      return ".qx-toolbar-separator";
    }, nil) && 'selector';
  })(self, $scope.get('CssRule'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["qx/ui/v_layout"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$nil?', '$initial_spacing', '$setSpacing', '$widget', '$new_ui_layout_vbox']);
  self.$require("qx/ui/widget");
  return (function($base, $super) {
    function $VLayout(){};
    var self = $VLayout = $klass($base, $super, 'VLayout', $VLayout);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    def.$initialize = TMP_1 = function(spacing) {
      var $a, self = this, $iter = TMP_1.$$p, $yield = $iter || nil, sp = nil;

      if (spacing == null) {
        spacing = nil
      }
      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      sp = (function() {if ((($a = spacing['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.$initial_spacing()
        } else {
        return spacing
      }; return nil; })();
      return self.$widget().$setSpacing(sp);
    };

    def.$initial_spacing = function() {
      var self = this;

      return 0;
    };

    return (def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_layout_vbox();
    }, nil) && 'initial_widget';
  })(self, $scope.get('Widget'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["qx/ui/container"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$include', '$initial_layout', '$nil?', '$set_layout', '$set_padding', '$initial_padding', '$add_items', '$add', '$new', '$new_ui_container_composite', '$setLayout', '$widget', '$as_qx', '$setPadding']);
  self.$require("core/qx_wrap");
  self.$require("core/util");
  self.$require("qx/ui/component");
  self.$require("qx/ui/v_layout");
  return (function($base, $super) {
    function $Container(){};
    var self = $Container = $klass($base, $super, 'Container', $Container);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    def.initial_layout = nil;
    self.$include($scope.get('Util'));

    def.$initialize = TMP_1 = function() {var $zuper = $slice.call(arguments, 0);
      var $a, self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, $iter).apply(self, $zuper);
      self.initial_layout = self.$initial_layout();
      if ((($a = self.initial_layout['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$set_layout(self.initial_layout)
      };
      self.$set_padding(self.$initial_padding());
      return self.$add_items();
    };

    def.$add_fill = function(item) {
      var self = this;

      return self.$add(item, $hash2(["flex"], {"flex": 1}));
    };

    def.$add_items = function() {
      var self = this;

      return nil;
    };

    def.$initial_layout = function() {
      var self = this;

      return $scope.get('VLayout').$new();
    };

    def.$initial_padding = function() {
      var self = this;

      return 0;
    };

    def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_container_composite();
    };

    def.$set_layout = function(aLayout) {
      var self = this;

      return self.$widget().$setLayout(self.$as_qx(aLayout));
    };

    return (def.$set_padding = function(n) {
      var self = this;

      return self.$widget().$setPadding(n, n, n, n);
    }, nil) && 'set_padding';
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["qx/ui/h_layout"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$nil?', '$initial_spacing', '$setSpacing', '$widget', '$new_ui_layout_hbox']);
  self.$require("qx/ui/widget");
  return (function($base, $super) {
    function $HLayout(){};
    var self = $HLayout = $klass($base, $super, 'HLayout', $HLayout);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    def.$initialize = TMP_1 = function(spacing) {
      var $a, self = this, $iter = TMP_1.$$p, $yield = $iter || nil, sp = nil;

      if (spacing == null) {
        spacing = nil
      }
      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      sp = (function() {if ((($a = spacing['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.$initial_spacing()
        } else {
        return spacing
      }; return nil; })();
      return self.$widget().$setSpacing(sp);
    };

    def.$initial_spacing = function() {
      var self = this;

      return 0;
    };

    return (def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_layout_hbox();
    }, nil) && 'initial_widget';
  })(self, $scope.get('Widget'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/form_button"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$setLabel', '$widget', '$nil?', '$on_click', '$new_ui_form_button', '$add_listener']);
  self.$require("core/qx_wrap");
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $FormButton(){};
    var self = $FormButton = $klass($base, $super, 'FormButton', $FormButton);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    def.$initialize = TMP_1 = function(label, fn) {
      var $a, self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      if (fn == null) {
        fn = nil
      }
      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      self.$widget().$setLabel(label);
      if ((($a = fn['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        return self.$on_click(fn)
      };
    };

    def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_form_button();
    };

    return (def.$on_click = function(fn) {
      var self = this;

      return self.$add_listener("execute", fn);
    }, nil) && 'on_click';
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/button_bar"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$add', '$new']);
  self.$require("core/util");
  self.$require("qx/ui/container");
  self.$require("qx/ui/h_layout");
  self.$require("widgets/form_button");
  return (function($base, $super) {
    function $ButtonBar(){};
    var self = $ButtonBar = $klass($base, $super, 'ButtonBar', $ButtonBar);

    var def = self.$$proto, $scope = self.$$scope;

    def.$add_button = function(label, fn) {
      var self = this;

      if (fn == null) {
        fn = nil
      }
      return self.$add($scope.get('FormButton').$new(label, fn));
    };

    def.$initial_classes = function() {
      var self = this;

      return ["buttonbar"];
    };

    def.$initial_layout = function() {
      var self = this;

      return $scope.get('HLayout').$new(5);
    };

    return (def.$initial_padding = function() {
      var self = this;

      return 5;
    }, nil) && 'initial_padding';
  })(self, $scope.get('Container'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/crud_buttons"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$include', '$add_crud_buttons', '$add_crud_refresh', '$add_crud_update', '$add_crud_new', '$add_crud_delete', '$[]', '$call', '$nil_handler', '$add_button']);
  self.$require("core/util");
  self.$require("widgets/button_bar");
  return (function($base, $super) {
    function $CrudButtons(){};
    var self = $CrudButtons = $klass($base, $super, 'CrudButtons', $CrudButtons);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    def.handlers = nil;
    self.$include($scope.get('Util'));

    def.$initialize = TMP_1 = function(handlers) {
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      if (handlers == null) {
        handlers = $hash2([], {})
      }
      TMP_1.$$p = null;
      self.handlers = handlers;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      return self.$add_crud_buttons();
    };

    def.$add_crud_buttons = function() {
      var self = this;

      self.$add_crud_refresh();
      self.$add_crud_update();
      self.$add_crud_new();
      return self.$add_crud_delete();
    };

    def.$add_crud_delete = function() {
      var $a, self = this, fn = nil;

      fn = (function() {if ((($a = self.handlers['$[]']("delete")) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.handlers['$[]']("delete")
        } else {
        return self.$nil_handler().$call("Delete")
      }; return nil; })();
      return self.$add_button("Delete", fn);
    };

    def.$add_crud_new = function() {
      var $a, self = this, fn = nil;

      fn = (function() {if ((($a = self.handlers['$[]']("create")) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.handlers['$[]']("create")
        } else {
        return self.$nil_handler().$call("New")
      }; return nil; })();
      return self.$add_button("New", fn);
    };

    def.$add_crud_refresh = function() {
      var $a, self = this, fn = nil;

      fn = (function() {if ((($a = self.handlers['$[]']("refresh")) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.handlers['$[]']("refresh")
        } else {
        return self.$nil_handler().$call("Refresh")
      }; return nil; })();
      return self.$add_button("Refresh", fn);
    };

    return (def.$add_crud_update = function() {
      var $a, self = this, fn = nil;

      fn = (function() {if ((($a = self.handlers['$[]']("update")) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.handlers['$[]']("update")
        } else {
        return self.$nil_handler().$call("Update")
      }; return nil; })();
      return self.$add_button("Update", fn);
    }, nil) && 'add_crud_update';
  })(self, $scope.get('ButtonBar'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["qx/ui/basic_layout"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$new_ui_layout_basic']);
  self.$require("core/qx_wrap");
  self.$require("qx/ui/widget");
  return (function($base, $super) {
    function $BasicLayout(){};
    var self = $BasicLayout = $klass($base, $super, 'BasicLayout', $BasicLayout);

    var def = self.$$proto, $scope = self.$$scope;

    return (def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_layout_basic();
    }, nil) && 'initial_widget'
  })(self, $scope.get('Widget'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/form_text_field"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$getValue', '$widget', '$new_ui_form_text_field', '$setValue']);
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $FormTextField(){};
    var self = $FormTextField = $klass($base, $super, 'FormTextField', $FormTextField);

    var def = self.$$proto, $scope = self.$$scope;

    def.$get_value = function() {
      var self = this;

      return self.$widget().$getValue();
    };

    def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_form_text_field();
    };

    return (def.$set_value = function(s) {
      var self = this;

      return self.$widget().$setValue(s);
    }, nil) && 'set_value';
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/form_form"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$include', '$attr_reader', '$add', '$widget', '$as_native', '$new', '$map_items', '$each', '$[]', '$item_map', '$[]=', '$keys', '$new_ui_form_form', '$reset', '$key?', '$id_sym', '$text_from_id', '$add_text_field', '$to_s']);
  self.$require("native");
  self.$require("core/qx_wrap");
  self.$require("core/util");
  self.$require("qx/ui/component");
  self.$require("widgets/form_text_field");
  return (function($base, $super) {
    function $FormForm(){};
    var self = $FormForm = $klass($base, $super, 'FormForm', $FormForm);

    var def = self.$$proto, $scope = self.$$scope;

    self.$include($scope.get('Util'));

    self.$attr_reader("item_map");

    def.$add = function(item, label, name, validator) {
      var self = this;

      if (validator == null) {
        validator = nil
      }
      return self.$widget().$add(self.$as_native(item), label, validator, name);
    };

    def.$add_text_field = function(label, name) {
      var self = this, field = nil;

      field = $scope.get('FormTextField').$new();
      return self.$add(field, label, name);
    };

    def.$get_values = function() {
      var $a, $b, TMP_1, self = this, values = nil;

      self.$map_items();
      values = $hash2([], {});
      ($a = ($b = self.$item_map().$keys()).$each, $a.$$p = (TMP_1 = function(k){var self = TMP_1.$$s || this, field = nil;
if (k == null) k = nil;
      field = self.$item_map()['$[]'](k);
        return values['$[]='](k, field.native.getValue());}, TMP_1.$$s = self, TMP_1), $a).call($b);
      return values;
    };

    def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_form_form();
    };

    def.$map_items = function() {
      var self = this;

      return self.item_map = $scope.get('Hash').$new(self.$widget().native.getItems());
    };

    def.$reset = function() {
      var self = this;

      return self.$widget().$reset();
    };

    def.$set_fields_from_config = function(fields) {
      var $a, $b, TMP_2, self = this;

      ($a = ($b = fields).$each, $a.$$p = (TMP_2 = function(field){var self = TMP_2.$$s || this, $a, label = nil, name = nil;
if (field == null) field = nil;
      label = (function() {if ((($a = (field['$key?']("text"))) !== nil && (!$a.$$is_boolean || $a == true))) {
          return field['$[]']("text")
          } else {
          return "--"
        }; return nil; })();
        name = (function() {if ((($a = (field['$key?']("id"))) !== nil && (!$a.$$is_boolean || $a == true))) {
          return field['$[]']("id")
          } else {
          return self.$id_sym(label)
        }; return nil; })();
        if ((($a = field['$key?']("text")) !== nil && (!$a.$$is_boolean || $a == true))) {
          } else {
          label = self.$text_from_id(name)
        };
        return self.$add_text_field(label, name);}, TMP_2.$$s = self, TMP_2), $a).call($b);
      return self.$map_items();
    };

    return (def.$set_values = function(record) {
      var $a, $b, TMP_3, self = this;

      return ($a = ($b = record).$each, $a.$$p = (TMP_3 = function(k, v){var self = TMP_3.$$s || this, $a, field = nil;
if (k == null) k = nil;if (v == null) v = nil;
      if ((($a = self.$item_map()['$key?'](k)) !== nil && (!$a.$$is_boolean || $a == true))) {
          field = self.$item_map()['$[]'](k);
          return field.native.setValue(v.$to_s());
          } else {
          return nil
        }}, TMP_3.$$s = self, TMP_3), $a).call($b);
    }, nil) && 'set_values';
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/form_renderer_single"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$include', '$as_qx', '$new_ui_form_renderer_single', '$as_native', '$adjust_layout', '$Native', '$getLayout', '$setColumnFlex']);
  self.$require("native");
  self.$require("core/qx_wrap");
  self.$require("core/util");
  self.$require("qx/ui/widget");
  return (function($base, $super) {
    function $FormRendererSingle(){};
    var self = $FormRendererSingle = $klass($base, $super, 'FormRendererSingle', $FormRendererSingle);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$include($scope.get('Util'));

    def.$initialize = TMP_1 = function(form) {
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      self.widget = self.$as_qx($scope.get('QxWrap').$new_ui_form_renderer_single(self.$as_native(form)));
      return self.$adjust_layout();
    };

    def.$adjust_layout = function() {
      var self = this, layout = nil;

      layout = self.$Native(self.$getLayout());
      layout.$setColumnFlex(0, nil);
      return layout.$setColumnFlex(1, 1);
    };

    return (def.$initial_widget = function() {
      var self = this;

      return nil;
    }, nil) && 'initial_widget';
  })(self, $scope.get('Widget'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/grid"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$attr_accessor', '$attr_reader', '$include', '$setStatusBarVisible', '$widget', '$add_selection_change_listener', '$Native', '$addListener', '$lambda', '$handle_selection_change', '$getData', '$first', '$nil?', '$reset_selection', '$show_selection', '$new_lib_ui_grid_grid', '$get_data', '$find_index', '$==', '$select_index', '$setColumnFlex', '$setColumnWidth', '$setColumns', '$each', '$key?', '$[]', '$id_sym', '$text_from_id', '$<<', '$[]=', '$set_columns', '$each_with_index', '$set_column_width', '$set_column_flex', '$setDataAsMapArray', '$set_data_as_map_array', '$data', '$store', '$select_id', '$add_data_consumer', '$selection_handler', '$call']);
  self.$require("core/util");
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $Grid(){};
    var self = $Grid = $klass($base, $super, 'Grid', $Grid);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    def.selection_model = nil;
    self.$attr_accessor("selection_handler");

    self.$attr_reader("selection_model");

    self.$attr_reader("store");

    self.$include($scope.get('Util'));

    def.$initialize = TMP_1 = function() {var $zuper = $slice.call(arguments, 0);
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, $iter).apply(self, $zuper);
      self.selection_handler = nil;
      self.store = nil;
      return self.$widget().$setStatusBarVisible(false);
    };

    def.$add_listeners = function() {
      var self = this;

      return self.$add_selection_change_listener();
    };

    def.$add_selection_change_listener = function() {
      var $a, $b, TMP_2, self = this;

      self.selection_model = self.$Native(self.$widget().native.$$user_selectionModel);
      return self.selection_model.$addListener("changeSelection", ($a = ($b = self).$lambda, $a.$$p = (TMP_2 = function(){var self = TMP_2.$$s || this;

      return self.$handle_selection_change()}, TMP_2.$$s = self, TMP_2), $a).call($b));
    };

    def.$get_data = function() {
      var self = this;

      return self.$widget().$getData();
    };

    def.$handle_selection_change = function() {
      var $a, self = this, ranges = nil, range = nil, minIndex = nil;

      ranges = self.$Native(self.selection_model.native.getSelectedRanges());
      range = self.$Native(ranges.$first());
      if ((($a = range['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.$reset_selection()
        } else {
        minIndex = range.native.minIndex;
        return self.$show_selection(minIndex);
      };
    };

    def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_lib_ui_grid_grid();
    };

    def.$reset_selection = function() {
      var self = this;

      return self.selection_model.native.resetSelection();
    };

    def.$select_id = function(id) {
      var $a, $b, TMP_3, self = this, data = nil, n = nil;

      self.$reset_selection();
      data = self.$get_data();
      n = ($a = ($b = data).$find_index, $a.$$p = (TMP_3 = function(x){var self = TMP_3.$$s || this;
if (x == null) x = nil;
      return x.$first()['$=='](id)}, TMP_3.$$s = self, TMP_3), $a).call($b);
      if ((($a = n['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        return self.$select_index(n)
      };
    };

    def.$select_index = function(n) {
      var self = this;

      return self.selection_model.native.setSelectionInterval(n, n);
    };

    def.$set_column_flex = function(col, flex) {
      var self = this;

      return self.$widget().$setColumnFlex(col, flex);
    };

    def.$set_column_width = function(col, width) {
      var self = this;

      return self.$widget().$setColumnWidth(col, width);
    };

    def.$set_columns = function(column_titles, column_ids) {
      var self = this;

      return self.$widget().$setColumns(column_titles, column_ids);
    };

    def.$set_columns_from_config = function(cols) {
      var $a, $b, TMP_4, $c, TMP_5, self = this, col_texts = nil, col_ids = nil, widths = nil;

      col_texts = [];
      col_ids = [];
      widths = [];
      ($a = ($b = cols).$each, $a.$$p = (TMP_4 = function(col){var self = TMP_4.$$s || this, $a, text = nil, id = nil, width = nil;
if (col == null) col = nil;
      text = (function() {if ((($a = (col['$key?']("text"))) !== nil && (!$a.$$is_boolean || $a == true))) {
          return col['$[]']("text")
          } else {
          return "--"
        }; return nil; })();
        id = (function() {if ((($a = (col['$key?']("id"))) !== nil && (!$a.$$is_boolean || $a == true))) {
          return col['$[]']("id")
          } else {
          return self.$id_sym(text)
        }; return nil; })();
        if ((($a = col['$key?']("text")) !== nil && (!$a.$$is_boolean || $a == true))) {
          } else {
          text = self.$text_from_id(id)
        };
        col_texts['$<<'](text);
        col_ids['$<<'](id);
        width = $hash2([], {});
        if ((($a = col['$key?']("width")) !== nil && (!$a.$$is_boolean || $a == true))) {
          width['$[]=']("width", col['$[]']("width"))};
        if ((($a = col['$key?']("flex")) !== nil && (!$a.$$is_boolean || $a == true))) {
          width['$[]=']("flex", col['$[]']("flex"))};
        return widths['$<<'](width);}, TMP_4.$$s = self, TMP_4), $a).call($b);
      self.$set_columns(col_texts, col_ids);
      return ($a = ($c = widths).$each_with_index, $a.$$p = (TMP_5 = function(width, i){var self = TMP_5.$$s || this, $a;
if (width == null) width = nil;if (i == null) i = nil;
      if ((($a = (width['$key?']("width"))) !== nil && (!$a.$$is_boolean || $a == true))) {
          return self.$set_column_width(i, width['$[]']("width"))
        } else if ((($a = (width['$key?']("flex"))) !== nil && (!$a.$$is_boolean || $a == true))) {
          return self.$set_column_flex(i, width['$[]']("flex"))
          } else {
          return self.$set_column_flex(i, 1)
        }}, TMP_5.$$s = self, TMP_5), $a).call($c);
    };

    def.$set_data_as_map_array = function(map_array) {
      var self = this;

      return self.$widget().$setDataAsMapArray(map_array);
    };

    def.$store_changed = function() {
      var self = this;

      return self.$set_data_as_map_array(self.$store().$data());
    };

    def.$store_changed_id = function(id) {
      var self = this;

      return self.$select_id(id);
    };

    def.$set_store = function(aStore) {
      var self = this;

      self.store = aStore;
      return self.$store().$add_data_consumer(self);
    };

    return (def.$show_selection = function(n) {
      var $a, self = this;

      if ((($a = self.$selection_handler()['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        return self.$selection_handler().$call(n)
      };
    }, nil) && 'show_selection';
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/split_pane"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$include', '$nil?', '$set_orientation', '$add', '$widget', '$as_qx', '$new_ui_splitpane_pane', '$setOrientation']);
  self.$require("core/util");
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $SplitPane(){};
    var self = $SplitPane = $klass($base, $super, 'SplitPane', $SplitPane);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$include($scope.get('Util'));

    def.$initialize = TMP_1 = function(orientation) {
      var $a, self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      if (orientation == null) {
        orientation = nil
      }
      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      if ((($a = orientation['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        return self.$set_orientation(orientation)
      };
    };

    def.$add = function(aWidget, flex) {
      var self = this;

      if (flex == null) {
        flex = 1
      }
      return self.$widget().$add(self.$as_qx(aWidget), flex);
    };

    def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_splitpane_pane();
    };

    return (def.$set_orientation = function(anOrientation) {
      var self = this;

      return self.$widget().$setOrientation(anOrientation);
    }, nil) && 'set_orientation';
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/grid_form"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$attr_reader', '$nil?', '$make_grid', '$make_form_container', '$add', '$grid', '$form_container', '$new', '$set_fields_from_config', '$form', '$set_padding', '$make_form', '$set_columns_from_config', '$selection_handler=', '$lambda', '$selection_changed', '$reset', '$reset_selection', '$select_id', '$[]', '$get_data', '$show_record', '$first', '$add_data_consumer', '$set_store', '$get_data_record', '$store', '$set_values', '$selected_record', '$reset_form', '$get_values']);
  self.$require("qx/ui/basic_layout");
  self.$require("qx/ui/container");
  self.$require("widgets/form_form");
  self.$require("widgets/form_renderer_single");
  self.$require("widgets/form_form");
  self.$require("widgets/form_renderer_single");
  self.$require("widgets/grid");
  self.$require("widgets/split_pane");
  return (function($base, $super) {
    function $GridForm(){};
    var self = $GridForm = $klass($base, $super, 'GridForm', $GridForm);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    def.form_config = def.grid_config = nil;
    self.$attr_reader("form");

    self.$attr_reader("form_container");

    self.$attr_reader("grid");

    self.$attr_reader("selected_record");

    self.$attr_reader("store");

    def.$initialize = TMP_1 = function(data_config, grid_config, form_config) {
      var $a, self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      if (grid_config == null) {
        grid_config = nil
      }
      if (form_config == null) {
        form_config = nil
      }
      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      self.data_config = data_config;
      self.grid_config = (function() {if ((($a = grid_config['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return data_config
        } else {
        return grid_config
      }; return nil; })();
      self.form_config = (function() {if ((($a = form_config['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return data_config
        } else {
        return form_config
      }; return nil; })();
      self.selected_record = nil;
      self.store = nil;
      self.$make_grid();
      self.$make_form_container();
      self.$add(self.$grid());
      return self.$add(self.$form_container());
    };

    def.$make_form = function() {
      var self = this;

      self.form = $scope.get('FormForm').$new();
      return self.$form().$set_fields_from_config(self.form_config);
    };

    def.$make_form_container = function() {
      var self = this, form_container = nil, renderer = nil;

      form_container = $scope.get('Container').$new();
      form_container.$set_padding(10);
      self.$make_form();
      renderer = $scope.get('FormRendererSingle').$new(self.$form());
      form_container.$add(renderer, $hash2(["flex"], {"flex": 1}));
      return self.form_container = form_container;
    };

    def.$make_grid = function() {
      var $a, $b, $c, $d, TMP_2, self = this;

      self.grid = $scope.get('Grid').$new();
      self.$grid().$set_columns_from_config(self.grid_config);
      return (($a = [($c = ($d = self).$lambda, $c.$$p = (TMP_2 = function(n){var self = TMP_2.$$s || this;
if (n == null) n = nil;
      return self.$selection_changed(n)}, TMP_2.$$s = self, TMP_2), $c).call($d)]), $b = self.$grid(), $b['$selection_handler='].apply($b, $a), $a[$a.length-1]);
    };

    def.$reset_form = function() {
      var self = this;

      return self.$form().$reset();
    };

    def.$reset_selection = function() {
      var self = this;

      return self.$grid().$reset_selection();
    };

    def.$select_grid_id = function(id) {
      var self = this;

      return self.$grid().$select_id(id);
    };

    def.$selection_changed = function(n) {
      var self = this, rec = nil;

      rec = self.$grid().$get_data()['$[]'](n);
      return self.$show_record(rec.$first());
    };

    def.$set_store = function(store) {
      var self = this;

      self.store = store;
      store.$add_data_consumer(self);
      return self.$grid().$set_store(store);
    };

    def.$show_record = function(rec_id) {
      var self = this;

      self.selected_record = self.$store().$get_data_record(rec_id);
      return self.$form().$set_values(self.$selected_record());
    };

    def.$store_changed = function() {
      var self = this;

      self.$reset_form();
      return self.$reset_selection();
    };

    def.$store_changed_id = function(id) {
      var self = this;

      return nil;
    };

    return (def.$update_selected_record = function() {
      var self = this;

      return self.selected_record = self.$form().$get_values();
    }, nil) && 'update_selected_record';
  })(self, $scope.get('SplitPane'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["panels/crud_panel"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$include', '$attr_reader', '$find_model', '$api_resource_name', '$model', '$data_config', '$grid_config', '$find_store', '$set_store', '$grid_form', '$store', '$initial_button_bar', '$nil?', '$bbar', '$add', '$initial_create_handler', '$initial_delete_handler', '$initial_refresh_handler', '$initial_update_handler', '$add_grid_form', '$add_button_bar', '$new', '$initial_handler_functions', '$create_handler', '$delete_handler', '$refresh_handler', '$update_handler', '$lambda', '$data_create', '$selected_record', '$data_delete', '$[]', '$data_load', '$update_selected_record', '$data_update']);
  self.$require("core/util");
  self.$require("managers/model_manager");
  self.$require("managers/store_manager");
  self.$require("qx/io/server");
  self.$require("qx/ui/container");
  self.$require("widgets/crud_buttons");
  self.$require("widgets/grid_form");
  return (function($base, $super) {
    function $CrudPanel(){};
    var self = $CrudPanel = $klass($base, $super, 'CrudPanel', $CrudPanel);

    var def = self.$$proto, $scope = self.$$scope, TMP_1, TMP_2;

    self.$include($scope.get('Util'));

    self.$attr_reader("api_resource_name");

    self.$attr_reader("bbar");

    self.$attr_reader("create_handler");

    self.$attr_reader("data_config");

    self.$attr_reader("delete_handler");

    self.$attr_reader("grid_config");

    self.$attr_reader("grid_form");

    self.$attr_reader("model");

    self.$attr_reader("refresh_handler");

    self.$attr_reader("store");

    self.$attr_reader("update_handler");

    def.$initialize = TMP_1 = function(model_name) {
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      if (model_name == null) {
        model_name = "dummy"
      }
      TMP_1.$$p = null;
      self.model = $scope.get('ModelManager').$find_model(model_name);
      self.api_resource_name = self.$model().$api_resource_name();
      self.data_config = self.$model().$data_config();
      self.grid_config = self.$model().$grid_config();
      self.store = $scope.get('StoreManager').$find_store(self.$api_resource_name());
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      return self.$grid_form().$set_store(self.$store());
    };

    def.$add_button_bar = function() {
      var $a, self = this;

      self.bbar = self.$initial_button_bar();
      if ((($a = self.$bbar()['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        return self.$add(self.$bbar())
      };
    };

    def.$add_handlers = TMP_2 = function() {var $zuper = $slice.call(arguments, 0);
      var self = this, $iter = TMP_2.$$p, $yield = $iter || nil;

      TMP_2.$$p = null;
      Opal.find_super_dispatcher(self, 'add_handlers', TMP_2, $iter).apply(self, $zuper);
      self.create_handler = self.$initial_create_handler();
      self.delete_handler = self.$initial_delete_handler();
      self.refresh_handler = self.$initial_refresh_handler();
      return self.update_handler = self.$initial_update_handler();
    };

    def.$add_items = function() {
      var self = this;

      self.$add_grid_form();
      return self.$add_button_bar();
    };

    def.$add_grid_form = function() {
      var self = this;

      self.grid_form = $scope.get('GridForm').$new(self.$data_config(), self.$grid_config());
      return self.$add(self.$grid_form(), $hash2(["flex"], {"flex": 1}));
    };

    def.$initial_button_bar = function() {
      var self = this;

      return $scope.get('CrudButtons').$new(self.$initial_handler_functions());
    };

    def.$initial_handler_functions = function() {
      var self = this;

      return $hash2(["create", "delete", "refresh", "update"], {"create": self.$create_handler(), "delete": self.$delete_handler(), "refresh": self.$refresh_handler(), "update": self.$update_handler()});
    };

    def.$initial_create_handler = function() {
      var $a, $b, TMP_3, self = this;

      return ($a = ($b = self).$lambda, $a.$$p = (TMP_3 = function(){var self = TMP_3.$$s || this;

      return self.$store().$data_create()}, TMP_3.$$s = self, TMP_3), $a).call($b);
    };

    def.$initial_delete_handler = function() {try {

      var $a, $b, TMP_4, self = this;

      return ($a = ($b = self).$lambda, $a.$$p = (TMP_4 = function(){var self = TMP_4.$$s || this, $a, selected_record = nil;

      selected_record = self.$grid_form().$selected_record();
        if ((($a = selected_record['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
          Opal.ret(nil)};
        return self.$store().$data_delete(selected_record['$[]']("id"));}, TMP_4.$$s = self, TMP_4), $a).call($b);
      } catch ($returner) { if ($returner === Opal.returner) { return $returner.$v } throw $returner; }
    };

    def.$initial_refresh_handler = function() {
      var $a, $b, TMP_5, self = this;

      return ($a = ($b = self).$lambda, $a.$$p = (TMP_5 = function(){var self = TMP_5.$$s || this;

      return self.$store().$data_load()}, TMP_5.$$s = self, TMP_5), $a).call($b);
    };

    def.$initial_update_handler = function() {try {

      var $a, $b, TMP_6, self = this;

      return ($a = ($b = self).$lambda, $a.$$p = (TMP_6 = function(){var self = TMP_6.$$s || this, $a, selected_record = nil;

      self.$grid_form().$update_selected_record();
        selected_record = self.$grid_form().$selected_record();
        if ((($a = selected_record['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
          Opal.ret(nil)};
        return self.$store().$data_update(selected_record);}, TMP_6.$$s = self, TMP_6), $a).call($b);
      } catch ($returner) { if ($returner === Opal.returner) { return $returner.$v } throw $returner; }
    };

    return (def.$model_name = function() {
      var self = this;

      return "dummy";
    }, nil) && 'model_name';
  })(self, $scope.get('Container'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/atom"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$attr_reader', '$include', '$setIconPosition', '$widget', '$setRich', '$nil?', '$on_click', '$%', '$resolve_color', '$label', '$new_ui_basic_atom', '$icon']);
  self.$require("managers/color_manager");
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $Atom(){};
    var self = $Atom = $klass($base, $super, 'Atom', $Atom);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$attr_reader("icon");

    self.$attr_reader("label");

    self.$include($scope.get('Util'));

    def.$initialize = TMP_1 = function(label, icon, fn) {
      var $a, self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      if (fn == null) {
        fn = nil
      }
      TMP_1.$$p = null;
      self.label = label;
      self.icon = icon;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      self.$widget().$setIconPosition("top");
      self.$widget().$setRich(true);
      if ((($a = fn['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        return self.$on_click(fn)
      };
    };

    return (def.$initial_widget = function() {
      var self = this, label_html = nil;

      label_html = "<span style=\"color:%s\">%s</span>"['$%']([$scope.get('ColorManager').$resolve_color("desktop_icon"), self.$label()]);
      return $scope.get('QxWrap').$new_ui_basic_atom(label_html, self.$icon());
    }, nil) && 'initial_widget';
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["panels/desktop_panel"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$include', '$attr_reader', '$set_background_color', '$resolve_color', '$each', '$add_icon', '$[]', '$icons', '$lambda', '$handle_action', '$instance', '$controller_class', '$new', '$add', '$top']);
  self.$require("core/icons");
  self.$require("qx/ui/basic_layout");
  self.$require("qx/ui/container");
  self.$require("widgets/atom");
  return (function($base, $super) {
    function $DesktopPanel(){};
    var self = $DesktopPanel = $klass($base, $super, 'DesktopPanel', $DesktopPanel);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$include($scope.get('Icons'));

    self.$attr_reader("controller_class");

    self.$attr_reader("icons");

    self.$attr_reader("left");

    self.$attr_reader("top");

    def.$initialize = TMP_1 = function(cls, icon_array) {
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      if (icon_array == null) {
        icon_array = []
      }
      TMP_1.$$p = null;
      self.controller_class = cls;
      self.icons = icon_array;
      self.top = 10;
      self.left = 10;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      return self.$set_background_color($scope.get('ColorManager').$resolve_color("desktop_background"));
    };

    def.$add_items = function() {
      var $a, $b, TMP_2, self = this;

      return ($a = ($b = self.$icons()).$each, $a.$$p = (TMP_2 = function(icon){var self = TMP_2.$$s || this;
if (icon == null) icon = nil;
      return self.$add_icon(icon['$[]']("label"), icon['$[]']("action"))}, TMP_2.$$s = self, TMP_2), $a).call($b);
    };

    def.$add_icon = function(label, action) {
      var $a, $b, TMP_3, self = this, fn = nil, atom = nil;

      fn = ($a = ($b = self).$lambda, $a.$$p = (TMP_3 = function(){var self = TMP_3.$$s || this;

      return self.$controller_class().$instance().$handle_action(action)}, TMP_3.$$s = self, TMP_3), $a).call($b);
      atom = $scope.get('Atom').$new(label, $scope.get('WINDOW_ICON'), fn);
      self.$add(atom, $hash2(["top", "left"], {"top": self.$top(), "left": 10}));
      return self.top = $rb_plus(self.$top(), 100);
    };

    return (def.$initial_layout = function() {
      var self = this;

      return $scope.get('BasicLayout').$new();
    }, nil) && 'initial_layout';
  })(self, $scope.get('Container'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["panels/nav_panel"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$attr_reader', '$include', '$setSpacing', '$getLayout', '$widget', '$setPadding', '$set_background_color', '$resolve_color', '$[]', '$config', '$each', '$new', '$lambda', '$nil?', '$controller', '$handle_action', '$add', '$add_buttons']);
  self.$require("core/dimensions");
  self.$require("qx/ui/container");
  self.$require("widgets/form_button");
  return (function($base, $super) {
    function $NavPanel(){};
    var self = $NavPanel = $klass($base, $super, 'NavPanel', $NavPanel);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$attr_reader("config");

    self.$attr_reader("controller");

    self.$include($scope.get('Dimensions'));

    def.$initialize = TMP_1 = function(config, controller) {
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      if (controller == null) {
        controller = nil
      }
      TMP_1.$$p = null;
      self.config = config;
      self.controller = controller;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      self.$widget().$getLayout().$setSpacing(10);
      self.$widget().$setPadding(35, 15);
      return self.$set_background_color($scope.get('ColorManager').$resolve_color("navpanel_background"));
    };

    def.$add_buttons = function() {
      var $a, $b, TMP_2, self = this, buttons = nil;

      buttons = self.$config()['$[]']("buttons");
      return ($a = ($b = buttons).$each, $a.$$p = (TMP_2 = function(button){var self = TMP_2.$$s || this, $a, $b, TMP_3, text = nil, action = nil, arg = nil, btn = nil;
if (button == null) button = nil;
      text = button['$[]']("text");
        action = button['$[]']("action");
        arg = button['$[]']("arg");
        btn = $scope.get('FormButton').$new(text, ($a = ($b = self).$lambda, $a.$$p = (TMP_3 = function(){var self = TMP_3.$$s || this, $a;

        if ((($a = self.$controller()['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return nil
            } else {
            return self.$controller().$handle_action(action, arg)
          }}, TMP_3.$$s = self, TMP_3), $a).call($b));
        return self.$add(btn);}, TMP_2.$$s = self, TMP_2), $a).call($b);
    };

    def.$add_items = function() {
      var self = this;

      return self.$add_buttons();
    };

    def.$initial_classes = function() {
      var self = this;

      return ["navpanel"];
    };

    return (def.$initial_width = function() {
      var self = this;

      return $scope.get('NAV_PANEL_WIDTH');
    }, nil) && 'initial_width';
  })(self, $scope.get('Container'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["qx/ui/scroll"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$new_ui_container_scroll']);
  self.$require("core/qx_wrap");
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $Scroll(){};
    var self = $Scroll = $klass($base, $super, 'Scroll', $Scroll);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    def.$initialize = TMP_1 = function(aComponent) {
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      TMP_1.$$p = null;
      return Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
    };

    return (def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_container_scroll();
    }, nil) && 'initial_widget';
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["qx/ui/stack_container"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$attr_reader', '$[]=', '$content_map', '$to_sym', '$add', '$new_ui_container_stack', '$[]', '$nil?', '$setSelection', '$as_qx']);
  self.$require("core/qx_wrap");
  self.$require("core/util");
  self.$require("qx/ui/container");
  self.$require("qx/ui/v_layout");
  return (function($base, $super) {
    function $StackContainer(){};
    var self = $StackContainer = $klass($base, $super, 'StackContainer', $StackContainer);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$attr_reader("content_map");

    def.$initialize = TMP_1 = function() {var $zuper = $slice.call(arguments, 0);
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, $iter).apply(self, $zuper);
      return self.content_map = $hash2([], {});
    };

    def.$add_item = function(tag, item) {
      var self = this;

      self.$content_map()['$[]='](tag.$to_sym(), item);
      return self.$add(item);
    };

    def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_container_stack();
    };

    return (def.$set_selection = function(tag) {
      var $a, self = this, item = nil;

      item = self.$content_map()['$[]'](tag.$to_sym());
      if ((($a = item['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        return self.$setSelection([self.$as_qx(item)])
      };
    }, nil) && 'set_selection';
  })(self, $scope.get('Container'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["qx/ui/window"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$attr_reader', '$add_button_bar', '$add_listener_appear', '$set_caption', '$initial_caption', '$set_content_padding', '$initial_content_padding', '$is_centered?', '$move_to', '$initial_x', '$initial_y', '$show', '$initial_button_bar', '$nil?', '$bbar', '$add', '$center', '$widget', '$new_ui_window_window', '$moveTo', '$setCaption', '$setContentPadding']);
  self.$require("core/qx_wrap");
  self.$require("qx/ui/container");
  return (function($base, $super) {
    function $Window(){};
    var self = $Window = $klass($base, $super, 'Window', $Window);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$attr_reader("bbar");

    def.$initialize = TMP_1 = function() {var $zuper = $slice.call(arguments, 0);
      var $a, self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, $iter).apply(self, $zuper);
      self.$add_button_bar();
      self.$add_listener_appear();
      self.$set_caption(self.$initial_caption());
      self.$set_content_padding(self.$initial_content_padding());
      if ((($a = self['$is_centered?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$move_to(self.$initial_x(), self.$initial_y())
      };
      return self.$show();
    };

    def.$add_button_bar = function() {
      var $a, self = this;

      self.bbar = self.$initial_button_bar();
      if ((($a = self.$bbar()['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        return self.$add(self.$bbar())
      };
    };

    def.$center = function() {
      var self = this;

      return self.$widget().$center();
    };

    def.$initial_button_bar = function() {
      var self = this;

      return nil;
    };

    def.$initial_caption = function() {
      var self = this;

      return "a Window";
    };

    def.$initial_content_padding = function() {
      var self = this;

      return 0;
    };

    def.$initial_handler_functions = function() {
      var self = this;

      return $hash2([], {});
    };

    def.$initial_height = function() {
      var self = this;

      return 350;
    };

    def.$initial_width = function() {
      var self = this;

      return 475;
    };

    def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_window_window();
    };

    def.$initial_x = function() {
      var self = this;

      return 15;
    };

    def.$initial_y = function() {
      var self = this;

      return 55;
    };

    def['$is_centered?'] = function() {
      var self = this;

      return false;
    };

    def.$move_to = function(x, y) {
      var self = this;

      return self.$widget().$moveTo(x, y);
    };

    def.$on_appear = function() {
      var $a, self = this;

      if ((($a = self['$is_centered?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.$center()
        } else {
        return nil
      };
    };

    def.$set_caption = function(aString) {
      var self = this;

      return self.$widget().$setCaption(aString);
    };

    def.$set_content_padding = function(n) {
      var self = this;

      return self.$widget().$setContentPadding(n);
    };

    return (def.$show = function() {
      var self = this;

      return self.$widget().$show();
    }, nil) && 'show';
  })(self, $scope.get('Container'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["set"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $module = Opal.module;

  Opal.add_stubs(['$include', '$new', '$nil?', '$===', '$raise', '$each', '$add', '$call', '$merge', '$class', '$respond_to?', '$subtract', '$dup', '$join', '$to_a', '$equal?', '$instance_of?', '$==', '$instance_variable_get', '$is_a?', '$size', '$all?', '$include?', '$[]=', '$enum_for', '$[]', '$<<', '$replace', '$delete', '$select', '$each_key', '$to_proc', '$empty?', '$eql?', '$instance_eval', '$clear', '$keys']);
  (function($base, $super) {
    function $Set(){};
    var self = $Set = $klass($base, $super, 'Set', $Set);

    var def = self.$$proto, $scope = self.$$scope, TMP_1, TMP_4, TMP_7, TMP_9, TMP_12;

    def.hash = nil;
    self.$include($scope.get('Enumerable'));

    Opal.defs(self, '$[]', function(ary) {
      var self = this;

      ary = $slice.call(arguments, 0);
      return self.$new(ary);
    });

    def.$initialize = TMP_1 = function(enum$) {
      var $a, $b, TMP_2, self = this, $iter = TMP_1.$$p, block = $iter || nil;

      if (enum$ == null) {
        enum$ = nil
      }
      TMP_1.$$p = null;
      self.hash = $scope.get('Hash').$new();
      if ((($a = enum$['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil};
      if ((($a = $scope.get('Enumerable')['$==='](enum$)) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise($scope.get('ArgumentError'), "value must be enumerable")
      };
      if (block !== false && block !== nil) {
        return ($a = ($b = enum$).$each, $a.$$p = (TMP_2 = function(item){var self = TMP_2.$$s || this;
if (item == null) item = nil;
        return self.$add(block.$call(item))}, TMP_2.$$s = self, TMP_2), $a).call($b)
        } else {
        return self.$merge(enum$)
      };
    };

    def.$dup = function() {
      var self = this, result = nil;

      result = self.$class().$new();
      return result.$merge(self);
    };

    def['$-'] = function(enum$) {
      var $a, self = this;

      if ((($a = enum$['$respond_to?']("each")) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise($scope.get('ArgumentError'), "value must be enumerable")
      };
      return self.$dup().$subtract(enum$);
    };

    Opal.defn(self, '$difference', def['$-']);

    def.$inspect = function() {
      var self = this;

      return "#<Set: {" + (self.$to_a().$join(",")) + "}>";
    };

    def['$=='] = function(other) {
      var $a, $b, TMP_3, self = this;

      if ((($a = self['$equal?'](other)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return true
      } else if ((($a = other['$instance_of?'](self.$class())) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.hash['$=='](other.$instance_variable_get("@hash"))
      } else if ((($a = ($b = other['$is_a?']($scope.get('Set')), $b !== false && $b !== nil ?self.$size()['$=='](other.$size()) : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return ($a = ($b = other)['$all?'], $a.$$p = (TMP_3 = function(o){var self = TMP_3.$$s || this;
          if (self.hash == null) self.hash = nil;
if (o == null) o = nil;
        return self.hash['$include?'](o)}, TMP_3.$$s = self, TMP_3), $a).call($b)
        } else {
        return false
      };
    };

    def.$add = function(o) {
      var self = this;

      self.hash['$[]='](o, true);
      return self;
    };

    Opal.defn(self, '$<<', def.$add);

    def.$classify = TMP_4 = function() {
      var $a, $b, TMP_5, $c, TMP_6, self = this, $iter = TMP_4.$$p, block = $iter || nil, result = nil;

      TMP_4.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("classify")
      };
      result = ($a = ($b = $scope.get('Hash')).$new, $a.$$p = (TMP_5 = function(h, k){var self = TMP_5.$$s || this;
if (h == null) h = nil;if (k == null) k = nil;
      return h['$[]='](k, self.$class().$new())}, TMP_5.$$s = self, TMP_5), $a).call($b);
      ($a = ($c = self).$each, $a.$$p = (TMP_6 = function(item){var self = TMP_6.$$s || this, $a;
if (item == null) item = nil;
      return result['$[]'](((($a = Opal.yield1(block, item)) === $breaker) ? $breaker.$v : $a)).$add(item)}, TMP_6.$$s = self, TMP_6), $a).call($c);
      return result;
    };

    def['$collect!'] = TMP_7 = function() {
      var $a, $b, TMP_8, self = this, $iter = TMP_7.$$p, block = $iter || nil, result = nil;

      TMP_7.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("collect!")
      };
      result = self.$class().$new();
      ($a = ($b = self).$each, $a.$$p = (TMP_8 = function(item){var self = TMP_8.$$s || this, $a;
if (item == null) item = nil;
      return result['$<<'](((($a = Opal.yield1(block, item)) === $breaker) ? $breaker.$v : $a))}, TMP_8.$$s = self, TMP_8), $a).call($b);
      return self.$replace(result);
    };

    Opal.defn(self, '$map!', def['$collect!']);

    def.$delete = function(o) {
      var self = this;

      self.hash.$delete(o);
      return self;
    };

    def['$delete?'] = function(o) {
      var $a, self = this;

      if ((($a = self['$include?'](o)) !== nil && (!$a.$$is_boolean || $a == true))) {
        self.$delete(o);
        return self;
        } else {
        return nil
      };
    };

    def.$delete_if = TMP_9 = function() {try {

      var $a, $b, TMP_10, $c, $d, TMP_11, self = this, $iter = TMP_9.$$p, $yield = $iter || nil;

      TMP_9.$$p = null;
      ((($a = ($yield !== nil)) !== false && $a !== nil) ? $a : Opal.ret(self.$enum_for("delete_if")));
      ($a = ($b = ($c = ($d = self).$select, $c.$$p = (TMP_11 = function(o){var self = TMP_11.$$s || this, $a;
if (o == null) o = nil;
      return $a = Opal.yield1($yield, o), $a === $breaker ? $a : $a}, TMP_11.$$s = self, TMP_11), $c).call($d)).$each, $a.$$p = (TMP_10 = function(o){var self = TMP_10.$$s || this;
        if (self.hash == null) self.hash = nil;
if (o == null) o = nil;
      return self.hash.$delete(o)}, TMP_10.$$s = self, TMP_10), $a).call($b);
      return self;
      } catch ($returner) { if ($returner === Opal.returner) { return $returner.$v } throw $returner; }
    };

    def['$add?'] = function(o) {
      var $a, self = this;

      if ((($a = self['$include?'](o)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        return self.$add(o)
      };
    };

    def.$each = TMP_12 = function() {
      var $a, $b, self = this, $iter = TMP_12.$$p, block = $iter || nil;

      TMP_12.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("each")
      };
      ($a = ($b = self.hash).$each_key, $a.$$p = block.$to_proc(), $a).call($b);
      return self;
    };

    def['$empty?'] = function() {
      var self = this;

      return self.hash['$empty?']();
    };

    def['$eql?'] = function(other) {
      var $a, $b, TMP_13, self = this;

      return self.hash['$eql?'](($a = ($b = other).$instance_eval, $a.$$p = (TMP_13 = function(){var self = TMP_13.$$s || this;
        if (self.hash == null) self.hash = nil;

      return self.hash}, TMP_13.$$s = self, TMP_13), $a).call($b));
    };

    def.$clear = function() {
      var self = this;

      self.hash.$clear();
      return self;
    };

    def['$include?'] = function(o) {
      var self = this;

      return self.hash['$include?'](o);
    };

    Opal.defn(self, '$member?', def['$include?']);

    def.$merge = function(enum$) {
      var $a, $b, TMP_14, self = this;

      ($a = ($b = enum$).$each, $a.$$p = (TMP_14 = function(item){var self = TMP_14.$$s || this;
if (item == null) item = nil;
      return self.$add(item)}, TMP_14.$$s = self, TMP_14), $a).call($b);
      return self;
    };

    def.$replace = function(enum$) {
      var self = this;

      self.$clear();
      self.$merge(enum$);
      return self;
    };

    def.$size = function() {
      var self = this;

      return self.hash.$size();
    };

    Opal.defn(self, '$length', def.$size);

    def.$subtract = function(enum$) {
      var $a, $b, TMP_15, self = this;

      ($a = ($b = enum$).$each, $a.$$p = (TMP_15 = function(item){var self = TMP_15.$$s || this;
if (item == null) item = nil;
      return self.$delete(item)}, TMP_15.$$s = self, TMP_15), $a).call($b);
      return self;
    };

    def['$|'] = function(enum$) {
      var $a, self = this;

      if ((($a = enum$['$respond_to?']("each")) !== nil && (!$a.$$is_boolean || $a == true))) {
        } else {
        self.$raise($scope.get('ArgumentError'), "value must be enumerable")
      };
      return self.$dup().$merge(enum$);
    };

    Opal.defn(self, '$+', def['$|']);

    Opal.defn(self, '$union', def['$|']);

    return (def.$to_a = function() {
      var self = this;

      return self.hash.$keys();
    }, nil) && 'to_a';
  })(self, null);
  return (function($base) {
    var self = $module($base, 'Enumerable');

    var def = self.$$proto, $scope = self.$$scope, TMP_16;

    Opal.defn(self, '$to_set', TMP_16 = function(klass, args) {
      var $a, $b, self = this, $iter = TMP_16.$$p, block = $iter || nil;

      args = $slice.call(arguments, 1);
      if (klass == null) {
        klass = $scope.get('Set')
      }
      TMP_16.$$p = null;
      return ($a = ($b = klass).$new, $a.$$p = block.$to_proc(), $a).apply($b, [self].concat(args));
    })
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/parser/sexp"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass, $range = Opal.range;

  Opal.add_stubs(['$attr_reader', '$attr_accessor', '$[]', '$[]=', '$send', '$to_proc', '$<<', '$push', '$new', '$dup', '$is_a?', '$==', '$array', '$join', '$map', '$inspect', '$line']);
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base, $super) {
      function $Sexp(){};
      var self = $Sexp = $klass($base, $super, 'Sexp', $Sexp);

      var def = self.$$proto, $scope = self.$$scope, TMP_1;

      def.array = def.source = nil;
      self.$attr_reader("array");

      self.$attr_accessor("source");

      def.$initialize = function(args) {
        var self = this;

        return self.array = args;
      };

      def.$type = function() {
        var self = this;

        return self.array['$[]'](0);
      };

      def['$type='] = function(type) {
        var self = this;

        return self.array['$[]='](0, type);
      };

      def.$children = function() {
        var self = this;

        return self.array['$[]']($range(1, -1, false));
      };

      def.$method_missing = TMP_1 = function(sym, args) {
        var $a, $b, self = this, $iter = TMP_1.$$p, block = $iter || nil;

        args = $slice.call(arguments, 1);
        TMP_1.$$p = null;
        return ($a = ($b = self.array).$send, $a.$$p = block.$to_proc(), $a).apply($b, [sym].concat(args));
      };

      def['$<<'] = function(other) {
        var self = this;

        self.array['$<<'](other);
        return self;
      };

      def.$push = function(parts) {
        var $a, self = this;

        parts = $slice.call(arguments, 0);
        ($a = self.array).$push.apply($a, [].concat(parts));
        return self;
      };

      def.$to_ary = function() {
        var self = this;

        return self.array;
      };

      def.$dup = function() {
        var self = this;

        return $scope.get('Sexp').$new(self.array.$dup());
      };

      def['$=='] = function(other) {
        var $a, self = this;

        if ((($a = other['$is_a?']($scope.get('Sexp'))) !== nil && (!$a.$$is_boolean || $a == true))) {
          return self.array['$=='](other.$array())
          } else {
          return self.array['$=='](other)
        };
      };

      Opal.defn(self, '$eql?', def['$==']);

      def.$line = function() {
        var $a, self = this;

        return ($a = self.source, $a !== false && $a !== nil ?self.source['$[]'](0) : $a);
      };

      def.$column = function() {
        var $a, self = this;

        return ($a = self.source, $a !== false && $a !== nil ?self.source['$[]'](1) : $a);
      };

      def.$inspect = function() {
        var $a, $b, TMP_2, self = this;

        return "(" + (($a = ($b = self.array).$map, $a.$$p = (TMP_2 = function(e){var self = TMP_2.$$s || this;
if (e == null) e = nil;
        return e.$inspect()}, TMP_2.$$s = self, TMP_2), $a).call($b).$join(", ")) + ")";
      };

      def.$pretty_inspect = function() {
        var $a, $b, TMP_3, self = this;

        return "(" + ((function() {if ((($a = self.$line()) !== nil && (!$a.$$is_boolean || $a == true))) {
          return "" + (self.$line()) + " "
          } else {
          return ""
        }; return nil; })()) + (($a = ($b = self.array).$map, $a.$$p = (TMP_3 = function(e){var self = TMP_3.$$s || this;
if (e == null) e = nil;
        return e.$inspect()}, TMP_3.$$s = self, TMP_3), $a).call($b).$join(", ")) + ")";
      };

      return Opal.defn(self, '$to_s', def.$inspect);
    })(self, null)
  })(self)
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/regexp_anchors"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module;

  Opal.add_stubs(['$==']);
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    Opal.cdecl($scope, 'REGEXP_START', (function() {if ($scope.get('RUBY_ENGINE')['$==']("opal")) {
      return "^"}; return nil; })());

    Opal.cdecl($scope, 'REGEXP_END', (function() {if ($scope.get('RUBY_ENGINE')['$==']("opal")) {
      return "$"}; return nil; })());
  })(self)
};
/* Generated by Opal 0.8.1 */
Opal.modules["strscan"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$attr_reader', '$length', '$pos=']);
  return (function($base, $super) {
    function $StringScanner(){};
    var self = $StringScanner = $klass($base, $super, 'StringScanner', $StringScanner);

    var def = self.$$proto, $scope = self.$$scope;

    def.pos = def.string = def.working = def.prev_pos = def.matched = def.match = nil;
    self.$attr_reader("pos");

    self.$attr_reader("matched");

    def.$initialize = function(string) {
      var self = this;

      self.string = string;
      self.pos = 0;
      self.matched = nil;
      self.working = string;
      return self.match = [];
    };

    self.$attr_reader("string");

    def['$beginning_of_line?'] = function() {
      var self = this;

      return self.pos === 0 || self.string.charAt(self.pos - 1) === "\n";
    };

    Opal.defn(self, '$bol?', def['$beginning_of_line?']);

    def.$scan = function(regex) {
      var self = this;

      
      var regex  = new RegExp('^' + regex.toString().substring(1, regex.toString().length - 1)),
          result = regex.exec(self.working);

      if (result == null) {
        return self.matched = nil;
      }
      else if (typeof(result) === 'object') {
        self.prev_pos = self.pos;
        self.pos     += result[0].length;
        self.working  = self.working.substring(result[0].length);
        self.matched  = result[0];
        self.match    = result;

        return result[0];
      }
      else if (typeof(result) === 'string') {
        self.pos     += result.length;
        self.working  = self.working.substring(result.length);

        return result;
      }
      else {
        return nil;
      }
    ;
    };

    def['$[]'] = function(idx) {
      var self = this;

      
      var match = self.match;

      if (idx < 0) {
        idx += match.length;
      }

      if (idx < 0 || idx >= match.length) {
        return nil;
      }

      if (match[idx] == null) {
        return nil;
      }

      return match[idx];
    ;
    };

    def.$check = function(regex) {
      var self = this;

      
      var regexp = new RegExp('^' + regex.toString().substring(1, regex.toString().length - 1)),
          result = regexp.exec(self.working);

      if (result == null) {
        return self.matched = nil;
      }

      return self.matched = result[0];
    ;
    };

    def.$peek = function(length) {
      var self = this;

      return self.working.substring(0, length);
    };

    def['$eos?'] = function() {
      var self = this;

      return self.working.length === 0;
    };

    def.$skip = function(re) {
      var self = this;

      
      re = new RegExp('^' + re.source)
      var result = re.exec(self.working);

      if (result == null) {
        return self.matched = nil;
      }
      else {
        var match_str = result[0];
        var match_len = match_str.length;
        self.matched = match_str;
        self.prev_pos = self.pos;
        self.pos += match_len;
        self.working = self.working.substring(match_len);
        return match_len;
      }
    ;
    };

    def.$get_byte = function() {
      var self = this;

      
      var result = nil;
      if (self.pos < self.string.length) {
        self.prev_pos = self.pos;
        self.pos += 1;
        result = self.matched = self.working.substring(0, 1);
        self.working = self.working.substring(1);
      }
      else {
        self.matched = nil;
      }

      return result;
    ;
    };

    Opal.defn(self, '$getch', def.$get_byte);

    def['$pos='] = function(pos) {
      var self = this;

      
      if (pos < 0) {
        pos += self.string.$length();
      }
    ;
      self.pos = pos;
      return self.working = self.string.slice(pos);
    };

    def.$reset = function() {
      var self = this;

      self.working = self.string;
      self.matched = nil;
      return self.pos = 0;
    };

    def.$rest = function() {
      var self = this;

      return self.working;
    };

    def['$rest?'] = function() {
      var self = this;

      return self.working.length !== 0;
    };

    def.$terminate = function() {
      var $a, $b, self = this;

      self.match = nil;
      return (($a = [self.string.$length()]), $b = self, $b['$pos='].apply($b, $a), $a[$a.length-1]);
    };

    return (def.$unscan = function() {
      var self = this;

      self.pos = self.prev_pos;
      self.prev_pos = nil;
      self.match = nil;
      return self;
    }, nil) && 'unscan';
  })(self, null)
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/parser/keywords"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$attr_accessor', '$map', '$new', '$each', '$[]=', '$name', '$[]']);
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Keywords');

      var def = self.$$proto, $scope = self.$$scope, $a, $b, TMP_1;

      (function($base, $super) {
        function $KeywordTable(){};
        var self = $KeywordTable = $klass($base, $super, 'KeywordTable', $KeywordTable);

        var def = self.$$proto, $scope = self.$$scope;

        self.$attr_accessor("name", "id", "state");

        return (def.$initialize = function(name, id, state) {
          var self = this;

          self.name = name;
          self.id = id;
          return self.state = state;
        }, nil) && 'initialize';
      })(self, null);

      Opal.cdecl($scope, 'KEYWORDS', ($a = ($b = [["__LINE__", ["k__LINE__", "k__LINE__"], "expr_end"], ["__FILE__", ["k__FILE__", "k__FILE__"], "expr_end"], ["alias", ["kALIAS", "kALIAS"], "expr_fname"], ["and", ["kAND", "kAND"], "expr_beg"], ["begin", ["kBEGIN", "kBEGIN"], "expr_beg"], ["break", ["kBREAK", "kBREAK"], "expr_mid"], ["case", ["kCASE", "kCASE"], "expr_beg"], ["class", ["kCLASS", "kCLASS"], "expr_class"], ["def", ["kDEF", "kDEF"], "expr_fname"], ["defined?", ["kDEFINED", "kDEFINED"], "expr_arg"], ["do", ["kDO", "kDO"], "expr_beg"], ["else", ["kELSE", "kELSE"], "expr_beg"], ["elsif", ["kELSIF", "kELSIF"], "expr_beg"], ["end", ["kEND", "kEND"], "expr_end"], ["ensure", ["kENSURE", "kENSURE"], "expr_beg"], ["false", ["kFALSE", "kFALSE"], "expr_end"], ["for", ["kFOR", "kFOR"], "expr_beg"], ["if", ["kIF", "kIF_MOD"], "expr_beg"], ["in", ["kIN", "kIN"], "expr_beg"], ["module", ["kMODULE", "kMODULE"], "expr_beg"], ["nil", ["kNIL", "kNIL"], "expr_end"], ["next", ["kNEXT", "kNEXT"], "expr_mid"], ["not", ["kNOT", "kNOT"], "expr_beg"], ["or", ["kOR", "kOR"], "expr_beg"], ["redo", ["kREDO", "kREDO"], "expr_end"], ["rescue", ["kRESCUE", "kRESCUE_MOD"], "expr_mid"], ["return", ["kRETURN", "kRETURN"], "expr_mid"], ["self", ["kSELF", "kSELF"], "expr_end"], ["super", ["kSUPER", "kSUPER"], "expr_arg"], ["then", ["kTHEN", "kTHEN"], "expr_beg"], ["true", ["kTRUE", "kTRUE"], "expr_end"], ["undef", ["kUNDEF", "kUNDEF"], "expr_fname"], ["unless", ["kUNLESS", "kUNLESS_MOD"], "expr_beg"], ["until", ["kUNTIL", "kUNTIL_MOD"], "expr_beg"], ["when", ["kWHEN", "kWHEN"], "expr_beg"], ["while", ["kWHILE", "kWHILE_MOD"], "expr_beg"], ["yield", ["kYIELD", "kYIELD"], "expr_arg"]]).$map, $a.$$p = (TMP_1 = function(decl){var self = TMP_1.$$s || this, $a;
if (decl == null) decl = nil;
      return ($a = $scope.get('KeywordTable')).$new.apply($a, [].concat(decl))}, TMP_1.$$s = self, TMP_1), $a).call($b));

      Opal.defs(self, '$map', function() {
        var $a, $b, TMP_2, self = this;
        if (self.map == null) self.map = nil;

        if ((($a = self.map) !== nil && (!$a.$$is_boolean || $a == true))) {
          } else {
          self.map = $hash2([], {});
          ($a = ($b = $scope.get('KEYWORDS')).$each, $a.$$p = (TMP_2 = function(k){var self = TMP_2.$$s || this;
            if (self.map == null) self.map = nil;
if (k == null) k = nil;
          return self.map['$[]='](k.$name(), k)}, TMP_2.$$s = self, TMP_2), $a).call($b);
        };
        return self.map;
      });

      Opal.defs(self, '$keyword', function(kw) {
        var self = this;

        return self.$map()['$[]'](kw);
      });
    })(self)
  })(self)
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/parser/lexer"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  function $rb_minus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs - rhs : lhs['$-'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$|', '$attr_reader', '$attr_accessor', '$new', '$yylex', '$yylval', '$has_local?', '$scope', '$parser', '$to_sym', '$<<', '$&', '$>>', '$!', '$==', '$include?', '$arg?', '$space?', '$check', '$after_operator?', '$scan', '$length', '$matched', '$pos=', '$pos', '$new_strterm', '$merge', '$yylval=', '$to_f', '$gsub', '$scanner', '$to_i', '$raise', '$peek', '$chr', '$%', '$[]', '$escape', '$peek_variable_name', '$bol?', '$eos?', '$read_escape', '$join', '$count', '$strterm', '$[]=', '$pushback', '$strterm=', '$add_string_content', '$line=', '$line', '$label_state?', '$end_with?', '$=~', '$keyword', '$state', '$name', '$id', '$cond?', '$cmdarg?', '$here_document', '$parse_string', '$skip', '$empty?', '$new_op_asgn', '$set_arg_state', '$spcarg?', '$beg?', '$===', '$new_strterm2', '$cond_push', '$cmdarg_push', '$cond_lexpop', '$cmdarg_lexpop', '$end?', '$heredoc_identifier', '$sub', '$inspect', '$process_numeric', '$process_identifier', '$size', '$pop', '$last']);
  self.$require("opal/regexp_anchors");
  self.$require("strscan");
  self.$require("opal/parser/keywords");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base, $super) {
      function $Lexer(){};
      var self = $Lexer = $klass($base, $super, 'Lexer', $Lexer);

      var def = self.$$proto, $scope = self.$$scope;

      def.scanner = def.tok_line = def.tok_column = def.column = def.line = def.cond = def.cmdarg = def.lex_state = def.space_seen = def.yylval = def.scanner_stack = def.start_of_lambda = def.file = nil;
      Opal.cdecl($scope, 'STR_FUNC_ESCAPE', 1);

      Opal.cdecl($scope, 'STR_FUNC_EXPAND', 2);

      Opal.cdecl($scope, 'STR_FUNC_REGEXP', 4);

      Opal.cdecl($scope, 'STR_FUNC_QWORDS', 8);

      Opal.cdecl($scope, 'STR_FUNC_SYMBOL', 16);

      Opal.cdecl($scope, 'STR_FUNC_INDENT', 32);

      Opal.cdecl($scope, 'STR_FUNC_XQUOTE', 64);

      Opal.cdecl($scope, 'STR_SQUOTE', 0);

      Opal.cdecl($scope, 'STR_DQUOTE', $scope.get('STR_FUNC_EXPAND'));

      Opal.cdecl($scope, 'STR_XQUOTE', $scope.get('STR_FUNC_EXPAND')['$|']($scope.get('STR_FUNC_XQUOTE')));

      Opal.cdecl($scope, 'STR_REGEXP', $scope.get('STR_FUNC_REGEXP')['$|']($scope.get('STR_FUNC_ESCAPE'))['$|']($scope.get('STR_FUNC_EXPAND')));

      Opal.cdecl($scope, 'STR_SWORD', $scope.get('STR_FUNC_QWORDS'));

      Opal.cdecl($scope, 'STR_DWORD', $scope.get('STR_FUNC_QWORDS')['$|']($scope.get('STR_FUNC_EXPAND')));

      Opal.cdecl($scope, 'STR_SSYM', $scope.get('STR_FUNC_SYMBOL'));

      Opal.cdecl($scope, 'STR_DSYM', $scope.get('STR_FUNC_SYMBOL')['$|']($scope.get('STR_FUNC_EXPAND')));

      self.$attr_reader("line", "column");

      self.$attr_reader("scope");

      self.$attr_reader("eof_content");

      self.$attr_accessor("lex_state");

      self.$attr_accessor("strterm");

      self.$attr_accessor("scanner");

      self.$attr_accessor("yylval");

      self.$attr_accessor("parser");

      def.$initialize = function(source, file) {
        var self = this;

        self.lex_state = "expr_beg";
        self.cond = 0;
        self.cmdarg = 0;
        self.line = 1;
        self.tok_line = 1;
        self.column = 0;
        self.tok_column = 0;
        self.file = file;
        self.scanner = $scope.get('StringScanner').$new(source);
        self.scanner_stack = [self.scanner];
        self.case_stmt = nil;
        return self.start_of_lambda = nil;
      };

      def.$next_token = function() {
        var self = this, token = nil, value = nil, location = nil;

        token = self.$yylex();
        value = self.$yylval();
        location = [self.tok_line, self.tok_column];
        self.tok_column = self.column;
        self.tok_line = self.line;
        return [token, [value, location]];
      };

      def['$has_local?'] = function(local) {
        var self = this;

        return self.$parser().$scope()['$has_local?'](local.$to_sym());
      };

      def.$cond_push = function(n) {
        var self = this;

        return self.cond = (self.cond['$<<'](1))['$|']((n['$&'](1)));
      };

      def.$cond_pop = function() {
        var self = this;

        return self.cond = self.cond['$>>'](1);
      };

      def.$cond_lexpop = function() {
        var self = this;

        return self.cond = (self.cond['$>>'](1))['$|']((self.cond['$&'](1)));
      };

      def['$cond?'] = function() {
        var self = this;

        return (self.cond['$&'](1))['$=='](0)['$!']();
      };

      def.$cmdarg_push = function(n) {
        var self = this;

        return self.cmdarg = (self.cmdarg['$<<'](1))['$|']((n['$&'](1)));
      };

      def.$cmdarg_pop = function() {
        var self = this;

        return self.cmdarg = self.cmdarg['$>>'](1);
      };

      def.$cmdarg_lexpop = function() {
        var self = this;

        return self.cmdarg = (self.cmdarg['$>>'](1))['$|']((self.cmdarg['$&'](1)));
      };

      def['$cmdarg?'] = function() {
        var self = this;

        return (self.cmdarg['$&'](1))['$=='](0)['$!']();
      };

      def['$arg?'] = function() {
        var self = this;

        return ["expr_arg", "expr_cmdarg"]['$include?'](self.lex_state);
      };

      def['$end?'] = function() {
        var self = this;

        return ["expr_end", "expr_endarg", "expr_endfn"]['$include?'](self.lex_state);
      };

      def['$beg?'] = function() {
        var self = this;

        return ["expr_beg", "expr_value", "expr_mid", "expr_class"]['$include?'](self.lex_state);
      };

      def['$after_operator?'] = function() {
        var self = this;

        return ["expr_fname", "expr_dot"]['$include?'](self.lex_state);
      };

      def['$label_state?'] = function() {
        var $a, self = this;

        return ((($a = ["expr_beg", "expr_endfn"]['$include?'](self.lex_state)) !== false && $a !== nil) ? $a : self['$arg?']());
      };

      def['$spcarg?'] = function() {
        var $a, $b, self = this;

        return ($a = ($b = self['$arg?'](), $b !== false && $b !== nil ?self.space_seen : $b), $a !== false && $a !== nil ?self['$space?']()['$!']() : $a);
      };

      def['$space?'] = function() {
        var self = this;

        return self.scanner.$check(/\s/);
      };

      def.$set_arg_state = function() {
        var $a, self = this;

        return self.lex_state = (function() {if ((($a = self['$after_operator?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
          return "expr_arg"
          } else {
          return "expr_beg"
        }; return nil; })();
      };

      def.$scan = function(regexp) {
        var $a, self = this, result = nil;

        if ((($a = result = self.scanner.$scan(regexp)) !== nil && (!$a.$$is_boolean || $a == true))) {
          self.column = $rb_plus(self.column, result.$length());
          self.yylval = $rb_plus(self.yylval, self.scanner.$matched());};
        return result;
      };

      def.$skip = function(regexp) {
        var $a, self = this, result = nil;

        if ((($a = result = self.scanner.$scan(regexp)) !== nil && (!$a.$$is_boolean || $a == true))) {
          self.column = $rb_plus(self.column, result.$length());
          self.tok_column = self.column;};
        return result;
      };

      def.$check = function(regexp) {
        var self = this;

        return self.scanner.$check(regexp);
      };

      def.$pushback = function(n) {
        var $a, self = this;

        return ($a = self.scanner, $a['$pos=']($rb_minus($a.$pos(), n)));
      };

      def.$matched = function() {
        var self = this;

        return self.scanner.$matched();
      };

      def['$line='] = function(line) {
        var self = this;

        self.column = self.tok_column = 0;
        return self.line = self.tok_line = line;
      };

      def.$new_strterm = function(func, term, paren) {
        var self = this;

        return $hash2(["type", "func", "term", "paren"], {"type": "string", "func": func, "term": term, "paren": paren});
      };

      def.$new_strterm2 = function(func, term, paren) {
        var self = this;

        term = self.$new_strterm(func, term, paren);
        return term.$merge($hash2(["balance", "nesting"], {"balance": true, "nesting": 0}));
      };

      def.$new_op_asgn = function(value) {
        var $a, $b, self = this;

        (($a = [value]), $b = self, $b['$yylval='].apply($b, $a), $a[$a.length-1]);
        return "tOP_ASGN";
      };

      def.$process_numeric = function() {
        var $a, $b, self = this;

        self.lex_state = "expr_end";
        if ((($a = self.$scan(/[\d_]+\.[\d_]+\b|[\d_]+(\.[\d_]+)?[eE][-+]?[\d_]+\b/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          (($a = [self.$scanner().$matched().$gsub(/_/, "").$to_f()]), $b = self, $b['$yylval='].apply($b, $a), $a[$a.length-1]);
          return "tFLOAT";
        } else if ((($a = self.$scan(/([^0][\d_]*|0)\b/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          (($a = [self.$scanner().$matched().$gsub(/_/, "").$to_i()]), $b = self, $b['$yylval='].apply($b, $a), $a[$a.length-1]);
          return "tINTEGER";
        } else if ((($a = self.$scan(/0[bB](0|1|_)+/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          (($a = [self.$scanner().$matched().$to_i(2)]), $b = self, $b['$yylval='].apply($b, $a), $a[$a.length-1]);
          return "tINTEGER";
        } else if ((($a = self.$scan(/0[xX](\d|[a-f]|[A-F]|_)+/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          (($a = [self.$scanner().$matched().$to_i(16)]), $b = self, $b['$yylval='].apply($b, $a), $a[$a.length-1]);
          return "tINTEGER";
        } else if ((($a = self.$scan(/0[oO]?([0-7]|_)+/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          (($a = [self.$scanner().$matched().$to_i(8)]), $b = self, $b['$yylval='].apply($b, $a), $a[$a.length-1]);
          return "tINTEGER";
        } else if ((($a = self.$scan(/0[dD]([0-9]|_)+/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          (($a = [self.$scanner().$matched().$gsub(/_/, "").$to_i()]), $b = self, $b['$yylval='].apply($b, $a), $a[$a.length-1]);
          return "tINTEGER";
          } else {
          return self.$raise("Lexing error on numeric type: `" + (self.$scanner().$peek(5)) + "`")
        };
      };

      def.$read_escape = function() {
        var $a, self = this;

        if ((($a = self.$scan(/\\/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return "\\"
        } else if ((($a = self.$scan(/n/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return "\n"
        } else if ((($a = self.$scan(/t/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return "\t"
        } else if ((($a = self.$scan(/r/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return "\r"
        } else if ((($a = self.$scan(/f/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return "\f"
        } else if ((($a = self.$scan(/v/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return "\v"
        } else if ((($a = self.$scan(/a/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return "\u0007"
        } else if ((($a = self.$scan(/b/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return "\b"
        } else if ((($a = self.$scan(/e/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return "\u001b"
        } else if ((($a = self.$scan(/s/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return " "
        } else if ((($a = self.$scan(/[0-7]{1,3}/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return (self.$matched().$to_i(8)['$%'](256)).$chr()
        } else if ((($a = self.$scan(/x([0-9a-fA-F]{1,2})/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return self.$scanner()['$[]'](1).$to_i(16).$chr()
        } else if ((($a = self.$scan(/u([0-9a-zA-Z]{1,4})/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return self.$scanner()['$[]'](1).$to_i(16).$chr((($scope.get('Encoding')).$$scope.get('UTF_8')))
          } else {
          return self.$scan(/./)
        };
      };

      def.$peek_variable_name = function() {
        var $a, self = this;

        if ((($a = self.$check(/[@$]/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return "tSTRING_DVAR"
        } else if ((($a = self.$scan(/\{/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return "tSTRING_DBEG"
          } else {
          return nil
        };
      };

      def.$here_document = function(str_parse) {
        var $a, $b, $c, self = this, eos_regx = nil, expand = nil, escape = nil, str_buffer = nil, tok = nil, reg = nil, complete_str = nil;

        eos_regx = (new RegExp("[ \\t]*" + $scope.get('Regexp').$escape(str_parse['$[]']("term")) + "(\\r*\\n|$)"));
        expand = true;
        escape = str_parse['$[]']("func")['$==']($scope.get('STR_SQUOTE'))['$!']();
        if ((($a = self.$check(eos_regx)) !== nil && (!$a.$$is_boolean || $a == true))) {
          self.$scan((new RegExp("[ \\t]*" + $scope.get('Regexp').$escape(str_parse['$[]']("term")))));
          if ((($a = str_parse['$[]']("scanner")) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.scanner_stack['$<<'](str_parse['$[]']("scanner"));
            self.scanner = str_parse['$[]']("scanner");};
          return "tSTRING_END";};
        str_buffer = [];
        if ((($a = self.$scan(/#/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          if ((($a = tok = self.$peek_variable_name()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return tok};
          str_buffer['$<<']("#");};
        while (!((($b = ($c = self.$check(eos_regx), $c !== false && $c !== nil ?self.$scanner()['$bol?']() : $c)) !== nil && (!$b.$$is_boolean || $b == true)))) {
        if ((($b = self.$scanner()['$eos?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
          self.$raise("reached EOF while in heredoc")};
        if ((($b = self.$scan(/\n/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          str_buffer['$<<'](self.$scanner().$matched())
        } else if ((($b = (($c = expand !== false && expand !== nil) ? self.$check(/#(?=[\$\@\{])/) : $c)) !== nil && (!$b.$$is_boolean || $b == true))) {
          break;
        } else if ((($b = self.$scan(/\\/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          str_buffer['$<<'](((function() {if (escape !== false && escape !== nil) {
            return self.$read_escape()
            } else {
            return self.$scanner().$matched()
          }; return nil; })()))
          } else {
          reg = $scope.get('Regexp').$new("[^#\u0000\\\\\n]+|.");
          self.$scan(reg);
          str_buffer['$<<'](self.$scanner().$matched());
        };};
        complete_str = str_buffer.$join("");
        self.line = $rb_plus(self.line, complete_str.$count("\n"));
        (($a = [complete_str]), $b = self, $b['$yylval='].apply($b, $a), $a[$a.length-1]);
        return "tSTRING_CONTENT";
      };

      def.$parse_string = function() {
        var $a, $b, self = this, str_parse = nil, func = nil, space = nil, qwords = nil, expand = nil, regexp = nil, str_buffer = nil, complete_str = nil;

        str_parse = self.$strterm();
        func = str_parse['$[]']("func");
        space = false;
        qwords = (func['$&']($scope.get('STR_FUNC_QWORDS')))['$=='](0)['$!']();
        expand = (func['$&']($scope.get('STR_FUNC_EXPAND')))['$=='](0)['$!']();
        regexp = (func['$&']($scope.get('STR_FUNC_REGEXP')))['$=='](0)['$!']();
        if ((($a = (($b = qwords !== false && qwords !== nil) ? self.$scan(/\s+/) : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
          space = true};
        str_buffer = [];
        if ((($a = self.$scan($scope.get('Regexp').$new($scope.get('Regexp').$escape(str_parse['$[]']("term"))))) !== nil && (!$a.$$is_boolean || $a == true))) {
          if ((($a = (($b = qwords !== false && qwords !== nil) ? str_parse['$[]']("done_last_space")['$!']() : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
            str_parse['$[]=']("done_last_space", true);
            self.$pushback(1);
            (($a = [" "]), $b = self, $b['$yylval='].apply($b, $a), $a[$a.length-1]);
            return "tSPACE";};
          if ((($a = str_parse['$[]']("balance")) !== nil && (!$a.$$is_boolean || $a == true))) {
            if (str_parse['$[]']("nesting")['$=='](0)) {
              if (regexp !== false && regexp !== nil) {
                (($a = [self.$scan(/\w+/)]), $b = self, $b['$yylval='].apply($b, $a), $a[$a.length-1]);
                return "tREGEXP_END";};
              return "tSTRING_END";
              } else {
              str_buffer['$<<'](self.$scanner().$matched());
              ($a = "nesting", $b = str_parse, $b['$[]=']($a, $rb_minus($b['$[]']($a), 1)));
              (($a = [str_parse]), $b = self, $b['$strterm='].apply($b, $a), $a[$a.length-1]);
            }
          } else if (regexp !== false && regexp !== nil) {
            self.lex_state = "expr_end";
            (($a = [self.$scan(/\w+/)]), $b = self, $b['$yylval='].apply($b, $a), $a[$a.length-1]);
            return "tREGEXP_END";
            } else {
            if ((($a = str_parse['$[]']("scanner")) !== nil && (!$a.$$is_boolean || $a == true))) {
              self.scanner_stack['$<<'](str_parse['$[]']("scanner"));
              self.scanner = str_parse['$[]']("scanner");};
            return "tSTRING_END";
          };};
        if (space !== false && space !== nil) {
          (($a = [" "]), $b = self, $b['$yylval='].apply($b, $a), $a[$a.length-1]);
          return "tSPACE";};
        if ((($a = ($b = str_parse['$[]']("balance"), $b !== false && $b !== nil ?self.$scan($scope.get('Regexp').$new($scope.get('Regexp').$escape(str_parse['$[]']("paren")))) : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
          str_buffer['$<<'](self.$scanner().$matched());
          ($a = "nesting", $b = str_parse, $b['$[]=']($a, $rb_plus($b['$[]']($a), 1)));
        } else if ((($a = self.$check(/#[@$]/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          self.$scan(/#/);
          if (expand !== false && expand !== nil) {
            return "tSTRING_DVAR"
            } else {
            str_buffer['$<<'](self.$scanner().$matched())
          };
        } else if ((($a = self.$scan(/#\{/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          if (expand !== false && expand !== nil) {
            return "tSTRING_DBEG"
            } else {
            str_buffer['$<<'](self.$scanner().$matched())
          }
        } else if ((($a = self.$scan(/\#/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          str_buffer['$<<']("#")};
        self.$add_string_content(str_buffer, str_parse);
        complete_str = str_buffer.$join("");
        self.line = $rb_plus(self.line, complete_str.$count("\n"));
        (($a = [complete_str]), $b = self, $b['$yylval='].apply($b, $a), $a[$a.length-1]);
        return "tSTRING_CONTENT";
      };

      def.$add_string_content = function(str_buffer, str_parse) {
        var $a, $b, $c, self = this, func = nil, end_str_re = nil, qwords = nil, expand = nil, regexp = nil, escape = nil, xquote = nil, c = nil, handled = nil, reg = nil;

        func = str_parse['$[]']("func");
        end_str_re = $scope.get('Regexp').$new($scope.get('Regexp').$escape(str_parse['$[]']("term")));
        qwords = (func['$&']($scope.get('STR_FUNC_QWORDS')))['$=='](0)['$!']();
        expand = (func['$&']($scope.get('STR_FUNC_EXPAND')))['$=='](0)['$!']();
        regexp = (func['$&']($scope.get('STR_FUNC_REGEXP')))['$=='](0)['$!']();
        escape = (func['$&']($scope.get('STR_FUNC_ESCAPE')))['$=='](0)['$!']();
        xquote = (func['$==']($scope.get('STR_XQUOTE')));
        while (!((($b = self.$scanner()['$eos?']()) !== nil && (!$b.$$is_boolean || $b == true)))) {
        c = nil;
        handled = true;
        if ((($b = self.$check(end_str_re)) !== nil && (!$b.$$is_boolean || $b == true))) {
          if ((($b = ($c = str_parse['$[]']("balance"), $c !== false && $c !== nil ?(str_parse['$[]']("nesting")['$=='](0)['$!']()) : $c)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.$scan(end_str_re);
            c = self.$scanner().$matched();
            ($b = "nesting", $c = str_parse, $c['$[]=']($b, $rb_minus($c['$[]']($b), 1)));
            } else {
            break;
          }
        } else if ((($b = ($c = str_parse['$[]']("balance"), $c !== false && $c !== nil ?self.$scan($scope.get('Regexp').$new($scope.get('Regexp').$escape(str_parse['$[]']("paren")))) : $c)) !== nil && (!$b.$$is_boolean || $b == true))) {
          ($b = "nesting", $c = str_parse, $c['$[]=']($b, $rb_plus($c['$[]']($b), 1)));
          c = self.$scanner().$matched();
        } else if ((($b = (($c = qwords !== false && qwords !== nil) ? self.$scan(/\s/) : $c)) !== nil && (!$b.$$is_boolean || $b == true))) {
          self.$pushback(1);
          break;;
        } else if ((($b = (($c = expand !== false && expand !== nil) ? self.$check(/#(?=[\$\@\{])/) : $c)) !== nil && (!$b.$$is_boolean || $b == true))) {
          break;
        } else if ((($b = (($c = qwords !== false && qwords !== nil) ? self.$scan(/\s/) : $c)) !== nil && (!$b.$$is_boolean || $b == true))) {
          self.$pushback(1);
          break;;
        } else if ((($b = self.$scan(/\\/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          if (xquote !== false && xquote !== nil) {
            c = $rb_plus("\\", self.$scan(/./))
          } else if ((($b = (($c = qwords !== false && qwords !== nil) ? self.$scan(/\n/) : $c)) !== nil && (!$b.$$is_boolean || $b == true))) {
            str_buffer['$<<']("\n");
            continue;;
          } else if ((($b = (($c = expand !== false && expand !== nil) ? self.$scan(/\n/) : $c)) !== nil && (!$b.$$is_boolean || $b == true))) {
            continue;
          } else if ((($b = (($c = qwords !== false && qwords !== nil) ? self.$scan(/\s/) : $c)) !== nil && (!$b.$$is_boolean || $b == true))) {
            c = " "
          } else if (regexp !== false && regexp !== nil) {
            if ((($b = self.$scan(/(.)/)) !== nil && (!$b.$$is_boolean || $b == true))) {
              c = $rb_plus("\\", self.$scanner().$matched())}
          } else if (expand !== false && expand !== nil) {
            c = self.$read_escape()
          } else if ((($b = self.$scan(/\n/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          } else if ((($b = self.$scan(/\\/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            if (escape !== false && escape !== nil) {
              c = "\\\\"
              } else {
              c = self.$scanner().$matched()
            }
          } else if ((($b = self.$scan(end_str_re)) !== nil && (!$b.$$is_boolean || $b == true))) {
            } else {
            str_buffer['$<<']("\\")
          }
          } else {
          handled = false
        };
        if (handled !== false && handled !== nil) {
          } else {
          reg = (function() {if (qwords !== false && qwords !== nil) {
            return $scope.get('Regexp').$new("[^" + ($scope.get('Regexp').$escape(str_parse['$[]']("term"))) + "#\u0000\n \\\\]+|.")
          } else if ((($b = str_parse['$[]']("balance")) !== nil && (!$b.$$is_boolean || $b == true))) {
            return $scope.get('Regexp').$new("[^" + ($scope.get('Regexp').$escape(str_parse['$[]']("term"))) + ($scope.get('Regexp').$escape(str_parse['$[]']("paren"))) + "#\u0000\\\\]+|.")
            } else {
            return $scope.get('Regexp').$new("[^" + ($scope.get('Regexp').$escape(str_parse['$[]']("term"))) + "#\u0000\\\\]+|.")
          }; return nil; })();
          self.$scan(reg);
          c = self.$scanner().$matched();
        };
        ((($b = c) !== false && $b !== nil) ? $b : c = self.$scanner().$matched());
        str_buffer['$<<'](c);};
        if ((($a = self.$scanner()['$eos?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
          return self.$raise("reached EOF while in string")
          } else {
          return nil
        };
      };

      def.$heredoc_identifier = function() {
        var $a, $b, self = this, escape_method = nil, heredoc = nil, end_of_line = nil;

        if ((($a = self.$scan(/(-?)(['"])?(\w+)\2?/)) !== nil && (!$a.$$is_boolean || $a == true))) {
          escape_method = (function() {if ((($a = (self.scanner['$[]'](2)['$==']("'"))) !== nil && (!$a.$$is_boolean || $a == true))) {
            return $scope.get('STR_SQUOTE')
            } else {
            return $scope.get('STR_DQUOTE')
          }; return nil; })();
          heredoc = self.scanner['$[]'](3);
          (($a = [self.$new_strterm(escape_method, heredoc, heredoc)]), $b = self, $b['$strterm='].apply($b, $a), $a[$a.length-1]);
          self.$strterm()['$[]=']("type", "heredoc");
          end_of_line = self.$scan(/.*\n/);
          if ((($a = end_of_line['$==']("\n")['$!']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.$strterm()['$[]=']("scanner", $scope.get('StringScanner').$new(end_of_line))};
          ($a = self, $a['$line=']($rb_plus($a.$line(), 1)));
          (($a = [heredoc]), $b = self, $b['$yylval='].apply($b, $a), $a[$a.length-1]);
          return "tSTRING_BEG";
          } else {
          return nil
        };
      };

      def.$process_identifier = function(matched, cmd_start) {
        var $a, $b, $c, self = this, last_state = nil, result = nil, kw = nil, old_state = nil;

        last_state = self.lex_state;
        if ((($a = ($b = ($c = self['$label_state?'](), $c !== false && $c !== nil ?self.$check(/::/)['$!']() : $c), $b !== false && $b !== nil ?self.$scan(/:/) : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
          self.lex_state = "expr_beg";
          (($a = [matched]), $b = self, $b['$yylval='].apply($b, $a), $a[$a.length-1]);
          return "tLABEL";};
        if (matched['$==']("defined?")) {
          if ((($a = self['$after_operator?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.lex_state = "expr_end";
            return "tIDENTIFIER";};
          self.lex_state = "expr_arg";
          return "kDEFINED";};
        if ((($a = matched['$end_with?']("?", "!")) !== nil && (!$a.$$is_boolean || $a == true))) {
          result = "tIDENTIFIER"
        } else if (self.lex_state['$==']("expr_fname")) {
          if ((($a = ($b = self.$check(/\=\>/)['$!'](), $b !== false && $b !== nil ?self.$scan(/\=/) : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
            result = "tIDENTIFIER";
            matched = $rb_plus(matched, self.$scanner().$matched());}
        } else if ((($a = matched['$=~']((new RegExp("" + $scope.get('REGEXP_START') + "[A-Z]")))) !== nil && (!$a.$$is_boolean || $a == true))) {
          result = "tCONSTANT"
          } else {
          result = "tIDENTIFIER"
        };
        if ((($a = ($b = self.lex_state['$==']("expr_dot")['$!'](), $b !== false && $b !== nil ?kw = $scope.get('Keywords').$keyword(matched) : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
          old_state = self.lex_state;
          self.lex_state = kw.$state();
          if (old_state['$==']("expr_fname")) {
            (($a = [kw.$name()]), $b = self, $b['$yylval='].apply($b, $a), $a[$a.length-1]);
            return kw.$id()['$[]'](0);};
          if (self.lex_state['$==']("expr_beg")) {
            cmd_start = true};
          if (matched['$==']("do")) {
            if ((($a = self['$after_operator?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
              self.lex_state = "expr_end";
              return "tIDENTIFIER";};
            if ((($a = self.start_of_lambda) !== nil && (!$a.$$is_boolean || $a == true))) {
              self.start_of_lambda = false;
              self.lex_state = "expr_beg";
              return "kDO_LAMBDA";
            } else if ((($a = self['$cond?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
              self.lex_state = "expr_beg";
              return "kDO_COND";
            } else if ((($a = ($b = self['$cmdarg?'](), $b !== false && $b !== nil ?self.lex_state['$==']("expr_cmdarg")['$!']() : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
              self.lex_state = "expr_beg";
              return "kDO_BLOCK";
            } else if (self.lex_state['$==']("expr_endarg")) {
              return "kDO_BLOCK"
              } else {
              self.lex_state = "expr_beg";
              return "kDO";
            };
          } else if ((($a = ((($b = old_state['$==']("expr_beg")) !== false && $b !== nil) ? $b : old_state['$==']("expr_value"))) !== nil && (!$a.$$is_boolean || $a == true))) {
            (($a = [matched]), $b = self, $b['$yylval='].apply($b, $a), $a[$a.length-1]);
            return kw.$id()['$[]'](0);
            } else {
            if ((($a = kw.$id()['$[]'](0)['$=='](kw.$id()['$[]'](1))['$!']()) !== nil && (!$a.$$is_boolean || $a == true))) {
              self.lex_state = "expr_beg"};
            (($a = [matched]), $b = self, $b['$yylval='].apply($b, $a), $a[$a.length-1]);
            return kw.$id()['$[]'](1);
          };};
        if ((($a = ["expr_beg", "expr_dot", "expr_mid", "expr_arg", "expr_cmdarg"]['$include?'](self.lex_state)) !== nil && (!$a.$$is_boolean || $a == true))) {
          self.lex_state = (function() {if (cmd_start !== false && cmd_start !== nil) {
            return "expr_cmdarg"
            } else {
            return "expr_arg"
          }; return nil; })()
        } else if (self.lex_state['$==']("expr_fname")) {
          self.lex_state = "expr_endfn"
          } else {
          self.lex_state = "expr_end"
        };
        if ((($a = ($b = ["expr_dot", "expr_fname"]['$include?'](last_state)['$!'](), $b !== false && $b !== nil ?self['$has_local?'](matched) : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
          self.lex_state = "expr_end"};
        return (function() {if ((($a = matched['$=~']((new RegExp("" + $scope.get('REGEXP_START') + "[A-Z]")))) !== nil && (!$a.$$is_boolean || $a == true))) {
          return "tCONSTANT"
          } else {
          return "tIDENTIFIER"
        }; return nil; })();
      };

      return (def.$yylex = function() {try {

        var $a, $b, $c, $d, self = this, cmd_start = nil, c = nil, token = nil, line_count = nil, result = nil, str_type = nil, paren = nil, term = nil, $case = nil, func = nil, start_word = nil, end_word = nil, matched = nil, sign = nil, utype = nil;

        self.yylval = "";
        self.space_seen = false;
        cmd_start = false;
        c = "";
        if ((($a = self.$strterm()) !== nil && (!$a.$$is_boolean || $a == true))) {
          if (self.$strterm()['$[]']("type")['$==']("heredoc")) {
            token = self.$here_document(self.$strterm())
            } else {
            token = self.$parse_string()
          };
          if ((($a = ((($b = token['$==']("tSTRING_END")) !== false && $b !== nil) ? $b : token['$==']("tREGEXP_END"))) !== nil && (!$a.$$is_boolean || $a == true))) {
            (($a = [nil]), $b = self, $b['$strterm='].apply($b, $a), $a[$a.length-1]);
            self.lex_state = "expr_end";};
          return token;};
        while ((($b = true) !== nil && (!$b.$$is_boolean || $b == true))) {
        if ((($b = self.$skip(/\ |\t|\r/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          self.space_seen = true;
          continue;;
        } else if ((($b = self.$skip(/(\n|#)/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          c = self.$scanner().$matched();
          if (c['$==']("#")) {
            self.$skip(/(.*)/)
            } else {
            ($b = self, $b['$line=']($rb_plus($b.$line(), 1)))
          };
          self.$skip(/(\n+)/);
          if ((($b = self.$scanner().$matched()) !== nil && (!$b.$$is_boolean || $b == true))) {
            ($b = self, $b['$line=']($rb_plus($b.$line(), self.$scanner().$matched().$length())))};
          if ((($b = ["expr_beg", "expr_dot"]['$include?'](self.lex_state)) !== nil && (!$b.$$is_boolean || $b == true))) {
            continue;};
          if ((($b = self.$skip(/([\ \t\r\f\v]*)\./)) !== nil && (!$b.$$is_boolean || $b == true))) {
            if ((($b = self.$scanner()['$[]'](1)['$empty?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
              } else {
              self.space_seen = true
            };
            self.$pushback(1);
            if ((($b = self.$check(/\.\./)) !== nil && (!$b.$$is_boolean || $b == true))) {
              } else {
              continue;
            };};
          cmd_start = true;
          self.lex_state = "expr_beg";
          (($b = ["\\n"]), $c = self, $c['$yylval='].apply($c, $b), $b[$b.length-1]);
          return "tNL";
        } else if ((($b = self.$scan(/\;/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          self.lex_state = "expr_beg";
          return "tSEMI";
        } else if ((($b = self.$check(/\*/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          if ((($b = self.$scan(/\*\*\=/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_beg";
            return self.$new_op_asgn("**");
          } else if ((($b = self.$scan(/\*\*/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.$set_arg_state();
            return "tPOW";
          } else if ((($b = self.$scan(/\*\=/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_beg";
            return self.$new_op_asgn("*");
            } else {
            self.$scan(/\*/);
            if ((($b = self['$after_operator?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
              self.lex_state = "expr_arg";
              return "tSTAR2";
            } else if ((($b = ($c = self.space_seen, $c !== false && $c !== nil ?self.$check(/\S/) : $c)) !== nil && (!$b.$$is_boolean || $b == true))) {
              self.lex_state = "expr_beg";
              return "tSTAR";
            } else if ((($b = ["expr_beg", "expr_mid"]['$include?'](self.lex_state)) !== nil && (!$b.$$is_boolean || $b == true))) {
              self.lex_state = "expr_beg";
              return "tSTAR";
              } else {
              self.lex_state = "expr_beg";
              return "tSTAR2";
            };
          }
        } else if ((($b = self.$scan(/\!/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          if ((($b = self['$after_operator?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_arg";
            if ((($b = self.$scan(/@/)) !== nil && (!$b.$$is_boolean || $b == true))) {
              return ["tBANG", "!"]};
            } else {
            self.lex_state = "expr_beg"
          };
          if ((($b = self.$scan(/\=/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            return "tNEQ"
          } else if ((($b = self.$scan(/\~/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            return "tNMATCH"};
          return "tBANG";
        } else if ((($b = self.$scan(/\=/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          if ((($b = (($c = self.lex_state['$==']("expr_beg")) ? self.space_seen['$!']() : $c)) !== nil && (!$b.$$is_boolean || $b == true))) {
            if ((($b = ($c = self.$scan(/begin/), $c !== false && $c !== nil ?self['$space?']() : $c)) !== nil && (!$b.$$is_boolean || $b == true))) {
              self.$scan(/(.*)/);
              line_count = 0;
              while ((($c = true) !== nil && (!$c.$$is_boolean || $c == true))) {
              if ((($c = self.$scanner()['$eos?']()) !== nil && (!$c.$$is_boolean || $c == true))) {
                self.$raise("embedded document meets end of file")};
              if ((($c = ($d = self.$scan(/\=end/), $d !== false && $d !== nil ?self['$space?']() : $d)) !== nil && (!$c.$$is_boolean || $c == true))) {
                self.line = $rb_plus(self.line, line_count);
                return self.$yylex();};
              if ((($c = self.$scan(/\n/)) !== nil && (!$c.$$is_boolean || $c == true))) {
                line_count = $rb_plus(line_count, 1);
                continue;;};
              self.$scan(/(.*)/);};}};
          self.$set_arg_state();
          if ((($b = self.$scan(/\=/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            if ((($b = self.$scan(/\=/)) !== nil && (!$b.$$is_boolean || $b == true))) {
              return "tEQQ"};
            return "tEQ";};
          if ((($b = self.$scan(/\~/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            return "tMATCH"
          } else if ((($b = self.$scan(/\>/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            return "tASSOC"};
          return "tEQL";
        } else if ((($b = self.$scan(/\"/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          (($b = [self.$new_strterm($scope.get('STR_DQUOTE'), "\"", "\x00")]), $c = self, $c['$strterm='].apply($c, $b), $b[$b.length-1]);
          return "tSTRING_BEG";
        } else if ((($b = self.$scan(/\'/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          (($b = [self.$new_strterm($scope.get('STR_SQUOTE'), "'", "\x00")]), $c = self, $c['$strterm='].apply($c, $b), $b[$b.length-1]);
          return "tSTRING_BEG";
        } else if ((($b = self.$scan(/\`/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          (($b = [self.$new_strterm($scope.get('STR_XQUOTE'), "`", "\x00")]), $c = self, $c['$strterm='].apply($c, $b), $b[$b.length-1]);
          return "tXSTRING_BEG";
        } else if ((($b = self.$scan(/\&/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          if ((($b = self.$scan(/\&/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_beg";
            if ((($b = self.$scan(/\=/)) !== nil && (!$b.$$is_boolean || $b == true))) {
              return self.$new_op_asgn("&&")};
            return "tANDOP";
          } else if ((($b = self.$scan(/\=/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_beg";
            return self.$new_op_asgn("&");};
          if ((($b = self['$spcarg?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
            result = "tAMPER"
          } else if ((($b = self['$beg?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
            result = "tAMPER"
            } else {
            result = "tAMPER2"
          };
          self.$set_arg_state();
          return result;
        } else if ((($b = self.$scan(/\|/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          if ((($b = self.$scan(/\|/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_beg";
            if ((($b = self.$scan(/\=/)) !== nil && (!$b.$$is_boolean || $b == true))) {
              return self.$new_op_asgn("||")};
            return "tOROP";
          } else if ((($b = self.$scan(/\=/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            return self.$new_op_asgn("|")};
          self.$set_arg_state();
          return "tPIPE";
        } else if ((($b = self.$scan(/\%[QqWwixrs]/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          str_type = self.$scanner().$matched()['$[]'](1, 1);
          paren = term = self.$scan(/./);
          $case = term;if ("("['$===']($case)) {term = ")"}else if ("["['$===']($case)) {term = "]"}else if ("{"['$===']($case)) {term = "}"}else if ("<"['$===']($case)) {term = ">"}else {paren = "\x00"};
          $b = Opal.to_ary((function() {$case = str_type;if ("Q"['$===']($case)) {return ["tSTRING_BEG", $scope.get('STR_DQUOTE')]}else if ("q"['$===']($case)) {return ["tSTRING_BEG", $scope.get('STR_SQUOTE')]}else if ("W"['$===']($case)) {self.$skip(/\s*/);
          return ["tWORDS_BEG", $scope.get('STR_DWORD')];}else if ("w"['$===']($case) || "i"['$===']($case)) {self.$skip(/\s*/);
          return ["tAWORDS_BEG", $scope.get('STR_SWORD')];}else if ("x"['$===']($case)) {return ["tXSTRING_BEG", $scope.get('STR_XQUOTE')]}else if ("r"['$===']($case)) {return ["tREGEXP_BEG", $scope.get('STR_REGEXP')]}else if ("s"['$===']($case)) {return ["tSTRING_BEG", $scope.get('STR_SQUOTE')]}else { return nil }})()), token = ($b[0] == null ? nil : $b[0]), func = ($b[1] == null ? nil : $b[1]);
          (($b = [self.$new_strterm2(func, term, paren)]), $c = self, $c['$strterm='].apply($c, $b), $b[$b.length-1]);
          return token;
        } else if ((($b = self.$scan(/\//)) !== nil && (!$b.$$is_boolean || $b == true))) {
          if ((($b = self['$beg?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
            (($b = [self.$new_strterm($scope.get('STR_REGEXP'), "/", "/")]), $c = self, $c['$strterm='].apply($c, $b), $b[$b.length-1]);
            return "tREGEXP_BEG";
          } else if ((($b = self.$scan(/\=/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_beg";
            return self.$new_op_asgn("/");};
          if ((($b = self['$arg?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
            if ((($b = ($c = self.$check(/\s/)['$!'](), $c !== false && $c !== nil ?self.space_seen : $c)) !== nil && (!$b.$$is_boolean || $b == true))) {
              (($b = [self.$new_strterm($scope.get('STR_REGEXP'), "/", "/")]), $c = self, $c['$strterm='].apply($c, $b), $b[$b.length-1]);
              return "tREGEXP_BEG";}};
          if ((($b = self['$after_operator?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_arg"
            } else {
            self.lex_state = "expr_beg"
          };
          return "tDIVIDE";
        } else if ((($b = self.$scan(/\%/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          if ((($b = self.$scan(/\=/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_beg";
            return self.$new_op_asgn("%");
          } else if ((($b = self.$check(/[^\s]/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            if ((($b = ((($c = self.lex_state['$==']("expr_beg")) !== false && $c !== nil) ? $c : ((($d = self.lex_state['$==']("expr_arg")) ? self.space_seen : $d)))) !== nil && (!$b.$$is_boolean || $b == true))) {
              start_word = self.$scan(/./);
              end_word = ((($b = $hash2(["(", "[", "{"], {"(": ")", "[": "]", "{": "}"})['$[]'](start_word)) !== false && $b !== nil) ? $b : start_word);
              (($b = [self.$new_strterm2($scope.get('STR_DQUOTE'), end_word, start_word)]), $c = self, $c['$strterm='].apply($c, $b), $b[$b.length-1]);
              return "tSTRING_BEG";}};
          self.$set_arg_state();
          return "tPERCENT";
        } else if ((($b = self.$scan(/\\/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          if ((($b = self.$scan(/\r?\n/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.space_seen = true;
            continue;;};
          self.$raise($scope.get('SyntaxError'), "backslash must appear before newline :" + (self.file) + ":" + (self.line));
        } else if ((($b = self.$scan(/\(/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          result = self.$scanner().$matched();
          if ((($b = self['$beg?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
            result = "tLPAREN"
          } else if ((($b = ($c = self.space_seen, $c !== false && $c !== nil ?self['$arg?']() : $c)) !== nil && (!$b.$$is_boolean || $b == true))) {
            result = "tLPAREN_ARG"
            } else {
            result = "tLPAREN2"
          };
          self.lex_state = "expr_beg";
          self.$cond_push(0);
          self.$cmdarg_push(0);
          return result;
        } else if ((($b = self.$scan(/\)/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          self.$cond_lexpop();
          self.$cmdarg_lexpop();
          self.lex_state = "expr_end";
          return "tRPAREN";
        } else if ((($b = self.$scan(/\[/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          result = self.$scanner().$matched();
          if ((($b = self['$after_operator?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_arg";
            if ((($b = self.$scan(/\]=/)) !== nil && (!$b.$$is_boolean || $b == true))) {
              return "tASET"
            } else if ((($b = self.$scan(/\]/)) !== nil && (!$b.$$is_boolean || $b == true))) {
              return "tAREF"
              } else {
              self.$raise("Unexpected '[' token")
            };
          } else if ((($b = self['$beg?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
            result = "tLBRACK"
          } else if ((($b = ($c = self['$arg?'](), $c !== false && $c !== nil ?self.space_seen : $c)) !== nil && (!$b.$$is_boolean || $b == true))) {
            result = "tLBRACK"
            } else {
            result = "tLBRACK2"
          };
          self.lex_state = "expr_beg";
          self.$cond_push(0);
          self.$cmdarg_push(0);
          return result;
        } else if ((($b = self.$scan(/\]/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          self.$cond_lexpop();
          self.$cmdarg_lexpop();
          self.lex_state = "expr_end";
          return "tRBRACK";
        } else if ((($b = self.$scan(/\}/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          self.$cond_lexpop();
          self.$cmdarg_lexpop();
          self.lex_state = "expr_end";
          return "tRCURLY";
        } else if ((($b = self.$scan(/\.\.\./)) !== nil && (!$b.$$is_boolean || $b == true))) {
          self.lex_state = "expr_beg";
          return "tDOT3";
        } else if ((($b = self.$scan(/\.\./)) !== nil && (!$b.$$is_boolean || $b == true))) {
          self.lex_state = "expr_beg";
          return "tDOT2";
        } else if ((($b = self.$scan(/\./)) !== nil && (!$b.$$is_boolean || $b == true))) {
          if (self.lex_state['$==']("expr_fname")) {
            } else {
            self.lex_state = "expr_dot"
          };
          return "tDOT";
        } else if ((($b = self.$scan(/\:\:/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          if ((($b = self['$beg?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_beg";
            return "tCOLON3";
          } else if ((($b = self['$spcarg?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_beg";
            return "tCOLON3";};
          self.lex_state = "expr_dot";
          return "tCOLON2";
        } else if ((($b = self.$scan(/\:/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          if ((($b = ((($c = self['$end?']()) !== false && $c !== nil) ? $c : self.$check(/\s/))) !== nil && (!$b.$$is_boolean || $b == true))) {
            if ((($b = self.$check(/\w/)) !== nil && (!$b.$$is_boolean || $b == true))) {
              } else {
              self.lex_state = "expr_beg";
              return "tCOLON";
            };
            self.lex_state = "expr_fname";
            return "tSYMBEG";};
          if ((($b = self.$scan(/\'/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            (($b = [self.$new_strterm($scope.get('STR_SSYM'), "'", "\x00")]), $c = self, $c['$strterm='].apply($c, $b), $b[$b.length-1])
          } else if ((($b = self.$scan(/\"/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            (($b = [self.$new_strterm($scope.get('STR_DSYM'), "\"", "\x00")]), $c = self, $c['$strterm='].apply($c, $b), $b[$b.length-1])};
          self.lex_state = "expr_fname";
          return "tSYMBEG";
        } else if ((($b = self.$scan(/\^\=/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          self.lex_state = "expr_beg";
          return self.$new_op_asgn("^");
        } else if ((($b = self.$scan(/\^/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          self.$set_arg_state();
          return "tCARET";
        } else if ((($b = self.$check(/\</)) !== nil && (!$b.$$is_boolean || $b == true))) {
          if ((($b = self.$scan(/\<\<\=/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_beg";
            return self.$new_op_asgn("<<");
          } else if ((($b = self.$scan(/\<\</)) !== nil && (!$b.$$is_boolean || $b == true))) {
            if ((($b = self['$after_operator?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
              self.lex_state = "expr_arg";
              return "tLSHFT";
            } else if ((($b = ($c = ($d = self['$after_operator?']()['$!'](), $d !== false && $d !== nil ?self['$end?']()['$!']() : $d), $c !== false && $c !== nil ?(((($d = self['$arg?']()['$!']()) !== false && $d !== nil) ? $d : self.space_seen)) : $c)) !== nil && (!$b.$$is_boolean || $b == true))) {
              if ((($b = token = self.$heredoc_identifier()) !== nil && (!$b.$$is_boolean || $b == true))) {
                return token};
              self.lex_state = "expr_beg";
              return "tLSHFT";};
            self.lex_state = "expr_beg";
            return "tLSHFT";
          } else if ((($b = self.$scan(/\<\=\>/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            if ((($b = self['$after_operator?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
              self.lex_state = "expr_arg"
              } else {
              if (self.lex_state['$==']("expr_class")) {
                cmd_start = true};
              self.lex_state = "expr_beg";
            };
            return "tCMP";
          } else if ((($b = self.$scan(/\<\=/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.$set_arg_state();
            return "tLEQ";
          } else if ((($b = self.$scan(/\</)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.$set_arg_state();
            return "tLT";}
        } else if ((($b = self.$check(/\>/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          if ((($b = self.$scan(/\>\>\=/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            return self.$new_op_asgn(">>")
          } else if ((($b = self.$scan(/\>\>/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.$set_arg_state();
            return "tRSHFT";
          } else if ((($b = self.$scan(/\>\=/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.$set_arg_state();
            return "tGEQ";
          } else if ((($b = self.$scan(/\>/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.$set_arg_state();
            return "tGT";}
        } else if ((($b = self.$scan(/->/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          self.lex_state = "expr_end";
          self.start_of_lambda = true;
          return "tLAMBDA";
        } else if ((($b = self.$scan(/[+-]/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          matched = self.$scanner().$matched();
          $b = Opal.to_ary((function() {if (matched['$==']("+")) {
            return ["tPLUS", "tUPLUS"]
            } else {
            return ["tMINUS", "tUMINUS"]
          }; return nil; })()), sign = ($b[0] == null ? nil : $b[0]), utype = ($b[1] == null ? nil : $b[1]);
          if ((($b = self['$beg?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_mid";
            (($b = [matched]), $c = self, $c['$yylval='].apply($c, $b), $b[$b.length-1]);
            if ((($b = ($c = self.$scanner().$peek(1)['$=~'](/\d/), $c !== false && $c !== nil ?Opal.ret((function() {if (utype['$==']("tUMINUS")) {
              return "-@NUM"
              } else {
              return "+@NUM"
            }; return nil; })()) : $c)) !== nil && (!$b.$$is_boolean || $b == true))) {
              } else {
              return utype
            };
          } else if ((($b = self['$after_operator?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_arg";
            if ((($b = self.$scan(/@/)) !== nil && (!$b.$$is_boolean || $b == true))) {
              (($b = [$rb_plus(matched, "@")]), $c = self, $c['$yylval='].apply($c, $b), $b[$b.length-1]);
              return "tIDENTIFIER";};
            (($b = [matched]), $c = self, $c['$yylval='].apply($c, $b), $b[$b.length-1]);
            return sign;};
          if ((($b = self.$scan(/\=/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_beg";
            return self.$new_op_asgn(matched);};
          if ((($b = self['$spcarg?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_mid";
            (($b = [matched]), $c = self, $c['$yylval='].apply($c, $b), $b[$b.length-1]);
            return utype;};
          self.lex_state = "expr_beg";
          (($b = [matched]), $c = self, $c['$yylval='].apply($c, $b), $b[$b.length-1]);
          return sign;
        } else if ((($b = self.$scan(/\?/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          if ((($b = self['$end?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_beg";
            return "tEH";};
          if ((($b = self.$check(/\ |\t|\r|\s/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_beg";
            return "tEH";
          } else if ((($b = self.$scan(/\\/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_end";
            (($b = [self.$read_escape()]), $c = self, $c['$yylval='].apply($c, $b), $b[$b.length-1]);
            return "tSTRING";};
          self.lex_state = "expr_end";
          (($b = [self.$scan(/./)]), $c = self, $c['$yylval='].apply($c, $b), $b[$b.length-1]);
          return "tSTRING";
        } else if ((($b = self.$scan(/\~/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          self.$set_arg_state();
          return "tTILDE";
        } else if ((($b = self.$check(/\$/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          if ((($b = self.$scan(/\$([1-9]\d*)/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_end";
            (($b = [self.$scanner().$matched().$sub("$", "")]), $c = self, $c['$yylval='].apply($c, $b), $b[$b.length-1]);
            return "tNTH_REF";
          } else if ((($b = self.$scan(/(\$_)(\w+)/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_end";
            return "tGVAR";
          } else if ((($b = self.$scan(/\$[\+\'\`\&!@\"~*$?\/\\:;=.,<>_]/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_end";
            return "tGVAR";
          } else if ((($b = self.$scan(/\$\w+/)) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.lex_state = "expr_end";
            return "tGVAR";
            } else {
            self.$raise("Bad gvar name: " + (self.$scanner().$peek(5).$inspect()))
          }
        } else if ((($b = self.$scan(/\$\w+/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          self.lex_state = "expr_end";
          return "tGVAR";
        } else if ((($b = self.$scan(/\@\@\w*/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          self.lex_state = "expr_end";
          return "tCVAR";
        } else if ((($b = self.$scan(/\@\w*/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          self.lex_state = "expr_end";
          return "tIVAR";
        } else if ((($b = self.$scan(/\,/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          self.lex_state = "expr_beg";
          return "tCOMMA";
        } else if ((($b = self.$scan(/\{/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          if ((($b = self.start_of_lambda) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.start_of_lambda = false;
            self.lex_state = "expr_beg";
            return "tLAMBEG";
          } else if ((($b = ((($c = self['$arg?']()) !== false && $c !== nil) ? $c : self.lex_state['$==']("expr_end"))) !== nil && (!$b.$$is_boolean || $b == true))) {
            result = "tLCURLY"
          } else if (self.lex_state['$==']("expr_endarg")) {
            result = "LBRACE_ARG"
            } else {
            result = "tLBRACE"
          };
          self.lex_state = "expr_beg";
          self.$cond_push(0);
          self.$cmdarg_push(0);
          return result;
        } else if ((($b = ($c = self.$scanner()['$bol?'](), $c !== false && $c !== nil ?self.$skip(/\__END__(\n|$)/) : $c)) !== nil && (!$b.$$is_boolean || $b == true))) {
          while ((($c = true) !== nil && (!$c.$$is_boolean || $c == true))) {
          if ((($c = self.$scanner()['$eos?']()) !== nil && (!$c.$$is_boolean || $c == true))) {
            self.eof_content = self.$yylval();
            return false;};
          self.$scan(/(.*)/);
          self.$scan(/\n/);}
        } else if ((($b = self.$check(/[0-9]/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          return self.$process_numeric()
        } else if ((($b = self.$scan(/(\w)+[\?\!]?/)) !== nil && (!$b.$$is_boolean || $b == true))) {
          return self.$process_identifier(self.$scanner().$matched(), cmd_start)};
        if ((($b = self.$scanner()['$eos?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
          if (self.scanner_stack.$size()['$=='](1)) {
            (($b = [false]), $c = self, $c['$yylval='].apply($c, $b), $b[$b.length-1]);
            return false;
            } else {
            self.scanner_stack.$pop();
            self.scanner = self.scanner_stack.$last();
            return self.$yylex();
          }};
        self.$raise("Unexpected content in parsing stream `" + (self.$scanner().$peek(5)) + "` :" + (self.file) + ":" + (self.line));};
        } catch ($returner) { if ($returner === Opal.returner) { return $returner.$v } throw $returner; }
      }, nil) && 'yylex';
    })(self, null)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["racc/parser"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  function $rb_lt(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs < rhs : lhs['$<'](rhs);
  }
  function $rb_gt(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs > rhs : lhs['$>'](rhs);
  }
  function $rb_minus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs - rhs : lhs['$-'](rhs);
  }
  function $rb_times(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs * rhs : lhs['$*'](rhs);
  }
  function $rb_ge(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs >= rhs : lhs['$>='](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass;

  Opal.add_stubs(['$class', '$_racc_do_parse_rb', '$_racc_setup', '$[]', '$!', '$==', '$next_token', '$racc_read_token', '$nil?', '$puts', '$push', '$<<', '$racc_shift', '$-@', '$last', '$pop', '$__send__', '$raise', '$racc_reduce', '$inspect', '$racc_next_state', '$racc_token2str', '$racc_print_stacks', '$empty?', '$map', '$racc_print_states', '$each_index', '$each']);
  return (function($base) {
    var self = $module($base, 'Racc');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base, $super) {
      function $Parser(){};
      var self = $Parser = $klass($base, $super, 'Parser', $Parser);

      var def = self.$$proto, $scope = self.$$scope;

      def.yydebug = nil;
      def.$_racc_setup = function() {
        var self = this;

        return ((self.$class()).$$scope.get('Racc_arg'));
      };

      def.$do_parse = function() {
        var self = this;

        return self.$_racc_do_parse_rb(self.$_racc_setup(), false);
      };

      def.$_racc_do_parse_rb = function(arg, in_debug) {
        var $a, $b, $c, $d, self = this, action_table = nil, action_check = nil, action_default = nil, action_pointer = nil, goto_table = nil, goto_check = nil, goto_default = nil, goto_pointer = nil, nt_base = nil, reduce_table = nil, token_table = nil, shift_n = nil, reduce_n = nil, use_result = nil, racc_state = nil, racc_tstack = nil, racc_vstack = nil, racc_t = nil, racc_tok = nil, racc_val = nil, racc_read_next = nil, racc_user_yyerror = nil, racc_error_status = nil, token = nil, act = nil, i = nil, nerr = nil, custate = nil, curstate = nil, reduce_i = nil, reduce_len = nil, reduce_to = nil, method_id = nil, tmp_t = nil, tmp_v = nil, reduce_call_result = nil, k1 = nil;

        action_table = arg['$[]'](0);
        action_check = arg['$[]'](1);
        action_default = arg['$[]'](2);
        action_pointer = arg['$[]'](3);
        goto_table = arg['$[]'](4);
        goto_check = arg['$[]'](5);
        goto_default = arg['$[]'](6);
        goto_pointer = arg['$[]'](7);
        nt_base = arg['$[]'](8);
        reduce_table = arg['$[]'](9);
        token_table = arg['$[]'](10);
        shift_n = arg['$[]'](11);
        reduce_n = arg['$[]'](12);
        use_result = arg['$[]'](13);
        racc_state = [0];
        racc_tstack = [];
        racc_vstack = [];
        racc_t = nil;
        racc_tok = nil;
        racc_val = nil;
        racc_read_next = true;
        racc_user_yyerror = false;
        racc_error_status = 0;
        token = nil;
        act = nil;
        i = nil;
        nerr = nil;
        custate = nil;
        while ((($b = true) !== nil && (!$b.$$is_boolean || $b == true))) {
        i = action_pointer['$[]'](racc_state['$[]'](-1));
        if (i !== false && i !== nil) {
          if (racc_read_next !== false && racc_read_next !== nil) {
            if ((($b = racc_t['$=='](0)['$!']()) !== nil && (!$b.$$is_boolean || $b == true))) {
              token = self.$next_token();
              racc_tok = token['$[]'](0);
              racc_val = token['$[]'](1);
              if (racc_tok['$=='](false)) {
                racc_t = 0
                } else {
                racc_t = token_table['$[]'](racc_tok);
                if (racc_t !== false && racc_t !== nil) {
                  } else {
                  racc_t = 1
                };
              };
              if ((($b = self.yydebug) !== nil && (!$b.$$is_boolean || $b == true))) {
                self.$racc_read_token(racc_t, racc_tok, racc_val)};
              racc_read_next = false;}};
          i = $rb_plus(i, racc_t);
          if ((($b = ((($c = ((($d = ($rb_lt(i, 0))) !== false && $d !== nil) ? $d : ((act = action_table['$[]'](i)))['$nil?']())) !== false && $c !== nil) ? $c : (action_check['$[]'](i)['$=='](racc_state['$[]'](-1))['$!']()))) !== nil && (!$b.$$is_boolean || $b == true))) {
            act = action_default['$[]'](racc_state['$[]'](-1))};
          } else {
          act = action_default['$[]'](racc_state['$[]'](-1))
        };
        if ((($b = self.yydebug) !== nil && (!$b.$$is_boolean || $b == true))) {
          self.$puts("(act: " + (act) + ", shift_n: " + (shift_n) + ", reduce_n: " + (reduce_n) + ")")};
        if ((($b = (($c = $rb_gt(act, 0)) ? $rb_lt(act, shift_n) : $c)) !== nil && (!$b.$$is_boolean || $b == true))) {
          if ($rb_gt(racc_error_status, 0)) {
            if ((($b = racc_t['$=='](1)['$!']()) !== nil && (!$b.$$is_boolean || $b == true))) {
              racc_error_status = $rb_minus(racc_error_status, 1)}};
          racc_vstack.$push(racc_val);
          curstate = act;
          racc_state['$<<'](act);
          racc_read_next = true;
          if ((($b = self.yydebug) !== nil && (!$b.$$is_boolean || $b == true))) {
            racc_tstack.$push(racc_t);
            self.$racc_shift(racc_t, racc_tstack, racc_vstack);};
        } else if ((($b = (($c = $rb_lt(act, 0)) ? $rb_gt(act, reduce_n['$-@']()) : $c)) !== nil && (!$b.$$is_boolean || $b == true))) {
          reduce_i = $rb_times(act, -3);
          reduce_len = reduce_table['$[]'](reduce_i);
          reduce_to = reduce_table['$[]']($rb_plus(reduce_i, 1));
          method_id = reduce_table['$[]']($rb_plus(reduce_i, 2));
          tmp_t = racc_tstack.$last(reduce_len);
          tmp_v = racc_vstack.$last(reduce_len);
          racc_state.$pop(reduce_len);
          racc_vstack.$pop(reduce_len);
          racc_tstack.$pop(reduce_len);
          if (use_result !== false && use_result !== nil) {
            reduce_call_result = self.$__send__(method_id, tmp_v, nil, tmp_v['$[]'](0));
            racc_vstack.$push(reduce_call_result);
            } else {
            self.$raise("not using result??")
          };
          racc_tstack.$push(reduce_to);
          if ((($b = self.yydebug) !== nil && (!$b.$$is_boolean || $b == true))) {
            self.$racc_reduce(tmp_t, reduce_to, racc_tstack, racc_vstack)};
          k1 = $rb_minus(reduce_to, nt_base);
          if ((($b = ((reduce_i = goto_pointer['$[]'](k1)))['$=='](nil)['$!']()) !== nil && (!$b.$$is_boolean || $b == true))) {
            reduce_i = $rb_plus(reduce_i, racc_state['$[]'](-1));
            if ((($b = ($c = ($d = ($rb_ge(reduce_i, 0)), $d !== false && $d !== nil ?(((curstate = goto_table['$[]'](reduce_i)))['$=='](nil)['$!']()) : $d), $c !== false && $c !== nil ?(goto_check['$[]'](reduce_i)['$=='](k1)) : $c)) !== nil && (!$b.$$is_boolean || $b == true))) {
              racc_state.$push(curstate)
              } else {
              racc_state.$push(goto_default['$[]'](k1))
            };
            } else {
            racc_state.$push(goto_default['$[]'](k1))
          };
        } else if (act['$=='](shift_n)) {
          return racc_vstack['$[]'](0)
        } else if (act['$=='](reduce_n['$-@']())) {
          self.$raise($scope.get('SyntaxError'), "unexpected '" + (racc_tok.$inspect()) + "'")
          } else {
          self.$raise("Rac: unknown action: " + (act))
        };
        if ((($b = self.yydebug) !== nil && (!$b.$$is_boolean || $b == true))) {
          self.$racc_next_state(racc_state['$[]'](-1), racc_state)};};
      };

      def.$racc_read_token = function(t, tok, val) {
        var self = this;

        self.$puts("read    " + (tok) + "(" + (self.$racc_token2str(t)) + ") " + (val.$inspect()));
        return self.$puts("\n");
      };

      def.$racc_shift = function(tok, tstack, vstack) {
        var self = this;

        self.$puts("shift  " + (self.$racc_token2str(tok)));
        self.$racc_print_stacks(tstack, vstack);
        return self.$puts("\n");
      };

      def.$racc_reduce = function(toks, sim, tstack, vstack) {
        var $a, $b, TMP_1, self = this;

        self.$puts("reduce " + ((function() {if ((($a = toks['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
          return "<none>"
          } else {
          return ($a = ($b = toks).$map, $a.$$p = (TMP_1 = function(t){var self = TMP_1.$$s || this;
if (t == null) t = nil;
          return self.$racc_token2str(t)}, TMP_1.$$s = self, TMP_1), $a).call($b)
        }; return nil; })()));
        self.$puts("  --> " + (self.$racc_token2str(sim)));
        return self.$racc_print_stacks(tstack, vstack);
      };

      def.$racc_next_state = function(curstate, state) {
        var self = this;

        self.$puts("goto  " + (curstate));
        self.$racc_print_states(state);
        return self.$puts("\n");
      };

      def.$racc_token2str = function(tok) {
        var self = this;

        return ((self.$class()).$$scope.get('Racc_token_to_s_table'))['$[]'](tok);
      };

      def.$racc_print_stacks = function(t, v) {
        var $a, $b, TMP_2, self = this;

        self.$puts("  [");
        ($a = ($b = t).$each_index, $a.$$p = (TMP_2 = function(i){var self = TMP_2.$$s || this;
if (i == null) i = nil;
        return self.$puts("    (" + (self.$racc_token2str(t['$[]'](i))) + " " + (v['$[]'](i).$inspect()) + ")")}, TMP_2.$$s = self, TMP_2), $a).call($b);
        return self.$puts("  ]");
      };

      return (def.$racc_print_states = function(s) {
        var $a, $b, TMP_3, self = this;

        self.$puts("  [");
        ($a = ($b = s).$each, $a.$$p = (TMP_3 = function(st){var self = TMP_3.$$s || this;
if (st == null) st = nil;
        return self.$puts("   " + (st))}, TMP_3.$$s = self, TMP_3), $a).call($b);
        return self.$puts("  ]");
      }, nil) && 'racc_print_states';
    })(self, null)
  })(self)
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/parser/grammar"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass, $hash = Opal.hash;

  Opal.add_stubs(['$require', '$new', '$each', '$empty?', '$[]=', '$to_i', '$split', '$new_compstmt', '$[]', '$new_block', '$<<', '$new_body', '$lex_state=', '$lexer', '$new_alias', '$s', '$to_sym', '$value', '$new_if', '$new_while', '$new_until', '$new_rescue_mod', '$new_assign', '$new_op_asgn', '$op_to_setter', '$new_unary_call', '$new_return', '$new_break', '$new_next', '$new_call', '$new_super', '$new_yield', '$new_assignable', '$new_attrasgn', '$new_colon2', '$new_colon3', '$new_const', '$new_sym', '$new_op_asgn1', '$new_irange', '$new_erange', '$new_binary_call', '$new_int', '$new_float', '$include?', '$type', '$==', '$-@', '$to_f', '$new_not', '$new_and', '$new_or', '$new_hash', '$add_block_pass', '$cmdarg_push', '$cmdarg_pop', '$new_block_pass', '$new_splat', '$line', '$new_paren', '$new_array', '$new_nil', '$cond_push', '$cond_pop', '$new_class', '$new_sclass', '$new_module', '$push_scope', '$new_def', '$pop_scope', '$new_iter', '$new_ident', '$new_block_args', '$push', '$intern', '$first', '$nil?', '$new_str', '$str_append', '$new_xstr', '$new_regexp', '$concat', '$new_str_content', '$strterm', '$strterm=', '$new_evstr', '$cond_lexpop', '$cmdarg_lexpop', '$new_gvar', '$new_ivar', '$new_cvar', '$new_dsym', '$negate_num', '$new_self', '$new_true', '$new_false', '$new___FILE__', '$new___LINE__', '$new_var_ref', '$new_kwrestarg', '$new_kwoptarg', '$new_kwarg', '$new_args_tail', '$new_args', '$add_local', '$scope', '$raise']);
  self.$require("racc/parser.rb");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base, $super) {
      function $Parser(){};
      var self = $Parser = $klass($base, $super, 'Parser', $Parser);

      var def = self.$$proto, $scope = self.$$scope, $a, $b, TMP_1, $c, TMP_3, $d, TMP_5, $e, TMP_7, clist = nil, racc_action_table = nil, arr = nil, idx = nil, racc_action_check = nil, racc_action_pointer = nil, racc_action_default = nil, racc_goto_table = nil, racc_goto_check = nil, racc_goto_pointer = nil, racc_goto_default = nil, racc_reduce_table = nil, racc_reduce_n = nil, racc_shift_n = nil, racc_token_table = nil, racc_nt_base = nil, racc_use_result_var = nil;

      clist = ["63,64,65,8,51,566,-89,-83,57,58,270,203,204,61,73,59,60,62,23,24,66", "67,74,-523,785,639,767,22,28,27,90,89,91,92,543,609,17,572,270,573,768", "265,7,41,6,9,94,93,566,84,50,86,85,87,-64,88,95,96,638,81,82,-94,38", "39,-88,535,587,790,537,573,-95,203,204,-525,-83,-90,-440,101,566,566", "-90,676,100,-440,36,-90,-91,30,-523,-92,52,-75,781,54,-92,32,269,608", "-92,40,101,-93,566,265,265,100,780,18,399,-523,-81,-83,79,73,75,76,77", "78,101,-83,565,74,80,100,269,708,63,64,65,56,51,-440,53,225,57,58,37", "83,542,61,-91,59,60,62,256,257,66,67,203,204,708,341,340,255,28,27,90", "89,91,92,305,101,217,565,-525,-86,100,-526,-80,41,708,586,94,93,-87", "84,50,86,85,87,259,88,95,96,-82,81,82,620,38,39,101,101,101,565,565", "100,100,100,-90,-91,-90,-91,-84,-90,-91,-85,-525,-92,225,-92,208,522", "-92,212,778,101,52,565,873,54,100,254,270,-277,101,40,707,-447,-84,100", "-277,-82,-84,216,-447,-526,-92,305,79,73,75,76,77,78,-83,-437,735,74", "80,101,790,707,-437,639,100,56,850,732,53,735,-94,388,37,83,63,64,65", "8,51,101,732,707,57,58,100,203,204,61,-277,59,60,62,23,24,66,67,638", "265,412,341,340,22,28,27,90,89,91,92,-82,-84,17,597,341,340,597,-89", "7,41,269,9,94,93,597,84,50,86,85,87,-449,88,95,96,597,81,82,733,38,39", "409,796,787,797,-278,411,410,-82,-84,597,733,-278,203,204,888,-82,-84", "446,549,735,36,-443,552,30,597,-278,52,602,-443,54,732,32,-278,599,598", "40,599,598,600,-88,518,519,751,18,599,598,604,522,79,73,75,76,77,78", "599,598,595,74,80,-278,338,337,341,340,639,56,-95,635,53,599,598,610", "37,83,63,64,65,8,51,639,-278,101,57,58,599,598,100,61,-86,59,60,62,23", "24,66,67,-94,638,733,809,790,22,28,27,90,89,91,92,101,101,17,-271,638", "100,100,615,7,41,-271,9,94,93,-449,84,50,86,85,87,583,88,95,96,483,81", "82,799,38,39,800,225,229,234,235,236,231,233,241,242,237,238,-277,218", "219,-277,101,239,240,-277,36,100,-277,30,-526,-80,52,-526,-271,54,619", "32,222,-88,228,40,224,223,220,221,232,230,226,18,227,582,801,202,79", "73,75,76,77,78,583,552,-448,74,80,200,243,581,-227,-448,101,56,201,-277", "53,100,-277,-82,37,83,63,64,65,8,51,-90,842,843,57,58,844,95,96,61,-87", "59,60,62,23,24,66,67,-95,535,804,757,537,22,28,27,90,89,91,92,919,582", "17,-448,341,340,199,920,7,41,481,9,94,93,770,84,50,86,85,87,525,88,95", "96,790,81,82,526,38,39,575,442,-81,225,-448,-442,225,576,443,-271,-89", "-448,-442,813,888,514,-271,-445,814,735,36,-446,515,30,-445,918,52,385", "-446,54,732,32,816,-279,222,40,490,222,224,223,-279,224,223,18,490,444", "445,592,79,73,75,76,77,78,593,574,444,74,80,-448,338,337,341,340,-271", "56,535,447,53,537,513,-258,37,83,63,64,65,8,51,-444,448,535,57,58,534", "-525,-444,61,-279,59,60,62,23,24,66,67,203,204,733,397,398,22,28,27", "90,89,91,92,809,790,17,-278,298,299,818,615,7,41,-278,9,94,93,753,84", "50,86,85,87,-279,88,95,96,213,81,82,-279,38,39,479,225,229,234,235,236", "231,233,241,242,237,238,-324,218,219,262,481,239,240,-324,36,483,263", "30,203,204,52,711,-278,54,573,32,222,826,228,40,224,223,220,221,232", "230,226,18,227,-279,703,828,79,73,75,76,77,78,690,831,832,74,80,834", "243,836,63,64,65,56,51,-324,53,-440,57,58,37,83,838,61,-440,59,60,62", "256,257,66,67,103,104,105,106,107,255,288,292,90,89,91,92,700,840,217", "698,338,337,341,340,490,41,688,-259,94,93,305,84,50,86,85,87,-63,88", "95,96,507,81,82,684,38,39,683,225,229,234,235,236,231,233,241,242,237", "238,682,218,219,297,508,239,240,296,208,509,676,212,851,852,52,853,854", "54,528,644,222,252,228,40,224,223,220,221,232,230,226,216,227,856,857", "676,79,73,75,76,77,78,516,859,668,74,80,-257,243,863,-227,666,-75,56", "690,665,53,868,274,870,37,83,63,64,65,8,51,663,244,305,57,58,655,654", "876,61,878,59,60,62,23,24,66,67,103,104,105,106,107,22,28,27,90,89,91", "92,879,305,17,305,338,337,341,340,7,41,265,9,94,93,225,84,50,86,85,87", "225,88,95,96,225,81,82,225,38,39,892,225,229,234,235,236,231,233,241", "242,237,238,-260,218,219,523,524,239,240,265,36,894,618,30,-257,198", "52,-526,552,54,617,32,222,904,228,40,224,223,220,221,232,230,226,18", "227,905,197,196,79,73,75,76,77,78,195,910,831,74,80,912,243,636,836", "836,838,56,225,614,53,611,607,603,37,83,63,64,65,8,51,921,305,108,57", "58,579,578,927,61,577,59,60,62,23,24,66,67,103,104,105,106,107,22,28", "27,90,89,91,92,401,554,17,333,331,330,682,332,7,41,97,9,94,93,562,84", "50,86,85,87,561,88,95,96,552,81,82,550,38,39,937,225,229,234,235,236", "231,233,241,242,237,238,836,218,219,939,940,239,240,549,36,546,483,30", "539,538,52,,,54,,32,222,,228,40,224,223,220,221,232,230,226,18,227,", ",,79,73,75,76,77,78,,,,74,80,,243,,63,64,65,56,51,,53,,57,58,37,83,", "61,,59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,554,217,333", "331,330,,332,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,225", "229,234,235,236,231,233,241,242,237,238,,218,219,,,239,240,,208,,,212", ",,52,,,54,,644,222,,228,40,224,223,220,221,232,230,226,216,227,,,,79", "73,75,76,77,78,,,,74,80,,243,,63,64,65,56,51,,53,,57,58,37,83,,61,,59", "60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,554,217,333,331,330", ",332,,289,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,745,,333,331,330", "735,332,,,,,,557,,,,732,,,,560,,,899,,,212,225,,52,,,54,,,,,,335,730", ",,,,,,338,337,341,340,,79,73,75,76,77,78,,222,,74,80,224,223,220,221", ",,56,,,53,,,,293,83,63,64,65,8,51,,,733,57,58,,,,61,,59,60,62,23,24", "66,67,,,,,,22,28,27,90,89,91,92,,,17,,,,,,7,41,,9,94,93,,84,50,86,85", "87,,88,95,96,,81,82,,38,39,,225,229,234,235,236,231,233,241,242,237", "238,,218,219,,,239,240,,36,,,30,,,52,,,54,,32,222,,228,40,224,223,220", "221,232,230,226,18,227,,,,79,73,75,76,77,78,,,,74,80,,243,,63,64,65", "56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292", "90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,259,88,95,96,,81", "82,,38,39,,225,229,234,235,236,231,233,241,242,237,238,,218,219,,,239", "240,,208,,,212,,,52,,,54,,,222,252,228,40,224,223,220,221,232,230,226", "216,227,,,,79,73,75,76,77,78,,,,74,80,,243,,63,64,65,56,51,,53,,57,58", "37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217", ",,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,225,229,234", "235,236,231,233,241,242,237,238,,218,219,,,239,240,,208,,,212,,,52,", ",54,,,222,,228,40,224,223,220,221,232,230,226,216,227,,,,79,73,75,76", "77,78,,,,74,80,,243,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,23", "24,66,67,,,,,,22,28,27,90,89,91,92,,,17,,,,,,,41,,,94,93,,84,50,86,85", "87,,88,95,96,,81,82,,38,39,,225,229,234,235,236,231,233,241,242,237", "238,,218,219,,,239,240,,208,,,212,213,,52,,,54,,,222,,228,40,224,223", "220,221,232,230,226,18,227,,,,79,73,75,76,77,78,,,,74,80,,243,,63,64", "65,56,51,,53,,57,58,37,83,,61,,59,60,62,23,24,66,67,,,,,,22,28,27,90", "89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38", "39,,225,229,234,235,236,231,233,241,242,237,238,,218,219,,,239,240,", "208,,,212,,,52,,,54,,,222,,228,40,224,223,220,221,232,230,226,216,227", ",,,79,73,75,76,77,78,,,,74,80,,243,,63,64,65,56,51,,53,,57,58,37,83", ",61,,59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,", ",41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,225,229,234,235", "236,231,233,241,242,237,238,,218,219,,,239,240,,208,,,212,,,52,,,54", ",,222,,228,40,224,223,220,221,232,230,226,216,227,,,,79,73,75,76,77", "78,,,,74,80,,243,,,,,56,,,53,,,,37,83,63,64,65,8,51,,,,57,58,,,,61,", "59,60,62,23,24,66,67,,,,,,22,28,27,90,89,91,92,,,17,,,,,,7,41,,9,94", "93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,225,229,234,235,236,231", "233,241,242,237,238,,218,219,,,239,240,,36,,,30,,,52,,,54,,32,222,,228", "40,224,223,220,221,232,230,226,18,227,,,,79,73,75,76,77,78,,,,74,80", ",243,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,", ",,,255,28,27,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,259", "88,95,96,,81,82,,38,39,,225,229,234,235,236,231,233,241,242,237,238", ",218,219,,,239,240,,208,,,212,,,52,,,54,,254,222,252,228,40,224,223", "220,221,232,230,226,216,227,,,,79,73,75,76,77,78,,,,74,80,,243,,63,64", "65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,28,27", "90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,259,88,95,96,,81", "82,,38,39,,225,229,234,235,236,231,233,241,242,237,238,,218,219,,,239", "240,,208,,,212,,,52,,,54,,254,222,252,228,40,224,223,220,221,232,230", "226,216,227,,,,79,73,75,76,77,78,,,,74,80,,243,,63,64,65,56,51,,53,", "57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,28,27,90,89,91,92,", ",217,,,,,,,41,,,94,93,,84,50,86,85,87,259,88,95,96,,81,82,,38,39,,225", "229,234,235,236,231,233,241,242,237,238,,218,219,,,239,240,,208,,,212", ",,52,,,54,,254,222,252,228,40,224,223,220,221,232,230,226,216,227,,", ",79,73,75,76,77,78,,,,74,80,,243,,,,,56,,,53,,,,37,83,63,64,65,8,51", ",,,57,58,,,,61,,59,60,62,23,24,66,67,,,,,,22,28,27,90,89,91,92,,,17", ",,,,,7,41,,9,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,225,229", "234,235,236,231,233,241,242,237,238,,218,219,,,239,240,,36,,,30,,,52", ",,54,,32,222,,228,40,224,223,220,221,232,230,226,18,227,,,,79,73,75", "76,77,78,,,,74,80,,243,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62", "23,24,66,67,,,,,,22,28,27,90,89,91,92,,554,17,333,331,330,225,332,,41", ",,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,554,,333,331,330,,332", "222,,557,,224,223,220,221,,,560,,,208,,,212,225,,52,,,54,,,,,,40,,,557", ",239,240,,18,,,806,,79,73,75,76,77,78,,222,,74,80,224,223,220,221,,", "56,,,53,,,,37,83,63,64,65,8,51,,,,57,58,,,,61,,59,60,62,23,24,66,67", ",,,,,22,28,27,90,89,91,92,,,17,,,,,,7,41,,9,94,93,,84,50,86,85,87,,88", "95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,36,,,30,,,52,,,54,,32,,,,40", ",,324,,333,331,330,18,332,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65", "56,51,,53,,57,58,37,83,,61,335,59,60,62,256,257,66,67,338,337,341,340", ",255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88", "95,96,,81,82,,38,39,,225,229,234,235,236,231,233,241,242,237,238,,-546", "-546,,,239,240,,208,,,212,,,52,,,54,,254,222,,228,40,224,223,220,221", "232,230,226,216,227,,,,79,73,75,76,77,78,,,,74,80,,,,,,,56,,,53,,,,37", "83,63,64,65,8,51,,,,57,58,,,,61,,59,60,62,23,24,66,67,,,,,,22,28,27", "90,89,91,92,,,17,,,,,,7,41,,9,94,93,,84,50,86,85,87,,88,95,96,,81,82", ",38,39,,225,,,,,,,,,,,,,,,,239,240,,36,,,278,,,52,,,54,,32,222,,228", "40,224,223,220,221,,,226,18,227,,,,79,73,75,76,77,78,,,,74,80,,,,63", "64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288", "292,90,89,91,92,,,217,,,,,,,289,,,94,93,,84,50,86,85,87,,88,95,96,,81", "82,324,,333,331,330,,332,,,,,,,,,,,,,,,,,286,,,283,,,52,,,54,,282,,", ",335,,,,,,,,338,337,341,340,,79,73,75,76,77,78,772,,,74,80,,,,63,64", "65,56,51,,53,,57,58,293,83,,61,,59,60,62,256,257,66,67,,,,,,255,288", "292,90,89,91,92,,,217,,,,,,,289,,,94,93,,84,50,86,85,87,,88,95,96,,81", "82,745,,333,331,330,735,332,,,,,,,,,,732,,,,,,,286,,,212,,,52,,,54,", ",,,,335,,,,,,,,338,337,341,340,,79,73,75,76,77,78,,,,74,80,,,,295,,", "56,,,53,,,,293,83,63,64,65,,51,,,733,57,58,,,,61,,59,60,62,256,257,66", "67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85", "87,,88,95,96,,81,82,,38,39,,225,229,234,235,236,231,233,,,237,238,,", ",,,239,240,,208,,,212,,,52,,,54,,,222,,228,40,224,223,220,221,232,230", "226,216,227,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57", "58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,", "217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,225,-546", "-546,-546,-546,231,233,,,-546,-546,,,,,,239,240,,208,,,212,,,52,,,54", ",,222,,228,40,224,223,220,221,232,230,226,216,227,,,,79,73,75,76,77", "78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257", "66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86", "85,87,,88,95,96,,81,82,,38,39,,225,229,234,235,236,231,233,241,,237", "238,,,,,,239,240,,208,,,212,,,52,,,54,,,222,,228,40,224,223,220,221", "232,230,226,216,227,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51", ",53,,57,58,37,83,,61,,59,60,62,23,24,66,67,,,,,,22,28,27,90,89,91,92", ",,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,225", ",,,,,,,,,,,,,,,239,240,,208,,,212,,,52,,,54,,,222,,228,40,224,223,220", "221,,,226,216,227,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53", ",57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92", ",,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,225", "-546,-546,-546,-546,231,233,,,-546,-546,,,,,,239,240,,208,,,212,,,52", ",,54,,,222,,228,40,224,223,220,221,232,230,226,216,227,,,,79,73,75,76", "77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256", "257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50", "86,85,87,,88,95,96,,81,82,,38,39,,225,229,234,235,236,231,233,241,242", "237,238,,-546,-546,,,239,240,,208,,,212,,,52,,,54,,,222,,228,40,224", "223,220,221,232,230,226,216,227,,,,79,73,75,76,77,78,,,,74,80,,,,63", "64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288", "292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81", "82,,38,39,,225,-546,-546,-546,-546,231,233,,,-546,-546,,,,,,239,240", ",208,,,212,,,52,,,54,,,222,,228,40,224,223,220,221,232,230,226,216,227", ",,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61", ",59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41", ",,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,225,-546,-546,-546", "-546,231,233,,,-546,-546,,,,,,239,240,,208,,,212,,,52,,,54,,,222,,228", "40,224,223,220,221,232,230,226,216,227,,,,79,73,75,76,77,78,,,,74,80", ",,,,,,56,,,53,,,,37,83,63,64,65,8,51,,,,57,58,,,,61,,59,60,62,23,24", "66,67,,,,,,22,28,27,90,89,91,92,,,17,,,,,,7,41,,9,94,93,,84,50,86,85", "87,,88,95,96,,81,82,,38,39,,225,,,,,,,,,,,,,,,,239,240,,36,,,30,,,52", ",,54,,32,222,,228,40,224,223,220,221,,,226,18,227,,,,79,73,75,76,77", "78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257", "66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86", "85,87,,88,95,96,,81,82,,38,39,,225,-546,-546,-546,-546,231,233,,,-546", "-546,,,,,,239,240,,208,,,212,,,52,,,54,,,222,,228,40,224,223,220,221", "232,230,226,216,227,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51", ",53,,57,58,37,83,,61,,59,60,62,23,24,66,67,,,,,,22,28,27,90,89,91,92", ",,17,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,225,", ",,,,,,,,,,,,,,239,240,,208,,,212,,,52,,,54,,,222,,228,40,224,223,220", "221,,,226,18,227,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53", ",57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,28,27,90,89,91,92", ",,217,,,,,,,41,,,94,93,,84,50,86,85,87,259,88,95,96,,81,82,,38,39,,225", "-546,-546,-546,-546,231,233,,,-546,-546,,,,,,239,240,,208,,,212,,,52", ",,54,,254,222,,228,40,224,223,220,221,232,230,226,216,227,,,,79,73,75", "76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256", "257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50", "86,85,87,259,88,95,96,,81,82,,38,39,,225,,,,,,,,,,,,,,,,239,240,,208", ",,212,,,52,,,54,,,222,,228,40,224,223,220,221,,,,216,,,,,79,73,75,76", "77,78,,,,74,80,,,,,,,56,,,53,,,,37,83,63,64,65,8,51,,,,57,58,,,,61,", "59,60,62,23,24,66,67,,,,,,22,28,27,90,89,91,92,,,17,,,,,,7,41,,9,94", "93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,225,,,,,,,,,,,,,,,,239,240", ",36,,,30,,,52,,,54,,32,222,,228,40,224,223,220,221,,,,18,,,,,79,73,75", "76,77,78,,,,74,80,,,,,,,56,,,53,,,,37,83,63,64,65,8,51,,,,57,58,,,,61", ",59,60,62,23,24,66,67,,,,,,22,28,27,90,89,91,92,,,17,,,,,,7,41,,9,94", "93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,36,,", "30,,,52,,,54,,32,,,,40,,,,,,,,18,,,,,79,73,75,76,77,78,,,,74,80,,,,63", "64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,23,24,66,67,,,,,,22,28,27", "90,89,91,92,,,17,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,", "38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,18,,,,", "79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59", "60,62,23,24,66,67,,,,,,22,28,27,90,89,91,92,,,17,,,,,,,41,,,94,93,,84", "50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,", "52,,,54,,,,,,40,,,,,,,,18,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65", "56,51,,53,,57,58,37,83,,61,,59,60,62,23,24,66,67,,,,,,22,28,27,90,89", "91,92,,,17,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39", ",,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,18,,,,,79,73", "75,76,77,78,,,,74,80,101,,,,,100,56,,,53,,,,37,83,63,64,65,,51,,,,57", "58,,,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217", ",,,,,,289,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,324,,333,331,330", ",332,,,,,,,,,,,,,,,,,353,,,30,,,52,,,54,,32,,,,335,319,,,,,,,338,337", "341,340,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,293", "83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,", ",,,,289,,,94,93,,84,50,86,85,358,,88,95,96,,81,82,745,,333,331,330,735", "332,,,,,,,,,,732,,,,364,,,359,,,212,,,52,,,54,,,,,,335,,,,,,,,338,337", "341,340,,79,73,75,76,77,78,,,,74,80,,,,,,,56,,,53,,,,293,83,63,64,65", "8,51,,,733,57,58,,,,61,,59,60,62,23,24,66,67,,,,,,22,28,27,90,89,91", "92,,,17,,,,,,7,41,,9,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,", ",,,,,,,,,,,,,,,,,,,36,,,30,,,52,,,54,,32,,,,40,,,,,,,,18,,,,,79,73,75", "76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,23", "24,66,67,,,,,,22,28,27,90,89,91,92,,,17,,,,,,,41,,,94,93,,84,50,86,85", "87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,", ",,,,40,,,,,,,,18,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53", ",57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92", ",,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,", ",,,,,,,,,,,,,,,208,,,212,,,52,,,54,,254,,,,40,,,,,,,,216,,,,,79,73,75", "76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256", "257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50", "86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52", ",,54,,418,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65", "56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292", "90,89,91,92,,,217,,,,,,,289,,,94,93,,84,50,86,85,87,,88,95,96,,81,82", "324,,333,331,330,,332,,,,,,,,,,,,,,,,,286,,,283,,,52,,,54,,,,,,335,", "545,,,,,,338,337,341,340,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56", "51,,53,,57,58,293,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90", "89,91,92,,,217,,,,,,,289,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,745", ",333,331,330,735,332,,,,,,,,,,732,,,,,,,286,,,283,,,52,,,54,,,,,,335", "730,,,,,,,338,337,341,340,,79,73,75,76,77,78,,,,74,80,,,,,,,56,,,53", ",,,293,83,63,64,65,8,51,,,733,57,58,,,,61,,59,60,62,23,24,66,67,,,,", ",22,28,27,90,89,91,92,,,17,,,,,,7,41,,9,94,93,,84,50,86,85,87,,88,95", "96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,36,,,30,,,52,,,54,,32,,,,40,,,", ",,,,18,,,,,79,73,75,76,77,78,,,,74,80,,,,,,,56,,,53,,,,37,83,63,64,65", "8,51,,,,57,58,,,,61,,59,60,62,23,24,66,67,,,,,,22,28,27,90,89,91,92", ",,17,,,,,,7,41,,9,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,", ",,,,,,,,,,,,,,,,36,,,30,,,52,,,54,,32,,,,40,,,,,,,,18,,,,,79,73,75,76", "77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256", "257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50", "86,85,87,259,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,", "52,,,54,,,,252,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,,,,56", ",,53,,,,37,83,63,64,65,8,51,,,,57,58,,,,61,,59,60,62,23,24,66,67,,,", ",,22,28,27,90,89,91,92,,,17,,,,,,7,41,6,9,94,93,,84,50,86,85,87,,88", "95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,36,,,30,,,52,,,54,,32,,,,40", ",,,,,,,18,,,,,79,73,75,76,77,78,,,,74,80,,,,,,401,56,,,53,,,,37,83,63", "64,65,,51,,,,57,58,,,,61,,59,60,62,23,24,66,67,,,,,,22,28,27,90,89,91", "92,,,17,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,", ",,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,18,,,,,79,73,75", "76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,23", "24,66,67,,,,,,22,28,27,90,89,91,92,,,17,,,,,,,41,,,94,93,,84,50,86,85", "87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,", ",,,,40,,,,,,,,18,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53", ",57,58,37,83,,61,,59,60,62,23,24,66,67,,,,,,22,28,27,90,89,91,92,,,17", ",,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,", ",,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,18,,,,,79,73,75,76,77,78", ",,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,23,24,66,67", ",,,,,22,28,27,90,89,91,92,,,17,,,,,,,41,,,94,93,,84,50,86,85,87,,88", "95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40", ",,,,,,,18,,,,,79,73,75,76,77,78,,,,74,80,,,,,,,56,,,53,,,,37,83,63,64", "65,8,51,,,,57,58,,,,61,,59,60,62,23,24,66,67,,,,,,22,28,27,90,89,91", "92,,,17,,,,,,7,41,,9,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,", ",,,,,,,,,,,,,,,,,,,36,,,30,,,52,,,54,,32,,,,40,,,,,,,,18,,,,,79,73,75", "76,77,78,,,,74,80,,,,,,,56,,,53,,,,37,83,63,64,65,8,51,,,,57,58,,,,61", ",59,60,62,23,24,66,67,,,,,,22,28,27,90,89,91,92,,,17,,,,,,7,41,6,9,94", "93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,36,,", "30,,,52,,,54,,32,,,,40,,,,,,,,18,,,,,79,73,75,76,77,78,,,,74,80,,,,63", "64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288", "292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,259,88,95,96", ",81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,644,,252,,40", ",,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,,,,56,,,53,,,,37,83,63", "64,65,8,51,,,,57,58,,,,61,,59,60,62,23,24,66,67,,,,,,22,28,27,90,89", "91,92,,,17,,,,,,7,41,,9,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39", ",,,,,,,,,,,,,,,,,,,,36,,,30,,,52,,,54,,32,,,,40,,,,,,,,18,,,,,79,73", "75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62", "256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84", "50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,", "52,,,54,,,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,,,,56,", ",53,,,,37,83,63,64,65,8,51,,,,57,58,,,,61,,59,60,62,23,24,66,67,,,,", ",22,28,27,90,89,91,92,,,17,,,,,,7,41,,9,94,93,,84,50,86,85,87,,88,95", "96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,36,,,30,,,52,,,54,,32,,,,40,,,", ",,,,18,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37", "83,,61,,59,60,62,23,24,66,67,,,,,,22,28,27,90,89,91,92,,,217,,,,,,,41", ",,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,", "208,,,212,,,52,,,54,,418,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74", "80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,23,24,66,67,,,,", ",22,28,27,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95", "96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,418,,,,40", ",,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57", "58,37,83,,61,,59,60,62,23,24,66,67,,,,,,22,28,27,90,89,91,92,,,217,", ",,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,", ",,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78", ",,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66", "67,,,,,,255,28,27,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87", "259,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,", "254,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,,,,56,,,53,,", ",37,83,63,64,65,8,51,,,,57,58,,,,61,,59,60,62,23,24,66,67,,,,,,22,28", "27,90,89,91,92,,,17,,,,,,7,41,,9,94,93,,84,50,86,85,87,,88,95,96,,81", "82,,38,39,,,,,,,,,,,,,,,,,,,,,36,,,30,,,52,,,54,,32,,,,40,,,,,,,,18", ",,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61", ",59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41", ",,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,", "208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80", ",,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,23,24,66,67,,,,,,22", "28,27,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,", "81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,", "216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83", ",61,,59,60,62,23,24,66,67,,,,,,22,28,27,90,89,91,92,,,17,,,,,,,41,,", "94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208", ",,212,,,52,,,54,,,,,,40,,,,,,,,18,,,,,79,73,75,76,77,78,,,,74,80,,,", "63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,23,24,66,67,,,,,,22,28", "27,90,89,91,92,,,17,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82", ",38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,18,,,", ",79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,", "59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41,", ",94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208", ",,212,,,52,,,54,,,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,", ",63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,23,24,66,67,,,,,,22,28", "27,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81", "82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,216", ",,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61", ",59,60,62,23,24,66,67,,,,,,22,28,27,90,89,91,92,,,217,,,,,,,41,,,94", "93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,", ",212,,,52,,,54,,,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,", "63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,23,24,66,67,,,,,,22,28", "27,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81", "82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,216", ",,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61", ",59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41", ",,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,", "208,,,212,,,52,,,54,,756,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74", "80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,", ",,,255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88", "95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40", ",,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,,,,56,,,53,,,,37,83,63", "64,65,8,51,,,,57,58,,,,61,,59,60,62,23,24,66,67,,,,,,22,28,27,90,89", "91,92,,,17,,,,,,7,41,,9,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39", ",,,,,,,,,,,,,,,,,,,,36,,,30,,,52,,,54,,32,,,,40,,,,,,,,18,,,,,79,73", "75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62", "23,24,66,67,,,,,,22,28,27,90,89,91,92,,,17,,,,,,,41,,,94,93,,84,50,86", "85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,450,52", ",,54,,,,,,40,,,,,,,,18,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56", "51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90", "89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38", "39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,,,,79", "73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60", "62,23,24,66,67,,,,,,22,28,27,90,89,91,92,,,17,,,,,,,41,,,94,93,,84,50", "86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52", ",,54,,,,,,40,,,,,,,,18,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56", "51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90", "89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38", "39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,,,,79", "73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60", "62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93", ",84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212", ",,52,,,54,,,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64", "65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292", "90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82", ",38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,", ",,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61", ",59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41", ",,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,", "208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80", ",,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,", "255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95", "96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,", ",,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58", "37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217", ",,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,", ",,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78", ",,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66", "67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85", "87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,", ",,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,", "53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91", "92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,", ",,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,,,,79,73,75", "76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256", "257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50", "86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52", ",,54,,,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56", "51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90", "89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38", "39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,,,,79", "73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60", "62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93", ",84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212", ",,52,,,54,,,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64", "65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292", "90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82", ",38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,", ",,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61", ",59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41", ",,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,", "208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80", ",,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,", "255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95", "96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,", ",,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58", "37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217", ",,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,", ",,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78", ",,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66", "67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85", "87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,", ",,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,", "53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91", "92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,", ",,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,,,,79,73,75", "76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256", "257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50", "86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52", ",,54,,,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56", "51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90", "89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38", "39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,,,,79", "73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60", "62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93", ",84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212", ",,52,,,54,,,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64", "65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292", "90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82", ",38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,", ",,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61", ",59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41", ",,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,", "208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80", ",,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,", "255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95", "96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,", ",,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58", "37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217", ",,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,", ",,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78", ",,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66", "67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85", "87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,", ",,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,", "53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91", "92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,", ",,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,,,,79,73,75", "76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,23", "24,66,67,,,,,,22,28,27,90,89,91,92,,,17,,,,,,,41,,,94,93,,84,50,86,85", "87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,", ",,,,40,,,,,,,,18,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53", ",57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,28,27,90,89,91,92", ",,217,,,,,,,41,,,94,93,,84,50,86,85,87,259,88,95,96,,81,82,,38,39,,", ",,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,254,,,,40,,,,,,,,216,,,,,79,73", "75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62", "256,257,66,67,,,,,,255,28,27,90,89,91,92,,,217,,,,,,,41,,,94,93,,84", "50,86,85,87,259,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212", ",,52,,,54,,254,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,63", "64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288", "292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81", "82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,216", ",,,,79,73,75,76,77,78,,,,74,80,,,,,,,56,,,53,,,,37,83,63,64,65,8,51", ",,,57,58,,,,61,,59,60,62,23,24,66,67,,,,,,22,28,27,90,89,91,92,,,17", ",,,,,7,41,,9,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,", ",,,,,,,,,,,36,,,30,,,52,,,54,,32,,,,40,,,,,,,,18,,,,,79,73,75,76,77", "78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257", "66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86", "85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54", ",,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51", ",53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90,89", "91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39", ",,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,,,,79,73", "75,76,77,78,,,,74,80,,,,,,,56,,,53,,,,37,83,63,64,65,8,51,,,,57,58,", ",,61,,59,60,62,23,24,66,67,,,,,,22,28,27,90,89,91,92,,,17,,,,,,7,41", ",9,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,", ",36,,,30,,,52,,,54,,32,,,,40,,,,,,,,18,,,,,79,73,75,76,77,78,,,,74,80", ",,,,,,56,,,53,,,,37,83,63,64,65,8,51,,,,57,58,,,,61,,59,60,62,23,24", "66,67,,,,,,22,28,27,90,89,91,92,,,17,,,,,,7,41,,9,94,93,,84,50,86,85", "87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,36,,,30,,,52,,,54,,32", ",,,40,,,,,,,,18,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53", ",57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92", ",,217,,,,,,,289,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,,,,,,,,,,", ",,,,,,,,,,,,694,,,212,,,52,,,54,,,,,,,,,,,,,,,,,,,79,73,75,76,77,78", ",,,74,80,,,,63,64,65,56,51,,53,,57,58,293,83,,61,,59,60,62,256,257,66", "67,,,,,,255,28,27,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87", "259,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,", "254,,252,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56", "51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,28,27,90,89", "91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,259,88,95,96,,81,82,,38", "39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,499,,,54,,254,,252,,40,,,,,,,,216", ",,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61", ",59,60,62,256,257,66,67,,,,,,255,28,27,90,89,91,92,,,217,,,,,,,41,,", "94,93,,84,50,86,85,87,259,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,", ",208,,,212,,503,52,,,54,,254,,252,,40,,,,,,,,216,,,,,79,73,75,76,77", "78,,,,74,80,,,,,,,56,,,53,,,,37,83,63,64,65,8,51,,,,57,58,,,,61,,59", "60,62,23,24,66,67,,,,,,22,28,27,90,89,91,92,,,17,,,,,,7,41,,9,94,93", ",84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,36,,,30", ",,52,,,54,,32,,,,40,,,,,,,,18,,,,,79,73,75,76,77,78,,,,74,80,,,,,,,56", ",,53,,,,37,83,63,64,65,8,51,,,,57,58,,,,61,,59,60,62,23,24,66,67,,,", ",,22,28,27,90,89,91,92,,,17,,,,,,7,41,,9,94,93,,84,50,86,85,87,,88,95", "96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,36,,,30,,,52,,,54,,32,,,,40,,,", ",,,,18,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37", "83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,", ",,,,41,,,94,93,,84,50,86,85,87,259,88,95,96,,81,82,,38,39,,,,,,,,,,", ",,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,,,,79,73,75,76,77", "78,,,,74,80,,,,,,,56,,,53,,,,37,83,63,64,65,8,51,,,,57,58,,,,61,,59", "60,62,23,24,66,67,,,,,,22,28,27,90,89,91,92,,,17,,,,,,7,41,,9,94,93", ",84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,36,,,278", ",,52,,,54,,32,,,,40,,,,,,,,18,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64", "65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292", "90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,259,88,95,96,,81", "82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,644,,,,40,,,,,,,", "216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83", ",61,,59,60,62,23,24,66,67,,,,,,22,28,27,90,89,91,92,,,17,,,,,,,41,,", "94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208", ",,212,,,52,,,54,,,,,,40,,,,,,,,18,,,,,79,73,75,76,77,78,,,,74,80,,,", "63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255", "288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96", ",81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,", ",216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37", "83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,", ",,,,289,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,,,,,,,,,,,,,,,,,,", ",,,,286,,,212,,,52,,,54,,,,,,,,,,,,,,,,,,,79,73,75,76,77,78,,,,74,80", ",,,511,,,56,,,53,,,,293,83,63,64,65,8,51,,,,57,58,,,,61,,59,60,62,23", "24,66,67,,,,,,22,28,27,90,89,91,92,,,17,,,,,,7,41,,9,94,93,,84,50,86", "85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,36,,,278,,,52,,,54", ",32,,,,40,,,,,,,,18,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51", ",53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90,89", "91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39", ",,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,,,,79,73", "75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62", "256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84", "50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,", "52,,,54,,,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65", "56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,28,27,90", "89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,259,88,95,96,,81,82", ",38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,254,,,,40,,,,,,,,216", ",,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61", ",59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,289", ",,94,93,,84,50,86,85,87,,88,95,96,,81,82,,,,,,,,,,,,,,,,,,,,,,,,286", ",,283,,,52,,,54,,,,,,,,,,,,,,,,,,,79,73,75,76,77,78,,,,74,80,,,,,,,56", ",,53,,,,293,83,63,64,65,8,51,,,,57,58,,,,61,,59,60,62,23,24,66,67,,", ",,,22,28,27,90,89,91,92,,,17,,,,,,7,41,,9,94,93,,84,50,86,85,87,,88", "95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,36,,,30,,,52,,,54,,32,,,,40", ",,,,,,,18,,,,,79,73,75,76,77,78,,,,74,80,,,,,,401,56,,,53,,,,37,83,63", "64,65,,51,,,,57,58,,,,61,,59,60,62,23,24,66,67,,,,,,22,28,27,90,89,91", "92,,,17,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,", ",,,,,,,,,,,,,,,,,208,,,212,,659,52,,,54,,,,252,,40,,,,,,,,18,,,,,79", "73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60", "62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93", ",84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212", ",,52,,,54,,,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64", "65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292", "90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,259,88,95,96,,81", "82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,252,,40,,,,,,,", "216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83", ",61,,59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,", ",41,,,94,93,,84,50,86,85,87,259,88,95,96,,81,82,,38,39,,,,,,,,,,,,,", ",,,,,,,208,,,212,,,52,,,54,,644,,252,,40,,,,,,,,216,,,,,79,73,75,76", "77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256", "257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,289,,,94,93,,84,50", "86,85,87,,88,95,96,,81,82,,,,,,,,,,,,,,,,,,,,,,,,286,,,283,,,52,,,54", ",,,,,,,,,,,,,,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,", "57,58,293,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92", ",,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,", ",,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,,,,79,73,75,76", "77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256", "257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50", "86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52", ",,54,,,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56", "51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90", "89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38", "39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,216,,,,,79", "73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60", "62,256,257,66,67,,,,,,255,28,27,90,89,91,92,,,217,,,,,,,41,,,94,93,", "84,50,86,85,87,259,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,", "212,,,52,,,54,,254,,,,40,,,,,,,,216,,,,,79,73,75,76,77,78,,,,74,80,", ",,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,256,257,66,67,,,,,,255", "288,292,90,89,91,92,,,217,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96", ",81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,", ",216,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37", "83,,61,,59,60,62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,", ",,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,", ",,,,,,,208,,,212,528,,52,,,54,,,,,,40,,,,,,,,216,,,,,79,73,75,76,77", "78,,,,74,80,,,,,,,56,,,53,,,,37,83,63,64,65,8,51,,,,57,58,,,,61,,59", "60,62,23,24,66,67,,,,,,22,28,27,90,89,91,92,,,17,,,,,,7,41,,9,94,93", ",84,50,86,85,87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,36,,,30", ",,52,,,54,,32,,,,40,,,,,,,,18,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64", "65,56,51,,53,,57,58,37,83,,61,,59,60,62,23,24,66,67,,,,,,22,28,27,90", "89,91,92,,,17,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38", "39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,18,,,,,79", "73,75,76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60", "62,256,257,66,67,,,,,,255,288,292,90,89,91,92,,,217,,,,,,,289,,,94,93", ",84,50,86,85,358,,88,95,96,,81,82,,,,,,,,,,,,,,,,,,,,,,,,359,,,212,", ",52,,,54,,,,,,,,,,,,,,,,,,,79,73,75,76,77,78,,,,74,80,,,,63,64,65,56", "51,,53,,57,58,293,83,,61,,59,60,62,23,24,66,67,,,,,,22,28,27,90,89,91", "92,,,17,,,,,,,41,,,94,93,,84,50,86,85,87,,88,95,96,,81,82,,38,39,,,", ",,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,,,,,,40,,,,,,,,18,,,,,79,73,75", "76,77,78,,,,74,80,,,,63,64,65,56,51,,53,,57,58,37,83,,61,,59,60,62,23", "24,66,67,,,,,,22,28,27,90,89,91,92,,,17,,,,,,,41,,,94,93,,84,50,86,85", "87,,88,95,96,,81,82,,38,39,,,,,,,,,,,,,,,,,,,,,208,,,212,,,52,,,54,", ",,,,40,,,,,,,,18,,,,-277,79,73,75,76,77,78,-277,-277,-277,74,80,,-277", "-277,,-277,,56,,,53,,,,37,83,,,,,,,,,,-277,-277,,-277,-277,-277,-277", "-277,,,,,,,,,,,,,,,,,,,,,,,,-277,-277,-277,-277,-277,-277,-277,-277", "-277,-277,-277,-277,-277,-277,,,-277,-277,-277,,628,,,,-277,,,,,,,-277", ",-277,,-277,-277,-277,-277,-277,-277,-277,,-277,,-277,,,,,,,,,,,,,-277", "-277,,-85,,-277,-277,,-277,,,-93,,-277,-277,-277,,,-277,-277,-277,,-277", ",,,,,,,,,-277,-277,,,,,,,,,-277,-277,,-277,-277,-277,-277,-277,,,,,", ",,,,,,,,,,,,,,,,,,-277,-277,-277,-277,-277,-277,-277,-277,-277,-277", "-277,-277,-277,-277,,,-277,-277,-277,,628,-277,,,-277,,,-277,,-277,", "-277,,-277,,-277,-277,-277,-277,-277,-277,-277,,-277,,-277,,,,,,,,,", ",,,-277,-277,-277,-277,,-277,-437,,-277,,,-93,,-437,-437,-437,,,-437", "-437,-437,,-437,,,,,,,,-437,,-437,-437,-437,,,,,,,,-437,-437,,-437,-437", "-437,-437,-437,,,,,,,,,,,,,,,,,,,,,,,,-437,-437,-437,-437,-437,-437", "-437,-437,-437,-437,-437,-437,-437,-437,,,-437,-437,-437,,-437,-437", ",,-437,,,-437,,-437,,-437,,-437,,-437,-437,-437,-437,-437,-437,-437", ",-437,,-437,,,,,,,,,,,,,-437,-437,-437,-437,,-437,-529,-437,-437,,,-437", ",-529,-529,-529,,,,-529,-529,,-529,,,,,,,,,-529,,,,,,,,,,,-529,-529", ",-529,-529,-529,-529,-529,,,,,,,,,,,,,,,,,,,,,,,,-529,-529,-529,-529", "-529,-529,-529,-529,-529,-529,-529,-529,-529,-529,,,-529,-529,-529,", "625,,,,-529,,,,,,,-529,,-529,,-529,-529,-529,-529,-529,-529,-529,,-529", "-529,-529,,,,,,,,,,,,,-529,-529,,-83,,-529,-529,,-529,,,-91,,-529,-529", "-529,,,-529,-529,-529,,-529,,,,,,,,,-529,-529,-529,,,,,,,,,-529,-529", ",-529,-529,-529,-529,-529,,,,,,,,,,,,,,,,,,,,,,,,-529,-529,-529,-529", "-529,-529,-529,-529,-529,-529,-529,-529,-529,-529,,,-529,-529,-529,", "769,-529,,,-529,,,-529,,-529,,-529,,-529,,-529,-529,-529,-529,-529,-529", "-529,,-529,-529,-529,,,,,,,,,,,,,-529,-529,-529,-529,,-529,-440,,-529", ",,-91,,-440,-440,-440,,,-440,-440,-440,,-440,,,,,,,,-440,,-440,-440", "-440,,,,,,,,-440,-440,,-440,-440,-440,-440,-440,,,,,,,,,,,,,,,,,,,,", ",,,-440,-440,-440,-440,-440,-440,-440,-440,-440,-440,-440,-440,-440", "-440,,,-440,-440,-440,,-440,-440,,,-440,,,-440,,-440,,-440,,-440,,-440", "-440,-440,-440,-440,-440,-440,,-440,,-440,,,,,,,,,,,,,-440,-440,-440", "-440,,-440,-373,-440,-440,,,-440,,-373,-373,-373,,,-373,-373,-373,,-373", ",,,,,,,,-373,-373,-373,,,,,,,,,-373,-373,,-373,-373,-373,-373,-373,", ",,,,,,,,,,,,,,,,,,,,,,-373,-373,-373,-373,-373,-373,-373,-373,-373,-373", "-373,-373,-373,-373,,,-373,-373,-373,,,-373,,265,-373,,,-373,,-373,", "-373,,-373,,-373,-373,-373,-373,-373,-373,-373,,-373,-373,-373,,,,,", ",,,,,,,-373,-373,-373,-373,-530,-373,,,-373,,,-530,-530,-530,,,-530", "-530,-530,,-530,,,,,,,,,-530,-530,-530,-530,,,,,,,,-530,-530,,-530,-530", "-530,-530,-530,,,,,,,,,,,,,,,,,,,,,,,,-530,-530,-530,-530,-530,-530", "-530,-530,-530,-530,-530,-530,-530,-530,,,-530,-530,-530,,,-530,,,-530", ",,-530,,-530,,-530,,-530,,-530,-530,-530,-530,-530,-530,-530,,-530,-530", "-530,,,,,,,,,,,,,-530,-530,-530,-530,-545,-530,,-530,-530,,,-545,-545", "-545,,,-545,-545,-545,,-545,,,,,,,,,-545,-545,-545,,,,,,,,,-545,-545", ",-545,-545,-545,-545,-545,,,,,,,,,,,,,,,,,,,,,,,,-545,-545,-545,-545", "-545,-545,-545,-545,-545,-545,-545,-545,-545,-545,,,-545,-545,-545,", ",-545,,265,-545,,,-545,,-545,,-545,,-545,,-545,-545,-545,-545,-545,-545", "-545,,-545,-545,-545,,,,,,,,,,,,,-545,-545,-545,-545,-531,-545,,,-545", ",,-531,-531,-531,,,-531,-531,-531,,-531,,,,,,,,,-531,-531,-531,-531", ",,,,,,,-531,-531,,-531,-531,-531,-531,-531,,,,,,,,,,,,,,,,,,,,,,,,-531", "-531,-531,-531,-531,-531,-531,-531,-531,-531,-531,-531,-531,-531,,,-531", "-531,-531,,,-531,,,-531,,,-531,,-531,,-531,,-531,,-531,-531,-531,-531", "-531,-531,-531,,-531,-531,-531,,,,,,,,,,,,,-531,-531,-531,-531,-285", "-531,,-531,-531,,,-285,-285,-285,,,-285,-285,-285,,-285,,,,,,,,,,-285", "-285,,,,,,,,,-285,-285,,-285,-285,-285,-285,-285,,,,,,,,,,,,,,,,,,,", ",,,,-285,-285,-285,-285,-285,-285,-285,-285,-285,-285,-285,-285,-285", "-285,,,-285,-285,-285,,,-285,,274,-285,,,-285,,-285,,-285,,-285,,-285", "-285,-285,-285,-285,-285,-285,,-285,,-285,,,,,,,,,,,,,-285,-285,-285", "-285,-272,-285,,,-285,,,-272,-272,-272,,,-272,-272,-272,,-272,,,,,,", ",,,-272,-272,-272,,,,,,,,-272,-272,,-272,-272,-272,-272,-272,,,,,,,", ",,,,,,,,,,,,,,,,-272,-272,-272,-272,-272,-272,-272,-272,-272,-272,-272", "-272,-272,-272,,,-272,-272,-272,,,-272,,,-272,,,-272,,-272,,-272,,-272", ",-272,-272,-272,-272,-272,-272,-272,,-272,,-272,,,,,,,,,,,,,-272,-272", "-272,-272,-545,-272,,-272,-272,,,-545,-545,-545,,,-545,-545,-545,,-545", ",,,,,,,,,-545,,,,,,,,,,-545,-545,,-545,-545,-545,-545,-545,,,,,,,,,", ",,,-545,,,,,,,-545,-545,-545,,,-545,-545,-545,,-545,,,,,,-545,,,,-545", ",,-545,,,,,265,-545,-545,-545,,-545,-545,-545,-545,-545,,,,,,,,,,,,", "-545,,,,,,,,,,,,,-545,,-545,,,-545,,,,-545,,,,,,,-545,,,,,265,-545,", ",,,,,,,,,,,,,,,,,,,-545,,,,,,,,,,,,,-545,,-545,,,-545,156,167,157,180", "153,173,163,162,188,191,178,161,160,155,181,189,190,165,154,168,172", "174,166,159,,,,175,182,177,176,169,179,164,152,171,170,183,184,185,186", "187,151,158,149,150,147,148,,111,113,,,112,,,,,,,,,142,143,,139,121", "122,123,130,127,129,,,124,125,,,,144,145,131,132,,,,,,,,,,,,,136,135", ",120,141,138,137,133,134,128,126,118,140,119,,,146,192,,,,,,,,,,80,156", "167,157,180,153,173,163,162,188,191,178,161,160,155,181,189,190,165", "154,168,172,174,166,159,,,,175,182,177,176,169,179,164,152,171,170,183", "184,185,186,187,151,158,149,150,147,148,,111,113,110,,112,,,,,,,,,142", "143,,139,121,122,123,130,127,129,,,124,125,,,,144,145,131,132,,,,,,", ",,,,,,136,135,,120,141,138,137,133,134,128,126,118,140,119,,,146,192", ",,,,,,,,,80,156,167,157,180,153,173,163,162,188,191,178,161,160,155", "181,189,190,165,154,168,172,174,166,159,,,,175,182,177,176,169,179,164", "152,171,170,183,184,185,186,187,151,158,149,150,147,148,,111,113,,,112", ",,,,,,,,142,143,,139,121,122,123,130,127,129,,,124,125,,,,144,145,131", "132,,,,,,,,,,,,,136,135,,120,141,138,137,133,134,128,126,118,140,119", ",,146,192,,,,,,,,,,80,156,167,157,180,153,173,163,162,188,191,178,161", "160,155,181,189,190,165,154,168,172,174,166,159,,,,175,182,177,176,169", "179,164,152,171,170,183,184,185,186,187,151,158,149,150,147,148,,111", "113,,,112,,,,,,,,,142,143,,139,121,122,123,130,127,129,,,124,125,,,", "144,145,131,132,,,,,,,,,,,,,136,135,,120,141,138,137,133,134,128,126", "118,140,119,,,146,192,,,,,,,,,,80,156,167,157,180,153,173,163,162,188", "191,178,161,160,155,181,189,190,165,154,168,172,174,166,159,,,,175,182", "177,176,169,179,164,152,171,170,183,184,185,186,187,151,158,149,150", "147,148,,111,113,,,112,,,,,,,,,142,143,,139,121,122,123,130,127,129", ",,124,125,,,,144,145,131,132,,,,,,,,,,,,,136,135,,120,141,138,137,133", "134,128,126,118,140,119,,,146,156,167,157,180,153,173,163,162,188,191", "178,161,160,155,181,189,190,165,154,168,172,174,166,159,,,,175,182,177", "176,169,179,164,152,171,170,183,184,185,186,187,151,158,149,150,147", "148,,111,113,395,394,112,,396,,,,,,,142,143,,139,121,122,123,130,127", "129,,,124,125,,,,144,145,131,132,,,,,,,,,,,,,136,135,,120,141,138,137", "133,134,128,126,118,140,119,,,146,156,167,157,180,153,173,163,162,188", "191,178,161,160,155,181,189,190,165,154,168,172,174,166,159,,,,175,182", "177,176,169,179,164,152,171,170,183,184,185,186,187,151,158,149,150", "147,148,,111,113,395,394,112,,396,,,,,,,142,143,,139,121,122,123,130", "127,129,,,124,125,,,,144,145,131,132,,,,,,,,,,,,,136,135,,120,141,138", "137,133,134,128,126,118,140,119,,,146,156,167,157,180,153,173,163,162", "188,191,178,161,160,155,181,189,190,165,154,168,172,174,166,159,,,,175", "182,177,373,372,374,371,152,171,170,183,184,185,186,187,151,158,149", "150,369,370,,367,113,86,85,368,,88,,,,,,,142,143,,139,121,122,123,130", "127,129,,,124,125,,,,144,145,131,132,,,,,,378,,,,,,,136,135,,120,141", "138,137,133,134,128,126,118,140,119,631,436,146,,632,,,,,,,,,142,143", ",139,121,122,123,130,127,129,,,124,125,,,,144,145,131,132,,,,,,,,,,", ",,136,135,,120,141,138,137,133,134,128,126,118,140,119,487,430,146,", "488,,,,,,,,,142,143,,139,121,122,123,130,127,129,,,124,125,,,,144,145", "131,132,,,,,,265,,,,,,,136,135,,120,141,138,137,133,134,128,126,118", "140,119,722,436,146,,720,,,,,,,,,142,143,,139,121,122,123,130,127,129", ",,124,125,,,,144,145,131,132,,,,,,,,,,,,,136,135,,120,141,138,137,133", "134,128,126,118,140,119,629,430,146,,630,,,,,,,,,142,143,,139,121,122", "123,130,127,129,,,124,125,,,,144,145,131,132,,,,,,265,,,,,,,136,135", ",120,141,138,137,133,134,128,126,118,140,119,717,436,146,,718,,,,,,", ",,142,143,,139,121,122,123,130,127,129,,,124,125,,,,144,145,131,132", ",,,,,,,,,,,,136,135,,120,141,138,137,133,134,128,126,118,140,119,715", "430,146,,716,,,,,,,,,142,143,,139,121,122,123,130,127,129,,,124,125", ",,,144,145,131,132,,,,,,265,,,,,,,136,135,,120,141,138,137,133,134,128", "126,118,140,119,932,430,146,,933,,,,,,,,,142,143,,139,121,122,123,130", "127,129,,,124,125,,,,144,145,131,132,,,,,,265,,,,,,,136,135,,120,141", "138,137,133,134,128,126,118,140,119,934,436,146,,935,,,,,,,,,142,143", ",139,121,122,123,130,127,129,,,124,125,,,,144,145,131,132,,,,,,,,,,", ",,136,135,,120,141,138,137,133,134,128,126,118,140,119,487,430,146,", "488,,,,,,,,,142,143,,139,121,122,123,130,127,129,,,124,125,,,,144,145", "131,132,,,,,,,,,,,,,136,135,,120,141,138,137,133,134,128,126,118,140", "119,487,430,146,,488,,,,,,,,,142,143,,139,121,122,123,130,127,129,,", "124,125,,,,144,145,131,132,,,,,,,,,,,,,136,135,,120,141,138,137,133", "134,128,126,118,140,119,631,436,146,,632,,,,,,,,,142,143,,139,121,122", "123,130,127,129,,,124,125,,,,144,145,131,132,,,,,,,,,,,,,136,135,,120", "141,138,137,133,134,128,126,118,140,119,432,436,146,,434,,,,,,,,,142", "143,,139,121,122,123,130,127,129,,,124,125,,,,144,145,131,132,,,,,,", ",,,,,,136,135,,120,141,138,137,133,134,128,126,118,140,119,673,436,146", ",674,,,,,,,,,142,143,,139,121,122,123,130,127,129,,,124,125,,,,144,145", "131,132,,,,,,,,,,,,,136,135,,120,141,138,137,133,134,128,126,118,140", "119,670,430,146,,671,,,,,,,,,142,143,,139,121,122,123,130,127,129,,", "124,125,,,,144,145,131,132,,,,,,265,,,,,,,136,135,,120,141,138,137,133", "134,128,126,118,140,119,426,430,146,,427,,,,,,,,,142,143,,139,121,122", "123,130,127,129,,,124,125,,,,144,145,131,132,,,,,,265,,,,,,,136,135", ",120,141,138,137,133,134,128,126,118,140,119,722,436,146,,865,,,,,,", ",,142,143,,139,121,122,123,130,127,129,,,124,125,,,,144,145,131,132", ",,,,,,,,,,,,136,135,,120,141,138,137,133,134,128,126,118,140,119,629", "430,146,,630,,,,,,,,,142,143,,139,121,122,123,130,127,129,,,124,125", ",,,144,145,131,132,,,,,,265,,,,,,,136,135,,120,141,138,137,133,134,128", "126,118,140,119,,,146"];

      racc_action_table = arr = Opal.get('Array').$new(23792, nil);

      idx = 0;

      ($a = ($b = clist).$each, $a.$$p = (TMP_1 = function(str){var self = TMP_1.$$s || this, $a, $b, TMP_2;
if (str == null) str = nil;
      return ($a = ($b = str.$split(",", -1)).$each, $a.$$p = (TMP_2 = function(i){var self = TMP_2.$$s || this, $a;
if (i == null) i = nil;
        if ((($a = i['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            } else {
            arr['$[]='](idx, i.$to_i())
          };
          return idx = $rb_plus(idx, 1);}, TMP_2.$$s = self, TMP_2), $a).call($b)}, TMP_1.$$s = self, TMP_1), $a).call($b);

      clist = ["0,0,0,0,0,345,816,673,0,0,55,727,727,0,71,0,0,0,0,0,0,0,71,358,669,477", "629,0,0,0,0,0,0,0,318,389,0,349,26,349,630,672,0,0,0,0,0,0,344,0,0,0", "0,0,665,0,0,0,477,0,0,577,0,0,351,685,363,907,685,907,354,594,594,673", "673,629,358,3,862,869,715,667,3,358,0,932,934,0,358,630,0,665,660,0", "933,0,55,389,716,0,727,718,815,26,675,727,657,0,97,358,816,673,0,0,0", "0,0,0,345,673,345,0,0,345,26,817,918,918,918,0,918,358,0,678,918,918", "0,0,318,918,717,918,918,918,918,918,918,918,307,307,568,831,831,918", "918,918,918,918,918,918,594,344,918,344,934,577,344,718,351,918,569", "363,918,918,354,918,918,918,918,918,918,918,918,918,715,918,918,427", "918,918,363,862,869,862,869,363,862,869,932,934,932,934,716,932,934", "718,717,933,679,933,918,445,933,918,653,815,918,815,815,918,815,918", "290,935,817,918,817,369,427,817,935,670,671,918,369,935,427,307,918", "918,918,918,918,918,717,367,836,918,918,568,681,568,367,501,568,918", "770,836,918,828,445,78,918,918,916,916,916,916,916,569,828,569,916,916", "569,680,680,916,935,916,916,916,916,916,916,916,501,290,193,836,836", "916,916,916,916,916,916,916,670,671,916,605,828,828,382,770,916,916", "290,916,916,916,386,916,916,916,916,916,207,916,916,916,380,916,916", "836,916,916,110,686,680,687,516,110,110,670,671,391,828,516,449,449", "834,670,671,209,690,834,916,373,692,916,384,921,916,384,373,916,834", "916,921,605,605,916,382,382,382,207,293,293,605,916,386,386,386,297", "916,916,916,916,916,916,380,380,380,916,916,516,834,834,834,834,500", "916,209,449,916,391,391,391,916,916,908,908,908,908,908,490,921,731", "908,908,384,384,731,908,297,908,908,908,908,908,908,908,297,500,834", "922,922,908,908,908,908,908,908,908,725,348,908,900,490,725,348,416", "908,908,900,908,908,908,35,908,908,908,908,908,693,908,908,908,643,908", "908,693,908,908,694,416,416,416,416,416,416,416,416,416,416,416,674", "416,416,720,275,416,416,674,908,275,720,908,674,35,908,720,900,908,426", "908,416,35,416,908,416,416,416,416,416,416,416,908,416,693,695,14,908", "908,908,908,908,908,361,696,897,908,908,13,416,361,416,897,587,908,13", "674,908,587,720,426,908,908,903,903,903,903,903,426,749,749,903,903", "749,749,749,903,14,903,903,903,903,903,903,903,14,311,699,618,311,903", "903,903,903,903,903,903,898,361,903,897,552,552,13,898,903,903,641,903", "903,903,633,903,903,903,903,903,300,903,903,903,704,903,903,300,903", "903,352,206,618,454,284,372,453,352,206,287,618,284,372,710,912,285", "287,371,712,912,903,370,285,903,371,898,903,77,370,903,912,903,714,785", "454,903,626,453,454,454,785,453,453,903,621,300,208,376,903,903,903", "903,903,903,376,352,206,903,903,284,912,912,912,912,287,903,314,210", "903,314,285,616,903,903,896,896,896,896,896,374,211,310,896,896,310", "722,374,896,785,896,896,896,896,896,896,896,343,343,912,83,83,896,896", "896,896,896,896,896,702,702,896,579,37,37,723,634,896,896,579,896,896", "896,609,896,896,896,896,896,940,896,896,896,217,896,896,940,896,896", "249,634,634,634,634,634,634,634,634,634,634,634,42,634,634,25,250,634", "634,42,896,253,25,896,15,15,896,571,579,896,570,896,634,734,634,896", "634,634,634,634,634,634,634,896,634,940,564,738,896,896,896,896,896", "896,801,739,741,896,896,742,634,743,894,894,894,896,894,42,896,368,894", "894,896,896,745,894,368,894,894,894,894,894,894,894,664,664,664,664", "664,894,894,894,894,894,894,894,559,748,894,555,801,801,801,801,264", "894,544,755,894,894,41,894,894,894,894,894,277,894,894,894,279,894,894", "531,894,894,530,656,656,656,656,656,656,656,656,656,656,656,529,656", "656,36,280,656,656,34,894,281,520,894,773,774,894,777,779,894,517,894", "656,894,656,894,656,656,656,656,656,656,656,894,656,782,783,784,894", "894,894,894,894,894,286,788,512,894,894,791,656,792,656,510,507,894", "546,506,894,807,288,810,894,894,877,877,877,877,877,502,20,289,877,877", "496,495,819,877,822,877,877,877,877,877,877,877,276,276,276,276,276", "877,877,877,877,877,877,877,823,824,877,492,546,546,546,546,877,877", "292,877,877,877,458,877,877,877,877,877,457,877,877,877,456,877,877", "455,877,877,846,476,476,476,476,476,476,476,476,476,476,476,847,476", "476,298,299,476,476,433,877,855,425,877,421,12,877,865,866,877,419,877", "476,871,476,877,476,476,476,476,476,476,476,877,476,872,11,10,877,877", "877,877,877,877,9,880,882,877,877,885,476,476,886,887,888,877,302,413", "877,407,388,385,877,877,874,874,874,874,874,899,309,6,874,874,359,355", "909,874,353,874,874,874,874,874,874,874,5,5,5,5,5,874,874,874,874,874", "874,874,350,560,874,560,560,560,917,560,874,874,1,874,874,874,342,874", "874,874,874,874,339,874,874,874,327,874,874,326,874,874,926,766,766", "766,766,766,766,766,766,766,766,766,928,766,766,929,931,766,766,324", "874,323,315,874,313,312,874,,,874,,874,766,,766,874,766,766,766,766", "766,766,766,874,766,,,,874,874,874,874,874,874,,,,874,874,,766,,873", "873,873,874,873,,874,,873,873,874,874,,873,,873,873,873,873,873,873", "873,,,,,,873,873,873,873,873,873,873,,806,873,806,806,806,,806,,873", ",,873,873,,873,873,873,873,873,,873,873,873,,873,873,,873,873,,849,849", "849,849,849,849,849,849,849,849,849,,849,849,,,849,849,,873,,,873,,", "873,,,873,,873,849,,849,873,849,849,849,849,849,849,849,873,849,,,,873", "873,873,873,873,873,,,,873,873,,849,,863,863,863,873,863,,873,,863,863", "873,873,,863,,863,863,863,863,863,863,863,,,,,,863,863,863,863,863,863", "863,,557,863,557,557,557,,557,,863,,,863,863,,863,863,863,863,863,,863", "863,863,,863,863,891,,891,891,891,891,891,,,,,,557,,,,891,,,,557,,,863", ",,863,472,,863,,,863,,,,,,891,891,,,,,,,891,891,891,891,,863,863,863", "863,863,863,,472,,863,863,472,472,472,472,,,863,,,863,,,,863,863,859", "859,859,859,859,,,891,859,859,,,,859,,859,859,859,859,859,859,859,,", ",,,859,859,859,859,859,859,859,,,859,,,,,,859,859,,859,859,859,,859", "859,859,859,859,,859,859,859,,859,859,,859,859,,439,439,439,439,439", "439,439,439,439,439,439,,439,439,,,439,439,,859,,,859,,,859,,,859,,859", "439,,439,859,439,439,439,439,439,439,439,859,439,,,,859,859,859,859", "859,859,,,,859,859,,439,,854,854,854,859,854,,859,,854,854,859,859,", "854,,854,854,854,854,854,854,854,,,,,,854,854,854,854,854,854,854,,", "854,,,,,,,854,,,854,854,,854,854,854,854,854,854,854,854,854,,854,854", ",854,854,,247,247,247,247,247,247,247,247,247,247,247,,247,247,,,247", "247,,854,,,854,,,854,,,854,,,247,854,247,854,247,247,247,247,247,247", "247,854,247,,,,854,854,854,854,854,854,,,,854,854,,247,,850,850,850", "854,850,,854,,850,850,854,854,,850,,850,850,850,850,850,850,850,,,,", ",850,850,850,850,850,850,850,,,850,,,,,,,850,,,850,850,,850,850,850", "850,850,,850,850,850,,850,850,,850,850,,527,527,527,527,527,527,527", "527,527,527,527,,527,527,,,527,527,,850,,,850,,,850,,,850,,,527,,527", "850,527,527,527,527,527,527,527,850,527,,,,850,850,850,850,850,850,", ",,850,850,,527,,17,17,17,850,17,,850,,17,17,850,850,,17,,17,17,17,17", "17,17,17,,,,,,17,17,17,17,17,17,17,,,17,,,,,,,17,,,17,17,,17,17,17,17", "17,,17,17,17,,17,17,,17,17,,424,424,424,424,424,424,424,424,424,424", "424,,424,424,,,424,424,,17,,,17,17,,17,,,17,,,424,,424,17,424,424,424", "424,424,424,424,17,424,,,,17,17,17,17,17,17,,,,17,17,,424,,18,18,18", "17,18,,17,,18,18,17,17,,18,,18,18,18,18,18,18,18,,,,,,18,18,18,18,18", "18,18,,,18,,,,,,,18,,,18,18,,18,18,18,18,18,,18,18,18,,18,18,,18,18", ",764,764,764,764,764,764,764,764,764,764,764,,764,764,,,764,764,,18", ",,18,,,18,,,18,,,764,,764,18,764,764,764,764,764,764,764,18,764,,,,18", "18,18,18,18,18,,,,18,18,,764,,838,838,838,18,838,,18,,838,838,18,18", ",838,,838,838,838,838,838,838,838,,,,,,838,838,838,838,838,838,838,", ",838,,,,,,,838,,,838,838,,838,838,838,838,838,,838,838,838,,838,838", ",838,838,,761,761,761,761,761,761,761,761,761,761,761,,761,761,,,761", "761,,838,,,838,,,838,,,838,,,761,,761,838,761,761,761,761,761,761,761", "838,761,,,,838,838,838,838,838,838,,,,838,838,,761,,,,,838,,,838,,,", "838,838,812,812,812,812,812,,,,812,812,,,,812,,812,812,812,812,812,812", "812,,,,,,812,812,812,812,812,812,812,,,812,,,,,,812,812,,812,812,812", ",812,812,812,812,812,,812,812,812,,812,812,,812,812,,677,677,677,677", "677,677,677,677,677,677,677,,677,677,,,677,677,,812,,,812,,,812,,,812", ",812,677,,677,812,677,677,677,677,677,677,677,812,677,,,,812,812,812", "812,812,812,,,,812,812,,677,,22,22,22,812,22,,812,,22,22,812,812,,22", ",22,22,22,22,22,22,22,,,,,,22,22,22,22,22,22,22,,,22,,,,,,,22,,,22,22", ",22,22,22,22,22,22,22,22,22,,22,22,,22,22,,19,19,19,19,19,19,19,19,19", "19,19,,19,19,,,19,19,,22,,,22,,,22,,,22,,22,19,22,19,22,19,19,19,19", "19,19,19,22,19,,,,22,22,22,22,22,22,,,,22,22,,19,,23,23,23,22,23,,22", ",23,23,22,22,,23,,23,23,23,23,23,23,23,,,,,,23,23,23,23,23,23,23,,,23", ",,,,,,23,,,23,23,,23,23,23,23,23,23,23,23,23,,23,23,,23,23,,754,754", "754,754,754,754,754,754,754,754,754,,754,754,,,754,754,,23,,,23,,,23", ",,23,,23,754,23,754,23,754,754,754,754,754,754,754,23,754,,,,23,23,23", "23,23,23,,,,23,23,,754,,24,24,24,23,24,,23,,24,24,23,23,,24,,24,24,24", "24,24,24,24,,,,,,24,24,24,24,24,24,24,,,24,,,,,,,24,,,24,24,,24,24,24", "24,24,24,24,24,24,,24,24,,24,24,,759,759,759,759,759,759,759,759,759", "759,759,,759,759,,,759,759,,24,,,24,,,24,,,24,,24,759,24,759,24,759", "759,759,759,759,759,759,24,759,,,,24,24,24,24,24,24,,,,24,24,,759,,", ",,24,,,24,,,,24,24,811,811,811,811,811,,,,811,811,,,,811,,811,811,811", "811,811,811,811,,,,,,811,811,811,811,811,811,811,,,811,,,,,,811,811", ",811,811,811,,811,811,811,811,811,,811,811,811,,811,811,,811,811,,771", "771,771,771,771,771,771,771,771,771,771,,771,771,,,771,771,,811,,,811", ",,811,,,811,,811,771,,771,811,771,771,771,771,771,771,771,811,771,,", ",811,811,811,811,811,811,,,,811,811,,771,,809,809,809,811,809,,811,", "809,809,811,811,,809,,809,809,809,809,809,809,809,,,,,,809,809,809,809", "809,809,809,,335,809,335,335,335,473,335,,809,,,809,809,,809,809,809", "809,809,,809,809,809,,809,809,,809,809,700,,700,700,700,,700,473,,335", ",473,473,473,473,,,335,,,809,,,809,461,,809,,,809,,,,,,809,,,700,,461", "461,,809,,,700,,809,809,809,809,809,809,,461,,809,809,461,461,461,461", ",,809,,,809,,,,809,809,790,790,790,790,790,,,,790,790,,,,790,,790,790", "790,790,790,790,790,,,,,,790,790,790,790,790,790,790,,,790,,,,,,790", "790,,790,790,790,,790,790,790,790,790,,790,790,790,,790,790,,790,790", ",,,,,,,,,,,,,,,,,,,,790,,,790,,,790,,,790,,790,,,,790,,,550,,550,550", "550,790,550,,,,790,790,790,790,790,790,,,,790,790,,,,780,780,780,790", "780,,790,,780,780,790,790,,780,550,780,780,780,780,780,780,780,550,550", "550,550,,780,780,780,780,780,780,780,,,780,,,,,,,780,,,780,780,,780", "780,780,780,780,,780,780,780,,780,780,,780,780,,451,451,451,451,451", "451,451,451,451,451,451,,451,451,,,451,451,,780,,,780,,,780,,,780,,780", "451,,451,780,451,451,451,451,451,451,451,780,451,,,,780,780,780,780", "780,780,,,,780,780,,,,,,,780,,,780,,,,780,780,30,30,30,30,30,,,,30,30", ",,,30,,30,30,30,30,30,30,30,,,,,,30,30,30,30,30,30,30,,,30,,,,,,30,30", ",30,30,30,,30,30,30,30,30,,30,30,30,,30,30,,30,30,,464,,,,,,,,,,,,,", ",,464,464,,30,,,30,,,30,,,30,,30,464,,464,30,464,464,464,464,,,464,30", "464,,,,30,30,30,30,30,30,,,,30,30,,,,31,31,31,30,31,,30,,31,31,30,30", ",31,,31,31,31,31,31,31,31,,,,,,31,31,31,31,31,31,31,,,31,,,,,,,31,,", "31,31,,31,31,31,31,31,,31,31,31,,31,31,638,,638,638,638,,638,,,,,,,", ",,,,,,,,,31,,,31,,,31,,,31,,31,,,,638,,,,,,,,638,638,638,638,,31,31", "31,31,31,31,638,,,31,31,,,,32,32,32,31,32,,31,,32,32,31,31,,32,,32,32", "32,32,32,32,32,,,,,,32,32,32,32,32,32,32,,,32,,,,,,,32,,,32,32,,32,32", "32,32,32,,32,32,32,,32,32,832,,832,832,832,832,832,,,,,,,,,,832,,,,", ",,32,,,32,,,32,,,32,,,,,,832,,,,,,,,832,832,832,832,,32,32,32,32,32", "32,,,,32,32,,,,32,,,32,,,32,,,,32,32,769,769,769,,769,,,832,769,769", ",,,769,,769,769,769,769,769,769,769,,,,,,769,769,769,769,769,769,769", ",,769,,,,,,,769,,,769,769,,769,769,769,769,769,,769,769,769,,769,769", ",769,769,,474,474,474,474,474,474,474,,,474,474,,,,,,474,474,,769,,", "769,,,769,,,769,,,474,,474,769,474,474,474,474,474,474,474,769,474,", ",,769,769,769,769,769,769,,,,769,769,,,,768,768,768,769,768,,769,,768", "768,769,769,,768,,768,768,768,768,768,768,768,,,,,,768,768,768,768,768", "768,768,,,768,,,,,,,768,,,768,768,,768,768,768,768,768,,768,768,768", ",768,768,,768,768,,471,471,471,471,471,471,471,,,471,471,,,,,,471,471", ",768,,,768,,,768,,,768,,,471,,471,768,471,471,471,471,471,471,471,768", "471,,,,768,768,768,768,768,768,,,,768,768,,,,767,767,767,768,767,,768", ",767,767,768,768,,767,,767,767,767,767,767,767,767,,,,,,767,767,767", "767,767,767,767,,,767,,,,,,,767,,,767,767,,767,767,767,767,767,,767", "767,767,,767,767,,767,767,,475,475,475,475,475,475,475,475,,475,475", ",,,,,475,475,,767,,,767,,,767,,,767,,,475,,475,767,475,475,475,475,475", "475,475,767,475,,,,767,767,767,767,767,767,,,,767,767,,,,757,757,757", "767,757,,767,,757,757,767,767,,757,,757,757,757,757,757,757,757,,,,", ",757,757,757,757,757,757,757,,,757,,,,,,,757,,,757,757,,757,757,757", "757,757,,757,757,757,,757,757,,757,757,,463,,,,,,,,,,,,,,,,463,463,", "757,,,757,,,757,,,757,,,463,,463,757,463,463,463,463,,,463,757,463,", ",,757,757,757,757,757,757,,,,757,757,,,,38,38,38,757,38,,757,,38,38", "757,757,,38,,38,38,38,38,38,38,38,,,,,,38,38,38,38,38,38,38,,,38,,,", ",,,38,,,38,38,,38,38,38,38,38,,38,38,38,,38,38,,38,38,,470,470,470,470", "470,470,470,,,470,470,,,,,,470,470,,38,,,38,,,38,,,38,,,470,,470,38", "470,470,470,470,470,470,470,38,470,,,,38,38,38,38,38,38,,,,38,38,,,", "39,39,39,38,39,,38,,39,39,38,38,,39,,39,39,39,39,39,39,39,,,,,,39,39", "39,39,39,39,39,,,39,,,,,,,39,,,39,39,,39,39,39,39,39,,39,39,39,,39,39", ",39,39,,452,452,452,452,452,452,452,452,452,452,452,,452,452,,,452,452", ",39,,,39,,,39,,,39,,,452,,452,39,452,452,452,452,452,452,452,39,452", ",,,39,39,39,39,39,39,,,,39,39,,,,40,40,40,39,40,,39,,40,40,39,39,,40", ",40,40,40,40,40,40,40,,,,,,40,40,40,40,40,40,40,,,40,,,,,,,40,,,40,40", ",40,40,40,40,40,,40,40,40,,40,40,,40,40,,468,468,468,468,468,468,468", ",,468,468,,,,,,468,468,,40,,,40,,,40,,,40,,,468,,468,40,468,468,468", "468,468,468,468,40,468,,,,40,40,40,40,40,40,,,,40,40,,,,756,756,756", "40,756,,40,,756,756,40,40,,756,,756,756,756,756,756,756,756,,,,,,756", "756,756,756,756,756,756,,,756,,,,,,,756,,,756,756,,756,756,756,756,756", ",756,756,756,,756,756,,756,756,,467,467,467,467,467,467,467,,,467,467", ",,,,,467,467,,756,,,756,,,756,,,756,,,467,,467,756,467,467,467,467,467", "467,467,756,467,,,,756,756,756,756,756,756,,,,756,756,,,,,,,756,,,756", ",,,756,756,750,750,750,750,750,,,,750,750,,,,750,,750,750,750,750,750", "750,750,,,,,,750,750,750,750,750,750,750,,,750,,,,,,750,750,,750,750", "750,,750,750,750,750,750,,750,750,750,,750,750,,750,750,,466,,,,,,,", ",,,,,,,,466,466,,750,,,750,,,750,,,750,,750,466,,466,750,466,466,466", "466,,,466,750,466,,,,750,750,750,750,750,750,,,,750,750,,,,736,736,736", "750,736,,750,,736,736,750,750,,736,,736,736,736,736,736,736,736,,,,", ",736,736,736,736,736,736,736,,,736,,,,,,,736,,,736,736,,736,736,736", "736,736,,736,736,736,,736,736,,736,736,,462,462,462,462,462,462,462", ",,462,462,,,,,,462,462,,736,,,736,,,736,,,736,,,462,,462,736,462,462", "462,462,462,462,462,736,462,,,,736,736,736,736,736,736,,,,736,736,,", ",52,52,52,736,52,,736,,52,52,736,736,,52,,52,52,52,52,52,52,52,,,,,", "52,52,52,52,52,52,52,,,52,,,,,,,52,,,52,52,,52,52,52,52,52,,52,52,52", ",52,52,,52,52,,465,,,,,,,,,,,,,,,,465,465,,52,,,52,,,52,,,52,,,465,", "465,52,465,465,465,465,,,465,52,465,,,,52,52,52,52,52,52,,,,52,52,,", ",53,53,53,52,53,,52,,53,53,52,52,,53,,53,53,53,53,53,53,53,,,,,,53,53", "53,53,53,53,53,,,53,,,,,,,53,,,53,53,,53,53,53,53,53,53,53,53,53,,53", "53,,53,53,,469,469,469,469,469,469,469,,,469,469,,,,,,469,469,,53,,", "53,,,53,,,53,,53,469,,469,53,469,469,469,469,469,469,469,53,469,,,,53", "53,53,53,53,53,,,,53,53,,,,54,54,54,53,54,,53,,54,54,53,53,,54,,54,54", "54,54,54,54,54,,,,,,54,54,54,54,54,54,54,,,54,,,,,,,54,,,54,54,,54,54", "54,54,54,54,54,54,54,,54,54,,54,54,,459,,,,,,,,,,,,,,,,459,459,,54,", ",54,,,54,,,54,,,459,,459,54,459,459,459,459,,,,54,,,,,54,54,54,54,54", "54,,,,54,54,,,,,,,54,,,54,,,,54,54,729,729,729,729,729,,,,729,729,,", ",729,,729,729,729,729,729,729,729,,,,,,729,729,729,729,729,729,729,", ",729,,,,,,729,729,,729,729,729,,729,729,729,729,729,,729,729,729,,729", "729,,729,729,,460,,,,,,,,,,,,,,,,460,460,,729,,,729,,,729,,,729,,729", "460,,460,729,460,460,460,460,,,,729,,,,,729,729,729,729,729,729,,,,729", "729,,,,,,,729,,,729,,,,729,729,728,728,728,728,728,,,,728,728,,,,728", ",728,728,728,728,728,728,728,,,,,,728,728,728,728,728,728,728,,,728", ",,,,,728,728,,728,728,728,,728,728,728,728,728,,728,728,728,,728,728", ",728,728,,,,,,,,,,,,,,,,,,,,,728,,,728,,,728,,,728,,728,,,,728,,,,,", ",,728,,,,,728,728,728,728,728,728,,,,728,728,,,,57,57,57,728,57,,728", ",57,57,728,728,,57,,57,57,57,57,57,57,57,,,,,,57,57,57,57,57,57,57,", ",57,,,,,,,57,,,57,57,,57,57,57,57,57,,57,57,57,,57,57,,57,57,,,,,,,", ",,,,,,,,,,,,,57,,,57,,,57,,,57,,,,,,57,,,,,,,,57,,,,,57,57,57,57,57", "57,,,,57,57,,,,58,58,58,57,58,,57,,58,58,57,57,,58,,58,58,58,58,58,58", "58,,,,,,58,58,58,58,58,58,58,,,58,,,,,,,58,,,58,58,,58,58,58,58,58,", "58,58,58,,58,58,,58,58,,,,,,,,,,,,,,,,,,,,,58,,,58,,,58,,,58,,,,,,58", ",,,,,,,58,,,,,58,58,58,58,58,58,,,,58,58,,,,61,61,61,58,61,,58,,61,61", "58,58,,61,,61,61,61,61,61,61,61,,,,,,61,61,61,61,61,61,61,,,61,,,,,", ",61,,,61,61,,61,61,61,61,61,,61,61,61,,61,61,,61,61,,,,,,,,,,,,,,,,", ",,,,61,,,61,,,61,,,61,,,,,,61,,,,,,,,61,,,,,61,61,61,61,61,61,,,,61", "61,61,,,,,61,61,,,61,,,,61,61,62,62,62,,62,,,,62,62,,,,62,,62,62,62", "62,62,62,62,,,,,,62,62,62,62,62,62,62,,,62,,,,,,,62,,,62,62,,62,62,62", "62,62,,62,62,62,,62,62,56,,56,56,56,,56,,,,,,,,,,,,,,,,,62,,,62,,,62", ",,62,,62,,,,56,56,,,,,,,56,56,56,56,,62,62,62,62,62,62,,,,62,62,,,,63", "63,63,62,63,,62,,63,63,62,62,,63,,63,63,63,63,63,63,63,,,,,,63,63,63", "63,63,63,63,,,63,,,,,,,63,,,63,63,,63,63,63,63,63,,63,63,63,,63,63,730", ",730,730,730,730,730,,,,,,,,,,730,,,,63,,,63,,,63,,,63,,,63,,,,,,730", ",,,,,,,730,730,730,730,,63,63,63,63,63,63,,,,63,63,,,,,,,63,,,63,,,", "63,63,724,724,724,724,724,,,730,724,724,,,,724,,724,724,724,724,724", "724,724,,,,,,724,724,724,724,724,724,724,,,724,,,,,,724,724,,724,724", "724,,724,724,724,724,724,,724,724,724,,724,724,,724,724,,,,,,,,,,,,", ",,,,,,,,724,,,724,,,724,,,724,,724,,,,724,,,,,,,,724,,,,,724,724,724", "724,724,724,,,,724,724,,,,719,719,719,724,719,,724,,719,719,724,724", ",719,,719,719,719,719,719,719,719,,,,,,719,719,719,719,719,719,719,", ",719,,,,,,,719,,,719,719,,719,719,719,719,719,,719,719,719,,719,719", ",719,719,,,,,,,,,,,,,,,,,,,,,719,,,719,,,719,,,719,,,,,,719,,,,,,,,719", ",,,,719,719,719,719,719,719,,,,719,719,,,,713,713,713,719,713,,719,", "713,713,719,719,,713,,713,713,713,713,713,713,713,,,,,,713,713,713,713", "713,713,713,,,713,,,,,,,713,,,713,713,,713,713,713,713,713,,713,713", "713,,713,713,,713,713,,,,,,,,,,,,,,,,,,,,,713,,,713,,,713,,,713,,713", ",,,713,,,,,,,,713,,,,,713,713,713,713,713,713,,,,713,713,,,,682,682", "682,713,682,,713,,682,682,713,713,,682,,682,682,682,682,682,682,682", ",,,,,682,682,682,682,682,682,682,,,682,,,,,,,682,,,682,682,,682,682", "682,682,682,,682,682,682,,682,682,,682,682,,,,,,,,,,,,,,,,,,,,,682,", ",682,,,682,,,682,,682,,,,682,,,,,,,,682,,,,,682,682,682,682,682,682", ",,,682,682,,,,676,676,676,682,676,,682,,676,676,682,682,,676,,676,676", "676,676,676,676,676,,,,,,676,676,676,676,676,676,676,,,676,,,,,,,676", ",,676,676,,676,676,676,676,676,,676,676,676,,676,676,319,,319,319,319", ",319,,,,,,,,,,,,,,,,,676,,,676,,,676,,,676,,,,,,319,,319,,,,,,319,319", "319,319,,676,676,676,676,676,676,,,,676,676,,,,666,666,666,676,666,", "676,,666,666,676,676,,666,,666,666,666,666,666,666,666,,,,,,666,666", "666,666,666,666,666,,,666,,,,,,,666,,,666,666,,666,666,666,666,666,", "666,666,666,,666,666,590,,590,590,590,590,590,,,,,,,,,,590,,,,,,,666", ",,666,,,666,,,666,,,,,,590,590,,,,,,,590,590,590,590,,666,666,666,666", "666,666,,,,666,666,,,,,,,666,,,666,,,,666,666,662,662,662,662,662,,", "590,662,662,,,,662,,662,662,662,662,662,662,662,,,,,,662,662,662,662", "662,662,662,,,662,,,,,,662,662,,662,662,662,,662,662,662,662,662,,662", "662,662,,662,662,,662,662,,,,,,,,,,,,,,,,,,,,,662,,,662,,,662,,,662", ",662,,,,662,,,,,,,,662,,,,,662,662,662,662,662,662,,,,662,662,,,,,,", "662,,,662,,,,662,662,661,661,661,661,661,,,,661,661,,,,661,,661,661", "661,661,661,661,661,,,,,,661,661,661,661,661,661,661,,,661,,,,,,661", "661,,661,661,661,,661,661,661,661,661,,661,661,661,,661,661,,661,661", ",,,,,,,,,,,,,,,,,,,,661,,,661,,,661,,,661,,661,,,,661,,,,,,,,661,,,", ",661,661,661,661,661,661,,,,661,661,,,,655,655,655,661,655,,661,,655", "655,661,661,,655,,655,655,655,655,655,655,655,,,,,,655,655,655,655,655", "655,655,,,655,,,,,,,655,,,655,655,,655,655,655,655,655,655,655,655,655", ",655,655,,655,655,,,,,,,,,,,,,,,,,,,,,655,,,655,,,655,,,655,,,,655,", "655,,,,,,,,655,,,,,655,655,655,655,655,655,,,,655,655,,,,,,,655,,,655", ",,,655,655,99,99,99,99,99,,,,99,99,,,,99,,99,99,99,99,99,99,99,,,,,", "99,99,99,99,99,99,99,,,99,,,,,,99,99,99,99,99,99,,99,99,99,99,99,,99", "99,99,,99,99,,99,99,,,,,,,,,,,,,,,,,,,,,99,,,99,,,99,,,99,,99,,,,99", ",,,,,,,99,,,,,99,99,99,99,99,99,,,,99,99,,,,,,99,99,,,99,,,,99,99,103", "103,103,,103,,,,103,103,,,,103,,103,103,103,103,103,103,103,,,,,,103", "103,103,103,103,103,103,,,103,,,,,,,103,,,103,103,,103,103,103,103,103", ",103,103,103,,103,103,,103,103,,,,,,,,,,,,,,,,,,,,,103,,,103,,,103,", ",103,,,,,,103,,,,,,,,103,,,,,103,103,103,103,103,103,,,,103,103,,,,104", "104,104,103,104,,103,,104,104,103,103,,104,,104,104,104,104,104,104", "104,,,,,,104,104,104,104,104,104,104,,,104,,,,,,,104,,,104,104,,104", "104,104,104,104,,104,104,104,,104,104,,104,104,,,,,,,,,,,,,,,,,,,,,104", ",,104,,,104,,,104,,,,,,104,,,,,,,,104,,,,,104,104,104,104,104,104,,", ",104,104,,,,105,105,105,104,105,,104,,105,105,104,104,,105,,105,105", "105,105,105,105,105,,,,,,105,105,105,105,105,105,105,,,105,,,,,,,105", ",,105,105,,105,105,105,105,105,,105,105,105,,105,105,,105,105,,,,,,", ",,,,,,,,,,,,,,105,,,105,,,105,,,105,,,,,,105,,,,,,,,105,,,,,105,105", "105,105,105,105,,,,105,105,,,,106,106,106,105,106,,105,,106,106,105", "105,,106,,106,106,106,106,106,106,106,,,,,,106,106,106,106,106,106,106", ",,106,,,,,,,106,,,106,106,,106,106,106,106,106,,106,106,106,,106,106", ",106,106,,,,,,,,,,,,,,,,,,,,,106,,,106,,,106,,,106,,,,,,106,,,,,,,,106", ",,,,106,106,106,106,106,106,,,,106,106,,,,,,,106,,,106,,,,106,106,107", "107,107,107,107,,,,107,107,,,,107,,107,107,107,107,107,107,107,,,,,", "107,107,107,107,107,107,107,,,107,,,,,,107,107,,107,107,107,,107,107", "107,107,107,,107,107,107,,107,107,,107,107,,,,,,,,,,,,,,,,,,,,,107,", ",107,,,107,,,107,,107,,,,107,,,,,,,,107,,,,,107,107,107,107,107,107", ",,,107,107,,,,,,,107,,,107,,,,107,107,108,108,108,108,108,,,,108,108", ",,,108,,108,108,108,108,108,108,108,,,,,,108,108,108,108,108,108,108", ",,108,,,,,,108,108,108,108,108,108,,108,108,108,108,108,,108,108,108", ",108,108,,108,108,,,,,,,,,,,,,,,,,,,,,108,,,108,,,108,,,108,,108,,,", "108,,,,,,,,108,,,,,108,108,108,108,108,108,,,,108,108,,,,654,654,654", "108,654,,108,,654,654,108,108,,654,,654,654,654,654,654,654,654,,,,", ",654,654,654,654,654,654,654,,,654,,,,,,,654,,,654,654,,654,654,654", "654,654,654,654,654,654,,654,654,,654,654,,,,,,,,,,,,,,,,,,,,,654,,", "654,,,654,,,654,,654,,654,,654,,,,,,,,654,,,,,654,654,654,654,654,654", ",,,654,654,,,,,,,654,,,654,,,,654,654,651,651,651,651,651,,,,651,651", ",,,651,,651,651,651,651,651,651,651,,,,,,651,651,651,651,651,651,651", ",,651,,,,,,651,651,,651,651,651,,651,651,651,651,651,,651,651,651,,651", "651,,651,651,,,,,,,,,,,,,,,,,,,,,651,,,651,,,651,,,651,,651,,,,651,", ",,,,,,651,,,,,651,651,651,651,651,651,,,,651,651,,,,644,644,644,651", "644,,651,,644,644,651,651,,644,,644,644,644,644,644,644,644,,,,,,644", "644,644,644,644,644,644,,,644,,,,,,,644,,,644,644,,644,644,644,644,644", ",644,644,644,,644,644,,644,644,,,,,,,,,,,,,,,,,,,,,644,,,644,,,644,", ",644,,,,,,644,,,,,,,,644,,,,,644,644,644,644,644,644,,,,644,644,,,,", ",,644,,,644,,,,644,644,195,195,195,195,195,,,,195,195,,,,195,,195,195", "195,195,195,195,195,,,,,,195,195,195,195,195,195,195,,,195,,,,,,195", "195,,195,195,195,,195,195,195,195,195,,195,195,195,,195,195,,195,195", ",,,,,,,,,,,,,,,,,,,,195,,,195,,,195,,,195,,195,,,,195,,,,,,,,195,,,", ",195,195,195,195,195,195,,,,195,195,,,,196,196,196,195,196,,195,,196", "196,195,195,,196,,196,196,196,196,196,196,196,,,,,,196,196,196,196,196", "196,196,,,196,,,,,,,196,,,196,196,,196,196,196,196,196,,196,196,196", ",196,196,,196,196,,,,,,,,,,,,,,,,,,,,,196,,,196,,,196,,,196,,196,,,", "196,,,,,,,,196,,,,,196,196,196,196,196,196,,,,196,196,,,,197,197,197", "196,197,,196,,197,197,196,196,,197,,197,197,197,197,197,197,197,,,,", ",197,197,197,197,197,197,197,,,197,,,,,,,197,,,197,197,,197,197,197", "197,197,,197,197,197,,197,197,,197,197,,,,,,,,,,,,,,,,,,,,,197,,,197", ",,197,,,197,,197,,,,197,,,,,,,,197,,,,,197,197,197,197,197,197,,,,197", "197,,,,198,198,198,197,198,,197,,198,198,197,197,,198,,198,198,198,198", "198,198,198,,,,,,198,198,198,198,198,198,198,,,198,,,,,,,198,,,198,198", ",198,198,198,198,198,,198,198,198,,198,198,,198,198,,,,,,,,,,,,,,,,", ",,,,198,,,198,,,198,,,198,,,,,,198,,,,,,,,198,,,,,198,198,198,198,198", "198,,,,198,198,,,,199,199,199,198,199,,198,,199,199,198,198,,199,,199", "199,199,199,199,199,199,,,,,,199,199,199,199,199,199,199,,,199,,,,,", ",199,,,199,199,,199,199,199,199,199,199,199,199,199,,199,199,,199,199", ",,,,,,,,,,,,,,,,,,,,199,,,199,,,199,,,199,,199,,,,199,,,,,,,,199,,,", ",199,199,199,199,199,199,,,,199,199,,,,,,,199,,,199,,,,199,199,640,640", "640,640,640,,,,640,640,,,,640,,640,640,640,640,640,640,640,,,,,,640", "640,640,640,640,640,640,,,640,,,,,,640,640,,640,640,640,,640,640,640", "640,640,,640,640,640,,640,640,,640,640,,,,,,,,,,,,,,,,,,,,,640,,,640", ",,640,,,640,,640,,,,640,,,,,,,,640,,,,,640,640,640,640,640,640,,,,640", "640,,,,636,636,636,640,636,,640,,636,636,640,640,,636,,636,636,636,636", "636,636,636,,,,,,636,636,636,636,636,636,636,,,636,,,,,,,636,,,636,636", ",636,636,636,636,636,,636,636,636,,636,636,,636,636,,,,,,,,,,,,,,,,", ",,,,636,,,636,,,636,,,636,,,,,,636,,,,,,,,636,,,,,636,636,636,636,636", "636,,,,636,636,,,,202,202,202,636,202,,636,,202,202,636,636,,202,,202", "202,202,202,202,202,202,,,,,,202,202,202,202,202,202,202,,,202,,,,,", ",202,,,202,202,,202,202,202,202,202,,202,202,202,,202,202,,202,202,", ",,,,,,,,,,,,,,,,,,,202,,,202,,,202,,,202,,,,,,202,,,,,,,,202,,,,,202", "202,202,202,202,202,,,,202,202,,,,203,203,203,202,203,,202,,203,203", "202,202,,203,,203,203,203,203,203,203,203,,,,,,203,203,203,203,203,203", "203,,,203,,,,,,,203,,,203,203,,203,203,203,203,203,,203,203,203,,203", "203,,203,203,,,,,,,,,,,,,,,,,,,,,203,,,203,,,203,,,203,,,,,,203,,,,", ",,,203,,,,,203,203,203,203,203,203,,,,203,203,,,,204,204,204,203,204", ",203,,204,204,203,203,,204,,204,204,204,204,204,204,204,,,,,,204,204", "204,204,204,204,204,,,204,,,,,,,204,,,204,204,,204,204,204,204,204,", "204,204,204,,204,204,,204,204,,,,,,,,,,,,,,,,,,,,,204,,,204,,,204,,", "204,,,,,,204,,,,,,,,204,,,,,204,204,204,204,204,204,,,,204,204,,,,628", "628,628,204,628,,204,,628,628,204,204,,628,,628,628,628,628,628,628", "628,,,,,,628,628,628,628,628,628,628,,,628,,,,,,,628,,,628,628,,628", "628,628,628,628,,628,628,628,,628,628,,628,628,,,,,,,,,,,,,,,,,,,,,628", ",,628,,,628,,,628,,,,,,628,,,,,,,,628,,,,,628,628,628,628,628,628,,", ",628,628,,,,625,625,625,628,625,,628,,625,625,628,628,,625,,625,625", "625,625,625,625,625,,,,,,625,625,625,625,625,625,625,,,625,,,,,,,625", ",,625,625,,625,625,625,625,625,,625,625,625,,625,625,,625,625,,,,,,", ",,,,,,,,,,,,,,625,,,625,,,625,,,625,,,,,,625,,,,,,,,625,,,,,625,625", "625,625,625,625,,,,625,625,,,,620,620,620,625,620,,625,,620,620,625", "625,,620,,620,620,620,620,620,620,620,,,,,,620,620,620,620,620,620,620", ",,620,,,,,,,620,,,620,620,,620,620,620,620,620,,620,620,620,,620,620", ",620,620,,,,,,,,,,,,,,,,,,,,,620,,,620,,,620,,,620,,,,,,620,,,,,,,,620", ",,,,620,620,620,620,620,620,,,,620,620,,,,619,619,619,620,619,,620,", "619,619,620,620,,619,,619,619,619,619,619,619,619,,,,,,619,619,619,619", "619,619,619,,,619,,,,,,,619,,,619,619,,619,619,619,619,619,,619,619", "619,,619,619,,619,619,,,,,,,,,,,,,,,,,,,,,619,,,619,,,619,,,619,,,,", ",619,,,,,,,,619,,,,,619,619,619,619,619,619,,,,619,619,,,,617,617,617", "619,617,,619,,617,617,619,619,,617,,617,617,617,617,617,617,617,,,,", ",617,617,617,617,617,617,617,,,617,,,,,,,617,,,617,617,,617,617,617", "617,617,,617,617,617,,617,617,,617,617,,,,,,,,,,,,,,,,,,,,,617,,,617", ",,617,,,617,,617,,,,617,,,,,,,,617,,,,,617,617,617,617,617,617,,,,617", "617,,,,615,615,615,617,615,,617,,615,615,617,617,,615,,615,615,615,615", "615,615,615,,,,,,615,615,615,615,615,615,615,,,615,,,,,,,615,,,615,615", ",615,615,615,615,615,,615,615,615,,615,615,,615,615,,,,,,,,,,,,,,,,", ",,,,615,,,615,,,615,,,615,,,,,,615,,,,,,,,615,,,,,615,615,615,615,615", "615,,,,615,615,,,,,,,615,,,615,,,,615,615,212,212,212,212,212,,,,212", "212,,,,212,,212,212,212,212,212,212,212,,,,,,212,212,212,212,212,212", "212,,,212,,,,,,212,212,,212,212,212,,212,212,212,212,212,,212,212,212", ",212,212,,212,212,,,,,,,,,,,,,,,,,,,,,212,,,212,,,212,,,212,,212,,,", "212,,,,,,,,212,,,,,212,212,212,212,212,212,,,,212,212,,,,213,213,213", "212,213,,212,,213,213,212,212,,213,,213,213,213,213,213,213,213,,,,", ",213,213,213,213,213,213,213,,,213,,,,,,,213,,,213,213,,213,213,213", "213,213,,213,213,213,,213,213,,213,213,,,,,,,,,,,,,,,,,,,,,213,,,213", ",213,213,,,213,,,,,,213,,,,,,,,213,,,,,213,213,213,213,213,213,,,,213", "213,,,,216,216,216,213,216,,213,,216,216,213,213,,216,,216,216,216,216", "216,216,216,,,,,,216,216,216,216,216,216,216,,,216,,,,,,,216,,,216,216", ",216,216,216,216,216,,216,216,216,,216,216,,216,216,,,,,,,,,,,,,,,,", ",,,,216,,,216,,,216,,,216,,,,,,216,,,,,,,,216,,,,,216,216,216,216,216", "216,,,,216,216,,,,346,346,346,216,346,,216,,346,346,216,216,,346,,346", "346,346,346,346,346,346,,,,,,346,346,346,346,346,346,346,,,346,,,,,", ",346,,,346,346,,346,346,346,346,346,,346,346,346,,346,346,,346,346,", ",,,,,,,,,,,,,,,,,,,346,,,346,,,346,,,346,,,,,,346,,,,,,,,346,,,,,346", "346,346,346,346,346,,,,346,346,,,,218,218,218,346,218,,346,,218,218", "346,346,,218,,218,218,218,218,218,218,218,,,,,,218,218,218,218,218,218", "218,,,218,,,,,,,218,,,218,218,,218,218,218,218,218,,218,218,218,,218", "218,,218,218,,,,,,,,,,,,,,,,,,,,,218,,,218,,,218,,,218,,,,,,218,,,,", ",,,218,,,,,218,218,218,218,218,218,,,,218,218,,,,219,219,219,218,219", ",218,,219,219,218,218,,219,,219,219,219,219,219,219,219,,,,,,219,219", "219,219,219,219,219,,,219,,,,,,,219,,,219,219,,219,219,219,219,219,", "219,219,219,,219,219,,219,219,,,,,,,,,,,,,,,,,,,,,219,,,219,,,219,,", "219,,,,,,219,,,,,,,,219,,,,,219,219,219,219,219,219,,,,219,219,,,,220", "220,220,219,220,,219,,220,220,219,219,,220,,220,220,220,220,220,220", "220,,,,,,220,220,220,220,220,220,220,,,220,,,,,,,220,,,220,220,,220", "220,220,220,220,,220,220,220,,220,220,,220,220,,,,,,,,,,,,,,,,,,,,,220", ",,220,,,220,,,220,,,,,,220,,,,,,,,220,,,,,220,220,220,220,220,220,,", ",220,220,,,,221,221,221,220,221,,220,,221,221,220,220,,221,,221,221", "221,221,221,221,221,,,,,,221,221,221,221,221,221,221,,,221,,,,,,,221", ",,221,221,,221,221,221,221,221,,221,221,221,,221,221,,221,221,,,,,,", ",,,,,,,,,,,,,,221,,,221,,,221,,,221,,,,,,221,,,,,,,,221,,,,,221,221", "221,221,221,221,,,,221,221,,,,222,222,222,221,222,,221,,222,222,221", "221,,222,,222,222,222,222,222,222,222,,,,,,222,222,222,222,222,222,222", ",,222,,,,,,,222,,,222,222,,222,222,222,222,222,,222,222,222,,222,222", ",222,222,,,,,,,,,,,,,,,,,,,,,222,,,222,,,222,,,222,,,,,,222,,,,,,,,222", ",,,,222,222,222,222,222,222,,,,222,222,,,,223,223,223,222,223,,222,", "223,223,222,222,,223,,223,223,223,223,223,223,223,,,,,,223,223,223,223", "223,223,223,,,223,,,,,,,223,,,223,223,,223,223,223,223,223,,223,223", "223,,223,223,,223,223,,,,,,,,,,,,,,,,,,,,,223,,,223,,,223,,,223,,,,", ",223,,,,,,,,223,,,,,223,223,223,223,223,223,,,,223,223,,,,224,224,224", "223,224,,223,,224,224,223,223,,224,,224,224,224,224,224,224,224,,,,", ",224,224,224,224,224,224,224,,,224,,,,,,,224,,,224,224,,224,224,224", "224,224,,224,224,224,,224,224,,224,224,,,,,,,,,,,,,,,,,,,,,224,,,224", ",,224,,,224,,,,,,224,,,,,,,,224,,,,,224,224,224,224,224,224,,,,224,224", ",,,225,225,225,224,225,,224,,225,225,224,224,,225,,225,225,225,225,225", "225,225,,,,,,225,225,225,225,225,225,225,,,225,,,,,,,225,,,225,225,", "225,225,225,225,225,,225,225,225,,225,225,,225,225,,,,,,,,,,,,,,,,,", ",,,225,,,225,,,225,,,225,,,,,,225,,,,,,,,225,,,,,225,225,225,225,225", "225,,,,225,225,,,,226,226,226,225,226,,225,,226,226,225,225,,226,,226", "226,226,226,226,226,226,,,,,,226,226,226,226,226,226,226,,,226,,,,,", ",226,,,226,226,,226,226,226,226,226,,226,226,226,,226,226,,226,226,", ",,,,,,,,,,,,,,,,,,,226,,,226,,,226,,,226,,,,,,226,,,,,,,,226,,,,,226", "226,226,226,226,226,,,,226,226,,,,227,227,227,226,227,,226,,227,227", "226,226,,227,,227,227,227,227,227,227,227,,,,,,227,227,227,227,227,227", "227,,,227,,,,,,,227,,,227,227,,227,227,227,227,227,,227,227,227,,227", "227,,227,227,,,,,,,,,,,,,,,,,,,,,227,,,227,,,227,,,227,,,,,,227,,,,", ",,,227,,,,,227,227,227,227,227,227,,,,227,227,,,,228,228,228,227,228", ",227,,228,228,227,227,,228,,228,228,228,228,228,228,228,,,,,,228,228", "228,228,228,228,228,,,228,,,,,,,228,,,228,228,,228,228,228,228,228,", "228,228,228,,228,228,,228,228,,,,,,,,,,,,,,,,,,,,,228,,,228,,,228,,", "228,,,,,,228,,,,,,,,228,,,,,228,228,228,228,228,228,,,,228,228,,,,229", "229,229,228,229,,228,,229,229,228,228,,229,,229,229,229,229,229,229", "229,,,,,,229,229,229,229,229,229,229,,,229,,,,,,,229,,,229,229,,229", "229,229,229,229,,229,229,229,,229,229,,229,229,,,,,,,,,,,,,,,,,,,,,229", ",,229,,,229,,,229,,,,,,229,,,,,,,,229,,,,,229,229,229,229,229,229,,", ",229,229,,,,230,230,230,229,230,,229,,230,230,229,229,,230,,230,230", "230,230,230,230,230,,,,,,230,230,230,230,230,230,230,,,230,,,,,,,230", ",,230,230,,230,230,230,230,230,,230,230,230,,230,230,,230,230,,,,,,", ",,,,,,,,,,,,,,230,,,230,,,230,,,230,,,,,,230,,,,,,,,230,,,,,230,230", "230,230,230,230,,,,230,230,,,,231,231,231,230,231,,230,,231,231,230", "230,,231,,231,231,231,231,231,231,231,,,,,,231,231,231,231,231,231,231", ",,231,,,,,,,231,,,231,231,,231,231,231,231,231,,231,231,231,,231,231", ",231,231,,,,,,,,,,,,,,,,,,,,,231,,,231,,,231,,,231,,,,,,231,,,,,,,,231", ",,,,231,231,231,231,231,231,,,,231,231,,,,232,232,232,231,232,,231,", "232,232,231,231,,232,,232,232,232,232,232,232,232,,,,,,232,232,232,232", "232,232,232,,,232,,,,,,,232,,,232,232,,232,232,232,232,232,,232,232", "232,,232,232,,232,232,,,,,,,,,,,,,,,,,,,,,232,,,232,,,232,,,232,,,,", ",232,,,,,,,,232,,,,,232,232,232,232,232,232,,,,232,232,,,,233,233,233", "232,233,,232,,233,233,232,232,,233,,233,233,233,233,233,233,233,,,,", ",233,233,233,233,233,233,233,,,233,,,,,,,233,,,233,233,,233,233,233", "233,233,,233,233,233,,233,233,,233,233,,,,,,,,,,,,,,,,,,,,,233,,,233", ",,233,,,233,,,,,,233,,,,,,,,233,,,,,233,233,233,233,233,233,,,,233,233", ",,,234,234,234,233,234,,233,,234,234,233,233,,234,,234,234,234,234,234", "234,234,,,,,,234,234,234,234,234,234,234,,,234,,,,,,,234,,,234,234,", "234,234,234,234,234,,234,234,234,,234,234,,234,234,,,,,,,,,,,,,,,,,", ",,,234,,,234,,,234,,,234,,,,,,234,,,,,,,,234,,,,,234,234,234,234,234", "234,,,,234,234,,,,235,235,235,234,235,,234,,235,235,234,234,,235,,235", "235,235,235,235,235,235,,,,,,235,235,235,235,235,235,235,,,235,,,,,", ",235,,,235,235,,235,235,235,235,235,,235,235,235,,235,235,,235,235,", ",,,,,,,,,,,,,,,,,,,235,,,235,,,235,,,235,,,,,,235,,,,,,,,235,,,,,235", "235,235,235,235,235,,,,235,235,,,,236,236,236,235,236,,235,,236,236", "235,235,,236,,236,236,236,236,236,236,236,,,,,,236,236,236,236,236,236", "236,,,236,,,,,,,236,,,236,236,,236,236,236,236,236,,236,236,236,,236", "236,,236,236,,,,,,,,,,,,,,,,,,,,,236,,,236,,,236,,,236,,,,,,236,,,,", ",,,236,,,,,236,236,236,236,236,236,,,,236,236,,,,237,237,237,236,237", ",236,,237,237,236,236,,237,,237,237,237,237,237,237,237,,,,,,237,237", "237,237,237,237,237,,,237,,,,,,,237,,,237,237,,237,237,237,237,237,", "237,237,237,,237,237,,237,237,,,,,,,,,,,,,,,,,,,,,237,,,237,,,237,,", "237,,,,,,237,,,,,,,,237,,,,,237,237,237,237,237,237,,,,237,237,,,,238", "238,238,237,238,,237,,238,238,237,237,,238,,238,238,238,238,238,238", "238,,,,,,238,238,238,238,238,238,238,,,238,,,,,,,238,,,238,238,,238", "238,238,238,238,,238,238,238,,238,238,,238,238,,,,,,,,,,,,,,,,,,,,,238", ",,238,,,238,,,238,,,,,,238,,,,,,,,238,,,,,238,238,238,238,238,238,,", ",238,238,,,,239,239,239,238,239,,238,,239,239,238,238,,239,,239,239", "239,239,239,239,239,,,,,,239,239,239,239,239,239,239,,,239,,,,,,,239", ",,239,239,,239,239,239,239,239,,239,239,239,,239,239,,239,239,,,,,,", ",,,,,,,,,,,,,,239,,,239,,,239,,,239,,,,,,239,,,,,,,,239,,,,,239,239", "239,239,239,239,,,,239,239,,,,240,240,240,239,240,,239,,240,240,239", "239,,240,,240,240,240,240,240,240,240,,,,,,240,240,240,240,240,240,240", ",,240,,,,,,,240,,,240,240,,240,240,240,240,240,,240,240,240,,240,240", ",240,240,,,,,,,,,,,,,,,,,,,,,240,,,240,,,240,,,240,,,,,,240,,,,,,,,240", ",,,,240,240,240,240,240,240,,,,240,240,,,,241,241,241,240,241,,240,", "241,241,240,240,,241,,241,241,241,241,241,241,241,,,,,,241,241,241,241", "241,241,241,,,241,,,,,,,241,,,241,241,,241,241,241,241,241,,241,241", "241,,241,241,,241,241,,,,,,,,,,,,,,,,,,,,,241,,,241,,,241,,,241,,,,", ",241,,,,,,,,241,,,,,241,241,241,241,241,241,,,,241,241,,,,242,242,242", "241,242,,241,,242,242,241,241,,242,,242,242,242,242,242,242,242,,,,", ",242,242,242,242,242,242,242,,,242,,,,,,,242,,,242,242,,242,242,242", "242,242,,242,242,242,,242,242,,242,242,,,,,,,,,,,,,,,,,,,,,242,,,242", ",,242,,,242,,,,,,242,,,,,,,,242,,,,,242,242,242,242,242,242,,,,242,242", ",,,243,243,243,242,243,,242,,243,243,242,242,,243,,243,243,243,243,243", "243,243,,,,,,243,243,243,243,243,243,243,,,243,,,,,,,243,,,243,243,", "243,243,243,243,243,,243,243,243,,243,243,,243,243,,,,,,,,,,,,,,,,,", ",,,243,,,243,,,243,,,243,,,,,,243,,,,,,,,243,,,,,243,243,243,243,243", "243,,,,243,243,,,,586,586,586,243,586,,243,,586,586,243,243,,586,,586", "586,586,586,586,586,586,,,,,,586,586,586,586,586,586,586,,,586,,,,,", ",586,,,586,586,,586,586,586,586,586,,586,586,586,,586,586,,586,586,", ",,,,,,,,,,,,,,,,,,,586,,,586,,,586,,,586,,,,,,586,,,,,,,,586,,,,,586", "586,586,586,586,586,,,,586,586,,,,582,582,582,586,582,,586,,582,582", "586,586,,582,,582,582,582,582,582,582,582,,,,,,582,582,582,582,582,582", "582,,,582,,,,,,,582,,,582,582,,582,582,582,582,582,582,582,582,582,", "582,582,,582,582,,,,,,,,,,,,,,,,,,,,,582,,,582,,,582,,,582,,582,,,,582", ",,,,,,,582,,,,,582,582,582,582,582,582,,,,582,582,,,,574,574,574,582", "574,,582,,574,574,582,582,,574,,574,574,574,574,574,574,574,,,,,,574", "574,574,574,574,574,574,,,574,,,,,,,574,,,574,574,,574,574,574,574,574", "574,574,574,574,,574,574,,574,574,,,,,,,,,,,,,,,,,,,,,574,,,574,,,574", ",,574,,574,,,,574,,,,,,,,574,,,,,574,574,574,574,574,574,,,,574,574", ",,,252,252,252,574,252,,574,,252,252,574,574,,252,,252,252,252,252,252", "252,252,,,,,,252,252,252,252,252,252,252,,,252,,,,,,,252,,,252,252,", "252,252,252,252,252,,252,252,252,,252,252,,252,252,,,,,,,,,,,,,,,,,", ",,,252,,,252,,,252,,,252,,,,,,252,,,,,,,,252,,,,,252,252,252,252,252", "252,,,,252,252,,,,,,,252,,,252,,,,252,252,572,572,572,572,572,,,,572", "572,,,,572,,572,572,572,572,572,572,572,,,,,,572,572,572,572,572,572", "572,,,572,,,,,,572,572,,572,572,572,,572,572,572,572,572,,572,572,572", ",572,572,,572,572,,,,,,,,,,,,,,,,,,,,,572,,,572,,,572,,,572,,572,,,", "572,,,,,,,,572,,,,,572,572,572,572,572,572,,,,572,572,,,,254,254,254", "572,254,,572,,254,254,572,572,,254,,254,254,254,254,254,254,254,,,,", ",254,254,254,254,254,254,254,,,254,,,,,,,254,,,254,254,,254,254,254", "254,254,,254,254,254,,254,254,,254,254,,,,,,,,,,,,,,,,,,,,,254,,,254", ",,254,,,254,,,,,,254,,,,,,,,254,,,,,254,254,254,254,254,254,,,,254,254", ",,,259,259,259,254,259,,254,,259,259,254,254,,259,,259,259,259,259,259", "259,259,,,,,,259,259,259,259,259,259,259,,,259,,,,,,,259,,,259,259,", "259,259,259,259,259,,259,259,259,,259,259,,259,259,,,,,,,,,,,,,,,,,", ",,,259,,,259,,,259,,,259,,,,,,259,,,,,,,,259,,,,,259,259,259,259,259", "259,,,,259,259,,,,,,,259,,,259,,,,259,259,567,567,567,567,567,,,,567", "567,,,,567,,567,567,567,567,567,567,567,,,,,,567,567,567,567,567,567", "567,,,567,,,,,,567,567,,567,567,567,,567,567,567,567,567,,567,567,567", ",567,567,,567,567,,,,,,,,,,,,,,,,,,,,,567,,,567,,,567,,,567,,567,,,", "567,,,,,,,,567,,,,,567,567,567,567,567,567,,,,567,567,,,,,,,567,,,567", ",,,567,567,563,563,563,563,563,,,,563,563,,,,563,,563,563,563,563,563", "563,563,,,,,,563,563,563,563,563,563,563,,,563,,,,,,563,563,,563,563", "563,,563,563,563,563,563,,563,563,563,,563,563,,563,563,,,,,,,,,,,,", ",,,,,,,,563,,,563,,,563,,,563,,563,,,,563,,,,,,,,563,,,,,563,563,563", "563,563,563,,,,563,563,,,,549,549,549,563,549,,563,,549,549,563,563", ",549,,549,549,549,549,549,549,549,,,,,,549,549,549,549,549,549,549,", ",549,,,,,,,549,,,549,549,,549,549,549,549,549,,549,549,549,,549,549", ",,,,,,,,,,,,,,,,,,,,,,,549,,,549,,,549,,,549,,,,,,,,,,,,,,,,,,,549,549", "549,549,549,549,,,,549,549,,,,265,265,265,549,265,,549,,265,265,549", "549,,265,,265,265,265,265,265,265,265,,,,,,265,265,265,265,265,265,265", ",,265,,,,,,,265,,,265,265,,265,265,265,265,265,265,265,265,265,,265", "265,,265,265,,,,,,,,,,,,,,,,,,,,,265,,,265,,,265,,,265,,265,,265,,265", ",,,,,,,265,,,,,265,265,265,265,265,265,,,,265,265,,,,266,266,266,265", "266,,265,,266,266,265,265,,266,,266,266,266,266,266,266,266,,,,,,266", "266,266,266,266,266,266,,,266,,,,,,,266,,,266,266,,266,266,266,266,266", "266,266,266,266,,266,266,,266,266,,,,,,,,,,,,,,,,,,,,,266,,,266,,,266", ",,266,,266,,266,,266,,,,,,,,266,,,,,266,266,266,266,266,266,,,,266,266", ",,,274,274,274,266,274,,266,,274,274,266,266,,274,,274,274,274,274,274", "274,274,,,,,,274,274,274,274,274,274,274,,,274,,,,,,,274,,,274,274,", "274,274,274,274,274,274,274,274,274,,274,274,,274,274,,,,,,,,,,,,,,", ",,,,,,274,,,274,,274,274,,,274,,274,,274,,274,,,,,,,,274,,,,,274,274", "274,274,274,274,,,,274,274,,,,,,,274,,,274,,,,274,274,543,543,543,543", "543,,,,543,543,,,,543,,543,543,543,543,543,543,543,,,,,,543,543,543", "543,543,543,543,,,543,,,,,,543,543,,543,543,543,,543,543,543,543,543", ",543,543,543,,543,543,,543,543,,,,,,,,,,,,,,,,,,,,,543,,,543,,,543,", ",543,,543,,,,543,,,,,,,,543,,,,,543,543,543,543,543,543,,,,543,543,", ",,,,,543,,,543,,,,543,543,542,542,542,542,542,,,,542,542,,,,542,,542", "542,542,542,542,542,542,,,,,,542,542,542,542,542,542,542,,,542,,,,,", "542,542,,542,542,542,,542,542,542,542,542,,542,542,542,,542,542,,542", "542,,,,,,,,,,,,,,,,,,,,,542,,,542,,,542,,,542,,542,,,,542,,,,,,,,542", ",,,,542,542,542,542,542,542,,,,542,542,,,,537,537,537,542,537,,542,", "537,537,542,542,,537,,537,537,537,537,537,537,537,,,,,,537,537,537,537", "537,537,537,,,537,,,,,,,537,,,537,537,,537,537,537,537,537,537,537,537", "537,,537,537,,537,537,,,,,,,,,,,,,,,,,,,,,537,,,537,,,537,,,537,,,,", ",537,,,,,,,,537,,,,,537,537,537,537,537,537,,,,537,537,,,,,,,537,,,537", ",,,537,537,278,278,278,278,278,,,,278,278,,,,278,,278,278,278,278,278", "278,278,,,,,,278,278,278,278,278,278,278,,,278,,,,,,278,278,,278,278", "278,,278,278,278,278,278,,278,278,278,,278,278,,278,278,,,,,,,,,,,,", ",,,,,,,,278,,,278,,,278,,,278,,278,,,,278,,,,,,,,278,,,,,278,278,278", "278,278,278,,,,278,278,,,,534,534,534,278,534,,278,,534,534,278,278", ",534,,534,534,534,534,534,534,534,,,,,,534,534,534,534,534,534,534,", ",534,,,,,,,534,,,534,534,,534,534,534,534,534,534,534,534,534,,534,534", ",534,534,,,,,,,,,,,,,,,,,,,,,534,,,534,,,534,,,534,,534,,,,534,,,,,", ",,534,,,,,534,534,534,534,534,534,,,,534,534,,,,528,528,528,534,528", ",534,,528,528,534,534,,528,,528,528,528,528,528,528,528,,,,,,528,528", "528,528,528,528,528,,,528,,,,,,,528,,,528,528,,528,528,528,528,528,", "528,528,528,,528,528,,528,528,,,,,,,,,,,,,,,,,,,,,528,,,528,,,528,,", "528,,,,,,528,,,,,,,,528,,,,,528,528,528,528,528,528,,,,528,528,,,,524", "524,524,528,524,,528,,524,524,528,528,,524,,524,524,524,524,524,524", "524,,,,,,524,524,524,524,524,524,524,,,524,,,,,,,524,,,524,524,,524", "524,524,524,524,,524,524,524,,524,524,,524,524,,,,,,,,,,,,,,,,,,,,,524", ",,524,,,524,,,524,,,,,,524,,,,,,,,524,,,,,524,524,524,524,524,524,,", ",524,524,,,,282,282,282,524,282,,524,,282,282,524,524,,282,,282,282", "282,282,282,282,282,,,,,,282,282,282,282,282,282,282,,,282,,,,,,,282", ",,282,282,,282,282,282,282,282,,282,282,282,,282,282,,,,,,,,,,,,,,,", ",,,,,,,,282,,,282,,,282,,,282,,,,,,,,,,,,,,,,,,,282,282,282,282,282", "282,,,,282,282,,,,282,,,282,,,282,,,,282,282,283,283,283,283,283,,,", "283,283,,,,283,,283,283,283,283,283,283,283,,,,,,283,283,283,283,283", "283,283,,,283,,,,,,283,283,,283,283,283,,283,283,283,283,283,,283,283", "283,,283,283,,283,283,,,,,,,,,,,,,,,,,,,,,283,,,283,,,283,,,283,,283", ",,,283,,,,,,,,283,,,,,283,283,283,283,283,283,,,,283,283,,,,523,523", "523,283,523,,283,,523,523,283,283,,523,,523,523,523,523,523,523,523", ",,,,,523,523,523,523,523,523,523,,,523,,,,,,,523,,,523,523,,523,523", "523,523,523,,523,523,523,,523,523,,523,523,,,,,,,,,,,,,,,,,,,,,523,", ",523,,,523,,,523,,,,,,523,,,,,,,,523,,,,,523,523,523,523,523,523,,,", "523,523,,,,522,522,522,523,522,,523,,522,522,523,523,,522,,522,522,522", "522,522,522,522,,,,,,522,522,522,522,522,522,522,,,522,,,,,,,522,,,522", "522,,522,522,522,522,522,,522,522,522,,522,522,,522,522,,,,,,,,,,,,", ",,,,,,,,522,,,522,,,522,,,522,,,,,,522,,,,,,,,522,,,,,522,522,522,522", "522,522,,,,522,522,,,,513,513,513,522,513,,522,,513,513,522,522,,513", ",513,513,513,513,513,513,513,,,,,,513,513,513,513,513,513,513,,,513", ",,,,,,513,,,513,513,,513,513,513,513,513,513,513,513,513,,513,513,,513", "513,,,,,,,,,,,,,,,,,,,,,513,,,513,,,513,,,513,,513,,,,513,,,,,,,,513", ",,,,513,513,513,513,513,513,,,,513,513,,,,511,511,511,513,511,,513,", "511,511,513,513,,511,,511,511,511,511,511,511,511,,,,,,511,511,511,511", "511,511,511,,,511,,,,,,,511,,,511,511,,511,511,511,511,511,,511,511", "511,,511,511,,,,,,,,,,,,,,,,,,,,,,,,511,,,511,,,511,,,511,,,,,,,,,,", ",,,,,,,,511,511,511,511,511,511,,,,511,511,,,,,,,511,,,511,,,,511,511", "505,505,505,505,505,,,,505,505,,,,505,,505,505,505,505,505,505,505,", ",,,,505,505,505,505,505,505,505,,,505,,,,,,505,505,,505,505,505,,505", "505,505,505,505,,505,505,505,,505,505,,505,505,,,,,,,,,,,,,,,,,,,,,505", ",,505,,,505,,,505,,505,,,,505,,,,,,,,505,,,,,505,505,505,505,505,505", ",,,505,505,,,,,,505,505,,,505,,,,505,505,499,499,499,,499,,,,499,499", ",,,499,,499,499,499,499,499,499,499,,,,,,499,499,499,499,499,499,499", ",,499,,,,,,,499,,,499,499,,499,499,499,499,499,,499,499,499,,499,499", ",499,499,,,,,,,,,,,,,,,,,,,,,499,,,499,,499,499,,,499,,,,499,,499,,", ",,,,,499,,,,,499,499,499,499,499,499,,,,499,499,,,,483,483,483,499,483", ",499,,483,483,499,499,,483,,483,483,483,483,483,483,483,,,,,,483,483", "483,483,483,483,483,,,483,,,,,,,483,,,483,483,,483,483,483,483,483,", "483,483,483,,483,483,,483,483,,,,,,,,,,,,,,,,,,,,,483,,,483,,,483,,", "483,,,,,,483,,,,,,,,483,,,,,483,483,483,483,483,483,,,,483,483,,,,481", "481,481,483,481,,483,,481,481,483,483,,481,,481,481,481,481,481,481", "481,,,,,,481,481,481,481,481,481,481,,,481,,,,,,,481,,,481,481,,481", "481,481,481,481,481,481,481,481,,481,481,,481,481,,,,,,,,,,,,,,,,,,", ",,481,,,481,,,481,,,481,,,,481,,481,,,,,,,,481,,,,,481,481,481,481,481", "481,,,,481,481,,,,479,479,479,481,479,,481,,479,479,481,481,,479,,479", "479,479,479,479,479,479,,,,,,479,479,479,479,479,479,479,,,479,,,,,", ",479,,,479,479,,479,479,479,479,479,479,479,479,479,,479,479,,479,479", ",,,,,,,,,,,,,,,,,,,,479,,,479,,,479,,,479,,479,,479,,479,,,,,,,,479", ",,,,479,479,479,479,479,479,,,,479,479,,,,295,295,295,479,295,,479,", "295,295,479,479,,295,,295,295,295,295,295,295,295,,,,,,295,295,295,295", "295,295,295,,,295,,,,,,,295,,,295,295,,295,295,295,295,295,,295,295", "295,,295,295,,,,,,,,,,,,,,,,,,,,,,,,295,,,295,,,295,,,295,,,,,,,,,,", ",,,,,,,,295,295,295,295,295,295,,,,295,295,,,,448,448,448,295,448,,295", ",448,448,295,295,,448,,448,448,448,448,448,448,448,,,,,,448,448,448", "448,448,448,448,,,448,,,,,,,448,,,448,448,,448,448,448,448,448,,448", "448,448,,448,448,,448,448,,,,,,,,,,,,,,,,,,,,,448,,,448,,,448,,,448", ",,,,,448,,,,,,,,448,,,,,448,448,448,448,448,448,,,,448,448,,,,447,447", "447,448,447,,448,,447,447,448,448,,447,,447,447,447,447,447,447,447", ",,,,,447,447,447,447,447,447,447,,,447,,,,,,,447,,,447,447,,447,447", "447,447,447,,447,447,447,,447,447,,447,447,,,,,,,,,,,,,,,,,,,,,447,", ",447,,,447,,,447,,,,,,447,,,,,,,,447,,,,,447,447,447,447,447,447,,,", "447,447,,,,446,446,446,447,446,,447,,446,446,447,447,,446,,446,446,446", "446,446,446,446,,,,,,446,446,446,446,446,446,446,,,446,,,,,,,446,,,446", "446,,446,446,446,446,446,,446,446,446,,446,446,,446,446,,,,,,,,,,,,", ",,,,,,,,446,,,446,,,446,,,446,,,,,,446,,,,,,,,446,,,,,446,446,446,446", "446,446,,,,446,446,,,,444,444,444,446,444,,446,,444,444,446,446,,444", ",444,444,444,444,444,444,444,,,,,,444,444,444,444,444,444,444,,,444", ",,,,,,444,,,444,444,,444,444,444,444,444,444,444,444,444,,444,444,,444", "444,,,,,,,,,,,,,,,,,,,,,444,,,444,,,444,,,444,,444,,,,444,,,,,,,,444", ",,,,444,444,444,444,444,444,,,,444,444,,,,418,418,418,444,418,,444,", "418,418,444,444,,418,,418,418,418,418,418,418,418,,,,,,418,418,418,418", "418,418,418,,,418,,,,,,,418,,,418,418,,418,418,418,418,418,,418,418", "418,,418,418,,418,418,,,,,,,,,,,,,,,,,,,,,418,,,418,,,418,,,418,,,,", ",418,,,,,,,,418,,,,,418,418,418,418,418,418,,,,418,418,,,,304,304,304", "418,304,,418,,304,304,418,418,,304,,304,304,304,304,304,304,304,,,,", ",304,304,304,304,304,304,304,,,304,,,,,,,304,,,304,304,,304,304,304", "304,304,,304,304,304,,304,304,,304,304,,,,,,,,,,,,,,,,,,,,,304,,,304", "304,,304,,,304,,,,,,304,,,,,,,,304,,,,,304,304,304,304,304,304,,,,304", "304,,,,,,,304,,,304,,,,304,304,306,306,306,306,306,,,,306,306,,,,306", ",306,306,306,306,306,306,306,,,,,,306,306,306,306,306,306,306,,,306", ",,,,,306,306,,306,306,306,,306,306,306,306,306,,306,306,306,,306,306", ",306,306,,,,,,,,,,,,,,,,,,,,,306,,,306,,,306,,,306,,306,,,,306,,,,,", ",,306,,,,,306,306,306,306,306,306,,,,306,306,,,,378,378,378,306,378", ",306,,378,378,306,306,,378,,378,378,378,378,378,378,378,,,,,,378,378", "378,378,378,378,378,,,378,,,,,,,378,,,378,378,,378,378,378,378,378,", "378,378,378,,378,378,,378,378,,,,,,,,,,,,,,,,,,,,,378,,,378,,,378,,", "378,,,,,,378,,,,,,,,378,,,,,378,378,378,378,378,378,,,,378,378,,,,366", "366,366,378,366,,378,,366,366,378,378,,366,,366,366,366,366,366,366", "366,,,,,,366,366,366,366,366,366,366,,,366,,,,,,,366,,,366,366,,366", "366,366,366,366,,366,366,366,,366,366,,,,,,,,,,,,,,,,,,,,,,,,366,,,366", ",,366,,,366,,,,,,,,,,,,,,,,,,,366,366,366,366,366,366,,,,366,366,,,", "347,347,347,366,347,,366,,347,347,366,366,,347,,347,347,347,347,347", "347,347,,,,,,347,347,347,347,347,347,347,,,347,,,,,,,347,,,347,347,", "347,347,347,347,347,,347,347,347,,347,347,,347,347,,,,,,,,,,,,,,,,,", ",,,347,,,347,,,347,,,347,,,,,,347,,,,,,,,347,,,,,347,347,347,347,347", "347,,,,347,347,,,,588,588,588,347,588,,347,,588,588,347,347,,588,,588", "588,588,588,588,588,588,,,,,,588,588,588,588,588,588,588,,,588,,,,,", ",588,,,588,588,,588,588,588,588,588,,588,588,588,,588,588,,588,588,", ",,,,,,,,,,,,,,,,,,,588,,,588,,,588,,,588,,,,,,588,,,,,,,,588,,,,434", "588,588,588,588,588,588,434,434,434,588,588,,434,434,,434,,588,,,588", ",,,588,588,,,,,,,,,,434,434,,434,434,434,434,434,,,,,,,,,,,,,,,,,,,", ",,,,434,434,434,434,434,434,434,434,434,434,434,434,434,434,,,434,434", "434,,434,,,,434,,,,,,,434,,434,,434,434,434,434,434,434,434,,434,,434", ",,,,,,,,,,,,434,434,,434,,434,632,,434,,,434,,632,632,632,,,632,632", "632,,632,,,,,,,,,,632,632,,,,,,,,,632,632,,632,632,632,632,632,,,,,", ",,,,,,,,,,,,,,,,,,632,632,632,632,632,632,632,632,632,632,632,632,632", "632,,,632,632,632,,632,632,,,632,,,632,,632,,632,,632,,632,632,632,632", "632,632,632,,632,,632,,,,,,,,,,,,,632,632,632,632,,632,84,,632,,,632", ",84,84,84,,,84,84,84,,84,,,,,,,,84,,84,84,84,,,,,,,,84,84,,84,84,84", "84,84,,,,,,,,,,,,,,,,,,,,,,,,84,84,84,84,84,84,84,84,84,84,84,84,84", "84,,,84,84,84,,84,84,,,84,,,84,,84,,84,,84,,84,84,84,84,84,84,84,,84", ",84,,,,,,,,,,,,,84,84,84,84,,84,432,84,84,,,84,,432,432,432,,,,432,432", ",432,,,,,,,,,432,,,,,,,,,,,432,432,,432,432,432,432,432,,,,,,,,,,,,", ",,,,,,,,,,,432,432,432,432,432,432,432,432,432,432,432,432,432,432,", ",432,432,432,,432,,,,432,,,,,,,432,,432,,432,432,432,432,432,432,432", ",432,432,432,,,,,,,,,,,,,432,432,,432,,432,631,,432,,,432,,631,631,631", ",,631,631,631,,631,,,,,,,,,631,631,631,,,,,,,,,631,631,,631,631,631", "631,631,,,,,,,,,,,,,,,,,,,,,,,,631,631,631,631,631,631,631,631,631,631", "631,631,631,631,,,631,631,631,,631,631,,,631,,,631,,631,,631,,631,,631", "631,631,631,631,631,631,,631,631,631,,,,,,,,,,,,,631,631,631,631,,631", "87,,631,,,631,,87,87,87,,,87,87,87,,87,,,,,,,,87,,87,87,87,,,,,,,,87", "87,,87,87,87,87,87,,,,,,,,,,,,,,,,,,,,,,,,87,87,87,87,87,87,87,87,87", "87,87,87,87,87,,,87,87,87,,87,87,,,87,,,87,,87,,87,,87,,87,87,87,87", "87,87,87,,87,,87,,,,,,,,,,,,,87,87,87,87,,87,27,87,87,,,87,,27,27,27", ",,27,27,27,,27,,,,,,,,,27,27,27,,,,,,,,,27,27,,27,27,27,27,27,,,,,,", ",,,,,,,,,,,,,,,,,27,27,27,27,27,27,27,27,27,27,27,27,27,27,,,27,27,27", ",,27,,27,27,,,27,,27,,27,,27,,27,27,27,27,27,27,27,,27,27,27,,,,,,,", ",,,,,27,27,27,27,436,27,,,27,,,436,436,436,,,436,436,436,,436,,,,,,", ",,436,436,436,436,,,,,,,,436,436,,436,436,436,436,436,,,,,,,,,,,,,,", ",,,,,,,,,436,436,436,436,436,436,436,436,436,436,436,436,436,436,,,436", "436,436,,,436,,,436,,,436,,436,,436,,436,,436,436,436,436,436,436,436", ",436,436,436,,,,,,,,,,,,,436,436,436,436,428,436,,436,436,,,428,428", "428,,,428,428,428,,428,,,,,,,,,428,428,428,,,,,,,,,428,428,,428,428", "428,428,428,,,,,,,,,,,,,,,,,,,,,,,,428,428,428,428,428,428,428,428,428", "428,428,428,428,428,,,428,428,428,,,428,,428,428,,,428,,428,,428,,428", ",428,428,428,428,428,428,428,,428,428,428,,,,,,,,,,,,,428,428,428,428", "437,428,,,428,,,437,437,437,,,437,437,437,,437,,,,,,,,,437,437,437,437", ",,,,,,,437,437,,437,437,437,437,437,,,,,,,,,,,,,,,,,,,,,,,,437,437,437", "437,437,437,437,437,437,437,437,437,437,437,,,437,437,437,,,437,,,437", ",,437,,437,,437,,437,,437,437,437,437,437,437,437,,437,437,437,,,,,", ",,,,,,,437,437,437,437,28,437,,437,437,,,28,28,28,,,28,28,28,,28,,,", ",,,,,,28,28,,,,,,,,,28,28,,28,28,28,28,28,,,,,,,,,,,,,,,,,,,,,,,,28", "28,28,28,28,28,28,28,28,28,28,28,28,28,,,28,28,28,,,28,,28,28,,,28,", "28,,28,,28,,28,28,28,28,28,28,28,,28,,28,,,,,,,,,,,,,28,28,28,28,50", "28,,,28,,,50,50,50,,,50,50,50,,50,,,,,,,,,,50,50,50,,,,,,,,50,50,,50", "50,50,50,50,,,,,,,,,,,,,,,,,,,,,,,,50,50,50,50,50,50,50,50,50,50,50", "50,50,50,,,50,50,50,,,50,,,50,,,50,,50,,50,,50,,50,50,50,50,50,50,50", ",50,,50,,,,,,,,,,,,,50,50,50,50,486,50,,50,50,,,486,486,486,,,486,486", "486,,486,,,,,,,,,,486,,,,,,,,,,486,486,,486,486,486,486,486,,,,,,,,", ",,,,489,,,,,,,489,489,489,,,489,489,489,,489,,,,,,486,,,,489,,,486,", ",,,486,486,489,489,,489,489,489,489,489,,,,,,,,,,,,,486,,,,,,,,,,,,", "486,,486,,,486,,,,489,,,,,,,489,,,,,489,489,,,,,,,,,,,,,,,,,,,,,489", ",,,,,,,,,,,,489,,489,,,489,408,408,408,408,408,408,408,408,408,408,408", "408,408,408,408,408,408,408,408,408,408,408,408,408,,,,408,408,408,408", "408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408,408", ",408,408,,,408,,,,,,,,,408,408,,408,408,408,408,408,408,408,,,408,408", ",,,408,408,408,408,,,,,,,,,,,,,408,408,,408,408,408,408,408,408,408", "408,408,408,408,,,408,408,,,,,,,,,,408,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7", "7,7,7,7,7,7,7,7,7,,,,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,,7,7", "7,,7,,,,,,,,,7,7,,7,7,7,7,7,7,7,,,7,7,,,,7,7,7,7,,,,,,,,,,,,,7,7,,7", "7,7,7,7,7,7,7,7,7,7,,,7,7,,,,,,,,,,7,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8", "8,8,8,8,8,8,8,8,,,,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,,8,8,,", "8,,,,,,,,,8,8,,8,8,8,8,8,8,8,,,8,8,,,,8,8,8,8,,,,,,,,,,,,,8,8,,8,8,8", "8,8,8,8,8,8,8,8,,,8,8,,,,,,,,,,8,412,412,412,412,412,412,412,412,412", "412,412,412,412,412,412,412,412,412,412,412,412,412,412,412,,,,412,412", "412,412,412,412,412,412,412,412,412,412,412,412,412,412,412,412,412", "412,412,,412,412,,,412,,,,,,,,,412,412,,412,412,412,412,412,412,412", ",,412,412,,,,412,412,412,412,,,,,,,,,,,,,412,412,,412,412,412,412,412", "412,412,412,412,412,412,,,412,412,,,,,,,,,,412,747,747,747,747,747,747", "747,747,747,747,747,747,747,747,747,747,747,747,747,747,747,747,747", "747,,,,747,747,747,747,747,747,747,747,747,747,747,747,747,747,747,747", "747,747,747,747,747,,747,747,,,747,,,,,,,,,747,747,,747,747,747,747", "747,747,747,,,747,747,,,,747,747,747,747,,,,,,,,,,,,,747,747,,747,747", "747,747,747,747,747,747,747,747,747,,,747,79,79,79,79,79,79,79,79,79", "79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,,,,79,79,79,79,79,79,79", "79,79,79,79,79,79,79,79,79,79,79,79,79,79,,79,79,79,79,79,,79,,,,,,", "79,79,,79,79,79,79,79,79,79,,,79,79,,,,79,79,79,79,,,,,,,,,,,,,79,79", ",79,79,79,79,79,79,79,79,79,79,79,,,79,192,192,192,192,192,192,192,192", "192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,,,,192", "192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,192", "192,192,192,,192,192,192,192,192,,192,,,,,,,192,192,,192,192,192,192", "192,192,192,,,192,192,,,,192,192,192,192,,,,,,,,,,,,,192,192,,192,192", "192,192,192,192,192,192,192,192,192,,,192,65,65,65,65,65,65,65,65,65", "65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,,,,65,65,65,65,65,65,65", "65,65,65,65,65,65,65,65,65,65,65,65,65,65,,65,65,65,65,65,,65,,,,,,", "65,65,,65,65,65,65,65,65,65,,,65,65,,,,65,65,65,65,,,,,,65,,,,,,,65", "65,,65,65,65,65,65,65,65,65,65,65,65,443,443,65,,443,,,,,,,,,443,443", ",443,443,443,443,443,443,443,,,443,443,,,,443,443,443,443,,,,,,,,,,", ",,443,443,,443,443,443,443,443,443,443,443,443,443,443,583,583,443,", "583,,,,,,,,,583,583,,583,583,583,583,583,583,583,,,583,583,,,,583,583", "583,583,,,,,,583,,,,,,,583,583,,583,583,583,583,583,583,583,583,583", "583,583,581,581,583,,581,,,,,,,,,581,581,,581,581,581,581,581,581,581", ",,581,581,,,,581,581,581,581,,,,,,,,,,,,,581,581,,581,581,581,581,581", "581,581,581,581,581,581,442,442,581,,442,,,,,,,,,442,442,,442,442,442", "442,442,442,442,,,442,442,,,,442,442,442,442,,,,,,442,,,,,,,442,442", ",442,442,442,442,442,442,442,442,442,442,442,576,576,442,,576,,,,,,", ",,576,576,,576,576,576,576,576,576,576,,,576,576,,,,576,576,576,576", ",,,,,,,,,,,,576,576,,576,576,576,576,576,576,576,576,576,576,576,575", "575,576,,575,,,,,,,,,575,575,,575,575,575,575,575,575,575,,,575,575", ",,,575,575,575,575,,,,,,575,,,,,,,575,575,,575,575,575,575,575,575,575", "575,575,575,575,919,919,575,,919,,,,,,,,,919,919,,919,919,919,919,919", "919,919,,,919,919,,,,919,919,919,919,,,,,,919,,,,,,,919,919,,919,919", "919,919,919,919,919,919,919,919,919,920,920,919,,920,,,,,,,,,920,920", ",920,920,920,920,920,920,920,,,920,920,,,,920,920,920,920,,,,,,,,,,", ",,920,920,,920,920,920,920,920,920,920,920,920,920,920,262,262,920,", "262,,,,,,,,,262,262,,262,262,262,262,262,262,262,,,262,262,,,,262,262", "262,262,,,,,,,,,,,,,262,262,,262,262,262,262,262,262,262,262,262,262", "262,263,263,262,,263,,,,,,,,,263,263,,263,263,263,263,263,263,263,,", "263,263,,,,263,263,263,263,,,,,,,,,,,,,263,263,,263,263,263,263,263", "263,263,263,263,263,263,526,526,263,,526,,,,,,,,,526,526,,526,526,526", "526,526,526,526,,,526,526,,,,526,526,526,526,,,,,,,,,,,,,526,526,,526", "526,526,526,526,526,526,526,526,526,526,201,201,526,,201,,,,,,,,,201", "201,,201,201,201,201,201,201,201,,,201,201,,,,201,201,201,201,,,,,,", ",,,,,,201,201,,201,201,201,201,201,201,201,201,201,201,201,515,515,201", ",515,,,,,,,,,515,515,,515,515,515,515,515,515,515,,,515,515,,,,515,515", "515,515,,,,,,,,,,,,,515,515,,515,515,515,515,515,515,515,515,515,515", "515,514,514,515,,514,,,,,,,,,514,514,,514,514,514,514,514,514,514,,", "514,514,,,,514,514,514,514,,,,,,514,,,,,,,514,514,,514,514,514,514,514", "514,514,514,514,514,514,200,200,514,,200,,,,,,,,,200,200,,200,200,200", "200,200,200,200,,,200,200,,,,200,200,200,200,,,,,,200,,,,,,,200,200", ",200,200,200,200,200,200,200,200,200,200,200,799,799,200,,799,,,,,,", ",,799,799,,799,799,799,799,799,799,799,,,799,799,,,,799,799,799,799", ",,,,,,,,,,,,799,799,,799,799,799,799,799,799,799,799,799,799,799,525", "525,799,,525,,,,,,,,,525,525,,525,525,525,525,525,525,525,,,525,525", ",,,525,525,525,525,,,,,,525,,,,,,,525,525,,525,525,525,525,525,525,525", "525,525,525,525,,,525"];

      racc_action_check = arr = Opal.get('Array').$new(23792, nil);

      idx = 0;

      ($a = ($c = clist).$each, $a.$$p = (TMP_3 = function(str){var self = TMP_3.$$s || this, $a, $b, TMP_4;
if (str == null) str = nil;
      return ($a = ($b = str.$split(",", -1)).$each, $a.$$p = (TMP_4 = function(i){var self = TMP_4.$$s || this, $a;
if (i == null) i = nil;
        if ((($a = i['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            } else {
            arr['$[]='](idx, i.$to_i())
          };
          return idx = $rb_plus(idx, 1);}, TMP_4.$$s = self, TMP_4), $a).call($b)}, TMP_3.$$s = self, TMP_3), $a).call($c);

      racc_action_pointer = [-2, 1124, nil, -48, nil, 1063, 976, 21953, 22076, 949, 917, 916, 944, 447, 426, 730, nil, 1846, 1972, 2362, 921, nil, 2362, 2488, 2614, 679, 13, 20871, 21387, nil, 3280, 3406, 3532, nil, 759, 363, 828, 655, 4174, 4300, 4426, 727, 676, nil, nil, nil, nil, nil, nil, nil, 21516, nil, 4942, 5068, 5194, -15, 5998, 5596, 5722, nil, nil, 5848, 5986, 6112, nil, 22658, nil, nil, nil, nil, nil, -101, nil, nil, nil, nil, nil, 502, 132, 22434, nil, nil, nil, 642, 20347, nil, nil, 20740, nil, nil, nil, nil, nil, nil, nil, nil, nil, 108, nil, 7420, nil, nil, nil, 7558, 7684, 7810, 7936, 8074, 8212, nil, 275, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, 22546, 160, nil, 8740, 8866, 8992, 9118, 9244, 23558, 23378, 9634, 9760, 9886, nil, 527, 231, 595, 258, 535, 597, 10780, 10906, nil, nil, 11032, 646, 11284, 11410, 11536, 11662, 11788, 11914, 12040, 12166, 12292, 12418, 12544, 12670, 12796, 12922, 13048, 13174, 13300, 13426, 13552, 13678, 13804, 13930, 14056, 14182, 14308, 14434, nil, nil, nil, 1594, nil, 614, 630, nil, 14938, 675, 15202, nil, nil, nil, nil, 15328, nil, nil, 23198, 23258, 750, 15856, 15982, nil, nil, nil, nil, nil, nil, nil, 16108, 357, 925, 767, 16648, 771, 793, 761, 17152, 17290, 530, 541, 867, 535, 848, 824, 197, nil, 896, 307, nil, 18574, nil, 288, 954, 955, 516, nil, 1005, nil, 19330, nil, 19468, 112, nil, 961, 560, 438, 1075, 1056, 545, 1078, nil, nil, 6, 6766, nil, nil, nil, 1037, 1027, nil, 1013, 1010, nil, nil, nil, nil, nil, nil, nil, 2864, nil, nil, nil, 1083, nil, nil, 1077, 664, 36, -7, 11158, 19846, 312, 23, 984, 40, 526, 1039, 46, 1067, nil, nil, -2, 1035, nil, 442, nil, 65, nil, nil, 19720, 167, 733, 149, 547, 543, 531, 271, 605, nil, 573, nil, 19594, nil, 262, nil, 244, nil, 292, 955, 252, nil, 954, -25, nil, 277, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, 963, 21830, nil, nil, nil, 22199, 961, nil, nil, 400, nil, 19204, 908, nil, 902, nil, nil, 1846, 934, 409, 100, 21129, nil, nil, nil, 20478, 935, 20085, nil, 21000, 21258, nil, 1468, nil, nil, 22898, 22718, 19078, 124, 18952, 18826, 18700, 304, nil, 3142, 4300, 543, 540, 936, 933, 929, 923, 5194, 5332, 2901, 4816, 4048, 3280, 4942, 4690, 4552, 4426, 5068, 4174, 3796, 1353, 2853, 3670, 3922, 940, -50, nil, 18448, nil, 18322, nil, 18196, nil, nil, 21645, nil, nil, 21701, 332, nil, 854, nil, nil, 825, 824, nil, nil, 18070, 317, 178, 856, nil, nil, 17932, 844, 804, nil, nil, 803, 17794, 833, 17668, 23498, 23438, 254, 808, nil, nil, 762, nil, 17542, 17416, 17026, 23678, 23318, 1720, 16900, 873, 859, 774, nil, nil, 16774, nil, nil, 16510, nil, nil, nil, nil, 16384, 16246, 757, nil, 883, nil, nil, 15730, 3069, nil, 479, nil, nil, 750, nil, 1316, nil, 710, 1064, nil, nil, 15604, 776, nil, nil, 15466, 124, 144, 756, 760, 15076, nil, 14812, 23018, 22958, 37, nil, 637, nil, 22838, 14686, 22778, nil, nil, 14560, 405, 19972, nil, 6892, nil, nil, nil, 35, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, 241, nil, nil, nil, 597, nil, nil, nil, nil, nil, 10642, 547, 10516, 478, 10390, 10264, 552, nil, nil, nil, 10138, 544, nil, 10012, -61, -47, 20609, 20216, 494, 676, nil, 9508, nil, 3418, nil, 9382, 456, nil, 372, 8602, nil, nil, nil, nil, nil, nil, 8476, nil, 123, 8338, 7282, 802, -22, nil, nil, 1, 7156, 7018, nil, 787, -37, 6880, -47, nil, -70, 207, 208, -49, -17, 400, 14, 6754, 2236, 66, 141, 239, 236, 6628, nil, nil, -60, 217, 322, nil, nil, 210, nil, 222, 378, 411, 384, 393, nil, nil, 473, 2893, nil, 699, nil, 584, nil, nil, nil, nil, nil, 608, nil, 613, 6502, 542, 56, 74, 116, 77, 6376, 403, nil, 599, 624, 6250, 311, nil, -25, 5470, 5332, 6124, 284, nil, nil, 724, nil, 4816, nil, 661, 669, nil, 670, 673, 675, nil, 680, nil, 22322, 748, 493, 4690, nil, nil, nil, 2488, 721, 4552, 4048, nil, 2614, nil, 2098, nil, nil, 1972, nil, 1078, 3922, 3796, 3670, 169, 2752, nil, 784, 884, nil, nil, 783, nil, 768, 3142, nil, 801, 905, 787, 559, nil, nil, 915, nil, 3016, 799, 841, nil, nil, nil, nil, nil, nil, 23618, nil, 745, nil, nil, nil, nil, 1190, 928, nil, 2878, 930, 2752, 2236, nil, nil, 90, -18, 99, nil, 945, nil, nil, 947, 967, 852, nil, nil, nil, 203, nil, nil, 52, 3544, nil, 291, nil, 190, nil, 2098, nil, nil, nil, nil, nil, nil, nil, 894, 890, nil, 1204, 1720, nil, nil, nil, 1594, 899, nil, nil, nil, 1468, nil, nil, 66, 1330, nil, 943, 906, nil, nil, 67, nil, 1030, 1042, 1204, 1078, nil, nil, 940, nil, nil, 970, nil, 934, nil, nil, 937, 940, 941, 934, nil, nil, 1342, nil, nil, 802, nil, 676, 444, 496, 1030, 361, nil, nil, 538, nil, nil, nil, 53, 400, 1083, nil, nil, 567, nil, nil, nil, 262, 1113, 124, 23078, 23138, 275, 414, nil, nil, nil, 1135, nil, 1028, 1150, nil, 1066, 73, 82, 74, 145, nil, nil, nil, nil, 654];

      racc_action_default = [-3, -546, -1, -534, -4, -6, -546, -546, -546, -546, -546, -546, -546, -546, -271, -36, -37, -546, -546, -42, -44, -45, -282, -320, -321, -49, -249, -249, -249, -61, -10, -65, -72, -74, -546, -448, -546, -546, -546, -546, -546, -536, -226, -264, -265, -266, -267, -268, -269, -270, -524, -273, -546, -545, -516, -290, -545, -546, -546, -295, -298, -534, -546, -546, -312, -546, -322, -323, -391, -392, -393, -394, -395, -545, -398, -545, -545, -545, -545, -545, -425, -431, -432, -546, -522, -438, -439, -523, -441, -442, -443, -444, -445, -446, -447, -450, -451, -546, -2, -535, -541, -542, -543, -546, -546, -546, -546, -546, -3, -13, -546, -100, -101, -102, -103, -104, -105, -106, -109, -110, -111, -112, -113, -114, -115, -116, -117, -118, -119, -120, -121, -122, -123, -124, -125, -126, -127, -128, -129, -130, -131, -132, -133, -134, -135, -136, -137, -138, -139, -140, -141, -142, -143, -144, -145, -146, -147, -148, -149, -150, -151, -152, -153, -154, -155, -156, -157, -158, -159, -160, -161, -162, -163, -164, -165, -166, -167, -168, -169, -170, -171, -172, -173, -174, -175, -176, -177, -178, -179, -180, -181, -182, -546, -18, -107, -10, -546, -546, -546, -545, -546, -546, -546, -546, -546, -40, -546, -448, -546, -271, -546, -546, -10, -546, -41, -218, -546, -546, -546, -546, -546, -546, -546, -546, -546, -546, -546, -546, -546, -546, -546, -546, -546, -546, -546, -546, -546, -546, -546, -546, -546, -546, -546, -546, -362, -364, -46, -227, -242, -256, -256, -246, -546, -257, -546, -282, -320, -321, -518, -546, -47, -48, -546, -546, -53, -545, -546, -289, -367, -374, -376, -59, -372, -60, -546, -534, -11, -61, -10, -546, -546, -66, -69, -10, -80, -546, -546, -87, -285, -536, -546, -324, -373, -546, -71, -546, -76, -278, -433, -434, -546, -203, -204, -219, -546, -537, -10, -536, -228, -536, -538, -538, -546, -546, -538, -546, -291, -292, -546, -546, -335, -336, -343, -545, -482, -350, -545, -545, -361, -481, -483, -484, -485, -486, -487, -546, -500, -505, -506, -508, -509, -510, -546, -43, -546, -546, -546, -546, -534, -546, -535, -448, -546, -546, -271, -546, -489, -490, -96, -546, -98, -546, -271, -546, -309, -448, -546, -100, -101, -138, -139, -155, -160, -167, -170, -315, -546, -514, -546, -396, -546, -411, -546, -413, -546, -546, -546, -403, -546, -546, -409, -546, -424, -426, -427, -428, -429, -435, -436, 941, -5, -544, -19, -20, -21, -22, -23, -546, -546, -15, -16, -17, -546, -546, -25, -33, -183, -257, -546, -546, -26, -34, -35, -27, -185, -546, -525, -526, -249, -369, -527, -528, -525, -249, -526, -371, -527, -528, -32, -192, -38, -39, -546, -546, -545, -278, -546, -546, -546, -546, -288, -193, -194, -195, -196, -197, -198, -199, -200, -205, -206, -207, -208, -209, -210, -211, -212, -213, -214, -215, -216, -217, -220, -221, -222, -223, -546, -545, -243, -546, -244, -546, -254, -546, -258, -521, -249, -525, -526, -249, -545, -54, -536, -237, -238, -256, -256, -250, -251, -546, -545, -545, -546, -284, -9, -535, -546, -62, -276, -77, -67, -546, -546, -545, -546, -546, -86, -546, -433, -434, -73, -78, -546, -546, -546, -546, -546, -224, -546, -383, -546, -546, -229, -230, -540, -539, -232, -540, -280, -281, -517, -332, -10, -10, -546, -334, -546, -352, -359, -546, -356, -357, -546, -360, -482, -546, -491, -546, -493, -495, -499, -507, -511, -10, -325, -326, -327, -10, -546, -546, -546, -546, -10, -378, -545, -546, -546, -278, -304, -96, -97, -546, -545, -546, -307, -452, -546, -546, -546, -313, -480, -317, -532, -533, -536, -397, -412, -415, -416, -418, -399, -414, -400, -401, -402, -546, -405, -407, -408, -546, -430, -7, -14, -108, -24, -546, -263, -546, -279, -546, -546, -55, -235, -236, -368, -546, -57, -370, -546, -525, -526, -525, -526, -546, -183, -287, -546, -346, -546, -348, -10, -256, -255, -259, -546, -519, -520, -50, -365, -51, -366, -10, -233, -546, -239, -241, -42, -546, -248, -252, -546, -10, -10, -283, -12, -62, -546, -70, -75, -546, -525, -526, -545, -529, -85, -546, -546, -191, -201, -202, -546, -545, -545, -274, -275, -538, -546, -546, -333, -344, -546, -351, -545, -345, -546, -545, -545, -501, -488, -546, -546, -498, -545, -328, -545, -296, -329, -330, -331, -299, -546, -302, -546, -546, -546, -525, -526, -529, -277, -546, -96, -99, -529, -546, -10, -546, -454, -546, -10, -10, -480, -546, -457, -458, -460, -461, -463, -464, -513, -513, -469, -471, -471, -471, -479, -482, -503, -546, -546, -546, -10, -404, -406, -410, -184, -261, -546, -546, -29, -187, -30, -188, -56, -31, -189, -58, -190, -546, -546, -546, -279, -225, -347, -546, -546, -245, -260, -546, -234, -256, -546, -253, -546, -546, -68, -81, -79, -286, -545, -341, -10, -384, -545, -385, -386, -231, -337, -338, -358, -546, -278, -546, -354, -355, -492, -494, -497, -546, -339, -546, -546, -10, -10, -301, -303, -546, -279, -546, -279, -546, -453, -310, -546, -546, -536, -456, -459, -462, -546, -467, -468, -546, -546, -475, -546, -477, -546, -478, -546, -318, -515, -417, -420, -421, -422, -423, -546, -262, -28, -186, -546, -349, -363, -52, -240, -256, -375, -377, -8, -10, -390, -342, -546, -546, -388, -277, -545, -496, -293, -546, -294, -546, -546, -546, -10, -305, -308, -10, -314, -316, -546, -465, -513, -512, -470, -471, -471, -471, -546, -504, -502, -480, -419, -247, -546, -389, -10, -88, -546, -546, -95, -387, -353, -10, -297, -300, -259, -545, -10, -546, -455, -466, -546, -473, -474, -476, -10, -383, -545, -546, -546, -94, -545, -379, -380, -381, -546, -311, -471, -546, -382, -546, -525, -526, -529, -93, -340, -306, -472, -319, -89];

      clist = ["26,304,375,556,248,248,248,214,428,433,328,312,320,571,520,102,393,26", "26,5,363,98,26,26,26,530,563,567,281,729,26,109,194,480,689,309,548", "291,291,551,553,415,422,681,117,117,344,345,741,744,348,807,26,26,99", "533,536,26,26,540,697,26,580,891,788,306,645,2,291,291,486,489,491,102", "294,596,264,271,273,349,555,606,889,114,114,346,279,810,249,249,249", "811,402,403,404,405,705,709,347,26,250,250,250,26,26,26,26,26,26,277", "642,812,642,829,830,692,13,719,5,696,382,384,645,908,391,648,406,5,650", "393,658,724,314,206,206,588,877,366,206,206,206,114,728,246,260,261", "13,285,285,590,833,835,837,747,591,114,318,425,541,743,889,376,695,858", "584,355,400,193,206,206,640,268,272,206,206,407,357,206,352,361,245", "309,477,651,500,414,420,423,741,744,501,438,923,661,662,26,26,26,26", "26,713,792,26,26,26,862,379,380,317,12,386,605,26,26,316,13,389,841", "749,206,206,206,206,13,13,556,750,824,701,882,667,881,313,377,710,497", "885,660,408,12,645,652,805,492,1,,,248,248,517,428,433,,,,,248,911,280", ",,,419,419,,26,26,531,114,532,,936,,328,26,544,,437,26,480,,,,26,721", "642,642,102,291,689,924,,521,504,913,914,915,,,743,752,291,699,,,,26", ",12,,,13,206,206,206,206,12,12,206,206,206,,672,589,510,,505,,13,206", "916,495,249,672,506,568,569,938,,512,249,,697,496,250,875,26,26,,741", "744,,250,,,,,277,,,102,,277,,,,570,,556,,866,,291,,,585,,,26,,,206,206", "672,,784,494,498,,,206,672,,,13,502,,321,285,13,,886,633,887,12,798", ",,802,803,,285,,,381,,383,383,387,390,383,12,13,413,775,,309,,,762,795", "930,612,,765,268,613,272,14,645,280,,,819,26,117,,822,823,117,,,653", "740,,,,,,,743,,206,206,,,,,14,287,287,669,,,867,,,,621,,,928,361,626", "114,642,,12,114,,,,12,,206,309,,874,,26,354,362,,,,26,280,,521,,,280", ",26,,12,,291,,437,,,,,664,642,,793,26,,,,714,,,,647,14,,649,723,,26", "26,896,14,14,,,,,903,,,748,291,309,641,,,206,,775,26,309,,,26,,,,925", "26,623,26,725,,,627,902,706,706,26,,,,26,,26,,,437,328,740,773,,909", ",,726,,,437,,,,493,,758,760,,,685,,763,206,,,,26,26,13,,,,26,,285,,206", ",623,,14,623,929,,,893,,26,,,,206,,,,,,14,26,437,,,,,437,13,13,,26,26", "521,547,693,,547,547,,,,291,786,,,,,,13,,,291,13,839,,,,13,,206,883", ",,883,740,,740,206,740,,,206,,206,,,,817,,,,,,12,,14,,,26,287,14,,,26", ",,672,26,26,,,779,287,,206,206,820,,821,,206,,825,14,,848,,419,,26,12", "12,,740,,13,26,,,,,,,,,,13,,622,686,687,12,,,740,12,13,13,815,,12,285", ",880,,,,,,26,702,285,,,704,,,869,,712,,35,,362,,,,,26,,26,26,,,,,623", "637,,627,,,114,,,,622,,706,622,637,35,284,284,206,,,,,13,637,637,,13", "13,855,,,12,,,,,,,,10,,26,12,,,,351,365,13,365,774,291,12,12,,206,26", "437,931,26,,777,,,,,,,,10,,782,783,,,,,,26,,,,309,35,,26,,,13,,26,35", "35,,,,,,26,,26,356,,,,315,,206,,13,13,,,,,12,,,,12,12,,,,,14,,,,,,287", ",,,,,,10,,,12,,,,,10,10,,,,,,,,,,13,,,846,898,14,14,,,,,,362,,,13,,", "13,,,437,35,,12,,14,,,,14,,,,,14,,13,622,35,,,,861,13,12,12,789,794", "13,,,,,,,,13,547,206,,547,547,,871,872,,,789,,789,,,,,,,10,,,,,,,,,", ",,,417,421,,12,10,,,,,,,,,14,,35,,,12,284,35,12,895,,14,,,,,,,,284,", "14,14,,907,,287,12,205,,35,,,,12,,287,,,12,,,,482,,484,917,12,,,485", "860,,922,,864,,10,926,,,,10,307,,,,,343,343,,,343,,,,,,,,,,,,14,10,", ",14,14,365,,,,,,,,,,,,,,,,,,,845,14,,,,343,343,343,343,,,,,,,,,,,,547", ",,,,,,,,,,,,,,,,,,,,14,,,,,,,,,,,,,,,,,,,,789,14,14,,,,,,,,,,,,,789", ",,,,,,,,,,,,,,,,,,,,,,,616,,,,,440,441,,,,14,,,,900,449,,,,,35,,,,,14", "284,,14,,,,,,,,,,,,,,,,,,,14,,,,,,,14,,,35,35,14,,643,,315,365,646,", "14,215,,,,247,247,247,10,,,35,,,657,35,,,,,35,301,302,303,,,,,,,,,,", ",,247,247,,,,,,,,10,10,,,643,,,315,,,,,,,,,,,,,,10,,,,10,,,,,10,,,,", ",343,343,,,,,,35,,,,,,,,,,,35,,,,,,,,,,35,35,,,594,284,,,,,,,,,,284", ",,,,,,,755,,,,,,,,,,,10,,,,,,,,,,,10,,,,,776,,,,,10,10,,,,643,315,,35", ",,,35,35,,416,247,424,247,,,439,,,,,,,,,,,,791,35,215,,451,452,453,454", "455,456,457,458,459,460,461,462,463,464,465,466,467,468,469,470,471", "472,473,474,475,476,10,417,,,10,10,,307,247,,247,35,,,,247,,,,,,247", "247,,827,,10,,,,247,,35,35,,,680,,,,,,,,847,,,,,,,,,,,,,,,,527,,,,,", "10,,417,,,,,,,,,,,,35,,,,897,,,10,10,,,,,,343,35,727,,35,,,,,,,,,,,", ",,,,,,,35,,,,,,,35,,890,,,35,,,,,,10,,35,,901,,,315,,,,,,,,10,,,10,", ",,,,,,906,,,,,,,,,,,10,,247,,,,,10,,,906,,10,,,,,,,,10,,,,,,,,247,,439", "634,424,,,,,,,,,,,,,,,,,,,,,,,343,,,,,,,,247,,247,,247,,,,,,,,,,,,,", ",,656,,,,,,,,,,,,,,247,,,,,,,,,677,678,679,,,,,,,,,,247,,,247,,,,,,", ",,,,,,,,,,,,,,,,,343,,,,,,,,,,,,,247,,,,,,,,247,,,,,,,,,,,,,,,,,,,,", ",,,,,,,,,,,,754,,247,,759,761,,,,,764,,,766,,,,,,,,771,,,,,,,,247,,", ",,,,,,,247,247,,,,,,,,,,,,,,,,,,,,,,,,,,,247,,,,,,,,,,,,,,,,,,,,,,,", ",,,,,,,247,,,,,,,,,,,,,,,,,,,,,,,247,,,,,,,,,,,,,,,,,,,,247,849,,,,", ",,,,,759,761,764,,,,,,,,,,,247,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,", ",,,,,,,,,,,,,,,,,,,,247,,,,,,,,,,,,849,,,,247,,,,,,,,,,,,,,,,,,,247", ",,,,,,,,,,,,,,,,,,,,247,,,,,,,,,,,,,,,,,,,,,,,,247"];

      racc_goto_table = arr = Opal.get('Array').$new(2256, nil);

      idx = 0;

      ($a = ($d = clist).$each, $a.$$p = (TMP_5 = function(str){var self = TMP_5.$$s || this, $a, $b, TMP_6;
if (str == null) str = nil;
      return ($a = ($b = str.$split(",", -1)).$each, $a.$$p = (TMP_6 = function(i){var self = TMP_6.$$s || this, $a;
if (i == null) i = nil;
        if ((($a = i['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            } else {
            arr['$[]='](idx, i.$to_i())
          };
          return idx = $rb_plus(idx, 1);}, TMP_6.$$s = self, TMP_6), $a).call($b)}, TMP_5.$$s = self, TMP_5), $a).call($d);

      clist = ["37,51,47,145,29,29,29,18,33,33,107,22,104,80,43,83,47,37,37,7,46,4,37", "37,37,8,77,77,41,84,37,14,14,61,106,29,109,52,52,109,109,24,24,10,50", "50,16,16,110,141,16,78,37,37,6,55,55,37,37,55,147,37,45,101,11,87,154", "2,52,52,33,33,35,83,42,129,34,34,34,4,148,129,151,48,48,88,39,11,54", "54,54,89,16,16,16,16,79,79,90,37,56,56,56,37,37,37,37,37,37,38,62,91", "62,142,142,111,21,92,7,111,126,126,154,93,126,60,7,7,60,47,62,94,56", "21,21,95,96,97,21,21,21,48,98,31,31,31,21,21,21,99,143,143,143,100,86", "48,102,22,103,111,151,85,105,12,82,81,5,15,21,21,36,57,57,21,21,2,19", "21,21,21,112,29,113,36,115,18,18,18,110,141,116,18,117,36,36,37,37,37", "37,37,118,119,37,37,37,120,124,125,76,20,127,128,37,37,74,21,130,131", "132,21,21,21,21,21,21,145,133,135,145,137,43,139,73,72,80,64,144,63", "27,20,154,59,149,58,1,,,29,29,51,33,33,,,,,29,142,9,,,,54,54,,37,37", "51,48,51,,78,,107,37,104,,48,37,61,,,,37,45,62,62,83,52,106,11,,41,4", "143,143,143,,,111,129,52,148,,,,37,,20,,,21,21,21,21,21,20,20,21,21", "21,,33,46,42,,6,,21,21,84,54,54,33,39,16,16,143,,39,54,,147,56,56,79", "37,37,,110,141,,56,,,,,38,,,83,,38,,,,4,,145,,111,,52,,,83,,,37,,,21", "21,33,,43,31,31,,,21,33,,,21,31,,53,21,21,,111,22,111,20,109,,,109,109", ",21,,,53,,53,53,53,53,53,20,21,9,61,,29,,,35,55,10,14,,35,57,14,57,23", "154,9,,,8,37,50,,8,8,50,,,51,108,,,,,,,111,,21,21,,,,,23,23,23,22,,", "145,,,,34,,,111,21,34,48,62,,20,48,,,,20,,21,29,,77,,37,23,23,,,,37", "9,,41,,,9,,37,,20,,52,,48,,,,,7,62,,24,37,,,,22,,,,34,23,,34,22,,37", "37,77,23,23,,,,,77,,,51,52,29,56,,,21,,61,37,29,,,37,,,,80,37,57,37", "16,,,57,109,83,83,37,,,,37,,37,,,48,107,108,104,,8,,,83,,,48,,,,53,", "18,18,,,56,,18,21,,,,37,37,21,,,,37,,21,,21,,57,,23,57,8,,,61,,37,,", ",21,,,,,,23,37,48,,,,,48,21,21,,37,37,41,53,21,,53,53,,,,52,41,,,,,", "21,,,52,21,47,,,,21,,21,108,,,108,108,,108,21,108,,,21,,21,,,,16,,,", ",,20,,23,,,37,23,23,,,37,,,33,37,37,,,56,23,,21,21,83,,83,,21,,83,23", ",18,,54,,37,20,20,,108,,21,37,,,,,,,,,,21,,53,9,9,20,,,108,20,21,21", "54,,20,21,,51,,,,,,37,9,21,,,9,,,16,,9,,44,,23,,,,,37,,37,37,,,,,57", "53,,57,,,48,,,,53,,83,53,53,44,44,44,21,,,,,21,53,53,,21,21,54,,,20", ",,,,,,,17,,37,20,,,,44,44,21,44,9,52,20,20,,21,37,48,22,37,,9,,,,,,", ",17,,9,9,,,,,,37,,,,29,44,,37,,,21,,37,44,44,,,,,,37,,37,17,,,,25,,21", ",21,21,,,,,20,,,,20,20,,,,,23,,,,,,23,,,,,,,17,,,20,,,,,17,17,,,,,,", ",,,21,,,9,21,23,23,,,,,,23,,,21,,,21,,,48,44,,20,,23,,,,23,,,,,23,,21", "53,44,,,,9,21,20,20,53,53,21,,,,,,,,21,53,21,,53,53,,9,9,,,53,,53,,", ",,,,17,,,,,,,,,,,,,25,25,,20,17,,,,,,,,,23,,44,,,20,44,44,20,9,,23,", ",,,,,,44,,23,23,,9,,23,20,26,,44,,,,20,,23,,,20,,,,25,,25,9,20,,,25", "53,,9,,53,,17,9,,,,17,26,,,,,26,26,,,26,,,,,,,,,,,,23,17,,,23,23,44", ",,,,,,,,,,,,,,,,,,23,23,,,,26,26,26,26,,,,,,,,,,,,53,,,,,,,,,,,,,,,", ",,,,,23,,,,,,,,,,,,,,,,,,,,53,23,23,,,,,,,,,,,,,53,,,,,,,,,,,,,,,,,", ",,,,,,25,,,,,26,26,,,,23,,,,23,26,,,,,44,,,,,23,44,,23,,,,,,,,,,,,,", ",,,,,23,,,,,,,23,,,44,44,23,,25,,25,44,25,,23,28,,,,28,28,28,17,,,44", ",,25,44,,,,,44,28,28,28,,,,,,,,,,,,,28,28,,,,,,,,17,17,,,25,,,25,,,", ",,,,,,,,,,17,,,,17,,,,,17,,,,,,26,26,,,,,,44,,,,,,,,,,,44,,,,,,,,,,44", "44,,,26,44,,,,,,,,,,44,,,,,,,,25,,,,,,,,,,,17,,,,,,,,,,,17,,,,,25,,", ",,17,17,,,,25,25,,44,,,,44,44,,28,28,28,28,,,28,,,,,,,,,,,,25,44,28", ",28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28,28", "28,28,28,17,25,,,17,17,,26,28,,28,44,,,,28,,,,,,28,28,,25,,17,,,,28", ",44,44,,,26,,,,,,,,25,,,,,,,,,,,,,,,,28,,,,,,17,,25,,,,,,,,,,,,44,,", ",44,,,17,17,,,,,,26,44,26,,44,,,,,,,,,,,,,,,,,,,44,,,,,,,44,,25,,,44", ",,,,,17,,44,,17,,,25,,,,,,,,17,,,17,,,,,,,,25,,,,,,,,,,,17,,28,,,,,17", ",,25,,17,,,,,,,,17,,,,,,,,28,,28,28,28,,,,,,,,,,,,,,,,,,,,,,,26,,,,", ",,,28,,28,,28,,,,,,,,,,,,,,,,28,,,,,,,,,,,,,,28,,,,,,,,,28,28,28,,,", ",,,,,,28,,,28,,,,,,,,,,,,,,,,,,,,,,,,26,,,,,,,,,,,,,28,,,,,,,,28,,,", ",,,,,,,,,,,,,,,,,,,,,,,,,,,,,28,,28,,28,28,,,,,28,,,28,,,,,,,,28,,,", ",,,,28,,,,,,,,,,28,28,,,,,,,,,,,,,,,,,,,,,,,,,,,28,,,,,,,,,,,,,,,,,", ",,,,,,,,,,,,,28,,,,,,,,,,,,,,,,,,,,,,,28,,,,,,,,,,,,,,,,,,,,28,28,,", ",,,,,,,28,28,28,,,,,,,,,,,28,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,", ",,,,,,,,,,,,,,,,,,28,,,,,,,,,,,,28,,,,28,,,,,,,,,,,,,,,,,,,28,,,,,,", ",,,,,,,,,,,,,,28,,,,,,,,,,,,,,,,,,,,,,,,28"];

      racc_goto_check = arr = Opal.get('Array').$new(2256, nil);

      idx = 0;

      ($a = ($e = clist).$each, $a.$$p = (TMP_7 = function(str){var self = TMP_7.$$s || this, $a, $b, TMP_8;
if (str == null) str = nil;
      return ($a = ($b = str.$split(",", -1)).$each, $a.$$p = (TMP_8 = function(i){var self = TMP_8.$$s || this, $a;
if (i == null) i = nil;
        if ((($a = i['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            } else {
            arr['$[]='](idx, i.$to_i())
          };
          return idx = $rb_plus(idx, 1);}, TMP_8.$$s = self, TMP_8), $a).call($b)}, TMP_7.$$s = self, TMP_7), $a).call($e);

      racc_goto_pointer = [nil, 244, 67, nil, 18, 67, 51, 19, -281, 228, -486, -617, -625, nil, 24, 159, -11, 857, -11, 114, 209, 116, -42, 438, -155, 869, 1089, 129, 1337, -18, nil, 121, nil, -192, 50, -192, -307, 0, 79, 56, nil, -3, 42, -281, 802, -297, -43, -63, 76, nil, 37, -40, 6, 341, 66, -255, 78, 145, -22, -251, -361, -217, -369, -262, -31, nil, nil, nil, nil, nil, nil, nil, 168, 178, 159, nil, 152, -318, -651, -472, -336, 103, -199, 12, -561, 96, -222, 14, 26, -614, 38, -598, -461, -752, -453, -229, -685, 73, -447, -226, -438, -776, 100, -160, -44, -388, -512, -46, -137, -287, -542, -431, 160, -62, nil, -85, -80, -715, -373, -481, -587, nil, nil, nil, 135, 134, 45, 133, -175, -305, 138, -532, -380, -373, nil, -503, nil, -599, nil, -597, nil, -541, -625, -591, -596, -332, nil, -490, -255, -458, nil, -752, nil, nil, -415];

      racc_goto_default = [nil, nil, nil, 3, nil, 4, 350, 276, nil, 529, nil, 808, nil, 275, nil, nil, nil, 210, 16, 11, 211, 300, nil, 209, nil, 253, 15, nil, 19, 20, 21, nil, 25, 675, nil, nil, nil, 290, 29, nil, 31, 34, 33, nil, 207, 360, nil, 116, 431, 115, 69, nil, 42, 308, 310, nil, 311, 429, nil, nil, 624, 478, 251, nil, nil, 266, 43, 44, 45, 46, 47, 48, 49, nil, 267, 55, nil, nil, nil, nil, nil, nil, nil, 564, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, 323, 322, 691, 325, nil, 326, 327, nil, nil, 435, nil, nil, nil, nil, nil, nil, 68, 70, 71, 72, nil, nil, nil, nil, 601, nil, nil, nil, nil, 392, 731, 734, 739, 736, 737, 738, 884, nil, nil, 742, 334, 329, 336, nil, 558, 559, 746, 339, 342, 258];

      racc_reduce_table = [0, 0, "racc_error", 1, 141, "_reduce_none", 2, 142, "_reduce_2", 0, 143, "_reduce_3", 1, 143, "_reduce_4", 3, 143, "_reduce_5", 1, 145, "_reduce_none", 4, 145, "_reduce_7", 4, 148, "_reduce_8", 2, 149, "_reduce_9", 0, 153, "_reduce_10", 1, 153, "_reduce_11", 3, 153, "_reduce_12", 0, 167, "_reduce_13", 4, 147, "_reduce_14", 3, 147, "_reduce_15", 3, 147, "_reduce_none", 3, 147, "_reduce_17", 2, 147, "_reduce_18", 3, 147, "_reduce_19", 3, 147, "_reduce_20", 3, 147, "_reduce_21", 3, 147, "_reduce_22", 3, 147, "_reduce_23", 4, 147, "_reduce_none", 3, 147, "_reduce_25", 3, 147, "_reduce_26", 3, 147, "_reduce_27", 6, 147, "_reduce_none", 5, 147, "_reduce_29", 5, 147, "_reduce_none", 5, 147, "_reduce_none", 3, 147, "_reduce_none", 3, 147, "_reduce_33", 3, 147, "_reduce_34", 3, 147, "_reduce_35", 1, 147, "_reduce_none", 1, 166, "_reduce_none", 3, 166, "_reduce_38", 3, 166, "_reduce_39", 2, 166, "_reduce_40", 2, 166, "_reduce_41", 1, 166, "_reduce_none", 1, 156, "_reduce_none", 1, 158, "_reduce_none", 1, 158, "_reduce_none", 2, 158, "_reduce_46", 2, 158, "_reduce_47", 2, 158, "_reduce_48", 1, 170, "_reduce_none", 4, 170, "_reduce_none", 4, 170, "_reduce_none", 4, 175, "_reduce_none", 2, 169, "_reduce_53", 3, 169, "_reduce_none", 4, 169, "_reduce_55", 5, 169, "_reduce_none", 4, 169, "_reduce_57", 5, 169, "_reduce_none", 2, 169, "_reduce_59", 2, 169, "_reduce_60", 1, 159, "_reduce_61", 3, 159, "_reduce_62", 1, 179, "_reduce_63", 3, 179, "_reduce_64", 1, 178, "_reduce_65", 2, 178, "_reduce_66", 3, 178, "_reduce_67", 5, 178, "_reduce_none", 2, 178, "_reduce_69", 4, 178, "_reduce_none", 2, 178, "_reduce_71", 1, 178, "_reduce_72", 3, 178, "_reduce_none", 1, 181, "_reduce_74", 3, 181, "_reduce_75", 2, 180, "_reduce_76", 3, 180, "_reduce_77", 1, 183, "_reduce_none", 3, 183, "_reduce_none", 1, 182, "_reduce_80", 4, 182, "_reduce_81", 3, 182, "_reduce_82", 3, 182, "_reduce_none", 3, 182, "_reduce_none", 3, 182, "_reduce_none", 2, 182, "_reduce_none", 1, 182, "_reduce_none", 1, 157, "_reduce_88", 4, 157, "_reduce_89", 3, 157, "_reduce_90", 3, 157, "_reduce_91", 3, 157, "_reduce_92", 3, 157, "_reduce_93", 2, 157, "_reduce_94", 1, 157, "_reduce_none", 1, 185, "_reduce_none", 2, 186, "_reduce_97", 1, 186, "_reduce_98", 3, 186, "_reduce_99", 1, 187, "_reduce_none", 1, 187, "_reduce_none", 1, 187, "_reduce_none", 1, 187, "_reduce_103", 1, 187, "_reduce_104", 1, 154, "_reduce_105", 1, 154, "_reduce_none", 1, 155, "_reduce_107", 3, 155, "_reduce_108", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 188, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 1, 189, "_reduce_none", 3, 168, "_reduce_183", 5, 168, "_reduce_184", 3, 168, "_reduce_185", 6, 168, "_reduce_186", 5, 168, "_reduce_187", 5, 168, "_reduce_none", 5, 168, "_reduce_none", 5, 168, "_reduce_none", 4, 168, "_reduce_none", 3, 168, "_reduce_none", 3, 168, "_reduce_193", 3, 168, "_reduce_194", 3, 168, "_reduce_195", 3, 168, "_reduce_196", 3, 168, "_reduce_197", 3, 168, "_reduce_198", 3, 168, "_reduce_199", 3, 168, "_reduce_200", 4, 168, "_reduce_201", 4, 168, "_reduce_202", 2, 168, "_reduce_203", 2, 168, "_reduce_204", 3, 168, "_reduce_205", 3, 168, "_reduce_206", 3, 168, "_reduce_207", 3, 168, "_reduce_208", 3, 168, "_reduce_209", 3, 168, "_reduce_210", 3, 168, "_reduce_211", 3, 168, "_reduce_212", 3, 168, "_reduce_213", 3, 168, "_reduce_214", 3, 168, "_reduce_215", 3, 168, "_reduce_216", 3, 168, "_reduce_217", 2, 168, "_reduce_218", 2, 168, "_reduce_219", 3, 168, "_reduce_220", 3, 168, "_reduce_221", 3, 168, "_reduce_222", 3, 168, "_reduce_223", 3, 168, "_reduce_224", 5, 168, "_reduce_225", 1, 168, "_reduce_none", 1, 165, "_reduce_none", 1, 162, "_reduce_228", 2, 162, "_reduce_229", 2, 162, "_reduce_230", 4, 162, "_reduce_231", 2, 162, "_reduce_232", 3, 197, "_reduce_233", 2, 199, "_reduce_none", 1, 200, "_reduce_235", 1, 200, "_reduce_none", 1, 198, "_reduce_237", 1, 198, "_reduce_none", 2, 198, "_reduce_239", 4, 198, "_reduce_240", 2, 198, "_reduce_241", 1, 171, "_reduce_242", 2, 171, "_reduce_243", 2, 171, "_reduce_244", 4, 171, "_reduce_245", 1, 171, "_reduce_246", 4, 203, "_reduce_none", 1, 203, "_reduce_none", 0, 205, "_reduce_249", 2, 174, "_reduce_250", 1, 204, "_reduce_none", 2, 204, "_reduce_252", 3, 204, "_reduce_253", 2, 202, "_reduce_254", 2, 201, "_reduce_255", 0, 201, "_reduce_256", 1, 194, "_reduce_257", 2, 194, "_reduce_258", 3, 194, "_reduce_259", 4, 194, "_reduce_260", 3, 164, "_reduce_261", 4, 164, "_reduce_262", 2, 164, "_reduce_263", 1, 192, "_reduce_none", 1, 192, "_reduce_none", 1, 192, "_reduce_none", 1, 192, "_reduce_none", 1, 192, "_reduce_none", 1, 192, "_reduce_none", 1, 192, "_reduce_none", 1, 192, "_reduce_none", 1, 192, "_reduce_none", 0, 227, "_reduce_273", 4, 192, "_reduce_274", 4, 192, "_reduce_275", 3, 192, "_reduce_276", 3, 192, "_reduce_277", 2, 192, "_reduce_278", 4, 192, "_reduce_279", 3, 192, "_reduce_280", 3, 192, "_reduce_281", 1, 192, "_reduce_282", 4, 192, "_reduce_283", 3, 192, "_reduce_284", 1, 192, "_reduce_285", 5, 192, "_reduce_286", 4, 192, "_reduce_287", 3, 192, "_reduce_288", 2, 192, "_reduce_289", 1, 192, "_reduce_none", 2, 192, "_reduce_291", 2, 192, "_reduce_292", 6, 192, "_reduce_293", 6, 192, "_reduce_294", 0, 228, "_reduce_295", 0, 229, "_reduce_296", 7, 192, "_reduce_297", 0, 230, "_reduce_298", 0, 231, "_reduce_299", 7, 192, "_reduce_300", 5, 192, "_reduce_301", 4, 192, "_reduce_302", 5, 192, "_reduce_303", 0, 232, "_reduce_304", 0, 233, "_reduce_305", 9, 192, "_reduce_306", 0, 234, "_reduce_307", 6, 192, "_reduce_308", 0, 235, "_reduce_309", 0, 236, "_reduce_310", 8, 192, "_reduce_311", 0, 237, "_reduce_312", 0, 238, "_reduce_313", 6, 192, "_reduce_314", 0, 239, "_reduce_315", 6, 192, "_reduce_316", 0, 240, "_reduce_317", 0, 241, "_reduce_318", 9, 192, "_reduce_319", 1, 192, "_reduce_320", 1, 192, "_reduce_321", 1, 192, "_reduce_322", 1, 192, "_reduce_none", 1, 161, "_reduce_none", 1, 217, "_reduce_none", 1, 217, "_reduce_none", 1, 217, "_reduce_none", 2, 217, "_reduce_none", 1, 219, "_reduce_none", 1, 219, "_reduce_none", 1, 219, "_reduce_none", 2, 216, "_reduce_332", 3, 242, "_reduce_333", 2, 242, "_reduce_334", 1, 242, "_reduce_none", 1, 242, "_reduce_none", 3, 243, "_reduce_337", 3, 243, "_reduce_338", 1, 218, "_reduce_339", 5, 218, "_reduce_340", 1, 151, "_reduce_none", 2, 151, "_reduce_342", 1, 245, "_reduce_343", 3, 245, "_reduce_344", 3, 246, "_reduce_345", 1, 176, "_reduce_none", 2, 176, "_reduce_347", 1, 176, "_reduce_348", 3, 176, "_reduce_349", 1, 247, "_reduce_350", 2, 249, "_reduce_351", 1, 249, "_reduce_352", 6, 244, "_reduce_353", 4, 244, "_reduce_354", 4, 244, "_reduce_355", 2, 244, "_reduce_356", 2, 244, "_reduce_357", 4, 244, "_reduce_358", 2, 244, "_reduce_359", 2, 244, "_reduce_360", 1, 244, "_reduce_361", 0, 253, "_reduce_362", 5, 252, "_reduce_363", 2, 172, "_reduce_364", 4, 172, "_reduce_none", 4, 172, "_reduce_none", 2, 215, "_reduce_367", 4, 215, "_reduce_368", 3, 215, "_reduce_369", 4, 215, "_reduce_370", 3, 215, "_reduce_371", 2, 215, "_reduce_372", 1, 215, "_reduce_373", 0, 255, "_reduce_374", 5, 214, "_reduce_375", 0, 256, "_reduce_376", 5, 214, "_reduce_377", 0, 258, "_reduce_378", 6, 220, "_reduce_379", 1, 257, "_reduce_380", 1, 257, "_reduce_none", 6, 150, "_reduce_382", 0, 150, "_reduce_383", 1, 259, "_reduce_384", 1, 259, "_reduce_none", 1, 259, "_reduce_none", 2, 260, "_reduce_387", 1, 260, "_reduce_388", 2, 152, "_reduce_389", 1, 152, "_reduce_none", 1, 206, "_reduce_none", 1, 206, "_reduce_none", 1, 206, "_reduce_none", 1, 207, "_reduce_394", 1, 263, "_reduce_none", 2, 263, "_reduce_396", 3, 264, "_reduce_397", 1, 264, "_reduce_398", 3, 208, "_reduce_399", 3, 209, "_reduce_400", 3, 210, "_reduce_401", 3, 210, "_reduce_402", 1, 267, "_reduce_403", 3, 267, "_reduce_404", 1, 268, "_reduce_405", 2, 268, "_reduce_406", 3, 211, "_reduce_407", 3, 211, "_reduce_408", 1, 270, "_reduce_409", 3, 270, "_reduce_410", 1, 265, "_reduce_411", 2, 265, "_reduce_412", 1, 266, "_reduce_413", 2, 266, "_reduce_414", 1, 269, "_reduce_415", 0, 272, "_reduce_416", 3, 269, "_reduce_417", 0, 273, "_reduce_418", 4, 269, "_reduce_419", 1, 271, "_reduce_420", 1, 271, "_reduce_421", 1, 271, "_reduce_422", 1, 271, "_reduce_none", 2, 190, "_reduce_424", 1, 190, "_reduce_425", 1, 274, "_reduce_none", 1, 274, "_reduce_none", 1, 274, "_reduce_none", 1, 274, "_reduce_none", 3, 262, "_reduce_430", 1, 261, "_reduce_431", 1, 261, "_reduce_432", 2, 261, "_reduce_433", 2, 261, "_reduce_434", 2, 261, "_reduce_435", 2, 261, "_reduce_436", 1, 184, "_reduce_437", 1, 184, "_reduce_438", 1, 184, "_reduce_439", 1, 184, "_reduce_440", 1, 184, "_reduce_441", 1, 184, "_reduce_442", 1, 184, "_reduce_443", 1, 184, "_reduce_444", 1, 184, "_reduce_445", 1, 184, "_reduce_446", 1, 184, "_reduce_447", 1, 212, "_reduce_448", 1, 160, "_reduce_449", 1, 163, "_reduce_450", 1, 163, "_reduce_none", 1, 222, "_reduce_452", 3, 222, "_reduce_453", 2, 222, "_reduce_454", 4, 224, "_reduce_455", 2, 224, "_reduce_456", 1, 276, "_reduce_none", 1, 276, "_reduce_none", 2, 277, "_reduce_459", 1, 277, "_reduce_460", 1, 278, "_reduce_461", 2, 279, "_reduce_462", 1, 279, "_reduce_463", 1, 280, "_reduce_464", 3, 280, "_reduce_465", 4, 281, "_reduce_466", 2, 281, "_reduce_467", 2, 281, "_reduce_468", 1, 281, "_reduce_469", 2, 283, "_reduce_470", 0, 283, "_reduce_471", 6, 275, "_reduce_472", 4, 275, "_reduce_473", 4, 275, "_reduce_474", 2, 275, "_reduce_475", 4, 275, "_reduce_476", 2, 275, "_reduce_477", 2, 275, "_reduce_478", 1, 275, "_reduce_479", 0, 275, "_reduce_480", 1, 285, "_reduce_none", 1, 285, "_reduce_482", 1, 286, "_reduce_483", 1, 286, "_reduce_484", 1, 286, "_reduce_485", 1, 286, "_reduce_486", 1, 287, "_reduce_487", 3, 287, "_reduce_488", 1, 221, "_reduce_none", 1, 221, "_reduce_none", 1, 289, "_reduce_491", 3, 289, "_reduce_none", 1, 290, "_reduce_493", 3, 290, "_reduce_494", 1, 288, "_reduce_none", 4, 288, "_reduce_none", 3, 288, "_reduce_none", 2, 288, "_reduce_none", 1, 288, "_reduce_none", 1, 250, "_reduce_500", 3, 250, "_reduce_501", 3, 291, "_reduce_502", 1, 284, "_reduce_503", 3, 284, "_reduce_504", 1, 292, "_reduce_none", 1, 292, "_reduce_none", 2, 251, "_reduce_507", 1, 251, "_reduce_508", 1, 293, "_reduce_none", 1, 293, "_reduce_none", 2, 248, "_reduce_511", 2, 282, "_reduce_512", 0, 282, "_reduce_513", 1, 225, "_reduce_514", 4, 225, "_reduce_515", 0, 213, "_reduce_516", 2, 213, "_reduce_517", 1, 196, "_reduce_518", 3, 196, "_reduce_519", 3, 294, "_reduce_520", 2, 294, "_reduce_521", 1, 177, "_reduce_none", 1, 177, "_reduce_none", 1, 177, "_reduce_none", 1, 173, "_reduce_none", 1, 173, "_reduce_none", 1, 173, "_reduce_none", 1, 173, "_reduce_none", 1, 254, "_reduce_none", 1, 254, "_reduce_none", 1, 254, "_reduce_none", 1, 226, "_reduce_none", 1, 226, "_reduce_none", 0, 144, "_reduce_none", 1, 144, "_reduce_none", 0, 191, "_reduce_none", 1, 191, "_reduce_none", 0, 195, "_reduce_none", 1, 195, "_reduce_none", 1, 195, "_reduce_none", 1, 223, "_reduce_none", 1, 223, "_reduce_none", 1, 146, "_reduce_none", 2, 146, "_reduce_none", 0, 193, "_reduce_545"];

      racc_reduce_n = 546;

      racc_shift_n = 941;

      racc_token_table = $hash(false, 0, "error", 1, "kCLASS", 2, "kMODULE", 3, "kDEF", 4, "kUNDEF", 5, "kBEGIN", 6, "kRESCUE", 7, "kENSURE", 8, "kEND", 9, "kIF", 10, "kUNLESS", 11, "kTHEN", 12, "kELSIF", 13, "kELSE", 14, "kCASE", 15, "kWHEN", 16, "kWHILE", 17, "kUNTIL", 18, "kFOR", 19, "kBREAK", 20, "kNEXT", 21, "kREDO", 22, "kRETRY", 23, "kIN", 24, "kDO", 25, "kDO_COND", 26, "kDO_BLOCK", 27, "kDO_LAMBDA", 28, "kRETURN", 29, "kYIELD", 30, "kSUPER", 31, "kSELF", 32, "kNIL", 33, "kTRUE", 34, "kFALSE", 35, "kAND", 36, "kOR", 37, "kNOT", 38, "kIF_MOD", 39, "kUNLESS_MOD", 40, "kWHILE_MOD", 41, "kUNTIL_MOD", 42, "kRESCUE_MOD", 43, "kALIAS", 44, "kDEFINED", 45, "klBEGIN", 46, "klEND", 47, "k__LINE__", 48, "k__FILE__", 49, "k__ENCODING__", 50, "tIDENTIFIER", 51, "tFID", 52, "tGVAR", 53, "tIVAR", 54, "tCONSTANT", 55, "tLABEL", 56, "tCVAR", 57, "tNTH_REF", 58, "tBACK_REF", 59, "tSTRING_CONTENT", 60, "tINTEGER", 61, "tFLOAT", 62, "tREGEXP_END", 63, "tUPLUS", 64, "tUMINUS", 65, "tUMINUS_NUM", 66, "tPOW", 67, "tCMP", 68, "tEQ", 69, "tEQQ", 70, "tNEQ", 71, "tGEQ", 72, "tLEQ", 73, "tANDOP", 74, "tOROP", 75, "tMATCH", 76, "tNMATCH", 77, "tDOT", 78, "tDOT2", 79, "tDOT3", 80, "tAREF", 81, "tASET", 82, "tLSHFT", 83, "tRSHFT", 84, "tCOLON2", 85, "tCOLON3", 86, "tOP_ASGN", 87, "tASSOC", 88, "tLPAREN", 89, "tLPAREN2", 90, "tRPAREN", 91, "tLPAREN_ARG", 92, "ARRAY_BEG", 93, "tRBRACK", 94, "tLBRACE", 95, "tLBRACE_ARG", 96, "tSTAR", 97, "tSTAR2", 98, "tAMPER", 99, "tAMPER2", 100, "tTILDE", 101, "tPERCENT", 102, "tDIVIDE", 103, "tPLUS", 104, "tMINUS", 105, "tLT", 106, "tGT", 107, "tPIPE", 108, "tBANG", 109, "tCARET", 110, "tLCURLY", 111, "tRCURLY", 112, "tBACK_REF2", 113, "tSYMBEG", 114, "tSTRING_BEG", 115, "tXSTRING_BEG", 116, "tREGEXP_BEG", 117, "tWORDS_BEG", 118, "tAWORDS_BEG", 119, "tSTRING_DBEG", 120, "tSTRING_DVAR", 121, "tSTRING_END", 122, "tSTRING", 123, "tSYMBOL", 124, "tNL", 125, "tEH", 126, "tCOLON", 127, "tCOMMA", 128, "tSPACE", 129, "tSEMI", 130, "tLAMBDA", 131, "tLAMBEG", 132, "tLBRACK2", 133, "tLBRACK", 134, "tDSTAR", 135, "tEQL", 136, "tLOWEST", 137, "-@NUM", 138, "+@NUM", 139);

      racc_nt_base = 140;

      racc_use_result_var = true;

      Opal.cdecl($scope, 'Racc_arg', [racc_action_table, racc_action_check, racc_action_default, racc_action_pointer, racc_goto_table, racc_goto_check, racc_goto_default, racc_goto_pointer, racc_nt_base, racc_reduce_table, racc_token_table, racc_shift_n, racc_reduce_n, racc_use_result_var]);

      Opal.cdecl($scope, 'Racc_token_to_s_table', ["$end", "error", "kCLASS", "kMODULE", "kDEF", "kUNDEF", "kBEGIN", "kRESCUE", "kENSURE", "kEND", "kIF", "kUNLESS", "kTHEN", "kELSIF", "kELSE", "kCASE", "kWHEN", "kWHILE", "kUNTIL", "kFOR", "kBREAK", "kNEXT", "kREDO", "kRETRY", "kIN", "kDO", "kDO_COND", "kDO_BLOCK", "kDO_LAMBDA", "kRETURN", "kYIELD", "kSUPER", "kSELF", "kNIL", "kTRUE", "kFALSE", "kAND", "kOR", "kNOT", "kIF_MOD", "kUNLESS_MOD", "kWHILE_MOD", "kUNTIL_MOD", "kRESCUE_MOD", "kALIAS", "kDEFINED", "klBEGIN", "klEND", "k__LINE__", "k__FILE__", "k__ENCODING__", "tIDENTIFIER", "tFID", "tGVAR", "tIVAR", "tCONSTANT", "tLABEL", "tCVAR", "tNTH_REF", "tBACK_REF", "tSTRING_CONTENT", "tINTEGER", "tFLOAT", "tREGEXP_END", "tUPLUS", "tUMINUS", "tUMINUS_NUM", "tPOW", "tCMP", "tEQ", "tEQQ", "tNEQ", "tGEQ", "tLEQ", "tANDOP", "tOROP", "tMATCH", "tNMATCH", "tDOT", "tDOT2", "tDOT3", "tAREF", "tASET", "tLSHFT", "tRSHFT", "tCOLON2", "tCOLON3", "tOP_ASGN", "tASSOC", "tLPAREN", "tLPAREN2", "tRPAREN", "tLPAREN_ARG", "ARRAY_BEG", "tRBRACK", "tLBRACE", "tLBRACE_ARG", "tSTAR", "tSTAR2", "tAMPER", "tAMPER2", "tTILDE", "tPERCENT", "tDIVIDE", "tPLUS", "tMINUS", "tLT", "tGT", "tPIPE", "tBANG", "tCARET", "tLCURLY", "tRCURLY", "tBACK_REF2", "tSYMBEG", "tSTRING_BEG", "tXSTRING_BEG", "tREGEXP_BEG", "tWORDS_BEG", "tAWORDS_BEG", "tSTRING_DBEG", "tSTRING_DVAR", "tSTRING_END", "tSTRING", "tSYMBOL", "tNL", "tEH", "tCOLON", "tCOMMA", "tSPACE", "tSEMI", "tLAMBDA", "tLAMBEG", "tLBRACK2", "tLBRACK", "tDSTAR", "tEQL", "tLOWEST", "\"-@NUM\"", "\"+@NUM\"", "$start", "program", "top_compstmt", "top_stmts", "opt_terms", "top_stmt", "terms", "stmt", "bodystmt", "compstmt", "opt_rescue", "opt_else", "opt_ensure", "stmts", "fitem", "undef_list", "expr_value", "lhs", "command_call", "mlhs", "var_lhs", "primary_value", "aref_args", "backref", "mrhs", "arg_value", "expr", "@1", "arg", "command", "block_command", "call_args", "block_call", "operation2", "command_args", "cmd_brace_block", "opt_block_var", "operation", "mlhs_basic", "mlhs_entry", "mlhs_head", "mlhs_item", "mlhs_node", "mlhs_post", "variable", "cname", "cpath", "fname", "op", "reswords", "symbol", "opt_nl", "primary", "none", "args", "trailer", "assocs", "paren_args", "opt_call_args", "rparen", "opt_paren_args", "opt_block_arg", "block_arg", "call_args2", "open_args", "@2", "literal", "strings", "xstring", "regexp", "words", "awords", "var_ref", "assoc_list", "brace_block", "method_call", "lambda", "then", "if_tail", "do", "case_body", "for_var", "superclass", "term", "f_arglist", "singleton", "dot_or_colon", "@3", "@4", "@5", "@6", "@7", "@8", "@9", "@10", "@11", "@12", "@13", "@14", "@15", "@16", "@17", "f_larglist", "lambda_body", "block_param", "f_block_optarg", "f_block_opt", "block_args_tail", "f_block_arg", "opt_block_args_tail", "f_arg", "f_rest_arg", "do_block", "@18", "operation3", "@19", "@20", "cases", "@21", "exc_list", "exc_var", "numeric", "dsym", "string", "string1", "string_contents", "xstring_contents", "word_list", "word", "string_content", "qword_list", "string_dvar", "@22", "@23", "sym", "f_args", "kwrest_mark", "f_kwrest", "f_label", "f_kw", "f_kwarg", "args_tail", "opt_f_block_arg", "opt_args_tail", "f_optarg", "f_norm_arg", "f_bad_arg", "f_arg_item", "f_margs", "f_marg", "f_marg_list", "f_opt", "restarg_mark", "blkarg_mark", "assoc"]);

      Opal.cdecl($scope, 'Racc_debug_parser', false);

      def.$_reduce_2 = function(val, _values, result) {
        var self = this;

        result = self.$new_compstmt(val['$[]'](0));
        return result;
      };

      def.$_reduce_3 = function(val, _values, result) {
        var self = this;

        result = self.$new_block();
        return result;
      };

      def.$_reduce_4 = function(val, _values, result) {
        var self = this;

        result = self.$new_block(val['$[]'](0));
        return result;
      };

      def.$_reduce_5 = function(val, _values, result) {
        var self = this;

        val['$[]'](0)['$<<'](val['$[]'](2));
        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_7 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](2);
        return result;
      };

      def.$_reduce_8 = function(val, _values, result) {
        var self = this;

        result = self.$new_body(val['$[]'](0), val['$[]'](1), val['$[]'](2), val['$[]'](3));
        return result;
      };

      def.$_reduce_9 = function(val, _values, result) {
        var self = this;

        result = self.$new_compstmt(val['$[]'](0));
        return result;
      };

      def.$_reduce_10 = function(val, _values, result) {
        var self = this;

        result = self.$new_block();
        return result;
      };

      def.$_reduce_11 = function(val, _values, result) {
        var self = this;

        result = self.$new_block(val['$[]'](0));
        return result;
      };

      def.$_reduce_12 = function(val, _values, result) {
        var self = this;

        val['$[]'](0)['$<<'](val['$[]'](2));
        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_13 = function(val, _values, result) {
        var $a, $b, self = this;

        (($a = ["expr_fname"]), $b = self.$lexer(), $b['$lex_state='].apply($b, $a), $a[$a.length-1]);
        return result;
      };

      def.$_reduce_14 = function(val, _values, result) {
        var self = this;

        result = self.$new_alias(val['$[]'](0), val['$[]'](1), val['$[]'](3));
        return result;
      };

      def.$_reduce_15 = function(val, _values, result) {
        var self = this;

        result = self.$s("valias", self.$value(val['$[]'](1)).$to_sym(), self.$value(val['$[]'](2)).$to_sym());
        return result;
      };

      def.$_reduce_17 = function(val, _values, result) {
        var self = this;

        result = self.$s("valias", self.$value(val['$[]'](1)).$to_sym(), self.$value(val['$[]'](2)).$to_sym());
        return result;
      };

      def.$_reduce_18 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_19 = function(val, _values, result) {
        var self = this;

        result = self.$new_if(val['$[]'](1), val['$[]'](2), val['$[]'](0), nil);
        return result;
      };

      def.$_reduce_20 = function(val, _values, result) {
        var self = this;

        result = self.$new_if(val['$[]'](1), val['$[]'](2), nil, val['$[]'](0));
        return result;
      };

      def.$_reduce_21 = function(val, _values, result) {
        var self = this;

        result = self.$new_while(val['$[]'](1), val['$[]'](2), val['$[]'](0));
        return result;
      };

      def.$_reduce_22 = function(val, _values, result) {
        var self = this;

        result = self.$new_until(val['$[]'](1), val['$[]'](2), val['$[]'](0));
        return result;
      };

      def.$_reduce_23 = function(val, _values, result) {
        var self = this;

        result = self.$new_rescue_mod(val['$[]'](1), val['$[]'](0), val['$[]'](2));
        return result;
      };

      def.$_reduce_25 = function(val, _values, result) {
        var self = this;

        result = self.$new_assign(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_26 = function(val, _values, result) {
        var self = this;

        result = self.$s("masgn", val['$[]'](0), self.$s("to_ary", val['$[]'](2)));
        return result;
      };

      def.$_reduce_27 = function(val, _values, result) {
        var self = this;

        result = self.$new_op_asgn(val['$[]'](1), val['$[]'](0), val['$[]'](2));
        return result;
      };

      def.$_reduce_29 = function(val, _values, result) {
        var self = this;

        result = self.$s("op_asgn2", val['$[]'](0), self.$op_to_setter(val['$[]'](2)), self.$value(val['$[]'](3)).$to_sym(), val['$[]'](4));
        return result;
      };

      def.$_reduce_33 = function(val, _values, result) {
        var self = this;

        result = self.$new_assign(val['$[]'](0), val['$[]'](1), self.$s("svalue", val['$[]'](2)));
        return result;
      };

      def.$_reduce_34 = function(val, _values, result) {
        var self = this;

        result = self.$s("masgn", val['$[]'](0), self.$s("to_ary", val['$[]'](2)));
        return result;
      };

      def.$_reduce_35 = function(val, _values, result) {
        var self = this;

        result = self.$s("masgn", val['$[]'](0), val['$[]'](2));
        return result;
      };

      def.$_reduce_38 = function(val, _values, result) {
        var self = this;

        result = self.$s("and", val['$[]'](0), val['$[]'](2));
        return result;
      };

      def.$_reduce_39 = function(val, _values, result) {
        var self = this;

        result = self.$s("or", val['$[]'](0), val['$[]'](2));
        return result;
      };

      def.$_reduce_40 = function(val, _values, result) {
        var self = this;

        result = self.$new_unary_call(["!", []], val['$[]'](1));
        return result;
      };

      def.$_reduce_41 = function(val, _values, result) {
        var self = this;

        result = self.$new_unary_call(val['$[]'](0), val['$[]'](1));
        return result;
      };

      def.$_reduce_46 = function(val, _values, result) {
        var self = this;

        result = self.$new_return(val['$[]'](0), val['$[]'](1));
        return result;
      };

      def.$_reduce_47 = function(val, _values, result) {
        var self = this;

        result = self.$new_break(val['$[]'](0), val['$[]'](1));
        return result;
      };

      def.$_reduce_48 = function(val, _values, result) {
        var self = this;

        result = self.$new_next(val['$[]'](0), val['$[]'](1));
        return result;
      };

      def.$_reduce_53 = function(val, _values, result) {
        var self = this;

        result = self.$new_call(nil, val['$[]'](0), val['$[]'](1));
        return result;
      };

      def.$_reduce_55 = function(val, _values, result) {
        var self = this;

        result = self.$new_call(val['$[]'](0), val['$[]'](2), val['$[]'](3));
        return result;
      };

      def.$_reduce_57 = function(val, _values, result) {
        var self = this;

        result = self.$new_call(val['$[]'](0), val['$[]'](2), val['$[]'](3));
        return result;
      };

      def.$_reduce_59 = function(val, _values, result) {
        var self = this;

        result = self.$new_super(val['$[]'](0), val['$[]'](1));
        return result;
      };

      def.$_reduce_60 = function(val, _values, result) {
        var self = this;

        result = self.$new_yield(val['$[]'](1));
        return result;
      };

      def.$_reduce_61 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_62 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_63 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_64 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_65 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_66 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0)['$<<'](val['$[]'](1));
        return result;
      };

      def.$_reduce_67 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0)['$<<'](self.$s("splat", val['$[]'](2)));
        return result;
      };

      def.$_reduce_69 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0)['$<<'](self.$s("splat"));
        return result;
      };

      def.$_reduce_71 = function(val, _values, result) {
        var self = this;

        result = self.$s("array", self.$s("splat", val['$[]'](1)));
        return result;
      };

      def.$_reduce_72 = function(val, _values, result) {
        var self = this;

        result = self.$s("array", self.$s("splat"));
        return result;
      };

      def.$_reduce_74 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_75 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_76 = function(val, _values, result) {
        var self = this;

        result = self.$s("array", val['$[]'](0));
        return result;
      };

      def.$_reduce_77 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0)['$<<'](val['$[]'](1));
        return result;
      };

      def.$_reduce_80 = function(val, _values, result) {
        var self = this;

        result = self.$new_assignable(val['$[]'](0));
        return result;
      };

      def.$_reduce_81 = function(val, _values, result) {
        var $a, self = this, args = nil;

        args = (function() {if ((($a = val['$[]'](2)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return val['$[]'](2)
          } else {
          return []
        }; return nil; })();
        result = self.$s("attrasgn", val['$[]'](0), "[]=", ($a = self).$s.apply($a, ["arglist"].concat(args)));
        return result;
      };

      def.$_reduce_82 = function(val, _values, result) {
        var self = this;

        result = self.$new_call(val['$[]'](0), val['$[]'](2), []);
        return result;
      };

      def.$_reduce_88 = function(val, _values, result) {
        var self = this;

        result = self.$new_assignable(val['$[]'](0));
        return result;
      };

      def.$_reduce_89 = function(val, _values, result) {
        var self = this;

        result = self.$new_attrasgn(val['$[]'](0), "[]=", val['$[]'](2));
        return result;
      };

      def.$_reduce_90 = function(val, _values, result) {
        var self = this;

        result = self.$new_attrasgn(val['$[]'](0), self.$op_to_setter(val['$[]'](2)));
        return result;
      };

      def.$_reduce_91 = function(val, _values, result) {
        var self = this;

        result = self.$new_attrasgn(val['$[]'](0), self.$op_to_setter(val['$[]'](2)));
        return result;
      };

      def.$_reduce_92 = function(val, _values, result) {
        var self = this;

        result = self.$new_attrasgn(val['$[]'](0), self.$op_to_setter(val['$[]'](2)));
        return result;
      };

      def.$_reduce_93 = function(val, _values, result) {
        var self = this;

        result = self.$new_colon2(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_94 = function(val, _values, result) {
        var self = this;

        result = self.$new_colon3(val['$[]'](0), val['$[]'](1));
        return result;
      };

      def.$_reduce_97 = function(val, _values, result) {
        var self = this;

        result = self.$new_colon3(val['$[]'](0), val['$[]'](1));
        return result;
      };

      def.$_reduce_98 = function(val, _values, result) {
        var self = this;

        result = self.$new_const(val['$[]'](0));
        return result;
      };

      def.$_reduce_99 = function(val, _values, result) {
        var self = this;

        result = self.$new_colon2(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_103 = function(val, _values, result) {
        var $a, $b, self = this;

        (($a = ["expr_end"]), $b = self.$lexer(), $b['$lex_state='].apply($b, $a), $a[$a.length-1]);
        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_104 = function(val, _values, result) {
        var $a, $b, self = this;

        (($a = ["expr_end"]), $b = self.$lexer(), $b['$lex_state='].apply($b, $a), $a[$a.length-1]);
        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_105 = function(val, _values, result) {
        var self = this;

        result = self.$new_sym(val['$[]'](0));
        return result;
      };

      def.$_reduce_107 = function(val, _values, result) {
        var self = this;

        result = self.$s("undef", val['$[]'](0));
        return result;
      };

      def.$_reduce_108 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0)['$<<'](val['$[]'](2));
        return result;
      };

      def.$_reduce_183 = function(val, _values, result) {
        var self = this;

        result = self.$new_assign(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_184 = function(val, _values, result) {
        var self = this;

        result = self.$new_assign(val['$[]'](0), val['$[]'](1), self.$s("rescue_mod", val['$[]'](2), val['$[]'](4)));
        return result;
      };

      def.$_reduce_185 = function(val, _values, result) {
        var self = this;

        result = self.$new_op_asgn(val['$[]'](1), val['$[]'](0), val['$[]'](2));
        return result;
      };

      def.$_reduce_186 = function(val, _values, result) {
        var self = this;

        result = self.$new_op_asgn1(val['$[]'](0), val['$[]'](2), val['$[]'](4), val['$[]'](5));
        return result;
      };

      def.$_reduce_187 = function(val, _values, result) {
        var self = this;

        result = self.$s("op_asgn2", val['$[]'](0), self.$op_to_setter(val['$[]'](2)), self.$value(val['$[]'](3)).$to_sym(), val['$[]'](4));
        return result;
      };

      def.$_reduce_193 = function(val, _values, result) {
        var self = this;

        result = self.$new_irange(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_194 = function(val, _values, result) {
        var self = this;

        result = self.$new_erange(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_195 = function(val, _values, result) {
        var self = this;

        result = self.$new_binary_call(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_196 = function(val, _values, result) {
        var self = this;

        result = self.$new_binary_call(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_197 = function(val, _values, result) {
        var self = this;

        result = self.$new_binary_call(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_198 = function(val, _values, result) {
        var self = this;

        result = self.$new_binary_call(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_199 = function(val, _values, result) {
        var self = this;

        result = self.$new_binary_call(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_200 = function(val, _values, result) {
        var self = this;

        result = self.$new_binary_call(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_201 = function(val, _values, result) {
        var self = this;

        result = self.$new_call(self.$new_binary_call(self.$new_int(val['$[]'](1)), val['$[]'](2), val['$[]'](3)), ["-@", []], []);
        return result;
      };

      def.$_reduce_202 = function(val, _values, result) {
        var self = this;

        result = self.$new_call(self.$new_binary_call(self.$new_float(val['$[]'](1)), val['$[]'](2), val['$[]'](3)), ["-@", []], []);
        return result;
      };

      def.$_reduce_203 = function(val, _values, result) {
        var $a, self = this;

        result = self.$new_call(val['$[]'](1), ["+@", []], []);
        if ((($a = ["int", "float"]['$include?'](val['$[]'](1).$type())) !== nil && (!$a.$$is_boolean || $a == true))) {
          result = val['$[]'](1)};
        return result;
      };

      def.$_reduce_204 = function(val, _values, result) {
        var self = this;

        result = self.$new_call(val['$[]'](1), ["-@", []], []);
        if (val['$[]'](1).$type()['$==']("int")) {
          val['$[]'](1)['$[]='](1, val['$[]'](1)['$[]'](1)['$-@']());
          result = val['$[]'](1);
        } else if (val['$[]'](1).$type()['$==']("float")) {
          val['$[]'](1)['$[]='](1, val['$[]'](1)['$[]'](1).$to_f()['$-@']());
          result = val['$[]'](1);};
        return result;
      };

      def.$_reduce_205 = function(val, _values, result) {
        var self = this;

        result = self.$new_binary_call(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_206 = function(val, _values, result) {
        var self = this;

        result = self.$new_binary_call(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_207 = function(val, _values, result) {
        var self = this;

        result = self.$new_binary_call(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_208 = function(val, _values, result) {
        var self = this;

        result = self.$new_binary_call(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_209 = function(val, _values, result) {
        var self = this;

        result = self.$new_binary_call(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_210 = function(val, _values, result) {
        var self = this;

        result = self.$new_binary_call(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_211 = function(val, _values, result) {
        var self = this;

        result = self.$new_binary_call(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_212 = function(val, _values, result) {
        var self = this;

        result = self.$new_binary_call(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_213 = function(val, _values, result) {
        var self = this;

        result = self.$new_binary_call(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_214 = function(val, _values, result) {
        var self = this;

        result = self.$new_binary_call(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_215 = function(val, _values, result) {
        var self = this;

        result = self.$new_unary_call(["!", []], self.$new_binary_call(val['$[]'](0), ["==", []], val['$[]'](2)));
        return result;
      };

      def.$_reduce_216 = function(val, _values, result) {
        var self = this;

        result = self.$new_binary_call(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_217 = function(val, _values, result) {
        var self = this;

        result = self.$new_not(val['$[]'](1), self.$new_binary_call(val['$[]'](0), ["=~", []], val['$[]'](2)));
        return result;
      };

      def.$_reduce_218 = function(val, _values, result) {
        var self = this;

        result = self.$new_unary_call(val['$[]'](0), val['$[]'](1));
        return result;
      };

      def.$_reduce_219 = function(val, _values, result) {
        var self = this;

        result = self.$new_unary_call(val['$[]'](0), val['$[]'](1));
        return result;
      };

      def.$_reduce_220 = function(val, _values, result) {
        var self = this;

        result = self.$new_binary_call(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_221 = function(val, _values, result) {
        var self = this;

        result = self.$new_binary_call(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_222 = function(val, _values, result) {
        var self = this;

        result = self.$new_and(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_223 = function(val, _values, result) {
        var self = this;

        result = self.$new_or(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_224 = function(val, _values, result) {
        var self = this;

        result = self.$s("defined", val['$[]'](2));
        return result;
      };

      def.$_reduce_225 = function(val, _values, result) {
        var self = this;

        result = self.$new_if(val['$[]'](1), val['$[]'](0), val['$[]'](2), val['$[]'](4));
        return result;
      };

      def.$_reduce_228 = function(val, _values, result) {
        var self = this;

        result = nil;
        return result;
      };

      def.$_reduce_229 = function(val, _values, result) {
        var self = this;

        result = [val['$[]'](0)];
        return result;
      };

      def.$_reduce_230 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_231 = function(val, _values, result) {
        var $a, self = this;

        val['$[]'](0)['$<<'](($a = self).$s.apply($a, ["hash"].concat(val['$[]'](2))));
        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_232 = function(val, _values, result) {
        var $a, self = this;

        result = [($a = self).$s.apply($a, ["hash"].concat(val['$[]'](0)))];
        return result;
      };

      def.$_reduce_233 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_235 = function(val, _values, result) {
        var self = this;

        result = [];
        return result;
      };

      def.$_reduce_237 = function(val, _values, result) {
        var self = this;

        result = [];
        return result;
      };

      def.$_reduce_239 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_240 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0);
        result['$<<'](self.$new_hash(nil, val['$[]'](2), nil));
        return result;
      };

      def.$_reduce_241 = function(val, _values, result) {
        var self = this;

        result = [self.$new_hash(nil, val['$[]'](0), nil)];
        return result;
      };

      def.$_reduce_242 = function(val, _values, result) {
        var self = this;

        result = [val['$[]'](0)];
        return result;
      };

      def.$_reduce_243 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0);
        self.$add_block_pass(val['$[]'](0), val['$[]'](1));
        return result;
      };

      def.$_reduce_244 = function(val, _values, result) {
        var self = this;

        result = [self.$new_hash(nil, val['$[]'](0), nil)];
        self.$add_block_pass(result, val['$[]'](1));
        return result;
      };

      def.$_reduce_245 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0);
        result['$<<'](self.$new_hash(nil, val['$[]'](2), nil));
        return result;
      };

      def.$_reduce_246 = function(val, _values, result) {
        var self = this;

        result = [];
        self.$add_block_pass(result, val['$[]'](0));
        return result;
      };

      def.$_reduce_249 = function(val, _values, result) {
        var self = this;

        self.$lexer().$cmdarg_push(1);
        return result;
      };

      def.$_reduce_250 = function(val, _values, result) {
        var self = this;

        self.$lexer().$cmdarg_pop();
        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_252 = function(val, _values, result) {
        var self = this;

        result = nil;
        return result;
      };

      def.$_reduce_253 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_254 = function(val, _values, result) {
        var self = this;

        result = self.$new_block_pass(val['$[]'](0), val['$[]'](1));
        return result;
      };

      def.$_reduce_255 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_256 = function(val, _values, result) {
        var self = this;

        result = nil;
        return result;
      };

      def.$_reduce_257 = function(val, _values, result) {
        var self = this;

        result = [val['$[]'](0)];
        return result;
      };

      def.$_reduce_258 = function(val, _values, result) {
        var self = this;

        result = [self.$new_splat(val['$[]'](0), val['$[]'](1))];
        return result;
      };

      def.$_reduce_259 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0)['$<<'](val['$[]'](2));
        return result;
      };

      def.$_reduce_260 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0)['$<<'](self.$new_splat(val['$[]'](2), val['$[]'](3)));
        return result;
      };

      def.$_reduce_261 = function(val, _values, result) {
        var $a, self = this;

        val['$[]'](0)['$<<'](val['$[]'](2));
        result = ($a = self).$s.apply($a, ["array"].concat(val['$[]'](0)));
        return result;
      };

      def.$_reduce_262 = function(val, _values, result) {
        var $a, self = this;

        val['$[]'](0)['$<<'](self.$s("splat", val['$[]'](3)));
        result = ($a = self).$s.apply($a, ["array"].concat(val['$[]'](0)));
        return result;
      };

      def.$_reduce_263 = function(val, _values, result) {
        var self = this;

        result = self.$s("splat", val['$[]'](1));
        return result;
      };

      def.$_reduce_273 = function(val, _values, result) {
        var self = this;

        result = self.$lexer().$line();
        return result;
      };

      def.$_reduce_274 = function(val, _values, result) {
        var self = this;

        result = self.$s("begin", val['$[]'](2));
        return result;
      };

      def.$_reduce_275 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_276 = function(val, _values, result) {
        var self = this;

        result = self.$new_paren(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_277 = function(val, _values, result) {
        var self = this;

        result = self.$new_colon2(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_278 = function(val, _values, result) {
        var self = this;

        result = self.$new_colon3(val['$[]'](0), val['$[]'](1));
        return result;
      };

      def.$_reduce_279 = function(val, _values, result) {
        var self = this;

        result = self.$new_call(val['$[]'](0), ["[]", []], val['$[]'](2));
        return result;
      };

      def.$_reduce_280 = function(val, _values, result) {
        var self = this;

        result = self.$new_array(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_281 = function(val, _values, result) {
        var self = this;

        result = self.$new_hash(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_282 = function(val, _values, result) {
        var self = this;

        result = self.$new_return(val['$[]'](0));
        return result;
      };

      def.$_reduce_283 = function(val, _values, result) {
        var self = this;

        result = self.$new_yield(val['$[]'](2));
        return result;
      };

      def.$_reduce_284 = function(val, _values, result) {
        var self = this;

        result = self.$s("yield");
        return result;
      };

      def.$_reduce_285 = function(val, _values, result) {
        var self = this;

        result = self.$s("yield");
        return result;
      };

      def.$_reduce_286 = function(val, _values, result) {
        var self = this;

        result = self.$s("defined", val['$[]'](3));
        return result;
      };

      def.$_reduce_287 = function(val, _values, result) {
        var self = this;

        result = self.$new_unary_call(["!", []], val['$[]'](2));
        return result;
      };

      def.$_reduce_288 = function(val, _values, result) {
        var self = this;

        result = self.$new_unary_call(["!", []], self.$new_nil(val['$[]'](0)));
        return result;
      };

      def.$_reduce_289 = function(val, _values, result) {
        var self = this;

        result = self.$new_call(nil, val['$[]'](0), []);
        result['$<<'](val['$[]'](1));
        return result;
      };

      def.$_reduce_291 = function(val, _values, result) {
        var self = this;

        val['$[]'](0)['$<<'](val['$[]'](1));
        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_292 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_293 = function(val, _values, result) {
        var self = this;

        result = self.$new_if(val['$[]'](0), val['$[]'](1), val['$[]'](3), val['$[]'](4));
        return result;
      };

      def.$_reduce_294 = function(val, _values, result) {
        var self = this;

        result = self.$new_if(val['$[]'](0), val['$[]'](1), val['$[]'](4), val['$[]'](3));
        return result;
      };

      def.$_reduce_295 = function(val, _values, result) {
        var self = this;

        self.$lexer().$cond_push(1);
        result = self.$lexer().$line();
        return result;
      };

      def.$_reduce_296 = function(val, _values, result) {
        var self = this;

        self.$lexer().$cond_pop();
        return result;
      };

      def.$_reduce_297 = function(val, _values, result) {
        var self = this;

        result = self.$s("while", val['$[]'](2), val['$[]'](5));
        return result;
      };

      def.$_reduce_298 = function(val, _values, result) {
        var self = this;

        self.$lexer().$cond_push(1);
        result = self.$lexer().$line();
        return result;
      };

      def.$_reduce_299 = function(val, _values, result) {
        var self = this;

        self.$lexer().$cond_pop();
        return result;
      };

      def.$_reduce_300 = function(val, _values, result) {
        var self = this;

        result = self.$s("until", val['$[]'](2), val['$[]'](5));
        return result;
      };

      def.$_reduce_301 = function(val, _values, result) {
        var $a, self = this;

        result = ($a = self).$s.apply($a, ["case", val['$[]'](1)].concat(val['$[]'](3)));
        return result;
      };

      def.$_reduce_302 = function(val, _values, result) {
        var $a, self = this;

        result = ($a = self).$s.apply($a, ["case", nil].concat(val['$[]'](2)));
        return result;
      };

      def.$_reduce_303 = function(val, _values, result) {
        var self = this;

        result = self.$s("case", nil, val['$[]'](3));
        return result;
      };

      def.$_reduce_304 = function(val, _values, result) {
        var self = this;

        self.$lexer().$cond_push(1);
        result = self.$lexer().$line();
        return result;
      };

      def.$_reduce_305 = function(val, _values, result) {
        var self = this;

        self.$lexer().$cond_pop();
        return result;
      };

      def.$_reduce_306 = function(val, _values, result) {
        var self = this;

        result = self.$s("for", val['$[]'](4), val['$[]'](1), val['$[]'](7));
        return result;
      };

      def.$_reduce_307 = function(val, _values, result) {
        var self = this;

        return result;
      };

      def.$_reduce_308 = function(val, _values, result) {
        var self = this;

        result = self.$new_class(val['$[]'](0), val['$[]'](1), val['$[]'](2), val['$[]'](4), val['$[]'](5));
        return result;
      };

      def.$_reduce_309 = function(val, _values, result) {
        var self = this;

        result = self.$lexer().$line();
        return result;
      };

      def.$_reduce_310 = function(val, _values, result) {
        var self = this;

        return result;
      };

      def.$_reduce_311 = function(val, _values, result) {
        var self = this;

        result = self.$new_sclass(val['$[]'](0), val['$[]'](3), val['$[]'](6), val['$[]'](7));
        return result;
      };

      def.$_reduce_312 = function(val, _values, result) {
        var self = this;

        result = self.$lexer().$line();
        return result;
      };

      def.$_reduce_313 = function(val, _values, result) {
        var self = this;

        return result;
      };

      def.$_reduce_314 = function(val, _values, result) {
        var self = this;

        result = self.$new_module(val['$[]'](0), val['$[]'](2), val['$[]'](4), val['$[]'](5));
        return result;
      };

      def.$_reduce_315 = function(val, _values, result) {
        var $a, $b, self = this;

        self.$push_scope();
        (($a = ["expr_endfn"]), $b = self.$lexer(), $b['$lex_state='].apply($b, $a), $a[$a.length-1]);
        return result;
      };

      def.$_reduce_316 = function(val, _values, result) {
        var self = this;

        result = self.$new_def(val['$[]'](0), nil, val['$[]'](1), val['$[]'](3), val['$[]'](4), val['$[]'](5));
        self.$pop_scope();
        return result;
      };

      def.$_reduce_317 = function(val, _values, result) {
        var $a, $b, self = this;

        (($a = ["expr_fname"]), $b = self.$lexer(), $b['$lex_state='].apply($b, $a), $a[$a.length-1]);
        return result;
      };

      def.$_reduce_318 = function(val, _values, result) {
        var $a, $b, self = this;

        self.$push_scope();
        (($a = ["expr_endfn"]), $b = self.$lexer(), $b['$lex_state='].apply($b, $a), $a[$a.length-1]);
        return result;
      };

      def.$_reduce_319 = function(val, _values, result) {
        var self = this;

        result = self.$new_def(val['$[]'](0), val['$[]'](1), val['$[]'](4), val['$[]'](6), val['$[]'](7), val['$[]'](8));
        self.$pop_scope();
        return result;
      };

      def.$_reduce_320 = function(val, _values, result) {
        var self = this;

        result = self.$new_break(val['$[]'](0));
        return result;
      };

      def.$_reduce_321 = function(val, _values, result) {
        var self = this;

        result = self.$s("next");
        return result;
      };

      def.$_reduce_322 = function(val, _values, result) {
        var self = this;

        result = self.$s("redo");
        return result;
      };

      def.$_reduce_332 = function(val, _values, result) {
        var self = this;

        result = self.$new_call(nil, ["lambda", []], []);
        result['$<<'](self.$new_iter(val['$[]'](0), val['$[]'](1)));
        return result;
      };

      def.$_reduce_333 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_334 = function(val, _values, result) {
        var self = this;

        result = nil;
        return result;
      };

      def.$_reduce_337 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_338 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_339 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_340 = function(val, _values, result) {
        var self = this;

        result = self.$new_if(val['$[]'](0), val['$[]'](1), val['$[]'](3), val['$[]'](4));
        return result;
      };

      def.$_reduce_342 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_343 = function(val, _values, result) {
        var self = this;

        result = self.$s("block", val['$[]'](0));
        return result;
      };

      def.$_reduce_344 = function(val, _values, result) {
        var self = this;

        val['$[]'](0)['$<<'](val['$[]'](2));
        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_345 = function(val, _values, result) {
        var self = this;

        result = self.$new_assign(self.$new_assignable(self.$new_ident(val['$[]'](0))), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_347 = function(val, _values, result) {
        var self = this;

        result = nil;
        return result;
      };

      def.$_reduce_348 = function(val, _values, result) {
        var self = this;

        result = nil;
        return result;
      };

      def.$_reduce_349 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_350 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_351 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_352 = function(val, _values, result) {
        var self = this;

        nil;
        return result;
      };

      def.$_reduce_353 = function(val, _values, result) {
        var self = this;

        result = self.$new_block_args(val['$[]'](0), val['$[]'](2), val['$[]'](4), val['$[]'](5));
        return result;
      };

      def.$_reduce_354 = function(val, _values, result) {
        var self = this;

        result = self.$new_block_args(val['$[]'](0), val['$[]'](2), nil, val['$[]'](3));
        return result;
      };

      def.$_reduce_355 = function(val, _values, result) {
        var self = this;

        result = self.$new_block_args(val['$[]'](0), nil, val['$[]'](2), val['$[]'](3));
        return result;
      };

      def.$_reduce_356 = function(val, _values, result) {
        var self = this;

        result = self.$new_block_args(val['$[]'](0), nil, nil, nil);
        return result;
      };

      def.$_reduce_357 = function(val, _values, result) {
        var self = this;

        result = self.$new_block_args(val['$[]'](0), nil, nil, val['$[]'](1));
        return result;
      };

      def.$_reduce_358 = function(val, _values, result) {
        var self = this;

        result = self.$new_block_args(nil, val['$[]'](0), val['$[]'](2), val['$[]'](3));
        return result;
      };

      def.$_reduce_359 = function(val, _values, result) {
        var self = this;

        result = self.$new_block_args(nil, val['$[]'](0), nil, val['$[]'](1));
        return result;
      };

      def.$_reduce_360 = function(val, _values, result) {
        var self = this;

        result = self.$new_block_args(nil, nil, val['$[]'](0), val['$[]'](1));
        return result;
      };

      def.$_reduce_361 = function(val, _values, result) {
        var self = this;

        result = self.$new_block_args(nil, nil, nil, val['$[]'](0));
        return result;
      };

      def.$_reduce_362 = function(val, _values, result) {
        var self = this;

        self.$push_scope("block");
        result = self.$lexer().$line();
        return result;
      };

      def.$_reduce_363 = function(val, _values, result) {
        var self = this;

        result = self.$new_iter(val['$[]'](2), val['$[]'](3));
        self.$pop_scope();
        return result;
      };

      def.$_reduce_364 = function(val, _values, result) {
        var self = this;

        val['$[]'](0)['$<<'](val['$[]'](1));
        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_367 = function(val, _values, result) {
        var self = this;

        result = self.$new_call(nil, val['$[]'](0), val['$[]'](1));
        return result;
      };

      def.$_reduce_368 = function(val, _values, result) {
        var self = this;

        result = self.$new_call(val['$[]'](0), val['$[]'](2), val['$[]'](3));
        return result;
      };

      def.$_reduce_369 = function(val, _values, result) {
        var self = this;

        result = self.$new_call(val['$[]'](0), ["call", []], val['$[]'](2));
        return result;
      };

      def.$_reduce_370 = function(val, _values, result) {
        var self = this;

        result = self.$new_call(val['$[]'](0), val['$[]'](2), val['$[]'](3));
        return result;
      };

      def.$_reduce_371 = function(val, _values, result) {
        var self = this;

        result = self.$new_call(val['$[]'](0), val['$[]'](2));
        return result;
      };

      def.$_reduce_372 = function(val, _values, result) {
        var self = this;

        result = self.$new_super(val['$[]'](0), val['$[]'](1));
        return result;
      };

      def.$_reduce_373 = function(val, _values, result) {
        var self = this;

        result = self.$new_super(val['$[]'](0), nil);
        return result;
      };

      def.$_reduce_374 = function(val, _values, result) {
        var self = this;

        self.$push_scope("block");
        result = self.$lexer().$line();
        return result;
      };

      def.$_reduce_375 = function(val, _values, result) {
        var self = this;

        result = self.$new_iter(val['$[]'](2), val['$[]'](3));
        self.$pop_scope();
        return result;
      };

      def.$_reduce_376 = function(val, _values, result) {
        var self = this;

        self.$push_scope("block");
        result = self.$lexer().$line();
        return result;
      };

      def.$_reduce_377 = function(val, _values, result) {
        var self = this;

        result = self.$new_iter(val['$[]'](2), val['$[]'](3));
        self.$pop_scope();
        return result;
      };

      def.$_reduce_378 = function(val, _values, result) {
        var self = this;

        result = self.$lexer().$line();
        return result;
      };

      def.$_reduce_379 = function(val, _values, result) {
        var $a, $b, self = this, part = nil;

        part = self.$s("when", ($a = self).$s.apply($a, ["array"].concat(val['$[]'](2))), val['$[]'](4));
        result = [part];
        if ((($b = val['$[]'](5)) !== nil && (!$b.$$is_boolean || $b == true))) {
          ($b = result).$push.apply($b, [].concat(val['$[]'](5)))};
        return result;
      };

      def.$_reduce_380 = function(val, _values, result) {
        var self = this;

        result = [val['$[]'](0)];
        return result;
      };

      def.$_reduce_382 = function(val, _values, result) {
        var $a, self = this, exc = nil;

        exc = ((($a = val['$[]'](1)) !== false && $a !== nil) ? $a : self.$s("array"));
        if ((($a = val['$[]'](2)) !== nil && (!$a.$$is_boolean || $a == true))) {
          exc['$<<'](self.$new_assign(val['$[]'](2), val['$[]'](2), self.$s("gvar", "$!".$intern())))};
        result = [self.$s("resbody", exc, val['$[]'](4))];
        if ((($a = val['$[]'](5)) !== nil && (!$a.$$is_boolean || $a == true))) {
          result.$push(val['$[]'](5).$first())};
        return result;
      };

      def.$_reduce_383 = function(val, _values, result) {
        var self = this;

        result = nil;
        return result;
      };

      def.$_reduce_384 = function(val, _values, result) {
        var self = this;

        result = self.$s("array", val['$[]'](0));
        return result;
      };

      def.$_reduce_387 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_388 = function(val, _values, result) {
        var self = this;

        result = nil;
        return result;
      };

      def.$_reduce_389 = function(val, _values, result) {
        var $a, self = this;

        result = (function() {if ((($a = val['$[]'](1)['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
          return self.$s("nil")
          } else {
          return val['$[]'](1)
        }; return nil; })();
        return result;
      };

      def.$_reduce_394 = function(val, _values, result) {
        var self = this;

        result = self.$new_str(val['$[]'](0));
        return result;
      };

      def.$_reduce_396 = function(val, _values, result) {
        var self = this;

        result = self.$str_append(val['$[]'](0), val['$[]'](1));
        return result;
      };

      def.$_reduce_397 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_398 = function(val, _values, result) {
        var self = this;

        result = self.$s("str", self.$value(val['$[]'](0)));
        return result;
      };

      def.$_reduce_399 = function(val, _values, result) {
        var self = this;

        result = self.$new_xstr(val['$[]'](0), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_400 = function(val, _values, result) {
        var self = this;

        result = self.$new_regexp(val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_401 = function(val, _values, result) {
        var self = this;

        result = self.$s("array");
        return result;
      };

      def.$_reduce_402 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_403 = function(val, _values, result) {
        var self = this;

        result = self.$s("array");
        return result;
      };

      def.$_reduce_404 = function(val, _values, result) {
        var self = this, part = nil;

        part = val['$[]'](1);
        if (part.$type()['$==']("evstr")) {
          part = self.$s("dstr", "", val['$[]'](1))};
        result = val['$[]'](0)['$<<'](part);
        return result;
      };

      def.$_reduce_405 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_406 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0).$concat([val['$[]'](1)]);
        return result;
      };

      def.$_reduce_407 = function(val, _values, result) {
        var self = this;

        result = self.$s("array");
        return result;
      };

      def.$_reduce_408 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_409 = function(val, _values, result) {
        var self = this;

        result = self.$s("array");
        return result;
      };

      def.$_reduce_410 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0)['$<<'](self.$s("str", self.$value(val['$[]'](1))));
        return result;
      };

      def.$_reduce_411 = function(val, _values, result) {
        var self = this;

        result = nil;
        return result;
      };

      def.$_reduce_412 = function(val, _values, result) {
        var self = this;

        result = self.$str_append(val['$[]'](0), val['$[]'](1));
        return result;
      };

      def.$_reduce_413 = function(val, _values, result) {
        var self = this;

        result = nil;
        return result;
      };

      def.$_reduce_414 = function(val, _values, result) {
        var self = this;

        result = self.$str_append(val['$[]'](0), val['$[]'](1));
        return result;
      };

      def.$_reduce_415 = function(val, _values, result) {
        var self = this;

        result = self.$new_str_content(val['$[]'](0));
        return result;
      };

      def.$_reduce_416 = function(val, _values, result) {
        var $a, $b, self = this;

        result = self.$lexer().$strterm();
        (($a = [nil]), $b = self.$lexer(), $b['$strterm='].apply($b, $a), $a[$a.length-1]);
        return result;
      };

      def.$_reduce_417 = function(val, _values, result) {
        var $a, $b, self = this;

        (($a = [val['$[]'](1)]), $b = self.$lexer(), $b['$strterm='].apply($b, $a), $a[$a.length-1]);
        result = self.$new_evstr(val['$[]'](2));
        return result;
      };

      def.$_reduce_418 = function(val, _values, result) {
        var $a, $b, self = this;

        self.$lexer().$cond_push(0);
        self.$lexer().$cmdarg_push(0);
        result = self.$lexer().$strterm();
        (($a = [nil]), $b = self.$lexer(), $b['$strterm='].apply($b, $a), $a[$a.length-1]);
        (($a = ["expr_beg"]), $b = self.$lexer(), $b['$lex_state='].apply($b, $a), $a[$a.length-1]);
        return result;
      };

      def.$_reduce_419 = function(val, _values, result) {
        var $a, $b, self = this;

        (($a = [val['$[]'](1)]), $b = self.$lexer(), $b['$strterm='].apply($b, $a), $a[$a.length-1]);
        self.$lexer().$cond_lexpop();
        self.$lexer().$cmdarg_lexpop();
        result = self.$new_evstr(val['$[]'](2));
        return result;
      };

      def.$_reduce_420 = function(val, _values, result) {
        var self = this;

        result = self.$new_gvar(val['$[]'](0));
        return result;
      };

      def.$_reduce_421 = function(val, _values, result) {
        var self = this;

        result = self.$new_ivar(val['$[]'](0));
        return result;
      };

      def.$_reduce_422 = function(val, _values, result) {
        var self = this;

        result = self.$new_cvar(val['$[]'](0));
        return result;
      };

      def.$_reduce_424 = function(val, _values, result) {
        var $a, $b, self = this;

        result = self.$new_sym(val['$[]'](1));
        (($a = ["expr_end"]), $b = self.$lexer(), $b['$lex_state='].apply($b, $a), $a[$a.length-1]);
        return result;
      };

      def.$_reduce_425 = function(val, _values, result) {
        var self = this;

        result = self.$new_sym(val['$[]'](0));
        return result;
      };

      def.$_reduce_430 = function(val, _values, result) {
        var self = this;

        result = self.$new_dsym(val['$[]'](1));
        return result;
      };

      def.$_reduce_431 = function(val, _values, result) {
        var self = this;

        result = self.$new_int(val['$[]'](0));
        return result;
      };

      def.$_reduce_432 = function(val, _values, result) {
        var self = this;

        result = self.$new_float(val['$[]'](0));
        return result;
      };

      def.$_reduce_433 = function(val, _values, result) {
        var self = this;

        result = self.$negate_num(self.$new_int(val['$[]'](1)));
        return result;
      };

      def.$_reduce_434 = function(val, _values, result) {
        var self = this;

        result = self.$negate_num(self.$new_float(val['$[]'](1)));
        return result;
      };

      def.$_reduce_435 = function(val, _values, result) {
        var self = this;

        result = self.$new_int(val['$[]'](1));
        return result;
      };

      def.$_reduce_436 = function(val, _values, result) {
        var self = this;

        result = self.$new_float(val['$[]'](1));
        return result;
      };

      def.$_reduce_437 = function(val, _values, result) {
        var self = this;

        result = self.$new_ident(val['$[]'](0));
        return result;
      };

      def.$_reduce_438 = function(val, _values, result) {
        var self = this;

        result = self.$new_ivar(val['$[]'](0));
        return result;
      };

      def.$_reduce_439 = function(val, _values, result) {
        var self = this;

        result = self.$new_gvar(val['$[]'](0));
        return result;
      };

      def.$_reduce_440 = function(val, _values, result) {
        var self = this;

        result = self.$new_const(val['$[]'](0));
        return result;
      };

      def.$_reduce_441 = function(val, _values, result) {
        var self = this;

        result = self.$new_cvar(val['$[]'](0));
        return result;
      };

      def.$_reduce_442 = function(val, _values, result) {
        var self = this;

        result = self.$new_nil(val['$[]'](0));
        return result;
      };

      def.$_reduce_443 = function(val, _values, result) {
        var self = this;

        result = self.$new_self(val['$[]'](0));
        return result;
      };

      def.$_reduce_444 = function(val, _values, result) {
        var self = this;

        result = self.$new_true(val['$[]'](0));
        return result;
      };

      def.$_reduce_445 = function(val, _values, result) {
        var self = this;

        result = self.$new_false(val['$[]'](0));
        return result;
      };

      def.$_reduce_446 = function(val, _values, result) {
        var self = this;

        result = self.$new___FILE__(val['$[]'](0));
        return result;
      };

      def.$_reduce_447 = function(val, _values, result) {
        var self = this;

        result = self.$new___LINE__(val['$[]'](0));
        return result;
      };

      def.$_reduce_448 = function(val, _values, result) {
        var self = this;

        result = self.$new_var_ref(val['$[]'](0));
        return result;
      };

      def.$_reduce_449 = function(val, _values, result) {
        var self = this;

        result = self.$new_assignable(val['$[]'](0));
        return result;
      };

      def.$_reduce_450 = function(val, _values, result) {
        var self = this;

        result = self.$s("nth_ref", self.$value(val['$[]'](0)));
        return result;
      };

      def.$_reduce_452 = function(val, _values, result) {
        var self = this;

        result = nil;
        return result;
      };

      def.$_reduce_453 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_454 = function(val, _values, result) {
        var self = this;

        result = nil;
        return result;
      };

      def.$_reduce_455 = function(val, _values, result) {
        var $a, $b, self = this;

        result = val['$[]'](1);
        (($a = ["expr_beg"]), $b = self.$lexer(), $b['$lex_state='].apply($b, $a), $a[$a.length-1]);
        return result;
      };

      def.$_reduce_456 = function(val, _values, result) {
        var $a, $b, self = this;

        result = val['$[]'](0);
        (($a = ["expr_beg"]), $b = self.$lexer(), $b['$lex_state='].apply($b, $a), $a[$a.length-1]);
        return result;
      };

      def.$_reduce_459 = function(val, _values, result) {
        var self = this;

        result = self.$new_kwrestarg(val['$[]'](1));
        return result;
      };

      def.$_reduce_460 = function(val, _values, result) {
        var self = this;

        result = self.$new_kwrestarg();
        return result;
      };

      def.$_reduce_461 = function(val, _values, result) {
        var self = this;

        result = self.$new_sym(val['$[]'](0));
        return result;
      };

      def.$_reduce_462 = function(val, _values, result) {
        var self = this;

        result = self.$new_kwoptarg(val['$[]'](0), val['$[]'](1));
        return result;
      };

      def.$_reduce_463 = function(val, _values, result) {
        var self = this;

        result = self.$new_kwarg(val['$[]'](0));
        return result;
      };

      def.$_reduce_464 = function(val, _values, result) {
        var self = this;

        result = [val['$[]'](0)];
        return result;
      };

      def.$_reduce_465 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0);
        result['$<<'](val['$[]'](2));
        return result;
      };

      def.$_reduce_466 = function(val, _values, result) {
        var self = this;

        result = self.$new_args_tail(val['$[]'](0), val['$[]'](2), val['$[]'](3));
        return result;
      };

      def.$_reduce_467 = function(val, _values, result) {
        var self = this;

        result = self.$new_args_tail(val['$[]'](0), nil, val['$[]'](1));
        return result;
      };

      def.$_reduce_468 = function(val, _values, result) {
        var self = this;

        result = self.$new_args_tail(nil, val['$[]'](0), val['$[]'](1));
        return result;
      };

      def.$_reduce_469 = function(val, _values, result) {
        var self = this;

        result = self.$new_args_tail(nil, nil, val['$[]'](0));
        return result;
      };

      def.$_reduce_470 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_471 = function(val, _values, result) {
        var self = this;

        result = self.$new_args_tail(nil, nil, nil);
        return result;
      };

      def.$_reduce_472 = function(val, _values, result) {
        var self = this;

        result = self.$new_args(val['$[]'](0), val['$[]'](2), val['$[]'](4), val['$[]'](5));
        return result;
      };

      def.$_reduce_473 = function(val, _values, result) {
        var self = this;

        result = self.$new_args(val['$[]'](0), val['$[]'](2), nil, val['$[]'](3));
        return result;
      };

      def.$_reduce_474 = function(val, _values, result) {
        var self = this;

        result = self.$new_args(val['$[]'](0), nil, val['$[]'](2), val['$[]'](3));
        return result;
      };

      def.$_reduce_475 = function(val, _values, result) {
        var self = this;

        result = self.$new_args(val['$[]'](0), nil, nil, val['$[]'](1));
        return result;
      };

      def.$_reduce_476 = function(val, _values, result) {
        var self = this;

        result = self.$new_args(nil, val['$[]'](0), val['$[]'](2), val['$[]'](3));
        return result;
      };

      def.$_reduce_477 = function(val, _values, result) {
        var self = this;

        result = self.$new_args(nil, val['$[]'](0), nil, val['$[]'](1));
        return result;
      };

      def.$_reduce_478 = function(val, _values, result) {
        var self = this;

        result = self.$new_args(nil, nil, val['$[]'](0), val['$[]'](1));
        return result;
      };

      def.$_reduce_479 = function(val, _values, result) {
        var self = this;

        result = self.$new_args(nil, nil, nil, val['$[]'](0));
        return result;
      };

      def.$_reduce_480 = function(val, _values, result) {
        var self = this;

        result = self.$new_args(nil, nil, nil, nil);
        return result;
      };

      def.$_reduce_482 = function(val, _values, result) {
        var self = this;

        result = self.$value(val['$[]'](0)).$to_sym();
        self.$scope().$add_local(result);
        return result;
      };

      def.$_reduce_483 = function(val, _values, result) {
        var self = this;

        self.$raise("formal argument cannot be a constant");
        return result;
      };

      def.$_reduce_484 = function(val, _values, result) {
        var self = this;

        self.$raise("formal argument cannot be an instance variable");
        return result;
      };

      def.$_reduce_485 = function(val, _values, result) {
        var self = this;

        self.$raise("formal argument cannot be a class variable");
        return result;
      };

      def.$_reduce_486 = function(val, _values, result) {
        var self = this;

        self.$raise("formal argument cannot be a global variable");
        return result;
      };

      def.$_reduce_487 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_488 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_491 = function(val, _values, result) {
        var self = this;

        result = self.$s("lasgn", val['$[]'](0));
        return result;
      };

      def.$_reduce_493 = function(val, _values, result) {
        var self = this;

        result = self.$s("array", val['$[]'](0));
        return result;
      };

      def.$_reduce_494 = function(val, _values, result) {
        var self = this;

        val['$[]'](0)['$<<'](val['$[]'](2));
        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_500 = function(val, _values, result) {
        var self = this;

        result = [val['$[]'](0)];
        return result;
      };

      def.$_reduce_501 = function(val, _values, result) {
        var self = this;

        val['$[]'](0)['$<<'](val['$[]'](2));
        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_502 = function(val, _values, result) {
        var self = this;

        result = self.$new_assign(self.$new_assignable(self.$new_ident(val['$[]'](0))), val['$[]'](1), val['$[]'](2));
        return result;
      };

      def.$_reduce_503 = function(val, _values, result) {
        var self = this;

        result = self.$s("block", val['$[]'](0));
        return result;
      };

      def.$_reduce_504 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0);
        val['$[]'](0)['$<<'](val['$[]'](2));
        return result;
      };

      def.$_reduce_507 = function(val, _values, result) {
        var self = this;

        result = (("*") + (self.$value(val['$[]'](1)))).$to_sym();
        return result;
      };

      def.$_reduce_508 = function(val, _values, result) {
        var self = this;

        result = "*";
        return result;
      };

      def.$_reduce_511 = function(val, _values, result) {
        var self = this;

        result = (("&") + (self.$value(val['$[]'](1)))).$to_sym();
        return result;
      };

      def.$_reduce_512 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_513 = function(val, _values, result) {
        var self = this;

        result = nil;
        return result;
      };

      def.$_reduce_514 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_515 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](1);
        return result;
      };

      def.$_reduce_516 = function(val, _values, result) {
        var self = this;

        result = [];
        return result;
      };

      def.$_reduce_517 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_518 = function(val, _values, result) {
        var self = this;

        result = val['$[]'](0);
        return result;
      };

      def.$_reduce_519 = function(val, _values, result) {
        var $a, self = this;

        result = ($a = val['$[]'](0)).$push.apply($a, [].concat(val['$[]'](2)));
        return result;
      };

      def.$_reduce_520 = function(val, _values, result) {
        var self = this;

        result = [val['$[]'](0), val['$[]'](2)];
        return result;
      };

      def.$_reduce_521 = function(val, _values, result) {
        var self = this;

        result = [self.$new_sym(val['$[]'](0)), val['$[]'](1)];
        return result;
      };

      def.$_reduce_545 = function(val, _values, result) {
        var self = this;

        result = nil;
        return result;
      };

      return (def.$_reduce_none = function(val, _values, result) {
        var self = this;

        return val['$[]'](0);
      }, nil) && '_reduce_none';
    })(self, (($scope.get('Racc')).$$scope.get('Parser')))
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/parser/parser_scope"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass;

  Opal.add_stubs(['$attr_reader', '$attr_accessor', '$==', '$<<', '$include?', '$has_local?']);
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base, $super) {
      function $ParserScope(){};
      var self = $ParserScope = $klass($base, $super, 'ParserScope', $ParserScope);

      var def = self.$$proto, $scope = self.$$scope;

      def.locals = def.parent = def.block = nil;
      self.$attr_reader("locals");

      self.$attr_accessor("parent");

      def.$initialize = function(type) {
        var self = this;

        self.block = type['$==']("block");
        self.locals = [];
        return self.parent = nil;
      };

      def.$add_local = function(local) {
        var self = this;

        return self.locals['$<<'](local);
      };

      return (def['$has_local?'] = function(local) {
        var $a, $b, self = this;

        if ((($a = self.locals['$include?'](local)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return true};
        if ((($a = ($b = self.parent, $b !== false && $b !== nil ?self.block : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return self.parent['$has_local?'](local)};
        return false;
      }, nil) && 'has_local?';
    })(self, null)
  })(self)
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/parser"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_minus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs - rhs : lhs['$-'](rhs);
  }
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  function $rb_times(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs * rhs : lhs['$*'](rhs);
  }
  function $rb_gt(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs > rhs : lhs['$>'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass, $range = Opal.range;

  Opal.add_stubs(['$require', '$attr_reader', '$new', '$parser=', '$parse_to_sexp', '$join', '$message', '$line', '$lexer', '$column', '$[]', '$split', '$raise', '$class', '$push_scope', '$do_parse', '$pop_scope', '$next_token', '$last', '$parent=', '$<<', '$pop', '$inspect', '$value', '$token_to_str', '$s', '$source=', '$s0', '$source', '$s1', '$file', '$to_sym', '$nil?', '$==', '$length', '$size', '$type', '$each', '$!', '$add_local', '$scope', '$to_s', '$empty?', '$is_a?', '$new_splat', '$new_call', '$[]=', '$array', '$-@', '$===', '$new_gettable', '$type=', '$has_local?']);
  self.$require("opal/parser/sexp");
  self.$require("opal/parser/lexer");
  self.$require("opal/parser/grammar");
  self.$require("opal/parser/parser_scope");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base, $super) {
      function $Parser(){};
      var self = $Parser = $klass($base, $super, 'Parser', $Parser);

      var def = self.$$proto, $scope = self.$$scope;

      def.lexer = def.file = def.scopes = nil;
      self.$attr_reader("lexer", "file", "scope");

      def.$parse = function(source, file) {
        var $a, $b, self = this, error = nil, message = nil;

        if (file == null) {
          file = "(string)"
        }
        try {
        self.file = file;
          self.scopes = [];
          self.lexer = $scope.get('Lexer').$new(source, file);
          (($a = [self]), $b = self.lexer, $b['$parser='].apply($b, $a), $a[$a.length-1]);
          return self.$parse_to_sexp();
        } catch ($err) {if (true) {error = $err;
          message = [nil, error.$message(), "Source: " + (self.file) + ":" + (self.$lexer().$line()) + ":" + (self.$lexer().$column()), source.$split("\n")['$[]']($rb_minus(self.$lexer().$line(), 1)), $rb_plus($rb_times("~", ($rb_minus(self.$lexer().$column(), 1))), "^")].$join("\n");
          return self.$raise(error.$class(), message);
          }else { throw $err; }
        };
      };

      def.$parse_to_sexp = function() {
        var self = this, result = nil;

        self.$push_scope();
        result = self.$do_parse();
        self.$pop_scope();
        return result;
      };

      def.$next_token = function() {
        var self = this;

        return self.lexer.$next_token();
      };

      def.$s = function(parts) {
        var self = this;

        parts = $slice.call(arguments, 0);
        return $scope.get('Sexp').$new(parts);
      };

      def.$push_scope = function(type) {
        var $a, $b, self = this, top = nil, scope = nil;

        if (type == null) {
          type = nil
        }
        top = self.scopes.$last();
        scope = $scope.get('ParserScope').$new(type);
        (($a = [top]), $b = scope, $b['$parent='].apply($b, $a), $a[$a.length-1]);
        self.scopes['$<<'](scope);
        return self.scope = scope;
      };

      def.$pop_scope = function() {
        var self = this;

        self.scopes.$pop();
        return self.scope = self.scopes.$last();
      };

      def.$on_error = function(t, val, vstack) {
        var $a, self = this;

        return self.$raise("parse error on value " + (self.$value(val).$inspect()) + " (" + (((($a = self.$token_to_str(t)) !== false && $a !== nil) ? $a : "?")) + ") :" + (self.file) + ":" + (self.$lexer().$line()));
      };

      def.$value = function(tok) {
        var self = this;

        return tok['$[]'](0);
      };

      def.$source = function(tok) {
        var self = this;

        if (tok !== false && tok !== nil) {
          return tok['$[]'](1)
          } else {
          return nil
        };
      };

      def.$s0 = function(type, source) {
        var $a, $b, self = this, sexp = nil;

        sexp = self.$s(type);
        (($a = [source]), $b = sexp, $b['$source='].apply($b, $a), $a[$a.length-1]);
        return sexp;
      };

      def.$s1 = function(type, first, source) {
        var $a, $b, self = this, sexp = nil;

        sexp = self.$s(type, first);
        (($a = [source]), $b = sexp, $b['$source='].apply($b, $a), $a[$a.length-1]);
        return sexp;
      };

      def.$new_nil = function(tok) {
        var self = this;

        return self.$s0("nil", self.$source(tok));
      };

      def.$new_self = function(tok) {
        var self = this;

        return self.$s0("self", self.$source(tok));
      };

      def.$new_true = function(tok) {
        var self = this;

        return self.$s0("true", self.$source(tok));
      };

      def.$new_false = function(tok) {
        var self = this;

        return self.$s0("false", self.$source(tok));
      };

      def.$new___FILE__ = function(tok) {
        var self = this;

        return self.$s1("str", self.$file(), self.$source(tok));
      };

      def.$new___LINE__ = function(tok) {
        var self = this;

        return self.$s1("int", self.$lexer().$line(), self.$source(tok));
      };

      def.$new_ident = function(tok) {
        var self = this;

        return self.$s1("identifier", self.$value(tok).$to_sym(), self.$source(tok));
      };

      def.$new_int = function(tok) {
        var self = this;

        return self.$s1("int", self.$value(tok), self.$source(tok));
      };

      def.$new_float = function(tok) {
        var self = this;

        return self.$s1("float", self.$value(tok), self.$source(tok));
      };

      def.$new_ivar = function(tok) {
        var self = this;

        return self.$s1("ivar", self.$value(tok).$to_sym(), self.$source(tok));
      };

      def.$new_gvar = function(tok) {
        var self = this;

        return self.$s1("gvar", self.$value(tok).$to_sym(), self.$source(tok));
      };

      def.$new_cvar = function(tok) {
        var self = this;

        return self.$s1("cvar", self.$value(tok).$to_sym(), self.$source(tok));
      };

      def.$new_const = function(tok) {
        var self = this;

        return self.$s1("const", self.$value(tok).$to_sym(), self.$source(tok));
      };

      def.$new_colon2 = function(lhs, tok, name) {
        var $a, $b, self = this, sexp = nil;

        sexp = self.$s("colon2", lhs, self.$value(name).$to_sym());
        (($a = [self.$source(tok)]), $b = sexp, $b['$source='].apply($b, $a), $a[$a.length-1]);
        return sexp;
      };

      def.$new_colon3 = function(tok, name) {
        var self = this;

        return self.$s1("colon3", self.$value(name).$to_sym(), self.$source(name));
      };

      def.$new_sym = function(tok) {
        var self = this;

        return self.$s1("sym", self.$value(tok).$to_sym(), self.$source(tok));
      };

      def.$new_alias = function(kw, new$, old) {
        var $a, $b, self = this, sexp = nil;

        sexp = self.$s("alias", new$, old);
        (($a = [self.$source(kw)]), $b = sexp, $b['$source='].apply($b, $a), $a[$a.length-1]);
        return sexp;
      };

      def.$new_break = function(kw, args) {
        var $a, self = this, sexp = nil;

        if (args == null) {
          args = nil
        }
        if ((($a = args['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
          sexp = self.$s("break")
        } else if (args.$length()['$=='](1)) {
          sexp = self.$s("break", args['$[]'](0))
          } else {
          sexp = self.$s("break", ($a = self).$s.apply($a, ["array"].concat(args)))
        };
        return sexp;
      };

      def.$new_return = function(kw, args) {
        var $a, self = this, sexp = nil;

        if (args == null) {
          args = nil
        }
        if ((($a = args['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
          sexp = self.$s("return")
        } else if (args.$length()['$=='](1)) {
          sexp = self.$s("return", args['$[]'](0))
          } else {
          sexp = self.$s("return", ($a = self).$s.apply($a, ["array"].concat(args)))
        };
        return sexp;
      };

      def.$new_next = function(kw, args) {
        var $a, self = this, sexp = nil;

        if (args == null) {
          args = []
        }
        if (args.$length()['$=='](1)) {
          sexp = self.$s("next", args['$[]'](0))
          } else {
          sexp = self.$s("next", ($a = self).$s.apply($a, ["array"].concat(args)))
        };
        return sexp;
      };

      def.$new_block = function(stmt) {
        var self = this, sexp = nil;

        if (stmt == null) {
          stmt = nil
        }
        sexp = self.$s("block");
        if (stmt !== false && stmt !== nil) {
          sexp['$<<'](stmt)};
        return sexp;
      };

      def.$new_compstmt = function(block) {
        var $a, $b, $c, self = this, comp = nil, result = nil;

        comp = (function() {if (block.$size()['$=='](1)) {
          return nil
        } else if (block.$size()['$=='](2)) {
          return block['$[]'](1)
          } else {
          return block
        }; return nil; })();
        if ((($a = ($b = (($c = comp !== false && comp !== nil) ? comp.$type()['$==']("begin") : $c), $b !== false && $b !== nil ?comp.$size()['$=='](2) : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
          result = comp['$[]'](1)
          } else {
          result = comp
        };
        return result;
      };

      def.$new_body = function(compstmt, res, els, ens) {
        var $a, $b, TMP_1, self = this, s = nil;

        s = ((($a = compstmt) !== false && $a !== nil) ? $a : self.$s("block"));
        if (res !== false && res !== nil) {
          s = self.$s("rescue", s);
          ($a = ($b = res).$each, $a.$$p = (TMP_1 = function(r){var self = TMP_1.$$s || this;
if (r == null) r = nil;
          return s['$<<'](r)}, TMP_1.$$s = self, TMP_1), $a).call($b);
          if (els !== false && els !== nil) {
            s['$<<'](els)};};
        if (ens !== false && ens !== nil) {
          return self.$s("ensure", s, ens)
          } else {
          return s
        };
      };

      def.$new_def = function(kw, recv, name, args, body, end_tok) {
        var $a, $b, self = this, sexp = nil;

        if ((($a = body.$type()['$==']("block")['$!']()) !== nil && (!$a.$$is_boolean || $a == true))) {
          body = self.$s("block", body)};
        if (body.$size()['$=='](1)) {
          body['$<<'](self.$s("nil"))};
        sexp = self.$s("def", recv, self.$value(name).$to_sym(), args, body);
        (($a = [self.$source(kw)]), $b = sexp, $b['$source='].apply($b, $a), $a[$a.length-1]);
        return sexp;
      };

      def.$new_class = function(start, path, sup, body, endt) {
        var $a, $b, self = this, sexp = nil;

        sexp = self.$s("class", path, sup, body);
        (($a = [self.$source(start)]), $b = sexp, $b['$source='].apply($b, $a), $a[$a.length-1]);
        return sexp;
      };

      def.$new_sclass = function(kw, expr, body, end_tok) {
        var $a, $b, self = this, sexp = nil;

        sexp = self.$s("sclass", expr, body);
        (($a = [self.$source(kw)]), $b = sexp, $b['$source='].apply($b, $a), $a[$a.length-1]);
        return sexp;
      };

      def.$new_module = function(kw, path, body, end_tok) {
        var $a, $b, self = this, sexp = nil;

        sexp = self.$s("module", path, body);
        (($a = [self.$source(kw)]), $b = sexp, $b['$source='].apply($b, $a), $a[$a.length-1]);
        return sexp;
      };

      def.$new_iter = function(args, body) {
        var $a, self = this, s = nil;

        ((($a = args) !== false && $a !== nil) ? $a : args = nil);
        s = self.$s("iter", args);
        if (body !== false && body !== nil) {
          s['$<<'](body)};
        return s;
      };

      def.$new_if = function(if_tok, expr, stmt, tail) {
        var $a, $b, self = this, sexp = nil;

        sexp = self.$s("if", expr, stmt, tail);
        (($a = [self.$source(if_tok)]), $b = sexp, $b['$source='].apply($b, $a), $a[$a.length-1]);
        return sexp;
      };

      def.$new_while = function(kw, test, body) {
        var $a, $b, self = this, sexp = nil;

        sexp = self.$s("while", test, body);
        (($a = [self.$source(kw)]), $b = sexp, $b['$source='].apply($b, $a), $a[$a.length-1]);
        return sexp;
      };

      def.$new_until = function(kw, test, body) {
        var $a, $b, self = this, sexp = nil;

        sexp = self.$s("until", test, body);
        (($a = [self.$source(kw)]), $b = sexp, $b['$source='].apply($b, $a), $a[$a.length-1]);
        return sexp;
      };

      def.$new_rescue_mod = function(kw, expr, resc) {
        var $a, $b, self = this, sexp = nil;

        sexp = self.$s("rescue_mod", expr, resc);
        (($a = [self.$source(kw)]), $b = sexp, $b['$source='].apply($b, $a), $a[$a.length-1]);
        return sexp;
      };

      def.$new_array = function(start, args, finish) {
        var $a, $b, $c, self = this, sexp = nil;

        ((($a = args) !== false && $a !== nil) ? $a : args = []);
        sexp = ($a = self).$s.apply($a, ["array"].concat(args));
        (($b = [self.$source(start)]), $c = sexp, $c['$source='].apply($c, $b), $b[$b.length-1]);
        return sexp;
      };

      def.$new_hash = function(open, assocs, close) {
        var $a, $b, $c, self = this, sexp = nil;

        sexp = ($a = self).$s.apply($a, ["hash"].concat(assocs));
        (($b = [self.$source(open)]), $c = sexp, $c['$source='].apply($c, $b), $b[$b.length-1]);
        return sexp;
      };

      def.$new_not = function(kw, expr) {
        var self = this;

        return self.$s1("not", expr, self.$source(kw));
      };

      def.$new_paren = function(open, expr, close) {
        var $a, $b, self = this;

        if ((($a = ((($b = expr['$nil?']()) !== false && $b !== nil) ? $b : expr['$=='](["block"]))) !== nil && (!$a.$$is_boolean || $a == true))) {
          return self.$s1("paren", self.$s0("nil", self.$source(open)), self.$source(open))
          } else {
          return self.$s1("paren", expr, self.$source(open))
        };
      };

      def.$new_args_tail = function(kwarg, kwrest, block) {
        var self = this;

        return [kwarg, kwrest, block];
      };

      def.$new_args = function(norm, opt, rest, tail) {
        var $a, $b, TMP_2, $c, TMP_3, $d, TMP_4, $e, self = this, res = nil, restname = nil, blockname = nil;

        res = self.$s("args");
        if (norm !== false && norm !== nil) {
          ($a = ($b = norm).$each, $a.$$p = (TMP_2 = function(arg){var self = TMP_2.$$s || this;
if (arg == null) arg = nil;
          self.$scope().$add_local(arg);
            return res['$<<'](self.$s("arg", arg));}, TMP_2.$$s = self, TMP_2), $a).call($b)};
        if (opt !== false && opt !== nil) {
          ($a = ($c = opt['$[]']($range(1, -1, false))).$each, $a.$$p = (TMP_3 = function(_opt){var self = TMP_3.$$s || this;
if (_opt == null) _opt = nil;
          return res['$<<'](self.$s("optarg", _opt['$[]'](1), _opt['$[]'](2)))}, TMP_3.$$s = self, TMP_3), $a).call($c)};
        if (rest !== false && rest !== nil) {
          restname = rest.$to_s()['$[]']($range(1, -1, false));
          if ((($a = restname['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            res['$<<'](self.$s("restarg"))
            } else {
            res['$<<'](self.$s("restarg", restname.$to_sym()));
            self.$scope().$add_local(restname.$to_sym());
          };};
        if ((($a = (($d = tail !== false && tail !== nil) ? tail['$[]'](0) : $d)) !== nil && (!$a.$$is_boolean || $a == true))) {
          ($a = ($d = tail['$[]'](0)).$each, $a.$$p = (TMP_4 = function(kwarg){var self = TMP_4.$$s || this;
if (kwarg == null) kwarg = nil;
          return res['$<<'](kwarg)}, TMP_4.$$s = self, TMP_4), $a).call($d)};
        if ((($a = (($e = tail !== false && tail !== nil) ? tail['$[]'](1) : $e)) !== nil && (!$a.$$is_boolean || $a == true))) {
          res['$<<'](tail['$[]'](1))};
        if ((($a = (($e = tail !== false && tail !== nil) ? tail['$[]'](2) : $e)) !== nil && (!$a.$$is_boolean || $a == true))) {
          blockname = tail['$[]'](2).$to_s()['$[]']($range(1, -1, false)).$to_sym();
          self.$scope().$add_local(blockname);
          res['$<<'](self.$s("blockarg", blockname));};
        return res;
      };

      def.$new_kwarg = function(name) {
        var self = this;

        self.$scope().$add_local(name['$[]'](1));
        return self.$s("kwarg", name['$[]'](1));
      };

      def.$new_kwoptarg = function(name, val) {
        var self = this;

        self.$scope().$add_local(name['$[]'](1));
        return self.$s("kwoptarg", name['$[]'](1), val);
      };

      def.$new_kwrestarg = function(name) {
        var self = this, result = nil;

        if (name == null) {
          name = nil
        }
        result = self.$s("kwrestarg");
        if (name !== false && name !== nil) {
          self.$scope().$add_local(name['$[]'](0).$to_sym());
          result['$<<'](name['$[]'](0).$to_sym());};
        return result;
      };

      def.$new_block_args = function(norm, opt, rest, block) {
        var $a, $b, TMP_5, $c, TMP_6, $d, self = this, res = nil, r = nil, b = nil, args = nil;

        res = self.$s("array");
        if (norm !== false && norm !== nil) {
          ($a = ($b = norm).$each, $a.$$p = (TMP_5 = function(arg){var self = TMP_5.$$s || this, $a;
if (arg == null) arg = nil;
          if ((($a = arg['$is_a?']($scope.get('Symbol'))) !== nil && (!$a.$$is_boolean || $a == true))) {
              self.$scope().$add_local(arg);
              return res['$<<'](self.$s("lasgn", arg));
              } else {
              return res['$<<'](arg)
            }}, TMP_5.$$s = self, TMP_5), $a).call($b)};
        if (opt !== false && opt !== nil) {
          ($a = ($c = opt['$[]']($range(1, -1, false))).$each, $a.$$p = (TMP_6 = function(_opt){var self = TMP_6.$$s || this;
if (_opt == null) _opt = nil;
          return res['$<<'](self.$s("lasgn", _opt['$[]'](1)))}, TMP_6.$$s = self, TMP_6), $a).call($c)};
        if (rest !== false && rest !== nil) {
          r = rest.$to_s()['$[]']($range(1, -1, false)).$to_sym();
          res['$<<'](self.$new_splat(nil, self.$s("lasgn", r)));
          self.$scope().$add_local(r);};
        if (block !== false && block !== nil) {
          b = block.$to_s()['$[]']($range(1, -1, false)).$to_sym();
          res['$<<'](self.$s("block_pass", self.$s("lasgn", b)));
          self.$scope().$add_local(b);};
        if (opt !== false && opt !== nil) {
          res['$<<'](opt)};
        args = (function() {if ((($a = (($d = res.$size()['$=='](2)) ? norm : $d)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return res['$[]'](1)
          } else {
          return self.$s("masgn", res)
        }; return nil; })();
        if (args.$type()['$==']("array")) {
          return self.$s("masgn", args)
          } else {
          return args
        };
      };

      def.$new_call = function(recv, meth, args) {
        var $a, $b, $c, self = this, sexp = nil;

        if (args == null) {
          args = nil
        }
        ((($a = args) !== false && $a !== nil) ? $a : args = []);
        sexp = self.$s("call", recv, self.$value(meth).$to_sym(), ($a = self).$s.apply($a, ["arglist"].concat(args)));
        (($b = [self.$source(meth)]), $c = sexp, $c['$source='].apply($c, $b), $b[$b.length-1]);
        return sexp;
      };

      def.$new_binary_call = function(recv, meth, arg) {
        var self = this;

        return self.$new_call(recv, meth, [arg]);
      };

      def.$new_unary_call = function(op, recv) {
        var self = this;

        return self.$new_call(recv, op, []);
      };

      def.$new_and = function(lhs, tok, rhs) {
        var $a, $b, self = this, sexp = nil;

        sexp = self.$s("and", lhs, rhs);
        (($a = [self.$source(tok)]), $b = sexp, $b['$source='].apply($b, $a), $a[$a.length-1]);
        return sexp;
      };

      def.$new_or = function(lhs, tok, rhs) {
        var $a, $b, self = this, sexp = nil;

        sexp = self.$s("or", lhs, rhs);
        (($a = [self.$source(tok)]), $b = sexp, $b['$source='].apply($b, $a), $a[$a.length-1]);
        return sexp;
      };

      def.$new_irange = function(beg, op, finish) {
        var $a, $b, self = this, sexp = nil;

        sexp = self.$s("irange", beg, finish);
        (($a = [self.$source(op)]), $b = sexp, $b['$source='].apply($b, $a), $a[$a.length-1]);
        return sexp;
      };

      def.$new_erange = function(beg, op, finish) {
        var $a, $b, self = this, sexp = nil;

        sexp = self.$s("erange", beg, finish);
        (($a = [self.$source(op)]), $b = sexp, $b['$source='].apply($b, $a), $a[$a.length-1]);
        return sexp;
      };

      def.$negate_num = function(sexp) {
        var self = this;

        sexp.$array()['$[]='](1, sexp.$array()['$[]'](1)['$-@']());
        return sexp;
      };

      def.$add_block_pass = function(arglist, block) {
        var self = this;

        if (block !== false && block !== nil) {
          arglist['$<<'](block)};
        return arglist;
      };

      def.$new_block_pass = function(amper_tok, val) {
        var self = this;

        return self.$s1("block_pass", val, self.$source(amper_tok));
      };

      def.$new_splat = function(tok, value) {
        var self = this;

        return self.$s1("splat", value, self.$source(tok));
      };

      def.$new_op_asgn = function(op, lhs, rhs) {
        var self = this, $case = nil, result = nil;

        $case = self.$value(op).$to_sym();if ("||"['$===']($case)) {result = self.$s("op_asgn_or", self.$new_gettable(lhs));
        result['$<<']((lhs['$<<'](rhs)));}else if ("&&"['$===']($case)) {result = self.$s("op_asgn_and", self.$new_gettable(lhs));
        result['$<<']((lhs['$<<'](rhs)));}else {result = lhs;
        result['$<<'](self.$new_call(self.$new_gettable(lhs), op, [rhs]));};
        return result;
      };

      def.$new_op_asgn1 = function(lhs, args, op, rhs) {
        var $a, $b, $c, self = this, arglist = nil, sexp = nil;

        arglist = ($a = self).$s.apply($a, ["arglist"].concat(args));
        sexp = self.$s("op_asgn1", lhs, arglist, self.$value(op), rhs);
        (($b = [self.$source(op)]), $c = sexp, $c['$source='].apply($c, $b), $b[$b.length-1]);
        return sexp;
      };

      def.$op_to_setter = function(op) {
        var self = this;

        return ((("") + (self.$value(op))) + "=").$to_sym();
      };

      def.$new_attrasgn = function(recv, op, args) {
        var $a, self = this, arglist = nil, sexp = nil;

        if (args == null) {
          args = []
        }
        arglist = ($a = self).$s.apply($a, ["arglist"].concat(args));
        sexp = self.$s("attrasgn", recv, op, arglist);
        return sexp;
      };

      def.$new_assign = function(lhs, tok, rhs) {
        var $a, $b, self = this, $case = nil;

        return (function() {$case = lhs.$type();if ("iasgn"['$===']($case) || "cdecl"['$===']($case) || "lasgn"['$===']($case) || "gasgn"['$===']($case) || "cvdecl"['$===']($case) || "nth_ref"['$===']($case)) {lhs['$<<'](rhs);
        return lhs;}else if ("call"['$===']($case) || "attrasgn"['$===']($case)) {lhs.$last()['$<<'](rhs);
        return lhs;}else if ("colon2"['$===']($case)) {lhs['$<<'](rhs);
        (($a = ["casgn"]), $b = lhs, $b['$type='].apply($b, $a), $a[$a.length-1]);
        return lhs;}else if ("colon3"['$===']($case)) {lhs['$<<'](rhs);
        (($a = ["casgn3"]), $b = lhs, $b['$type='].apply($b, $a), $a[$a.length-1]);
        return lhs;}else {return self.$raise("Bad lhs for new_assign: " + (lhs.$type()))}})();
      };

      def.$new_assignable = function(ref) {
        var $a, $b, self = this, $case = nil;

        $case = ref.$type();if ("ivar"['$===']($case)) {(($a = ["iasgn"]), $b = ref, $b['$type='].apply($b, $a), $a[$a.length-1])}else if ("const"['$===']($case)) {(($a = ["cdecl"]), $b = ref, $b['$type='].apply($b, $a), $a[$a.length-1])}else if ("identifier"['$===']($case)) {if ((($a = self.$scope()['$has_local?'](ref['$[]'](1))) !== nil && (!$a.$$is_boolean || $a == true))) {
          } else {
          self.$scope().$add_local(ref['$[]'](1))
        };
        (($a = ["lasgn"]), $b = ref, $b['$type='].apply($b, $a), $a[$a.length-1]);}else if ("gvar"['$===']($case)) {(($a = ["gasgn"]), $b = ref, $b['$type='].apply($b, $a), $a[$a.length-1])}else if ("cvar"['$===']($case)) {(($a = ["cvdecl"]), $b = ref, $b['$type='].apply($b, $a), $a[$a.length-1])}else {self.$raise($scope.get('SyntaxError'), "Bad new_assignable type: " + (ref.$type()))};
        return ref;
      };

      def.$new_gettable = function(ref) {
        var $a, $b, self = this, res = nil, $case = nil;

        res = (function() {$case = ref.$type();if ("lasgn"['$===']($case)) {return self.$s("lvar", ref['$[]'](1))}else if ("iasgn"['$===']($case)) {return self.$s("ivar", ref['$[]'](1))}else if ("gasgn"['$===']($case)) {return self.$s("gvar", ref['$[]'](1))}else if ("cvdecl"['$===']($case)) {return self.$s("cvar", ref['$[]'](1))}else if ("cdecl"['$===']($case)) {return self.$s("const", ref['$[]'](1))}else {return self.$raise("Bad new_gettable ref: " + (ref.$type()))}})();
        (($a = [ref.$source()]), $b = res, $b['$source='].apply($b, $a), $a[$a.length-1]);
        return res;
      };

      def.$new_var_ref = function(ref) {
        var $a, $b, self = this, $case = nil, result = nil;

        return (function() {$case = ref.$type();if ("self"['$===']($case) || "nil"['$===']($case) || "true"['$===']($case) || "false"['$===']($case) || "line"['$===']($case) || "file"['$===']($case)) {return ref}else if ("const"['$===']($case)) {return ref}else if ("ivar"['$===']($case) || "gvar"['$===']($case) || "cvar"['$===']($case)) {return ref}else if ("int"['$===']($case)) {return ref}else if ("str"['$===']($case)) {return ref}else if ("identifier"['$===']($case)) {result = (function() {if ((($a = self.$scope()['$has_local?'](ref['$[]'](1))) !== nil && (!$a.$$is_boolean || $a == true))) {
          return self.$s("lvar", ref['$[]'](1))
          } else {
          return self.$s("call", nil, ref['$[]'](1), self.$s("arglist"))
        }; return nil; })();
        (($a = [ref.$source()]), $b = result, $b['$source='].apply($b, $a), $a[$a.length-1]);
        return result;}else {return self.$raise("Bad var_ref type: " + (ref.$type()))}})();
      };

      def.$new_super = function(kw, args) {
        var $a, $b, $c, self = this, sexp = nil;

        if ((($a = args['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
          sexp = self.$s("super", nil)
          } else {
          sexp = self.$s("super", ($a = self).$s.apply($a, ["arglist"].concat(args)))
        };
        (($b = [self.$source(kw)]), $c = sexp, $c['$source='].apply($c, $b), $b[$b.length-1]);
        return sexp;
      };

      def.$new_yield = function(args) {
        var $a, self = this;

        ((($a = args) !== false && $a !== nil) ? $a : args = []);
        return ($a = self).$s.apply($a, ["yield"].concat(args));
      };

      def.$new_xstr = function(start_t, str, end_t) {
        var $a, $b, self = this, $case = nil;

        if (str !== false && str !== nil) {
          } else {
          return self.$s("xstr", "")
        };
        $case = str.$type();if ("str"['$===']($case)) {(($a = ["xstr"]), $b = str, $b['$type='].apply($b, $a), $a[$a.length-1])}else if ("dstr"['$===']($case)) {(($a = ["dxstr"]), $b = str, $b['$type='].apply($b, $a), $a[$a.length-1])}else if ("evstr"['$===']($case)) {str = self.$s("dxstr", "", str)};
        (($a = [self.$source(start_t)]), $b = str, $b['$source='].apply($b, $a), $a[$a.length-1]);
        return str;
      };

      def.$new_dsym = function(str) {
        var $a, $b, self = this, $case = nil;

        if (str !== false && str !== nil) {
          } else {
          return self.$s("sym", "")
        };
        $case = str.$type();if ("str"['$===']($case)) {(($a = ["sym"]), $b = str, $b['$type='].apply($b, $a), $a[$a.length-1]);
        str['$[]='](1, str['$[]'](1).$to_sym());}else if ("dstr"['$===']($case)) {(($a = ["dsym"]), $b = str, $b['$type='].apply($b, $a), $a[$a.length-1])}else if ("evstr"['$===']($case)) {str = self.$s("dsym", str)};
        return str;
      };

      def.$new_evstr = function(str) {
        var self = this;

        return self.$s("evstr", str);
      };

      def.$new_str = function(str) {
        var $a, $b, $c, self = this;

        if (str !== false && str !== nil) {
          } else {
          return self.$s("str", "")
        };
        if ((($a = ($b = (($c = str.$size()['$=='](3)) ? str['$[]'](1)['$==']("") : $c), $b !== false && $b !== nil ?str.$type()['$==']("str") : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return str['$[]'](2)
        } else if ((($a = (($b = str.$type()['$==']("str")) ? $rb_gt(str.$size(), 3) : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
          (($a = ["dstr"]), $b = str, $b['$type='].apply($b, $a), $a[$a.length-1]);
          return str;
        } else if (str.$type()['$==']("evstr")) {
          return self.$s("dstr", "", str)
          } else {
          return str
        };
      };

      def.$new_regexp = function(reg, ending) {
        var $a, $b, self = this, $case = nil;

        if (reg !== false && reg !== nil) {
          } else {
          return self.$s("regexp", "")
        };
        return (function() {$case = reg.$type();if ("str"['$===']($case)) {return self.$s("regexp", reg['$[]'](1), self.$value(ending))}else if ("evstr"['$===']($case)) {return self.$s("dregx", "", reg)}else if ("dstr"['$===']($case)) {(($a = ["dregx"]), $b = reg, $b['$type='].apply($b, $a), $a[$a.length-1]);
        return reg;}else { return nil }})();
      };

      def.$str_append = function(str, str2) {
        var self = this;

        if (str !== false && str !== nil) {
          } else {
          return str2
        };
        if (str2 !== false && str2 !== nil) {
          } else {
          return str
        };
        if (str.$type()['$==']("evstr")) {
          str = self.$s("dstr", "", str)
        } else if (str.$type()['$==']("str")) {
          str = self.$s("dstr", str['$[]'](1))};
        str['$<<'](str2);
        return str;
      };

      return (def.$new_str_content = function(tok) {
        var self = this;

        return self.$s1("str", self.$value(tok), self.$source(tok));
      }, nil) && 'new_str_content';
    })(self, (($scope.get('Racc')).$$scope.get('Parser')))
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/fragment"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass;

  Opal.add_stubs(['$attr_reader', '$to_s', '$line', '$column', '$inspect']);
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base, $super) {
      function $Fragment(){};
      var self = $Fragment = $klass($base, $super, 'Fragment', $Fragment);

      var def = self.$$proto, $scope = self.$$scope;

      def.sexp = def.code = nil;
      self.$attr_reader("code");

      def.$initialize = function(code, sexp) {
        var self = this;

        if (sexp == null) {
          sexp = nil
        }
        self.code = code.$to_s();
        return self.sexp = sexp;
      };

      def.$to_code = function() {
        var $a, self = this;

        if ((($a = self.sexp) !== nil && (!$a.$$is_boolean || $a == true))) {
          return "/*:" + (self.sexp.$line()) + ":" + (self.sexp.$column()) + "*/" + (self.code)
          } else {
          return self.code
        };
      };

      def.$inspect = function() {
        var self = this;

        return "f(" + (self.code.$inspect()) + ")";
      };

      def.$line = function() {
        var $a, self = this;

        if ((($a = self.sexp) !== nil && (!$a.$$is_boolean || $a == true))) {
          return self.sexp.$line()
          } else {
          return nil
        };
      };

      return (def.$column = function() {
        var $a, self = this;

        if ((($a = self.sexp) !== nil && (!$a.$$is_boolean || $a == true))) {
          return self.sexp.$column()
          } else {
          return nil
        };
      }, nil) && 'column';
    })(self, null)
  })(self)
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/helpers"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module;

  Opal.add_stubs(['$require', '$valid_name?', '$inspect', '$=~', '$!', '$to_s', '$to_sym', '$indent', '$to_proc', '$compiler', '$parser_indent', '$push', '$current_indent', '$js_truthy_optimize', '$with_temp', '$fragment', '$expr', '$==', '$type', '$[]', '$uses_block!', '$scope', '$block_name', '$include?', '$dup']);
  self.$require("opal/regexp_anchors");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base) {
        var self = $module($base, 'Helpers');

        var def = self.$$proto, $scope = self.$$scope, TMP_1;

        Opal.cdecl($scope, 'ES51_RESERVED_WORD', (new RegExp("" + $scope.get('REGEXP_START') + "(?:do|if|in|for|let|new|try|var|case|else|enum|eval|false|null|this|true|void|with|break|catch|class|const|super|throw|while|yield|delete|export|import|public|return|static|switch|typeof|default|extends|finally|package|private|continue|debugger|function|arguments|interface|protected|implements|instanceof)" + $scope.get('REGEXP_END'))));

        Opal.cdecl($scope, 'ES3_RESERVED_WORD_EXCLUSIVE', (new RegExp("" + $scope.get('REGEXP_START') + "(?:int|byte|char|goto|long|final|float|short|double|native|throws|boolean|abstract|volatile|transient|synchronized)" + $scope.get('REGEXP_END'))));

        Opal.cdecl($scope, 'IMMUTABLE_PROPS', (new RegExp("" + $scope.get('REGEXP_START') + "(?:NaN|Infinity|undefined)" + $scope.get('REGEXP_END'))));

        Opal.cdecl($scope, 'BASIC_IDENTIFIER_RULES', (new RegExp("" + $scope.get('REGEXP_START') + "[$_a-z][$_a-z\\d]*" + $scope.get('REGEXP_END'))));

        Opal.defn(self, '$property', function(name) {
          var $a, self = this;

          if ((($a = self['$valid_name?'](name)) !== nil && (!$a.$$is_boolean || $a == true))) {
            return "." + (name)
            } else {
            return "[" + (name.$inspect()) + "]"
          };
        });

        Opal.defn(self, '$valid_name?', function(name) {
          var $a, $b, $c, self = this;

          return ($a = $scope.get('BASIC_IDENTIFIER_RULES')['$=~'](name), $a !== false && $a !== nil ?(((($b = ((($c = $scope.get('ES51_RESERVED_WORD')['$=~'](name)) !== false && $c !== nil) ? $c : $scope.get('ES3_RESERVED_WORD_EXCLUSIVE')['$=~'](name))) !== false && $b !== nil) ? $b : $scope.get('IMMUTABLE_PROPS')['$=~'](name)))['$!']() : $a);
        });

        Opal.defn(self, '$variable', function(name) {
          var $a, self = this;

          if ((($a = self['$valid_name?'](name.$to_s())) !== nil && (!$a.$$is_boolean || $a == true))) {
            return name
            } else {
            return "" + (name) + "$"
          };
        });

        Opal.defn(self, '$lvar_to_js', function(var$) {
          var $a, self = this;

          if ((($a = self['$valid_name?'](var$.$to_s())) !== nil && (!$a.$$is_boolean || $a == true))) {
            } else {
            var$ = "" + (var$) + "$"
          };
          return var$.$to_sym();
        });

        Opal.defn(self, '$mid_to_jsid', function(mid) {
          var $a, self = this;

          if ((($a = /\=|\+|\-|\*|\/|\!|\?|\<|\>|\&|\||\^|\%|\~|\[/['$=~'](mid.$to_s())) !== nil && (!$a.$$is_boolean || $a == true))) {
            return "['$" + (mid) + "']"
            } else {
            return $rb_plus(".$", mid)
          };
        });

        Opal.defn(self, '$indent', TMP_1 = function() {
          var $a, $b, self = this, $iter = TMP_1.$$p, block = $iter || nil;

          TMP_1.$$p = null;
          return ($a = ($b = self.$compiler()).$indent, $a.$$p = block.$to_proc(), $a).call($b);
        });

        Opal.defn(self, '$current_indent', function() {
          var self = this;

          return self.$compiler().$parser_indent();
        });

        Opal.defn(self, '$line', function(strs) {
          var $a, self = this;

          strs = $slice.call(arguments, 0);
          self.$push("\n" + (self.$current_indent()));
          return ($a = self).$push.apply($a, [].concat(strs));
        });

        Opal.defn(self, '$empty_line', function() {
          var self = this;

          return self.$push("\n");
        });

        Opal.defn(self, '$js_truthy', function(sexp) {
          var $a, $b, TMP_2, self = this, optimize = nil;

          if ((($a = optimize = self.$js_truthy_optimize(sexp)) !== nil && (!$a.$$is_boolean || $a == true))) {
            return optimize};
          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_2 = function(tmp){var self = TMP_2.$$s || this;
if (tmp == null) tmp = nil;
          return [self.$fragment("((" + (tmp) + " = "), self.$expr(sexp), self.$fragment(") !== nil && (!" + (tmp) + ".$$is_boolean || " + (tmp) + " == true))")]}, TMP_2.$$s = self, TMP_2), $a).call($b);
        });

        Opal.defn(self, '$js_falsy', function(sexp) {
          var $a, $b, TMP_3, self = this, mid = nil;

          if (sexp.$type()['$==']("call")) {
            mid = sexp['$[]'](2);
            if (mid['$==']("block_given?")) {
              self.$scope()['$uses_block!']();
              return "" + (self.$scope().$block_name()) + " === nil";};};
          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_3 = function(tmp){var self = TMP_3.$$s || this;
if (tmp == null) tmp = nil;
          return [self.$fragment("((" + (tmp) + " = "), self.$expr(sexp), self.$fragment(") === nil || (" + (tmp) + ".$$is_boolean && " + (tmp) + " == false))")]}, TMP_3.$$s = self, TMP_3), $a).call($b);
        });

        Opal.defn(self, '$js_truthy_optimize', function(sexp) {
          var $a, self = this, mid = nil;

          if (sexp.$type()['$==']("call")) {
            mid = sexp['$[]'](2);
            if (mid['$==']("block_given?")) {
              return self.$expr(sexp)
            } else if ((($a = (($scope.get('Compiler')).$$scope.get('COMPARE'))['$include?'](mid.$to_s())) !== nil && (!$a.$$is_boolean || $a == true))) {
              return self.$expr(sexp)
            } else if (mid['$==']("==")) {
              return self.$expr(sexp)
              } else {
              return nil
            };
          } else if ((($a = ["lvar", "self"]['$include?'](sexp.$type())) !== nil && (!$a.$$is_boolean || $a == true))) {
            return [self.$expr(sexp.$dup()), self.$fragment(" !== false && "), self.$expr(sexp.$dup()), self.$fragment(" !== nil")]
            } else {
            return nil
          };
        });
      })(self)
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/base"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass, $hash2 = Opal.hash2, $range = Opal.range;

  Opal.add_stubs(['$require', '$include', '$each', '$[]=', '$handlers', '$each_with_index', '$define_method', '$[]', '$attr_reader', '$type', '$compile', '$raise', '$is_a?', '$fragment', '$<<', '$unshift', '$reverse', '$push', '$new', '$error', '$scope', '$s', '$==', '$process', '$expr', '$add_scope_local', '$to_sym', '$add_scope_ivar', '$add_scope_gvar', '$add_scope_temp', '$helper', '$with_temp', '$to_proc', '$in_while?', '$instance_variable_get']);
  self.$require("opal/nodes/helpers");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $Base(){};
        var self = $Base = $klass($base, $super, 'Base', $Base);

        var def = self.$$proto, $scope = self.$$scope, TMP_6;

        def.sexp = def.fragments = def.compiler = def.level = nil;
        self.$include($scope.get('Helpers'));

        Opal.defs(self, '$handlers', function() {
          var $a, self = this;
          if (self.handlers == null) self.handlers = nil;

          return ((($a = self.handlers) !== false && $a !== nil) ? $a : self.handlers = $hash2([], {}));
        });

        Opal.defs(self, '$handle', function(types) {
          var $a, $b, TMP_1, self = this;

          types = $slice.call(arguments, 0);
          return ($a = ($b = types).$each, $a.$$p = (TMP_1 = function(type){var self = TMP_1.$$s || this;
if (type == null) type = nil;
          return $scope.get('Base').$handlers()['$[]='](type, self)}, TMP_1.$$s = self, TMP_1), $a).call($b);
        });

        Opal.defs(self, '$children', function(names) {
          var $a, $b, TMP_2, self = this;

          names = $slice.call(arguments, 0);
          return ($a = ($b = names).$each_with_index, $a.$$p = (TMP_2 = function(name, idx){var self = TMP_2.$$s || this, $a, $b, TMP_3;
if (name == null) name = nil;if (idx == null) idx = nil;
          return ($a = ($b = self).$define_method, $a.$$p = (TMP_3 = function(){var self = TMP_3.$$s || this;
              if (self.sexp == null) self.sexp = nil;

            return self.sexp['$[]']($rb_plus(idx, 1))}, TMP_3.$$s = self, TMP_3), $a).call($b, name)}, TMP_2.$$s = self, TMP_2), $a).call($b);
        });

        self.$attr_reader("compiler", "type");

        def.$initialize = function(sexp, level, compiler) {
          var self = this;

          self.sexp = sexp;
          self.type = sexp.$type();
          self.level = level;
          return self.compiler = compiler;
        };

        def.$children = function() {
          var self = this;

          return self.sexp['$[]']($range(1, -1, false));
        };

        def.$compile_to_fragments = function() {
          var $a, $b, self = this;

          if ((($a = (($b = self['fragments'], $b != null && $b !== nil) ? 'instance-variable' : nil)) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.fragments};
          self.fragments = [];
          self.$compile();
          return self.fragments;
        };

        def.$compile = function() {
          var self = this;

          return self.$raise("Not Implemented");
        };

        def.$push = function(strs) {
          var $a, $b, TMP_4, self = this;

          strs = $slice.call(arguments, 0);
          return ($a = ($b = strs).$each, $a.$$p = (TMP_4 = function(str){var self = TMP_4.$$s || this, $a;
            if (self.fragments == null) self.fragments = nil;
if (str == null) str = nil;
          if ((($a = str['$is_a?']($scope.get('String'))) !== nil && (!$a.$$is_boolean || $a == true))) {
              str = self.$fragment(str)};
            return self.fragments['$<<'](str);}, TMP_4.$$s = self, TMP_4), $a).call($b);
        };

        def.$unshift = function(strs) {
          var $a, $b, TMP_5, self = this;

          strs = $slice.call(arguments, 0);
          return ($a = ($b = strs.$reverse()).$each, $a.$$p = (TMP_5 = function(str){var self = TMP_5.$$s || this, $a;
            if (self.fragments == null) self.fragments = nil;
if (str == null) str = nil;
          if ((($a = str['$is_a?']($scope.get('String'))) !== nil && (!$a.$$is_boolean || $a == true))) {
              str = self.$fragment(str)};
            return self.fragments.$unshift(str);}, TMP_5.$$s = self, TMP_5), $a).call($b);
        };

        def.$wrap = function(pre, post) {
          var self = this;

          self.$unshift(pre);
          return self.$push(post);
        };

        def.$fragment = function(str) {
          var self = this;

          return (($scope.get('Opal')).$$scope.get('Fragment')).$new(str, self.sexp);
        };

        def.$error = function(msg) {
          var self = this;

          return self.compiler.$error(msg);
        };

        def.$scope = function() {
          var self = this;

          return self.compiler.$scope();
        };

        def.$s = function(args) {
          var $a, self = this;

          args = $slice.call(arguments, 0);
          return ($a = self.compiler).$s.apply($a, [].concat(args));
        };

        def['$expr?'] = function() {
          var self = this;

          return self.level['$==']("expr");
        };

        def['$recv?'] = function() {
          var self = this;

          return self.level['$==']("recv");
        };

        def['$stmt?'] = function() {
          var self = this;

          return self.level['$==']("stmt");
        };

        def.$process = function(sexp, level) {
          var self = this;

          if (level == null) {
            level = "expr"
          }
          return self.compiler.$process(sexp, level);
        };

        def.$expr = function(sexp) {
          var self = this;

          return self.compiler.$process(sexp, "expr");
        };

        def.$recv = function(sexp) {
          var self = this;

          return self.compiler.$process(sexp, "recv");
        };

        def.$stmt = function(sexp) {
          var self = this;

          return self.compiler.$process(sexp, "stmt");
        };

        def.$expr_or_nil = function(sexp) {
          var self = this;

          if (sexp !== false && sexp !== nil) {
            return self.$expr(sexp)
            } else {
            return "nil"
          };
        };

        def.$add_local = function(name) {
          var self = this;

          return self.$scope().$add_scope_local(name.$to_sym());
        };

        def.$add_ivar = function(name) {
          var self = this;

          return self.$scope().$add_scope_ivar(name);
        };

        def.$add_gvar = function(name) {
          var self = this;

          return self.$scope().$add_scope_gvar(name);
        };

        def.$add_temp = function(temp) {
          var self = this;

          return self.$scope().$add_scope_temp(temp);
        };

        def.$helper = function(name) {
          var self = this;

          return self.compiler.$helper(name);
        };

        def.$with_temp = TMP_6 = function() {
          var $a, $b, self = this, $iter = TMP_6.$$p, block = $iter || nil;

          TMP_6.$$p = null;
          return ($a = ($b = self.compiler).$with_temp, $a.$$p = block.$to_proc(), $a).call($b);
        };

        def['$in_while?'] = function() {
          var self = this;

          return self.compiler['$in_while?']();
        };

        return (def.$while_loop = function() {
          var self = this;

          return self.compiler.$instance_variable_get("@while_loop");
        }, nil) && 'while_loop';
      })(self, null)
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/literal"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass, $hash2 = Opal.hash2, $gvars = Opal.gvars;

  Opal.add_stubs(['$require', '$handle', '$push', '$to_s', '$type', '$children', '$value', '$recv?', '$wrap', '$join', '$keys', '$gsub', '$even?', '$length', '$chop', '$[]', '$translate_escape_chars', '$inspect', '$===', '$new', '$flags', '$each_line', '$==', '$s', '$source=', '$line', '$include', '$stmt?', '$!', '$include?', '$compile_split_lines', '$needs_semicolon?', '$each_with_index', '$expr', '$raise', '$last', '$each', '$requires_semicolon', '$helper', '$start', '$finish']);
  self.$require("opal/nodes/base");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $ValueNode(){};
        var self = $ValueNode = $klass($base, $super, 'ValueNode', $ValueNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("true", "false", "self", "nil");

        return (def.$compile = function() {
          var self = this;

          return self.$push(self.$type().$to_s());
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $NumericNode(){};
        var self = $NumericNode = $klass($base, $super, 'NumericNode', $NumericNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("int", "float");

        self.$children("value");

        return (def.$compile = function() {
          var $a, self = this;

          self.$push(self.$value().$to_s());
          if ((($a = self['$recv?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$wrap("(", ")")
            } else {
            return nil
          };
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $StringNode(){};
        var self = $StringNode = $klass($base, $super, 'StringNode', $StringNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("str");

        self.$children("value");

        Opal.cdecl($scope, 'ESCAPE_CHARS', $hash2(["a", "e"], {"a": "\\u0007", "e": "\\u001b"}));

        Opal.cdecl($scope, 'ESCAPE_REGEX', (new RegExp("(\\\\+)([" + $scope.get('ESCAPE_CHARS').$keys().$join("") + "])")));

        def.$translate_escape_chars = function(inspect_string) {
          var $a, $b, TMP_1, self = this;

          return ($a = ($b = inspect_string).$gsub, $a.$$p = (TMP_1 = function(original){var self = TMP_1.$$s || this, $a, $b;
if (original == null) original = nil;
          if ((($a = (($b = $gvars['~']) === nil ? nil : $b['$[]'](1)).$length()['$even?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
              return original
              } else {
              return $rb_plus((($a = $gvars['~']) === nil ? nil : $a['$[]'](1)).$chop(), $scope.get('ESCAPE_CHARS')['$[]']((($a = $gvars['~']) === nil ? nil : $a['$[]'](2))))
            }}, TMP_1.$$s = self, TMP_1), $a).call($b, $scope.get('ESCAPE_REGEX'));
        };

        return (def.$compile = function() {
          var self = this;

          return self.$push(self.$translate_escape_chars(self.$value().$inspect()));
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $SymbolNode(){};
        var self = $SymbolNode = $klass($base, $super, 'SymbolNode', $SymbolNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("sym");

        self.$children("value");

        return (def.$compile = function() {
          var self = this;

          return self.$push(self.$value().$to_s().$inspect());
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $RegexpNode(){};
        var self = $RegexpNode = $klass($base, $super, 'RegexpNode', $RegexpNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("regexp");

        self.$children("value", "flags");

        return (def.$compile = function() {
          var self = this, $case = nil, message = nil;

          return (function() {$case = self.$value();if (""['$===']($case)) {return self.$push("/(?:)/")}else if (/\?\<\w+\>/['$===']($case)) {message = "named captures are not supported in javascript: " + (self.$value().$inspect());
          return self.$push("self.$raise(new SyntaxError('" + (message) + "'))");}else {return self.$push("" + ($scope.get('Regexp').$new(self.$value()).$inspect()) + (self.$flags()))}})();
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base) {
        var self = $module($base, 'XStringLineSplitter');

        var def = self.$$proto, $scope = self.$$scope;

        Opal.defn(self, '$compile_split_lines', function(value, sexp) {
          var $a, $b, TMP_2, self = this, idx = nil;

          idx = 0;
          return ($a = ($b = value).$each_line, $a.$$p = (TMP_2 = function(line){var self = TMP_2.$$s || this, $a, $b, line_sexp = nil, frag = nil;
if (line == null) line = nil;
          if (idx['$=='](0)) {
              self.$push(line)
              } else {
              line_sexp = self.$s();
              (($a = [[$rb_plus(sexp.$line(), idx), 0]]), $b = line_sexp, $b['$source='].apply($b, $a), $a[$a.length-1]);
              frag = $scope.get('Fragment').$new(line, line_sexp);
              self.$push(frag);
            };
            return idx = $rb_plus(idx, 1);}, TMP_2.$$s = self, TMP_2), $a).call($b);
        })
      })(self);

      (function($base, $super) {
        function $XStringNode(){};
        var self = $XStringNode = $klass($base, $super, 'XStringNode', $XStringNode);

        var def = self.$$proto, $scope = self.$$scope;

        def.sexp = nil;
        self.$include($scope.get('XStringLineSplitter'));

        self.$handle("xstr");

        self.$children("value");

        def['$needs_semicolon?'] = function() {
          var $a, self = this;

          return ($a = self['$stmt?'](), $a !== false && $a !== nil ?self.$value().$to_s()['$include?'](";")['$!']() : $a);
        };

        def.$compile = function() {
          var $a, self = this;

          self.$compile_split_lines(self.$value().$to_s(), self.sexp);
          if ((($a = self['$needs_semicolon?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.$push(";")};
          if ((($a = self['$recv?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$wrap("(", ")")
            } else {
            return nil
          };
        };

        return (def.$start_line = function() {
          var self = this;

          return self.sexp.$line();
        }, nil) && 'start_line';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $DynamicStringNode(){};
        var self = $DynamicStringNode = $klass($base, $super, 'DynamicStringNode', $DynamicStringNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("dstr");

        return (def.$compile = function() {
          var $a, $b, TMP_3, self = this;

          return ($a = ($b = self.$children()).$each_with_index, $a.$$p = (TMP_3 = function(part, idx){var self = TMP_3.$$s || this, $a;
if (part == null) part = nil;if (idx == null) idx = nil;
          if (idx['$=='](0)) {
              } else {
              self.$push(" + ")
            };
            if ((($a = $scope.get('String')['$==='](part)) !== nil && (!$a.$$is_boolean || $a == true))) {
              self.$push(part.$inspect())
            } else if (part.$type()['$==']("evstr")) {
              self.$push("(");
              self.$push(self.$expr(part['$[]'](1)));
              self.$push(")");
            } else if (part.$type()['$==']("str")) {
              self.$push(part['$[]'](1).$inspect())
            } else if (part.$type()['$==']("dstr")) {
              self.$push("(");
              self.$push(self.$expr(part));
              self.$push(")");
              } else {
              self.$raise("Bad dstr part " + (part.$inspect()))
            };
            if ((($a = self['$recv?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
              return self.$wrap("(", ")")
              } else {
              return nil
            };}, TMP_3.$$s = self, TMP_3), $a).call($b);
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $DynamicSymbolNode(){};
        var self = $DynamicSymbolNode = $klass($base, $super, 'DynamicSymbolNode', $DynamicSymbolNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("dsym");

        return (def.$compile = function() {
          var $a, $b, TMP_4, self = this;

          ($a = ($b = self.$children()).$each_with_index, $a.$$p = (TMP_4 = function(part, idx){var self = TMP_4.$$s || this, $a;
if (part == null) part = nil;if (idx == null) idx = nil;
          if (idx['$=='](0)) {
              } else {
              self.$push(" + ")
            };
            if ((($a = $scope.get('String')['$==='](part)) !== nil && (!$a.$$is_boolean || $a == true))) {
              return self.$push(part.$inspect())
            } else if (part.$type()['$==']("evstr")) {
              return self.$push(self.$expr(self.$s("call", part.$last(), "to_s", self.$s("arglist"))))
            } else if (part.$type()['$==']("str")) {
              return self.$push(part.$last().$inspect())
              } else {
              return self.$raise("Bad dsym part")
            };}, TMP_4.$$s = self, TMP_4), $a).call($b);
          return self.$wrap("(", ")");
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $DynamicXStringNode(){};
        var self = $DynamicXStringNode = $klass($base, $super, 'DynamicXStringNode', $DynamicXStringNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$include($scope.get('XStringLineSplitter'));

        self.$handle("dxstr");

        def.$requires_semicolon = function(code) {
          var $a, self = this;

          return ($a = self['$stmt?'](), $a !== false && $a !== nil ?code['$include?'](";")['$!']() : $a);
        };

        return (def.$compile = function() {
          var $a, $b, TMP_5, self = this, needs_semicolon = nil;

          needs_semicolon = false;
          ($a = ($b = self.$children()).$each, $a.$$p = (TMP_5 = function(part){var self = TMP_5.$$s || this, $a;
            if (self.sexp == null) self.sexp = nil;
if (part == null) part = nil;
          if ((($a = $scope.get('String')['$==='](part)) !== nil && (!$a.$$is_boolean || $a == true))) {
              self.$compile_split_lines(part.$to_s(), self.sexp);
              if ((($a = self.$requires_semicolon(part.$to_s())) !== nil && (!$a.$$is_boolean || $a == true))) {
                return needs_semicolon = true
                } else {
                return nil
              };
            } else if (part.$type()['$==']("evstr")) {
              return self.$push(self.$expr(part['$[]'](1)))
            } else if (part.$type()['$==']("str")) {
              self.$compile_split_lines(part.$last().$to_s(), part);
              if ((($a = self.$requires_semicolon(part.$last().$to_s())) !== nil && (!$a.$$is_boolean || $a == true))) {
                return needs_semicolon = true
                } else {
                return nil
              };
              } else {
              return self.$raise("Bad dxstr part")
            }}, TMP_5.$$s = self, TMP_5), $a).call($b);
          if (needs_semicolon !== false && needs_semicolon !== nil) {
            self.$push(";")};
          if ((($a = self['$recv?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$wrap("(", ")")
            } else {
            return nil
          };
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $DynamicRegexpNode(){};
        var self = $DynamicRegexpNode = $klass($base, $super, 'DynamicRegexpNode', $DynamicRegexpNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("dregx");

        return (def.$compile = function() {
          var $a, $b, TMP_6, self = this;

          ($a = ($b = self.$children()).$each_with_index, $a.$$p = (TMP_6 = function(part, idx){var self = TMP_6.$$s || this, $a;
if (part == null) part = nil;if (idx == null) idx = nil;
          if (idx['$=='](0)) {
              } else {
              self.$push(" + ")
            };
            if ((($a = $scope.get('String')['$==='](part)) !== nil && (!$a.$$is_boolean || $a == true))) {
              return self.$push(part.$inspect())
            } else if (part.$type()['$==']("str")) {
              return self.$push(part['$[]'](1).$inspect())
              } else {
              return self.$push(self.$expr(part['$[]'](1)))
            };}, TMP_6.$$s = self, TMP_6), $a).call($b);
          return self.$wrap("(new RegExp(", "))");
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $InclusiveRangeNode(){};
        var self = $InclusiveRangeNode = $klass($base, $super, 'InclusiveRangeNode', $InclusiveRangeNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("irange");

        self.$children("start", "finish");

        return (def.$compile = function() {
          var self = this;

          self.$helper("range");
          return self.$push("$range(", self.$expr(self.$start()), ", ", self.$expr(self.$finish()), ", false)");
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $ExclusiveRangeNode(){};
        var self = $ExclusiveRangeNode = $klass($base, $super, 'ExclusiveRangeNode', $ExclusiveRangeNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("erange");

        self.$children("start", "finish");

        return (def.$compile = function() {
          var self = this;

          self.$helper("range");
          return self.$push("$range(", self.$expr(self.$start()), ", ", self.$expr(self.$finish()), ", true)");
        }, nil) && 'compile';
      })(self, $scope.get('Base'));
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/variables"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass, $range = Opal.range;

  Opal.add_stubs(['$require', '$handle', '$children', '$irb?', '$compiler', '$top?', '$scope', '$using_irb?', '$push', '$variable', '$to_s', '$var_name', '$with_temp', '$property', '$wrap', '$expr', '$value', '$add_local', '$recv?', '$[]', '$name', '$add_ivar', '$helper', '$==', '$handle_global_match', '$handle_post_match', '$handle_pre_match', '$add_gvar', '$index']);
  self.$require("opal/nodes/base");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $LocalVariableNode(){};
        var self = $LocalVariableNode = $klass($base, $super, 'LocalVariableNode', $LocalVariableNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("lvar");

        self.$children("var_name");

        def['$using_irb?'] = function() {
          var $a, self = this;

          return ($a = self.$compiler()['$irb?'](), $a !== false && $a !== nil ?self.$scope()['$top?']() : $a);
        };

        return (def.$compile = function() {
          var $a, $b, TMP_1, self = this;

          if ((($a = self['$using_irb?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            } else {
            return self.$push(self.$variable(self.$var_name().$to_s()))
          };
          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_1 = function(tmp){var self = TMP_1.$$s || this;
if (tmp == null) tmp = nil;
          self.$push(self.$property(self.$var_name().$to_s()));
            return self.$wrap("((" + (tmp) + " = Opal.irb_vars", ") == null ? nil : " + (tmp) + ")");}, TMP_1.$$s = self, TMP_1), $a).call($b);
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $LocalAssignNode(){};
        var self = $LocalAssignNode = $klass($base, $super, 'LocalAssignNode', $LocalAssignNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("lasgn");

        self.$children("var_name", "value");

        def['$using_irb?'] = function() {
          var $a, self = this;

          return ($a = self.$compiler()['$irb?'](), $a !== false && $a !== nil ?self.$scope()['$top?']() : $a);
        };

        return (def.$compile = function() {
          var $a, self = this;

          if ((($a = self['$using_irb?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.$push("Opal.irb_vars" + (self.$property(self.$var_name().$to_s())) + " = ");
            self.$push(self.$expr(self.$value()));
            } else {
            self.$add_local(self.$variable(self.$var_name().$to_s()));
            self.$push("" + (self.$variable(self.$var_name().$to_s())) + " = ");
            self.$push(self.$expr(self.$value()));
          };
          if ((($a = self['$recv?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$wrap("(", ")")
            } else {
            return nil
          };
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $InstanceVariableNode(){};
        var self = $InstanceVariableNode = $klass($base, $super, 'InstanceVariableNode', $InstanceVariableNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("ivar");

        self.$children("name");

        def.$var_name = function() {
          var self = this;

          return self.$name().$to_s()['$[]']($range(1, -1, false));
        };

        return (def.$compile = function() {
          var self = this, name = nil;

          name = self.$property(self.$var_name());
          self.$add_ivar(name);
          return self.$push("self" + (name));
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $InstanceAssignNode(){};
        var self = $InstanceAssignNode = $klass($base, $super, 'InstanceAssignNode', $InstanceAssignNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("iasgn");

        self.$children("name", "value");

        def.$var_name = function() {
          var self = this;

          return self.$name().$to_s()['$[]']($range(1, -1, false));
        };

        return (def.$compile = function() {
          var self = this, name = nil;

          name = self.$property(self.$var_name());
          self.$push("self" + (name) + " = ");
          return self.$push(self.$expr(self.$value()));
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $GlobalVariableNode(){};
        var self = $GlobalVariableNode = $klass($base, $super, 'GlobalVariableNode', $GlobalVariableNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("gvar");

        self.$children("name");

        def.$var_name = function() {
          var self = this;

          return self.$name().$to_s()['$[]']($range(1, -1, false));
        };

        def.$compile = function() {
          var self = this, name = nil;

          self.$helper("gvars");
          if (self.$var_name()['$==']("&")) {
            return self.$handle_global_match()
          } else if (self.$var_name()['$==']("'")) {
            return self.$handle_post_match()
          } else if (self.$var_name()['$==']("`")) {
            return self.$handle_pre_match()};
          name = self.$property(self.$var_name());
          self.$add_gvar(name);
          return self.$push("$gvars" + (name));
        };

        def.$handle_global_match = function() {
          var $a, $b, TMP_2, self = this;

          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_2 = function(tmp){var self = TMP_2.$$s || this;
if (tmp == null) tmp = nil;
          return self.$push("((" + (tmp) + " = $gvars['~']) === nil ? nil : " + (tmp) + "['$[]'](0))")}, TMP_2.$$s = self, TMP_2), $a).call($b);
        };

        def.$handle_pre_match = function() {
          var $a, $b, TMP_3, self = this;

          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_3 = function(tmp){var self = TMP_3.$$s || this;
if (tmp == null) tmp = nil;
          return self.$push("((" + (tmp) + " = $gvars['~']) === nil ? nil : " + (tmp) + ".$pre_match())")}, TMP_3.$$s = self, TMP_3), $a).call($b);
        };

        return (def.$handle_post_match = function() {
          var $a, $b, TMP_4, self = this;

          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_4 = function(tmp){var self = TMP_4.$$s || this;
if (tmp == null) tmp = nil;
          return self.$push("((" + (tmp) + " = $gvars['~']) === nil ? nil : " + (tmp) + ".$post_match())")}, TMP_4.$$s = self, TMP_4), $a).call($b);
        }, nil) && 'handle_post_match';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $GlobalAssignNode(){};
        var self = $GlobalAssignNode = $klass($base, $super, 'GlobalAssignNode', $GlobalAssignNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("gasgn");

        self.$children("name", "value");

        def.$var_name = function() {
          var self = this;

          return self.$name().$to_s()['$[]']($range(1, -1, false));
        };

        return (def.$compile = function() {
          var self = this, name = nil;

          self.$helper("gvars");
          name = self.$property(self.$var_name());
          self.$push("$gvars" + (name) + " = ");
          return self.$push(self.$expr(self.$value()));
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $BackrefNode(){};
        var self = $BackrefNode = $klass($base, $super, 'BackrefNode', $BackrefNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("nth_ref");

        self.$children("index");

        return (def.$compile = function() {
          var $a, $b, TMP_5, self = this;

          self.$helper("gvars");
          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_5 = function(tmp){var self = TMP_5.$$s || this;
if (tmp == null) tmp = nil;
          return self.$push("((" + (tmp) + " = $gvars['~']) === nil ? nil : " + (tmp) + "['$[]'](" + (self.$index()) + "))")}, TMP_5.$$s = self, TMP_5), $a).call($b);
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $ClassVariableNode(){};
        var self = $ClassVariableNode = $klass($base, $super, 'ClassVariableNode', $ClassVariableNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("cvar");

        self.$children("name");

        return (def.$compile = function() {
          var $a, $b, TMP_6, self = this;

          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_6 = function(tmp){var self = TMP_6.$$s || this;
if (tmp == null) tmp = nil;
          return self.$push("((" + (tmp) + " = Opal.cvars['" + (self.$name()) + "']) == null ? nil : " + (tmp) + ")")}, TMP_6.$$s = self, TMP_6), $a).call($b);
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $ClassVarAssignNode(){};
        var self = $ClassVarAssignNode = $klass($base, $super, 'ClassVarAssignNode', $ClassVarAssignNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("casgn");

        self.$children("name", "value");

        return (def.$compile = function() {
          var self = this;

          self.$push("(Opal.cvars['" + (self.$name()) + "'] = ");
          self.$push(self.$expr(self.$value()));
          return self.$push(")");
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $ClassVarDeclNode(){};
        var self = $ClassVarDeclNode = $klass($base, $super, 'ClassVarDeclNode', $ClassVarDeclNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("cvdecl");

        self.$children("name", "value");

        return (def.$compile = function() {
          var self = this;

          self.$push("(Opal.cvars['" + (self.$name()) + "'] = ");
          self.$push(self.$expr(self.$value()));
          return self.$push(")");
        }, nil) && 'compile';
      })(self, $scope.get('Base'));
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/constants"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$handle', '$children', '$==', '$name', '$eof_content', '$compiler', '$push', '$expr', '$base', '$wrap', '$value']);
  self.$require("opal/nodes/base");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $ConstNode(){};
        var self = $ConstNode = $klass($base, $super, 'ConstNode', $ConstNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("const");

        self.$children("name");

        return (def.$compile = function() {
          var $a, $b, self = this;

          if ((($a = (($b = self.$name()['$==']("DATA")) ? self.$compiler().$eof_content() : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$push("$__END__")
            } else {
            return self.$push("$scope.get('" + (self.$name()) + "')")
          };
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $ConstDeclarationNode(){};
        var self = $ConstDeclarationNode = $klass($base, $super, 'ConstDeclarationNode', $ConstDeclarationNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("cdecl");

        self.$children("name", "base");

        return (def.$compile = function() {
          var self = this;

          self.$push(self.$expr(self.$base()));
          return self.$wrap("Opal.cdecl($scope, '" + (self.$name()) + "', ", ")");
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $ConstAssignNode(){};
        var self = $ConstAssignNode = $klass($base, $super, 'ConstAssignNode', $ConstAssignNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("casgn");

        self.$children("base", "name", "value");

        return (def.$compile = function() {
          var self = this;

          self.$push("Opal.casgn(");
          self.$push(self.$expr(self.$base()));
          self.$push(", '" + (self.$name()) + "', ");
          self.$push(self.$expr(self.$value()));
          return self.$push(")");
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $ConstGetNode(){};
        var self = $ConstGetNode = $klass($base, $super, 'ConstGetNode', $ConstGetNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("colon2");

        self.$children("base", "name");

        return (def.$compile = function() {
          var self = this;

          self.$push("((");
          self.$push(self.$expr(self.$base()));
          return self.$push(").$$scope.get('" + (self.$name()) + "'))");
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $TopConstNode(){};
        var self = $TopConstNode = $klass($base, $super, 'TopConstNode', $TopConstNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("colon3");

        self.$children("name");

        return (def.$compile = function() {
          var self = this;

          return self.$push("Opal.get('" + (self.$name()) + "')");
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $TopConstAssignNode(){};
        var self = $TopConstAssignNode = $klass($base, $super, 'TopConstAssignNode', $TopConstAssignNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("casgn3");

        self.$children("name", "value");

        return (def.$compile = function() {
          var self = this;

          self.$push("Opal.casgn(Opal.Object, '" + (self.$name()) + "', ");
          self.$push(self.$expr(self.$value()));
          return self.$push(")");
        }, nil) && 'compile';
      })(self, $scope.get('Base'));
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["pathname"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $module = Opal.module;

  Opal.add_stubs(['$==', '$raise', '$attr_reader', '$path', '$start_with?', '$!', '$absolute?', '$sub', '$new']);
  (function($base, $super) {
    function $Pathname(){};
    var self = $Pathname = $klass($base, $super, 'Pathname', $Pathname);

    var def = self.$$proto, $scope = self.$$scope;

    def.path = nil;
    def.$initialize = function(path) {
      var self = this;

      if (path['$==']("\x00")) {
        self.$raise($scope.get('ArgumentError'))};
      return self.path = path;
    };

    self.$attr_reader("path");

    def['$=='] = function(other) {
      var self = this;

      return other.$path()['$=='](self.path);
    };

    def['$absolute?'] = function() {
      var self = this;

      return self.path['$start_with?']("/");
    };

    def['$relative?'] = function() {
      var self = this;

      return self['$absolute?']()['$!']();
    };

    def['$root?'] = function() {
      var self = this;

      return self.path['$==']("/");
    };

    def.$parent = function() {
      var $a, self = this, new_path = nil;

      new_path = self.path.$sub(/\/([^\/]+\/?$)/, "");
      if (new_path['$==']("")) {
        new_path = (function() {if ((($a = self['$absolute?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
          return "/"
          } else {
          return "."
        }; return nil; })()};
      return $scope.get('Pathname').$new(new_path);
    };

    def.$sub = function(args) {
      var $a, self = this;

      args = $slice.call(arguments, 0);
      return $scope.get('Pathname').$new(($a = self.path).$sub.apply($a, [].concat(args)));
    };

    def.$cleanpath = function() {
      var self = this;

      return Opal.normalize_loadable_path(self.path);
    };

    def.$to_path = function() {
      var self = this;

      return self.path;
    };

    def.$hash = function() {
      var self = this;

      return self.path;
    };

    Opal.defn(self, '$to_str', def.$to_path);

    return Opal.defn(self, '$to_s', def.$to_path);
  })(self, null);
  return (function($base) {
    var self = $module($base, 'Kernel');

    var def = self.$$proto, $scope = self.$$scope;

    Opal.defn(self, '$Pathname', function(path) {
      var self = this;

      return $scope.get('Pathname').$new(path);
    })
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/runtime_helpers"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$new', '$children', '$==', '$include?', '$to_sym', '$<<', '$define_method', '$to_proc', '$meth', '$__send__', '$raise', '$helper', '$[]', '$arglist', '$js_truthy', '$js_falsy']);
  self.$require("set");
  self.$require("opal/nodes/base");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $RuntimeHelpers(){};
        var self = $RuntimeHelpers = $klass($base, $super, 'RuntimeHelpers', $RuntimeHelpers);

        var def = self.$$proto, $scope = self.$$scope, TMP_1, $a, $b, TMP_2, $c, TMP_3;

        Opal.cdecl($scope, 'HELPERS', $scope.get('Set').$new());

        self.$children("recvr", "meth", "arglist");

        Opal.defs(self, '$compatible?', function(recvr, meth, arglist) {
          var $a, self = this;

          return (($a = recvr['$=='](["const", "Opal"])) ? $scope.get('HELPERS')['$include?'](meth.$to_sym()) : $a);
        });

        Opal.defs(self, '$helper', TMP_1 = function(name) {
          var $a, $b, self = this, $iter = TMP_1.$$p, block = $iter || nil;

          TMP_1.$$p = null;
          $scope.get('HELPERS')['$<<'](name);
          return ($a = ($b = self).$define_method, $a.$$p = block.$to_proc(), $a).call($b, "compile_" + (name));
        });

        def.$compile = function() {
          var $a, self = this;

          if ((($a = $scope.get('HELPERS')['$include?'](self.$meth().$to_sym())) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$__send__("compile_" + (self.$meth()))
            } else {
            return self.$raise("Helper not supported: " + (self.$meth()))
          };
        };

        ($a = ($b = self).$helper, $a.$$p = (TMP_2 = function(){var self = TMP_2.$$s || this, $a, sexp = nil;

        if ((($a = sexp = self.$arglist()['$[]'](1)) !== nil && (!$a.$$is_boolean || $a == true))) {
            } else {
            self.$raise("truthy? requires an object")
          };
          return self.$js_truthy(sexp);}, TMP_2.$$s = self, TMP_2), $a).call($b, "truthy?");

        return ($a = ($c = self).$helper, $a.$$p = (TMP_3 = function(){var self = TMP_3.$$s || this, $a, sexp = nil;

        if ((($a = sexp = self.$arglist()['$[]'](1)) !== nil && (!$a.$$is_boolean || $a == true))) {
            } else {
            self.$raise("falsy? requires an object")
          };
          return self.$js_falsy(sexp);}, TMP_3.$$s = self, TMP_3), $a).call($c, "falsy?");
      })(self, $scope.get('Base'))
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/call"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass, $hash2 = Opal.hash2, $range = Opal.range;

  Opal.add_stubs(['$require', '$handle', '$children', '$[]=', '$define_method', '$to_proc', '$handle_special', '$compile_default?', '$<<', '$method_calls', '$compiler', '$to_sym', '$meth', '$using_irb?', '$compile_irb_var', '$default_compile', '$mid_to_jsid', '$to_s', '$any?', '$==', '$first', '$[]', '$arglist', '$===', '$last', '$type', '$pop', '$iter', '$new_temp', '$scope', '$expr', '$recv', '$recv_sexp', '$s', '$!', '$insert', '$push', '$unshift', '$queue_temp', '$recvr', '$=~', '$with_temp', '$variable', '$intern', '$irb?', '$top?', '$nil?', '$include?', '$__send__', '$compatible?', '$compile', '$new', '$each', '$add_special', '$inline_operators?', '$operator_helpers', '$fragment', '$compile_default!', '$resolve', '$requires', '$file', '$dirname', '$cleanpath', '$join', '$Pathname', '$inspect', '$process', '$class_scope?', '$required_trees', '$handle_block_given_call', '$def?', '$mid', '$handle_part', '$map', '$expand_path', '$split', '$dynamic_require_severity', '$error', '$line', '$warning', '$inject']);
  self.$require("set");
  self.$require("pathname");
  self.$require("opal/nodes/base");
  self.$require("opal/nodes/runtime_helpers");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $CallNode(){};
        var self = $CallNode = $klass($base, $super, 'CallNode', $CallNode);

        var def = self.$$proto, $scope = self.$$scope, TMP_1, $a, $b, TMP_6, $c, TMP_8, $d, TMP_9, $e, TMP_10, $f, TMP_11, $g, TMP_12, $h, TMP_13, $i, TMP_14, $j, TMP_15;

        def.assignment = def.compiler = def.sexp = def.level = def.compile_default = nil;
        self.$handle("call");

        self.$children("recvr", "meth", "arglist", "iter");

        Opal.cdecl($scope, 'SPECIALS', $hash2([], {}));

        Opal.cdecl($scope, 'OPERATORS', $hash2(["+", "-", "*", "/", "<", "<=", ">", ">="], {"+": "plus", "-": "minus", "*": "times", "/": "divide", "<": "lt", "<=": "le", ">": "gt", ">=": "ge"}));

        Opal.defs(self, '$add_special', TMP_1 = function(name, options) {
          var $a, $b, self = this, $iter = TMP_1.$$p, handler = $iter || nil;

          if (options == null) {
            options = $hash2([], {})
          }
          TMP_1.$$p = null;
          $scope.get('SPECIALS')['$[]='](name, options);
          return ($a = ($b = self).$define_method, $a.$$p = handler.$to_proc(), $a).call($b, "handle_" + (name));
        });

        def.$compile = function() {
          var $a, self = this;

          self.$handle_special();
          if ((($a = self['$compile_default?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            } else {
            return nil
          };
          self.$compiler().$method_calls()['$<<'](self.$meth().$to_sym());
          if ((($a = self['$using_irb?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$compile_irb_var()};
          return self.$default_compile();
        };

        def.$default_compile = function() {
          var $a, $b, TMP_2, $c, self = this, mid = nil, splat = nil, block = nil, blktmp = nil, tmprecv = nil, recv_code = nil, call_recv = nil, args = nil;

          mid = self.$mid_to_jsid(self.$meth().$to_s());
          splat = ($a = ($b = self.$arglist()['$[]']($range(1, -1, false)))['$any?'], $a.$$p = (TMP_2 = function(a){var self = TMP_2.$$s || this;
if (a == null) a = nil;
          return a.$first()['$==']("splat")}, TMP_2.$$s = self, TMP_2), $a).call($b);
          if ((($a = ($c = $scope.get('Sexp')['$==='](self.$arglist().$last()), $c !== false && $c !== nil ?self.$arglist().$last().$type()['$==']("block_pass") : $c)) !== nil && (!$a.$$is_boolean || $a == true))) {
            block = self.$arglist().$pop()
          } else if ((($a = self.$iter()) !== nil && (!$a.$$is_boolean || $a == true))) {
            block = self.$iter()};
          if (block !== false && block !== nil) {
            blktmp = self.$scope().$new_temp()};
          if ((($a = ((($c = splat) !== false && $c !== nil) ? $c : blktmp)) !== nil && (!$a.$$is_boolean || $a == true))) {
            tmprecv = self.$scope().$new_temp()};
          if (block !== false && block !== nil) {
            block = self.$expr(block)};
          recv_code = self.$recv(self.$recv_sexp());
          call_recv = self.$s("js_tmp", ((($a = tmprecv) !== false && $a !== nil) ? $a : recv_code));
          if ((($a = (($c = blktmp !== false && blktmp !== nil) ? splat['$!']() : $c)) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.$arglist().$insert(1, call_recv)};
          args = self.$expr(self.$arglist());
          if (tmprecv !== false && tmprecv !== nil) {
            self.$push("(" + (tmprecv) + " = ", recv_code, ")" + (mid))
            } else {
            self.$push(recv_code, mid)
          };
          if (blktmp !== false && blktmp !== nil) {
            self.$unshift("(" + (blktmp) + " = ");
            self.$push(", " + (blktmp) + ".$$p = ", block, ", " + (blktmp) + ")");};
          if (splat !== false && splat !== nil) {
            self.$push(".apply(", (((($a = tmprecv) !== false && $a !== nil) ? $a : recv_code)), ", ", args, ")")
          } else if (blktmp !== false && blktmp !== nil) {
            self.$push(".call(", args, ")")
            } else {
            self.$push("(", args, ")")
          };
          if (blktmp !== false && blktmp !== nil) {
            return self.$scope().$queue_temp(blktmp)
            } else {
            return nil
          };
        };

        def.$recv_sexp = function() {
          var $a, self = this;

          return ((($a = self.$recvr()) !== false && $a !== nil) ? $a : self.$s("self"));
        };

        def['$attr_assignment?'] = function() {
          var $a, self = this;

          return ((($a = self.assignment) !== false && $a !== nil) ? $a : self.assignment = self.$meth().$to_s()['$=~']((new RegExp("" + $scope.get('REGEXP_START') + "[\\da-z]+\\=" + $scope.get('REGEXP_END')))));
        };

        def.$compile_irb_var = function() {
          var $a, $b, TMP_3, self = this;

          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_3 = function(tmp){var self = TMP_3.$$s || this, lvar = nil, call = nil;
if (tmp == null) tmp = nil;
          lvar = self.$variable(self.$meth());
            call = self.$s("call", self.$s("self"), self.$meth().$intern(), self.$s("arglist"));
            return self.$push("((" + (tmp) + " = Opal.irb_vars." + (lvar) + ") == null ? ", self.$expr(call), " : " + (tmp) + ")");}, TMP_3.$$s = self, TMP_3), $a).call($b);
        };

        def.$compile_assignment = function() {
          var $a, $b, TMP_4, self = this;

          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_4 = function(args_tmp){var self = TMP_4.$$s || this, $a, $b, TMP_5;
if (args_tmp == null) args_tmp = nil;
          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_5 = function(recv_tmp){var self = TMP_5.$$s || this, args = nil, mid = nil;
if (recv_tmp == null) recv_tmp = nil;
            args = self.$expr(self.$arglist());
              mid = self.$mid_to_jsid(self.$meth().$to_s());
              return self.$push("((" + (args_tmp) + " = [", args, $rb_plus("]), ", "" + (recv_tmp) + " = "), self.$recv(self.$recv_sexp()), ", ", recv_tmp, mid, $rb_plus(".apply(" + (recv_tmp) + ", " + (args_tmp) + "), ", "" + (args_tmp) + "[" + (args_tmp) + ".length-1])"));}, TMP_5.$$s = self, TMP_5), $a).call($b)}, TMP_4.$$s = self, TMP_4), $a).call($b);
        };

        def['$using_irb?'] = function() {
          var $a, $b, $c, $d, self = this;

          return ($a = ($b = ($c = ($d = self.compiler['$irb?'](), $d !== false && $d !== nil ?self.$scope()['$top?']() : $d), $c !== false && $c !== nil ?self.$arglist()['$=='](self.$s("arglist")) : $c), $b !== false && $b !== nil ?self.$recvr()['$nil?']() : $b), $a !== false && $a !== nil ?self.$iter()['$nil?']() : $a);
        };

        def.$handle_special = function() {
          var $a, self = this;

          self.compile_default = true;
          if ((($a = $scope.get('SPECIALS')['$include?'](self.$meth())) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.compile_default = false;
            return self.$__send__("handle_" + (self.$meth()));
          } else if ((($a = $scope.get('RuntimeHelpers')['$compatible?'](self.$recvr(), self.$meth(), self.$arglist())) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.compile_default = false;
            return self.$push($scope.get('RuntimeHelpers').$new(self.sexp, self.level, self.compiler).$compile());
            } else {
            return nil
          };
        };

        def['$compile_default!'] = function() {
          var self = this;

          return self.compile_default = true;
        };

        def['$compile_default?'] = function() {
          var self = this;

          return self.compile_default;
        };

        ($a = ($b = $scope.get('OPERATORS')).$each, $a.$$p = (TMP_6 = function(operator, name){var self = TMP_6.$$s || this, $a, $b, TMP_7;
if (operator == null) operator = nil;if (name == null) name = nil;
        return ($a = ($b = self).$add_special, $a.$$p = (TMP_7 = function(){var self = TMP_7.$$s || this, $a, lhs = nil, rhs = nil;

          if ((($a = self.$compiler()['$inline_operators?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
              self.$compiler().$operator_helpers()['$<<'](operator.$to_sym());
              $a = [self.$expr(self.$recvr()), self.$expr(self.$arglist()['$[]'](1))], lhs = $a[0], rhs = $a[1];
              self.$push(self.$fragment("$rb_" + (name) + "("));
              self.$push(lhs);
              self.$push(self.$fragment(", "));
              self.$push(rhs);
              return self.$push(self.$fragment(")"));
              } else {
              return self['$compile_default!']()
            }}, TMP_7.$$s = self, TMP_7), $a).call($b, operator.$to_sym())}, TMP_6.$$s = self, TMP_6), $a).call($b);

        ($a = ($c = self).$add_special, $a.$$p = (TMP_8 = function(){var self = TMP_8.$$s || this, $a, str = nil;

        self['$compile_default!']();
          str = $scope.get('DependencyResolver').$new(self.$compiler(), self.$arglist()['$[]'](1)).$resolve();
          if ((($a = str['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            } else {
            self.$compiler().$requires()['$<<'](str)
          };
          return self.$push(self.$fragment(""));}, TMP_8.$$s = self, TMP_8), $a).call($c, "require");

        ($a = ($d = self).$add_special, $a.$$p = (TMP_9 = function(){var self = TMP_9.$$s || this, arg = nil, file = nil, dir = nil;

        arg = self.$arglist()['$[]'](1);
          file = self.$compiler().$file();
          if (arg['$[]'](0)['$==']("str")) {
            dir = $scope.get('File').$dirname(file);
            self.$compiler().$requires()['$<<'](self.$Pathname(dir).$join(arg['$[]'](1)).$cleanpath().$to_s());};
          self.$push(self.$fragment("self.$require(" + (file.$inspect()) + "+ '/../' + "));
          self.$push(self.$process(self.$arglist()));
          return self.$push(self.$fragment(")"));}, TMP_9.$$s = self, TMP_9), $a).call($d, "require_relative");

        ($a = ($e = self).$add_special, $a.$$p = (TMP_10 = function(){var self = TMP_10.$$s || this, $a, str = nil;

        if ((($a = self.$scope()['$class_scope?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            self['$compile_default!']();
            str = $scope.get('DependencyResolver').$new(self.$compiler(), self.$arglist()['$[]'](2)).$resolve();
            if ((($a = str['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
              } else {
              self.$compiler().$requires()['$<<'](str)
            };
            return self.$push(self.$fragment(""));
            } else {
            return nil
          }}, TMP_10.$$s = self, TMP_10), $a).call($e, "autoload");

        ($a = ($f = self).$add_special, $a.$$p = (TMP_11 = function(){var self = TMP_11.$$s || this, arg = nil, relative_path = nil, dir = nil, full_path = nil;

        arg = self.$arglist()['$[]'](1);
          if (arg['$[]'](0)['$==']("str")) {
            relative_path = arg['$[]'](1);
            self.$compiler().$required_trees()['$<<'](relative_path);
            dir = $scope.get('File').$dirname(self.$compiler().$file());
            full_path = self.$Pathname(dir).$join(relative_path).$cleanpath().$to_s();
            arg['$[]='](1, full_path);};
          self['$compile_default!']();
          return self.$push(self.$fragment(""));}, TMP_11.$$s = self, TMP_11), $a).call($f, "require_tree");

        ($a = ($g = self).$add_special, $a.$$p = (TMP_12 = function(){var self = TMP_12.$$s || this;
          if (self.sexp == null) self.sexp = nil;

        return self.$push(self.$compiler().$handle_block_given_call(self.sexp))}, TMP_12.$$s = self, TMP_12), $a).call($g, "block_given?");

        ($a = ($h = self).$add_special, $a.$$p = (TMP_13 = function(){var self = TMP_13.$$s || this, $a;

        if ((($a = self.$scope()['$def?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$push(self.$fragment(self.$scope().$mid().$to_s().$inspect()))
            } else {
            return self.$push(self.$fragment("nil"))
          }}, TMP_13.$$s = self, TMP_13), $a).call($h, "__callee__");

        ($a = ($i = self).$add_special, $a.$$p = (TMP_14 = function(){var self = TMP_14.$$s || this, $a;

        if ((($a = self.$scope()['$def?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$push(self.$fragment(self.$scope().$mid().$to_s().$inspect()))
            } else {
            return self.$push(self.$fragment("nil"))
          }}, TMP_14.$$s = self, TMP_14), $a).call($i, "__method__");

        ($a = ($j = self).$add_special, $a.$$p = (TMP_15 = function(){var self = TMP_15.$$s || this;

        return self.$push(self.$fragment("debugger"))}, TMP_15.$$s = self, TMP_15), $a).call($j, "debugger");

        return (function($base, $super) {
          function $DependencyResolver(){};
          var self = $DependencyResolver = $klass($base, $super, 'DependencyResolver', $DependencyResolver);

          var def = self.$$proto, $scope = self.$$scope;

          def.sexp = def.compiler = nil;
          def.$initialize = function(compiler, sexp) {
            var self = this;

            self.compiler = compiler;
            return self.sexp = sexp;
          };

          def.$resolve = function() {
            var self = this;

            return self.$handle_part(self.sexp);
          };

          def.$handle_part = function(sexp) {
            var $a, $b, TMP_16, self = this, type = nil, _ = nil, recv = nil, meth = nil, args = nil, parts = nil, msg = nil, $case = nil;

            type = sexp.$type();
            if (type['$==']("str")) {
              return sexp['$[]'](1)
            } else if (type['$==']("call")) {
              $a = Opal.to_ary(sexp), _ = ($a[0] == null ? nil : $a[0]), recv = ($a[1] == null ? nil : $a[1]), meth = ($a[2] == null ? nil : $a[2]), args = ($a[3] == null ? nil : $a[3]);
              parts = ($a = ($b = args['$[]']($range(1, -1, false))).$map, $a.$$p = (TMP_16 = function(s){var self = TMP_16.$$s || this;
if (s == null) s = nil;
              return self.$handle_part(s)}, TMP_16.$$s = self, TMP_16), $a).call($b);
              if (recv['$=='](["const", "File"])) {
                if (meth['$==']("expand_path")) {
                  return ($a = self).$expand_path.apply($a, [].concat(parts))
                } else if (meth['$==']("join")) {
                  return self.$expand_path(parts.$join("/"))
                } else if (meth['$==']("dirname")) {
                  return self.$expand_path(parts['$[]'](0).$split("/")['$[]']($range(0, -1, true)).$join("/"))}};};
            msg = "Cannot handle dynamic require";
            return (function() {$case = self.compiler.$dynamic_require_severity();if ("error"['$===']($case)) {return self.compiler.$error(msg, self.sexp.$line())}else if ("warning"['$===']($case)) {return self.compiler.$warning(msg, self.sexp.$line())}else { return nil }})();
          };

          return (def.$expand_path = function(path, base) {
            var $a, $b, TMP_17, self = this;

            if (base == null) {
              base = ""
            }
            return ($a = ($b = (((("") + (base)) + "/") + (path)).$split("/")).$inject, $a.$$p = (TMP_17 = function(p, part){var self = TMP_17.$$s || this;
if (p == null) p = nil;if (part == null) part = nil;
            if (part['$==']("")) {
              } else if (part['$==']("..")) {
                p.$pop()
                } else {
                p['$<<'](part)
              };
              return p;}, TMP_17.$$s = self, TMP_17), $a).call($b, []).$join("/");
          }, nil) && 'expand_path';
        })(self, null);
      })(self, $scope.get('Base'))
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/call_special"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass, $range = Opal.range;

  Opal.add_stubs(['$require', '$handle', '$children', '$=~', '$to_s', '$meth', '$with_temp', '$expr', '$arglist', '$mid_to_jsid', '$push', '$recv', '$recv_sexp', '$s', '$lhs', '$rhs', '$process', '$recvr', '$[]', '$args', '$op', '$===', '$compile_or', '$compile_and', '$compile_operator', '$to_sym', '$first_arg', '$mid']);
  self.$require("opal/nodes/base");
  self.$require("opal/nodes/call");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $AttrAssignNode(){};
        var self = $AttrAssignNode = $klass($base, $super, 'AttrAssignNode', $AttrAssignNode);

        var def = self.$$proto, $scope = self.$$scope, TMP_1;

        self.$handle("attrasgn");

        self.$children("recvr", "meth", "arglist");

        return (def.$default_compile = TMP_1 = function() {var $zuper = $slice.call(arguments, 0);
          var $a, $b, TMP_2, self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

          TMP_1.$$p = null;
          if ((($a = ($b = self.$meth().$to_s()['$=~']((new RegExp("" + $scope.get('REGEXP_START') + "\\w+=" + $scope.get('REGEXP_END')))), ($b === nil || $b === false))) !== nil && (!$a.$$is_boolean || $a == true))) {
            return Opal.find_super_dispatcher(self, 'default_compile', TMP_1, $iter).apply(self, $zuper)};
          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_2 = function(args_tmp){var self = TMP_2.$$s || this, $a, $b, TMP_3;
if (args_tmp == null) args_tmp = nil;
          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_3 = function(recv_tmp){var self = TMP_3.$$s || this, args = nil, mid = nil;
if (recv_tmp == null) recv_tmp = nil;
            args = self.$expr(self.$arglist());
              mid = self.$mid_to_jsid(self.$meth().$to_s());
              return self.$push("((" + (args_tmp) + " = [", args, $rb_plus("]), ", "" + (recv_tmp) + " = "), self.$recv(self.$recv_sexp()), ", ", recv_tmp, mid, $rb_plus(".apply(" + (recv_tmp) + ", " + (args_tmp) + "), ", "" + (args_tmp) + "[" + (args_tmp) + ".length-1])"));}, TMP_3.$$s = self, TMP_3), $a).call($b)}, TMP_2.$$s = self, TMP_2), $a).call($b);
        }, nil) && 'default_compile';
      })(self, $scope.get('CallNode'));

      (function($base, $super) {
        function $Match3Node(){};
        var self = $Match3Node = $klass($base, $super, 'Match3Node', $Match3Node);

        var def = self.$$proto, $scope = self.$$scope;

        def.level = nil;
        self.$handle("match3");

        self.$children("lhs", "rhs");

        return (def.$compile = function() {
          var self = this, sexp = nil;

          sexp = self.$s("call", self.$lhs(), "=~", self.$s("arglist", self.$rhs()));
          return self.$push(self.$process(sexp, self.level));
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $OpAsgnOrNode(){};
        var self = $OpAsgnOrNode = $klass($base, $super, 'OpAsgnOrNode', $OpAsgnOrNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("op_asgn_or");

        self.$children("recvr", "rhs");

        return (def.$compile = function() {
          var self = this, sexp = nil;

          sexp = self.$s("or", self.$recvr(), self.$rhs());
          return self.$push(self.$expr(sexp));
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $OpAsgnAndNode(){};
        var self = $OpAsgnAndNode = $klass($base, $super, 'OpAsgnAndNode', $OpAsgnAndNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("op_asgn_and");

        self.$children("recvr", "rhs");

        return (def.$compile = function() {
          var self = this, sexp = nil;

          sexp = self.$s("and", self.$recvr(), self.$rhs());
          return self.$push(self.$expr(sexp));
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $OpAsgn1Node(){};
        var self = $OpAsgn1Node = $klass($base, $super, 'OpAsgn1Node', $OpAsgn1Node);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("op_asgn1");

        self.$children("lhs", "args", "op", "rhs");

        def.$first_arg = function() {
          var self = this;

          return self.$args()['$[]'](1);
        };

        def.$compile = function() {
          var self = this, $case = nil;

          return (function() {$case = self.$op().$to_s();if ("||"['$===']($case)) {return self.$compile_or()}else if ("&&"['$===']($case)) {return self.$compile_and()}else {return self.$compile_operator()}})();
        };

        def.$compile_operator = function() {
          var $a, $b, TMP_4, self = this;

          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_4 = function(a){var self = TMP_4.$$s || this, $a, $b, TMP_5;
if (a == null) a = nil;
          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_5 = function(r){var self = TMP_5.$$s || this, cur = nil, rhs = nil, call = nil;
if (r == null) r = nil;
            cur = self.$s("call", self.$s("js_tmp", r), "[]", self.$s("arglist", self.$s("js_tmp", a)));
              rhs = self.$s("call", cur, self.$op().$to_sym(), self.$s("arglist", self.$rhs()));
              call = self.$s("call", self.$s("js_tmp", r), "[]=", self.$s("arglist", self.$s("js_tmp", a), rhs));
              self.$push("(" + (a) + " = ", self.$expr(self.$first_arg()), ", " + (r) + " = ", self.$expr(self.$lhs()));
              return self.$push(", ", self.$expr(call), ")");}, TMP_5.$$s = self, TMP_5), $a).call($b)}, TMP_4.$$s = self, TMP_4), $a).call($b);
        };

        def.$compile_or = function() {
          var $a, $b, TMP_6, self = this;

          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_6 = function(a){var self = TMP_6.$$s || this, $a, $b, TMP_7;
if (a == null) a = nil;
          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_7 = function(r){var self = TMP_7.$$s || this, aref = nil, aset = nil, orop = nil;
if (r == null) r = nil;
            aref = self.$s("call", self.$s("js_tmp", r), "[]", self.$s("arglist", self.$s("js_tmp", a)));
              aset = self.$s("call", self.$s("js_tmp", r), "[]=", self.$s("arglist", self.$s("js_tmp", a), self.$rhs()));
              orop = self.$s("or", aref, aset);
              self.$push("(" + (a) + " = ", self.$expr(self.$first_arg()), ", " + (r) + " = ", self.$expr(self.$lhs()));
              return self.$push(", ", self.$expr(orop), ")");}, TMP_7.$$s = self, TMP_7), $a).call($b)}, TMP_6.$$s = self, TMP_6), $a).call($b);
        };

        return (def.$compile_and = function() {
          var $a, $b, TMP_8, self = this;

          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_8 = function(a){var self = TMP_8.$$s || this, $a, $b, TMP_9;
if (a == null) a = nil;
          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_9 = function(r){var self = TMP_9.$$s || this, aref = nil, aset = nil, andop = nil;
if (r == null) r = nil;
            aref = self.$s("call", self.$s("js_tmp", r), "[]", self.$s("arglist", self.$s("js_tmp", a)));
              aset = self.$s("call", self.$s("js_tmp", r), "[]=", self.$s("arglist", self.$s("js_tmp", a), self.$rhs()));
              andop = self.$s("and", aref, aset);
              self.$push("(" + (a) + " = ", self.$expr(self.$first_arg()), ", " + (r) + " = ", self.$expr(self.$lhs()));
              return self.$push(", ", self.$expr(andop), ")");}, TMP_9.$$s = self, TMP_9), $a).call($b)}, TMP_8.$$s = self, TMP_8), $a).call($b);
        }, nil) && 'compile_and';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $OpAsgn2Node(){};
        var self = $OpAsgn2Node = $klass($base, $super, 'OpAsgn2Node', $OpAsgn2Node);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("op_asgn2");

        self.$children("lhs", "mid", "op", "rhs");

        def.$meth = function() {
          var self = this;

          return self.$mid().$to_s()['$[]']($range(0, -2, false));
        };

        def.$compile = function() {
          var self = this, $case = nil;

          return (function() {$case = self.$op().$to_s();if ("||"['$===']($case)) {return self.$compile_or()}else if ("&&"['$===']($case)) {return self.$compile_and()}else {return self.$compile_operator()}})();
        };

        def.$compile_or = function() {
          var $a, $b, TMP_10, self = this;

          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_10 = function(tmp){var self = TMP_10.$$s || this, getr = nil, asgn = nil, orop = nil;
if (tmp == null) tmp = nil;
          getr = self.$s("call", self.$s("js_tmp", tmp), self.$meth(), self.$s("arglist"));
            asgn = self.$s("call", self.$s("js_tmp", tmp), self.$mid(), self.$s("arglist", self.$rhs()));
            orop = self.$s("or", getr, asgn);
            return self.$push("(" + (tmp) + " = ", self.$expr(self.$lhs()), ", ", self.$expr(orop), ")");}, TMP_10.$$s = self, TMP_10), $a).call($b);
        };

        def.$compile_and = function() {
          var $a, $b, TMP_11, self = this;

          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_11 = function(tmp){var self = TMP_11.$$s || this, getr = nil, asgn = nil, andop = nil;
if (tmp == null) tmp = nil;
          getr = self.$s("call", self.$s("js_tmp", tmp), self.$meth(), self.$s("arglist"));
            asgn = self.$s("call", self.$s("js_tmp", tmp), self.$mid(), self.$s("arglist", self.$rhs()));
            andop = self.$s("and", getr, asgn);
            return self.$push("(" + (tmp) + " = ", self.$expr(self.$lhs()), ", ", self.$expr(andop), ")");}, TMP_11.$$s = self, TMP_11), $a).call($b);
        };

        return (def.$compile_operator = function() {
          var $a, $b, TMP_12, self = this;

          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_12 = function(tmp){var self = TMP_12.$$s || this, getr = nil, oper = nil, asgn = nil;
if (tmp == null) tmp = nil;
          getr = self.$s("call", self.$s("js_tmp", tmp), self.$meth(), self.$s("arglist"));
            oper = self.$s("call", getr, self.$op(), self.$s("arglist", self.$rhs()));
            asgn = self.$s("call", self.$s("js_tmp", tmp), self.$mid(), self.$s("arglist", oper));
            return self.$push("(" + (tmp) + " = ", self.$expr(self.$lhs()), ", ", self.$expr(asgn), ")");}, TMP_12.$$s = self, TMP_12), $a).call($b);
        }, nil) && 'compile_operator';
      })(self, $scope.get('Base'));
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/scope"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$attr_accessor', '$attr_reader', '$indent', '$scope', '$compiler', '$scope=', '$call', '$==', '$!', '$class?', '$dup', '$push', '$map', '$ivars', '$gvars', '$parser_indent', '$empty?', '$join', '$proto', '$%', '$fragment', '$def_in_class?', '$add_proto_ivar', '$include?', '$<<', '$has_local?', '$pop', '$next_temp', '$succ', '$uses_block!', '$identify!', '$unique_temp', '$add_scope_temp', '$parent', '$def?', '$type', '$mid']);
  self.$require("opal/nodes/base");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $ScopeNode(){};
        var self = $ScopeNode = $klass($base, $super, 'ScopeNode', $ScopeNode);

        var def = self.$$proto, $scope = self.$$scope, TMP_1, TMP_2;

        def.type = def.defs = def.parent = def.temps = def.locals = def.compiler = def.proto_ivars = def.ivars = def.gvars = def.args = def.queue = def.unique = def.while_stack = def.identity = def.uses_block = nil;
        self.$attr_accessor("parent");

        self.$attr_accessor("name");

        self.$attr_accessor("block_name");

        self.$attr_reader("scope_name");

        self.$attr_reader("ivars");

        self.$attr_reader("gvars");

        self.$attr_accessor("mid");

        self.$attr_accessor("defs");

        self.$attr_reader("methods");

        self.$attr_accessor("uses_super");

        self.$attr_accessor("uses_zuper");

        self.$attr_accessor("catch_return");

        def.$initialize = TMP_1 = function() {var $zuper = $slice.call(arguments, 0);
          var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

          TMP_1.$$p = null;
          Opal.find_super_dispatcher(self, 'initialize', TMP_1, $iter).apply(self, $zuper);
          self.locals = [];
          self.temps = [];
          self.args = [];
          self.ivars = [];
          self.gvars = [];
          self.parent = nil;
          self.queue = [];
          self.unique = "a";
          self.while_stack = [];
          self.identity = nil;
          self.defs = nil;
          self.methods = [];
          self.uses_block = false;
          return self.proto_ivars = [];
        };

        def.$in_scope = TMP_2 = function() {
          var $a, $b, TMP_3, self = this, $iter = TMP_2.$$p, block = $iter || nil;

          TMP_2.$$p = null;
          return ($a = ($b = self).$indent, $a.$$p = (TMP_3 = function(){var self = TMP_3.$$s || this, $a, $b;
            if (self.parent == null) self.parent = nil;

          self.parent = self.$compiler().$scope();
            (($a = [self]), $b = self.$compiler(), $b['$scope='].apply($b, $a), $a[$a.length-1]);
            block.$call(self);
            return (($a = [self.parent]), $b = self.$compiler(), $b['$scope='].apply($b, $a), $a[$a.length-1]);}, TMP_3.$$s = self, TMP_3), $a).call($b);
        };

        def['$class_scope?'] = function() {
          var $a, self = this;

          return ((($a = self.type['$==']("class")) !== false && $a !== nil) ? $a : self.type['$==']("module"));
        };

        def['$class?'] = function() {
          var self = this;

          return self.type['$==']("class");
        };

        def['$module?'] = function() {
          var self = this;

          return self.type['$==']("module");
        };

        def['$sclass?'] = function() {
          var self = this;

          return self.type['$==']("sclass");
        };

        def['$top?'] = function() {
          var self = this;

          return self.type['$==']("top");
        };

        def['$iter?'] = function() {
          var self = this;

          return self.type['$==']("iter");
        };

        def['$def?'] = function() {
          var self = this;

          return self.type['$==']("def");
        };

        def['$def_in_class?'] = function() {
          var $a, $b, $c, self = this;

          return ($a = ($b = ($c = self.defs['$!'](), $c !== false && $c !== nil ?self.type['$==']("def") : $c), $b !== false && $b !== nil ?self.parent : $b), $a !== false && $a !== nil ?self.parent['$class?']() : $a);
        };

        def.$proto = function() {
          var self = this;

          return "def";
        };

        def.$to_vars = function() {
          var $a, $b, $c, TMP_4, $d, TMP_5, $e, TMP_6, $f, TMP_7, self = this, vars = nil, iv = nil, gv = nil, indent = nil, str = nil, pvars = nil, result = nil;

          vars = self.temps.$dup();
          ($a = vars).$push.apply($a, [].concat(($b = ($c = self.locals).$map, $b.$$p = (TMP_4 = function(l){var self = TMP_4.$$s || this;
if (l == null) l = nil;
          return "" + (l) + " = nil"}, TMP_4.$$s = self, TMP_4), $b).call($c)));
          iv = ($b = ($d = self.$ivars()).$map, $b.$$p = (TMP_5 = function(ivar){var self = TMP_5.$$s || this;
if (ivar == null) ivar = nil;
          return "if (self" + (ivar) + " == null) self" + (ivar) + " = nil;\n"}, TMP_5.$$s = self, TMP_5), $b).call($d);
          gv = ($b = ($e = self.$gvars()).$map, $b.$$p = (TMP_6 = function(gvar){var self = TMP_6.$$s || this;
if (gvar == null) gvar = nil;
          return "if ($gvars" + (gvar) + " == null) $gvars" + (gvar) + " = nil;\n"}, TMP_6.$$s = self, TMP_6), $b).call($e);
          indent = self.compiler.$parser_indent();
          str = (function() {if ((($b = vars['$empty?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
            return ""
            } else {
            return "var " + (vars.$join(", ")) + ";\n"
          }; return nil; })();
          if ((($b = self.$ivars()['$empty?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
            } else {
            str = $rb_plus(str, "" + (indent) + (iv.$join(indent)))
          };
          if ((($b = self.$gvars()['$empty?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
            } else {
            str = $rb_plus(str, "" + (indent) + (gv.$join(indent)))
          };
          if ((($b = ($f = self['$class?'](), $f !== false && $f !== nil ?self.proto_ivars['$empty?']()['$!']() : $f)) !== nil && (!$b.$$is_boolean || $b == true))) {
            pvars = ($b = ($f = self.proto_ivars).$map, $b.$$p = (TMP_7 = function(i){var self = TMP_7.$$s || this;
if (i == null) i = nil;
            return "" + (self.$proto()) + (i)}, TMP_7.$$s = self, TMP_7), $b).call($f).$join(" = ");
            result = "%s\n%s%s = nil;"['$%']([str, indent, pvars]);
            } else {
            result = str
          };
          return self.$fragment(result);
        };

        def.$add_scope_ivar = function(ivar) {
          var $a, self = this;

          if ((($a = self['$def_in_class?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.parent.$add_proto_ivar(ivar)
          } else if ((($a = self.ivars['$include?'](ivar)) !== nil && (!$a.$$is_boolean || $a == true))) {
            return nil
            } else {
            return self.ivars['$<<'](ivar)
          };
        };

        def.$add_scope_gvar = function(gvar) {
          var $a, self = this;

          if ((($a = self.gvars['$include?'](gvar)) !== nil && (!$a.$$is_boolean || $a == true))) {
            return nil
            } else {
            return self.gvars['$<<'](gvar)
          };
        };

        def.$add_proto_ivar = function(ivar) {
          var $a, self = this;

          if ((($a = self.proto_ivars['$include?'](ivar)) !== nil && (!$a.$$is_boolean || $a == true))) {
            return nil
            } else {
            return self.proto_ivars['$<<'](ivar)
          };
        };

        def.$add_arg = function(arg) {
          var $a, self = this;

          if ((($a = self.args['$include?'](arg)) !== nil && (!$a.$$is_boolean || $a == true))) {
            } else {
            self.args['$<<'](arg)
          };
          return arg;
        };

        def.$add_scope_local = function(local) {
          var $a, self = this;

          if ((($a = self['$has_local?'](local)) !== nil && (!$a.$$is_boolean || $a == true))) {
            return nil};
          return self.locals['$<<'](local);
        };

        def['$has_local?'] = function(local) {
          var $a, $b, self = this;

          if ((($a = ((($b = self.locals['$include?'](local)) !== false && $b !== nil) ? $b : self.args['$include?'](local))) !== nil && (!$a.$$is_boolean || $a == true))) {
            return true};
          if ((($a = ($b = self.parent, $b !== false && $b !== nil ?self.type['$==']("iter") : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.parent['$has_local?'](local)};
          return false;
        };

        def.$add_scope_temp = function(tmps) {
          var $a, self = this;

          tmps = $slice.call(arguments, 0);
          return ($a = self.temps).$push.apply($a, [].concat(tmps));
        };

        def['$has_temp?'] = function(tmp) {
          var self = this;

          return self.temps['$include?'](tmp);
        };

        def.$new_temp = function() {
          var $a, self = this, tmp = nil;

          if ((($a = self.queue['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            } else {
            return self.queue.$pop()
          };
          tmp = self.$next_temp();
          self.temps['$<<'](tmp);
          return tmp;
        };

        def.$next_temp = function() {
          var self = this, tmp = nil;

          tmp = "$" + (self.unique);
          self.unique = self.unique.$succ();
          return tmp;
        };

        def.$queue_temp = function(name) {
          var self = this;

          return self.queue['$<<'](name);
        };

        def.$push_while = function() {
          var self = this, info = nil;

          info = $hash2([], {});
          self.while_stack.$push(info);
          return info;
        };

        def.$pop_while = function() {
          var self = this;

          return self.while_stack.$pop();
        };

        def['$in_while?'] = function() {
          var self = this;

          return self.while_stack['$empty?']()['$!']();
        };

        def['$uses_block!'] = function() {
          var $a, $b, self = this;

          if ((($a = (($b = self.type['$==']("iter")) ? self.parent : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.parent['$uses_block!']()
            } else {
            self.uses_block = true;
            return self['$identify!']();
          };
        };

        def['$identify!'] = function() {
          var $a, self = this;

          if ((($a = self.identity) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.identity};
          self.identity = self.compiler.$unique_temp();
          if ((($a = self.parent) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.parent.$add_scope_temp(self.identity)};
          return self.identity;
        };

        def.$identity = function() {
          var self = this;

          return self.identity;
        };

        def.$find_parent_def = function() {
          var $a, $b, self = this, scope = nil;

          scope = self;
          while ((($b = scope = scope.$parent()) !== nil && (!$b.$$is_boolean || $b == true))) {
          if ((($b = scope['$def?']()) !== nil && (!$b.$$is_boolean || $b == true))) {
            return scope}};
          return nil;
        };

        def.$get_super_chain = function() {
          var $a, $b, self = this, chain = nil, scope = nil, defn = nil, mid = nil;

          $a = [[], self, "null", "null"], chain = $a[0], scope = $a[1], defn = $a[2], mid = $a[3];
          while (scope !== false && scope !== nil) {
          if (scope.$type()['$==']("iter")) {
            chain['$<<'](scope['$identify!']());
            if ((($b = scope.$parent()) !== nil && (!$b.$$is_boolean || $b == true))) {
              scope = scope.$parent()};
          } else if (scope.$type()['$==']("def")) {
            defn = scope['$identify!']();
            mid = "'" + (scope.$mid()) + "'";
            break;;
            } else {
            break;
          }};
          return [chain, defn, mid];
        };

        return (def['$uses_block?'] = function() {
          var self = this;

          return self.uses_block;
        }, nil) && 'uses_block?';
      })(self, $scope.get('Base'))
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/module"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$handle', '$children', '$name_and_base', '$helper', '$push', '$line', '$in_scope', '$name=', '$scope', '$add_temp', '$proto', '$stmt', '$body', '$s', '$empty_line', '$to_vars', '$==', '$type', '$cid', '$to_s', '$[]', '$expr', '$raise']);
  self.$require("opal/nodes/scope");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $ModuleNode(){};
        var self = $ModuleNode = $klass($base, $super, 'ModuleNode', $ModuleNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("module");

        self.$children("cid", "body");

        def.$compile = function() {
          var $a, $b, TMP_1, self = this, name = nil, base = nil;

          $a = Opal.to_ary(self.$name_and_base()), name = ($a[0] == null ? nil : $a[0]), base = ($a[1] == null ? nil : $a[1]);
          self.$helper("module");
          self.$push("(function($base) {");
          self.$line("  var self = $module($base, '" + (name) + "');");
          ($a = ($b = self).$in_scope, $a.$$p = (TMP_1 = function(){var self = TMP_1.$$s || this, $a, $b, body_code = nil;

          (($a = [name]), $b = self.$scope(), $b['$name='].apply($b, $a), $a[$a.length-1]);
            self.$add_temp("" + (self.$scope().$proto()) + " = self.$$proto");
            self.$add_temp("$scope = self.$$scope");
            body_code = self.$stmt(((($a = self.$body()) !== false && $a !== nil) ? $a : self.$s("nil")));
            self.$empty_line();
            self.$line(self.$scope().$to_vars());
            return self.$line(body_code);}, TMP_1.$$s = self, TMP_1), $a).call($b);
          return self.$line("})(", base, ")");
        };

        return (def.$name_and_base = function() {
          var self = this;

          if (self.$cid().$type()['$==']("const")) {
            return [self.$cid()['$[]'](1).$to_s(), "self"]
          } else if (self.$cid().$type()['$==']("colon2")) {
            return [self.$cid()['$[]'](2).$to_s(), self.$expr(self.$cid()['$[]'](1))]
          } else if (self.$cid().$type()['$==']("colon3")) {
            return [self.$cid()['$[]'](1).$to_s(), "Opal.Object"]
            } else {
            return self.$raise("Bad receiver in module")
          };
        }, nil) && 'name_and_base';
      })(self, $scope.get('ScopeNode'))
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/class"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$handle', '$children', '$name_and_base', '$helper', '$push', '$line', '$in_scope', '$name=', '$scope', '$add_temp', '$proto', '$body_code', '$empty_line', '$to_vars', '$super_code', '$sup', '$expr', '$stmt', '$returns', '$compiler', '$body', '$s']);
  self.$require("opal/nodes/module");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $ClassNode(){};
        var self = $ClassNode = $klass($base, $super, 'ClassNode', $ClassNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("class");

        self.$children("cid", "sup", "body");

        def.$compile = function() {
          var $a, $b, TMP_1, self = this, name = nil, base = nil;

          $a = Opal.to_ary(self.$name_and_base()), name = ($a[0] == null ? nil : $a[0]), base = ($a[1] == null ? nil : $a[1]);
          self.$helper("klass");
          self.$push("(function($base, $super) {");
          self.$line("  function $" + (name) + "(){};");
          self.$line("  var self = $" + (name) + " = $klass($base, $super, '" + (name) + "', $" + (name) + ");");
          ($a = ($b = self).$in_scope, $a.$$p = (TMP_1 = function(){var self = TMP_1.$$s || this, $a, $b, body_code = nil;

          (($a = [name]), $b = self.$scope(), $b['$name='].apply($b, $a), $a[$a.length-1]);
            self.$add_temp("" + (self.$scope().$proto()) + " = self.$$proto");
            self.$add_temp("$scope = self.$$scope");
            body_code = self.$body_code();
            self.$empty_line();
            self.$line(self.$scope().$to_vars());
            return self.$line(body_code);}, TMP_1.$$s = self, TMP_1), $a).call($b);
          return self.$line("})(", base, ", ", self.$super_code(), ")");
        };

        def.$super_code = function() {
          var $a, self = this;

          if ((($a = self.$sup()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$expr(self.$sup())
            } else {
            return "null"
          };
        };

        return (def.$body_code = function() {
          var $a, self = this;

          return self.$stmt(self.$compiler().$returns(((($a = self.$body()) !== false && $a !== nil) ? $a : self.$s("nil"))));
        }, nil) && 'body_code';
      })(self, $scope.get('ModuleNode'))
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/singleton_class"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$handle', '$children', '$push', '$in_scope', '$add_temp', '$stmt', '$returns', '$compiler', '$body', '$line', '$to_vars', '$scope', '$recv', '$object']);
  self.$require("opal/nodes/scope");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $SingletonClassNode(){};
        var self = $SingletonClassNode = $klass($base, $super, 'SingletonClassNode', $SingletonClassNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("sclass");

        self.$children("object", "body");

        return (def.$compile = function() {
          var $a, $b, TMP_1, self = this;

          self.$push("(function(self) {");
          ($a = ($b = self).$in_scope, $a.$$p = (TMP_1 = function(){var self = TMP_1.$$s || this, body_stmt = nil;

          self.$add_temp("$scope = self.$$scope");
            self.$add_temp("def = self.$$proto");
            body_stmt = self.$stmt(self.$compiler().$returns(self.$body()));
            self.$line(self.$scope().$to_vars());
            return self.$line(body_stmt);}, TMP_1.$$s = self, TMP_1), $a).call($b);
          return self.$line("})(", self.$recv(self.$object()), ".$singleton_class())");
        }, nil) && 'compile';
      })(self, $scope.get('ScopeNode'))
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/iter"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_minus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs - rhs : lhs['$-'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass, $range = Opal.range;

  Opal.add_stubs(['$require', '$handle', '$children', '$extract_opt_args', '$extract_block_arg', '$is_a?', '$last', '$args', '$==', '$type', '$[]', '$pop', '$length', '$args_to_params', '$<<', '$in_scope', '$identify!', '$scope', '$add_temp', '$compile_args', '$add_arg', '$push', '$block_name=', '$line', '$stmt', '$body', '$to_vars', '$unshift', '$join', '$each_with_index', '$variable', '$find', '$to_sym', '$expr', '$raise', '$shift', '$===', '$args_sexp', '$nil?', '$s', '$returns', '$compiler', '$body_sexp', '$each', '$next_temp']);
  self.$require("opal/nodes/scope");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $IterNode(){};
        var self = $IterNode = $klass($base, $super, 'IterNode', $IterNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("iter");

        self.$children("args_sexp", "body_sexp");

        def.$compile = function() {
          var $a, $b, TMP_1, self = this, opt_args = nil, block_arg = nil, splat = nil, len = nil, params = nil, to_vars = nil, identity = nil, body_code = nil;

          opt_args = self.$extract_opt_args();
          block_arg = self.$extract_block_arg();
          if ((($a = ($b = self.$args().$last()['$is_a?']($scope.get('Sexp')), $b !== false && $b !== nil ?self.$args().$last().$type()['$==']("splat") : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
            splat = self.$args().$last()['$[]'](1)['$[]'](1);
            self.$args().$pop();
            len = self.$args().$length();};
          params = self.$args_to_params(self.$args()['$[]']($range(1, -1, false)));
          if (splat !== false && splat !== nil) {
            params['$<<'](splat)};
          to_vars = identity = body_code = nil;
          ($a = ($b = self).$in_scope, $a.$$p = (TMP_1 = function(){var self = TMP_1.$$s || this, $a, $b, scope_name = nil;

          identity = self.$scope()['$identify!']();
            self.$add_temp("self = " + (identity) + ".$$s || this");
            self.$compile_args(self.$args()['$[]']($range(1, -1, false)), opt_args, params);
            if (splat !== false && splat !== nil) {
              self.$scope().$add_arg(splat);
              self.$push("" + (splat) + " = $slice.call(arguments, " + ($rb_minus(len, 1)) + ");");};
            if (block_arg !== false && block_arg !== nil) {
              (($a = [block_arg]), $b = self.$scope(), $b['$block_name='].apply($b, $a), $a[$a.length-1]);
              self.$scope().$add_temp(block_arg);
              scope_name = self.$scope()['$identify!']();
              self.$line("" + (block_arg) + " = " + (scope_name) + ".$$p || nil, " + (scope_name) + ".$$p = null;");};
            body_code = self.$stmt(self.$body());
            return to_vars = self.$scope().$to_vars();}, TMP_1.$$s = self, TMP_1), $a).call($b);
          self.$line(body_code);
          self.$unshift(to_vars);
          self.$unshift("(" + (identity) + " = function(" + (params.$join(", ")) + "){");
          return self.$push("}, " + (identity) + ".$$s = self, " + (identity) + ")");
        };

        def.$compile_args = function(args, opt_args, params) {
          var $a, $b, TMP_2, self = this;

          return ($a = ($b = args).$each_with_index, $a.$$p = (TMP_2 = function(arg, idx){var self = TMP_2.$$s || this, $a, $b, $c, $d, TMP_3, TMP_4, current_opt = nil;
if (arg == null) arg = nil;if (idx == null) idx = nil;
          if (arg.$type()['$==']("lasgn")) {
              arg = self.$variable(arg['$[]'](1));
              if ((($a = (($b = opt_args !== false && opt_args !== nil) ? current_opt = ($c = ($d = opt_args).$find, $c.$$p = (TMP_3 = function(s){var self = TMP_3.$$s || this;
if (s == null) s = nil;
              return s['$[]'](1)['$=='](arg.$to_sym())}, TMP_3.$$s = self, TMP_3), $c).call($d) : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
                return self.$push("if (" + (arg) + " == null) " + (arg) + " = ", self.$expr(current_opt['$[]'](2)), ";")
                } else {
                return self.$push("if (" + (arg) + " == null) " + (arg) + " = nil;")
              };
            } else if (arg.$type()['$==']("array")) {
              return ($a = ($b = arg['$[]']($range(1, -1, false))).$each_with_index, $a.$$p = (TMP_4 = function(_arg, _idx){var self = TMP_4.$$s || this;
if (_arg == null) _arg = nil;if (_idx == null) _idx = nil;
              _arg = self.$variable(_arg['$[]'](1));
                return self.$push("var " + (_arg) + " = " + (params['$[]'](idx)) + "[" + (_idx) + "];");}, TMP_4.$$s = self, TMP_4), $a).call($b)
              } else {
              return self.$raise("Bad block arg type")
            }}, TMP_2.$$s = self, TMP_2), $a).call($b);
        };

        def.$extract_opt_args = function() {
          var $a, $b, self = this, opt_args = nil;

          if ((($a = ($b = self.$args().$last()['$is_a?']($scope.get('Sexp')), $b !== false && $b !== nil ?self.$args().$last().$type()['$==']("block") : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
            opt_args = self.$args().$pop();
            opt_args.$shift();
            return opt_args;
            } else {
            return nil
          };
        };

        def.$extract_block_arg = function() {
          var $a, $b, self = this, block_arg = nil;

          if ((($a = ($b = self.$args().$last()['$is_a?']($scope.get('Sexp')), $b !== false && $b !== nil ?self.$args().$last().$type()['$==']("block_pass") : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
            block_arg = self.$args().$pop();
            return block_arg = block_arg['$[]'](1)['$[]'](1).$to_sym();
            } else {
            return nil
          };
        };

        def.$args = function() {
          var $a, $b, self = this;

          if ((($a = ((($b = $scope.get('Fixnum')['$==='](self.$args_sexp())) !== false && $b !== nil) ? $b : self.$args_sexp()['$nil?']())) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$s("array")
          } else if (self.$args_sexp().$type()['$==']("lasgn")) {
            return self.$s("array", self.$args_sexp())
            } else {
            return self.$args_sexp()['$[]'](1)
          };
        };

        def.$body = function() {
          var $a, self = this;

          return self.$compiler().$returns(((($a = self.$body_sexp()) !== false && $a !== nil) ? $a : self.$s("nil")));
        };

        return (def.$args_to_params = function(sexp) {
          var $a, $b, TMP_5, self = this, result = nil;

          result = [];
          ($a = ($b = sexp).$each, $a.$$p = (TMP_5 = function(arg){var self = TMP_5.$$s || this, ref = nil;
if (arg == null) arg = nil;
          if (arg['$[]'](0)['$==']("lasgn")) {
              ref = self.$variable(arg['$[]'](1));
              self.$add_arg(ref);
              return result['$<<'](ref);
            } else if (arg['$[]'](0)['$==']("array")) {
              return result['$<<'](self.$scope().$next_temp())
              } else {
              return self.$raise("Bad js_block_arg: " + (arg['$[]'](0)))
            }}, TMP_5.$$s = self, TMP_5), $a).call($b);
          return result;
        }, nil) && 'args_to_params';
      })(self, $scope.get('ScopeNode'))
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/def"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_minus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs - rhs : lhs['$-'](rhs);
  }
  function $rb_lt(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs < rhs : lhs['$<'](rhs);
  }
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass, $range = Opal.range;

  Opal.add_stubs(['$require', '$handle', '$children', '$select', '$==', '$first', '$[]', '$args', '$find', '$include?', '$length', '$block_arg', '$rest_arg', '$size', '$keyword_args', '$mid_to_jsid', '$to_s', '$mid', '$to_sym', '$variable', '$arity_check?', '$compiler', '$arity_check', '$opt_args', '$in_scope', '$mid=', '$scope', '$recvr', '$defs=', '$uses_block!', '$add_arg', '$block_name=', '$process', '$stmt', '$returns', '$stmts', '$add_temp', '$compile_rest_arg', '$compile_opt_args', '$compile_keyword_args', '$identity', '$compile_block_arg', '$unshift', '$current_indent', '$to_vars', '$line', '$uses_zuper', '$catch_return', '$push', '$recv', '$uses_defn?', '$wrap', '$class?', '$proto', '$sclass?', '$top?', '$expr?', '$uses_block?', '$block_name', '$argc', '$each', '$expr', '$empty?', '$helper', '$with_temp', '$last', '$===', '$add_local', '$map', '$inspect', '$join', '$raise', '$iter?', '$module?', '$name', '$!', '$-@', '$each_with_index', '$nil?']);
  self.$require("opal/nodes/scope");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $DefNode(){};
        var self = $DefNode = $klass($base, $super, 'DefNode', $DefNode);

        var def = self.$$proto, $scope = self.$$scope;

        def.opt_args = def.rest_arg = def.keyword_args = def.block_arg = def.argc = nil;
        self.$handle("def");

        self.$children("recvr", "mid", "args", "stmts");

        def.$opt_args = function() {
          var $a, $b, $c, TMP_1, self = this;

          return ((($a = self.opt_args) !== false && $a !== nil) ? $a : self.opt_args = ($b = ($c = self.$args()['$[]']($range(1, -1, false))).$select, $b.$$p = (TMP_1 = function(arg){var self = TMP_1.$$s || this;
if (arg == null) arg = nil;
          return arg.$first()['$==']("optarg")}, TMP_1.$$s = self, TMP_1), $b).call($c));
        };

        def.$rest_arg = function() {
          var $a, $b, $c, TMP_2, self = this;

          return ((($a = self.rest_arg) !== false && $a !== nil) ? $a : self.rest_arg = ($b = ($c = self.$args()['$[]']($range(1, -1, false))).$find, $b.$$p = (TMP_2 = function(arg){var self = TMP_2.$$s || this;
if (arg == null) arg = nil;
          return arg.$first()['$==']("restarg")}, TMP_2.$$s = self, TMP_2), $b).call($c));
        };

        def.$keyword_args = function() {
          var $a, $b, $c, TMP_3, self = this;

          return ((($a = self.keyword_args) !== false && $a !== nil) ? $a : self.keyword_args = ($b = ($c = self.$args()['$[]']($range(1, -1, false))).$select, $b.$$p = (TMP_3 = function(arg){var self = TMP_3.$$s || this;
if (arg == null) arg = nil;
          return ["kwarg", "kwoptarg", "kwrestarg"]['$include?'](arg.$first())}, TMP_3.$$s = self, TMP_3), $b).call($c));
        };

        def.$block_arg = function() {
          var $a, $b, $c, TMP_4, self = this;

          return ((($a = self.block_arg) !== false && $a !== nil) ? $a : self.block_arg = ($b = ($c = self.$args()['$[]']($range(1, -1, false))).$find, $b.$$p = (TMP_4 = function(arg){var self = TMP_4.$$s || this;
if (arg == null) arg = nil;
          return arg.$first()['$==']("blockarg")}, TMP_4.$$s = self, TMP_4), $b).call($c));
        };

        def.$argc = function() {
          var $a, self = this;

          if ((($a = self.argc) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.argc};
          self.argc = $rb_minus(self.$args().$length(), 1);
          if ((($a = self.$block_arg()) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.argc = $rb_minus(self.argc, 1)};
          if ((($a = self.$rest_arg()) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.argc = $rb_minus(self.argc, 1)};
          self.argc = $rb_minus(self.argc, self.$keyword_args().$size());
          return self.argc;
        };

        def.$compile = function() {
          var $a, $b, TMP_5, self = this, jsid = nil, params = nil, scope_name = nil, block_name = nil, arity_code = nil;

          jsid = self.$mid_to_jsid(self.$mid().$to_s());
          params = nil;
          scope_name = nil;
          if ((($a = self.$block_arg()) !== nil && (!$a.$$is_boolean || $a == true))) {
            block_name = self.$variable(self.$block_arg()['$[]'](1)).$to_sym()};
          if ((($a = self.$compiler()['$arity_check?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            arity_code = self.$arity_check(self.$args(), self.$opt_args(), self.$rest_arg(), self.$keyword_args(), block_name, self.$mid())};
          ($a = ($b = self).$in_scope, $a.$$p = (TMP_5 = function(){var self = TMP_5.$$s || this, $a, $b, $c, stmt_code = nil;

          (($a = [self.$mid()]), $b = self.$scope(), $b['$mid='].apply($b, $a), $a[$a.length-1]);
            if ((($a = self.$recvr()) !== nil && (!$a.$$is_boolean || $a == true))) {
              (($a = [true]), $b = self.$scope(), $b['$defs='].apply($b, $a), $a[$a.length-1])};
            if (block_name !== false && block_name !== nil) {
              self.$scope()['$uses_block!']();
              self.$scope().$add_arg(block_name);};
            (($a = [((($c = block_name) !== false && $c !== nil) ? $c : "$yield")]), $b = self.$scope(), $b['$block_name='].apply($b, $a), $a[$a.length-1]);
            params = self.$process(self.$args());
            stmt_code = self.$stmt(self.$compiler().$returns(self.$stmts()));
            self.$add_temp("self = this");
            self.$compile_rest_arg();
            self.$compile_opt_args();
            self.$compile_keyword_args();
            scope_name = self.$scope().$identity();
            self.$compile_block_arg();
            self.$unshift("\n" + (self.$current_indent()), self.$scope().$to_vars());
            self.$line(stmt_code);
            if (arity_code !== false && arity_code !== nil) {
              self.$unshift(arity_code)};
            if ((($a = self.$scope().$uses_zuper()) !== nil && (!$a.$$is_boolean || $a == true))) {
              self.$unshift("var $zuper = $slice.call(arguments, 0);")};
            if ((($a = self.$scope().$catch_return()) !== nil && (!$a.$$is_boolean || $a == true))) {
              self.$unshift("try {\n");
              self.$line("} catch ($returner) { if ($returner === Opal.returner) { return $returner.$v }");
              return self.$push(" throw $returner; }");
              } else {
              return nil
            };}, TMP_5.$$s = self, TMP_5), $a).call($b);
          self.$unshift(") {");
          self.$unshift(params);
          self.$unshift("function(");
          if (scope_name !== false && scope_name !== nil) {
            self.$unshift("" + (scope_name) + " = ")};
          self.$line("}");
          if ((($a = self.$recvr()) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.$unshift("Opal.defs(", self.$recv(self.$recvr()), ", '$" + (self.$mid()) + "', ");
            self.$push(")");
          } else if ((($a = self['$uses_defn?'](self.$scope())) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.$wrap("Opal.defn(self, '$" + (self.$mid()) + "', ", ")")
          } else if ((($a = self.$scope()['$class?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.$unshift("" + (self.$scope().$proto()) + (jsid) + " = ")
          } else if ((($a = self.$scope()['$sclass?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.$unshift("self.$$proto" + (jsid) + " = ")
          } else if ((($a = self.$scope()['$top?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.$unshift("Opal.Object.$$proto" + (jsid) + " = ")
            } else {
            self.$unshift("def" + (jsid) + " = ")
          };
          if ((($a = self['$expr?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$wrap("(", ", nil) && '" + (self.$mid()) + "'")
            } else {
            return nil
          };
        };

        def.$compile_block_arg = function() {
          var $a, self = this, scope_name = nil, yielder = nil;

          if ((($a = self.$scope()['$uses_block?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            scope_name = self.$scope().$identity();
            yielder = self.$scope().$block_name();
            self.$add_temp("$iter = " + (scope_name) + ".$$p");
            self.$add_temp("" + (yielder) + " = $iter || nil");
            return self.$line("" + (scope_name) + ".$$p = null;");
            } else {
            return nil
          };
        };

        def.$compile_rest_arg = function() {
          var $a, $b, self = this, splat = nil;

          if ((($a = ($b = self.$rest_arg(), $b !== false && $b !== nil ?self.$rest_arg()['$[]'](1) : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
            splat = self.$variable(self.$rest_arg()['$[]'](1).$to_sym());
            return self.$line("" + (splat) + " = $slice.call(arguments, " + (self.$argc()) + ");");
            } else {
            return nil
          };
        };

        def.$compile_opt_args = function() {
          var $a, $b, TMP_6, self = this;

          return ($a = ($b = self.$opt_args()).$each, $a.$$p = (TMP_6 = function(arg){var self = TMP_6.$$s || this;
if (arg == null) arg = nil;
          if (arg['$[]'](2)['$[]'](2)['$==']("undefined")) {
              return nil;};
            self.$line("if (" + (self.$variable(arg['$[]'](1))) + " == null) {");
            self.$line("  " + (self.$variable(arg['$[]'](1))) + " = ", self.$expr(arg['$[]'](2)));
            return self.$line("}");}, TMP_6.$$s = self, TMP_6), $a).call($b);
        };

        def.$compile_keyword_args = function() {
          var $a, $b, TMP_7, $c, TMP_8, self = this, last_opt_arg = nil, opt_arg_name = nil;

          if ((($a = self.$keyword_args()['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return nil};
          self.$helper("hash2");
          if ((($a = self.$rest_arg()) !== nil && (!$a.$$is_boolean || $a == true))) {
            ($a = ($b = self).$with_temp, $a.$$p = (TMP_7 = function(tmp){var self = TMP_7.$$s || this, rest_arg_name = nil;
if (tmp == null) tmp = nil;
            rest_arg_name = self.$variable(self.$rest_arg()['$[]'](1).$to_sym());
              self.$line("" + (tmp) + " = " + (rest_arg_name) + "[" + (rest_arg_name) + ".length - 1];");
              self.$line("if (" + (tmp) + " == null || !" + (tmp) + ".$$is_hash) {");
              self.$line("  $kwargs = $hash2([], {});");
              self.$line("} else {");
              self.$line("  $kwargs = " + (rest_arg_name) + ".pop();");
              return self.$line("}");}, TMP_7.$$s = self, TMP_7), $a).call($b)
          } else if ((($a = last_opt_arg = self.$opt_args().$last()) !== nil && (!$a.$$is_boolean || $a == true))) {
            opt_arg_name = self.$variable(last_opt_arg['$[]'](1));
            self.$line("if (" + (opt_arg_name) + " == null) {");
            self.$line("  $kwargs = $hash2([], {});");
            self.$line("}");
            self.$line("else if (" + (opt_arg_name) + ".$$is_hash) {");
            self.$line("  $kwargs = " + (opt_arg_name) + ";");
            self.$line("  " + (opt_arg_name) + " = ", self.$expr(last_opt_arg['$[]'](2)), ";");
            self.$line("}");
            } else {
            self.$line("if ($kwargs == null) {");
            self.$line("  $kwargs = $hash2([], {});");
            self.$line("}");
          };
          self.$line("if (!$kwargs.$$is_hash) {");
          self.$line("  throw Opal.ArgumentError.$new('expecting keyword args');");
          self.$line("}");
          return ($a = ($c = self.$keyword_args()).$each, $a.$$p = (TMP_8 = function(kwarg){var self = TMP_8.$$s || this, $a, $b, TMP_9, $c, $d, TMP_10, $case = nil, arg_name = nil, var_name = nil, kwarg_names = nil, used_args = nil;
if (kwarg == null) kwarg = nil;
          return (function() {$case = kwarg.$first();if ("kwoptarg"['$===']($case)) {arg_name = kwarg['$[]'](1);
            var_name = self.$variable(arg_name.$to_s());
            self.$add_local(var_name);
            self.$line("if ((" + (var_name) + " = $kwargs.smap['" + (arg_name) + "']) == null) {");
            self.$line("  " + (var_name) + " = ", self.$expr(kwarg['$[]'](2)));
            return self.$line("}");}else if ("kwarg"['$===']($case)) {arg_name = kwarg['$[]'](1);
            var_name = self.$variable(arg_name.$to_s());
            self.$add_local(var_name);
            self.$line("if ((" + (var_name) + " = $kwargs.smap['" + (arg_name) + "']) == null) {");
            self.$line("  throw new Error('expecting keyword arg: " + (arg_name) + "')");
            return self.$line("}");}else if ("kwrestarg"['$===']($case)) {arg_name = kwarg['$[]'](1);
            var_name = self.$variable(arg_name.$to_s());
            self.$add_local(var_name);
            kwarg_names = ($a = ($b = ($c = ($d = self.$keyword_args()).$select, $c.$$p = (TMP_10 = function(kw){var self = TMP_10.$$s || this;
if (kw == null) kw = nil;
            return ["kwoptarg", "kwarg"]['$include?'](kw.$first())}, TMP_10.$$s = self, TMP_10), $c).call($d)).$map, $a.$$p = (TMP_9 = function(kw){var self = TMP_9.$$s || this;
if (kw == null) kw = nil;
            return "" + (kw['$[]'](1).$to_s().$inspect()) + ": true"}, TMP_9.$$s = self, TMP_9), $a).call($b);
            used_args = "{" + (kwarg_names.$join(",")) + "}";
            return self.$line("" + (var_name) + " = Opal.kwrestargs($kwargs, " + (used_args) + ");");}else {return self.$raise("unknown kwarg type " + (kwarg.$first()))}})()}, TMP_8.$$s = self, TMP_8), $a).call($c);
        };

        def['$uses_defn?'] = function(scope) {
          var $a, $b, self = this;

          if ((($a = ((($b = scope['$iter?']()) !== false && $b !== nil) ? $b : scope['$module?']())) !== nil && (!$a.$$is_boolean || $a == true))) {
            return true
          } else if ((($a = ($b = scope['$class?'](), $b !== false && $b !== nil ?["Object", "BasicObject"]['$include?'](scope.$name()) : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
            return true
            } else {
            return false
          };
        };

        return (def.$arity_check = function(args, opt, splat, kwargs, block_name, mid) {
          var $a, $b, $c, self = this, meth = nil, arity = nil, aritycode = nil;

          meth = mid.$to_s().$inspect();
          arity = $rb_minus(args.$size(), 1);
          arity = $rb_minus(arity, (opt.$size()));
          if (splat !== false && splat !== nil) {
            arity = $rb_minus(arity, 1)};
          arity = $rb_minus(arity, (kwargs.$size()));
          if (block_name !== false && block_name !== nil) {
            arity = $rb_minus(arity, 1)};
          if ((($a = ((($b = ((($c = opt['$empty?']()['$!']()) !== false && $c !== nil) ? $c : kwargs['$empty?']()['$!']())) !== false && $b !== nil) ? $b : splat)) !== nil && (!$a.$$is_boolean || $a == true))) {
            arity = $rb_minus(arity['$-@'](), 1)};
          aritycode = "var $arity = arguments.length;";
          if ($rb_lt(arity, 0)) {
            return $rb_plus(aritycode, "if ($arity < " + (($rb_plus(arity, 1))['$-@']()) + ") { Opal.ac($arity, " + (arity) + ", this, " + (meth) + "); }")
            } else {
            return $rb_plus(aritycode, "if ($arity !== " + (arity) + ") { Opal.ac($arity, " + (arity) + ", this, " + (meth) + "); }")
          };
        }, nil) && 'arity_check';
      })(self, $scope.get('ScopeNode'));

      (function($base, $super) {
        function $ArgsNode(){};
        var self = $ArgsNode = $klass($base, $super, 'ArgsNode', $ArgsNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("args");

        return (def.$compile = function() {
          var $a, $b, TMP_11, self = this, done_kwargs = nil;

          done_kwargs = false;
          return ($a = ($b = self.$children()).$each_with_index, $a.$$p = (TMP_11 = function(child, idx){var self = TMP_11.$$s || this, $a, $b, $case = nil;
if (child == null) child = nil;if (idx == null) idx = nil;
          if ("blockarg"['$=='](child.$first())) {
              return nil;};
            if ((($a = (($b = "restarg"['$=='](child.$first())) ? child['$[]'](1)['$nil?']() : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
              return nil;};
            return (function() {$case = child.$first();if ("kwarg"['$===']($case) || "kwoptarg"['$===']($case) || "kwrestarg"['$===']($case)) {if (done_kwargs !== false && done_kwargs !== nil) {
              return nil
              } else {
              done_kwargs = true;
              if (idx['$=='](0)) {
                } else {
                self.$push(", ")
              };
              self.$scope().$add_arg("$kwargs");
              return self.$push("$kwargs");
            }}else {child = child['$[]'](1).$to_sym();
            if (idx['$=='](0)) {
              } else {
              self.$push(", ")
            };
            child = self.$variable(child);
            self.$scope().$add_arg(child.$to_sym());
            return self.$push(child.$to_s());}})();}, TMP_11.$$s = self, TMP_11), $a).call($b);
        }, nil) && 'compile';
      })(self, $scope.get('Base'));
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/if"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$handle', '$children', '$truthy', '$falsy', '$skip_check_present?', '$skip_check_present_not?', '$push', '$js_truthy', '$test', '$indent', '$line', '$stmt', '$==', '$type', '$needs_wrapper?', '$wrap', '$returns', '$compiler', '$true_body', '$s', '$false_body', '$expr?', '$recv?']);
  self.$require("opal/nodes/base");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $IfNode(){};
        var self = $IfNode = $klass($base, $super, 'IfNode', $IfNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("if");

        self.$children("test", "true_body", "false_body");

        Opal.cdecl($scope, 'RUBY_ENGINE_CHECK', ["call", ["const", "RUBY_ENGINE"], "==", ["arglist", ["str", "opal"]]]);

        Opal.cdecl($scope, 'RUBY_PLATFORM_CHECK', ["call", ["const", "RUBY_PLATFORM"], "==", ["arglist", ["str", "opal"]]]);

        Opal.cdecl($scope, 'RUBY_ENGINE_CHECK_NOT', ["call", ["call", ["const", "RUBY_ENGINE"], "==", ["arglist", ["str", "opal"]]], "!", ["arglist"]]);

        Opal.cdecl($scope, 'RUBY_PLATFORM_CHECK_NOT', ["call", ["call", ["const", "RUBY_PLATFORM"], "==", ["arglist", ["str", "opal"]]], "!", ["arglist"]]);

        def.$compile = function() {
          var $a, $b, TMP_1, $c, TMP_2, self = this, truthy = nil, falsy = nil;

          $a = [self.$truthy(), self.$falsy()], truthy = $a[0], falsy = $a[1];
          if ((($a = self['$skip_check_present?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            falsy = nil};
          if ((($a = self['$skip_check_present_not?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            truthy = nil};
          self.$push("if (", self.$js_truthy(self.$test()), ") {");
          if (truthy !== false && truthy !== nil) {
            ($a = ($b = self).$indent, $a.$$p = (TMP_1 = function(){var self = TMP_1.$$s || this;

            return self.$line(self.$stmt(truthy))}, TMP_1.$$s = self, TMP_1), $a).call($b)};
          if (falsy !== false && falsy !== nil) {
            if (falsy.$type()['$==']("if")) {
              self.$line("} else ", self.$stmt(falsy))
              } else {
              ($a = ($c = self).$indent, $a.$$p = (TMP_2 = function(){var self = TMP_2.$$s || this;

              self.$line("} else {");
                return self.$line(self.$stmt(falsy));}, TMP_2.$$s = self, TMP_2), $a).call($c);
              self.$line("}");
            }
            } else {
            self.$push("}")
          };
          if ((($a = self['$needs_wrapper?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$wrap("(function() {", "; return nil; })()")
            } else {
            return nil
          };
        };

        def['$skip_check_present?'] = function() {
          var $a, self = this;

          return ((($a = self.$test()['$==']($scope.get('RUBY_ENGINE_CHECK'))) !== false && $a !== nil) ? $a : self.$test()['$==']($scope.get('RUBY_PLATFORM_CHECK')));
        };

        def['$skip_check_present_not?'] = function() {
          var $a, self = this;

          return ((($a = self.$test()['$==']($scope.get('RUBY_ENGINE_CHECK_NOT'))) !== false && $a !== nil) ? $a : self.$test()['$==']($scope.get('RUBY_PLATFORM_CHECK_NOT')));
        };

        def.$truthy = function() {
          var $a, self = this;

          if ((($a = self['$needs_wrapper?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$compiler().$returns(((($a = self.$true_body()) !== false && $a !== nil) ? $a : self.$s("nil")))
            } else {
            return self.$true_body()
          };
        };

        def.$falsy = function() {
          var $a, self = this;

          if ((($a = self['$needs_wrapper?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$compiler().$returns(((($a = self.$false_body()) !== false && $a !== nil) ? $a : self.$s("nil")))
            } else {
            return self.$false_body()
          };
        };

        return (def['$needs_wrapper?'] = function() {
          var $a, self = this;

          return ((($a = self['$expr?']()) !== false && $a !== nil) ? $a : self['$recv?']());
        }, nil) && 'needs_wrapper?';
      })(self, $scope.get('Base'))
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/logic"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_gt(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs > rhs : lhs['$>'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$handle', '$children', '$in_while?', '$push', '$expr_or_nil', '$value', '$wrap', '$compile_while', '$iter?', '$scope', '$compile_iter', '$error', '$[]', '$while_loop', '$stmt?', '$[]=', '$identity', '$with_temp', '$expr', '$==', '$empty_splat?', '$type', '$recv', '$lhs', '$rhs', '$js_truthy_optimize', '$nil?', '$s', '$size', '$find_parent_def', '$expr?', '$def?', '$return_in_iter?', '$return_expr_in_def?', '$scope_to_catch_return', '$catch_return=', '$return_val', '$raise', '$to_s']);
  self.$require("opal/nodes/base");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $NextNode(){};
        var self = $NextNode = $klass($base, $super, 'NextNode', $NextNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("next");

        self.$children("value");

        return (def.$compile = function() {
          var $a, self = this;

          if ((($a = self['$in_while?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$push("continue;")};
          self.$push(self.$expr_or_nil(self.$value()));
          return self.$wrap("return ", ";");
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $BreakNode(){};
        var self = $BreakNode = $klass($base, $super, 'BreakNode', $BreakNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("break");

        self.$children("value");

        def.$compile = function() {
          var $a, self = this;

          if ((($a = self['$in_while?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$compile_while()
          } else if ((($a = self.$scope()['$iter?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$compile_iter()
            } else {
            return self.$error("void value expression: cannot use break outside of iter/while")
          };
        };

        def.$compile_while = function() {
          var $a, self = this;

          if ((($a = self.$while_loop()['$[]']("closure")) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$push("return ", self.$expr_or_nil(self.$value()))
            } else {
            return self.$push("break;")
          };
        };

        return (def.$compile_iter = function() {
          var $a, self = this;

          if ((($a = self['$stmt?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            } else {
            self.$error("break must be used as a statement")
          };
          self.$push(self.$expr_or_nil(self.$value()));
          return self.$wrap("return ($breaker.$v = ", ", $breaker)");
        }, nil) && 'compile_iter';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $RedoNode(){};
        var self = $RedoNode = $klass($base, $super, 'RedoNode', $RedoNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("redo");

        def.$compile = function() {
          var $a, self = this;

          if ((($a = self['$in_while?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$compile_while()
          } else if ((($a = self.$scope()['$iter?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$compile_iter()
            } else {
            return self.$push("REDO()")
          };
        };

        def.$compile_while = function() {
          var self = this;

          self.$while_loop()['$[]=']("use_redo", true);
          return self.$push("" + (self.$while_loop()['$[]']("redo_var")) + " = true");
        };

        return (def.$compile_iter = function() {
          var self = this;

          return self.$push("return " + (self.$scope().$identity()) + ".apply(null, $slice.call(arguments))");
        }, nil) && 'compile_iter';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $NotNode(){};
        var self = $NotNode = $klass($base, $super, 'NotNode', $NotNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("not");

        self.$children("value");

        return (def.$compile = function() {
          var $a, $b, TMP_1, self = this;

          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_1 = function(tmp){var self = TMP_1.$$s || this;
if (tmp == null) tmp = nil;
          self.$push(self.$expr(self.$value()));
            return self.$wrap("(" + (tmp) + " = ", ", (" + (tmp) + " === nil || " + (tmp) + " === false))");}, TMP_1.$$s = self, TMP_1), $a).call($b);
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $SplatNode(){};
        var self = $SplatNode = $klass($base, $super, 'SplatNode', $SplatNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("splat");

        self.$children("value");

        def['$empty_splat?'] = function() {
          var $a, self = this;

          return ((($a = self.$value()['$=='](["nil"])) !== false && $a !== nil) ? $a : self.$value()['$=='](["paren", ["nil"]]));
        };

        return (def.$compile = function() {
          var $a, self = this;

          if ((($a = self['$empty_splat?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$push("[]")
          } else if (self.$value().$type()['$==']("sym")) {
            return self.$push("[", self.$expr(self.$value()), "]")
            } else {
            return self.$push(self.$recv(self.$value()))
          };
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $OrNode(){};
        var self = $OrNode = $klass($base, $super, 'OrNode', $OrNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("or");

        self.$children("lhs", "rhs");

        return (def.$compile = function() {
          var $a, $b, TMP_2, self = this;

          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_2 = function(tmp){var self = TMP_2.$$s || this;
if (tmp == null) tmp = nil;
          self.$push("(((" + (tmp) + " = ");
            self.$push(self.$expr(self.$lhs()));
            self.$push(") !== false && " + (tmp) + " !== nil) ? " + (tmp) + " : ");
            self.$push(self.$expr(self.$rhs()));
            return self.$push(")");}, TMP_2.$$s = self, TMP_2), $a).call($b);
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $AndNode(){};
        var self = $AndNode = $klass($base, $super, 'AndNode', $AndNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("and");

        self.$children("lhs", "rhs");

        return (def.$compile = function() {
          var $a, $b, TMP_3, self = this, truthy_opt = nil;

          truthy_opt = nil;
          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_3 = function(tmp){var self = TMP_3.$$s || this, $a;
if (tmp == null) tmp = nil;
          if ((($a = truthy_opt = self.$js_truthy_optimize(self.$lhs())) !== nil && (!$a.$$is_boolean || $a == true))) {
              self.$push("((" + (tmp) + " = ", truthy_opt);
              self.$push(") ? ");
              self.$push(self.$expr(self.$rhs()));
              return self.$push(" : " + (tmp) + ")");
              } else {
              self.$push("(" + (tmp) + " = ");
              self.$push(self.$expr(self.$lhs()));
              self.$push(", " + (tmp) + " !== false && " + (tmp) + " !== nil ?");
              self.$push(self.$expr(self.$rhs()));
              return self.$push(" : " + (tmp) + ")");
            }}, TMP_3.$$s = self, TMP_3), $a).call($b);
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $ReturnNode(){};
        var self = $ReturnNode = $klass($base, $super, 'ReturnNode', $ReturnNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("return");

        self.$children("value");

        def.$return_val = function() {
          var $a, self = this;

          if ((($a = self.$value()['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$expr(self.$s("nil"))
          } else if ($rb_gt(self.$children().$size(), 1)) {
            return self.$expr(($a = self).$s.apply($a, ["array"].concat(self.$children())))
            } else {
            return self.$expr(self.$value())
          };
        };

        def['$return_in_iter?'] = function() {
          var $a, $b, self = this, parent_def = nil;

          if ((($a = ($b = self.$scope()['$iter?'](), $b !== false && $b !== nil ?parent_def = self.$scope().$find_parent_def() : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
            return parent_def
            } else {
            return nil
          };
        };

        def['$return_expr_in_def?'] = function() {
          var $a, $b, self = this;

          if ((($a = ($b = self['$expr?'](), $b !== false && $b !== nil ?self.$scope()['$def?']() : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$scope()
            } else {
            return nil
          };
        };

        def.$scope_to_catch_return = function() {
          var $a, self = this;

          return ((($a = self['$return_in_iter?']()) !== false && $a !== nil) ? $a : self['$return_expr_in_def?']());
        };

        return (def.$compile = function() {
          var $a, $b, self = this, def_scope = nil;

          if ((($a = def_scope = self.$scope_to_catch_return()) !== nil && (!$a.$$is_boolean || $a == true))) {
            (($a = [true]), $b = def_scope, $b['$catch_return='].apply($b, $a), $a[$a.length-1]);
            return self.$push("Opal.ret(", self.$return_val(), ")");
          } else if ((($a = self['$stmt?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$push("return ", self.$return_val())
            } else {
            return self.$raise($scope.get('SyntaxError'), "void value expression: cannot return as an expression")
          };
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $JSReturnNode(){};
        var self = $JSReturnNode = $klass($base, $super, 'JSReturnNode', $JSReturnNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("js_return");

        self.$children("value");

        return (def.$compile = function() {
          var self = this;

          self.$push("return ");
          return self.$push(self.$expr(self.$value()));
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $JSTempNode(){};
        var self = $JSTempNode = $klass($base, $super, 'JSTempNode', $JSTempNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("js_tmp");

        self.$children("value");

        return (def.$compile = function() {
          var self = this;

          return self.$push(self.$value().$to_s());
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $BlockPassNode(){};
        var self = $BlockPassNode = $klass($base, $super, 'BlockPassNode', $BlockPassNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("block_pass");

        self.$children("value");

        return (def.$compile = function() {
          var self = this;

          return self.$push(self.$expr(self.$s("call", self.$value(), "to_proc", self.$s("arglist"))));
        }, nil) && 'compile';
      })(self, $scope.get('Base'));
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/definitions"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass, $range = Opal.range;

  Opal.add_stubs(['$require', '$handle', '$children', '$push', '$process', '$value', '$proto', '$scope', '$mid_to_jsid', '$to_s', '$[]', '$mid', '$new_name', '$old_name', '$class?', '$module?', '$<<', '$methods', '$old_mid', '$new_mid', '$!', '$stmt?', '$==', '$type', '$body', '$stmt', '$returns', '$compiler', '$wrap', '$each_with_index', '$expr', '$empty?', '$stmt_join', '$find_inline_yield', '$child_is_expr?', '$class_scope?', '$current_indent', '$raw_expression?', '$include?', '$first', '$===', '$[]=', '$s', '$has_temp?', '$add_temp']);
  self.$require("opal/nodes/base");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $SvalueNode(){};
        var self = $SvalueNode = $klass($base, $super, 'SvalueNode', $SvalueNode);

        var def = self.$$proto, $scope = self.$$scope;

        def.level = nil;
        self.$handle("svalue");

        self.$children("value");

        return (def.$compile = function() {
          var self = this;

          return self.$push(self.$process(self.$value(), self.level));
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $UndefNode(){};
        var self = $UndefNode = $klass($base, $super, 'UndefNode', $UndefNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("undef");

        self.$children("mid");

        return (def.$compile = function() {
          var self = this;

          return self.$push("delete " + (self.$scope().$proto()) + (self.$mid_to_jsid(self.$mid()['$[]'](1).$to_s())));
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $AliasNode(){};
        var self = $AliasNode = $klass($base, $super, 'AliasNode', $AliasNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("alias");

        self.$children("new_name", "old_name");

        def.$new_mid = function() {
          var self = this;

          return self.$mid_to_jsid(self.$new_name()['$[]'](1).$to_s());
        };

        def.$old_mid = function() {
          var self = this;

          return self.$mid_to_jsid(self.$old_name()['$[]'](1).$to_s());
        };

        return (def.$compile = function() {
          var $a, $b, self = this;

          if ((($a = ((($b = self.$scope()['$class?']()) !== false && $b !== nil) ? $b : self.$scope()['$module?']())) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.$scope().$methods()['$<<']("$" + (self.$new_name()['$[]'](1)));
            return self.$push("Opal.defn(self, '$" + (self.$new_name()['$[]'](1)) + "', " + (self.$scope().$proto()) + (self.$old_mid()) + ")");
            } else {
            return self.$push("self.$$proto" + (self.$new_mid()) + " = self.$$proto" + (self.$old_mid()))
          };
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $BeginNode(){};
        var self = $BeginNode = $klass($base, $super, 'BeginNode', $BeginNode);

        var def = self.$$proto, $scope = self.$$scope;

        def.level = nil;
        self.$handle("begin");

        self.$children("body");

        return (def.$compile = function() {
          var $a, $b, self = this;

          if ((($a = ($b = self['$stmt?']()['$!'](), $b !== false && $b !== nil ?self.$body().$type()['$==']("block") : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.$push(self.$stmt(self.$compiler().$returns(self.$body())));
            return self.$wrap("(function() {", "})()");
            } else {
            return self.$push(self.$process(self.$body(), self.level))
          };
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $ParenNode(){};
        var self = $ParenNode = $klass($base, $super, 'ParenNode', $ParenNode);

        var def = self.$$proto, $scope = self.$$scope;

        def.level = nil;
        self.$handle("paren");

        self.$children("body");

        return (def.$compile = function() {
          var $a, $b, TMP_1, self = this;

          if (self.$body().$type()['$==']("block")) {
            ($a = ($b = self.$body().$children()).$each_with_index, $a.$$p = (TMP_1 = function(child, idx){var self = TMP_1.$$s || this;
if (child == null) child = nil;if (idx == null) idx = nil;
            if (idx['$=='](0)) {
                } else {
                self.$push(", ")
              };
              return self.$push(self.$expr(child));}, TMP_1.$$s = self, TMP_1), $a).call($b);
            return self.$wrap("(", ")");
            } else {
            self.$push(self.$process(self.$body(), self.level));
            if ((($a = self['$stmt?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
              return nil
              } else {
              return self.$wrap("(", ")")
            };
          };
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $BlockNode(){};
        var self = $BlockNode = $klass($base, $super, 'BlockNode', $BlockNode);

        var def = self.$$proto, $scope = self.$$scope;

        def.level = nil;
        self.$handle("block");

        def.$compile = function() {
          var $a, $b, TMP_2, self = this;

          if ((($a = self.$children()['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$push("nil")};
          return ($a = ($b = self.$children()).$each_with_index, $a.$$p = (TMP_2 = function(child, idx){var self = TMP_2.$$s || this, $a, yasgn = nil;
            if (self.level == null) self.level = nil;
if (child == null) child = nil;if (idx == null) idx = nil;
          if (idx['$=='](0)) {
              } else {
              self.$push(self.$stmt_join())
            };
            if ((($a = yasgn = self.$find_inline_yield(child)) !== nil && (!$a.$$is_boolean || $a == true))) {
              self.$push(self.$compiler().$process(yasgn, self.level));
              self.$push(";");};
            self.$push(self.$compiler().$process(child, self.level));
            if ((($a = self['$child_is_expr?'](child)) !== nil && (!$a.$$is_boolean || $a == true))) {
              return self.$push(";")
              } else {
              return nil
            };}, TMP_2.$$s = self, TMP_2), $a).call($b);
        };

        def.$stmt_join = function() {
          var $a, self = this;

          if ((($a = self.$scope()['$class_scope?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return "\n\n" + (self.$current_indent())
            } else {
            return "\n" + (self.$current_indent())
          };
        };

        def['$child_is_expr?'] = function(child) {
          var $a, self = this;

          return ($a = self['$raw_expression?'](child), $a !== false && $a !== nil ?["stmt", "stmt_closure"]['$include?'](self.level) : $a);
        };

        def['$raw_expression?'] = function(child) {
          var self = this;

          return ["xstr", "dxstr"]['$include?'](child.$type())['$!']();
        };

        return (def.$find_inline_yield = function(stmt) {
          var $a, $b, TMP_3, $c, TMP_4, self = this, found = nil, $case = nil, arglist = nil;

          found = nil;
          $case = stmt.$first();if ("js_return"['$===']($case)) {if ((($a = found = self.$find_inline_yield(stmt['$[]'](1))) !== nil && (!$a.$$is_boolean || $a == true))) {
            found = found['$[]'](2)}}else if ("array"['$===']($case)) {($a = ($b = stmt['$[]']($range(1, -1, false))).$each_with_index, $a.$$p = (TMP_3 = function(el, idx){var self = TMP_3.$$s || this;
if (el == null) el = nil;if (idx == null) idx = nil;
          if (el.$first()['$==']("yield")) {
              found = el;
              return stmt['$[]=']($rb_plus(idx, 1), self.$s("js_tmp", "$yielded"));
              } else {
              return nil
            }}, TMP_3.$$s = self, TMP_3), $a).call($b)}else if ("call"['$===']($case)) {arglist = stmt['$[]'](3);
          ($a = ($c = arglist['$[]']($range(1, -1, false))).$each_with_index, $a.$$p = (TMP_4 = function(el, idx){var self = TMP_4.$$s || this;
if (el == null) el = nil;if (idx == null) idx = nil;
          if (el.$first()['$==']("yield")) {
              found = el;
              return arglist['$[]=']($rb_plus(idx, 1), self.$s("js_tmp", "$yielded"));
              } else {
              return nil
            }}, TMP_4.$$s = self, TMP_4), $a).call($c);};
          if (found !== false && found !== nil) {
            if ((($a = self.$scope()['$has_temp?']("$yielded")) !== nil && (!$a.$$is_boolean || $a == true))) {
              } else {
              self.$scope().$add_temp("$yielded")
            };
            return self.$s("yasgn", "$yielded", found);
            } else {
            return nil
          };
        }, nil) && 'find_inline_yield';
      })(self, $scope.get('Base'));
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/yield"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass, $range = Opal.range;

  Opal.add_stubs(['$require', '$find_yielding_scope', '$uses_block!', '$block_name', '$yields_single_arg?', '$push', '$expr', '$first', '$wrap', '$s', '$uses_splat?', '$scope', '$def?', '$parent', '$!', '$==', '$size', '$any?', '$type', '$handle', '$compile_call', '$children', '$stmt?', '$with_temp', '$[]', '$yield_args', '$var_name']);
  self.$require("opal/nodes/base");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $BaseYieldNode(){};
        var self = $BaseYieldNode = $klass($base, $super, 'BaseYieldNode', $BaseYieldNode);

        var def = self.$$proto, $scope = self.$$scope;

        def.$compile_call = function(children, level) {
          var $a, $b, self = this, yielding_scope = nil, block_name = nil;

          yielding_scope = self.$find_yielding_scope();
          yielding_scope['$uses_block!']();
          block_name = ((($a = yielding_scope.$block_name()) !== false && $a !== nil) ? $a : "$yield");
          if ((($a = self['$yields_single_arg?'](children)) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.$push(self.$expr(children.$first()));
            return self.$wrap("Opal.yield1(" + (block_name) + ", ", ")");
            } else {
            self.$push(self.$expr(($a = self).$s.apply($a, ["arglist"].concat(children))));
            if ((($b = self['$uses_splat?'](children)) !== nil && (!$b.$$is_boolean || $b == true))) {
              return self.$wrap("Opal.yieldX(" + (block_name) + ", ", ")")
              } else {
              return self.$wrap("Opal.yieldX(" + (block_name) + ", [", "])")
            };
          };
        };

        def.$find_yielding_scope = function() {
          var $a, $b, $c, self = this, working = nil;

          working = self.$scope();
          while (working !== false && working !== nil) {
          if ((($b = ((($c = working.$block_name()) !== false && $c !== nil) ? $c : working['$def?']())) !== nil && (!$b.$$is_boolean || $b == true))) {
            break;};
          working = working.$parent();};
          return working;
        };

        def['$yields_single_arg?'] = function(children) {
          var $a, self = this;

          return ($a = self['$uses_splat?'](children)['$!'](), $a !== false && $a !== nil ?children.$size()['$=='](1) : $a);
        };

        return (def['$uses_splat?'] = function(children) {
          var $a, $b, TMP_1, self = this;

          return ($a = ($b = children)['$any?'], $a.$$p = (TMP_1 = function(child){var self = TMP_1.$$s || this;
if (child == null) child = nil;
          return child.$type()['$==']("splat")}, TMP_1.$$s = self, TMP_1), $a).call($b);
        }, nil) && 'uses_splat?';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $YieldNode(){};
        var self = $YieldNode = $klass($base, $super, 'YieldNode', $YieldNode);

        var def = self.$$proto, $scope = self.$$scope;

        def.level = nil;
        self.$handle("yield");

        return (def.$compile = function() {
          var $a, $b, TMP_2, self = this;

          self.$compile_call(self.$children(), self.level);
          if ((($a = self['$stmt?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$wrap("if (", " === $breaker) return $breaker.$v")
            } else {
            return ($a = ($b = self).$with_temp, $a.$$p = (TMP_2 = function(tmp){var self = TMP_2.$$s || this;
if (tmp == null) tmp = nil;
            return self.$wrap("(((" + (tmp) + " = ", ") === $breaker) ? $breaker.$v : " + (tmp) + ")")}, TMP_2.$$s = self, TMP_2), $a).call($b)
          };
        }, nil) && 'compile';
      })(self, $scope.get('BaseYieldNode'));

      (function($base, $super) {
        function $YasgnNode(){};
        var self = $YasgnNode = $klass($base, $super, 'YasgnNode', $YasgnNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("yasgn");

        self.$children("var_name", "yield_args");

        return (def.$compile = function() {
          var $a, self = this;

          self.$compile_call(($a = self).$s.apply($a, [].concat(self.$yield_args()['$[]']($range(1, -1, false)))), "stmt");
          return self.$wrap("if ((" + (self.$var_name()) + " = ", ") === $breaker) return $breaker.$v");
        }, nil) && 'compile';
      })(self, $scope.get('BaseYieldNode'));

      (function($base, $super) {
        function $ReturnableYieldNode(){};
        var self = $ReturnableYieldNode = $klass($base, $super, 'ReturnableYieldNode', $ReturnableYieldNode);

        var def = self.$$proto, $scope = self.$$scope;

        def.level = nil;
        self.$handle("returnable_yield");

        return (def.$compile = function() {
          var $a, $b, TMP_3, self = this;

          self.$compile_call(self.$children(), self.level);
          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_3 = function(tmp){var self = TMP_3.$$s || this;
if (tmp == null) tmp = nil;
          return self.$wrap("return " + (tmp) + " = ", ", " + (tmp) + " === $breaker ? " + (tmp) + " : " + (tmp))}, TMP_3.$$s = self, TMP_3), $a).call($b);
        }, nil) && 'compile';
      })(self, $scope.get('BaseYieldNode'));
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/rescue"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass, $range = Opal.range;

  Opal.add_stubs(['$require', '$handle', '$children', '$stmt?', '$lhs', '$returns', '$compiler', '$rhs', '$push', '$expr', '$body', '$rescue_val', '$wrap', '$line', '$process', '$body_sexp', '$ensr_sexp', '$wrap_in_closure?', '$begn', '$ensr', '$s', '$recv?', '$expr?', '$indent', '$body_code', '$each_with_index', '$==', '$type', '$[]', '$!', '$empty?', '$rescue_exprs', '$rescue_variable', '$[]=', '$rescue_body', '$===', '$include?', '$rescue_variable?', '$last', '$args', '$dup', '$pop']);
  self.$require("opal/nodes/base");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $RescueModNode(){};
        var self = $RescueModNode = $klass($base, $super, 'RescueModNode', $RescueModNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("rescue_mod");

        self.$children("lhs", "rhs");

        def.$body = function() {
          var $a, self = this;

          if ((($a = self['$stmt?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$lhs()
            } else {
            return self.$compiler().$returns(self.$lhs())
          };
        };

        def.$rescue_val = function() {
          var $a, self = this;

          if ((($a = self['$stmt?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$rhs()
            } else {
            return self.$compiler().$returns(self.$rhs())
          };
        };

        return (def.$compile = function() {
          var $a, self = this;

          self.$push("try {", self.$expr(self.$body()), " } catch ($err) { ", self.$expr(self.$rescue_val()), " }");
          if ((($a = self['$stmt?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return nil
            } else {
            return self.$wrap("(function() {", "})()")
          };
        }, nil) && 'compile';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $EnsureNode(){};
        var self = $EnsureNode = $klass($base, $super, 'EnsureNode', $EnsureNode);

        var def = self.$$proto, $scope = self.$$scope;

        def.level = nil;
        self.$handle("ensure");

        self.$children("begn", "ensr");

        def.$compile = function() {
          var $a, self = this;

          self.$push("try {");
          self.$line(self.$compiler().$process(self.$body_sexp(), self.level));
          self.$line("} finally {");
          self.$line(self.$compiler().$process(self.$ensr_sexp(), self.level));
          self.$line("}");
          if ((($a = self['$wrap_in_closure?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$wrap("(function() {", "; })()")
            } else {
            return nil
          };
        };

        def.$body_sexp = function() {
          var $a, self = this;

          if ((($a = self['$wrap_in_closure?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$compiler().$returns(self.$begn())
            } else {
            return self.$begn()
          };
        };

        def.$ensr_sexp = function() {
          var $a, self = this;

          return ((($a = self.$ensr()) !== false && $a !== nil) ? $a : self.$s("nil"));
        };

        return (def['$wrap_in_closure?'] = function() {
          var $a, self = this;

          return ((($a = self['$recv?']()) !== false && $a !== nil) ? $a : self['$expr?']());
        }, nil) && 'wrap_in_closure?';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $RescueNode(){};
        var self = $RescueNode = $klass($base, $super, 'RescueNode', $RescueNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("rescue");

        self.$children("body");

        def.$compile = function() {
          var $a, $b, TMP_1, $c, TMP_2, self = this, handled_else = nil;

          handled_else = false;
          self.$push("try {");
          self.$line(($a = ($b = self).$indent, $a.$$p = (TMP_1 = function(){var self = TMP_1.$$s || this;
            if (self.level == null) self.level = nil;

          return self.$process(self.$body_code(), self.level)}, TMP_1.$$s = self, TMP_1), $a).call($b));
          self.$line("} catch ($err) {");
          ($a = ($c = self.$children()['$[]']($range(1, -1, false))).$each_with_index, $a.$$p = (TMP_2 = function(child, idx){var self = TMP_2.$$s || this, $a, $b, TMP_3;
if (child == null) child = nil;if (idx == null) idx = nil;
          if (child.$type()['$==']("resbody")) {
              } else {
              handled_else = true
            };
            if (idx['$=='](0)) {
              } else {
              self.$push("else ")
            };
            return self.$push(($a = ($b = self).$indent, $a.$$p = (TMP_3 = function(){var self = TMP_3.$$s || this;
              if (self.level == null) self.level = nil;

            return self.$process(child, self.level)}, TMP_3.$$s = self, TMP_3), $a).call($b));}, TMP_2.$$s = self, TMP_2), $a).call($c);
          if (handled_else !== false && handled_else !== nil) {
            } else {
            self.$push("else { throw $err; }")
          };
          self.$line("}");
          if ((($a = self['$expr?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$wrap("(function() { ", "})()")
            } else {
            return nil
          };
        };

        return (def.$body_code = function() {
          var $a, self = this, body_code = nil;

          body_code = ((function() {if (self.$body().$type()['$==']("resbody")) {
            return self.$s("nil")
            } else {
            return self.$body()
          }; return nil; })());
          if ((($a = self['$stmt?']()['$!']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$compiler().$returns(body_code)
            } else {
            return body_code
          };
        }, nil) && 'body_code';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $ResBodyNode(){};
        var self = $ResBodyNode = $klass($base, $super, 'ResBodyNode', $ResBodyNode);

        var def = self.$$proto, $scope = self.$$scope;

        def.level = nil;
        self.$handle("resbody");

        self.$children("args", "body");

        def.$compile = function() {
          var $a, $b, TMP_4, self = this, variable = nil;

          self.$push("if (");
          if ((($a = self.$rescue_exprs()['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.$push("true")
            } else {
            self.$push("Opal.rescue($err, [");
            ($a = ($b = self.$rescue_exprs()).$each_with_index, $a.$$p = (TMP_4 = function(rexpr, idx){var self = TMP_4.$$s || this;
if (rexpr == null) rexpr = nil;if (idx == null) idx = nil;
            if (idx['$=='](0)) {
                } else {
                self.$push(", ")
              };
              return self.$push(self.$expr(rexpr));}, TMP_4.$$s = self, TMP_4), $a).call($b);
            self.$push("])");
          };
          self.$push(") {");
          if ((($a = variable = self.$rescue_variable()) !== nil && (!$a.$$is_boolean || $a == true))) {
            variable['$[]='](2, self.$s("js_tmp", "$err"));
            self.$push(self.$expr(variable), ";");};
          self.$line(self.$process(self.$rescue_body(), self.level));
          return self.$line("}");
        };

        def['$rescue_variable?'] = function(variable) {
          var $a, self = this;

          return ($a = $scope.get('Sexp')['$==='](variable), $a !== false && $a !== nil ?["lasgn", "iasgn"]['$include?'](variable.$type()) : $a);
        };

        def.$rescue_variable = function() {
          var $a, self = this;

          if ((($a = self['$rescue_variable?'](self.$args().$last())) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$args().$last().$dup()
            } else {
            return nil
          };
        };

        def.$rescue_exprs = function() {
          var $a, self = this, exprs = nil;

          exprs = self.$args().$dup();
          if ((($a = self['$rescue_variable?'](exprs.$last())) !== nil && (!$a.$$is_boolean || $a == true))) {
            exprs.$pop()};
          return exprs.$children();
        };

        return (def.$rescue_body = function() {
          var $a, self = this, body_code = nil;

          body_code = (((($a = self.$body()) !== false && $a !== nil) ? $a : self.$s("nil")));
          if ((($a = self['$stmt?']()['$!']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            body_code = self.$compiler().$returns(body_code)};
          return body_code;
        }, nil) && 'rescue_body';
      })(self, $scope.get('Base'));
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/case"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass, $range = Opal.range;

  Opal.add_stubs(['$require', '$handle', '$children', '$in_case', '$condition', '$[]=', '$case_stmt', '$add_local', '$push', '$expr', '$each_with_index', '$==', '$type', '$needs_closure?', '$returns', '$compiler', '$stmt', '$case_parts', '$!', '$wrap', '$stmt?', '$[]', '$s', '$js_truthy', '$when_checks', '$process', '$body_code', '$whens', '$body']);
  self.$require("opal/nodes/base");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $CaseNode(){};
        var self = $CaseNode = $klass($base, $super, 'CaseNode', $CaseNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("case");

        self.$children("condition");

        def.$compile = function() {
          var $a, $b, TMP_1, self = this, handled_else = nil;

          handled_else = false;
          return ($a = ($b = self.$compiler()).$in_case, $a.$$p = (TMP_1 = function(){var self = TMP_1.$$s || this, $a, $b, TMP_2, $c;

          if ((($a = self.$condition()) !== nil && (!$a.$$is_boolean || $a == true))) {
              self.$case_stmt()['$[]=']("cond", true);
              self.$add_local("$case");
              self.$push("$case = ", self.$expr(self.$condition()), ";");};
            ($a = ($b = self.$case_parts()).$each_with_index, $a.$$p = (TMP_2 = function(wen, idx){var self = TMP_2.$$s || this, $a, $b;
if (wen == null) wen = nil;if (idx == null) idx = nil;
            if ((($a = (($b = wen !== false && wen !== nil) ? wen.$type()['$==']("when") : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
                if ((($a = self['$needs_closure?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
                  self.$compiler().$returns(wen)};
                if (idx['$=='](0)) {
                  } else {
                  self.$push("else ")
                };
                return self.$push(self.$stmt(wen));
              } else if (wen !== false && wen !== nil) {
                handled_else = true;
                if ((($a = self['$needs_closure?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
                  wen = self.$compiler().$returns(wen)};
                return self.$push("else {", self.$stmt(wen), "}");
                } else {
                return nil
              }}, TMP_2.$$s = self, TMP_2), $a).call($b);
            if ((($a = ($c = self['$needs_closure?'](), $c !== false && $c !== nil ?handled_else['$!']() : $c)) !== nil && (!$a.$$is_boolean || $a == true))) {
              self.$push("else { return nil }")};
            if ((($a = self['$needs_closure?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
              return self.$wrap("(function() {", "})()")
              } else {
              return nil
            };}, TMP_1.$$s = self, TMP_1), $a).call($b);
        };

        def['$needs_closure?'] = function() {
          var self = this;

          return self['$stmt?']()['$!']();
        };

        def.$case_parts = function() {
          var self = this;

          return self.$children()['$[]']($range(1, -1, false));
        };

        return (def.$case_stmt = function() {
          var self = this;

          return self.$compiler().$case_stmt();
        }, nil) && 'case_stmt';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $WhenNode(){};
        var self = $WhenNode = $klass($base, $super, 'WhenNode', $WhenNode);

        var def = self.$$proto, $scope = self.$$scope;

        def.level = nil;
        self.$handle("when");

        self.$children("whens", "body");

        def.$compile = function() {
          var $a, $b, TMP_3, self = this;

          self.$push("if (");
          ($a = ($b = self.$when_checks()).$each_with_index, $a.$$p = (TMP_3 = function(check, idx){var self = TMP_3.$$s || this, $a, call = nil;
if (check == null) check = nil;if (idx == null) idx = nil;
          if (idx['$=='](0)) {
              } else {
              self.$push(" || ")
            };
            if (check.$type()['$==']("splat")) {
              self.$push("(function($splt) { for (var i = 0; i < $splt.length; i++) {");
              self.$push("if ($splt[i]['$===']($case)) { return true; }");
              return self.$push("} return false; })(", self.$expr(check['$[]'](1)), ")");
            } else if ((($a = self.$case_stmt()['$[]']("cond")) !== nil && (!$a.$$is_boolean || $a == true))) {
              call = self.$s("call", check, "===", self.$s("arglist", self.$s("js_tmp", "$case")));
              return self.$push(self.$expr(call));
              } else {
              return self.$push(self.$js_truthy(check))
            };}, TMP_3.$$s = self, TMP_3), $a).call($b);
          return self.$push(") {", self.$process(self.$body_code(), self.level), "}");
        };

        def.$when_checks = function() {
          var self = this;

          return self.$whens().$children();
        };

        def.$case_stmt = function() {
          var self = this;

          return self.$compiler().$case_stmt();
        };

        return (def.$body_code = function() {
          var $a, self = this;

          return ((($a = self.$body()) !== false && $a !== nil) ? $a : self.$s("nil"));
        }, nil) && 'body_code';
      })(self, $scope.get('Base'));
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/super"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$children', '$arglist', '$iter', '$expr', '$iter_sexp', '$uses_block!', '$scope', '$def?', '$identify!', '$name', '$parent', '$defs', '$push', '$to_s', '$mid', '$iter?', '$get_super_chain', '$join', '$map', '$raise', '$s', '$handle', '$compile_dispatcher', '$wrap', '$has_splat?', '$args', '$fragment', '$uses_zuper=', '$any?', '$==', '$type']);
  self.$require("opal/nodes/base");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $BaseSuperNode(){};
        var self = $BaseSuperNode = $klass($base, $super, 'BaseSuperNode', $BaseSuperNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$children("arglist", "iter");

        def.$compile_dispatcher = function() {
          var $a, $b, TMP_1, self = this, iter = nil, scope_name = nil, class_name = nil, chain = nil, cur_defn = nil, mid = nil, trys = nil;

          if ((($a = ((($b = self.$arglist()) !== false && $b !== nil) ? $b : self.$iter())) !== nil && (!$a.$$is_boolean || $a == true))) {
            iter = self.$expr(self.$iter_sexp())
            } else {
            self.$scope()['$uses_block!']();
            iter = "$iter";
          };
          if ((($a = self.$scope()['$def?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.$scope()['$uses_block!']();
            scope_name = self.$scope()['$identify!']();
            class_name = (function() {if ((($a = self.$scope().$parent().$name()) !== nil && (!$a.$$is_boolean || $a == true))) {
              return "$" + (self.$scope().$parent().$name())
              } else {
              return "self.$$class.$$proto"
            }; return nil; })();
            if ((($a = self.$scope().$defs()) !== nil && (!$a.$$is_boolean || $a == true))) {
              self.$push("Opal.find_super_dispatcher(self, '" + (self.$scope().$mid().$to_s()) + "', " + (scope_name) + ", ");
              self.$push(iter);
              return self.$push(", " + (class_name) + ")");
              } else {
              self.$push("Opal.find_super_dispatcher(self, '" + (self.$scope().$mid().$to_s()) + "', " + (scope_name) + ", ");
              self.$push(iter);
              return self.$push(")");
            };
          } else if ((($a = self.$scope()['$iter?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            $a = Opal.to_ary(self.$scope().$get_super_chain()), chain = ($a[0] == null ? nil : $a[0]), cur_defn = ($a[1] == null ? nil : $a[1]), mid = ($a[2] == null ? nil : $a[2]);
            trys = ($a = ($b = chain).$map, $a.$$p = (TMP_1 = function(c){var self = TMP_1.$$s || this;
if (c == null) c = nil;
            return "" + (c) + ".$$def"}, TMP_1.$$s = self, TMP_1), $a).call($b).$join(" || ");
            return self.$push("Opal.find_iter_super_dispatcher(self, " + (mid) + ", (" + (trys) + " || " + (cur_defn) + "), null)");
            } else {
            return self.$raise("Cannot call super() from outside a method block")
          };
        };

        def.$args = function() {
          var $a, self = this;

          return ((($a = self.$arglist()) !== false && $a !== nil) ? $a : self.$s("arglist"));
        };

        return (def.$iter_sexp = function() {
          var $a, self = this;

          return ((($a = self.$iter()) !== false && $a !== nil) ? $a : self.$s("js_tmp", "null"));
        }, nil) && 'iter_sexp';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $DefinedSuperNode(){};
        var self = $DefinedSuperNode = $klass($base, $super, 'DefinedSuperNode', $DefinedSuperNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("defined_super");

        return (def.$compile = function() {
          var self = this;

          self.$compile_dispatcher();
          return self.$wrap("((", ") != null ? \"super\" : nil)");
        }, nil) && 'compile';
      })(self, $scope.get('BaseSuperNode'));

      (function($base, $super) {
        function $SuperNode(){};
        var self = $SuperNode = $klass($base, $super, 'SuperNode', $SuperNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("super");

        self.$children("arglist", "iter");

        def.$compile = function() {
          var $a, $b, self = this, splat = nil, args = nil;

          if ((($a = ((($b = self.$arglist()) !== false && $b !== nil) ? $b : self.$iter())) !== nil && (!$a.$$is_boolean || $a == true))) {
            splat = self['$has_splat?']();
            args = self.$expr(self.$args());
            if (splat !== false && splat !== nil) {
              } else {
              args = [self.$fragment("["), args, self.$fragment("]")]
            };
          } else if ((($a = self.$scope()['$def?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            (($a = [true]), $b = self.$scope(), $b['$uses_zuper='].apply($b, $a), $a[$a.length-1]);
            args = self.$fragment("$zuper");
            } else {
            args = self.$fragment("$slice.call(arguments)")
          };
          self.$compile_dispatcher();
          self.$push(".apply(self, ");
          ($a = self).$push.apply($a, [].concat(args));
          return self.$push(")");
        };

        return (def['$has_splat?'] = function() {
          var $a, $b, TMP_2, self = this;

          return ($a = ($b = self.$args().$children())['$any?'], $a.$$p = (TMP_2 = function(child){var self = TMP_2.$$s || this;
if (child == null) child = nil;
          return child.$type()['$==']("splat")}, TMP_2.$$s = self, TMP_2), $a).call($b);
        }, nil) && 'has_splat?';
      })(self, $scope.get('BaseSuperNode'));
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/version"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module;

  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    Opal.cdecl($scope, 'VERSION', "0.8.1")
  })(self)
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/top"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$handle', '$children', '$push', '$version_comment', '$opening', '$in_scope', '$line', '$inspect', '$to_s', '$dynamic_require_severity', '$compiler', '$stmt', '$stmts', '$is_a?', '$add_temp', '$add_used_helpers', '$add_used_operators', '$to_vars', '$scope', '$compile_method_stubs', '$compile_irb_vars', '$compile_end_construct', '$closing', '$requirable?', '$cleanpath', '$Pathname', '$file', '$returns', '$body', '$irb?', '$to_a', '$helpers', '$each', '$operator_helpers', '$[]', '$method_missing?', '$method_calls', '$join', '$map', '$empty?', '$eof_content']);
  self.$require("pathname");
  self.$require("opal/version");
  self.$require("opal/nodes/scope");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $TopNode(){};
        var self = $TopNode = $klass($base, $super, 'TopNode', $TopNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("top");

        self.$children("body");

        def.$compile = function() {
          var $a, $b, TMP_1, self = this;

          self.$push(self.$version_comment());
          self.$opening();
          ($a = ($b = self).$in_scope, $a.$$p = (TMP_1 = function(){var self = TMP_1.$$s || this, $a, body_code = nil;

          self.$line("Opal.dynamic_require_severity = " + (self.$compiler().$dynamic_require_severity().$to_s().$inspect()) + ";");
            body_code = self.$stmt(self.$stmts());
            if ((($a = body_code['$is_a?']($scope.get('Array'))) !== nil && (!$a.$$is_boolean || $a == true))) {
              } else {
              body_code = [body_code]
            };
            self.$add_temp("self = Opal.top");
            self.$add_temp("$scope = Opal");
            self.$add_temp("nil = Opal.nil");
            self.$add_used_helpers();
            self.$add_used_operators();
            self.$line(self.$scope().$to_vars());
            self.$compile_method_stubs();
            self.$compile_irb_vars();
            self.$compile_end_construct();
            return self.$line(body_code);}, TMP_1.$$s = self, TMP_1), $a).call($b);
          return self.$closing();
        };

        def.$opening = function() {
          var $a, self = this, path = nil;

          if ((($a = self.$compiler()['$requirable?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            path = self.$Pathname(self.$compiler().$file()).$cleanpath().$to_s();
            return self.$line("Opal.modules[" + (path.$inspect()) + "] = function(Opal) {");
            } else {
            return self.$line("(function(Opal) {")
          };
        };

        def.$closing = function() {
          var $a, self = this;

          if ((($a = self.$compiler()['$requirable?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$line("};\n")
            } else {
            return self.$line("})(Opal);\n")
          };
        };

        def.$stmts = function() {
          var self = this;

          return self.$compiler().$returns(self.$body());
        };

        def.$compile_irb_vars = function() {
          var $a, self = this;

          if ((($a = self.$compiler()['$irb?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$line("if (!Opal.irb_vars) { Opal.irb_vars = {}; }")
            } else {
            return nil
          };
        };

        def.$add_used_helpers = function() {
          var $a, $b, TMP_2, self = this, helpers = nil;

          helpers = self.$compiler().$helpers().$to_a();
          return ($a = ($b = helpers.$to_a()).$each, $a.$$p = (TMP_2 = function(h){var self = TMP_2.$$s || this;
if (h == null) h = nil;
          return self.$add_temp("$" + (h) + " = Opal." + (h))}, TMP_2.$$s = self, TMP_2), $a).call($b);
        };

        def.$add_used_operators = function() {
          var $a, $b, TMP_3, self = this, operators = nil;

          operators = self.$compiler().$operator_helpers().$to_a();
          return ($a = ($b = operators).$each, $a.$$p = (TMP_3 = function(op){var self = TMP_3.$$s || this, name = nil;
if (op == null) op = nil;
          name = (((($scope.get('Nodes')).$$scope.get('CallNode'))).$$scope.get('OPERATORS'))['$[]'](op);
            self.$line("function $rb_" + (name) + "(lhs, rhs) {");
            self.$line("  return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs " + (op) + " rhs : lhs['$" + (op) + "'](rhs);");
            return self.$line("}");}, TMP_3.$$s = self, TMP_3), $a).call($b);
        };

        def.$compile_method_stubs = function() {
          var $a, $b, TMP_4, self = this, calls = nil, stubs = nil;

          if ((($a = self.$compiler()['$method_missing?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            calls = self.$compiler().$method_calls();
            stubs = ($a = ($b = calls.$to_a()).$map, $a.$$p = (TMP_4 = function(k){var self = TMP_4.$$s || this;
if (k == null) k = nil;
            return "'$" + (k) + "'"}, TMP_4.$$s = self, TMP_4), $a).call($b).$join(", ");
            if ((($a = stubs['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
              return nil
              } else {
              return self.$line("Opal.add_stubs([" + (stubs) + "]);")
            };
            } else {
            return nil
          };
        };

        def.$compile_end_construct = function() {
          var $a, self = this, content = nil;

          if ((($a = content = self.$compiler().$eof_content()) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.$line("var $__END__ = Opal.Object.$new();");
            return self.$line("$__END__.$read = function() { return " + (content.$inspect()) + "; };");
            } else {
            return nil
          };
        };

        return (def.$version_comment = function() {
          var self = this;

          return "/* Generated by Opal " + ((($scope.get('Opal')).$$scope.get('VERSION'))) + " */";
        }, nil) && 'version_comment';
      })(self, $scope.get('ScopeNode'))
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/while"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$handle', '$children', '$with_temp', '$js_truthy', '$test', '$in_while', '$wrap_in_closure?', '$[]=', '$while_loop', '$stmt', '$body', '$uses_redo?', '$push', '$while_open', '$while_close', '$line', '$compiler', '$wrap', '$[]', '$expr?', '$recv?']);
  self.$require("opal/nodes/base");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $WhileNode(){};
        var self = $WhileNode = $klass($base, $super, 'WhileNode', $WhileNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("while");

        self.$children("test", "body");

        def.$compile = function() {
          var $a, $b, TMP_1, self = this;

          ($a = ($b = self).$with_temp, $a.$$p = (TMP_1 = function(redo_var){var self = TMP_1.$$s || this, $a, $b, TMP_2, test_code = nil;
if (redo_var == null) redo_var = nil;
          test_code = self.$js_truthy(self.$test());
            return ($a = ($b = self.$compiler()).$in_while, $a.$$p = (TMP_2 = function(){var self = TMP_2.$$s || this, $a, body_code = nil;

            if ((($a = self['$wrap_in_closure?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
                self.$while_loop()['$[]=']("closure", true)};
              self.$while_loop()['$[]=']("redo_var", redo_var);
              body_code = self.$stmt(self.$body());
              if ((($a = self['$uses_redo?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
                self.$push("" + (redo_var) + " = false; " + (self.$while_open()) + (redo_var) + " || ");
                self.$push(test_code);
                self.$push(self.$while_close());
                } else {
                self.$push(self.$while_open(), test_code, self.$while_close())
              };
              if ((($a = self['$uses_redo?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
                self.$push("" + (redo_var) + " = false;")};
              return self.$line(body_code, "}");}, TMP_2.$$s = self, TMP_2), $a).call($b);}, TMP_1.$$s = self, TMP_1), $a).call($b);
          if ((($a = self['$wrap_in_closure?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$wrap("(function() {", "; return nil; })()")
            } else {
            return nil
          };
        };

        def.$while_open = function() {
          var self = this;

          return "while (";
        };

        def.$while_close = function() {
          var self = this;

          return ") {";
        };

        def['$uses_redo?'] = function() {
          var self = this;

          return self.$while_loop()['$[]']("use_redo");
        };

        return (def['$wrap_in_closure?'] = function() {
          var $a, self = this;

          return ((($a = self['$expr?']()) !== false && $a !== nil) ? $a : self['$recv?']());
        }, nil) && 'wrap_in_closure?';
      })(self, $scope.get('Base'));

      (function($base, $super) {
        function $UntilNode(){};
        var self = $UntilNode = $klass($base, $super, 'UntilNode', $UntilNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("until");

        def.$while_open = function() {
          var self = this;

          return "while (!(";
        };

        return (def.$while_close = function() {
          var self = this;

          return ")) {";
        }, nil) && 'while_close';
      })(self, $scope.get('WhileNode'));
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/for"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$handle', '$children', '$with_temp', '$==', '$type', '$args_sexp', '$s', '$<<', '$body_sexp', '$first', '$insert', '$value', '$push', '$expr']);
  self.$require("opal/nodes/base");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $ForNode(){};
        var self = $ForNode = $klass($base, $super, 'ForNode', $ForNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("for");

        self.$children("value", "args_sexp", "body_sexp");

        return (def.$compile = function() {
          var $a, $b, TMP_1, self = this;

          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_1 = function(loop_var){var self = TMP_1.$$s || this, $a, assign = nil, iter = nil, sexp = nil;
if (loop_var == null) loop_var = nil;
          if (self.$args_sexp().$type()['$==']("array")) {
              assign = self.$s("masgn", self.$args_sexp());
              assign['$<<'](self.$s("to_ary", self.$s("js_tmp", loop_var)));
              } else {
              assign = self.$args_sexp()['$<<'](self.$s("js_tmp", loop_var))
            };
            if ((($a = self.$body_sexp()) !== nil && (!$a.$$is_boolean || $a == true))) {
              if (self.$body_sexp().$first()['$==']("block")) {
                self.$body_sexp().$insert(1, assign);
                assign = self.$body_sexp();
                } else {
                assign = self.$s("block", assign, self.$body_sexp())
              }};
            iter = self.$s("iter", self.$s("lasgn", loop_var), assign);
            sexp = self.$s("call", self.$value(), "each", self.$s("arglist"), iter);
            return self.$push(self.$expr(sexp));}, TMP_1.$$s = self, TMP_1), $a).call($b);
        }, nil) && 'compile';
      })(self, $scope.get('Base'))
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/hash"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$handle', '$each_with_index', '$even?', '$<<', '$children', '$all?', '$include?', '$type', '$keys_and_values', '$simple_keys?', '$compile_hash2', '$compile_hash', '$helper', '$==', '$push', '$expr', '$wrap', '$times', '$inspect', '$to_s', '$[]', '$[]=', '$size', '$join']);
  self.$require("opal/nodes/base");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $HashNode(){};
        var self = $HashNode = $klass($base, $super, 'HashNode', $HashNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("hash");

        def.$keys_and_values = function() {
          var $a, $b, TMP_1, self = this, keys = nil, values = nil;

          $a = [[], []], keys = $a[0], values = $a[1];
          ($a = ($b = self.$children()).$each_with_index, $a.$$p = (TMP_1 = function(obj, idx){var self = TMP_1.$$s || this, $a;
if (obj == null) obj = nil;if (idx == null) idx = nil;
          if ((($a = idx['$even?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
              return keys['$<<'](obj)
              } else {
              return values['$<<'](obj)
            }}, TMP_1.$$s = self, TMP_1), $a).call($b);
          return [keys, values];
        };

        def['$simple_keys?'] = function(keys) {
          var $a, $b, TMP_2, self = this;

          return ($a = ($b = keys)['$all?'], $a.$$p = (TMP_2 = function(key){var self = TMP_2.$$s || this;
if (key == null) key = nil;
          return ["sym", "str"]['$include?'](key.$type())}, TMP_2.$$s = self, TMP_2), $a).call($b);
        };

        def.$compile = function() {
          var $a, self = this, keys = nil, values = nil;

          $a = Opal.to_ary(self.$keys_and_values()), keys = ($a[0] == null ? nil : $a[0]), values = ($a[1] == null ? nil : $a[1]);
          if ((($a = self['$simple_keys?'](keys)) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$compile_hash2(keys, values)
            } else {
            return self.$compile_hash()
          };
        };

        def.$compile_hash = function() {
          var $a, $b, TMP_3, self = this;

          self.$helper("hash");
          ($a = ($b = self.$children()).$each_with_index, $a.$$p = (TMP_3 = function(child, idx){var self = TMP_3.$$s || this;
if (child == null) child = nil;if (idx == null) idx = nil;
          if (idx['$=='](0)) {
              } else {
              self.$push(", ")
            };
            return self.$push(self.$expr(child));}, TMP_3.$$s = self, TMP_3), $a).call($b);
          return self.$wrap("$hash(", ")");
        };

        return (def.$compile_hash2 = function(keys, values) {
          var $a, $b, TMP_4, $c, TMP_5, self = this, hash_obj = nil, hash_keys = nil;

          $a = [$hash2([], {}), []], hash_obj = $a[0], hash_keys = $a[1];
          self.$helper("hash2");
          ($a = ($b = keys.$size()).$times, $a.$$p = (TMP_4 = function(idx){var self = TMP_4.$$s || this, $a, key = nil;
if (idx == null) idx = nil;
          key = keys['$[]'](idx)['$[]'](1).$to_s().$inspect();
            if ((($a = hash_obj['$include?'](key)) !== nil && (!$a.$$is_boolean || $a == true))) {
              } else {
              hash_keys['$<<'](key)
            };
            return hash_obj['$[]='](key, self.$expr(values['$[]'](idx)));}, TMP_4.$$s = self, TMP_4), $a).call($b);
          ($a = ($c = hash_keys).$each_with_index, $a.$$p = (TMP_5 = function(key, idx){var self = TMP_5.$$s || this;
if (key == null) key = nil;if (idx == null) idx = nil;
          if (idx['$=='](0)) {
              } else {
              self.$push(", ")
            };
            self.$push("" + (key) + ": ");
            return self.$push(hash_obj['$[]'](key));}, TMP_5.$$s = self, TMP_5), $a).call($c);
          return self.$wrap("$hash2([" + (hash_keys.$join(", ")) + "], {", "})");
        }, nil) && 'compile_hash2';
      })(self, $scope.get('Base'))
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/array"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$handle', '$empty?', '$children', '$push', '$each', '$==', '$type', '$expr', '$<<', '$fragment']);
  self.$require("opal/nodes/base");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $ArrayNode(){};
        var self = $ArrayNode = $klass($base, $super, 'ArrayNode', $ArrayNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("array");

        return (def.$compile = function() {
          var $a, $b, TMP_1, self = this, code = nil, work = nil, join = nil;

          if ((($a = self.$children()['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$push("[]")};
          $a = [[], []], code = $a[0], work = $a[1];
          ($a = ($b = self.$children()).$each, $a.$$p = (TMP_1 = function(child){var self = TMP_1.$$s || this, $a, splat = nil, part = nil;
if (child == null) child = nil;
          splat = child.$type()['$==']("splat");
            part = self.$expr(child);
            if (splat !== false && splat !== nil) {
              if ((($a = work['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
                if ((($a = code['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
                  code['$<<'](self.$fragment("[].concat("))['$<<'](part)['$<<'](self.$fragment(")"))
                  } else {
                  code['$<<'](self.$fragment(".concat("))['$<<'](part)['$<<'](self.$fragment(")"))
                }
                } else {
                if ((($a = code['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
                  code['$<<'](self.$fragment("["))['$<<'](work)['$<<'](self.$fragment("]"))
                  } else {
                  code['$<<'](self.$fragment(".concat(["))['$<<'](work)['$<<'](self.$fragment("])"))
                };
                code['$<<'](self.$fragment(".concat("))['$<<'](part)['$<<'](self.$fragment(")"));
              };
              return work = [];
              } else {
              if ((($a = work['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
                } else {
                work['$<<'](self.$fragment(", "))
              };
              return work['$<<'](part);
            };}, TMP_1.$$s = self, TMP_1), $a).call($b);
          if ((($a = work['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            } else {
            join = [self.$fragment("["), work, self.$fragment("]")];
            if ((($a = code['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
              code = join
              } else {
              code.$push([self.$fragment(".concat("), join, self.$fragment(")")])
            };
          };
          return self.$push(code);
        }, nil) && 'compile';
      })(self, $scope.get('Base'))
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/defined"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass, $range = Opal.range;

  Opal.add_stubs(['$require', '$handle', '$children', '$type', '$value', '$===', '$push', '$inspect', '$to_s', '$expr', '$s', '$[]', '$respond_to?', '$__send__', '$mid_to_jsid', '$with_temp', '$handle_block_given_call', '$compiler', '$wrap', '$include?']);
  self.$require("opal/nodes/base");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $DefinedNode(){};
        var self = $DefinedNode = $klass($base, $super, 'DefinedNode', $DefinedNode);

        var def = self.$$proto, $scope = self.$$scope;

        def.sexp = nil;
        self.$handle("defined");

        self.$children("value");

        def.$compile = function() {
          var $a, self = this, type = nil, $case = nil;

          type = self.$value().$type();
          return (function() {$case = type;if ("self"['$===']($case) || "nil"['$===']($case) || "false"['$===']($case) || "true"['$===']($case)) {return self.$push(type.$to_s().$inspect())}else if ("lasgn"['$===']($case) || "iasgn"['$===']($case) || "gasgn"['$===']($case) || "cvdecl"['$===']($case) || "masgn"['$===']($case) || "op_asgn_or"['$===']($case) || "op_asgn_and"['$===']($case)) {return self.$push("'assignment'")}else if ("paren"['$===']($case) || "not"['$===']($case)) {return self.$push(self.$expr(self.$s("defined", self.$value()['$[]'](1))))}else if ("lvar"['$===']($case)) {return self.$push("'local-variable'")}else {if ((($a = self['$respond_to?']("compile_" + (type))) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$__send__("compile_" + (type))
            } else {
            return self.$push("'expression'")
          }}})();
        };

        def.$compile_call = function() {
          var $a, $b, TMP_1, self = this, mid = nil, recv = nil;

          mid = self.$mid_to_jsid(self.$value()['$[]'](2).$to_s());
          recv = (function() {if ((($a = self.$value()['$[]'](1)) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$expr(self.$value()['$[]'](1))
            } else {
            return "self"
          }; return nil; })();
          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_1 = function(tmp){var self = TMP_1.$$s || this;
if (tmp == null) tmp = nil;
          self.$push("(((" + (tmp) + " = ", recv, "" + (mid) + ") && !" + (tmp) + ".$$stub) || ", recv);
            return self.$push("['$respond_to_missing?']('" + (self.$value()['$[]'](2).$to_s()) + "') ? 'method' : nil)");}, TMP_1.$$s = self, TMP_1), $a).call($b);
        };

        def.$compile_ivar = function() {
          var $a, $b, TMP_2, self = this;

          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_2 = function(tmp){var self = TMP_2.$$s || this, name = nil;
if (tmp == null) tmp = nil;
          name = self.$value()['$[]'](1).$to_s()['$[]']($range(1, -1, false));
            self.$push("((" + (tmp) + " = self['" + (name) + "'], " + (tmp) + " != null && " + (tmp) + " !== nil) ? ");
            return self.$push("'instance-variable' : nil)");}, TMP_2.$$s = self, TMP_2), $a).call($b);
        };

        def.$compile_super = function() {
          var self = this;

          return self.$push(self.$expr(self.$s("defined_super", self.$value())));
        };

        def.$compile_yield = function() {
          var self = this;

          self.$push(self.$compiler().$handle_block_given_call(self.sexp));
          return self.$wrap("((", ") != null ? \"yield\" : nil)");
        };

        def.$compile_xstr = function() {
          var self = this;

          self.$push(self.$expr(self.$value()));
          return self.$wrap("(typeof(", ") !== \"undefined\")");
        };

        Opal.defn(self, '$compile_dxstr', def.$compile_xstr);

        def.$compile_const = function() {
          var self = this;

          return self.$push("($scope." + (self.$value()['$[]'](1)) + " != null)");
        };

        def.$compile_colon2 = function() {
          var self = this;

          self.$push("(function(){ try { return ((");
          self.$push(self.$expr(self.$value()));
          self.$push(") != null ? 'constant' : nil); } catch (err) { if (err.$$class");
          return self.$push(" === Opal.NameError) { return nil; } else { throw(err); }}; })()");
        };

        def.$compile_colon3 = function() {
          var self = this;

          return self.$push("(Opal.Object.$$scope." + (self.$value()['$[]'](1)) + " == null ? nil : 'constant')");
        };

        def.$compile_cvar = function() {
          var self = this;

          return self.$push("(Opal.cvars['" + (self.$value()['$[]'](1)) + "'] != null ? 'class variable' : nil)");
        };

        def.$compile_gvar = function() {
          var $a, $b, TMP_3, self = this, name = nil;

          name = self.$value()['$[]'](1).$to_s()['$[]']($range(1, -1, false));
          if ((($a = ["~", "!"]['$include?'](name)) !== nil && (!$a.$$is_boolean || $a == true))) {
            return self.$push("'global-variable'")
          } else if ((($a = ["`", "'", "+", "&"]['$include?'](name)) !== nil && (!$a.$$is_boolean || $a == true))) {
            return ($a = ($b = self).$with_temp, $a.$$p = (TMP_3 = function(tmp){var self = TMP_3.$$s || this;
if (tmp == null) tmp = nil;
            self.$push("((" + (tmp) + " = $gvars['~'], " + (tmp) + " != null && " + (tmp) + " !== nil) ? ");
              return self.$push("'global-variable' : nil)");}, TMP_3.$$s = self, TMP_3), $a).call($b)
            } else {
            return self.$push("($gvars[" + (name.$inspect()) + "] != null ? 'global-variable' : nil)")
          };
        };

        return (def.$compile_nth_ref = function() {
          var $a, $b, TMP_4, self = this;

          return ($a = ($b = self).$with_temp, $a.$$p = (TMP_4 = function(tmp){var self = TMP_4.$$s || this;
if (tmp == null) tmp = nil;
          self.$push("((" + (tmp) + " = $gvars['~'], " + (tmp) + " != null && " + (tmp) + " != nil) ? ");
            return self.$push("'global-variable' : nil)");}, TMP_4.$$s = self, TMP_4), $a).call($b);
        }, nil) && 'compile_nth_ref';
      })(self, $scope.get('Base'))
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/masgn"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_minus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs - rhs : lhs['$-'](rhs);
  }
  function $rb_ge(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs >= rhs : lhs['$>='](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$handle', '$children', '$new_temp', '$scope', '$==', '$type', '$rhs', '$size', '$push', '$expr', '$[]', '$raise', '$each_with_index', '$dup', '$<<', '$s', '$[]=', '$to_sym', '$last', '$lhs', '$queue_temp']);
  self.$require("opal/nodes/base");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $MassAssignNode(){};
        var self = $MassAssignNode = $klass($base, $super, 'MassAssignNode', $MassAssignNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("masgn");

        self.$children("lhs", "rhs");

        return (def.$compile = function() {
          var $a, $b, TMP_1, self = this, tmp = nil, len = nil;

          tmp = self.$scope().$new_temp();
          len = 0;
          if (self.$rhs().$type()['$==']("array")) {
            len = $rb_minus(self.$rhs().$size(), 1);
            self.$push("" + (tmp) + " = ", self.$expr(self.$rhs()));
          } else if (self.$rhs().$type()['$==']("to_ary")) {
            self.$push("" + (tmp) + " = Opal.to_ary(", self.$expr(self.$rhs()['$[]'](1)), ")")
          } else if (self.$rhs().$type()['$==']("splat")) {
            self.$push("(" + (tmp) + " = ", self.$expr(self.$rhs()['$[]'](1)), ")['$to_a'] && !" + (tmp) + "['$to_a'].$$stub ? (" + (tmp) + " = " + (tmp) + "['$to_a']())");
            self.$push(" : (" + (tmp) + ").$$is_array ? " + (tmp) + " : (" + (tmp) + " = [" + (tmp) + "])");
            } else {
            self.$raise("unsupported mlhs type")
          };
          ($a = ($b = self.$lhs().$children()).$each_with_index, $a.$$p = (TMP_1 = function(child, idx){var self = TMP_1.$$s || this, $a, $b, $c, $d, part = nil, assign = nil;
if (child == null) child = nil;if (idx == null) idx = nil;
          self.$push(", ");
            if (child.$type()['$==']("splat")) {
              if ((($a = part = child['$[]'](1)) !== nil && (!$a.$$is_boolean || $a == true))) {
                part = part.$dup();
                part['$<<'](self.$s("js_tmp", "$slice.call(" + (tmp) + ", " + (idx) + ")"));
                return self.$push(self.$expr(part));
                } else {
                return nil
              }
              } else {
              if ($rb_ge(idx, len)) {
                assign = self.$s("js_tmp", "(" + (tmp) + "[" + (idx) + "] == null ? nil : " + (tmp) + "[" + (idx) + "])")
                } else {
                assign = self.$s("js_tmp", "" + (tmp) + "[" + (idx) + "]")
              };
              part = child.$dup();
              if ((($a = ((($b = ((($c = ((($d = child.$type()['$==']("lasgn")) !== false && $d !== nil) ? $d : child.$type()['$==']("iasgn"))) !== false && $c !== nil) ? $c : child.$type()['$==']("lvar"))) !== false && $b !== nil) ? $b : child.$type()['$==']("gasgn"))) !== nil && (!$a.$$is_boolean || $a == true))) {
                part['$<<'](assign)
              } else if (child.$type()['$==']("call")) {
                part['$[]='](2, ((("") + (part['$[]'](2))) + "=").$to_sym());
                part.$last()['$<<'](assign);
              } else if (child.$type()['$==']("attrasgn")) {
                part.$last()['$<<'](assign)
                } else {
                self.$raise("Bad lhs for masgn")
              };
              return self.$push(self.$expr(part));
            };}, TMP_1.$$s = self, TMP_1), $a).call($b);
          return self.$scope().$queue_temp(tmp);
        }, nil) && 'compile';
      })(self, $scope.get('Base'))
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes/arglist"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$handle', '$each', '$==', '$first', '$expr', '$empty?', '$<<', '$fragment', '$children', '$push']);
  self.$require("opal/nodes/base");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'Nodes');

      var def = self.$$proto, $scope = self.$$scope;

      (function($base, $super) {
        function $ArglistNode(){};
        var self = $ArglistNode = $klass($base, $super, 'ArglistNode', $ArglistNode);

        var def = self.$$proto, $scope = self.$$scope;

        self.$handle("arglist");

        return (def.$compile = function() {
          var $a, $b, TMP_1, self = this, code = nil, work = nil, join = nil;

          $a = [[], []], code = $a[0], work = $a[1];
          ($a = ($b = self.$children()).$each, $a.$$p = (TMP_1 = function(current){var self = TMP_1.$$s || this, $a, splat = nil, arg = nil;
if (current == null) current = nil;
          splat = current.$first()['$==']("splat");
            arg = self.$expr(current);
            if (splat !== false && splat !== nil) {
              if ((($a = work['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
                if ((($a = code['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
                  code['$<<'](self.$fragment("[].concat("));
                  code['$<<'](arg);
                  code['$<<'](self.$fragment(")"));
                  } else {
                  code = $rb_plus(code, ".concat(" + (arg) + ")")
                }
                } else {
                if ((($a = code['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
                  code['$<<']([self.$fragment("["), work, self.$fragment("]")])
                  } else {
                  code['$<<']([self.$fragment(".concat(["), work, self.$fragment("])")])
                };
                code['$<<']([self.$fragment(".concat("), arg, self.$fragment(")")]);
              };
              return work = [];
              } else {
              if ((($a = work['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
                } else {
                work['$<<'](self.$fragment(", "))
              };
              return work['$<<'](arg);
            };}, TMP_1.$$s = self, TMP_1), $a).call($b);
          if ((($a = work['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            } else {
            join = work;
            if ((($a = code['$empty?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
              code = join
              } else {
              code['$<<'](self.$fragment(".concat("))['$<<'](join)['$<<'](self.$fragment(")"))
            };
          };
          return ($a = self).$push.apply($a, [].concat(code));
        }, nil) && 'compile';
      })(self, $scope.get('Base'))
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/nodes"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice;

  Opal.add_stubs(['$require']);
  self.$require("opal/nodes/base");
  self.$require("opal/nodes/literal");
  self.$require("opal/nodes/variables");
  self.$require("opal/nodes/constants");
  self.$require("opal/nodes/call");
  self.$require("opal/nodes/call_special");
  self.$require("opal/nodes/module");
  self.$require("opal/nodes/class");
  self.$require("opal/nodes/singleton_class");
  self.$require("opal/nodes/iter");
  self.$require("opal/nodes/def");
  self.$require("opal/nodes/if");
  self.$require("opal/nodes/logic");
  self.$require("opal/nodes/definitions");
  self.$require("opal/nodes/yield");
  self.$require("opal/nodes/rescue");
  self.$require("opal/nodes/case");
  self.$require("opal/nodes/super");
  self.$require("opal/nodes/top");
  self.$require("opal/nodes/while");
  self.$require("opal/nodes/for");
  self.$require("opal/nodes/hash");
  self.$require("opal/nodes/array");
  self.$require("opal/nodes/defined");
  self.$require("opal/nodes/masgn");
  return self.$require("opal/nodes/arglist");
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/compiler"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  function $rb_gt(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs > rhs : lhs['$>'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $hash2 = Opal.hash2, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$compile', '$new', '$[]', '$define_method', '$fetch', '$!', '$include?', '$raise', '$inspect', '$compiler_option', '$attr_reader', '$attr_accessor', '$s', '$parse', '$file', '$eof_content', '$lexer', '$flatten', '$process', '$join', '$map', '$to_proc', '$message', '$class', '$backtrace', '$warn', '$<<', '$helpers', '$new_temp', '$queue_temp', '$push_while', '$pop_while', '$in_while?', '$==', '$fragment', '$handlers', '$type', '$compile_to_fragments', '$returns', '$===', '$[]=', '$length', '$=~', '$tap', '$source=', '$source', '$uses_block!', '$block_name', '$find_parent_def']);
  self.$require("set");
  self.$require("opal/parser");
  self.$require("opal/fragment");
  self.$require("opal/nodes");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    Opal.defs(self, '$compile', function(source, options) {
      var self = this;

      if (options == null) {
        options = $hash2([], {})
      }
      return $scope.get('Compiler').$new(source, options).$compile();
    });

    (function($base, $super) {
      function $Compiler(){};
      var self = $Compiler = $klass($base, $super, 'Compiler', $Compiler);

      var def = self.$$proto, $scope = self.$$scope, TMP_3, TMP_4, TMP_5, TMP_6;

      def.parser = def.source = def.sexp = def.fragments = def.helpers = def.operator_helpers = def.method_calls = def.indent = def.unique = def.scope = def.case_stmt = def.handlers = def.requires = def.required_trees = nil;
      Opal.cdecl($scope, 'INDENT', "  ");

      Opal.cdecl($scope, 'COMPARE', ["<", ">", "<=", ">="]);

      Opal.defs(self, '$compiler_option', function(name, default_value, options) {
        var $a, $b, TMP_1, $c, self = this, mid = nil, valid_values = nil;

        if (options == null) {
          options = $hash2([], {})
        }
        mid = options['$[]']("as");
        valid_values = options['$[]']("valid_values");
        return ($a = ($b = self).$define_method, $a.$$p = (TMP_1 = function(){var self = TMP_1.$$s || this, $a, $b, TMP_2, $c, value = nil;
          if (self.options == null) self.options = nil;

        value = ($a = ($b = self.options).$fetch, $a.$$p = (TMP_2 = function(){var self = TMP_2.$$s || this;

          return default_value}, TMP_2.$$s = self, TMP_2), $a).call($b, name);
          if ((($a = (($c = valid_values !== false && valid_values !== nil) ? (valid_values['$include?'](value))['$!']() : $c)) !== nil && (!$a.$$is_boolean || $a == true))) {
            self.$raise($scope.get('ArgumentError'), $rb_plus("invalid value " + (value.$inspect()) + " for option " + (name.$inspect()) + " ", "(valid values: " + (valid_values.$inspect()) + ")"))};
          return value;}, TMP_1.$$s = self, TMP_1), $a).call($b, ((($c = mid) !== false && $c !== nil) ? $c : name));
      });

      self.$compiler_option("file", "(file)");

      self.$compiler_option("method_missing", true, $hash2(["as"], {"as": "method_missing?"}));

      self.$compiler_option("arity_check", false, $hash2(["as"], {"as": "arity_check?"}));

      self.$compiler_option("irb", false, $hash2(["as"], {"as": "irb?"}));

      self.$compiler_option("dynamic_require_severity", "error", $hash2(["valid_values"], {"valid_values": ["error", "warning", "ignore"]}));

      self.$compiler_option("requirable", false, $hash2(["as"], {"as": "requirable?"}));

      self.$compiler_option("inline_operators", true, $hash2(["as"], {"as": "inline_operators?"}));

      self.$attr_reader("result");

      self.$attr_reader("fragments");

      self.$attr_accessor("scope");

      self.$attr_reader("case_stmt");

      self.$attr_reader("eof_content");

      def.$initialize = function(source, options) {
        var self = this;

        if (options == null) {
          options = $hash2([], {})
        }
        self.source = source;
        self.indent = "";
        self.unique = 0;
        return self.options = options;
      };

      def.$compile = function() {
        var $a, $b, self = this, error = nil, message = nil;

        try {
        self.parser = $scope.get('Parser').$new();
          self.sexp = self.$s("top", ((($a = self.parser.$parse(self.source, self.$file())) !== false && $a !== nil) ? $a : self.$s("nil")));
          self.eof_content = self.parser.$lexer().$eof_content();
          self.fragments = self.$process(self.sexp).$flatten();
          return self.result = ($a = ($b = self.fragments).$map, $a.$$p = "code".$to_proc(), $a).call($b).$join("");
        } catch ($err) {if (true) {error = $err;
          message = "An error occurred while compiling: " + (self.$file()) + "\n" + (error.$message());
          return self.$raise(error.$class(), message, error.$backtrace());
          }else { throw $err; }
        };
      };

      def.$source_map = function(source_file) {
        var $a, self = this;

        if (source_file == null) {
          source_file = nil
        }
        return (($scope.get('Opal')).$$scope.get('SourceMap')).$new(self.fragments, ((($a = source_file) !== false && $a !== nil) ? $a : self.$file()));
      };

      def.$helpers = function() {
        var $a, self = this;

        return ((($a = self.helpers) !== false && $a !== nil) ? $a : self.helpers = $scope.get('Set').$new(["breaker", "slice"]));
      };

      def.$operator_helpers = function() {
        var $a, self = this;

        return ((($a = self.operator_helpers) !== false && $a !== nil) ? $a : self.operator_helpers = $scope.get('Set').$new());
      };

      def.$method_calls = function() {
        var $a, self = this;

        return ((($a = self.method_calls) !== false && $a !== nil) ? $a : self.method_calls = $scope.get('Set').$new());
      };

      def.$error = function(msg, line) {
        var self = this;

        if (line == null) {
          line = nil
        }
        return self.$raise($scope.get('SyntaxError'), "" + (msg) + " :" + (self.$file()) + ":" + (line));
      };

      def.$warning = function(msg, line) {
        var self = this;

        if (line == null) {
          line = nil
        }
        return self.$warn("WARNING: " + (msg) + " -- " + (self.$file()) + ":" + (line));
      };

      def.$parser_indent = function() {
        var self = this;

        return self.indent;
      };

      def.$s = function(parts) {
        var self = this;

        parts = $slice.call(arguments, 0);
        return $scope.get('Sexp').$new(parts);
      };

      def.$fragment = function(str, sexp) {
        var self = this;

        if (sexp == null) {
          sexp = nil
        }
        return $scope.get('Fragment').$new(str, sexp);
      };

      def.$unique_temp = function() {
        var self = this;

        return "TMP_" + (self.unique = $rb_plus(self.unique, 1));
      };

      def.$helper = function(name) {
        var self = this;

        return self.$helpers()['$<<'](name);
      };

      def.$indent = TMP_3 = function() {
        var $a, self = this, $iter = TMP_3.$$p, block = $iter || nil, indent = nil, res = nil;

        TMP_3.$$p = null;
        indent = self.indent;
        self.indent = $rb_plus(self.indent, $scope.get('INDENT'));
        self.space = "\n" + (self.indent);
        res = ((($a = Opal.yieldX(block, [])) === $breaker) ? $breaker.$v : $a);
        self.indent = indent;
        self.space = "\n" + (self.indent);
        return res;
      };

      def.$with_temp = TMP_4 = function() {
        var $a, self = this, $iter = TMP_4.$$p, block = $iter || nil, tmp = nil, res = nil;

        TMP_4.$$p = null;
        tmp = self.scope.$new_temp();
        res = ((($a = Opal.yield1(block, tmp)) === $breaker) ? $breaker.$v : $a);
        self.scope.$queue_temp(tmp);
        return res;
      };

      def.$in_while = TMP_5 = function() {
        var $a, self = this, $iter = TMP_5.$$p, $yield = $iter || nil, result = nil;

        TMP_5.$$p = null;
        if (($yield !== nil)) {
          } else {
          return nil
        };
        self.while_loop = self.scope.$push_while();
        result = ((($a = Opal.yieldX($yield, [])) === $breaker) ? $breaker.$v : $a);
        self.scope.$pop_while();
        return result;
      };

      def.$in_case = TMP_6 = function() {
        var self = this, $iter = TMP_6.$$p, $yield = $iter || nil, old = nil;

        TMP_6.$$p = null;
        if (($yield !== nil)) {
          } else {
          return nil
        };
        old = self.case_stmt;
        self.case_stmt = $hash2([], {});
        if (Opal.yieldX($yield, []) === $breaker) return $breaker.$v;
        return self.case_stmt = old;
      };

      def['$in_while?'] = function() {
        var self = this;

        return self.scope['$in_while?']();
      };

      def.$process = function(sexp, level) {
        var $a, self = this, handler = nil;

        if (level == null) {
          level = "expr"
        }
        if (sexp['$=='](nil)) {
          return self.$fragment("")};
        if ((($a = handler = self.$handlers()['$[]'](sexp.$type())) !== nil && (!$a.$$is_boolean || $a == true))) {
          return handler.$new(sexp, level, self).$compile_to_fragments()
          } else {
          return self.$raise("Unsupported sexp: " + (sexp.$type()))
        };
      };

      def.$handlers = function() {
        var $a, self = this;

        return ((($a = self.handlers) !== false && $a !== nil) ? $a : self.handlers = (((($scope.get('Opal')).$$scope.get('Nodes'))).$$scope.get('Base')).$handlers());
      };

      def.$requires = function() {
        var $a, self = this;

        return ((($a = self.requires) !== false && $a !== nil) ? $a : self.requires = []);
      };

      def.$required_trees = function() {
        var $a, self = this;

        return ((($a = self.required_trees) !== false && $a !== nil) ? $a : self.required_trees = []);
      };

      def.$returns = function(sexp) {
        var $a, $b, TMP_7, self = this, $case = nil;

        if (sexp !== false && sexp !== nil) {
          } else {
          return self.$returns(self.$s("nil"))
        };
        return (function() {$case = sexp.$type();if ("break"['$===']($case) || "next"['$===']($case) || "redo"['$===']($case)) {return sexp}else if ("yield"['$===']($case)) {sexp['$[]='](0, "returnable_yield");
        return sexp;}else if ("scope"['$===']($case)) {sexp['$[]='](1, self.$returns(sexp['$[]'](1)));
        return sexp;}else if ("block"['$===']($case)) {if ($rb_gt(sexp.$length(), 1)) {
          sexp['$[]='](-1, self.$returns(sexp['$[]'](-1)))
          } else {
          sexp['$<<'](self.$returns(self.$s("nil")))
        };
        return sexp;}else if ("when"['$===']($case)) {sexp['$[]='](2, self.$returns(sexp['$[]'](2)));
        return sexp;}else if ("rescue"['$===']($case)) {sexp['$[]='](1, self.$returns(sexp['$[]'](1)));
        if ((($a = ($b = sexp['$[]'](2), $b !== false && $b !== nil ?sexp['$[]'](2)['$[]'](0)['$==']("resbody") : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
          if ((($a = sexp['$[]'](2)['$[]'](2)) !== nil && (!$a.$$is_boolean || $a == true))) {
            sexp['$[]'](2)['$[]='](2, self.$returns(sexp['$[]'](2)['$[]'](2)))
            } else {
            sexp['$[]'](2)['$[]='](2, self.$returns(self.$s("nil")))
          }};
        return sexp;}else if ("ensure"['$===']($case)) {sexp['$[]='](1, self.$returns(sexp['$[]'](1)));
        return sexp;}else if ("begin"['$===']($case)) {sexp['$[]='](1, self.$returns(sexp['$[]'](1)));
        return sexp;}else if ("rescue_mod"['$===']($case)) {sexp['$[]='](1, self.$returns(sexp['$[]'](1)));
        sexp['$[]='](2, self.$returns(sexp['$[]'](2)));
        return sexp;}else if ("while"['$===']($case)) {return sexp}else if ("return"['$===']($case) || "js_return"['$===']($case)) {return sexp}else if ("xstr"['$===']($case)) {if ((($a = /return|;/['$=~'](sexp['$[]'](1))) !== nil && (!$a.$$is_boolean || $a == true))) {
          } else {
          sexp['$[]='](1, "return " + (sexp['$[]'](1)) + ";")
        };
        return sexp;}else if ("dxstr"['$===']($case)) {if ((($a = /return|;|\n/['$=~'](sexp['$[]'](1))) !== nil && (!$a.$$is_boolean || $a == true))) {
          } else {
          sexp['$[]='](1, "return " + (sexp['$[]'](1)))
        };
        return sexp;}else if ("if"['$===']($case)) {sexp['$[]='](2, self.$returns(((($a = sexp['$[]'](2)) !== false && $a !== nil) ? $a : self.$s("nil"))));
        sexp['$[]='](3, self.$returns(((($a = sexp['$[]'](3)) !== false && $a !== nil) ? $a : self.$s("nil"))));
        return sexp;}else {return ($a = ($b = self.$s("js_return", sexp)).$tap, $a.$$p = (TMP_7 = function(s){var self = TMP_7.$$s || this, $a, $b;
if (s == null) s = nil;
        return (($a = [sexp.$source()]), $b = s, $b['$source='].apply($b, $a), $a[$a.length-1])}, TMP_7.$$s = self, TMP_7), $a).call($b)}})();
      };

      return (def.$handle_block_given_call = function(sexp) {
        var $a, $b, self = this, scope = nil;

        self.scope['$uses_block!']();
        if ((($a = self.scope.$block_name()) !== nil && (!$a.$$is_boolean || $a == true))) {
          return self.$fragment("(" + (self.scope.$block_name()) + " !== nil)", sexp)
        } else if ((($a = ($b = scope = self.scope.$find_parent_def(), $b !== false && $b !== nil ?scope.$block_name() : $b)) !== nil && (!$a.$$is_boolean || $a == true))) {
          return self.$fragment("(" + (scope.$block_name()) + " !== nil)", sexp)
          } else {
          return self.$fragment("false", sexp)
        };
      }, nil) && 'handle_block_given_call';
    })(self, null);
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal/erb"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $klass = Opal.klass, $gvars = Opal.gvars;

  Opal.add_stubs(['$require', '$compile', '$new', '$fix_quotes', '$find_contents', '$find_code', '$wrap_compiled', '$require_erb', '$prepared_source', '$gsub', '$=~', '$sub']);
  self.$require("opal/compiler");
  return (function($base) {
    var self = $module($base, 'Opal');

    var def = self.$$proto, $scope = self.$$scope;

    (function($base) {
      var self = $module($base, 'ERB');

      var def = self.$$proto, $scope = self.$$scope;

      Opal.defs(self, '$compile', function(source, file_name) {
        var self = this;

        if (file_name == null) {
          file_name = "(erb)"
        }
        return $scope.get('Compiler').$new(source, file_name).$compile();
      });

      (function($base, $super) {
        function $Compiler(){};
        var self = $Compiler = $klass($base, $super, 'Compiler', $Compiler);

        var def = self.$$proto, $scope = self.$$scope;

        def.prepared_source = def.source = def.file_name = nil;
        Opal.cdecl($scope, 'BLOCK_EXPR', /\s+(do|\{)(\s*\|[^|]*\|)?\s*\Z/);

        def.$initialize = function(source, file_name) {
          var $a, self = this;

          if (file_name == null) {
            file_name = "(erb)"
          }
          return $a = [source, file_name, source], self.source = $a[0], self.file_name = $a[1], self.result = $a[2];
        };

        def.$prepared_source = function() {
          var $a, self = this, source = nil;

          return ((($a = self.prepared_source) !== false && $a !== nil) ? $a : self.prepared_source = (function() {source = self.source;
          source = self.$fix_quotes(source);
          source = self.$find_contents(source);
          source = self.$find_code(source);
          source = self.$wrap_compiled(source);
          source = self.$require_erb(source);
          return source;})());
        };

        def.$compile = function() {
          var self = this;

          return $scope.get('Opal').$compile(self.$prepared_source());
        };

        def.$fix_quotes = function(result) {
          var self = this;

          return result.$gsub("\"", "\\\"");
        };

        def.$require_erb = function(result) {
          var self = this;

          return $rb_plus("require \"erb\";", result);
        };

        def.$find_contents = function(result) {
          var $a, $b, TMP_1, self = this;

          return ($a = ($b = result).$gsub, $a.$$p = (TMP_1 = function(){var self = TMP_1.$$s || this, $a, inner = nil;

          inner = (($a = $gvars['~']) === nil ? nil : $a['$[]'](1)).$gsub(/\\'/, "'").$gsub(/\\"/, "\"");
            if ((($a = inner['$=~']($scope.get('BLOCK_EXPR'))) !== nil && (!$a.$$is_boolean || $a == true))) {
              return "\")\noutput_buffer.append= " + (inner) + "\noutput_buffer.append(\""
              } else {
              return "\")\noutput_buffer.append=(" + (inner) + ")\noutput_buffer.append(\""
            };}, TMP_1.$$s = self, TMP_1), $a).call($b, /<%=([\s\S]+?)%>/);
        };

        def.$find_code = function(result) {
          var $a, $b, TMP_2, self = this;

          return ($a = ($b = result).$gsub, $a.$$p = (TMP_2 = function(){var self = TMP_2.$$s || this, $a;

          return "\")\n" + ((($a = $gvars['~']) === nil ? nil : $a['$[]'](1))) + "\noutput_buffer.append(\""}, TMP_2.$$s = self, TMP_2), $a).call($b, /<%([\s\S]+?)%>/);
        };

        return (def.$wrap_compiled = function(result) {
          var self = this, path = nil;

          path = self.file_name.$sub((new RegExp("\\.opalerb" + $scope.get('REGEXP_END'))), "");
          return result = "Template.new('" + (path) + "') do |output_buffer|\noutput_buffer.append(\"" + (result) + "\")\noutput_buffer.join\nend\n";
        }, nil) && 'wrap_compiled';
      })(self, null);
    })(self)
  })(self);
};
/* Generated by Opal 0.8.1 */
Opal.modules["encoding"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  var $a, $b, TMP_4, $c, TMP_6, $d, TMP_8, self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$[]', '$new', '$to_proc', '$each', '$const_set', '$sub', '$upcase', '$===', '$const_get', '$==', '$name', '$include?', '$names', '$constants', '$raise', '$attr_accessor', '$attr_reader', '$register', '$length', '$bytes', '$to_a', '$each_byte', '$bytesize', '$enum_for', '$find', '$getbyte']);
  self.$require("corelib/string");
  (function($base, $super) {
    function $Encoding(){};
    var self = $Encoding = $klass($base, $super, 'Encoding', $Encoding);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    def.ascii = def.dummy = def.name = nil;
    Opal.defs(self, '$register', TMP_1 = function(name, options) {
      var $a, $b, $c, TMP_2, self = this, $iter = TMP_1.$$p, block = $iter || nil, names = nil, encoding = nil;

      if (options == null) {
        options = $hash2([], {})
      }
      TMP_1.$$p = null;
      names = $rb_plus([name], (((($a = options['$[]']("aliases")) !== false && $a !== nil) ? $a : [])));
      encoding = ($a = ($b = $scope.get('Class')).$new, $a.$$p = block.$to_proc(), $a).call($b, self).$new(name, names, ((($a = options['$[]']("ascii")) !== false && $a !== nil) ? $a : false), ((($a = options['$[]']("dummy")) !== false && $a !== nil) ? $a : false));
      return ($a = ($c = names).$each, $a.$$p = (TMP_2 = function(name){var self = TMP_2.$$s || this;
if (name == null) name = nil;
      return self.$const_set(name.$sub("-", "_"), encoding)}, TMP_2.$$s = self, TMP_2), $a).call($c);
    });

    Opal.defs(self, '$find', function(name) {try {

      var $a, $b, TMP_3, self = this, upcase_name = nil;

      upcase_name = name.$upcase();
      if ((($a = self['$==='](upcase_name)) !== nil && (!$a.$$is_boolean || $a == true))) {
        return upcase_name};
      ($a = ($b = self.$constants()).$each, $a.$$p = (TMP_3 = function(const$){var self = TMP_3.$$s || this, $a, $b, encoding = nil;
if (const$ == null) const$ = nil;
      encoding = self.$const_get(const$);
        if ((($a = ((($b = encoding.$name()['$=='](upcase_name)) !== false && $b !== nil) ? $b : encoding.$names()['$include?'](upcase_name))) !== nil && (!$a.$$is_boolean || $a == true))) {
          Opal.ret(encoding)
          } else {
          return nil
        };}, TMP_3.$$s = self, TMP_3), $a).call($b);
      return self.$raise($scope.get('ArgumentError'), "unknown encoding name - " + (name));
      } catch ($returner) { if ($returner === Opal.returner) { return $returner.$v } throw $returner; }
    });

    (function(self) {
      var $scope = self.$$scope, def = self.$$proto;

      return self.$attr_accessor("default_external")
    })(self.$singleton_class());

    self.$attr_reader("name", "names");

    def.$initialize = function(name, names, ascii, dummy) {
      var self = this;

      self.name = name;
      self.names = names;
      self.ascii = ascii;
      return self.dummy = dummy;
    };

    def['$ascii_compatible?'] = function() {
      var self = this;

      return self.ascii;
    };

    def['$dummy?'] = function() {
      var self = this;

      return self.dummy;
    };

    def.$to_s = function() {
      var self = this;

      return self.name;
    };

    def.$inspect = function() {
      var $a, self = this;

      return "#<Encoding:" + (self.name) + ((function() {if ((($a = self.dummy) !== nil && (!$a.$$is_boolean || $a == true))) {
        return " (dummy)"
        } else {
        return nil
      }; return nil; })()) + ">";
    };

    def.$each_byte = function() {
      var self = this;

      return self.$raise($scope.get('NotImplementedError'));
    };

    def.$getbyte = function() {
      var self = this;

      return self.$raise($scope.get('NotImplementedError'));
    };

    def.$bytesize = function() {
      var self = this;

      return self.$raise($scope.get('NotImplementedError'));
    };

    (function($base, $super) {
      function $EncodingError(){};
      var self = $EncodingError = $klass($base, $super, 'EncodingError', $EncodingError);

      var def = self.$$proto, $scope = self.$$scope;

      return nil;
    })(self, $scope.get('StandardError'));

    return (function($base, $super) {
      function $CompatibilityError(){};
      var self = $CompatibilityError = $klass($base, $super, 'CompatibilityError', $CompatibilityError);

      var def = self.$$proto, $scope = self.$$scope;

      return nil;
    })(self, $scope.get('EncodingError'));
  })(self, null);
  ($a = ($b = $scope.get('Encoding')).$register, $a.$$p = (TMP_4 = function(){var self = TMP_4.$$s || this, TMP_5;

  Opal.defn(self, '$each_byte', TMP_5 = function(string) {
      var $a, self = this, $iter = TMP_5.$$p, block = $iter || nil;

      TMP_5.$$p = null;
      
      for (var i = 0, length = string.length; i < length; i++) {
        var code = string.charCodeAt(i);

        if (code <= 0x7f) {
          ((($a = Opal.yield1(block, code)) === $breaker) ? $breaker.$v : $a);
        }
        else {
          var encoded = encodeURIComponent(string.charAt(i)).substr(1).split('%');

          for (var j = 0, encoded_length = encoded.length; j < encoded_length; j++) {
            ((($a = Opal.yield1(block, parseInt(encoded[j], 16))) === $breaker) ? $breaker.$v : $a);
          }
        }
      }
    
    });
    return (Opal.defn(self, '$bytesize', function() {
      var self = this;

      return self.$bytes().$length();
    }), nil) && 'bytesize';}, TMP_4.$$s = self, TMP_4), $a).call($b, "UTF-8", $hash2(["aliases", "ascii"], {"aliases": ["CP65001"], "ascii": true}));
  ($a = ($c = $scope.get('Encoding')).$register, $a.$$p = (TMP_6 = function(){var self = TMP_6.$$s || this, TMP_7;

  Opal.defn(self, '$each_byte', TMP_7 = function(string) {
      var $a, self = this, $iter = TMP_7.$$p, block = $iter || nil;

      TMP_7.$$p = null;
      
      for (var i = 0, length = string.length; i < length; i++) {
        var code = string.charCodeAt(i);

        ((($a = Opal.yield1(block, code & 0xff)) === $breaker) ? $breaker.$v : $a);
        ((($a = Opal.yield1(block, code >> 8)) === $breaker) ? $breaker.$v : $a);
      }
    
    });
    return (Opal.defn(self, '$bytesize', function() {
      var self = this;

      return self.$bytes().$length();
    }), nil) && 'bytesize';}, TMP_6.$$s = self, TMP_6), $a).call($c, "UTF-16LE");
  ($a = ($d = $scope.get('Encoding')).$register, $a.$$p = (TMP_8 = function(){var self = TMP_8.$$s || this, TMP_9;

  Opal.defn(self, '$each_byte', TMP_9 = function(string) {
      var $a, self = this, $iter = TMP_9.$$p, block = $iter || nil;

      TMP_9.$$p = null;
      
      for (var i = 0, length = string.length; i < length; i++) {
        ((($a = Opal.yield1(block, string.charCodeAt(i) & 0xff)) === $breaker) ? $breaker.$v : $a);
      }
    
    });
    return (Opal.defn(self, '$bytesize', function() {
      var self = this;

      return self.$bytes().$length();
    }), nil) && 'bytesize';}, TMP_8.$$s = self, TMP_8), $a).call($d, "ASCII-8BIT", $hash2(["aliases", "ascii"], {"aliases": ["BINARY"], "ascii": true}));
  return (function($base, $super) {
    function $String(){};
    var self = $String = $klass($base, $super, 'String', $String);

    var def = self.$$proto, $scope = self.$$scope, TMP_10;

    def.encoding = nil;
    def.encoding = (($scope.get('Encoding')).$$scope.get('UTF_16LE'));

    def.$bytes = function() {
      var self = this;

      return self.$each_byte().$to_a();
    };

    def.$bytesize = function() {
      var self = this;

      return self.encoding.$bytesize(self);
    };

    def.$each_byte = TMP_10 = function() {
      var $a, $b, self = this, $iter = TMP_10.$$p, block = $iter || nil;

      TMP_10.$$p = null;
      if ((block !== nil)) {
        } else {
        return self.$enum_for("each_byte")
      };
      ($a = ($b = self.encoding).$each_byte, $a.$$p = block.$to_proc(), $a).call($b, self);
      return self;
    };

    def.$encoding = function() {
      var self = this;

      return self.encoding;
    };

    def.$force_encoding = function(encoding) {
      var self = this;

      encoding = $scope.get('Encoding').$find(encoding);
      if (encoding['$=='](self.encoding)) {
        return self};
      
      var result = new String(self);
      result.encoding = encoding;

      return result;
    
    };

    return (def.$getbyte = function(idx) {
      var self = this;

      return self.encoding.$getbyte(self, idx);
    }, nil) && 'getbyte';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["opal-parser"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $module = Opal.module, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$compile', '$eval']);
  self.$require("opal/compiler");
  self.$require("opal/erb");
  self.$require("opal/version");
  self.$require("encoding");
  (function($base) {
    var self = $module($base, 'Kernel');

    var def = self.$$proto, $scope = self.$$scope;

    Opal.defn(self, '$eval', function(str) {
      var self = this, code = nil;

      code = $scope.get('Opal').$compile(str, $hash2(["file"], {"file": "(eval)"}));
      return eval(code);
    });

    Opal.defn(self, '$require_remote', function(url) {
      var self = this;

      
      var r = new XMLHttpRequest();
      r.open("GET", url, false);
      r.send('');
    
      return self.$eval(r.responseText);
    });
  })(self);
  
  Opal.compile = function(str, options) {
    if (options) {
      options = Opal.hash(options);
    }
    return Opal.Opal.$compile(str, options);
  };

  Opal.eval = function(str, options) {
   return eval(Opal.compile(str, options));
  };

  function run_ruby_scripts() {
    var tag, tags = document.getElementsByTagName('script');

    for (var i = 0, len = tags.length; i < len; i++) {
      tag = tags[i];
      if (tag.type === "text/ruby") {
        if (tag.src)       Opal.Kernel.$require_remote(tag.src);
        if (tag.innerHTML) Opal.Kernel.$eval(tag.innerHTML);
      }
    }
  }

  if (typeof(document) !== 'undefined') {
    if (window.addEventListener) {
      window.addEventListener('DOMContentLoaded', run_ruby_scripts, false);
    }
    else {
      window.attachEvent('onload', run_ruby_scripts);
    }
  }

};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/console_buttons"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$include', '$add_console_buttons', '$add_console_show_it', '$add_console_clear_top', '$add_console_clear_bottom', '$[]', '$call', '$nil_handler', '$add_button']);
  self.$require("core/util");
  self.$require("widgets/button_bar");
  return (function($base, $super) {
    function $ConsoleButtons(){};
    var self = $ConsoleButtons = $klass($base, $super, 'ConsoleButtons', $ConsoleButtons);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    def.handlers = nil;
    self.$include($scope.get('Util'));

    def.$initialize = TMP_1 = function(handlers) {
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      if (handlers == null) {
        handlers = $hash2([], {})
      }
      TMP_1.$$p = null;
      self.handlers = handlers;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      return self.$add_console_buttons();
    };

    def.$add_console_buttons = function() {
      var self = this;

      self.$add_console_show_it();
      self.$add_console_clear_top();
      return self.$add_console_clear_bottom();
    };

    def.$add_console_clear_bottom = function() {
      var $a, self = this, fn = nil;

      fn = (function() {if ((($a = self.handlers['$[]']("clear_bottom")) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.handlers['$[]']("clear_bottom")
        } else {
        return self.$nil_handler().$call("Clear bottom")
      }; return nil; })();
      return self.$add_button("Clear bottom", fn);
    };

    def.$add_console_clear_top = function() {
      var $a, self = this, fn = nil;

      fn = (function() {if ((($a = self.handlers['$[]']("clear_top")) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.handlers['$[]']("clear_top")
        } else {
        return self.$nil_handler().$call("Clear top")
      }; return nil; })();
      return self.$add_button("Clear top", fn);
    };

    return (def.$add_console_show_it = function() {
      var $a, self = this, fn = nil;

      fn = (function() {if ((($a = self.handlers['$[]']("show_it")) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.handlers['$[]']("show_it")
        } else {
        return self.$nil_handler().$call("Show it")
      }; return nil; })();
      return self.$add_button("Show it", fn);
    }, nil) && 'add_console_show_it';
  })(self, $scope.get('ButtonBar'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/form_text_area"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$set_value', '$getValue', '$widget', '$new_ui_form_text_area', '$setValue']);
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $FormTextArea(){};
    var self = $FormTextArea = $klass($base, $super, 'FormTextArea', $FormTextArea);

    var def = self.$$proto, $scope = self.$$scope;

    def.$clear = function() {
      var self = this;

      return self.$set_value("");
    };

    def.$get_value = function() {
      var self = this;

      return self.$widget().$getValue();
    };

    def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_form_text_area();
    };

    return (def.$set_value = function(s) {
      var self = this;

      return self.$widget().$setValue(s);
    }, nil) && 'set_value';
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["views/windows/console_window"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$attr_reader', '$new', '$add', '$split_pane', '$cin', '$cout', '$add_cout', '$add_cin', '$initial_handler_functions', '$lambda', '$clear', '$prn', '$eval', '$get_value', '$set_value']);
  self.$require("opal-parser");
  self.$require("qx/ui/window");
  self.$require("widgets/console_buttons");
  self.$require("widgets/split_pane");
  self.$require("widgets/form_text_area");
  return (function($base, $super) {
    function $ConsoleWindow(){};
    var self = $ConsoleWindow = $klass($base, $super, 'ConsoleWindow', $ConsoleWindow);

    var def = self.$$proto, $scope = self.$$scope;

    self.$attr_reader("cin");

    self.$attr_reader("cout");

    self.$attr_reader("split_pane");

    def.$add_cin = function() {
      var self = this;

      self.cin = $scope.get('FormTextArea').$new();
      return self.$split_pane().$add(self.$cin());
    };

    def.$add_cout = function() {
      var self = this;

      self.cout = $scope.get('FormTextArea').$new();
      return self.$split_pane().$add(self.$cout());
    };

    def.$add_items = function() {
      var self = this;

      self.split_pane = $scope.get('SplitPane').$new("vertical");
      self.$add(self.$split_pane(), $hash2(["flex"], {"flex": 1}));
      self.$add_cout();
      return self.$add_cin();
    };

    def.$initial_button_bar = function() {
      var self = this;

      return $scope.get('ConsoleButtons').$new(self.$initial_handler_functions());
    };

    def.$initial_caption = function() {
      var self = this;

      return "Console Window";
    };

    def.$initial_handler_functions = function() {
      var $a, $b, TMP_1, $c, TMP_2, $d, TMP_3, self = this;

      return $hash2(["clear_bottom", "clear_top", "show_it"], {"clear_bottom": ($a = ($b = self).$lambda, $a.$$p = (TMP_1 = function(){var self = TMP_1.$$s || this;

      return self.$cin().$clear()}, TMP_1.$$s = self, TMP_1), $a).call($b), "clear_top": ($a = ($c = self).$lambda, $a.$$p = (TMP_2 = function(){var self = TMP_2.$$s || this;

      return self.$cout().$clear()}, TMP_2.$$s = self, TMP_2), $a).call($c), "show_it": ($a = ($d = self).$lambda, $a.$$p = (TMP_3 = function(){var self = TMP_3.$$s || this;

      return self.$prn(self.$eval(self.$cin().$get_value()))}, TMP_3.$$s = self, TMP_3), $a).call($d)});
    };

    def['$is_centered?'] = function() {
      var self = this;

      return true;
    };

    return (def.$prn = function(s) {
      var self = this, text = nil;

      text = self.$cout().$get_value();
      text = $rb_plus(text, "" + (s) + "\n");
      return self.$cout().$set_value(text);
    }, nil) && 'prn';
  })(self, $scope.get('Window'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["views/windows/crud_window"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$attr_reader', '$nil?', '$capitalize', '$to_s', '$add_fill', '$new', '$model', '$caption']);
  self.$require("panels/crud_panel");
  self.$require("qx/ui/window");
  return (function($base, $super) {
    function $CrudWindow(){};
    var self = $CrudWindow = $klass($base, $super, 'CrudWindow', $CrudWindow);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$attr_reader("caption");

    self.$attr_reader("model");

    def.$initialize = TMP_1 = function(model, caption) {
      var $a, self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      if (model == null) {
        model = "dummy"
      }
      if (caption == null) {
        caption = nil
      }
      TMP_1.$$p = null;
      self.model = model;
      self.caption = (function() {if ((($a = caption['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return "" + (model.$to_s().$capitalize()) + "s Window"
        } else {
        return caption
      }; return nil; })();
      return Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
    };

    def.$add_items = function() {
      var self = this;

      return self.$add_fill($scope.get('CrudPanel').$new(self.$model()));
    };

    def.$initial_caption = function() {
      var self = this;

      return self.$caption();
    };

    return (def['$is_centered?'] = function() {
      var self = this;

      return true;
    }, nil) && 'is_centered?';
  })(self, $scope.get('Window'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["views/windows/message_window"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$attr_reader', '$add_fill', '$new', '$message', '$caption']);
  self.$require("qx/ui/window");
  self.$require("widgets/html");
  return (function($base, $super) {
    function $MessageWindow(){};
    var self = $MessageWindow = $klass($base, $super, 'MessageWindow', $MessageWindow);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$attr_reader("caption");

    self.$attr_reader("message");

    def.$initialize = TMP_1 = function(aMessage, caption) {
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      if (caption == null) {
        caption = "Message"
      }
      TMP_1.$$p = null;
      self.message = aMessage;
      self.caption = caption;
      return Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
    };

    def.$add_items = function() {
      var self = this;

      return self.$add_fill($scope.get('Html').$new(self.$message()));
    };

    def.$initial_caption = function() {
      var self = this;

      return self.$caption();
    };

    return (def['$is_centered?'] = function() {
      var self = this;

      return true;
    }, nil) && 'is_centered?';
  })(self, $scope.get('Window'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/footer"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$include', '$new', '$set_color', '$resolve_color', '$set_width', '$add', '$set_background_color']);
  self.$require("core/dimensions");
  self.$require("core/qx_wrap");
  self.$require("managers/color_manager");
  self.$require("qx/ui/container");
  self.$require("qx/ui/h_layout");
  self.$require("qx/ui/widget");
  self.$require("widgets/html");
  return (function($base, $super) {
    function $Footer(){};
    var self = $Footer = $klass($base, $super, 'Footer', $Footer);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$include($scope.get('Dimensions'));

    def.$initialize = TMP_1 = function(text) {
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil, html_widget = nil;

      if (text == null) {
        text = ""
      }
      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      html_widget = $scope.get('Html').$new(text);
      html_widget.$set_color($scope.get('ColorManager').$resolve_color("footer_text"));
      html_widget.$set_width(200);
      self.$add($scope.get('Widget').$new(), $hash2(["flex"], {"flex": 1}));
      self.$add(html_widget);
      self.$add($scope.get('Widget').$new(), $hash2(["flex"], {"flex": 1}));
      return self.$set_background_color($scope.get('ColorManager').$resolve_color("footer_background"));
    };

    def.$initial_classes = function() {
      var self = this;

      return ["footer"];
    };

    def.$initial_height = function() {
      var self = this;

      return $scope.get('FOOTER_HEIGHT');
    };

    def.$initial_layout = function() {
      var self = this;

      return $scope.get('HLayout').$new();
    };

    return (def.$initial_padding = function() {
      var self = this;

      return $scope.get('FOOTER_PADDING');
    }, nil) && 'initial_padding';
  })(self, $scope.get('Container'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/form_combo_box"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$add_items', '$set_value', '$on_change_value', '$each', '$new_ui_form_list_item', '$add', '$widget', '$getValue', '$new_ui_form_combo_box', '$nil?', '$lambda', '$call', '$add_listener', '$resetValue', '$setValue']);
  self.$require("core/qx_wrap");
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $FormComboBox(){};
    var self = $FormComboBox = $klass($base, $super, 'FormComboBox', $FormComboBox);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    def.$initialize = TMP_1 = function(item_labels, value, fn) {
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      if (item_labels == null) {
        item_labels = []
      }
      if (value == null) {
        value = nil
      }
      if (fn == null) {
        fn = nil
      }
      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      self.$add_items(item_labels);
      self.$set_value(value);
      return self.$on_change_value(fn);
    };

    def.$add_items = function(item_labels) {
      var $a, $b, TMP_2, self = this;

      return ($a = ($b = item_labels).$each, $a.$$p = (TMP_2 = function(label){var self = TMP_2.$$s || this, item = nil;
if (label == null) label = nil;
      item = $scope.get('QxWrap').$new_ui_form_list_item(label);
        return self.$widget().$add(item);}, TMP_2.$$s = self, TMP_2), $a).call($b);
    };

    def.$get_value = function() {
      var self = this;

      return self.$widget().$getValue();
    };

    def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_form_combo_box();
    };

    def.$on_change_value = function(fn) {
      var $a, $b, TMP_3, self = this, listener_fn = nil;

      if ((($a = fn['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        listener_fn = ($a = ($b = self).$lambda, $a.$$p = (TMP_3 = function(data_event){var self = TMP_3.$$s || this;
if (data_event == null) data_event = nil;
        return fn.$call(data_event.getData())}, TMP_3.$$s = self, TMP_3), $a).call($b);
        return self.$add_listener("changeValue", listener_fn);
      };
    };

    def.$reset_value = function() {
      var self = this;

      return self.$widget().$resetValue();
    };

    return (def.$set_value = function(str) {
      var $a, self = this;

      if ((($a = str['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        return self.$widget().$setValue(str)
      };
    }, nil) && 'set_value';
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/form_list"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$attr_reader', '$add_controller', '$setDecorator', '$widget', '$setPadding', '$setMargin', '$set_model', '$set_template', '$new_data_array', '$new_data_controller_list', '$items', '$setDelegate', '$controller', '$lambda', '$new_lib_data_list_model', '$push', '$new', '$model', '$render', '$template', '$add_item', '$new_ui_form_list', '$getSelection', '$Native', '$call', '$addListener', '$removeAll', '$removeAt', '$find_model', '$find_template']);
  self.$require("core/qx_wrap");
  self.$require("managers/model_manager");
  self.$require("managers/template_manager");
  self.$require("qx/ui/component");
  self.$require("widgets/html");
  self.$require("native");
  return (function($base, $super) {
    function $FormList(){};
    var self = $FormList = $klass($base, $super, 'FormList', $FormList);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$attr_reader("controller");

    self.$attr_reader("items");

    self.$attr_reader("model");

    self.$attr_reader("template");

    def.$initialize = TMP_1 = function(model_name, template_name) {
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      if (model_name == null) {
        model_name = "dummy"
      }
      if (template_name == null) {
        template_name = "dummy"
      }
      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      self.$add_controller();
      self.$widget().$setDecorator(nil);
      self.$widget().$setPadding(0);
      self.$widget().$setMargin(0);
      self.$set_model(model_name);
      return self.$set_template(template_name);
    };

    def.$add_controller = function() {
      var $a, $b, TMP_2, self = this;

      self.items = $scope.get('QxWrap').$new_data_array();
      self.controller = $scope.get('QxWrap').$new_data_controller_list(self.$items(), self.$widget());
      return self.$controller().$setDelegate($hash2(["configureItem"], {"configureItem": ($a = ($b = self).$lambda, $a.$$p = (TMP_2 = function(item){var self = TMP_2.$$s || this;
if (item == null) item = nil;
      return item.setRich(true);}, TMP_2.$$s = self, TMP_2), $a).call($b)}));
    };

    def.$add_item = function(html, data) {
      var self = this, item = nil;

      if (data == null) {
        data = nil
      }
      item = $scope.get('QxWrap').$new_lib_data_list_model(html, data);
      return self.$items().$push(item);
    };

    def.$add_record = function(data) {
      var self = this, record = nil, html = nil;

      record = self.$model().$new(data);
      html = self.$template().$render(record);
      return self.$add_item(html, data);
    };

    def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_form_list();
    };

    def.$on_select = function(fn) {
      var $a, $b, TMP_3, self = this, handler = nil;

      handler = ($a = ($b = self).$lambda, $a.$$p = (TMP_3 = function(){var self = TMP_3.$$s || this, $a, selection = nil, selected_model = nil, html = nil, data = nil;

      selection = self.$widget().$getSelection();
        selected_model = selection.length > 0 ? selection[0].getModel() : null;
        if ((($a = selected_model == null) !== nil && (!$a.$$is_boolean || $a == true))) {
          } else {
          html = selected_model.getHtml();
          data = selected_model.getData();
          selected_model = $hash2(["html", "data"], {"html": self.$Native(html), "data": data});
        };
        return fn.$call(selected_model);}, TMP_3.$$s = self, TMP_3), $a).call($b);
      return self.$controller().$addListener("changeSelection", handler);
    };

    def.$remove_all = function() {
      var self = this;

      return self.$items().$removeAll();
    };

    def.$remove_at = function(n) {
      var self = this;

      return self.$items().$removeAt(n);
    };

    def.$set_model = function(model_name) {
      var self = this;

      return self.model = $scope.get('ModelManager').$find_model(model_name);
    };

    return (def.$set_template = function(template_name) {
      var self = this;

      return self.template = $scope.get('TemplateManager').$find_template(template_name);
    }, nil) && 'set_template';
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/form_select_box"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$add_items', '$on_change_selection', '$each', '$new_ui_form_list_item', '$add', '$widget', '$new_ui_form_select_box', '$nil?', '$lambda', '$call', '$add_listener']);
  self.$require("core/qx_wrap");
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $FormSelectBox(){};
    var self = $FormSelectBox = $klass($base, $super, 'FormSelectBox', $FormSelectBox);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    def.$initialize = TMP_1 = function(item_labels, fn) {
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      if (item_labels == null) {
        item_labels = []
      }
      if (fn == null) {
        fn = nil
      }
      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      self.$add_items(item_labels);
      return self.$on_change_selection(fn);
    };

    def.$add_items = function(item_labels) {
      var $a, $b, TMP_2, self = this;

      return ($a = ($b = item_labels).$each, $a.$$p = (TMP_2 = function(label){var self = TMP_2.$$s || this, item = nil;
if (label == null) label = nil;
      item = $scope.get('QxWrap').$new_ui_form_list_item(label);
        return self.$widget().$add(item);}, TMP_2.$$s = self, TMP_2), $a).call($b);
    };

    def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_form_select_box();
    };

    return (def.$on_change_selection = function(fn) {
      var $a, $b, TMP_3, self = this, listener_fn = nil;

      if ((($a = fn['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        listener_fn = ($a = ($b = self).$lambda, $a.$$p = (TMP_3 = function(data_event){var self = TMP_3.$$s || this;
if (data_event == null) data_event = nil;
        return fn.$call(data_event.getData()[0].getLabel())}, TMP_3.$$s = self, TMP_3), $a).call($b);
        return self.$add_listener("changeSelection", listener_fn);
      };
    }, nil) && 'on_change_selection';
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/h_separator"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require']);
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $HSeparator(){};
    var self = $HSeparator = $klass($base, $super, 'HSeparator', $HSeparator);

    var def = self.$$proto, $scope = self.$$scope;

    return (def.$initial_height = function() {
      var self = this;

      return 1;
    }, nil) && 'initial_height'
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/h_spacer"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$setHeight', '$widget', '$setWidth']);
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $HSpacer(){};
    var self = $HSpacer = $klass($base, $super, 'HSpacer', $HSpacer);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    return (def.$initialize = TMP_1 = function(w) {
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      self.$widget().$setHeight(1);
      return self.$widget().$setWidth(w);
    }, nil) && 'initialize'
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/image"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$attr_reader', '$new_ui_basic_image', '$source']);
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $Image(){};
    var self = $Image = $klass($base, $super, 'Image', $Image);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$attr_reader("source");

    def.$initialize = TMP_1 = function(source) {
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      TMP_1.$$p = null;
      self.source = source;
      return Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
    };

    return (def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_basic_image(self.$source());
    }, nil) && 'initial_widget';
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/label"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$setRich', '$widget', '$setValue', '$new_ui_basic_label']);
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $Label(){};
    var self = $Label = $klass($base, $super, 'Label', $Label);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    def.$initialize = TMP_1 = function(html) {
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      self.$widget().$setRich(true);
      return self.$widget().$setValue(html);
    };

    return (def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_basic_label();
    }, nil) && 'initial_widget';
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/menu_button"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$setLabel', '$widget', '$nil?', '$on_click', '$new_ui_menu_button', '$add_listener']);
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $MenuButton(){};
    var self = $MenuButton = $klass($base, $super, 'MenuButton', $MenuButton);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    def.$initialize = TMP_1 = function(label, fn) {
      var $a, self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      if (fn == null) {
        fn = nil
      }
      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      self.$widget().$setLabel(label);
      if ((($a = fn['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        return self.$on_click(fn)
      };
    };

    def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_menu_button();
    };

    return (def.$on_click = function(fn) {
      var self = this;

      return self.$add_listener("execute", fn);
    }, nil) && 'on_click';
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/menu_menu"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$respond_to?', '$widget', '$nil?', '$add', '$new_ui_menu_menu']);
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $MenuMenu(){};
    var self = $MenuMenu = $klass($base, $super, 'MenuMenu', $MenuMenu);

    var def = self.$$proto, $scope = self.$$scope;

    def.$add = function(aComponent, arg) {
      var $a, self = this, component = nil;

      if (arg == null) {
        arg = nil
      }
      component = (function() {if ((($a = (aComponent['$respond_to?']("widget"))) !== nil && (!$a.$$is_boolean || $a == true))) {
        return aComponent.$widget()
        } else {
        return aComponent
      }; return nil; })();
      if ((($a = arg['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.$widget().$add(component)
        } else {
        return self.$widget().$add(component, arg)
      };
    };

    return (def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_menu_menu();
    }, nil) && 'initial_widget';
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/menubar_button"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$%', '$new', '$setRich', '$widget', '$setLabel', '$nil?', '$on_click', '$respond_to?', '$setMenu', '$new_ui_menubar_button', '$add_listener']);
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $MenubarButton(){};
    var self = $MenubarButton = $klass($base, $super, 'MenubarButton', $MenubarButton);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    Opal.defs(self, '$new_with_style', function(label, style, fn) {
      var self = this, html = nil, btn = nil;

      if (fn == null) {
        fn = nil
      }
      html = "<span style=\"%s\">%s</style>"['$%']([style, label]);
      btn = self.$new(html, fn);
      btn.$widget().$setRich(true);
      return btn;
    });

    def.$initialize = TMP_1 = function(label, fn) {
      var $a, self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      if (fn == null) {
        fn = nil
      }
      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      self.$widget().$setLabel(label);
      if ((($a = fn['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        return self.$on_click(fn)
      };
    };

    def.$set_menu = function(aMenu) {
      var $a, self = this, menu = nil;

      menu = (function() {if ((($a = (aMenu['$respond_to?']("widget"))) !== nil && (!$a.$$is_boolean || $a == true))) {
        return aMenu.$widget()
        } else {
        return aMenu
      }; return nil; })();
      return self.$widget().$setMenu(menu);
    };

    def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_menubar_button();
    };

    return (def.$on_click = function(fn) {
      var self = this;

      return self.$add_listener("execute", fn);
    }, nil) && 'on_click';
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/menubar_menubar"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$setDecorator', '$widget', '$respond_to?', '$nil?', '$add', '$addSeparator', '$addSpacer', '$new_ui_menubar_menubar']);
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $MenubarBar(){};
    var self = $MenubarBar = $klass($base, $super, 'MenubarBar', $MenubarBar);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    def.$initialize = TMP_1 = function() {var $zuper = $slice.call(arguments, 0);
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, $iter).apply(self, $zuper);
      return self.$widget().$setDecorator(nil);
    };

    def.$add = function(aComponent, arg) {
      var $a, self = this, component = nil;

      if (arg == null) {
        arg = nil
      }
      component = (function() {if ((($a = (aComponent['$respond_to?']("widget"))) !== nil && (!$a.$$is_boolean || $a == true))) {
        return aComponent.$widget()
        } else {
        return aComponent
      }; return nil; })();
      if ((($a = arg['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return self.$widget().$add(component)
        } else {
        return self.$widget().$add(component, arg)
      };
    };

    def.$add_separator = function() {
      var self = this;

      return self.$widget().$addSeparator();
    };

    def.$add_spacer = function() {
      var self = this;

      return self.$widget().$addSpacer();
    };

    return (def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_menubar_menubar();
    }, nil) && 'initial_widget';
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/nav_bar"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  function $rb_minus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs - rhs : lhs['$-'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$attr_reader', '$include', '$setPadding', '$widget', '$set_background_color', '$resolve_color', '$[]', '$config', '$each', '$new', '$lambda', '$nil?', '$controller', '$handle_action', '$add', '$menu_bar', '$create_label', '$create_menu_bar', '$to_s', '$set_color', '$set_width', '$add_buttons', '$add_spacer', '$add_separator', '$menu_bar_login_buttons', '$new_with_style', '$set_menu']);
  self.$require("core/dimensions");
  self.$require("core/qx_wrap");
  self.$require("managers/color_manager");
  self.$require("qx/ui/container");
  self.$require("qx/ui/h_layout");
  self.$require("widgets/h_spacer");
  self.$require("widgets/label");
  self.$require("widgets/menu_button");
  self.$require("widgets/menu_menu");
  self.$require("widgets/menubar_button");
  self.$require("widgets/menubar_menubar");
  return (function($base, $super) {
    function $NavBar(){};
    var self = $NavBar = $klass($base, $super, 'NavBar', $NavBar);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$attr_reader("controller");

    self.$attr_reader("config");

    self.$attr_reader("menu_bar");

    self.$include($scope.get('Dimensions'));

    def.$initialize = TMP_1 = function(config, controller) {
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      if (controller == null) {
        controller = nil
      }
      TMP_1.$$p = null;
      self.config = config;
      self.controller = controller;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      self.$widget().$setPadding(5, 0, 5, 35);
      return self.$set_background_color($scope.get('ColorManager').$resolve_color("navbar_background"));
    };

    def.$add_buttons = function() {
      var $a, $b, TMP_2, self = this, buttons = nil;

      buttons = self.$config()['$[]']("buttons");
      return ($a = ($b = buttons).$each, $a.$$p = (TMP_2 = function(button){var self = TMP_2.$$s || this, $a, $b, TMP_3, text = nil, action = nil, mb_btn = nil;
if (button == null) button = nil;
      text = button['$[]']("text");
        action = button['$[]']("action");
        mb_btn = $scope.get('MenubarButton').$new(text, ($a = ($b = self).$lambda, $a.$$p = (TMP_3 = function(){var self = TMP_3.$$s || this, $a;

        if ((($a = self.$controller()['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
            return nil
            } else {
            return self.$controller().$handle_action(action)
          }}, TMP_3.$$s = self, TMP_3), $a).call($b));
        return self.$menu_bar().$add(mb_btn);}, TMP_2.$$s = self, TMP_2), $a).call($b);
    };

    def.$add_items = function() {
      var self = this;

      self.$add(self.$create_label());
      return self.$add(self.$create_menu_bar(), $hash2(["flex"], {"flex": 1}));
    };

    def.$create_label = function() {
      var self = this, text = nil, labelHtml = nil, label = nil;

      text = self.$config()['$[]']("label").$to_s();
      labelHtml = "<span style=\"font-size:1.8em;\">";
      labelHtml = $rb_plus(labelHtml, text);
      labelHtml = $rb_plus(labelHtml, "</span>");
      label = $scope.get('Label').$new(labelHtml);
      label.$set_color($scope.get('ColorManager').$resolve_color("navbar_label"));
      label.$set_width($rb_minus($scope.get('NAV_PANEL_WIDTH'), 35));
      return label;
    };

    def.$create_menu_bar = function() {
      var self = this;

      self.menu_bar = $scope.get('MenubarBar').$new();
      self.$add_buttons();
      self.$menu_bar().$add_spacer();
      self.$menu_bar().$add_separator();
      self.$menu_bar().$add(self.$menu_bar_login_buttons());
      self.$menu_bar().$add($scope.get('HSpacer').$new(5));
      return self.$menu_bar();
    };

    def.$initial_classes = function() {
      var self = this;

      return ["navbar"];
    };

    def.$initial_layout = function() {
      var self = this;

      return $scope.get('HLayout').$new();
    };

    def.$initial_padding = function() {
      var self = this;

      return 0;
    };

    return (def.$menu_bar_login_buttons = function() {
      var self = this, color = nil, mb_btn = nil, login_btn = nil, register_btn = nil, menu = nil;

      color = $scope.get('ColorManager').$resolve_color("navbar_label");
      mb_btn = $scope.get('MenubarButton').$new_with_style("Login", "color:" + (color));
      login_btn = $scope.get('MenuButton').$new("Login");
      register_btn = $scope.get('MenuButton').$new("Register");
      menu = $scope.get('MenuMenu').$new();
      menu.$add(login_btn);
      menu.$add(register_btn);
      mb_btn.$set_menu(menu);
      return mb_btn;
    }, nil) && 'menu_bar_login_buttons';
  })(self, $scope.get('Container'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/tab_button"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$new_ui_tabview_tab_button']);
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $TabButton(){};
    var self = $TabButton = $klass($base, $super, 'TabButton', $TabButton);

    var def = self.$$proto, $scope = self.$$scope;

    return (def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_tabview_tab_button();
    }, nil) && 'initial_widget'
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/tab_page"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$attr_reader', '$nil?', '$add_fill', '$new_ui_tabview_page', '$label']);
  self.$require("core/qx_wrap");
  self.$require("qx/ui/container");
  return (function($base, $super) {
    function $TabPage(){};
    var self = $TabPage = $klass($base, $super, 'TabPage', $TabPage);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$attr_reader("label");

    def.$initialize = TMP_1 = function(label, item) {
      var $a, self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      if (item == null) {
        item = nil
      }
      TMP_1.$$p = null;
      self.label = label;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      if ((($a = item['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
        return nil
        } else {
        return self.$add_fill(item)
      };
    };

    return (def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_tabview_page(self.$label());
    }, nil) && 'initial_widget';
  })(self, $scope.get('Container'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/tab_view"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$include', '$add', '$widget', '$as_qx', '$new_ui_tabview_tab_view']);
  self.$require("core/qx_wrap");
  self.$require("core/util");
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $TabView(){};
    var self = $TabView = $klass($base, $super, 'TabView', $TabView);

    var def = self.$$proto, $scope = self.$$scope;

    self.$include($scope.get('Util'));

    def.$add = function(page) {
      var self = this;

      return self.$widget().$add(self.$as_qx(page));
    };

    return (def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_tabview_tab_view();
    }, nil) && 'initial_widget';
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/tree"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$include', '$attr_reader', '$create_model_from_object', '$new_ui_tree_virtual_tree', '$model']);
  self.$require("core/marshal");
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $Tree(){};
    var self = $Tree = $klass($base, $super, 'Tree', $Tree);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$include($scope.get('Marshal'));

    self.$attr_reader("model");

    def.$initialize = TMP_1 = function(model) {
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      if (model == null) {
        model = nil
      }
      TMP_1.$$p = null;
      self.model = self.$create_model_from_object(model);
      return Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
    };

    return (def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_tree_virtual_tree(self.$model());
    }, nil) && 'initial_widget';
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/tree_file"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$new_ui_tree_file']);
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $TreeFile(){};
    var self = $TreeFile = $klass($base, $super, 'TreeFile', $TreeFile);

    var def = self.$$proto, $scope = self.$$scope;

    return (def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_tree_file();
    }, nil) && 'initial_widget'
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/tree_folder"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$new_ui_tree_folder']);
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $TreeFolder(){};
    var self = $TreeFolder = $klass($base, $super, 'TreeFolder', $TreeFolder);

    var def = self.$$proto, $scope = self.$$scope;

    return (def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_tree_folder();
    }, nil) && 'initial_widget'
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/v_separator"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require']);
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $VSeparator(){};
    var self = $VSeparator = $klass($base, $super, 'VSeparator', $VSeparator);

    var def = self.$$proto, $scope = self.$$scope;

    return (def.$initial_width = function() {
      var self = this;

      return 1;
    }, nil) && 'initial_width'
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/virtual_tree"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$include', '$attr_reader', '$update_model', '$new_ui_tree_virtual_tree', '$create_model_from_object', '$model', '$setModel', '$widget']);
  self.$require("core/marshal");
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $VirtualTree(){};
    var self = $VirtualTree = $klass($base, $super, 'VirtualTree', $VirtualTree);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$include($scope.get('Marshal'));

    self.$attr_reader("model");

    def.$initialize = TMP_1 = function(model) {
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      if (model == null) {
        model = nil
      }
      TMP_1.$$p = null;
      self.model = model;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, null).apply(self, []);
      return self.$update_model();
    };

    def.$initial_widget = function() {
      var self = this;

      return $scope.get('QxWrap').$new_ui_tree_virtual_tree();
    };

    def.$set_model = function(model) {
      var self = this;

      self.model = model;
      return self.$update_model();
    };

    return (def.$update_model = function() {
      var self = this, qx_model = nil;

      qx_model = self.$create_model_from_object(self.$model());
      return self.$widget().$setModel(qx_model);
    }, nil) && 'update_model';
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["widgets/virtual_tree_item"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require']);
  self.$require("qx/ui/component");
  return (function($base, $super) {
    function $VirtualTreeItem(){};
    var self = $VirtualTreeItem = $klass($base, $super, 'VirtualTreeItem', $VirtualTreeItem);

    var def = self.$$proto, $scope = self.$$scope;

    return nil;
  })(self, $scope.get('Component'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["configs/colors_config"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  return (function($base, $super) {
    function $ColorsConfig(){};
    var self = $ColorsConfig = $klass($base, $super, 'ColorsConfig', $ColorsConfig);

    var def = self.$$proto, $scope = self.$$scope;

    return (Opal.defs(self, '$value', function() {
      var self = this;

      return $hash2(["palette"], {"palette": "default"});
    }), nil) && 'value'
  })(self, null)
};
/* Generated by Opal 0.8.1 */
Opal.modules["models/message_model"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$attr_accessor', '$take', '$data_config', '$default=', '$[]']);
  self.$require("data/model");
  return (function($base, $super) {
    function $MessageModel(){};
    var self = $MessageModel = $klass($base, $super, 'MessageModel', $MessageModel);

    var def = self.$$proto, $scope = self.$$scope;

    self.$attr_accessor("sender_email");

    self.$attr_accessor("receiver_email");

    self.$attr_accessor("subject");

    self.$attr_accessor("text");

    self.$attr_accessor("created_at");

    self.$attr_accessor("updated_at");

    Opal.defs(self, '$api_resource_name', function() {
      var self = this;

      return "messages";
    });

    Opal.defs(self, '$data_config', function() {
      var self = this;

      return [$hash2(["id", "width"], {"id": "id", "width": 50}), $hash2(["id"], {"id": "sender_email"}), $hash2(["id"], {"id": "receiver_email"}), $hash2(["id"], {"id": "subject"}), $hash2(["id"], {"id": "text"}), $hash2(["id"], {"id": "created_at"}), $hash2(["id"], {"id": "updated_at"})];
    });

    Opal.defs(self, '$grid_config', function() {
      var self = this;

      return self.$data_config().$take(4);
    });

    return (def.$initialize = function(record) {
      var $a, $b, self = this;

      if (record == null) {
        record = $hash2([], {})
      }
      (($a = [""]), $b = record, $b['$default='].apply($b, $a), $a[$a.length-1]);
      self.sender_email = record['$[]']("sender_email");
      self.receiver_email = record['$[]']("receiver_email");
      self.subject = record['$[]']("subject");
      self.text = record['$[]']("text");
      self.created_at = record['$[]']("created_at");
      return self.updated_at = record['$[]']("updated_at");
    }, nil) && 'initialize';
  })(self, $scope.get('Model'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["models/user_model"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$attr_accessor', '$take', '$data_config', '$default=', '$[]']);
  self.$require("data/model");
  return (function($base, $super) {
    function $UserModel(){};
    var self = $UserModel = $klass($base, $super, 'UserModel', $UserModel);

    var def = self.$$proto, $scope = self.$$scope;

    self.$attr_accessor("email");

    self.$attr_accessor("first_name");

    self.$attr_accessor("last_name");

    self.$attr_accessor("created_at");

    self.$attr_accessor("updated_at");

    Opal.defs(self, '$api_resource_name', function() {
      var self = this;

      return "users";
    });

    Opal.defs(self, '$data_config', function() {
      var self = this;

      return [$hash2(["id", "width"], {"id": "id", "width": 50}), $hash2(["id"], {"id": "email"}), $hash2(["id"], {"id": "last_name"}), $hash2(["id"], {"id": "first_name"}), $hash2(["id"], {"id": "created_at"}), $hash2(["id"], {"id": "updated_at"})];
    });

    Opal.defs(self, '$grid_config', function() {
      var self = this;

      return self.$data_config().$take(4);
    });

    return (def.$initialize = function(record) {
      var $a, $b, self = this;

      if (record == null) {
        record = $hash2([], {})
      }
      (($a = [""]), $b = record, $b['$default='].apply($b, $a), $a[$a.length-1]);
      self.email = record['$[]']("email");
      self.first_name = record['$[]']("first_name");
      self.last_name = record['$[]']("last_name");
      self.created_at = record['$[]']("created_at");
      return self.updated_at = record['$[]']("updated_at");
    }, nil) && 'initialize';
  })(self, $scope.get('Model'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["views/panel_html"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$render', '$new', '$find_template']);
  return (function($base, $super) {
    function $PanelHtml(){};
    var self = $PanelHtml = $klass($base, $super, 'PanelHtml', $PanelHtml);

    var def = self.$$proto, $scope = self.$$scope;

    return (Opal.defs(self, '$title_template', function(title, template_name) {
      var self = this, body = nil, model = nil, template = nil, text = nil;

      body = $scope.get('TemplateManager').$render(template_name);
      model = $scope.get('PanelModel').$new($hash2(["title", "body"], {"title": title, "body": body}));
      template = $scope.get('TemplateManager').$find_template("bp_panel");
      text = template.$render(model);
      return $scope.get('Html').$new(text);
    }), nil) && 'title_template'
  })(self, null)
};
/* Generated by Opal 0.8.1 */
Opal.modules["views/viewport"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$include', '$attr_reader', '$desktop', '$instance', '$footer', '$nav_bar', '$add_nav_bar', '$add_desktop', '$add_footer', '$add', '$new', '$footer_html', '$nav_bar_config', '$nav_bar_controller']);
  self.$require("singleton");
  self.$require("qx/ui/container");
  return (function($base, $super) {
    function $Viewport(){};
    var self = $Viewport = $klass($base, $super, 'Viewport', $Viewport);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$include($scope.get('Singleton'));

    self.$attr_reader("desktop");

    self.$attr_reader("footer");

    self.$attr_reader("nav_bar");

    Opal.defs(self, '$desktop', function() {
      var self = this;

      return self.$instance().$desktop();
    });

    Opal.defs(self, '$footer', function() {
      var self = this;

      return self.$instance().$footer();
    });

    Opal.defs(self, '$nav_bar', function() {
      var self = this;

      return self.$instance().$nav_bar();
    });

    def.$initialize = TMP_1 = function() {var $zuper = $slice.call(arguments, 0);
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, $iter).apply(self, $zuper);
      self.$add_nav_bar();
      self.$add_desktop();
      return self.$add_footer();
    };

    def.$add_desktop = function() {
      var self = this;

      self.desktop = $scope.get('Workspace').$instance();
      return self.$add(self.$desktop(), $hash2(["flex"], {"flex": 1}));
    };

    def.$add_footer = function() {
      var self = this;

      self.footer = $scope.get('Footer').$new(self.$footer_html());
      return self.$add(self.$footer());
    };

    def.$add_nav_bar = function() {
      var self = this;

      self.nav_bar = $scope.get('NavBar').$new(self.$nav_bar_config(), self.$nav_bar_controller());
      return self.$add(self.$nav_bar());
    };

    def.$footer_html = function() {
      var self = this;

      return "Copyright &copy; 2016 Peter Fisk";
    };

    def.$nav_bar_config = function() {
      var self = this;

      return $hash2(["buttons", "label"], {"buttons": [$hash2(["text", "action"], {"text": "Users", "action": "open_users_window"}), $hash2(["text", "action"], {"text": "Console", "action": "open_console_window"})], "label": "RailsQx"});
    };

    return (def.$nav_bar_controller = function() {
      var self = this;

      return $scope.get('NavBarController').$instance();
    }, nil) && 'nav_bar_controller';
  })(self, $scope.get('Container'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["views/workspace"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$include', '$attr_reader', '$content_panel', '$instance', '$nav_panel', '$new', '$nav_panel_config', '$nav_panel_controller', '$add', '$add_pages', '$add_item', '$desktop=']);
  self.$require("singleton");
  return (function($base, $super) {
    function $Workspace(){};
    var self = $Workspace = $klass($base, $super, 'Workspace', $Workspace);

    var def = self.$$proto, $scope = self.$$scope;

    self.$include($scope.get('Singleton'));

    self.$attr_reader("content_panel");

    self.$attr_reader("nav_panel");

    Opal.defs(self, '$content_panel', function() {
      var self = this;

      return self.$instance().$content_panel();
    });

    Opal.defs(self, '$nav_panel', function() {
      var self = this;

      return self.$instance().$nav_panel();
    });

    def.$add_items = function() {
      var self = this;

      self.content_panel = $scope.get('StackContainer').$new();
      self.nav_panel = $scope.get('NavPanel').$new(self.$nav_panel_config(), self.$nav_panel_controller());
      self.$add(self.$nav_panel());
      self.$add(self.$content_panel(), $hash2(["flex"], {"flex": 1}));
      return self.$add_pages();
    };

    def.$add_pages = function() {
      var self = this;

      self.$content_panel().$add_item("welcome", $scope.get('WelcomePage').$new());
      self.$content_panel().$add_item("demos", $scope.get('DemosPage').$new());
      self.$content_panel().$add_item("widgets", $scope.get('WidgetsPage').$new());
      return self.$content_panel().$add_item("license", $scope.get('LicensePage').$new());
    };

    def.$initial_layout = function() {
      var self = this;

      return $scope.get('HLayout').$new();
    };

    def.$nav_panel_config = function() {
      var self = this;

      return $hash2(["buttons"], {"buttons": [$hash2(["text", "action", "arg"], {"text": "Welcome", "action": "select_page", "arg": "welcome"}), $hash2(["text", "action", "arg"], {"text": "Demos", "action": "select_page", "arg": "demos"}), $hash2(["text", "action", "arg"], {"text": "Widgets", "action": "select_page", "arg": "widgets"}), $hash2(["text", "action", "arg"], {"text": "License", "action": "select_page", "arg": "license"})]});
    };

    return (def.$nav_panel_controller = function() {
      var $a, $b, self = this, controller = nil;

      controller = $scope.get('NavPanelController').$instance();
      (($a = [self]), $b = controller, $b['$desktop='].apply($b, $a), $a[$a.length-1]);
      return controller;
    }, nil) && 'nav_panel_controller';
  })(self, $scope.get('Container'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["app/application"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$include', '$start', '$instance', '$require_tree', '$set_color_palette', '$[]', '$color_config', '$set_model_configs', '$model_configs', '$set_panel_configs', '$panel_configs', '$set_root', '$set_template_configs', '$template_configs', '$set_viewport_class', '$build', '$value']);
  self.$require("singleton");
  self.$require("configs/colors_config");
  self.$require("models/message_model");
  self.$require("models/user_model");
  self.$require("views/panel_html");
  self.$require("views/viewport");
  self.$require("views/workspace");
  return (function($base, $super) {
    function $Application(){};
    var self = $Application = $klass($base, $super, 'Application', $Application);

    var def = self.$$proto, $scope = self.$$scope;

    self.$include($scope.get('Singleton'));

    Opal.defs(self, '$start', function(root) {
      var self = this;

      return self.$instance().$start(root);
    });

    def.$start = function(root) {
      var self = this;

      self.$require_tree(".");
      $scope.get('DesktopLoader').$set_color_palette(self.$color_config()['$[]']("palette"));
      $scope.get('DesktopLoader').$set_model_configs(self.$model_configs());
      $scope.get('DesktopLoader').$set_panel_configs(self.$panel_configs());
      $scope.get('DesktopLoader').$set_root(root);
      $scope.get('DesktopLoader').$set_template_configs(self.$template_configs());
      $scope.get('DesktopLoader').$set_viewport_class($scope.get('Viewport'));
      return $scope.get('DesktopLoader').$build();
    };

    def.$color_config = function() {
      var self = this;

      return $scope.get('ColorsConfig').$value();
    };

    def.$model_configs = function() {
      var self = this;

      return [$hash2(["name", "model_class"], {"name": "message", "model_class": $scope.get('MessageModel')}), $hash2(["name", "model_class"], {"name": "user", "model_class": $scope.get('UserModel')})];
    };

    def.$panel_configs = function() {
      var self = this;

      return [$hash2(["name", "template"], {"name": "license", "template": "license"}), $hash2(["name", "template", "model", "init_data"], {"name": "user", "template": "user", "model": "user", "init_data": $hash2(["email", "first_name", "last_name"], {"email": "dude@test.com", "first_name": "Dude", "last_name": "Test"})}), $hash2(["name", "template"], {"name": "welcome", "template": "welcome"})];
    };

    return (def.$template_configs = function() {
      var self = this;

      return [$hash2(["name"], {"name": "bp_panel"}), $hash2(["name"], {"name": "guides"}), $hash2(["name"], {"name": "license"}), $hash2(["name", "model_name"], {"name": "user", "model_name": "user"}), $hash2(["name"], {"name": "welcome_bl"}), $hash2(["name"], {"name": "welcome_br"}), $hash2(["name"], {"name": "welcome_tp"})];
    }, nil) && 'template_configs';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["application"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice;

  Opal.add_stubs(['$require']);
  return self.$require("app/application")
};
/* Generated by Opal 0.8.1 */
Opal.modules["controllers/desktop_controller"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$include', '$===', '$open_console_window', '$open_messages_window', '$open_prompt_window', '$open_users_window', '$puts', '$%', '$new']);
  self.$require("native");
  self.$require("singleton");
  self.$require("views/windows/console_window");
  self.$require("views/windows/crud_window");
  self.$require("views/windows/message_window");
  return (function($base, $super) {
    function $DesktopController(){};
    var self = $DesktopController = $klass($base, $super, 'DesktopController', $DesktopController);

    var def = self.$$proto, $scope = self.$$scope;

    self.$include($scope.get('Singleton'));

    def.$handle_action = function(action) {
      var self = this, $case = nil;

      return (function() {$case = action;if ("open_console_window"['$===']($case)) {return self.$open_console_window()}else if ("open_messages_window"['$===']($case)) {return self.$open_messages_window()}else if ("open_prompt_window"['$===']($case)) {return self.$open_prompt_window()}else if ("open_users_window"['$===']($case)) {return self.$open_users_window()}else {return self.$puts("DesktopController action %s not found"['$%'](action))}})();
    };

    def.$open_console_window = function() {
      var self = this;

      return $scope.get('ConsoleWindow').$new();
    };

    def.$open_messages_window = function() {
      var self = this;

      return $scope.get('CrudWindow').$new("message");
    };

    def.$open_prompt_window = function() {
      var self = this;

      return $scope.get('MessageWindow').$new("This is a prompt window", "Prompt");
    };

    return (def.$open_users_window = function() {
      var self = this;

      return $scope.get('CrudWindow').$new("user");
    }, nil) && 'open_users_window';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["controllers/nav_bar_controller"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$include', '$===', '$open_console_window', '$open_users_window', '$puts', '$%', '$new']);
  self.$require("native");
  self.$require("singleton");
  self.$require("views/windows/console_window");
  self.$require("views/windows/crud_window");
  return (function($base, $super) {
    function $NavBarController(){};
    var self = $NavBarController = $klass($base, $super, 'NavBarController', $NavBarController);

    var def = self.$$proto, $scope = self.$$scope;

    self.$include($scope.get('Singleton'));

    def.$handle_action = function(action) {
      var self = this, $case = nil;

      return (function() {$case = action;if ("open_console_window"['$===']($case)) {return self.$open_console_window()}else if ("open_users_window"['$===']($case)) {return self.$open_users_window()}else {return self.$puts("NavBarController action %s not found"['$%'](action))}})();
    };

    def.$open_console_window = function() {
      var self = this;

      return $scope.get('ConsoleWindow').$new();
    };

    return (def.$open_users_window = function() {
      var self = this;

      return $scope.get('CrudWindow').$new("user");
    }, nil) && 'open_users_window';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["controllers/nav_panel_controller"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass;

  Opal.add_stubs(['$require', '$include', '$attr_accessor', '$===', '$select_page', '$puts', '$%', '$set_selection', '$content_panel', '$desktop']);
  self.$require("native");
  self.$require("singleton");
  return (function($base, $super) {
    function $NavPanelController(){};
    var self = $NavPanelController = $klass($base, $super, 'NavPanelController', $NavPanelController);

    var def = self.$$proto, $scope = self.$$scope, TMP_1;

    self.$include($scope.get('Singleton'));

    self.$attr_accessor("desktop");

    def.$initialize = TMP_1 = function() {var $zuper = $slice.call(arguments, 0);
      var self = this, $iter = TMP_1.$$p, $yield = $iter || nil;

      TMP_1.$$p = null;
      Opal.find_super_dispatcher(self, 'initialize', TMP_1, $iter).apply(self, $zuper);
      return self.desktop = nil;
    };

    def.$handle_action = function(action, arg) {
      var self = this, $case = nil;

      if (arg == null) {
        arg = nil
      }
      return (function() {$case = action;if ("select_page"['$===']($case)) {return self.$select_page(arg)}else {return self.$puts("NavPanelController action %s not found"['$%'](action))}})();
    };

    return (def.$select_page = function(page_tag) {
      var self = this;

      return self.$desktop().$content_panel().$set_selection(page_tag);
    }, nil) && 'select_page';
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["models/panel_model"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$attr_accessor', '$first', '$data_config', '$default=', '$[]']);
  self.$require("data/model");
  return (function($base, $super) {
    function $PanelModel(){};
    var self = $PanelModel = $klass($base, $super, 'PanelModel', $PanelModel);

    var def = self.$$proto, $scope = self.$$scope;

    self.$attr_accessor("title");

    self.$attr_accessor("body");

    Opal.defs(self, '$api_resource_name', function() {
      var self = this;

      return nil;
    });

    Opal.defs(self, '$data_config', function() {
      var self = this;

      return [$hash2(["id"], {"id": "title"}), $hash2(["id"], {"id": "body"})];
    });

    Opal.defs(self, '$grid_config', function() {
      var self = this;

      return self.$data_config().$first();
    });

    return (def.$initialize = function(record) {
      var $a, $b, self = this;

      if (record == null) {
        record = $hash2([], {})
      }
      (($a = [""]), $b = record, $b['$default='].apply($b, $a), $a[$a.length-1]);
      self.title = record['$[]']("title");
      return self.body = record['$[]']("body");
    }, nil) && 'initialize';
  })(self, $scope.get('Model'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["erb"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $module = Opal.module;

  Opal.add_stubs(['$require', '$module_function']);
  self.$require("template");
  return (function($base, $super) {
    function $ERB(){};
    var self = $ERB = $klass($base, $super, 'ERB', $ERB);

    var def = self.$$proto, $scope = self.$$scope;

    return (function($base) {
      var self = $module($base, 'Util');

      var def = self.$$proto, $scope = self.$$scope;

      var escapes = { '&': '&amp;', '<': '&lt;', '>': '&gt;', '"': '&quot;', "'": '&#39;'};

      var escape_regexp = /[&<>"']/g;

      Opal.defn(self, '$html_escape', function(str) {
        var self = this;

        return ("" + str).replace(escape_regexp, function (m) { return escapes[m] });
      });

      Opal.defn(self, '$h', def.$html_escape);

      self.$module_function("h");

      self.$module_function("html_escape");
    })(self)
  })(self, null);
};
/* Generated by Opal 0.8.1 */
Opal.modules["templates/bp_panel"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var $a, $b, TMP_1, self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice;

  Opal.add_stubs(['$require', '$new', '$append', '$append=', '$title', '$body', '$join']);
  self.$require("erb");
  return ($a = ($b = $scope.get('Template')).$new, $a.$$p = (TMP_1 = function(output_buffer){var self = TMP_1.$$s || this, $a, $b;
if (output_buffer == null) output_buffer = nil;
  output_buffer.$append("<div class=\"panel1 panel-default1\">\n  <div class=\"panel-heading\">\n    <h3 class=\"panel-title\">");
    (($a = [(self.$title())]), $b = output_buffer, $b['$append='].apply($b, $a), $a[$a.length-1]);
    output_buffer.$append("</h3>\n  </div>\n  <div style=\"overflow: visible\">\n    ");
    (($a = [(self.$body())]), $b = output_buffer, $b['$append='].apply($b, $a), $a[$a.length-1]);
    output_buffer.$append("\n  </div>\n</div>\n");
    return output_buffer.$join();}, TMP_1.$$s = self, TMP_1), $a).call($b, "bp_panel");
};
/* Generated by Opal 0.8.1 */
Opal.modules["templates/guides"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var $a, $b, TMP_1, self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice;

  Opal.add_stubs(['$require', '$new', '$append', '$join']);
  self.$require("erb");
  return ($a = ($b = $scope.get('Template')).$new, $a.$$p = (TMP_1 = function(output_buffer){var self = TMP_1.$$s || this;
if (output_buffer == null) output_buffer = nil;
  output_buffer.$append("<p>Guide information to be written.</p>\n");
    return output_buffer.$join();}, TMP_1.$$s = self, TMP_1), $a).call($b, "guides");
};
/* Generated by Opal 0.8.1 */
Opal.modules["templates/license"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var $a, $b, TMP_1, self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice;

  Opal.add_stubs(['$require', '$new', '$append', '$join']);
  self.$require("erb");
  return ($a = ($b = $scope.get('Template')).$new, $a.$$p = (TMP_1 = function(output_buffer){var self = TMP_1.$$s || this;
if (output_buffer == null) output_buffer = nil;
  output_buffer.$append("<b>The MIT License (MIT)</b>\n\n<p>Copyright (c) 2016 Peter Fisk</p>\n\n<p>Permission is hereby granted, free of charge, to any person obtaining a copy\n  of this software and associated documentation files (the \"Software\"), to deal\n  in the Software without restriction, including without limitation the rights\n  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell\n  copies of the Software, and to permit persons to whom the Software is\n  furnished to do so, subject to the following conditions:</p>\n\n<p>The above copyright notice and this permission notice shall be included in\n  all copies or substantial portions of the Software.</p>\n\n<p>THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\n  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN\n  THE SOFTWARE.</p>\n\n\n");
    return output_buffer.$join();}, TMP_1.$$s = self, TMP_1), $a).call($b, "license");
};
/* Generated by Opal 0.8.1 */
Opal.modules["templates/user"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var $a, $b, TMP_1, self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice;

  Opal.add_stubs(['$require', '$new', '$append', '$append=', '$email', '$first_name', '$last_name', '$join']);
  self.$require("erb");
  return ($a = ($b = $scope.get('Template')).$new, $a.$$p = (TMP_1 = function(output_buffer){var self = TMP_1.$$s || this, $a, $b;
if (output_buffer == null) output_buffer = nil;
  output_buffer.$append("<div class=\"panel panel-default\">\n  <div class=\"panel-heading\">\n    <h3 class=\"panel-title\">User Profile</h3>\n  </div>\n  <div class=\"panel-body\">\n    Email ");
    (($a = [(self.$email())]), $b = output_buffer, $b['$append='].apply($b, $a), $a[$a.length-1]);
    output_buffer.$append("</br>\n    First name ");
    (($a = [(self.$first_name())]), $b = output_buffer, $b['$append='].apply($b, $a), $a[$a.length-1]);
    output_buffer.$append("</br>\n    Last name ");
    (($a = [(self.$last_name())]), $b = output_buffer, $b['$append='].apply($b, $a), $a[$a.length-1]);
    output_buffer.$append("</br>\n  </div>\n</div>\n\n");
    return output_buffer.$join();}, TMP_1.$$s = self, TMP_1), $a).call($b, "user");
};
/* Generated by Opal 0.8.1 */
Opal.modules["templates/welcome_bl"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var $a, $b, TMP_1, self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice;

  Opal.add_stubs(['$require', '$new', '$append', '$join']);
  self.$require("erb");
  return ($a = ($b = $scope.get('Template')).$new, $a.$$p = (TMP_1 = function(output_buffer){var self = TMP_1.$$s || this;
if (output_buffer == null) output_buffer = nil;
  output_buffer.$append("<!-- HTML generated using hilite.me -->\n<div style=\"background: #ffffff;\">\n  <table>\n    <tr>\n      <td><pre style=\"margin: 0; line-height: 125%\"> 1\n 2\n 3\n 4\n 5\n 6\n 7\n 8\n 9\n10\n11\n12\n13\n14\n15\n16\n17\n18\n19\n20\n21\n22\n23\n24\n25\n26\n27\n28\n29\n30\n31\n32\n33\n34\n35\n36\n37\n38\n39\n40\n41\n42\n43\n44\n45\n46\n47\n48\n49\n50\n51\n52\n53\n54\n55\n56\n57\n58\n59\n60\n61\n62\n63\n64\n65\n66\n67\n68\n69\n70\n71\n72\n73\n74\n75\n76\n77\n78\n79\n80\n81\n82\n83\n84\n85\n86\n87</pre>\n      </td>\n      <td><pre style=\"margin: 0; line-height: 125%\"><span style=\"color: #008800; font-weight: bold\">class</span> <span style=\"color: #BB0066; font-weight: bold\">DemosPage</span> <span style=\"color: #333333\">&lt;</span> <span style=\"color: #003366; font-weight: bold\">Container</span>\n\t<span style=\"color: #003388; font-weight: bold\">attr_reader</span> <span style=\"color: #AA6600\">:tabpage_crud_panels</span>\n\t<span style=\"color: #003388; font-weight: bold\">attr_reader</span> <span style=\"color: #AA6600\">:tabpage_crud_panels_messages</span>\n\t<span style=\"color: #003388; font-weight: bold\">attr_reader</span> <span style=\"color: #AA6600\">:tabpage_crud_panels_tabview</span>\n\t<span style=\"color: #003388; font-weight: bold\">attr_reader</span> <span style=\"color: #AA6600\">:tabpage_crud_panels_users</span>\n\t<span style=\"color: #003388; font-weight: bold\">attr_reader</span> <span style=\"color: #AA6600\">:tabpage_desktop</span>\n\t<span style=\"color: #003388; font-weight: bold\">attr_reader</span> <span style=\"color: #AA6600\">:tabpage_formatted_lists</span>\n\t<span style=\"color: #003388; font-weight: bold\">attr_reader</span> <span style=\"color: #AA6600\">:tabpage_html_templates</span>\n\t<span style=\"color: #003388; font-weight: bold\">attr_reader</span> <span style=\"color: #AA6600\">:tabview</span>\n\n\t<span style=\"color: #008800; font-weight: bold\">def</span> <span style=\"color: #0066BB; font-weight: bold\">add_items</span>\n\t\tadd_tabview\n\t<span style=\"color: #008800; font-weight: bold\">end</span>\n\n\t<span style=\"color: #008800; font-weight: bold\">def</span> <span style=\"color: #0066BB; font-weight: bold\">add_tab_pages</span>\n\t\tadd_tab_page_desktop\n\t\tadd_tab_page_html_templates\n\t\tadd_tab_page_formatted_lists\n\t\tadd_tab_page_crud_panels\n\t<span style=\"color: #008800; font-weight: bold\">end</span>\n\n\t<span style=\"color: #008800; font-weight: bold\">def</span> <span style=\"color: #0066BB; font-weight: bold\">add_tab_page_crud_panels</span>\n\t\t<span style=\"color: #3333BB\">@tabpage_crud_panels_tabview</span> <span style=\"color: #333333\">=</span> <span style=\"color: #003366; font-weight: bold\">TabView</span><span style=\"color: #333333\">.</span>new\n\t\t<span style=\"color: #3333BB\">@tabpage_crud_panels</span> <span style=\"color: #333333\">=</span> <span style=\"color: #003366; font-weight: bold\">TabPage</span><span style=\"color: #333333\">.</span>new <span style=\"background-color: #fff0f0\">&#39;Crud Panels&#39;</span>, tabpage_crud_panels_tabview\n\t\ttabview<span style=\"color: #333333\">.</span>add tabpage_crud_panels\n\t\tadd_tab_page_crud_panels_tab_page_messages\n\t\tadd_tab_page_crud_panels_tab_page_users\n\t<span style=\"color: #008800; font-weight: bold\">end</span>\n\n\t<span style=\"color: #008800; font-weight: bold\">def</span> <span style=\"color: #0066BB; font-weight: bold\">add_tab_page_crud_panels_tab_page_messages</span>\n\t\tmessages_panel <span style=\"color: #333333\">=</span> <span style=\"color: #003366; font-weight: bold\">CrudPanel</span><span style=\"color: #333333\">.</span>new <span style=\"color: #AA6600\">:message</span>\n\t\t<span style=\"color: #3333BB\">@tabpage_crud_panels_messages</span> <span style=\"color: #333333\">=</span> <span style=\"color: #003366; font-weight: bold\">TabPage</span><span style=\"color: #333333\">.</span>new <span style=\"background-color: #fff0f0\">&#39;Messages&#39;</span>, messages_panel\n\t\ttabpage_crud_panels_tabview<span style=\"color: #333333\">.</span>add tabpage_crud_panels_messages\n\t<span style=\"color: #008800; font-weight: bold\">end</span>\n\n\t<span style=\"color: #008800; font-weight: bold\">def</span> <span style=\"color: #0066BB; font-weight: bold\">add_tab_page_crud_panels_tab_page_users</span>\n\t\tusers_panel <span style=\"color: #333333\">=</span> <span style=\"color: #003366; font-weight: bold\">CrudPanel</span><span style=\"color: #333333\">.</span>new <span style=\"color: #AA6600\">:user</span>\n\t\t<span style=\"color: #3333BB\">@tabpage_crud_panels_users</span> <span style=\"color: #333333\">=</span> <span style=\"color: #003366; font-weight: bold\">TabPage</span><span style=\"color: #333333\">.</span>new <span style=\"background-color: #fff0f0\">&#39;Users&#39;</span>, users_panel\n\t\ttabpage_crud_panels_tabview<span style=\"color: #333333\">.</span>add <span style=\"color: #3333BB\">@tabpage_crud_panels_users</span>\n\t<span style=\"color: #008800; font-weight: bold\">end</span>\n\n\t<span style=\"color: #008800; font-weight: bold\">def</span> <span style=\"color: #0066BB; font-weight: bold\">add_tab_page_desktop</span>\n\t\ticons <span style=\"color: #333333\">=</span> <span style=\"color: #333333\">[</span>\n\t\t\t\t{<span style=\"color: #AA6600\">label</span>: <span style=\"background-color: #fff0f0\">&#39;Console&#39;</span>, <span style=\"color: #AA6600\">action</span>: <span style=\"color: #AA6600\">:open_console_window</span>},\n\t\t\t\t{<span style=\"color: #AA6600\">label</span>: <span style=\"background-color: #fff0f0\">&#39;Messages&#39;</span>, <span style=\"color: #AA6600\">action</span>: <span style=\"color: #AA6600\">:open_messages_window</span>},\n\t\t\t\t{<span style=\"color: #AA6600\">label</span>: <span style=\"background-color: #fff0f0\">&#39;Prompt&#39;</span>, <span style=\"color: #AA6600\">action</span>: <span style=\"color: #AA6600\">:open_prompt_window</span>},\n\t\t\t\t{<span style=\"color: #AA6600\">label</span>: <span style=\"background-color: #fff0f0\">&#39;Users&#39;</span>, <span style=\"color: #AA6600\">action</span>: <span style=\"color: #AA6600\">:open_users_window</span>}\n\t\t<span style=\"color: #333333\">]</span>\n\t\tpanel <span style=\"color: #333333\">=</span> <span style=\"color: #003366; font-weight: bold\">DesktopPanel</span><span style=\"color: #333333\">.</span>new <span style=\"color: #003366; font-weight: bold\">DesktopController</span>, icons\n\t\t<span style=\"color: #3333BB\">@tabpage_desktop</span> <span style=\"color: #333333\">=</span> <span style=\"color: #003366; font-weight: bold\">TabPage</span><span style=\"color: #333333\">.</span>new <span style=\"background-color: #fff0f0\">&#39;Desktop&#39;</span>, panel\n\t\ttabview<span style=\"color: #333333\">.</span>add tabpage_desktop\n\t<span style=\"color: #008800; font-weight: bold\">end</span>\n\n\t<span style=\"color: #008800; font-weight: bold\">def</span> <span style=\"color: #0066BB; font-weight: bold\">add_tab_page_formatted_lists</span>\n\t\tformatted_list <span style=\"color: #333333\">=</span> <span style=\"color: #003366; font-weight: bold\">FormList</span><span style=\"color: #333333\">.</span>new <span style=\"color: #AA6600\">:user</span>, <span style=\"color: #AA6600\">:user</span>\n\t\tformatted_list<span style=\"color: #333333\">.</span>add_record({\n\t\t\t\t                          <span style=\"color: #AA6600\">email</span>: <span style=\"background-color: #fff0f0\">&#39;dude@test.com&#39;</span>,\n\t\t\t\t                          first_name: <span style=\"background-color: #fff0f0\">&#39;Dude&#39;</span>,\n\t\t\t\t                          last_name: <span style=\"background-color: #fff0f0\">&#39;Test&#39;</span>\n\t\t                          })\n\t\tformatted_list<span style=\"color: #333333\">.</span>add_record({\n\t\t\t\t                          <span style=\"color: #AA6600\">email</span>: <span style=\"background-color: #fff0f0\">&#39;dudette@test.com&#39;</span>,\n\t\t\t\t                          first_name: <span style=\"background-color: #fff0f0\">&#39;Dudette&#39;</span>,\n\t\t\t\t                          last_name: <span style=\"background-color: #fff0f0\">&#39;Testette&#39;</span>\n\t\t                          })\n\t\tformatted_list<span style=\"color: #333333\">.</span>on_select <span style=\"color: #007020\">lambda</span> { <span style=\"color: #333333\">|</span>x<span style=\"color: #333333\">|</span> <span style=\"color: #007020\">puts</span> x<span style=\"color: #333333\">[</span><span style=\"color: #AA6600\">:data</span><span style=\"color: #333333\">]</span> <span style=\"color: #008800; font-weight: bold\">unless</span> x<span style=\"color: #333333\">.</span>nil? }\n\t\t<span style=\"color: #3333BB\">@tabpage_formatted_lists</span> <span style=\"color: #333333\">=</span> <span style=\"color: #003366; font-weight: bold\">TabPage</span><span style=\"color: #333333\">.</span>new <span style=\"background-color: #fff0f0\">&#39;Formatted Lists&#39;</span>, formatted_list\n\t\ttabview<span style=\"color: #333333\">.</span>add tabpage_formatted_lists\n\t<span style=\"color: #008800; font-weight: bold\">end</span>\n\n\t<span style=\"color: #008800; font-weight: bold\">def</span> <span style=\"color: #0066BB; font-weight: bold\">add_tab_page_html_templates</span>\n\t\thtml <span style=\"color: #333333\">=</span> <span style=\"color: #003366; font-weight: bold\">HtmlManager</span><span style=\"color: #333333\">.</span>get_panel <span style=\"color: #AA6600\">:user</span>\n\t\t<span style=\"color: #3333BB\">@tabpage_html_templates</span> <span style=\"color: #333333\">=</span> <span style=\"color: #003366; font-weight: bold\">TabPage</span><span style=\"color: #333333\">.</span>new <span style=\"background-color: #fff0f0\">&#39;Html Templates&#39;</span>, html\n\t\ttabview<span style=\"color: #333333\">.</span>add tabpage_html_templates\n\t<span style=\"color: #008800; font-weight: bold\">end</span>\n\n\t<span style=\"color: #008800; font-weight: bold\">def</span> <span style=\"color: #0066BB; font-weight: bold\">add_tabview</span>\n\t\t<span style=\"color: #3333BB\">@tabview</span> <span style=\"color: #333333\">=</span> <span style=\"color: #003366; font-weight: bold\">TabView</span><span style=\"color: #333333\">.</span>new\n\t\tadd tabview, {<span style=\"color: #AA6600\">flex</span>: <span style=\"color: #0000DD; font-weight: bold\">1</span>}\n\t\tadd_tab_pages\n\t<span style=\"color: #008800; font-weight: bold\">end</span>\n\n\t<span style=\"color: #008800; font-weight: bold\">def</span> <span style=\"color: #0066BB; font-weight: bold\">initial_background_color</span>\n\t\t<span style=\"color: #003366; font-weight: bold\">WHITE</span>\n\t<span style=\"color: #008800; font-weight: bold\">end</span>\n\n<span style=\"color: #008800; font-weight: bold\">end</span>\n</pre>\n      </td>\n    </tr>\n  </table>\n</div>\n");
    return output_buffer.$join();}, TMP_1.$$s = self, TMP_1), $a).call($b, "welcome_bl");
};
/* Generated by Opal 0.8.1 */
Opal.modules["templates/welcome_br"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var $a, $b, TMP_1, self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice;

  Opal.add_stubs(['$require', '$new', '$append', '$join']);
  self.$require("erb");
  return ($a = ($b = $scope.get('Template')).$new, $a.$$p = (TMP_1 = function(output_buffer){var self = TMP_1.$$s || this;
if (output_buffer == null) output_buffer = nil;
  output_buffer.$append("<p>An introduction to RailsQx classes will go here.</p>\n");
    return output_buffer.$join();}, TMP_1.$$s = self, TMP_1), $a).call($b, "welcome_br");
};
/* Generated by Opal 0.8.1 */
Opal.modules["templates/welcome_tp"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var $a, $b, TMP_1, self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice;

  Opal.add_stubs(['$require', '$new', '$append', '$join']);
  self.$require("erb");
  return ($a = ($b = $scope.get('Template')).$new, $a.$$p = (TMP_1 = function(output_buffer){var self = TMP_1.$$s || this;
if (output_buffer == null) output_buffer = nil;
  output_buffer.$append("<div style=\"padding:15px; font-size:1.2em;\">\n  <b>RailsQx</b> is a framework for developing Rich Internet Applications\n  using Ruby-on-Rails on the server-side and the Qooxdoo JavaScript libraries\n  for client-side layout, and widgets.\n\n  Styling is done using Twitter Bootstrap and client-side application logic is written in\n  Ruby (with EBR templating).\n\n  Client-side Ruby code is automatically converted to JavaScript by RoR Sprockets during deployment.\n</div>\n");
    return output_buffer.$join();}, TMP_1.$$s = self, TMP_1), $a).call($b, "welcome_tp");
};
/* Generated by Opal 0.8.1 */
Opal.modules["views/pages/demos_page"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$attr_reader', '$add_tabview', '$add_tab_page_desktop', '$add_tab_page_html_templates', '$add_tab_page_formatted_lists', '$add_tab_page_crud_panels', '$new', '$tabpage_crud_panels_tabview', '$add', '$tabview', '$tabpage_crud_panels', '$add_tab_page_crud_panels_tab_page_messages', '$add_tab_page_crud_panels_tab_page_users', '$tabpage_crud_panels_messages', '$tabpage_desktop', '$add_record', '$on_select', '$lambda', '$nil?', '$puts', '$[]', '$tabpage_formatted_lists', '$get_panel', '$tabpage_html_templates', '$add_tab_pages']);
  self.$require("controllers/desktop_controller");
  self.$require("managers/html_manager");
  self.$require("panels/crud_panel");
  self.$require("widgets/form_list");
  self.$require("widgets/tab_page");
  self.$require("widgets/tab_view");
  return (function($base, $super) {
    function $DemosPage(){};
    var self = $DemosPage = $klass($base, $super, 'DemosPage', $DemosPage);

    var def = self.$$proto, $scope = self.$$scope;

    def.tabpage_crud_panels_users = nil;
    self.$attr_reader("tabpage_crud_panels");

    self.$attr_reader("tabpage_crud_panels_messages");

    self.$attr_reader("tabpage_crud_panels_tabview");

    self.$attr_reader("tabpage_crud_panels_users");

    self.$attr_reader("tabpage_desktop");

    self.$attr_reader("tabpage_formatted_lists");

    self.$attr_reader("tabpage_html_templates");

    self.$attr_reader("tabview");

    def.$add_items = function() {
      var self = this;

      return self.$add_tabview();
    };

    def.$add_tab_pages = function() {
      var self = this;

      self.$add_tab_page_desktop();
      self.$add_tab_page_html_templates();
      self.$add_tab_page_formatted_lists();
      return self.$add_tab_page_crud_panels();
    };

    def.$add_tab_page_crud_panels = function() {
      var self = this;

      self.tabpage_crud_panels_tabview = $scope.get('TabView').$new();
      self.tabpage_crud_panels = $scope.get('TabPage').$new("Crud Panels", self.$tabpage_crud_panels_tabview());
      self.$tabview().$add(self.$tabpage_crud_panels());
      self.$add_tab_page_crud_panels_tab_page_messages();
      return self.$add_tab_page_crud_panels_tab_page_users();
    };

    def.$add_tab_page_crud_panels_tab_page_messages = function() {
      var self = this, messages_panel = nil;

      messages_panel = $scope.get('CrudPanel').$new("message");
      self.tabpage_crud_panels_messages = $scope.get('TabPage').$new("Messages", messages_panel);
      return self.$tabpage_crud_panels_tabview().$add(self.$tabpage_crud_panels_messages());
    };

    def.$add_tab_page_crud_panels_tab_page_users = function() {
      var self = this, users_panel = nil;

      users_panel = $scope.get('CrudPanel').$new("user");
      self.tabpage_crud_panels_users = $scope.get('TabPage').$new("Users", users_panel);
      return self.$tabpage_crud_panels_tabview().$add(self.tabpage_crud_panels_users);
    };

    def.$add_tab_page_desktop = function() {
      var self = this, icons = nil, panel = nil;

      icons = [$hash2(["label", "action"], {"label": "Console", "action": "open_console_window"}), $hash2(["label", "action"], {"label": "Messages", "action": "open_messages_window"}), $hash2(["label", "action"], {"label": "Prompt", "action": "open_prompt_window"}), $hash2(["label", "action"], {"label": "Users", "action": "open_users_window"})];
      panel = $scope.get('DesktopPanel').$new($scope.get('DesktopController'), icons);
      self.tabpage_desktop = $scope.get('TabPage').$new("Desktop", panel);
      return self.$tabview().$add(self.$tabpage_desktop());
    };

    def.$add_tab_page_formatted_lists = function() {
      var $a, $b, TMP_1, self = this, formatted_list = nil;

      formatted_list = $scope.get('FormList').$new("user", "user");
      formatted_list.$add_record($hash2(["email", "first_name", "last_name"], {"email": "dude@test.com", "first_name": "Dude", "last_name": "Test"}));
      formatted_list.$add_record($hash2(["email", "first_name", "last_name"], {"email": "dudette@test.com", "first_name": "Dudette", "last_name": "Testette"}));
      formatted_list.$on_select(($a = ($b = self).$lambda, $a.$$p = (TMP_1 = function(x){var self = TMP_1.$$s || this, $a;
if (x == null) x = nil;
      if ((($a = x['$nil?']()) !== nil && (!$a.$$is_boolean || $a == true))) {
          return nil
          } else {
          return self.$puts(x['$[]']("data"))
        }}, TMP_1.$$s = self, TMP_1), $a).call($b));
      self.tabpage_formatted_lists = $scope.get('TabPage').$new("Formatted Lists", formatted_list);
      return self.$tabview().$add(self.$tabpage_formatted_lists());
    };

    def.$add_tab_page_html_templates = function() {
      var self = this, html = nil;

      html = $scope.get('HtmlManager').$get_panel("user");
      self.tabpage_html_templates = $scope.get('TabPage').$new("Html Templates", html);
      return self.$tabview().$add(self.$tabpage_html_templates());
    };

    return (def.$add_tabview = function() {
      var self = this;

      self.tabview = $scope.get('TabView').$new();
      self.$add(self.$tabview(), $hash2(["flex"], {"flex": 1}));
      return self.$add_tab_pages();
    }, nil) && 'add_tabview';
  })(self, $scope.get('Container'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["views/pages/license_page"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$add_license', '$title_template', '$add']);
  self.$require("views/panel_html");
  return (function($base, $super) {
    function $LicensePage(){};
    var self = $LicensePage = $klass($base, $super, 'LicensePage', $LicensePage);

    var def = self.$$proto, $scope = self.$$scope;

    def.$add_items = function() {
      var self = this;

      return self.$add_license();
    };

    def.$add_license = function() {
      var self = this, title = nil, html = nil;

      title = "MIT License";
      html = $scope.get('PanelHtml').$title_template(title, "license");
      return self.$add(html, $hash2(["flex"], {"flex": 1}));
    };

    return (def.$initial_padding = function() {
      var self = this;

      return 25;
    }, nil) && 'initial_padding';
  })(self, $scope.get('Container'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["views/pages/welcome_page"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2;

  Opal.add_stubs(['$require', '$attr_reader', '$add_top_panel', '$add_bottom_panel', '$new', '$set_layout', '$bottom_panel', '$add', '$title_template', '$setOverflowX', '$widget', '$setOverflowY', '$top_panel']);
  self.$require("views/panel_html");
  return (function($base, $super) {
    function $WelcomePage(){};
    var self = $WelcomePage = $klass($base, $super, 'WelcomePage', $WelcomePage);

    var def = self.$$proto, $scope = self.$$scope;

    self.$attr_reader("bottom_panel");

    self.$attr_reader("top_panel");

    def.$add_items = function() {
      var self = this;

      self.$add_top_panel();
      return self.$add_bottom_panel();
    };

    def.$add_bottom_panel = function() {
      var self = this, title = nil, html = nil;

      self.bottom_panel = $scope.get('Container').$new();
      self.$bottom_panel().$set_layout($scope.get('HLayout').$new());
      self.$add(self.$bottom_panel(), $hash2(["flex"], {"flex": 2}));
      title = "Client-side Ruby Code Sample (automatically converted to JavaScript)";
      html = $scope.get('PanelHtml').$title_template(title, "welcome_bl");
      html.$widget().$setOverflowX("auto");
      html.$widget().$setOverflowY("auto");
      return self.$bottom_panel().$add(html, $hash2(["flex"], {"flex": 1}));
    };

    return (def.$add_top_panel = function() {
      var self = this, title = nil, html = nil;

      self.top_panel = $scope.get('Container').$new();
      self.$add(self.$top_panel(), $hash2(["height"], {"height": 17}));
      title = "RailsQx Introduction";
      html = $scope.get('PanelHtml').$title_template(title, "welcome_tp");
      return self.$top_panel().$add(html, $hash2(["flex"], {"flex": 1}));
    }, nil) && 'add_top_panel';
  })(self, $scope.get('Container'));
};
/* Generated by Opal 0.8.1 */
Opal.modules["views/pages/widgets_page"] = function(Opal) {
  Opal.dynamic_require_severity = "ignore";
  function $rb_gt(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs > rhs : lhs['$>'](rhs);
  }
  function $rb_plus(lhs, rhs) {
    return (typeof(lhs) === 'number' && typeof(rhs) === 'number') ? lhs + rhs : lhs['$+'](rhs);
  }
  var self = Opal.top, $scope = Opal, nil = Opal.nil, $breaker = Opal.breaker, $slice = Opal.slice, $klass = Opal.klass, $hash2 = Opal.hash2, $range = Opal.range;

  Opal.add_stubs(['$require', '$attr_reader', '$add_combo_box', '$add_select_box', '$add_tree', '$new', '$items', '$set_value', '$first', '$on_change_value', '$lambda', '$puts', '$add', '$on_change_selection', '$add_tree_node_children', '$each', '$<<', '$rand', '$[]=']);
  self.$require("widgets/form_combo_box");
  self.$require("widgets/form_select_box");
  self.$require("widgets/tree");
  return (function($base, $super) {
    function $WidgetsPage(){};
    var self = $WidgetsPage = $klass($base, $super, 'WidgetsPage', $WidgetsPage);

    var def = self.$$proto, $scope = self.$$scope;

    self.$attr_reader("items");

    def.$add_items = function() {
      var self = this;

      self.items = ["One", "Two", "Three", "Four", "Five"];
      self.$add_combo_box();
      self.$add_select_box();
      return self.$add_tree();
    };

    def.$add_combo_box = function() {
      var $a, $b, TMP_1, self = this, combo_box = nil;

      combo_box = $scope.get('FormComboBox').$new(self.$items());
      combo_box.$set_value(self.$items().$first());
      combo_box.$on_change_value(($a = ($b = self).$lambda, $a.$$p = (TMP_1 = function(x){var self = TMP_1.$$s || this;
if (x == null) x = nil;
      return self.$puts("on_change_value", x)}, TMP_1.$$s = self, TMP_1), $a).call($b));
      return self.$add(combo_box);
    };

    def.$add_select_box = function() {
      var $a, $b, TMP_2, self = this, select_box = nil;

      select_box = $scope.get('FormSelectBox').$new(self.$items());
      select_box.$on_change_selection(($a = ($b = self).$lambda, $a.$$p = (TMP_2 = function(x){var self = TMP_2.$$s || this;
if (x == null) x = nil;
      return self.$puts("on_change_selection", x)}, TMP_2.$$s = self, TMP_2), $a).call($b));
      return self.$add(select_box);
    };

    def.$add_tree = function() {
      var self = this, root = nil, tree = nil;

      root = $hash2(["name"], {"name": "Root"});
      self.$add_tree_node_children(root, "Node", 0);
      tree = $scope.get('Tree').$new(root);
      return self.$add(tree);
    };

    def.$add_tree_node_children = function(node, pname, depth) {
      var $a, $b, TMP_3, self = this, children = nil;

      if ($rb_gt(depth, 3)) {
        return nil
        } else {
        children = [];
        ($a = ($b = ($range(1, ($rb_plus(2, self.$rand(5))), false))).$each, $a.$$p = (TMP_3 = function(n){var self = TMP_3.$$s || this, name = nil, child = nil;
if (n == null) n = nil;
        name = "" + (pname) + "." + (n);
          child = $hash2(["name"], {"name": name});
          self.$add_tree_node_children(child, name, $rb_plus(depth, 1));
          return children['$<<'](child);}, TMP_3.$$s = self, TMP_3), $a).call($b);
        return node['$[]=']("children", children);
      };
    };

    return (def.$initial_padding = function() {
      var self = this;

      return 25;
    }, nil) && 'initial_padding';
  })(self, $scope.get('Container'));
};
